!----------------------------------------------------
! purpose: a combined subroutines for grid 
!          manipulation
! revised: zy
! date   : 2011-06-16
!----------------------------------------------------

module  acd_grid 

! !USES:
  use dynamics_vars, only: T_FVDYCORE_GRID
  use shr_kind_mod, only : r8 => shr_kind_r8

#ifdef NO_R16
   integer,parameter :: r16= selected_real_kind(12) ! 8 byte real
#else
   integer,parameter :: r16= selected_real_kind(24) ! 16 byte real
#endif
!
! !PUBLIC MEMBER FUNCTIONS:
      public :: a2c3d, d2a2c_winds, a2d3d
!EOP

! Magic numbers used in this module

      real(r8), parameter ::  D0_0                    =  0.0_r8
      real(r8), parameter ::  D0_125                  =  0.125_r8
      real(r8), parameter ::  D0_25                   =  0.25_r8
      real(r8), parameter ::  D0_5                    =  0.5_r8
      real(r8), parameter ::  D1_0                    =  1.0_r8
      real(r8), parameter ::  D2_0                    =  2.0_r8
      real(r8), parameter ::  D1E30                   =  1.0e30_r8

contains


!-----------------------------------------------------------------------
!BOP
! !IROUTINE: d2a2c_winds --- Interpolate winds
!
! !INTERFACE:

! old one
! subroutine d2a2c_winds(grid, u, v, ua, va, uc, vc, u_cen, v_cen, reset_winds, met_rlx)

! zy uses this for ffsl offline
  subroutine d2a2c_winds(grid, u, v, ua, va, uc, vc)

  implicit none

! !PARAMETERS:
  type (T_FVDYCORE_GRID), intent(in) :: grid

  real(r8), intent(in   ):: u(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_s)
  real(r8), intent(inout):: v(grid%im,grid%jfirst-grid%ng_s:grid%jlast+grid%ng_d)
  real(r8), intent(  out):: ua(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d)
  real(r8), intent(  out):: va(grid%im,grid%jfirst-grid%ng_s:grid%jlast+grid%ng_d)
  real(r8), intent(  out):: uc(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d)
 ! real(r8), intent(  out):: vc(grid%im,grid%jfirst-3:grid%jlast+3 )
  real(r8), intent(  out):: vc(grid%im,grid%jfirst:grid%jlast )

! cell centered winds
!  real(r8), intent(in):: u_cen(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d)
!  real(r8), intent(in):: v_cen(grid%im,grid%jfirst-grid%ng_s:grid%jlast+grid%ng_d)

!  zy 2011.06.15
!  real(r8), intent(in):: met_rlx

! !DESCRIPTION:
!
!   Calculate the cell-centered (A-grid) winds and the cell-wall perpendicular
!   (C-grid) winds from the cell-wall parallel (D-grid) winds.
!
!   This routine assumes that U and V have complete haloes!  As a result,
!   the A-grid and C-grid results should have complete haloes from +/- ng_c
!   (which is generally smaller than ng_d).  This feature has not been 
!   thoroughly tested.
!
! !REVISION HISTORY:
!   WP   2007.06.01     Creation
!   WS   2007.07.03     Added ProTeX documentation, removed unused vars.
!   WS   2009.04.15     Fixed numerous problems in indexing bounds
!
!EOP
!-----------------------------------------------------------------------
!BOC

  real(r8) us, vs, un, vn
  real(r8) uanp(grid%im), uasp(grid%im), vanp(grid%im), vasp(grid%im), r2im

  real(r8), pointer:: sinlon(:)
  real(r8), pointer:: coslon(:)
  real(r8), pointer:: sinl5(:)
  real(r8), pointer:: cosl5(:)

  integer :: i, j, im2
  integer :: im, jm, jfirst, jlast, ng_s, ng_c, ng_d
  integer :: jn1gc, js1gc, jn2gc, js2gc   ! ng_c ghosted bounds
  integer :: js2gd, jn2gd                 ! ng_d ghosted bounds
  integer :: js2gs, jn2gsm1               ! ng_s ghosted bounds
  integer :: js2g2, jn1g2                 ! 2-lat ghosted bounds

  im     = grid%im
  jm     = grid%jm
  jfirst = grid%jfirst
  jlast  = grid%jlast

  ng_c   = grid%ng_c
  ng_d   = grid%ng_d
  ng_s   = grid%ng_s

  im2 = im/2

  js1gc  = max(1,jfirst-ng_c) ! ng_c lats on S (starting at 1)
  jn1gc  = min(jm,jlast+ng_c) ! ng_c latitudes on N (ending at jm)
  js2gc  = max(2,jfirst-ng_c) ! ng_c lats on S (starting at 2)
  jn2gc  = min(jm-1,jlast+ng_c)   ! ng_c latitudes on N (ending at jm-1)
  js2gs  = max(2,jfirst-ng_s) ! ng_s latitudes on S (starting at 2)
  jn2gsm1= min(jm-1,jlast+ng_s-1) ! ng_s-1 latitudes on N (ending at jm-1)
  js2gd  = max(2,jfirst-ng_d) ! ng_d latitudes on S (starting at 2)
  jn2gd  = min(jm-1,jlast+ng_d)   ! ng_d latitudes on N (ending at jm-1)
  js2g2  = max(2,jfirst-2)    ! 2 latitudes on S (starting at 2)
  jn1g2  = min(jm,jlast+2)    ! 2 latitudes on N (ending at jm)

  sinlon => grid%sinlon
  coslon => grid%coslon
  sinl5  => grid%sinl5
  cosl5  => grid%cosl5

! Get D-grid V-wind at the poles.

    r2im = 0.5_r16/real(im,r16)

    if ( jfirst-ng_d <= 1 ) then

!
! Treat SP
!
       do i=1,im-1
          uasp(i) = u(i,2) + u(i,3)
          vasp(i) = v(i,2) + v(i+1,2)
       enddo

       uasp(im) = u(im,2) + u(im,3)
       vasp(im) = v(im,2) + v(1,2)

! Projection at SP

       us = D0_0
       vs = D0_0

       do i=1,im2
          us = us + (uasp(i+im2)-uasp(i))*sinlon(i)    &
                  + (vasp(i)-vasp(i+im2))*coslon(i)
          vs = vs + (uasp(i+im2)-uasp(i))*coslon(i)    &
                  + (vasp(i+im2)-vasp(i))*sinlon(i)
       enddo
       us = us*r2im
       vs = vs*r2im

! get V-wind at SP

       do i=1,im2
          v(i,  1) =  us*cosl5(i) - vs*sinl5(i)
          v(i+im2,1) = -v(i,  1)
       enddo

    endif

    if ( jlast+ng_d >= jm ) then

!
! Treat NP
!
       do i=1,im-1
          uanp(i) = u(i,jm-1) + u(i,jm)
          vanp(i) = v(i,jm-1) + v(i+1,jm-1)
       enddo

       uanp(im) = u(im,jm-1) + u(im,jm)
       vanp(im) = v(im,jm-1) + v(1,jm-1)

! Projection at NP

       un = D0_0
       vn = D0_0
       do i=1,im2
          un = un + (uanp(i+im2)-uanp(i))*sinlon(i)   &
                  + (vanp(i+im2)-vanp(i))*coslon(i)
          vn = vn + (uanp(i)-uanp(i+im2))*coslon(i)   &
                  + (vanp(i+im2)-vanp(i))*sinlon(i)
       enddo
       un = un*r2im
       vn = vn*r2im

! get V-wind at NP

       do i=1,im2
          v(i,jm) = -un*cosl5(i) - vn*sinl5(i)
          v(i+im2,jm) = -v(i,jm)
       enddo

    endif

    ua(:,:) = D0_0
    va(:,:) = D0_0

    do j=js2gs,jn2gd
       do i=1,im-1
          va(i,j) = v(i,j) + v(i+1,j)
       enddo
          va(im,j) = v(im,j) + v(1,j)
    enddo

    do j=js2gd,jn2gsm1  ! WS: should be safe since u defined to jn2gs
       do i=1,im
          ua(i,j) = u(i,j) + u(i,j+1)
       enddo
    enddo

!
! reset cell center winds to the offline meteorlogy data
!
! zy comments these

!    if ( reset_winds .and. met_rlx > D0_0 ) then          
!       ua(:,:) = (D1_0-met_rlx)*ua(:,:) + met_rlx*( D2_0*u_cen(:,:) )
!       va(:,:) = (D1_0-met_rlx)*va(:,:) + met_rlx*( D2_0*v_cen(:,:) )
!    endif

    if ( jfirst-ng_d <= 1 ) then
! Projection at SP
       us = D0_0
       vs = D0_0


       do i=1,im2
          us = us + (ua(i+im2,2)-ua(i    ,2))*sinlon(i)         &
               + (va(i    ,2)-va(i+im2,2))*coslon(i)
          vs = vs + (ua(i+im2,2)-ua(i    ,2))*coslon(i)         &
               + (va(i+im2,2)-va(i    ,2))*sinlon(i)
       enddo

       us = us/im
       vs = vs/im

       ! SP
       do i=1,im2
          ua(i,1)  = -us*sinlon(i) - vs*coslon(i)
          va(i,1)  =  us*coslon(i) - vs*sinlon(i)
          ua(i+im2,1)  = -ua(i,1)
          va(i+im2,1)  = -va(i,1)
       enddo

    endif

    if ( jlast+ng_d >= jm ) then

! Projection at NP
       un = D0_0
       vn = D0_0

       j = jm-1
       do i=1,im2
          un = un + (ua(i+im2,j)-ua(i    ,j))*sinlon(i)        &
               + (va(i+im2,j)-va(i    ,j))*coslon(i)
          vn = vn + (ua(i    ,j)-ua(i+im2,j))*coslon(i)        &
               + (va(i+im2,j)-va(i    ,j))*sinlon(i)
       enddo

       un = un/im
       vn = vn/im

       ! NP
       do i=1,im2
          ua(i,jm) = -un*sinlon(i) + vn*coslon(i)
          va(i,jm) = -un*coslon(i) - vn*sinlon(i)
          ua(i+im2,jm) = -ua(i,jm)
          va(i+im2,jm) = -va(i,jm)
       enddo

    endif


! A -> C
        do j=js1gc,jn1gc       ! uc needed N*ng_c S*ng_c, ua defined at poles
! i=1
            uc(1,j) = D0_25*(ua(1,j)+ua(im,j))


          do i=2,im

            uc(i,j) = D0_25*(ua(i,j)+ua(i-1,j))
          enddo
        enddo

        do j=js2g2,jn1g2       ! vc needed N*2, S*2 (for ycc), va defined at poles
          do i=1,im
            vc(i,j) = D0_25*(va(i,j)+va(i,j-1))  ! va needed N*2 S*3
          enddo
        enddo
        if ( jfirst==1 ) then
           do i=1,im
              vc(i,1) = D0_0   ! Needed to avoid undefined values in VC
           enddo
        endif
!EOC
 end subroutine d2a2c_winds
!-----------------------------------------------------------------------


     subroutine a2d3d( grid, u, v )

! !USES:

!#if defined( SPMD )
!      use mod_comm, only: mp_send3d, mp_recv3d
!#endif

      implicit none
! !INPUT PARAMETERS:
      type (T_FVDYCORE_GRID), intent(in) :: grid

! !INPUT/OUTPUT PARAMETERS:
      real(r8), intent(inout) :: u(grid%ifirst:grid%ilast,grid%jfirst:grid%jlast,grid%km) ! U-Wind
      real(r8), intent(inout) :: v(grid%ifirst:grid%ilast,grid%jfirst:grid%jlast,grid%km) ! V-Wind

      integer  :: im      ! Dimensions longitude (total)
      integer  :: jm      ! Dimensions latitude (total)
      integer  :: km      ! Dimensions vertical (total)
      integer  :: ifirst  ! longitude strip start
      integer  :: ilast   ! longitude strip finish
      integer  :: jfirst  ! latitude strip start
      integer  :: jlast   ! latitude strip finish
      integer  :: iam     ! process identifier
      integer  :: myidxy_y, myidxy_x, nprxy_x
      integer  :: commxy

      real(r8), parameter :: UNDEFINED = 1.0D15

      integer  :: i, j, k, itot, jtot
      real(r8) :: vwest(grid%jfirst:grid%jlast,grid%km)
      real(r8) :: usouth(grid%ifirst:grid%ilast,grid%km)

!#if defined( SPMD )
!      integer dest, src
!#endif


      im     = grid%im
      jm     = grid%jm
      km     = grid%km
      ifirst = grid%ifirst
      ilast  = grid%ilast
      jfirst = grid%jfirst
      jlast  = grid%jlast

!      iam      = grid%iam
!      myidxy_x = grid%myidxy_x
!      myidxy_y = grid%myidxy_y
!      nprxy_x  = grid%nprxy_x

!      commxy   = grid%commxy

      itot = ilast-ifirst+1
      jtot = jlast-jfirst+1

!#if defined( SPMD )
! Send one latitude to the north
!      call mp_send3d( commxy, iam+nprxy_x, iam-nprxy_x, im, jm, km,    &
!                      ifirst, ilast, jfirst, jlast, 1, km,             &
!                      ifirst, ilast, jlast, jlast, 1, km, u )
!      call mp_recv3d( commxy, iam-nprxy_x, im, jm, km,                 &
!                      ifirst, ilast, jfirst-1, jfirst-1, 1, km,        &
!                      ifirst, ilast, jfirst-1, jfirst-1, 1, km, usouth )
!#endif


     do k=1,km
        do j=jlast, jfirst+1, -1
          do i=ifirst,ilast
            u(i,j,k) = D0_5*(u(i,j-1,k) + u(i,j,k))
          enddo
        enddo
      enddo

!#if defined( SPMD )
!      if ( jfirst > 1 ) then
!!$omp  parallel do private(i, k)
!         do k=1,km
!            do i=ifirst,ilast
!              u(i,jfirst,k) = D0_5 * ( u(i,jfirst,k) + usouth(i,k) )
!            enddo
!         enddo
!      endif
!#endif

      if ( jfirst ==  1 ) then
!$omp  parallel do private(i,k)
         do k=1,km
            do i=ifirst,ilast
               u(i,1,k) = UNDEFINED
            enddo
         enddo
      endif

!
! V-winds
!

! Pack vwest with wrap-around condition

!$omp  parallel do private(j,k)
      do k = 1,km
         do j=jfirst,jlast
            vwest(j,k) = v(ilast,j,k)
         enddo
      enddo

!#if defined( SPMD )
!      if (itot /= im) then
!         dest = myidxy_y*nprxy_x + MOD(iam+1,nprxy_x)
!         src  = myidxy_y*nprxy_x + MOD(iam+nprxy_x-1,nprxy_x)
!         call mp_send3d( commxy, dest, src, im, jm, km,                   &
!                         ifirst, ilast, jfirst, jlast, 1, km,             &
!                         ilast, ilast, jfirst, jlast, 1, km, v )
!         call mp_recv3d( commxy, src, im, jm, km,                         &
!                         ifirst-1, ifirst-1, jfirst, jlast, 1, km,        &
!                         ifirst-1, ifirst-1, jfirst, jlast, 1, km, vwest )
!      endif
!#endif

!
! Beware: ilast is en route, don't alter its value
!

!$omp  parallel do private(i,j,k)
      do k=1,km
         do j=jfirst, jlast
            do i=ilast,ifirst+1,-1
               v(i,j,k) = D0_5*(v(i-1,j,k) + v(i,j,k))
            enddo
         enddo
      enddo
!
! Clean up shop
!

!$omp  parallel do private(i,j,k)
      do k=1,km
         do j=jfirst, jlast
            v(ifirst,j,k)= D0_5*(vwest(j,k) + v(ifirst,j,k))
         enddo
      enddo

      return
!EOC
      end subroutine a2d3d

 subroutine a2c3d(grid, ua, va, uc, vc)

! IN/OUT
   type (T_FVDYCORE_GRID), intent(in) :: grid   
   real(r8), intent(in) :: ua(grid%im,grid%km,grid%jfirstxy:grid%jlastxy)  ! ng_d, ng_s are zero under no spmd
   real(r8), intent(in) :: va(grid%im,grid%km,grid%jfirstxy:grid%jlastxy)
   real(r8), intent(out):: uc(grid%im,grid%jfirst-grid%ng_c:grid%jlast+grid%ng_c, grid%km)
   real(r8), intent(out):: vc(grid%im,grid%jfirst-grid%ng_d:grid%jlast+grid%ng_d, grid%km)
! LOCAL
   integer :: js1gc, jn1gc, js2g2, jn1g2 ! set
   integer :: i, j, k                    ! index
   integer :: im, km                     ! for convienence


!  js1gc  = max(1,grid%jfirst-grid%ng_c) ! ng_c lats on S (starting at 1)
!  jn1gc  = min(grid%jm,grid%jlast+grid%ng_c) ! ng_c latitudes on N (ending at jm)
!  js2g2  = max(2,grid%jfirst-2)         ! 2 latitudes on S (starting at 2)
!  jn1g2  = min(grid%jm,grid%jlast+2)    ! 2 latitudes on N (ending at jm)
   js1gc  = max(grid%jfirstxy,grid%jfirst-grid%ng_c) ! ng_c lats on S (starting at 1)
   jn1gc  = min(grid%jlastxy,grid%jlast+grid%ng_c) ! ng_c latitudes on N (ending at jm)
   js2g2  = max(1+grid%jfirstxy,grid%jfirst-grid%ng_d)         ! 2 latitudes on S (starting at 2)
   jn1g2  = min(grid%jlastxy,grid%jlast+grid%ng_d)    ! 2 latitudes on N (ending at jm)

   im     = grid%im
   km     = grid%km

!$omp  parallel do private(k, i, j)
     do k =1, km
         do j=js1gc,jn1gc       ! uc needed N*ng_c S*ng_c, ua defined at poles
            uc(1,j,k) = D0_5*(ua(1,k,j)+ua(im,k,j))
          do i=2,im
            uc(i,j,k) = D0_5*(ua(i,k,j)+ua(i-1,k,j))
          enddo
         enddo

         do j=js2g2,jn1g2       ! vc needed N*2, S*2 (for ycc), va defined at poles
           do i=1,im
              vc(i,j,k) = D0_5*(va(i,k,j)+va(i,k,j-1))  ! va needed N*2 S*3
           enddo
         enddo
     enddo

   end subroutine a2c3d
end module acd_grid

