#include <misc.h>

subroutine herxinw(pf      ,pkcnst  ,fb      ,fxl     ,fxr     , &
                  x       ,idp     ,fint    , nlonex,      &
                  nlon    , plon, beglatex, endlatex, plev, jcen  )

!----------------------------------------------------------------------- 
! 
! Purpose: 
! 
! Method: 
! For each departure point in the latitude slice being forecast,
! interpolate (using equally spaced Hermite cubic formulas) to its
! x value at each latitude required for later interpolation in the y
! direction.
! 
! Author: 
! Original version:  J. Olson
! Standardized:      J. Rosinski, June 1992
! Reviewed:          D. Williamson, P. Rasch, August 1992
! Reviewed:          D. Williamson, P. Rasch, March 1996
!
!-----------------------------------------------------------------------
!
! $Id: herxinw.F90,v 1.1.1.1 2014/08/26 01:56:31 wangzzh Exp $
! $Author: wangzzh $
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!------------------------------Arguments--------------------------------
!
! Input arguments
!
   integer, intent(in) :: pf                   ! dimension (number of fields)
   integer, intent(in) :: pkcnst               ! dimension,=p3d
   integer, intent(in) :: plon                 !
   integer, intent(in) :: beglatex, endlatex   !
   integer, intent(in) :: plev                 !
   integer, intent(in) :: jcen                 !
!  integer, intent(in) :: jcen, ng             !
!
   real(r8), intent(in) :: fb (plon,plev,beglatex:endlatex,pkcnst) ! field
   real(r8), intent(in) :: fxl(plon,plev,beglatex:endlatex,pf)     ! left  x derivative
   real(r8), intent(in) :: fxr(plon,plev,beglatex:endlatex,pf)     ! right x derivative
   real(r8), intent(in) :: x(plon,beglatex:endlatex)      ! longitudinal grid coordinates
!
   integer, intent(in) :: idp(plon,beglatex:endlatex)     ! longitude index of dep pt.
   integer, intent(in) :: nlonex(beglatex:endlatex)     ! 
   integer, intent(in) :: nlon
!
! Output arguments
!
   real(r8), intent(out) :: fint(plon,plev,beglatex:endlatex,pf) ! x-interpolants
!
!-----------------------------------------------------------------------
!
!  pf      Number of fields being interpolated.
!  pkcnst  Dimensioning construct for 3-D arrays.
!  fb      extended array of data to be interpolated.
!  fxl     x derivatives at the left edge of each interval containing 
!          the departure point
!  fxr     x derivatives at the right edge of each interval containing 
!          the departure point
!  x       Equally spaced x grid values in extended arrays.
!  idp     idp(i,k) is the index of the x-interval (extended grid) that
!          contains the departure point corresponding to global grid
!          point (i,k) in the latitude slice being forecasted.
!          Note that
!                x(idp(i,k)) .le. xdp(i,k) .lt. x(idp(i,k)+1) .
!  fint    (fint(i,k,j,n),j=1,ppdy) contains the x interpolants at each
!          latitude needed for the y derivative estimates at the
!          endpoints of the interval that contains the departure point
!          for grid point (i,k).  The last index of fint allows for
!          interpolation of multiple fields.
!
!---------------------------Local workspace-----------------------------
!
   integer i,j,k,m           ! indices
   integer idp1
!
   real(r8) dx (beglatex:endlatex)           ! x-increment
   real(r8) rdx(beglatex:endlatex)           ! 1./dx
   real(r8) xl                   ! |
   real(r8) xr                   ! |
   real(r8) hl                   ! | --interpolation coeffs
   real(r8) hr                   ! |
   real(r8) dhl                  ! |
   real(r8) dhr                  ! |

   integer n

!
!-----------------------------------------------------------------------
!
      do j = beglatex, endlatex
         dx (j) = x(2,j)-x(1,j)
         rdx(j) = 1._r8/dx(j)
      end do
!
! x interpolation at each latitude needed for y interpolation.
! Once for each field.
! 
      do j = beglatex, endlatex
         if(nlonex(j)==nlon) then
            do m = 1,pf
            do k = 1,plev
            do i = 1,nlon
               fint(i,k,j,m) = fb(i,k,j,m)
            enddo
            enddo
            enddo
         else
         do i = 1,nlon
            if(idp(i,j)==nlonex(j)) then
            idp1 = 1
!           xr  = ( x(i, jcen) - x(idp(i,j),j))* rdx(j)
!           xl  = 1._r8 - xr
            else
            idp1 = idp(i,j) + 1
!           xl  = ( x(idp1,j) - x(i, jcen) )* rdx(j)
!           xr  = 1._r8 - xl
            endif
            xr  = ( x(i, jcen) - x(idp(i,j),j))* rdx(j)
            xl  = 1._r8 - xr
            hl  = ( 3.0_r8 - 2.0_r8*xl )*xl*xl
            hr  = ( 3.0_r8 - 2.0_r8*xr )*xr*xr
            dhl = -dx(j)*( xl - 1._r8 )*xl*xl
            dhr =  dx(j)*( xr - 1._r8 )*xr*xr
            do m = 1,pf
            do k = 1,plev
               fint(i,k,j,m) = &
                  fb (idp(i,j)  ,k,j,m)*hl  + &
                  fb (idp1,k,j,m)*hr  + &
                  fxl(idp(i,j)  ,k,j, m)*dhl + &
                  fxr(idp(i,j)  ,k,j, m)*dhr
!           if(m==2.and.i==2.and.jcen==5.and.j==jcen+1) then
!             write(*,*)'wzz: fint=', fint(i,k,j,m), idp(i,j), idp1,fb(idp(i,j),k,j,m), &
!             fb(idp1,k,j,m), fxl(idp(i,j)  ,k,j, m), fxr(idp(i,j)  ,k,j, m), hl, hr, dhl, dhr
!           endif
            end do
            end do
         end do
         end if
      end do
!
   return
end subroutine herxinw
