#if defined( UNICOSMP ) || defined ( NEC_SX )
#define VECTORIZE
#endif
module tp_core
!BOP
!
! !MODULE: tp_core --- Utilities for the transport core
!
! !USES:
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   implicit none

!
! !PUBLIC MEMBER FUNCTIONS:
      public tp2c
!     public tp2c, tp2d, xtpv, fxppm, xmist, steepx, lmppm
!     public huynh, ytp, ymist, fyppm, tpcc, ycc
!
! !DESCRIPTION:
!
!      This module provides 
!
!      \begin{tabular}{|l|l|} \hline \hline
!       tp2c  &   \\ \hline
!       tp2d  &   \\ \hline 
!       xtp  &   \\ \hline 
!       fxppm  &   \\ \hline 
!       xmist  &   \\ \hline 
!       steepx  &   \\ \hline 
!       lmppm  &   \\ \hline 
!       huynh  &   \\ \hline 
!       ytp  &   \\ \hline 
!       ymist  &   \\ \hline 
!       fyppm  &   \\ \hline 
!       tpcc  &   \\ \hline 
!       ycc  &   \\ \hline
!                                \hline
!      \end{tabular}
!
! !REVISION HISTORY:
!   01.01.15   Lin        Routines coalesced into this module
!   01.03.26   Sawyer     Additional ProTeX documentation
!   03.11.19   Sawyer     Merged in CAM changes by Mirin
!   04.10.07   Sawyer     ompinner now from dynamics_vars
!   05.03.25   Todling    shr_kind_r8 can only be referenced once (MIPSpro-7.4.2)
!   05.05.25   Sawyer     Merged CAM and GEOS5 versions (mostly CAM)
!   06.09.06   Sawyer     Turned "magic numbers" into F90 parameters
!
!EOP
!-----------------------------------------------------------------------

! Magic numbers used in this module

   private
   real(r8), parameter ::  D0_0                    =  0.0_r8
   real(r8), parameter ::  D0_05                   =  0.05_r8
   real(r8), parameter ::  D0_25                   =  0.25_r8
   real(r8), parameter ::  D0_5                    =  0.5_r8
   real(r8), parameter ::  D1_0                    =  1.0_r8
   real(r8), parameter ::  D2_0                    =  2.0_r8
   real(r8), parameter ::  D3_0                    =  3.0_r8
   real(r8), parameter ::  D4_0                    =  4.0_r8
   real(r8), parameter ::  D8_0                    =  8.0_r8
   real(r8), parameter ::  D12_0                   = 12.0_r8
   real(r8), parameter ::  D24_0                   = 24.0_r8

CONTAINS

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: tp2c --- Perform transport on a C grid
!
! !INTERFACE: 
 subroutine tp2c(dh, va, h, crx, cry, im, jm, nq,                        &
                 iord, jord, ng, fx, fy, ffsl, ffsly,                    &
                 rcap, acosp, xfx, yfx, cosp, id, jfirst, jlast, nx)
!-----------------------------------------------------------------------

! !INPUT PARAMETERS:
   integer, intent(in) :: im, jm                  ! Dimensions
   integer, intent(in) :: nq                      ! Tracers
   integer, intent(in) :: nx                      !
   integer, intent(in) :: jfirst, jlast           ! Latitude strip
   integer, intent(in) :: iord, jord              ! Interpolation order in x,y
   integer, intent(in) :: ng                      ! Max. NS dependencies
   integer, intent(in) :: id                      ! density (0)  (mfx = C)
   real(r8), intent(in) :: rcap                  ! Ask S.-J. (polar constant?)
   real(r8), intent(in) :: acosp(jfirst:jlast)             ! Ask S.-J. (difference to cosp??)
!  real (r8) acosp(jm)             ! Ask S.-J. (difference to cosp??)
!  logical ffsl(jm)                ! Use flux-form semi-Lagrangian trans.?
   logical ffsl(jfirst-ng:jlast+ng)                ! Use flux-form semi-Lagrangian trans.?
   logical ffsly(jfirst-ng:jlast+ng)               ! Use flux-form semi-Lagrangian trans.?
                                      ! (N*NG S*NG)
   real(r8) cosp(jfirst-ng:jlast+ng)                   ! Critical angle
!  real(r8) cosp(jm)                   ! Critical angle
   real(r8) va(im,jfirst:jlast)        ! Courant  (unghosted)
   real(r8) h(im,jfirst-ng:jlast+ng,nq)   ! Pressure ( N*NG S*NG )
   real(r8) crx(im,jfirst-ng:jlast+ng) ! Ask S.-J. ( Courant X N*NG S*NG )
   real(r8) cry(im,jfirst:jlast+1)     ! Ask S.-J. ( Courant Y N like FY )
   real(r8) xfx(im,jfirst:jlast)       ! Ask S.-J. ( In Flux X unghosted like FX )
   real(r8) yfx(im,jfirst:jlast+1)     ! Ask S.-J. ( In Flux Y N like FY )

! !OUTPUT PARAMETERS:
   real(r8) dh(im,jfirst:jlast,nq)        ! Ask S.-J. ( Tendency unghosted )
   real(r8) fx(im,jfirst:jlast)        ! Flux in x ( Out Flux X unghosted )
   real(r8) fy(im,jfirst:jlast+1)      ! Flux in y ( Out Flux Y N, see tp2c )

! !DESCRIPTION:
!     Perform transport on a C grid.   The number of ghost
!     latitudes (NG) depends on what method (JORD) will be used
!     subsequentally.    NG is equal to MIN(ABS(JORD),3).
!     Ask S.-J. how exactly this differs from TP2C.
!
! !REVISION HISTORY:
!
!EOP
!-----------------------------------------------------------------------
!BOC
   integer i, j, iq, js2g0, jn2g0
   real(r8)  sum1

!  js2g0  = max(2,jfirst)          !  No ghosting
!  jn2g0  = min(jm-1,jlast)        !  No ghosting
   js2g0  = max(1,jfirst)          !  No ghosting
   jn2g0  = min(jm,jlast)        !  No ghosting

!  if(id==1 .and. jfirst==17) then
!  write(*,*)'wzz: js2g0, jn2g0 in tp2c=', js2g0, jn2g0, jfirst, jlast, id, ng, h(2,19,1), va(2,18), &
!             fy(2,19), crx(3,18),crx(3,19)
!  endif
   do iq =1, nq
   call tp2d(va(1:nx,jfirst:jlast), h(1:nx,jfirst-ng:jlast+ng,iq), &
             crx(1:nx,jfirst-ng:jlast+ng), cry(1:nx,jfirst:jlast+1), nx, jm,  &
             iord, jord, ng,fx(1:nx,jfirst:jlast), fy(1:nx,jfirst:jlast+1), ffsl, ffsly,   &
             xfx(1:nx,jfirst:jlast), yfx(1:nx,jfirst:jlast+1), cosp, id, jfirst, jlast)

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
   do j=js2g0,jn2g0
      do i=1,nx-1
         dh(i,j,iq)  = fx(i,j)  - fx(i+1,j) + (fy(i,j)-fy(i,j+1))*acosp(j)
      enddo
      dh(nx,j,iq) = fx(nx,j) - fx(1,j)   + (fy(nx,j)-fy(nx,j+1))*acosp(j)
   enddo

! Poles
   if ( jfirst ==  0 ) then
!       sum1 = - SUM( fy(1:im, 1) ) * rcap
        sum1 = D0_0
        do i=1,nx
          sum1 = sum1 + fy(i,1)
        enddo
          sum1 = -sum1*rcap
        do i=1,nx
          dh(i, 0,iq) = sum1
        enddo
   endif
   
   if ( jlast == jm+1 ) then
!       sum1 = SUM( fy(1:im,jm+1) ) * rcap
        sum1 = D0_0
        do i=1,nx
          sum1 = sum1 + fy(i,jm+1)
        enddo
          sum1 = sum1*rcap
        do i=1,nx
          dh(i,jm+1,iq) = sum1
        enddo
   endif
   end do

   return
!EOC
 end subroutine tp2c
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: tp2d --- Perform transport on a D grid
!
! !INTERFACE: 
 subroutine tp2d(va, q, crx, cry, im, jm, iord, jord, ng, fx, fy,        &
                 ffsl, ffsly, xfx, yfx, cosp, id, jfirst, jlast)
!-----------------------------------------------------------------------
! !USES:

! !INPUT PARAMETERS:
   integer, intent(in) :: im, jm                    ! Dimensions
   integer jfirst, jlast             ! Latitude strip
   integer iord, jord                ! Interpolation order in x,y
   integer ng                        ! Max. NS dependencies
   integer id                        ! density (0)  (mfx = C)
                                     ! mixing ratio (1) (mfx = mass flux)
!  logical ffsl(jm)                  ! Use flux-form semi-Lagrangian trans.?
   logical ffsl(jfirst-ng:jlast+ng)                  ! Use flux-form semi-Lagrangian trans.?
   logical ffsly(jfirst-ng:jlast+ng)                 ! Use flux-form semi-Lagrangian trans.?
                                     ! ghosted N*ng S*ng
!  real(r8) cosp(jm)                     ! Critical angle
   real(r8) cosp(jfirst-ng:jlast+ng)                     ! Critical angle
   real(r8) va(im,jfirst:jlast)          ! Courant  (unghosted)
   real(r8) q(im,jfirst-ng:jlast+ng)     ! transported scalar ( N*NG S*NG )
   real(r8) crx(im,jfirst-ng:jlast+ng)   ! Ask S.-J. ( N*NG S*NG )
   real(r8) cry(im,jfirst:jlast+1)       ! Ask S.-J. ( N like FY )
   real(r8) xfx(im,jfirst:jlast)         ! Ask S.-J. ( unghosted like FX )
   real(r8) yfx(im,jfirst:jlast+1)       ! Ask S.-J. ( N like FY )

! !OUTPUT PARAMETERS:
   real(r8) fx(im,jfirst:jlast)          ! Flux in x ( unghosted )
   real(r8) fy(im,jfirst:jlast+1)        ! Flux in y ( N, see tp2c )

! !DESCRIPTION:
!     Perform transport on a D grid.   The number of ghost
!     latitudes (NG) depends on what method (JORD) will be used
!     subsequentally.    NG is equal to MIN(ABS(JORD),3).
!
!
! !REVISION HISTORY:
!   WS  99.04.13:  Added jfirst:jlast concept
!       99.04.21:  Removed j1 and j2 (j1=2, j2=jm-1 consistently)
!       99.04.27:  Removed dc, wk2 as arguments (local to YTP)
!       99.04.27:  Removed adx as arguments (local here)
!   SJL 99.07.26:  ffsl flag added
!   WS  99.09.07:  Restructuring, cleaning, documentation
!   WS  99.10.22:  NG now argument; arrays pruned
!   WS  00.05.14:  Renamed ghost indices as per Kevin's definitions
!
!EOP
!---------------------------------------------------------------------
!BOC

! Local:
   integer i, j, iad, jp, js2g0, js2gng, jn2g0, jn2gng
   integer i2
   real(r8) adx(im,jfirst-ng:jlast+ng)
   real(r8) wk1v(im,jfirst-ng:jlast+ng)
   real(r8)   dm(-3:im+3)
   real(r8) qtmpv(-3:im+3)
   real(r8)   al(-3:im+3)
   real(r8)   ar(-3:im+3)
   real(r8)   a6(-3:im+3)

! Number of ghost latitudes
!   js2g0  = max(2,jfirst)          !  No ghosting
!   js2gng = max(2,jfirst-ng)       !  Number needed on S
!   jn2g0  = min(jm-1,jlast)        !  No ghosting
!   jn2gng = min(jm-1,jlast+ng)     !  Number needed on N
    js2g0  = max(1,jfirst)          !  No ghosting
    js2gng = max(1,jfirst-ng)       !  Number needed on S
    jn2g0  = min(jm,jlast)        !  No ghosting
    jn2gng = min(jm,jlast+ng)     !  Number needed on N

    iad = 1
    call xtpv(im,  ffsl, wk1v, q, crx, iad, crx,        &
             cosp, 0, dm, qtmpv, al, ar, a6,            &
             jfirst, jlast, js2gng, jn2gng, jm,         &
             1, jm, jfirst-ng, jlast+ng,                &
             jfirst-ng, jlast+ng, jfirst-ng, jlast+ng,  &
             jfirst-ng, jlast+ng, jfirst-ng, jlast+ng)

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
    do j=js2gng,jn2gng               !  adx needed on N*ng S*ng

       do i=1,im-1
          adx(i,j) = q(i,j) + D0_5 *                       &
                     (wk1v(i,j)-wk1v(i+1,j) + q(i,j)*(crx(i+1,j)-crx(i,j)))
       enddo
          adx(im,j) = q(im,j) + D0_5 *                     &
                      (wk1v(im,j)-wk1v(1,j) + q(im,j)*(crx(1,j)-crx(im,j)))
    enddo

! WS 99.09.07 : Split up north and south pole

!    if ( jfirst-ng <= 1 ) then
     if ( jfirst-ng <= 0 ) then
        do i=1,im 
          adx(i, 0) = q(i,0)
        enddo
!  wzz, 2012.11.28
        do j = jfirst-ng, -1
        do i=1,im 
          i2 = i+im/2
          if(i2>im) i2=i2-im
          adx(i,j) = adx(i2,0-j)
        enddo
        enddo
     endif 
!    if ( jlast+ng >= jm ) then
     if ( jlast+ng >= jm+1 ) then
        do i=1,im 
          adx(i,jm+1) = q(i,jm+1)
        enddo
!  wzz, 2012.11.28
        do j = jm+2, jlast+ng
        do i=1,im 
          i2 = i+im/2
          if(i2>im) i2=i2-im
          adx(i,j) = adx(i2,jm+1+jm+1-j)
        enddo
        enddo
     endif

     call ytp(im,jm,ffsly,fy, adx,cry,yfx,ng,jord,0,jfirst,jlast)

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i,jp,i2)
!!!#endif
      do j=js2g0,jn2g0
       if(ffsly(j)) then
        do i=1,im
           i2 = int(va(i,j))
           jp = j-i2
           if(jp-1<jfirst-ng.or.jp+1>jlast+ng) write(*,*)'wzz: jp=', jp, j, i, va(i,j)
           if(va(i,j) >=0. ) then
           wk1v(i,j) = q(i,jp) - q(i,j) + (va(i,j)-i2)*(q(i,jp-1)-q(i,jp))
           else
           wk1v(i,j) = q(i,jp) - q(i,j) + (va(i,j)-i2)*(q(i,jp)-q(i,jp+1))
           endif
!
           wk1v(i,j) = q(i,j) +D0_5*wk1v(i,j)
        enddo
       else
        do i=1,im
           jp = j-va(i,j)
           wk1v(i,j) = q(i,j) +D0_5*va(i,j)*(q(i,jp)-q(i,jp+1))
        enddo
       endif
      enddo

      call xtpv(im,  ffsl, fx, wk1v, crx, iord, xfx,        &
               cosp, id, dm, qtmpv, al, ar, a6,             &
               jfirst, jlast, js2g0, jn2g0, jm,             &
               1, jm, jfirst, jlast,                        &
               jfirst-ng, jlast+ng, jfirst-ng, jlast+ng,    &
               jfirst, jlast, jfirst-ng, jlast+ng)

    return
!EOC
 end subroutine tp2d
!-----------------------------------------------------------------------

#ifndef VECTORIZE
!-----------------------------------------------------------------------
!BOP
! !IROUTINE: xtpv
!
! !INTERFACE: 
 subroutine xtpv(im, ffslv,  fxv,  qv,  cv,  iord,  mfxv,        &
                cosav, id, dm, qtmpv, al, ar, a6,            &
                jfirst, jlast, jlow, jhigh, jm,                  &
                jl2, jh2, jl3, jh3,                              &
                jl4, jh4, jl5, jh5,                              &
                jl7, jh7, jl11, jh11)
!-----------------------------------------------------------------------

! !INPUT PARAMETERS:
   integer id               ! ID = 0: density (mfx = C)
                            ! ID = 1: mixing ratio (mfx is mass flux)

   integer im               ! Total longitudes
   integer iord
   integer jfirst, jlast, jlow, jhigh, jm
   integer jl2, jh2, jl3, jh3, jl4, jh4, jl5, jh5
   integer jl7, jh7, jl11, jh11 
   real(r8) cv(im,jl5:jh5)          ! Courant numbers
   real(r8) qv(im,jl4:jh4)
   real(r8) mfxv(im,jl7:jh7)
!  logical ffslv(jl2:jh2)
   logical ffslv(jl5:jh5)
   real(r8) cosav(jl4:jh4)

! !INPUT/OUTPUT PARAMETERS:
   real(r8) dm(-3:im+3)
   real(r8) qtmpv(-3:im+3)   ! Input work arrays:
   real(r8) al(-3:im+3)
   real(r8) ar(-3:im+3)
   real(r8) a6(-3:im+3)

! !OUTPUT PARAMETERS:
   real(r8) fxv(im,jl3:jh3)

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC

! Local:
   real(r8)       cos_upw               !critical cosine for upwind
   real(r8)       cos_van               !critical cosine for van Leer
   real(r8)       cos_ppm               !critical cosine for ppm

   parameter (cos_upw = D0_05)       !roughly at 87 deg.
!  parameter (cos_van = D0_25)       !roughly at 75 deg.
!  parameter (cos_ppm = D0_25)
   parameter (cos_van = 0.170)       !roughly at 75 deg.
   parameter (cos_ppm = 0.170)

   integer i, imp, j
   real(r8) qmax, qmin
   real(r8) rut, tmp
   integer iu, itmp, ist
   integer isave(im)
   integer iuw, iue

   imp = im + 1
   iuw =  -1              ! wzz, 2012.12.4
   iue = im + 2       ! wzz, 2012.12.4

!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i,iuw,iue,iu,itmp,isave,tmp,qmax,qmin,dm,rut,ist,al,ar,a6)
!#endif
  do j = jlow, jhigh

   do i=1,im
      qtmpv(i) = qv(i,j)
   enddo
   qtmpv(imp) = qv(1,j)
   qtmpv(  0) = qv(im,j)
   qtmpv(-1)    = qv(im-1,j)
   qtmpv(imp+1) = qv(2,j)

   if( ffslv(j) ) then
! Flux-Form Semi-Lagrangian (FFSL) transport

! Figure out ghost zone for the western edge:
!     iuw =  -cv(1,j)
!     iuw =  -im/3           ! wzz, 2012.12.4
!     iuw =  -1              ! wzz, 2012.12.4
!     iuw = min(0, iuw)
 
! Figure out ghost zone for the eastern edge:
!     iue = im - cv(im,j)
!     iue = im + im/3    ! wzz, 2012.12.4
!     iue = im + 2       ! wzz, 2012.12.4
!     iue = max(imp, iue)

! To compute the fractional fluxes first.
      if( iord == 1 .or. cosav(j) < cos_upw) then
        do i=1,im
          iu = cv(i,j)
          if(cv(i,j) .le. D0_0) then
            itmp = i - iu
            isave(i) = itmp - 1
          else
            itmp = i - iu - 1
            isave(i) = itmp + 1
          endif
          itmp = mod(itmp+im,im)
          fxv(i,j) = (cv(i,j)-iu) * qtmpv(itmp)   ! 1st-order upwind scheme here.
        enddo
      else

        do i=1,im
! 2nd order slope
           tmp = D0_25*(qtmpv(i+1) - qtmpv(i-1))
           qmax = max(qtmpv(i-1), qtmpv(i), qtmpv(i+1)) - qtmpv(i)
           qmin = qtmpv(i) - min(qtmpv(i-1), qtmpv(i), qtmpv(i+1))
           dm(i) = sign(min(abs(tmp),qmax,qmin), tmp)
        enddo

        do i=iuw, 0
           dm(i) = dm(im+i)
        enddo 

        do i=imp, iue
           dm(i) = dm(i-im)
        enddo

        if(iord .ge. 3 .and. cosav(j) .gt. cos_ppm) then
           call fxppm(im, cv(:,j), mfxv(:,j), qtmpv, dm, fxv(:,j), iord, al, ar, a6,  &
                      iuw, iue, ffslv(j), isave)
        else
          do i=1,im
            iu  = cv(i,j)
            rut = cv(i,j) - iu
           if(cv(i,j) .le. D0_0) then
              itmp = i - iu
              isave(i) = itmp - 1
              itmp = mod(itmp,im)
              fxv(i,j) = rut*(qtmpv(itmp)-dm(itmp)*(D1_0+rut))
           else
              itmp = i - iu - 1
              isave(i) = itmp + 1
              itmp = mod(itmp+im,im)
              fxv(i,j) = rut*(qtmpv(itmp)+dm(itmp)*(D1_0-rut))
           endif
          enddo
        endif

      endif

! To compute the integer fluxes.
      do i=1,im
      if(cv(i,j) .ge. D1_0) then
        do ist = isave(i),i-1
           itmp = mod(ist+im,im)
           fxv(i,j) = fxv(i,j) + qtmpv(itmp)
!          fxv(i,j) = fxv(i,j) + qtmpv(ist)
        enddo
      elseif(cv(i,j) .le. -D1_0) then
        do ist = i,isave(i)
!          fxv(i,j) = fxv(i,j) - qtmpv(ist)   !  ????
           itmp = mod(ist,im)
           fxv(i,j) = fxv(i,j) - qtmpv(itmp)   !  ????
        enddo
      endif
      enddo

      if(id .ne. 0) then
         do i=1,im
            fxv(i,j) =  fxv(i,j)*mfxv(i,j)
         enddo
      endif

   else
! Regular PPM (Eulerian without FFSL extension)

!     if(iord == 1 .or. cosav(j) < cos_upw) then
      if(iord == 1 ) then
         do i=1,im
            iu = real(i,r8) - cv(i,j)
            fxv(i,j) = mfxv(i,j)*qtmpv(iu)    ! 1st-order upwind scheme
         enddo
      else

!        if(iord > 0 .or. cosav(j) < cos_van) then
         if(iord > 0 .or. abs(cosav(j)) < cos_van) then
!           call xmist(im, qtmpv(:,j), dm, 2)
            call xmist(im, qtmpv, dm, iord)
         else
            call xmist(im, qtmpv, dm, iord)
         endif

         dm(0) = dm(im)
         dm(imp) = dm(1)

!        if( abs(iord).eq.2 .or. cosav(j) .lt. cos_van ) then
         if( abs(iord).eq.2 .or. abs(cosav(j)) .lt. cos_van ) then
            do i=1,im
               iu = real(i,r8) - cv(i,j)
               fxv(i,j) =  mfxv(i,j)*qtmpv(iu)   ! wzz for test 2013.3.11
!              fxv(i,j) =  mfxv(i,j)*(qtmpv(iu)+dm(iu)*(sign(D1_0,cv(i,j))-cv(i,j)))

!              if(cv(i,j) .le. 0.) then
!                 fxv(i,j) = qtmpv(i) - dm(i)*(1.+cv(i,j))
!              else
!                 fxv(i,j) = qtmpv(i-1) + dm(i-1)*(1.-cv(i,j))
!              endif
!                 fxv(i,j) = fxv(i,j)*mfxv(i,j)

            enddo
         else
            call fxppm(im, cv(:,j), mfxv(:,j), qtmpv, dm, fxv(:,j), iord, al, ar, a6,       &
                       iuw, iue, ffslv(j), isave)
         endif
      endif

   endif

  enddo

   return
!EOC
 end subroutine xtpv
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: xmist
!
! !INTERFACE: 
 subroutine xmist(im,  q,  dm,  id)
!-----------------------------------------------------------------------

! !INPUT PARAMETERS:
 integer im                   ! Total number of longitudes
 integer id                   ! ID = 0: density (mfx = C)
                              ! ID = 1: mixing ratio (mfx is mass flux)
 real(r8)  q(-3:im+3)   ! Input latitude 

! !OUTPUT PARAMETERS:
 real(r8) dm(-3:im+3)   ! 

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC

 real(r8) r24
 parameter( r24 = D1_0/D24_0)

 integer i
 real(r8) qmin, qmax

    if(id .le. 2) then
       do i=1,im
          dm(i) = r24*(D8_0*(q(i+1) - q(i-1)) + q(i-2) - q(i+2))
       enddo
    else
       do i=1,im
          dm(i) = D0_25*(q(i+1) - q(i-1))
       enddo
    endif

    if( id < 0 ) return

! Apply monotonicity constraint (Lin et al. 1994, MWR)
      do i=1,im
         qmax = max( q(i-1), q(i), q(i+1) ) - q(i)
         qmin = q(i) - min( q(i-1), q(i), q(i+1) )
         dm(i) = sign( min(abs(dm(i)), qmax, qmin), dm(i) )
      enddo
  return
!EOC
 end subroutine xmist
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: fxppm
!
! !INTERFACE: 
 subroutine fxppm(im, c, mfx,  p, dm, fx, iord, al, ar, a6,        &
                  iuw, iue, ffsl, isave)
!-----------------------------------------------------------------------
!
! !INPUT PARAMETERS:
 integer im, iord
 real(r8)  c(im)
 real(r8) p(-3:im+3)
 real(r8) dm(-3:im+3)
 real(r8) mfx(im)
 integer iuw, iue
 logical ffsl
 integer isave(im)

! !INPUT/OUTPUT PARAMETERS:
 real(r8) al(-3:im+3)
 real(r8) ar(-3:im+3)
 real(r8) a6(-3:im+3)

! !OUTPUT PARAMETERS:
 real(r8) fx(im)

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 real(r8) r3, r23
 parameter ( r3 = D1_0/D3_0, r23 = D2_0/D3_0 )

 integer i, lmt
 integer iu, itmp
 real(r8) ru
 logical steep

  if( iord == 6 ) then
      steep = .true.
  else
      steep = .false.
  endif

  do i=1,im
     al(i) = D0_5*(p(i-1)+p(i)) + (dm(i-1) - dm(i))*r3
  enddo

  if( steep ) call steepx( im, p, al(1), dm )

     do i=1,im-1
        ar(i) = al(i+1)
     enddo
        ar(im) = al(1)

  if(iord == 7) then
     call huynh(im, ar(1), al(1), p(1), a6(1), dm(1))
  else
     if(iord .eq. 3 .or. iord .eq. 5) then
         do i=1,im
            a6(i) = D3_0*(p(i)+p(i)  - (al(i)+ar(i)))
         enddo
     endif
     lmt = iord - 3
     call lmppm( dm(1), a6(1), ar(1), al(1), p(1), im, lmt )
  endif

  if( ffsl ) then

      do i=iuw, 0
         al(i) = al(im+i)
         ar(i) = ar(im+i)
         a6(i) = a6(im+i)
      enddo

      do i=im+1, iue
         al(i) = al(i-im)
         ar(i) = ar(i-im)
         a6(i) = a6(i-im)
      enddo

      do i=1,im
            iu = c(i)
            ru = c(i) - iu
         if(c(i) .gt. D0_0) then
            itmp = i - iu - 1
            isave(i) = itmp + 1
            itmp = mod(itmp+im,im)
            fx(i) = ru*(ar(itmp)+D0_5*ru*(al(itmp)-ar(itmp) +     &
                        a6(itmp)*(D1_0-r23*ru)) )
         else
            itmp = i - iu
            isave(i) = itmp - 1
            itmp = mod(itmp,im)
            fx(i) = ru*(al(itmp)-D0_5*ru*(ar(itmp)-al(itmp) +     &
                        a6(itmp)*(D1_0+r23*ru)) )
         endif
      enddo

  else
         al(0) = al(im)
         ar(0) = ar(im)
         a6(0) = a6(im)
      do i=1,im
         if(c(i) .gt. D0_0) then
            fx(i) = ar(i-1) + D0_5*c(i)*(al(i-1) - ar(i-1) +   &
                    a6(i-1)*(D1_0-r23*c(i)) )
!          if(i==9.and.fx(i)<-100.) then
!            write(*,*)'wzz: fx in fxppm=', fx(i),ar(i-1),al(i-1),a6(i-1)
!          endif
         else
            fx(i) = al(i) - D0_5*c(i)*(ar(i) - al(i) +         &
                    a6(i)*(D1_0+r23*c(i)))
         endif
            fx(i) = mfx(i) * fx(i)
      enddo
  endif
  return
!EOC
 end subroutine fxppm
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: steepx
!
! !INTERFACE: 
 subroutine  steepx(im, p, al, dm)
!-----------------------------------------------------------------------

! !USES:
 implicit none

! !INPUT PARAMETERS:
 integer im
 real(r8)  p(-3:im+3)
 real(r8) dm(-3:im+3)

! !INPUT/OUTPUT PARAMETERS:
 real(r8) al(im)

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 integer i
 real(r8) r3
 parameter ( r3 = D1_0/D3_0 )

 real(r8) dh(0:im)
 real(r8) d2(0:im+1)
 real(r8) eta(0:im)
 real(r8) xxx, bbb, ccc

   do i=0,im
      dh(i) = p(i+1) - p(i)
   enddo

! Needs dh(0:im)
   do i=1,im
      d2(i) = dh(i) - dh(i-1)
   enddo
   d2(0) = d2(im)
   d2(im+1) = d2(1)

! needs p(-1:im+2), d2(0:im+1)
   do i=1,im
      if( d2(i+1)*d2(i-1).lt.D0_0 .and. p(i+1).ne.p(i-1) ) then
          xxx    = D1_0 - D0_5 * ( p(i+2) - p(i-2) ) / ( p(i+1) - p(i-1) )
          eta(i) = max(D0_0, min(xxx, D0_5) )
      else
          eta(i) = D0_0
      endif
    enddo

    eta(0) = eta(im)

! needs eta(0:im), dh(0:im-1), dm(0:im)
   do i=1,im
      bbb = ( D2_0*eta(i  ) - eta(i-1) ) * dm(i-1) 
      ccc = ( D2_0*eta(i-1) - eta(i  ) ) * dm(i  ) 
      al(i) = al(i) + D0_5*( eta(i-1) - eta(i)) * dh(i-1) + (bbb - ccc) * r3
   enddo
   return
!EOC
 end subroutine steepx
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: lmppm
!
! !INTERFACE: 
 subroutine lmppm(dm, a6, ar, al, p, im, lmt)
!-----------------------------------------------------------------------

 implicit none

! !INPUT PARAMETERS:
 integer im   ! Total longitudes
 integer lmt  ! LMT = 0: full monotonicity
              ! LMT = 1: Improved and simplified full monotonic constraint
              ! LMT = 2: positive-definite constraint
              ! LMT = 3: Quasi-monotone constraint
 real(r8) p(im)
 real(r8) dm(im)

! !OUTPUT PARAMETERS:
 real(r8) a6(im)
 real(r8) ar(im)
 real(r8) al(im)

! !DESCRIPTION:
!
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 real(r8) r12
 parameter ( r12 = D1_0/D12_0 )

 real(r8) da1, da2, fmin, a6da
 real(r8) dr, dl

 integer i

! LMT = 0: full monotonicity
! LMT = 1: Improved and simplified full monotonic constraint
! LMT = 2: positive-definite constraint
! LMT = 3: Quasi-monotone constraint

  if( lmt == 0 ) then

! Full constraint
  do i=1,im
     if(dm(i) .eq. D0_0) then
         ar(i) = p(i)
         al(i) = p(i)
         a6(i) = D0_0
     else
         da1  = ar(i) - al(i)
         da2  = da1**2
         a6da = a6(i)*da1
         if(a6da .lt. -da2) then
            a6(i) = D3_0*(al(i)-p(i))
            ar(i) = al(i) - a6(i)
         elseif(a6da .gt. da2) then
            a6(i) = D3_0*(ar(i)-p(i))
            al(i) = ar(i) - a6(i)
         endif
     endif
  enddo

  elseif( lmt == 1 ) then

! Improved (Lin 2001?) full constraint
      do i=1,im
           da1 = dm(i) + dm(i)
            dl = sign(min(abs(da1),abs(al(i)-p(i))), da1)
            dr = sign(min(abs(da1),abs(ar(i)-p(i))), da1)
         ar(i) = p(i) + dr
         al(i) = p(i) - dl
         a6(i) = D3_0*(dl-dr)
      enddo

  elseif( lmt == 2 ) then
! Positive definite constraint
      do 250 i=1,im
      if(abs(ar(i)-al(i)) .ge. -a6(i)) go to 250
      fmin = p(i) + D0_25*(ar(i)-al(i))**2/a6(i) + a6(i)*r12
      if(fmin.ge.D0_0) go to 250
      if(p(i).lt.ar(i) .and. p(i).lt.al(i)) then
            ar(i) = p(i)
            al(i) = p(i)
            a6(i) = D0_0
      elseif(ar(i) .gt. al(i)) then
            a6(i) = D3_0*(al(i)-p(i))
            ar(i) = al(i) - a6(i)
      else
            a6(i) = D3_0*(ar(i)-p(i))
            al(i) = ar(i) - a6(i)
      endif
250   continue

  elseif(lmt .eq. 3) then
! Quasi-monotone constraint
      do i=1,im
         da1 = D4_0*dm(i)
          dl = sign(min(abs(da1),abs(al(i)-p(i))), da1)
          dr = sign(min(abs(da1),abs(ar(i)-p(i))), da1)
         ar(i) = p(i) + dr
         al(i) = p(i) - dl
         a6(i) = D3_0*(dl-dr)
      enddo
  endif
  return
!EOC
 end subroutine lmppm
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: huynh --- Enforce Huynh's 2nd constraint in 1D periodic domain
!
! !INTERFACE: 
 subroutine huynh(im, ar, al, p, d2, d1)
!-----------------------------------------------------------------------

! !USES:

 implicit none

! !INPUT PARAMETERS:
 integer im
 real(r8)  p(im)

! !OUTPUT PARAMETERS:
 real(r8) ar(im)
 real(r8) al(im)
 real(r8) d2(im)
 real(r8) d1(im)

! !DESCRIPTION:
!
!   Enforce Huynh's 2nd constraint in 1D periodic domain
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 integer  i
 real(r8) pmp
 real(r8) lac
 real(r8) pmin
 real(r8) pmax

! Compute d1 and d2
      d1(1) = p(1) - p(im)
      do i=2,im
         d1(i) = p(i) - p(i-1)
      enddo

      do i=1,im-1
         d2(i) = d1(i+1) - d1(i)
      enddo
      d2(im) = d1(1) - d1(im)

! Constraint for AR
!            i = 1
         pmp   = p(1) + D2_0 * d1(1)
         lac   = p(1) + D0_5 * (d1(1)+d2(im)) + d2(im) 
         pmin  = min(p(1), pmp, lac)
         pmax  = max(p(1), pmp, lac)
         ar(1) = min(pmax, max(ar(1), pmin))

      do i=2, im
         pmp   = p(i) + D2_0*d1(i)
         lac   = p(i) + D0_5*(d1(i)+d2(i-1)) + d2(i-1)
         pmin  = min(p(i), pmp, lac)
         pmax  = max(p(i), pmp, lac)
         ar(i) = min(pmax, max(ar(i), pmin))
      enddo

! Constraint for AL
      do i=1, im-1
         pmp   = p(i) - D2_0*d1(i+1)
         lac   = p(i) + D0_5*(d2(i+1)-d1(i+1)) + d2(i+1)
         pmin  = min(p(i), pmp, lac)
         pmax  = max(p(i), pmp, lac)
         al(i) = min(pmax, max(al(i), pmin))
      enddo

! i=im
         i = im
         pmp    = p(im) - D2_0*d1(1)
         lac    = p(im) + D0_5*(d2(1)-d1(1)) + d2(1)
         pmin   = min(p(im), pmp, lac)
         pmax   = max(p(im), pmp, lac)
         al(im) = min(pmax, max(al(im), pmin))

! compute A6 (d2)
      do i=1, im
         d2(i) = D3_0*(p(i)+p(i)  - (al(i)+ar(i)))
      enddo
    return
!EOC
 end subroutine huynh
!-----------------------------------------------------------------------
#endif

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: ytp
!
! !INTERFACE: 
 subroutine ytp(im, jm, ffsl, fy, q, c, yfx, ng, jord, iv, jfirst, jlast)
!-----------------------------------------------------------------------

! !USES:
 implicit none

! !INPUT PARAMETERS:
 integer, intent(in) :: im, jm                      !  Dimensions
 integer, intent(in) :: jfirst, jlast               !  Latitude strip
 integer, intent(in) :: ng                          !  Max. NS dependencies
 integer, intent(in) :: jord                        !  order of subgrid dist
 integer, intent(in) :: iv                          !  Scalar=0, Vector=1
 real(r8) q(im,jfirst-ng:jlast+ng)       !  advected scalar N*jord S*jord
 real(r8) c(im,jfirst:jlast+1)           !  Courant   N (like FY)
 real(r8) yfx(im,jfirst:jlast+1)         !  Backgrond mass flux

 logical ffsl(jfirst-ng:jlast+ng)                  ! Use flux-form semi-Lagrangian trans.?

! !OUTPUT PARAMETERS:
 real(r8) fy(im,jfirst:jlast+1)          !  Flux      N (see tp2c)

! !DESCRIPTION:
!     This routine calculates the flux FX.  The method chosen
!     depends on the order of the calculation JORD (currently
!     1, 2 or 3).  
!
! !CALLED FROM:
!     cd_core
!     tp2d
!
! !REVISION HISTORY:
!
!  SJL 99.04.13:  Delivery
!  WS  99.04.13:  Added jfirst:jlast concept
!  WS  99.04.21:  Removed j1 and j2 (j1=2, j2=jm-1 consistently)
!                 removed a6,ar,al from argument list
!  WS  99.04.27:  DM made local to this routine
!  WS  99.09.09:  Documentation; indentation; cleaning
!  WS  99.10.22:  Added NG as argument; pruned arrays
!  SJL 99.12.24:  Revised documentation; optimized for better cache usage
!  WS  00.05.14:  Renamed ghost indices as per Kevin's definitions
!
!EOP
!---------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 integer i, j, jt
 integer js2g0, jn1g1
 integer iu, jsave, jst

! work arrays (should pass in eventually for performance enhancement):
 real(r8) dm(im,jfirst-ng:jlast+ng)

!  js2g0  = max(2,jfirst)       ! No ghosting
!  jn1g1  = min(jm,jlast+1)     ! Ghost N*1
   js2g0  = max(1,jfirst)       ! No ghosting
   jn1g1  = min(jm+1,jlast+1)     ! Ghost N*1
     
   if(jord == 1) then
!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i,jt,iu,jsave,jst)
!#endif
     do j=js2g0,jn1g1
      if ( ffsl(j) ) then
       do i=1,im
          iu = c(i,j)
          if(c(i,j).le.D0_0) then
            jt = j - iu
            if ( jt>jlast+ng) then
               write(*,*)'FFSL: ytp is out of N.bounds: ', i, j, jt, c(i,j)
               call endrun
            endif
            fy(i,j) = (c(i,j)-iu)*q(i,jt)
! To compute the integer fluxes.
            jsave = jt - 1
            do jst = j, jsave
               fy(i,j) = fy(i,j) + q(i,jst)
            enddo
          else
            jt = j - iu - 1
            if ( jt<jfirst-ng) then
               write(*,*)'FFSL: ytp is out of S.bounds: ', i, j, jt, c(i,j)
               call endrun
            endif
            fy(i,j) = (c(i,j)-iu)*q(i,jt)
! To compute the integer fluxes.
            jsave = jt + 1
            do jst = jsave, j-1
               fy(i,j) = fy(i,j) + q(i,jst)
            enddo
          endif
       enddo
      else
        do i=1,im
         jt = real(j,r8) - c(i,j)
!        if(jt.lt.0 .or. jt.gt.jm+1) then
!          write(*,*)'wzz: jt=', jt,i,j,c(i,j),q(i,jt), yfx(i,j)
!        endif
         jt  = min(jt,jlast+ng)     ! Ghost N*1
         jt  = max(jt,jfirst-ng)
         fy(i,j) = q(i,jt)
       enddo
      endif
     enddo
   else

!
! YMIST requires q on NS;  Only call to YMIST here
!
     call ymist(im, jm, q, dm, ng, jord, iv, jfirst, jlast)

     if( abs(jord) .ge. 3 ) then
 
       call fyppm(c,q,dm,fy,im,jm,ng,jord,iv,jfirst,jlast, ffsl)

     else
!
! JORD can either have the value 2 or -2 at this point
!
!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i,jt)
!!!#endif
       do j=js2g0,jn1g1
         do i=1,im
           jt = real(j,r8) - c(i,j)
!  if(jt<0 .or. jt>jm+1) ???
           fy(i,j) = q(i,jt) + (sign(D1_0,c(i,j))-c(i,j))*dm(i,jt)
         enddo
       enddo
     endif
   endif

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
   do j=js2g0,jn1g1
     do i=1,im
       fy(i,j) = fy(i,j)*yfx(i,j)
     enddo
   enddo
 return
!EOC
 end subroutine ytp
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: ymist
!
! !INTERFACE: 
 subroutine ymist(im, jm, q, dm, ng, jord, iv, jfirst, jlast)
!-----------------------------------------------------------------------

! !USES:
 implicit none

! !INPUT PARAMETERS:
 integer, intent(in) :: im, jm                     !  Dimensions
 integer, intent(in) :: jfirst, jlast              !  Latitude strip
 integer, intent(in) :: ng                         !  NS dependencies
 integer, intent(in) :: jord                       !  order of subgrid distribution
 integer, intent(in) :: iv                         !  Scalar (==0) Vector (==1)
 real(r8), intent(in) :: q(im,jfirst-ng:jlast+ng)  !  transported scalar  N*ng S*ng

! !OUTPUT PARAMETERS:
 real(r8), intent(out) :: dm(im,jfirst-ng:jlast+ng)      !  Slope only N*(ng-1) S*(ng-1) used

! !DESCRIPTION:
!     Calculate the slope of the pressure.  The number of ghost
!     latitudes (NG) depends on what method (JORD) will be used
!     subsequentally.    NG is equal to MIN(ABS(JORD),3).
!
! !CALLED FROM:
!     ytp
!
! !REVISION HISTORY:
!
!  SJL 99.04.13:  Delivery
!  WS  99.04.13:  Added jfirst:jlast concept
!  WS  99.09.09:  Documentation; indentation; cleaning
!  SJL 00.01.06:  Documentation
!  WS  00.05.14:  Renamed ghost indices as per Kevin's definitions
!
!EOP
!---------------------------------------------------------------------
!BOC

! Local variables

 integer i, j, jm1, im2, js2gng1, jn2gng1
 real(r8) qmax, qmin, tmp

!   js2gng1 = max(2,   jfirst-ng+1)     !  Number needed on S
!   jn2gng1 = min(jm-1,jlast+ng-1)      !  Number needed on N
!   js2gng1 = max(1,   jfirst-ng+1)     !  Number needed on S
!   jn2gng1 = min(jm,  jlast+ng-1)      !  Number needed on N
! wzz. 2012.12.20
    js2gng1 = jfirst-ng+1     !  Number needed on S
    jn2gng1 = jlast+ng-1      !  Number needed on N

    jm1 = jm - 1
    im2 = im / 2

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
      do j=js2gng1,jn2gng1
        do i=1,im
           dm(i,j) = D0_25*(q(i,j+1) - q(i,j-1))
        enddo
      enddo

   if( iv == 0 ) then
! For ghost regions out of pole

!       if ( jfirst-ng <= 1 ) then
        if ( jfirst-ng <= 0 ) then
! S pole
          do i=1,im2
            tmp = D0_25*(q(i,1)-q(i+im2,1))
            qmax = max(q(i,1),q(i,0), q(i+im2,1)) - q(i,0)
            qmin = q(i,0) - min(q(i,1),q(i,0), q(i+im2,1))
            dm(i,0) = sign(min(abs(tmp),qmax,qmin),tmp)
          enddo

          do i=im2+1,im
            dm(i, 0) =  - dm(i-im2, 0)
          enddo
        endif

!       if ( jlast+ng >= jm ) then
        if ( jlast+ng >= jm+1 ) then
! N pole
          do i=1,im2
            tmp = D0_25*(q(i+im2,jm)-q(i,jm))
            qmax = max(q(i+im2,jm),q(i,jm+1), q(i,jm)) - q(i,jm+1)
            qmin = q(i,jm+1) - min(q(i+im2,jm),q(i,jm+1), q(i,jm))
            dm(i,jm+1) = sign(min(abs(tmp),qmax,qmin),tmp)
          enddo

          do i=im2+1,im
            dm(i,jm+1) =  - dm(i-im2,jm+1)
          enddo
        endif

   else

!       if ( jfirst-ng <= 1 ) then
        if ( jfirst-ng <= 0 ) then
! South
          do i=1,im2
            tmp  = D0_25*(q(i,1)+q(i+im2,1))
            qmax = max(q(i,1),q(i,0), -q(i+im2,1)) - q(i,0)
            qmin = q(i,0) - min(q(i,1),q(i,0),-q(i+im2,1))
            dm(i,0) = sign(min(abs(tmp),qmax,qmin),tmp)
          enddo

          do i=im2+1,im
            dm(i, 0) = dm(i-im2, 0)
          enddo
        endif

!       if ( jlast+ng >= jm ) then
        if ( jlast+ng >= jm+1 ) then
! North
          do i=1,im2
            tmp  = -D0_25*(q(i+im2,jm)+q(i,jm))
            qmax = max(-q(i+im2,jm),q(i,jm+1), q(i,jm)) - q(i,jm+1)
            qmin = q(i,jm+1) - min(-q(i+im2,jm),q(i,jm+1), q(i,jm))
            dm(i,jm+1) = sign(min(abs(tmp),qmax,qmin),tmp)
          enddo

          do i=im2+1,im
            dm(i,jm+1) = dm(i-im2,jm+1)
          enddo
        endif

   endif

!  if( jord > 0 ) then
!
! Applies monotonic slope constraint (off if jord less than zero)
!
!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i,qmax,qmin,tmp)
!!!#endif
        do j=js2gng1,jn2gng1
          do i=1,im
            qmax = max(q(i,j-1),q(i,j),q(i,j+1)) - q(i,j)
            qmin = q(i,j) - min(q(i,j-1),q(i,j),q(i,j+1))
            tmp  = dm(i,j)
            dm(i,j) = sign(min(abs(tmp),qmin,qmax),tmp)  ! wzz, 2014.12.17
!           dm(i,j) = sign(min(abs(dm(i,j)),qmin,qmax),dm(i,j))
          enddo
        enddo
!  endif
    return
!EOC
 end subroutine ymist
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: fyppm
!
! !INTERFACE: 
 subroutine fyppm(c,  q,  dm, flux, im, jm, ng, jord, iv,   &
                  jfirst, jlast, ffsl)
!-----------------------------------------------------------------------

! !USES:
 implicit none

! !INPUT PARAMETERS:
 integer im, jm                      !  Dimensions
 integer jfirst, jlast               !  Latitude strip
 integer ng                          !  Max. NS dependencies
 integer jord                        !  Approximation order
 integer iv                          !  Scalar=0, Vector=1
 real(r8)  q(im,jfirst-ng:jlast+ng) !  mean value needed only N*2 S*2
 real(r8) dm(im,jfirst-ng:jlast+ng) !  Slope     needed only N*2 S*2
 real(r8)  c(im,jfirst:jlast+1)     !  Courant   N (like FLUX)
 logical ffsl(jfirst-ng:jlast+ng)                    ! Use flux-form semi-Lagrangian trans.?

! !INPUT/OUTPUT PARAMETERS:
 real(r8) ar(im,jfirst-ng:jlast+ng)   ! AR needs to be ghosted on NS
 real(r8) al(im,jfirst-ng:jlast+ng)   ! AL needs to be ghosted on N2S
 real(r8) a6(im,jfirst-ng:jlast+ng)   ! A6 needs to be ghosted on NS

! !OUTPUT PARAMETERS:
 real(r8) flux(im,jfirst:jlast+1)   !  Flux      N (see tp2c)

! !DESCRIPTION:
!
!   NG is passed from YTP for convenience -- it is actually 1 more in NS
!   than the actual number of latitudes needed here.  But in the shared-memory 
!   case it becomes 0, which is much cleaner.
!
! !CALLED FROM:
!      ytp
!
! !REVISION HISTORY:
!
!  SJL 99.04.13:  Delivery
!  WS  99.04.19:  Added jfirst:jlast concept; FYPPM only called from YTP
!  WS  99.04.21:  Removed j1, j2  (j1=2, j2=jm-1 consistently)
!                 removed a6,ar,al from argument list
!  WS  99.09.09:  Documentation; indentation; cleaning
!  WS  99.10.22:  Added ng as argument; Pruned arrays
!  WS  00.05.14:  Renamed ghost indices as per Kevin's definitions
!
!EOP
!---------------------------------------------------------------------
!BOC
 real(r8)   r3, r23
 real(r8)   ru
 parameter ( r3 = D1_0/D3_0, r23 = D2_0/D3_0 )
 integer i, j, imh, jm1, lmt
 integer js1g1, js2g0, js2g1, jn1g2, jn1g1, jn2g1
 integer jan, jlow, jhigh, ilow, ihigh
 integer ja(jlast-jfirst+3)
 integer iu, jtmp, jsave, jst
 integer im2
!     logical steep

!     if(jord .eq. 6) then
!        steep = .true.
!     else
!        steep = .false.
!     endif

      imh = im / 2
      jm1 = jm - 1

!     js1g1  = max(1,jfirst-1)         ! Ghost S*1
!     js2g0  = max(2,jfirst)           ! No ghosting
!     js2g1  = max(2,jfirst-1)         ! Ghost S*1
!     jn1g1  = min(jm,jlast+1)         ! Ghost N*1
!     jn1g2  = min(jm,jlast+2)         ! Ghost N*2
!     jn2g1  = min(jm-1,jlast+1)       ! Ghost N*1
      js1g1  = max(0,jfirst-1)         ! Ghost S*1
      js2g0  = max(1,jfirst)           ! No ghosting
      js2g1  = max(1,jfirst-1)         ! Ghost S*1
      jn1g1  = min(jm+1,jlast+1)         ! Ghost N*1
      jn1g2  = min(jm+1,jlast+2)         ! Ghost N*2
      jn2g1  = min(jm,jlast+1)       ! Ghost N*1

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
!     do j=js2g1,jn1g2                 ! AL needed N2S
      do j=jfirst-ng+1,jlast+ng                 ! AL needed N2S
        do i=1,im                      ! P, dm ghosted N2S2 (at least)
          al(i,j) = D0_5*(q(i,j-1)+q(i,j)) + r3*(dm(i,j-1) - dm(i,j))
        enddo
      enddo
! exchange the s. boundaty of al
      j = jfirst-ng
      if(j<1) then
        do i =1, im
           im2 = i+imh
           if(im2>im) im2=i-im
           al(i,j) = al(im2, -j+1)
        enddo
      endif

! Yeh's steepening procedure; to be implemented
!     if(steep) call steepy(im,   jm,   jfirst,   jlast,       &
!                           ng,    q,       al,   dm )

!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
!     do j=js1g1,jn2g1                 ! AR needed NS
      do j=jfirst-ng,jlast+ng-1      ! AR needed NS
        do i=1,im
          ar(i,j) = al(i,j+1)          ! AL ghosted N2S
        enddo
      enddo
! exchange the N. boundaty of ar
      j = jlast+ng
      if(j>jm-1) then
        do i = 1, im
           im2 = i+imh
           if(im2>im) im2=i-im
           ar(i,j) = ar(im2,jm+jm+1-j)   !
        enddo
      endif

! WS 990726 :  Added condition to decide if poles are on this processor

! Poles:

   if( iv == 0 ) then

        if ( jfirst == 1 ) then
          do i=1,imh
            al(i,    0) = al(i+imh,1)
            al(i+imh,0) = al(i,    1)
          enddo
        endif

        if ( jlast == jm ) then
          do i=1,imh
            ar(i,    jm+1) = ar(i+imh,jm)
            ar(i+imh,jm+1) = ar(i,    jm)
          enddo
        endif

   else

        if ( jfirst == 1 ) then
          do i=1,imh
            al(i,    0) = -al(i+imh,1)
            al(i+imh,0) = -al(i,    1)
          enddo
        endif

        if ( jlast == jm ) then
          do i=1,imh
            ar(i,    jm+1) = -ar(i+imh,jm)
            ar(i+imh,jm+1) = -ar(i,    jm)
          enddo
        endif

   endif

!  for test here, wzz
     js1g1 = jfirst-ng
     jn1g1 = jlast+ng
!
   if( jord == 3 .or. jord == 5 ) then
!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i)
!!!#endif
      do j=js1g1,jn1g1               ! A6 needed NS
        do i=1,im
          a6(i,j) = D3_0*(q(i,j)+q(i,j) - (al(i,j)+ar(i,j)))
        enddo
      enddo
   endif

      lmt = jord - 3

!       do j=js1g1,jn1g1             !  A6, AR, AL needed NS
!         call lmppm(dm(1,j),a6(1,j),ar(1,j),al(1,j),q(1,j),im,lmt)
!       enddo

#ifdef VECTORIZE
        jan = 1
        ja(1) = 1
        ilow = 1
        ihigh = im*(jn1g1-js1g1+1)
        jlow = 1
        jhigh = 1
        call lmppmv(dm(1,js1g1), a6(1,js1g1), ar(1,js1g1),               &
                   al(1,js1g1),  q(1,js1g1), im*(jn1g1-js1g1+1), lmt,    &
                   jan, ja, ilow, ihigh, jlow, jhigh, jlow, jhigh)
#else
        call lmppm(dm(1,js1g1), a6(1,js1g1), ar(1,js1g1),               &
                   al(1,js1g1),  q(1,js1g1), im*(jn1g1-js1g1+1), lmt)
#endif

      jn1g1  = min(jm+1,jlast+1)         ! Ghost N*1
!!!#if defined(INNER_OMP)
!!!$omp parallel do default(shared) private(j,i,iu,jtmp,jsave,jst,ru)
!!!#endif
      do j=js2g0,jn1g1                 ! flux needed N
       if ( ffsl(j) ) then
        do i=1,im
          iu = c(i,j)    ! for FFSL
          ru = c(i,j) - iu
          if(c(i,j).gt.D0_0) then
            jtmp = j-iu-1
            jsave = jtmp + 1
!           if(jtmp<jfirst-ng) then
!             write(*,*)'Out of the S. boundary in fyppm', i, j, jtmp, jfirst-ng,c(i,j)
!             call endrun
!           endif
            flux(i,j) = ru*(ar(i,jtmp) + D0_5*ru*(al(i,jtmp) - ar(i,jtmp) +  &
                        a6(i,jtmp)*(D1_0-r23*ru) ))
          else
            jtmp = j-iu
            jsave = jtmp - 1
!           if(jtmp>jlast+ng) then
!             write(*,*)'Out of the N. boundary in fyppm', i, j, jtmp, jlast+ng,c(i,j)
!             call endrun
!           endif
            flux(i,j) = ru*(al(i,jtmp) - D0_5*ru*(ar(i,jtmp) - al(i,jtmp) +   &
                        a6(i,jtmp)*(D1_0+r23*ru)))
          endif
! To compute the integer fluxes.
          if(c(i,j).ge.D1_0) then
            do jst = jsave, j-1
               flux(i,j) = flux(i,j) + q(i,jst)
            enddo
          else if(c(i,j).le.-D1_0) then
            do jst = j, jsave
               flux(i,j) = flux(i,j) - q(i,jst)
            enddo
          endif
        enddo
       else
        do i=1,im
          if(c(i,j).gt.D0_0) then
            flux(i,j) = ar(i,j-1) + D0_5*c(i,j)*(al(i,j-1) - ar(i,j-1) +  &
                        a6(i,j-1)*(D1_0-r23*c(i,j)) )
          else
            flux(i,j) = al(i,j) - D0_5*c(i,j)*(ar(i,j) - al(i,j) +        &
                        a6(i,j)*(D1_0+r23*c(i,j)))
          endif
        enddo
       endif
      enddo
!
    return
!EOC
 end subroutine fyppm 
!-----------------------------------------------------------------------

#ifdef VECTORIZE
!-----------------------------------------------------------------------
!BOP
! !IROUTINE: xtpv
!
! !INTERFACE: 
 subroutine xtpv(im, ffslv,  fxv,  qv,  cv,  iord,  mfxv,        &
                cosav, id, dm, qtmpv, al, ar, a6,                &
                jfirst, jlast, jlow, jhigh, jm,                  &
                jl2, jh2, jl3, jh3,                              &
                jl4, jh4, jl5, jh5,                              &
                jl7, jh7, jl11, jh11)
!-----------------------------------------------------------------------

 implicit none
 
! !INPUT PARAMETERS:
   integer id               ! ID = 0: density (mfx = C)
                            ! ID = 1: mixing ratio (mfx is mass flux)

   integer im               ! Total longitudes
   real (r8) cv(im,jl5:jh5)          ! Courant numbers
   real (r8) qv(im,jl4:jh4)
   real (r8) mfxv(im,jl7:jh7)
   logical ffslv(jl2:jh2)
   integer iord
   integer jfirst, jlast, jlow, jhigh, jm
   integer jl2, jh2, jl3, jh3, jl4, jh4, jl5, jh5
   integer jl7, jh7, jl11, jh11 
   real (r8) cosav(jm)

! !INPUT/OUTPUT PARAMETERS:
   real (r8) qtmpv(-im/3:im+im/3,jl11:jh11)   ! Input work arrays:
   real (r8)   dm(-im/3:im+im/3)
   real (r8)   al(-im/3:im+im/3)
   real (r8)   ar(-im/3:im+im/3)
   real (r8)   a6(-im/3:im+im/3)

! !OUTPUT PARAMETERS:
   real (r8) fxv(im,jl3:jh3)

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC

! Local:
   real (r8)       cos_upw               !critical cosine for upwind
   real (r8)       cos_van               !critical cosine for van Leer
   real (r8)       cos_ppm               !critical cosine for ppm

   parameter (cos_upw = D0_05)       !roughly at 87 deg.
   parameter (cos_van = D0_25)       !roughly at 75 deg.
   parameter (cos_ppm = D0_25)

   real (r8) r24
   parameter (r24 = D1_0/D24_0)

   integer i, imp, j
   real (r8) qmax, qmin
   real (r8) rut, tmp
   real (r8) dmv(-im/3:im+im/3,jlow:jhigh)
   integer iu, itmp, ist
   integer isave(im,jlow:jhigh)
   integer iuwv(jlow:jhigh), iuev(jlow:jhigh)

   integer jatn, jafn, ja
   integer jat(jhigh-jlow+1), jaf(jhigh-jlow+1)
   integer jattn, jatfn, jaftn, jaffn
   integer jatt(jhigh-jlow+1), jatf(jhigh-jlow+1)
   integer jaft(jhigh-jlow+1), jaff(jhigh-jlow+1)
   integer jatftn, jatffn
   integer jatft(jhigh-jlow+1), jatff(jhigh-jlow+1)
   integer jafftn1, jafffn1
   integer jafft1(jhigh-jlow+1), jafff1(jhigh-jlow+1)
   integer jafftn2, jafffn2
   integer jafft2(jhigh-jlow+1), jafff2(jhigh-jlow+1)
   real (r8) qsum((-im/3)-1:im+im/3,jlow:jhigh)   ! work arrays


   jatn = 0
   jafn = 0
   jattn = 0
   jatfn = 0
   jaftn = 0
   jaffn = 0
   jatftn = 0
   jatffn = 0
   jafftn1 = 0
   jafffn1 = 0
   jafftn2 = 0
   jafffn2 = 0
!call ftrace_region_begin("xtpv_1")
   do j = jlow, jhigh
     if (ffslv(j)) then
        jatn = jatn + 1
        jat(jatn) = j
        if( iord == 1 .or. cosav(j) < cos_upw) then
          jattn = jattn + 1
          jatt(jattn) = j
        else
          jatfn = jatfn + 1
          jatf(jatfn) = j
          if(iord .ge. 3 .and. cosav(j) .gt. cos_ppm) then
            jatftn = jatftn + 1
            jatft(jatftn) = j
          else
            jatffn = jatffn + 1
            jatff(jatffn) = j
          endif
        endif
     else
        jafn = jafn + 1
        jaf(jafn) = j
        if( iord == 1 .or. cosav(j) < cos_upw) then
          jaftn = jaftn + 1
          jaft(jaftn) = j
        else
          jaffn = jaffn + 1
          jaff(jaffn) = j
          if(iord > 0 .or. cosav(j) < cos_van) then
            jafftn1 = jafftn1 + 1
            jafft1(jafftn1) = j
          else
            jafffn1 = jafffn1 + 1
            jafff1(jafffn1) = j
          endif
          if( abs(iord).eq.2 .or. cosav(j) .lt. cos_van ) then
            jafftn2 = jafftn2 + 1
            jafft2(jafftn2) = j
          else
            jafffn2 = jafffn2 + 1
            jafff2(jafffn2) = j
          endif
        endif
     endif
   enddo
!call ftrace_region_end("xtpv_1")

   imp = im + 1

   do j = jlow, jhigh
     do i=1,im
       qtmpv(i,j) = qv(i,j)
     enddo
   enddo

! Flux-Form Semi-Lagrangian transport

!call ftrace_region_begin("xtpv_2")
!dir$ concurrent
   do ja = 1, jatn
      j = jat(ja)

! Figure out ghost zone for the western edge:
      iuwv(j) =  -cv(1,j)
      iuwv(j) = min(0, iuwv(j))
 
      do i=iuwv(j), 0
         qtmpv(i,j) = qv(im+i,j)
      enddo 

! Figure out ghost zone for the eastern edge:
      iuev(j) = im - cv(im,j)
      iuev(j) = max(imp, iuev(j))

      do i=imp, iuev(j)
         qtmpv(i,j) = qv(i-im,j)
      enddo

   enddo
!call ftrace_region_end("xtpv_2")

!dir$ concurrent
!call ftrace_region_begin("xtpv_3")
   do ja = 1, jattn
      j = jatt(ja)

      do i=1,im
        iu = cv(i,j)
        if(cv(i,j) .le. D0_0) then
          itmp = i - iu
          isave(i,j) = itmp - 1
        else
          itmp = i - iu - 1
          isave(i,j) = itmp + 1
        endif
        fxv(i,j) = (cv(i,j)-iu) * qtmpv(itmp,j)
      enddo

   enddo
!call ftrace_region_end("xtpv_3")

!dir$ concurrent
!call ftrace_region_begin("xtpv_4")
   do ja = 1, jatfn
      j = jatf(ja)

      do i=1,im
! 2nd order slope
        tmp = D0_25*(qtmpv(i+1,j) - qtmpv(i-1,j))
        qmax = max(qtmpv(i-1,j), qtmpv(i,j), qtmpv(i+1,j)) - qtmpv(i,j)
        qmin = qtmpv(i,j) - min(qtmpv(i-1,j), qtmpv(i,j), qtmpv(i+1,j))
        dmv(i,j) = sign(min(abs(tmp),qmax,qmin), tmp)
      enddo

      do i=iuwv(j), 0
        dmv(i,j) = dmv(im+i,j)
      enddo 

      do i=imp, iuev(j)
        dmv(i,j) = dmv(i-im,j)
      enddo

   enddo
!call ftrace_region_end("xtpv_4")

   call fxppmv(im, cv, mfxv, qtmpv, dmv, fxv, iord,                     &
              iuwv, iuev, ffslv, isave, jatftn, jatft, jlow, jhigh,     &
              jl2, jh2, jl3, jh3, jl5, jh5, jl7, jh7, jl11, jh11)

!dir$ concurrent
!call ftrace_region_begin("xtpv_5")
   do ja = 1, jatffn
      j = jatff(ja)

      do i=1,im
        iu  = cv(i,j)
        rut = cv(i,j) - iu
        if(cv(i,j) .le. D0_0) then
          itmp = i - iu
          isave(i,j) = itmp - 1
          fxv(i,j) = rut*(qtmpv(itmp,j)-dmv(itmp,j)*(D1_0+rut))
        else
          itmp = i - iu - 1
          isave(i,j) = itmp + 1
          fxv(i,j) = rut*(qtmpv(itmp,j)+dmv(itmp,j)*(D1_0-rut))
        endif
      enddo

   enddo
!call ftrace_region_end("xtpv_5")

!dir$ concurrent
!call ftrace_region_begin("xtpv_6")
   do ja = 1, jatn
      j = jat(ja)
      qsum(iuwv(j)-1,j) = D0_0
      do i = iuwv(j), iuev(j)
        qsum(i,j) = qsum(i-1,j) + qtmpv(i,j)
      end do

!
! The boolean terms:
! a)    .and. (isave(i,j) < i)
! b)    .and. (i <= isave(i,j))
! are needed in the IF statements below because I cannot prove to myself
! that the relationship between i and isave are such to guarantee that
! there is always at least one term from qsum (qtmpv,j) contributed to fxv.
!

      do i=1,im
        if(cv(i,j) >= D1_0 .and. (isave(i,j) < i) ) then
            fxv(i,j) = fxv(i,j) + (qsum(i-1,j) - qsum(isave(i,j) - 1,j))
        else if (cv(i,j) <= -D1_0 .and. (i <= isave(i,j)) ) then
            fxv(i,j) = fxv(i,j) - (qsum(isave(i,j),j) - qsum(i-1,j))
        end if
      end do

      if(id .ne. 0) then
         do i=1,im
            fxv(i,j) =  fxv(i,j)*mfxv(i,j)
         enddo
      endif

   enddo
!call ftrace_region_end("xtpv_6")

! Regular PPM (Eulerian without FFSL extension)

!call ftrace_region_begin("xtpv_7")
!dir$ concurrent
!cdir nodep
   do ja = 1, jafn
      j = jaf(ja)

      qtmpv(imp,j) = qv(1,j)
      qtmpv(  0,j) = qv(im,j)
   enddo

!dir$ concurrent
   do ja = 1, jaftn
      j = jaft(ja)

      do i=1,im
         iu = real(i,r8) - cv(i,j)
         fxv(i,j) = mfxv(i,j)*qtmpv(iu,j)
      enddo
   enddo

!dir$ concurrent
!cdir nodep
   do ja = 1, jaffn
      j = jaff(ja)

      qtmpv(-1,j)    = qv(im-1,j)
      qtmpv(imp+1,j) = qv(2,j)

   enddo
!call ftrace_region_end("xtpv_7")

!dir$ concurrent
!call ftrace_region_begin("xtpv_8")
   do ja = 1, jafftn1
      j = jafft1(ja)

! In-line xmist

      do i=1,im
         dmv(i,j) = r24*(D8_0*(qtmpv(i+1,j) - qtmpv(i-1,j)) + qtmpv(i-2,j) - qtmpv(i+2,j))
      enddo

! Apply monotonicity constraint (Lin et al. 1994, MWR)
      do i=1,im
         qmax = max( qtmpv(i-1,j), qtmpv(i,j), qtmpv(i+1,j) ) - qtmpv(i,j)
         qmin = qtmpv(i,j) - min( qtmpv(i-1,j), qtmpv(i,j), qtmpv(i+1,j) )
         dmv(i,j) = sign( min(abs(dmv(i,j)), qmax, qmin), dmv(i,j) )
      enddo

   enddo
!call ftrace_region_end("xtpv_8")

!dir$ concurrent
!call ftrace_region_begin("xtpv_9")
   do ja = 1, jafffn1
      j = jafff1(ja)

! In-line xmist

      if(iord .le. 2) then
         do i=1,im
            dmv(i,j) = r24*(D8_0*(qtmpv(i+1,j) - qtmpv(i-1,j)) + qtmpv(i-2,j) - qtmpv(i+2,j))
         enddo
      else
         do i=1,im
            dmv(i,j) = D0_25*(qtmpv(i+1,j) - qtmpv(i-1,j))
         enddo
      endif

      if( iord >= 0 ) then

! Apply monotonicity constraint (Lin et al. 1994, MWR)
         do i=1,im
            qmax = max( qtmpv(i-1,j), qtmpv(i,j), qtmpv(i+1,j) ) - qtmpv(i,j)
            qmin = qtmpv(i,j) - min( qtmpv(i-1,j), qtmpv(i,j), qtmpv(i+1,j) )
            dmv(i,j) = sign( min(abs(dmv(i,j)), qmax, qmin), dmv(i,j) )
         enddo
      endif

   enddo
!call ftrace_region_end("xtpv_9")

!call ftrace_region_begin("xtpv_10")
!dir$ concurrent
!cdir nodep
   do ja = 1, jaffn
      j = jaff(ja)

      dmv(0,j) = dmv(im,j)

   enddo
!call ftrace_region_end("xtpv_10")

!dir$ concurrent
!call ftrace_region_begin("xtpv_11")
   do ja = 1, jafftn2
      j = jafft2(ja)

      do i=1,im
         iu = real(i,r8) - cv(i,j)
         fxv(i,j) =  mfxv(i,j)*(qtmpv(iu,j)+dmv(iu,j)*(sign(D1_0,cv(i,j))-cv(i,j)))
      enddo

   enddo
!call ftrace_region_end("xtpv_11")

   call fxppmv(im, cv, mfxv, qtmpv, dmv, fxv, iord,                     &
              iuwv, iuev, ffslv, isave, jafffn2, jafff2, jlow, jhigh,   &
              jl2, jh2, jl3, jh3, jl5, jh5, jl7, jh7, jl11, jh11)

   return
!EOC
 end subroutine xtpv
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: fxppmv
!
! !INTERFACE: 

 subroutine fxppmv(im, c, mfx, p, dm, fx, iord,                         &
                  iuw, iue, ffsl, isave, jan, ja, jlow, jhigh,          &
                  jl2, jh2, jl3, jh3, jl5, jh5, jl7, jh7, jl11, jh11)
!-----------------------------------------------------------------------
!
! !USES:
 implicit none

! !INPUT PARAMETERS:
 integer jan, ja(jan), jlow, jhigh, jj, j
 integer jl2, jh2, jl3, jh3, jl5, jh5, jl7, jh7, jl11, jh11
 integer im, iord
 real (r8)  c(im,jl5:jh5)
 real (r8) p(-im/3:im+im/3,jl11:jh11)
 real (r8) dm(-im/3:im+im/3,jlow:jhigh)
 real (r8) mfx(im,jl7:jh7)
 integer iuw(jlow:jhigh), iue(jlow:jhigh)
 logical ffsl(jl2:jh2)
 integer isave(im,jlow:jhigh)

! !OUTPUT PARAMETERS:
 real (r8) fx(im,jl3:jh3)

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 real (r8) r3, r23
 parameter ( r3 = D1_0/D3_0, r23 = D2_0/D3_0 )

 integer i, lmt
 integer iu, itmp
 real (r8) ru
 logical steep
 real (r8) al(-im/3:im+im/3,jlow:jhigh)
 real (r8) ar(-im/3:im+im/3,jlow:jhigh)
 real (r8) a6(-im/3:im+im/3,jlow:jhigh)

 integer jbtn, jbfn
 integer jbt(jan), jbf(jan)
 integer ilow, ihigh

 ilow = -im/3
 ihigh = im + im/3

 if( iord == 6 ) then
   steep = .true.
 else
   steep = .false.
 endif

!dir$ concurrent
 do jj = 1, jan
   j = ja(jj)

   do i=1,im
     al(i,j) = D0_5*(p(i-1,j)+p(i,j)) + (dm(i-1,j) - dm(i,j))*r3
   enddo

 enddo

 if (steep) then

   call steepxv( im, p, al, dm, jan, ja, jlow, jhigh, jl11, jh11 )

 endif

!dir$ concurrent
 do jj = 1, jan
   j = ja(jj)

   do i=1,im-1
     ar(i,j) = al(i+1,j)
   enddo
   ar(im,j) = al(1,j)

 enddo

 if(iord == 7) then

   call huynhv(im, ar, al, p, a6, dm, jan, ja, jlow, jhigh, jl11, jh11 )

 else

   if(iord .eq. 3 .or. iord .eq. 5) then

!dir$ concurrent
     do jj = 1, jan
       j = ja(jj)

       do i=1,im
         a6(i,j) = D3_0*(p(i,j)+p(i,j)  - (al(i,j)+ar(i,j)))
       enddo

     enddo
   endif

   lmt = iord - 3

   call lmppmv( dm, a6, ar, al, p, im, lmt, jan, ja, ilow, ihigh,    &
                jlow, jhigh, jl11, jh11 )

 endif

 jbtn = 0
 jbfn = 0
!dir$ concurrent
 do jj = 1, jan
   j = ja(jj)
   if( ffsl(j) ) then
     jbtn = jbtn + 1
     jbt(jbtn) = j
   else
     jbfn = jbfn + 1
     jbf(jbfn) = j
   endif
 enddo

!dir$ concurrent
 do jj = 1, jbtn
   j = jbt(jj)

   do i=iuw(j), 0
     al(i,j) = al(im+i,j)
     ar(i,j) = ar(im+i,j)
     a6(i,j) = a6(im+i,j)
   enddo

   do i=im+1, iue(j)
     al(i,j) = al(i-im,j)
     ar(i,j) = ar(i-im,j)
     a6(i,j) = a6(i-im,j)
   enddo

   do i=1,im
     iu = c(i,j)
     ru = c(i,j) - iu
     if(c(i,j) .gt. D0_0) then
       itmp = i - iu - 1
       isave(i,j) = itmp + 1
       fx(i,j) = ru*(ar(itmp,j)+D0_5*ru*(al(itmp,j)-ar(itmp,j) +     &
                 a6(itmp,j)*(D1_0-r23*ru)) )
     else
       itmp = i - iu
       isave(i,j) = itmp - 1
       fx(i,j) = ru*(al(itmp,j)-D0_5*ru*(ar(itmp,j)-al(itmp,j) +     &
                 a6(itmp,j)*(D1_0+r23*ru)) )
     endif
   enddo

 enddo

!dir$ concurrent
 do jj = 1, jbfn
   j = jbf(jj)

   al(0,j) = al(im,j)
   ar(0,j) = ar(im,j)
   a6(0,j) = a6(im,j)
   do i=1,im
     if(c(i,j) .gt. D0_0) then
       fx(i,j) = ar(i-1,j) + D0_5*c(i,j)*(al(i-1,j) - ar(i-1,j) +   &
                 a6(i-1,j)*(D1_0-r23*c(i,j)) )
     else
       fx(i,j) = al(i,j) - D0_5*c(i,j)*(ar(i,j) - al(i,j) +         &
                 a6(i,j)*(D1_0+r23*c(i,j)))
     endif
     fx(i,j) = mfx(i,j) * fx(i,j)
   enddo

 enddo

 return
!EOC
 end subroutine fxppmv
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: steepxv
!
! !INTERFACE: 
 subroutine steepxv(im, p, al, dm, jan, ja, jlow, jhigh, jl11, jh11 )
!-----------------------------------------------------------------------

! !USES:
 implicit none

! !INPUT PARAMETERS:
 integer im
 integer jan, ja(jan), jlow, jhigh, jl11, jh11
 real (r8)  p(-im/3:im+im/3,jl11:jh11)
 real (r8) dm(-im/3:im+im/3,jlow:jhigh)

! !INPUT/OUTPUT PARAMETERS:
 real (r8) al(im,jlow:jhigh)

! !DESCRIPTION:
!   
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 integer i, jj, j
 real (r8) r3
 parameter ( r3 = D1_0/D3_0 )

 real (r8) dh(0:im,jlow:jhigh)
 real (r8) d2(0:im+1,jlow:jhigh)
 real (r8) eta(0:im,jlow:jhigh)
 real (r8) xxx, bbb, ccc

!dir$ concurrent
 do jj = 1, jan
   j = ja(jj)

   do i=0,im
      dh(i,j) = p(i+1,j) - p(i,j)
   enddo

! Needs dh(0:im,j)
   do i=1,im
      d2(i,j) = dh(i,j) - dh(i-1,j)
   enddo
   d2(0,j) = d2(im,j)
   d2(im+1,j) = d2(1,j)

! needs p(-1:im+2,j), d2(0:im+1,j)
   do i=1,im
      if( d2(i+1,j)*d2(i-1,j).lt.D0_0 .and. p(i+1,j).ne.p(i-1,j) ) then
          xxx    = D1_0 - D0_5 * ( p(i+2,j) - p(i-2,j) ) / ( p(i+1,j) - p(i-1,j) )
          eta(i,j) = max(D0_0, min(xxx, D0_5) )
      else
          eta(i,j) = D0_0
      endif
   enddo

   eta(0,j) = eta(im,j)

! needs eta(0:im,j), dh(0:im-1,j), dm(0:im,j)
   do i=1,im
      bbb = ( D2_0*eta(i,j  ) - eta(i-1,j) ) * dm(i-1,j) 
      ccc = ( D2_0*eta(i-1,j) - eta(i,j  ) ) * dm(i,j  ) 
      al(i,j) = al(i,j) + D0_5*( eta(i-1,j) - eta(i,j)) * dh(i-1,j) + (bbb - ccc) * r3
   enddo

 enddo

 return
!EOC
 end subroutine steepxv
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: huynhv --- Enforce Huynh's 2nd constraint in 1D periodic domain
!
! !INTERFACE: 
 subroutine huynhv(im, ar, al, p, d2, d1, jan, ja, jlow, jhigh, jl11, jh11)
!-----------------------------------------------------------------------

! !USES:

 implicit none

! !INPUT PARAMETERS:
 integer im
 integer jan, ja(jan), jlow, jhigh, jl11, jh11
 real(r8)  p(im,jl11:jh11)

! !OUTPUT PARAMETERS:
 real(r8) ar(im,jlow:jhigh)
 real(r8) al(im,jlow:jhigh)
 real(r8) d2(im,jlow:jhigh)
 real(r8) d1(im,jlow:jhigh)

! !DESCRIPTION:
!
!   Enforce Huynh's 2nd constraint in 1D periodic domain
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 integer  i, jj, j
 real(r8) pmp
 real(r8) lac
 real(r8) pmin
 real(r8) pmax

!dir$ concurrent
   do jj = 1, jan
      j = ja(jj)

! Compute d1 and d2
      d1(1,j) = p(1,j) - p(im,j)
      do i=2,im
         d1(i,j) = p(i,j) - p(i-1,j)
      enddo

      do i=1,im-1
         d2(i,j) = d1(i+1,j) - d1(i,j)
      enddo
      d2(im,j) = d1(1,j) - d1(im,j)

! Constraint for AR
!     i = 1
      pmp   = p(1,j) + D2_0 * d1(1,j)
      lac   = p(1,j) + D0_5 * (d1(1,j)+d2(im,j)) + d2(im,j) 
      pmin  = min(p(1,j), pmp, lac)
      pmax  = max(p(1,j), pmp, lac)
      ar(1,j) = min(pmax, max(ar(1,j), pmin))

      do i=2, im
         pmp   = p(i,j) + D2_0*d1(i,j)
         lac   = p(i,j) + D0_5*(d1(i,j)+d2(i-1,j)) + d2(i-1,j)
         pmin  = min(p(i,j), pmp, lac)
         pmax  = max(p(i,j), pmp, lac)
         ar(i,j) = min(pmax, max(ar(i,j), pmin))
      enddo

! Constraint for AL
      do i=1, im-1
         pmp   = p(i,j) - D2_0*d1(i+1,j)
         lac   = p(i,j) + D0_5*(d2(i+1,j)-d1(i+1,j)) + d2(i+1,j)
         pmin  = min(p(i,j), pmp, lac)
         pmax  = max(p(i,j), pmp, lac)
         al(i,j) = min(pmax, max(al(i,j), pmin))
      enddo

! i=im
      i = im
      pmp    = p(im,j) - D2_0*d1(1,j)
      lac    = p(im,j) + D0_5*(d2(1,j)-d1(1,j)) + d2(1,j)
      pmin   = min(p(im,j), pmp, lac)
      pmax   = max(p(im,j), pmp, lac)
      al(im,j) = min(pmax, max(al(im,j), pmin))

! compute A6 (d2)
      do i=1, im
         d2(i,j) = D3_0*(p(i,j)+p(i,j)  - (al(i,j)+ar(i,j)))
      enddo

   enddo

   return
!EOC
 end subroutine huynhv
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!BOP
! !IROUTINE: lmppmv
!
! !INTERFACE: 
 subroutine lmppmv(dm, a6, ar, al, p, im, lmt, jan, ja,           &
                  ilow, ihigh, jlow, jhigh, jl11, jh11)
!-----------------------------------------------------------------------

 implicit none

! !INPUT PARAMETERS:
 integer im   ! Total longitudes
 integer jan, ja(jan), ilow, ihigh, jlow, jhigh, jl11, jh11
 integer lmt  ! LMT = 0: full monotonicity
              ! LMT = 1: Improved and simplified full monotonic constraint
              ! LMT = 2: positive-definite constraint
              ! LMT = 3: Quasi-monotone constraint
 real(r8) p(ilow:ihigh,jl11:jh11)
 real(r8) dm(ilow:ihigh,jlow:jhigh)

! !OUTPUT PARAMETERS:
 real(r8) a6(ilow:ihigh,jlow:jhigh)
 real(r8) ar(ilow:ihigh,jlow:jhigh)
 real(r8) al(ilow:ihigh,jlow:jhigh)

! !DESCRIPTION:
!
!
! !REVISION HISTORY:
!   99.01.01 Lin      Creation
!   01.03.27 Sawyer   Additional ProTeX documentation
!
!EOP
!-----------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
 real (r8) r12
 parameter ( r12 = D1_0/D12_0 )

 real (r8) da1, da2, fmin, a6da
 real (r8) dr, dl

 integer i, jj, j

! LMT = 0: full monotonicity
! LMT = 1: Improved and simplified full monotonic constraint
! LMT = 2: positive-definite constraint
! LMT = 3: Quasi-monotone constraint

  if( lmt == 0 ) then

! Full constraint

!dir$ concurrent
    do jj = 1, jan
      j = ja(jj)

        do i=1,im
           if(dm(i,j) .eq. D0_0) then
               ar(i,j) = p(i,j)
               al(i,j) = p(i,j)
               a6(i,j) = D0_0
           else
               da1  = ar(i,j) - al(i,j)
               da2  = da1**2
               a6da = a6(i,j)*da1
               if(a6da .lt. -da2) then
                  a6(i,j) = D3_0*(al(i,j)-p(i,j))
                  ar(i,j) = al(i,j) - a6(i,j)
               elseif(a6da .gt. da2) then
                  a6(i,j) = D3_0*(ar(i,j)-p(i,j))
                  al(i,j) = ar(i,j) - a6(i,j)
               endif
           endif
        enddo

    enddo

  elseif( lmt == 1 ) then

! Improved (Lin 2001?) full constraint

!dir$ concurrent
    do jj = 1, jan
      j = ja(jj)

        do i=1,im
            da1 = dm(i,j) + dm(i,j)
            dl = sign(min(abs(da1),abs(al(i,j)-p(i,j))), da1)
            dr = sign(min(abs(da1),abs(ar(i,j)-p(i,j))), da1)
            ar(i,j) = p(i,j) + dr
            al(i,j) = p(i,j) - dl
            a6(i,j) = D3_0*(dl-dr)
        enddo

    enddo

  elseif( lmt == 2 ) then

! Positive definite constraint

!dir$ concurrent
    do jj = 1, jan
      j = ja(jj)

      do i=1,im
        if(abs(ar(i,j)-al(i,j)) .lt. -a6(i,j)) then
          fmin = p(i,j) + D0_25*(ar(i,j)-al(i,j))**2/a6(i,j) + a6(i,j)*r12
          if(fmin.lt.D0_0) then
            if(p(i,j).lt.ar(i,j) .and. p(i,j).lt.al(i,j)) then
                ar(i,j) = p(i,j)
                al(i,j) = p(i,j)
                a6(i,j) = D0_0
            elseif(ar(i,j) .gt. al(i,j)) then
                a6(i,j) = D3_0*(al(i,j)-p(i,j))
                ar(i,j) = al(i,j) - a6(i,j)
            else
                a6(i,j) = D3_0*(ar(i,j)-p(i,j))
                al(i,j) = ar(i,j) - a6(i,j)
            endif
          endif
        endif
      enddo

    enddo

  elseif(lmt .eq. 3) then

! Quasi-monotone constraint

!dir$ concurrent
    do jj = 1, jan
      j = ja(jj)

      do i=1,im
         da1 = D4_0*dm(i,j)
         dl = sign(min(abs(da1),abs(al(i,j)-p(i,j))), da1)
         dr = sign(min(abs(da1),abs(ar(i,j)-p(i,j))), da1)
         ar(i,j) = p(i,j) + dr
         al(i,j) = p(i,j) - dl
         a6(i,j) = D3_0*(dl-dr)
      enddo

    enddo

  endif
  return
!EOC
 end subroutine lmppmv
!-----------------------------------------------------------------------
#endif

end module tp_core
