#include <misc.h>
#include <params.h>
!-----------------------------------------------------------------------
!
! BOP
!
! !MODULE: ozn_data
!
! !DESCRIPTION: Module to handle dealing with the OZONE datasets. 
!
! Public interfaces:
!
!       oznini -- Initialization and reading of dataset.
!       oznint -- Interpolate dataset OZONE to current time.
!
! $Id: ozn_data.F90,v 1.3 2009/10/19 01:00:57 wangzzh Exp $
!
!-----------------------------------------------------------------------

module ozn_data
!
! USES:
!
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use ppgrid,       only: begchunk, endchunk, pcols
   use phys_grid,    only: get_ncols_p, scatter_field_to_chunk
   use comozp
   use pspect
   use rgrid
   use commap
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use abortutils,   only: endrun
   use infnan,       only: inf
#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none
   public oznini
   public oznint

   integer ncid_oz       !  ozone dataset
   logical  :: reverp = .false.

   interface

   subroutine bilin (arrin, xin, yin, nlondin, nlonin, &
                     nlevdin, nlev, nlatin, arrout, xout, &
                     yout, nlondout, nlonout, nlevdout, nlatout, fillvalue)

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   integer, intent(in) :: nlondin                        ! longitude dimension of input grid
   integer, intent(in) :: nlonin                         ! number of real longitudes (input)
   integer, intent(in) :: nlevdin                        ! vertical dimension of input grid
   integer, intent(in) :: nlev                           ! number of vertical levels
   integer, intent(in) :: nlatin                         ! number of input latitudes
   integer, intent(in) :: nlatout                        ! number of output latitudes
   integer, intent(in) :: nlondout                       ! longitude dimension of output grid
   integer, intent(in) :: nlonout(nlatout)               ! number of output longitudes per lat
   integer, intent(in) :: nlevdout                       ! vertical dimension of output grid

   real(r8), intent(in) :: arrin(nlondin,nlevdin,nlatin) ! input array of values to interpolate
   real(r8), intent(in) :: xin(nlondin)                  ! input x mesh
   real(r8), intent(in) :: yin(nlatin)                   ! input y mesh
   real(r8), intent(in) :: xout(nlondout,nlatout)        ! output x mesh
   real(r8), intent(in) :: yout(nlatout)                 ! output y mesh
   real(r8), optional, intent(in) :: fillvalue           ! undef. value, wzz, 2009.8.31

   real(r8), intent(out) :: arrout(nlondout,nlevdout,nlatout) ! interpolated array

   end subroutine bilin

   end interface

!===============================================================================
!EOP
!===============================================================================

!===============================================================================
CONTAINS
!===============================================================================

subroutine oznini
!----------------------------------------------------------------------- 
! 
! Purpose: Do initial read of time-variant ozone boundary dataset, containing
!          ozone mixing ratios as a function of latitude and pressure.  Read two
!          consecutive months between which the current date lies.  Routine
!          RADOZ2 then evaluates the two path length integrals (with and without
!          pressure weighting) from zero to the interfaces between the input
!          levels.  It also stores the contribution to the integral from each
!          layer.
! 
! Method: Call appropriate netcdf wrapper routines and interpolate to model grid
! 
! Author: CCM Core Group
! Modified: P. Worley, August 2003, for chunking and performance optimization
! 
!-----------------------------------------------------------------------
   use filenames,    only: bndtvo           ! wzz
   use ioFileMod,    only: getfil

!-----------------------------------------------------------------------
   include 'netcdf.inc'
#include <comctl.h>
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer dateid                          ! netcdf id for date variable
   integer secid                           ! netcdf id for seconds variable
   integer londimid                        ! netcdf id for longitude dimension
   integer latdimid                        ! netcdf id for latitude dimension
   integer levdimid                        ! netcdf id for level dimension
   integer lonid                           ! netcdf id for longitude variable
   integer latid                           ! netcdf id for latitude variable
   integer levid                           ! netcdf id for level variable
   integer timeid                          ! netcdf id for time variable
   integer dimids(nf_max_var_dims)         ! variable shape
   integer cnt4(4)                         ! array of counts for each dimension
   integer strt4(4)                        ! array of starting indices
   integer i, k, lat, n                    ! longitude, level, latitude, time indices
   integer  :: yr, mon, day                ! components of a date
   integer  :: ncdate                      ! current date in integer format [yyyymmdd]
   integer  :: ncsec                       ! current time of day [seconds]
   integer  :: status
   real(r8) :: calday                      ! current calendar day
   real(r8) caldayloc                      ! calendar day (includes yr if no cycling)
   real(r8), allocatable :: ozmix2D(:,:)   ! temporary ozmix arrays
   real(r8), allocatable :: ozmix3D(:,:,:)
   real(r8), allocatable :: oznbdym(:,:,:) ! ozone data previous time sample
   real(r8), allocatable :: oznbdyp(:,:,:) ! ozone data next time sample

   character(len=256) :: locfn    ! netcdf local filename to open

   nm = 1
   np = 2

   fillvalue = inf    ! wzz, 2009.10.10
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
   if (masterproc) then
!
! Use year information only if not cycling ozone dataset
!
      calday = get_curr_calday()
      if ( is_perpetual() ) then
         call get_perp_date(yr, mon, day, ncsec)
      else
         call get_curr_date(yr, mon, day, ncsec)
      end if
      ncdate = yr*10000 + mon*100 + day
      if (ozncyc) then
         caldayloc = calday
      else
         caldayloc = calday + yr*365.
      end if

      call getfil (bndtvo, locfn)
      call wrap_open (locfn, 0, ncid_oz)
      write(6,*)'OZNINI: NCOPN returns id ',ncid_oz,' for file ',trim(locfn)

!
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_oz, 'lon', londimid   )
      CALL WRAP_INQ_DIMID( ncid_oz, 'lev', levdimid   )
      CALL WRAP_INQ_DIMID( ncid_oz, 'time', timeid  )
      CALL WRAP_INQ_DIMID( ncid_oz, 'lat', latdimid   )

      CALL WRAP_INQ_DIMLEN( ncid_oz, londimid, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_oz, levdimid, levsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_oz, latdimid, latsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_oz, timeid, timesiz   )

      CALL WRAP_INQ_VARID( ncid_oz, 'date', dateid   )
      CALL WRAP_INQ_VARID( ncid_oz, 'datesec', secid   )
      CALL WRAP_INQ_VARID( ncid_oz, 'OZONE', oznid   )
      CALL WRAP_INQ_VARID( ncid_oz, 'lon', lonid   )
      CALL WRAP_INQ_VARID( ncid_oz, 'lat', latid   )
      CALL WRAP_INQ_VARID( ncid_oz, 'lev', levid   )

      CALL WRAP_INQ_VARDIMID (ncid_oz, oznid, dimids)
      if (dimids(1) /= londimid .or. dimids(2) /= levdimid .or. dimids(3) /= latdimid) then
         call endrun ('OZNINI: Data must be ordered lon, lev, lat, time')
      end if
! get the fillvalue if it exists, wzz, 2009.10.10
      status = nf_get_att_double(ncid_oz, oznid, '_FillValue', fillvalue)
      write(*,*)'wzz: status for getting fillvalue=', status

   end if

#if (defined SPMD )
   call mpibcast( lonsiz, 1, mpiint, 0, mpicom )
   call mpibcast( latsiz, 1, mpiint, 0, mpicom )
   call mpibcast( levsiz, 1, mpiint, 0, mpicom )
   call mpibcast( timesiz, 1, mpiint, 0, mpicom )
   call mpibcast( fillvalue, 1, mpir8, 0, mpicom )
#endif
!   if ( fillvalue /= inf ) then
!     if (masterproc) write(*,*)' The fillvalue of OZONE data = ', fillvalue
!   endif

!
! Dynamically allocated memory for module comozp 
!
   allocate (date_oz(timesiz))
   allocate (sec_oz(timesiz))
   allocate (pin(levsiz))
   allocate (ozmixm(nm)%val(pcols,levsiz,begchunk:endchunk))
   allocate (ozmixm(np)%val(pcols,levsiz,begchunk:endchunk))
   allocate (ozmix(pcols,levsiz,begchunk:endchunk))
!
! Locally dynamic that will be deallocated before "return"
!
   allocate (ozmix3D(plond,levsiz,plat))

   if (masterproc) then
!
! More dynamically allocated memory for module comozp 
! (for masterproc only)
!
      allocate (ozlon(lonsiz))
      allocate (ozlat(latsiz))
!
! More locally dynamic that will be deallocated before "return"
!
      allocate (oznbdym(lonsiz,levsiz,latsiz))
      allocate (oznbdyp(lonsiz,levsiz,latsiz))
      allocate (ozmix2D(levsiz,plat))
!
! Retrieve longitude, latitude and level arrays for interpolation.
!
      CALL WRAP_GET_VAR_REALX (NCID_OZ, lonid,ozlon)
      CALL WRAP_GET_VAR_REALX (NCID_OZ, latid,ozlat)
      CALL WRAP_GET_VAR_REALX (NCID_OZ, levid,pin)
!
! Convert from millibars to pascals
!
      do k=1,levsiz
         pin(k) = pin(k)*100.
      end do

!
! Retrieve entire date and sec variables.
!
      CALL WRAP_GET_VAR_INT (ncid_oz,dateid,date_oz)
      CALL WRAP_GET_VAR_INT (ncid_oz,secid,sec_oz)
      if (ozncyc) then
         if (timesiz < 12) then 
            write(6,*)'OZNINI: When cycling ozone, dataset must have 12 consecutive ', &
                      'months of data starting with Jan'
            write(6,*)'Current dataset has only ',timesiz,' months'
            call endrun
         end if
         do n = 1,12
            if (mod(date_oz(n),10000)/100 /= n) then
               write(6,*)'OZNINI: When cycling ozone, dataset must have 12 consecutive ', &
                         'months of data starting with Jan'
               write(6,*)'Month ',n,' of dataset says date=',date_oz(n)
               call endrun
            end if
         end do
      end if

      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1)  = lonsiz
      cnt4(2)  = levsiz
      cnt4(3)  = latsiz
      cnt4(4)  = 1
!
! Special code for interpolation between December and January
!
      if (ozncyc) then
         n = 12
         np1 = 1
         call bnddyi(date_oz(n  ), sec_oz(n  ), cdayozm)
         call bnddyi(date_oz(np1), sec_oz(np1), cdayozp)
         if (caldayloc <= cdayozp .or. caldayloc > cdayozm) then
            strt4(4) = n
            call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdym)

            strt4(4) = np1
            call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdyp)
            goto 10
         end if
      end if
!
! Normal interpolation between consecutive time slices.
!
      do n=1,timesiz-1
         np1 = n + 1
         call bnddyi(date_oz(n  ), sec_oz(n  ), cdayozm)
         call bnddyi(date_oz(np1), sec_oz(np1), cdayozp)
         if (.not.ozncyc) then
            yr = date_oz(n)/10000
            cdayozm = cdayozm + yr*365.
            yr = date_oz(np1)/10000
            cdayozp = cdayozp + yr*365.
         end if
         if (caldayloc > cdayozm .and. caldayloc <= cdayozp) then
            strt4(4) = n
            call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdym)

            strt4(4) = np1
            call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdyp)
            goto 10
         end if
      end do
      n = 1
      np1 = n
      call bnddyi(date_oz(n  ), sec_oz(n  ), cdayozm)
      call bnddyi(date_oz(np1), sec_oz(np1), cdayozp)
      if (.not.ozncyc) then
         yr = date_oz(n)/10000
         cdayozm = cdayozm + yr*365.
         yr = date_oz(np1)/10000
         cdayozp = cdayozp + yr*365.
      end if
      if ( caldayloc <= cdayozm) then
         strt4(4) = n
         call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdym)
!        oznbdyp(:,:,:) = oznbdym(:,:,:)

         strt4(4) = np1
         call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdyp)
         goto 10
      endif
      n = timesiz-1
      np1 = n + 1
      call bnddyi(date_oz(n  ), sec_oz(n  ), cdayozm)
      call bnddyi(date_oz(np1), sec_oz(np1), cdayozp)
         if (.not.ozncyc) then
            yr = date_oz(n)/10000
            cdayozm = cdayozm + yr*365.
            yr = date_oz(np1)/10000
            cdayozp = cdayozp + yr*365.
         end if
      if ( caldayloc > cdayozp) then
         strt4(4) = n
         call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdym)

         strt4(4) = np1
         call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdyp)
!        oznbdym = oznbdyp
            goto 10
      endif
      write(6,*)'OZNINI: Failed to find dates bracketing ncdate, ncsec=', ncdate, ncsec
      call endrun
10    continue
      write(6,*)'OZNINI: Read ozone data for dates ',date_oz(n), &
                sec_oz(n),' and ',date_oz(np1),sec_oz(np1)
!
! Spatial interpolation.  If ozone dataset is 2-d (i.e. lonsiz = 1) and 
! thus only latitude interpolation is necessary, expand to 3-d after 
! interpolation.
!
      if (lonsiz == 1) then
         call lininterp (oznbdym ,ozlat   ,levsiz  ,latsiz  ,ozmix2D, &
                         latdeg  ,plat    )
         do lat=1,plat
            do k=1,levsiz
               do i=1,nlon(lat)
                  ozmix3D(i,k,lat) = ozmix2D(k,lat)
               end do
            end do
         end do
 
         call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(nm)%val)

         call lininterp (oznbdyp ,ozlat   ,levsiz  ,latsiz  ,ozmix2D, &
                         latdeg  ,plat)
         do lat=1,plat
            do k=1,levsiz
               do i=1,nlon(lat)
                  ozmix3D(i,k,lat) = ozmix2D(k,lat)
               end do
            end do
         end do

        call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(np)%val)

      else

        if ( fillvalue == inf ) then
         call bilin (oznbdym, ozlon, ozlat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, ozmix3D, londeg, &
                     latdeg, plond, nlon, levsiz, plat)
        else
         call bilin (oznbdym, ozlon, ozlat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, ozmix3D, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)
        endif

         call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(nm)%val)

        if ( fillvalue == inf ) then
         call bilin (oznbdyp, ozlon, ozlat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, ozmix3D, londeg, &
                     latdeg, plond, nlon, levsiz, plat)
        else
         call bilin (oznbdyp, ozlon, ozlat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, ozmix3D, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)
        endif

        call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(np)%val)
      end if
!
! Deallocate dynamic memory for local workspace.  NOT for pointers in common.
!
      deallocate (oznbdym)
      deallocate (oznbdyp)
      deallocate (ozmix2D)
      deallocate (ozmix3D)

#if (defined SPMD )
   else

      call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(nm)%val)
      call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(np)%val)
!
! Deallocate dynamic memory for local workspace.  NOT for pointers in common.
!
      deallocate (ozmix3D)

#endif
   end if

#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (pin, levsiz, mpir8, 0, mpicom)
   call mpibcast (date_oz, timesiz, mpiint, 0, mpicom )
   call mpibcast (sec_oz, timesiz, mpiint, 0, mpicom )
   call mpibcast (cdayozm, 1, mpir8, 0, mpicom)
   call mpibcast (cdayozp, 1, mpir8, 0, mpicom)
#endif
!
   if ( pin(1)>pin(2) ) then
     reverp = .true.
   endif
   if ( reverp ) then
      pin(1:levsiz) = pin(levsiz:1:-1)
!     write(*,*)'ozn=', ozmixm(nm)%val(1,:,begchunk)
      ozmixm(nm)%val(:,1:levsiz,:) = ozmixm(nm)%val(:,levsiz:1:-1,:)
      ozmixm(np)%val(:,1:levsiz,:) = ozmixm(np)%val(:,levsiz:1:-1,:)
!     write(*,*)'pin=', pin
!     write(*,*)'ozn1=', ozmixm(nm)%val(1,:,begchunk)
   endif

   return
end subroutine oznini


subroutine oznint
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate ozone mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
! Method: Find next month of ozone data to interpolate.  Linearly interpolate 
!         vertically and horizontally
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
  use timeinterp, only: getfactors

#include <comctl.h>
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer cnt4(4)                ! array of counts for each dimension
   integer strt4(4)               ! array of starting indices
   integer i, k, lat              ! longitude/column, level, latitude indices
   integer c, ncol                ! chunk index, number of columns in chunk
   integer ntmp                   ! temporary
   integer :: yr, mon, day        ! components of a date
   integer :: ncdate              ! current date in integer format [yyyymmdd]
   integer :: ncsec               ! current time of day [seconds]

   real(r8) fact1, fact2          ! time interpolation factors
   real(r8) :: calday             ! current calendar day
   real(r8) caldayloc             ! calendar day (includes yr if no cycling)
   real(r8) deltat                ! time (days) between interpolating ozone data

   real(r8), allocatable :: oznbdyp(:,:,:)  ! ozone data from next time sample
   real(r8), allocatable :: ozmix2D(:,:)    ! temporary ozmix arrays
   real(r8), allocatable :: ozmix3D(:,:,:)
!
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   if ( is_perpetual() ) then
      call get_perp_date(yr, mon, day, ncsec)
   else
      call get_curr_date(yr, mon, day, ncsec)
   end if
   ncdate = yr*10000 + mon*100 + day
   if (ozncyc) then
      caldayloc = calday
   else
      caldayloc = calday + yr*365.
   end if
!
! If model time is past current forward ozone timeslice, then
! masterproc reads in the next timeslice for time interpolation.  Messy logic is 
! for ozncyc = .true. interpolation between December and January (np1 == 1).  Note 
! that np1 is never 1 when ozncyc is .false.
!
!   if (caldayloc > cdayozp .and. .not. (np1 == 1 .and. caldayloc > cdayozm)) then
   if (caldayloc > cdayozp .and.  &
   (.not.(np1 == 1 .and. caldayloc > cdayozm).or.cdayozp==cdayozm)) then
!
      if (ozncyc) then
         np1 = mod(np1,12) + 1
      else
         np1 = np1 + 1
      end if
      if (np1 > timesiz) then
!         call endrun ('OZNINT: Attempt to read past end of O3 dataset')
          np1 = timesiz
      end if
      cdayozm = cdayozp
      call bnddyi(date_oz(np1), sec_oz(np1), cdayozp)
      if (.not.ozncyc) then
         yr = date_oz(np1)/10000
         cdayozp = cdayozp + yr*365.
      end if

      if (.not. (np1 == 1 .or. caldayloc <= cdayozp)) then
         if (masterproc) then
            write(6,*)'OZNINT: Input ozone for date',date_oz(np1),' sec ',sec_oz(np1), &
                      'does not exceed model date',ncdate,' sec ',ncsec,' Stopping.'
         endif
         call endrun ()
      end if

      ntmp = nm
      nm = np
      np = ntmp
!
! Allocate memory for dynamic local workspace
!
      allocate (ozmix3D(plond,levsiz,plat))

      if (masterproc) then
         strt4(1) = 1
         strt4(2) = 1
         strt4(3) = 1
         strt4(4) = np1
         cnt4(1)  = lonsiz
         cnt4(2)  = levsiz
         cnt4(3)  = latsiz
         cnt4(4)  = 1
!
! Allocate memory for more dynamic local workspace
!
         allocate (oznbdyp(lonsiz,levsiz,latsiz))
         allocate (ozmix2D(levsiz,plat))

         call wrap_get_vara_realx (ncid_oz,oznid,strt4,cnt4,oznbdyp)
         write(6,*)'OZNINT: Read ozone for date (yyyymmdd) ', date_oz(np1),' sec ',sec_oz(np1)
!
! Spatial interpolation.  If ozone dataset is only 2-d (i.e. lonsiz = 1) and 
! thus only latitude interpolation is necessary, expand to 3-d after 
! interpolation.
!
         if (lonsiz == 1) then
            call lininterp (oznbdyp, ozlat, levsiz, latsiz, ozmix2D, &
                            latdeg, plat)
            do lat=1,plat
               do k=1,levsiz
                  do i=1,nlon(lat)
                     ozmix3D(i,k,lat) = ozmix2D(k,lat)
                  end do
               end do
            end do
         else
          if ( fillvalue == inf ) then
            call bilin (oznbdyp ,ozlon   ,ozlat   ,lonsiz  ,lonsiz  , &
                        levsiz  ,levsiz  ,latsiz  ,ozmix3D , londeg  , &
                        latdeg  ,plond   ,nlon    ,levsiz  ,plat    )
          else
            call bilin (oznbdyp ,ozlon   ,ozlat   ,lonsiz  ,lonsiz  , &
                        levsiz  ,levsiz  ,latsiz  ,ozmix3D , londeg  , &
                        latdeg  ,plond   ,nlon    ,levsiz  ,plat, fillvalue    )
          endif
         end if
      end if

      call scatter_field_to_chunk(1,levsiz,1,plond,ozmix3D,ozmixm(np)%val)
!
! wzz, 2009.10.12
      if ( reverp ) then
      ozmixm(np)%val(:,1:levsiz,:) = ozmixm(np)%val(:,levsiz:1:-1,:)
      endif
!
! Deallocate dynamic memory for local workspace.
!
      deallocate (ozmix3D)
      if (masterproc) then
         deallocate (oznbdyp)
         deallocate (ozmix2D)
      end if
   end if
!
! Determine time interpolation factors.
!
   if(cdayozm==cdayozp) then
   fact1 = 1.
   fact2 = 0.
   else
   call getfactors (ozncyc, np1, cdayozm, cdayozp, caldayloc, &
                    fact1, fact2, 'OZNINT:')
   endif
!
! Time interpolation.
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do k=1,levsiz
         do i=1,ncol
            ozmix(i,k,c) = ozmixm(nm)%val(i,k,c)*fact1 + ozmixm(np)%val(i,k,c)*fact2
            if (ozmixm(nm)%val(i,k,c) == fillvalue ) then
              ozmix(i,k,c) = ozmixm(np)%val(i,k,c)
            else if (ozmixm(np)%val(i,k,c) == fillvalue ) then
              ozmix(i,k,c) = ozmixm(nm)%val(i,k,c)
            end if
         end do
      end do
   end do

   return
end subroutine oznint

end module ozn_data

