#include <misc.h>
#include <params.h>

!-------------------------------------------------------------------------------
! dynamics - physics coupling module
!-------------------------------------------------------------------------------
module dp_coupling

   use shr_kind_mod,  only: r8 => shr_kind_r8
   use ppgrid,        only: pcols, pver
   use rgrid,         only: nlon
   use pmgrid
   use phys_buffer,   only: pbuf_fld, pbuf_size_max
   use phys_grid
!#ifdef TDK_CU
   use physics_types, only: physics_state, physics_tend, dynamics_tend  
   use constituents,  only: pcnst, ppcnst, pnats
!#else
!  use physics_types, only: physics_state, physics_tend
!  use constituents,  only: pcnst, ppcnst
!#endif
   use physconst,     only: cpair, gravit, rair, zvir
   use geopotential,  only: geopotential_t
   use check_energy,  only: check_energy_timestep_init
#if (defined SPMD)
   use spmd_dyn,      only: buf1, buf2, spmdbuf_resize, spmdbuf_siz
#endif

   implicit none

!===============================================================================
CONTAINS
!===============================================================================

!===============================================================================
#ifdef TDK_CU
  subroutine d_p_coupling(ps, t3, u3, v3, q3, &
                          omga, phis, phys_state, phys_tend, pbuf, &
						  pstend, utend, vtend, ttend, qtend, tatend, qatend, dyn_tend)
#else
!------------------------------------
#ifdef WU_CONV
  subroutine d_p_coupling(ps, t3, u3, v3, q3, q31, t31, q32, t32,  t3obs, q3obs, u3obs, v3obs, vt3obs, dv3obs,&  ! jwh
                 omga, phis, phys_state, phys_tend, pbuf, pstend , dyn_tend)
#else
  subroutine d_p_coupling(ps, t3, u3, v3, q3, &
                          omga, phis, phys_state, phys_tend, pbuf)
#endif
!-----------------------------
#endif
!------------------------------------------------------------------------------
! Coupler for converting dynamics output variables into physics input variables
! also writes dynamics variables (on physics grid) to history file
!------------------------------------------------------------------------------
    use physconst,    only: cappa
    use constituents, only: cnst_get_type_byind
!------------------------------Arguments--------------------------------
    real(r8), intent(in) :: ps  (plond, beglat:endlat)            ! surface pressure
    real(r8), intent(in) :: t3  (plond, plev, beglatex:beglatex+numlats-1)  ! temperature
    real(r8), intent(in) :: u3  (plond, plev, beglatex:beglatex+numlats-1)  ! u-wind component
    real(r8), intent(in) :: v3  (plond, plev, beglatex:beglatex+numlats-1)  ! v-wind component
    real(r8), intent(in) :: q3  (plond, plev, ppcnst, beglatex:beglatex+numlats-1) ! constituents
    real(r8), intent(in) :: omga(plond, plev, beglat:endlat)      ! vertical velocity
    real(r8), intent(in) :: phis(plond, beglat:endlat)            ! Surface geopotential

#ifdef WU_CONV
    real(r8), intent(in) :: t31 (plond, plev, beglatex:beglatex+numlats-1)  ! temperature
    real(r8), intent(in) :: t32 (plond, plev, beglatex:beglatex+numlats-1)  ! temperature
    real(r8), intent(in) :: q31 (plond, plev, beglatex:beglatex+numlats-1)  ! constituents
    real(r8), intent(in) :: q32 (plond, plev, beglatex:beglatex+numlats-1)  ! constituents
    real(r8), intent(in) :: t3obs (plond, plev, beglatex:beglatex+numlats-1)  ! jwh
    real(r8), intent(in) :: q3obs (plond, plev, beglatex:beglatex+numlats-1)  ! jwh
    real(r8), intent(in) :: u3obs (plond, plev, beglatex:beglatex+numlats-1)  ! jwh
    real(r8), intent(in) :: v3obs (plond, plev, beglatex:beglatex+numlats-1)  ! jwh
    real(r8), intent(in) :: vt3obs (plond, plev, beglatex:beglatex+numlats-1)  ! jwh
    real(r8), intent(in) :: dv3obs (plond, plev, beglatex:beglatex+numlats-1)  ! jwh
#endif

#ifdef TDK_CU
    real(r8), intent(in) :: ttend  (plond, plev, beglatex:beglatex+numlats-1)              ! temperature
    real(r8), intent(in) :: utend  (plond, plev, beglatex:beglatex+numlats-1)              ! u-wind component
    real(r8), intent(in) :: vtend  (plond, plev, beglatex:beglatex+numlats-1)              ! v-wind component
    real(r8), intent(in) :: qtend  (plond, plev, pcnst+pnats, beglatex:beglatex+numlats-1) ! constituents
    real(r8), intent(in) :: tatend (plond, plev, beglatex:beglatex+numlats-1)              ! temperature
    real(r8), intent(in) :: qatend (plond, plev, pcnst, beglatex:beglatex+numlats-1)       ! constituents
#endif
    type(physics_state), intent(out), dimension(begchunk:endchunk) :: phys_state
    type(physics_tend ), intent(out), dimension(begchunk:endchunk) :: phys_tend
    type(pbuf_fld),    intent(inout), dimension(pbuf_size_max)     :: pbuf

!#ifdef TDK_CU
!   real(r8), intent(in) :: pstend (plond, beglat:endlat)                                  ! surface pressure
!   type(dynamics_tend), intent(out), dimension(begchunk:endchunk) :: dyn_tend
!#endif
#if (defined TDK_CU) || (defined WU_CONV)
    real(r8), intent(in) :: pstend (plond, beglat:endlat)                                  ! surface pressure
    type(dynamics_tend), intent(out), dimension(begchunk:endchunk) :: dyn_tend
#endif
!
!---------------------------Local workspace-----------------------------
#if (! defined SPMD)
    real(r8) :: buf1(1), buf2(1)     ! transpose buffers
    integer  :: spmdbuf_siz
#endif

    integer :: i,k,j,m,lchnk         ! indices
    integer :: ncol                  ! number of columns in current chunk
    integer :: lats(pcols)           ! array of latitude indices
    integer :: lons(pcols)           ! array of longitude indices
    integer :: tsize                 ! amount of data per grid point passed to physics
    integer :: bpter(plon,0:plev)    ! offsets into block buffer for packing data
    integer :: cpter(pcols,0:pver)   ! offsets into chunk buffer for unpacking data
    logical :: wetq(ppcnst)          ! 'moist-type' constituent flag
!-----------------------------------------------------------------------

! Determine which constituents are wet and which are dry
    do m=2,ppcnst
       if (cnst_get_type_byind(m).eq.'wet') then  
          wetq(m) = .true.
       else
          wetq(m) = .false.
       endif
    enddo
!-----------------------------------------------------------------------
! copy data from dynamics data structure to physics data structure
!-----------------------------------------------------------------------
    if (local_dp_map) then

!$OMP PARALLEL DO PRIVATE (LCHNK, NCOL, I, K, M, LONS, LATS)
       do lchnk = begchunk,endchunk
          ncol = get_ncols_p(lchnk)
          call get_lon_all_p(lchnk, ncol, lons)
          call get_lat_all_p(lchnk, ncol, lats)
          phys_state(lchnk)%ncol  = ncol
          phys_state(lchnk)%lchnk = lchnk

          do i=1,ncol
             phys_state(lchnk)%ps   (i)     = ps  (lons(i),lats(i))
             phys_state(lchnk)%phis (i)     = phis(lons(i),lats(i))
!#ifdef TDK_CU
#if (defined TDK_CU) || (defined WU_CONV)
             dyn_tend(lchnk)%pstend (i)     = pstend (lons(i),lats(i))
#endif
          end do
  
          do k=1,plev
             do i=1,ncol
                phys_state(lchnk)%t    (i,k)   = t3  (lons(i),k,lats(i))
                phys_state(lchnk)%u    (i,k)   = u3  (lons(i),k,lats(i))
                phys_state(lchnk)%v    (i,k)   = v3  (lons(i),k,lats(i))
                phys_state(lchnk)%omega(i,k)   = omga(lons(i),k,lats(i))
                phys_state(lchnk)%q(i,k,1)     = q3  (lons(i),k,1,lats(i))
#ifdef WU_CONV 
                phys_state(lchnk)%t1   (i,k)   = t31 (lons(i),k,lats(i))
                phys_state(lchnk)%t2   (i,k)   = t32 (lons(i),k,lats(i))
                phys_state(lchnk)%q1(i,k)      = q31 (lons(i),k,lats(i))
                phys_state(lchnk)%q2(i,k)      = q32 (lons(i),k,lats(i))
                phys_state(lchnk)%tobs(i,k)    = t3obs(lons(i),k,lats(i))
                phys_state(lchnk)%qobs(i,k)    = q3obs(lons(i),k,lats(i))
                phys_state(lchnk)%uobs(i,k)    = u3obs(lons(i),k,lats(i))
                phys_state(lchnk)%vobs(i,k)    = v3obs(lons(i),k,lats(i))
                phys_state(lchnk)%vtobs(i,k)    = vt3obs(lons(i),k,lats(i))
                phys_state(lchnk)%dvobs(i,k)    = dv3obs(lons(i),k,lats(i))
#endif
#ifdef TDK_CU
                dyn_tend(lchnk)%ttend  (i,k)   = ttend(lons(i),k,lats(i))
                dyn_tend(lchnk)%utend  (i,k)   = utend(lons(i),k,lats(i))
                dyn_tend(lchnk)%vtend  (i,k)   = vtend(lons(i),k,lats(i))
                dyn_tend(lchnk)%tatend (i,k)   = tatend(lons(i),k,lats(i))
#endif

             end do
          end do

          ! convert moist-type constituents from dry to moist mixing ratio
          do m=2,ppcnst
             if (wetq(m)) then  
                do k=1,plev
                   do i=1,ncol
                      phys_state(lchnk)%q(i,k,m) = q3(lons(i),k,m,lats(i))*(1. - q3(lons(i),k,1,lats(i)))
                   end do
                end do
             else
                do k=1,plev
                   do i=1,ncol
                      phys_state(lchnk)%q(i,k,m) = q3(lons(i),k,m,lats(i))
                   end do
                end do
             endif
          end do
#ifdef TDK_CU
          do m=1,pcnst+pnats
             do k=1,plev
                do i=1,ncol
                   dyn_tend(lchnk)%qtend(i,k,m) = qtend(lons(i),k,m,lats(i))
                end do
             end do
          end do
          do m=1,pcnst
             do k=1,plev
                do i=1,ncol                   
                   dyn_tend(lchnk)%qatend(i,k,m) = qatend(lons(i),k,m,lats(i))
                end do
             end do
          end do    
#endif
       end do

    else

#ifdef TDK_CU
       tsize = 4 + ppcnst + 4 + (pcnst+pnats) + pcnst 
#else
#ifdef WU_CONV
!---------------------------
       tsize = 14 + ppcnst ! jwh (add 6 vars)
!---------------------------------
#else 
       tsize = 4 + ppcnst
#endif
#endif

       if (tsize*max(block_buf_nrecs,chunk_buf_nrecs) > spmdbuf_siz) then
#if (defined SPMD)
          call spmdbuf_resize(tsize*max(block_buf_nrecs,chunk_buf_nrecs))
#endif
       endif

!$OMP PARALLEL DO PRIVATE (J, BPTER, I, K, M)
       do j=beglat,endlat

          call block_to_chunk_send_pters(j,plon,plev+1,tsize,bpter)

!DIR$ CONCURRENT
          do i=1,nlon(j)

             buf1(bpter(i,0))   = ps  (i,j)
             buf1(bpter(i,0)+1) = phis(i,j)

#if (defined TDK_CU) || (defined WU_CONV)
             buf1(bpter(i,0)+2) = pstend(i,j)   
#endif

!DIR$ CONCURRENT
             do k=1,plev

                buf1(bpter(i,k))   = t3  (i,k,j)
                buf1(bpter(i,k)+1) = u3  (i,k,j)
                buf1(bpter(i,k)+2) = v3  (i,k,j)
                buf1(bpter(i,k)+3) = omga(i,k,j)
                buf1(bpter(i,k)+4) = q3  (i,k,1,j)
#ifdef WU_CONV
                buf1(bpter(i,k)+5) = q31 (i,k,j)
                buf1(bpter(i,k)+6) = t31 (i,k,j)
                buf1(bpter(i,k)+7) = q32 (i,k,j)
                buf1(bpter(i,k)+8) = t32 (i,k,j)
                buf1(bpter(i,k)+9) = t3obs (i,k,j)      ! jwh
                buf1(bpter(i,k)+10) = q3obs (i,k,j)
                buf1(bpter(i,k)+11) = u3obs (i,k,j)
                buf1(bpter(i,k)+12) = v3obs (i,k,j)
                buf1(bpter(i,k)+13) = vt3obs (i,k,j)
                buf1(bpter(i,k)+14) = dv3obs (i,k,j)
#endif
                ! convert moist-type constituents from dry to moist mixing ratio

                do m=2,ppcnst
                   if (wetq(m)) then  
#ifdef WU_CONV
!------------------------------
                     buf1(bpter(i,k)+13+m) = q3(i,k,m,j)*(1. - q3(i,k,1,j))
!------------------
#else
                      buf1(bpter(i,k)+3+m) = q3(i,k,m,j)*(1. - q3(i,k,1,j))
#endif
                   else
#ifdef WU_CONV
                      buf1(bpter(i,k)+13+m) = q3(i,k,m,j)
#else
                      buf1(bpter(i,k)+3+m) = q3(i,k,m,j)
#endif
                   endif
                end do
#ifdef TDK_CU
                buf1(bpter(i,k)+3+ppcnst+1) = ttend  (i,k,j)
                buf1(bpter(i,k)+3+ppcnst+2) = utend  (i,k,j)
                buf1(bpter(i,k)+3+ppcnst+3) = vtend  (i,k,j)
                buf1(bpter(i,k)+3+ppcnst+4) = tatend (i,k,j)

                do m=1,pcnst+pnats
                   buf1(bpter(i,k)+3+ppcnst+4+m) = qtend  (i,k,m,j)
                end do
                do m=1,pcnst
                   buf1(bpter(i,k)+3+ppcnst+4+pcnst+pnats+m) = qatend (i,k,m,j)
                end do
#endif
             end do

          end do

       end do

       call transpose_block_to_chunk(tsize, buf1, buf2)

!$OMP PARALLEL DO PRIVATE (LCHNK, NCOL, CPTER, I, K, M)
       do lchnk = begchunk,endchunk
          ncol = get_ncols_p(lchnk)
          phys_state(lchnk)%ncol  = ncol
          phys_state(lchnk)%lchnk = lchnk

          call block_to_chunk_recv_pters(lchnk,pcols,plev+1,tsize,cpter)

          do i=1,ncol

             phys_state(lchnk)%ps   (i)     = buf2(cpter(i,0))
             phys_state(lchnk)%phis (i)     = buf2(cpter(i,0)+1)
!#ifdef TDK_CU
#if (defined TDK_CU) || (defined WU_CONV)
             dyn_tend(lchnk)%pstend (i)     = buf2(cpter(i,0)+2)  
#endif
             do k=1,plev

                phys_state(lchnk)%t    (i,k)   = buf2(cpter(i,k))
                phys_state(lchnk)%u    (i,k)   = buf2(cpter(i,k)+1)
                phys_state(lchnk)%v    (i,k)   = buf2(cpter(i,k)+2)
                phys_state(lchnk)%omega (i,k)   = buf2(cpter(i,k)+3)
#ifdef WU_CONV
                phys_state(lchnk)%q   (i,k,1)   = buf2(cpter(i,k)+4)
                phys_state(lchnk)%q1    (i,k)   = buf2(cpter(i,k)+5)
                phys_state(lchnk)%t1    (i,k)   = buf2(cpter(i,k)+6)
                phys_state(lchnk)%q2    (i,k)   = buf2(cpter(i,k)+7)
                phys_state(lchnk)%t2    (i,k)   = buf2(cpter(i,k)+8)
                phys_state(lchnk)%tobs  (i,k)   = buf2(cpter(i,k)+9)  ! jwh
                phys_state(lchnk)%qobs  (i,k)   = buf2(cpter(i,k)+10)
                phys_state(lchnk)%uobs  (i,k)   = buf2(cpter(i,k)+11)
                phys_state(lchnk)%vobs  (i,k)   = buf2(cpter(i,k)+12)
                phys_state(lchnk)%vtobs  (i,k)   = buf2(cpter(i,k)+13)
                phys_state(lchnk)%dvobs  (i,k)   = buf2(cpter(i,k)+14)
                do m=2,ppcnst
                   phys_state(lchnk)%q (i,k,m) = buf2(cpter(i,k)+13+m)
                end do
#else
                do m=1,ppcnst
                   phys_state(lchnk)%q (i,k,m) = buf2(cpter(i,k)+3+m)
                end do
#endif

#ifdef TDK_CU
                dyn_tend(lchnk)%ttend  (i,k)   = buf2(cpter(i,k)+3+ppcnst+1)
                dyn_tend(lchnk)%utend  (i,k)   = buf2(cpter(i,k)+3+ppcnst+2)
                dyn_tend(lchnk)%vtend  (i,k)   = buf2(cpter(i,k)+3+ppcnst+3)
                dyn_tend(lchnk)%tatend (i,k)   = buf2(cpter(i,k)+3+ppcnst+4)
                do m=1,pcnst+pnats
                   dyn_tend(lchnk)%qtend(i,k,m)= buf2(cpter(i,k)+3+ppcnst+4+m) 
                end do
                do m=1,pcnst
                   dyn_tend(lchnk)%qatend(i,k,m)=buf2(cpter(i,k)+3+ppcnst+4+pcnst+pnats+m) 
                end do
#endif
             end do

          end do

       end do

    endif

!-----------------------------------------------------------------------
! Fill auxilliary arrays in physics data structure
!-----------------------------------------------------------------------
!$OMP PARALLEL DO PRIVATE (LCHNK, NCOL, I, K, M, LONS, LATS)

    do lchnk = begchunk,endchunk
       ncol = phys_state(lchnk)%ncol

! pressure arrays
       call plevs0(ncol, pcols, pver, &
                   phys_state(lchnk)%ps,   phys_state(lchnk)%pint,    &
                   phys_state(lchnk)%pmid, phys_state(lchnk)%pdel)

! log(pressure) arrays and Exner function
       do k=1,pver+1
          do i=1,ncol
             phys_state(lchnk)%lnpint(i,k) = log(phys_state(lchnk)%pint(i,k))
          end do
       end do
       do k=1,pver
          do i=1,ncol
             phys_state(lchnk)%rpdel(i,k)  = 1./phys_state(lchnk)%pdel(i,k)
             phys_state(lchnk)%lnpmid(i,k) = log(phys_state(lchnk)%pmid(i,k))
             phys_state(lchnk)%exner (i,k) = (phys_state(lchnk)%pint(i,pver+1) &
                                             / phys_state(lchnk)%pmid(i,k))**cappa
          end do
       end do

! Compute initial geopotential heights
       call geopotential_t (phys_state(lchnk)%lnpint, phys_state(lchnk)%lnpmid  , phys_state(lchnk)%pint  , &
                            phys_state(lchnk)%pmid  , phys_state(lchnk)%pdel    , phys_state(lchnk)%rpdel , &
                            phys_state(lchnk)%t     , phys_state(lchnk)%q(1,1,1), rair,  gravit,  zvir    , &
                            phys_state(lchnk)%zi    , phys_state(lchnk)%zm      , ncol                )

! Compute initial dry static energy, include surface geopotential
       do k = 1, pver
          do i=1,ncol
             phys_state(lchnk)%s(i,k) = cpair*phys_state(lchnk)%t(i,k) &
                                      + gravit*phys_state(lchnk)%zm(i,k) + phys_state(lchnk)%phis(i)
          end do
       end do

! Compute energy and water integrals of input state
       call check_energy_timestep_init(phys_state(lchnk), phys_tend(lchnk), pbuf)

    end do

    return
  end subroutine d_p_coupling

!===============================================================================
#ifdef TDK_CU
  subroutine p_d_coupling(phys_state, phys_tend, t2, fu, fv, flx_net, qminus, qnats, t2rad)
#else
#ifdef WU_CONV
  subroutine p_d_coupling(phys_state, phys_tend, t2, fu, fv, flx_net, qminus, qnats, &
                          q31, t31, t3obs, q3obs, u3obs, v3obs, vt3obs, dv3obs) !jwh 
#else
  subroutine p_d_coupling(phys_state, phys_tend, t2, fu, fv, flx_net, qminus, qnats)
#endif
#endif
!------------------------------------------------------------------------------
! Coupler for converting physics output variables into dynamics input variables
!------------------------------Arguments--------------------------------
    use constituents, only: cnst_get_type_byind

    type(physics_state),intent(in), dimension(begchunk:endchunk) :: phys_state
    type(physics_tend), intent(in), dimension(begchunk:endchunk) :: phys_tend

    real(r8), intent(out) :: t2(plond, plev, beglat:endlat)        ! temp tendency
    real(r8), intent(out) :: fu(plond, plev, beglat:endlat)        ! u wind tendency
    real(r8), intent(out) :: fv(plond, plev, beglat:endlat)        ! v wind tendency
    real(r8), intent(out) :: flx_net(plond,beglat:endlat)          ! net flux
    real(r8), intent(out) :: qminus(plond, plev, pcnst, beglat:endlat) ! constituents
    real(r8), intent(out) :: qnats(plond, plev, ppcnst, beglat:endlat) ! non-adv constituents
#ifdef WU_CONV
    real(r8), intent(out) :: t31(plond, plev, beglat:endlat)       ! temp (K)
    real(r8), intent(out) :: q31(plond, plev, beglat:endlat)       ! moisture
    real(r8), intent(out) :: t3obs(plond, plev, beglat:endlat)     ! new T (jwh)
    real(r8), intent(out) :: q3obs(plond, plev, beglat:endlat)       ! new Q
    real(r8), intent(out) :: u3obs(plond, plev, beglat:endlat)       ! new U
    real(r8), intent(out) :: v3obs(plond, plev, beglat:endlat)       ! new V
    real(r8), intent(out) :: vt3obs(plond, plev, beglat:endlat)       ! new VORT
    real(r8), intent(out) :: dv3obs(plond, plev, beglat:endlat)       ! new DIV
#endif
#ifdef TDK_CU
    real(r8), intent(out) :: t2rad(plond, plev, beglat:endlat)     ! rad temp tendency
#endif
!
!---------------------------Local workspace-----------------------------
#if (! defined SPMD)
    real(r8) :: buf1(1), buf2(1)     ! transpose buffers
    integer  :: spmdbuf_siz
#endif

    integer :: i,j,k,m,lchnk         ! indices
    integer :: ncol                  ! number of columns in current chunk
    integer :: lats(pcols)           ! array of latitude indices
    integer :: lons(pcols)           ! array of longitude indices
    integer :: tsize                 ! amount of data per grid point passed to physics
    integer :: bpter(plon,0:plev)    ! offsets into block buffer for packing data
    integer :: cpter(pcols,0:pver)   ! offsets into chunk buffer for unpacking data
    logical :: wetq(ppcnst)          ! 'wet' constituent flag
!-----------------------------------------------------------------------

! Determine which constituents are wet and which are dry
    do m=2,ppcnst
       if (cnst_get_type_byind(m).eq.'wet') then  
          wetq(m) = .true.
       else
          wetq(m) = .false.
       endif
    enddo
!-----------------------------------------------------------------------
! copy data from physics data structure to dynamics data structure
!-----------------------------------------------------------------------
    if (local_dp_map) then

!$OMP PARALLEL DO PRIVATE (LCHNK, NCOL, I, K, M, LONS, LATS)

       do lchnk = begchunk,endchunk
          ncol = get_ncols_p(lchnk)
          call get_lon_all_p(lchnk, ncol, lons)
          call get_lat_all_p(lchnk, ncol, lats)

          do k=1,plev
!DIR$ CONCURRENT
             do i=1,ncol
                t2(lons(i),k,lats(i))   = phys_tend(lchnk)%dTdt (i,k)
                fu(lons(i),k,lats(i))   = phys_tend(lchnk)%dudt (i,k)
                fv(lons(i),k,lats(i))   = phys_tend(lchnk)%dvdt (i,k)
                qminus(lons(i),k,1,lats(i)) = phys_state(lchnk)%q(i,k,1)
#ifdef WU_CONV
                t31(lons(i),k,lats(i))   = phys_state(lchnk)%t1 (i,k)
                q31(lons(i),k,lats(i))   = phys_state(lchnk)%q1 (i,k)
                t3obs(lons(i),k,lats(i)) = phys_state(lchnk)%tobs (i,k)  ! jwh
                q3obs(lons(i),k,lats(i)) = phys_state(lchnk)%qobs (i,k)
                u3obs(lons(i),k,lats(i)) = phys_state(lchnk)%uobs (i,k)
                v3obs(lons(i),k,lats(i)) = phys_state(lchnk)%vobs (i,k)
                vt3obs(lons(i),k,lats(i)) = phys_state(lchnk)%vtobs (i,k)
                dv3obs(lons(i),k,lats(i)) = phys_state(lchnk)%dvobs (i,k)
#endif
#ifdef TDK_CU
                t2rad(lons(i),k,lats(i))= phys_tend(lchnk)%dTrdt (i,k)
#endif
             end do
          end do

!DIR$ CONCURRENT
          do i=1,ncol
             flx_net(lons(i),lats(i))   = phys_tend(lchnk)%flx_net(i)
          end do
          
          ! convert moist-type constituents from dry to moist mixing ratio
          do m=2,pcnst
             if (wetq(m)) then  
                do k=1,plev
!DIR$ CONCURRENT
                   do i=1,ncol
                      qminus(lons(i),k,m,lats(i)) = phys_state(lchnk)%q(i,k,m) /     &
                           (1. - phys_state(lchnk)%q(i,k,1))
                   end do
                end do
             else
                do k=1,plev
!DIR$ CONCURRENT
                   do i=1,ncol
                      qminus(lons(i),k,m,lats(i)) = phys_state(lchnk)%q(i,k,m)
                   end do
                end do
             endif
          end do

!----------------------------------------
! wtw 200506 modified
!
        if( ppcnst .ge. (pcnst+1) ) then  ! wtw 200508 add
          do m= pcnst+1, ppcnst
             if (wetq(m)) then  
                do k=1,plev
                   do i=1,ncol
                      qnats(lons(i),k,m,lats(i)) = phys_state(lchnk)%q(i,k,m) /     &
                                                   (1. - phys_state(lchnk)%q(i,k,1))
                   end do
                end do
             else 
                do k=1,plev
                   do i=1,ncol
                      qnats(lons(i),k,m,lats(i)) = phys_state(lchnk)%q(i,k,m)
                   end do
                end do
              endif 
          end do

         endif   ! wtw 200508 add

       end do

    else

#ifdef TDK_CU
       tsize = 3 + ppcnst + 1
#else
#ifdef WU_CONV
       tsize = 11 + ppcnst ! jwh
#else
       tsize = 3 + ppcnst
#endif
#endif

       if (tsize*max(block_buf_nrecs,chunk_buf_nrecs) > spmdbuf_siz) then
#if (defined SPMD)
          call spmdbuf_resize(tsize*max(block_buf_nrecs,chunk_buf_nrecs))
#endif
       endif

!$OMP PARALLEL DO PRIVATE (LCHNK, NCOL, CPTER, I, K, M)
       do lchnk = begchunk,endchunk
          ncol = get_ncols_p(lchnk)

          call chunk_to_block_send_pters(lchnk,pcols,pver+1,tsize,cpter)

!DIR$ CONCURRENT
          do i=1,ncol

             buf2(cpter(i,0)) = phys_tend(lchnk)%flx_net(i)

!DIR$ CONCURRENT
             do k=1,plev

                buf2(cpter(i,k))   = phys_tend(lchnk)%dTdt (i,k)
                buf2(cpter(i,k)+1) = phys_tend(lchnk)%dudt (i,k)
                buf2(cpter(i,k)+2) = phys_tend(lchnk)%dvdt (i,k)
                buf2(cpter(i,k)+3) = phys_state(lchnk)%q(i,k,1)
#ifdef WU_CONV
                buf2(cpter(i,k)+4) = phys_state(lchnk)%t1(i,k)
                buf2(cpter(i,k)+5) = phys_state(lchnk)%q1(i,k)

                buf2(cpter(i,k)+6) = phys_state(lchnk)%tobs (i,k)  ! jwh
                buf2(cpter(i,k)+7) = phys_state(lchnk)%qobs (i,k)  ! jwh
                buf2(cpter(i,k)+8) = phys_state(lchnk)%uobs (i,k)  ! jwh
                buf2(cpter(i,k)+9) = phys_state(lchnk)%vobs (i,k)  ! jwh
                buf2(cpter(i,k)+10) = phys_state(lchnk)%vtobs (i,k)  ! jwh
                buf2(cpter(i,k)+11) = phys_state(lchnk)%dvobs (i,k)  ! jwh
#endif
                ! convert moist-type constituents from dry to moist mixing ratio

                do m=2,ppcnst
                   if (wetq(m)) then  
#ifdef WU_CONV
                      buf2(cpter(i,k)+10+m) = phys_state(lchnk)%q(i,k,m) /     &
                           (1. - phys_state(lchnk)%q(i,k,1))
#else
                      buf2(cpter(i,k)+2+m) = phys_state(lchnk)%q(i,k,m) /     &
                           (1. - phys_state(lchnk)%q(i,k,1))
#endif
                   else
#ifdef WU_CONV
                      buf2(cpter(i,k)+10+m) = phys_state(lchnk)%q(i,k,m)
#else
                      buf2(cpter(i,k)+2+m) = phys_state(lchnk)%q(i,k,m)
#endif
                   endif
		enddo
#ifdef TDK_CU
                buf2(cpter(i,k)+2+ppcnst+1) = phys_tend(lchnk)%dTrdt (i,k)
#endif

             end do

          end do

       end do

       call transpose_chunk_to_block(tsize, buf2, buf1)

!$OMP PARALLEL DO PRIVATE (J, BPTER, I, K, M)
       do j=beglat,endlat

          call chunk_to_block_recv_pters(j,plon,plev+1,tsize,bpter)

          do i=1,nlon(j)

             flx_net(i,j) = buf1(bpter(i,0))

             do k=1,plev

                t2(i,k,j) = buf1(bpter(i,k))
                fu(i,k,j) = buf1(bpter(i,k)+1)
                fv(i,k,j) = buf1(bpter(i,k)+2)
#ifdef WU_CONV
                qminus(i,k,1,j) = buf1(bpter(i,k)+3)
                t31(i,k,j) = buf1(bpter(i,k)+4)
                q31(i,k,j) = buf1(bpter(i,k)+5)
                t3obs(i,k,j) = buf1(bpter(i,k)+6)  ! jwh
                q3obs(i,k,j) = buf1(bpter(i,k)+7)
                u3obs(i,k,j) = buf1(bpter(i,k)+8)
                v3obs(i,k,j) = buf1(bpter(i,k)+9)
                vt3obs(i,k,j) = buf1(bpter(i,k)+10)
                dv3obs(i,k,j) = buf1(bpter(i,k)+11)
                do m=2,pcnst
                   qminus(i,k,m,j) = buf1(bpter(i,k)+10+m)
                end do
                do m=pcnst+1,ppcnst
                   qnats(i,k,m,j)  = buf1(bpter(i,k)+10+m)
                end do
#else
                do m=1,pcnst
                   qminus(i,k,m,j) = buf1(bpter(i,k)+2+m)
                end do
                do m=pcnst+1,ppcnst
                   qnats(i,k,m,j)  = buf1(bpter(i,k)+2+m)
                end do
#endif
#ifdef TDK_CU
                t2rad(i,k,j) = buf1(bpter(i,k)+2+ppcnst+1)
#endif

             end do

          end do

       end do

    endif

  end subroutine p_d_coupling
end module dp_coupling

