#include <misc.h>
#include <params.h>

module prognostics

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Prognostic variables held in-core for convenient access.
! q3 is specific humidity (water vapor) and other constituents.
! pcnst is advected constituents, pnats is non-advected.
! 
! Author: G. Grant
! 
!-----------------------------------------------------------------------

   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid 
   use infnan
   use constituents, only: pcnst, pnats

   implicit none

   integer, parameter :: ptimelevels = 3  ! number of time levels in the dycore
   integer :: n3   = 3
   integer :: n3m1 = 2
   integer :: n3m2 = 1

   real(r8), allocatable :: ps(:,:,:)
   real(r8), allocatable :: u3(:,:,:,:)
   real(r8), allocatable :: v3(:,:,:,:)
   real(r8), allocatable :: t3(:,:,:,:)
   real(r8), allocatable :: q3(:,:,:,:,:)
   real(r8), allocatable :: qminus(:,:,:,:)
   real(r8), allocatable :: qminustp(:,:,:,:) !only for tspas

#ifdef WU_CONV
   real(r8), allocatable :: t31(:,:,:)
   real(r8), allocatable :: t32(:,:,:)
   real(r8), allocatable :: q31(:,:,:)
   real(r8), allocatable :: q32(:,:,:)
   real(r8), allocatable :: t3obs(:,:,:)    ! jwh
   real(r8), allocatable :: q3obs(:,:,:)
   real(r8), allocatable :: u3obs(:,:,:)
   real(r8), allocatable :: v3obs(:,:,:)
   real(r8), allocatable :: vt3obs(:,:,:)
   real(r8), allocatable :: dv3obs(:,:,:)
#ifdef BCC_DRAG
   real(r8), allocatable :: frontgf3(:,:,:)
   real(r8), allocatable :: frontga3(:,:,:)
#endif
#endif
   real(r8), allocatable :: vort(:,:,:,:)   ! vorticity
   real(r8), allocatable :: div(:,:,:,:)    ! divergence

   real(r8), allocatable :: dpsl(:,:)       ! longitudinal pressure gradient
   real(r8), allocatable :: dpsm(:,:)       ! meridional pressure gradient
   real(r8), allocatable :: dps(:,:)        ! pressure gradient
   real(r8), allocatable :: phis(:,:)       ! surface geopotential
   real(r8), allocatable :: omga(:,:,:)     ! vertical velocity

   real(r8), allocatable :: pstend(:,:)   
#ifdef TDK_CU
   real(r8), allocatable :: utend (:,:,:)   
   real(r8), allocatable :: vtend (:,:,:)   
   real(r8), allocatable :: ttend (:,:,:)   
   real(r8), allocatable :: qtend (:,:,:,:)   
   real(r8), allocatable :: tatend(:,:,:)   
   real(r8), allocatable :: qatend(:,:,:,:)   
#endif
!
! wu tongwen in May, 2004 added 
!
   real(r8), allocatable :: phisgrid(:,:)       ! surface geopotential
   real(r8), allocatable :: phisspec(:,:)       ! surface geopotential

   real(r8), allocatable :: lpsbar(:,:)       ! surface pressure (Pa)
   real(r8), allocatable :: lpsbardl(:,:)       ! surface pressure (Pa)
   real(r8), allocatable :: lpsbardm(:,:)       ! surface pressure (Pa)
   real(r8), allocatable :: lpsbard2(:,:)       ! surface pressure (Pa)
   real(r8), allocatable :: lpsbard4(:,:)       ! surface pressure (Pa)

   real(r8), allocatable :: psex(:,:)         ! extended ps for FFSL

   ! zhangyi adds for tspas
   real(r8), allocatable :: omega(:,:,:,:)  ! vertical velocity

CONTAINS

   subroutine initialize_prognostics
!
! Purpose:  Allocate and initialize the prognostic arrays.
!
      allocate (ps    (plond                 ,beglat:endlat    ,ptimelevels))
      allocate (u3    (plond,plev            ,beglatex:endlatex,ptimelevels))
      allocate (v3    (plond,plev            ,beglatex:endlatex,ptimelevels))
      allocate (t3    (plond,plev            ,beglatex:endlatex,ptimelevels))
      allocate (q3    (plond,plev,pcnst+pnats,beglatex:endlatex,ptimelevels))
      allocate (qminus  (plond,plev,pcnst      ,beglatex:endlatex  ))
      allocate (qminustp(plond,plev,pcnst      ,beglatex:endlatex  ))

#ifdef WU_CONV
      allocate (t31   (plond,plev            ,beglatex:endlatex))
      allocate (t32   (plond,plev            ,beglatex:endlatex))
      allocate (q31   (plond,plev            ,beglatex:endlatex))
      allocate (q32   (plond,plev            ,beglatex:endlatex))
      allocate (t3obs   (plond,plev            ,beglatex:endlatex))  !jwh
      allocate (q3obs   (plond,plev            ,beglatex:endlatex))
      allocate (u3obs   (plond,plev            ,beglatex:endlatex))
      allocate (v3obs   (plond,plev            ,beglatex:endlatex))
      allocate (vt3obs   (plond,plev            ,beglatex:endlatex))
      allocate (dv3obs   (plond,plev            ,beglatex:endlatex))
#ifdef BCC_DRAG
      allocate (frontgf3 (plond,plev           ,beglatex:endlatex))
      allocate (frontga3 (plond,plev           ,beglatex:endlatex))
#endif
#endif
      allocate (vort  (plond,plev,beglat:endlat,ptimelevels))   
      allocate (div   (plond,plev,beglat:endlat,ptimelevels))    

      allocate (dpsl  (plond,beglat:endlat))        
      allocate (dpsm  (plond,beglat:endlat))        
      allocate (dps   (plond,beglat:endlat))         
      allocate (phis  (plond,beglat:endlat))        
      allocate (omga  (plond,plev,beglat:endlat))    

      allocate (pstend   (plond                 ,beglat:endlat    ))        
#ifdef TDK_CU
      allocate (utend    (plond,plev            ,beglatex:endlatex))
      allocate (vtend    (plond,plev            ,beglatex:endlatex)) 
      allocate (ttend    (plond,plev            ,beglatex:endlatex)) 
      allocate (qtend    (plond,plev,pcnst+pnats,beglatex:endlatex))  
      allocate (tatend   (plond,plev            ,beglatex:endlatex))
      allocate (qatend   (plond,plev,pcnst      ,beglatex:endlatex))  
#endif
!
! wu tongwen in May, 2004 added 
!
      allocate (phisgrid  (plond,beglat:endlat))        
      allocate (phisspec  (plond,beglat:endlat))        

      allocate (lpsbar    (plond,beglat:endlat))        
      allocate (lpsbardl  (plond,beglat:endlat))        
      allocate (lpsbardm  (plond,beglat:endlat))        
      allocate (lpsbard2  (plond,beglat:endlat))        
      allocate (lpsbard4  (plond,beglat:endlat))        

      allocate (psex  (plond,beglatex:endlatex) )

      ! zhangyi adds for tspas
      allocate (omega (plon ,plevp,beglat:endlat,ptimelevels))
!
      ps(:,:,:)       = inf
      u3(:,:,:,:)     = inf
      v3(:,:,:,:)     = inf
      t3(:,:,:,:)     = inf
      q3(:,:,:,:,:)   = inf
      qminus(:,:,:,:) = inf
      qminustp(:,:,:,:) = inf

#ifdef WU_CONV
      t31(:,:,:)      = inf
      t32(:,:,:)      = inf
      q31(:,:,:)      = inf
      q32(:,:,:)      = inf
      t3obs(:,:,:)    = inf  ! jwh
      q3obs(:,:,:)    = inf
      u3obs(:,:,:)    = inf  
      v3obs(:,:,:)    = inf  
      vt3obs(:,:,:)   = inf  
      dv3obs(:,:,:)   = inf  
#ifdef BCC_DRAG
      frontgf3(:,:,:)  = 0.0
      frontga3(:,:,:)  = 0.0
#endif
#endif
      vort(:,:,:,:)   = inf
      div (:,:,:,:)   = inf

      dpsl  (:,:) = inf
      dpsm  (:,:) = inf
      dps   (:,:) = inf
      phis  (:,:) = inf
      omga  (:,:,:) = inf

      pstend(:,:)      = 0.0 
#ifdef TDK_CU
      utend (:,:,:)    = 0.0
      vtend (:,:,:)    = 0.0
      ttend (:,:,:)    = 0.0
      qtend (:,:,:,:)  = 0.0 
      tatend(:,:,:)    = 0.0
      qatend(:,:,:,:)  = 0.0 
#endif
!
! wu tongwen in May, 2004 added 
!
      phisgrid  (:,:) = inf
      phisspec  (:,:) = inf

      lpsbar   (:,:) = inf
      lpsbardl (:,:) = inf
      lpsbardm (:,:) = inf
      lpsbard2 (:,:) = inf
      lpsbard4 (:,:) = inf

      psex  (:,:)      = inf
!
      ! zhangyi adds for tspas
      omega (:,:,:,:) = inf

      return
   end subroutine initialize_prognostics

   subroutine shift_time_indices
!
! Purpose: 
! Shift the indices that keep track of which index stores
! the relative times (current time, previous, time before previous etc).
!
      integer :: itmp

      itmp = n3m2

      n3m2 = n3m1
      n3m1 = n3
      n3   = itmp
   end subroutine shift_time_indices

end module prognostics
