#include <misc.h>
#include <params.h>

subroutine scanffsl(ztodt   ,pmap    ,etadot  ,kdpmpf  ,kdpmph  , &
                   lam     ,phi     ,dphi    ,sinlam  ,coslam  ,  &
                   lbasdy  ,lbasdz  ,lbassd  ,lbasiy  ,detam   ,  &
                   detai   ,dlam    ,cwava   ,etamid  ,etaint, grid  )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Driving routine for Flux Form Semi-Lagrangian (FFSL) transport.
! 
! Method: 
! The latitude loop in this routine is multitasked.
! 
! Author: 
! Original version:  Zhang Yi, 2012.08
! Reviewed:          Wang Zaizhi, 2012.10
!
!-----------------------------------------------------------------------
!
! $Id: scanffsl.F90,v 1.4 2015/11/18 07:50:44 wangzzh Exp $
! $Author: wangzzh $
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use shr_const_mod,only: pi=>shr_const_pi, ae=>shr_const_rearth
   use pmgrid,      only: plon, plond, plev, plevp, plat, platd, beglat, endlat, beglatex, &
                          endlatex, i1, j1
   use constituents,only: pcnst
   use comslt,      only: qfcst, lammp, phimp, sigmp, hw1lat, trcavg
   use prognostics, only: u3, v3, qminus, ps, n3m2, n3m1, q3, n3, psex
   use rgrid,       only: nlon, nlonex
   use commap,      only: w
   use dynconst,    only: ra
   use time_manager,only: get_nstep

#if ( defined SCAM )
   use physconst, only: gravit
#endif

#if (defined SPMD)
# if ( defined TIMING_BARRIERS )
   use mpishorthand, only: mpicom
# endif
#endif
! For FFSL
   use dynamics_vars,only: T_FVDYCORE_GRID
   use acd_grid,     only: a2c3d
   use ffsl_mod,     only: ffsl_run
!-----------------------------------------------------------------------
   implicit none
!
! Input arguments
!
   real(r8), intent(in) :: ztodt              ! twice the time step unless nstep = 0
!
   integer, intent(in) :: pmap                ! dimension of artificial array
!                                 !  used to locate vertical interval
!                                 !  in which departure point falls
   real(r8), intent(in) :: etadot(plon,plevp,beglat:endlat)! vertical motion (slt)
!
   integer, intent(in) :: kdpmpf(pmap)        ! mapping from artificial grid to
!                                 ! model levels
   integer, intent(in) :: kdpmph(pmap)        ! mapping from artificial grid to
!                                 ! interfaces
   real(r8), intent(in) :: lam(plond,platd)       ! longitude coordinates of model grid
   real(r8), intent(in) :: phi(platd)             ! latitude  coordinates of model grid
   real(r8), intent(in) :: dphi(platd)            ! latitudinal grid increments
   real(r8), intent(in) :: sinlam(plond,platd)    ! sine of longitude
   real(r8), intent(in) :: coslam(plond,platd)    ! cosine of longitude
   real(r8), intent(in) :: lbasdy(4,2,platd)      ! basis functions for lat deriv est.
   real(r8), intent(in) :: lbasdz(4,2,plev)       ! basis functions for vert deriv est.
!                                 !  (levels)
   real(r8), intent(in) :: lbassd(4,2,plevp)  ! basis functions for vert deriv est.
!                                 !  (levels)
   real(r8), intent(in) :: lbasiy(4,2,platd)  ! basis functions for Lagrange interp
   real(r8), intent(in) :: detam(plev)        ! delta eta at levels
   real(r8), intent(in) :: detai(plevp)       ! delta eta at interfaces
   real(r8), intent(in) :: dlam(platd)        ! longitudinal grid increment
   real(r8), intent(in) :: cwava(plat)        ! weight for global water vapor int.
   real(r8), intent(in) :: etamid(plev)       ! eta at levels
   real(r8), intent(in) :: etaint(plevp)      ! eta at interfaces

! For FFSL
   type(T_FVDYCORE_GRID)    :: grid

!
#include <comctl.h>
!---------------------------Local workspace-----------------------------
!
   integer m
   integer lat                 ! latitude index
   integer irow                ! N/S latitude pair index
   integer jcen                ! lat index (extended grid) of forecast
   integer :: nstep            ! current timestep number

! Dynamic (SPMD) vs stack (shared memory)
!
!  real(r8) uxl(plond,plev,beglatex:endlatex)     ! left  x-deriv of u/v
!  real(r8) uxr(plond,plev,beglatex:endlatex)     ! left  x-deriv of u/v
!  real(r8) vxl(plond,plev,beglatex:endlatex)     ! left  x-deriv of u/v
!  real(r8) vxr(plond,plev,beglatex:endlatex)     ! left  x-deriv of u/v
!  real(r8) qxl(plond,plev,pcnst,beglatex:endlatex) ! left  x-deriv of constituents
!  real(r8) qxr(plond,plev,pcnst,beglatex:endlatex) ! right  x-deriv of constituents
   real(r8) pe(plon,beglatex:endlatex,plevp)
   real(r8) delp(plond,beglatex:endlatex,plev)
   real(r8) uc(plon,beglat-grid%ng_c:endlat+grid%ng_c, plev)
   real(r8) vc(plon,beglat-grid%ng_d:endlat+grid%ng_d, plev)
#if ( defined SCAM )
   real(r8) gw(plat)                ! Gaussian weights
#endif
   integer k, i, j
!------------------------------Externals--------------------------------
!
   external extx,extys,extyv

!
!-----------------------------------------------------------------------
#if ( !defined SCAM )
#if ( defined SPMD )
#ifdef TIMING_BARRIERS
   call t_startf ('sync_bndexch')
   call mpibarrier (mpicom)
   call t_stopf ('sync_bndexch')
#endif
!
! Communicate boundary information 
!
   psex(:,:) = 0.0_r8
   psex(i1:i1+plon-1,beglat+j1-1:endlat+j1-1) = ps(1:plon,beglat:endlat,n3m2)
   call t_startf ('bndexch')
   call ffsl_bndexch
   call t_stopf ('bndexch')
#endif

   nstep = get_nstep()
!
! Initialize extended arrays
!
   call t_startf('sltini')
!  call sltini (dlam,    sinlam,  coslam,  uxl,     uxr, &
!               vxl,     vxr,     qxl,     qxr,     u3,  &
!               v3,      qminus,  n3m1)
! Fill latitude extensions beyond the southern- and northern-most
! latitudes in the global grid
!
   call extyv(1, plev, coslam, sinlam, u3(1,1,beglatex,n3m2), &
                                       v3(1,1,beglatex,n3m2))
   call extys(pcnst, plev    ,qminus, pcnst)
   call extys(  1  , 1       ,psex  , 1)

!$OMP PARALLEL DO PRIVATE (K, I, J)
      do k=1,plevp
         do j = beglatex, endlatex
           do i = 1, nlonex(j)
               pe(i,j,k) = grid%ak(k) + grid%bk(k) * psex(i+i1-1,j)   !  pint
           enddo
         enddo
      enddo

!$OMP PARALLEL DO PRIVATE (K, I, J)
      do k = 1, plev
         do j = beglatex, endlatex
            do i = 1, nlonex(j)
                delp(i,j,k) = pe(i,j,k+1) - pe(i,j,k)
            enddo
         enddo
      enddo
!
!  convert winds on A-grid to C-grid
!

   if ( fullgrid ) then
      call a2c3d(grid, u3(i1:plon+i1-1,1:plev,beglatex:endlatex,n3m2), &
                 v3(i1:plon+i1-1,1:plev,beglatex:endlatex,n3m2), uc ,vc)
   else
! uc, vc for temperal use, wzz
      do k = 1, plev
        uc(1:plon,beglat-grid%ng_c:endlat+grid%ng_c,k) = u3(i1:plon+i1-1,k,beglat+j1-1-grid%ng_c:endlat+j1-1+grid%ng_c,n3m2)
        vc(1:plon,beglat-grid%ng_d:endlat+grid%ng_d,k) = v3(i1:plon+i1-1,k,beglat+j1-1-grid%ng_d:endlat+j1-1+grid%ng_d,n3m2)
      enddo
   endif

   call t_stopf('sltini')
#else
!************************************************************************
!     IF surface pressure changes with time we need to remap the vertical
!     coordinate for the slt advection process.  It has been empirically
!     determined that we can get away with 500 for pmap (instead of 20000)
!     This is necessary to make the procedure computationally feasible
!
!
!   call grdini(pmap    ,etamid    ,etaint    ,gravit  ,dlam    , &
!        lam     ,phi     ,dphi    ,gw      ,sinlam  , &
!        coslam  ,lbasdy  ,lbasdz  ,lbassd  ,lbasiy  , &
!        detam   ,detai   ,kdpmpf  ,kdpmph  ,cwava   )
      
!   ps_prev = ps
!     
! Initial guess for trajectory midpoints in spherical coords.
! nstep = 0:  use arrival points as initial guess for trajectory midpoints.
! nstep > 0:  use calculated trajectory midpoints from previous time
! step as first guess.
!
   do k=1,plev
      sigmp(1,k,beglat) = etamid(k)
   end do

   nstep = get_nstep()
#endif
!
! Initialize moisture mass integrals.
!
   do m=1,pcnst
      do lat=1,plat
         hw1lat(m,lat) = 0.0_r8
      end do
   end do
!
! Loop through latitudes producing forecast
!
   call t_startf('ffslb1')

!  call t_startf('qmassa')
#ifdef OUTER_OMP
!$OMP PARALLEL DO PRIVATE (LAT, IROW, JCEN )
#endif
   do lat=beglat,endlat
      if(lat.le.plat/2) then
         irow = lat
      else
         irow = plat + 1 - lat
      end if
      jcen = j1 - 1 + lat
!
! Calculate mass of moisture in field being advected by slt.
!
!  q3     is plond,plev,pcnst+pnats,beglattex:endlatex,ptimelevs
!  qminus is plond,plev,pcnst+pnats,beglattex:endlatex
!  qfcst  is plond,plev,pcnst,beglat:endlat
      call qmassa (cwava(lat),w(irow) ,qminus(i1,1,1,jcen),delp(:,jcen,1:plev)    , &
                   hw1lat(1,lat),nlon(lat), q3(i1,1,1,jcen,n3m2), lat)
!
   end do
!  call t_stopf('qmassa')

!----------------------------------------------------------------------------------
! zy for ffsl 
!
!  if(beglat==1)then
!  write(*,*)'wzz: beglat, endlat=', beglat, endlat, beglatex, endlatex, j1, i1
!  write(*,*)'wzz: v3=', v3(i1, 2,:,n3m2)
!  write(*,*)'wzz: ps=', psex(i1, :)
!  write(*,*)'wzz: qm=', qminus(i1-1+6,4,1, beglat+j1-1)
!  end if

   call ffsl_run( grid, ztodt, uc, vc, delp(1:plon,beglat+j1-1-grid%ng_c:endlat+j1-1+grid%ng_c,1:plev), &
        pe, pi ,ae, qminus(i1:plon+i1-1,:,:,:), &
        lam(i1:plon+i1-1,beglat+j1-1-grid%ng_c:endlat+j1-1+grid%ng_c), &
        dlam(beglat+j1-1-grid%ng_c:endlat+j1-1+grid%ng_c), nlonex(beglat+j1-1-grid%ng_c:endlat+j1-1+grid%ng_c), &
        fullgrid, qfcst(i1:plon+i1-1,:,:,:))

   call t_stopf('ffslb1')

end subroutine scanffsl
