#include <misc.h>
#include <params.h>
#ifdef OBS-FORCE
!-----------------------------------------------------------------------
!
! BOP
!
! !MODULE: atm_force_fnl for T639
!
! !DESCRIPTION: Module to handle dealing with the OZONE datasets. 
!
! Public interfaces:
!
!       tempini -- Initialization and reading of dataset.
!       tempint -- Interpolate dataset temperature to current time.
!
! $Id: atm_force_fnl.F90,v 1.3 2010/7/11  jwh Exp $
!
!-----------------------------------------------------------------------

module atm_force_fnl
!
! USES:
!
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use ppgrid,       only: begchunk, endchunk, pcols
   use phys_grid,    only: get_ncols_p, scatter_field_to_chunk, gather_chunk_to_field
   use comtp
   use prognostics
   use pspect
   use rgrid
   use comspe        ! jwh
   use commap
   use time_manager, only: tmpcyc
   use physics_types,only: physics_state
   use constituents, only: ppcnst, qmin
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual, force_start, force_stop,start_ymd,stop_ymd
   use abortutils,   only: endrun
   use infnan,       only: inf
   use spetru
   use fft99, only: fft991
    
#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none
   public tempini_fnl
   public tempint_fnl

   integer ncid_tp       !  temperature dataset
   integer ncid_ps       
   integer ncid_un     
   integer ncid_vn     
   integer ncid_qn  

   interface

   subroutine bilin (arrin, xin, yin, nlondin, nlonin, &
                     nlevdin, nlev, nlatin, arrout, xout, &
                     yout, nlondout, nlonout, nlevdout, nlatout, fillvalue)

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   integer, intent(in) :: nlondin                        ! longitude dimension of input grid
   integer, intent(in) :: nlonin                         ! number of real longitudes (input)
   integer, intent(in) :: nlevdin                        ! vertical dimension of input grid
   integer, intent(in) :: nlev                           ! number of vertical levels
   integer, intent(in) :: nlatin                         ! number of input latitudes
   integer, intent(in) :: nlatout                        ! number of output latitudes
   integer, intent(in) :: nlondout                       ! longitude dimension of output grid
   integer, intent(in) :: nlonout(nlatout)               ! number of output longitudes per lat
   integer, intent(in) :: nlevdout                       ! vertical dimension of output grid

   real(r8), intent(in) :: arrin(nlondin,nlevdin,nlatin) ! input array of values to interpolate
   real(r8), intent(in) :: xin(nlondin)                  ! input x mesh
   real(r8), intent(in) :: yin(nlatin)                   ! input y mesh
   real(r8), intent(in) :: xout(nlondout,nlatout)        ! output x mesh
   real(r8), intent(in) :: yout(nlatout)                 ! output y mesh
   real(r8), optional, intent(in) :: fillvalue           ! undef. value, wzz, 2009.8.31

   real(r8), intent(out) :: arrout(nlondout,nlevdout,nlatout) ! interpolated array

   end subroutine bilin

   end interface

!===============================================================================
!EOP
!===============================================================================

!===============================================================================
CONTAINS
!===============================================================================

subroutine tempini_fnl
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate temp to current time
!
! Method: Find next step of temp data to interpolate. 
!         vertically 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use filenames, only: bndobspath
   use ioFileMod,    only: getfil
!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
#include <comctl.h>
#include <comlun.h>
#include <comhyb.h>
!
! Local workspace
!
   integer londimid                        ! netcdf id for longitude dimension
   integer latdimid                        ! netcdf id for latitude dimension
   integer levdimid                        ! netcdf id for level dimension
   integer londimid2                        ! netcdf id for longitude dimension
   integer latdimid2                        ! netcdf id for latitude dimension
   integer levdimid2                        ! netcdf id for level dimension
   integer levdimid8                       ! netcdf id for level dimension
   integer lonid                           ! netcdf id for longitude variable
   integer latid                           ! netcdf id for latitude variable
   integer levid                           ! netcdf id for level variable
   integer levid8                          ! netcdf id for level variable
   integer timeid                          ! netcdf id for time variable
   integer timeid2                          ! netcdf id for time variable
   integer strt4(4)               ! array of starting indices
   integer cnt4(4)                ! array of counts for each dimension
   integer strt2d(3)              ! array of starting indices
   integer cnt2d(3)               ! array of counts for each dimension
   integer cnt3d(4)               ! array of counts for each dimension
   integer strt2d_org(3)              ! array of starting indices
   integer cnt2d_org(3)               ! array of counts for each dimension
   data strt2d_org/1,1,1/
   data cnt2d_org/plon,1,1/
   integer i, j, k, n                 
   integer c, ncol                ! chunk index, number of columns in chunk
   integer :: yr, mon, day       ! components of a date
   integer :: data_time_end
   integer :: ncsec               ! current time of day [seconds]
   integer ntmp                   ! temporary
   integer dimids(nf_max_var_dims)! variable shape
   integer ndays(12)               !day of different month
   integer nday(12)               !day of different month  
   integer yymmdd_model_m, yymmdd_model_p              ! 2012
   data    ndays/31,28,31,30,31,30,31,31,30,31,30,31/   !different month to day
   data    nday/0,31,59,90,120,151,181,212,243,273,304,334/   !different month to day
   real(r8) coef1, coef2          ! time interpolation factors
   real(r8) calhorloc             ! calendar hour ( no cycling)
   real(r8), allocatable :: tpbdym(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: tpbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: qnbdym(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: qnbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: psbdym(:,:)  ! temp data from pre time sample
   real(r8), allocatable :: psbdyp(:,:)  ! temp data from next time sample
   real(r8), allocatable :: tp_covm(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: tp_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: qn_covm(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: qn_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: un_covm(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: un_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: vn_covm(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: vn_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: u3_tmpm(:,:,:)             
   real(r8), allocatable :: v3_tmpm(:,:,:)             
   real(r8), allocatable :: u3_tmpp(:,:,:)             
   real(r8), allocatable :: v3_tmpp(:,:,:)             
   real(r8), allocatable :: unbdym(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: unbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: vnbdym(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: vnbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: vtbdym(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: vtbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: dvbdym(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: dvbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_prem(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: pobs_prep(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_postm(:,:)  ! temp data from pre time sample
   real(r8), allocatable :: pobs_postp(:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_intm(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: pobs_intp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: tobs_intm(:,:,:)
   real(r8), allocatable :: tobs_intp(:,:,:)
   real(r8), allocatable :: qobs_intm(:,:,:)
   real(r8), allocatable :: qobs_intp(:,:,:)
   real(r8), allocatable :: uobs_intm(:,:,:)
   real(r8), allocatable :: uobs_intp(:,:,:)
   real(r8), allocatable :: vobs_intm(:,:,:)
   real(r8), allocatable :: vobs_intp(:,:,:)
   real(r8), allocatable :: uobs_sigmam(:,:,:)
   real(r8), allocatable :: uobs_sigmap(:,:,:)
   real(r8), allocatable :: vobs_sigmam(:,:,:)
   real(r8), allocatable :: vobs_sigmap(:,:,:)
   character(len=256) :: locfn    ! netcdf local filename to open 
   character(len=256) :: locfn_tmp    ! 2012 
   character(len=256) :: yymmdd_nc_m   ! 2012
   character(len=256) :: yymmdd_nc_p   ! 2012

   real(r8) :: ps_tmp(plond,plat)
   integer psid
!
!----------------------------------------------
   integer, parameter :: tflag = 1        ! 1 for temperature / 0 for others
   integer, parameter :: vflag = 0        ! 1 for temperature / 0 for others

   nm = 1
   np = 2

   fillvalue = inf    ! jwh, 2010.11
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
   if (masterproc) then

!wtw      if (start_ymd > force_start .or. force_stop > stop_ymd) then
!wtw	     write(*,*)'dataini: force_date is not in model_date'
!wtw         call endrun
!wtw      end if

      call get_curr_date(yr, mon, day, ncsec)
      yc = yr         ! record start year      
      calhorloc = nday(mon)*24 + (day-1)*24 + ncsec/3600.
      yymmdd_model_m = yr*10000 + mon*100 + day      ! current model time
      call bndyymmdd(yr, mon, day, yymmdd_model_p)                  ! next day yymmdd

      write(yymmdd_nc_m,"(i8)") yymmdd_model_m        ! convert to string

print*,'tempini', yymmdd_model_m, yr, mon, day

! Read DATA (jwh)
!-----------------------------------------------------------------------
      call read_ncid(bndobspath, 'air', yymmdd_nc_m, ncid_tp)
      call read_ncid(bndobspath, 'pres', yymmdd_nc_m, ncid_ps)
      call read_ncid(bndobspath, 'uwnd', yymmdd_nc_m, ncid_un)
      call read_ncid(bndobspath, 'vwnd', yymmdd_nc_m, ncid_vn)
      call read_ncid(bndobspath, 'shum', yymmdd_nc_m, ncid_qn)	! wj 2016	

!<====== jwh 2010.9
!
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_tp, 'lon', londimid   )
      CALL WRAP_INQ_DIMID( ncid_tp, 'level', levdimid   )
!     CALL WRAP_INQ_DIMID( ncid_qn, 'level', levdimid8   )          ! 8  levels
      CALL WRAP_INQ_DIMID( ncid_tp, 'time', timeid  )
      CALL WRAP_INQ_DIMID( ncid_tp, 'lat', latdimid   )

      CALL WRAP_INQ_DIMLEN( ncid_tp, londimid, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_tp, levdimid, levsiz   )
!     CALL WRAP_INQ_DIMLEN( ncid_qn, levdimid8, levsiz8   )       ! 8  levels
      CALL WRAP_INQ_DIMLEN( ncid_tp, latdimid, latsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_tp, timeid, timesiz   )

      CALL WRAP_INQ_VARID( ncid_tp, 'lon', lonid   )
      CALL WRAP_INQ_VARID( ncid_tp, 'lat', latid   )
      CALL WRAP_INQ_VARID( ncid_tp, 'level', levid   )
!     CALL WRAP_INQ_VARID( ncid_qn, 'level', levid8    )            ! 8  levels
      CALL WRAP_INQ_VARID( ncid_tp, 'air', tpid   )
      CALL WRAP_INQ_VARID( ncid_ps, 'pres', ncpsid   )
      CALL WRAP_INQ_VARID( ncid_qn, 'shum', qnid   )                ! wj 2016
      CALL WRAP_INQ_VARID( ncid_un, 'uwnd', unid   )
      CALL WRAP_INQ_VARID( ncid_vn, 'vwnd', vnid   )

      CALL WRAP_INQ_VARDIMID (ncid_tp, tpid, dimids)
      CALL WRAP_INQ_VARDIMID (ncid_un, unid, dimids)
      CALL WRAP_INQ_VARDIMID (ncid_vn, vnid, dimids)
!  
   end if    
!   
#if (defined SPMD )
   call mpibcast( lonsiz, 1, mpiint, 0, mpicom )
   call mpibcast( latsiz, 1, mpiint, 0, mpicom )
   call mpibcast( levsiz, 1, mpiint, 0, mpicom )
   call mpibcast( levsiz8, 1, mpiint, 0, mpicom )         ! 8  levels
   call mpibcast( timesiz, 1, mpiint, 0, mpicom )
#endif

! Dynamically allocated memory for module comtp
   allocate (pin(levsiz))
   allocate (tempt(nm)%val(pcols,levsiz,begchunk:endchunk))
   allocate (tempt(np)%val(pcols,levsiz,begchunk:endchunk))
!   allocate (tempq(nm)%val(pcols,levsiz8,begchunk:endchunk))       ! wj 2016
!   allocate (tempq(np)%val(pcols,levsiz8,begchunk:endchunk))       ! wj 2016
   allocate (tempq(nm)%val(pcols,levsiz,begchunk:endchunk))       ! wj 2016
   allocate (tempq(np)%val(pcols,levsiz,begchunk:endchunk))       ! wj 2016
   allocate (tempu(nm)%val(pcols,plev,begchunk:endchunk))
   allocate (tempu(np)%val(pcols,plev,begchunk:endchunk))
   allocate (tempv(nm)%val(pcols,plev,begchunk:endchunk))
   allocate (tempv(np)%val(pcols,plev,begchunk:endchunk))
   allocate (tempvt(nm)%val(pcols,plev,begchunk:endchunk))
   allocate (tempvt(np)%val(pcols,plev,begchunk:endchunk))
   allocate (tempdv(nm)%val(pcols,plev,begchunk:endchunk))
   allocate (tempdv(np)%val(pcols,plev,begchunk:endchunk))
!  
   allocate (tpbdym(lonsiz,latsiz,levsiz))
   allocate (tpbdyp(lonsiz,latsiz,levsiz))
!   allocate (qnbdym(lonsiz,latsiz,levsiz8))           ! wj 2016
!   allocate (qnbdyp(lonsiz,latsiz,levsiz8))           ! wj 2016
   allocate (qnbdym(lonsiz,latsiz,levsiz))             ! wj 2016
   allocate (qnbdyp(lonsiz,latsiz,levsiz))             ! wj 2016
   allocate (psbdym(lonsiz,latsiz))
   allocate (psbdyp(lonsiz,latsiz))
   allocate (unbdym(lonsiz,latsiz,levsiz))
   allocate (unbdyp(lonsiz,latsiz,levsiz))
   allocate (vnbdym(lonsiz,latsiz,levsiz))
   allocate (vnbdyp(lonsiz,latsiz,levsiz))
!
   allocate (tp_covm(lonsiz,levsiz,latsiz))
   allocate (tp_covp(lonsiz,levsiz,latsiz))
!   allocate (qn_covm(lonsiz,levsiz8,latsiz))        ! wj 2016
!   allocate (qn_covp(lonsiz,levsiz8,latsiz))        ! wj 2016
   allocate (qn_covm(lonsiz,levsiz,latsiz))          ! wj 2016
   allocate (qn_covp(lonsiz,levsiz,latsiz))          ! wj 2016
   allocate (un_covm(lonsiz,levsiz,latsiz))
   allocate (un_covp(lonsiz,levsiz,latsiz))
   allocate (vn_covm(lonsiz,levsiz,latsiz))
   allocate (vn_covp(lonsiz,levsiz,latsiz))
!
   allocate (pobs_prem(lonsiz,1,latsiz))  
   allocate (pobs_prep(lonsiz,1,latsiz))  
   allocate (pobs_intm(plond,1,plat))  
   allocate (pobs_intp(plond,1,plat))  
   allocate (pobs_postm(plond,plat))  
   allocate (pobs_postp(plond,plat))  
   allocate (tobs_intm(plond,levsiz,plat))  
   allocate (tobs_intp(plond,levsiz,plat))  
!   allocate (qobs_intm(plond,levsiz8,plat))     ! wj 2016
!   allocate (qobs_intp(plond,levsiz8,plat))     ! wj 2016
   allocate (qobs_intm(plond,levsiz,plat))       ! wj 2016
   allocate (qobs_intp(plond,levsiz,plat))       ! wj 2016 
   allocate (uobs_intm(plond,levsiz,plat))  
   allocate (uobs_intp(plond,levsiz,plat))  
   allocate (vobs_intm(plond,levsiz,plat))  
   allocate (vobs_intp(plond,levsiz,plat))  
!
   allocate (uobs_sigmam(plond,plev,plat))  
   allocate (uobs_sigmap(plond,plev,plat))  
   allocate (vobs_sigmam(plond,plev,plat))  
   allocate (vobs_sigmap(plond,plev,plat))  
   allocate ( u3_tmpm(plond,plev,plat) )             
   allocate ( v3_tmpm(plond,plev,plat) )             
   allocate ( u3_tmpp(plond,plev,plat) )             
   allocate ( v3_tmpp(plond,plev,plat) )             
   allocate (vtbdym(plond,plev,plat))
   allocate (vtbdyp(plond,plev,plat))
   allocate (dvbdym(plond,plev,plat))
   allocate (dvbdyp(plond,plev,plat))
!
   allocate (tplon(lonsiz))
   allocate (tplat(latsiz))

!
   if (masterproc) then
!
      call initcom_tmp
!
!--------------------------------------------------
! check dat(:)
      CALL WRAP_GET_VAR_REALX (ncid_tp, levid,pin)
      CALL WRAP_GET_VAR_REALX (ncid_tp, lonid,tplon)
      CALL WRAP_GET_VAR_REALX (ncid_tp, latid,tplat)
!----------------------------------
!
! Convert from millibars to pascals
!
!----------------------------------
         strt4(1) = 1
         strt4(2) = 1
         strt4(3) = 1
         cnt4(1)  = lonsiz
         cnt4(2)  = latsiz
         cnt4(3)  = levsiz
         cnt4(4)  = 1

         cnt3d(1)  = lonsiz
         cnt3d(2)  = latsiz
         cnt3d(3)  = levsiz8
         cnt3d(4)  = 1
!-----------------------------------
         strt2d(1) = 1
         strt2d(2) = 1

         cnt2d(1)  = lonsiz
         cnt2d(2)  = latsiz
         cnt2d(3)  = 1
!-----------------------------------
!
! Normal interpolation between consecutive time slices.
!         
      if (tmpcyc) then   ! cycle in file1
 
         n = timesiz
         np1 = 1

         cdaytmpm = nday(12)*24 + (ndays(12)-1)*24 + 6*3  ! mon 12  
         cdaytmpp = nday(1)*24 + (ndays(12)-1)*24    ! return to mon 1 

         if (calhorloc <= cdaytmpp .or. calhorloc > cdaytmpm) then         
            yymmdd_model_m = yr*10000 + 12*100 + 31      ! 12.31
            write(yymmdd_nc_m,"(i8)") yymmdd_model_m       ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_m, ncid_tp)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_m, ncid_ps)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_m, ncid_un)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_m, ncid_vn)
            call read_ncid(bndobspath, 'shum', yymmdd_nc_m, ncid_qn)  ! wj 2016			

            strt2d(3) = n
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdym)

            strt4(4) = n
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdym)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdym)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdym) ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdym)         
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdym)         

!--------------------------------
          
            yymmdd_model_p = yr*10000 + 1*100 + 1      ! 1.1
            write(yymmdd_nc_p,"(i8)") yymmdd_model_p        ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_p, ncid_tp2)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_p, ncid_ps2)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_p, ncid_un2)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_p, ncid_vn2)
            call read_ncid(bndobspath, 'shum', yymmdd_nc_p, ncid_qn2)  ! wj 2016
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_ps2,ncpsid,strt2d,cnt2d,psbdyp)

            strt4(4) = np1
            call wrap_get_vara_realx (ncid_tp2,tpid,strt4,cnt4,tpbdyp)
!           call wrap_get_vara_realx (ncid_qn2,qnid,strt4,cnt3d,qnbdyp)
            call wrap_get_vara_realx (ncid_qn2,qnid,strt4,cnt4,qnbdyp)  ! wj 2016
            call wrap_get_vara_realx (ncid_un2,unid,strt4,cnt4,unbdyp)
            call wrap_get_vara_realx (ncid_vn2,vnid,strt4,cnt4,vnbdyp)
 
            goto 10
         end if
      else            ! not cycle
!----------------------------------------------------------------------
         n = timesiz
         np1 = 1

         cdaytmpm = nday(12)*24 + (ndays(12)-1)*24 + 6*3
         cdaytmpp = nday(1)*24 + (0)*24 
         if ((calhorloc <= cdaytmpp) .or. &    ! keep between end of file1 and start of file2
            (calhorloc > cdaytmpm)) then

            yymmdd_model_m = (yr-1)*10000 + 12*100 + 31      ! 12.31
            write(yymmdd_nc_m,"(i8)") yymmdd_model_m        ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_m, ncid_tp)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_m, ncid_ps)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_m, ncid_un)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_m, ncid_vn)
            call read_ncid(bndobspath, 'shum', yymmdd_nc_m, ncid_qn)  ! wj 2016

            strt2d(3) = n
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdym)

            strt4(4) = n
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdym)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdym)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdym)  ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdym)         
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdym)         
!---------------------------
            yymmdd_model_p = yr*10000 + 1*100 + 1      ! next year 1.1
            write(yymmdd_nc_p,"(i8)") yymmdd_model_p        ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_p, ncid_tp2)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_p, ncid_ps2)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_p, ncid_un2)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_p, ncid_vn2)
			
            call read_ncid(bndobspath, 'shum', yymmdd_nc_p, ncid_qn2)  ! wj 2016			
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_ps2,ncpsid,strt2d,cnt2d,psbdyp)

            strt4(4) = np1
            call wrap_get_vara_realx (ncid_tp2,tpid,strt4,cnt4,tpbdyp)
!           call wrap_get_vara_realx (ncid_qn2,qnid,strt4,cnt,qnbdyp)
            call wrap_get_vara_realx (ncid_qn2,qnid,strt4,cnt4,qnbdyp)  ! wj 2016
            call wrap_get_vara_realx (ncid_un2,unid,strt4,cnt4,unbdyp)
            call wrap_get_vara_realx (ncid_vn2,vnid,strt4,cnt4,vnbdyp)

            goto 10
         end if
!----------------------------------------------------------------------
         n = 1
         np1 = n

         cdaytmpm = nday(1)*24 + (0)*24 
         cdaytmpp = nday(1)*24 + (0)*24 


         if (calhorloc <= cdaytmpm) then       ! start from pre of file1 

            yymmdd_model_m = yr*10000 + 1*100 + 1      ! 1.1
            write(yymmdd_nc_m,"(i8)") yymmdd_model_m        ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_m, ncid_tp)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_m, ncid_ps)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_m, ncid_un)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_m, ncid_vn)
            call read_ncid(bndobspath, 'shum', yymmdd_nc_m, ncid_qn)  ! wj 2016
 
            strt2d(3) = n
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdym)

            strt4(4) = n
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdym)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdym)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdym)  ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdym)         
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdym)         

!---------------
            yymmdd_model_p = yr*10000 + 1*100 + 1      ! 1.1
            write(yymmdd_nc_p,"(i8)") yymmdd_model_p        ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_p, ncid_tp)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_p, ncid_ps)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_p, ncid_un)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_p, ncid_vn)
            call read_ncid(bndobspath, 'shum', yymmdd_nc_p, ncid_qn)  ! wj 2016

            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdyp)
            strt4(4) = np1
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdyp)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdyp)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdyp)  ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdyp)
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdyp)
 
            goto 10
      end if   
!----------------------------------------------------------------------
      if (.not. tmpcyc .and. int(force_start/1000000) .eq. yr+1) then  ! if start from file2, file1 == file2
         print*, 'Start from 2th year '
      end if
!----------------------------------------------------------------------
   end if !masterproc

      do n=1,timesiz-1
         np1 = n + 1
         cdaytmpm = nday(mon)*24 + (day-1)*24 + (n-1)*6
!-------------------------------------
         cdaytmpp = nday(mon)*24 + (day-1)*24 + (np1-1)*6
!-------------------------------------
         if (calhorloc >= cdaytmpm .and. calhorloc < cdaytmpp) then
            strt2d(3) = n
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdym)

            strt4(4) = n
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdym)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdym)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdym)  ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdym)
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdym)

!--------------
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdyp)

            strt4(4) = np1
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdyp)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdyp)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdyp)  ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdyp)
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdyp)

            goto 10
                       
         end if
      end do
      write(*,*)'dataini: Failed to find dates bracketing ncdate'
      call endrun

10    continue

      write(*,*)'dataini successful'
!
      do i=1,latsiz
         do k=1,levsiz                ! levsiz is read from obs
            tp_covm(:,k,i) = tpbdym(:,i,k)
            tp_covp(:,k,i) = tpbdyp(:,i,k)
            un_covm(:,k,i) = unbdym(:,i,k)
            un_covp(:,k,i) = unbdyp(:,i,k)
            vn_covm(:,k,i) = vnbdym(:,i,k)
            vn_covp(:,k,i) = vnbdyp(:,i,k)
            qn_covm(:,k,i) = qnbdym(:,i,k)  ! wj 2016
            qn_covp(:,k,i) = qnbdyp(:,i,k)  ! wj 2016				
         end do
!        do k=1,levsiz8
!           qn_covm(:,k,i) = qnbdym(:,i,k)
!           qn_covp(:,k,i) = qnbdyp(:,i,k)
!        end do
      end do
!
      if (tplat(1) > tplat(2)) then
         reverlat = .true.
      end if
      if (reverlat) then
         tplat(1:latsiz) = tplat(latsiz:1:-1)   
         tp_covm(:,:,1:latsiz) = tp_covm(:,:,latsiz:1:-1)
         tp_covp(:,:,1:latsiz) = tp_covp(:,:,latsiz:1:-1)
         qn_covm(:,:,1:latsiz) = qn_covm(:,:,latsiz:1:-1)
         qn_covp(:,:,1:latsiz) = qn_covp(:,:,latsiz:1:-1)
         psbdym(:,1:latsiz)    = psbdym(:,latsiz:1:-1)
         psbdyp(:,1:latsiz)    = psbdyp(:,latsiz:1:-1)
         un_covm(:,:,1:latsiz) = un_covm(:,:,latsiz:1:-1)
         un_covp(:,:,1:latsiz) = un_covp(:,:,latsiz:1:-1)
         vn_covm(:,:,1:latsiz) = vn_covm(:,:,latsiz:1:-1)
         vn_covp(:,:,1:latsiz) = vn_covp(:,:,latsiz:1:-1)
      end if
!
      if (lonsiz.eq.plon .and. latsiz.eq.plat) then
          tobs_intm(:plond,:levsiz,:plat) = tp_covm(:lonsiz,:levsiz,:latsiz)
          tobs_intp(:plond,:levsiz,:plat) = tp_covp(:lonsiz,:levsiz,:latsiz)
!          qobs_intm(:plond,:levsiz8,:plat) = qn_covm(:lonsiz,:levsiz8,:latsiz)
!          qobs_intp(:plond,:levsiz8,:plat) = qn_covp(:lonsiz,:levsiz8,:latsiz)
          qobs_intm(:plond,:levsiz,:plat) = qn_covm(:lonsiz,:levsiz,:latsiz) ! wj 2016
          qobs_intp(:plond,:levsiz,:plat) = qn_covp(:lonsiz,:levsiz,:latsiz) ! wj 2016
          uobs_intm(:plond,:levsiz,:plat) = un_covm(:lonsiz,:levsiz,:latsiz)
          uobs_intp(:plond,:levsiz,:plat) = un_covp(:lonsiz,:levsiz,:latsiz)
          vobs_intm(:plond,:levsiz,:plat) = vn_covm(:lonsiz,:levsiz,:latsiz)
          vobs_intp(:plond,:levsiz,:plat) = vn_covp(:lonsiz,:levsiz,:latsiz)
      else
          call bilin (tp_covm, tplon, tplat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, tobs_intm, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)

          call bilin (tp_covp, tplon, tplat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, tobs_intp, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)

!         call bilin (qn_covm, tplon, tplat, lonsiz, lonsiz, &
!                    levsiz, levsiz8, latsiz, qobs_intm, londeg, &
!                    latdeg, plond, nlon, levsiz8, plat, fillvalue)

!         call bilin (qn_covp, tplon, tplat, lonsiz, lonsiz, &
!                    levsiz, levsiz8, latsiz, qobs_intp, londeg, &
!                    latdeg, plond, nlon, levsiz8, plat, fillvalue)

          call bilin (qn_covm, tplon, tplat, lonsiz, lonsiz, &       ! wj 2016
                     levsiz, levsiz, latsiz, qobs_intm, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)

          call bilin (qn_covp, tplon, tplat, lonsiz, lonsiz, &      ! wj 2016
                     levsiz, levsiz, latsiz, qobs_intp, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)

          call bilin (un_covm, tplon, tplat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, uobs_intm, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)
          call bilin (un_covp, tplon, tplat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, uobs_intp, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)

          call bilin (vn_covm, tplon, tplat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, vobs_intm, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)
          call bilin (vn_covp, tplon, tplat, lonsiz, lonsiz, &
                     levsiz, levsiz, latsiz, vobs_intp, londeg, &
                     latdeg, plond, nlon, levsiz, plat, fillvalue)
       end if


       if (lonsiz.eq.plon .and. latsiz.eq.plat) then
          pobs_postm(:plond,:plat) = psbdym(:lonsiz,:latsiz)
          pobs_postp(:plond,:plat) = psbdyp(:lonsiz,:latsiz)
       else
          do i=1,lonsiz
             pobs_prem(i,1,:) = psbdym(i,:)
             pobs_prep(i,1,:) = psbdyp(i,:)
          end do
          call bilin (pobs_prem, tplon, tplat, lonsiz, lonsiz, &
                     1, 1, latsiz, pobs_intm, londeg, &
                     latdeg, plond, nlon, 1, plat, fillvalue)
 
          call bilin (pobs_prep, tplon, tplat, lonsiz, lonsiz, &
                     1, 1, latsiz, pobs_intp, londeg, &
                     latdeg, plond, nlon, 1, plat, fillvalue)
          do i=1,plond
             pobs_postm(i,:) = pobs_intm(i,1,:)
             pobs_postp(i,:) = pobs_intp(i,1,:)
          end do
       end if
!
!----------PS-----------------
!
         call wrap_inq_varid (ncid_ini, 'PS'  , psid)

         strt2d_org(1) = 1
         strt2d_org(2) = 1
         strt2d_org(3) = 1

         do j=1,plat
            strt2d_org(2) = j
            call wrap_get_vara_realx(ncid_ini, psid, strt2d_org, cnt2d_org, ps_tmp(1,j))
         end do
!-------------------------------

       if (pin(1)>pin(2)) then
          reverp = .true.
       end if
       if ( reverp ) then
          pin(1:levsiz) = pin(levsiz:1:-1)
          tobs_intm(:,1:levsiz,:) = tobs_intm(:,levsiz:1:-1,:)
          tobs_intp(:,1:levsiz,:) = tobs_intp(:,levsiz:1:-1,:)
!          qobs_intm(:,1:levsiz8,:) = qobs_intm(:,levsiz8:1:-1,:)
!          qobs_intp(:,1:levsiz8,:) = qobs_intp(:,levsiz8:1:-1,:)
          qobs_intm(:,1:levsiz,:) = qobs_intm(:,levsiz:1:-1,:)  ! wj 2016
          qobs_intp(:,1:levsiz,:) = qobs_intp(:,levsiz:1:-1,:)  ! wj 2016
          uobs_intm(:,1:levsiz,:) = uobs_intm(:,levsiz:1:-1,:)
          uobs_intp(:,1:levsiz,:) = uobs_intp(:,levsiz:1:-1,:)
          vobs_intm(:,1:levsiz,:) = vobs_intm(:,levsiz:1:-1,:)
          vobs_intp(:,1:levsiz,:) = vobs_intp(:,levsiz:1:-1,:)
       end if

       do k=1,levsiz
          pin(k) = pin(k)*100.
       end do
       ! write(*,*)"pin(k)=",pin 
       call interplot (uobs_intm, pin, levsiz, uobs_sigmam, ps_tmp, plev, hyam, hybm, vflag)
       call interplot (vobs_intm, pin, levsiz, vobs_sigmam, ps_tmp, plev, hyam, hybm, vflag)
       call interplot (uobs_intp, pin, levsiz, uobs_sigmap, ps_tmp, plev, hyam, hybm, vflag)
       call interplot (vobs_intp, pin, levsiz, vobs_sigmap, ps_tmp, plev, hyam, hybm, vflag)

       u3_tmpm(:plond,:plev,:plat) = uobs_sigmam(:plond,:plev,:plat)
       v3_tmpm(:plond,:plev,:plat) = vobs_sigmam(:plond,:plev,:plat)
       u3_tmpp(:plond,:plev,:plat) = uobs_sigmap(:plond,:plev,:plat)
       v3_tmpp(:plond,:plev,:plat) = vobs_sigmap(:plond,:plev,:plat)

       call convert_uv   (u3_tmpm, v3_tmpm, vtbdym, dvbdym)
       call convert_uv   (u3_tmpp, v3_tmpp, vtbdyp, dvbdyp)

       deallocate(alp)
       deallocate(dalp)
   end if
!----------------------------------------------------------------------------------
#if (defined SPMD)
         call scatter_field_to_chunk(1,levsiz,1,plond-3,tobs_intm(:,:,:),tempt(nm)%val)   
                                     ! the observed temperature at top 6 levels not include
!         call scatter_field_to_chunk(1,levsiz8,1,plond,qobs_intm,tempq(nm)%val)  ! wj 2016
         call scatter_field_to_chunk(1,levsiz,1,plond,qobs_intm,tempq(nm)%val)  ! wj 2016
         call scatter_field_to_chunk(1,plev,1,plond,u3_tmpm,tempu(nm)%val)
         call scatter_field_to_chunk(1,plev,1,plond,v3_tmpm,tempv(nm)%val)
         call scatter_field_to_chunk(1,plev,1,plond,vtbdym,tempvt(nm)%val)
         call scatter_field_to_chunk(1,plev,1,plond,dvbdym,tempdv(nm)%val)

         call scatter_field_to_chunk(1,levsiz,1,plond,tobs_intp(:,:,:),tempt(np)%val)    
                                     ! the observed temperature at top 6 levels not include
!         call scatter_field_to_chunk(1,levsiz8,1,plond,qobs_intp,tempq(np)%val)  ! wj 2016
         call scatter_field_to_chunk(1,levsiz,1,plond,qobs_intp,tempq(np)%val)  ! wj 2016
         call scatter_field_to_chunk(1,plev,1,plond,u3_tmpp,tempu(np)%val)
         call scatter_field_to_chunk(1,plev,1,plond,v3_tmpp,tempv(np)%val)
         call scatter_field_to_chunk(1,plev,1,plond,vtbdyp,tempvt(np)%val)
         call scatter_field_to_chunk(1,plev,1,plond,dvbdyp,tempdv(np)%val)
#endif
!  deallocate (date_tp)
   deallocate (tpbdym)
   deallocate (tpbdyp)
   deallocate (qnbdym)
   deallocate (qnbdyp)
   deallocate (psbdym)
   deallocate (psbdyp)
!
   deallocate (tp_covm)
   deallocate (tp_covp)
   deallocate (qn_covm)
   deallocate (qn_covp)
   deallocate (un_covm)
   deallocate (un_covp)
   deallocate (vn_covm)
   deallocate (vn_covp)
   deallocate (unbdym)
   deallocate (unbdyp)
   deallocate (vnbdym)
   deallocate (vnbdyp)
!
   deallocate (tobs_intm)
   deallocate (tobs_intp)
   deallocate (qobs_intm)
   deallocate (qobs_intp)
   deallocate (pobs_intm)
   deallocate (pobs_intp)
   deallocate (pobs_prem)
   deallocate (pobs_prep)
   deallocate (pobs_postm)
   deallocate (pobs_postp)
   deallocate (uobs_intm)
   deallocate (uobs_intp)
   deallocate (vobs_intm)
   deallocate (vobs_intp)
!
   deallocate (uobs_sigmam)
   deallocate (uobs_sigmap)
   deallocate (vobs_sigmam)
   deallocate (vobs_sigmap)
   deallocate (u3_tmpm)
   deallocate (u3_tmpp)
   deallocate (v3_tmpm)
   deallocate (v3_tmpp)
   deallocate (vtbdym)
   deallocate (vtbdyp)
   deallocate (dvbdym)
   deallocate (dvbdyp)
!  deallocate (temp(1)%val)
!  deallocate (temp(2)%val)
!------------------------------------------------------
#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (yc, 1, mpiint, 0, mpicom)
   call mpibcast (pin, levsiz, mpir8, 0, mpicom)
   call mpibcast (tplon, lonsiz, mpir8, 0, mpicom)
   call mpibcast (tplat, latsiz, mpir8, 0, mpicom)
   call mpibcast (cdaytmpm, 1, mpir8, 0, mpicom)
   call mpibcast (cdaytmpp, 1, mpir8, 0, mpicom)
   call mpibcast (reverp,    1, mpilog, 0,mpicom)
   call mpibcast (reverlat,  1, mpilog, 0,mpicom)
#endif
!
   return

end subroutine tempini_fnl


subroutine tempint_fnl(state)
!----------------------------------------------------------------------- 
! Purpose: Interpolate temperature to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
! Method: Find next month of temperature data to interpolate.  Linearly interpolate 
!         vertically and horizontally
! 
! Author: jwh 2010
! 
!-----------------------------------------------------------------------
   use filenames, only: bndobspath
   use ioFileMod, only: getfil
   use time_manager, only: get_nstep, get_step_size
!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
#include <comctl.h>
#include <comlun.h>
#include <comhyb.h>
!-----------------------------------------------------------------------
!
! Local workspace
!
   type(physics_state), intent(inout), dimension(begchunk:endchunk) :: state
 
   integer levdimid                        ! netcdf id for level dimension
   integer levdimid8                        ! netcdf id for level dimension
   integer levid                           ! netcdf id for level variable
   integer levid8                           ! netcdf id for level variable
   integer timeid                          ! netcdf id for time variable
   integer :: hyamid           
   integer :: hybmid           
   integer cnt4(4)                ! array of counts for each dimension
   integer cnt3d(4)               ! array of counts for each dimension
   integer strt4(4)               ! array of starting indices
   integer cnt2d(3)               ! array of counts for each dimension
   integer strt2d(3)               ! array of starting indices
   integer i, j, k, n                 
   integer c, ncol                ! chunk index, number of columns in chunk
   integer lchnk                  ! chunk identifier
   integer yymmdd_model_p         ! 2012
   integer :: yr, mon, day        ! components of a date
   integer :: ncsec               ! current time of day [seconds]
   integer ntmp                   ! temporary
   integer dimids(nf_max_var_dims)! variable shape
   integer ncdays(12)               !day of different month
   data    ncdays/31,28,31,30,31,30,31,31,30,31,30,31/   !different month to day
   integer nday(12)               !day of different month  
   data    nday/0,31,59,90,120,151,181,212,243,273,304,334/   !different month to day
   real(r8) coef1, coef2          ! time interpolation factors
   real(r8) calhorloc             ! calendar hour ( no cycling)
   real(r8), allocatable :: tpbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: qnbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: psbdyp(:,:)  ! temp data from next time sample
   real(r8), allocatable :: tp_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: qn_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: un_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: vn_covp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: u3_tmpp(:,:,:)             
   real(r8), allocatable :: v3_tmpp(:,:,:)             
   real(r8), allocatable :: pobs_intp(:,:,:)
   real(r8), allocatable :: pobs_prep(:,:,:)
   real(r8), allocatable :: pobs_postp(:,:)
   real(r8), allocatable :: tobs_intp(:,:,:)
   real(r8), allocatable :: qobs_intp(:,:,:)
   real(r8), allocatable :: uobs_intp(:,:,:)  ! interpolat
   real(r8), allocatable :: vobs_intp(:,:,:)
   real(r8), allocatable :: uobs_sigmap(:,:,:)
   real(r8), allocatable :: vobs_sigmap(:,:,:)
   real(r8), allocatable :: unbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: vnbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: vtbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: dvbdyp(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: tempi(:,:,:)  
   real(r8), allocatable :: tempi8(:,:,:)  
   real(r8), allocatable :: ps_tmp2(:,:)
   real(r8), allocatable :: tobs_pre(:)
   real(r8), allocatable :: qobs_pre(:)

   real(r8), dimension(plond,plat) :: ps_gather

   character(len=256) :: locfn    ! netcdf local filename to open 
   character(len=256) :: yymmdd_nc_p   ! 2012
!-----------------------------------------------------------------------
! Purpose: 
! Vertically interpolate input array to output sigma level
! Copy values at boundaries.
   
   integer, parameter :: tflag = 1        ! 1 for temperature / 0 for others
   integer, parameter :: vflag = 0        ! 1 for temperature / 0 for others
!----------------------------------------------
   real(r8) dtime                 ! timestep size
   real(r8) srtstep, endstep
   integer  nstep, yrsrt, yrsrt_force, yrend_force
   real(r8) daysrt, daysrt_force, dayend_force
!-------------------------
   real(r8)  pmid(plev)
!   real(r8)  tobs_pre(plev)
   real(r8)  tobs_sigma(plev)
!   real(r8)  qobs_pre(levsiz8)      ! wj 2016
!   real(r8)  qobs_pre(plev)          ! wj 2006
   real(r8)  qobs_sigma(plev)
   real(r8)  p0
   p0 = 100000.0

!   
! SPMD: Master does all the work.  Sends needed info to slaves
!
   call get_curr_date(yr, mon, day, ncsec)
   calhorloc = nday(mon)*24 + (day-1)*24 + ncsec/3600.
!--------------------------------------------------
!  wtw 
!
  dtime = get_step_size()
  nstep = get_nstep()

  call bnday(start_ymd,daysrt,yrsrt)
  call bnday_hour(force_start,daysrt_force,yrsrt_force)
  call bnday_hour(force_stop,dayend_force,yrend_force)
  if (yrend_force > yrsrt) then
    srtstep = (86400/dtime)*daysrt_force + (86400/dtime)*(365*(yrsrt_force - yrsrt) - daysrt)
    endstep = (86400/dtime)*dayend_force + (86400/dtime)*(365*(yrend_force - yrsrt) - daysrt)
  else
    srtstep = (86400/dtime)*(daysrt_force - daysrt)
    endstep = (86400/dtime)*(dayend_force - daysrt)
  end if
  if (nstep > endstep ) then
      return
  endif
!--------------------------------------
   if (masterproc) then
!----------------------------------------------------------
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_tp, 'level', levdimid   )
!     CALL WRAP_INQ_DIMID( ncid_qn, 'level', levdimid8   )
      CALL WRAP_INQ_VARID( ncid_tp, 'level', levid   )
!     CALL WRAP_INQ_VARID( ncid_qn, 'level', levid8   )
      CALL WRAP_INQ_VARID( ncid_tp, 'air', tpid   )
      CALL WRAP_INQ_VARID( ncid_ps, 'pres', ncpsid   )
      CALL WRAP_INQ_VARID( ncid_qn, 'shum', qnid   )       ! wj 2016
      CALL WRAP_INQ_VARID( ncid_un, 'uwnd', unid   )
      CALL WRAP_INQ_VARID( ncid_vn, 'vwnd', vnid   )

      call wrap_inq_varid (ncid_ini, 'hyam'   , hyamid)
      call wrap_inq_varid (ncid_ini, 'hybm'   , hybmid)

!     CALL WRAP_INQ_DIMLEN( ncid_qn, levdimid8, levsiz8   )       ! 8  levels
!  
   end if    
!
!----------------------------------------------------------
! Dynamically allocated memory for module comtp

!  allocate (date_tp(timesiz))
   allocate (lev(levsiz))
   allocate (lev8(levsiz8))
!  allocate (temp(nm)%val(pcols,levsiz,begchunk:endchunk))
!  allocate (temp(np)%val(pcols,levsiz,begchunk:endchunk))
   allocate (tmp%val(pcols,levsiz,begchunk:endchunk))
   allocate (tmp8%val(pcols,levsiz8,begchunk:endchunk))
   allocate (tpbdyp(lonsiz,latsiz,levsiz))   
!   allocate (qnbdyp(lonsiz,latsiz,levsiz8))           ! wj 2016   
   allocate (qnbdyp(lonsiz,latsiz,levsiz))            ! wj 2016
   allocate (psbdyp(lonsiz,latsiz))   
   allocate (unbdyp(lonsiz,latsiz,levsiz))   
   allocate (vnbdyp(lonsiz,latsiz,levsiz))   
!
   allocate (tp_covp(lonsiz,levsiz,latsiz))   
!   allocate (qn_covp(lonsiz,levsiz8,latsiz))           ! wj 2016
   allocate (qn_covp(lonsiz,levsiz,latsiz))             ! wj 2016  
   allocate (un_covp(lonsiz,levsiz,latsiz))   
   allocate (vn_covp(lonsiz,levsiz,latsiz))   
!
   allocate (pobs_prep(lonsiz,1,latsiz))  
   allocate (pobs_postp(plond,plat))  
   allocate (pobs_intp(plond,1,plat))  
   allocate (tobs_intp(plond,levsiz,plat))  
!   allocate (qobs_intp(plond,levsiz8,plat))           ! wj 2016
   allocate (qobs_intp(plond,levsiz,plat))             ! wj 2016 
   allocate (uobs_intp(plond,levsiz,plat))  
   allocate (vobs_intp(plond,levsiz,plat))  
!
   allocate (uobs_sigmap(plond,plev,plat))  
   allocate (vobs_sigmap(plond,plev,plat))  
   allocate ( u3_tmpp(plond,plev,plat) )             
   allocate ( v3_tmpp(plond,plev,plat) )             
   allocate (vtbdyp(plond,plev,plat))   
   allocate (dvbdyp(plond,plev,plat))   
   allocate (tempi(lonsiz,levsiz,latsiz))
   allocate (tempi8(lonsiz,levsiz8,latsiz))

   allocate (ps_tmp2(pcols,begchunk:endchunk))
   allocate (tobs_pre(levsiz))
   allocate (qobs_pre(levsiz))
!
      if (masterproc) then
!
!--------------------------------------------------
      CALL WRAP_GET_VAR_REALX (ncid_tp, levid, lev)
!     CALL WRAP_GET_VAR_REALX (ncid_qn, levid8, lev8)
      call wrap_get_var_realx (ncid_ini, hyamid,hyam)
      call wrap_get_var_realx (ncid_ini, hybmid,hybm)
!--------------------------------------------------
      do i=1, lonsiz
        do j=1, latsiz
          do k=1, levsiz
             tempi(i,k,j)  =lev(k)
          end do
        end do
      end do


!      do i=1, lonsiz
!        do j=1, latsiz
!          do k=1, levsiz8
!             tempi8(i,k,j)  =lev8(k)
!          end do
!        end do
!      end do
!--------------------------------------------------

         strt4(1) = 1
         strt4(2) = 1
         strt4(3) = 1
         cnt4(1)  = lonsiz
         cnt4(2)  = latsiz
         cnt4(3)  = levsiz
         cnt4(4)  = 1

         cnt3d(1)  = lonsiz
         cnt3d(2)  = latsiz
         cnt3d(3)  = levsiz8
         cnt3d(4)  = 1
!-----------------------------------
         strt2d(1) = 1
         strt2d(2) = 1

         cnt2d(1)  = lonsiz
         cnt2d(2)  = latsiz
         cnt2d(3)  = 1
!-----------------------------------
      end if
!
! Normal interpolation between consecutive time slices.
!
!---------------------------------------------------------------------------------
   if (calhorloc >= cdaytmpp .and. &
      (.not. (np1 == 1 .and. calhorloc > 8754.0) .or. cdaytmpm == cdaytmpp)) then
      np1 = mod(np1,timesiz) + 1
      if (np1 == 1) then   
         if (masterproc) then     
            if (tmpcyc .and. mon.eq.12 .and. day.eq.31) then  ! read next file
               call bndyymmdd(min(yc,yr), 1, 1, yymmdd_model_p)         ! next day yymmdd
            else
               call bndyymmdd(yr, mon, day+1, yymmdd_model_p)                  ! next day yymmdd
            end if
            call wrap_close (ncid_tp)
            call wrap_close (ncid_ps)
            call wrap_close (ncid_un)
            call wrap_close (ncid_vn)
            call wrap_close (ncid_qn)

            write(yymmdd_nc_p,"(i8)") yymmdd_model_p        ! convert to string
            call read_ncid(bndobspath, 'air', yymmdd_nc_p, ncid_tp)
            call read_ncid(bndobspath, 'pres', yymmdd_nc_p, ncid_ps)
            call read_ncid(bndobspath, 'uwnd', yymmdd_nc_p, ncid_un)
            call read_ncid(bndobspath, 'vwnd', yymmdd_nc_p, ncid_vn)
            call read_ncid(bndobspath, 'shum', yymmdd_nc_p, ncid_qn)      ! wj 2016			
            print*,'model_time=',yr, mon, day, ncsec/3600
            print*,'fnl_time=', yymmdd_nc_p
         end if
         cdaytmpm = cdaytmpp
         cdaytmpp = nday(mon)*24 + (day)*24 + (np1-1)*6
         if(cdaytmpp.eq.365*24)cdaytmpp=0
     else
         cdaytmpm = cdaytmpp   
         cdaytmpp = nday(mon)*24 + (day-1)*24 + (np1-1)*6
     end if
!
     ntmp = nm
     nm = np
     np = ntmp
!--------------------
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      ps_tmp2(:ncol,c) = state(c)%ps(:ncol)
   end do
   call gather_chunk_to_field(1,1,1,plond,ps_tmp2,ps_gather(1,1))

!---------------------
!
     if (masterproc) then

        call initcom_tmp                 

! Read DATA (jwh)
!-----------------------------------------------------------------------
            
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_ps,ncpsid,strt2d,cnt2d,psbdyp)

            strt4(4) = np1
            call wrap_get_vara_realx (ncid_tp,tpid,strt4,cnt4,tpbdyp)
!           call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt3d,qnbdyp)
            call wrap_get_vara_realx (ncid_qn,qnid,strt4,cnt4,qnbdyp)     ! wj 2016
            call wrap_get_vara_realx (ncid_un,unid,strt4,cnt4,unbdyp)         
            call wrap_get_vara_realx (ncid_vn,vnid,strt4,cnt4,vnbdyp)         
!
            do i=1,latsiz
               do k=1,levsiz
                  tp_covp(:,k,i) = tpbdyp(:,i,k)
                  un_covp(:,k,i) = unbdyp(:,i,k)
                  vn_covp(:,k,i) = vnbdyp(:,i,k)
                  qn_covp(:,k,i) = qnbdyp(:,i,k)        ! wj 2016				  
               end do
!               do k=1,levsiz8                          ! wj 2016
!                  qn_covp(:,k,i) = qnbdyp(:,i,k)       ! wj 2016
!               end do                                  ! wj 2016
            end do
!
            if ( reverlat ) then
               psbdyp(:,1:latsiz) = psbdyp(:,latsiz:1:-1)
               tp_covp(:,:,1:latsiz) = tp_covp(:,:,latsiz:1:-1)
               qn_covp(:,:,1:latsiz) = qn_covp(:,:,latsiz:1:-1)
               un_covp(:,:,1:latsiz) = un_covp(:,:,latsiz:1:-1)
               vn_covp(:,:,1:latsiz) = vn_covp(:,:,latsiz:1:-1)
            end if
!
            if (lonsiz.eq.plon .and. latsiz.eq.plat) then

               pobs_postp(:plond,:plat) = psbdyp(:lonsiz,:latsiz)
               tobs_intp(:plond,:levsiz,:plat) = tp_covp(:lonsiz,:levsiz,:latsiz)
!               qobs_intp(:plond,:levsiz8,:plat) = qn_covp(:lonsiz,:levsiz8,:latsiz) ! wj 2016
               qobs_intp(:plond,:levsiz,:plat) = qn_covp(:lonsiz,:levsiz,:latsiz) ! wj 2016			   
               uobs_intp(:plond,:levsiz,:plat) = un_covp(:lonsiz,:levsiz,:latsiz)
               vobs_intp(:plond,:levsiz,:plat) = vn_covp(:lonsiz,:levsiz,:latsiz)

            else

               do i=1,lonsiz
                  pobs_prep(i,1,:) = psbdyp(i,:)
               end do

               call bilin (pobs_prep, tplon, tplat, lonsiz, lonsiz, &
                          1, 1, latsiz, pobs_intp, londeg, &
                          latdeg, plond, nlon, 1, plat, fillvalue)

               do i=1,plond
                  pobs_postp(i,:) = pobs_intp(i,1,:)
               end do

               call bilin (tp_covp, tplon, tplat, lonsiz, lonsiz, &
                          levsiz, levsiz, latsiz, tobs_intp, londeg, &
                          latdeg, plond, nlon, levsiz, plat, fillvalue)

!              call bilin (qn_covp, tplon, tplat, lonsiz, lonsiz, &
!                         levsiz, levsiz8, latsiz, qobs_intp, londeg, &
!                         latdeg, plond, nlon, levsiz8, plat, fillvalue)

               call bilin (qn_covp, tplon, tplat, lonsiz, lonsiz, &         ! wj 2016
                          levsiz, levsiz, latsiz, qobs_intp, londeg, &
                          latdeg, plond, nlon, levsiz, plat, fillvalue)


               call bilin (un_covp, tplon, tplat, lonsiz, lonsiz, &
                          levsiz, levsiz, latsiz, uobs_intp, londeg, &
                          latdeg, plond, nlon, levsiz, plat, fillvalue)

               call bilin (vn_covp, tplon, tplat, lonsiz, lonsiz, &
                          levsiz, levsiz, latsiz, vobs_intp, londeg, &
                          latdeg, plond, nlon, levsiz, plat, fillvalue)

            end if

            if ( reverp ) then
              tobs_intp(:,1:levsiz,:) = tobs_intp(:,levsiz:1:-1,:)
!              qobs_intp(:,1:levsiz8,:) = qobs_intp(:,levsiz8:1:-1,:)     ! wj 2016
              qobs_intp(:,1:levsiz8,:) = qobs_intp(:,levsiz:1:-1,:)       ! wj 2016
              uobs_intp(:,1:levsiz,:) = uobs_intp(:,levsiz:1:-1,:)
              vobs_intp(:,1:levsiz,:) = vobs_intp(:,levsiz:1:-1,:)
            end if

            call interplot (uobs_intp, pin, levsiz, uobs_sigmap, ps_gather, plev, hyam, hybm, vflag)
            call interplot (vobs_intp, pin, levsiz, vobs_sigmap, ps_gather, plev, hyam, hybm, vflag)

            u3_tmpp(:plond,:plev,:plat) = uobs_sigmap(:plond,:plev,:plat)
            v3_tmpp(:plond,:plev,:plat) = vobs_sigmap(:plond,:plev,:plat)

            call convert_uv   (u3_tmpp, v3_tmpp, vtbdyp, dvbdyp)

            deallocate(alp)
            deallocate(dalp)

     end if
#if (defined SPMD)
         call scatter_field_to_chunk(1,levsiz,1,plond,tobs_intp(:,:,:),tempt(np)%val) 
                                       ! 26 levels top from 7 down to 32 levels
!         call scatter_field_to_chunk(1,levsiz8,1,plond,qobs_intp,tempq(np)%val) ! wj 2016
         call scatter_field_to_chunk(1,levsiz,1,plond,qobs_intp,tempq(np)%val)   ! wj 2016
         call scatter_field_to_chunk(1,plev,1,plond,u3_tmpp,tempu(np)%val)
         call scatter_field_to_chunk(1,plev,1,plond,v3_tmpp,tempv(np)%val)
         call scatter_field_to_chunk(1,plev,1,plond,vtbdyp,tempvt(np)%val)
         call scatter_field_to_chunk(1,plev,1,plond,dvbdyp,tempdv(np)%val)
#endif
   end if
! call scatter_field_to_chunk(1,levsiz,1,lonsiz,tempi,tmp%val)
! call scatter_field_to_chunk(1,levsiz8,1,lonsiz,tempi8,tmp8%val)
!----------------------------------------------------------------------------------
!
! Determine time interpolation factors.

      if (cdaytmpm == cdaytmpp) then
         coef1 = 1.
         coef2 = 0.
      else
          call getfact (tmpcyc, np1, cdaytmpm, cdaytmpp, calhorloc, coef1, coef2 ,'TEMPINT_T639:')
!         write(*,*) coef1,coef2
      end if
!
! Time interpolation.
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do k=1,plev
         do i=1,ncol
            state(c)%uobs(i,k) = tempu(nm)%val(i,k,c)*coef1 + tempu(np)%val(i,k,c)*coef2
            state(c)%vobs(i,k) = tempv(nm)%val(i,k,c)*coef1 + tempv(np)%val(i,k,c)*coef2
            state(c)%vtobs(i,k) = tempvt(nm)%val(i,k,c)*coef1 + tempvt(np)%val(i,k,c)*coef2
            state(c)%dvobs(i,k) = tempdv(nm)%val(i,k,c)*coef1 + tempdv(np)%val(i,k,c)*coef2
         end do
      end do
   end do

!-------------------------
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do i=1,ncol
!--
         do k=1, levsiz
            tobs_pre(k) = tempt(nm)%val(i,k,c)*coef1 + tempt(np)%val(i,k,c)*coef2
!           pre17(k)    = tmp%val(i,k,c)*100.0
         end do

!	 do k=1, levsiz8      ! wj 2016
         do k=1, levsiz       ! wj 2016
            qobs_pre(k) = tempq(nm)%val(i,k,c)*coef1 + tempq(np)%val(i,k,c)*coef2    ! wj 2016
!           pre8(k)    = tmp8%val(i,k,c)*100.0
         end do
!--
         do k=1,plev
            pmid(k) = hyam(k)*p0+hybm(k)*state(c)%ps(i)
         end do

!    	    call vinterp(tobs_pre, pre17, levsiz, tobs_sigma , pmid, plev, tflag)		 
!           call vinterp(qobs_pre, pre8, levsiz8, qobs_sigma , pmid, plev, vflag)
!         do k=1,plev
!             state(c)%tobs(i,k)=tobs_sigma(k)
!             state(c)%qobs(i,k)=qobs_sigma(k)
!         end do

!         if( levsiz .gt. plev) then
!             write(*,*) 'there is a error. levsiz .gt.plev '
!             call endrun
!         else
             do k=1, levsiz
                state(c)%tobs(i,k)= tobs_pre(k)
                state(c)%qobs(i,k)= qobs_pre(k)
             end do
!         end if

      end do

      call qneg3_tmp('CYC', lchnk, ncol, pcols, plev,  ppcnst, qmin, state(c)%qobs )

   end do

   deallocate (lev)
   deallocate (lev8)
!  deallocate (date_tp)
!  deallocate (tplon)
!  deallocate (tplat)
   deallocate (tpbdyp)
   deallocate (qnbdyp)
   deallocate (psbdyp)
   deallocate (tp_covp)
   deallocate (qn_covp)
   deallocate (un_covp)
   deallocate (vn_covp)
   deallocate (tobs_intp)
   deallocate (qobs_intp)
   deallocate (pobs_prep)
   deallocate (pobs_postp)
   deallocate (pobs_intp)
   deallocate (uobs_intp)
   deallocate (vobs_intp)
   deallocate (uobs_sigmap)
   deallocate (vobs_sigmap)
   deallocate (u3_tmpp)
   deallocate (v3_tmpp)
   deallocate (unbdyp)
   deallocate (vnbdyp)
   deallocate (vtbdyp)
   deallocate (dvbdyp)
   deallocate (tempi)
   deallocate (tempi8)
!  deallocate (temp(1)%val)
!  deallocate (temp(2)%val)
   deallocate (tmp%val)
   deallocate (tmp8%val)
   deallocate (ps_tmp2)
   deallocate (tobs_pre)
   deallocate (qobs_pre)
   return
end subroutine tempint_fnl

!***********************************************************************
   subroutine wrap_get_vara_factor (nfid, varid, factid, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets a range of the given real variable from a input file
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#include <netcdf.inc>

   integer, intent(in):: nfid
   integer, intent(in):: varid
 
   character(len=*), intent(in) :: factid ! name of calling routine
   real(r8), intent(out):: arr 
   integer ret      ! NetCDF return code

   ret = nf_get_att_double(nfid, varid, factid, arr)

   if (ret/=NF_NOERR) then
     write(6,*)'wrap_get_vara_factor: error reading varid =', varid
     call handle_error (ret)
   end if
   end subroutine wrap_get_vara_factor
!-----------------------------------------------------------------------
#include <misc.h>
#include <params.h>
subroutine interplot (data_pre, lev, lev_num, data_sigma, ps_model, lev_out, hyam, hybm, flag)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert input levsize data to model needs at masterproc
! 
! Method: vinterp
! 
! Author: JWH
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid,   only: plon, plond, plev, plat
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
! Arguments
!
   integer, intent(in) :: lev_num           ! input levsize
   integer, intent(in) :: flag           ! input levsize
   real(r8), intent(in) :: data_pre(plond,lev_num,plat)      ! input data
   real(r8), intent(in) :: lev(lev_num)           ! input levsize
   integer, intent(in) :: lev_out       ! ouput levsize
   real(r8), intent(in) :: ps_model(plond,plat)
   real(r8), intent(in) :: hyam(lev_out)
   real(r8), intent(in) :: hybm(lev_out)
   real(r8), intent(out) :: data_sigma(plond,plev,plat)       ! Day of year

   integer i,j,k
   real(r8) press(lev_out)
   real(r8) temp_pre(lev_num)
   real(r8) temp_sigma(plev)
   real(r8)  p0
   p0 = 100000.0

      do i=1,plond
	     do j=1,plat
            do k=1,plev
			   press(k) = hyam(k)*p0+hybm(k)*ps_model(i,j)
            end do
            
			do k=1, lev_num
			   temp_pre(k) = data_pre(i,k,j)
            end do
			
			call vinterp(temp_pre, lev, lev_num, temp_sigma , press, lev_out, flag)

		    do k=1,plev
               data_sigma(i,k,j) = temp_sigma(k)
			end do

		 end do
      end do
   
   return
end subroutine interplot

!-----------------------------------------------------------------------
subroutine bndyear (ncdate, year)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(in) :: ncdate      ! Current date as yymmdd or yyyymmdd

   integer, intent(out) :: year       ! number of year
!
! Decode month and day
!
   year = int(ncdate/1000000)
   return
end subroutine bndyear
!
!-----------------------------------------------------------------------
subroutine bndhour (ncdate, hour)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(in) :: ncdate      ! Current date as yymmdd or yyyymmdd

   real(r8), intent(out) :: hour       ! Day of year
!
! Local Variables
!
   integer mnth        ! Month number
   integer mday        ! Day number of month
   integer mhour       ! hour number of day
   integer jdcon(12)   ! Starting day number for each month
   save jdcon
   data jdcon/0,31,59,90,120,151,181,212,243,273,304,334/
!
! Decode month and day
!
   mnth = mod(ncdate,1000000)/10000
   mday = mod(ncdate,10000)/100
   mhour= mod(ncdate, 100)
   hour = jdcon(mnth)*24 + (mday-1)*24 + mhour
   
   return
end subroutine bndhour
!
!-----------------------------------------------------------------------
subroutine bndyymmdd (yr, mon, day, ncdate)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
!
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(out) :: ncdate      ! Current date as yymmdd or yyyymmdd

   integer, intent(in) :: yr       ! year
   integer, intent(in) :: mon      ! month
   integer, intent(in) :: day      ! day 
!
! Local Variables
!
   integer  ndays(12)
   integer  nd, nm, ny
   data ndays/31,28,31,30,31,30,31,31,30,31,30,31/
         
!
   nd = day
   nm = mon
   ny = yr
   if ((mod(yr,4).eq.0 .and. mod(yr,100).ne.0) .or. mod(yr,400).eq.0 ) then
      if (mon .eq. 2 .and. day .eq. 29) then
         nm = nm + 1
         nd = 1 
      end if
   end if
   if (nd .gt. ndays(mon)) then 
      nd = nd - ndays(mon) 
      nm = nm + 1
          if (nm .gt. 12) then
              nm = 1
              ny = ny + 1
          end if
   end if  
   ncdate = ny*10000 + nm*100 + nd
   return
end subroutine bndyymmdd
!
!-----------------------------------------------------------------------
subroutine read_ncid (pass_id, name_id, yymmdd, ncid)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ioFileMod,    only: getfil
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments

   character(len=256) :: locfn    ! netcdf local filename to open 
   character(len=256) :: locfn_tmp    ! 2012 
   character(len=256), intent(in) :: pass_id  ! pass id
   character*(*), intent(in) :: name_id  ! file name id
   character(len=256), intent(in) :: yymmdd   ! time of nc
   integer, intent(out) :: ncid  ! file ncid

   if (name_id .eq. 'pres') then
      locfn_tmp = trim(pass_id)//trim(yymmdd(1:4))//'/surface/T106.'//trim(name_id)//'.'//trim(yymmdd)//'.nc'
   else
      locfn_tmp = trim(pass_id)//trim(yymmdd(1:4))//'/pressure/T106.'//trim(name_id)//'.'//trim(yymmdd)//'.nc'
   end if
   !locfn_tmp = trim(pass_id)//trim(yymmdd(1:4))//'/FNL.'//trim(name_id)//'.'//trim(yymmdd)//'.nc'
   call getfil (locfn_tmp, locfn)
   call wrap_open (locfn_tmp, 0, ncid)
   write(6,*)'INITEXT: NCOPN returns id ',ncid_tp,' for file ',trim(locfn)
   return
end subroutine read_ncid
!-------------------------------------------------------------------------
subroutine convert_uv(u3      ,v3      ,vort    ,div)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! 
! Method: 
! Spectrally truncate U, V input fields.
! 
! Author: 
! Original version:  J. Rosinski
! Standardized:      J. Rosinski, June 1992
! Reviewed:          B. Boville, J. Hack, August 1992
! Modified:          P. Worley, May 2003
!
!-----------------------------------------------------------------------

   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid,   only: plon, plond, plev, plat
   use pspect
   use comspe
   use rgrid,    only: nlon, nmmax
   use commap,   only: w, xm, rsq, cs
   use dynconst, only: ez, ra

   implicit none

#include <comctl.h>
#include <comfft.h>
!
! Input/Output arguments
!
   real(r8), intent(inout) :: u3(plond,plev,plat)    ! Fourier -> spec. coeffs. for u-wind
   real(r8), intent(inout) :: v3(plond,plev,plat)    ! Fourier -> spec. coeffs. for v-wind
!
! Output arguments
!
   real(r8), intent(out) :: vort(plond,plev,plat)    ! Spectrally truncated vorticity
   real(r8), intent(out) :: div(plond,plev,plat)     ! Spectrally truncated divergence

!
!---------------------------Local workspace-----------------------------
!
   real(r8) d_tmp(psp)         ! used in spectral truncation of div
   real(r8) vz_tmp(psp)        ! used in spectral truncation of vort
   real(r8) alpn(pspt)         ! alp*rsq*xm*ra
   real(r8) dalpn(pspt)        ! dalp*rsq*ra
   real(r8) tmp1               ! vector temporary
   real(r8) tmp2               ! vector temporary
   real(r8) tmpr               ! vector temporary (real)
   real(r8) tmpi               ! vector temporary (imaginary)
   real(r8) zcor               ! correction for absolute vorticity
   real(r8) zwalp              ! zw*alp
   real(r8) zwdalp             ! zw*dalp
   real(r8) zrcsj              ! ra/(cos**2 latitude)
   real(r8) zw                 ! w**2
#if ( ! defined USEFFTLIB )
   real(r8) work((plon+1)*plev)  ! Workspace for fft
#else
   real(r8) work((plon+1)*pcray)   ! Workspace for fft
#endif
   real(r8) zsqcs

   integer ir,ii               ! indices complex coeffs. of spec. arrs.
   integer i,k                 ! longitude, level indices
   integer irow                ! latitude pair index
   integer latm,latp           ! symmetric latitude indices
   integer lat
   integer m                   ! longitudinal wavenumber index
   integer n                   ! latitudinal wavenumber index
   integer nspec
   integer mr,mc               ! spectral indices
   
   call print_memusage ('convert_uv')
!
!-----------------------------------------------------------------------
!
! Compute the quantities which are transformed to spectral space:
!   1. u = u*sqrt(1-mu*mu),   u * cos(phi)
!   2. v = v*sqrt(1-mu*mu),   v * cos(phi)
!
   do lat=1,plat
      irow = lat
      if (lat.gt.plat/2) irow = plat - lat + 1
      zsqcs = sqrt(cs(irow))
      do k=1,plev
         do i=1,nlon(lat)
            u3(i,k,lat) = u3(i,k,lat)*zsqcs
            v3(i,k,lat) = v3(i,k,lat)*zsqcs
         end do
      end do
!
! Transform grid -> fourier
! 1st transform: U,V,T: note contiguity assumptions
! 2nd transform: LN(PS).  3rd transform: surface geopotential
!
      call fft991(u3(1,1,lat),work,trig(1,irow),ifax(1,irow),1,plond, &
                  nlon(lat),plev,-1)
      call fft991(v3(1,1,lat),work,trig(1,irow),ifax(1,irow),1,plond, &
                  nlon(lat),plev,-1)

   end do                    ! lat=1,plat
!
! Multi-level fields: U, V
!
   do k=1,plev
!
! Zero spectral arrays
!
      vz_tmp(:) = 0.
      d_tmp(:) = 0.
!
! Loop over latitude pairs
!
      do irow=1,plat/2
         latp = irow
         latm = plat - irow + 1
         zrcsj = ra/cs(irow)
         zw = w(irow)*2.
         do i=1,2*nmmax(irow)

            tmp1 = 0.5*(u3(i,k,latm) - u3(i,k,latp))
            tmp2 = 0.5*(u3(i,k,latm) + u3(i,k,latp))
            u3(i,k,latm) = tmp1
            u3(i,k,latp) = tmp2

            tmp1 = 0.5*(v3(i,k,latm) - v3(i,k,latp))
            tmp2 = 0.5*(v3(i,k,latm) + v3(i,k,latp))
            v3(i,k,latm) = tmp1
            v3(i,k,latp) = tmp2

         end do
!     
! Compute vzmn and dmn
!
         do m=1,nmmax(irow)
            mr = nstart(m)
            mc = 2*mr
            do n=1,nlen(m),2
               zwdalp = zw*dalp(mr+n,irow)
               zwalp  = zw*alp (mr+n,irow)
               ir = mc + 2*n - 1
               ii = ir + 1
               d_tmp(ir) = d_tmp(ir) - (zwdalp*v3(2*m-1,k,latm) + &
                  xm(m)*zwalp*u3(2*m  ,k,latp))*zrcsj
               d_tmp(ii) = d_tmp(ii) - (zwdalp*v3(2*m  ,k,latm) - &
                  xm(m)*zwalp*u3(2*m-1,k,latp))*zrcsj
               vz_tmp(ir) = vz_tmp(ir) + (zwdalp*u3(2*m-1,k,latm) - &
                  xm(m)*zwalp*v3(2*m  ,k,latp))*zrcsj
               vz_tmp(ii) = vz_tmp(ii) + (zwdalp*u3(2*m  ,k,latm) + &
                  xm(m)*zwalp*v3(2*m-1,k,latp))*zrcsj
            end do
         end do
         do m=1,nmmax(irow)
            mr = nstart(m)
            mc = 2*mr
            do n=2,nlen(m),2
               zwdalp = zw*dalp(mr+n,irow)
               zwalp  = zw*alp (mr+n,irow)
               ir = mc + 2*n - 1
               ii = ir + 1
               d_tmp(ir) = d_tmp(ir) - (zwdalp*v3(2*m-1,k,latp) + &
                  xm(m)*zwalp*u3(2*m  ,k,latm))*zrcsj
               d_tmp(ii) = d_tmp(ii) - (zwdalp*v3(2*m  ,k,latp) - &
                  xm(m)*zwalp*u3(2*m-1,k,latm))*zrcsj
               vz_tmp(ir) = vz_tmp(ir) + (zwdalp*u3(2*m-1,k,latp) - &
                  xm(m)*zwalp*v3(2*m  ,k,latm))*zrcsj
               vz_tmp(ii) = vz_tmp(ii) + (zwdalp*u3(2*m  ,k,latp) + &
                  xm(m)*zwalp*v3(2*m-1,k,latm))*zrcsj
            end do
         end do
      enddo               ! irow=1,plat/2
! Compute grid point values of:u,v,vz, and d.
!
      do irow=1,plat/2
         latp = irow
         latm = plat - irow + 1
         zcor = ez*alp(2,irow)
!
! Compute(u,v,vz,d)m
!
         do m=1,nmmax(irow)
            mr = nstart(m)
            do n=1,nlen(m)
!
! These statements will likely not be bfb since xm*ra is now a scalar
!
               alpn (mr+n) =  alp(mr+n,irow)*rsq(n+m-1)*xm(m)*ra
               dalpn(mr+n) = dalp(mr+n,irow)*rsq(n+m-1)      *ra
            end do
         end do
!
! Zero fourier fields
!
         u3(:,k,latm) = 0.
         u3(:,k,latp) = 0.

         v3(:,k,latm) = 0.
         v3(:,k,latp) = 0.

         vort(:,k,latm) = 0.
         vort(:,k,latp) = 0.

         div(:,k,latm) = 0.
         div(:,k,latp) = 0.

         do m=1,nmmax(irow)
            mr = nstart(m)
            mc = 2*mr
            do n=1,nlen(m),2
               ir = mc + 2*n - 1
               ii = ir + 1
!
               tmpr = d_tmp(ir)*alpn(mr+n)
               tmpi = d_tmp(ii)*alpn(mr+n)
               u3(2*m-1,k,latm) = u3(2*m-1,k,latm) + tmpi
               u3(2*m  ,k,latm) = u3(2*m  ,k,latm) - tmpr
!
               tmpr = d_tmp(ir)*dalpn(mr+n)
               tmpi = d_tmp(ii)*dalpn(mr+n)
               v3(2*m-1,k,latp) = v3(2*m-1,k,latp) - tmpr
               v3(2*m  ,k,latp) = v3(2*m  ,k,latp) - tmpi
!
               tmpr = vz_tmp(ir)*dalpn(mr+n)
               tmpi = vz_tmp(ii)*dalpn(mr+n)
               u3(2*m-1,k,latp) = u3(2*m-1,k,latp) + tmpr
               u3(2*m  ,k,latp) = u3(2*m  ,k,latp) + tmpi
!
               tmpr = vz_tmp(ir)*alpn(mr+n)
               tmpi = vz_tmp(ii)*alpn(mr+n)
               v3(2*m-1,k,latm) = v3(2*m-1,k,latm) + tmpi
               v3(2*m  ,k,latm) = v3(2*m  ,k,latm) - tmpr
!
               tmpr = d_tmp(ir)*alp(mr+n,irow)
               tmpi = d_tmp(ii)*alp(mr+n,irow)
               div(2*m-1,k,latm) = div(2*m-1,k,latm) + tmpr
               div(2*m  ,k,latm) = div(2*m  ,k,latm) + tmpi
!
               tmpr = vz_tmp(ir)*alp(mr+n,irow)
               tmpi = vz_tmp(ii)*alp(mr+n,irow)
               vort(2*m-1,k,latm) = vort(2*m-1,k,latm) + tmpr
               vort(2*m  ,k,latm) = vort(2*m  ,k,latm) + tmpi
            end do
         end do

         do m=1,nmmax(irow)
            mr = nstart(m)
            mc = 2*mr
            do n=2,nlen(m),2
               ir = mc + 2*n - 1
               ii = ir + 1
!     
               tmpr = d_tmp(ir)*alpn(mr+n)
               tmpi = d_tmp(ii)*alpn(mr+n)
               u3(2*m-1,k,latp) = u3(2*m-1,k,latp) + tmpi
               u3(2*m  ,k,latp) = u3(2*m  ,k,latp) - tmpr
!
               tmpr = d_tmp(ir)*dalpn(mr+n)
               tmpi = d_tmp(ii)*dalpn(mr+n)
               v3(2*m-1,k,latm) = v3(2*m-1,k,latm) - tmpr
               v3(2*m  ,k,latm) = v3(2*m  ,k,latm) - tmpi
!
               tmpr = vz_tmp(ir)*dalpn(mr+n)
               tmpi = vz_tmp(ii)*dalpn(mr+n)
               u3(2*m-1,k,latm) = u3(2*m-1,k,latm) + tmpr
               u3(2*m  ,k,latm) = u3(2*m  ,k,latm) + tmpi
!
               tmpr = vz_tmp(ir)*alpn(mr+n)
               tmpi = vz_tmp(ii)*alpn(mr+n)
               v3(2*m-1,k,latp) = v3(2*m-1,k,latp) + tmpi
               v3(2*m  ,k,latp) = v3(2*m  ,k,latp) - tmpr
!
               tmpr = d_tmp(ir)*alp(mr+n,irow)
               tmpi = d_tmp(ii)*alp(mr+n,irow)
               div(2*m-1,k,latp) = div(2*m-1,k,latp) + tmpr
               div(2*m  ,k,latp) = div(2*m  ,k,latp) + tmpi
!
               tmpr = vz_tmp(ir)*alp(mr+n,irow)
               tmpi = vz_tmp(ii)*alp(mr+n,irow)
               vort(2*m-1,k,latp) = vort(2*m-1,k,latp) + tmpr
               vort(2*m  ,k,latp) = vort(2*m  ,k,latp) + tmpi
            end do
         end do
!
! Correction to get the absolute vorticity.
!     
         vort(1,k,latp) = vort(1,k,latp) + zcor
!
! Recompute real fields from symmetric and antisymmetric parts
!
         do i=1,nlon(latm)+2
            tmp1 = u3(i,k,latm) + u3(i,k,latp)
            tmp2 = u3(i,k,latm) - u3(i,k,latp)
            u3(i,k,latm) = tmp1
            u3(i,k,latp) = tmp2
!
            tmp1 = v3(i,k,latm) + v3(i,k,latp)
            tmp2 = v3(i,k,latm) - v3(i,k,latp)
            v3(i,k,latm) = tmp1
            v3(i,k,latp) = tmp2
!
            tmp1 = vort(i,k,latm) + vort(i,k,latp)
            tmp2 = vort(i,k,latm) - vort(i,k,latp)
            vort(i,k,latm) = tmp1
            vort(i,k,latp) = tmp2
!
            tmp1 = div(i,k,latm) + div(i,k,latp)
            tmp2 = div(i,k,latm) - div(i,k,latp)
            div(i,k,latm) = tmp1
            div(i,k,latp) = tmp2
         end do
      enddo               ! irow=1,plat/2
   enddo                  ! k=1,plev
!
   do lat=1,plat
!     
! Transform Fourier -> grid, obtaining spectrally truncated
! grid point values.
!
      irow = lat
      if (lat.gt.plat/2) irow = plat - lat + 1

      call fft991(u3(1,1,lat),work,trig(1,irow),ifax(1,irow),1,plond, &
                  nlon(lat),plev,+1)
      call fft991(v3(1,1,lat),work,trig(1,irow),ifax(1,irow),1,plond, &
                  nlon(lat),plev,+1)
      call fft991(vort(1,1,lat),work,trig(1,irow),ifax(1,irow),1, &
                  plond,nlon(lat),plev,+1)
      call fft991(div(1,1,lat),work,trig(1,irow),ifax(1,irow),1,plond, &
                  nlon(lat),plev,+1)
!
! Convert U,V to u,v
!
      zsqcs = sqrt(cs(irow))
      do k=1,plev
         do i=1,nlon(lat)
            u3(i,k,lat) = u3(i,k,lat)/zsqcs
            v3(i,k,lat) = v3(i,k,lat)/zsqcs
         end do
      end do
   enddo

   return
end subroutine convert_uv
!-----------------------------------------------------------------------

subroutine initcom_tmp

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Initialize Model commons, including COMCON, COMHYB, COMMAP, COMSPE,
! and COMTRCNM
! 
! Method: 
! 
! Author: 
! Original version:  CCM1
! Standardized:      L. Bath, Jun 1992
!                    L. Buja, Feb 1996
!
!-----------------------------------------------------------------------
!
! $Id: initcom.F90,v 1.16.2.7.4.1 2004/05/20 18:36:05 mvr Exp $
! $Author: mvr $
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use pspect
   use comspe
   use rgrid
   use gauaw_mod, only: gauaw
   use commap
   use abortutils, only: endrun
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
#include <comctl.h>
!-----------------------------------------------------------------------
#include <comfft.h>
!-----------------------------------------------------------------------
#include <comhyb.h>
!-----------------------------------------------------------------------

!
! Local workspace
!
   real(r8) zsi(plat)      ! sine of latitudes
   real(r8) zw(plat)       ! Gaussian weights
   real(r8) zalp(2*pspt)   ! Legendre function array
   real(r8) zdalp(2*pspt)  ! Derivative array
   real(r8) zslat          ! sin of lat  and cosine of colatitude
   real(r8) xlat           ! Latitude (radians)

   integer i           ! longitude index
   integer j           ! Latitude index
   integer itmp        ! Dimension of polynomial arrays temporary.
   integer irow        ! Latitude pair index
   integer lat         ! Latitude index
!
!-----------------------------------------------------------------------!
! MMAX dependent arrays
!
   do j=1,pmmax
      xm(j) = j-1
   end do
#if ( !defined SCAM )
!
! Gaussian latitude dependent arrays
!
   call gauaw(zsi     ,zw      ,plat    )
   do irow=1,plat/2
      slat(irow) = zsi(irow)
      w(irow)              = zw(irow)
      w(plat - irow + 1)   = zw(irow)
      cs(irow)  = 1. - zsi(irow)*zsi(irow)
      xlat = asin(slat(irow))
      clat(irow) = -xlat
      clat(plat - irow + 1) = xlat
   end do

   do lat=1,plat
      latdeg(lat) = clat(lat)*45./atan(1._r8)
   end do
#endif
!
! Compute constants related to Legendre transforms
! Compute and reorder ALP and DALP
!
   allocate( alp  (pspt,plat/2) )
   allocate( dalp (pspt,plat/2) )
   do j=1,plat/2
      zslat = slat(j)
      itmp = 2*pspt - 1
      call phcs  (zalp    ,zdalp   ,itmp    ,zslat    )
      call reordp(j       ,itmp    ,zalp    ,zdalp   )
   end do
!
! Copy and save local ALP and DALP
!
end subroutine initcom_tmp

subroutine getfact (cycflag, np1, chourminus, chourplus, chour, &
                       fact1, fact2, str)
!---------------------------------------------------------------------------
!
! Purpose: Determine time interpolation factors (normally for a boundary dataset)
!          for linear interpolation.
!
! Method:  Assume 365 days per year.  Output variable fact1 will be the weight to
!          apply to data at calendar time "cdayminus", and fact2 the weight to apply
!          to data at time "cdayplus".  Combining these values will produce a result 
!          valid at time "cday".  Output arguments fact1 and fact2 will be between 
!          0 and 1, and fact1 + fact2 = 1 to roundoff.
!
! Author:  Jim Rosinski
!
!--------------------------------------------------------------------------- 

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils, only: endrun
!---------------------------------------------------------------------------
   implicit none
!
! Arguments
!
   logical, intent(in) :: cycflag             ! flag indicates whether dataset is being cycled yearly

   integer, intent(in) :: np1                 ! index points to forward time slice matching cdayplus

   real(r8), intent(in) :: chourminus          ! calendar day of rearward time slice
   real(r8), intent(in) :: chourplus           ! calendar day of forward time slice
   real(r8), intent(in) :: chour               ! calenar day to be interpolated to
   real(r8), intent(out) :: fact1             ! time interpolation factor to apply to rearward time slice
   real(r8), intent(out) :: fact2             ! time interpolation factor to apply to forward time slice

   character(len=*), intent(in) :: str        ! string to be added to print in case of error (normally the callers name)
!
! Local workspace
!
   real(r8) :: deltat                         ! time difference (days) between cdayminus and cdayplus
   real(r8), parameter :: daysperyear = 365.  ! number of days in a year
!
! Initial sanity checks
!
! Determine time interpolation factors.  Account for December-January 
! interpolation if dataset is being cycled yearly.
!
!  if (cycflag .and. np1 == 1) then                     ! Dec-Jan interpolation
   if (np1 == 1 .and. chour >= 8754.0) then             ! non-tmpcyc/tmpcyc Dec-Jan interpolation
      deltat = chourplus + daysperyear*24 - chourminus
      if (chour> chourplus) then                        ! We are in December
         fact1 = (chourplus + daysperyear*24 - chour)/deltat
         fact2 = (chour - chourminus)/deltat
      else                                              ! We are in January
         fact1 = (chourplus - chour)/deltat
         fact2 = (chour + daysperyear*24 - chourminus)/deltat
      end if
   else
      deltat = chourplus - chourminus
      fact1 = (chourplus - chour)/deltat
      fact2 = (chour- chourminus)/deltat
   end if

   return
end subroutine getfact
!----------------------------------------------------------------------!(jiweihua)

subroutine vinterp ( win,ply_in ,nlev_in,wout,ply_out,nlev_out,flag) 

!-------------------------------------------------------------------------------------------
! ATTENTION: Both the input and output pressure sufaces should be MONOTONICALLY INCREASING
!            within each column. Otherwise modify the code.   
!
! (WAN Hui 2003.06)
!------------------------------------------------------------------------------------------- 
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
! parameters, for extrapolation of temperature
!
    real,parameter :: rd    = 287.0
    real,parameter :: cp    = 1004.6
    real,parameter :: capa  = rd/cp
!
! input and output variables
!
    integer,intent(in) :: flag        ! 1 for temperature
    integer,intent(in) :: nlev_in     ! number of vertical layers (input )
    integer,intent(in) :: nlev_out    ! number of vertical layers (output)

    real(r8),intent(in)    :: win ( nlev_in )  ! input  variable to be interpolated
    real(r8),intent(out)   :: wout( nlev_out)  ! output variable after interpolation

    real(r8),intent(in)    :: ply_in ( nlev_in )
    real(r8),intent(in)    :: ply_out( nlev_out)
!
! local workspace
!    
    real(r8) :: p_in (nlev_in)
    real(r8) :: p_out(nlev_out)
    real(r8) :: press
    real(r8) :: pmax,pmin

    integer k,kk,k1,k2
!--------------------------------------------------------------------------
    
        do k=1,nlev_in
           p_in(k) = ply_in(k)
        enddo
		
           pmin = p_in(1      )
           pmax = p_in(nlev_in) 

                     
        do k=1,nlev_out
           p_out(k) = ply_out(k)
        enddo

!---    
          do k=1,nlev_out
           press = p_out(k)
 
! when the boundary of the output pressure surface exceeds the range of the input
          
           if ( press >= pmax ) then
              if (flag==1) then
                wout(k) = win(nlev_in) * ( (press/pmax)**capa )
              else
                wout(k) = win(nlev_in)
              endif
              
           elseif(press <= pmin) then
                wout(k) = win(1)
!
! the normal condition
!             	  
           else
          
             do kk = 2,nlev_in
               k1 = kk-1
               k2 = kk
               if ( (press>=p_in(k1)).and.(press<p_in(k2)) ) then
                if (flag==1) then
                 wout(k) = win(k1) +(win(k2)-win(k1))*log(press/p_in(k1))/log(p_in(k2)/p_in(k1))
                else
                 wout(k) = win(k1)+(win(k2)-win(k1))*(press-p_in(k1))/(p_in(k2)-p_in(k1))
                endif
               endif
             enddo
            
            endif
           
         enddo  !(k=1,nlev_out)
        
!--------------------------------------------------------------------------
    return

end subroutine vinterp
!--------------------------------------------------------------------------

subroutine qneg3_tmp (subnam  ,idx     ,ncol    ,ncold   ,lver    ,lconst  , &
                  qmin    ,q       )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Check moisture and tracers for minimum value, reset any below
! minimum value to minimum value and return information to allow
! warning message to be printed. The global average is NOT preserved.
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author: J. Rosinski
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8

   implicit none

!------------------------------Arguments--------------------------------
!
! Input arguments
!
   character*(*), intent(in) :: subnam ! name of calling routine

   integer, intent(in) :: idx          ! chunk/latitude index
   integer, intent(in) :: ncol         ! number of atmospheric columns
   integer, intent(in) :: ncold        ! declared number of atmospheric columns
   integer, intent(in) :: lver         ! number of vertical levels in column
   integer, intent(in) :: lconst       ! number of constituents

   real(r8), intent(in) :: qmin(lconst)      ! Global minimum constituent concentration

!
! Input/Output arguments
!
   real(r8), intent(inout) :: q(ncold,lver) ! moisture/tracer field
!
!---------------------------Local workspace-----------------------------
!
   integer indx(ncol )      ! array of indices of points < qmin
   integer nval             ! number of points < qmin for 1 level
   integer nvals            ! number of values found < qmin
   integer i,ii,k           ! longitude, level indices
   integer m                ! constituent index
   integer iw,kw            ! i,k indices of worst violator

   logical found            ! true => at least 1 minimum violator found

   real(r8) worst           ! biggest violator
!
!-----------------------------------------------------------------------
!
      m=1
#ifdef HADVTEST
!jr Disable this routine for purposes of advection test
      return
#endif
      nvals = 0
      found = .false.
      worst = 1.e35
!
! Test all field values for being less than minimum value. Set q = qmin
! for all such points. Trace offenders and identify worst one.
!
      do k=1,lver
         nval = 0
         do i=1,ncol
            if (q(i,k) < qmin(m)) then
               nval = nval + 1
               indx(nval) = i
            end if
         end do

         if (nval > 0) then
            found = .true.
            nvals = nvals + nval
            do ii=1,nval
               i = indx(ii)
               if (q(i,k) < worst) then
                  worst = q(i,k)
                  kw = k
                  iw = i
               end if
               q(i,k) = qmin(m)
            end do
         end if
      end do
      if (found .and. abs(worst)>1.e-16) then
!         write(6,9000)subnam,m,idx,nvals,qmin(m),worst,iw,kw
      end if
!
   return
9000 format(' QNEG3 from ',a,':m=',i3,' lat/lchnk=',i3, &
            ' Min. mixing ratio violated at ',i4,' points.  Reset to ', &
            1p,e8.1,' Worst =',e8.1,' at i,k=',i4,i3)
end subroutine qneg3_tmp

!-----------------------------------------------------------
subroutine convert_ncep_time(ncdate,iyear,imon,idays,ihour)
!-----------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
   implicit none
!
! input and output variables
!
    integer,intent(in) :: ncdate      ! time for NCEP
    integer,intent(out) :: iyear     ! number of vertical layers (input )
    integer,intent(out) :: imon      ! number of vertical layers (output)
    integer,intent(out) :: idays      ! number of vertical layers (output)
    integer,intent(out) :: ihour      ! number of vertical layers (output)

    integer time
    integer hor
    integer  ndays(12)
    data ndays/31,28,31,30,31,30,31,31,30,31,30,31/

    time = 0
    do iyear=1,2100
      do imon=1,12
         if (imon .eq. 2) then
            if ((mod(iyear,4).eq.0 .and. mod(iyear,100).ne.0) .or. mod(iyear,400).eq.0 ) then
               ndays(imon) = 29
            else
               ndays(imon) = 28
            end if
         end if
         do idays=1,ndays(imon)
            do hor=1,4
               if (ncdate .eq. time + 24*2 ) then
                   ihour = (hor-1)*6
                   return
               end if
               time = time + 6
            end do
         end do
       end do
    end do
    return
end subroutine convert_ncep_time 

end module atm_force_fnl
#endif

