#include <misc.h>
#include <params.h>
!-----------------------------------------------------------------------
!
! BOP
!
! !MODULE: atm_force_ncep
!
! !DESCRIPTION: Module to handle dealing with the OZONE datasets. 
!
! Public interfaces:
!
!       rainini -- Initialization and reading of dataset.
!       rainint -- Interpolate dataset temperature to current time.
!
! $Id: atm_force_ncep.F90,v 1.3 2010/7/11  jwh Exp $
!
!-----------------------------------------------------------------------

module rain_force_ncep
!
! USES:
!
   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid
   use ppgrid,       only: begchunk, endchunk, pcols
   use phys_grid,    only: get_ncols_p, scatter_field_to_chunk
   use compre
   use prognostics
   use pspect
   use rgrid
   use comspe        ! jwh
   use commap
   use physics_types,only: physics_state
   use constituents, only: ppcnst, qmin
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual, force_start, force_stop,start_ymd,stop_ymd
   use time_manager, only: tmpcyc
   use abortutils,   only: endrun
   use infnan,       only: inf
   use spetru
    
#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none
   public rainini
   public rainint

   integer ncid_pre       !  temperature dataset

   interface

   subroutine bilin (arrin, xin, yin, nlondin, nlonin, &
                     nlevdin, nlev, nlatin, arrout, xout, &
                     yout, nlondout, nlonout, nlevdout, nlatout, fillvalue)

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   integer, intent(in) :: nlondin                        ! longitude dimension of input grid
   integer, intent(in) :: nlonin                         ! number of real longitudes (input)
   integer, intent(in) :: nlevdin                        ! vertical dimension of input grid
   integer, intent(in) :: nlev                           ! number of vertical levels
   integer, intent(in) :: nlatin                         ! number of input latitudes
   integer, intent(in) :: nlatout                        ! number of output latitudes
   integer, intent(in) :: nlondout                       ! longitude dimension of output grid
   integer, intent(in) :: nlonout(nlatout)               ! number of output longitudes per lat
   integer, intent(in) :: nlevdout                       ! vertical dimension of output grid

   real(r8), intent(in) :: arrin(nlondin,nlevdin,nlatin) ! input array of values to interpolate
   real(r8), intent(in) :: xin(nlondin)                  ! input x mesh
   real(r8), intent(in) :: yin(nlatin)                   ! input y mesh
   real(r8), intent(in) :: xout(nlondout,nlatout)        ! output x mesh
   real(r8), intent(in) :: yout(nlatout)                 ! output y mesh
   real(r8), optional, intent(in) :: fillvalue           ! undef. value, wzz, 2009.8.31

   real(r8), intent(out) :: arrout(nlondout,nlevdout,nlatout) ! interpolated array

   end subroutine bilin

   end interface

!===============================================================================
!EOP
!===============================================================================

!===============================================================================
CONTAINS
!===============================================================================

subroutine rainini
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate temp to current time
!
! Method: Find next step of temp data to interpolate. 
!         vertically 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use filenames, only: bndrainpath
   use ioFileMod,    only: getfil
!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
#include <comctl.h>
#include <comlun.h>
#include <comhyb.h>
!
! Local workspace
!
   integer dateid                          ! netcdf id for date variable
   integer dateid2                          ! netcdf id for date variable
   integer londimid                        ! netcdf id for longitude dimension
   integer latdimid                        ! netcdf id for latitude dimension
   integer levdimid                        ! netcdf id for level dimension
   integer londimid2                        ! netcdf id for longitude dimension
   integer latdimid2                        ! netcdf id for latitude dimension
   integer levdimid2                        ! netcdf id for level dimension
   integer levdimid8                       ! netcdf id for level dimension
   integer lonid                           ! netcdf id for longitude variable
   integer latid                           ! netcdf id for latitude variable
   integer levid                           ! netcdf id for level variable
   integer levid8                          ! netcdf id for level variable
   integer timeid                          ! netcdf id for time variable
   integer timeid2                          ! netcdf id for time variable
   integer strt4(4)               ! array of starting indices
   integer cnt4(4)                ! array of counts for each dimension
   integer strt2d(3)              ! array of starting indices
   integer cnt2d(3)               ! array of counts for each dimension
   integer cnt3d(4)               ! array of counts for each dimension
   integer i, j, k, n                 
   integer c, ncol                ! chunk index, number of columns in chunk
   integer :: yr, mon, day        ! components of a date
   integer :: ncyear, ncmon, ncday, nchour
   integer :: data_time_end
   integer :: ncsec               ! current time of day [seconds]
   integer ntmp                   ! temporary
   integer dimids(nf_max_var_dims)! variable shape
   integer ncdays(12)               !day of different month
   integer nday(12)               !day of different month  
   data    ncdays/31,28,31,30,31,30,31,31,30,31,30,31/   !different month to day
   data    nday/0,31,59,90,120,151,181,212,243,273,304,334/   !different month to day
   real(r8) coef1, coef2          ! time interpolation factors
   real(r8) calhorloc             ! calendar hour ( no cycling)
   real(r8), allocatable :: psbdym(:,:)  ! temp data from pre time sample
   real(r8), allocatable :: psbdyp(:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_prem(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: pobs_prep(:,:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_postm(:,:)  ! temp data from pre time sample
   real(r8), allocatable :: pobs_postp(:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_intm(:,:,:)  ! temp data from pre time sample
   real(r8), allocatable :: pobs_intp(:,:,:)  ! temp data from next time sample
   character(len=256) :: locfn    ! netcdf local filename to open 
   character(len=256) :: yymmdd_nc_m   ! 2012
   character(len=256) :: yymmdd_nc_p   ! 2012
!
!----------------------------------------------
   integer, parameter :: tflag = 1        ! 1 for temperature / 0 for others
   integer, parameter :: vflag = 0        ! 1 for temperature / 0 for others

   nm = 1
   np = 2

   fillvalue = inf    ! jwh, 2010.11
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
   if (masterproc) then

      call get_curr_date(yr, mon, day, ncsec)
      calhorloc = nday(mon)*24 + (day-1)*24 + ncsec/3600.
      yc = yr         ! record start year      
      calhorloc = nday(mon)*24 + (day-1)*24 + ncsec/3600.
      write(yymmdd_nc_m,"(i4)") yr-1        ! convert to string
      write(yymmdd_nc_p,"(i4)") yr      ! convert to string

! Read DATA (jwh)
!-----------------------------------------------------------------------
      call read_ncid(bndrainpath, 'prate', yymmdd_nc_m, ncid_pre2)
!
      call read_ncid(bndrainpath, 'prate', yymmdd_nc_p, ncid_pre)
!
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_pre, 'lon', londimid   )
      CALL WRAP_INQ_DIMID( ncid_pre2, 'lon', londimid2   )
      CALL WRAP_INQ_DIMID( ncid_pre, 'time', timeid  )
      CALL WRAP_INQ_DIMID( ncid_pre2, 'time', timeid2  )
      CALL WRAP_INQ_DIMID( ncid_pre, 'lat', latdimid   )
      CALL WRAP_INQ_DIMID( ncid_pre2, 'lat', latdimid2   )


      CALL WRAP_INQ_DIMLEN( ncid_pre, londimid, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_pre, latdimid, latsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_pre, timeid, timesiz   )
      CALL WRAP_INQ_DIMLEN( ncid_pre2, timeid2, timesiz2   )
      CALL WRAP_INQ_DIMLEN( ncid_pre2, londimid2, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_pre2, latdimid2, latsiz   )

      CALL WRAP_INQ_VARID( ncid_pre, 'time', dateid   )
      CALL WRAP_INQ_VARID( ncid_pre2, 'time', dateid2   )
      CALL WRAP_INQ_VARID( ncid_pre, 'lon', lonid   )
      CALL WRAP_INQ_VARID( ncid_pre, 'lat', latid   )
      CALL WRAP_INQ_VARID( ncid_pre, 'prate', preid   )
      CALL WRAP_INQ_VARID( ncid_pre2, 'prate', preid2   )

      CALL WRAP_INQ_VARDIMID (ncid_pre, preid, dimids)
!  
   end if    
!   
#if (defined SPMD )
   call mpibcast( lonsiz, 1, mpiint, 0, mpicom )
   call mpibcast( latsiz, 1, mpiint, 0, mpicom )
   call mpibcast( timesiz, 1, mpiint, 0, mpicom )
   call mpibcast( timesiz2, 1, mpiint, 0, mpicom )
#endif

! Dynamically allocated memory for module comtp

   allocate (date_pre(timesiz))
   allocate (date_pre2(timesiz2))
   allocate (temprain(nm)%val(pcols,begchunk:endchunk))
   allocate (temprain(np)%val(pcols,begchunk:endchunk))
!  
   allocate (psbdym(lonsiz,latsiz))
   allocate (psbdyp(lonsiz,latsiz))
!
   allocate (pobs_prem(lonsiz,1,latsiz))  
   allocate (pobs_prep(lonsiz,1,latsiz))  
   allocate (pobs_intm(plond,1,plat))  
   allocate (pobs_intp(plond,1,plat))  
   allocate (pobs_postm(plond,plat))  
   allocate (pobs_postp(plond,plat))  
!
   allocate (tplon(lonsiz))
   allocate (tplat(latsiz))

   if (masterproc) then
!
!--------------------------------------------------
! read dat(:)

      CALL WRAP_GET_VAR_INT (ncid_pre, dateid, date_pre)
      CALL WRAP_GET_VAR_INT (ncid_pre2, dateid2, date_pre2)

      CALL WRAP_GET_VAR_REALX (ncid_pre, lonid,tplon)
      CALL WRAP_GET_VAR_REALX (ncid_pre, latid,tplat)
!----------------------------------
!
! Convert from millibars to pascals
!
!-----------------------------------
         strt2d(1) = 1
         strt2d(2) = 1

         cnt2d(1)  = lonsiz
         cnt2d(2)  = latsiz
         cnt2d(3)  = 1
!-----------------------------------
!
! Normal interpolation between consecutive time slices.
!         
      if (tmpcyc) then   ! cycle in file1
         n = timesiz
         np1 = 1

         call convert_ncep_time(date_pre(n),ncyear,ncmon,ncday,nchour)
         cdaytmpm = nday(ncmon)*24 + (ncday-1)*24 + nchour 
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         cdaytmpp = nday(ncmon)*24 + (ncday-1)*24 + nchour

!         call bndhour(date_pre(n),  cdaytmpm)
!         call bndhour(date_pre(np1), cdaytmpp)

         if (calhorloc <= cdaytmpp .or. calhorloc > cdaytmpm) then         

         print*,'model_time=',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre(n),ncyear,ncmon,ncday,nchour)
         print*,'obs_time', ncyear,ncmon,ncday,nchour

            strt2d(3) = n
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdym)

!--------------------------------

         print*,'model_time=',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         print*,'obs_time', ncyear,ncmon,ncday,nchour
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdyp)

            goto 10
         end if
      else            ! not cycle
!----------------------------------------------------------------------
         n = timesiz2
         np1 = 1

         call convert_ncep_time(date_pre2(n),ncyear,ncmon,ncday,nchour)
         cdaytmpm = nday(ncmon)*24 + (ncday-1)*24 + nchour
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         cdaytmpp = nday(ncmon-1)*24 + (ncday-1)*24 + nchour

!         call bndhour(date_pre(n),  cdaytmpm)
!         call bndhour(date_pre(np1), cdaytmpp)

         if (calhorloc <= cdaytmpp  .or. &    ! keep between end of file1 and start of file2
            calhorloc > cdaytmpm ) then

         print*,'non-cyc',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre2(n),ncyear,ncmon,ncday,nchour)
         print*,'non-cyc', ncyear,ncmon,ncday,nchour
            strt2d(3) = n
            call wrap_get_vara_realx (ncid_pre2,preid2,strt2d,cnt2d,psbdym)

!---------------------------
         print*,'non-cyc',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         print*,'non-cyc', ncyear,ncmon,ncday,nchour
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdyp)

            goto 10
         end if
!----------------------------------------------------------------------
         n = 1
         np1 = n
         call convert_ncep_time(date_pre(n),ncyear,ncmon,ncday,nchour)
         cdaytmpm = nday(ncmon)*24 + (ncday-1)*24 + nchour
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         cdaytmpp = nday(ncmon)*24 + (ncday-1)*24 + nchour

!         call bndhour(date_pre(n),  cdaytmpm)
!         call bndhour(date_pre(np1), cdaytmpp)

         if (calhorloc <= cdaytmpm ) then       ! start from pre of file1 
            strt2d(3) = n
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdym)

!---------------
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdyp)

            goto 10
      end if   
      end if   
!----------------------------------------------------------------------
      do n=1,timesiz-1
         np1 = n + 1
         call convert_ncep_time(date_pre(n),ncyear,ncmon,ncday,nchour)
         cdaytmpm = nday(ncmon)*24 + (ncday-1)*24 + nchour
         if (ncmon .eq. 2 .and. ncday .eq. 29 .and. nchour .eq. 0) then
           strt2d(3) = n + 4
           strt4(4)  = n + 4
           np1 = n + 1
           cdaytmpm = cdaytmpm + 24
         end if

         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         cdaytmpp = nday(ncmon)*24 + (ncday-1)*24 + nchour
         if (ncmon .eq. 2 .and. ncday .eq. 29 .and. nchour .eq. 0) then
           np1 = np1 + 4
           cdaytmpp = cdaytmpp + 24
         end if

!          call bndhour(date_pre(n),  cdaytmpm)
!          call bndhour(date_pre(np1), cdaytmpp)	 
         if (calhorloc > cdaytmpm .and. calhorloc <= cdaytmpp) then
         print*,'ini',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre(n),ncyear,ncmon,ncday,nchour)
         print*,'ini', ncyear,ncmon,ncday,nchour
            strt2d(3) = n
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdym)

         print*,'ini',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         print*,'ini', ncyear,ncmon,ncday,nchour
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdyp)

            goto 10
                       
         end if
      end do
      write(*,*)'rainini: Failed to find dates bracketing ncdate'
      call endrun

10    continue
      write(*,*)'rainini successful'
!
      call wrap_get_vara_factor (ncid_pre, preid, 'scale_factor', pre_factor)
      call wrap_get_vara_factor (ncid_pre, preid, 'add_offset', pre_offset)
      do i=1,lonsiz
         do j=1,latsiz
            psbdym(i,j) = psbdym(i,j)*pre_factor + pre_offset
            psbdyp(i,j) = psbdyp(i,j)*pre_factor + pre_offset
         end do
      end do
!
      if (tplat(1) > tplat(2)) then
          reverlat = .true.
      end if
      if (reverlat) then
         tplat(1:latsiz) = tplat(latsiz:1:-1)   
         psbdym(:,1:latsiz)    = psbdym(:,latsiz:1:-1)
         psbdyp(:,1:latsiz)    = psbdyp(:,latsiz:1:-1)
      end if

       if (lonsiz.eq.plon .and. latsiz.eq.plat) then
          pobs_postm(:plond,:plat) = psbdym(:lonsiz,:latsiz)
          pobs_postp(:plond,:plat) = psbdyp(:lonsiz,:latsiz)
       else
          do i=1,lonsiz
             pobs_prem(i,1,:) = psbdym(i,:)
             pobs_prep(i,1,:) = psbdyp(i,:)
          end do

          call bilin (pobs_prem, tplon, tplat, lonsiz, lonsiz, &
                     1, 1, latsiz, pobs_intm, londeg, &
                     latdeg, plond, nlon, 1, plat, fillvalue)
 
          call bilin (pobs_prep, tplon, tplat, lonsiz, lonsiz, &
                     1, 1, latsiz, pobs_intp, londeg, &
                     latdeg, plond, nlon, 1, plat, fillvalue)
          do i=1,plond
             pobs_postm(i,:) = pobs_intm(i,1,:)
             pobs_postp(i,:) = pobs_intp(i,1,:)
          end do
       end if
!
   end if
!----------------------------------------------------------------------------------
#if (defined SPMD)
        call scatter_field_to_chunk(1,1,1,plond,pobs_postm,temprain(nm)%val)
        call scatter_field_to_chunk(1,1,1,plond,pobs_postp,temprain(np)%val)
#endif
!  deallocate (date_pre)
   deallocate (psbdym)
   deallocate (psbdyp)
!
   deallocate (pobs_intm)
   deallocate (pobs_intp)
   deallocate (pobs_prem)
   deallocate (pobs_prep)
   deallocate (pobs_postm)
   deallocate (pobs_postp)

!------------------------------------------------------
#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (tplon, lonsiz, mpir8, 0, mpicom)
   call mpibcast (tplat, latsiz, mpir8, 0, mpicom)
   call mpibcast (cdaytmpm, 1, mpir8, 0, mpicom)
   call mpibcast (cdaytmpp, 1, mpir8, 0, mpicom)
   call mpibcast (date_pre, timesiz, mpiint, 0, mpicom )
   call mpibcast (pre_factor, 1, mpir8, 0, mpicom)
   call mpibcast (pre_offset, 1, mpir8, 0, mpicom)
   call mpibcast (reverp,    1, mpilog, 0,mpicom)
   call mpibcast (reverlat,  1, mpilog, 0,mpicom)
#endif
!
   return

end subroutine rainini


subroutine rainint(state)
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate temperature to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
! Method: Find next month of temperature data to interpolate.  Linearly interpolate 
!         vertically and horizontally
! 
! Author: jwh 2010
! 
!-----------------------------------------------------------------------
   use filenames, only: bndrainpath
   use ioFileMod, only: getfil
   use time_manager, only: get_nstep, get_step_size
!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
#include <comctl.h>
#include <comlun.h>
#include <comhyb.h>
!-----------------------------------------------------------------------
!
! Local workspace
!
   type(physics_state), intent(inout), dimension(begchunk:endchunk) :: state
 
   integer dateid
   integer levdimid                        ! netcdf id for level dimension
   integer levdimid8                        ! netcdf id for level dimension
   integer levid                           ! netcdf id for level variable
   integer levid8                           ! netcdf id for level variable
   integer timeid                          ! netcdf id for time variable
   integer :: hyamid           
   integer :: hybmid           
   integer cnt4(4)                ! array of counts for each dimension
   integer cnt3d(4)               ! array of counts for each dimension
   integer strt4(4)               ! array of starting indices
   integer cnt2d(3)               ! array of counts for each dimension
   integer strt2d(3)               ! array of starting indices
   integer i, j, k, n                 
   integer c, ncol                ! chunk index, number of columns in chunk
   integer lchnk                  ! chunk identifier
   integer :: ncyear, ncmon, ncday, nchour
   integer :: yr, mon, day        ! components of a date
   integer :: ncsec               ! current time of day [seconds]
   integer ntmp                   ! temporary
   integer dimids(nf_max_var_dims)! variable shape
   integer ncdays(12)               !day of different month
   data    ncdays/31,28,31,30,31,30,31,31,30,31,30,31/   !different month to day
   integer nday(12)               !day of different month  
   data    nday/0,31,59,90,120,151,181,212,243,273,304,334/   !different month to day
   real(r8) coef1, coef2          ! time interpolation factors
   real(r8) calhorloc             ! calendar hour ( no cycling)
   real(r8), allocatable :: psbdyp(:,:)  ! temp data from next time sample
   real(r8), allocatable :: pobs_intp(:,:,:)
   real(r8), allocatable :: pobs_prep(:,:,:)
   real(r8), allocatable :: pobs_postp(:,:)
   character(len=256) :: locfn    ! netcdf local filename to open 
   character(len=256) :: yymmdd_nc_p   ! 2012
!-----------------------------------------------------------------------
! Purpose: 
! Vertically interpolate input array to output sigma level
! Copy values at boundaries.
   
   integer, parameter :: nlev = 17        !change 'nlev' to 17 for 'u,v,t'; 8 for 'q'
   integer, parameter :: qlev = 8         !change 'nlev' to 17 for 'u,v,t'; 8 for 'q'
   integer, parameter :: tflag = 1        ! 1 for temperature / 0 for others
   integer, parameter :: vflag = 0        ! 1 for temperature / 0 for others
!----------------------------------------------
   real(r8) dtime                 ! timestep size
   real(r8) srtstep, endstep
   integer  nstep, yrsrt, yrsrt_force, yrend_force
   real(r8) daysrt, daysrt_force, dayend_force
!-------------------------
!  real(r8)  lev( nlev )   ! levels of factors
   real(r8)  pre17(nlev)
   real(r8)  pre8(qlev)
   real(r8)  pmid(plev)
   real(r8)  tobs_pre(nlev)
   real(r8)  tobs_sigma(plev)
   real(r8)  qobs_pre(qlev)
   real(r8)  qobs_sigma(plev)
   real(r8)  p0
!  data lev/1000.0,2000.0,3000.0,5000.0,7000.0,10000.0,15000.0,20000.0,25000.0,&
!           30000.0,40000.0,50000.0,60000.0,70000.0,85000.0,92500.0,100000.0/  !17 levs	     
   p0 = 100000.0
!   
! SPMD: Master does all the work.  Sends needed info to slaves
!
   call get_curr_date(yr, mon, day, ncsec)
   calhorloc = nday(mon)*24 + (day-1)*24 + ncsec/3600.

!--------------------------------------------------
!  wtw 
!
  dtime = get_step_size()
  nstep = get_nstep()

  call bnday(start_ymd,daysrt,yrsrt)

!wtw  call bnday(force_start,daysrt_force,yrsrt_force)
  call bnday_hour(force_start,daysrt_force,yrsrt_force)

!wtw  call bnday(force_stop,dayend_force,yrend_force)
  call bnday_hour(force_stop,dayend_force,yrend_force)

  if (yrend_force > yrsrt) then
    srtstep = (86400/dtime)*daysrt_force + (86400/dtime)*(365*(yrsrt_force - yrsrt) - daysrt)
    endstep = (86400/dtime)*dayend_force + (86400/dtime)*(365*(yrend_force - yrsrt) - daysrt)
  else
    srtstep = (86400/dtime)*(daysrt_force - daysrt)
    endstep = (86400/dtime)*(dayend_force - daysrt)
  end if
  if (nstep > endstep ) then
!      return
  endif

!--------------------------------------

   if (masterproc) then
!----------------------------------------------------------
! Get and check dimension info
!
      CALL WRAP_INQ_VARID( ncid_pre, 'prate', preid   )

!     CALL WRAP_INQ_DIMLEN( ncid_qn, levdimid8, levsiz8   )       ! 8  levels
!  
   end if    
!
#if (defined SPMD )
   call mpibcast( lonsiz, 1, mpiint, 0, mpicom )
   call mpibcast( latsiz, 1, mpiint, 0, mpicom )
   call mpibcast( timesiz, 1, mpiint, 0, mpicom )
#endif
!----------------------------------------------------------
! Dynamically allocated memory for module comtp

!  allocate (date_pre(timesiz))
   allocate (psbdyp(lonsiz,latsiz))   
!
   allocate (pobs_prep(lonsiz,1,latsiz))  
   allocate (pobs_postp(plond,plat))  
   allocate (pobs_intp(plond,1,plat))  
!
   if (masterproc) then
!
         strt2d(1) = 1
         strt2d(2) = 1

         cnt2d(1)  = lonsiz
         cnt2d(2)  = latsiz
         cnt2d(3)  = 1
!-----------------------------------
   end if
!
! Normal interpolation between consecutive time slices.
!
!---------------------------------------------------------------------------------
   if (calhorloc > cdaytmpp .and. &
      (.not. (np1 == 1 .and. calhorloc > cdaytmpm) .or. cdaytmpm == cdaytmpp)) then

       np1 = mod(np1,timesiz) + 1
         
       if (.not. tmpcyc .and. np1 == 1) then  ! read next file

          if (masterproc) then
             write(yymmdd_nc_p,"(i4)") yr+1      ! convert to string

             call read_ncid(bndrainpath, 'prate', yymmdd_nc_p, ncid_pre)

             CALL WRAP_INQ_DIMID( ncid_pre, 'time', timeid  )
             CALL WRAP_INQ_DIMLEN( ncid_pre, timeid, timesiz   )

             CALL WRAP_INQ_VARID( ncid_pre, 'time', dateid   )
             CALL WRAP_INQ_VARID( ncid_pre, 'prate', preid   )

          end if
#if (defined SPMD )
          call mpibcast( timesiz, 1, mpiint, 0, mpicom )
#endif
          deallocate(date_pre)
          allocate (date_pre(timesiz))

          if (masterproc) then
             CALL WRAP_GET_VAR_INT (ncid_pre, dateid, date_pre)
          end if

#if (defined SPMD )
          call mpibcast (date_pre, timesiz, mpiint, 0, mpicom )
#endif
       end if 
         cdaytmpm = cdaytmpp
     
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         cdaytmpp = nday(ncmon)*24 + (ncday-1)*24 + nchour
         if (ncmon .eq. 2 .and. ncday .eq. 29 .and. nchour .eq. 0) then
             np1 = np1 + 4
             cdaytmpp = cdaytmpp + 24
         end if

!         call bndhour(date_pre(np1), cdaytmpp)

         ntmp = nm
         nm = np
         np = ntmp
            
     if (masterproc) then

         print*,'model_time=',yr, mon, day, ncsec/3600.
         call convert_ncep_time(date_pre(np1),ncyear,ncmon,ncday,nchour)
         print*,'obs_time', ncyear,ncmon,ncday,nchour
            strt2d(3) = np1
            call wrap_get_vara_realx (ncid_pre,preid,strt2d,cnt2d,psbdyp)
            do i=1,lonsiz
               do j=1,latsiz
                  psbdyp(i,j) = psbdyp(i,j)*pre_factor + pre_offset
               end do
            end do

            if ( reverlat ) then
               psbdyp(:,1:latsiz) = psbdyp(:,latsiz:1:-1)
            end if
!
            if (lonsiz.eq.plon .and. latsiz.eq.plat) then
               pobs_postp(:plond,:plat) = psbdyp(:lonsiz,:latsiz)
            else

               do i=1,lonsiz
                  pobs_prep(i,1,:) = psbdyp(i,:)
               end do

               call bilin (pobs_prep, tplon, tplat, lonsiz, lonsiz, &
                          1, 1, latsiz, pobs_intp, londeg, &
                          latdeg, plond, nlon, 1, plat, fillvalue)

               do i=1,plond
                  pobs_postp(i,:) = pobs_intp(i,1,:)
               end do

            end if
     end if

         call scatter_field_to_chunk(1,1,1,plond,pobs_postp,temprain(np)%val)
   end if
! call scatter_field_to_chunk(1,levsiz,1,lonsiz,tempi,tmp%val)
! call scatter_field_to_chunk(1,levsiz8,1,lonsiz,tempi8,tmp8%val)
!----------------------------------------------------------------------------------
!
! Determine time interpolation factors.
      if (cdaytmpm == cdaytmpp) then
         coef1 = 1.
         coef2 = 0.
      else
          call getfact (tmpcyc, np1, cdaytmpm, cdaytmpp, calhorloc, coef1, coef2 ,'TEMPINT:')
!         write(*,*) coef1,coef2
      end if
!
! Time interpolation.
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
         do i=1,ncol
            state(c)%preobs(i) = temprain(nm)%val(i,c)*coef1 + temprain(np)%val(i,c)*coef2
         end do
   end do
!-------------------------

   deallocate (psbdyp)
   deallocate (pobs_intp)
   deallocate (pobs_prep)
   deallocate (pobs_postp)

   return
end subroutine rainint

!***********************************************************************
   subroutine wrap_get_vara_factor (nfid, varid, factid, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets a range of the given real variable from a input file
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#include <netcdf.inc>

   integer, intent(in):: nfid
   integer, intent(in):: varid
 
   character(len=*), intent(in) :: factid ! name of calling routine
   real(r8), intent(out):: arr 
   integer ret      ! NetCDF return code

   ret = nf_get_att_double(nfid, varid, factid, arr)

   if (ret/=NF_NOERR) then
     write(6,*)'wrap_get_vara_factor: error reading varid =', varid
     call handle_error (ret)
   end if
   end subroutine wrap_get_vara_factor
!-----------------------------------------------------------------------
subroutine read_ncid (pass_id, name_id, yymmdd, ncid)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ioFileMod,    only: getfil
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   character(len=256) :: locfn    ! netcdf local filename to open 
   character(len=256) :: locfn_tmp    ! 2012 
   character(len=256), intent(in) :: pass_id  ! pass id
   character*(*), intent(in) :: name_id  ! file name id
   character(len=256), intent(in) :: yymmdd   ! time of nc
   integer, intent(out) :: ncid  ! file ncid

   if (name_id .eq. 'prate') then
      locfn_tmp = trim(pass_id)//trim(name_id)//'.sfc.gauss.'//trim(yymmdd)//'.nc'
   else
      locfn_tmp = trim(pass_id)//trim(name_id)//'.'//trim(yymmdd)//'.nc'
   end if
   call getfil (locfn_tmp, locfn)
   call wrap_open (locfn_tmp, 0, ncid)
   write(6,*)'INITEXT: NCOPN returns id ',name_id,' for file ',trim(locfn)
   return
end subroutine read_ncid
!-----------------------------------------------------------------------
subroutine bndyear (ncdate, year)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(in) :: ncdate      ! Current date as yymmdd or yyyymmdd

   integer, intent(out) :: year       ! number of year
!
! Decode month and day
!
   year = int(ncdate/1000000)
   return
end subroutine bndyear
!
!-----------------------------------------------------------------------
subroutine bndhour (ncdate, hour)
!----------------------------------------------------------------------- 
! 
! Purpose: Convert date and seconds of day to floating point calendar day, for
!          boundary dataset handling
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
!-----------------------------------------------------------------------
   implicit none
!--------------------------Arguments------------------------------------
!
! Arguments
!
   integer, intent(in) :: ncdate      ! Current date as yymmdd or yyyymmdd

   real(r8), intent(out) :: hour       ! Day of year
!
! Local Variables
!
   integer mnth        ! Month number
   integer mday        ! Day number of month
   integer mhour       ! hour number of day
   integer jdcon(12)   ! Starting day number for each month
   save jdcon
   data jdcon/0,31,59,90,120,151,181,212,243,273,304,334/
!
! Decode month and day
!
   mnth = mod(ncdate,1000000)/10000
   mday = mod(ncdate,10000)/100
   mhour= mod(ncdate, 100)
   hour = jdcon(mnth)*24 + (mday-1)*24 + mhour
   
   return
end subroutine bndhour
!
subroutine getfact (cycflag, np1, chourminus, chourplus, chour, &
                       fact1, fact2, str)
!---------------------------------------------------------------------------
!
! Purpose: Determine time interpolation factors (normally for a boundary dataset)
!          for linear interpolation.
!
! Method:  Assume 365 days per year.  Output variable fact1 will be the weight to
!          apply to data at calendar time "cdayminus", and fact2 the weight to apply
!          to data at time "cdayplus".  Combining these values will produce a result 
!          valid at time "cday".  Output arguments fact1 and fact2 will be between 
!          0 and 1, and fact1 + fact2 = 1 to roundoff.
!
! Author:  Jim Rosinski
!
!--------------------------------------------------------------------------- 

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils, only: endrun
!---------------------------------------------------------------------------
   implicit none
!
! Arguments
!
   logical, intent(in) :: cycflag             ! flag indicates whether dataset is being cycled yearly

   integer, intent(in) :: np1                 ! index points to forward time slice matching cdayplus

   real(r8), intent(in) :: chourminus          ! calendar day of rearward time slice
   real(r8), intent(in) :: chourplus           ! calendar day of forward time slice
   real(r8), intent(in) :: chour               ! calenar day to be interpolated to
   real(r8), intent(out) :: fact1             ! time interpolation factor to apply to rearward time slice
   real(r8), intent(out) :: fact2             ! time interpolation factor to apply to forward time slice

   character(len=*), intent(in) :: str        ! string to be added to print in case of error (normally the callers name)
!
! Local workspace
!
   real(r8) :: deltat                         ! time difference (days) between cdayminus and cdayplus
   real(r8), parameter :: daysperyear = 365.  ! number of days in a year
!
! Initial sanity checks
!
! Determine time interpolation factors.  Account for December-January 
! interpolation if dataset is being cycled yearly.
!
!  if (cycflag .and. np1 == 1) then                     ! Dec-Jan interpolation
   if (np1 == 1) then                 ! non-tmpcyc/tmpcyc Dec-Jan interpolation
      deltat = chourplus + daysperyear*24 - chourminus
      if (chour> chourplus) then                        ! We are in December
         fact1 = (chourplus + daysperyear*24 - chour)/deltat
         fact2 = (chour - chourminus)/deltat
      else                                              ! We are in January
         fact1 = (chourplus - chour)/deltat
         fact2 = (chour + daysperyear*24 - chourminus)/deltat
      end if
   else
      deltat = chourplus - chourminus
      fact1 = (chourplus - chour)/deltat
      fact2 = (chour- chourminus)/deltat
   end if

   return
end subroutine getfact
!-----------------------------------------------------------
subroutine convert_ncep_time(ncdate,iyear,imon,idays,ihour)
!-----------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun
   implicit none
!
! input and output variables
!
    integer,intent(in) :: ncdate      ! time for NCEP
    integer,intent(out) :: iyear     ! number of vertical layers (input )
    integer,intent(out) :: imon      ! number of vertical layers (output)
    integer,intent(out) :: idays      ! number of vertical layers (output)
    integer,intent(out) :: ihour      ! number of vertical layers (output)

    integer time
    integer hor
    integer  ndays(12)
    data ndays/31,28,31,30,31,30,31,31,30,31,30,31/

    time = 0
    do iyear=1800,2100
      do imon=1,12
         if (imon .eq. 2) then
            if ((mod(iyear,4).eq.0 .and. mod(iyear,100).ne.0) .or. mod(iyear,400).eq.0 ) then
               ndays(imon) = 29
            else
               ndays(imon) = 28
            end if
         end if
         do idays=1,ndays(imon)
            do hor=1,4
               if (ncdate .eq. time) then
                   ihour = (hor-1)*6
                   return
               end if
               time = time + 6
            end do
         end do
       end do
    end do
    return
end subroutine convert_ncep_time 
!-----------------------------
end module rain_force_ncep

