! --------------------------------------------------
MODULE AVERAGE_FIELDS_MOD
! --------------------------------------------------
  IMPLICIT NONE
  
CONTAINS
  
! --------------------------------------------------
  SUBROUTINE average_fields(tabs, qv, qc, qi, &
                            ave_t, ave_qv, cloud_index)
! --------------------------------------------------
    
! --------------------------------------------------
! Computes two vertical profiles of temperature and water vapour,
! one for clear columns and one for cloudy columns.
! --------------------------------------------------

    USE shr_kind_mod,only: r8 => shr_kind_r8
    USE ppgrid

    IMPLICIT NONE
    INCLUDE 'rad_PSU.inc'

!
! INPUT DATA
!
    REAL(r8), INTENT(IN) ::  &
    tabs(NX,NY,NZ),          & ! Domain temperature     (K)
    qv(NX,NY,NZ),            & ! Domain water vapour    (g/kg)
    qc(NX,NY,NZ),            & ! Domain LWC             (g/kg)
    qi(NX,NY,NZ)               ! Domain IWC             (g/kg)

!    LOGICAL, INTENT(IN) :: &
!    ocean                   ! Is the surface mostly ocean or land

!
! OUTPUT DATA
!
    
    REAL, INTENT(OUT) :: &
    ave_t(1,2,NZP2),     & ! Clear and cloudy temperature profiles, mid-level                (K)
    ave_qv(1,2,NZP2)       ! Clear and cloudy water vapour mixing ratio profiles, mid-level  (g/kg)
  
    INTEGER, INTENT(OUT) :: &
    cloud_index(NX,NY)       ! Clear and cloudy columns
  
!
! LOCAL VARIABLES
!

    INTEGER :: &
    ix,        & ! X direction column number
    iy,        & ! Y direction column number
    iz,        & ! Z direction layer number
    flag,      & ! Cloud flag
    count        ! Counter

    REAL(r8) :: &
    r_count,    & ! Counter
    CLD_EPS_L,  & ! Minimum cloud water content cutoff
    CLD_EPS_I     ! Minimum cloud ice content cutoff

!    IF (ocean) THEN ! Over ocean
!       CLD_EPS_L = OCEAN_CLD_EPS_L
!       CLD_EPS_I = OCEAN_CLD_EPS_I
!    ELSE ! Over land
!       CLD_EPS_L = LAND_CLD_EPS_L
!       CLD_EPS_I = LAND_CLD_EPS_I
!    END IF

! Go through field and find the cloudy and clear columns

    cloud_index(:,:) = 0
    flag = 0
    DO ix = 1,NX
       DO iy = 1,NY
          DO iz = 1,NZ
             IF (qc(ix,iy,iz) > LAND_CLD_EPS_L & 
                  .OR. qi(ix,iy,iz) > LAND_CLD_EPS_I) THEN ! Cloudy
                flag = 1
             END IF
          END DO
          IF (flag == 1) THEN ! Cloudy
             cloud_index(ix,iy) = 1
             flag = 0
          END IF
       END DO
    END DO
    
! Generate average pressure, temperature and water vapour profiles for
! clear and cloudy columns

    ave_t(:,:,:)  = 0.0_r8
    ave_qv(:,:,:) = 0.0_r8
    
    DO ix = 1,NX
       DO iy = 1,NY
          IF (cloud_index(ix,iy) .EQ. 0) THEN ! Clear column
             ave_t(1,1,1:NZ)  = ave_t(1,1,1:NZ) + tabs(ix,iy,1:NZ)
             ave_qv(1,1,1:NZ) = ave_qv(1,1,1:NZ) + qv(ix,iy,1:NZ)
          ELSE ! Cloudy column
             ave_t(1,2,1:NZ)  = ave_t(1,2,1:NZ) + tabs(ix,iy,1:NZ)
             ave_qv(1,2,1:NZ) = ave_qv(1,2,1:NZ) + qv(ix,iy,1:NZ)           
          END IF
       END DO
    END DO
    
    count = NX*NY - sum(cloud_index(:,:)) ! Number clear columns
    r_count = REAL(count)
    IF (count .GT. 0) THEN
       ave_t(1,1,:)  = ave_t(1,1,:)/r_count
       ave_qv(1,1,:) = ave_qv(1,1,:)/r_count
    ELSE ! No clear columns, divide by number of cloudy columns
       count = NX*NY - count
       r_count = REAL(count)
       ave_t(1,1,:)  = ave_t(1,2,:)/r_count
       ave_qv(1,1,:) = ave_qv(1,2,:)/r_count
    END IF
    
    count = sum(cloud_index(:,:)) ! Number cloudy columns
    r_count = REAL(count)
    IF (count .GT. 0) THEN
       ave_t(1,2,:)  = ave_t(1,2,:)/r_count
       ave_qv(1,2,:) = ave_qv(1,2,:)/r_count  
    ELSE ! No cloudy columns
       ave_t(1,2,:)  = ave_t(1,1,:)
       ave_qv(1,2,:) = ave_qv(1,1,:)
    END IF
    
    RETURN
  END SUBROUTINE average_fields

END MODULE AVERAGE_FIELDS_MOD
