! --------------------------------------------------
MODULE COMPUTE_EXTRA_LAYERS_ZHANG_MOD
! --------------------------------------------------

  IMPLICIT NONE
  
CONTAINS

! --------------------------------------------------
  SUBROUTINE compute_extra_layers_zhang(interp_z,  ave_p, &
                                  interp_p, ave_t, interp_t, ave_qv, o3)
! --------------------------------------------------

    USE shr_kind_mod,only: r8 => shr_kind_r8
    USE ppgrid
    USE ATMOSPHERE_MOD

    IMPLICIT NONE
    INCLUDE 'rad_PSU.inc'
    INTEGER, PARAMETER :: NZLS0 = 500

! --------------------------------------------------
! When using a mesoscale model, the top of the domain is
! usually around 10 hPa (about 30 km).  This can't be used as the 
! top of atmosphere since it is in the heart of the ozone layer
! and so the heating rates at the top of the model will be
! incorrect.  So we add four layers to the top of the model
! which will take us from the model top to 70 km 
! --------------------------------------------------

!
! OUTPUT DATA
!

    REAL(r8), INTENT(INOUT) :: &
    interp_z(NZP2+1),        & ! Heights at full-level                              (m)
    ave_p(1,2,NZP2),         & ! Clear and cloudy pressure profiles, mid-level      (hPa)
    interp_p(1,2,NZP2+1),    & ! Clear and cloudy pressure profiles, full-level     (hPa)
    ave_t(1,2,NZP2),         & ! Clear and cloudy temperature profiles, mid-level   (K)
    interp_t(1,2,NZP2+1),    & ! Clear and cloudy temperature profiles, full-level  (K)
    ave_qv(1,2,NZP2),        & ! Clear and cloudy water vapour  profiles, mid-level (g/kg)
    o3(NZP2)                   ! Vertical ozone profile                             (g/g)
    
!
! LOCAL VARIABLES
!

    INTEGER :: &
    ii,        &
    jj,        &
    ix,        &
    iy,        &
    iz,        &
    index(5)
    
    REAL(r8) :: &
    slope,      & ! Slope
    num,        & ! Numerator
    denom,      & ! Denominator
    new_pt        ! Interpolated value

    REAL(r8) :: &
    sigma1,     & ! Ratio pressure to sea level density
    sigma2,     & ! Ratio pressure to sea level density
    delta1,     & ! Ratio pressure to sea level pressure 
    delta2,     & ! Ratio pressure to sea level pressure
    theta1,     & ! Ratio pressure to sea level temperature
    theta2        ! Ratio pressure to sea level temperature
    
! Check if the top of the model is >= 50 km.  If so then halt the code as
! the fluxes will be garbage
    
    IF (interp_z(NZ+1) >= 65000.0) THEN   ! replace 50000. with 6500, test by Jing X. W.
       WRITE (*,*) 'Top of model = ',interp_z(NZ+1)
       STOP "Top of model domain >= 65 km, radiative fluxes and heating rates will be garbage!"
    END IF

! Build the extra layers

! Height (fixed) 
    
    interp_z(NZ+2) = 65000.0_r8 ! metres  ! replace 65000. with 65000, test by Jing X. W.
    interp_z(NZ+3) = 67500.0_r8 ! metres  ! replace 67500. with 65000, test by Jing X. W.
    interp_z(NZ+4) = 70000.0_r8 ! metres  ! replace 70000. with 65000, test by Jing X. W.
!    interp_z(NZ+5) = 70000.0_r8 ! metres ! replace 50000. with 65000, test by Jing X. W.    

! Compute pressure and temperature for extra levels and layers
! Instead of doing this as a function of time do it only for the first 
! profile and reuse it 
    
    DO iz = 2,4

       CALL atmosphere(interp_z(NZ+iz-1)/1000.0_r8,sigma1,delta1,theta1)
       CALL atmosphere(interp_z(NZ+iz)/1000.0_r8,sigma2,delta2,theta2)
       
       interp_t(:,:,NZ+iz)   = (theta2/theta1)*interp_t(:,:,NZ+iz-1)
       interp_p(:,:,NZ+iz)   = (delta2/delta1)*interp_p(:,:,NZ+iz-1)    
       
       ave_p(:,:,NZ+iz-1) = exp((LOG(interp_p(:,:,NZ+iz)) + LOG(interp_p(:,:,NZ+iz-1)))*0.5_r8)
       ave_t(:,:,NZ+iz-1) = (interp_t(:,:,NZ+iz) + interp_t(:,:,NZ+iz-1))*0.5_r8
    END DO


! Estimate the ozone and water vapour mixing ratio for extra layers
! Do this by scaling with pressure from the topmost layer

    DO iz = 1,3
     !  ave_qv(:,:,NZ+iz) = MAX(1.0E-6,ave_qv(:,:,NZ)*(ave_p(:,:,NZ+iz)/ave_p(:,:,NZ))) !JNSC
     !  o3(NZ+iz)         = MAX(1.0E-6,o3(NZ)*(ave_p(1,2,NZ+iz)/ave_p(1,2,NZ)))         !JNSC

     ! modified by jxw, to coodinate with new units(ppmv), 2010.10.26
       ave_qv(:,:,NZ+iz) = MAX(1.0E-6,ave_qv(:,:,NZ)*(ave_p(:,:,NZ+iz)/ave_p(:,:,NZ)))
     !  o3(NZ+iz)         = MAX(8.0,o3(NZ)*(ave_p(1,2,NZ+iz)/ave_p(1,2,NZ)))
     !  o3(NZ+iz)         = o3(NZ)*(ave_p(1,2,NZ+iz)/ave_p(1,2,NZ))
       o3(NZ+iz)         = o3(NZ+iz-1)*1.0_r8
    END DO
    RETURN
  END SUBROUTINE compute_extra_layers_zhang

END MODULE COMPUTE_EXTRA_LAYERS_ZHANG_MOD
