! --------------------------------------------------------------------
MODULE MSC_CLD_GENERATOR_MRO_MOD
! --------------------------------------------------------------------

  IMPLICIT NONE

CONTAINS

! --------------------------------------------------------------------
  SUBROUTINE MSC_CLD_GENERATOR_MRO(LMAXRAN, LPPH, ZF, AVG_CF, AVG_QLW, AVG_QIW, &
                               RLC_CF, RLC_CW, SIGMA_QCW, QLW, QIW)
! --------------------------------------------------------------------
! --------------------------------------------------------------------
! Computes a cloudy column column using Petri Raisanen and 
! Howard Barker's algorithms.  Domain averages of many columns tend toward
! input profiles.  This is a simplfied version to suit my limited purposes
! of coupling the output to my McICA code.  Removed generation of 
! horizontally varying effective radii, cloud mask, etc.
! --------------------------------------------------------------------   

    USE shr_kind_mod,only: r8 => shr_kind_r8  ! Precision used by reals
    USE ppgrid                ! GCM dimensions
    USE rad_PSU_dims_include_mod  ! McICA dimensions
    USE RAN_NUMBERS_MOD          ! Random number code 
    
!    USE XCW_DATA_MOD

    IMPLICIT NONE

! Note: NZ => Number of layers in GCM, should equal pver
!       NY & NX are dimensions for McICA fields   


!
! INPUT DATA
!

    REAL(r8), INTENT(IN) :: &
    ZF(NZ),                 & ! Full-level (layer midpoint) altitude (km)
    AVG_CF(NZ),             & ! Cloud fraction for each layer
    AVG_QLW(NZ),            & ! Cloud mean liquid water mixing ratio (kg/kg)
    AVG_QIW(NZ),            & ! Cloud mean ice mixing ratio          (kg/kg)
    SIGMA_QCW(NZ),          & ! Normalized cloud condensate std. dev.
    RLC_CF(NZ-1),           & ! Cloud fraction decorrelation length
    RLC_CW(NZ-1)              ! Cloud condensate decorrelation length

    
    LOGICAL, INTENT(IN) :: &
    LMAXRAN,               & ! Maximum-random overlap flag
    LPPH                     ! Plane-parallel homogeneous flag

!
! OUTPUT DATA
!

    REAL(r8),INTENT(OUT) :: &
    QLW(NX,NY,NZ),          & ! Liquid water mixing ratio field (kg/kg)
    QIW(NX,NY,NZ)             ! Ice mixing ratio field          (kg/kg)


!
! LOCAL DATA
!

    INTEGER :: &
    I,         &
    J,         &
    K,         &
    K_TOP,     &
    K_BASE,    &
    IND1,      &
    IND2

    INTEGER, PARAMETER :: N1=1000, N2=140
    REAL(R8) :: XCW(1000, 140)
    
    REAL(r8) ::  &
    ALPHA(NZ-1), &
    RCORR(NZ-1), &
    X(NX),       &
    Y(NX),       &
    RIND1,       &
    RIND2,       &
    ZCW,         &
    RLWC,        &
    B

!    REAL :: &
!    rand_     ! Random number generator
    
!************************************************************
! Generate "binary" cloud field for the McICA algorithm
!************************************************************


! Initialization of subgrid cloud field
      DO K=1,NZ
         DO J = 1,NY
            DO I=1,NX
               QLW(I,J,K)  = 0.0
               QIW(I,J,K)  = 0.0
            END DO ! I
         END DO ! J
      END DO ! K

! Find uppermost cloudy layer

      K_TOP = 0
      DO K=1,NZ
         K_TOP = K
         IF (AVG_CF(K) .GT. 0.0) EXIT
      END DO

! If no cloudy layers, exit
      IF (K_TOP .EQ. 0) THEN         
         RETURN
      END IF   

! Find lowermost cloudy layer

      K_BASE = 0
      DO K=NZ,1,-1
         K_BASE = K
         IF (AVG_CF(K) .GT. 0.0) EXIT
      END DO

! Calculate overlap factors ALPHA for cloud fraction and RCORR for cloud 
! condensate based on layer midpoint distances and decorrelation depths

      DO K=K_TOP,K_BASE-1
         IF (RLC_CF(K) .GT. 0.0) THEN
            ALPHA(K) = EXP(-abs(ZF(K) - ZF(K+1)) / RLC_CF(K)) 
         ELSE
            ALPHA(K) = 0.0
         END IF
         IF (RLC_CW(K) .GT. 0.0) THEN
            RCORR(K) = EXP(-abs(ZF(K) - ZF(K+1)) / RLC_CW(K)) 
         ELSE
            RCORR(K) = 0.0
         END IF
      END DO

! Generate NXxNY subcolumns
      DO K=K_TOP,K_BASE
         DO J = 1,NY
            DO I=1,NX
               
               IF (K .EQ. K_TOP) THEN
                  X(I) = rand_()
                  Y(I) = rand_()
               ELSE
                  
! Maximum-random overlap
                  IF (LMAXRAN) THEN 
                     IF (X(I) .LT. 1.0-AVG_CF(K-1)) THEN  !It is clear above
                        X(I) = rand_() * (1.0 - AVG_CF(K-1)) 
                        Y(I) = rand_() 
                     END IF
! Generalized overlap
                  ELSE
                     IF (rand_() .GT. ALPHA(K-1)) X(I) = rand_()
                     IF (rand_() .GT. RCORR(K-1)) Y(I) = rand_()
                  END IF
               END IF


! Treatment of cloudy cells    
               IF (X(I) .GT. 1.0-AVG_CF(K)) THEN ! Plane-parallel clouds
                  IF (LPPH) THEN
                     ZCW = 1.0
                  ELSE
! Horizontally veriable clouds:   
! Determine ZCW = ratio of cloud condensate miximg ratio QC for this cell to 
! its mean value for all cloudy cells in this layer.
! Use bilinear interpolation of ZCW tabulated in array XCW as a function of
!    * cumulative probability Y
!    * relative standard deviation SIGMA
! Take care that the definition of RIND2 is consistent with subroutine
! TABULATE_XCW

                     RIND1 = Y(I) * (N1 - 1) + 1.0
                     IND1  = MAX(1, MIN(INT(RIND1), N1-1))
                     RIND1 = RIND1 - IND1       
                     RIND2 = 40.0 * SIGMA_QCW(K) - 3.0
                     IND2  = MAX(1, MIN(INT(RIND2), N2-1))
                     RIND2 = RIND2 - IND2
                     
                     ZCW = (1.0-RIND1) * (1.0-RIND2) * XCW(IND1,IND2)   &
                         + (1.0-RIND1) * RIND2       * XCW(IND1,IND2+1) &
                         + RIND1 * (1.0-RIND2)       * XCW(IND1+1,IND2) &
                         + RIND1 * RIND2             * XCW(IND1+1,IND2+1)
                  END IF
! A horizontally constant IWC/LWC ratio is assumed for each layer so far
                  QLW(I,J,K) = ZCW * AVG_QLW(K)
                  QIW(I,J,K) = ZCW * AVG_QIW(K)
               END IF
            END DO ! I
         END DO ! J
      END DO ! K
      RETURN
    END SUBROUTINE MSC_CLD_GENERATOR_MRO
  END MODULE MSC_CLD_GENERATOR_MRO_MOD
