!___------------------------------------------------------------------
MODULE MSC_CLD_GEN_DRIVER_MRO_MOD
! --------------------------------------------------------------------

  IMPLICIT NONE

CONTAINS

! --------------------------------------------------------------------
  SUBROUTINE MSC_CLD_GEN_DRIVER_MRO(cf_in, t_in, qv_in, qi_in, qc_in,   &
                                zm_in, t_rad, qv_rad, qi_rad, qc_rad)
! --------------------------------------------------------------------
! --------------------------------------------------------------------
! Driver to compute required fields needed by McICA. Computes a 
! cloud field using Petri Raisanen and Howard Barker's algorithms.  
! Create other fields needed for the McICA code while here as well.
! --------------------------------------------------------------------   

    USE shr_kind_mod,only: r8 => shr_kind_r8 ! Precision used by reals
    USE ppgrid               ! GCM dimensions
    USE rad_PSU_dims_include_mod ! McICA dimensions
    USE MSC_CLD_GENERATOR_MRO_MOD  ! Cloud generator

    IMPLICIT NONE

! Note: NZ => Number of layers in GCM, should equal pver
!       NY & NX are dimensions for McICA fields   

!
! INPUT DATA
!

    REAL(r8), INTENT(IN) :: &
    cf_in(NZ),              & ! GCM column cloud fraction
    t_in(NZ),               & ! GCM column temperature profile               (K)
    qv_in(NZ),              & ! GCM column water vapour profile              (g/g)
    qi_in(NZ),              & ! GCM column ice water mixing ratio profile    (kg/kg)
    qc_in(NZ),              & ! GCM column liquid water mixing ratio profile (kg/kg)
    zm_in(NZ)                 ! GCM vertical layer mid-points                (m)

!
! OUTPUT DATA
!

    REAL(r8), INTENT(OUT) :: & 
    t_rad(NX,NY,NZ),         & ! McICA temperature field                (K)  
    qv_rad(NX,NY,NZ),        & ! McICA water vapour field               (g/kg)
    qi_rad(NX,NY,NZ),        & ! McICA ice water mixing ratio field     (kg/kg)
    qc_rad(NX,NY,NZ)           ! McICA liquid water mixing ratio field  (kg/kg)

!
! LOCAL DATA
!

    INTEGER :: &
    ii,        &
    jj,        &
    kk,        &
    kkinv

    REAL(r8) ::    &
    zm(NZ),        & ! GCM vertical layer mid-points                (km)
    rlc_cf(NZ-1),  & ! Cloud fraction decorrelation length
    rlc_cw(NZ-1),  & ! Cloud condensate decorrelation length
    sigma_qcw(NZ)    ! Normalized standard deviation of cloud condensate

    LOGICAL :: &
    lmaxran,   & ! Maximum-random overlap flag
    lpph         ! Plan-parallel homogeneous flag

    REAL(r8) :: &
    tflip1(NZ), & ! Temp array for flipping LWC fields
    tflip2(NZ)    ! Temp array for flipping IWC fields

! Zero out fields
    DO kk = 1,NZ
       DO jj = 1,NY
          DO ii = 1,NX
             t_rad(ii,jj,kk)  = 0.0_r8
             qv_rad(ii,jj,kk) = 0.0_r8
             qi_rad(ii,jj,kk) = 0.0_r8
             qc_rad(ii,jj,kk) = 0.0_r8
          END DO
       END DO
    END DO

! Convert zm_in from metres to kilometers

    DO kk = 1,NZ
       zm(kk) = zm_in(kk)/1000.0_r8
    END DO

! Just put the same temperature and water vapour into each column of field
! Remember to flip the GCM profiles (go top to bottom, want the other way)

    DO kk = 1,NZ
       kkinv = NZ-kk+1
       DO jj = 1,NY
          DO ii = 1,NX
             t_rad(ii,jj,kk)  = t_in(kkinv)
             qv_rad(ii,jj,kk) = qv_in(kkinv)*1000.0_r8 ! kg/kg to g/kg
          END DO ! ii
       END DO ! jj
    END DO ! kk

! Set values needed by cloud generator

! Cloud overlap

    lmaxran = .FALSE. ! Use maximum-random overlap?
    lpph    = .TRUE. ! Use horizontally homogeneous clouds?

! Cloud fraction, cloud condensate decorrelation depth
! and normalized cloud condensate standard deviation
! Just set single values for now
    
    DO kk = 1,NZ-1
       rlc_cf(kk)    = 2.0 
       rlc_cw(kk)    = 1.0
       sigma_qcw(kk) = 0.0 
    END DO ! kk

    sigma_qcw(NZ) = 0.0

! Call cloud generator

    CALL MSC_CLD_GENERATOR_MRO(lmaxran, lpph, zm, cf_in, qc_in, qi_in, rlc_cf, &
                           rlc_cw, sigma_qcw, qc_rad, qi_rad)

! Cloud generator works from top to bottom, need to flip cloud fields for 
! radiation which works from bottom to top

    DO jj = 1,NY
       DO ii = 1,NX

          DO kk = 1,NZ
             kkinv = NZ-kk+1
             tflip1(kkinv) = qc_rad(ii,jj,kk)
             tflip2(kkinv) = qi_rad(ii,jj,kk)
          END DO ! kk
          
          DO kk = 1,NZ
             qc_rad(ii,jj,kk) = tflip1(kk)
             qi_rad(ii,jj,kk) = tflip2(kk)
          END DO ! kk

       END DO ! ii
    END DO ! jj

    RETURN

  END SUBROUTINE MSC_CLD_GEN_DRIVER_MRO
  
END MODULE MSC_CLD_GEN_DRIVER_MRO_MOD
