! --------------------------------------------------------------------
MODULE RAN_NUMBERS_MOD
! --------------------------------------------------------------------

! --------------------------------------------------------------------
! This random number generator is taken from Numerical Recipes, who
! borrowed it from Knuth.  To use the random number generator in
! the GCM I REALLY needed to have a stream of random numbers that I can reproduce.
! After doing it incorrectly and talking to Robert Pincus figure the best way to do things
! is to generate different random number streams for each GCM column.  The stream is made
! reproducable by re-seeding the model at each GCM timestep based on latitude, longitude 
! and time.
! --------------------------------------------------------------------

  IMPLICIT NONE

CONTAINS

! --------------------------------------------------------------------
  SUBROUTINE ranset(idum_in)
! --------------------------------------------------------------------
! --------------------------------------------------------------------
! Create a new random number stream
! --------------------------------------------------------------------

    USE shr_kind_mod,only: r8 => shr_kind_r8

    IMPLICIT NONE

!
! INPUT DATA
!

    INTEGER, INTENT(IN) :: &
    idum_in                    ! Seed for random number generator, must be < 0

!
! LOCAL DATA
!

    REAL(r8) :: &
    rdummy       ! Random number

    INTEGER :: &
    idum        ! Seed for random number generator, must be < 0

! Check that random number seed is less than zero

    IF (idum_in .GT. 0) THEN
       idum = -idum_in
    ELSE IF (idum_in .EQ. 0) THEN
       idum = -100
    ELSE
       idum = idum_in
    END IF

    CALL ran3(idum, rdummy)

    RETURN
  END SUBROUTINE ranset

! --------------------------------------------------------------------
  FUNCTION rand_()
! --------------------------------------------------------------------
! --------------------------------------------------------------------
! Create a new random number
! --------------------------------------------------------------------

    USE shr_kind_mod,only: r8 => shr_kind_r8

    IMPLICIT NONE

!
! LOCAL DATA
!

    INTEGER::      &
    idum             ! Seed for random number generator

    REAL :: &
    rand_    ! Random number

    REAL(r8) :: &
    rdummy

    idum = 0 ! Flag to get the random number state

    CALL ran3(idum, rdummy)

    rand_ = rdummy

  END FUNCTION rand_

! --------------------------------------------------------------------
  SUBROUTINE RAN3(idum, random_number)
! --------------------------------------------------------------------
! --------------------------------------------------------------------
! Random number generator from Numerical Recipes which is from Knuth
! --------------------------------------------------------------------

! IDUM < 0 Seeds generator
! IDUM = 0 Generates new psuedo random number

    USE shr_kind_mod,only: r8 => shr_kind_r8

    IMPLICIT NONE

!
! PARAMETERS

    INTEGER, PARAMETER :: n_MA = 55

!
! INPUT DATA
!

    INTEGER, INTENT(IN) :: &
    idum                    ! Flag or seed

!
! OUTPUT DATA
!

    REAL(r8), INTENT(OUT) :: &
    random_number             ! A random number

!
! LOCAL DATA
!

    INTEGER, PARAMETER ::     &
    MBIG  = 1000000000,       &
    MSEED = 161803398,        &
    MZ    = 0

    REAL(r8) ::       &
    FAC 

    INTEGER ::     &
    I,             &
    II,            &
    K,             &
    MJ,            &
    MK

    INTEGER :: &
    INEXT,     &
    INEXTP,    &
    MA(n_MA)


    COMMON /RAN_COM/ I, II, K, MJ, MK, INEXT, INEXTP, MA

!$OMP THREADPRIVATE (/RAN_COM/)

    FAC = 1.0/REAL(MBIG)

! IF IDUM < 0 generate a new stream

    IF(IDUM .LT. 0) THEN

       MJ=MSEED-IABS(IDUM)
       MJ=MOD(MJ,MBIG)
       MA(55)=MJ
       MK=1

       DO I=1,54
          II=MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK

          IF(MK.LT.MZ) MK=MK+MBIG

          MJ=MA(II)
       END DO ! I

       DO K=1,4
          DO I=1,55
             MA(I)=MA(I)-MA(1+MOD(I+30,55))
             IF(MA(I).LT.MZ)MA(I)=MA(I)+MBIG
          END DO ! I
       END DO ! K

       INEXT=0
       INEXTP=31
    ENDIF

    INEXT=INEXT+1
    IF(INEXT.EQ.56)INEXT=1

    INEXTP=INEXTP+1
    IF(INEXTP.EQ.56)INEXTP=1

    MJ=MA(INEXT)-MA(INEXTP)

    IF(MJ.LT.MZ) MJ=MJ+MBIG

    MA(INEXT)=MJ

    random_number=MJ*FAC

    IF(random_number.LT.1.0E-6)    random_number=1.0E-6
    IF(random_number.GT.0.999999)  random_number=0.999999

    RETURN

  END SUBROUTINE RAN3

END MODULE RAN_NUMBERS_MOD
