#include <misc.h>
#include <params.h>

!#if ( defined SCAM )
!#include <max.h>
!#endif
!----------------------------------------------------------------------- 
!
! BOP
!
! !MODULE: prod_data
!
! Handles reading and interpolating solar proton ionization data.
! 
! Public interfaces:
! 
!       prodataini ---Initialization and reading of dataset.
!       prodataint -- Interpolate dataset to current time.
!
! $Id: bcc_prodata.F90 2013/02/06
!
!----------------------------------------------------------------------- 

module bcc_spedata
!
! USES:
!
  use shr_kind_mod, only:  r8 => shr_kind_r8
  use pmgrid,       only:  masterproc
  use ppgrid,       only:  pver
  use abortutils,   only:  endrun

  implicit none

  public spedata_ini
  public spe_prod
 
  save 

  integer  :: ncid_prod

  integer  :: levsiz
  integer  :: timesiz

!===============================================================================
!EOP
!===============================================================================
!-----------------------------------------------------------------------
! PRIVATE: Everthing else is private to this module
!-----------------------------------------------------------------------
!  private   ! By default all data is private to this module

   real(r8), allocatable :: prodbdy(:,:) 
   integer,  allocatable :: date_prod(:)
   integer,  allocatable :: sec_prod (:)
   real(r8), allocatable :: pin      (:)
   real(r8), allocatable :: prod     (:)   
  
   real(r8) :: cdayprodm
   real(r8) :: cdayprodp

   integer :: prodid
   integer :: dateid                          ! netcdf id for date variable
   integer :: secid                           ! netcdf id for seconds variable
   integer :: levdimid                        ! netcdf id for level dimension
   integer :: levid                           ! netcdf id for level variable
   integer :: timeid                          ! netcdf id for time variable
   integer :: nm,np                        ! Array indices for prv., nxt month

   integer  :: np1
!===============================================================================
CONTAINS
!===============================================================================

!======================================================================
! PUBLIC ROUTINES: Following routines are publically accessable
!======================================================================
!-----------------------------------------------------------------------
!
! BOP
!
! Author: ZF
!
!-----------------------------------------------------------------------
!
! !INTERFACE
!
!===============================================================================
subroutine spedata_ini(spe_data_file)

   use pspect
   use rgrid
   use commap
   use ioFileMod,    only: getfil
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use abortutils,   only: endrun

#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none

#include <comctl.h>
#include <comlun.h>
!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
   character(*) spe_data_file

!-------------------------------------------
! Local workspace
!
   integer  :: cnt2(2)                         ! array of counts for each dimension
   integer  :: strt2(2)                        ! array of starting indices
   integer  :: i, k,  n                    ! longitude, level, latitude, time indices
   integer  :: yr, mon, day                ! components of a date
   integer  :: ncdate                      ! current date in integer format [yyyymmdd]
   integer  :: ncsec                       ! current time of day [seconds]
   real(r8) :: calday                      ! current calendar day
   real(r8) :: caldayloc                      ! calendar day (includes yr if no cycling)

   character(len=256) :: locfn    ! netcdf local filename to open 

!------------------------------------------------
  nm = 1
  np = 2
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
!
! Use year information only if not cycling prod dataset
!
      calday = get_curr_calday()
      if ( is_perpetual() ) then
         call get_perp_date(yr, mon, day, ncsec)
      else
         call get_curr_date(yr, mon, day, ncsec)
      end if
      ncdate = yr*10000 + mon*100 + day
      caldayloc = calday + yr*365.
!
! Get and check dimension info
!
!   if (masterproc) then
      call getfil (spe_data_file, locfn)
      call wrap_open(locfn, 0, ncid_prod)
!--
      CALL WRAP_INQ_DIMID( ncid_prod, 'pressure', levdimid   )
      CALL WRAP_INQ_DIMID( ncid_prod, 'time', timeid  )

      CALL WRAP_INQ_DIMLEN( ncid_prod, levdimid, levsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_prod, timeid, timesiz   )

      CALL WRAP_INQ_VARID( ncid_prod, 'date', dateid   )
      CALL WRAP_INQ_VARID( ncid_prod, 'datesec', secid   )
      CALL WRAP_INQ_VARID( ncid_prod, 'pressure', levid   )
      CALL WRAP_INQ_VARID( ncid_prod, 'Prod', prodid   )
!   end if

#if (defined SPMD )
!   call mpibcast( ncid_prod, 1, mpiint, 0, mpicom )
!   call mpibcast( levsiz, 1, mpiint, 0, mpicom )
!   call mpibcast( timesiz, 1, mpiint, 0, mpicom )
!   call mpibcast( prodid, 1, mpiint, 0, mpicom )
!   call mpibcast( levid, 1, mpiint, 0, mpicom )
!   call mpibcast( secid, 1, mpiint, 0, mpicom )
!   call mpibcast( dateid, 1, mpiint, 0, mpicom )
#endif

!
! Dynamically allocated memory for module comozp
!
   allocate (date_prod(timesiz))
   allocate (sec_prod(timesiz))
   allocate (pin (levsiz))
   allocate (prod(levsiz))
   allocate (prodbdy(levsiz, 2))

!
! Locally dynamic that will be deallocated before "return"
!

!   if (masterproc) then
!
! More dynamically allocated memory for module comozp
! (for masterproc only)
!
!
! Retrieve longitude, latitude and level arrays for interpolation.
!
      CALL WRAP_GET_VAR_REALX (NCID_prod, levid, pin)
!
! Convert from millibars to pascals
!
      do k=1,levsiz
         pin(k) = pin(k)*100.
      end do
!
! Retrieve entire date and sec variables.
!

      CALL WRAP_GET_VAR_INT (ncid_prod,dateid,date_prod)
      CALL WRAP_GET_VAR_INT (ncid_prod,secid,sec_prod)

      strt2(1) = 1
      cnt2(1)  = levsiz
      cnt2(2)  = 1
!
! Normal interpolation between consecutive time slices.
!
      do n=1,timesiz-1
         np1 = n + 1
         call bnddyi(date_prod(n  ), sec_prod(n  ), cdayprodm)
         call bnddyi(date_prod(np1), sec_prod(np1), cdayprodp)
         yr = date_prod(n)/10000
         cdayprodm = cdayprodm + yr*365.
         yr = date_prod(np1)/10000
         cdayprodp = cdayprodp + yr*365.
         if (caldayloc > cdayprodm .and. caldayloc <= cdayprodp) then
            strt2(2) = n
            call wrap_get_vara_realx (ncid_prod,prodid,strt2,cnt2,prodbdy(1,nm))

            strt2(2) = np1
            call wrap_get_vara_realx (ncid_prod,prodid,strt2,cnt2,prodbdy(1,np))
            goto 10
         end if
      end do
      write(6,*)'PRODINI: Failed to find dates bracketing ncdate, ncsec=', ncdate, ncsec
      call endrun
10    continue
      write(6,*)'PRODINI: Read prod data for dates ',date_prod(n), &
                sec_prod(n),' and ',date_prod(np1),sec_prod(np1)
!   end if   !masterproc

#if (defined SPMD )
!   call mpibcast (prodbdy, levsiz*2, mpir8, 0, mpicom)
!   call mpibcast (np1, 1, mpiint, 0, mpicom)
!   call mpibcast (pin, levsiz, mpir8, 0, mpicom)
!   call mpibcast (date_prod, timesiz, mpiint, 0, mpicom )
!   call mpibcast (sec_prod, timesiz, mpiint, 0, mpicom )
!   call mpibcast (cdayprodm, 1, mpir8, 0, mpicom)
!   call mpibcast (cdayprodp, 1, mpir8, 0, mpicom)
#endif

   return
end subroutine spedata_ini

!===============================================================================
subroutine get_ionpairs_profile( pmid, spedata )

   use pspect
   use rgrid
   use commap
   use ppgrid,       only: pver
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use abortutils,   only: endrun
   use timeinterp,   only: getfactors
#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none

#include <comctl.h>
#include <comlun.h>
!--------------------------
   real(r8), intent(in)  :: pmid   (pver)
   real(r8), intent(out) :: spedata(pver)
!--------------------------------
! Local workspace
!
   integer cnt2(2)                ! array of counts for each dimension
   integer strt2(2)               ! array of starting indices
   integer i, k                   ! longitude/column, level, latitude indices
   integer c, ncol                ! chunk index, number of columns in chunk
   integer ntmp                   ! temporary
   integer :: yr, mon, day        ! components of a date
   integer :: ncdate              ! current date in integer format [yyyymmdd]
   integer :: ncsec               ! current time of day [seconds]

   real(r8) fact1, fact2          ! time interpolation factors
   real(r8) :: calday             ! current calendar day
   real(r8) caldayloc             ! calendar day (includes yr if no cycling)
   real(r8) deltat                ! time (days) between interpolating prod data
!---
!
   real(r8) cdayprdp
   real(r8) prod(levsiz)

!---------------
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   if ( is_perpetual() ) then
      call get_perp_date(yr, mon, day, ncsec)
   else
      call get_curr_date(yr, mon, day, ncsec)
   end if
   ncdate = yr*10000 + mon*100 + day
   caldayloc = calday + yr*365.

!
   if (caldayloc > cdayprodp .and. .not. (np1 == 1 .and. caldayloc > cdayprodm)) then
!
      np1 = np1 + 1
      if (np1 > timesiz) then
         call endrun ('bcc_spedata: Attempt to read past end of O3 dataset')
      end if
      cdayprodm = cdayprodp
      call bnddyi(date_prod(np1), sec_prod(np1), cdayprdp)

      yr = date_prod(np1)/10000
      cdayprodp = cdayprodp + yr*365.

      if (.not. (np1 == 1 .or. caldayloc <= cdayprodp)) then
         if (masterproc) then
            write(6,*)'bcc_spedata: Input prod for date',date_prod(np1),' sec ',sec_prod(np1), &
                      'does not exceed model date',ncdate,' sec ',ncsec,' Stopping.'
         endif
         call endrun ()
      end if
!
! Allocate memory for dynamic local workspace
!
!      if ( masterproc ) then

         prodbdy(1:levsiz,nm) =  prodbdy(1:levsiz,np)

         strt2(1) = 1
         strt2(2) = np1
         cnt2(1)  = levsiz
         cnt2(2)  = 1
!
! Allocate memory for more dynamic local workspace
!
         call wrap_get_vara_realx (ncid_prod,prodid,strt2,cnt2,prodbdy(1,np) )

!      end if ! masterproc

#if (defined SPMD )
!      call mpibcast (prodbdy, levsiz*2, mpir8, 0, mpicom)
#endif

   endif
!
! Determine time interpolation factors.
!
   call getfactors (.false., np1, cdayprodm, cdayprodp, caldayloc, &
                          fact1, fact2, 'PRODINT:')
!
! Time interpolation.
!
   do k=1,levsiz
      prod(k) = prodbdy(k,nm)*fact1 + prodbdy(k,np)*fact2
   end do
!
! Spatial interpolation.  If prod dataset is only 2-d (i.e. lonsiz = 1) and
! thus only latitude interpolation is necessary, expand to 3-d after
! interpolation.
!
   call lininterp1d(prod, levsiz, pin, spedata, plev, pmid )
	   
   return
end subroutine get_ionpairs_profile

  subroutine lininterp1d (arrin, nin, pin, arrout, nout, pout)
    !-----------------------------------------------------------------------
    !
    ! 
    !-----------------------------------------------------------------------
    !-----------------------------------------------------------------------
    implicit none
    !-----------------------------------------------------------------------
    !
    ! Arguments
    !
    integer, intent(in) :: nin               ! number of input lev
    integer, intent(in) :: nout              ! number of output lev

    real(r8), intent(in) :: arrin(nin)    ! input array of values to interpolate
    real(r8), intent(in) :: pin(nin)      
    real(r8), intent(out) :: arrout(nout) ! interpolated array
    real(r8), intent(in)  :: pout(nout)      

    !
    ! Local workspace
    !
    integer k, i


    !
    ! Do the interpolation
    !
    do k = 1, nout
      if ( pout(k) .le. pin(nin) ) then
        arrout(k)  = arrin(nin)
      end if
      if ( pout(k) .gt. pin(1) ) then
        arrout(k)  = arrin(1)
      end if

      do i = 1, nin-1
        if ( pout(k) .le. pin(i) .and. pout(k) .gt. pin(i+1) ) then
          arrout(k) = arrin(i+1) + (arrin(i) - arrin(i+1))*(pout(k) - pin(i+1))/(pin(i)-pin(i+i))
        end if
      end do
    end do

    return
  end subroutine lininterp1d

  !-----------------------------------------------------------------------
  !
  !     ... calculates NO production on calday (output in molec/cm3/s)
  !
  !-----------------------------------------------------------------------
  subroutine spe_prod( noxprod, hoxprod, pmid, zmid, lchnk, ncol)

    use physconst,   only:  pi
    use bcc_apex,    only : alatm              ! magnetic latitude grid (radians)
    use ppgrid,      only : pcols, pver
    use bcc_spehox,  only : hox_prod_factor
    use phys_grid,       only: get_lat_p, get_lon_p

    implicit none

    !-----------------------------------------------------------------------
    ! 	... dummy arguments
    !-----------------------------------------------------------------------
    integer, intent(in) ::  &
         ncol, &                           ! column count
         lchnk                             ! chunk index
    real(r8), intent(in) :: &
         pmid(pcols,pver)                 ! midpoint pressure (Pa)
    real(r8), intent(in) :: &
         zmid(pcols,pver)                  ! midpoint altitude (km)
    real(r8), intent(out) :: &
         noxprod(pcols,pver)                ! NO production
    real(r8), intent(out) :: &
         hoxprod(pcols,pver)               ! HOx production

    !-----------------------------------------------------------------------
    ! 	... local variables
    !-----------------------------------------------------------------------

    integer  :: i,ic,jc,k
    real(r8) :: dlat_aur
    logical  :: do_spe(pcols)

    real(r8) :: ion_pairs(pver)
    real(r8), parameter :: noxprod_factor = 1._r8 
    real(r8) :: hoxprod_factor(pver)
    real(r8) :: pmidver(pver), zmidver(pver)
    
    !-----------------------------------------------------------------------
    ! 	... intialize NO production
    !-----------------------------------------------------------------------

    noxprod(:ncol,:pver) = 0._r8
    hoxprod(:ncol,:pver) = 0._r8

    !-----------------------------------------------------------------------
    ! 	... check magnetic latitudes, and return if all below 60 deg
    !-----------------------------------------------------------------------

    do i = 1,ncol
       ic                = get_lon_p(lchnk,i)
       jc                = get_lat_p(lchnk,i)
       dlat_aur = alatm(ic,jc)
       do_spe(i) = abs( dlat_aur ) > pi/3._r8
    enddo

    if( all( .not. do_spe(:) ) ) then
       return
    end if

    do i = 1,ncol
       if( do_spe(i) ) then
!--
          do k=1, pver
             pmidver(k) = pmid(i,k)
             zmidver(k) = zmid(i,k)
          enddo

          call get_ionpairs_profile( pmidver(:pver), ion_pairs(:pver) )
!
!wtw          ion_pairs(:pver) = spedata(:pver)      
!--------
          noxprod(i,:pver) = noxprod_factor * ion_pairs(:pver)
          hoxprod_factor(:pver) = hox_prod_factor( ion_pairs(:pver), zmidver(:pver) )
          hoxprod(i,:pver) = hoxprod_factor(:pver)* ion_pairs(:pver)
       end if
    end do

  end subroutine spe_prod

end module bcc_spedata
