#include <misc.h>
#include <params.h>

module bcc_strato_sad
!---------------------------------------------------------------
! 	... prescribed strat aero surf area density module
!---------------------------------------------------------------

  use bcc_m_types,  only : time_ramp
  use shr_kind_mod, only : r8 => shr_kind_r8
  use ppgrid,       only : pver, pcols, pverp, begchunk, endchunk
  use time_manager, only : get_curr_date
  use bcc_chem_utls,only : flt_date, moz_findplb
  use abortutils,   only : endrun
  use ioFileMod,    only : getfil
  use pmgrid,       only : masterproc
  use mpishorthand, only : mpir8, mpiint, mpicom ! MPI interface

  implicit none

  private
  public  :: strato_sad_inti
  public  :: strato_sad_timestep_init
  public  :: strato_sad_set

  save

  integer, parameter    :: time_span = 1

  integer               :: ntimes
  integer               :: nlon
  integer               :: nlat
  integer               :: nlev
  integer               :: tim_ndx(2)
  integer,  allocatable :: dates(:)
  real(r8), allocatable :: times(:)
  real(r8), allocatable :: sad_lats(:)
  real(r8), allocatable :: sad_lons(:)
  real(r8), allocatable :: sad_levs(:)
  real(r8), allocatable :: sage_sad(:,:,:,:)
  character(len=256)    :: filename

  logical :: has_sulfate_rxts
 
  character(len=10) time_type    !  'SERIAL', or 'CYCLICAL'
  integer           cycle_yr     !   for time_type = 'CYCLICAL'

!  time_type = 'SERIAL' 
!  cycle_yr  =  1971              !  only for time_type = 'CYCLICAL'

  character(len=256) :: locfn     ! netcdf local filename to open 

contains

  subroutine strato_sad_inti( sad_file, time_type0, cycle_year )
!-----------------------------------------------------------------------
! 	... initialize the strato sad module
!-----------------------------------------------------------------------
    
    use bcc_const_mozart, only : d2r
    use bcc_chem_utls,    only : get_rxt_ndx
    use bcc_strato_rates, only : has_strato_chem    ! .true.
    use string_utils,     only : to_upper

    implicit none

!-----------------------------------------------------------------------
! 	... dummy arguments
!-----------------------------------------------------------------------
    character(len=*), intent(in) :: sad_file
    integer                      :: cycle_year
!-----------------------------------------------------------------------
! 	... local variables
!-----------------------------------------------------------------------
    real(r8), parameter :: hPa2Pa = 100.
    integer             :: astat
    integer             :: j, l, m, n                           ! Indices
    integer             :: t1, t2
    integer             :: dimid
    integer             :: varid
    integer             :: yr, wrk_date, wrk_sec
    real(r8)            :: seq
    real(r8)            :: wrk_time
        

    integer :: mon, day, tod, ncdate, ncsec
    integer :: ierr
    integer :: usrrxt_ndx(4)
    integer :: ncid
 
    character(len=*) time_type0  
!--------------------------------------------------------------
    cycle_yr = cycle_year

    usrrxt_ndx(1) = get_rxt_ndx( 'usr_N2O5_aer' )
    usrrxt_ndx(2) = get_rxt_ndx( 'usr_NO3_aer' )
    usrrxt_ndx(3) = get_rxt_ndx( 'usr_NO2_aer' )
    usrrxt_ndx(4) = get_rxt_ndx( 'usr_HO2_aer' )

    has_sulfate_rxts = any( usrrxt_ndx > 0 )                     ! has_sulfate_rxts => .false.
    has_sulfate_rxts = has_sulfate_rxts .or. has_strato_chem     ! has_strato_chem  => .true.

    if ( .not. has_sulfate_rxts) return

    call get_curr_date( yr, mon, day, tod )
    ncdate = yr*10000 + mon*100 + day
    ncsec = tod

!-----------------------------------------------------------------------
! 	... check timing
!-----------------------------------------------------------------------
    time_type = trim(time_type0)
    wrk_sec  = ncsec
    if( (trim(time_type) == 'SERIAL') ) then
       wrk_date = ncdate
    else if( trim(time_type) == 'CYCLICAL' ) then
       wrk_date = cycle_yr*10000 + mod( ncdate,10000 )
    else
       write(*,*) 'time_type is not correct'
       call endrun
    end if
    wrk_time = flt_date( wrk_date, wrk_sec )

!-----------------------------------------------------------------------
! 	... open netcdf file
!-----------------------------------------------------------------------
    filename = sad_file

if (masterproc) then
    call getfil (sad_file, locfn)
    call wrap_open (locfn, 0, ncid)

!-----------------------------------------------------------------------
! 	... get timing information, allocate arrays, and read in dates
!-----------------------------------------------------------------------
    call wrap_inq_dimid( ncid, 'time', dimid )
    call wrap_inq_dimlen( ncid, dimid, ntimes )
end if

#if (defined SPMD )  
    call mpibcast( ntimes, 1, mpiint, 0, mpicom ) 
#endif

    allocate( dates(ntimes),stat=astat )
    if( astat/= 0 ) then
       write(*,*) 'strato_sad_inti: failed to allocate dates array; error = ',astat
       call endrun
    end if
    allocate( times(ntimes),stat=astat )
    if( astat/= 0 ) then
       write(*,*) 'strato_sad_inti: failed to allocate times array; error = ',astat
       call endrun
    end if

if (masterproc) then
    call wrap_inq_varid( ncid, 'date', varid )
    call wrap_get_var_int( ncid, varid, dates )
end if

#if (defined SPMD )  
    call mpibcast( dates, ntimes, mpiint, 0, mpicom ) 
#endif

    do n = 1,ntimes
       times(n) = flt_date( dates(n), 0 )
    end do

    if( trim(time_type) /= 'CYCLICAL' ) then
       if( wrk_time < times(1) .or. wrk_time > times(ntimes) ) then
          write(*,*) 'strato_sad_inti: time out of bounds for dataset = ',trim(filename)
          call endrun
       end if
       do n = 2,ntimes
          if( wrk_time <= times(n) ) then
             exit
          end if
       end do
       tim_ndx(1) = n - 1
    else
       yr = cycle_yr
       do n = 1,ntimes
          if( yr == dates(n)/10000 ) then
             exit
          end if
       end do
       if( n >= ntimes ) then
          write(*,*) 'strato_sad_inti: time out of bounds for dataset = ',trim(filename)
          call endrun
       end if
       tim_ndx(1) = n
    end if

    select case( time_type )
    case( 'CYCLICAL' )
       do n = tim_ndx(1),ntimes
          if( yr /= dates(n)/10000 ) then
             exit
          end if
       end do
       tim_ndx(2) = n - 1
       if( (tim_ndx(2) - tim_ndx(1)) < 2 ) then
          write(*,*) 'strato_sad_inti: cyclical sad require at least two time points'
          call endrun
       end if
    case( 'SERIAL' )
       tim_ndx(2) = min( ntimes,tim_ndx(1) + time_span )
    end select
    t1 = tim_ndx(1)
    t2 = tim_ndx(2)
    
    if (masterproc) then
       write(*,*) ' '
       write(*,*) 'strato_sad time cnt = ',ntimes
       write(*,*) 'strato_sad times'
       write(*,'(10i10)') dates(:)
       write(*,'(1p,5g15.7)') times(:)
       write(*,*) 'strato_sad time indicies = ',tim_ndx(:)
       write(*,'(10i10)') dates(tim_ndx(1):tim_ndx(2))
       write(*,*) ' '
    endif

!-----------------------------------------------------------------------
!     	... inquire about latitudes
!-----------------------------------------------------------------------
if (masterproc) then
    call wrap_inq_dimid( ncid, 'lat', dimid )
    call wrap_inq_dimlen( ncid, dimid, nlat )
end if
#if (defined SPMD )  
    call mpibcast( nlat, 1, mpiint, 0, mpicom ) 
#endif
!-----------------------------------------------------------------------
!     	... allocate space for latitude coordinates
!-----------------------------------------------------------------------
    allocate( sad_lats(nlat),stat=astat )
    if( astat /= 0 ) then
       write(*,*) ' strato_sad_inti : failed to allocate latitudes array; error = ',astat
       call endrun
    end if
!-----------------------------------------------------------------------
!     	... get latitudes
!-----------------------------------------------------------------------
if (masterproc) then
    call wrap_inq_varid( ncid, 'lat', varid )
    call wrap_get_var_realx( ncid, varid, sad_lats  )

!-----------------------------------------------------------------------
!     	... inquire about longitudes
!-----------------------------------------------------------------------
    call wrap_inq_dimid( ncid, 'lon', dimid )
    call wrap_inq_dimlen( ncid, dimid, nlon )
end if
#if (defined SPMD )  
    call mpibcast( nlon, 1, mpiint, 0, mpicom ) 
    call mpibcast( sad_lats, nlat, mpir8, 0, mpicom ) 
#endif
!-----------------------------------------------------------------------
!     	... allocate space for longitude coordinates
!-----------------------------------------------------------------------
    allocate( sad_lons(nlon),stat=astat )
    if( astat /= 0 ) then
       write(*,*) ' strato_sad_inti : failed to allocate longitudes array; error = ',astat
       call endrun
    end if
!-----------------------------------------------------------------------
!     	... get longitudes
!-----------------------------------------------------------------------
if (masterproc) then
    call wrap_inq_varid( ncid, 'lon', varid )
    call wrap_get_var_realx( ncid, varid, sad_lons )
end if

#if (defined SPMD )  
    call mpibcast( sad_lons, nlon, mpir8, 0, mpicom ) 
#endif
!-----------------------------------------------------------------------
!     	... convert to radians and setup regridding
!-----------------------------------------------------------------------
    sad_lats(:) = d2r * sad_lats(:)
    sad_lons(:) = d2r * sad_lons(:)

!-----------------------------------------------------------------------
!     	... inquire about levels
!-----------------------------------------------------------------------
if (masterproc) then
    call wrap_inq_dimid( ncid, 'lev', dimid )
    call wrap_inq_dimlen( ncid, dimid, nlev )
end if
#if (defined SPMD )  
    call mpibcast( nlev, 1, mpiint, 0, mpicom ) 
#endif
!-----------------------------------------------------------------------
!     	... allocate space for level coordinates
!-----------------------------------------------------------------------
    allocate( sad_levs(nlev),stat=astat )
    if( astat /= 0 ) then
       write(*,*) ' strato_sad_inti : failed to allocate levels array; error = ',astat
       call endrun
    end if
!-----------------------------------------------------------------------
!     	... get levels
!-----------------------------------------------------------------------
if (masterproc) then
    call wrap_inq_varid( ncid, 'lev', varid )
    call wrap_get_var_realx( ncid, varid, sad_levs )
end if
#if (defined SPMD )  
    call mpibcast( sad_levs, nlev, mpir8, 0, mpicom ) 
#endif
    sad_levs(:) = hPa2Pa * sad_levs(:)

    if (masterproc) then
       write(*,*) ' '
       write(*,*) 'strato_sad_inti: sad_levs'
       write(*,'(10g12.5)') sad_levs(:)
       write(*,*) ' '
    endif

!-----------------------------------------------------------------------
!     	... allocate module sad variable
!-----------------------------------------------------------------------
    allocate( sage_sad(pcols,nlev,begchunk:endchunk,t1:t2),stat=astat )
    if( astat /= 0 ) then
       write(*,*) ' strato_sad_inti : failed to allocate sage_sad array; error = ',astat
       call endrun
    end if
!-----------------------------------------------------------------------
! 	... read and interpolate sage sad data
!-----------------------------------------------------------------------
    call strato_sad_get( ncid )
!
!    call strato_sad_get 
!-----------------------------------------------------------------------
! 	... close netcdf file
!-----------------------------------------------------------------------
if (masterproc) then
    call wrap_close( ncid )
end if

  end subroutine strato_sad_inti

  subroutine strato_sad_timestep_init( )
!-----------------------------------------------------------------------
!       ... check serial case for time span
!-----------------------------------------------------------------------
    implicit none

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
    integer                     :: m
    integer                     :: t1, t2, tcnt
    integer                     :: astat
    real(r8)                    :: wrk_time
    integer                     :: ncid

    integer ::  yr, mon, day, tod, ncdate, ncsec

    if ( .not. has_sulfate_rxts) return

    call get_curr_date( yr, mon, day, tod )
    ncdate = yr*10000 + mon*100 + day
    ncsec = tod

    if( trim(time_type) == 'SERIAL' ) then
       wrk_time = flt_date( ncdate, ncsec )
       if( wrk_time > times(tim_ndx(2)) ) then
          tcnt = tim_ndx(2) - tim_ndx(1)
          tim_ndx(1) = tim_ndx(2)
          tim_ndx(2) = min( ntimes,tim_ndx(1) + time_span )
          t1 = tim_ndx(1)
          t2 = tim_ndx(2)

!-----------------------------------------------------------------------
! 	... allocate array
!-----------------------------------------------------------------------
             if( allocated( sage_sad ) ) then
                deallocate( sage_sad,stat=astat )
                if( astat/= 0 ) then
                   write(*,*) 'strato_sad_timestep_init: failed to deallocate strato_sad vmr; error = ',astat
                   call endrun
                end if
             end if
             allocate( sage_sad(pcols,nlev,begchunk:endchunk,t1:t2),stat=astat )
             if( astat/= 0 ) then
                write(*,*) 'strato_sad_timestep_init: failed to allocate sage_sad; error = ',astat
                call endrun
             end if
!!$          end if
!-----------------------------------------------------------------------
! 	... open netcdf file
!-----------------------------------------------------------------------
          call wrap_open (trim(filename), 0, ncid)
!-----------------------------------------------------------------------
! 	... read and interpolate sage sad data
!-----------------------------------------------------------------------
          call strato_sad_get( ncid )
!-----------------------------------------------------------------------
! 	... close netcdf file
!-----------------------------------------------------------------------
          call wrap_close( ncid )
       end if
    end if

  end subroutine strato_sad_timestep_init

  subroutine strato_sad_get( ncid )
!-----------------------------------------------------------------------
!       ... read sad values
!-----------------------------------------------------------------------
    use phys_grid,            only : scatter_field_to_chunk 
    use bcc_interpolate_data, only : lininterp_init, lininterp, lininterp_finish, &
                                     interp_type
    use phys_grid,            only : get_ncols_p, get_rlat_all_p, get_rlon_all_p
    use bcc_const_mozart,     only : pi
    implicit none

!-----------------------------------------------------------------------
!       ... local variables
!-----------------------------------------------------------------------
    integer               :: i, c, j, k, n                            ! Indices
    integer               :: t1, t2, tcnt
    integer               :: ierr
    integer               :: vid
    real(r8), allocatable :: sage_sad_new(:,:,:,:), sage_sad_3d(:,:,:)
    real(r8), allocatable :: tmp_3d(:,:,:)
    integer               :: ndims(20), ncols
    integer               :: ncid
    real(r8)              :: to_lats(pcols), to_lons(pcols)

    type(interp_type)     :: lon_wgts, lat_wgts
      
    real(r8), parameter :: zero=0._r8, twopi=2._r8*pi
    
!-----------------------------------------------------------------------
!       ... read sage data
!-----------------------------------------------------------------------
    t1 = tim_ndx(1)
    t2 = tim_ndx(2)
    tcnt = t2 - t1 + 1

!-----------------------------------------------------------------------
!       ... allocate local wrk arrays
!-----------------------------------------------------------------------
    allocate( sage_sad_new(nlon, nlev, nlat, t1:t2), stat = ierr)
    allocate( sage_sad_3d(nlon,nlat,nlev), stat=ierr )

    if( ierr /= 0 ) then
       write(*,*) 'strato_sad_get: wrk allocation error = ',ierr
       call endrun
    end if

if (masterproc) then

    call wrap_inq_varid(  ncid, 'sad_sage', vid )
    call wrap_inq_vardimid (ncid, vid, ndims)

      do n = t1, t2
          call wrap_get_vara_realx( ncid, vid, &
            (/ 1, 1, 1, n/), &                     ! start
            (/ nlon, nlat, nlev, 1 /), &  ! count
            sage_sad_3d (:,:,:) )
            do i = 1, nlon
            do j = 1, nlat
            do k = 1, nlev
	     sage_sad_new(i,k,j,n) = sage_sad_3d(i,j,k)
	enddo
	enddo
	enddo
      end do

!-----------------------------------
  endif   ! masterproc

    do n = t1, t2
      call scatter_field_to_chunk(1,nlev,1,nlon,sage_sad_new(:,:,:,n),sage_sad(:,:,:,n ))
    end do

    deallocate( sage_sad_new, stat=ierr )
 !----------------------------------------------
    deallocate( sage_sad_3d, stat=ierr )

    if( ierr /= 0 ) then
       write(*,*) 'strato_sad_get: failed to deallocate sage_sad_in, ierr = ',ierr
       call endrun
    end if

  end subroutine strato_sad_get

  subroutine strato_sad_set( pmid, sad, ncol, lchnk )
!--------------------------------------------------------
!	... set the sad values
!--------------------------------------------------------
    
    implicit none

!--------------------------------------------------------
!	... dummy arguments
!--------------------------------------------------------
    integer, intent(in)     :: ncol, lchnk
    real(r8), intent(in)    :: pmid(pcols,pver) ! midpoint pressure (Pa)
    real(r8), intent(inout) :: sad(pcols, pver) ! stratospheric aerosol surface area density (1/cm)

!--------------------------------------------------------
!	... local variables
!--------------------------------------------------------
    integer  ::  i, k, n
    integer  ::  last, next
    integer  ::  wrk_date, wrk_sec
    integer  ::  tcnt
    integer  ::  astat
    real(r8) ::  dels
    real(r8) ::  wrk_time
    real(r8) ::  sad_int(pcols,pver,2)
    integer  ::  yr, mon, day, tod, ncdate, ncsec

    if ( .not. has_sulfate_rxts) return  !  has_sulfate_rxts = .true.

    call get_curr_date( yr, mon, day, tod )
    ncdate = yr*10000 + mon*100 + day
    ncsec = tod

!--------------------------------------------------------
!	... setup the time interpolation
!--------------------------------------------------------

    wrk_sec  = ncsec
    if( (trim(time_type) == 'SERIAL') ) then
       wrk_date = ncdate
    else if( trim(time_type) == 'CYCLICAL' ) then
       wrk_date = cycle_yr*10000 + mod( ncdate,10000 )
    else
       write(*,*) 'time_type is not correct'
       call endrun
    end if
    wrk_time = flt_date( wrk_date, wrk_sec )

!--------------------------------------------------------
!	... set time interpolation factor
!--------------------------------------------------------
    if( trim(time_type) /= 'CYCLICAL' ) then
       do n = tim_ndx(1)+1,tim_ndx(2)
          if( wrk_time <= times(n) ) then
             last = n - 1
             next = n
             exit
          end if
       end do
       if( n > ntimes ) then
          write(*,*) 'strato_sad_set: interp time is out of bounds'
          call endrun
       end if
       dels = (wrk_time - times(last))/(times(next) - times(last))
    else
       tcnt = tim_ndx(2) - tim_ndx(1) + 1
       call moz_findplb( times(tim_ndx(1)), tcnt, wrk_time, n )
       if( n < tcnt ) then
          last = tim_ndx(1) + n - 1
          next = last + 1
          dels = (wrk_time - times(last))/(times(next) - times(last))
       else
          next = tim_ndx(1)
          last = tim_ndx(2)
          dels = wrk_time - times(last)
          if( dels < 0. ) then
             dels = 365. + dels
          end if
          dels = dels/(365. + times(next) - times(last))
       end if
    end if
    
    dels = max( min( 1._r8,dels ),0._r8 )

    call vinterp( pmid, sage_sad(:,:,lchnk,last), sad_int(:,:,1), ncol )
    call vinterp( pmid, sage_sad(:,:,lchnk,next), sad_int(:,:,2), ncol )

    do i=1,ncol
       sad(i,:) = sad_int(i,:,1) + dels * (sad_int(i,:,2) - sad_int(i,:,1))
    enddo

  end subroutine strato_sad_set

  subroutine vinterp( pmid, sad_src, sad_int, ncol )
!-----------------------------------------------------------------------
!   	... vertically interpolate input data
!-----------------------------------------------------------------------
    implicit none

!-----------------------------------------------------------------------
!   	... dummy arguments
!-----------------------------------------------------------------------
    integer,  intent(in)  :: ncol
    real(r8), intent(in)  :: pmid(:,:)
    real(r8), intent(in)  :: sad_src(:,:)
    real(r8), intent(out) :: sad_int(:,:)

!-----------------------------------------------------------------------
!   	... local variables
!-----------------------------------------------------------------------
    integer  :: i
    integer  :: k, kl, ku
    real(r8) :: delp, pinterp

    level_loop : do k = 1,pver
       long_loop : do i = 1,ncol
          pinterp = pmid(i,k)
          if( pinterp <= sad_levs(1) ) then
             sad_int(i,k) = sad_src(i,1)
          else if( pinterp > sad_levs(nlev) ) then
             sad_int(i,k) = sad_src(i,nlev)
          else
             do ku = 2,nlev
                if( pinterp <= sad_levs(ku) ) then
                   kl = ku - 1
                   delp = log( pinterp/sad_levs(kl) ) &
                        / log( sad_levs(ku)/sad_levs(kl) )
                   sad_int(i,k) = sad_src(i,kl) + delp * (sad_src(i,ku) - sad_src(i,kl))
                   exit
                end if
             end do
          end if
       end do long_loop
    end do level_loop

  end subroutine vinterp

end module bcc_strato_sad
