#include <misc.h>
#include <params.h>

module bcc_synoz

  !--------------------------------------------------------------------
  !	... synthetic stratospheric ozone emission source
  !--------------------------------------------------------------------

  use shr_kind_mod, only : r8 => shr_kind_r8
  use cam_logfile,  only : iulog

  implicit none

  save 

  real(r8), allocatable :: po3(:,:,:)

  private
  public :: synoz_inti
  public :: po3

contains

  subroutine synoz_inti( )
    !-----------------------------------------------------------------------
    ! 	... initialize synoz emissions
    !	    note: the emissions are in in units of molecules/cm**3/s
    !-----------------------------------------------------------------------

!    use dyn_grid,        only : get_dyn_grid_parm
    use ppgrid,          only : pcols, begchunk, endchunk, pver, pverp
    use pmgrid,          only:  plon, plat, plev, plevp
!
!wtw    use hycoef,          only : ps0, hyam, hybm,hyai,hybi
!
    use phys_grid,        only : scatter_field_to_chunk
    use commap,           only : lat => latdeg
    use commap,           only : latwts => w
    use bcc_chem_mods,    only : adv_mass
    use bcc_chem_utls,    only : get_spc_ndx
    use pmgrid,           only : masterproc
    use physconst,        only : pi, &
                                 grav => gravit, & ! m/s^2
                                 dry_mass => mwdry, & ! kg/kmole
                                 seconds_in_day => cday, &
                                 rearth  ! m
    use abortutils,       only : endrun

    implicit none

#include <comhyb.h>

    !-----------------------------------------------------------------------
    ! 	... dummy arguments
    !-----------------------------------------------------------------------

    !-----------------------------------------------------------------------
    !	... local variables
    !-----------------------------------------------------------------------
    real(r8), parameter :: latmin                = -30._r8
    real(r8), parameter :: latmax                = 30._r8
    real(r8), parameter :: prsmin                = 1000._r8
    real(r8), parameter :: prsmax                = 7000._r8
    real(r8), parameter :: ps                    = 100000._r8
    real(r8), parameter :: ozone_source_per_year = 500._r8     ! global/annual average of ozone source (Tg/yr)
    real(r8), parameter :: tg2kg                 = 1.e9_r8
    real(r8), parameter :: days_per_year         = 365._r8

    integer :: i, j, k, jl, ju
    integer :: jmin, jmax, kmin, kmax
    integer :: synoz_ndx
    integer :: astat
!
!wtw    integer :: plon, plat, plev
!
    real(r8)    :: diff, diff_min, diff_max
    real(r8)    :: total_mass = 0._r8
    real(r8)    :: seq
    real(r8), allocatable    :: sf(:)
    real(r8), allocatable :: prs(:)
    real(r8), allocatable :: dp(:)
    real(r8), allocatable :: wk(:,:,:)

    allocate( po3(pcols,pver,begchunk:endchunk), stat=astat )
    if( astat /= 0 ) then
       write(*,*) 'synoz_inti: failed to allocate po3; error = ',astat
       call endrun
    end if

!wtw    plon = get_dyn_grid_parm('plon')
!wtw    plat = get_dyn_grid_parm('plat')
!wtw    plev = get_dyn_grid_parm('plev')

    allocate( wk(plon,plev,plat), stat=astat )
    if( astat /= 0 ) then
       write(*,*) 'synoz_inti: failed to allocate wk; error = ',astat
       call endrun
    end if

    Masterproc_only :  if( masterproc ) then
       !-----------------------------------------------------------------------
       ! 	... allocate memory -- for global grid
       !-----------------------------------------------------------------------
       allocate(  prs(plev), dp(plev), stat=astat )
       if( astat /= 0 ) then
          write(*,*) 'synoz_inti: failed to allocate prs, dp; error = ',astat
          call endrun
       end if

       !-----------------------------------------------------------------------
       ! 	... find indices of the latitudinal box
       !-----------------------------------------------------------------------
       jmin = -99
       jmax = -99
       diff_min = 1.e20_r8
       diff_max = 1.e20_r8
       do j = 1,plat
          diff = abs(lat(j)-latmin)
          if( diff < diff_min ) then
             diff_min = diff
             jmin = j
          end if
          diff = abs(lat(j)-latmax)
          if( diff < diff_max ) then
             diff_max = diff
             jmax = j
          end if
       end do

       !-----------------------------------------------------------------------
       ! 	... make sure we found them
       !-----------------------------------------------------------------------
       if( jmin < 0 .or. jmax < 0 ) then
          write(*,*) 'synoz_inti: problem finding the min/max lat in synoz_inti',jmin,jmax
          call endrun
       end if
       !-----------------------------------------------------------------------
       ! 	... define pressure arrays, assuming surface pressure = 1000 hPa
       !-----------------------------------------------------------------------
       prs(:) = hyam(:) * ps0 + hybm(:) * ps
       dp(:)  = (hyai(2:pverp) - hyai(1:pver)) * ps0 + (hybi(2:pverp) - hybi(1:pver)) * ps

       !-----------------------------------------------------------------------
       ! 	... find indices of the pressure box
       !-----------------------------------------------------------------------
       kmin = -99
       kmax = -99
       diff_min = 1.e20_r8
       diff_max = 1.e20_r8
       do k = 1,plev
          diff = abs( prs(k) - prsmin )
          if( diff < diff_min ) then
             diff_min = diff
             kmin = k
          end if
          diff = abs( prs(k) - prsmax )
          if( diff < diff_max ) then
             diff_max = diff
             kmax = k
          end if
       end do

       !-----------------------------------------------------------------------
       ! 	... make sure we found them
       !-----------------------------------------------------------------------
       if( kmin < 0 .or. kmax < 0 ) then
          write(*,*) 'synoz_inti: problem finding the min/max prs in synoz_inti',kmin,kmax
          call endrun
       end if

       !-----------------------------------------------------------------------
       ! 	... define geometric factors (in SI)
       !-----------------------------------------------------------------------
       seq = 2.*pi*rearth**2/real(plon)
       allocate(sf(plat))
       do j = 1,plat
          sf(j) = seq*latwts(j)
       end do

       !-----------------------------------------------------------------------
       ! 	... find index of synoz
       !-----------------------------------------------------------------------
       synoz_ndx = get_spc_ndx('SYNOZ')
       has_synoz : if( synoz_ndx > 0 ) then
          !-----------------------------------------------------------------------
          ! 	... compute total mass (in kg) over the domain for which
          !           the ozone source will be defined
          !-----------------------------------------------------------------------
          total_mass = 0.
          do k = kmin,kmax
             do j = jmin,jmax
                total_mass = total_mass + dp(k)/grav * sf(j)
             end do
          end do
          total_mass = total_mass * plon
#ifdef DEBUG
          write(*,*)'synoz_inti: total mass = ',total_mass
#endif
          !-----------------------------------------------------------------------
          ! 	... define the location of the ozone source
          !-----------------------------------------------------------------------
          wk(:,:,:) = 0._r8
          do k = kmin,kmax
             do j = jmin,jmax
                wk(1:plon,k,j) = 1._r8
             end do
          end do
          !-----------------------------------------------------------------------
          ! 	... define the ozone source as vmr/s (what is needed for the chemistry solver)
          !           note : a change in chemdr is made to avoid the division by invariants
          !-----------------------------------------------------------------------
          wk(:,:,:) = wk(:,:,:) * (ozone_source_per_year*tg2kg/total_mass) &
                                / (seconds_in_day*days_per_year) &
                                * (dry_mass/adv_mass(synoz_ndx))
          write(*,*) 'synoz_inti: max wk = ',maxval( wk(:,:,:) )
          deallocate( prs, dp )
       end if has_synoz

       deallocate(sf)

    endif Masterproc_only

    !-----------------------------------------------------------------------
    ! 	... scatter to mpi tasks
    !-----------------------------------------------------------------------
    call scatter_field_to_chunk(1, plev, 1, plon, wk, po3)

    !-----------------------------------------------------------------------
    ! 	... deallocate memory
    !-----------------------------------------------------------------------
    deallocate( wk )

  end subroutine synoz_inti

end module bcc_synoz
