! This module is used to diagnose the location of the tropopause. Multiple
! algorithms are provided, some of which may not be able to identify a
! tropopause in all situations. To handle these cases, an analytic
! defintion and a climatology are provided that can be used to fill in
! when the original algorithm fails. The troppause temperature and
! pressure are determined and can be output to the history file.
!
! These routines are based upon code in the WACCM chemistry module
! including mo_tropoause.F90 and llnl_set_chem_trop.F90. The code
! for the Reichler et al. [2003] algorithm is from:
!
!   http://www.gfdl.noaa.gov/~tjr/TROPO/tropocode.htm
!
! Author: Charles Bardeen
! Created: April, 2009

#include <misc.h>
#include <params.h>

module bcc_tropopause
  !---------------------------------------------------------------
  ! ... variables for the tropopause module
  !---------------------------------------------------------------

  use shr_kind_mod,  only : r8 => shr_kind_r8
  use ppgrid,        only : pcols, pver
  use physics_types, only : physics_state
  use physconst,     only : gravit, cappa, rair

  implicit none

  private
  
  public  :: tropopause_twmo

  save

  integer, parameter :: NOTFOUND    = -1
  real(r8),parameter :: cnst_kap    = cappa
  real(r8),parameter :: cnst_faktor = -gravit/rair
  real(r8),parameter :: cnst_ka1    = cnst_kap - 1._r8
    
!================================================================================================
contains
!================================================================================================

  !-------------------------------------------------------------------------------
  ! This routine uses an implementation of Reichler et al. [2003] done by
  ! Reichler and downloaded from his web site. This is similar to the WMO
  ! routines, but is designed for GCMs with a coarse vertical grid.
  !-------------------------------------------------------------------------------
  subroutine tropopause_twmo(state, tropLev, tropP, tropT, tropZ)

    implicit none

    type(physics_state), intent(in)    :: state 
    integer,            intent(inout)  :: tropLev(pcols)            ! tropopause level index   
    real(r8), optional, intent(inout)  :: tropP(pcols)              ! tropopause pressure (Pa)   
    real(r8), optional, intent(inout)  :: tropT(pcols)              ! tropopause temperature (K)
    real(r8), optional, intent(inout)  :: tropZ(pcols)              ! tropopause height (m)

    ! Local Variables 
    real(r8), parameter     :: gamma    = -0.002_r8         ! K/m
    real(r8), parameter     :: plimu    = 45000._r8         ! Pa
    real(r8), parameter     :: pliml    = 7500._r8          ! Pa
     
    integer                 :: i
    integer                 :: k
    integer                 :: ncol                         ! number of columns in the chunk
    integer                 :: lchnk                        ! chunk identifier
    real(r8)                :: tP                       ! tropopause pressure (Pa)

    ! Information about the chunk.  
    lchnk = state%lchnk
    ncol  = state%ncol

    ! Iterate over all of the columns.
    do i = 1, ncol
     
      ! Skip column in which the tropopause has already been found.
      if (tropLev(i) == NOTFOUND) then

        ! Use the routine from Reichler.
        call twmo(pver, state%t(i, :), state%pmid(i, :), plimu, pliml, gamma, tP)
     
        ! if successful, store of the results and find the level and temperature.
        if (tP > 0) then
        
          ! Find the associated level.
          do k = pver, 2, -1
            if (tP >= state%pint(i, k)) then
              tropLev(i) = k
              exit
            end if
          end do
          
          ! Return the optional outputs
          if (present(tropP)) tropP(i) = tP
          
          if (present(tropT)) then
            tropT(i) = tropopause_interpolateT(state, i, tropLev(i), tP)
          end if

          if (present(tropZ)) then
            tropZ(i) = tropopause_interpolateZ(state, i, tropLev(i), tP)
          end if
        end if
      end if
    end do
    
    return
  end subroutine tropopause_twmo
  
    !----------------------------------------------------------------------------
  ! This routine is an implementation of Reichler et al. [2003] done by
  ! Reichler and downloaded from his web site. Minimal modifications were
  ! made to have the routine work within the CAM framework (i.e. using
  ! CAM constants and types).
  !
  ! NOTE: I am not a big fan of the goto's and multiple returns in this
  ! code, but for the moment I have left them to preserve as much of the
  ! original and presumably well tested code as possible.
  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  ! determination of tropopause height from gridded temperature data
  !
  ! reference: Reichler, T., M. Dameris, and R. Sausen (2003)
  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  subroutine twmo(level, t, p, plimu, pliml, gamma, trp)

    integer, intent(in)                     :: level
    real(r8), intent(in), dimension(level)  :: t, p
    real(r8), intent(in)                    :: plimu, pliml, gamma
    real(r8), intent(out)                   :: trp
    
    real(r8), parameter                     :: deltaz = 2000.0_r8

    real(r8)                                :: pmk, pm, a, b, tm, dtdp, dtdz
    real(r8)                                :: ag, bg, ptph
    real(r8)                                :: pm0, pmk0, dtdz0
    real(r8)                                :: p2km, asum, aquer
    real(r8)                                :: pmk2, pm2, a2, b2, tm2, dtdp2, dtdz2
    integer                                 :: icount, jj
    integer                                 :: j
    !-------------------------------------------------------------------------------

    trp=-99.0_r8                           ! negative means not valid

    do j=level,2,-1
    
      ! dt/dz
      pmk= .5_r8 * (p(j-1)**cnst_kap+p(j)**cnst_kap)
      pm = pmk**(1/cnst_kap)               
      a = (t(j-1)-t(j))/(p(j-1)**cnst_kap-p(j)**cnst_kap)
      b = t(j)-(a*p(j)**cnst_kap)
      tm = a * pmk + b               
      dtdp = a * cnst_kap * (pm**cnst_ka1)
      dtdz = cnst_faktor*dtdp*pm/tm
      ! dt/dz valid?
      if (j.eq.level)    go to 999     ! no, start level, initialize first
      if (dtdz.le.gamma) go to 999     ! no, dt/dz < -2 K/km
      if (pm.gt.plimu)   go to 999     ! no, too low
  
      ! dtdz is valid, calculate tropopause pressure
      if (dtdz0.lt.gamma) then
        ag = (dtdz-dtdz0) / (pmk-pmk0)     
        bg = dtdz0 - (ag * pmk0)          
        ptph = exp(log((gamma-bg)/ag)/cnst_kap)
      else
        ptph = pm
      endif
  
      if (ptph.lt.pliml) go to 999     
      if (ptph.gt.plimu) go to 999           
  
      ! 2nd test: dtdz above 2 km must not exceed gamma
      p2km = ptph + deltaz*(pm/tm)*cnst_faktor     ! p at ptph + 2km
      asum = 0.0_r8                                ! dtdz above
      icount = 0                                   ! number of levels above
  
      ! test until apm < p2km
      do jj=j,2,-1
    
        pmk2 = .5_r8 * (p(jj-1)**cnst_kap+p(jj)**cnst_kap) ! p mean ^kappa
        pm2 = pmk2**(1/cnst_kap)                           ! p mean
        if(pm2.gt.ptph) go to 110                ! doesn't happen
        if(pm2.lt.p2km) go to 888                ! ptropo is valid

        a2 = (t(jj-1)-t(jj))                     ! a
        a2 = a2/(p(jj-1)**cnst_kap-p(jj)**cnst_kap)
        b2 = t(jj)-(a2*p(jj)**cnst_kap)          ! b
        tm2 = a2 * pmk2 + b2                     ! T mean
        dtdp2 = a2 * cnst_kap * (pm2**(cnst_kap-1))  ! dt/dp
        dtdz2 = cnst_faktor*dtdp2*pm2/tm2
        asum = asum+dtdz2
        icount = icount+1
        aquer = asum/float(icount)               ! dt/dz mean
   
        ! discard ptropo ?
        if (aquer.le.gamma) go to 999           ! dt/dz above < gamma
    
110     continue 
      enddo                           ! test next level
    
888   continue                        ! ptph is valid
      trp = ptph
      return
    
999   continue                        ! continue search at next higher level 
      pm0 = pm
      pmk0 = pmk
      dtdz0  = dtdz
    enddo 
    
    ! no tropopouse found
    return
  end subroutine twmo
  
  !--------------------------------------------------------------------
  ! This routine interpolates the temperatures in the physics state to
  ! find the temperature at the specified tropopause pressure.
  !-------------------------------------------------------------------
  function tropopause_interpolateT(state, icol, tropLev, tropP)
 
    implicit none

    type(physics_state), intent(in)     :: state 
    integer, intent(in)                 :: icol               ! column being processed
    integer, intent(in)                 :: tropLev            ! tropopause level index   
    real(r8), optional, intent(in)      :: tropP              ! tropopause pressure (Pa)
    real(r8)                            :: tropopause_interpolateT
    
    ! Local Variables
    real(r8)   :: tropT              ! tropopause temperature (K)
    real(r8)   :: dTdlogP            ! dT/dlog(P)
    
    ! Intrepolate the temperature linearly against log(P)
    
    ! Is the tropoause at the midpoint?
    if (tropP == state%pmid(icol, tropLev)) then
      tropT = state%t(icol, tropLev)
    
    else if (tropP < state%pmid(icol, tropLev)) then
    
      ! It is above the midpoint? Make sure we aren't at the top.
      if (tropLev > 1) then
        dTdlogP = (state%t(icol, tropLev) - state%t(icol, tropLev - 1)) / & 
          (log(state%pmid(icol, tropLev)) - log(state%pmid(icol, tropLev - 1)))
        tropT = state%t(icol, tropLev) + (log(tropP) - log(state%pmid(icol, tropLev))) * dTdlogP
      end if
    else
      
      ! It is below the midpoint. Make sure we aren't at the bottom.
      if (tropLev < pver) then
        dTdlogP = (state%t(icol, tropLev + 1) - state%t(icol, tropLev)) / &
          (log(state%pmid(icol, tropLev + 1)) - log(state%pmid(icol, tropLev)))
        tropT = state%t(icol, tropLev) + (log(tropP) - log(state%pmid(icol, tropLev))) * dTdlogP
      end if
    end if
    
    tropopause_interpolateT = tropT
  end function tropopause_interpolateT

  !----------------------------------------------------------------------------
  ! This routine interpolates the geopotential height in the physics state to
  ! find the geopotential height at the specified tropopause pressure.
  !------------------------------------------------------------------------------
  function tropopause_interpolateZ(state, icol, tropLev, tropP)
 
    implicit none

    type(physics_state), intent(in)     :: state 
    integer, intent(in)                 :: icol               ! column being processed
    integer, intent(in)                 :: tropLev            ! tropopause level index   
    real(r8), optional, intent(in)      :: tropP              ! tropopause pressure (Pa)
    real(r8)                            :: tropopause_interpolateZ
    
    ! Local Variables
    real(r8)   :: tropZ              ! tropopause geopotential height (m)
    real(r8)   :: dZdlogP            ! dZ/dlog(P)
    
    ! Intrepolate the geopotential height linearly against log(P)
    
    ! Is the tropoause at the midpoint?
    if (tropP == state%pmid(icol, tropLev)) then
      tropZ = state%zm(icol, tropLev)
    
    else if (tropP < state%pmid(icol, tropLev)) then
    
      ! It is above the midpoint? Make sure we aren't at the top.
      dZdlogP = (state%zm(icol, tropLev) - state%zi(icol, tropLev)) / &
        (log(state%pmid(icol, tropLev)) - log(state%pint(icol, tropLev)))
      tropZ = state%zm(icol, tropLev) + (log(tropP) - log(state%pmid(icol, tropLev))) * dZdlogP
    else
      
      ! It is below the midpoint. Make sure we aren't at the bottom.
      dZdlogP = (state%zm(icol, tropLev) - state%zi(icol, tropLev+1)) / &
        (log(state%pmid(icol, tropLev)) - log(state%pint(icol, tropLev+1)))
      tropZ = state%zm(icol, tropLev) + (log(tropP) - log(state%pmid(icol, tropLev))) * dZdlogP
    end if
    
    tropopause_interpolateZ = tropZ
  end function tropopause_interpolateZ

end module bcc_tropopause
