#include <misc.h>

      module mo_strato_aerosol
!---------------------------------------------------------------
!	... annual cycle for sulfur
!---------------------------------------------------------------

      use pmgrid,       only : plon, plat
      use ppgrid,       only : pcols, pver
      use shr_kind_mod, only : r8 => shr_kind_r8
      use abortutils,   only : endrun

      implicit none
      private
      save

      public  :: aero_init
      public  :: aero_timestep_init
      public  :: aero_set

      ! set via namelist
      character(len=16)  :: sad_scenario
      integer            :: sad_Fixed_date
      character(len=256) :: sad_file

      integer, parameter :: time_span = 12
      integer  :: ncid
      integer  :: ntimes
      integer  :: gndx  = 0
      integer  :: nlon, nlev, nlat
      integer  :: tim_ndx(2)
      integer, allocatable :: dates(:)
      real(r8), allocatable :: times(:)
      real(r8), allocatable :: sad_sage(:,:,:)
      real(r8), allocatable :: sad(:,:,:,:)
      logical :: doRamp_sad

      contains

      subroutine aero_init( scenario, fixed_date, file )
!-----------------------------------------------------------------------
! 	... open netCDF file containing annual sulfur data.  Initialize
!           arrays with the data to be interpolated to the current time.
!
!           It is assumed that the time coordinate is increasing
!           and represents calendar days; range = [1.,366.).
!-----------------------------------------------------------------------

      use ioFileMod,     only : getfil
      use pmgrid,        only : masterproc
      use time_manager,  only : get_curr_date
#ifdef SPMD
      use mpishorthand,  only : mpicom, mpir8, mpiint
#endif

      implicit none

!-----------------------------------------------------------------------
!	... dummy args
!-----------------------------------------------------------------------
      character(len=*), intent(in) :: scenario
      integer,          intent(in) :: fixed_date
      character(len=*), intent(in) :: file

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
      integer  :: i, k, n              ! indicies
      integer  :: ret                  ! return code
      integer  :: vid, dimid           ! netcdf ids
      integer  :: yr, mon, day         ! components of a date
      integer  :: ncsec
      integer  :: wrk_date
      real(r8) :: wrk_time
      character(len=256) :: locfn

#include <netcdf.inc>
!-----------------------------------------------------------------------

      sad_scenario   = scenario
      sad_fixed_date = fixed_date
      sad_file       = file

!-----------------------------------------------------------------------
!     	... check scenario
!-----------------------------------------------------------------------
      write(*,*) 'aero_init: aerosol values are as follows'
      if( sad_scenario == 'FIXED' ) then
         if( sad_Fixed_date == 0 ) then
            write(*,*)'aero_init: FIXED aerosols requires namelist sad_Fixed_date setting'
            call endrun
         end if
         doRamp_sad = .false.
         wrk_date = sad_Fixed_date
         write(*,*) 'FIXED values at date ',sad_Fixed_date
      else if( sad_scenario == 'RAMPED' ) then
         doRamp_sad  = .true.
         call get_curr_date( yr, mon, day, ncsec )
         wrk_date = 10000*yr + 100*mon + day
         write(*,*) 'RAMPED values from date ',wrk_date
      else
         write(*,*)'aero_init: input namelist SAD_SCENARIO must be set to either FIXED or RAMPED'
         call endrun
      end if

Masterproc_only : &
      if( masterproc ) then

!-----------------------------------------------------------------------
!     	... open netcdf file
!-----------------------------------------------------------------------
         call getfil (sad_file, locfn, 0)
         call wrap_open (trim(locfn), NF_NOWRITE, ncid)
!-----------------------------------------------------------------------
!     	... get and check spatial dimensions
!-----------------------------------------------------------------------
         call wrap_inq_dimid( ncid, 'lev', dimid )
         call wrap_inq_dimlen( ncid, dimid, nlev )
         if( nlev /= pver ) then
            write(*,*) 'aero_init : levels in netcdf file do not match simulation grid'
            call endrun
         end if
         call wrap_inq_dimid( ncid, 'lat', dimid )
         call wrap_inq_dimlen( ncid, dimid, i )
         if( i /= plat ) then
            write(*,*) 'aero_init : latitudes in netcdf file do not match simulation grid'
            call endrun
         end if
!-----------------------------------------------------------------------
!     	... get time dimension
!-----------------------------------------------------------------------
         call wrap_inq_dimid( ncid, 'time', dimid )
         call wrap_inq_dimlen( ncid, dimid, ntimes )
!-----------------------------------------------------------------------
!     	... allocate array and get time coordinate data
!-----------------------------------------------------------------------
         allocate( dates(ntimes), times(ntimes), stat = ret )
         if( ret /= 0 ) then
            write(*,*) 'aero_init : Failed to allocate dates,times ; error = ', ret
            call endrun
         end if
         call wrap_inq_varid( ncid, 'date', vid )
         call wrap_get_var_int( ncid, vid, dates )
         do n = 1,ntimes
            call days0( dates(n), 0, times(n) )
         end do
         call days0( wrk_date, 0, wrk_time )
         if( wrk_time < times(1) .or. wrk_time > times(ntimes) ) then
            write(*,*) 'aero_init: initial time is out of range of aerosol times'
            call endrun
         end if
         do n = 2,ntimes
            if( wrk_time <= times(n) ) then
               exit
            end if
         end do
         tim_ndx(1) = n - 1
         if( doRamp_sad ) then
            tim_ndx(2) = min( ntimes,tim_ndx(1) + time_span )
         else
            tim_ndx(2) = n
         end if

         write(*,*) '----------------------------------------------------'
         write(*,*) 'aero_init: diagnostics'
         write(*,*) 'ntimes = ',ntimes
	 write(*,*) 'dates'
	 write(*,'(10i10)') dates
	 write(*,*) ' '
	 write(*,*) 'time indicies = ',tim_ndx
	 write(*,*) '----------------------------------------------------'

!-----------------------------------------------------------------------
!     	... read in the sage sad data
!-----------------------------------------------------------------------
         call aero_get
	 call wrap_close( ncid )
      end if Masterproc_only

#ifdef SPMD
      call mpibcast( ntimes, 1, mpiint, 0, mpicom )
      call mpibcast( tim_ndx, 2, mpiint, 0, mpicom )
      if( .not. masterproc ) then
         allocate( sad_sage(plat,pver,tim_ndx(1):tim_ndx(2)),stat=ret )
         if( ret /= 0 ) then
	    write(*,*) 'aero_init : Failed to allocate sad_sage array; error = ',ret
            call endrun
         end if
         allocate( times(ntimes),stat=ret )
         if( ret /= 0 ) then
            write(*,*) 'aero_init : Failed to allocate times; error = ',ret
            call endrun
         end if
      end if
      call mpibcast( times, ntimes, mpir8, 0, mpicom )
      call mpibcast( sad_sage, plat*pver*(tim_ndx(2)-tim_ndx(1)+1), mpir8, 0, mpicom )
#endif
      call aero_dist
      deallocate( sad_sage )

      end subroutine aero_init

      subroutine aero_get
!--------------------------------------------------------------------
!	... read in sage aerosol surface area density
!	    (masterproc only)
!--------------------------------------------------------------------

      implicit none

!----------------------------------------------------------------------- 
!	... local variables
!----------------------------------------------------------------------- 
      integer  :: k, tcnt, t, vid
      integer  :: astat
      integer  :: start(3)
      integer  :: cnt(3)

      tcnt     = tim_ndx(2) - tim_ndx(1) + 1
      cnt(:)   = (/ plat,nlev,tcnt /)
!--------------------------------------------------------------------
!	... allocate variable
!--------------------------------------------------------------------
      allocate( sad_sage(plat,pver,tim_ndx(1):tim_ndx(2)),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'aero_get: failed to allocate sad_sage; error = ',astat
         call endrun
      end if

      call wrap_inq_varid( ncid, 'sad_sage', vid )
      start(:) = (/ 1,1,tim_ndx(1) /)
      call wrap_get_vara_realx( ncid, vid, start, cnt, sad_sage )

      end subroutine aero_get

      subroutine aero_dist
!----------------------------------------------------------------------- 
! 	... "chunk" the sage sad data
!----------------------------------------------------------------------- 

      use ppgrid,    only : pcols, begchunk, endchunk
      use phys_grid, only : get_lat_p, get_ncols_p

      implicit none

!----------------------------------------------------------------------- 
!	... local variables
!----------------------------------------------------------------------- 
      integer :: c, ncols
      integer :: i, jc, k
      integer :: astat

      if( allocated( sad ) ) then
         deallocate( sad,stat=astat )
         if( astat /= 0 ) then
            write(*,*) 'aero_dist: failed to deallocate sad; error = ',astat
            call endrun
         end if
      end if
      allocate( sad(pcols,pver,begchunk:endchunk,tim_ndx(1):tim_ndx(2)),stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'aero_dist: failed to allocate sad; error = ',astat
         call endrun
      end if

!-----------------------------------------------------------------------
! 	... distribute global array to chunks
!-----------------------------------------------------------------------
      do c = begchunk,endchunk
         ncols = get_ncols_p( c )
         do k = 1,pver
            do i = 1,ncols
               jc = get_lat_p( c,i )
               sad(i,k,c,:) = sad_sage(jc,k,:)
            end do
         end do
      end do

      end subroutine aero_dist

      subroutine aero_timestep_init
!-----------------------------------------------------------------------
! 	... check sad data times
!-----------------------------------------------------------------------

      use pmgrid,        only : masterproc
      use time_manager,  only : get_curr_date
      use ioFileMod,     only : getfil
#ifdef SPMD
      use mpishorthand,  only : mpiint, mpir8, mpicom
#endif

      implicit none

!-----------------------------------------------------------------------
! 	... local variables
!-----------------------------------------------------------------------
      integer  :: yr, mon, day      ! components of a date
      integer  :: ncdate            ! current date in integer format [yyyymmdd]
      integer  :: ncsec             ! current seconds in ncdate
      integer  :: astat
      real(r8) :: wrk_time
      character(len=256) :: locfn

#include <netcdf.inc>
!-----------------------------------------------------------------------

ramping : &
      if( doRamp_sad ) then
         call get_curr_date(yr, mon, day, ncsec)
         ncdate = yr*10000 + mon*100 + day
         call days0( ncdate, ncsec, wrk_time )
!-----------------------------------------------------------------------
!	... if ramping read next set of data ?
!-----------------------------------------------------------------------
         if( wrk_time > times(tim_ndx(2)) ) then
            tim_ndx(1) = tim_ndx(2)
            tim_ndx(2) = min( ntimes,tim_ndx(1) + time_span )
            if( masterproc ) then
               call getfil (sad_file, locfn, 0)
               call wrap_open (trim(locfn), NF_NOWRITE, ncid)
               call aero_get
               call wrap_close( ncid )
            end if
#ifdef SPMD
            call mpibcast( tim_ndx, 2, mpiint, 0, mpicom )
            if( .not. masterproc ) then
               allocate( sad_sage(plat,pver,tim_ndx(1):tim_ndx(2)),stat=astat )
               if( astat /= 0 ) then
                  write(*,*) 'aero_set: failed to allocate sad_sage; error = ',astat
                  call endrun
               end if
            end if
            call mpibcast( sad_sage, plat*pver*(tim_ndx(2)-tim_ndx(1)+1), mpir8, 0, mpicom )
            call aero_dist
            deallocate( sad_sage )
#else
            call aero_dist
            deallocate( sad_sage )
#endif
         end if
      end if ramping

      end subroutine aero_timestep_init

      subroutine aero_set( lchnk, ncols, model_sad )
!-----------------------------------------------------------------------
! 	... set model aerosol sad
!-----------------------------------------------------------------------

      use time_manager,  only: get_curr_date

      implicit none

!-----------------------------------------------------------------------
! 	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)     :: lchnk                      ! chunk id
      integer, intent(in)     :: ncols                      ! columns in chunk
      real(r8), intent(out)   :: model_sad(ncols,pver)      ! output surface area density

!-----------------------------------------------------------------------
! 	... local variables
!-----------------------------------------------------------------------
      integer  :: k                 ! working indicies
      integer  :: n, nm1            ! indicies
      integer  :: yr, mon, day      ! components of a date
      integer  :: ncdate            ! current date in integer format [yyyymmdd]
      integer  :: ncsec             ! current seconds in ncdate

      real(r8) :: wrk_time          ! wrk time
      real(r8) :: fac1              ! time interpolation factor

      if( doRamp_sad ) then
         call get_curr_date(yr, mon, day, ncsec)
         ncdate = yr*10000 + mon*100 + day
         call days0( ncdate, ncsec, wrk_time )
      else
         call days0( sad_Fixed_date, 0, wrk_time )
      end if

!-----------------------------------------------------------------------
! determine time index into input data
!-----------------------------------------------------------------------
      do n = tim_ndx(1)+1,tim_ndx(2)
         if( wrk_time <= times(n) ) then
            exit
         end if
      end do

!-----------------------------------------------------------------------
! check time index
!-----------------------------------------------------------------------
      if( n > tim_ndx(2) ) then
         write(*,*) 'aero_set: failed to find present time in lbc array'
         call endrun
      end if

!-----------------------------------------------------------------------
! set timing factor and time interpolate the sad data
!-----------------------------------------------------------------------
      nm1  = n - 1
      fac1 = (wrk_time - times(nm1))/(times(n) - times(nm1))
      do k = 1,pver
         model_sad(:ncols,k) = sad(:ncols,k,lchnk,nm1) &
                             + fac1*(sad(:ncols,k,lchnk,n) - sad(:ncols,k,lchnk,nm1))
      end do

      end subroutine aero_set

subroutine days0( ncdate, ncsec, days )
!----------------------------------------------------------------------- 
! Purpose: Convert date and seconds of day to floating point days since
!          00/01/01
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
!-----------------------------------------------------------------------


   implicit none

!-----------------------------------------------------------------------
!	... dummy arguments
!-----------------------------------------------------------------------
   integer, intent(in)   :: ncdate      ! Current date as yymmdd or yyyymmdd
   integer, intent(in)   :: ncsec       ! Seconds of day for current date
   real(r8), intent(out) :: days        ! Day of year

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
   integer  :: year        ! year
   integer  :: mnth        ! Month number
   integer  :: mday        ! Day number of month
   integer  :: jdcon(12) & ! Starting day number for each month
                    = (/ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /)
   real(r8) :: doy

!-----------------------------------------------------------------------
! Decode month and day
!-----------------------------------------------------------------------
   year = ncdate/10000
   mnth = mod( ncdate,10000 )/100
   if( mnth < 1 .or. mnth > 12) then
      write(*,*) 'days0: Bad month index=', mnth
      call endrun
   end if
   mday = mod(ncdate,100)
   doy  = jdcon(mnth) + mday + ncsec/86400._r8

   if( doy < 1._r8 .or. doy > 366._r8) then
      write(*,*) 'days0: bad day of year = ',doy
      call endrun
   end if

   days = 365._r8*year + doy

end subroutine days0

      end module mo_strato_aerosol
