#include <misc.h>
#include <params.h>

module upper_bc

!---------------------------------------------------------------------------------
! Module to compute the upper boundary condition for temperature (dry static energy)
! and trace gases. Uses the MSIS model, and SNOE and TIME GCM data.
!
! original code by Stacy Walters
! adapted by B. A. Boville
!---------------------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  use ppgrid,       only: pcols, pver, pverp
  use constituents, only: pcnst, pnats, ppcnst
  use physconst,    only: rair, mwdry

  implicit none
  private
  save
!
! Public interfaces
!
  public :: ubc_init           ! global initialization
  public :: ubc_timestep_init  ! time step initialization
  public :: ubc_get_vals       ! get ubc values for this step

!================================================================================================
contains
!================================================================================================

  subroutine ubc_init(tgcm_ubc_file, snoe_ubc_file) 
!-----------------------------------------------------------------------
! Initialization of time independent fields for the upper boundary condition
! Calls initialization routine for MSIS, TGCM and SNOE
!-----------------------------------------------------------------------
    use mo_tgcm_ubc, only: tgcm_ubc_inti
    use mo_snoe,     only: snoe_inti
    use mo_msis_ubc, only: msis_ubc_inti
    use time_manager,only: get_step_size

    character(len=256) :: tgcm_ubc_file
    character(len=256) :: snoe_ubc_file

!---------------------------Local workspace-----------------------------
    integer :: steps_per_day
    logical :: zonal_avg
!-----------------------------------------------------------------------

    steps_per_day = 86400/get_step_size()
    zonal_avg     = .false.

!-----------------------------------------------------------------------
!	... initialize the tgcm upper boundary module
!-----------------------------------------------------------------------
    call tgcm_ubc_inti(tgcm_ubc_file, zonal_avg, steps_per_day)
    write(*,*) 'ubc_init: after tgcm_ubc_inti'

!-----------------------------------------------------------------------
!	... initialize the snoe module
!-----------------------------------------------------------------------
    call snoe_inti(snoe_ubc_file)
    write(*,*) 'ubc_init: after snoe_inti'

!-----------------------------------------------------------------------
!	... initialize the msis module
!-----------------------------------------------------------------------
    call msis_ubc_inti( zonal_avg, steps_per_day )
    write(*,*) 'ubc_init: after msis_ubc_inti'

  end subroutine ubc_init

!===============================================================================

  subroutine ubc_timestep_init
!-----------------------------------------------------------------------
! timestep dependent setting
!-----------------------------------------------------------------------

    use bcc_solar_parms, only: get_solar_parms
    use mo_msis_ubc,    only: msis_timestep_init
    use mo_tgcm_ubc,    only: tgcm_timestep_init
    use mo_snoe,        only: snoe_timestep_init

    real(r8) :: ap, kp, f107, f107a

    call get_solar_parms( kp_s = kp, f107_s = f107, f107a_s = f107a, ap_s = ap )
    call msis_timestep_init( ap, f107, f107a )
    call tgcm_timestep_init
    call snoe_timestep_init( kp, f107 )

  end subroutine ubc_timestep_init

!===============================================================================

  subroutine ubc_get_vals (lchnk, ncol, ntop_molec, pint, zi, msis_temp, ubc_mmr)
!-----------------------------------------------------------------------
! interface routine for vertical diffusion and pbl scheme
!-----------------------------------------------------------------------
    use mo_msis_ubc,  only: get_msis_ubc
    use mo_snoe,      only: set_no_ubc
    use mo_tgcm_ubc,  only: set_tgcm_ubc

!------------------------------Arguments--------------------------------
    integer,  intent(in)  :: lchnk                 ! chunk identifier
    integer,  intent(in)  :: ncol                  ! number of atmospheric columns
    integer,  intent(in)  :: ntop_molec            ! top of molecular diffusion region (=1)
    real(r8), intent(in)  :: pint(pcols,pverp)     ! interface pressures
    real(r8), intent(in)  :: zi(pcols,pverp)       ! interface geoptl height above sfc

    real(r8), intent(out) :: ubc_mmr(pcols,pcnst+pnats) ! upper bndy mixing ratios (kg/kg)
    real(r8), intent(out) :: msis_temp(pcols)           ! upper bndy temperature (K)

!---------------------------Local storage-------------------------------
    real(r8), parameter :: m2km = 1.e-3_r8         ! meter to km
    real(r8) :: rho_top(pcols)                     ! density at top interface
    real(r8) :: z_top(pcols)                       ! height of top interface (km)
!-----------------------------------------------------------------------

    ubc_mmr(:,:) = 0._r8

    call get_msis_ubc( lchnk, ncol, msis_temp, ubc_mmr )
    rho_top(:ncol) = pint(:ncol,ntop_molec) / (rair*msis_temp(:ncol))
    z_top(:ncol)   = m2km * zi(:ncol,ntop_molec)
    call set_no_ubc  ( lchnk, ncol, z_top, ubc_mmr, rho_top )
    call set_tgcm_ubc( lchnk, ncol, ubc_mmr, mwdry )

  end subroutine ubc_get_vals

end module upper_bc
