#include <misc.h>
#include <params.h>

module aero_properties
 !
 ! Purpose:
 ! 
 
 
 implicit none
 
 public :: aero_properties_cam
 public :: aero_properties_bcc
 
contains

 subroutine aero_properties_cam(lchnk,ncol,ns,Nday,Nnite,IdxDay,IdxNite,pint,rh,aermmr, &
                                 tauxar,wa,ga,fa, &
				 frc_day, &
				 aertau,aerssa,aerasm,aerfwd)
  ! Purpose:
  
  use shr_kind_mod, only: r8 => shr_kind_r8
  use ppgrid
  use prescribed_aerosols, only: idxBG, idxSUL, idxSSLT, idxOCPHO, idxBCPHO, idxOCPHI, idxBCPHI, &
     idxDUSTfirst, numDUST, idxVOLC, naer_all
  use aer_optics, only: nrh, ndstsz, ksul, wsul, gsul, &
     ksslt, wsslt, gsslt, kcphil, wcphil, gcphil, kcphob, wcphob, gcphob, &
     kcb, wcb, gcb, kdst, wdst, gdst, kbg, wbg, gbg, kvolc, wvolc, gvolc
  use abortutils, only: endrun
   
  implicit none

#include <crdcon.h>

  integer nspint            ! Num of spctrl intervals across solar spectrum
  integer naer_groups       ! Num of aerosol groups for optical diagnostics

  parameter ( nspint = 19 )
  parameter ( naer_groups = 7 )    ! current groupings are sul, sslt, all carbons, all dust, and all aerosols

  !
  ! Input arguments
  !
  integer, intent(in) :: lchnk
  integer, intent(in) :: ncol              ! number of atmospheric columns
  integer, intent(in) :: ns                ! Spectral loop index
  integer, intent(in) :: Nday                      ! Number of daylight columns
  integer, intent(in) :: Nnite                      ! Number of night columns
  integer, intent(in), dimension(pcols) :: IdxDay  ! Indicies of daylight coumns
  integer, intent(in), dimension(pcols) :: IdxNite  ! Indicies of night coumns
  real(r8), intent(in) :: pint(pcols,pverp) ! Interface pressure
  real(r8), intent(in) :: rh(pcols,pver)   ! Relative humidity (fraction)
  real(r8), intent(in) :: aermmr(pcols,pver,naer_all) ! Aerosol mass mixing ratio

  ! 
  ! Output arguments
  ! Aerosol radiative property arrays
  ! 
  real(r8) , intent(out) :: tauxar(pcols,0:pver) ! aerosol extinction optical depth
  real(r8) , intent(out) :: wa(pcols,0:pver) ! aerosol single scattering albedo
  real(r8) , intent(out) :: ga(pcols,0:pver) ! aerosol assymetry parameter
  real(r8) , intent(out) :: fa(pcols,0:pver) ! aerosol forward scattered fraction
  real(r8) , intent(out) :: frc_day(pcols) ! = 1 for daylight, =0 for night columns
  real(r8) :: aertau(pcols,nspint,naer_groups) ! Aerosol column optical depth
  real(r8) :: aerssa(pcols,nspint,naer_groups) ! Aerosol column averaged single scattering albedo
  real(r8) :: aerasm(pcols,nspint,naer_groups) ! Aerosol column averaged asymmetry parameter
  real(r8) :: aerfwd(pcols,nspint,naer_groups) ! Aerosol column averaged forward scattering


  !
  !--------------------------Local variables--------------------------
  !
  integer kk
  integer i                 ! Longitude loop index
  integer k                 ! Level loop index
  integer n                 ! Loop index for daylight
  integer indxsl            ! Index for cloud particle properties
  integer ksz               ! dust size bin index
  integer krh               ! relative humidity bin index
  integer kaer              ! aerosol group index
  real(r8) wrh              ! weight for linear interpolation between lut points
  real(r8) :: rhtrunc       ! rh, truncated for the purposes of extrapolating
                             ! aerosol optical properties
  !
  ! Aerosol mass paths by species
  !
  real(r8) usul(pcols,pver)   ! sulfate (SO4)
  real(r8) ubg(pcols,pver)    ! background aerosol
  real(r8) usslt(pcols,pver)  ! sea-salt (SSLT)
  real(r8) ucphil(pcols,pver) ! hydrophilic organic carbon (OCPHI)
  real(r8) ucphob(pcols,pver) ! hydrophobic organic carbon (OCPHO)
  real(r8) ucb(pcols,pver)    ! black carbon (BCPHI + BCPHO)
  real(r8) uvolc(pcols,pver) ! volcanic mass
  real(r8) udst(pcols,ndstsz,pver) ! dust

!
! local variables used for the external mixing of aerosol species
!
  real(r8) tau_sul             ! optical depth, sulfate
  real(r8) tau_bg              ! optical depth, background aerosol
  real(r8) tau_sslt            ! optical depth, sea-salt
  real(r8) tau_cphil           ! optical depth, hydrophilic carbon
  real(r8) tau_cphob           ! optical depth, hydrophobic carbon
  real(r8) tau_cb              ! optical depth, black carbon
  real(r8) tau_volc            ! optical depth, volcanic
  real(r8) tau_dst(ndstsz)     ! optical depth, dust, by size category
  real(r8) tau_dst_tot         ! optical depth, total dust
  real(r8) tau_tot             ! optical depth, total aerosol

  real(r8) tau_w_sul           ! optical depth * single scattering albedo, sulfate
  real(r8) tau_w_bg            ! optical depth * single scattering albedo, background aerosol
  real(r8) tau_w_sslt          ! optical depth * single scattering albedo, sea-salt
  real(r8) tau_w_cphil         ! optical depth * single scattering albedo, hydrophilic carbon
  real(r8) tau_w_cphob         ! optical depth * single scattering albedo, hydrophobic carbon
  real(r8) tau_w_cb            ! optical depth * single scattering albedo, black carbon
  real(r8) tau_w_volc          ! optical depth * single scattering albedo, volcanic
  real(r8) tau_w_dst(ndstsz)   ! optical depth * single scattering albedo, dust, by size
  real(r8) tau_w_dst_tot       ! optical depth * single scattering albedo, total dust
  real(r8) tau_w_tot           ! optical depth * single scattering albedo, total aerosol

  real(r8) tau_w_g_sul         ! optical depth * single scattering albedo * asymmetry parameter, sulfate
  real(r8) tau_w_g_bg          ! optical depth * single scattering albedo * asymmetry parameter, background aerosol
  real(r8) tau_w_g_sslt        ! optical depth * single scattering albedo * asymmetry parameter, sea-salt
  real(r8) tau_w_g_cphil       ! optical depth * single scattering albedo * asymmetry parameter, hydrophilic carbon
  real(r8) tau_w_g_cphob       ! optical depth * single scattering albedo * asymmetry parameter, hydrophobic carbon
  real(r8) tau_w_g_cb          ! optical depth * single scattering albedo * asymmetry parameter, black carbon
  real(r8) tau_w_g_volc        ! optical depth * single scattering albedo * asymmetry parameter, volcanic
  real(r8) tau_w_g_dst(ndstsz) ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
  real(r8) tau_w_g_dst_tot     ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) tau_w_g_tot         ! optical depth * single scattering albedo * asymmetry parameter, total aerosol

  real(r8) f_sul               ! forward scattering fraction, sulfate
  real(r8) f_bg                ! forward scattering fraction, background aerosol
  real(r8) f_sslt              ! forward scattering fraction, sea-salt
  real(r8) f_cphil             ! forward scattering fraction, hydrophilic carbon
  real(r8) f_cphob             ! forward scattering fraction, hydrophobic carbon
  real(r8) f_cb                ! forward scattering fraction, black carbon
  real(r8) f_volc              ! forward scattering fraction, volcanic
  real(r8) f_dst(ndstsz)       ! forward scattering fraction, dust, by size
  real(r8) f_dst_tot           ! forward scattering fraction, total dust
  real(r8) f_tot               ! forward scattering fraction, total aerosol

  real(r8) tau_w_f_sul         ! optical depth * forward scattering fraction * single scattering albedo, sulfate
  real(r8) tau_w_f_bg          ! optical depth * forward scattering fraction * single scattering albedo, background
  real(r8) tau_w_f_sslt        ! optical depth * forward scattering fraction * single scattering albedo, sea-salt
  real(r8) tau_w_f_cphil       ! optical depth * forward scattering fraction * single scattering albedo, hydrophilic C
  real(r8) tau_w_f_cphob       ! optical depth * forward scattering fraction * single scattering albedo, hydrophobic C
  real(r8) tau_w_f_cb          ! optical depth * forward scattering fraction * single scattering albedo, black C
  real(r8) tau_w_f_volc        ! optical depth * forward scattering fraction * single scattering albedo, volcanic
  real(r8) tau_w_f_dst(ndstsz) ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
  real(r8) tau_w_f_dst_tot     ! optical depth * forward scattering fraction * single scattering albedo, total dust
  real(r8) tau_w_f_tot         ! optical depth * forward scattering fraction * single scattering albedo, total aerosol
  real(r8) w_dst_tot           ! single scattering albedo, total dust
  real(r8) w_tot               ! single scattering albedo, total aerosol
  real(r8) g_dst_tot           ! asymmetry parameter, total dust
  real(r8) g_tot               ! asymmetry parameter, total aerosol
  real(r8) ksuli               ! specific extinction interpolated between rh look-up-table points, sulfate
  real(r8) ksslti              ! specific extinction interpolated between rh look-up-table points, sea-salt
  real(r8) kcphili             ! specific extinction interpolated between rh look-up-table points, hydrophilic carbon
  real(r8) wsuli               ! single scattering albedo interpolated between rh look-up-table points, sulfate
  real(r8) wsslti              ! single scattering albedo interpolated between rh look-up-table points, sea-salt
  real(r8) wcphili             ! single scattering albedo interpolated between rh look-up-table points, hydrophilic carbon
  real(r8) gsuli               ! asymmetry parameter interpolated between rh look-up-table points, sulfate
  real(r8) gsslti              ! asymmetry parameter interpolated between rh look-up-table points, sea-salt
  real(r8) gcphili             ! asymmetry parameter interpolated between rh look-up-table points, hydrophilic carbon


  real(r8) pflx(pcols,0:pverp) ! Interface press, including extra layer
  real(r8) path             ! Mass path of layer
  real(r8) pdel             ! Pressure difference across layer


  !
  ! Arrays used in vectorization
  !
  real(r8), dimension(pcols,ndstsz) :: v_tau_dst             ! optical depth, dust, by size category
  real(r8), dimension(pcols,ndstsz) :: v_tau_w_dst           ! optical depth * single scattering albedo, dust, by size
  real(r8), dimension(pcols,ndstsz) :: v_tau_w_g_dst         ! optical depth * single scattering albedo * asymmetry parameter, dust, by size
  real(r8), dimension(pcols,ndstsz) :: v_tau_w_f_dst         ! optical depth * forward scattering fraction * single scattering albedo, dust, by size
  real(r8), dimension(pcols)        :: v_tau_dst_tot         ! optical depth, total dust
  real(r8), dimension(pcols)        :: v_tau_w_dst_tot       ! optical depth * single scattering albedo, total dust
  real(r8), dimension(pcols)        :: v_tau_w_g_dst_tot     ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8), dimension(pcols)        :: v_tau_w_f_dst_tot     ! optical depth * forward scattering fraction * single scattering albedo, total dust

  real(r8) :: Tv_tau_dst
  real(r8) :: Tv_tau_w_dst
  real(r8) :: Tv_tau_w_g_dst
  real(r8) :: Tv_tau_w_f_dst


  logical :: lrhtrunc_lt0(pver)      ! Logical rhtrunc < 0.0_r8
  logical :: lg_tot_gt1(pver)        ! Logical g_tot > 1.0_r8
  logical :: lg_tot_ltm1(pver)       ! Logical g_tot < -1.0_r8
  logical :: lf_tot_gt1(pver)        ! Logical f_tot > 1.0_r8
  logical :: lf_tot_lt0(pver)        ! Logical f_tot < 0.0_r8


   
  ! initialize aerosol diagnostic fields to 0.0 
  ! Average can be obtained by dividing <aerod>/<frc_day>
  aertau(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
  aerssa(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
  aerasm(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
  aerfwd(1:ncol,1:nspint,1:naer_groups) = 0.0_r8

  do k=1,pverp
    do i=1,Nday
      pflx(i,k) = pint(i,k)
    end do
  end do
  do i=1,Nday
    pflx(i,0) = 0._r8
  enddo
   
  do k=1,pver
!cdir nodep
    do i=1,Nday
      pdel      = pflx(i,k+1) - pflx(i,k)
      path      = pdel / gravit
         
      usul(i,k) = aermmr(i,k,idxSUL) * path 
      ubg(i,k) = aermmr(i,k,idxBG) * path 
      usslt(i,k) = aermmr(i,k,idxSSLT) * path
      if (usslt(i,k) .lt. 0.0) then  ! usslt is sometimes small and negative, will be fixed
        usslt(i,k) = 0.0
      end if
      ucphil(i,k) = aermmr(i,k,idxOCPHI) * path
      ucphob(i,k) = aermmr(i,k,idxOCPHO) * path
      ucb(i,k) = ( aermmr(i,k,idxBCPHO) + aermmr(i,k,idxBCPHI) ) * path
      uvolc(i,k) =  aermmr(i,k,idxVOLC)
!cdir expand=ndstsz
      do ksz = 1, ndstsz
        udst(i,ksz,k) = aermmr(i,k,idxDUSTfirst-1+ksz) * path
      end do
    end do ! End  do i=1,Nday
  end do ! End  k=1,pver
 
!
! Compute tau_dst_tot, tau_w_dst_tot, tau_w_g_dst_tot, and, tau_w_f_dst_tot
!
  do i=1,Nday
    frc_day(i) = 1.0_r8
    do kaer = 1, naer_groups
      aertau(i,ns,kaer) = 0.0
      aerssa(i,ns,kaer) = 0.0
      aerasm(i,ns,kaer) = 0.0
      aerfwd(i,ns,kaer) = 0.0
    end do
  end do ! End do i=1,Nday

  f_cphob  = gcphob(ns) * gcphob(ns)
  f_cb     = gcb(ns) * gcb(ns)
  f_volc   = gvolc(ns) * gvolc(ns)
  f_bg     = gbg(ns) * gbg(ns)
  f_dst(:) = gdst(:,ns) * gdst(:,ns)

!CSD$ PARALLEL DO PRIVATE( v_tau_dst_tot ) &
!CSD$ PRIVATE( v_tau_w_dst_tot, v_tau_w_g_dst_tot, v_tau_w_f_dst_tot, Tv_tau_dst, Tv_tau_w_dst, Tv_tau_w_g_dst ) &
!CSD$ PRIVATE( Tv_tau_w_f_dst, tmp1l, tmp2l, tmp3l, tmp1i, tmp2i, tmp3i, rhtrunc, krh, wrh, ksuli, ksslti ) &
!CSD$ PRIVATE( kcphili, wsuli, wsslti, wcphili, gsuli, gsslti, gcphili, tau_sul, tau_sslt, tau_cphil, tau_cphob ) &
!CSD$ PRIVATE( tau_cb, tau_volc, tau_bg, tau_w_sul, tau_w_sslt, tau_w_cphil, tau_w_cphob, tau_w_cb, tau_w_volc, tau_w_bg ) &
!CSD$ PRIVATE( tau_w_g_sul, tau_w_g_sslt, tau_w_g_cphil, tau_w_g_cphob, tau_w_g_cb, tau_w_g_volc, tau_w_g_bg, f_sul,f_sslt ) &
!CSD$ PRIVATE(  f_cphil, tau_w_f_sul, tau_w_f_bg, tau_w_f_sslt, tau_w_f_cphil, tau_w_f_cphob, tau_w_f_cb, tau_w_f_volc ) &
!CSD$ PRIVATE( tau_dst_tot, tau_w_dst_tot, tau_w_g_dst_tot, tau_w_f_dst_tot, w_dst_tot, g_dst_tot, f_dst_tot, tau_tot ) &
!CSD$ PRIVATE( tau_w_tot, tau_w_g_tot, tau_w_f_tot, w_tot, g_tot, f_tot, k, i, kk )
      
	  do k=1,pver
!
! The following logicals are used to check to see whether we have invalid
! values for rhtrunc, g_tot, and f_tot.
! The above values are checked for every level of K and column of I.
! However, since the out of bounds conditions are *extremely* rare (in fact
! we abort the run if we find any one), let's reduce the number of 'if' tests
! by 1/K, and only check for aborting the run after scanning all Ks and Is.
!
         lrhtrunc_lt0(k) = .false.
         lg_tot_gt1(k)   = .false.
         lg_tot_ltm1(k)  = .false.
         lf_tot_gt1(k)   = .false.
         lf_tot_lt0(k)   = .false.

         
         v_tau_dst_tot(1:Nday)     = 0.0_r8
         v_tau_w_dst_tot(1:Nday)   = 0.0_r8
         v_tau_w_g_dst_tot(1:Nday) = 0.0_r8
         v_tau_w_f_dst_tot(1:Nday) = 0.0_r8


         do i=1,Nday
!cdir expand
            do kk = 1, ndstsz

               Tv_tau_dst           = 1.e4 * kdst(kk,ns) * udst(i,kk,k)
               Tv_tau_w_dst         = Tv_tau_dst         * wdst(kk,ns)
               Tv_tau_w_g_dst       = Tv_tau_w_dst       * gdst(kk,ns)
               Tv_tau_w_f_dst       = Tv_tau_w_dst       * f_dst(kk)

               v_tau_dst_tot(i)     = v_tau_dst_tot(i)     + Tv_tau_dst
               v_tau_w_dst_tot(i)   = v_tau_w_dst_tot(i)   + Tv_tau_w_dst
               v_tau_w_g_dst_tot(i) = v_tau_w_g_dst_tot(i) + Tv_tau_w_g_dst
               v_tau_w_f_dst_tot(i) = v_tau_w_f_dst_tot(i) + Tv_tau_w_f_dst

            end do
!
! End do i=1,Nday
!
         end do

         do i=1,Nday
!
! Set aerosol properties
! Conversion factor to adjust aerosol extinction (m2/g)
! 
               rhtrunc = rh(i,k)
               rhtrunc = min(rh(i,k),1._r8)
               if ( rhtrunc < 0.0_r8 ) lrhtrunc_lt0(k) = .true.
               krh = min(floor( rhtrunc * nrh ) + 1, nrh - 1)
               wrh = rhtrunc * nrh - krh

               ! linear interpolation of optical properties between rh table points
               ksuli = ksul(krh + 1, ns) * (wrh + 1) - ksul(krh, ns) * wrh
               ksslti = ksslt(krh + 1, ns) * (wrh + 1) - ksslt(krh, ns) * wrh
               kcphili = kcphil(krh + 1, ns) * (wrh + 1) - kcphil(krh, ns) * wrh
               wsuli = wsul(krh + 1, ns) * (wrh + 1) - wsul(krh, ns) * wrh
               wsslti = wsslt(krh + 1, ns) * (wrh + 1) - wsslt(krh, ns) * wrh
               wcphili = wcphil(krh + 1, ns) * (wrh + 1) - wcphil(krh, ns) * wrh
               gsuli = gsul(krh + 1, ns) * (wrh + 1) - gsul(krh, ns) * wrh
               gsslti = gsslt(krh + 1, ns) * (wrh + 1) - gsslt(krh, ns) * wrh
               gcphili = gcphil(krh + 1, ns) * (wrh + 1) - gcphil(krh, ns) * wrh

               tau_sul = 1.e4 * ksuli * usul(i,k)
               tau_sslt = 1.e4 * ksslti * usslt(i,k)
               tau_cphil = 1.e4 * kcphili * ucphil(i,k)
               tau_cphob = 1.e4 * kcphob(ns) * ucphob(i,k)
               tau_cb = 1.e4 * kcb(ns) * ucb(i,k)
               tau_volc = 1.e3 * kvolc(ns) * uvolc(i,k)
               tau_bg = 1.e4 * kbg(ns) * ubg(i,k)

               tau_w_sul = tau_sul * wsuli
               tau_w_sslt = tau_sslt * wsslti
               tau_w_cphil = tau_cphil * wcphili
               tau_w_cphob = tau_cphob * wcphob(ns)
               tau_w_cb = tau_cb * wcb(ns)
               tau_w_volc = tau_volc * wvolc(ns)
               tau_w_bg = tau_bg * wbg(ns)

               tau_w_g_sul = tau_w_sul * gsuli
               tau_w_g_sslt = tau_w_sslt * gsslti
               tau_w_g_cphil = tau_w_cphil * gcphili
               tau_w_g_cphob = tau_w_cphob * gcphob(ns)
               tau_w_g_cb = tau_w_cb * gcb(ns)
               tau_w_g_volc = tau_w_volc * gvolc(ns)
               tau_w_g_bg = tau_w_bg * gbg(ns)

               f_sul = gsuli * gsuli
               f_sslt = gsslti * gsslti
               f_cphil = gcphili * gcphili

               tau_w_f_sul = tau_w_sul * f_sul
               tau_w_f_bg = tau_w_bg * f_bg
               tau_w_f_sslt = tau_w_sslt * f_sslt
               tau_w_f_cphil = tau_w_cphil * f_cphil
               tau_w_f_cphob = tau_w_cphob * f_cphob
               tau_w_f_cb = tau_w_cb * f_cb
               tau_w_f_volc = tau_w_volc * f_volc
!
! mix dust aerosol size bins
!   w_dst_tot, g_dst_tot, w_dst_tot are currently not used anywhere
!   but calculate them anyway for future use
!
               tau_dst_tot     = v_tau_dst_tot(i)
               tau_w_dst_tot   = v_tau_w_dst_tot(i)
               tau_w_g_dst_tot = v_tau_w_g_dst_tot(i)
               tau_w_f_dst_tot = v_tau_w_f_dst_tot(i)

               if (tau_dst_tot .gt. 0.0) then
                 w_dst_tot = tau_w_dst_tot / tau_dst_tot
               else
                 w_dst_tot = 0.0
               endif

               if (tau_w_dst_tot .gt. 0.0) then
                 g_dst_tot = tau_w_g_dst_tot / tau_w_dst_tot
                 f_dst_tot = tau_w_f_dst_tot / tau_w_dst_tot
               else
                 g_dst_tot = 0.0
                 f_dst_tot = 0.0
               endif
!
! mix aerosols
!
               tau_tot     = tau_sul + tau_sslt &
                           + tau_cphil + tau_cphob + tau_cb + tau_dst_tot
               tau_tot     = tau_tot + tau_bg + tau_volc

               tau_w_tot   = tau_w_sul + tau_w_sslt &
                           + tau_w_cphil + tau_w_cphob + tau_w_cb + tau_w_dst_tot
               tau_w_tot   = tau_w_tot + tau_w_bg + tau_w_volc

               tau_w_g_tot = tau_w_g_sul + tau_w_g_sslt &
                           + tau_w_g_cphil + tau_w_g_cphob + tau_w_g_cb + tau_w_g_dst_tot
               tau_w_g_tot = tau_w_g_tot + tau_w_g_bg + tau_w_g_volc

               tau_w_f_tot = tau_w_f_sul + tau_w_f_sslt &
                           + tau_w_f_cphil + tau_w_f_cphob + tau_w_f_cb + tau_w_f_dst_tot
               tau_w_f_tot = tau_w_f_tot + tau_w_f_bg + tau_w_f_volc

               if (tau_tot .gt. 0.0) then
                 w_tot = tau_w_tot / tau_tot
               else
                 w_tot = 0.0
               endif

               if (tau_w_tot .gt. 0.0) then
                 g_tot = tau_w_g_tot / tau_w_tot
                 f_tot = tau_w_f_tot / tau_w_tot
               else
                 g_tot = 0.0
                 f_tot = 0.0
               endif

               if ( g_tot > 1.0_r8 )  lg_tot_gt1(k)  = .true.
               if ( g_tot < -1.0_r8 ) lg_tot_ltm1(k) = .true.
               if ( f_tot > 1.0_r8 )  lf_tot_gt1(k)  = .true.
               if ( f_tot < 0.0_r8 )  lf_tot_lt0(k)  = .true.

               tauxar(i,k) = tau_tot
               wa(i,k)     = min(w_tot, 0.999999_r8)
               ga(i,k)     = g_tot
               fa(i,k)     = f_tot

               aertau(i,ns,1) = aertau(i,ns,1) + tau_sul
               aertau(i,ns,2) = aertau(i,ns,2) + tau_sslt
               aertau(i,ns,3) = aertau(i,ns,3) + tau_cphil + tau_cphob + tau_cb
               aertau(i,ns,4) = aertau(i,ns,4) + tau_dst_tot
               aertau(i,ns,5) = aertau(i,ns,5) + tau_bg
               aertau(i,ns,6) = aertau(i,ns,6) + tau_volc
               aertau(i,ns,7) = aertau(i,ns,7) + tau_tot

               aerssa(i,ns,1) = aerssa(i,ns,1) + tau_w_sul
               aerssa(i,ns,2) = aerssa(i,ns,2) + tau_w_sslt
               aerssa(i,ns,3) = aerssa(i,ns,3) + tau_w_cphil + tau_w_cphob + tau_w_cb
               aerssa(i,ns,4) = aerssa(i,ns,4) + tau_w_dst_tot
               aerssa(i,ns,5) = aerssa(i,ns,5) + tau_w_bg
               aerssa(i,ns,6) = aerssa(i,ns,6) + tau_w_volc
               aerssa(i,ns,7) = aerssa(i,ns,7) + tau_w_tot

               aerasm(i,ns,1) = aerasm(i,ns,1) + tau_w_g_sul
               aerasm(i,ns,2) = aerasm(i,ns,2) + tau_w_g_sslt
               aerasm(i,ns,3) = aerasm(i,ns,3) + tau_w_g_cphil + tau_w_g_cphob + tau_w_g_cb
               aerasm(i,ns,4) = aerasm(i,ns,4) + tau_w_g_dst_tot
               aerasm(i,ns,5) = aerasm(i,ns,5) + tau_w_g_bg
               aerasm(i,ns,6) = aerasm(i,ns,6) + tau_w_g_volc
               aerasm(i,ns,7) = aerasm(i,ns,7) + tau_w_g_tot

               aerfwd(i,ns,1) = aerfwd(i,ns,1) + tau_w_f_sul
               aerfwd(i,ns,2) = aerfwd(i,ns,2) + tau_w_f_sslt
               aerfwd(i,ns,3) = aerfwd(i,ns,3) + tau_w_f_cphil + tau_w_f_cphob + tau_w_f_cb
               aerfwd(i,ns,4) = aerfwd(i,ns,4) + tau_w_f_dst_tot
               aerfwd(i,ns,5) = aerfwd(i,ns,5) + tau_w_f_bg
               aerfwd(i,ns,6) = aerfwd(i,ns,6) + tau_w_f_volc
               aerfwd(i,ns,7) = aerfwd(i,ns,7) + tau_w_f_tot

! 
! End do i=1,Nday
! 
         end do
! 
! End do k=1,pver
! 
      end do
!CSD$ END PARALLEL 

      if (any( lrhtrunc_lt0(:) )) then
         write(6,*) "rhtrunc < 0.0"
         call endrun('RADCSWMX')
      end if
      if (any( lg_tot_gt1(:) )) then
         write(6,*) "g_tot > 1.0"
         call endrun('RADCSWMX')
      end if
      if (any( lg_tot_ltm1(:) )) then
         write(6,*) "g_tot < -1.0"
         call endrun('RADCSWMX')
      end if
      if (any( lf_tot_gt1(:) )) then
         write(6,*) "f_tot > 1.0"
         call endrun('RADCSWMX')
      end if
      if (any( lf_tot_lt0(:) )) then
         write(6,*) "f_tot < 0.0"
         call endrun('RADCSWMX')
      end if


         ! normalize aerosol optical diagnostic fields
      do kaer = 1, naer_groups

         do i=1,Nday


               if (aerssa(i,ns,kaer) .gt. 0.0) then   ! aerssa currently holds product of tau and ssa
                  aerasm(i,ns,kaer) = aerasm(i,ns,kaer) / aerssa(i,ns,kaer)
                  aerfwd(i,ns,kaer) = aerfwd(i,ns,kaer) / aerssa(i,ns,kaer)
               else
                  aerasm(i,ns,kaer) = 0.0_r8
                  aerfwd(i,ns,kaer) = 0.0_r8
               end if

               if (aertau(i,ns,kaer) .gt. 0.0) then
                  aerssa(i,ns,kaer) = aerssa(i,ns,kaer) / aertau(i,ns,kaer)
               else
                  aerssa(i,ns,kaer) = 0.0_r8
               end if

! 
! End do i=1,Nday
! 
         end do
!
! End do kaer = 1, naer_groups
!

      end do


  return
   
 end subroutine aero_properties_cam


 subroutine aero_properties_bcc(lchnk,ncol,ns,Nday,Nnite,IdxDay,IdxNite,pint,rh,aermmr, &
                                 tauxar,wa,ga,fa, &
				 frc_day, &
				 aertau,aerssa,aerasm,aerfwd)
  ! Purpose:compute aerosols' optical properties using Zhang Hua's scheme, but bg and volc use CAM's method.
  
  use shr_kind_mod, only: r8 => shr_kind_r8
  use ppgrid
  use prescribed_aerosols, only: idxBG, idxSUL, idxSSLT, idxOCPHO, idxBCPHO, idxOCPHI, idxBCPHI, &
     idxDUSTfirst, numDUST, idxVOLC, naer_all
  use aer_optics, only: nrh, ndstsz, ksul, wsul, gsul, &
     ksslt, wsslt, gsslt, kcphil, wcphil, gcphil, kcphob, wcphob, gcphob, &
     kcb, wcb, gcb, kdst, wdst, gdst, kbg, wbg, gbg, kvolc, wvolc, gvolc
  use abortutils, only: endrun
  use constituents, only: ppcnst   ! by wzl
  use physics_types, only: physics_state !by wangfang,2010.1.14
  use cmparray_mod, only: CmpDayNite, ExpDayNite !by wangfang,2010.1.14
   
  implicit none

#include <crdcon.h>

  integer nspint            ! Num of spctrl intervals across solar spectrum
  integer naer_groups       ! Num of aerosol groups for optical diagnostics

  parameter ( nspint = 19 )
  parameter ( naer_groups = 7 )    ! current groupings are sul, sslt, all carbons, all dust, and all aerosols

  !
  ! Input arguments
  !
  integer, intent(in) :: lchnk
  integer, intent(in) :: ncol              ! number of atmospheric columns
  integer, intent(in) :: ns                ! Spectral loop index
  integer, intent(in) :: Nday                      ! Number of daylight columns
  integer, intent(in) :: Nnite                      ! Number of night columns
  integer, intent(in), dimension(pcols) :: IdxDay  ! Indicies of daylight coumns
  integer, intent(in), dimension(pcols) :: IdxNite  ! Indicies of night coumns
  real(r8), intent(in) :: pint(pcols,pverp) ! Interface pressure
  real(r8), intent(in) :: rh(pcols,pver)   ! Relative humidity (fraction)
  real(r8), intent(in) :: aermmr(pcols,pver,naer_all) ! Aerosol mass mixing ratio

  ! 
  ! Output arguments
  ! Aerosol radiative property arrays
  ! 
  real(r8) , intent(out) :: tauxar(pcols,0:pver) ! aerosol extinction optical depth
  real(r8) , intent(out) :: wa(pcols,0:pver) ! aerosol single scattering albedo
  real(r8) , intent(out) :: ga(pcols,0:pver) ! aerosol assymetry parameter
  real(r8) , intent(out) :: fa(pcols,0:pver) ! aerosol forward scattered fraction
  real(r8) , intent(out) :: frc_day(pcols) ! = 1 for daylight, =0 for night columns
  real(r8) :: aertau(pcols,nspint,naer_groups) ! Aerosol column optical depth
  real(r8) :: aerssa(pcols,nspint,naer_groups) ! Aerosol column averaged single scattering albedo
  real(r8) :: aerasm(pcols,nspint,naer_groups) ! Aerosol column averaged asymmetry parameter
  real(r8) :: aerfwd(pcols,nspint,naer_groups) ! Aerosol column averaged forward scattering


  !
  !--------------------------Local variables--------------------------
  !
   type(physics_state) :: state  !get CAUCE aerosols(state%q) and state%zi and state%t

   real(r8) :: aer_tr(pcols,pver,ppcnst)    ! by 11
   !real(r8) :: zi(pcols,pverp) ! by 11  !not used below, wangfang,2010.1.14
   !real(r8) :: t(pcols,pver)  ! by wzl  !not used below, wangfang,2010.1.14
!----------------------------------------------------------------------------------
   real(r8) pflx(pcols,0:pverp) ! Interface press, including extra layer
   real(r8) ubg(pcols,pver)    ! background aerosol
   real(r8) uvolc(pcols,pver)    ! background aerosol
   real(r8) path             ! Mass path of layer
   real(r8) pdel             ! Pressure difference across layer

!
  real(r8) tau_bg              ! optical depth, background aerosol
  real(r8) tau_volc            ! optical depth, volcanic

  real(r8) tau_w_bg            ! optical depth * single scattering albedo, background aerosol
  real(r8) tau_w_volc          ! optical depth * single scattering albedo, volcanic
  real(r8) tau_w_g_bg          ! optical depth * single scattering albedo * asymmetry parameter, background aerosol
  real(r8) tau_w_g_volc        ! optical depth * single scattering albedo * asymmetry parameter, volcanic
  real(r8) f_bg                ! forward scattering fraction, background aerosol
  real(r8) f_volc

  real(r8) tau_w_f_bg          ! optical depth * forward scattering fraction * single scattering albedo, background
  real(r8) tau_w_f_volc        ! optical depth * forward scattering fraction * single scattering albedo, volcanic

  logical :: lrhtrunc_lt0(pver)      ! Logical rhtrunc < 0.0_r8
  logical :: lg_tot_gt1(pver)        ! Logical g_tot > 1.0_r8
  logical :: lg_tot_ltm1(pver)       ! Logical g_tot < -1.0_r8
  logical :: lf_tot_gt1(pver)        ! Logical f_tot > 1.0_r8
  logical :: lf_tot_lt0(pver)        ! Logical f_tot < 0.0_r8
! new aerosol/wangzhili
   integer  IRH
   real(r8) ss_ex(12,19), oc_ex(12,19), sf_ex(12,19)
   real(r8) ss_w(12,19),  oc_w(12,19), sf_w(12,19)
   real(r8) ss_g(12,19),  oc_g(12,19), sf_g(12,19)
   real(r8) ss_f(12), bc_f(12), oc_f(12), sd_f(12), sf_f(12)
   real(r8) u_ss(pcols,12,pver), u_bc(pcols,12,pver)
   real(r8) u_oc(pcols,12,pver), u_sd(pcols,12,pver), u_sf(pcols,12,pver)
   real(r8) Tv_tau_ss, Tv_tau_bc, Tv_tau_oc, Tv_tau_sd, Tv_tau_sf
   real(r8) Tv_tau_w_ss,Tv_tau_w_bc,Tv_tau_w_oc,Tv_tau_w_sd,Tv_tau_w_sf
   real(r8) Tv_tau_w_g_ss,Tv_tau_w_g_bc,Tv_tau_w_g_oc,Tv_tau_w_g_sd,Tv_tau_w_g_sf
   real(r8) Tv_tau_w_f_ss,Tv_tau_w_f_bc,Tv_tau_w_f_oc,Tv_tau_w_f_sd,Tv_tau_w_f_sf
   real(r8) tau_ss_tot,	tau_bc_tot,tau_oc_tot,tau_sd_tot,tau_sf_tot
   real(r8) tau_w_ss_tot,tau_w_bc_tot,tau_w_oc_tot,	tau_w_sd_tot,tau_w_sf_tot
   real(r8) tau_w_g_ss_tot,tau_w_g_bc_tot,tau_w_g_oc_tot,tau_w_g_sd_tot,tau_w_g_sf_tot
   real(r8) tau_w_f_ss_tot,tau_w_f_bc_tot,tau_w_f_oc_tot,tau_w_f_sd_tot,tau_w_f_sf_tot
   real(r8) tau_tot_new
   real(r8) tau_w_tot_new
   real(r8) tau_w_g_tot_new
   real(r8) tau_w_f_tot_new
   real(r8) w_tot_new
   real(r8) g_tot_new
   real(r8) f_tot_new

   integer iband
   integer i, k , kk, kaer

  real(r8) v_tau_ss(pcols,12)       ! optical depth, dust, by size category
  real(r8) v_tau_w_ss(pcols,12)     ! optical depth * single scattering albedo, dust, by size
  real(r8) v_tau_w_g_ss(pcols,12)   ! optical depth * single scattering albedo * asymmetry parameter, by size
  real(r8) v_tau_w_f_ss(pcols,12)   ! optical depth*forward scattering fraction*single scattering albedo,by size
  real(r8) v_tau_ss_tot(pcols)      ! optical depth, total dust
  real(r8) v_tau_w_ss_tot(pcols)    ! optical depth * single scattering albedo, total dust
  real(r8) v_tau_w_g_ss_tot(pcols)  ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) v_tau_w_f_ss_tot(pcols)  ! optical depth*forward scattering fraction*single scattering albedo,total

  real(r8) v_tau_sf(pcols,12)       ! optical depth, dust, by size category
  real(r8) v_tau_w_sf(pcols,12)     ! optical depth * single scattering albedo, dust, by size
  real(r8) v_tau_w_g_sf(pcols,12)   ! optical depth * single scattering albedo * asymmetry parameter,, by size
  real(r8) v_tau_w_f_sf(pcols,12)   ! optical depth*forward scattering fraction*single scattering albedo,by size
  real(r8) v_tau_sf_tot(pcols)      ! optical depth, total dust
  real(r8) v_tau_w_sf_tot(pcols)    ! optical depth * single scattering albedo, total dust
  real(r8) v_tau_w_g_sf_tot(pcols)  ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) v_tau_w_f_sf_tot(pcols)  ! optical depth*forward scattering fraction*single scattering albedo, total
  
  real(r8) v_tau_bc(pcols,12)       ! optical depth, dust, by size category
  real(r8) v_tau_w_bc(pcols,12)     ! optical depth * single scattering albedo, dust, by size
  real(r8) v_tau_w_g_bc(pcols,12)   ! optical depth * single scattering albedo * asymmetry parameter,, by size
  real(r8) v_tau_w_f_bc(pcols,12)   ! optical depth*forward scattering fraction*single scattering albedo,by size
  real(r8) v_tau_bc_tot(pcols)      ! optical depth, total dust
  real(r8) v_tau_w_bc_tot(pcols)    ! optical depth * single scattering albedo, total dust
  real(r8) v_tau_w_g_bc_tot(pcols)  ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) v_tau_w_f_bc_tot(pcols)  ! optical depth*forward scattering fraction*single scattering albedo, total

  real(r8) v_tau_oc(pcols,12)       ! optical depth, dust, by size category
  real(r8) v_tau_w_oc(pcols,12)     ! optical depth * single scattering albedo, dust, by size
  real(r8) v_tau_w_g_oc(pcols,12)   ! optical depth * single scattering albedo * asymmetry parameter, by size
  real(r8) v_tau_w_f_oc(pcols,12)   ! optical depth*forward scattering fraction*single scattering albedo,by size
  real(r8) v_tau_oc_tot(pcols)      ! optical depth, total dust
  real(r8) v_tau_w_oc_tot(pcols)    ! optical depth * single scattering albedo, total dust
  real(r8) v_tau_w_g_oc_tot(pcols)  ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) v_tau_w_f_oc_tot(pcols)  ! optical depth*forward scattering fraction*single scattering albedo, total

  real(r8) v_tau_sd(pcols,12)       ! optical depth, dust, by size category
  real(r8) v_tau_w_sd(pcols,12)     ! optical depth * single scattering albedo, dust, by size
  real(r8) v_tau_w_g_sd(pcols,12)   ! optical depth*single scattering albedo*asymmetry parameter,by size
  real(r8) v_tau_w_f_sd(pcols,12)   ! optical depth*forward scattering fraction*single scattering albedo,by size
  real(r8) v_tau_sd_tot(pcols)      ! optical depth, total dust
  real(r8) v_tau_w_sd_tot(pcols)    ! optical depth * single scattering albedo, total dust
  real(r8) v_tau_w_g_sd_tot(pcols)  ! optical depth * single scattering albedo * asymmetry parameter, total dust
  real(r8) v_tau_w_f_sd_tot(pcols)  ! optical depth*forward scattering fraction*single scattering albedo, total
!------------------------------------------------------------------


  ! aerosol optical property  by wangzhili
#include <aero_opt.h>
  ! 

  ! initialize aerosol diagnostic fields to 0.0 
  ! Average can be obtained by dividing <aerod>/<frc_day>

   aertau(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
   aerssa(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
   aerasm(1:ncol,1:nspint,1:naer_groups) = 0.0_r8
   aerfwd(1:ncol,1:nspint,1:naer_groups) = 0.0_r8

  !Rearange CAUCE aerosols
   call CmpDayNite(state%q,aer_tr, Nday, IdxDay, Nnite, IdxNite, 1, pcols, 1, pver, 1, ppcnst)

  do k=1,pverp
    do i=1,Nday
      pflx(i,k) = pint(i,k)
    end do
  end do
  do i=1,Nday
    pflx(i,0) = 0._r8
  enddo
  
    do k=1,pver
    !cdir nodep
      do i=1,Nday
         pdel      = pflx(i,k+1) - pflx(i,k)
         path      = pdel / gravit
         ubg(i,k) = aermmr(i,k,idxBG) * path 
         uvolc(i,k) =  aermmr(i,k,idxVOLC)

!new aerosol//wangzhili
         do iband=1,12
           u_ss(i,iband,k)=aer_tr(i,k,3+iband)*path
           u_bc(i,iband,k)=aer_tr(i,k,15+iband)*path
           u_oc(i,iband,k)=aer_tr(i,k,27+iband)*path
           u_sd(i,iband,k)=aer_tr(i,k,39+iband)*path
           u_sf(i,iband,k)=aer_tr(i,k,51+iband)*path
         enddo

! 
! End  do i=1,Nday
! 
      end do
! 
! End  k=1,pver
! 
   end do

!
! Compute tau_dst_tot, tau_w_dst_tot, tau_w_g_dst_tot, and, tau_w_f_dst_tot
!
      do i=1,Nday

         frc_day(i) = 1.0_r8
         do kaer = 1, naer_groups
            aertau(i,ns,kaer) = 0.0
            aerssa(i,ns,kaer) = 0.0
            aerasm(i,ns,kaer) = 0.0
            aerfwd(i,ns,kaer) = 0.0
         end do
! 
! End do i=1,Nday
! 
      end do

      f_volc   = gvolc(ns) * gvolc(ns)
      f_bg     = gbg(ns) * gbg(ns)

!------------------------------------------
!new aerosol//wangzhili
      bc_f(:)  = bc_g(:,ns)*bc_g(:,ns)
      sd_f(:)  = sd_g(:,ns)*sd_g(:,ns)
!------------------------------------------

!CSD$ PARALLEL DO PRIVATE( v_tau_dst_tot ) &
!CSD$ PRIVATE( v_tau_w_dst_tot, v_tau_w_g_dst_tot, v_tau_w_f_dst_tot, Tv_tau_dst, Tv_tau_w_dst, Tv_tau_w_g_dst ) &
!CSD$ PRIVATE( Tv_tau_w_f_dst, tmp1l, tmp2l, tmp3l, tmp1i, tmp2i, tmp3i, rhtrunc, krh, wrh, ksuli, ksslti ) &
!CSD$ PRIVATE( kcphili, wsuli, wsslti, wcphili, gsuli, gsslti, gcphili, tau_sul, tau_sslt, tau_cphil, tau_cphob ) &
!CSD$ PRIVATE( tau_cb, tau_volc, tau_bg, tau_w_sul, tau_w_sslt, tau_w_cphil, tau_w_cphob, tau_w_cb, tau_w_volc, tau_w_bg ) &
!CSD$ PRIVATE( tau_w_g_sul, tau_w_g_sslt, tau_w_g_cphil, tau_w_g_cphob, tau_w_g_cb, tau_w_g_volc, tau_w_g_bg, f_sul,f_sslt ) &
!CSD$ PRIVATE(  f_cphil, tau_w_f_sul, tau_w_f_bg, tau_w_f_sslt, tau_w_f_cphil, tau_w_f_cphob, tau_w_f_cb, tau_w_f_volc ) &
!CSD$ PRIVATE( tau_dst_tot, tau_w_dst_tot, tau_w_g_dst_tot, tau_w_f_dst_tot, w_dst_tot, g_dst_tot, f_dst_tot, tau_tot ) &
!CSD$ PRIVATE( tau_w_tot, tau_w_g_tot, tau_w_f_tot, w_tot, g_tot, f_tot, k, i, kk )


      do k=1,pver

!
! The following logicals are used to check to see whether we have invalid
! values for rhtrunc, g_tot, and f_tot.
! The above values are checked for every level of K and column of I.
! However, since the out of bounds conditions are *extremely* rare (in fact
! we abort the run if we find any one), let's reduce the number of 'if' tests
! by 1/K, and only check for aborting the run after scanning all Ks and Is.
!
         lrhtrunc_lt0(k) = .false.
         lg_tot_gt1(k)   = .false.
         lg_tot_ltm1(k)  = .false.
         lf_tot_gt1(k)   = .false.
         lf_tot_lt0(k)   = .false.

!-----------------------------------------------
!new aerosol//wangzhili
  		 v_tau_ss_tot(1:Nday)     = 0.0_r8
		 v_tau_w_ss_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_ss_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_ss_tot(1:Nday) = 0.0_r8

		 v_tau_bc_tot(1:Nday)     = 0.0_r8
		 v_tau_w_bc_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_bc_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_bc_tot(1:Nday) = 0.0_r8

		 v_tau_oc_tot(1:Nday)     = 0.0_r8
		 v_tau_w_oc_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_oc_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_oc_tot(1:Nday) = 0.0_r8

		 v_tau_sd_tot(1:Nday)     = 0.0_r8
		 v_tau_w_sd_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_sd_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_sd_tot(1:Nday) = 0.0_r8

		 v_tau_sf_tot(1:Nday)     = 0.0_r8
		 v_tau_w_sf_tot(1:Nday)   = 0.0_r8
		 v_tau_w_g_sf_tot(1:Nday) = 0.0_r8
		 v_tau_w_f_sf_tot(1:Nday) = 0.0_r8

!new aerosol//wangzhili
     do i=1,Nday
        if (rh(i,k) .LT. BRH(2)) then
               IRH = 1                                   ! RH  0-45%
        else if (rh(i,k) .GE. BRH(2) .and. rh(i,k) .LT. BRH(3)) then
               IRH = 2                                   !     45-50
        else if (rh(i,k) .GE. BRH(3) .and. rh(i,k) .LT. BRH(4)) then
               IRH = 3                                   !     50-60
        else if (rh(i,k) .GE. BRH(4) .and. rh(i,k) .LT. BRH(5)) then
               IRH = 4                                   !     60-70
        else if (rh(i,k) .GE. BRH(5) .and. rh(i,k) .LT. BRH(6)) then
               IRH = 5                                   !     70-80
        else if (rh(i,k) .GE. BRH(6) .and. rh(i,k) .LT. BRH(7)) then
               IRH = 6                                   !     80-90
        else if (rh(i,k) .GE. BRH(7) .and. rh(i,k) .LT. BRH(8)) THEN
               IRH = 7                                   !     90-95
        else if (rh(i,k) .GE. BRH(8) .and. rh(i,k) .LT. BRH(9)) then
               IRH = 8                                   !     95-98
        else if (rh(i,k) .GE. BRH(9) .and. rh(i,k) .LT. BRH(10))then
               IRH = 9                                   !     98-99
	else
	       IRH = 10                                  !      > 99
        endif
        do kk=1,12
         if (IRH .EQ. 10) then
    	    ss_ex(kk,ns) = qexss(IRH,kk,ns)
	    ss_w(kk,ns)  = wss(IRH,kk,ns)
	    ss_g(kk,ns)  = gss(IRH,kk,ns)
            oc_ex(kk,ns) = qexoc(IRH,kk,ns)
            oc_w(kk,ns)  = woc(IRH,kk,ns)
	    oc_g(kk,ns)  = goc(IRH,kk,ns)
	    sf_ex(kk,ns) = qexsf(IRH,kk,ns)
            sf_w(kk,ns)  = wsf(IRH,kk,ns)
	    sf_g(kk,ns)  = gsf(IRH,kk,ns)
          else
            ss_ex(kk,ns) = ((rh(i,k)-BRH(IRH))*qexss(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *qexss(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))
            ss_w(kk,ns)  = ((rh(i,k)-BRH(IRH))*wss(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *wss(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))				
 	    ss_g(kk,ns)  = ((rh(i,k)-BRH(IRH))*gss(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *gss(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))
            oc_ex(kk,ns) = ((rh(i,k)-BRH(IRH))*qexoc(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *qexoc(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))
            oc_w(kk,ns)  = ((rh(i,k)-BRH(IRH))*woc(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *woc(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))				
	    oc_g(kk,ns)  = ((rh(i,k)-BRH(IRH))*goc(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *goc(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))
	    sf_ex(kk,ns) = ((rh(i,k)-BRH(IRH))*qexsf(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *qexsf(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))
            sf_w(kk,ns)  = ((rh(i,k)-BRH(IRH))*wsf(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *wsf(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))				
	    sf_g(kk,ns)  = ((rh(i,k)-BRH(IRH))*gsf(IRH+1,kk,ns)+(BRH(IRH+1)-rh(i,k)) &
                            *gsf(IRH,kk,ns))/(BRH(IRH+1)-BRH(IRH))
	  end if

         ss_f(kk) = ss_g(kk,ns)*ss_g(kk,ns)
         oc_f(kk) = oc_g(kk,ns)*oc_g(kk,ns)
         sf_f(kk) = sf_g(kk,ns)*sf_g(kk,ns)

         tv_tau_ss     = 1.e4*u_ss(i,kk,k)*ss_ex(kk,ns)/(vv(kk)*2.17)
         tv_tau_w_ss   = tv_tau_ss*ss_w(kk,ns)
	 tv_tau_w_g_ss = tv_tau_ss*ss_w(kk,ns)*ss_g(kk,ns)
	 tv_tau_w_f_ss = tv_tau_ss*ss_w(kk,ns)*ss_f(kk)
		 
         tv_tau_bc     = 1.e4*u_bc(i,kk,k)*bc_ex(kk,ns)/(vv(kk)*1.5)
         tv_tau_w_bc   = tv_tau_bc*bc_w(kk,ns)
	 tv_tau_w_g_bc = tv_tau_bc*bc_w(kk,ns)*bc_g(kk,ns)
	 tv_tau_w_f_bc = tv_tau_bc*bc_w(kk,ns)*bc_f(kk)

         tv_tau_oc     = 1.e4*u_oc(i,kk,k)*oc_ex(kk,ns)/(vv(kk)*1.3)
	 tv_tau_w_oc   = tv_tau_oc*oc_w(kk,ns)
	 tv_tau_w_g_oc = tv_tau_oc*oc_w(kk,ns)*oc_g(kk,ns)
	 tv_tau_w_f_oc = tv_tau_oc*oc_w(kk,ns)*oc_f(kk)

         tv_tau_sd     = 1.e4*u_sd(i,kk,k)*sd_ex(kk,ns)/(vv(kk)*2.65)
	 tv_tau_w_sd   = tv_tau_sd*sd_w(kk,ns)
	 tv_tau_w_g_sd = tv_tau_sd*sd_w(kk,ns)*sd_g(kk,ns)
	 tv_tau_w_f_sd = tv_tau_sd*sd_w(kk,ns)*sd_f(kk)

         tv_tau_sf     = 1.e4*u_sf(i,kk,k)*sf_ex(kk,ns)/(vv(kk)*1.769)
	 tv_tau_w_sf   = tv_tau_sf*sf_w(kk,ns)
	 tv_tau_w_g_sf = tv_tau_sf*sf_w(kk,ns)*sf_g(kk,ns)
	 tv_tau_w_f_sf = tv_tau_sf*sf_w(kk,ns)*sf_f(kk)

	 v_tau_ss_tot(i)     = v_tau_ss_tot(i)+ tv_tau_ss
	 v_tau_w_ss_tot(i)   = v_tau_w_ss_tot(i)+ tv_tau_w_ss
	 v_tau_w_g_ss_tot(i) = v_tau_w_g_ss_tot(i)+ tv_tau_w_g_ss
	 v_tau_w_f_ss_tot(i) = v_tau_w_f_ss_tot(i)+ tv_tau_w_f_ss

	 v_tau_bc_tot(i)     = v_tau_bc_tot(i)+ tv_tau_bc
	 v_tau_w_bc_tot(i)   = v_tau_w_bc_tot(i)+ tv_tau_w_bc
	 v_tau_w_g_bc_tot(i) = v_tau_w_g_bc_tot(i)+ tv_tau_w_g_bc
	 v_tau_w_f_bc_tot(i) = v_tau_w_f_bc_tot(i)+ tv_tau_w_f_bc

	 v_tau_oc_tot(i)     = v_tau_oc_tot(i)+ tv_tau_oc
	 v_tau_w_oc_tot(i)   = v_tau_w_oc_tot(i)+ tv_tau_w_oc
	 v_tau_w_g_oc_tot(i) = v_tau_w_g_oc_tot(i)+ tv_tau_w_g_oc
	 v_tau_w_f_oc_tot(i) = v_tau_w_f_oc_tot(i)+ tv_tau_w_f_oc

	 v_tau_sd_tot(i)     = v_tau_sd_tot(i)+ tv_tau_sd
	 v_tau_w_sd_tot(i)   = v_tau_w_sd_tot(i)+ tv_tau_w_sd
	 v_tau_w_g_sd_tot(i) = v_tau_w_g_sd_tot(i)+ tv_tau_w_g_sd
	 v_tau_w_f_sd_tot(i) = v_tau_w_f_sd_tot(i)+ tv_tau_w_f_sd

         v_tau_sf_tot(i)     = v_tau_sf_tot(i)+ tv_tau_sf
	 v_tau_w_sf_tot(i)   = v_tau_w_sf_tot(i)+ tv_tau_w_sf
	 v_tau_w_g_sf_tot(i) = v_tau_w_g_sf_tot(i)+ tv_tau_w_g_sf
	 v_tau_w_f_sf_tot(i) = v_tau_w_f_sf_tot(i)+ tv_tau_w_f_sf
	enddo
      enddo
!--------------------------------------------------------------------------

         do i=1,Nday
! 
! Set aerosol properties
! Conversion factor to adjust aerosol extinction (m2/g)
! 
               tau_volc = 1.e3 * kvolc(ns) * uvolc(i,k)
               tau_bg = 1.e4 * kbg(ns) * ubg(i,k)

               tau_w_volc = tau_volc * wvolc(ns)
               tau_w_bg = tau_bg * wbg(ns)

               tau_w_g_volc = tau_w_volc * gvolc(ns)
               tau_w_g_bg = tau_w_bg * gbg(ns)

               tau_w_f_bg = tau_w_bg * f_bg
               tau_w_f_volc = tau_w_volc * f_volc
!
! mix dust aerosol size bins
!   w_dst_tot, g_dst_tot, w_dst_tot are currently not used anywhere
!   but calculate them anyway for future use
!
!new aerosol//wangzhili
         tau_ss_tot    =v_tau_ss_tot(i)
         tau_w_ss_tot  =v_tau_w_ss_tot(i)
         tau_w_g_ss_tot=v_tau_w_g_ss_tot(i)
         tau_w_f_ss_tot=v_tau_w_g_ss_tot(i)

         tau_bc_tot    =v_tau_bc_tot(i)
         tau_w_bc_tot  =v_tau_w_bc_tot(i)
         tau_w_g_bc_tot=v_tau_w_g_bc_tot(i)
         tau_w_f_bc_tot=v_tau_w_f_bc_tot(i)

         tau_oc_tot    =v_tau_oc_tot(i)
         tau_w_oc_tot  =v_tau_w_oc_tot(i)
         tau_w_g_oc_tot=v_tau_w_g_oc_tot(i)
         tau_w_f_oc_tot=v_tau_w_f_oc_tot(i)

         tau_sd_tot    =v_tau_sd_tot(i)
         tau_w_sd_tot  =v_tau_w_sd_tot(i)
         tau_w_g_sd_tot=v_tau_w_g_sd_tot(i)
         tau_w_f_sd_tot=v_tau_w_f_sd_tot(i)

         tau_sf_tot    =v_tau_sf_tot(i)
         tau_w_sf_tot  =v_tau_w_sf_tot(i)
         tau_w_g_sf_tot=v_tau_w_g_sf_tot(i)
         tau_w_f_sf_tot=v_tau_w_f_sf_tot(i)
!new  aersol//wangzhili
!--------------------------------------------------------------------------------
   tau_tot_new     = tau_sf_tot+tau_ss_tot+tau_bc_tot+tau_oc_tot+tau_sd_tot
   tau_tot_new     = tau_tot_new + tau_bg + tau_volc
   tau_w_tot_new   = tau_w_sf_tot+tau_w_ss_tot+tau_w_bc_tot+tau_w_oc_tot+tau_w_sd_tot
   tau_w_tot_new   = tau_w_tot_new + tau_w_bg + tau_w_volc
   tau_w_g_tot_new = tau_w_g_sf_tot+tau_w_g_ss_tot+tau_w_g_bc_tot+tau_w_g_oc_tot+tau_w_g_sd_tot
   tau_w_g_tot_new = tau_w_g_tot_new + tau_w_g_bg + tau_w_g_volc
   tau_w_f_tot_new = tau_w_f_sf_tot+tau_w_f_ss_tot+tau_w_f_bc_tot+tau_w_f_oc_tot+tau_w_f_sd_tot
   tau_w_f_tot_new = tau_w_f_tot_new + tau_w_f_bg + tau_w_f_volc
!--------------------------------------------------------------------------------
!new aerosol//wangzhili
!-----------------------------------------------------------------------
               if (tau_tot_new .gt. 0.0) then
                 w_tot_new = tau_w_tot_new / tau_tot_new
               else
                 w_tot_new = 0.0
               endif

               if (tau_w_tot_new .gt. 0.0) then
                 g_tot_new = tau_w_g_tot_new / tau_w_tot_new
                 f_tot_new = tau_w_f_tot_new / tau_w_tot_new
               else
                 g_tot_new = 0.0
                 f_tot_new = 0.0
               endif

               tauxar(i,k) = max(tau_tot_new,0.0)
               wa(i,k)     = min(w_tot_new, 0.999999_r8)
               ga(i,k)     = min(g_tot_new, 0.999999_r8)
               fa(i,k)     = min(f_tot_new, 0.999999_r8)
!---------------------------------wangzhili----------------------------------
               aertau(i,ns,1) = aertau(i,ns,1) + tau_sf_tot
               aertau(i,ns,2) = aertau(i,ns,2) + tau_ss_tot
               aertau(i,ns,3) = aertau(i,ns,3) + tau_oc_tot + tau_bc_tot
               aertau(i,ns,4) = aertau(i,ns,4) + tau_sd_tot
               aertau(i,ns,5) = aertau(i,ns,5) + tau_bg
               aertau(i,ns,6) = aertau(i,ns,6) + tau_volc
               aertau(i,ns,7) = aertau(i,ns,7) + tau_tot_new

               aerssa(i,ns,1) = aerssa(i,ns,1) + tau_w_sf_tot
               aerssa(i,ns,2) = aerssa(i,ns,2) + tau_w_ss_tot
               aerssa(i,ns,3) = aerssa(i,ns,3) + tau_w_oc_tot + tau_w_bc_tot
               aerssa(i,ns,4) = aerssa(i,ns,4) + tau_w_sd_tot
               aerssa(i,ns,5) = aerssa(i,ns,5) + tau_w_bg
               aerssa(i,ns,6) = aerssa(i,ns,6) + tau_w_volc
               aerssa(i,ns,7) = aerssa(i,ns,7) + tau_w_tot_new

               aerasm(i,ns,1) = aerasm(i,ns,1) + tau_w_g_sf_tot
               aerasm(i,ns,2) = aerasm(i,ns,2) + tau_w_g_ss_tot
               aerasm(i,ns,3) = aerasm(i,ns,3) + tau_w_g_oc_tot + tau_w_g_bc_tot
               aerasm(i,ns,4) = aerasm(i,ns,4) + tau_w_g_sd_tot
               aerasm(i,ns,5) = aerasm(i,ns,5) + tau_w_g_bg
               aerasm(i,ns,6) = aerasm(i,ns,6) + tau_w_g_volc
               aerasm(i,ns,7) = aerasm(i,ns,7) + tau_w_g_tot_new

               aerfwd(i,ns,1) = aerfwd(i,ns,1) + tau_w_f_sf_tot
               aerfwd(i,ns,2) = aerfwd(i,ns,2) + tau_w_f_ss_tot
               aerfwd(i,ns,3) = aerfwd(i,ns,3) + tau_w_f_oc_tot + tau_w_f_bc_tot
               aerfwd(i,ns,4) = aerfwd(i,ns,4) + tau_w_f_sd_tot
               aerfwd(i,ns,5) = aerfwd(i,ns,5) + tau_w_f_bg
               aerfwd(i,ns,6) = aerfwd(i,ns,6) + tau_w_f_volc
               aerfwd(i,ns,7) = aerfwd(i,ns,7) + tau_w_f_tot_new

! 
! End do i=1,Nday
! 
         end do
! 
! End do k=1,pver
! 
      end do
!CSD$ END PARALLEL 

      if (any( lrhtrunc_lt0(:) )) then
         write(6,*) "rhtrunc < 0.0"
         call endrun('RADCSWMX')
      end if
      if (any( lg_tot_gt1(:) )) then
         write(6,*) "g_tot > 1.0"
         call endrun('RADCSWMX')
      end if
      if (any( lg_tot_ltm1(:) )) then
         write(6,*) "g_tot < -1.0"
         call endrun('RADCSWMX')
      end if
      if (any( lf_tot_gt1(:) )) then
         write(6,*) "f_tot > 1.0"
         call endrun('RADCSWMX')
      end if
      if (any( lf_tot_lt0(:) )) then
         write(6,*) "f_tot < 0.0"
         call endrun('RADCSWMX')
      end if


      ! normalize aerosol optical diagnostic fields
      do kaer = 1, naer_groups

         do i=1,Nday


               if (aerssa(i,ns,kaer) .gt. 0.0) then   ! aerssa currently holds product of tau and ssa
                  aerasm(i,ns,kaer) = aerasm(i,ns,kaer) / aerssa(i,ns,kaer)
                  aerfwd(i,ns,kaer) = aerfwd(i,ns,kaer) / aerssa(i,ns,kaer)
               else
                  aerasm(i,ns,kaer) = 0.0_r8
                  aerfwd(i,ns,kaer) = 0.0_r8
               end if

               if (aertau(i,ns,kaer) .gt. 0.0) then
                  aerssa(i,ns,kaer) = aerssa(i,ns,kaer) / aertau(i,ns,kaer)
               else
                  aerssa(i,ns,kaer) = 0.0_r8
               end if

! 
! End do i=1,Nday
! 
         end do
!
! End do kaer = 1, naer_groups
!

      end do


 end subroutine aero_properties_bcc



end module aero_properties
