#include <misc.h>
#include <params.h>

module aerosol_intr

!---------------------------------------------------------------------------------
! Module to interface the aerosol parameterizations with CAM
! Phil Rasch, Jan 2003
!---------------------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  use ppgrid,       only: pcols, pver
  use physconst,    only: mwdry, mwh2o, gravit
  use constituents, only: ppcnst, cnst_add, cnst_name, advected
  use constituents, only: iaero, ixaer, ncnst_aer
  use abortutils,   only: endrun

  implicit none

  private          ! Make default type private to the module
  save
!
! Public interfaces
!
  public microp_aerosol_wet_intr                             ! interface to wet deposition
  public aerosol_register_cnst                        ! register consituents
  public aerosol_implements_cnst                      ! returns true if consituent is implemented by this package
  public aerosol_init_cnst                            ! initialize mixing ratios if not read from initial file
  public aerosol_defaultopts                          ! get default runtime options
  public aerosol_setopts                              ! set runtime options
  public prognostic_aerosol_initialize                ! initialize (history) variables
  public aerosol_drydep_intr                          ! interface to dry deposition
  public aerosol_wet_intr                             ! interface to wet deposition
  public aerosol_srcsnk_intr                          ! interface to aerosol transformations
  public aerosol_emis_intr                            ! interface to surface emissions
  public aerosol_time_interp                          ! interpolate oxidant and fluxes to current time
  public set_aerosol_from_prognostics
  public :: feedback_sulfur, feedback_carbon, feedback_bcarbon, feedback_soil_dust, &
            feedback_sea_salt, feedback_ocarbon
! Private data
!

  character(len=16), private, parameter :: def_prognostic_sulfur = 'off'  ! default
! Set this flag to .TRUE. to turn on sulfur
  logical, private :: sulfur = .FALSE.
! Set this flag to .TRUE. to turn on sulfur feedback in 
! set_aerosol_from_prognostics()
  logical :: feedback_sulfur = .FALSE.
! logical, private :: feedback_sulfur = .FALSE.

! Set this flag to .TRUE. to turn on carbon
  logical, private, parameter :: def_carbon = .FALSE.    ! default
  logical, private :: carbon = def_carbon
!--------------the following 6 lines, lqx 070817----------
! Set this flag to .TRUE. to turn on black carbon
  logical, private, parameter :: def_bcarbon = .FALSE.    ! default
  logical, private :: bcarbon = def_bcarbon

! Set this flag to .TRUE. to turn on organic carbon
  logical, private, parameter :: def_ocarbon = .FALSE.    ! default
  logical, private :: ocarbon = def_ocarbon
! Set this flag to .TRUE. to turn on carbon feedback in 
! set_aerosol_from_prognostics()
  logical, private, parameter :: def_feedback_carbon = .FALSE.  ! default
  logical :: feedback_carbon = def_feedback_carbon
! logical, private :: feedback_carbon = def_feedback_carbon

!--------------the following 8 lines, lqx 20070817---
! Set this flag to .TRUE. to turn on black carbon feedback in
! set_aerosol_from_prognostics()
  logical, private, parameter :: def_feedback_bcarbon = .FALSE.  ! default
  logical :: feedback_bcarbon = def_feedback_bcarbon
! logical, private :: feedback_bcarbon = def_feedback_bcarbon

! Set this flag to .TRUE. to turn on organic carbon feedback in
! set_aerosol_from_prognostics()
  logical, private, parameter :: def_feedback_ocarbon = .FALSE.  ! default
  logical :: feedback_ocarbon = def_feedback_ocarbon
! logical, private :: feedback_ocarbon = def_feedback_ocarbon
!---------lqx 20071009--------------------------------
  ! Set this flag to .TRUE. to turn on soil dust
  logical, private, parameter :: def_soil_dust = .FALSE.  ! default
  logical, private :: soil_dust = def_soil_dust
! Set this flag to .TRUE. to turn on soil dust feedback in
! set_aerosol_from_prognostics()
  logical, private, parameter :: def_feedback_soil_dust = .FALSE.  ! default
  logical :: feedback_soil_dust = def_feedback_soil_dust
! logical, private :: feedback_soil_dust = def_feedback_soil_dust

! Set this flag to .TRUE. to turn on sea salt
  logical, private, parameter :: def_sea_salt = .FALSE.  ! default
  logical, private :: sea_salt = def_sea_salt
! Set this flag to .TRUE. to turn on sea salt feedback in 
! set_aerosol_from_prognostics()
  logical, private, parameter :: def_feedback_sea_salt = .FALSE.  ! default
  logical :: feedback_sea_salt = def_feedback_sea_salt
! logical, private :: feedback_sea_salt = def_feedback_sea_salt

!  logical, private, parameter :: def_feedback_dust = .FALSE.  ! default
!  logical, private :: feedback_dust = def_feedback_dust

contains

!===============================================================================
  subroutine aerosol_register_cnst
!----------------------------------------------------------------------- 
! 
! Purpose: register aerosols
! 
! Method: 

! Author: P. J. Rasch
! 
!-----------------------------------------------------------------------
    use pmgrid,       only: masterproc
    use sulfur_intr,  only: sulfur_register_cnst, sulfur_n => ncnst
    use scyc,         only: scyc_idx1
    use carbon_intr,  only: carbon_register_cnst, carbon_idx1, carbon_n => ncnst
    use bcarbon_intr, only: bcarbon_register_cnst, bcyc_idx1, bcarbon_n => ncnst  !liuqx 070919
    use ocarbon_intr, only: ocarbon_register_cnst, ocyc_idx1, ocarbon_n => ncnst  !liuqx 070919
    use cuace_driver, only: cam_set_gem             ! wzz, 2008.9.9
    use dust_intr,    only: dust_register_cnst, dcyc_idx1, dust_n => ncnst
    use seasalt_intr, only: seasalt_register_cnst, ssyc_idx1, seaslt_n => ncnst

!---------------------------Local workspace-----------------------------
    integer :: m                                   ! tracer index
    integer :: ixaer0      ! the default value
!-----------------------------------------------------------------------

    ixaer0 = ixaer

    if ( iaero == 1 ) then
! -----get aerosol properties of cuace scheme -----------
       call cam_set_gem
    endif

    if ( carbon ) then
! Set names of carbon species and declare them as history variables
      call carbon_register_cnst
      if ( ixaer /= ixaer0 .and. carbon_n>0) then
         ixaer = carbon_idx1
      endif
      ncnst_aer = ncnst_aer + carbon_n
    endif

    if ( sea_salt ) then
! Set names of sea salt species and declare them as history variables
      call seasalt_register_cnst
      if ( ixaer /= ixaer0 .and. seaslt_n>0) then
         ixaer = ssyc_idx1()
      endif
      ncnst_aer = ncnst_aer + seaslt_n
    endif
!
!------the following 8 lines added by liuqx 20070905--------
    if ( bcarbon ) then
! Set names of carbon species and declare them as history variables
       call bcarbon_register_cnst
      if ( ixaer /= ixaer0 .and. bcarbon_n>0) then
         ixaer = bcyc_idx1()
      endif
      ncnst_aer = ncnst_aer + bcarbon_n
    endif

    if ( ocarbon ) then
! Set names of carbon species and declare them as history variables
       call ocarbon_register_cnst
      if ( ixaer /= ixaer0 .and. ocarbon_n>0) then
         ixaer = ocyc_idx1()
      endif
      ncnst_aer = ncnst_aer + ocarbon_n
    endif
!---------------------

    if ( soil_dust ) then
! Set names of dust species and declare them as history variables
      call dust_register_cnst
      if ( ixaer /= ixaer0 .and. dust_n>0) then
         ixaer = dcyc_idx1()
      endif
      ncnst_aer = ncnst_aer + dust_n
    endif

! Set names of sulfur species and declare them as history variables
    if ( sulfur ) then
      call sulfur_register_cnst
      if ( ixaer /= ixaer0 .and. sulfur_n>0) then
         ixaer = scyc_idx1()
      endif
      ncnst_aer = ncnst_aer + sulfur_n
    endif

  end subroutine aerosol_register_cnst


!=======================================================================
  function aerosol_implements_cnst(name)
!----------------------------------------------------------------------- 
! 
! Purpose: return true if specified constituent is implemented by this 
!          package
! 
! Author: T. Henderson
! 
!-----------------------------------------------------------------------
    use sulfur_intr,  only: sulfur_implements_cnst
    use carbon_intr,  only: carbon_implements_cnst
    use bcarbon_intr, only: bcarbon_implements_cnst
    use ocarbon_intr, only: ocarbon_implements_cnst
    use dust_intr,    only: dust_implements_cnst
    use seasalt_intr, only: seasalt_implements_cnst
!-----------------------------Arguments---------------------------------

     character(len=*), intent(in) :: name   ! constituent name
     logical :: aerosol_implements_cnst        ! return value
!---------------------------Local workspace-----------------------------
     integer :: m
!-----------------------------------------------------------------------

     aerosol_implements_cnst = .false.

     if ( sulfur ) then
       aerosol_implements_cnst = &
         (aerosol_implements_cnst.OR.sulfur_implements_cnst (name))
     endif

     if ( carbon ) then
       aerosol_implements_cnst = &
         (aerosol_implements_cnst.OR.carbon_implements_cnst (name))
     endif

     if ( bcarbon ) then
       aerosol_implements_cnst = &
         (aerosol_implements_cnst.OR.bcarbon_implements_cnst (name))
     endif

     if ( ocarbon ) then
       aerosol_implements_cnst = &
         (aerosol_implements_cnst.OR.ocarbon_implements_cnst (name))
     endif

     if ( soil_dust ) then
        aerosol_implements_cnst = &
          (aerosol_implements_cnst.OR.dust_implements_cnst (name))
     endif

     if ( sea_salt ) then
       aerosol_implements_cnst = &
         (aerosol_implements_cnst.OR.seasalt_implements_cnst (name))
     endif

  end function aerosol_implements_cnst


!=======================================================================
  subroutine aerosol_init_cnst(name, q)
!----------------------------------------------------------------------- 
! 
! Purpose:  
! Set initial mass mixing ratios.  
!
!-----------------------------------------------------------------------
    use pmgrid,       only: masterproc
    use sulfur_intr,  only: sulfur_implements_cnst, sulfur_init_cnst
    use carbon_intr,  only: carbon_implements_cnst, carbon_init_cnst
    use bcarbon_intr, only: bcarbon_implements_cnst, bcarbon_init_cnst
    use ocarbon_intr, only: ocarbon_implements_cnst, ocarbon_init_cnst
    use dust_intr,    only: dust_implements_cnst, dust_init_cnst
    use seasalt_intr, only: seasalt_implements_cnst, seasalt_init_cnst
!-----------------------------Arguments---------------------------------

    character(len=*), intent(in) :: name         ! constituent name
    real(r8), intent(out) :: q(:,:,:)            !  mass mixing ratio
!-----------------------------------------------------------------------

    if (masterproc) then           ! wzz, 2008.6.27
      if (sulfur_implements_cnst(name)) then
       write(6,*)'name=',name,sulfur
      endif
    endif
    if ( sulfur ) then
      if (sulfur_implements_cnst(name)) then
        call sulfur_init_cnst(name, q)
      endif
    endif

    if ( carbon ) then
      if (carbon_implements_cnst(name)) then
        call carbon_init_cnst(name, q)
      endif
    endif

    if ( bcarbon ) then
      if (bcarbon_implements_cnst(name)) then
        call bcarbon_init_cnst(name, q)
      endif
    endif

    if ( ocarbon ) then
      if (ocarbon_implements_cnst(name)) then
        call ocarbon_init_cnst(name, q)
      endif
    endif

    if ( soil_dust ) then
       if (dust_implements_cnst(name)) then
         call dust_init_cnst(name, q)
       endif
    endif

    if ( sea_salt ) then
      if (seasalt_implements_cnst(name)) then
        call seasalt_init_cnst(name, q)
      endif
    endif

  end subroutine aerosol_init_cnst



!=======================================================================
  subroutine aerosol_defaultopts(prognostic_sulfur_out,     &
                                 aero_carbon_out,           &
                                 aero_feedback_carbon_out,  &
                                 aero_bcarbon_out,          & !lqx 20071008
                                 aero_feedback_bcarbon_out, & !lqx
                                 aero_ocarbon_out,          & !lqx
                                 aero_feedback_ocarbon_out, & !lqx
                                 aero_soil_dust_out, &        !lqx
                                 aero_feedback_soil_dust_out, & !lqx
                                 aero_sea_salt_out,        &
                                 aero_feedback_sea_salt_out)
!----------------------------------------------------------------------- 
!
! Purpose: Return default runtime options
!
! Author: Tom Henderson
!
!-----------------------------------------------------------------------
!
! Arguments:
!
    ! prognostic aerosol sulfur option
    character(len=16), intent(out), optional :: prognostic_sulfur_out
    ! prognostic aerosol carbon option
    logical, intent(out), optional :: aero_carbon_out
    ! prognostic aerosol feedback option for carbon
    logical, intent(out), optional :: aero_feedback_carbon_out
!---------------liuqx 20081008-------------------
    ! prognostic aerosol black carbon option
    logical, intent(out), optional :: aero_bcarbon_out
    ! prognostic aerosol feedback option for black carbon
    logical, intent(out), optional :: aero_feedback_bcarbon_out
    ! prognostic aerosol organic carbon option
    logical, intent(out), optional :: aero_ocarbon_out
    ! prognostic aerosol feedback option for organic carbon
    logical, intent(out), optional :: aero_feedback_ocarbon_out
    ! prognostic aerosol soil dust option
    logical, intent(out), optional :: aero_soil_dust_out
    ! prognostic aerosol feedback option for soil dust
    logical, intent(out), optional :: aero_feedback_soil_dust_out
!-----------------------------------
    ! prognostic aerosol sea salt option
    logical, intent(out), optional :: aero_sea_salt_out
    ! prognostic aerosol feedback option for sea salt
    logical, intent(out), optional :: aero_feedback_sea_salt_out
!-----------------------------------------------------------------------
    if ( present(prognostic_sulfur_out) ) then
      prognostic_sulfur_out = def_prognostic_sulfur
    endif

    if ( present(aero_carbon_out) ) then
      aero_carbon_out = def_carbon
    endif
    if ( present(aero_feedback_carbon_out) ) then
      aero_feedback_carbon_out = def_feedback_carbon
    endif

!----------the following added by liuqx 20070930------------
    if ( present(aero_bcarbon_out) ) then
      aero_bcarbon_out = def_bcarbon
    endif
    if ( present(aero_feedback_bcarbon_out) ) then
      aero_feedback_bcarbon_out = def_feedback_bcarbon
    endif

    if ( present(aero_ocarbon_out) ) then
      aero_ocarbon_out = def_ocarbon
    endif
    if ( present(aero_feedback_ocarbon_out) ) then
      aero_feedback_ocarbon_out = def_feedback_ocarbon
    endif

   if ( present(aero_soil_dust_out) ) then
      aero_soil_dust_out = def_soil_dust
    endif
    if ( present(aero_feedback_soil_dust_out) ) then
      aero_feedback_soil_dust_out = def_feedback_soil_dust
    endif

!---------------------
    if ( present(aero_sea_salt_out) ) then
      aero_sea_salt_out = def_sea_salt
    endif
    if ( present(aero_feedback_sea_salt_out) ) then
      aero_feedback_sea_salt_out = def_feedback_sea_salt
    endif

  end subroutine aerosol_defaultopts



!=======================================================================
  subroutine aerosol_setopts(prognostic_sulfur_in,    &
                             aero_carbon_in,          &
                             aero_feedback_carbon_in, &
                             aero_bcarbon_in,          &  !lqx 20071009
                             aero_feedback_bcarbon_in, &  !
                             aero_ocarbon_in,          &  !
                             aero_feedback_ocarbon_in, &  !
                             aero_soil_dust_in,          &  !
                             aero_feedback_soil_dust_in, &  !
                             aero_sea_salt_in,        &
                             aero_feedback_sea_salt_in)
!----------------------------------------------------------------------- 
!
! Purpose: Set runtime options and print them
!
! Author: Tom Henderson
!
!-----------------------------------------------------------------------
    use pmgrid, only: masterproc
!-----------------------------------------------------------------------
!
! Arguments:
!
    ! prognostic aerosol sulfur option
    character(len=16), intent(in), optional :: prognostic_sulfur_in
    ! prognostic aerosol carbon option
    logical, intent(in), optional :: aero_carbon_in
    ! prognostic aerosol feedback option for carbon
    logical, intent(in), optional :: aero_feedback_carbon_in
!-----------liuqx 20071009--------
    ! prognostic aerosol black carbon option
    logical, intent(in), optional :: aero_bcarbon_in
    ! prognostic aerosol feedback option for black carbon
    logical, intent(in), optional :: aero_feedback_bcarbon_in
    ! prognostic aerosol organic carbon option
    logical, intent(in), optional :: aero_ocarbon_in
    ! prognostic aerosol feedback option for organic carbon
    logical, intent(in), optional :: aero_feedback_ocarbon_in
    ! prognostic aerosol soil dust
    logical, intent(in), optional :: aero_soil_dust_in
    ! prognostic aerosol feedback option for soil dust
    logical, intent(in), optional :: aero_feedback_soil_dust_in
!------------------------
    ! prognostic aerosol sea salt option
    logical, intent(in), optional :: aero_sea_salt_in
    ! prognostic aerosol feedback option for sea salt
    logical, intent(in), optional :: aero_feedback_sea_salt_in
!-----------------------------------------------------------------------
!
! Local variables:
!
    character(len=8) :: onoff
!-----------------------------------------------------------------------
    if ( present(prognostic_sulfur_in) ) then
      if ( prognostic_sulfur_in== "off") then
        sulfur = .FALSE.
        feedback_sulfur = .FALSE.
      elseif( prognostic_sulfur_in == "passive") then
        sulfur = .TRUE.
        feedback_sulfur = .FALSE.
      elseif( prognostic_sulfur_in== "direct") then
        sulfur = .TRUE.
        feedback_sulfur = .TRUE.
      else
        write(6,*) "prognostic_sulfur must be one of off, passive or direct"
        call endrun
      endif
    endif
!
    if ( present(aero_carbon_in) ) then
      carbon = aero_carbon_in
    endif
    if ( present(aero_feedback_carbon_in) ) then
      feedback_carbon = aero_feedback_carbon_in
    endif
!----------lqx 20071208----------------------------
    if ( iaero == 1 ) then
    if ( present(aero_bcarbon_in) ) then
      bcarbon = aero_bcarbon_in
    endif
    if ( present(aero_feedback_bcarbon_in) ) then
      feedback_bcarbon = aero_feedback_bcarbon_in
    endif

    if ( present(aero_ocarbon_in) ) then
      ocarbon = aero_ocarbon_in
    endif
    if ( present(aero_feedback_ocarbon_in) ) then
      feedback_ocarbon = aero_feedback_ocarbon_in
    endif
    endif     ! iaero == 1
!----------lqx 20071208------------------------------
    if ( present(aero_sea_salt_in) ) then
      sea_salt = aero_sea_salt_in
    endif
    if ( present(aero_feedback_sea_salt_in) ) then
      feedback_sea_salt = aero_feedback_sea_salt_in
    endif
    if ( present(aero_soil_dust_in) ) then
       soil_dust= aero_soil_dust_in
    endif
    if ( present(aero_feedback_soil_dust_in) ) then
      feedback_soil_dust = aero_feedback_soil_dust_in
    endif

    ! error check...
    if (feedback_carbon.and.(.not.carbon)) then
      write(6,*) 'ERROR:  AEROSOL_SETOPTS:  Feedback of prognostic'
      write(6,*) 'carbon aerosols is only allowed when these'
      write(6,*) 'prognostic aerosols are enabled.' 
      call endrun
    endif
    if (feedback_bcarbon.and.(.not.bcarbon)) then
      write(6,*) 'ERROR:  AEROSOL_SETOPTS:  Feedback of prognostic'
      write(6,*) 'bcarbon aerosols is only allowed when these'
      write(6,*) 'prognostic aerosols are enabled.' 
      call endrun
    endif
    if (feedback_ocarbon.and.(.not.ocarbon)) then
      write(6,*) 'ERROR:  AEROSOL_SETOPTS:  Feedback of prognostic'
      write(6,*) 'ocarbon aerosols is only allowed when these'
      write(6,*) 'prognostic aerosols are enabled.' 
      call endrun
    endif
    if (feedback_soil_dust.and.(.not.soil_dust)) then
      write(6,*) 'ERROR:  AEROSOL_SETOPTS:  Feedback of prognostic'
      write(6,*) 'soil_dust aerosols is only allowed when these'
      write(6,*) 'prognostic aerosols are enabled.' 
      call endrun
    endif
    if (feedback_sea_salt.and.(.not.sea_salt)) then
      write(6,*) 'ERROR:  AEROSOL_SETOPTS:  Feedback of prognostic'
      write(6,*) 'sea salt aerosols is only allowed when these'
      write(6,*) 'prognostic aerosols are enabled.' 
      call endrun
    endif
    if (masterproc) then
      ! TBH:  refactor this mess...  
      !----------------------------------
      if (sulfur) then
        onoff = 'on      '
      else
        onoff = 'off     '
      endif
      write(6,*) 'AEROSOL_SETOPTS:  prognostic sulfur aerosols are ', &
        onoff
      if (feedback_sulfur) then
        onoff = 'enabled '
      else
        onoff = 'disabled'
      endif
      write(6,*) 'AEROSOL_SETOPTS:  feedback of prognostic sulfur aerosols is ', &
        onoff
      !----------------------------------
      if (carbon) then
        onoff = 'on      '
      else
        onoff = 'off     '
      endif
      write(6,*) 'AEROSOL_SETOPTS:  prognostic carbon aerosols are ', &
        onoff
      if (feedback_carbon) then
        onoff = 'enabled '
      else
        onoff = 'disabled'
      endif
      write(6,*) 'AEROSOL_SETOPTS:  feedback of prognostic carbon aerosols is ', &
        onoff
!--------------liuqx 20071009-------------------
      if (bcarbon) then
        onoff = 'on      '
      else
        onoff = 'off     '
      endif
      write(6,*) 'AEROSOL_SETOPTS:  prognostic black carbon aerosols are ', &
        onoff
      if (feedback_bcarbon) then
        onoff = 'enabled '
      else
        onoff = 'disabled'
      endif
      write(6,*) 'AEROSOL_SETOPTS:  feedback of prognostic black carbon aerosols is ', &
        onoff
!--------------liuqx 20071009-------------------
      if (ocarbon) then
        onoff = 'on      '
      else
        onoff = 'off     '
      endif
      write(6,*) 'AEROSOL_SETOPTS:  prognostic organic carbon aerosols are ', &
        onoff
      if (feedback_ocarbon) then
        onoff = 'enabled '
      else
        onoff = 'disabled'
      endif
      write(6,*) 'AEROSOL_SETOPTS:  feedback of prognostic carbon aerosols is ', &
        onoff
!--------------liuqx 20071009-------------------
      if (soil_dust) then
        onoff = 'on      '
      else
        onoff = 'off     '
      endif
      write(6,*) 'AEROSOL_SETOPTS:  prognostic soil_dust aerosols are ', &
        onoff
      if (feedback_soil_dust) then
        onoff = 'enabled '
      else
        onoff = 'disabled'
      endif
      write(6,*) 'AEROSOL_SETOPTS:  feedback of prognostic soil_dust aerosols is ', &
        onoff

      !----------------------------------
      if (sea_salt) then
        onoff = 'on      '
      else
        onoff = 'off     '
      endif
      write(6,*) 'AEROSOL_SETOPTS:  prognostic sea salt aerosols are ', &
        onoff
      if (feedback_sea_salt) then
        onoff = 'enabled '
      else
        onoff = 'disabled'
      endif
      write(6,*) 'AEROSOL_SETOPTS:  feedback of prognostic sea salt aerosols is ', &
        onoff
    endif

  end subroutine aerosol_setopts



!===============================================================================
  subroutine prognostic_aerosol_initialize
!----------------------------------------------------------------------- 
! 
! Purpose: initialize aerosol parameterizations
!          (declare history variables)
! 
! Method: 
! call subroutines
! 
! Author: Phil Rasch
! 
!-----------------------------------------------------------------------
    use history,      only: addfld, add_default, phys_decomp
    use sulfur_intr,  only: sulfur_initialize
    use carbon_intr,  only: carbon_initialize
    use bcarbon_intr, only: bcarbon_initialize
    use ocarbon_intr, only: ocarbon_initialize
    use dust_intr,    only: dust_initialize
    use seasalt_intr, only: seasalt_initialize

    if ( sulfur ) then
      call sulfur_initialize
    endif

    if ( carbon ) then
      call carbon_initialize
    endif

    if ( bcarbon ) then
      call bcarbon_initialize
    endif

    if ( ocarbon ) then
      call ocarbon_initialize
    endif

!   if ( soil_dust ) then
       call dust_initialize
!   endif

    if ( sea_salt ) then
      call seasalt_initialize
    endif

    if ( iaero == 1 ) then
!      call addfld ('U10M','m/s     ',1, 'A',' 10m wind speed'  ,phys_decomp)
       call addfld ('SOILW','g/g     ',1, 'A','Soil moisture  '  ,phys_decomp)
    endif

  end subroutine prognostic_aerosol_initialize

!===============================================================================
  subroutine microp_aerosol_wet_intr (state, ptend, dt, pbuf, dlf, cflx, fracis)      ! wtw 2012.05
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to wet processing of all aerosols
! 
! Method: 
!  use a modified version of the scavenging parameterization described in
!     Barth et al, 2000, JGR (sulfur cycle paper)
!     Rasch et al, 2001, JGR (INDOEX paper)
! 
! Author: Phil Rasch
! 
!-----------------------------------------------------------------------
    use physics_types, only: physics_state, physics_ptend
    use phys_buffer,   only: pbuf_size_max, pbuf_fld, pbuf_old_tim_idx, pbuf_get_fld_idx

    use phys_grid,     only: get_lat_all_p, get_lon_all_p, get_rlat_all_p, get_rlon_all_p
    use dust_intr,     only: dust_wet_intr
    use sulfur_intr,   only: sulfur_wet_intr
    use carbon_intr,   only: carbon_wet_intr

    use wetdep,        only: microp_clddiag
    use time_manager,  only: get_curr_date, get_perp_date, get_curr_calday
    use time_manager,  only: is_perpetual, get_nstep
    use scyc,          only: scyc_idx1
!cam5   use progseasalts_intr,only: progseasalts_wet_intr
!cam5   use mz_aerosols_intr,       only:  mz_aero_wet_intr

#if ( defined MODAL_AERO )
    use modal_aero_calcsize,    only:  modal_aero_calcsize_sub
    use modal_aero_wateruptake, only:  modal_aero_wateruptake_sub
    use modal_aero_data,        only:  ntot_amode
#endif
!-----------------------------------------------------------------------
    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables

    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies
    type(pbuf_fld),      intent(inout) :: pbuf(pbuf_size_max)

    real(r8), intent(in) :: dlf(pcols,pver)            ! shallow+deep convective detrainment [kg/kg/s]


! local vars
    real(r8) :: calday        ! current calendar day
    integer :: yr, mon, day, ncsec
    integer :: ncdate
    real(r8):: rainmr(pcols,pver)  ! mixing ratio of rain within cloud volume
    real(r8):: cldv(pcols,pver)     ! cloudy volume undergoing wet chem and scavenging
    real(r8):: cldvcu(pcols,pver)   ! Convective precipitation area at the top interface of current layer
    real(r8):: cldvst(pcols,pver)   ! Stratiform precipitation area at the top interface of current layer 
    integer ix
    integer  lat(pcols)                  ! latitude indices
    real(r8) clat(pcols)                    ! latitudes
    integer  lon(pcols)                  ! longtitude indices
    real(r8) clon(pcols)                    ! longitudes
    integer  nstep
    real(r8) conicw(pcols,pver)          ! convective in-cloud water
    real(r8) cmfdqr(pcols,pver)          ! convective production of rain
    real(r8) cldc(pcols,pver)            ! convective cloud fraction, currently empty
    real(r8) clds(pcols,pver)            ! Stratiform cloud fraction
    real(r8) evapc(pcols,pver)           ! Evaporation rate of convective precipitation

#if ( defined MODAL_AERO )
    integer  i, k
#endif

!---
! wtw
!
real(r8), intent(inout)  :: cflx(pcols,ppcnst)     ! Surface constituent flux (kg/m^2/s)
real(r8), intent(inout)  :: fracis(pcols,pver,ppcnst)    ! fraction of transported species that are insoluble

integer  ::    cld_idx    
integer  ::    qme_idx    
integer  ::    prain_idx  
integer  ::    nevapr_idx 
integer  ::    icwmrdp_idx
integer  ::    rprddp_idx 
integer  ::    icwmrsh_idx
integer  ::    rprdsh_idx 
integer  ::    sh_frac_idx
integer  ::    dp_frac_idx
integer  ::    nevapr_shcu_idx
integer  ::    nevapr_dpcu_idx

!---
! physics buffer 
    integer itim
    real(r8), pointer, dimension(:,:) :: cldn       ! cloud fraction
    real(r8), pointer, dimension(:,:) :: cme
    real(r8), pointer, dimension(:,:) :: prain
    real(r8), pointer, dimension(:,:) :: evapr
    real(r8), pointer, dimension(:,:) ::  icwmrdp ! in cloud water mixing ratio, deep convection
    real(r8), pointer, dimension(:,:) ::  rprddp ! rain production, deep convection
    real(r8), pointer, dimension(:,:) ::  icwmrsh ! in cloud water mixing ratio, deep convection
    real(r8), pointer, dimension(:,:) ::  rprdsh ! rain production, deep convection
! Dec.29.2009. Sungsu
    real(r8), pointer, dimension(:,:) ::  sh_frac  ! Shallow convective cloud fraction
    real(r8), pointer, dimension(:,:) ::  dp_frac  ! Deep convective cloud fraction
    real(r8), pointer, dimension(:,:) ::  evapcsh  ! Evaporation rate of shallow convective precipitation >=0.
    real(r8), pointer, dimension(:,:) ::  evapcdp  ! Evaporation rate of deep    convective precipitation >=0.
! Dec.29.2009. Sungsu 
#if ( defined MODAL_AERO )
    real(r8), pointer, dimension(:,:,:) :: dgnum_pbuf, dgnumwet_pbuf, wetdens_pbuf
    real(r8), pointer, dimension(:,:,:) :: qaerwat  ! aerosol water
    real(r8), pointer, dimension(:,:) :: rate1ord_cw2pr_st   ! 1st order rate for direct conversion of
                                                    ! strat. cloud water to precip (1/s)    ! rce 2010/05/01
#endif


    integer ncol,lchnk

    nstep = get_nstep()

    calday = get_curr_calday()
    if ( is_perpetual() ) then
       call get_perp_date(yr, mon, day, ncsec)
    else
       call get_curr_date(yr, mon, day, ncsec)
    end if
    ncdate = yr*10000 + mon*100 + day

    ncol = state%ncol
    lchnk = state%lchnk

   call get_lat_all_p(lchnk, ncol, lat)
   call get_lon_all_p(lchnk, ncol, lon)
   call get_rlat_all_p(lchnk, ncol, clat)
   call get_rlon_all_p(lchnk, ncol, clon)

! Associate pointers with physics buffer fields
   itim = pbuf_old_tim_idx()

   cld_idx             = pbuf_get_fld_idx('CLD')
   cldn => pbuf(cld_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

   qme_idx             = pbuf_get_fld_idx('QME')    
   cme  => pbuf(qme_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   prain_idx           = pbuf_get_fld_idx('PRAIN')  
   prain  => pbuf(prain_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   nevapr_idx          = pbuf_get_fld_idx('NEVAPR') 
   evapr  => pbuf(nevapr_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   icwmrdp_idx         = pbuf_get_fld_idx('ICWMRDP') 
   icwmrdp  => pbuf(icwmrdp_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   rprddp_idx          = pbuf_get_fld_idx('RPRDDP')  
   rprddp  => pbuf(rprddp_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   icwmrsh_idx         = pbuf_get_fld_idx('ICWMRSH') 
   icwmrsh  => pbuf(icwmrsh_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   rprdsh_idx          = pbuf_get_fld_idx('RPRDSH')  
   rprdsh  => pbuf(rprdsh_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

! Dec.29.2009. Sungsu
   sh_frac_idx         = pbuf_get_fld_idx('SH_FRAC' )
   sh_frac => pbuf(sh_frac_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)

   dp_frac_idx         = pbuf_get_fld_idx('DP_FRAC') 
   dp_frac => pbuf(dp_frac_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)

   nevapr_shcu_idx     = pbuf_get_fld_idx('NEVAPR_SHCU') 
   evapcsh  => pbuf(nevapr_shcu_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   nevapr_dpcu_idx     = pbuf_get_fld_idx('NEVAPR_DPCU') 
   evapcdp  => pbuf(nevapr_dpcu_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

   cldc(:ncol,:)  = dp_frac(:ncol,:) + sh_frac(:ncol,:) ! Sungsu included this.
   evapc(:ncol,:) = evapcsh(:ncol,:) + evapcdp(:ncol,:) ! Sungsu included this.
   clds(:ncol,:)  = cldn(:ncol,:) - cldc(:ncol,:)       ! Stratiform cloud fraction


   ! sum deep and shallow convection contributions

!   if (cam_physpkg_is('cam5')) then
!      ! Dec.29.2009. Sungsu

   conicw(:ncol,:) = (icwmrdp(:ncol,:)*dp_frac(:ncol,:) + icwmrsh(:ncol,:)*sh_frac(:ncol,:))/ &
                        max(0.01_r8, sh_frac(:ncol,:) + dp_frac(:ncol,:))
!   else
!      conicw(:ncol,:) = icwmrdp(:ncol,:) + icwmrsh(:ncol,:)
!   end if

   cmfdqr(:ncol,:) = rprddp(:ncol,:)  + rprdsh(:ncol,:)

!   fields needed for wet scavenging

    call microp_clddiag( state%t, state%pmid, state%pdel, cmfdqr, evapc, cldn, cldc, clds, cme, evapr, prain, &
                  cldv, cldvcu, cldvst, rainmr, ncol )

    ptend%name = 'wetdep'

#if ( defined MODAL_AERO )
!    if ( associated(pbuf(dgnum_idx)%fld_ptr) ) then
!       dgnum_pbuf => pbuf(dgnum_idx)%fld_ptr( 1, 1:pcols, 1:pver, lchnk, 1:ntot_amode )
!    else
!       call endrun( 'pbuf for DGNUM not allocated in aerosol_wet_intr' )
!    end if
!    if ( associated(pbuf(dgnumwet_idx)%fld_ptr) ) then
!       dgnumwet_pbuf => pbuf(dgnumwet_idx)%fld_ptr( 1, 1:pcols, 1:pver, lchnk, 1:ntot_amode )
!    else
!       call endrun( 'pbuf for DGNUMWET not allocated in aerosol_wet_intr' )
!    end if
!    if ( associated(pbuf(wetdens_ap_idx)%fld_ptr) ) then
!       wetdens_pbuf => pbuf(wetdens_ap_idx)%fld_ptr( 1, 1:pcols, 1:pver, lchnk, 1:ntot_amode )
!    else
!       call endrun( 'pbuf for WETDENS_AP not allocated in aerosol_wet_intr' )
!    end if
!    if ( associated(pbuf(qaerwat_idx)%fld_ptr) ) then
!       qaerwat => pbuf(qaerwat_idx)%fld_ptr( 1, 1:pcols, 1:pver, lchnk, 1:ntot_amode )
!    else
!       call endrun( 'pbuf for QAERWAT not allocated in aerosol_wet_intr' )
!    end if
!    rate1ord_cw2pr_st => pbuf(rate1_cw2pr_st_idx)%fld_ptr( 1, 1:pcols, 1:pver, lchnk, 1 )  ! rce 2010/05/01
!
!    call t_startf('calcsize')
!
!    call modal_aero_calcsize_sub(                                &
!                       lchnk,   ncol,    nstep,                  &
!                       1,                0,                      &
!                       .true.,           dt,                     &
!                       state%t,          state%pmid,             &
!                       state%pdel,       state%q,                &
!                       ptend%q,          ptend%lq,               &
!                                         dgnum_pbuf              )
!
!    call t_stopf('calcsize')
!    call t_startf('wateruptake')
!
!    call modal_aero_wateruptake_sub(                             &
!                       lchnk,   ncol,    nstep,                  &
!                       1,                0,                      &
!                       .true.,           .true.,                 &
!                       dt,               state%q(:,:,1),         &
!                       state%t,          state%pmid,             &
!                       state%pdel, cldn, state%q,                &
!                       ptend%q,          ptend%lq,               &
!                       qaerwat,                                  &
!                       dgnum_pbuf,       dgnumwet_pbuf,          &
!                       wetdens_pbuf                              )
!
!    call t_stopf('wateruptake')
!    call mz_aero_wet_intr (state, ptend, nstep, dt, cme, prain, &
!            evapr, cldv, cldvcu, cldvst, cldc, cldn, fracis, calday, cmfdqr, evapc, conicw, rainmr, &
!            rate1ord_cw2pr_st, &   ! rce 2010/05/01
!            dgnumwet_pbuf, qaerwat, cam_out, dlf)
!
!
#else
!
!cam5    if ( dust ) then
!cam5       !   wet scavenging for dust
!cam5       call dust_wet_intr (state, ptend, nstep, dt, lat, clat, cme, prain, &
!cam5            evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr, cam_out)
!cam5    endif

!cam5    if ( progsslt ) then
!cam5       !   wet scavenging for prognostic sea salts
!cam5       call progseasalts_wet_intr (state, ptend, nstep, dt, lat, clat, cme, prain, &
!cam5            evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
!    endif

!   wet scavenging for mozart aerosols
! can not be done under trop_mozart chem_timestep_tend --
! this need to be done before deep convection so that fracis is set correctly
! for the mozart aerosols -- fracis is used in deep convection routine
!cam5    call mz_aero_wet_intr (state, ptend, nstep, dt, cme, prain, &
!cam5            evapr, cldv, cldvcu, cldvst, cldc, cldn, fracis, calday, cmfdqr, evapc, conicw, rainmr, cam_out, dlf)

!----------
! cam3
!
    if ( sulfur ) then
!     wet chemistry and scavenging of fields for sulfur cycle
      call sulfur_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, lon, cme, prain, &
         evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
    endif

    if ( carbon ) then
!      wet scavenging for carbon
       if ( iaero == 0 ) then
       call carbon_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
         evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
       endif
    endif

    if ( soil_dust ) then
!      wet scavenging for dust
       if ( iaero == 0 ) then
       call dust_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
          evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
       endif
    endif

#endif
    return

  end subroutine microp_aerosol_wet_intr

!===============================================================================
  subroutine aerosol_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, lon, cme, prain, &
       evapr, cldc, cldn, fracis, calday, cmfdqr, conicw)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to wet processing of all aerosols
! 
! Method: 
!  use a modified version of the scavenging parameterization described in
!     Barth et al, 2000, JGR (sulfur cycle paper)
!     Rasch et al, 2001, JGR (INDOEX paper)
! 
! Author: Phil Rasch
! 
!-----------------------------------------------------------------------
    use history,       only: outfld
    use physics_types, only: physics_state, physics_ptend
    use phys_grid,     only: get_lat_all_p
    use sulfur_intr,   only: sulfur_wet_intr
    use carbon_intr,   only: carbon_wet_intr
    use dust_intr,     only: dust_wet_intr
    use wetdep,        only: clddiag
    use time_manager,  only: get_curr_date, get_perp_date, get_curr_calday, &
                             is_perpetual
    use scyc,          only: scyc_idx1
!-----------------------------------------------------------------------
!
! Arguments:
!
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer,  intent(in) :: nstep
    integer,  intent(in) :: lat(pcols)                 ! latitude indices
    real(r8), intent(in) :: clat(pcols)                ! latitudes
    integer,  intent(in) :: lon(pcols)                 ! longtitude indices
    real(r8), intent(in) :: cme(pcols,pver)            ! local condensation of cloud water
    real(r8), intent(in) :: prain(pcols,pver)          ! production of rain
    real(r8), intent(in) :: evapr(pcols,pver)          ! evaporation of rain
    real(r8), intent(in) :: cldn(pcols,pver)           ! cloud fraction
    real(r8), intent(in) :: cldc(pcols,pver)           ! convective cloud fraction
    real(r8), intent(in) :: conicw(pcols,pver)         ! convective in-cloud water
    real(r8), intent(in) :: cmfdqr(pcols,pver)         ! convective production of rain

    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies
    real(r8), intent(inout)  :: cflx(pcols,ppcnst)     ! Surface constituent flux (kg/m^2/s)

    real(r8), intent(inout) :: fracis(pcols,pver,ppcnst)    ! fraction of transported species that are insoluble


! local vars
    real(r8) :: calday        ! current calendar day
    integer :: yr, mon, day, ncsec
    integer :: ncdate
    real(r8):: rainmr(pcols,pver)  ! mixing ratio of rain within cloud volume
    real(r8):: cldv(pcols,pver)     ! cloudy volume undergoing wet chem and scavenging
    integer ix
    integer m
    integer ncol

    calday = get_curr_calday()
    if ( is_perpetual() ) then
       call get_perp_date(yr, mon, day, ncsec)
    else
       call get_curr_date(yr, mon, day, ncsec)
    end if
    ncdate = yr*10000 + mon*100 + day

    ncol = state%ncol

!   fields needed for wet scavenging
    call clddiag( state%t, state%pmid, state%pdel, cmfdqr, cldn, cme, evapr, prain, &
         cldv, rainmr, ncol )

    if ( sulfur ) then
!   wet chemistry and scavenging of fields for sulfur cycle
      call sulfur_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, lon, cme, prain, &
         evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
    endif

    if ( carbon ) then
!   wet scavenging for carbon
       if ( iaero == 0 ) then
       call carbon_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
         evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
       endif
    endif

    if ( soil_dust ) then
!   wet scavenging for dust
       if ( iaero == 0 ) then
       call dust_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
          evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
       endif
    endif

  end subroutine aerosol_wet_intr

  subroutine aerosol_drydep_intr (state, ptend, wvflx, dt, &
       fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, landfrac, icefrac, &
       ocnfrac, fv,ram1)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to dry deposition parameterizions and sedimentation of all aerosols
! 
! Method: 
! Use prescribed dry deposition velocities for sulfur and carbon
! Use calculated dry dep velocities from CLM for dust
! 
! Author: Phil Rasch
! 
!-----------------------------------------------------------------------
    use history,       only: outfld
    use physics_types, only: physics_state, physics_ptend
    use phys_grid,     only: get_lat_all_p, get_rlat_all_p
    use sulfur_intr,   only: sulfur_drydep_intr
    use carbon_intr,   only: carbon_drydep_intr
    use dust_intr,     only: dust_drydep_intr
    use time_manager,  only: get_curr_date, get_perp_date, get_curr_calday, &
                             is_perpetual
    use scyc,          only: scyc_idx1
!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    real(r8), intent(in) :: fsds(pcols)                ! longwave down at sfc
    real(r8), intent(in) :: obklen(pcols)              ! longwave down at sfc
    real(r8), intent(in) :: ustar(pcols)               ! sfc fric vel
    real(r8), intent(in) :: ts(pcols)                  ! sfc temp
    real(r8), intent(in) :: landfrac(pcols)            ! land fraction
    real(r8), intent(in) :: icefrac(pcols)             ! ice fraction
    real(r8), intent(in) :: ocnfrac(pcols)             ! ocn fraction
    real(r8), intent(in) :: hflx(pcols)                ! sensible heat flux
    real(r8), intent(in) :: prect(pcols)               ! prect
    real(r8), intent(in) :: snowh(pcols)               ! snow depth
    real(r8), intent(in) :: pblh(pcols)                ! pbl height
    real(r8), intent(in) :: wvflx(pcols)               ! water vapor flux
    real(r8), intent(in) :: fv(pcols)                  !  friction velocity
    real(r8), intent(in) :: ram1(pcols)                ! aerodynamic resistance from land model

    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies

    integer  lat(pcols)                  ! latitude index for S->N storage
    real(r8) clat(pcols)                 ! latitude 
    integer lchnk
    integer ncol
    integer ix
    integer m

    integer :: yr, mon, day, ncsec
    integer :: ncdate

    real(r8) fvtmp(pcols)         ! temporary friction velocity
    real(r8) ram1tmp(pcols)       ! temporary aerodynamic resistance from land model

    if ( is_perpetual() ) then
       call get_perp_date(yr, mon, day, ncsec)
    else
       call get_curr_date(yr, mon, day, ncsec)
    end if

    lchnk = state%lchnk
    ncol = state%ncol
    
    call get_lat_all_p(lchnk, ncol, lat)
    call get_rlat_all_p(lchnk, ncol, clat)

!   note that tendencies are not only in sfc layer (because of sedimentation)
!   and that ptend is updated within each subroutine for different species

    if ( sulfur ) then
      call sulfur_drydep_intr (state, ptend, wvflx, dt, lat, clat, &
         fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, mon, landfrac, icefrac, ocnfrac)
    endif

    if ( carbon ) then
       if ( iaero == 0 ) then
      call carbon_drydep_intr (state, ptend, wvflx, dt, lat, clat, &
         fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, mon, landfrac, icefrac, ocnfrac)
       endif
    endif

!    fvtmp = 0.1
!    ram1tmp = 0.1
    if ( soil_dust ) then
       if ( iaero == 0 ) then
       call dust_drydep_intr (state, ptend, wvflx, dt, lat, clat, &
        fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, mon, landfrac, icefrac, ocnfrac,fv,ram1)
       endif
    endif

  end subroutine aerosol_drydep_intr

  subroutine aerosol_time_interp

!-----------------------------------------------------------------------
! Purpose: 
! interpolate the emission inventory to correct point in time
! 
! Method: 
! read in data from netcdf files for surrounding time periods
! do linear interpolation
! 
! Author: Phil Rasch
! 
!-----------------------------------------------------------------------

    use sulfur_intr, only: sulfur_time_interp
    use carbon_intr, only: carbon_time_interp
    use dust_intr,   only: dust_time_interp


    if ( sulfur ) then
       if ( iaero == 0 ) then
          call sulfur_time_interp
       endif
    endif

    if ( carbon ) then
      call carbon_time_interp
    endif

!lqx if ( soil_dust ) then
     if ( iaero == 1 ) then      ! liuqx 2011.4.19
         call dust_time_interp    ! wzz, 2008.3.11
    endif

  end subroutine aerosol_time_interp

  subroutine aerosol_emis_intr (state, ptend, cflx, dt)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! return surface fluxes of aerosol species and tendencies in surface layer
! due to surface emissions
! 
! Method: 
! 
! Author: Phil Rasch
! 
!-----------------------------------------------------------------------
    use history,       only: outfld
    use physics_types, only: physics_state, physics_ptend
    use phys_grid,     only: get_lon_all_p, get_lat_all_p, get_rlat_all_p
    use time_manager,  only: get_curr_date, get_perp_date, get_curr_calday, &
                             is_perpetual
    use sulfur_intr,   only: sulfur_emis_intr

    use carbon_intr,   only: carbon_emis_intr

    use dust_intr,     only: dust_emis_intr
!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables

    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies
    real(r8), intent(inout)  :: cflx(pcols,ppcnst)     ! Surface constituent flux (kg/m^2/s)

    integer  lat(pcols)                  ! latitude index 
    integer  lon(pcols)                  ! longitude index
    real(r8) clat(pcols)                 ! latitude 
    integer lchnk
    integer ncol
    integer i
    integer m
    real astmp(pcols,pver,3)
!
    real(r8) :: calday        ! current calendar day
    integer :: yr, mon, day, ncsec
    integer :: ncdate
    real(r8) :: so2sf(pcols), so4sf(pcols), dmssf(pcols)
    integer :: k

    if ( sulfur ) then
!   if ( iaero == 0 ) then   ! wzz, 2008.6.27
      call sulfur_emis_intr(state, ptend, dt)
!   endif
    endif

    if ( carbon ) then
    if ( iaero == 0 ) then   ! wzz, 2008.6.27
      call carbon_emis_intr(state, ptend, dt)
    endif
    endif

    if ( soil_dust ) then
    if ( iaero == 0 ) then   ! wzz, 2008.6.27
       call dust_emis_intr(state,ptend,dt,cflx)
    endif
    endif

  end subroutine aerosol_emis_intr 

  subroutine aerosol_srcsnk_intr (state, ptend, dt, ocnfrc)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! convert aerosols from one category to another by non-wet processes
! gas phase chemistry could go here also
! 

! Author: Phil Rasch
! 
!-----------------------------------------------------------------------
    use history,       only: outfld
    use physics_types, only: physics_state, physics_ptend
    use phys_grid,     only: get_lon_all_p, get_lat_all_p, get_rlat_all_p
    use caer,          only: caer_idx1, caercv
    use seasalt_intr,  only: seasalt_srcsnk
!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    real(r8),            intent(in)  :: ocnfrc(pcols)  !

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies

    integer i
    integer m

    if ( carbon ) then
      i = caer_idx1()-1
      ! conversion from hydrophobic to hydrophilic form
      ptend%lq(i+1:i+4) = .true.
      do m = 1,2
         call caercv(state%ncol, state%q(:,:,i+m), dt, ptend%q(:,:,i+m), ptend%q(:,:,i+m+2))
      end do
    endif

    if ( sea_salt ) then
      call seasalt_srcsnk(state, dt, state%u(:,pver), ocnfrc, ptend)
    endif

    return
  end subroutine aerosol_srcsnk_intr 

  subroutine set_aerosol_from_prognostics(state, aeromix, scale)

!----------------------------------------------------------------------- 
    use prescribed_aerosols, only: naer_all, idxSUL, idxSSLT, idxOCPHO, idxBCPHO, idxOCPHI, idxBCPHI
    use prescribed_aerosols, only: idxDUSTfirst, numDUST,  idxBG
    use constituents,        only: ppcnst, cnst_name
    use physics_types,       only: physics_state

    type(physics_state), intent(in) :: state                            ! Physics state variables
    real(r8), optional,  intent(in) :: scale(naer_all)    ! scale each aerosol by this amount, wzz, 2008.5.27

    real(r8), intent(inout)          :: aeromix(pcols, pver, naer_all)   ! aerosol mass mixing ratios
!
! Local workspace
!
    integer m
    integer ncol

    ncol = state%ncol

! TBH:  Commented out the following line at request of Andrew Conley.  9/17/03
!    aeromix(:ncol,:,idxBG) = 0 ! background aerosol to zero
    do m = 1,ppcnst
!       write (6,*) '  set_aerosol_from_prognostics >', trim(cnst_name(m)),'<'
       if (feedback_sulfur) then    
          if (trim(cnst_name(m)) == 'SO4') then
!          write (6,*) ' sulfate found '
             aeromix(:ncol,:,idxSUL) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxSUL) = aeromix(:ncol,:,idxSUL)*scale(idxSUL)
             endif
          endif
       endif
       if (feedback_carbon) then    
          if ( iaero == 0 ) then
          if (trim(cnst_name(m)) == 'BCPHO') then
!          write (6,*) ' BCPHO found '
             aeromix(:ncol,:,idxBCPHO) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxBCPHO) = aeromix(:ncol,:,idxBCPHO)*scale(idxBCPHO)
             endif
          else if (trim(cnst_name(m)) == 'BCPHI') then
!          write (6,*) ' BCPHI found '
             aeromix(:ncol,:,idxBCPHI) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxBCPHI) = aeromix(:ncol,:,idxBCPHI)*scale(idxBCPHI)
             endif
          else if (trim(cnst_name(m)) == 'OCPHO') then
!          write (6,*) ' OCPHO found '
             aeromix(:ncol,:,idxOCPHO) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxOCPHO) = aeromix(:ncol,:,idxOCPHO)*scale(idxOCPHO)
             endif
          else if (trim(cnst_name(m)) == 'OCPHI') then
!          write (6,*) ' OCPHI found '
             aeromix(:ncol,:,idxOCPHI) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxOCPHI) = aeromix(:ncol,:,idxOCPHI)*scale(idxOCPHI)
             endif
          endif
          endif
       endif
       if (feedback_sea_salt) then    
          if ( iaero == 0 ) then
          if (trim(cnst_name(m)) == 'SSLT') then
!          write (6,*) ' SSLT found '
             aeromix(:ncol,:,idxSSLT) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxSSLT) = aeromix(:ncol,:,idxSSLT)*scale(idxSSLT)
             endif
          endif
          endif
       endif
!       if (feedback_dust) then    
       if (feedback_soil_dust) then    
          if ( iaero == 0 ) then
          if (trim(cnst_name(m)) == 'DSTX01') then
!          write (6,*) ' DSTX01 found '
             aeromix(:ncol,:,idxDUSTfirst) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst) = aeromix(:ncol,:,idxDUSTfirst)*scale(idxDUSTfirst)
             endif
          else if (trim(cnst_name(m)) == 'DSTX02') then
!          write (6,*) ' DSTX02 found '
             aeromix(:ncol,:,idxDUSTfirst+1) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst+1) = aeromix(:ncol,:,idxDUSTfirst+1)*scale(idxDUSTfirst+1)
             endif
          else if (trim(cnst_name(m)) == 'DSTX03') then
!          write (6,*) ' DSTX03 found '
             aeromix(:ncol,:,idxDUSTfirst+2) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst+2) = aeromix(:ncol,:,idxDUSTfirst+2)*scale(idxDUSTfirst+2)
             endif
          else if (trim(cnst_name(m)) == 'DSTX04') then
!          write (6,*) ' DSTX04 found '
             aeromix(:ncol,:,idxDUSTfirst+3) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst+3) = aeromix(:ncol,:,idxDUSTfirst+3)*scale(idxDUSTfirst+3)
             endif
          endif
          else if ( iaero == 1 ) then
          if (trim(cnst_name(m)) == 'SD01') then
!          write (6,*) ' SD1 found '
             aeromix(:ncol,:,idxDUSTfirst) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst) = aeromix(:ncol,:,idxDUSTfirst)*scale(idxDUSTfirst)
             endif
          else if (trim(cnst_name(m)) == 'SD02') then
!          write (6,*) ' SD2 found '
             aeromix(:ncol,:,idxDUSTfirst+1) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst+1) = aeromix(:ncol,:,idxDUSTfirst+1)*scale(idxDUSTfirst+1)
             endif
          else if (trim(cnst_name(m)) == 'SD03') then
!          write (6,*) ' SD3 found '
             aeromix(:ncol,:,idxDUSTfirst+2) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst+2) = aeromix(:ncol,:,idxDUSTfirst+2)*scale(idxDUSTfirst+2)
             endif
          else if (trim(cnst_name(m)) == 'SD04') then
!          write (6,*) ' SD4 found '
             aeromix(:ncol,:,idxDUSTfirst+3) = state%q(:ncol,:,m)
             if (present(scale)) then
                aeromix(:ncol,:,idxDUSTfirst+3) = aeromix(:ncol,:,idxDUSTfirst+3)*scale(idxDUSTfirst+3)
             endif
          endif
          endif
       endif
    end do

  end subroutine set_aerosol_from_prognostics

end module aerosol_intr
