#include <misc.h>
#include <params.h>

module buffer

!----------------------------------------------------------------------- 
! 
! Purpose: 
!   Definition and initialization of time-cycling physics arrays
!
! Author: 
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  use constituents, only: pcnst, pnats
  use ppgrid, only: pcols, pver, pverp, begchunk, endchunk
  use prognostics, only: ptimelevels

  real(r8), allocatable :: qrs(:,:,:)      ! shortwave radiative heating rate 
  real(r8), allocatable :: qrl(:,:,:)      ! longwave  radiative heating rate 
#ifdef MOZART2 
  real(r8), allocatable :: chemflx(:,:,:)  
  real(r8), allocatable :: dq_chem(:,:,:,:)          ! tracer tendency due to chemistry process
#endif

#ifdef NSTEP_MOIST
  real(r8), allocatable :: dq_moist(:,:,:,:)         ! tracer tendency due to moist process
  real(r8), allocatable :: dt_moist(:,:,:)           ! temperture tendency due to moist process
  real(r8), allocatable :: du_moist(:,:,:)           ! u tendency due to moist process
  real(r8), allocatable :: dv_moist(:,:,:)           ! v tendency due to moist process
  real(r8), allocatable :: cnt(:,:)  
  real(r8), allocatable :: cnb(:,:)  
  real(r8), allocatable :: cmfdqr(:,:,:)  
  real(r8), allocatable :: nevapr(:,:,:)  
  real(r8), allocatable :: prain(:,:,:)  
  real(r8), allocatable :: cmfmc(:,:,:)  
  real(r8), allocatable :: q0_conv(:,:,:)  
  real(r8), allocatable :: t0_conv(:,:,:) 
  real(r8), allocatable :: date0_conv(:,:) 
#endif

  real(r8), allocatable :: pblht(:,:)      ! PBL height
  real(r8), allocatable :: tpert(:,:)      ! temperature perturbation (PBL)
  real(r8), allocatable :: qpert(:,:,:)    ! moisture/constituent perturb.(PBL)
  real(r8), allocatable :: kbfs(:,:)      ! surface buoyancy flux

#ifdef UWMT
  real(r8), allocatable :: kvm(:,:,:)
  real(r8), allocatable :: kvh(:,:,:)
#endif

#ifdef NSTEP_GW
  real(r8), allocatable :: utgw(:,:,:)
  real(r8), allocatable :: vtgw(:,:,:)
  real(r8), allocatable :: stgw(:,:,:)
  real(r8), allocatable :: qtgw(:,:,:,:)
#endif

CONTAINS

  subroutine initialize_buffer
!
! Allocate memory
!
#ifdef MOZART2
    allocate (dq_chem(pcols,pver,pcnst+pnats,begchunk:endchunk))
    allocate (chemflx(pcols,pcnst+pnats,begchunk:endchunk))
#endif
#ifdef NSTEP_MOIST
    allocate (dq_moist(pcols,pver,pcnst+pnats,begchunk:endchunk))     
    allocate (dt_moist(pcols,pver,begchunk:endchunk))
    allocate (du_moist(pcols,pver,begchunk:endchunk))
    allocate (dv_moist(pcols,pver,begchunk:endchunk))
    allocate (cnt(pcols,begchunk:endchunk))
    allocate (cnb(pcols,begchunk:endchunk))
    allocate (cmfdqr(pcols,pver,begchunk:endchunk))
    allocate (nevapr(pcols,pver,begchunk:endchunk))
    allocate (prain(pcols,pver,begchunk:endchunk))
    allocate (cmfmc(pcols,pverp,begchunk:endchunk))
    allocate (q0_conv(pcols,pver,begchunk:endchunk))
    allocate (t0_conv(pcols,pver,begchunk:endchunk))
    allocate (date0_conv(pcols,begchunk:endchunk))
#endif
    allocate (qrs   (pcols,pver,begchunk:endchunk))     
    allocate (qrl   (pcols,pver,begchunk:endchunk))     
    allocate (pblht (pcols,begchunk:endchunk))       
    allocate (tpert (pcols,begchunk:endchunk))       
    allocate (qpert (pcols,pcnst+pnats,begchunk:endchunk)) 
    allocate (kbfs  (pcols,begchunk:endchunk))
#ifdef UWMT
    allocate (kvm   (pcols,pver+1,begchunk:endchunk))
    allocate (kvh   (pcols,pver+1,begchunk:endchunk))
#endif
#ifdef NSTEP_GW
    allocate (utgw(pcols,pver,begchunk:endchunk))
    allocate (vtgw(pcols,pver,begchunk:endchunk))
    allocate (stgw(pcols,pver,begchunk:endchunk))
    allocate (qtgw(pcols,pver,pcnst+pnats,begchunk:endchunk))
#endif
!
! Initialize to NaN or Inf
!
#ifdef MOZART2
    chemflx(:,:,:) = 0.0_r8
    dq_chem(:,:,:,:) = 0.0_r8
#endif
#ifdef NSTEP_MOIST
    dq_moist(:,:,:,:) = 0.0_r8
    dt_moist(:,:,:)   = 0.0_r8
    du_moist(:,:,:)   = 0.0_r8
    dv_moist(:,:,:)   = 0.0_r8
    cnt(:,:)   = 0.0_r8
    cnb(:,:)   = 0.0_r8
    cmfdqr(:,:,:) = 0.0_r8
    nevapr(:,:,:) = 0.0_r8
    prain(:,:,:) = 0.0_r8
    cmfmc(:,:,:) = 0.0_r8
    q0_conv(:,:,:) = 0.0_r8
    t0_conv(:,:,:) = 0.0_r8
#endif
    qrs   (:,:,:) = 0.0_r8
    qrl   (:,:,:) = 0.0_r8
    pblht (:,:) = 0.0_r8
    tpert (:,:) = 0.0_r8
    qpert (:,:,:) = 0.0_r8
    kbfs (:,:) = 0.0_r8
#ifdef UWMT
    kvm   (:,:,:) = 0.
    kvh   (:,:,:) = 0.
#endif
#ifdef NSTEP_GW
    utgw(:,:,:)   = 0.0
    vtgw(:,:,:)   = 0.0
    stgw(:,:,:)   = 0.0
    qtgw(:,:,:,:) = 0.0
#endif

    return
  end subroutine initialize_buffer

end module buffer
