#include <misc.h>
#include <params.h>
module cloud_fraction
  !
  ! Module for cloud fraction routines.
  !
  ! $Id: cloud_fraction.F90,v 1.11 2010/07/30 06:48:40 wangzzh Exp $
  !
  use shr_kind_mod, only: r8 => shr_kind_r8
  use pmgrid,       only: masterproc
  use ppgrid,        only:  pver

  implicit none

  private
  save
  !
  ! Public interfaces
#ifdef MICROP-MG              
  public microp_cldfrc_init   !
  public microp_cldfrc        ! Computation of cloud fraction
  public cldfrc_init          ! Inititialization of cloud_fraction run-time parameters
  public cldfrc               ! Computation of cloud fraction
  
  ! Private data
  real(r8), parameter :: unset_r8 = huge(1.0_r8)
  
  ! Physics buffer indices 
  integer :: sh_frac_idx   = 0            
  integer :: dp_frac_idx   = 0           

  ! Namelist variables           
  logical  :: cldfrc_freeze_dry           ! switch for Vavrus correction
  logical  :: cldfrc_ice                  ! switch to compute ice cloud fraction
  real(r8), public :: rhminl             ! set from namelist input cldfrc_rhminl
  real(r8), public :: rhminh             ! set from namelist input cldfrc_rhminh 
  real(r8) :: sh1, sh2           ! set from namelist input cldfrc_sh1, cldfrc_sh2 
  real(r8) :: dp1,dp2            ! set from namelist input cldfrc_dp1, cldfrc_dp2 
  real(r8), public :: premit             ! set from namelist input cldfrc_premit
  real(r8), parameter :: pnot = 1.e5_r8         ! reference pressure
  real(r8), parameter :: lapse = 6.5e-3_r8      ! U.S. Standard Atmsophere lapse rate
  real(r8), parameter :: premib_uw  = 750.e2_r8 ! bottom pressure bound of middle cloud for UW
  real(r8), parameter :: premib_cam = 750.e2_r8 ! bottom pressure bound of middle cloud for CAM
  real(r8), public    :: premib                 ! bottom pressure bound of middle cloud
  real(r8), parameter :: pretop = 1.0e2_r8      ! pressure bounding high cloud

  integer :: iceopt = 2                       ! option for ice cloud closure
                                              ! 1=wang & sassen 2=schiller (iciwc)
                                              ! 3=wood & field, 4=Wilson (based on smith)
  real(r8), parameter :: icecrit = 0.95_r8    ! Critical RH for ice clouds in Wilson & Ballard closure (smaller = more ice clouds)

  integer count
  character(len=16) :: microp_scheme

  logical :: inversion_cld_off    ! Turns off stratification-based cld frc

#else
  !
  public cldfrc_init ! Inititialization of cloud_fraction run-time parameters
  public cldfrc   !  Computation of cloud fraction
  !
  ! Private data
  !
  real(r8) rhminl                ! minimum rh for low stable clouds
  real(r8) rhminh                ! minimum rh for high stable clouds
  real(r8) sh1,sh2               ! parameters for shallow convection cloud fraction
  real(r8) dp1,dp2               ! parameters for deep convection cloud fraction
  real(r8) premit                ! top pressure bound for mid level cloud
  real(r8) premib
  !
#endif

  public strat_EIS

contains  

!=================================================================
  subroutine cldfrc_init()
    !
    ! Purpose:
    ! Initialize cloud fraction run-time parameters
    !
    ! Author: J. McCaa
    !    
    use dycore, only: dycore_is, get_resolution
!
    logical :: exists
    character(len=16) :: varname
    real :: varqi, varke, varp
    integer ioerr
!
! read the tunable parameters from file 'parameters', wzz, 2007.5.23
   varqi = -1.
   varke = -1.
   inquire(file='parameters',exist=exists)
   if (exists) then
      open(10,file='parameters',form='formatted')
      do
         read(10,fmt="(a16,4x,e14.7)",iostat=ioerr) varname, varp
         if (ioerr < 0 ) exit
         if (ioerr /= 0 ) cycle
         if (trim(varname).eq.'RHlow_Value') then
            varqi = varp
         endif
         if (trim(varname).eq.'RHhigh_Value') then
            varke = varp
         endif
      enddo
      close(10)
   endif
   write(6,*)'wzz: varlow =', varqi, varke
!
    if ( dycore_is ('LR') ) then
       if ( get_resolution() == '1x1.25' ) then
          rhminl = .88
          rhminh = .77
          sh1 = 0.04
          sh2 = 500.0
          dp1 = 0.10
          dp2 = 500.0
          premit = 750.e2  ! top of area defined to be mid-level cloud
       else
          rhminl = .90
          rhminh = .80
          sh1 = 0.04
          sh2 = 500.0
          dp1 = 0.10
          dp2 = 500.0
          premit = 750.e2  ! top of area defined to be mid-level cloud
       endif
    else
       if ( get_resolution() == 'T106' ) then   ! wzz, 2007.04.18

#ifdef COUP_CSM
          rhminl = 0.945   ! 0.945
          rhminh = 0.87   ! 0.87
#else
          rhminl = 0.945   ! 0.945
          rhminh = 0.87   ! 0.87
#endif
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 400.e2  ! top of area defined to be mid-level cloud
       elseif ( get_resolution() == 'T266' ) then   ! wzz, 2010.08.06
          if( pver > 26) then
#ifdef COUP_CSM
              rhminl = 0.94
              rhminh = 0.87
#else
              rhminl = 0.94
              rhminh = 0.87
#endif

          else
#ifdef COUP_CSM
              rhminl = 0.935   
              rhminh = 0.87   ! 0.87
#else
              rhminl = 0.935
              rhminh = 0.87   ! 0.87
#endif
          endif
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 400.e2  ! 400.e2  ! top of area defined to be mid-level cloud
       elseif ( get_resolution() == 'T213' ) then   ! wzz, 2010.01.06
          rhminl = .91
          rhminh = .70
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 400.e2  ! top of area defined to be mid-level cloud
       elseif ( get_resolution() == 'T85' ) then
          rhminl = .91
          rhminh = .70
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 250.e2  ! top of area defined to be mid-level cloud
       elseif ( get_resolution() == 'T63' ) then   ! wzz, 2007.04.18
       !   rhminl = .91
       !   rhminh = .70
          rhminl = .90  ! wzz, 2007.05.04
          rhminh = .80      !  wzz, 2007.04.28
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 400.e2  ! top of area defined to be mid-level cloud
       elseif ( get_resolution() == 'T31' ) then
          rhminl = .88
          rhminh = .80
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 750.e2  ! top of area defined to be mid-level cloud
       else
#ifdef MOZART2
#ifdef MG08
          rhminl = 0.83
          rhminh = 0.78
#else     
          rhminl = 0.92
          rhminh = 0.87
#endif    
!-----------------
#else  
#ifdef MG08
          rhminl = 0.83   
          rhminh = 0.78     
#else     
#ifdef COUP_CSM
          rhminl = 0.92
          rhminh = 0.87
#else
          rhminl = 0.92
          rhminh = 0.87
#endif
#endif    
!----------------- 
#endif
          sh1 = 0.07
          sh2 = 500.0
          dp1 = 0.14
          dp2 = 500.0
          premit = 400.e2
       endif
    endif
          if ( varqi > 0) rhminl = varqi  ! wzz, 2007.5.24
          if ( varke > 0) rhminh = varke  ! wzz, 2007.5.24

  end subroutine cldfrc_init

  subroutine cldfrc(lchnk   ,ncol    , tdeepcld, qdeepcld, deepcld,  tshallowcld, &
       qshallowcld, shallowcld , &
       pmid    ,pint,   ztodt,  dlf,  temp    ,q       ,qi      ,ql      ,omga    , phis, &
       cldtop  ,cldbot  ,lcl     ,cloud   ,clc     ,pdel    , &
       cmfmc   ,cmfmc2  ,landfrac,snowh   ,concld  ,cldst   , &
       cld_env ,t_env   ,q_env     , &
       ts      ,sst     ,ps      ,zdu     ,ocnfrac ,&
       rhu00   ,relhum  ,dindex, zm,   cstcld )
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    ! Compute cloud fraction 
    ! 
    ! 
    ! Method: 
    ! This calculate cloud fraction using a relative humidity threshold
    ! The threshold depends upon pressure, and upon the presence or absence 
    ! of convection as defined by a reasonably large vertical mass flux 
    ! entering that layer from below.
    ! 
    ! Author: Many. Last modified by Jim McCaa
    ! 
    !-----------------------------------------------------------------------
    use ppgrid
    use physconst, only: cappa, gravit, rair, tmelt, cpair
    use cldconst
    use wv_saturation, only: aqsat
    use phys_grid,     only: get_rlat_all_p, get_rlon_all_p
    use dycore,        only: dycore_is, get_resolution
    use history,       only: outfld    !zf2016.10.21

    real(r8), parameter :: pnot = 1.e5       ! reference pressure
    real(r8), parameter :: lapse = 6.5e-3    ! U.S. Standard Atmsophere lapse rate
    real(r8), parameter :: premib = 750.e2   ! bottom pressure bound of middle cloud
    real(r8), parameter :: pretop = 1.0e2    ! pressure bounding high cloud
    !
    ! Arguments
    !
    integer, intent(in) :: lchnk                  ! chunk identifier
    integer, intent(in) :: ncol                   ! number of atmospheric columns
    integer, intent(in) :: dindex                 ! 0 or 1 to perturb rh

    real(r8), intent(in) :: pmid(pcols,pver)      ! midpoint pressures
    real(r8), intent(in) :: pint(pcols,pverp)     ! interface pressures
    real(r8), intent(in) :: dlf(pcols,pver)
    real(r8), intent(in) :: ztodt
    real(r8), intent(in) :: temp(pcols,pver)      ! temperature
    real(r8), intent(in) :: q(pcols,pver)         ! specific humidity
    real(r8), intent(in) :: omga(pcols,pver)      ! vertical pressure velocity
    real(r8), intent(in) :: cldtop(pcols)         ! top level of convection
    real(r8), intent(in) :: cldbot(pcols)         ! bottom level of convection
    real(r8), intent(in) :: cmfmc(pcols,pverp)    ! convective mass flux--m sub c
    real(r8), intent(in) :: cmfmc2(pcols,pverp)   ! shallow convective mass flux--m sub c
    real(r8), intent(in) :: snowh(pcols)          ! snow depth (liquid water equivalent)
    real(r8), intent(in) :: pdel(pcols,pver)      ! pressure depth of layer
    real(r8), intent(in) :: landfrac(pcols)       ! Land fraction
    real(r8), intent(in) :: ocnfrac(pcols)        ! Ocean fraction
    real(r8), intent(in) :: ts(pcols)             ! surface temperature
    real(r8), intent(in) :: sst(pcols)            ! sea surface temperature
    real(r8), intent(in) :: ps(pcols)             ! surface pressure
    real(r8), intent(in) :: zdu(pcols,pver)       ! detrainment rate from deep convection
    real(r8), intent(inout) :: deepcld(pcols,pver)   
    real(r8), intent(inout) :: shallowcld(pcols,pver) 
    real(r8), intent(inout) :: qdeepcld(pcols,pver)      ! water vapor inside deep convective cld
    real(r8), intent(inout) :: qshallowcld(pcols,pver)   ! water vapor inside shallow convective cld
    real(r8), intent(inout) :: tdeepcld(pcols,pver)   
    real(r8), intent(inout) :: tshallowcld(pcols,pver)
 
    real(r8), intent(in) :: phis(pcols)           ! surface geopotential

    real(r8), intent(in) :: qi(pcols,pver)        ! cloud ice content
    real(r8), intent(in) :: ql(pcols,pver)        ! cloud liquid content

    !
    ! Output arguments
    !
    real(r8), intent(out) :: q_env(pcols,pver)      ! water vapor out of convective cloud package
    real(r8), intent(out) :: t_env(pcols,pver)      ! temperature out of convective cloud package
    real(r8), intent(out) :: cld_env(pcols,pver)  

    real(r8), intent(out) :: cloud(pcols,pver)     ! cloud fraction
    real(r8), intent(out) :: clc(pcols)            ! column convective cloud amount
    real(r8), intent(out) :: cldst(pcols,pver)     ! stratus cloud fraction over ocean
    real(r8), intent(out) :: rhu00(pcols,pver)     ! RH threshold for cloud
    real(r8), intent(out) :: relhum(pcols,pver)    ! RH 
    real(r8), intent(out) :: concld(pcols,pver)    ! convective cloud cover
    real(r8), optional, intent(out) :: cstcld(pcols,pver)    ! layered cloud cover
    !      real(r8) dmudp                 ! measure of mass detraining in a layer
    !
    !---------------------------Local workspace-----------------------------
    !
    real(r8), intent(in) :: zm(pcols,pver)        !
    real(r8) ETAM_k700(pcols)         
    real(r8) ETAM_lcl (pcols)
    real(r8) EIS      (pcols)
    integer :: lcl(pcols)
    integer    cnb

    real(r8) cld                   ! intermediate scratch variable (low cld)
    real(r8) dthdpmn(pcols)         ! most stable lapse rate below 750 mb
    real(r8) dthdp                 ! lapse rate (intermediate variable)
    real(r8) es(pcols,pver)        ! saturation vapor pressure
    real(r8) qs(pcols,pver)        ! saturation specific humidity
    real(r8) rhwght                ! weighting function for rhlim transition
    real(r8) rh(pcols,pver)        ! relative humidity
    real(r8) rhdif                 ! intermediate scratch variable
    real(r8) strat(pcols)          ! intermediate scratch variable
    real(r8) theta(pcols,pver)     ! potential temperature
    real(r8) rhlim                 ! local rel. humidity threshold estimate
    real(r8) coef1                 ! coefficient to convert mass flux to mb/d
    real(r8) clrsky(pcols)         ! temporary used in random overlap calc
    real(r8) rpdeli(pcols,pver-1) ! 1./(pmid(k+1)-pmid(k))
    real(r8) rhpert                !the specified perturbation to rh
    real(r8) deepcu                ! deep convection cloud fraction
    real(r8) shallowcu             ! shallow convection cloud fraction
    real(r8) zwcrit

    logical cldbnd(pcols)          ! region below high cloud boundary
    logical lmdz_scheme

    integer i, ierror, k           ! column, level indices
    integer kp1
    integer kdthdp(pcols)
    integer numkcld                ! number of levels in which to allow clouds

    real(r8) thetas(pcols)                    ! ocean surface potential temperature
    real(r8) :: clat(pcols)                   ! current latitudes(radians)
    real(r8) :: clon(pcols)                   ! current longitudes(radians)
    !
    ! Statement functions
    !
    logical land

    land(i) = nint(landfrac(i)) == 1
    call get_rlat_all_p(lchnk, ncol, clat)
    call get_rlon_all_p(lchnk, ncol, clon)

    !==================================================================================
    ! PHILOSOPHY OF PRESENT IMPLEMENTATION
    !
    ! There are three co-existing cloud types: convective, inversion related low-level
    ! stratocumulus, and layered cloud (based on relative humidity).  Layered and 
    ! stratocumulus clouds do not compete with convective cloud for which one creates 
    ! the most cloud.  They contribute collectively to the total grid-box average cloud 
    ! amount.  This is reflected in the way in which the total cloud amount is evaluated 
    ! (a sum as opposed to a logical "or" operation)
    !
    !==================================================================================
    ! set defaults for rhu00
    rhu00(:,:) = 2.0
    ! define rh perturbation in order to estimate rhdfda
    rhpert = 0.01 

    !
    ! Evaluate potential temperature and relative humidity
    !
    call aqsat(temp    ,pmid    ,es      ,qs      ,pcols   , &
         ncol    ,pver    ,1       ,pver    )
    do k=1,pver
       do i=1,ncol
          theta(i,k)  = temp(i,k)*(pnot/pmid(i,k))**cappa
          rh(i,k)     = q(i,k)/qs(i,k)*(1.0+float(dindex)*rhpert)       
          !
          !  record relhum, rh itself will later be modified related with concld
          !
          relhum(i,k) = rh(i,k)
          cloud(i,k)  = 0.
          cldst(i,k)  = 0.
          concld(i,k) = 0.
          cld_env(i,k)= 0.
       end do
    end do
    if ( present(cstcld) ) then   ! wzz, 2008.2.28
       cstcld(:,:) = 0.0
    endif
    !
    ! Initialize other temporary variables
    !
    ierror = 0
    do i=1,ncol
       ! Adjust thetas(i) in the presence of non-zero ocean heights.
       ! This reduces the temperature for positive heights according to a standard lapse rate.
       if(ocnfrac(i).gt.0.01) thetas(i)  = &
            ( sst(i) - lapse * phis(i) / gravit) * (pnot/ps(i))**cappa
       if(ocnfrac(i).gt.0.01.and.sst(i).lt.260.) ierror = i
       clc(i) = 0.0
    end do
    coef1 = gravit*864.0    ! conversion to millibars/day

    do k=1,pver-1
       do i=1,ncol
          rpdeli(i,k) = 1./(pmid(i,k+1) - pmid(i,k))
       end do
    end do

    !
    ! Estimate of local convective cloud cover based on convective mass flux
    ! Modify local large-scale relative humidity to account for presence of 
    ! convective cloud when evaluating relative humidity based layered cloud amount
    !
    !
    ! cloud mass flux in SI units of kg/m2/s; should produce typical numbers of 20%
    ! shallow and deep convective cloudiness are evaluated separately (since processes
    ! are evaluated separately) and summed
    !   
!-------------------------------------
!    do k=1,pver-1
!       do i=1,ncol
!          shallowcu = max(0.0,min(sh1*log(1.0+sh2*cmfmc2(i,k+1)),0.30))
!          deepcu = max(0.0,min(dp1*log(1.0+dp2*(cmfmc(i,k+1)-cmfmc2(i,k+1))),0.60))
!          concld(i,k) = min(shallowcu + deepcu,0.80)
!          rh(i,k) = (rh(i,k) - concld(i,k))/(1.0 - concld(i,k))
!          q_env(i,k) = rh(i,k) * qs(i,k)    ! water vaper out of the convective cloud
!        end do
!    end do
!------------------------------------

    do k=1,pver  
       do i=1,ncol
          shallowcu = shallowcld(i,k)
          deepcu = deepcld(i,k)
          concld(i,k) = min(shallowcu + deepcu,0.95)
          if( shallowcu + deepcu > concld(i,k)) then
              shallowcld(i,k) = shallowcld(i,k)/(shallowcu + deepcu)*concld(i,k)
              deepcld(i,k)    = deepcld(i,k)/(shallowcu + deepcu)*concld(i,k)
          endif 
       enddo
    enddo

    do k=1,pver  
       do i=1,ncol
          if( concld(i,K) > 0.001 ) then
              q_env(i,k) = ( q(i,k) &
                           - qdeepcld(i,k)*deepcld(i,k) - qshallowcld(i,k)*shallowcld(i,k)  ) &
                               /(1.0-concld(i,k))
              t_env(i,k) = (temp(i,k) - tdeepcld(i,k)*deepcld(i,k) - tshallowcld(i,k)*shallowcld(i,k) ) &
                               /(1.0-concld(i,k))
              if( q_env(i,k) < 1.e-06) then
                  q_env(i,k) = 1.e-06
                  t_env(i,k) = temp(i,k)
              endif
              if( q_env(i,k) > qs(i,k) ) then
                  q_env(i,k) = qs(i,k)
                  t_env(i,k) = temp(i,k)
              endif
          else
             q_env(i,k) = q(i,k)
             t_env(i,k) = temp(i,k)
          endif
        end do
    end do
    call aqsat(t_env    ,pmid    ,es      ,qs      ,pcols   , &
         ncol    ,pver    ,1       ,pver    )
   
    do k=1,pver
       do i=1,ncol
          rh(i,k)     = max( min(q_env(i,k)/qs(i,k), 0.999), 0.1) *(1.0-float(dindex)*rhpert)

!          rh(i,k)     = max(q_env(i,k)/qs(i,k), 0.1) *(1.0-float(dindex)*rhpert)
!                                ! relative humidity outside of convective cloud
          relhum(i,k) = rh(i,k)
       enddo
    enddo
!-----------------------------------------------------------------------------

    lmdz_scheme = .false.

    if( lmdz_scheme ) then 

!       call cldnrh_lmdz(ncol,   ps,    pmid,  t_env,   q_env,     qi   , &
!                       ql  ,    pdel,  cloud )
        call cldnrh_lmdz(lchnk,  ncol,                       &   
                         pmid, pint, ztodt,dlf, t_env, q_env,             &
                         qi,ql , pdel, cloud)

       do k=1,pver
          do i=1,ncol
             cloud(i,k) = min(cloud(i,k), 1.0) 
          enddo
       enddo

    else  
    !==================================================================================
    !
    !          ****** Compute layer cloudiness ******
    !
    !====================================================================
    ! Begin the evaluation of layered cloud amount based on (modified) RH 
    !====================================================================
    !
    numkcld = pver
    do k=2,numkcld
       kp1 = min(k + 1,pver)
       do i=1,ncol
          !
          cldbnd(i) = pmid(i,k).ge.pretop
          !
          if ( pmid(i,k).ge.premib ) then
             !==============================================================
             ! This is the low cloud (below premib) block
             !==============================================================
             ! enhance low cloud activation over land with no snow cover
             if (land(i) .and. (snowh(i) <= 0.000001)) then
                if (get_resolution() == 'T266' ) then
                   rhlim = rhminl - 0.03
                elseif (get_resolution() == 'T106' ) then
                   rhlim = rhminl - 0.03
                else
                   rhlim = rhminl - 0.05
                endif
             else
                rhlim = rhminl - 0.00
             endif
             rhlim = rhminl

             rhdif = (rh(i,k) - rhlim)/(1.0_r8-rhlim)
             cloud(i,k) = min(0.999_r8,(max(rhdif,0.0_r8))**2)
             
!            rhdif = 1.0 - sqrt((1.0 - rh(i,k))/(1.0_r8-rhlim))
!            cloud(i,k) = min(0.999_r8, max(rhdif,0.0_r8))


          else if ( pmid(i,k).lt.premit ) then
             !==============================================================
             ! This is the high cloud (above premit) block
             !==============================================================
             !
             rhlim = rhminh
             !
             rhdif = (rh(i,k) - rhlim)/(1.0_r8-rhlim)   
             cloud(i,k) = min(0.999_r8,(max(rhdif,0.0_r8))**2)

!             rhdif = 1.0 - sqrt((1.0 - rh(i,k))/(1.0_r8-rhlim))
!             cloud(i,k) = min(0.999_r8, max(rhdif,0.0_r8))
          else
             !==============================================================
             ! This is the middle cloud block
             !==============================================================
             !
             !       linear rh threshold transition between thresholds for low & high cloud
             !
             rhwght = (premib-(max(pmid(i,k),premit)))/(premib-premit)
             
             if (land(i) .and. (snowh(i) <= 0.000001)) then
               if (get_resolution() == 'T266' ) then
                  rhlim = rhminh*rhwght + (rhminl - 0.03)*(1.0-rhwght)
               elseif (get_resolution() == 'T106' ) then
                  rhlim = rhminh*rhwght + (rhminl - 0.03)*(1.0-rhwght)
               else
                  rhlim = rhminh*rhwght + (rhminl - 0.05)*(1.0-rhwght)
               endif
             else
                rhlim = rhminh*rhwght + (rhminl - 0.00)*(1.0-rhwght)
             endif
            
             rhlim = rhminh*rhwght + (rhminl - 0.00)*(1.0-rhwght)
 
             rhdif = (rh(i,k) - rhlim)/(1.0_r8-rhlim)
             cloud(i,k) = min(0.999_r8,(max(rhdif,0.0_r8))**2)

!             rhdif = 1.0 - sqrt((1.0 - rh(i,k))/(1.0_r8-rhlim))
!             cloud(i,k) = min(0.999_r8, max(rhdif,0.0_r8))
          end if
          !==================================================================================
          ! WE NEED TO DOCUMENT THE PURPOSE OF THIS TYPE OF CODE (ASSOCIATED WITH 2ND CALL)
          !==================================================================================
          !      !
          !      ! save rhlim to rhu00, it handles well by itself for low/high cloud
          !      !
          rhu00(i,k)=rhlim
          !==================================================================================
       end do
       !
       ! Final evaluation of layered cloud fraction
       !
    end do
    endif
!------------------------------------------------------------------------------

    !
    ! Add in the marine strat
    ! MARINE STRATUS SHOULD BE A SPECIAL CASE OF LAYERED CLOUD
    ! CLOUD CURRENTLY CONTAINS LAYERED CLOUD DETERMINED BY RH CRITERIA
    ! TAKE THE MAXIMUM OF THE DIAGNOSED LAYERED CLOUD OR STRATOCUMULUS
    !
    !===================================================================================
    !
    !  SOME OBSERVATIONS ABOUT THE FOLLOWING SECTION OF CODE (missed in earlier look)
    !  K700 IS SET AS A CONSTANT BASED ON HYBRID COORDINATE: IT DOES NOT DEPEND ON 
    !  LOCAL PRESSURE; THERE IS NO PRESSURE RAMP => LOOKS LEVEL DEPENDENT AND 
    !  DISCONTINUOUS IN SPACE (I.E., STRATUS WILL END SUDDENLY WITH NO TRANSITION)
    !
    !  IT APPEARS THAT STRAT IS EVALUATED ACCORDING TO KLEIN AND HARTMANN; HOWEVER,
    !  THE ACTUAL STRATUS AMOUNT (CLDST) APPEARS TO DEPEND DIRECTLY ON THE RH BELOW
    !  THE STRONGEST PART OF THE LOW LEVEL INVERSION.  
    !
    !==================================================================================
    !
    ! Find most stable level below 750 mb for evaluating stratus regimes
    !
    do i=1,ncol
       ! Nothing triggers unless a stability greater than this minimum threshold is found
       dthdpmn(i) = -0.125
       kdthdp(i) = 0
    end do
    !
    do k=2,pver
       do i=1,ncol
          if (pmid(i,k) >= premib .and. ocnfrac(i).gt. 0.01) then
             ! I think this is done so that dtheta/dp is in units of dg/mb (JJH)
             dthdp = 100.0*(theta(i,k) - theta(i,k-1))*rpdeli(i,k-1)
             if (dthdp < dthdpmn(i)) then
                dthdpmn(i) = dthdp
                kdthdp(i) = k     ! index of interface of max inversion
             end if
          end if
       end do
    end do

    ! Also check between the bottom layer and the surface
    ! Only perform this check if the criteria were not met above

    do i = 1,ncol
       if ( kdthdp(i) .eq. 0 .and. ocnfrac(i).gt.0.01) then
          dthdp = 100.0 * (thetas(i) - theta(i,pver)) / (ps(i)-pmid(i,pver))
          if (dthdp < dthdpmn(i)) then
             dthdpmn(i) = dthdp
             kdthdp(i) = pver     ! index of interface of max inversion
          endif
       endif
    enddo

!---------------------------
! CAM3 sheme
!
!    do i=1,ncol
!       if (kdthdp(i) /= 0) then
!          k = kdthdp(i)
!          kp1 = min(k+1,pver)
!          ! Note: strat will be zero unless ocnfrac > 0.01
!         strat(i) = min(1._r8,max(0._r8, ocnfrac(i) * ((theta(i,k700)-thetas(i))*.057_r8-.5573_r8) ) )
!         cldst(i,k) = min(strat(i),max(rh(i,k),rh(i,kp1)))
!       end if
!    end do
!
!---
! New scheme
! 
    call strat_EIS(ncol, lchnk, temp, theta, thetas, landfrac, ocnfrac, ts, sst, ps, kdthdp, strat) 

    do i=1,ncol
       if (kdthdp(i) /= 0) then
          k = kdthdp(i)
          kp1 = min(k+1,pver)

          cldst(i,k) = min(strat(i),max(rh(i,k),rh(i,kp1)))
       end if
    end do 

!-------------------
    !
    ! AGGREGATE CLOUD CONTRIBUTIONS (cldst should be zero everywhere except at level kdthdp(i))
    !
    do k=1,pver
       do i=1,ncol
          !
          !       which is greater; standard layered cloud amount or stratocumulus diagnosis
          !
!          if( pmid(i,k)/ps(i) > 0.45 ) then
!              if( omga(i,k) .gt. 0.0) then
!                  cloud(i,k) = 0.0
!              endif
!          endif

#ifdef UWMT
!do nothing
          cloud(i,k) = min(cloud(i,k), rh(i,k)) 
#else
          cloud(i,k) = max(cloud(i,k),cldst(i,k))
#endif
          !
          if ( present(cstcld) ) then   ! wzz, 2008.2.28
             cstcld(i,k) = cloud(i,k)
          endif
          !       add in the contributions of convective cloud (determined separately and accounted
          !       for by modifications to the large-scale relative humidity.
          !
!         cloud(i,k) = min(cloud(i,k)+concld(i,k), 1.0_r8)

         cld_env(i,k) = cloud(i,k)
         cloud(i,k) = min(cloud(i,k)*(1.-concld(i,k))+concld(i,k), 1.0_r8)
       end do
    end do

    call outfld('WBPTEMP', theta, pcols, lchnk)   !zf 2016.10.20

    !
  end subroutine cldfrc

    subroutine strat_EIS(ncol,   lchnk, t,     theta,   thetas, landfrac, ocnfrac,    & 
                         ts,     sst,   ps,    kdthdp,  strat   )
    !-----------------------------------------------------------------------
    !
    ! Purpose:
    ! Compute stratocumulus
    !
    ! Method:
    ! This calculate straticumlus fraction using a estimated inversion strength (EIS)
    !
    ! Author: GZ 2011.6
    !
    !-----------------------------------------------------------------------
        use ppgrid  ,  only: pcols, pver
        use cldconst,  only: k700
        integer,  intent(in) :: lchnk                  ! chunk identifier
        integer,  intent(in) :: ncol                   ! number of atmospheric columns
        real(r8), intent(in) :: t(pcols, pver )       ! Land fraction
        real(r8), intent(in) :: landfrac(pcols)       ! Land fraction
        real(r8), intent(in) :: ocnfrac(pcols)        ! Ocean fraction
        real(r8), intent(in) :: ts(pcols)             ! surface temperature
        real(r8), intent(in) :: sst(pcols)            ! sea surface temperature
        real(r8), intent(in) :: ps(pcols)             ! surface pressure (Pa)
        real(r8), intent(in) :: theta(pcols,pver)     ! potential temperature (K)
        real(r8), intent(in) :: thetas(pcols)         ! ocean surface potential temperature (K)
        integer , intent(in) :: kdthdp(pcols)          ! index of interface of max inversion

        real(r8), intent(out) :: strat(pcols)    ! stratus cloud fraction

        real(r8), parameter   ::  aa     = 17.271_r8
        real(r8), parameter   ::  bb     = 237.7_r8
        real(r8), parameter   ::  aa1    = 1013.250_r8
        real(r8), parameter   ::  aa2    = 2.256e-5_r8
        real(r8), parameter   ::  aa3    = 5.256_r8
        real(r8), parameter   ::  L_v    = 2.4e6_r8     ! latent heat of evaporation [approx 2400 kJ/kg at 25C] 
        real(r8), parameter   ::  Cpd    = 1004_r8      ! specific heat at constant pressure for air [approx 1004 J/(kg-K)] 
        real(r8), parameter   ::  Ra     = 287_r8       ! specific gas constant for air [J/(kg-K)] 
        real(r8), parameter   ::  Rv     = 1.608*287_r8 ! specific gas constant for water vapor [J/(kg-K)] 
        real(r8), parameter   ::  in_reg = .055_r8      ! tunning
        real(r8), parameter   ::  in_scl = 0.235_r8     ! ~
!---------------------------
!   local variables
!
        real(r8) EIS1(pcols)            ! estimated inversion strength
        real(r8) EIS2(pcols)            ! estimated inversion strength
        real(r8) Ya(pcols)              ! gradient
        real(r8) Yl(pcols)              ! gradient
        real(r8) Y7(pcols)              ! gradient
        real(r8) lcl_p(pcols)           ! lifting condensation level Press
        real(r8) lcl_h(pcols)           ! lifting condensation level 
        real(r8) Z700(pcols)            ! z700
        real(r8) CHI (pcols)            ! index of LCL
        real(r8) Ml (pcols)             ! qs mixing ratio
        real(r8) Ma (pcols)             ! qs mixing ratio
        real(r8) M7 (pcols)             ! qs mixing ratio
        real(r8) tave(pcols)            ! ave of t700 & ts
        real(r8) tlcl(pcols)            ! temp LCL
        real(r8) Ea(pcols)              ! moist
        real(r8) E7(pcols)              ! moist
        real(r8) El(pcols)              ! moist
     
        integer   i

        ! ---------------------------------------------------------------------------
        !  Calculate marine stratus fraction
        ! ---------------------------------------------------------------------------
        !
        ! Marine stratus should be a special case of layered cloud.
        ! Cloud currently contains layered cloud determined by rh criteria.
        ! Take the maximum of the diagnosed layered cloud or stratocumulus
        !
        ! Some observations about the following section of code (missed in earlier look)
        ! K700 is set as a constant based on hybrid coordinate: it does not depend on
        ! local pressure; there is no pressure ramp => looks level dependent and
        ! discontinuous in space (i.e., stratus will end suddenly with no transition)
        !
        ! It appears that stratus is evaluated according to Klein and Hartmann; however,
        ! The actual stratus amount (cldst) appears to depend directly on the rh below
        ! the strongest part of the low level inversion.
        !
            do i = 1, ncol
               Z700(i)  = Ra * sst(i) / 9.8*(log(ps(i)/70000))
               CHI(i)   = sst(i)/(1669.0-122.0*(0.8)-sst(i))
               lcl_p(i) = ps(i)/100.0*(0.8)**CHI(i)         !  0.8 is relhum at surface
               lcl_h(i) = 1/aa2*( 1-(lcl_p(i)/aa1)**(1/aa3) )
               tave(i)  = (sst(i)+t(i,k700))/2
               tlcl(i)  = sst(i)-lcl_h(i)*9.8/Cpd
               Ea(i)    = 6.1*10**(7.5*(tave(i)  -273.15)/(237.3+tave(i)  -273.15))
               E7(i)    = 6.1*10**(7.5*(t(i,k700)-273.15)/(237.3+t(i,k700)-273.15))
               El(i)    = 6.1*10**(7.5*(tlcl(i)  -273.15)/(237.3+tlcl(i)  -273.15))
               Ma(i)  = 0.622*Ea(i)/(850.0   - Ea(i))
               Ml(i)  = 0.622*El(i)/(lcl_p(i)- El(i))
               M7(i)  = 0.622*E7(i)/(700.0   - E7(i))
               Ya(i) = 9.8/Cpd*(L_v**2*Ma(i) - Cpd*L_v*Ma(i)*Rv*tave(i)/Ra)/(Cpd*Rv*tave(i)**2+ L_v**2*Ma(i))
               Y7(i) = 9.8/Cpd*(L_v**2*M7(i) - Cpd*L_v*M7(i)*Rv*t(i,k700)/Ra)/(Cpd*Rv*t(i,k700)**2+ L_v**2*M7(i))
               Yl(i) = 9.8/Cpd*(L_v**2*Ml(i) - Cpd*L_v*Ml(i)*Rv*tlcl(i)/Ra)/(Cpd*Rv*tlcl(i)**2+ L_v**2*Ml(i))
               EIS1(i)  = (theta(i,k700)-thetas(i)) - Ya(i)*(Z700(i)- lcl_h(i) )
               EIS2(i)  = (theta(i,k700)-thetas(i)) - Y7(i)* Z700(i)+ Yl(i)*lcl_h(i)
               strat(i) = min(1._r8,max(0._r8,ocnfrac(i)*(EIS2(i)*in_reg+in_scl))) 
            end do
        return
    end subroutine strat_EIS

end module cloud_fraction
