module dewpoint 
!-------------------------------------------------------------- 
!    pv=prsl(i,k)*1.e-3*qenv(i,k)/(0.622+0.378*qenv(i,k))   
!	   ! PRSL - midlayer pressure in centibars (1000 Pa)
!	   ! ! qenv - real (ilev,klev) environment specific humidities
!    call dewp(pv) 
!-------------------------------------------
  use shr_kind_mod,    only: r8 => shr_kind_r8
  use ppgrid,          only: pcols, pver
  implicit none

  save
  private                        
!
! PUBLIC: interfaces
!
   public dewp       !  Compute dewpoint temperature 
   public dewp_init
   public ftdpxg
!
! Private data
!
   integer, parameter :: nxtdp = 5001
   real(r8) :: tbtdp(nxtdp) 
   real(r8) :: c1xtdp
   real(r8) :: c2xtdp

contains

  subroutine dewp(lchnk   ,ncol  ,   ps  ,    q  , tdewp)
 
!-------------------------------------------------------------------------
! Subprogram: ftdp         Compute dewpoint temperature
!   Author: N Phillips            w/NMC2X2   Date: 30 dec 82
!
! Abstract: Compute dewpoint temperature from vapor pressure.
!   A linear interpolation is done between values in a lookup table
!   computed in gtdp. See documentation for ftdpxg for details.
!   Input values outside table range are reset to table extrema.
!   The interpolation accuracy is better than 0.0005 Kelvin
!   for dewpoint temperatures greater than 250 Kelvin,
!   but decreases to 0.02 Kelvin for a dewpoint around 230 Kelvin.
!   On the Cray, ftdp is about 75 times faster than exact calculation.
!   This function should be expanded inline in the calling routine.
!
! Program History Log:
!   91-05-07  Iredell             made into inlinable function
!   94-12-30  Iredell             expand table
! 1999-03-01  Iredell             f90 module
! 2001-02-26  Iredell             ice phase
!
! Usage:   tdp=ftdp(pv)
!
!   Input argument list:
!     pv         real(r8) vapor pressure in Pascals
!
!   Output argument list:
!     ftdp       real(r8) dewpoint temperature in Kelvin
!
! Attributes:
!   Language: Fortran 90.
!
!------------------------------------------------------------
    implicit none

    integer, intent(in) :: lchnk                   ! chunk identifier
    integer, intent(in) :: ncol                    ! 

    real(r8), intent(in)  :: ps(pcols)      ! surface pressure (Pa)
    real(r8), intent(in)  :: q(pcols, pver)      ! specific humidity
    real(r8), intent(out) :: tdewp(pcols)  ! dewpoint temperature (K) 
    
    real(r8) ftdp
    real(r8) pv

    integer  jx , i
    real(r8) xj
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    do i=1, ncol
       pv= ps(i)*q(i,pver)/(0.622+0.378*q(i,pver)) 
       xj=min(max(c1xtdp+c2xtdp*pv,1.),real(nxtdp))
       jx=min(xj,nxtdp-1.)
       ftdp=tbtdp(jx)+(xj-jx)*(tbtdp(jx+1)-tbtdp(jx))
       tdewp(i) = ftdp
    enddo
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  end subroutine dewp

  subroutine dewp_init
!$$$     Subprogram Documentation Block
!
! Subprogram: gtdp         Compute dewpoint temperature table
!   Author: N Phillips            w/NMC2X2   Date: 30 dec 82
!
! Abstract: Compute dewpoint temperature table as a function of
!   vapor pressure for inlinable function ftdp.
!   Exact dewpoint temperatures are calculated in subprogram ftdpxg.
!   The current implementation computes a table with a length
!   of 5001 for vapor pressures ranging from 0.5 to 1000.5 Pascals
!   giving a dewpoint temperature range of 208 to 319 Kelvin.
!
! Program History Log:
!   91-05-07  Iredell
!   94-12-30  Iredell             expand table
! 1999-03-01  Iredell             f90 module
! 2001-02-26  Iredell             ice phase
!
! Usage:  call gtdp
!
! Subprograms called:
!   (ftdpxg)   inlinable function to compute dewpoint temperature
!
! Attributes:
!   Language: Fortran 90.
!
!$$$
    implicit none
    integer jx
    real(r8) xmin,xmax,xinc,t,x,pv
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    xmin=0.5
    xmax=10000.5
    xinc=(xmax-xmin)/(nxtdp-1)
    c1xtdp=1.-xmin/xinc
    c2xtdp=1./xinc
    t=208.0
    do jx=1,nxtdp
      x=xmin+(jx-1)*xinc
      pv=x
      t=ftdpxg(t,pv)
      tbtdp(jx)=t
    enddo
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  end subroutine dewp_init

  function ftdpxg(tg,pv)
!-------------------------------------------------------------------------
! Subprogram: ftdpxg       Compute dewpoint temperature
!   Author: N Phillips            w/NMC2X2   Date: 30 dec 82
!
! Abstract: Exactly compute dewpoint temperature from vapor pressure.
!   A guess dewpoint temperature must be provided.
!   The saturation vapor pressure over either liquid and ice is computed
!   over liquid for temperatures above the triple point,
!   over ice for temperatures 20 degress below the triple point,
!   and a linear combination of the two for temperatures in between.
!   The water model assumes a perfect gas, constant specific heats
!   for gas, liquid and ice, and neglects the volume of the condensate.
!   The model does account for the variation of the latent heat
!   of condensation and sublimation with temperature.
!   The Clausius-Clapeyron equation is integrated from the triple point
!   to get the formula
!       pvsl=con_psat*(tr**xa)*exp(xb*(1.-tr))
!   where tr is ttp/t and other values are physical constants.
!   The reference for this decision is Emanuel(1994), pages 116-117.
!   The formula is inverted by iterating Newtonian approximations
!   for each pvs until t is found to within 1.e-6 Kelvin.
!   This function can be expanded inline in the calling routine.
!
! Program History Log:
!   91-05-07  Iredell             made into inlinable function
!   94-12-30  Iredell             exact computation
! 1999-03-01  Iredell             f90 module
! 2001-02-26  Iredell             ice phase
!
! Usage:   tdp=ftdpxg(tg,pv)
!
!   Input argument list:
!     tg         real(r8) guess dewpoint temperature in Kelvin
!     pv         real(r8) vapor pressure in Pascals
!
!   Output argument list:
!     ftdpxg     real(r8) dewpoint temperature in Kelvin
!
! Attributes:
!   Language: Fortran 90.
!
!---------------------------------------------------------------------------------
    implicit none
    real(r8) ::  ftdpxg
    real(r8),intent(in) :: tg,pv
    real(r8),parameter:: terrm=1.e-6

    real(r8),parameter:: con_ttp     =2.7316e+2 ! temp at H2O 3pt     (K)
    real(r8),parameter:: con_cvap    =1.8460e+3 ! spec heat H2O gas   (J/kg/K)
    real(r8),parameter:: con_cliq    =4.1855e+3 ! spec heat H2O liq   (J/kg/K)
    real(r8),parameter:: con_hvap    =2.5000e+6 ! lat heat H2O cond   (J/kg)
    real(r8),parameter:: con_rv      =4.6150e+2 ! gas constant H2O    (J/kg/K)
    real(r8),parameter:: con_csol    =2.1060e+3 ! spec heat H2O ice   (J/kg/K)
    real(r8),parameter:: con_hfus    =3.3358e+5 ! lat heat H2O fusion (J/kg)
    real(r8),parameter:: con_psat    =6.1078e+2 ! pres at H2O 3pt     (Pa)

    real(r8),parameter:: tliq=con_ttp
    real(r8),parameter:: tice=con_ttp-20.0
    real(r8),parameter:: dldtl=con_cvap-con_cliq
    real(r8),parameter:: heatl=con_hvap
    real(r8),parameter:: xponal=-dldtl/con_rv

    real(r8),parameter:: xponbl=-dldtl/con_rv+heatl/(con_rv*con_ttp)
    real(r8),parameter:: dldti=con_cvap-con_csol
    real(r8),parameter:: heati=con_hvap+con_hfus
    real(r8),parameter:: xponai=-dldti/con_rv
    real(r8),parameter:: xponbi=-dldti/con_rv+heati/(con_rv*con_ttp)
    real(r8) t,tr,w,pvtl,pvti,pvt,ell,eli,el,dpvt,terr
    integer i
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    t=tg
    do i=1,100
      tr=con_ttp/t
      if(t.ge.tliq) then
        pvt=con_psat*(tr**xponal)*exp(xponbl*(1.-tr))
        el=heatl+dldtl*(t-con_ttp)
        dpvt=el*pvt/(con_rv*t**2)
      elseif(t.lt.tice) then
        pvt=con_psat*(tr**xponai)*exp(xponbi*(1.-tr))
        el=heati+dldti*(t-con_ttp)
        dpvt=el*pvt/(con_rv*t**2)
      else
        w=(t-tice)/(tliq-tice)
        pvtl=con_psat*(tr**xponal)*exp(xponbl*(1.-tr))
        pvti=con_psat*(tr**xponai)*exp(xponbi*(1.-tr))
        pvt=w*pvtl+(1.-w)*pvti
        ell=heatl+dldtl*(t-con_ttp)
        eli=heati+dldti*(t-con_ttp)
        dpvt=(w*ell*pvtl+(1.-w)*eli*pvti)/(con_rv*t**2)
      endif
      terr=(pvt-pv)/dpvt
      t=t-terr
      if(abs(terr).le.terrm) exit
    enddo
    ftdpxg=t
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  end function ftdpxg

end module dewpoint
