#include <misc.h>
#include <params.h>

module dust_intr

!---------------------------------------------------------------------------------
! Module to interface the aerosol parameterizations with CAM
! written by PJR (extensively modified from chemistry module)
!---------------------------------------------------------------------------------

!-----the following 7 lines are added by lqx, 20070710----------------------
  use shr_kind_mod,only: r8 => shr_kind_r8
  use pmgrid,      only: plat, plev, plevp, masterproc
  use ppgrid,      only: pcols, pver
  use shr_const_mod, only: SHR_CONST_G
  use constituents,only: ppcnst, cnst_add, cnst_name, advected, cnst_get_ind, iaero
  use cam_drydep_mod, only: setdvel, ddflux
!------------------------------------------------------
  use abortutils,  only: endrun

  implicit none

  private          ! Make default type private to the module

  save

!------The following 5 lines are added by Liu Qianxia, 20070710------------
  character(len=80) :: srcnam (12) ! names of source/sink tendencies, 12 advected species
  integer ncyear

  integer :: ncnst                                      ! number of constituents
  character(len=8), allocatable :: cnst_names(:)     ! constituent names
  integer dust_idx1                                  ! start constituent index for first DUST cycle component
!
! Public interfaces
!
  public dust_register_cnst                        ! register consituents
  public dust_implements_cnst                      ! returns true if consituent is implemented by this package
  public dust_init_cnst                            ! initialize mixing ratios if not read from initial file
  public dust_initialize                           ! initialize (history) variables
  public dust_wet_intr                             ! interface to wet deposition
  public dust_emis_intr                            ! interface to emission
  public dust_drydep_intr                          ! interface to tendency computation
  public dust_time_interp                          ! interpolate oxidants and fluxes to current time
  public dcyc_idx1, ncnst                          ! allow other parts of code to know where dust is
!-----------defining mw and cp of all the dust bins by Liuqx 20070709--------------------

  real(r8), parameter:: cpSD1 = 666.
  real(r8), parameter:: cpSD2 = 666.
  real(r8), parameter:: cpSD3 = 666.
  real(r8), parameter:: cpSD4 = 666.
  real(r8), parameter:: cpSD5 = 666.
  real(r8), parameter:: cpSD6 = 666.
  real(r8), parameter:: cpSD7 = 666.
  real(r8), parameter:: cpSD8 = 666.
  real(r8), parameter:: cpSD9 = 666.
  real(r8), parameter:: cpSD10 = 666.
  real(r8), parameter:: cpSD11 = 666.
  real(r8), parameter:: cpSD12 = 666.


  real(r8), parameter:: mwSD1  = 60.08
  real(r8), parameter:: mwSD2  = 9999.
  real(r8), parameter:: mwSD3 = 9999.
  real(r8), parameter:: mwSD4  = 9999.
  real(r8), parameter:: mwSD5  = 9999.
  real(r8), parameter:: mwSD6  = 9999.
  real(r8), parameter:: mwSD7 = 9999.
  real(r8), parameter:: mwSD8  = 9999.
  real(r8), parameter:: mwSD9  = 9999.
  real(r8), parameter:: mwSD10  = 9999.
  real(r8), parameter:: mwSD11= 9999.
  real(r8), parameter:: mwSD12 = 9999.

  real(r8), parameter::  gravit = SHR_CONST_G

contains

!===============================================================================
  subroutine dust_register_cnst
!----------------------------------------------------------------------- 
! 
! Purpose: register advected constituents for all aerosols
! 
! Adopt the registering method of CUACEAero which defines 12 bins of dust species
!       Liu Qianxia 2007.07.06
! Method: 
!-----------------------------------------------------------------------

!---------------------------Local workspace-----------------------------
    integer :: m, i                                ! tracer index
    integer :: istat
    real(r8), parameter :: zero  = 0._r8
    character names*8
!-----------------------------------------------------------------------

! Define constituent names
    ncnst = dustcyc_ncomp()

    if ( ncnst > 0 ) then
    allocate( cnst_names(ncnst), stat=istat )
    if ( istat /= 0 ) then
      call endrun ('DUST_REGISTER_CNST: ERROR: allocate of cnst_names failed')
    end if
    end if

    select case ( iaero )
    case ( 0 )
       call endrun('DUST_REGISTER_CNST:  code under development')
    case ( 1 )
       if ( ncnst < 1 ) then
       call endrun('DUST_REGISTER_CNST:  the selected species are not correct')
       end if
       names = 'SD01'
       do i = 1, ncnst
          write(names(3:4),'(i2.2)') i
          cnst_names(i) = names
          call cnst_add(cnst_names(i), advected, mwSD1, cpSD1, zero, m)
          if ( i == 1 ) then
             call dcyc_idx_set(m)
          endif
       enddo

    case default
       call endrun('DUST_REGISTER_CNST:  unknown scheme')
    end select

  end subroutine dust_register_cnst

!=======================================================================
  function dust_implements_cnst(name)
!----------------------------------------------------------------------- 
! 
! Purpose: return true if specified constituent is implemented by this 
!          package
! 
!-----------------------------------------------------------------------
!-----------------------------Arguments---------------------------------

     character(len=*), intent(in) :: name   ! constituent name
     logical :: dust_implements_cnst        ! return value
     integer :: m
!-----------------------------------------------------------------------

     select case ( iaero )
     case ( 0 )
        call endrun('DUST_IMPLEMENTS_CNST:  code under development')
     case ( 1 )
        dust_implements_cnst = .false.
        do m = 1, ncnst
        if (name == cnst_names(m)) then
           dust_implements_cnst = .true.
           exit
        end if
        end do
     case default
        call endrun('DUST_IMPLEMENTS_CNST:  unknown scheme')
     end select

  end function dust_implements_cnst


!=======================================================================
  subroutine dust_init_cnst(name, q)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Set initial mass mixing ratios.
!
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8

!-----------------------------Arguments---------------------------------
    
    character(len=*), intent(in) :: name         ! constituent name
    
    real(r8), intent(out) :: q(:,:,:)            !  mass mixing ratio
!-----------------------------------------------------------------------
    
!-----------------------------------------------------------------------
    select case ( iaero )
    case ( 0 )
       call endrun('DUST_INIT_CNST:  code under development')
    case ( 1 )
     if ( name == cnst_names(1) ) then
       q = 0._r8
     else if ( name == cnst_names(2) ) then
       q = 0._r8
     else if ( name == cnst_names(3) ) then
       q = 0._r8
     else if ( name == cnst_names(4) ) then
       q = 0._r8
     else if ( name == cnst_names(5) ) then
       q = 0._r8
  !     q = 0.01_r8    ! wzz for test, 2008.1.14
  !     q(30:40,26,10:20) = 0.01_r8    ! wzz for test, 2008.1.14
     else if ( name == cnst_names(6) ) then
       q = 0._r8
     else if ( name == cnst_names(7) ) then
       q = 0._r8
     else if ( name == cnst_names(8) ) then
       q = 0._r8
     else if ( name == cnst_names(9) ) then
       q = 0._r8
     else if ( name == cnst_names(10) ) then
       q = 0._r8
     else if ( name == cnst_names(11) ) then
       q = 0._r8
     else if ( name == cnst_names(12) ) then
       q = 0._r8
     end if
    case default
       call endrun('DUST_INIT_CNST:  unknown scheme')
    end select

  end subroutine dust_init_cnst

!===============================================================================
  subroutine dust_initialize 
!----------------------------------------------------------------------- 
! 
! Purpose: initialize parameterization of dust chemistry
!          (declare history variables)
! 
! Method: 
!-----------------------------------------------------------------------

    use history,      only: addfld, add_default, phys_decomp
    use ccrnbnd,      only: ccrnbndini
    use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                            is_perpetual

    use shr_const_mod,    only: SHR_CONST_RDAIR, SHR_CONST_REARTH
    use commap,    only: clat, clon, w
    use ppgrid,    only: pcols, begchunk, endchunk
    use ioFileMod, only: getfil
   !---------------------------Local workspace-----------------------------
    integer :: m, ix                                ! tracer index
    real(r8) :: calday        ! current calendar day
    integer :: yr, mon, day, ncsec
    integer :: ncdate
    character lpthncsf*200       ! local file for netCDF surface flux data
!-----------------------------------------------------------------------

    calday = get_curr_calday()
    if ( masterproc ) then
    write (6,*) ' dust_initialize: positioning at calday ', calday
    endif
    if ( is_perpetual() ) then
       call get_perp_date(yr, mon, day, ncsec)
    else
       call get_curr_date(yr, mon, day, ncsec)
    end if
    ncyear = yr
    if ( ncyear .lt. 1000 ) ncyear = ncyear + 1900

    select case ( iaero )
    case ( 0 )
!      call endrun('DUST_INITIALIZE:  code under development')
     if ( masterproc ) then
       write(*,*) 'DUST_INITIALIZE:  code under development for scheme ', iaero
     endif
    case ( 1 )
!
       call ccrnbndini( ncyear )
!
! wzz, 2008.8.6
       do m = 1, ncnst
          call addfld (trim(cnst_names(m))//'DRY','kg/m2/s ',1, 'A', &
                      'SD dry deposition',phys_decomp)
          call add_default (trim(cnst_names(m))//'DRY', 1, ' ')
!
          call addfld (trim(cnst_names(m))//'WETB','kg/m2/s ',1, 'A', &
                      'SD wet deposition below cloud',phys_decomp)
          call add_default (trim(cnst_names(m))//'WETB', 1, ' ')
!
          call addfld (trim(cnst_names(m))//'WETI','kg/m2/s ',1, 'A', &
                      'SD wet deposition in cloud',phys_decomp)
          call add_default (trim(cnst_names(m))//'WETI', 1, ' ')
       enddo

    case default
       call endrun('DUST_INITIALIZE:  unknown scheme')
    end select

  end subroutine dust_initialize


!===============================================================================
  subroutine dust_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
       evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to wet processing of aerosols (source and sinks).
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols, pver
    use constituents,only: ppcnst

!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer, intent(in) :: nstep
    integer, intent(in) :: lat(pcols)                  ! latitude index for S->N storage
    real(r8), intent(in) :: clat(pcols)                ! latitude 
    real(r8), intent(in) :: cme(pcols,pver)            ! local condensation of cloud water
    real(r8), intent(in) :: prain(pcols,pver)          ! production of rain
    real(r8), intent(in) :: evapr(pcols,pver)          ! evaporation of rain
    real(r8), intent(in) :: cldn(pcols,pver)           ! cloud fraction
    real(r8), intent(in) :: cldc(pcols,pver)           ! convective cloud fraction
    real(r8), intent(in) :: cldv(pcols,pver)           ! cloudy volume undergoing scavenging

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies
    real(r8), intent(inout)  :: cflx(pcols,ppcnst)       ! Surface constituent flux (kg/m^2/s)

    real(r8), intent(inout) :: fracis(pcols,pver,ppcnst)         ! fraction of transported species that are insoluble

    real(r8), intent(in) :: conicw(pcols, pver)
    real(r8), intent(in) :: cmfdqr(pcols, pver)
    real(r8), intent(in) :: rainmr(pcols, pver) ! rain mixing ratio
    real(r8) :: calday        ! current calendar day

!-----------------------------------------------------------------------

    if ( iaero == 0 ) then
       call endrun('DUST_WET_INTR:  code under development')
    endif

  end subroutine dust_wet_intr

  subroutine dust_drydep_intr (state, ptend, wvflx, dt, lat, clat, &
       fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, month, landfrac, &
       icefrac, ocnfrac,fv,ram1)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to dry deposition and sedimentation of dust
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols

!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer, intent(in) :: lat(pcols)                  ! latitude index for S->N storage
    real(r8), intent(in) :: clat(pcols)                 ! latitude 
    real(r8), intent(in) :: fsds(pcols)                 ! longwave down at sfc
    real(r8), intent(in) :: obklen(pcols)                 ! obklen
    real(r8), intent(in) :: ustar(pcols)                  ! sfc fric vel--used over oceans and sea ice.
    real(r8), intent(in) :: ts(pcols)                     ! sfc temp
    real(r8), intent(in) :: landfrac(pcols)               ! land fraction
    real(r8), intent(in) :: icefrac(pcols)                ! ice fraction
    real(r8), intent(in) :: ocnfrac(pcols)                ! ocean fraction
    real(r8), intent(in) :: hflx(pcols)                  ! sensible heat flux
    real(r8), intent(in) :: prect(pcols)                     ! prect
    real(r8), intent(in) :: snowh(pcols)                     ! snow depth
    real(r8), intent(in) :: pblh(pcols)                     ! pbl height
    integer, intent(in)  :: month
    real(r8), intent(in) :: wvflx(pcols)       ! water vapor flux
    real(r8), intent(in) :: fv(pcols)        ! for dry dep velocities from land model for dust
    real(r8), intent(in) :: ram1(pcols)       ! for dry dep velocities from land model for dust

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies

!-----------------------------------------------------------------------

    call endrun('DUST_DRYDEP_INTR:  code under development')

  end subroutine dust_drydep_intr

  subroutine dust_time_interp

    use ccrnbnd,      only: ccrnbndint
    use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                            is_perpetual

    real(r8) calday
    integer :: yr, mon, day, ncsec

    calday = get_curr_calday()
    if ( is_perpetual() ) then
       call get_perp_date(yr, mon, day, ncsec)
    else
       call get_curr_date(yr, mon, day, ncsec)
    end if

    select case ( iaero )
    case ( 0 )
       call endrun('DUST_TIME_INTERP:  code under development')
    case ( 1 )
       call ccrnbndint(yr, calday) ! interpolate emissions for CUACE
    case default
       call endrun('DUST_TIME_INTERP:  unknown scheme')
    end select

  end subroutine dust_time_interp

  subroutine dust_emis_intr (state, ptend, dt,cflx)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to emission of all dusts.
! Notice that the mobilization is calculated in the land model (need #define BGC) and
! the soil erodibility factor is applied here.
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols
    use constituents,only: ppcnst

!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies
    real(r8),            intent(inout) :: cflx(pcols,ppcnst)

    select case ( iaero )
    case ( 0 )
       call endrun('DUST_EMIS_INTR:  code under development')
    case ( 1 )
       write(0,*)'DUST_EMIS_INTR: need to be developed'
    case default
       call endrun('DUST_EMIS_INTR:  unknown scheme')
    end select

  end subroutine dust_emis_intr 

   integer function dustcyc_ncomp()
#include <aero_chem_order_cam9.cdk>
    character*8 aeroname(aerochem_np)
    COMMON /CAM_C/ AERONAME
    integer i

    dustcyc_ncomp = 0

    if ( iaero == 1 ) then
      do i = 1, aerochem_np
         if ( AERONAME(i) == 'SOILDUST' ) then
            dustcyc_ncomp = aero_nsize
            exit
         end if
      enddo
    endif

   end function dustcyc_ncomp

   subroutine dcyc_idx_set(m)
     ! store the start index of the DUST species
     integer m
!    write (6,*) ' dcyc: dcyc_idx_set to ', m
     dust_idx1 = m
   end subroutine dcyc_idx_set

   integer function dcyc_idx1()
      dcyc_idx1 = dust_idx1
   end function dcyc_idx1

end module dust_intr
