#include <misc.h>
#include <params.h> 

module ghg_surfvals_2d
!-----------------------------------------------------------------------------------
! Purpose: Provides greenhouse gas (ghg) values at the Earth's surface.
!          These values may be time dependent.
!
! Author: Brian Eaton (assembled module from existing scattered code pieces)
!         change by xinxiaoge for CMIP6 monthly latitude data, 2016July
!-----------------------------------------------------------------------------------

   use shr_kind_mod, only: r8=>shr_kind_r8
   use pmgrid,       only: plon, plat, plond, masterproc, plev, plevp
   use abortutils,   only: endrun
   use comsrf,       only: srfflx_state2d
   use time_manager, only: get_curr_date, get_curr_calday, get_nstep,get_perp_date,&
                           is_perpetual
   use ppgrid,         only: begchunk, endchunk, pcols, pver
   use phys_grid,      only: scatter_field_to_chunk, get_ncols_p
   use history,         only: outfld, addfld, add_default, phys_decomp

!-----------------------------------------------------------------------
!- module boilerplate --------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
   private                   ! Make default access private
   save

! Public methods
   public ::&
      ghg_surfvals_ramp_2d, & 
      co2_surfvals_ramp_2d, & 
      ghg_surfvals_init_2d,         & ! returns true when ramp option is on
      ghg_ramp_read_2d,          &
      co2_ramp_time_2d,          &
      ghg_surfvals_get_ghg_2d, &
      ghg_surfvals_get_co2vmr_2d

! Public data
#ifdef CARBONCYC
   character(len=16), public :: scenario_ghg = 'CO2CYC'          ! wzz, 2009.5.12
#else
   character(len=16), public :: scenario_ghg = 'FIXED' ! 'FIXED','RAMPED' or 'RAMP_CO2_ONLY'
#endif

   real(r8), private, allocatable :: co2vmrx(:,:,:)

   integer, public  :: rampYear_ghg = 0                ! ramped gases fixed at this year (if > 0)
   character(len=256), public  :: bndtvghg_2d             ! filename for ramped data
  character(len=256), public  :: bndtvco2             ! filename for ramped data   lijd, 2009.11.09
   integer, public  :: ramp_co2_start_ymd = 0          ! start date for co2 ramping (yyyymmdd)
   real(r8), public :: ramp_co2_annual_rate = 1.0      ! % amount of co2 ramping per yr; default is 1% 
   real(r8), public :: ramp_co2_cap = -9999.0          ! co2 ramp cap if rate>0, floor otherwise 

   logical :: doRamp_ghg    ! true => turn on ramping for ghg
   logical :: ramp_just_co2    ! true => turn on ramping for ghg
   logical :: ghgcyc    ! true => turn on ramping for ghg
   logical :: abrupt4xCO2    ! true => turn on ramping for ghg
   integer :: fixYear_ghg   ! year at which Ramped gases are fixed
   integer :: co2_start     ! date at which co2 begins ramping
   real(r8) :: co2_limit    ! value of co2vmr where ramping ends
   real(r8) :: co2_base     ! initial co2 volume mixing ratio, before any ramping
   integer :: ntim                ! number of yearly data values
   integer :: nlat,np1                    ! number of latitude grids
   integer :: nprev,nnext,ntmp
   integer :: ncid
   integer :: co2_id
   integer :: ch4_id
   integer :: n2o_id
   integer :: f11_id
   integer :: f12_id
   integer :: date_id
   integer :: sec_id
   integer :: time_id
   integer :: lat_id
   integer,  allocatable :: date(:) ! yearly data values
   integer,  allocatable :: datesec(:) ! yearly data values
   integer,  allocatable :: yrdata_em(:) ! yearly data values for emission data
   real(r8), allocatable :: co2total(:)    ! co2 total emission in TgC/yr   lijd, 2009.11.09
   real(r8), allocatable :: co2fossil(:)    ! co2 total emission in TgC/yr   lijd, 2009.11.09
   real(r8) cdayprev  ! dataset calendar day previous month
   real(r8) cdaynext  ! dataset calendar day next month

   real(r8), allocatable :: ch4vmrx(:,:,:)    ! ppbv
   real(r8), allocatable :: n2ovmrx(:,:,:)    ! ppbv
   real(r8), allocatable :: f11vmrx(:,:,:)    ! ppbv
   real(r8), allocatable :: f12vmrx(:,:,:)    ! ppbv
   
   real(r8), allocatable :: co2vmr_2d(:,:)    ! co2 mixing ratios in ppmv xinxg,201604
   real(r8), allocatable :: ch4vmr_2d(:,:)    ! ppbv
   real(r8), allocatable :: n2ovmr_2d(:,:)    ! ppbv
   real(r8), allocatable :: f11vmr_2d(:,:)    ! ppbv
   real(r8), allocatable :: f12vmr_2d(:,:)    ! ppbv
   type ghgco2_pointers
     real(r8), dimension(:,:,:), pointer :: ghgco2  ! (pcols,levsiz,begchunk:endchunk)
   end type ghgco2_pointers
    type (ghgco2_pointers) :: co2p(2)      ! pointers in 2 bounding months

  type ghgn2o_pointers
    real(r8), dimension(:,:,:), pointer :: ghgn2o  ! (pcols,levsiz,begchunk:endchunk)
  end type ghgn2o_pointers
    type (ghgn2o_pointers) :: n2op(2)      ! pointers in 2 bounding months

  
  type ghgch4_pointers
    real(r8), dimension(:,:,:), pointer :: ghgch4  ! (pcols,levsiz,begchunk:endchunk)
  end type ghgch4_pointers
   type (ghgch4_pointers) :: ch4p(2)

   type ghgf11_pointers
    real(r8), dimension(:,:,:), pointer :: ghgf11  ! (pcols,levsiz,begchunk:endchunk)
  end type ghgf11_pointers
   type(ghgf11_pointers) :: f11p(2)      ! pointers in 2 bounding months
  
  type ghgf12_pointers
    real(r8), dimension(:,:,:), pointer :: ghgf12  ! (pcols,levsiz,begchunk:endchunk)
  end type ghgf12_pointers
   type (ghgf12_pointers) :: f12p(2)      ! pointers in 2 bounding months
   


!
   integer  :: istep = 0              ! the accumulate number for co2 flux, wzz, 2009.2.18
   real(r8) :: co2f_glb = 0.0         ! the accumulated global CO2 flux
   real(r8) :: carb_glob(2) = 0.0     ! the global carbon flux (g/s)
   logical  :: cycl_just_co2 ! true => cycling to be done just for co2 and not other ghg's

!=========================================================================================
contains
!=========================================================================================

subroutine ghg_surfvals_init_2d()

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Initialize the ramp options that are controlled by namelist input.
! Set surface values at initial time.
! N.B. This routine must be called after the time manager has been initialized
!      since ghg_surfvals_set calls time manager methods.
! 
! Author: B. Eaton - merged code from parse_namelist and rampnl_ghg.
! 
!-----------------------------------------------------------------------

     if (scenario_ghg == 'RAMPED') then
      doRamp_ghg = .true.
      ghgcyc     = .false.
      ramp_just_co2 = .false.
      abrupt4xCO2 = .false.
     else if  (scenario_ghg == 'FIXED') then
      doRamp_ghg = .true.
      ramp_just_co2 = .false.
      ghgcyc     = .true.
      abrupt4xCO2 = .false.
     else if  (scenario_ghg == 'abrupt4xCO2') then
      doRamp_ghg = .true.
      ghgcyc     = .true.
      ramp_just_co2 = .false.
      abrupt4xCO2 = .true.
     else if  (scenario_ghg == 'RAMP_CO2_ONLY') then
       doRamp_ghg = .false.
       co2_start = ramp_co2_start_ymd
       ramp_just_co2 = .true.
       abrupt4xCO2 = .false.
     end if
 
      call ghg_initialize()

end subroutine ghg_surfvals_init_2d

!-------------------------------------------------------

function ghg_surfvals_ramp_2d()
   logical :: ghg_surfvals_ramp_2d
   ghg_surfvals_ramp_2d = doRamp_ghg
end function ghg_surfvals_ramp_2d

function co2_surfvals_ramp_2d()
   logical :: co2_surfvals_ramp_2d
   co2_surfvals_ramp_2d = ramp_just_co2
end function co2_surfvals_ramp_2d 


!=========================================================================================

subroutine ghg_initialize()

!----------------------------------------------------------------------- 
! 
! Purpose: 
! initialize the  greenhouse gas surface data.  
! 
! Author: xinxiaoge as read the ozone data
! 
!-----------------------------------------------------------------------

   use ioFileMod, only: getfil
   use rgrid, only: nlon
   use history,      only: addfld, phys_decomp
#if ( defined SPMD )
   use mpishorthand, only: mpicom, mpiint, mpir8
#endif

   include 'netcdf.inc'

!---------------------------Local variables-----------------------------
   integer :: ierror
   integer :: lchnk
   integer :: i,j,k,n
   integer dim_strt(2)
   integer dim_cnt(2)
   integer  :: yr, mon, day                ! components of a date
  integer  :: ncdate                      ! current date in integer format [yyyymmdd]
  integer  :: ncsec,time              ! current time of day [seconds]
  real(r8) :: calday                      ! current calendar day
  real(r8) :: caldayloc                      ! current calendar day

  real(r8), allocatable :: co2bdyprev3D(:,:,:) 
  real(r8), allocatable :: co2bdynext3D(:,:,:) 
  real(r8), allocatable :: n2obdyprev3D(:,:,:) 
 real(r8), allocatable :: n2obdynext3D(:,:,:) 
  real(r8), allocatable :: ch4bdyprev3D(:,:,:) 
  real(r8), allocatable :: ch4bdynext3D(:,:,:) 
  real(r8), allocatable :: f11bdyprev3D(:,:,:)
  real(r8), allocatable :: f11bdynext3D(:,:,:)
  real(r8), allocatable :: f12bdyprev3D(:,:,:)
  real(r8), allocatable :: f12bdynext3D(:,:,:)
  real(r8), allocatable :: co2bdyprev(:,:)
  real(r8), allocatable :: co2bdynext(:,:)
  real(r8), allocatable :: n2obdyprev(:,:)
  real(r8), allocatable :: n2obdynext(:,:)
  real(r8), allocatable :: ch4bdyprev(:,:)
  real(r8), allocatable :: ch4bdynext(:,:)
  real(r8), allocatable :: f11bdyprev(:,:) 
  real(r8), allocatable :: f11bdynext(:,:)
  real(r8), allocatable :: f12bdyprev(:,:) 
  real(r8), allocatable :: f12bdynext(:,:)

  character(len=256) :: locfn          ! netcdf local filename to open


   if (masterproc) then
     call getfil (bndtvghg_2d, locfn, 0)
     call wrap_open (trim(locfn), NF_NOWRITE, ncid)
  write(6,*)'GHG_RAMP_READ:  reading ramped greenhouse gas surface data from file ',trim(locfn)
     call wrap_inq_varid( ncid, 'date', date_id )
     call wrap_inq_varid( ncid, 'datesec', sec_id )
     call wrap_inq_varid( ncid, 'CO2', co2_id )
     call wrap_inq_varid( ncid, 'CH4', ch4_id )
     call wrap_inq_varid( ncid, 'N2O', n2o_id )
     call wrap_inq_varid( ncid, 'CFC11', f11_id )
     call wrap_inq_varid( ncid, 'CFC12', f12_id )
     call wrap_inq_dimid( ncid, 'time', time_id )
     call wrap_inq_dimlen( ncid, time_id, ntim )
     call wrap_inq_dimid( ncid, 'lat', lat_id )
     call wrap_inq_dimlen( ncid, lat_id, nlat )
!    if (ierror /= 0) then
!     write(6,*)'GHG_RAMP_READ:  ERROR, allocate() failed!'
!     call endrun
!   endif
  end if

#if (defined SPMD )
   call mpibcast (ntim, 1, mpiint, 0, mpicom)
   call mpibcast (nlat, 1, mpiint, 0, mpicom)
#endif
   ! these arrays are never deallocated
   allocate (date(ntim), datesec(ntim),stat=ierror )        
  nprev = 1
  nnext = 2

  allocate (co2p(nprev)%ghgco2(pcols,pver,begchunk:endchunk))
  allocate (co2p(nnext)%ghgco2(pcols,pver,begchunk:endchunk))

  allocate (ch4p(nprev)%ghgch4(pcols,pver,begchunk:endchunk))
  allocate (ch4p(nnext)%ghgch4(pcols,pver,begchunk:endchunk))

  allocate (n2op(nprev)%ghgn2o(pcols,pver,begchunk:endchunk))
 allocate (n2op(nnext)%ghgn2o(pcols,pver,begchunk:endchunk))


   allocate (f11p(nprev)%ghgf11(pcols,pver,begchunk:endchunk))
  allocate (f11p(nnext)%ghgf11(pcols,pver,begchunk:endchunk))


  allocate (f12p(nprev)%ghgf12(pcols,pver,begchunk:endchunk))
  allocate (f12p(nnext)%ghgf12(pcols,pver,begchunk:endchunk))


 allocate( co2vmrx(pcols,pver,begchunk:endchunk),n2ovmrx(pcols,pver,begchunk:endchunk), &
           ch4vmrx(pcols,pver,begchunk:endchunk),f11vmrx(pcols,pver,begchunk:endchunk), &
           f12vmrx(pcols,pver,begchunk:endchunk) )

 allocate( co2vmr_2d(pcols,pver),n2ovmr_2d(pcols,pver), &
           ch4vmr_2d(pcols,pver),f11vmr_2d(pcols,pver), &
           f12vmr_2d(pcols,pver) )

  allocate (co2bdyprev3D(plon,pver,plat))
  allocate (co2bdynext3D(plon,pver,plat))

  allocate (ch4bdyprev3D(plon,pver,plat))
  allocate (ch4bdynext3D(plon,pver,plat)) 

  allocate (f11bdyprev3D(plon,pver,plat))
  allocate (f11bdynext3D(plon,pver,plat))

  allocate (f12bdyprev3D(plon,pver,plat))
  allocate (f12bdynext3D(plon,pver,plat))

  allocate (n2obdyprev3D(plon,pver,plat))
  allocate (n2obdynext3D(plon,pver,plat))

   if (masterproc) then
      calday = get_curr_calday()
      if ( is_perpetual() ) then
         call get_perp_date(yr, mon, day, ncsec)
      else
         call get_curr_date(yr, mon, day, ncsec)
      end if
      ncdate = yr*10000 + mon*100 + day
    if (ghgcyc) then
      caldayloc = calday
    else
      caldayloc = calday + yr*365.
    end if
	 allocate (co2bdyprev(nlat,1),n2obdyprev(nlat,1),ch4bdyprev(nlat,1),f11bdyprev(nlat,1),f12bdyprev(nlat,1))
	 allocate (co2bdynext(nlat,1),n2obdynext(nlat,1),ch4bdynext(nlat,1),f11bdynext(nlat,1),f12bdynext(nlat,1))
	     call wrap_get_var_int   (ncid, date_id, date )
	     call wrap_get_var_int   (ncid, sec_id, datesec )
	    dim_strt(1) = 1
	    dim_cnt(2)  = 1
	    dim_cnt(1)  = nlat

       if (ghgcyc) then
         time = 12
         np1 = 1
         call bnddyi(date(time  ), datesec(time  ), cdayprev)
         call bnddyi(date(np1), datesec(np1), cdaynext)
         if (caldayloc <= cdaynext .or. caldayloc > cdayprev) then
            dim_strt(2) = time
            call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdyprev)
            call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdyprev)
            call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdyprev)
            call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdyprev)
            call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdyprev)

            dim_strt(2) = np1
             call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdynext)
             call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdynext)
             call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdynext)
             call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdynext)
             call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdynext)
            goto 10
         end if
      end if



	    do time=1,ntim-1
	      np1 = time + 1
	      call bnddyi(date(time), datesec(time), cdayprev)
	      call bnddyi(date(np1 ), datesec(np1 ), cdaynext)

              if (.not.ghgcyc) then
               yr = date(time)/10000
	       cdayprev = cdayprev + yr*365
	       yr = date(np1)/10000
	       cdaynext = cdaynext + yr*365.
              end if
	      if (caldayloc > cdayprev .and. caldayloc <= cdaynext) then
		dim_strt(2) = time
		call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdyprev)
               call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdyprev)
               call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdyprev)
               call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdyprev)
               call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdyprev)
               dim_strt(2) = np1
               call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdynext)
               call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdynext)
               call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdynext)
               call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdynext)
               call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdynext)
                goto 10
              endif
            end do
         time = 1
         np1 = time
          call bnddyi(date(time), datesec(time), cdayprev)
          call bnddyi(date(np1 ), datesec(np1 ), cdaynext)
              if (.not.ghgcyc) then
               yr = date(time)/10000
               cdayprev = cdayprev + yr*365
               yr = date(np1)/10000
               cdaynext = cdaynext + yr*365.
              end if
             if (caldayloc <= cdayprev) then
               dim_strt(2) = time
               call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdyprev)
               call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdyprev)
               call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdyprev)
               call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdyprev)
               call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdyprev)
               dim_strt(2) = np1
               call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdynext)
               call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdynext)
               call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdynext)
               call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdynext)
               call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdynext)
                go to 10
            end if       

         time = ntim-1
         np1 = time+1
         call bnddyi(date(time), datesec(time), cdayprev)
          call bnddyi(date(np1 ), datesec(np1 ), cdaynext)
             if (.not.ghgcyc) then
               yr = date(time)/10000
               cdayprev = cdayprev + yr*365
               yr = date(np1)/10000
               cdaynext = cdaynext + yr*365.
              end if
           if ( caldayloc > cdaynext) then
              dim_strt(2) = time
               call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdyprev)
               call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdyprev)
               call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdyprev)
               call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdyprev)
               call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdyprev)
               dim_strt(2) = np1
               call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdynext)
               call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdynext)
               call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdynext)
               call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdynext)
               call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdynext)
                go to 10  
           end if
       write(6,*)'ghg_initialize: Failed to find dates bracketing ncdate', ncdate
       call endrun
10  continue
    write(6,*)'ghg_initialize: Read greenhouse gas data for dates ',date(np1)
     
    do j=1,plat
    do k=1,plev
      do i=1,nlon(j)
       co2bdyprev3D(i,k,j) = co2bdyprev(j,1) 
       co2bdynext3D(i,k,j) = co2bdynext(j,1)  
       ch4bdyprev3D(i,k,j) = ch4bdyprev(j,1)  
       ch4bdynext3D(i,k,j) = ch4bdynext(j,1)  

       n2obdyprev3D(i,k,j) = n2obdyprev(j,1)  
       n2obdynext3D(i,k,j) = n2obdynext(j,1)  
       f11bdyprev3D(i,k,j) = f11bdyprev(j,1)
       f11bdynext3D(i,k,j) = f11bdynext(j,1) 
       f12bdyprev3D(i,k,j) = f12bdyprev(j,1)
       f12bdynext3D(i,k,j) = f12bdynext(j,1)
      end do
     end do
     end do
   call scatter_field_to_chunk(1,plev,1,plon,co2bdyprev3D,co2p(nprev)%ghgco2)
   call scatter_field_to_chunk(1,plev,1,plon,co2bdynext3D,co2p(nnext)%ghgco2)
   call scatter_field_to_chunk(1,plev,1,plon,ch4bdyprev3D,ch4p(nprev)%ghgch4)
   call scatter_field_to_chunk(1,plev,1,plon,ch4bdynext3D,ch4p(nnext)%ghgch4)
   call scatter_field_to_chunk(1,plev,1,plon,n2obdyprev3D,n2op(nprev)%ghgn2o)
   call scatter_field_to_chunk(1,plev,1,plon,n2obdynext3D,n2op(nnext)%ghgn2o)

   call scatter_field_to_chunk(1,plev,1,plon,f11bdyprev3D,f11p(nprev)%ghgf11)
   call scatter_field_to_chunk(1,plev,1,plon,f11bdynext3D,f11p(nnext)%ghgf11)

   call scatter_field_to_chunk(1,plev,1,plon,f12bdyprev3D,f12p(nprev)%ghgf12)
   call scatter_field_to_chunk(1,plev,1,plon,f12bdynext3D,f12p(nnext)%ghgf12)
#if (defined SPMD )
   else
   call scatter_field_to_chunk(1,pver,1,plon,co2bdyprev3D,co2p(nprev)%ghgco2)
   call scatter_field_to_chunk(1,pver,1,plon,co2bdynext3D,co2p(nnext)%ghgco2)
   call scatter_field_to_chunk(1,pver,1,plon,ch4bdyprev3D,ch4p(nprev)%ghgch4)
   call scatter_field_to_chunk(1,pver,1,plon,ch4bdynext3D,ch4p(nnext)%ghgch4)

   call scatter_field_to_chunk(1,pver,1,plon,n2obdyprev3D,n2op(nprev)%ghgn2o)
   call scatter_field_to_chunk(1,pver,1,plon,n2obdynext3D,n2op(nnext)%ghgn2o)

   call scatter_field_to_chunk(1,pver,1,plon,f11bdyprev3D,f11p(nprev)%ghgf11)
   call scatter_field_to_chunk(1,pver,1,plon,f11bdynext3D,f11p(nnext)%ghgf11)

   call scatter_field_to_chunk(1,pver,1,plon,f12bdyprev3D,f12p(nprev)%ghgf12)
   call scatter_field_to_chunk(1,pver,1,plon,f12bdynext3D,f12p(nnext)%ghgf12)
    deallocate(co2bdyprev3D)
    deallocate(co2bdynext3D)
    deallocate(ch4bdyprev3D)
    deallocate(ch4bdynext3D)
    deallocate(n2obdyprev3D)
    deallocate(n2obdynext3D)
    deallocate(f11bdyprev3D)
    deallocate(f11bdynext3D)

    deallocate(f12bdyprev3D)
    deallocate(f12bdynext3D)
#endif
   end if
  call  set_ghg_timeinterp(caldayloc)
  
#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom) 
   call mpibcast (date, ntim, mpiint, 0, mpicom )
   call mpibcast (datesec,ntim, mpiint, 0, mpicom )
   call mpibcast (cdayprev, 1, mpir8, 0, mpicom)
   call mpibcast (cdaynext, 1, mpir8, 0, mpicom)
#endif

   return

end subroutine ghg_initialize



subroutine ghg_ramp_read_2d
!----------------------------------------------------------------------- 
! 
! Purpose: 
! ramp the  greenhouse gas surface data.  
! 
! Author: xinxiaoge as read the ozone data
! 
!-----------------------------------------------------------------------

  use abortutils,   only: endrun
  use rgrid,        only: nlon
#if ( defined SPMD )
   use mpishorthand, only: mpicom, mpiint, mpir8
#endif

  integer :: dim_cnt(4)                ! array of counts for each dimension
  integer :: dim_strt(4)               ! array of starting indices
  integer :: i,j,k,n,c,ilev,icol 
  integer :: lchnk,ncols                     ! chunk index
  integer :: yr,mon,day                ! components of date
  integer :: ncdate                    ! formated date [yyyymmdd]
  integer :: ncsec                     ! seconds past midnight of current ncdate
  real(r8):: calday                    ! current calendar day
  real(r8):: caldayloc                    ! current calendar day
 
  real(r8), allocatable :: co2bdynext3D(:,:,:)
  real(r8), allocatable :: n2obdynext3D(:,:,:)
  real(r8), allocatable :: ch4bdynext3D(:,:,:)
  real(r8), allocatable :: f11bdynext3D(:,:,:)
  real(r8), allocatable :: f12bdynext3D(:,:,:)   
  real(r8), allocatable :: co2bdynext(:,:)
  real(r8), allocatable :: n2obdynext(:,:)
  real(r8), allocatable :: ch4bdynext(:,:)
  real(r8), allocatable :: f11bdynext(:,:)
  real(r8), allocatable :: f12bdynext(:,:)

 
  include 'netcdf.inc'

  calday = get_curr_calday()
  
   if ( is_perpetual() ) then
      call get_perp_date(yr, mon, day, ncsec)
   else
      call get_curr_date(yr, mon, day, ncsec)
   end if
   ncdate = yr*10000 + mon*100 + day

  if(ghgcyc ) then
   caldayloc = calday
  else
   caldayloc = calday + yr*365.
  end if


  if (caldayloc > cdaynext .and. & 
    (.not.(np1 == 1 .and. caldayloc > cdayprev).or.cdaynext==cdayprev)) then
    if(ghgcyc ) then
     np1 = mod(np1,12) + 1
    else
     np1 = np1 + 1
    end if
    if (np1 >ntim) then
      call endrun ('READ_GHG: Attempt to read past end of GHG dataset')
      np1=ntim
    end if
    cdayprev = cdaynext
      
    call bnddyi(date(np1), datesec(np1), cdaynext)
   if(.not.ghgcyc ) then
     yr = date(np1)/10000
     cdaynext = cdaynext + yr*365.
    end if   

      ntmp = nprev
      nprev = nnext
      nnext = ntmp
!
! Allocate memory for dynamic local workspace
!
      allocate (co2bdynext3D(plon,pver,plat))
      allocate (n2obdynext3D(plon,pver,plat))
      allocate (ch4bdynext3D(plon,pver,plat))
      allocate (f12bdynext3D(plon,pver,plat))
      allocate (f11bdynext3D(plon,pver,plat))
       if (masterproc) then
            dim_strt(1) = 1
            dim_strt(2) = np1
            dim_cnt(2)  = 1
            dim_cnt(1)  = nlat
        allocate ( co2bdynext(nlat,1),n2obdynext(nlat,1),ch4bdynext(nlat,1))
        allocate ( f11bdynext(nlat,1),f12bdynext(nlat,1))
          call wrap_get_vara_realx (ncid,co2_id,dim_strt,dim_cnt,co2bdynext)
          call wrap_get_vara_realx (ncid,n2o_id,dim_strt,dim_cnt,n2obdynext)
          call wrap_get_vara_realx (ncid,ch4_id,dim_strt,dim_cnt,ch4bdynext)        
          call wrap_get_vara_realx (ncid,f11_id,dim_strt,dim_cnt,f11bdynext)        
          call wrap_get_vara_realx (ncid,f12_id,dim_strt,dim_cnt,f12bdynext)        
        write(6,*)'READ_GHG: date from dataset ',date(np1)     
    do j=1,plat
    do k=1,plev
      do i=1,nlon(j)
       co2bdynext3D(i,k,j) = co2bdynext(j,1)  
       ch4bdynext3D(i,k,j) = ch4bdynext(j,1)  
       n2obdynext3D(i,k,j) = n2obdynext(j,1)  
       f11bdynext3D(i,k,j) = f11bdynext(j,1)
       f12bdynext3D(i,k,j) = f12bdynext(j,1)
      end do
    end do
    end do
    call scatter_field_to_chunk(1,plev,1,plon,co2bdynext3D,co2p(nnext)%ghgco2)
    call scatter_field_to_chunk(1,plev,1,plon,n2obdynext3D,n2op(nnext)%ghgn2o)
    call scatter_field_to_chunk(1,plev,1,plon,ch4bdynext3D,ch4p(nnext)%ghgch4)
    call scatter_field_to_chunk(1,plev,1,plon,f11bdynext3D,f11p(nnext)%ghgf11)
    call scatter_field_to_chunk(1,plev,1,plon,f12bdynext3D,f12p(nnext)%ghgf12)

    
   deallocate(co2bdynext3D) 
   deallocate(n2obdynext3D) 
   deallocate(ch4bdynext3D) 
   deallocate(f11bdynext3D) 
   deallocate(f12bdynext3D) 

  else  ! masterproc
    call scatter_field_to_chunk(1,plev,1,plon,co2bdynext3D,co2p(nnext)%ghgco2)
    call scatter_field_to_chunk(1,plev,1,plon,n2obdynext3D,n2op(nnext)%ghgn2o)
    call scatter_field_to_chunk(1,plev,1,plon,ch4bdynext3D,ch4p(nnext)%ghgch4)
    call scatter_field_to_chunk(1,plev,1,plon,f11bdynext3D,f11p(nnext)%ghgf11)
    call scatter_field_to_chunk(1,plev,1,plon,f12bdynext3D,f12p(nnext)%ghgf12)

   deallocate(co2bdynext3D)
   deallocate(n2obdynext3D)
   deallocate(ch4bdynext3D)
   deallocate(f11bdynext3D)
   deallocate(f12bdynext3D)
  endif ! masterproc
 end if

 call set_ghg_timeinterp(caldayloc)

#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (cdayprev, 1, mpir8, 0, mpicom)
   call mpibcast (cdaynext, 1, mpir8, 0, mpicom)
#endif

end subroutine ghg_ramp_read_2d

subroutine co2_ramp_time_2d
!----------------------------------------------------------------------- 
! 
! Purpose: 
! ramp the time for RAMP_CO2_ONLY   
! 
! Author: xinxiaoge 2016July
! 
!-----------------------------------------------------------------------


  real(r8):: calday                    ! current calendar day
  real(r8):: caldayloc                    ! current calendar day
  integer :: yr,mon,day,ncsec

 call get_curr_date(yr, mon, day, ncsec)
 calday = get_curr_calday()
 caldayloc = calday + yr*365.
 call set_ghg_timeinterp(caldayloc)
end subroutine co2_ramp_time_2d


subroutine set_ghg_timeinterp(caldayloc) 
!----------------------------------------------------------------------- 
! 
! Purpose: 
! timeinterp the ghg data between two months  
! 
! Author: xinxiaoge as read the ozone data
! 
!-----------------------------------------------------------------------

  use timeinterp, only: getfactors
  use time_manager, only: timemgr_datediff
  real(r8):: fact1,fact2
  real(r8) :: daydiff
  real(r8) :: caldayloc
  real(r8) :: co2_daily_factor 
  integer :: i,j,k,n,c,ilev,icol
  integer :: lchnk,ncols,ncdate,ncsec,yr,mon,day


  if ( is_perpetual() ) then
      call get_perp_date(yr, mon, day, ncsec)
   else
      call get_curr_date(yr, mon, day, ncsec)
   end if
   ncdate = yr*10000 + mon*100 + day

 if(cdayprev==cdaynext) then
   fact1 = 1.
   fact2 = 0.
   else
    call getfactors (ghgcyc, np1, cdayprev, cdaynext, caldayloc, &
                    fact1, fact2, 'GHGINT:')
   endif



  do c=begchunk,endchunk
  ncols = get_ncols_p(c)
  do ilev=1,plev
    do icol=1,ncols
      co2vmrx(icol,ilev,c) = (co2p(nprev)%ghgco2(icol,ilev,c)*fact1 + co2p(nnext)%ghgco2(icol,ilev,c)*fact2)*1.0e-6
      ch4vmrx(icol,ilev,c) = (ch4p(nprev)%ghgch4(icol,ilev,c)*fact1 + ch4p(nnext)%ghgch4(icol,ilev,c)*fact2)*1.0e-9
      n2ovmrx(icol,ilev,c) = (n2op(nprev)%ghgn2o(icol,ilev,c)*fact1 + n2op(nnext)%ghgn2o(icol,ilev,c)*fact2)*1.0e-9
      f11vmrx(icol,ilev,c) = (f11p(nprev)%ghgf11(icol,ilev,c)*fact1 + f11p(nnext)%ghgf11(icol,ilev,c)*fact2)*1.0e-12
      f12vmrx(icol,ilev,c) = (f12p(nprev)%ghgf12(icol,ilev,c)*fact1 + f12p(nnext)%ghgf12(icol,ilev,c)*fact2)*1.0e-12
    enddo
  enddo
  enddo

  if (abrupt4xCO2) then
    do c=begchunk,endchunk
     ncols = get_ncols_p(c)
    do ilev=1,plev
    do icol=1,ncols
      co2vmrx(icol,ilev,c) = co2vmrx(icol,ilev,c)*4.0
    end do
    end do
   end do   
  end if

  if (ramp_just_co2) then
      co2_daily_factor = (ramp_co2_annual_rate*0.01+1.0)**(1.0/365.0)
     call timemgr_datediff(co2_start, 0, ncdate, ncsec, daydiff)
      if (masterproc) then
        write(6,*) 'ncdate=', ncdate, 'co2_daily_factor**daydiff',co2_daily_factor,daydiff
      end if
    if  (daydiff > 0.0) then 
    do c=begchunk,endchunk
     ncols = get_ncols_p(c)
    do ilev=1,plev
    do icol=1,ncols
      co2vmrx(icol,ilev,c) = co2vmrx(icol,ilev,c)*(co2_daily_factor)**daydiff
    end do
    end do
   end do
   end if
  end if

 return
end subroutine set_ghg_timeinterp



!=========================================================================

 subroutine ghg_surfvals_get_co2vmr_2d(lchnk, ncol, co2vmr_2d)
   integer, intent(in) :: lchnk               ! chunk identifier
   integer , intent(in)   :: ncol             ! number of atmospheric columns
   real(r8), intent(out) :: co2vmr_2d(pcols,pver)
   integer i,j,k

  do i=1,ncol
  do j=1,pver
    co2vmr_2d(i,j) = co2vmrx(i,j,lchnk)
  end do
  end do
end subroutine ghg_surfvals_get_co2vmr_2d


!=========================================================================================
  subroutine ghg_surfvals_get_ghg_2d(lchnk,ch4vmr_2d,n2ovmr_2d,f11vmr_2d,f12vmr_2d)
   integer, intent(in) :: lchnk               ! chunk identifier
   real(r8), intent(out) :: ch4vmr_2d(pcols,pver)
   real(r8), intent(out) :: n2ovmr_2d(pcols,pver)
   real(r8), intent(out) :: f11vmr_2d(pcols,pver)
   real(r8), intent(out) :: f12vmr_2d(pcols,pver)
   integer i,j,k

  do i=1,pcols
  do j=1,pver
    ch4vmr_2d(i,j) = ch4vmrx(i,j,lchnk)
    n2ovmr_2d(i,j) = n2ovmrx(i,j,lchnk)
    f11vmr_2d(i,j) = f11vmrx(i,j,lchnk)
    f12vmr_2d(i,j) = f12vmrx(i,j,lchnk)
  end do
  end do
 end subroutine ghg_surfvals_get_ghg_2d

end module ghg_surfvals_2d
