#include <misc.h>
#include <params.h>

SUBROUTINE Lagrain(lchnk   ,ncol   ,ztodt  ,&
                  pmid    ,pdel    ,t      ,q      , &
				  rainls  ,snowls   )                   
!-----------------------------------------------------------------------
! purpose: ESTIMATE LARGE SCALE UNCONVECTIVE LATENT HEAT RELEASE     
!                    AND PRECIPITATION (RAIN:MM)                        
! author: wu tongwen
!----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid      , only: pver, pverp, pcols
   implicit none
!
! Arguments
!
   integer, intent(in) :: lchnk               ! chunk identifier
   integer, intent(in) :: ncol                ! number of atmospheric columns

   real(r8), intent(in) :: pmid(pcols,pver)   ! pressure at model levels (Pa)
   real(r8), intent(in) :: pdel(pcols,pver)   ! vertical delta-p (Pa)
   real(r8), intent(in) :: ztodt              ! 2 delta t (model time increment)
!                    
! Input/output arguments
!
   real(r8), intent(inout) :: t(pcols,pver)        ! temperature (K)
   real(r8), intent(inout) :: q(pcols,pver)        ! specific humidity

   real(r8), intent(out)   :: rainls(pcols)         ! large scale precipitation (m/s)
   real(r8), intent(out)   :: snowls(pcols)         ! large scale snowfall (m/s)
!
!---------------------------Local workspace-----------------------------
!
   integer i, k
   
   real(r8)  pmid0  ! hPa
   real(r8)  pdel0  ! hPa
   real(r8)  relag, gqs
   real(r8)  ES
   external  relag
   external  gqs

   real(r8)  QS
   real(r8)  GPR
   real(r8)  DQ ! specific humidity DIFFERENCE DUE TO LG.SCL. precipitation
   real(r8)  DT ! TEMP. DIFFERENCE DUE TO LG.SCL.LATENT HEAT .RELEASE 
   real(r8)  FR

   DATA GPR/10.1973/                                                 

   FR = 1.0     ! here, the moisture can be over-saturated .  wtw
!-------------------------------------------------------
do i=1,ncol

   rainls(i) = 0.0
   snowls(i) = 0.0

   do k=1, pver-2  ! assumed there is no rain in the bottom layer 
      pmid0 = pmid(i,k)/100.0        ! Pa-> hPa
      pdel0 = pdel(i,k)/100.0
      
         QS=GQS(t(i,k), pmid0, ES)                                                    
         DQ=RELAG(t(i,k), q(i,k), QS, pmid0, ES, FR, DT)                                       
!
         rainls(i)= rainls(i) + pdel0*GPR*DQ    ! mm water
         t(i, k) = t(i,k) + dt
   enddo
   rainls(i) = rainls(i)/ztodt*1.0e-03            ! m/s large scale precipitation
   
   if(t(i,pver).lt.(273.16+2.0).and.rainls(i).gt.0.0) then
         snowls(i) = rainls(i)
		 rainls(i) = 0.0
   else
         snowls(i) = 0.0
   endif
enddo
return
END SUBROUTINE Lagrain                                                            


FUNCTION RELAG(T,Q,QS,P,ES,FR,DT)                                 
! ----------------------------------------------------------------------
! purpose: *ITERATION METHOD IS USED TO RELAX T AND Q DURING LARGE    
!                 SCALE HEAT RELEASING                                  
!          *FUNCTION RELAG(T,Q,QS,P,ES,FR,DT)                           
!          *INPUT:     T: TEMPERATURE                                   
!                      Q: SPECIFIC HUMIDITY                             
!                     QS: SATURATION SPECIFIC HUMIDITY                  
!                      P: PRESSURE                                      
!                     FR: CRITICAL RELATIVE HUMIDITY                    
!          *OUTPUT:RELAG: DIFFERENCE OF Q DUE TO L.H.R.                 
!                     DT: DIFFERENCE OF T DUE TO L.H.R                  
!          *GQS                                                         
!     .......................................                           
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
!
   real(r8) T
   real(r8) Q
   real(r8) QS
   real(r8) P
   real(r8) ES
   real(r8) FR
   real(r8) DT

   real(r8) gqs, ern, t1, t2, relag
   external gqs
!
   real(r8) cpl, b, c, alfa, dtmin, t0
   DATA  CPL/2489.139/ ,    B/24.77514/,  C/  4.9283/   
   data  ALFA/0.378/   , DTMIN/0.005/,     T0/273.15/                
!------------------------------------------------------------
   QS=QS*FR                                                          
   IF(Q.LE.QS) then
      DT=0.0                                                            
      RELAG=0.0                                                         
      RETURN                                                            
   else
      T1=T+CPL*(Q-QS)/(CPL*QS*P/(P-ALFA*ES)*(B*T0/T-C)/T+1.0)           
  100 QS=GQS(T1,P,ES)*FR                                                
      ERN=T1-(T+CPL*(Q-QS))                                             
      T2=T1-ERN/(1.0+CPL*QS*P/(P-ALFA*ES)*(B*T0/T1-C)/T1)               
      IF(ABS(T2-T1).LE.DTMIN) then
         QS=GQS(T2,P,ES)*FR                                                
         DT=T2-T                                                           
         RELAG=Q-QS                                                        
         Q= QS
         RETURN                                                            
	  else
         T1=T2                                                             
         GO TO 100                                                         
      endif
   endif
   RETURN                                                            
END FUNCTION RELAG                                                              

FUNCTION GQS(T,P,E)                                               
!-----------------------------------------------------------------------
! purpose: *CALCULATE SATURATION SPECIFIC HUMIDITY                      
!          *FUNCTION GQS(T,P,E)                                         
!          *INPUT:    T: TEMPERATURE                                    
!                     P: PRESSURE                                       
!          *OUTPUT:   E: SATURATION WATER VAPOR PRESSURE                
!                   GQS: SATURATION SPECIFIC HUMIDITY.                  
!------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use wv_saturation, only: estblf
   implicit none
!
   real(r8) t
   real(r8) P
   real(r8) E
!
   real(r8) E0, B, C, ALFA
   real(r8) X
   real(r8) GQS
   DATA E0/6.10875/,B/24.7751/,C/4.9283/,ALFA/0.378/                 
!-----------------------------------------------------------
   X=273.16/T                                                        
!-----------
!   E=E0*EXP(B*(1.0-X))*X**C                                 
   E = estblf(T)/100.0
!---------------------------
   GQS=0.622*E/(P-ALFA*E)                                            
   GQS=AMAX1(GQS,1.0E-10)
   GQS=AMIN1(GQS,1.0    )
!----------------------------------
   RETURN                                                            
END FUNCTION GQS
                                                              
