!+ master routine - provides interface for: cumastr (cumulus parameterization)

!#define DEBUG

!sxlMODULE m_cucall

!sxlCONTAINS

SUBROUTINE cucall(klp2,k2lp2,klon,klev,klevp1,klevm1,ztodt,&
                  ptm1,pqm1,pum1,pvm1,pxm1,ptte,pqte,pvom,pvol,pxte,pverv,pqhfl,     &
                  papp1,paphp1,pgeo,ldland,ttend,utend,vtend,qtend,precc,pxtec,ptopmax, &
                  cnb,zdu,cmfmc,mu2,md2,du2,eu2,ed2,ptate,pqate)


!SUBROUTINE cucall(klp2,k2lp2,klon,klev,klevp1,klevm1,ilab,&
!                  ptm1,pqm1,pum1,pvm1,pxm1,ptte,pqte,pvom,pvol,pxte,pverv,pqhfl,pxtec,     &
!                  papp1,paphp1,pgeo,prsfc,pssfc,paprc,paprs,ktype,ldland,ptopmax,ptopmaxm, &
!                  cnb,cmfmc,zlude,ztodt,mu2,md2,du2,eu2,ed2)


  ! ------------------> ldland included for AMIP2

  ! Description:
  !
  ! Master routine - provides interface for: cumastr (cumulus parameterization)
  !
  ! Method:
  !
  ! *cucall* - interface for *cumastr*:
  ! Provides input for cumastr.
  ! Receives updated tendencies, precipitation.
  !
  ! *cucall* is called from *physc*
  !
  ! Authors:
  !
  ! M. Tiedtke, ECMWF, December 1989, original source
  ! L. Kornblueh, MPI, May 1998, f90 rewrite
  ! U. Schulzweida, MPI, May 1998, f90 rewrite
  ! A. Rhodin, MPI, December 1998, lookup tables removed
  ! 
  ! for more details see file AUTHORS
  !

  USE mo_cumulus_flux,   ONLY: cuparam
  USE time_manager,      ONLY: is_first_step,         &! initial run
                               is_first_restart_step   ! restart run 
  USE mo_constants,      ONLY: vtmpc1,  &! vtmpc1=rv/rd-1
                               inicon, cpd, g
  USE mo_convect_tables, ONLY: tlucua, jptlucu1, jptlucu2, &
                               lookuperror, lookupoverflow, &
                               set_lookup_tables
  IMPLICIT NONE

  !  Scalar arguments with intent(In):
  INTEGER, INTENT (IN) :: k2lp2, klev, klevm1, klevp1, klon, klp2

  !  Array arguments with intent(In):
  REAL, INTENT (IN) :: paphp1(klp2,klevp1), papp1(klp2,klev), pgeo(klp2,klev), &
                       pqhfl(klp2), pqm1(k2lp2,klev), ptm1(k2lp2,klev),        &
                       pum1(k2lp2,klev), pverv(klp2,klev),     &
                       pvm1(k2lp2,klev), pxm1(klp2,klev), pxte(klp2,klev),     &
                       pqte(klp2,klev),  ptte(klp2,klev), pvol(k2lp2,klev),    &
                       pvom(k2lp2,klev), ztodt, pqate(klp2,klev),  ptate(klp2,klev)
  LOGICAL, INTENT (IN) :: ldland(klp2)    ! INCLUDED FOR AMIP2 !!

  !  Scalar arguments with intent(InOut):

  !  Array arguments with intent(InOut):
  REAL, INTENT (OUT)   :: ptopmax(klp2), precc(klp2),  &                          
                          pxtec(klp2,klev),ttend(klp2,klev), utend(klp2,klev),  &
                          vtend(klp2,klev),qtend(klp2,klev), zdu(klp2,klev),    &
                          cmfmc(klp2,klev),cnb(klp2),mu2(klp2,klev),md2(klp2,klev),&
                          du2(klp2,klev),eu2(klp2,klev),ed2(klp2,klev)
    

!sxl  INTEGER, INTENT (INOUT) :: ilab(klp2,klev), ktype(klp2)

  !  Local scalars: 
  REAL    :: ztmst
  INTEGER :: ilevmin, jk, jl, jt
  INTEGER :: it
  INTEGER :: ilab(klp2,klev), ktype(klp2) 

  !  Local arrays: 
  REAL :: zlu(klp2,klev), zlude(klp2,klev), zmfd(klp2,klev), zmfu(klp2,klev), &
          zqp1(klp2,klev), zqsat(klp2,klev), zqu(klp2,klev), zrain(klp2), &
          ztp1(klp2,klev), ztu(klp2,klev), zup1(klp2,klev), zvp1(klp2,klev), &
          zxp1(klp2,klev)
  INTEGER :: icbot(klp2), ictop(klp2), itopec2(klp2)
  LOGICAL :: locum(klp2)
  REAL :: paprc(klp2), paprs(klp2), prsfc(klp2), pssfc(klp2), ptopmaxm(klp2)


  !  External subroutines 
  EXTERNAL cumastr

  !  Intrinsic functions 
  INTRINSIC MIN

  !  Executable statements 

  lookupoverflow = .FALSE.

!-- 1. Calculate t,q and qs at main levels

  ztmst = ztodt
!s  IF (is_first_step())  ztmst = 0.5*ztodt
  IF (is_first_step().or.is_first_restart_step()) then
    CALL inicon()
    CALL cuparam()
    CALL set_lookup_tables
  END IF
  DO jk = 1, klev
    DO jl = 1, klon
      ztp1(jl,jk) = ptm1(jl,jk) + ptte(jl,jk)*ztmst
      zqp1(jl,jk) = pqm1(jl,jk) + pqte(jl,jk)*ztmst
      zxp1(jl,jk) = pxm1(jl,jk) + pxte(jl,jk)*ztmst
      zup1(jl,jk) = pum1(jl,jk) + pvom(jl,jk)*ztmst
      zvp1(jl,jk) = pvm1(jl,jk) + pvol(jl,jk)*ztmst
      ttend(jl,jk)= ptte(jl,jk)
      qtend(jl,jk)= pqte(jl,jk)
      utend(jl,jk)= pvom(jl,jk)
      vtend(jl,jk)= pvol(jl,jk)  
      it = INT(ztp1(jl,jk)*1000.)
      IF (it<jptlucu1 .OR. it>jptlucu2) lookupoverflow = .TRUE.
      it = MAX(MIN(it,jptlucu2),jptlucu1)
      zqsat(jl,jk) = tlucua(it)/papp1(jl,jk)
      zqsat(jl,jk) = MIN(0.5,zqsat(jl,jk))
      zqsat(jl,jk) = zqsat(jl,jk)/(1.-vtmpc1*zqsat(jl,jk))
    END DO
    IF (lookupoverflow) CALL lookuperror ('cucall      ')

  END DO

  DO jl = 1, klon
    zrain(jl) = 0.
    locum(jl) = .FALSE.
  END DO
!-- 2. Call 'cumastr'(master-routine for cumulus parameterization)
!      -----------------------------------------------------------
  CALL cumastr(klp2,k2lp2,klon,klev,klevp1,klevm1,ilab,ztp1,zqp1,zxp1,zup1,          &
               zvp1,ldland,pverv,zqsat,pqhfl,papp1,paphp1,pgeo,ptte, &
               pqte,pvom,pvol,prsfc,pssfc,paprc,paprs,pxtec,locum,ktype,icbot,ictop, &
               ztu,zqu,zlu,zlude,zmfu,zmfd,zrain,ztodt,du2,eu2,ed2,ptate,pqate)

!-- 3. Pressure altitude of convective cloud tops
!  ilevmin = klev - 4

!  DO jl = 1, klon
!    itopec2(jl) = klevp1
!  END DO

!  DO jk = 1, ilevmin
!    DO jl = 1, klon
!      IF (ilab(jl,jk)==2 .AND. itopec2(jl)==klevp1) THEN
!        itopec2(jl) = jk
!      END IF
!    END DO
!  END DO

!  DO jl = 1, klon
!    IF (itopec2(jl)==1) THEN
!      ptopmax(jl) = papp1(jl,1)
!    ELSE IF (itopec2(jl)/=klevp1) THEN
!      ptopmax(jl) = paphp1(jl,itopec2(jl))
!    ELSE
!      ptopmax(jl) = 99999.
!    END IF
!    ptopmax(jl) = MIN(ptopmax(jl),ptopmaxm(jl))
!  END DO

  cnb(:)=icbot(:)
! ptopmax(:)=itopec2(:)
  ptopmax(:)=ictop(:)
  cmfmc(:,:)=zmfu(:,:)+zmfd(:,:)
  mu2(:,:)=zmfu(:,:)
  md2(:,:)=zmfd(:,:)
! zdu(:,:)=du2(:,:)
  precc(:) = (prsfc(:) + pssfc(:))/1000.
  DO jk = 1, klev
    DO jl = 1, klon
      ttend(jl,jk)= (ptte(jl,jk)-ttend(jl,jk))*cpd
      qtend(jl,jk)= pqte(jl,jk)-qtend(jl,jk)
      utend(jl,jk)= pvom(jl,jk)-utend(jl,jk)
      vtend(jl,jk)= pvol(jl,jk)-vtend(jl,jk)
      zdu  (jl,jk)= 0.0
    END DO
  END DO

  DO jk = klevm1, 2, -1
     DO jl = 1, klon
       zdu(jl,jk) = du2(jl,jk)*g/(paphp1(jl,jk+1)-paphp1(jl,jk))
     END DO
  END DO



! DO jl = 1, klon
!   if(ktype(jl)==1) then
!     write(6,*) 'cnt=',ptopmax(jl),'cnb=',cnb(jl)
!   end if
! END DO
!  DO jl = 1, klon
!     DO jk = 1, klev 
!        write(6,*) 'pxtec=',pxtec(jl,jk),'jk=',jk,'jl=',jl
!     if(zdu(jl,jk)>1.) write(6,*) 'du2=',du2(jl,jk),'jk=',jk,'jl=',jl
!     if(ktype(jl)==1 .and. zdu(jl,jk)>0.) write(6,*) 'zdu=',zdu(jl,jk),'jk=',jk,'jl=',jl
!     END DO
!  END DO


  RETURN
END SUBROUTINE cucall

!sxlEND MODULE m_cucall
