#include <misc.h>
#include <params.h>


#ifdef MAC_SP
module mo_simple_plumes
!=========================================================================
!>
!!
!! @brief Module MO_SIMPLE_PLUMES: provides anthropogenic aerorol optial properties as a function of lat, lon
!!   height, time, and wavelength
!!
!! @remarks
!!
!! @author Bjorn Stevens & Karsten Peters MPI-M, Hamburg (v1-beta release 2015-12-05)
!!
!! @revisions:
!!    2016-06-20: Bug fix in array indexing in of look distributing plume properties over profile
!!    2016-06-20: Removal of beta flag and reference to beta input.  Final version 1 release.
!!
!! $ID: n/a$
!!
!! @par Origin
!!   Based on code originally developed at the MPI by Karsten Peters, Bjorn Stevens, Stephanie Fiedler
!!   and Stefan Kinne with input from Thorsten Mauritsen and Robert Pincus
!!
!! @par Copyright
!! 
!==================================================================


  use shr_kind_mod,only: r8=>shr_kind_r8 !precision used by reals
  use abortutils,only: endrun
  use filenames,    only: bndmacsp

  IMPLICIT NONE
  INTEGER, PARAMETER ::                        &
       nplumes   = 9                          ,& !< Number of plumes
       nfeatures = 2                          ,& !< Number of features per plume
       ntimes    = 52                         ,& !< Number of times resolved per year (52 => weekly resolution)
       nyears    = 252                           !< Number of years of available forcing

  REAL(r8), PARAMETER    ::                        &
       pi      = 2.*ASIN(1.),                  & !< half the unit circle (radians)
       deg2rad = pi/180.                         !< conversion factor from degrees to radians

  LOGICAL, SAVE ::                             &
       sp_initialized = .FALSE.                  !< parameter determining whether input needs to be read

  REAL(r8) ::                                      &
       plume_lat   (nplumes)                  ,& !< latitude where plume maximizes
       plume_lon   (nplumes)                  ,& !< longitude where plume maximizes
       beta_a      (nplumes)                  ,& !< parameter a for beta function vertical profile
       beta_b      (nplumes)                  ,& !< parameter b for beta function vertical profile
       aod_spmx    (nplumes)                  ,& !< aod at 550 for simple plume (maximum)
       aod_fmbg    (nplumes)                  ,& !< aod at 550 for fine mode background (for twomey effect)
       asy550      (nplumes)                  ,& !< asymmetry parameter for plume at 550nm
       ssa550      (nplumes)                  ,& !< single scattering albedo for plume at 550nm
       angstrom    (nplumes)                  ,& !< angstrom parameter for plume 
       sig_lon_E   (nfeatures,nplumes)        ,& !< Eastward extent of plume feature
       sig_lon_W   (nfeatures,nplumes)        ,& !< Westward extent of plume feature
       sig_lat_E   (nfeatures,nplumes)        ,& !< Southward extent of plume feature
       sig_lat_W   (nfeatures,nplumes)        ,& !< Northward extent of plume feature
       theta       (nfeatures,nplumes)        ,& !< Rotation angle of feature
       ftr_weight  (nfeatures,nplumes)        ,& !< Feature weights = (nfeatures + 1) to account for BB background
       time_weight (nfeatures,nplumes)        ,& !< Time-weights = (nfeatures +1) to account for BB background
       year_weight (nyears,nplumes)           ,& !< Yearly weight for plume
       ann_cycle   (nfeatures,ntimes,nplumes)    !< annual cycle for feature
  INTEGER :: years (nyears)

  PUBLIC sp_aop_profile
  PUBLIC sp_setup

CONTAINS
  !
  ! ------------------------------------------------------------------------------------------------------------------------
  ! SP_SETUP:  This subroutine should be called at initialization to read the netcdf data that describes the simple plume
  ! climatology.  The information needs to be either read by each processor or distributed to processors.
  !
  SUBROUTINE sp_setup
    !
    ! ---------- 
    !
    include 'netcdf.inc'

    INTEGER :: iret, ncid, DimID, VarID, xdmy
    !
    ! ---------- 
    !    
      call WRAP_OPEN(bndmacsp,0,ncid)
    !
    ! read dimensions and make sure file conforms to expected size
    !
      call WRAP_INQ_DIMID (ncid, "plume_number"  , DimId)
      call WRAP_INQ_DIMLEN (ncid, DimId, xdmy)
      IF (xdmy /= nplumes) STOP 'NetCDF improperly dimensioned -- plume_number'

      call WRAP_INQ_DIMID (ncid, "plume_feature"  , DimId)
      call WRAP_INQ_DIMLEN (ncid, DimId, xdmy)
      IF (xdmy /= nfeatures) STOP 'NetCDF improperly dimensioned -- plume_feature' 
      
      
      call WRAP_INQ_DIMID (ncid, "year_fr"  , DimId)
      call WRAP_INQ_DIMLEN (ncid, DimId, xdmy)
      IF (xdmy /= ntimes) STOP 'NetCDF improperly dimensioned -- year_fr'

      call WRAP_INQ_DIMID (ncid, "years"  , DimId)
      call WRAP_INQ_DIMLEN (ncid, DimId, xdmy)
      IF (xdmy /= nyears) STOP 'NetCDF improperly dimensioned -- years'


    !
    ! read variables that define the simple plume climatology
    !
      call WRAP_INQ_VARID (ncid, "years"  , VarId)
      call wrap_get_vara_int (ncid, Varid, 1, nyears, years)

      call WRAP_INQ_VARID (ncid, "plume_lat"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes, plume_lat(:))

      call WRAP_INQ_VARID (ncid, "plume_lon"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes, plume_lon(:))

      call WRAP_INQ_VARID (ncid, "beta_a"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,beta_a(:))

      call WRAP_INQ_VARID (ncid, "beta_b"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,beta_b(:))
      
      
      call WRAP_INQ_VARID (ncid, "aod_spmx"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,aod_spmx(:))
      
      call WRAP_INQ_VARID (ncid, "aod_fmbg"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,aod_fmbg(:))
      
      call WRAP_INQ_VARID (ncid, "ssa550"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,ssa550(:))

      call WRAP_INQ_VARID (ncid, "asy550"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,asy550(:))
    
      call WRAP_INQ_VARID (ncid, "angstrom"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, 1, nplumes,angstrom(:))

      call WRAP_INQ_VARID (ncid, "sig_lat_W"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nfeatures,nplumes/),sig_lat_W(:,:))

      call WRAP_INQ_VARID (ncid, "sig_lat_E"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nfeatures,nplumes/),sig_lat_E(:,:))

      call WRAP_INQ_VARID (ncid, "sig_lon_E"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nfeatures,nplumes/),sig_lon_E(:,:))
  
            call WRAP_INQ_VARID (ncid, "sig_lon_W"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nfeatures,nplumes/),sig_lon_W(:,:))

      call WRAP_INQ_VARID (ncid, "theta"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nfeatures,nplumes/),theta(:,:))

      call WRAP_INQ_VARID (ncid, "ftr_weight"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nfeatures,nplumes/),ftr_weight(:,:))

      call WRAP_INQ_VARID (ncid, "year_weight"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1/), (/nyears,nplumes/),year_weight(:,:))

      call WRAP_INQ_VARID (ncid, "ann_cycle"  , VarId)
      call WRAP_GET_VARA_REALX(ncid, VarId, (/1,1,1/), (/nfeatures,ntimes,nplumes/),ann_cycle(:,:,:))

      call wrap_close(ncid)

      sp_initialized = .TRUE.

    RETURN
  END SUBROUTINE sp_setup

  !
  ! ------------------------------------------------------------------------------------------------------------------------
  ! SET_TIME_WEIGHT:  The simple plume model assumes that meteorology constrains plume shape and that only source strength
  ! influences the amplitude of a plume associated with a given source region.   This routine retrieves the temporal weights
  ! for the plumes.  Each plume feature has its own temporal weights which varies yearly.  The annual cycle is indexed by
  ! week in the year and superimposed on the yearly mean value of the weight. 
  !
  SUBROUTINE set_time_weight(year_fr)
    !
    ! ---------- 
    !
    REAL(r8), INTENT(IN) ::  &
         year_fr           !< Fractional Year (1850.0 - 2100.99)

    INTEGER          ::  &
         idate(8)       ,& !< integer array of system clock date yyyy, mm, dd
         iyear          ,& !< Integer year values between 1 and 156 (1850-2100) 
         iweek          ,& !< Integer index (between 1 and ntimes); for ntimes=52 this corresponds to weeks (roughly)
         iplume            ! plume number
    !
    ! ---------- 
    !
!   iyear = FLOOR(year_fr) - 1850 + 1
    iyear = FLOOR(year_fr) - years(1) + 1
    iweek = FLOOR((year_fr - FLOOR(year_fr)) * ntimes) + 1

    IF ((iweek > ntimes) .OR. (iweek < 1) .OR. (iyear > nyears) .OR. (iyear < 1)) STOP 'Time out of bounds in set_time_weight'
    DO iplume=1,nplumes
      time_weight(1,iplume) = year_weight(iyear,iplume) * ann_cycle(1,iweek,iplume)
      time_weight(2,iplume) = year_weight(iyear,iplume) * ann_cycle(2,iweek,iplume)
    END DO
    
    RETURN
  END SUBROUTINE set_time_weight
  !
  ! ------------------------------------------------------------------------------------------------------------------------
  ! SP_AOP_PROFILE:  This subroutine calculates the simple plume aerosol and cloud active optical properites based on the
  ! the simple plume fit to the MPI Aerosol Climatology (Version 2).  It sums over nplumes to provide a profile of aerosol
  ! optical properties on a host models vertical grid. 
  !
  SUBROUTINE sp_aop_profile                                                                           ( &
       nlevels        ,ncol_max       ,ncol           ,lambda         ,oro            ,lon            , &
       lat            ,year_fr        ,z              ,dz             ,dNovrN         ,aod_prof       , &
       ssa_prof       ,asy_prof        )
    !
    ! ---------- 
    !
    INTEGER, INTENT(IN)            :: &
         nlevels,                     & !< number of levels
         ncol_max,                    & !< maximum number of columns
         ncol                           !< number of columns

    REAL(r8), INTENT(IN)               :: &
         lambda,                      & !< wavelength
         year_fr,                     & !< Fractional Year (1903.0 is the 0Z on the first of January 1903, Gregorian)
         oro(ncol_max),               & !< orographic height (m)
         lon(ncol_max),               & !< longitude 
         lat(ncol_max),               & !< latitude
         z (ncol_max,nlevels),        & !< height above sea-level (m)
         dz(ncol_max,nlevels)           !< level thickness (difference between half levels)

    REAL(r8), INTENT(OUT)              :: &
         dNovrN(ncol_max)           , & !< anthropogenic incroment to cloud drop number concentration 
         aod_prof(ncol_max,nlevels) , & !< profile of aerosol optical depth
         ssa_prof(ncol_max,nlevels) , & !< profile of single scattering albedo
         asy_prof(ncol_max,nlevels)     !< profile of asymmetry parameter

    INTEGER                        :: iplume, icol, k

    REAL(r8)                           :: &
         eta(ncol_max,nlevels),       & !< normalized height (by 15 km)
         z_beta(ncol_max,nlevels),    & !< profile for scaling column optical depth
         prof(ncol_max,nlevels),      & !< scaled profile (by beta function)
         beta_sum(ncol_max),          & !< vertical sum of beta function
         ssa(ncol_max),               & !< aerosol optical depth 
         asy(ncol_max),               & !< aerosol optical depth 
         cw_an(ncol_max),             & !< column weight for simple plume (anthropogenic) aod at 550 nm
         cw_bg(ncol_max),             & !< column weight for fine-mode background aod at 550 nm
         caod_sp(ncol_max),           & !< column simple plume (anthropogenic) aod at 550 nm
         caod_bg(ncol_max),           & !< column fine-mode background aod at 550 nm
         a_plume1,                    & !< gaussian longitude factor for feature 1
         a_plume2,                    & !< gaussian longitude factor for feature 2
         b_plume1,                    & !< gaussian latitude factor for feature 1
         b_plume2,                    & !< gaussian latitude factor for feature 2
         delta_lat,                   & !< latitude offset
         delta_lon,                   & !< longitude offset
         lon1,                        & !< rotated longitude for feature 1
         lat1,                        & !< rotated latitude for feature 2
         lon2,                        & !< rotated longitude for feature 1
         lat2,                        & !< rotated latitude for feature 2
         f1,                          & !< contribution from feature 1
         f2,                          & !< contribution from feature 2
         aod_550,                     & !< aerosol optical depth at 550nm
         aod_lmd,                     & !< aerosol optical depth at input wavelength
         lfactor                        !< factor to compute wavelength dependence of optical properties
    !
    ! ----------
    !
    ! check array bounds
    !
    IF (ncol .gt. ncol_max) STOP 'Array out of bounds in mo_simple-plumes_v2'
    !
    ! initialize input data (by calling setup at first instance) 
    !
    IF (.NOT.sp_initialized) CALL sp_setup
    !
    ! get time weights
    !
    CALL set_time_weight(year_fr)
    !
    ! initialize variables, including output
    !
    DO k=1,nlevels
      DO icol=1,ncol
        aod_prof(icol,k) = 0.0
        ssa_prof(icol,k) = 0.0
        asy_prof(icol,k) = 0.0
        z_beta(icol,k)   = MERGE(1.0, 0.0, z(icol,k) >= oro(icol))
        eta(icol,k)      = MAX(0.0,MIN(1.0,z(icol,k)/15000.))
      END DO
    END DO
    DO icol=1,ncol
      dNovrN(icol)   = 1.0
      caod_sp(icol)  = 0.00
      caod_bg(icol)  = 0.02
    END DO
    !
    ! sum contribution from plumes to construct composite profiles of aerosol otpical properties
    !
    DO iplume=1,nplumes
      !
      ! calculate vertical distribution function from parameters of beta distribution
      !
      DO icol=1,ncol
        beta_sum(icol) = 0.
      END DO
      DO k=1,nlevels
        DO icol=1,ncol
          prof(icol,k)   = (eta(icol,k)**(beta_a(iplume)-1.) * (1.-eta(icol,k))**(beta_b(iplume)-1.))*dz(icol,k)
          beta_sum(icol) = beta_sum(icol) + prof(icol,k)
        END DO
      END DO
      DO k=1,nlevels
        DO icol=1,ncol
          prof(icol,k)   = prof(icol,k) / beta_sum(icol) * z_beta(icol,k)
        END DO
      END DO
      !
      ! calculate plume weights
      !
      DO icol=1,ncol
        !
        ! get plume-center relative spatial parameters for specifying amplitude of plume at given lat and lon
        !
        delta_lat = lat(icol) - plume_lat(iplume)
        delta_lon = lon(icol) - plume_lon(iplume)
        delta_lon = MERGE ( delta_lon-SIGN(360.,delta_lon) , delta_lon , ABS(delta_lon) > 180. )
        a_plume1  = 0.5 / (MERGE(sig_lon_E(1,iplume), sig_lon_W(1,iplume), delta_lon > 0)**2)
        b_plume1  = 0.5 / (MERGE(sig_lat_E(1,iplume), sig_lat_W(1,iplume), delta_lon > 0)**2)
        a_plume2  = 0.5 / (MERGE(sig_lon_E(2,iplume), sig_lon_W(2,iplume), delta_lon > 0)**2)
        b_plume2  = 0.5 / (MERGE(sig_lat_E(2,iplume), sig_lat_W(2,iplume), delta_lon > 0)**2)
        !
        ! adjust for a plume specific rotation which helps match plume state to climatology.
        !
        lon1 =   COS(theta(1,iplume))*(delta_lon) + SIN(theta(1,iplume))*(delta_lat)
        lat1 = - SIN(theta(1,iplume))*(delta_lon) + COS(theta(1,iplume))*(delta_lat)
        lon2 =   COS(theta(2,iplume))*(delta_lon) + SIN(theta(2,iplume))*(delta_lat)
        lat2 = - SIN(theta(2,iplume))*(delta_lon) + COS(theta(2,iplume))*(delta_lat)
        !
        ! calculate contribution to plume from its different features, to get a column weight for the anthropogenic
        ! (cw_an) and the fine-mode background aerosol (cw_bg)
        !
        f1 = time_weight(1,iplume) * ftr_weight(1,iplume) * EXP(-1.* (a_plume1 * ((lon1)**2) + (b_plume1 * ((lat1)**2)))) 
        f2 = time_weight(2,iplume) * ftr_weight(2,iplume) * EXP(-1.* (a_plume2 * ((lon2)**2) + (b_plume2 * ((lat2)**2)))) 

        cw_an(icol) = f1 * aod_spmx(iplume) + f2 * aod_spmx(iplume)  
        cw_bg(icol) = f1 * aod_fmbg(iplume) + f2 * aod_fmbg(iplume) 
        !
        ! calculate wavelength-dependent scattering properties
        !
        lfactor   = MIN(1.0,700.0/lambda)
        ssa(icol) = (ssa550(iplume) * lfactor**4) / ((ssa550(iplume) * lfactor**4) + ((1-ssa550(iplume)) * lfactor))
        asy(icol) =  asy550(iplume) * SQRT(lfactor)
      END DO
      !
      ! distribute plume optical properties across its vertical profile weighting by optical depth and scaling for
      ! wavelength using the anstrom parameter. 
      !      
      lfactor = EXP(-angstrom(iplume) * LOG(lambda/550.0))
      DO k=1,nlevels
        DO icol = 1,ncol
          aod_550          = prof(icol,k)     * cw_an(icol)
          aod_lmd          = aod_550          * lfactor
          caod_sp(icol)    = caod_sp(icol)    + prof(icol,k) * cw_an(icol)
          caod_bg(icol)    = caod_bg(icol)    + prof(icol,k) * cw_bg(icol)
          asy_prof(icol,k) = asy_prof(icol,k) + aod_lmd * ssa(icol) * asy(icol)
          ssa_prof(icol,k) = ssa_prof(icol,k) + aod_lmd * ssa(icol)
          aod_prof(icol,k) = aod_prof(icol,k) + aod_lmd
        END DO
      END DO
    END DO
    !
    ! complete optical depth weighting
    !
    DO k=1,nlevels
      DO icol = 1,ncol
        asy_prof(icol,k) = MERGE(asy_prof(icol,k)/ssa_prof(icol,k), 0.0, ssa_prof(icol,k) > TINY(1.))
        ssa_prof(icol,k) = MERGE(ssa_prof(icol,k)/aod_prof(icol,k), 1.0, aod_prof(icol,k) > TINY(1.))
      END DO
    END DO
    !
    ! calcuate effective radius normalization (divisor) factor
    !
    DO icol=1,ncol
      dNovrN(icol) = LOG((1000.0 * (caod_sp(icol) + caod_bg(icol))) + 3.0)/LOG((1000.0 * caod_bg(icol)) + 3.0)
    END DO

    RETURN
  END SUBROUTINE sp_aop_profile

END MODULE mo_simple_plumes
#endif
