#include <misc.h> 
#include <params.h>
#if ( defined SCAM )
#include <max.h>
#endif

module prescribed_aerosols
!----------------------------------------------------------------------- 
! 
! Purposes: 
!       read, store, interpolate, and return fields
!         of aerosols to CAM.  The initialization
!         file (mass.nc) is assumed to be a monthly climatology
!         of aerosols from MATCH (on a sigma pressure
!         coordinate system).
!       also provide a "background" aerosol field to correct
!         for any deficiencies in the physical parameterizations
!         This fields is a "tuning" parameter.
!       Public methods:
!       (1) - initialization
!          read aerosol masses from external file
!             also pressure coordinates
!          convert from monthly average values to mid-month values
!       (2) - interpolation (time and vertical)
!          interpolate onto pressure levels of CAM
!          interpolate to time step of CAM
!          return mmr's of aerosols 
!          provide background aerosol based on "tauback"
!       (3) - diagnostics
!          write out various diagnostic fields
! 
! Calling Heirarchy
!   aerosol_initialize (public)
!     add_aer_diag_fields
!   get_aerosol (public)
!     vert_interpolate
!     background
!     scale_aerosols
!   aerosol_diagnostics (public)
!   aerosol_indirect (public)
!   get_rf_scales (public)
!   get_int_scales (public)
#include <misc.h> 
!
!-----------------------------------------------------------------------

   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid, only: pcols, pver, pverp, begchunk, endchunk
   use pmgrid, only: plon, plat, plev, plevp, masterproc
   use phys_grid, only: get_ncols_p, scatter_field_to_chunk
   use time_manager, only: get_curr_calday, get_curr_date
   use infnan, only: inf, bigint
   use abortutils, only: endrun
!---------------------------zf 2010.11.08------------------------------
   use rgrid,          only: nlon
   use commap,         only: latdeg, londeg
   use pmgrid,         only: plond
!-----------------------------------------------------------------------
#if ( defined SCAM )
   use scamMod, only :initlonidx,initlatidx
   use getnetcdfdata
#endif

   implicit none

! naer_all is total number of species
! naer is number of species in climatology
! naer_all = naer + 1 (background "species") + 1 (volcanic)
!-------------
! cam3
! cam3  integer, public, parameter :: naer_all = 12
! cam3  integer, private, parameter :: naer = 10 

  integer, public, parameter  :: naer_all = 14        ! wtw
!++sxj
  !integer, private, parameter :: naer = 12           ! wtw 
  integer, public, parameter  :: naer = 12
!--sxj

! indices to aerosol array (species portion)

  integer, public, parameter :: &
      idxSUL   =  1, &
      idxSSLT  =  2, &
      idxOCPHO =  7, &
      idxBCPHO =  8, &
      idxOCPHI =  9, &
      idxBCPHI = 10, &
      idxSSLT01= 11, &  ! wtw
      idxNH4NO3= 12, &  ! wtw
      idxBG    = 13, &
      idxVOLC  = 14

!      idxSOA   = 13, & ! wtw

#ifdef MICROP-MG
   integer, public, parameter ::   &
       idxdst1 = 3 ,   &
       idxdst2 = 4 ,   &
       idxdst3 = 5 ,   &
       idxdst4 = 6
#endif

! indices to sections of array that represent 
! groups of aerosols

  integer, public, parameter :: &
      idxDUSTfirst    = 3, &
      numDUST         = 4, &
      idxCARBONfirst = 7, &
      numCARBON      = 4

! names of aerosols are they are represented in
! the climatology file and for the purposes
! of outputting mmr's.  Appended '_V' indicates field has been vertically summed.

  character(len=8), public, parameter :: aerosol_name(naer_all) =  &
     (/"MSUL_V  "&
      ,"MSSLT_V "&
      ,"MDUST1_V"&
      ,"MDUST2_V"&
      ,"MDUST3_V"&
      ,"MDUST4_V"&
      ,"MOCPHO_V"&
      ,"MBCPHO_V"&
      ,"MOCPHI_V"&
      ,"MBCPHI_V"&
      ,"MSSLT01 "&   
      ,"MNH4NO3 "&
      ,"Backgrnd"&
      ,"MVOLC   "/)

!     ,"MSOA    "&  
! 
! default values for namelist variables
!

! compute carbons scaling from population
  character(len=16), public :: scenario_carbon_scale   = 'FIXED' ! 'FIXED' or 'RAMPED'
  character(len=16), public :: scenario_prescribed_sulfur   = 'FIXED' ! 'FIXED' or 'RAMPED'
  character(len=16), public :: prescribed_sulfur   = 'direct' ! 'off', 'passive' or 'direct'
  integer, public :: rampyear_prescribed_sulfur   = bigint ! this is the only acceptable value
  integer, public :: rampyear_prescribed_aeros     = bigint ! this is the only acceptable value
  character(len=16), public :: scenario_prescribed_aeros   = 'FIXED' ! 'FIXED' or 'RAMPED'

! compute radiative forcing per aerosol
  logical, public :: radforce   = .false. 

!++sxj 
! compute radiative forcing without aerosol, in order to diagnose AIE 
  logical, public :: radAIE   = .false. 
!--sxj 

! compute stratospheric volcanic aerosol fields and forcing
  logical, public :: strat_volcanic   = .false.

! portion of each species group to use in computation
! of relative radiative forcing.

  real(r8), public :: sulscl_rf  = 0._r8 ! 
  real(r8), public :: carscl_rf  = 0._r8
  real(r8), public :: ssltscl_rf = 0._r8
  real(r8), public :: dustscl_rf = 0._r8
  real(r8), public :: bgscl_rf   = 0._r8
  real(r8), public :: volcscl_rf = 0._r8

! "background" aerosol species mmr.
  real(r8), public :: tauback = 0._r8

! portion of each species group to use in computation
! of aerosol forcing in driving the climate
  real(r8), public :: sulscl  = 1._r8
  real(r8), public :: carscl  = 1._r8
  real(r8), public :: ssltscl = 1._r8
  real(r8), public :: dustscl = 1._r8
  real(r8), public :: volcscl = 1._r8

  integer,public,parameter :: nfile = 27    ! number of input data files
  integer,public,parameter :: yr_sulf(nfile) = (/ &
                      1840, 1850, 1900, 1920, 1930, 1940, &
                      1950, 1960, 1970, 1980, 1990, &
                      2000, 2010, 2020, 2030, 2040, &
                      2050, 2060, 2070, 2100, 2110, &
                      2120, 2130, 2140, 2150, 2160, &
                      2170         /)
  integer,public,parameter :: yr_aero(nfile) = (/ &
                      1845, 1855, 1865, 1875, 1885, 1895, 1905, &
                      1915, 1925, 1935, 1945, 1955, &
                      1965, 1975, 1985, 1995, 2005, &
                      2015, 2025, 2035, 2045, 2055, &  
                      2065, 2075, 2085, 2095, 2105  &
                                                  /)

  private
  save

  integer :: aernid = -1           ! netcdf id for aerosol file (init to invalid)
  integer :: sulnid = -1           ! netcdf id for sulfurl file (init to invalid)
  integer :: species_id(naer) = -1 ! netcdf_id of each aerosol species (init to invalid)
  integer :: Mpsid                 ! netcdf id for MATCH PS
  integer :: nm = 1                ! index to prv month in array. init to 1 and toggle between 1 and 2
  integer :: np = 2                ! index to nxt month in array. init to 2 and toggle between 1 and 2
  integer :: mo_nxt = bigint       ! index to nxt month in file

  real(r8) :: cdaym = inf          ! calendar day of prv month
  real(r8) :: cdayp = inf          ! calendar day of next month
  real(r8) :: cdaymsul = inf          ! calendar day of prv month for sulfur
  real(r8) :: cdaypsul = inf          ! calendar day of next month for sulfur
  real(r8) :: Mid(12)              ! Days into year for mid month date
  data Mid/16.5, 46.0, 75.5, 106.0, 136.5, 167.0, 197.5, 228.5, 259.0, 289.5, 320.0, 350.5 /
  
  public aerosol_initialize  ! read from file, interpolate onto horiz grid
  public get_aerosol         ! interpolate onto pressure levels, and time
  public aerosol_diagnostics ! write out various data about aerosols
  public aerosol_indirect    ! compute change in effective liqw radius from sulfate
  public get_rf_scales       ! compute scale factors for forcing computation
  public get_int_scales      ! compute scale factors for interactive comps
  public aerint              ! read next month info

!
!  values read from file and temporary values used for interpolation
!
!  AEROSOLc is:
!  Cumulative Mass at midpoint of each month
!    on CAM's horizontal grid (col)
!    on MATCH's levels (lev)
!  AEROSOLc(month_ind, col_ind, Match_lev_ind, chunk_ind, species_ind) 
!
  integer, parameter :: paerlev = 28           ! number of levels for aerosol fields (MUST = naerlev)
  integer naerlev                              ! size of level dimension in MATCH data
  integer nsullev                              ! size of level dimension in sulfur data
  real(r8) :: M_hybi(paerlev+1)                ! MATCH hybi
  real(r8) :: M_hybi_sul(paerlev+1)            ! sulfur hybi
  real(r8) :: M_ps(plon,plat)                  ! surface pressure from MATCH file
  real(r8), allocatable :: AEROSOLc(:,:,:,:,:) ! Aerosol cumulative mass from MATCH
  real(r8), allocatable :: M_ps_cam_col(:,:,:) ! PS from MATCH on Cam Columns
  real(r8), allocatable :: M_ps_sul_col(:,:,:) ! PS for sulfur on Cam Columns
  real(r8), allocatable :: Match_ps_chunk(:,:) ! surface pressure array section at a particular time step

  integer :: mxaerl                            ! Maximum level of background aerosol
!

! for IPCC data 

!--------------------zf 2010.11.10------------------
   integer,private   :: lonaerosiz     ! size of longitude dimension on aerosol dataset
   integer,private   :: lataerosiz     ! size of latitude dimension on aerosol dataset
   integer,private   :: levaerosiz     ! size of latitude dimension on aerosol dataset
   integer,private   :: naerlon, naerlat
   integer,private   :: lonso4siz, latso4siz

   real(r8),private, allocatable :: aerolon(:)
   real(r8),private, allocatable :: aerolat(:)
   real(r8),private, allocatable :: so4lon(:)
   real(r8),private, allocatable :: so4lat(:)
!---------------------------------------------------

   real(r8),private,allocatable,dimension(:,:,:,:) ::  sulfi
                      ! input sulfate bio mixing ratios (pcols,pver,begchunk:endchunk,2)
   real(r8),private,allocatable,dimension(:,:,:)   ::  sulf
                      ! time interpolated sulfate bio mixing ratios (pcols,pver,begchunk:endchunk)
   real(r8),private,allocatable,dimension(:,:,:,:,:) ::  aeroi
                      ! input sulfate bio mixing ratios (pcols,pver,begchunk:endchunk,2,naer)
   real(r8),private,allocatable,dimension(:,:,:,:)   ::  aero
                      ! time interpolated sulfate bio mixing ratios (pcols,pver,begchunk:endchunk,naer)

   real(r8), private :: cdaysulfm        ! calendar day for prv. month sulfate values read in
   real(r8), private :: cdaysulfp        ! calendar day for nxt. month sulfate values read in
   real(r8), private :: cdayaerom        ! calendar day for prv. month sulfate values read in
   real(r8), private :: cdayaerop        ! calendar day for nxt. month sulfate values read in

   real(r8),private :: bndry_m          ! calendar day for the lower time-boundary of inputdata
   real(r8),private :: bndry_p          ! calendar day for the upper time-boundary of inputdata
   logical, private :: outofbndry       ! whether current model time is out of the time-boundary

!--- wtw
!   character*8, private, parameter :: varname(10)=          &
!   (/"SO4     ","SSLT    ","DST01   ","DST02   ","DST03   ", &
!     "DST04   ","OC1     ","CB1     ","OC2     ","CB2     "/)

   character*8, private, parameter :: varname(12)=          &
   (/"SO4     ","SSLT    ","DST01   ","DST02   ","DST03   ", &
     "DST04   ","OC1     ","CB1     ","OC2     ","CB2     ", &
     "SSLT01  ","NH4NO3  "/)
!--- wtw

   integer :: nmsul = 1             ! index to prv month in array. init to 1 and toggle between 1 and 2
   integer :: npsul = 2             ! index to nxt month in array. init to 2 and toggle between 1 and 2

   character*256,private :: file_sulf(nfile)
   character*256,private :: file_aero(nfile)

   integer, private,parameter :: nsample = 12   ! number of samples in each input data file
   real(r8),private           :: sample_sulf(nsample) =      (/&
                1296000.0,  3888000.0,  6393600.0,  9072000.0, &
               11664000.0, 14342400.0, 16934400.0, 19612800.0, &  
               22291200.0, 24883200.0, 27561600.0, 30153600.0 /)
!               1296000.0,  3888000.0,  6480000.0,  9072000.0, &
!              11664000.0, 14256000.0, 16848000.0, 19440000.0, &  
!              22032000.0, 24624000.0, 27216000.0, 29808000.0 /)
   real(r8),private           :: sample_aero(nsample) =      (/&
                1296000.0,  3888000.0,  6393600.0,  9072000.0, &
               11664000.0, 14342400.0, 16934400.0, 19612800.0, &  
               22291200.0, 24883200.0, 27561600.0, 30153600.0 /)
! Integer data
!
   integer, private :: sp_m, sp_p
   integer, private :: yrm,  yrp
   integer, private :: yr_am, yr_bm, yr_ap, yr_bp, indxbp
!
   integer, private :: ncid_sulf_ap, ncid_sulf_bp  ! sulfate dataset id
   integer, private :: ncid_aero_ap, ncid_aero_bp  ! sulfate dataset id
   integer, private :: sulf_id_ap,   sulf_id_bp    ! netcdf id for sulfate mmr bio variable

   character*80,  private ::  sulfdata=''        ! full pathname for sulfate dataset
   character*256, private ::  sulfdatafile_bp ! full pathname for sulfate dataset
   character*80,  private ::  aerodata=''        ! full pathname for sulfate dataset
   character*256, private ::  aerodatafile_bp ! full pathname for sulfate dataset

!------------------zf 2010.11.15---------------------------------------------------
   interface

   subroutine bilin (arrin, xin, yin, nlondin, nlonin, &
                     nlevdin, nlev, nlatin, arrout, xout, &
                     yout, nlondout, nlonout, nlevdout, nlatout, fillvalue)

   use shr_kind_mod, only: r8 => shr_kind_r8
   use abortutils,   only: endrun

   integer, intent(in) :: nlondin                        ! longitude dimension of input grid
   integer, intent(in) :: nlonin                         ! number of real longitudes (input)
   integer, intent(in) :: nlevdin                        ! vertical dimension of input grid
   integer, intent(in) :: nlev                           ! number of vertical levels
   integer, intent(in) :: nlatin                         ! number of input latitudes
   integer, intent(in) :: nlatout                        ! number of output latitudes
   integer, intent(in) :: nlondout                       ! longitude dimension of output grid
   integer, intent(in) :: nlonout(nlatout)               ! number of output longitudes per lat
   integer, intent(in) :: nlevdout                       ! vertical dimension of output grid

   real(r8), intent(in) :: arrin(nlondin,nlevdin,nlatin) ! input array of values to interpolate
   real(r8), intent(in) :: xin(nlondin)                  ! input x mesh
   real(r8), intent(in) :: yin(nlatin)                   ! input y mesh
   real(r8), intent(in) :: xout(nlondout,nlatout)        ! output x mesh
   real(r8), intent(in) :: yout(nlatout)                 ! output y mesh
   real(r8), optional, intent(in) :: fillvalue           ! undef. value, wzz, 2009.8.31

   real(r8), intent(out) :: arrout(nlondout,nlevdout,nlatout) ! interpolated array

   end subroutine bilin

   end interface
!----------------------------------------------------------------------------------

contains

subroutine aerosol_initialize
!------------------------------------------------------------------
!  Reads in:
!     file from which to read aerosol Masses on CAM grid. Currently
!        assumed to be MATCH ncep runs, averaged by month.
!     NOTE (Data have been externally interpolated onto CAM grid 
!        and backsolved to provide Mid-month values)
!     
!  Populates:
!     module variables:
!       AEROSOLc(pcols,paerlev+1,begchunk:endchunk,naer,2))
!       AEROSOLc(  column_index
!                , level_index (match levels)
!                , chunk_index 
!                , species_index
!                , month = 1:2 )
!       M_hybi(level_index = Lev_MATCH) = pressure at mid-level.
!       M_ps_cam_col(column,chunk,month) ! PS from MATCH on Cam Columns
!
!  Method:
!    read data from file
!    allocate memory for storage of aerosol data on CAM horizontal grid
!    distribute data to remote nodes
!    populates the module variables
!
!------------------------------------------------------------------
   use ioFileMod, only: getfil
   use filenames, only: bndtvaer, bndtvsul
   use carbonscales, only: init_scale
#ifdef MAC_SP
   use volcanicradi, only: volcanic_radi_initialize
#endif
#ifdef VOL_DENS
   use volcanicdens, only: volcanic_dens_initialize
#else
   use volcanicmass, only: volcanic_initialize
#endif

#if ( defined SPMD )
   use mpishorthand
#endif

   include 'netcdf.inc'

! local variables

   real(r8) AEROSOLm(pcols,pver,begchunk:endchunk,naer,2) ! aerosol mmr from MATCH in column at previous (minus) month
!  real(r8) AEROSOLp(pcols,pver,begchunk:endchunk,naer) ! aerosol mmr from MATCH in column at next (plus) month 

   integer dateid                       ! netcdf id for date variable
   integer secid                        ! netcdf id for seconds variable
   integer londimid                     ! netcdf id for longitude dimension
   integer latdimid                     ! netcdf id for latitude dimension
   integer levdimid                     ! netcdf id for level dimension
   integer lonid                        ! netcdf id for longitude variable

   integer timesiz                      ! number of time samples (=12) in netcdf file
   integer latid                        ! netcdf id for latitude variable
   integer Mhybiid                      ! netcdf id for MATCH hybi
   integer timeid                       ! netcdf id for time variable
   integer dimids(nf_max_var_dims)      ! variable shape
   integer :: start(4)                  ! start vector for netcdf calls
   integer :: kount(4)                  ! count vector for netcdf calls
   integer mo                           ! month index
   integer m                            ! constituent index
   integer :: n                         ! loop index
   integer :: i,j,k                     ! spatial indices
   integer :: date_aer(12)              ! Date on aerosol dataset (YYYYMMDD)
   integer :: attnum                    ! attribute number

#if ( defined SCAM )
   real(r8) ::  coldata(paerlev)    ! aerosol field read in from dataset
   integer :: ret
#endif
   integer mo_prv                       ! index to previous month

   character(len=256) :: locfn          ! netcdf local filename to open
!
! aerosol_data will be read in from the aerosol boundary dataset, then scattered to chunks
! after filling in the bottom level with zeros
! 
   real(r8) :: aerosol_data(plon,plat,paerlev)    ! aerosol field read in from dataset
   real(r8) :: aerosol_field(plon,paerlev+1,plat) ! aerosol field to be scattered
   real(r8) :: caldayloc                          ! calendar day of current timestep

   call t_startf ('aerosol_init')
   call print_memusage ('Start aerosol_initialize')

   if(scenario_carbon_scale.eq."RAMPED") then
     write(6,*)"aerosol_init: carbon ramping overrides carscl"
   else if(scenario_carbon_scale.eq."FIXED") then
     write(6,*) "aerosol_init: using namelist value for carscl, or default"
   else
     call endrun ('scenario_carbon_scale must be either RAMPED or FIXED')
   endif

! wzz, 2010.4.25
   if(scenario_prescribed_aeros.eq."RAMPED") then
     write(6,*) "aerosol_init: Using aerosols from RAMPED aerosol dataset"
   else if(scenario_prescribed_aeros.eq."FIXED") then
     write(6,*) "aerosol_init: Using aerosols from cyclic climatology"
   else
     call endrun ('scenario_prescribed_aeros must be either RAMPED or FIXED')
   endif

   if(scenario_prescribed_sulfur.eq."RAMPED") then
     write(6,*) "aerosol_init: Using sulfate from RAMPED aerosol dataset"
     if ( rampyear_prescribed_sulfur .ne. bigint ) then
        write(6,*) "aerosol_init: Ramped sulfate being fixed at year", rampyear_prescribed_sulfur
     endif
   else if(scenario_prescribed_sulfur.eq."FIXED") then
     write(6,*) "aerosol_init: Using sulfate from cyclic climatology"
     if ( rampyear_prescribed_sulfur .ne. bigint ) then
       call endrun ('aerosol_init: rampyear_prescribed_sulfur set but there is only one year in the climatology')
     endif
   else
     call endrun ('scenario_prescribed_sulfur must be either RAMPED or FIXED')
   endif

  if(prescribed_sulfur  == 'off') then
    call endrun ('aerosol_init: prescribed sulfur = off is not implemented, try passive')
  else if(.not.(prescribed_sulfur == 'direct' .or. prescribed_sulfur == 'passive')) then
    write(6,*) 
    call endrun ('prescribed sulfur must be one of off, direct, or passive')
  endif
!
! Allocate memory for dynamic arrays local to this module
!
   allocate (AEROSOLc(pcols,paerlev+1,begchunk:endchunk,naer,2))
   allocate (M_ps_cam_col(pcols,begchunk:endchunk,2))
   allocate (M_ps_sul_col(pcols,begchunk:endchunk,2))
   allocate (Match_ps_chunk(pcols,begchunk:endchunk))

! TBH:  HACK to avoid use of uninitialized values when ncols < pcols
  AEROSOLc(:,:,:,:,:) = 0.
  M_ps_cam_col(:,:,:) = 0.
  M_ps_sul_col(:,:,:) = 0.
  Match_ps_chunk(:,:) = 0.
! TBH:  END HACK

!
!  Add fields to master list for output
!
   call add_aer_diag_fields ()
!
! compute mxaerl
!
   call setmxaerl ()

   if (masterproc) then
!
! find and open file; abort if fail (getfil(,,0)).
!
      call getfil (bndtvaer, locfn, 0)
      call wrap_open (locfn, 0, aernid)
      write(6,*)'aerosol_initialize: reading aerosol dataset....  If this seems to be taking too'
      write(6,*)'long, perhaps the dataset is being read from an nfs-mounted filesystem.'
      if (scenario_prescribed_sulfur=='FIXED') then
#ifdef SCAM
      bndtvsul = bndtvaer
#endif
      if ( bndtvaer==bndtvsul) then
         sulnid = aernid
      else
         call getfil (bndtvsul, locfn, 0)
         call wrap_open (locfn, 0, sulnid)
      endif
      endif
!
! First ensure dataset is CAM-ready
!
      if (nf_inq_attid (aernid, nf_global, 'cam-ready', attnum) /= nf_noerr) then
         write(6,*)'interpaerosols needs to be run to create a cam-ready aerosol dataset'
         call endrun ('aerosol_initialize: global attribute cam-ready not found')
      end if
   end if

! wzz, 2009.9.2

   if (scenario_prescribed_aeros=='RAMPED') then

! wzz for test, 2010.4.26
      do i = len_trim(bndtvaer), 1, -1
         if(bndtvaer(i:i).eq.'/') exit
      enddo
      aerodata = bndtvaer(1:i)

! read historic AERO data
      call aeroini_IPCC(nsullev)
!
   else
!
      if (scenario_prescribed_sulfur=='RAMPED') then
! get the fullpath of sulfate data here
         do i = len_trim(bndtvsul), 1, -1
            if(bndtvsul(i:i).eq.'/') exit
         enddo
         sulfdata = bndtvsul(1:i)
!        write(*,*)'wzz: sulfdata= ', i, sulfdata
! read historic SULFUR data
         call sulfini_IPCC(nsullev, M_hybi_sul)
      else
! read climate SULFUR data
         call aerosol_initialize_read(sulnid, aerosol_name(idxSUL), 1, M_hybi_sul, &
                                   M_ps_sul_col, AEROSOLc(:,:,:,idxSUL,:), nsullev,  &
                                   scenario_prescribed_sulfur,   &
                                   species_id(idxSUL), cdaymsul, cdaypsul)
      endif
#if ( defined SPMD )
      call mpibcast (M_hybi_sul, paerlev+1, mpir8, 0, mpicom)
      call mpibcast (nsullev,1, mpiint, 0, mpicom)
#endif
      write(*,*)'wzz: nsullev=', nsullev
!     write(*,*)'wzz: plevs  =', M_hybi_sul(1:nsullev)
      if (scenario_prescribed_sulfur=='RAMPED') then
         M_ps_sul_col = M_hybi_sul(nsullev)
         M_hybi_sul(:) = M_hybi_sul(:)/M_hybi_sul(nsullev)
      endif
!
! read climate data except SULFUR
      call aerosol_initialize_read(aernid, aerosol_name(2:naer), naer-1, M_hybi, &
                                   M_ps_cam_col, AEROSOLc(:,:,:,2:naer,:), naerlev,  &
                                   scenario_carbon_scale,  &
                                   species_id(2:naer), cdaym, cdayp)
!
#if ( defined SPMD )
      call mpibcast (M_hybi, paerlev+1, mpir8, 0, mpicom)
      call mpibcast (naerlev,1, mpiint, 0, mpicom)
#endif
   endif    ! (scenario_prescribed_aero=='RAMPED')
!
!
! read in volcanic grid data (structural only, no masses)
!
   if (strat_volcanic) then
#ifdef VOL_DENS
      call volcanic_dens_initialize(rampyear_prescribed_aeros)
#else
      call volcanic_initialize(rampyear_prescribed_aeros)
#endif
   endif

#ifdef MAC_SP
      call volcanic_radi_initialize(rampyear_prescribed_aeros)
#endif

   if ( scenario_carbon_scale == "RAMPED") then
      call init_scale()
   endif

   call print_memusage('End aerosol_initialize')
   call t_stopf ('aerosol_init')

   return
end subroutine aerosol_initialize

subroutine aerosol_initialize_read(aernid, aerosol_nameloc, naerloc, M_hybiloc, &
                                   M_ps_col, AEROSOLcloc, naerlevloc, scenario_aero,  &
                                   species_idloc, cdaym, cdayp)
!
#if ( defined SPMD )
   use mpishorthand
#endif

   include 'netcdf.inc'

   integer, intent(in) :: aernid
   integer, intent(in) :: naerloc
   character(len=8), intent(in) :: aerosol_nameloc(naerloc)
   character(len=16), intent(in) :: scenario_aero
   integer, intent(out) :: naerlevloc
   integer, intent(out) :: species_idloc(naerloc)  ! netcdf_id of each aerosol species (init to invalid)
   real(r8),intent(out) :: cdaym                   ! calendar day of prv month
   real(r8),intent(out) :: cdayp                   ! calendar day of next month
   real(r8), intent(out) :: M_hybiloc(paerlev+1)                ! aeros hybi
   real(r8), intent(out) :: M_ps_col(pcols,begchunk:endchunk,2)
   real(r8), intent(out) :: AEROSOLcloc(pcols,paerlev+1,begchunk:endchunk,naerloc,2)

   integer,dimension(:), allocatable :: date_aer              ! Date on aerosol dataset (YYYYMMDD)  ???
!
! local variables

!zf   integer :: naerlon
!zf   integer :: naerlat

   integer dateid                       ! netcdf id for date variable
   integer secid                        ! netcdf id for seconds variable
   integer londimid                     ! netcdf id for longitude dimension
   integer latdimid                     ! netcdf id for latitude dimension
   integer levdimid                     ! netcdf id for level dimension
   integer lonid                        ! netcdf id for longitude variable
   integer mo_prv                       ! index to previous month

   integer timesiz                      ! number of time samples (=12) in netcdf file
   integer latid                        ! netcdf id for latitude variable
   integer Mhybiid                      ! netcdf id for MATCH hybi
   integer timeid                       ! netcdf id for time variable
   integer dimids(nf_max_var_dims)      ! variable shape
   integer :: start(4)                  ! start vector for netcdf calls
   integer :: kount(4)                  ! count vector for netcdf calls
   integer mo                           ! month index
   integer m                            ! constituent index
   integer :: n                         ! loop index
   integer :: i,j,k                     ! spatial indices
   integer :: attnum                    ! attribute number
   real(r8) :: caldayloc                          ! calendar day of current timestep
!zf   real(r8) :: M_psloc(plon,plat)                  ! surface pressure from MATCH file
   real(r8) :: aerosol_data(plon,plat,paerlev)    ! aerosol field read in from dataset
!zf   real(r8) :: aerosol_field(plon,paerlev+1,plat) ! aerosol field to be scattered

   integer :: yr, mon, day   ! components of a date
   integer :: ncdate         ! current date in integer format [yyyymmdd]
   integer :: ncsec          ! current time of day [seconds]

!-----------------------zf 2010.11.16---------------------
   integer  :: istat                 !   error return
   real(r8) :: aerosol_field(plond,paerlev+1,plat) ! aerosol field to be scattered
   real(r8) :: M_psloc(plond,plat)                  ! surface pressure from MATCH file
   real(r8), allocatable :: aero_tempa(:,:,:)
   real(r8), allocatable :: aero_tempb(:,:,:)
   real(r8), allocatable :: psloc_temp(:,:) 

#if ( defined SCAM )
   real(r8) ::  coldata(paerlev)    ! aerosol field read in from dataset
   integer :: ret
#endif
!----------------------------------------------------------

! Get and check dimension info
!
      aerosol_data = 0.0
      species_idloc = -1
!
     if (masterproc) then
!
      call wrap_inq_dimid( aernid,  'lon', londimid )
      call wrap_inq_dimid( aernid,  'lev', levdimid )
      call wrap_inq_dimid( aernid, 'time',   timeid )
      call wrap_inq_dimid( aernid,  'lat', latdimid )

      call wrap_inq_dimlen( aernid, londimid, naerlon )
      call wrap_inq_dimlen( aernid, levdimid, naerlevloc )
      call wrap_inq_dimlen( aernid, latdimid, naerlat )
      call wrap_inq_dimlen( aernid,   timeid, timesiz )

#if ( !defined SCAM )
!zf      if (naerlon /= plon .or. naerlat /= plat .or. naerlevloc > paerlev .or. timesiz < 12) then
      if ( naerlevloc > paerlev .or. timesiz < 12) then   !zf 2010.11.16
         write(6,*)'aerosol_initialize: grid mismatch'
         write(6,*)'Model:   plon,    plat,    paerlev, 12     =', plon, plat, paerlev, 12
         write(6,*)'Dataset: naerlon, naerlat, naerlev, timesiz=', naerlon, naerlat, naerlevloc, timesiz
         call endrun ()
      end if
#endif 

      allocate (date_aer(timesiz))

      call wrap_inq_varid( aernid, 'date',   dateid )
      call wrap_inq_varid( aernid, 'datesec', secid )
      do m = 1, naerloc
!        write(*,*)'wzz: aerosol_name=',aerosol_nameloc(m),m
         call wrap_inq_varid( aernid, TRIM(aerosol_nameloc(m)), species_idloc(m))
      end do

      call wrap_inq_varid( aernid, 'lon', lonid   )
      call wrap_inq_varid( aernid, 'lat', latid   )
!---------------------zf 2010.11.16----------------------------
      allocate( aero_tempa(naerlon,naerlat,paerlev), stat=istat )
      allocate( aero_tempb(naerlon,paerlev+1,naerlat), stat=istat )
      allocate( psloc_temp(naerlon,naerlat), stat=istat )

      allocate( aerolon(naerlon), stat=istat )
      allocate( aerolat(naerlat), stat=istat )

      call wrap_get_var_realx( aernid, lonid, aerolon )
      call wrap_get_var_realx( aernid, latid, aerolat )
!----------------------------------------------------------------
!
! quick sanity check on one field
!
      call wrap_inq_vardimid (aernid, species_idloc(1), dimids)
      if ( (dimids(4) /= timeid) .or. &
           (dimids(3) /= levdimid) .or. &
           (dimids(2) /= latdimid) .or. &
           (dimids(1) /= londimid) ) then
         write(6,*)'AEROSOL READ: Data must be ordered time, lev, lat, lon'
         write(6,*)'data are       ordered as', dimids(4), dimids(3), dimids(2), dimids(1)
         write(6,*)'data should be ordered as', timeid, levdimid, latdimid, londimid
         call endrun ()
      end if
!
! use hybi,PS from MATCH
!
      call wrap_inq_varid( aernid, 'hybi', Mhybiid   )
      call wrap_inq_varid( aernid, 'PS', Mpsid   )
!
! check dimension order for MATCH's surface pressure
!
      call wrap_inq_vardimid (aernid, Mpsid, dimids)
      if ( (dimids(3) /= timeid) .or. &
           (dimids(2) /= latdimid) .or. &
           (dimids(1) /= londimid) ) then
         write(6,*)'AEROSOL READ: Pressure must be ordered time, lat, lon'
         write(6,*)'data are       ordered as', dimids(3), dimids(2), dimids(1)
         write(6,*)'data should be ordered as', timeid, levdimid, latdimid, londimid
         call endrun ()
      end if
! 
! read in hybi from dataset
!
      call wrap_get_var_realx (aernid, Mhybiid, M_hybiloc)
!
! Retrieve date and sec variables.
!
      call wrap_get_var_int (aernid, dateid, date_aer)
      if (timesiz < 12) then
         write(6,*)'AEROSOL READ: When cycling aerosols, dataset must have 12 consecutive ', &
                   'months of data starting with Jan'
         write(6,*)'Current dataset has only ',timesiz,' months'
         call endrun ()
      end if
      if (scenario_aero=='FIXED') then
      do mo = 1,12
         if (mod(date_aer(mo),10000)/100 /= mo) then
            write(6,*)'AEROSOL READ: When cycling aerosols, dataset must have 12 consecutive ', &
                      'months of data starting with Jan'
            write(6,*)'Month ',mo,' of dataset says date=',date_aer(mo)
            call endrun ()
         end if
      end do
      endif
      if (scenario_aero=='RAMPED') then
      do mo = 1,timesiz
         if (mod(date_aer(mo),10000)/100 < 0 .or. mod(date_aer(mo),10000)/100 > 12) then
            write(6,*)'Month ',mo,' of dataset says date=',date_aer(mo)
            call endrun ()
         end if
      end do
      endif
   end if          ! masterproc
!
! broadcast hybi to nodes
!
#if ( defined SPMD )
!  call mpibcast (M_hybi, paerlev+1, mpir8, 0, mpicom)
#endif

   caldayloc = get_curr_calday ()
  
   if (caldayloc < Mid(1)) then
      mo_prv = 12
      mo_nxt =  1
   else if (caldayloc >= Mid(12)) then
      mo_prv = 12
      mo_nxt =  1
   else
      do i = 2 , 12
         if (caldayloc < Mid(i)) then
            mo_prv = i-1
            mo_nxt = i
            exit
         end if
      end do
   end if
!
! Set initial calendar day values
!
   cdaym = Mid(mo_prv)
   cdayp = Mid(mo_nxt)
!
   if (scenario_aero=='RAMPED') then
      call get_curr_date(yr, mon, day, ncsec)
      ncdate = yr*10000 + mon*100 + day
      if (masterproc) then
      if(ncdate<date_aer(1) .or. ncdate>date_aer(timesiz)) then
            write(6,*)'Data range ERROR: ',ncdate, date_aer(1), date_aer(timesiz)
            call endrun ()
      endif
      do mo = 1,timesiz-1
         if (ncdate>=date_aer(mo).and.ncdate<=date_aer(mo+1)) then
            exit
         endif
      enddo
!     write(*,*)'wzz: mo in date_aer= ',mo,ncdate,date_aer(mo),date_aer(mo+1)
      deallocate (date_aer)
      endif
   endif

! Retrieve Aerosol Masses (kg/m^2 in each layer), transpose to model order (lon,lev,lat),
! then scatter to slaves.
!
   if (nm /= 1 .or. np /= 2) call endrun ('AEROSOL_INITIALIZE: bad nm or np value')
   do n=nm,np
      if (scenario_aero=='FIXED') then
      if (n == 1) then
         mo = mo_prv
      else
         mo = mo_nxt
      end if
      else if (scenario_aero=='RAMPED') then   ! wzz, 2008.9.2
      if (n == 1) then
         mo = mo
      else
         mo = mo + 1
      end if
      endif

      do m=1,naerloc
         if (masterproc) then
#if ( defined SCAM )
            call getncdata (aernid, initLatIdx, initLonIdx, mo, &
                 TRIM(aerosol_nameloc(m)), coldata(:), RET)
            aerosol_data(1,1,:)=coldata(:)
#else
            start(:) = (/1,1,1,mo/)
!zf            kount(:) = (/plon,plat,naerlevloc,1/)
            kount(:) = (/naerlon,naerlat,naerlevloc,1/)   !zf
!zf         call wrap_get_vara_realx (aernid, species_idloc(m), start, kount, aerosol_data(1,1,1))
            call wrap_get_vara_realx (aernid, species_idloc(m), start, kount, aero_tempa(1,1,1)) !zf
#endif
!zf            do j=1,plat
!zf               do k=1,paerlev
!zf                  aerosol_field(:,k,j) = aerosol_data(:,j,k)
!zf               end do
!zf               aerosol_field(:,paerlev+1,j) = 0.   ! value at bottom
!zf            end do
!--------------zf 2010.11.16-------------------------------------
           do j=1,naerlat
             do k=1,paerlev
               aero_tempb(:,k,j) = aero_tempa(:,j,k)    
             enddo
             aero_tempb(:,paerlev+1,j) = 0.   ! value at bottom
           end do

           aerosol_field = 0.
           if(naerlon .eq. plon .and. naerlat .eq. plat) then
             aerosol_field(1:plon,:,:) = aero_tempb(1:plon,:,:)
           else

             call bilin(aero_tempb,aerolon,aerolat,naerlon,naerlon,paerlev+1,paerlev+1,   &
                        naerlat,aerosol_field,londeg,latdeg,plond,nlon,paerlev+1,plat)
           end if
!----------------------------------------------------------------------------------------
         end if
!zf         call scatter_field_to_chunk (1, paerlev+1, 1, plon, aerosol_field, AEROSOLcloc(1,1,begchunk,m,n))
         call scatter_field_to_chunk (1, paerlev+1, 1, plon, aerosol_field(1:plon,:,:), AEROSOLcloc(1,1,begchunk,m,n)) !zf
      end do
!
! Retrieve PS from dataset
!
      if (masterproc) then
#if ( defined SCAM )
         call getncdata (aernid, initLatIdx, initLonIdx, mo, &
                'PS', M_ps(1,1), RET)
#else
         start(:) = (/1,1,mo,-1/)
!zf         kount(:) = (/plon,plat,1,-1/)
         kount(:) = (/naerlon,naerlat,1,-1/)   !zf
!zf      call wrap_get_vara_realx (aernid, Mpsid, start, kount, M_psloc(1,1))
         call wrap_get_vara_realx (aernid, Mpsid, start, kount, psloc_temp(1,1))
#endif

!-------------------------------------zf 2010.11.16--------------------------------------
     M_psloc = 0.
     if(naerlon .eq. plon .and. naerlat .eq. plat) then
      M_psloc(1:plon,:) = psloc_temp(1:plon,:)
     else
      call bilin(psloc_temp,aerolon,aerolat,naerlon,naerlon,1,1,   &
                 naerlat,M_psloc,londeg,latdeg,plond,nlon,1,plat)
     end if

!------------------------------------------------------------

      end if
!zf      call scatter_field_to_chunk (1, 1, 1, plon, M_psloc(1,1), M_ps_col(1,begchunk,n))
      call scatter_field_to_chunk (1, 1, 1, plon, M_psloc(1:plon,:), M_ps_col(1,begchunk,n)) !zf
   end do     ! n=nm,np (=1,2)
!--------------------zf 2010.11.17-----------------------------------
   if (masterproc) then
     deallocate (aero_tempa)
     deallocate (aero_tempb)
     deallocate (psloc_temp)
   endif
!--------------------------------------------------------------------
!
end subroutine aerosol_initialize_read

subroutine get_aerosol(c, pint, AEROSOLt, scale, state)
!------------------------------------------------------------------
!
!  Input:
!     time at which aerosol mmrs are needed (get_curr_calday())
!     chunk index
!     CAM's vertical grid (pint)
!
!  Output:
!     values for Aerosol Mass Mixing Ratios at specified time
!     on vertical grid specified by CAM (AEROSOLt)
!
!  Method:
!     first determine which indexs of aerosols are the bounding data sets
!     interpolate both onto vertical grid aerm(),aerp().
!     from those two, interpolate in time.
!
!------------------------------------------------------------------

   use timeinterp,    only: getfactors
   use physics_types, only: physics_state

#ifdef VOL_DENS
   use volcanicdens,  only: get_volcanic_dens
#else
   use volcanicmass,  only: get_volcanic_mass
#endif

#ifdef MOZART2
   use chemistry,         only: ixchm
#endif
!
! aerosol fields interpolated to current time step
!   on pressure levels of this time step.
! these should be made read-only for other modules
! Is allocation done correctly here?
!
   type(physics_state), intent(in) :: state
!
   integer, intent(in) :: c                   ! Chunk Id.
   real(r8), intent(in) :: pint(pcols,pverp)  ! midpoint pres.
   real(r8), intent(in) :: scale(naer_all)    ! scale each aerosol by this amount

   real(r8), intent(out) :: AEROSOLt(pcols, pver, naer_all) ! aerosols
!
! Local workspace
!
   real(r8) caldayloc                     ! calendar day of current timestep
   real(r8) fact1, fact2                  ! time interpolation factors

   integer i, k, j                        ! spatial indices
   integer m                              ! constituent index
   integer lats(pcols),lons(pcols)        ! latitude and longitudes of column
   integer ncol                           ! number of columns
   integer lchnk  
   
   real(r8) speciesmin(naer)              ! minimal value for each species
!
! values before current time step "the minus month"
! aerosolm(pcols,pver) is value of preceeding month's aerosol mmr
! aerosolp(pcols,pver) is value of next month's aerosol mmr
!  (think minus and plus or values to left and right of point to be interpolated)
!
   real(r8) AEROSOLm(pcols,pver,naer) ! aerosol mmr from MATCH in column at previous (minus) month
!
! values beyond (or at) current time step "the plus month"
!
   real(r8) AEROSOLp(pcols,pver,naer) ! aerosol mmr from MATCH in column at next (plus) month 

   logical error_found
!------------------------------------------------------------------

   caldayloc = get_curr_calday ()
!
! Determine time interpolation factors.  1st arg says we are cycling 1 year of data
!
!  write(*,*)'call getfact=',mo_nxt, cdaym, cdayp, caldayloc

   call getfactors (.true., mo_nxt, cdaym, cdayp, caldayloc, &
                    fact1, fact2, 'GET_AEROSOL:')
!
! interpolate (prv and nxt month) bounding datasets onto cam vertical grid.
! compute mass mixing ratios on CAMS's pressure coordinate
!  for both the "minus" and "plus" months
!
   ncol = get_ncols_p(c)

   if (scenario_prescribed_aeros=='RAMPED') then
      call  getaerbnd_IPCC( c, ncol, AEROSOLt(:,:,1:naer) )

   else
! for sulphure first.
      if (scenario_prescribed_sulfur=='FIXED') then
          call vert_interpolate (M_ps_sul_col(1,c,nm), pint, nm, AEROSOLm(:,:,idxSUL), ncol, c,  &
                          1, M_hybi_sul, nsullev, AEROSOLc(:,:,:,idxSUL,:))
          call vert_interpolate (M_ps_sul_col(1,c,np), pint, np, AEROSOLp(:,:,idxSUL), ncol, c,  &
                          1, M_hybi_sul, nsullev, AEROSOLc(:,:,:,idxSUL,:))
! Time interpolate.
          do k=1,pver
             do i=1,ncol
                AEROSOLt(i,k,idxSUL) = AEROSOLm(i,k,idxSUL)*fact1 + AEROSOLp(i,k,idxSUL)*fact2
             end do
          end do
      endif
!
      if (scenario_prescribed_sulfur=='RAMPED') then
          call  getso4bnd_IPCC( c, ncol, AEROSOLm(:,:,idxSUL) )

! compute the vertical accumulated flux. kg/m3 -> kg/m2, wzz, 2009.9.29
          call IPCCsul_diagnostics(state, AEROSOLm(:,:,idxSUL), nsullev, M_hybi_sul,   &
                               AEROSOLc(:,:,c,idxSUL,nm), M_ps_sul_col(1,c,nm))
          call vert_interpolate (M_ps_sul_col(1,c,nm), pint, nm, AEROSOLt(:,:,idxSUL), ncol, c,  &
                          1, M_hybi_sul, nsullev-1, AEROSOLc(:,:,:,idxSUL,:))

!         call  getso4bnd_IPCC( c, ncol, AEROSOLt(:,:,idxSUL) )
      endif

! for aerosols except sulphure.
      call vert_interpolate (M_ps_cam_col(1,c,nm), pint, nm, AEROSOLm(:,:,2:naer), ncol, c,  &
                          naer-1, M_hybi, naerlev, AEROSOLc(:,:,:,2:naer,:))
      call vert_interpolate (M_ps_cam_col(1,c,np), pint, np, AEROSOLp(:,:,2:naer), ncol, c,  &
                          naer-1, M_hybi, naerlev, AEROSOLc(:,:,:,2:naer,:))
!
! Time interpolate.
!
      do m=2,naer
         do k=1,pver
            do i=1,ncol
               AEROSOLt(i,k,m) = AEROSOLm(i,k,m)*fact1 + AEROSOLp(i,k,m)*fact2
            end do
         end do
      end do

      do i=1,ncol
         Match_ps_chunk(i,c) = M_ps_cam_col(i,c,nm)*fact1 + M_ps_cam_col(i,c,np)*fact2
      end do
  
   endif   ! (scenario_prescribed_aeros= 'RAMPED')

! get background aerosol (tuning) field
!
   call background (c, ncol, pint, AEROSOLt(:, :, idxBG))

!
! find volcanic aerosol masses
!
   if (strat_volcanic) then
#ifdef VOL_DENS
      call get_volcanic_dens(c, AEROSOLt(:,:,idxVOLC),state,rampyear_prescribed_aeros)
#else
      call get_volcanic_mass(c, AEROSOLt(:,:,idxVOLC),rampyear_prescribed_aeros)
#endif
   else
      AEROSOLt(:,:,idxVOLC) = 0._r8
   endif

!
! exit if mmr is negative (we have previously set
!  cumulative mass to be a decreasing function.)
!
   speciesmin(:) = 0. ! speciesmin(m) = 0 is minimum mmr for each species
 
   error_found = .false.
   do m=1,naer
      do k=1,pver
         do i=1,ncol
            if (AEROSOLt(i, k, m) < speciesmin(m)) error_found = .true.
         end do
      end do
   end do
   if (error_found) then
      do m=1,naer
         do k=1,pver
            do i=1,ncol
               if (AEROSOLt(i, k, m) < speciesmin(m)) then
                  write(6,*) 'AEROSOL_INTERPOLATE: negative mass mixing ratio, exiting'
                  write(6,*) 'm, column, pver',m, i, k ,AEROSOLt(i, k, m)
                  call endrun ()
               end if
            end do
         end do
      end do
   end if

!-----------------------------------------------
#ifndef MAC_SP
#ifdef MOZART2

   lchnk = state%lchnk

!  call IPCC_aerosol_output(AEROSOLt, lchnk)                   !zf 20150615
!
! use prognostic aerosol
!
   call get_aerosol_mozart(state%q(1,1,ixchm), AEROSOLt)

#endif
#endif

!--------------------------------------------------
!
! scale any AEROSOLS as required
!
   call scale_aerosols (AEROSOLt, ncol, c, scale)

end subroutine get_aerosol

subroutine vert_interpolate (Match_ps, pint, n, AEROSOL_mmr, ncol, c,  &
                          naerloc, M_hybi, naerlev, AEROSOLc)
!--------------------------------------------------------------------
! Input: match surface pressure, cam interface pressure, 
!        month index, number of columns, chunk index
! 
! Output: Aerosol mass mixing ratio (AEROSOL_mmr)
!
! Method:
!         interpolate column mass (cumulative) from match onto
!           cam's vertical grid (pressure coordinate)
!         convert back to mass mixing ratio
!
!--------------------------------------------------------------------

   use physconst,     only: gravit

   integer, intent(in) :: ncol,c                          ! chunk index and number of columns
   integer, intent(in) :: n                               ! prv or nxt month index
   integer, intent(in) :: naerloc                         ! number of aerosols
   integer, intent(in) :: naerlev                         ! levels of aerosols

   real(r8), intent(in) :: Match_ps(pcols)                ! surface pressure at a particular month
   real(r8), intent(in) :: pint(pcols,pverp)              ! interface pressure from CAM
   real(r8), intent(in) :: AEROSOLc(pcols,paerlev+1,begchunk:endchunk,naerloc,2)

   real(r8), intent(in) :: M_hybi(paerlev+1)                ! dataset hybi

   real(r8), intent(out) :: AEROSOL_mmr(pcols,pver,naerloc)  ! aerosol mmr from dataset
!
! Local workspace
!
   integer m                           ! index to aerosol species
   integer kupper(pcols)               ! last upper bound for interpolation
   integer i, k, kk, kkstart, kount    ! loop vars for interpolation
   integer isv, ksv, msv               ! loop indices to save

   logical bad                         ! indicates a bad point found
   logical lev_interp_comp             ! interpolation completed for a level 
   logical error_found

   real(r8) AEROSOL(pcols,pverp,naer)  ! cumulative mass of aerosol in column beneath upper 
                                       ! interface of level in column at particular month
   real(r8) dpl, dpu                   ! lower and upper intepolation factors
   real(r8) v_coord                    ! vertical coordinate
   real(r8) m_to_mmr                   ! mass to mass mixing ratio conversion factor
   real(r8) AER_diff                   ! temp var for difference between aerosol masses

   call t_startf ('vert_interpolate')
!
! Initialize index array 
!
   do i=1,ncol
      kupper(i) = 1
   end do
!
! assign total mass to topmost level
!
   AEROSOL(:,1,1:naerloc) = AEROSOLc(:,1,c,1:naerloc,n)
!
! At every pressure level, interpolate onto that pressure level
!
!  do k=2,pver
   do k=2,pverp
!
! Top level we need to start looking is the top level for the previous k
! for all longitude points
!
!     kkstart = paerlev+1
      kkstart = naerlev+1
      do i=1,ncol
         kkstart = min0(kkstart,kupper(i))
      end do
      kount = 0
!
! Store level indices for interpolation
!
! for the pressure interpolation should be comparing
! pint(column,lev) with M_hybi(lev)*M_ps_cam_col(month,column,chunk)
!
      lev_interp_comp = .false.
!     do kk=kkstart,paerlev
      do kk=kkstart,naerlev
         if(.not.lev_interp_comp) then
         do i=1,ncol
            v_coord = pint(i,k)
            if (M_hybi(kk)*Match_ps(i) .lt. v_coord .and. v_coord .le. M_hybi(kk+1)*Match_ps(i)) then
               kupper(i) = kk
               kount = kount + 1
            end if
         end do
!
! If all indices for this level have been found, do the interpolation and
! go to the next level
!
! Interpolate in pressure.
!
         if (kount.eq.ncol) then
            do m=1,naerloc
               do i=1,ncol
                  dpu = pint(i,k) - M_hybi(kupper(i))*Match_ps(i)
                  dpl = M_hybi(kupper(i)+1)*Match_ps(i) - pint(i,k)
                  AEROSOL(i,k,m) = &
                     (AEROSOLc(i,kupper(i)  ,c,m,n)*dpl + &
                     AEROSOLc(i,kupper(i)+1,c,m,n)*dpu)/(dpl + dpu)
               enddo !i
            end do
            lev_interp_comp = .true.
         end if
         end if
      end do
!
! If we've fallen through the kk=1,levsiz-1 loop, we cannot interpolate and
! must extrapolate from the bottom or top pressure level for at least some
! of the longitude points.
!

      if(.not.lev_interp_comp) then
         do m=1,naerloc
            do i=1,ncol
               if (pint(i,k) .lt. M_hybi(1)*Match_ps(i)) then
                  AEROSOL(i,k,m) =  AEROSOLc(i,1,c,m,n)
!              else if (pint(i,k) .gt. M_hybi(paerlev+1)*Match_ps(i)) then
               else if (pint(i,k) .gt. M_hybi(naerlev+1)*Match_ps(i)) then
                  AEROSOL(i,k,m) = 0.0
               else
                  dpu = pint(i,k) - M_hybi(kupper(i))*Match_ps(i)
                  dpl = M_hybi(kupper(i)+1)*Match_ps(i) - pint(i,k)
                  AEROSOL(i,k,m) = &
                     (AEROSOLc(i,kupper(i)  ,c,m,n)*dpl + &
                     AEROSOLc(i,kupper(i)+1,c,m,n)*dpu)/(dpl + dpu)
               end if
            end do
         end do

         if (kount.gt.ncol) then
            call endrun ('VERT_INTERPOLATE: Bad data: non-monotonicity suspected in dependent variable')
         end if
      end if
   end do

!   call t_startf ('vi_checks')
!
! aerosol mass beneath lowest interface (pverp) must be 0
!  ???, wzz
!
!  AEROSOL(1:ncol,pverp,:) = 0.
!
! Set mass in layer to zero whenever it is less than 
!   1.e-40 kg/m^2 in the layer
!
   do m = 1, naerloc
      do k = 1, pverp
!     do k = 1, pver
         do i = 1, ncol
            if (AEROSOL(i,k,m) < 1.e-40) AEROSOL(i,k,m) = 0.
         end do
      end do
   end do
!
! Set mass in layer to zero whenever it is less than 
!   10^-15 relative to column total mass
! convert back to mass mixing ratios. 
! exit if mmr is negative
!
   error_found = .false.
   do m = 1, naerloc
      do k = 1, pver
         do i = 1, ncol
            AER_diff = AEROSOL(i,k,m) - AEROSOL(i,k+1,m)
            if( abs(AER_diff) < 1e-15*AEROSOL(i,1,m)) then
               AER_diff = 0.
            end if
            m_to_mmr = gravit / (pint(i,k+1)-pint(i,k))
            AEROSOL_mmr(i,k,m)= AER_diff * m_to_mmr
            if (AEROSOL_mmr(i,k,m) < 0) error_found = .true.
         end do
      end do
   end do
   if (error_found) then
      do m = 1, naerloc
         do k = 1, pver
            do i = 1, ncol
               if (AEROSOL_mmr(i,k,m) < 0) then
                  write(6,*)'vert_interpolate: mmr < 0, m, col, lev, mmr',m, i, k, AEROSOL_mmr(i,k,m)
                  write(6,*)'vert_interpolate: aerosol(k),(k+1)',AEROSOL(i,k,m),AEROSOL(i,k+1,m)
                  write(6,*)'vert_interpolate: pint(k+1),(k)',pint(i,k+1),pint(i,k)
                  write(6,*)'n,c',n,c
                  call endrun()
               end if
            end do
         end do
      end do
   end if

!   call t_stopf ('vi_checks')
   call t_stopf ('vert_interpolate')

   return
end subroutine vert_interpolate

subroutine scale_aerosols(AEROSOLt, ncol, lchnk, scale)
!-----------------------------------------------------------------
! scale each species as determined by scale factors
!-----------------------------------------------------------------
  integer, intent(in) :: ncol, lchnk ! number of columns and chunk index
  real(r8), intent(in) :: scale(naer_all) ! scale each aerosol by this amount
  real(r8), intent(inout) :: AEROSOLt(pcols, pver, naer_all) ! aerosols
  integer m

  do m = 1, naer_all
     AEROSOLt(:ncol, :, m) = scale(m)*AEROSOLt(:ncol, :, m)
  end do

  return
end subroutine scale_aerosols

subroutine background(lchnk, ncol, pint, mmr)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Set global mean tropospheric aerosol background (or tuning) field
! 
! Method: 
! Specify aerosol mixing ratio.
! Aerosol mass mixing ratio
! is specified so that the column visible aerosol optical depth is a
! specified global number (tauback). This means that the actual mixing
! ratio depends on pressure thickness of the lowest three atmospheric
! layers near the surface.
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use aer_optics, only: kbg,idxVIS
   use physconst, only: gravit
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
#include <ptrrgrid.h>
!------------------------------Arguments--------------------------------
!
! Input arguments
!
   integer, intent(in) :: lchnk                 ! chunk identifier
   integer, intent(in) :: ncol                  ! number of atmospheric columns

   real(r8), intent(in) :: pint(pcols,pverrp)   ! Interface pressure (mks)
!
! Output arguments
!
   real(r8), intent(out) :: mmr(pcols,pverr)    ! "background" aerosol mass mixing ratio
!
!---------------------------Local variables-----------------------------
!
   integer i          ! Longitude index
   integer k          ! Level index
!
   real(r8) mass2mmr  ! Factor to convert mass to mass mixing ratio
   real(r8) mass      ! Mass of "background" aerosol as specified by tauback
!
!-----------------------------------------------------------------------
!
   do i=1,ncol
      mass2mmr =  gravit / (pint(i,pverrp)-pint(i,pverrp-mxaerl))
      do k=1,pverr
!
! Compute aerosol mass mixing ratio for specified levels (1.e3 factor is
! for units conversion of the extinction coefficiant from m2/g to m2/kg)
!
         if ( k >= pverrp-mxaerl ) then
! kaervs is not consistent with the values in aer_optics
! this ?should? be changed.
! rhfac is also implemented differently
            mass = tauback / (1.e3 * kbg(idxVIS))
            mmr(i,k) = mass2mmr*mass
         else
            mmr(i,k) = 0._r8
         endif
!
      enddo
   enddo
!
end subroutine background


subroutine aerosol_diagnostics(state, AEROSOL_mmr)
!-----------------------------------------------------------------
! write out the mass of aerosol in each layer
!-----------------------------------------------------------------

  use physics_types, only: physics_state
  use physconst,     only: rga
  use history,       only: outfld

  type(physics_state), intent(in) :: state     ! state from physics code
  real(r8), intent(in) :: AEROSOL_mmr(pcols, pver, naer_all) ! aerosols

  integer lchnk,ncol  ! chunk index and number of columns
  real(r8) :: MAerosol(pcols, pver, naer_all)  !mass of aerosols in each layer
! column totals of masses
  real(r8) :: TMAllAerosols(pcols), TMCarbon(pcols), TMDust(pcols) 
  real(r8) :: TMAerosol(pcols,naer_all)
  integer k,m         ! level, constituent indices

  TMAllAerosols = 0.0
  TMCarbon = 0.0
  TMDust = 0.0
  TMAerosol = 0.0

  ncol = state%ncol
  lchnk = state%lchnk
!
! output surface pressure from MATCH
!
  call outfld('PS_match',Match_ps_chunk(1,lchnk),pcols,lchnk)
!
! compute column mass of aerosols (and groups)
!
! first, mass in each layer
!
  do m = 1, naer_all-1
    MAerosol(:ncol,:,m) = AEROSOL_mmr(:ncol,:,m)*state%pdel(:ncol,:)*rga
  enddo
  MAerosol(:ncol,:,idxVOLC) = AEROSOL_mmr(:ncol,:,idxVOLC)
!
! now accumulate mass below upper interface bounding each layer
!
  do k = pver-1,1,-1
    MAerosol(:ncol,k,:) = MAerosol(:ncol,k+1,:) + MAerosol(:ncol,k,:)
  enddo
!
! total mass in column is given by cumulative mass at level = 1
!
  TMAerosol(:ncol,:) = MAerosol(:ncol,1,:)

  do m = idxCARBONfirst, idxCARBONfirst+numCARBON-1
    TMCarbon(:ncol) = TMCarbon(:ncol) + TMAerosol(:ncol,m)
  enddo

  do m = idxDUSTfirst, idxDUSTfirst+numDUST-1
    TMDust(:ncol) = TMDust(:ncol) + TMAerosol(:ncol,m)
  enddo

  TMAllAerosols(:ncol)=TMCarbon(:ncol)            &
                      +TMDust(:ncol)              &
                      +TMAerosol(:ncol,idxSUL )   &
                      +TMAerosol(:ncol,idxSSLT)   &
                      +TMAerosol(:ncol,idxVOLC)
!
! write out column totals of aerosol mmr's and groups
!
  call outfld('MSUL_V  ', MAerosol(:,:,idxSUL) ,pcols, lchnk)
! call outfld('MSUL_V  ', AEROSOL_mmr(:,:,idxSUL) ,pcols, lchnk)  ! wzz, 2009.9.23
  call outfld('MSSLT_V ', MAerosol(:,:,idxSSLT) ,pcols, lchnk)
  call outfld('MDUST1_V', MAerosol(:,:,idxDUSTfirst) ,pcols, lchnk)
  call outfld('MDUST2_V', MAerosol(:,:,idxDUSTfirst+1) ,pcols, lchnk)
  call outfld('MDUST3_V', MAerosol(:,:,idxDUSTfirst+2) ,pcols, lchnk)
  call outfld('MDUST4_V', MAerosol(:,:,idxDUSTfirst+3) ,pcols, lchnk)
  call outfld('MOCPHO_V', MAerosol(:,:,idxOCPHO) ,pcols, lchnk)
  call outfld('MOCPHI_V', MAerosol(:,:,idxOCPHI) ,pcols, lchnk)
  call outfld('MBCPHO_V', MAerosol(:,:,idxBCPHO) ,pcols, lchnk)
  call outfld('MBCPHI_V', MAerosol(:,:,idxBCPHI) ,pcols, lchnk)
  call outfld('MBG_V   ', MAerosol(:,:,idxBG) ,pcols, lchnk)
  call outfld('MVOLC   ', MAerosol(:,:,idxVOLC) ,pcols, lchnk)

  return
end subroutine aerosol_diagnostics

subroutine aerosol_indirect(ncol,lchnk,landfrac,pmid,t,qm1,cld,zm,rel)
!--------------------------------------------------------------
! Compute effect of sulfate on effective liquid water radius
!  Method of Martin et. al.
!--------------------------------------------------------------

  use constituents, only: ppcnst, cnst_get_ind
  use history,      only: outfld

#include <comctl.h>

  integer, intent(in) :: ncol                  ! number of atmospheric columns
  integer, intent(in) :: lchnk                 ! chunk identifier

  real(r8), intent(in) :: landfrac(pcols)      ! land fraction
  real(r8), intent(in) :: pmid(pcols,pver)     ! Model level pressures
  real(r8), intent(in) :: t(pcols,pver)        ! Model level temperatures
  real(r8), intent(in) :: qm1(pcols,pver,ppcnst) ! Specific humidity and tracers
  real(r8), intent(in) :: cld(pcols,pver)      ! Fractional cloud cover
  real(r8), intent(in) :: zm(pcols,pver)       ! Height of midpoints (above surface)
  real(r8), intent(in) :: rel(pcols,pver)      ! liquid effective drop size (microns)
!
! local variables
!
  real(r8) locrhoair(pcols,pver)  ! dry air density            [kg/m^3 ]
  real(r8) lwcwat(pcols,pver)     ! in-cloud liquid water path [kg/m^3 ]
  real(r8) sulfmix(pcols,pver)    ! sulfate mass mixing ratio  [kg/kg  ]
  real(r8) so4mass(pcols,pver)    ! sulfate mass concentration [g/cm^3 ]
  real(r8) Aso4(pcols,pver)       ! sulfate # concentration    [#/cm^3 ]
  real(r8) Ntot(pcols,pver)       ! ccn # concentration        [#/cm^3 ]
  real(r8) relmod(pcols,pver)     ! effective radius           [microns]

  real(r8) wrel(pcols,pver)       ! weighted effective radius    [microns]
  real(r8) wlwc(pcols,pver)       ! weighted liq. water content  [kg/m^3 ]
  real(r8) cldfrq(pcols,pver)     ! frequency of occurance of...
!                                  ! clouds (cld => 0.01)         [fraction]
  real(r8) locPi                  ! my piece of the pi
  real(r8) Rdryair                ! gas constant of dry air   [J/deg/kg]
  real(r8) rhowat                 ! density of water          [kg/m^3  ]
  real(r8) Acoef                  ! m->A conversion factor; assumes
!                                  ! Dbar=0.10, sigma=2.0      [g^-1    ]
  real(r8) rekappa                ! kappa in evaluation of re(lmod)
  real(r8) recoef                 ! temp. coeficient for calc of re(lmod)
  real(r8) reexp                  ! 1.0/3.0
  real(r8) Ntotb                  ! temp var to hold below cloud ccn
! -- Parameters for background CDNC (from `ambient' non-sulfate aerosols)...
  real(r8) Cmarn                  ! Coef for CDNC_marine         [cm^-3]
  real(r8) Cland                  ! Coef for CDNC_land           [cm^-3]
  real(r8) Hmarn                  ! Scale height for CDNC_marine [m]
  real(r8) Hland                  ! Scale height for CDNC_land   [m]
  parameter ( Cmarn = 50.0, Cland = 100.0 )
  parameter ( Hmarn = 1000.0, Hland = 2000.0 )
  real(r8) bgaer                  ! temp var to hold background CDNC
  integer :: ixcldliq               ! index of liquid cloud water

  integer i,k     ! loop indices
!
! Statement functions
!
  logical land    ! is this a column over land?
  land(i) = nint(landfrac(i)).gt.0.5_r8

  if (indirect) then
 
    call endrun ('AEROSOL_INDIRECT:  indirect effect is obsolete')

!   ramping is not yet resolved so sulfmix is 0.
    sulfmix(1:ncol,1:pver) = 0._r8

    locPi = 3.141592654
    Rdryair = 287.04
    rhowat = 1000.0
    Acoef = 1.2930E14
    recoef = 3.0/(4.0*locPi*rhowat)
    reexp = 1.0/3.0

    call cnst_get_ind('CLDLIQ', ixcldliq)
    do k=pver,1,-1
      do i = 1,ncol
        locrhoair(i,k) = pmid(i,k)/( Rdryair*t(i,k) )
        lwcwat(i,k) = ( qm1(i,k,ixcldliq)/max(0.01_r8,cld(i,k)) )* &
                      locrhoair(i,k)
!          NOTE: 0.001 converts kg/m3 -> g/cm3
        so4mass(i,k) = sulfmix(i,k)*locrhoair(i,k)*0.001
        Aso4(i,k) = so4mass(i,k)*Acoef

        if (Aso4(i,k) <= 280.0) then
           Aso4(i,k) = max(36.0_r8,Aso4(i,k))
           Ntot(i,k) = -1.15E-3*Aso4(i,k)**2 + 0.963*Aso4(i,k)+5.30
           rekappa = 0.80
        else
           Aso4(i,k) = min(1500.0_r8,Aso4(i,k))
           Ntot(i,k) = -2.10E-4*Aso4(i,k)**2 + 0.568*Aso4(i,k)-27.9
           rekappa = 0.67
        end if
        if (land(i)) then ! Account for local background aerosol;
           bgaer = Cland*exp(-(zm(i,k)/Hland))
           Ntot(i,k) = max(bgaer,Ntot(i,k))
        else
           bgaer = Cmarn*exp(-(zm(i,k)/Hmarn))
           Ntot(i,k) = max(bgaer,Ntot(i,k))
        end if

        if (k == pver) then
           Ntotb = Ntot(i,k)
        else
           Ntotb = Ntot(i,k+1)
        end if

        relmod(i,k) = (( (recoef*lwcwat(i,k))/(rekappa*Ntotb))**reexp)*10000.0
        relmod(i,k) = max(4.0_r8,relmod(i,k))
        relmod(i,k) = min(20.0_r8,relmod(i,k))
        if (cld(i,k) >= 0.01) then
           cldfrq(i,k) = 1.0
        else
           cldfrq(i,k) = 0.0
        end if
        wrel(i,k) = relmod(i,k)*cldfrq(i,k)
        wlwc(i,k) = lwcwat(i,k)*cldfrq(i,k)
      end do
    end do
    call outfld('MSO4    ',so4mass,pcols,lchnk)
    call outfld('LWC     ',lwcwat ,pcols,lchnk)
    call outfld('CLDFRQ  ',cldfrq ,pcols,lchnk)
    call outfld('WREL    ',wrel   ,pcols,lchnk)
    call outfld('WLWC    ',wlwc   ,pcols,lchnk)
    write(6,*)'WARNING: indirect calculation has no effects'
  else
    do k = 1, pver
      do i = 1, ncol
        relmod(i,k) = rel(i,k)
      end do
    end do
  endif

  return
end subroutine aerosol_indirect

subroutine setmxaerl()
!------------------------------------
!
! set levels for background aerosol
!
!-----------------------------------

! need hypm from comhyb.h
#include <comhyb.h>

   integer k ! index through vertical levels
!
! mxaerl = max number of levels (from bottom) for background aerosol
! Limit background aerosol height to regions below 900 mb
!
   mxaerl = 0
   do k=pver,1,-1
      if (hypm(k) >= 9.e4) mxaerl = mxaerl + 1
   end do
   mxaerl = max(mxaerl,1)
   if (masterproc) then
      write(6,*)'AEROSOLS:  Background aerosol will be limited to ', &
                'bottom ',mxaerl,' model interfaces. Top interface is ', &
                hypi(pverp-mxaerl),' pascals'
   end if

   return
end subroutine setmxaerl

subroutine add_aer_diag_fields()
!-------------------------------------------------------------
! make sure aerosols are in the master list so they can be written
!-------------------------------------------------------------

  use history, only: phys_decomp, addfld, add_default  ! sxj add add_default
!++sxj
  use pmgrid,        only: masterproc,iam    
!--sxj

  call addfld ('PS_match','N/m^2   ',1, 'I','Surface Pressure from aerosol climatology' ,phys_decomp)
  call addfld ('frc_day ','None    ',1, 'I','Portion of time column is lit' ,phys_decomp)

  call addfld ('MSUL_V  ','Kg/m^2  ',pver,'I','Mass of Sulfate in and below layer',phys_decomp)
  call addfld ('MSSLT_V ','Kg/m^2  ',pver,'I','Mass of Sea Salt in and below layer',phys_decomp)
  call addfld ('MDUST1_V','Kg/m^2  ',pver,'I','Mass of Dust bin 1 in and below layer',phys_decomp)
  call addfld ('MDUST2_V','Kg/m^2  ',pver,'I','Mass of Dust bin 2 in and below layer',phys_decomp)
  call addfld ('MDUST3_V','Kg/m^2  ',pver,'I','Mass of Dust bin 3 in and below layer',phys_decomp)
  call addfld ('MDUST4_V','Kg/m^2  ',pver,'I','Mass of Dust bin 4 in and below layer',phys_decomp)
  call addfld ('MOCPHO_V','Kg/m^2  ',pver,'I','Mass of OCPHO in and below layer',phys_decomp)
  call addfld ('MOCPHI_V','Kg/m^2  ',pver,'I','Mass of OCPHI in and below layer',phys_decomp)
  call addfld ('MBCPHO_V','Kg/m^2  ',pver,'I','Mass of BCPHO in and below layer',phys_decomp)
  call addfld ('MBCPHI_V','Kg/m^2  ',pver,'I','Mass of BCPHI in and below layer',phys_decomp)
  call addfld ('MBG_V   ','Kg/m^2  ',pver,'I','Mass of Background Aerosol in and below layer',phys_decomp)
  call addfld ('MVOLC   ','Kg/m^2  ',pver,'I','Mass of Volcanic Aerosol in layer',phys_decomp)

!++sxj
!  call addfld ('SULOD_v ','None    ',1, 'I','Sulfate Optical Depth in visible', phys_decomp)
!  call addfld ('SSLTOD_v','None    ',1, 'I','Sea Salt Optical Depth in visible', phys_decomp)
!  call addfld ('CAROD_v ','None    ',1, 'I','Carbon Optical Depth in visible', phys_decomp)
!  call addfld ('DUSTOD_v','None    ',1, 'I','Dust Optical Depth in visible', phys_decomp)
!  call addfld ('BGOD_v  ','None    ',1, 'I','Background Aerosol Optical Depth in visible', phys_decomp)
!  call addfld ('VOLCOD_v','None    ',1, 'I','Volcanic Aerosol Optical Depth in visible', phys_decomp)
!  call addfld ('AEROD_v ','None    ',1, 'I','Total Aerosol Optical Depth in visible', phys_decomp)
!  call addfld ('AERSSA_v','None    ',1, 'I','Total Aerosol Single Scattering Albedo in visible', phys_decomp)
!  call addfld ('AERASM_v','None    ',1, 'I','Total Aerosol Asymmetry Parameter in visible', phys_decomp)
!  call addfld ('AERFWD_v','None    ',1, 'I','Total Aerosol Forward Scattering in visible', phys_decomp)
  call addfld ('SULOD_v ','None    ',1, 'A','Sulfate Optical Depth in visible', phys_decomp)
  call addfld ('SSLTOD_v','None    ',1, 'A','Sea Salt Optical Depth in visible', phys_decomp)
  call addfld ('CAROD_v ','None    ',1, 'A','Carbon Optical Depth in visible', phys_decomp)
  call addfld ('DUSTOD_v','None    ',1, 'A','Dust Optical Depth in visible', phys_decomp)
#ifdef MAC_SP
  call addfld ('AERANT_v','None    ',1, 'A','Anthropogenic & Background Aerosol Optical Depth in visible', phys_decomp)
#else
  call addfld ('BGOD_v  ','None    ',1, 'A','Background Aerosol Optical Depth in visible', phys_decomp)
#endif
  call addfld ('VOLCOD_v','None    ',1, 'A','Volcanic Aerosol Optical Depth in visible', phys_decomp)
  call addfld ('AEROD_v ','None    ',1, 'A','Total Aerosol Optical Depth in visible', phys_decomp)
  call addfld ('AERSSA_v','None    ',1, 'A','Total Aerosol Single Scattering Albedo in visible', phys_decomp)
  call addfld ('AERASM_v','None    ',1, 'A','Total Aerosol Asymmetry Parameter in visible', phys_decomp)
  call addfld ('AERFWD_v','None    ',1, 'A','Total Aerosol Forward Scattering in visible', phys_decomp)

  !if ( masterproc) write(6,*) "SXJ prescribed_aerosols.F90  add_aer_diag_fields radAIE",radAIE
  if(radAIE) then   
     call addfld ('FSNT_NA ','W/m^2   ',1, 'A','Total column absorbed solar flux (no arosol)' ,phys_decomp)
     call addfld ('FSNTC_NA','W/m^2   ',1, 'A','Clear sky total column absorbed solar flux (no arosol)' ,phys_decomp)
     call addfld ('FSNS_NA ','W/m^2   ',1, 'A','Surface absorbed solar flux (no arosol)' ,phys_decomp)
     call addfld ('FSNSC_NA','W/m^2   ',1, 'A','Clear sky surface absorbed solar flux (no arosol)' ,phys_decomp)
     call addfld ('QRS_NA'  ,'K/s  ',pver, 'A','Solar heating rate (no arosol)' ,phys_decomp)
     call add_default ('FSNT_NA'   , 1, ' ')
     call add_default ('FSNTC_NA'  , 1, ' ')
     call add_default ('FSNS_NA'   , 1, ' ')
     call add_default ('FSNSC_NA'  , 1, ' ')
     call add_default ('QRS_NA'    , 1, ' ')  
     call add_default ('SULOD_v'   , 1, ' ')
     call add_default ('SSLTOD_v'  , 1, ' ')
     call add_default ('CAROD_v'   , 1, ' ')
     call add_default ('DUSTOD_v'  , 1, ' ')
     call add_default ('BGOD_v'    , 1, ' ') 
     call add_default ('VOLCOD_v'  , 1, ' ')
     call add_default ('AEROD_v'   , 1, ' ')
     call add_default ('AERSSA_v'  , 1, ' ')
     call add_default ('AERASM_v'  , 1, ' ')
     call add_default ('AERFWD_v'  , 1, ' ')   
  endif
!--sxj  

 
  
  return

end subroutine add_aer_diag_fields

subroutine get_rf_scales(scales)

  real(r8), intent(out)::scales(naer_all)  ! scale aerosols by this amount
    
  integer i                                  ! loop index

  scales(idxBG) = bgscl_rf
  scales(idxSUL) = sulscl_rf
  scales(idxSSLT) = ssltscl_rf
!---wtw
  scales(idxSSLT01) = ssltscl_rf
  scales(idxNH4NO3) = ssltscl_rf
!---
  do i = idxCARBONfirst, idxCARBONfirst+numCARBON-1
    scales(i) = carscl_rf
  enddo

  do i = idxDUSTfirst, idxDUSTfirst+numDUST-1
    scales(i) = dustscl_rf
  enddo

  scales(idxVOLC) = volcscl_rf

end subroutine get_rf_scales

subroutine get_int_scales(scales)
  real(r8), intent(out)::scales(naer_all)  ! scale each aerosol by this amount
  integer i                                  ! index through species

  scales(idxBG) = 1._r8
  scales(idxSUL) = sulscl 
  scales(idxSSLT) = ssltscl 
!---wtw
  scales(idxSSLT01) = ssltscl
  scales(idxNH4NO3) = ssltscl
!---
  do i = idxCARBONfirst, idxCARBONfirst+numCARBON-1
    scales(i) = carscl
  enddo

  do i = idxDUSTfirst, idxDUSTfirst+numDUST-1
    scales(i) = dustscl 
  enddo

  scales(idxVOLC) = volcscl

  return
end subroutine get_int_scales

subroutine aerint ()
   use carbonscales, only: get_carbonscale
!----------------------zf 2010.11.08
   use history,        only: outfld
!---------------------

   implicit none

   integer :: ntmp                                ! used in index swapping
   integer :: start(4)                            ! start vector for netcdf calls
   integer :: kount(4)                            ! count vector for netcdf calls
   integer :: i,j,k                               ! spatial indices
   integer :: m                                   ! constituent index

   real(r8) :: caldayloc                          ! calendar day of current timestep
   real(r8) :: aerosol_data(plon,plat,paerlev)    ! aerosol field read in from dataset
!zf   real(r8) :: aerosol_field(plon,paerlev+1,plat) ! aerosol field to be scattered
!-----------------------zf 2010.11.16---------------------
   integer  :: istat          !   error return
   real(r8) :: aerosol_field(plond,paerlev+1,plat) ! aerosol field to be scattered
   real(r8), allocatable :: aero_tempa(:,:,:)
   real(r8), allocatable :: aero_tempb(:,:,:)
   real(r8), allocatable :: psloc_tempa(:,:)
   real(r8), allocatable :: psloc_tempb(:,:)
!----------------------------------------------------------
!
! determine if need to read in next month data
! also determine time interpolation factors
!
   caldayloc = get_curr_calday ()  
!
! If model time is past current forward ozone timeslice, then
! masterproc reads in the next timeslice for time interpolation.  Messy logic is 
! for interpolation between December and January (mo_nxt == 1).  Just like
! oznint, sstint.
!
!---------------------zf 2010.11.16----------------------------
      allocate( aero_tempa(naerlon,naerlat,paerlev), stat=istat )
      allocate( aero_tempb(naerlon,paerlev+1,naerlat), stat=istat )
      allocate( psloc_tempa(naerlon,naerlat), stat=istat ) 
      allocate( psloc_tempb(plond,plat), stat=istat ) 
!--------------------------------------------------------------- 
   if (scenario_prescribed_aeros=='FIXED') then
   if (caldayloc > cdayp .and. .not. (mo_nxt == 1 .and. caldayloc > cdaym)) then
      mo_nxt = mod(mo_nxt,12) + 1
      cdaym = cdayp
      cdayp = Mid(mo_nxt)
!
! Check for valid date info
!
      if (.not. (mo_nxt == 1 .or. caldayloc <= cdayp)) then
         call endrun ('AERINT: Non-monotonicity suspected in input aerosol data')
      end if

      ntmp = nm
      nm = np
      np = ntmp

!
! for sulfur data, 2009.9.8
      if (scenario_prescribed_sulfur=='FIXED') then
         if (masterproc) then
            aerosol_data = 0.0
            start(:) = (/1,1,1,mo_nxt/)
!zf            kount(:) = (/plon,plat,nsullev,1/)
!zf         call wrap_get_vara_realx (sulnid, species_id(idxSUL), start, kount, aerosol_data(1,1,1))
!zf            do j=1,plat
!zf               do k=1,paerlev
!zf                  aerosol_field(:,k,j) = aerosol_data(:,j,k)
!zf               end do
!zf               aerosol_field(:,paerlev+1,j) = 0.   ! value at bottom
!zf            end do
!zf         end if
!zf         call scatter_field_to_chunk (1, paerlev+1, 1, plon, aerosol_field, AEROSOLc(1,1,begchunk,idxSUL,np))

!----------------------------zf 2010.11.16--------------------------------------------------------
        kount(:) = (/naerlon,naerlat,nsullev,1/)
        call wrap_get_vara_realx (sulnid, species_id(idxSUL), start, kount, aero_tempa(1,1,1))
        do j = 1, naerlat
          do k=1,paerlev
            aero_tempb(:,k,j) = aero_tempa(:,j,k)
          end do
          aero_tempb(:,paerlev+1,j) = 0.   ! value at bottom
        end do
        
        aerosol_field = 0.
        if(naerlon .eq. plon .and. naerlat .eq. plat) then
          aerosol_field(1:plon,:,:) = aero_tempb(1:plon,:,:)
        else
          
          call bilin(aero_tempb,aerolon,aerolat,naerlon,naerlon,paerlev+1,paerlev+1,   &
                     naerlat,aerosol_field,londeg,latdeg,plond,nlon,paerlev+1,plat)
        end if
        
        end if
        call scatter_field_to_chunk (1, paerlev+1, 1, plon, aerosol_field(1:plon,:,:), AEROSOLc(1,1,begchunk,idxSUL,np))
!--------------------------------------------------------------------------------------------------
!
         if (masterproc) then
            start(:) = (/1,1,mo_nxt,-1/)
!zf            kount(:) = (/plon,plat,1,-1/)
!zf            call wrap_get_vara_realx (aernid, Mpsid, start, kount, M_ps(1,1))
!zf            write(6,*)'AERINT: Read sulfate  data for julian day', Mid(mo_nxt)
!zf         end if
!zf         call scatter_field_to_chunk (1, 1, 1, plon, M_ps(1,1), M_ps_sul_col(1,begchunk,np))
!-------------------------zf 2010.11.16----------------------------------------------------------
           kount(:) = (/naerlon,naerlat,1,-1/)
           call wrap_get_vara_realx (aernid, Mpsid, start, kount, psloc_tempa(1,1))
           write(6,*)'AERINT: Read sulfate  data for julian day', Mid(mo_nxt)
           M_ps = 0.
           if(naerlon .eq. plon .and. naerlat .eq. plat) then
             M_ps(1:plon,:) = psloc_tempa(1:plon,:)
           else
             call bilin(psloc_tempa,aerolon,aerolat,naerlon,naerlon,1,1,   &
                        naerlat,psloc_tempb,londeg,latdeg,plond,nlon,1,plat)
             M_ps(1:plon,:) = psloc_tempb(1:plon,:)
           end if
         end if
         call scatter_field_to_chunk (1, 1, 1, plon, M_ps, M_ps_sul_col(1,begchunk,np))
!--------------------------------------------------------------------------------------------------
      endif

!
!     do m=1,naer
      do m=2,naer
         if (masterproc) then
            start(:) = (/1,1,1,mo_nxt/)
!zf            kount(:) = (/plon,plat,paerlev,1/)
!zf            call wrap_get_vara_realx (aernid, species_id(m), start, kount, aerosol_data(1,1,1))
!zf            do j=1,plat
!zf               do k=1,paerlev
!zf                  aerosol_field(:,k,j) = aerosol_data(:,j,k)
!zf               end do
!zf               aerosol_field(:,paerlev+1,j) = 0.   ! value at bottom
!zf            end do
!zf         end if
!zf         call scatter_field_to_chunk (1, paerlev+1, 1, plon, aerosol_field, AEROSOLc(1,1,begchunk,m,np))
!----------------------------zf 2010.11.16--------------------------------------------------------
        kount(:) = (/naerlon,naerlat,paerlev,1/) 
        call wrap_get_vara_realx (aernid, species_id(m), start, kount, aero_tempa(1,1,1))
        do j = 1, naerlat
          do k=1,paerlev
            aero_tempb(:,k,j) = aero_tempa(:,j,k)
          end do
          aero_tempb(:,paerlev+1,j) = 0.   ! value at bottom
        end do
                        
        aerosol_field = 0.
        if(naerlon .eq. plon .and. naerlat .eq. plat) then
          aerosol_field(1:plon,:,:) = aero_tempb(1:plon,:,:)
        else
      
          call bilin(aero_tempb,aerolon,aerolat,naerlon,naerlon,paerlev+1,paerlev+1,   &
                     naerlat,aerosol_field,londeg,latdeg,plond,nlon,paerlev+1,plat)
        end if   
         
        end if
        call scatter_field_to_chunk (1, paerlev+1, 1, plon, aerosol_field(1:plon,:,:), AEROSOLc(1,1,begchunk,m,np))
!--------------------------------------------------------------------------------------------------
      end do
!
! Retrieve PS from Match
!
      if (masterproc) then
         start(:) = (/1,1,mo_nxt,-1/)
!zf         kount(:) = (/plon,plat,1,-1/)
!zf         call wrap_get_vara_realx (aernid, Mpsid, start, kount, M_ps(1,1))
!zf         write(6,*)'AERINT: Read aerosols data for julian day', Mid(mo_nxt)
!zf      end if
!zf      call scatter_field_to_chunk (1, 1, 1, plon, M_ps(1,1), M_ps_cam_col(1,begchunk,np))
!-------------------------zf 2010.11.16----------------------------------------------------------
      kount(:) = (/naerlon,naerlat,1,-1/)  
      call wrap_get_vara_realx (aernid, Mpsid, start, kount, psloc_tempa(1,1))
      write(6,*)'AERINT: Read aerosols data for julian day', Mid(mo_nxt)
      M_ps = 0.
      if(naerlon .eq. plon .and. naerlat .eq. plat) then
        M_ps(1:plon,:) = psloc_tempa(1:plon,:)
      else
        call bilin(psloc_tempa,aerolon,aerolat,naerlon,naerlon,1,1,   &
                   naerlat,psloc_tempb,londeg,latdeg,plond,nlon,1,plat)
        M_ps(1:plon,:) = psloc_tempb(1:plon,:)
      end if
      end if
      call scatter_field_to_chunk (1, 1, 1, plon, M_ps, M_ps_cam_col(1,begchunk,np))
!--------------------------------------------------------------------------------------------------
   end if
   endif !(scenario_prescribed_aero=='FIXED')

!
   if (scenario_prescribed_aeros=='RAMPED') then
!  wzz for test, 2010.4.26

      call aeroint_IPCC
   else
   if (scenario_prescribed_sulfur=='RAMPED') then
!
      call sulfint_IPCC
   endif
   endif !(scenario_prescribed_aero=='RAMPED')
! Call get_carbonscale (if required) to set carbon scale
!   at this particular time step
!
 
   if(scenario_carbon_scale.eq."RAMPED") then
     call get_carbonscale(carscl)
!     if(masterproc) then
!       write(6,*) "aerint: carscl" , carscl, 'caldayloc',caldayloc
!     endif
   endif

!------------zf--------------
   deallocate (aero_tempa)
   deallocate (aero_tempb)
   deallocate (psloc_tempa)
   deallocate (psloc_tempb)
!----------------------------
   return
end subroutine aerint

subroutine sulfini_IPCC(levsiz, m_bi)
!----------------------------------------------------------------------- 
! 
! Purpose: Do initial read of time-variant sulfate dataset, containing
!          sulfate mixing ratios as a function of time.  It is currently
!          required that the sulfate dataset have the *SAME* horizontal
!          and vertical resolution as the model. Therefore, ONLY a time
!          interpolation of the dataset is currently performed.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use error_messages, only: alloc_err, handle_ncerr
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
!-----------------------------------------------------------------------
   implicit none

   integer,  intent(out) :: levsiz     ! size of level dimension on sulfate dataset
   real(r8), intent(out) :: m_bi(paerlev+1)       ! vertical level values of sulfate dataset
!
   integer :: ncid_sulf_am=-1, ncid_sulf_bm  ! sulfate dataset id
   integer :: sulf_id_am,   sulf_id_bm    ! netcdf id for sulfate mmr bio variable

   integer :: lonsiz     ! size of longitude dimension on sulfate dataset
   integer :: latsiz     ! size of latitude dimension on sulfate dataset
   integer :: timsiz     ! size of time dimension on sulfate dataset

   character*16  ::  IPCC_scenario   ! name of the IPCC scenario
   character*256 ::  sulfdatafile_am ! full pathname for sulfate dataset
   character*256 ::  sulfdatafile_bm ! full pathname for sulfate dataset
   character*256 ::  sulfdatafile_ap ! full pathname for sulfate dataset

   character*80, parameter ::  p1='so4.'   ! part of the name of sulfate data file
   character*80, parameter ::  p2='.nc'   ! part of the name of sulfate data file

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!
! Local workspace
!
   character(len=256) :: locfn_am, locfn_bm, locfn_ap, locfn_bp     ! local filenames

   integer londimid              ! netcdf id for longitude dimension
   integer latdimid              ! netcdf id for latitude dimension
   integer levdimid              ! netcdf id for level dimension
   integer lonid                 ! netcdf id for longitude variable
   integer latid                 ! netcdf id for latitude variable
   integer levid                 ! netcdf id for level variable
   integer timeid                ! netcdf id for time variable
   integer cnt4(4)               ! array of counts for each dimension
   integer strt4(4)              ! array of starting indices
   integer i, k, lat, n          ! longitude, level, latitude, time indices
   integer istat                 ! error return
   integer dimids(nf_max_var_dims) ! netcdf variable shape

   integer yr, mon, day, ncsec ! components of a date
   real(r8) calday                   ! current calendar day
   real(r8) caldayloc                ! calendar day (includes yr if no cycling)
   real(r8) fact1, fact2, deltat

   character*4 yrtmp

!zf   real(r8) xsulfi_am(plon,pver,plat)  ! input sulfate bio mixing ratios
!zf   real(r8) xsulfi_bm(plon,pver,plat)  ! input sulfate ant mixing ratios
!zf   real(r8) xsulfi_ap(plon,pver,plat)  ! input sulfate bio mixing ratios
!zf   real(r8) xsulfi_bp(plon,pver,plat)  ! input sulfate ant mixing ratios

!---------------------zf 2010.11.18----------------------------------------
   real(r8) xsulfi_am(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xsulfi_bm(plond,pver,plat)  ! input sulfate ant mixing ratios
   real(r8) xsulfi_ap(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xsulfi_bp(plond,pver,plat)  ! input sulfate ant mixing ratios
   real(r8), allocatable :: xsulfi_temp(:,:,:)   
!---------------------------------------------------------------------------
   real(r8) xsulfi(plon,pver,plat,2)   ! input sulfate ant mixing ratios
!
!-----------------------------------------------------------------------
!
! Allocate space for data.
!
  allocate( sulfi(pcols,pver,begchunk:endchunk,2), stat=istat )
  call alloc_err( istat, 'sulfini', 'sulfi', pcols*pver*(endchunk-begchunk+1)*2 )
  allocate( sulf(pcols,pver,begchunk:endchunk), stat=istat )
  call alloc_err( istat, 'sulfini', 'sulf',  pcols*pver*(endchunk-begchunk+1) )
!
! unit shift from second to day
!
   do n=1,nsample
      sample_sulf(n) = sample_sulf(n)/86400.0+1.0
   enddo
!
! the names of rawdata files
!
   do n=1,nfile
      write(yrtmp,'(i4.4)') yr_sulf(n)
      file_sulf(n) = trim(p1)//yrtmp//trim(p2)
   enddo

! 2009.11.21
!  if (trim(IPCC_scenario)/='20CN') then
!     do n=12,19
!        write(yrtmp,'(i4.4)') yr_sulf(n)
!        file_sulf(n) = trim(p1)//trim(IPCC_scenario)//'.'//yrtmp//trim(p2)
!     enddo
!  end if

!  if (masterproc) then
!     do n=1,nfile
!       write(*,*) n,trim(file_sulf(n))
!     enddo
!      write(*,*) '-----------------------'
!     write(*,*)'wzz: sample_sulf= ', sample_sulf
!  endif
!
! set the time_boundaries of input dataset
!
   bndry_m = yr_sulf(1    )*365.0 + sample_sulf(1) 
   bndry_p = yr_sulf(nfile)*365.0 + sample_sulf(nsample) 
  
!!   if (masterproc) then
!!       write(*,*) 'bndry_m=',bndry_m
!!       write(*,*) 'bndry_p=',bndry_p
!!   endif
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
!  if (masterproc) then

     calday = get_curr_calday()
     call get_curr_date(yr, mon, day, ncsec)
     if ( rampyear_prescribed_sulfur .ne. bigint ) then
        yr = rampyear_prescribed_sulfur        ! wzz, 2009.10.15
     endif
     caldayloc = calday + yr*365.

!----------------------------------------------------------------
! locate the model time among the samples of sulfate data file      
!---------------------------------------------------------------
    if ( caldayloc.lt.bndry_m ) then

        outofbndry = .true.
        cdaysulfp  = bndry_m
        sp_p   = 1
        yrp    = yr_sulf(1)
        yr_ap  = yr_sulf(1)
        yr_bp  = yr_sulf(2)
        indxbp = 2
        sulfdatafile_ap = trim(sulfdata)//file_sulf(1)
        sulfdatafile_bp = trim(sulfdata)//file_sulf(2)

    elseif ( caldayloc.ge.bndry_p ) then

        outofbndry = .true.
        sp_p   = nsample 
        yrp    = yr_sulf(nfile)
        yr_ap  = yr_sulf(nfile)
        yr_bp  = yr_sulf(nfile)
        sulfdatafile_ap = trim(sulfdata)//file_sulf(nfile)

    else
        outofbndry = .false.
!
! month
!
       if ( calday.lt.sample_sulf(1) ) then
          yrm  = yr-1
          yrp  = yr
          sp_m = nsample 
          sp_p = 1
          goto 100
       elseif (calday.ge.sample_sulf(nsample) ) then
          yrm = yr
          yrp = yr+1
          sp_m = nsample
          sp_p = 1 
          goto 100
       else 
         do n=1,nsample-1
            if ( calday.ge.sample_sulf(n) .and. calday.lt.sample_sulf(n+1) ) then
               yrm  = yr
               yrp  = yr
               sp_m = n
               sp_p = n+1
               goto 100
            endif
         enddo
       endif
       write(*,*) 'SULFINI: Failed to find seconds bracketing model_time =',calday
       call endrun
100    continue
       if ( rampyear_prescribed_sulfur .ne. bigint ) then
          yrm = rampyear_prescribed_sulfur        ! wzz, 2009.10.15
          yrp = rampyear_prescribed_sulfur        ! wzz, 2009.10.15
       endif
       cdaysulfm = yrm*365.0+sample_sulf(sp_m)
       cdaysulfp = yrp*365.0+sample_sulf(sp_p)
!
! year ( both yrm & yrp )
!
       do n=1,nfile-1
          yr_am = yr_sulf(n)
          yr_bm = yr_sulf(n+1)
          if ( yrm.ge.yr_am .and. yrm.lt.yr_bm ) goto 200
       enddo
       write(*,*) 'SULFINI: Failed to find years bracketing year  yrm=',yrm
       call endrun
200    continue
       sulfdatafile_am = trim(sulfdata)//trim(file_sulf(n))
       sulfdatafile_bm = trim(sulfdata)//trim(file_sulf(n+1))
 
       do n=1,nfile-1
          yr_ap = yr_sulf(n)
          yr_bp = yr_sulf(n+1)
          if ( yrp.ge.yr_ap .and. yrp.lt.yr_bp ) goto 300
       enddo
       write(*,*) 'SULFINI: Failed to find years bracketing year  yrp=',yrp
       call endrun
300    indxbp = n+1 
       sulfdatafile_ap = trim(sulfdata)//trim(file_sulf(n))
       sulfdatafile_bp = trim(sulfdata)//trim(file_sulf(n+1))

    endif

   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    if ( outofbndry ) then

      call getfil(sulfdatafile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_sulf_ap)
!
! Obtain dimension id's
!
      call wrap_inq_dimid( ncid_sulf_ap, 'lon', londimid)
      call wrap_inq_dimid( ncid_sulf_ap, 'lat', latdimid)
      call wrap_inq_dimid( ncid_sulf_ap, 'lev', levdimid)
      call wrap_inq_dimid( ncid_sulf_ap, 'time',timeid  )
!
! Obtain size of dimensions.
! Check that horizontal and vertical dimensions are same as model's
! (check one file only and assume that all the other files are of the same setting)
!
      call wrap_inq_dimlen( ncid_sulf_ap, londimid, lonso4siz   )
!zf      call wrap_inq_dimlen( ncid_sulf_ap, londimid, lonsiz   )
!zf      if (lonsiz /= plon) then
!zf         write(6,*)'SULFINI: lonsiz=',lonsiz,' must = ',plon
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_sulf_ap, latdimid, latso4siz   )
!zf      call wrap_inq_dimlen( ncid_sulf_ap, latdimid, latsiz   )
!zf      if (latsiz /= plat) then
!zf         write(6,*)'SULFINI: latsiz=',latsiz,' must = ',plat
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_sulf_ap, levdimid, levsiz   )
!     if (levsiz /= pver) then
      if (levsiz > pver) then
         write(6,*)'SULFINI: levsiz=',levsiz,' must le ',pver
         call endrun
      end if

!-----------------zf 2010.11.17------------
      allocate( xsulfi_temp(lonso4siz,pver,latso4siz), stat=istat )
      allocate( so4lon(lonso4siz), stat=istat )  
      allocate( so4lat(latso4siz), stat=istat )

      call wrap_inq_varid( ncid_sulf_ap, 'lon', lonid   )
      call wrap_inq_varid( ncid_sulf_ap, 'lat', latid   )
      call wrap_get_var_realx( ncid_sulf_ap, lonid, so4lon )
      call wrap_get_var_realx( ncid_sulf_ap, latid, so4lat )
!----------------------------------------------------------------
      call wrap_inq_dimlen( ncid_sulf_ap, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'SULFINI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!
! wzz, 2009.9.28
      call wrap_inq_varid( ncid_sulf_ap, 'lev' , sulf_id_ap )
      call wrap_get_var_realx (ncid_sulf_ap,sulf_id_ap,m_bi)
!
! the variable
!
!     call wrap_inq_varid( ncid_sulf_ap, 'sulfmmrbio' , sulf_id_ap )
      call wrap_inq_varid( ncid_sulf_ap, 'so4' , sulf_id_ap )

      call wrap_inq_vardimid (ncid_sulf_ap, sulf_id_ap, dimids)
      if (dimids(1) /= londimid .and. dimids(2) /= levdimid .and. dimids(3) /= latdimid) then
         write(6,*)'SULFINI: Data must be ordered lon, lev, lat, time'
         call endrun
      end if
!
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
!-------------------zf 2010.11.18-------------------
      cnt4(1)  = lonso4siz
      cnt4(2)  = levsiz
      cnt4(3)  = latso4siz
!---------------------------------------------------
!zf      cnt4(1)  = lonsiz
!zf      cnt4(2)  = levsiz
!zf      cnt4(3)  = latsiz
      cnt4(4)  = 1
!
! need to read in only 1 sample of rawdata
!
      strt4(4) = sp_p 
      xsulfi_ap = 0.0
!zf      call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
!zf           xsulfi_ap(:,1:levsiz,:))

!-------------zf 2010.11.17-----------------
     xsulfi_temp = 0.
     call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
           xsulfi_temp(:,1:levsiz,:))
     if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
      xsulfi_ap(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
     else

      call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                 latso4siz,xsulfi_ap,londeg,latdeg,plond,nlon,pver,plat)
     end if
!-----------------------------------------------
      do n    = 1,2
       do lat = 1,plat
        do k  = 1,pver
         do i = 1,plon
           xsulfi(i,k,lat,n)= xsulfi_ap(i,k,lat)
         enddo
        enddo
       enddo
      enddo

!---------------------------------------------------------------
! if not out of boundary
!---------------------------------------------------------------
    else   
  
      write(*,*)'wzz: locfn_am'
      call getfil(sulfdatafile_am, locfn_am)
      call wrap_open(locfn_am, 0, ncid_sulf_am)

      write(*,*)'wzz: locfn_bm'
      call getfil(sulfdatafile_bm, locfn_bm)
      call wrap_open(locfn_bm, 0, ncid_sulf_bm)

      write(*,*)'wzz: locfn_ap'
      call getfil(sulfdatafile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_sulf_ap)

      write(*,*)'wzz: locfn_bp'
      call getfil(sulfdatafile_bp, locfn_bp)
      call wrap_open(locfn_bp, 0, ncid_sulf_bp)
  
!
      call wrap_inq_dimid( ncid_sulf_am, 'lon', londimid)
      call wrap_inq_dimid( ncid_sulf_am, 'lat', latdimid)
      call wrap_inq_dimid( ncid_sulf_am, 'lev', levdimid)
      call wrap_inq_dimid( ncid_sulf_am, 'time',timeid  )

      call wrap_inq_dimlen( ncid_sulf_am, londimid, lonso4siz   )
!zf      call wrap_inq_dimlen( ncid_sulf_am, londimid, lonsiz   )
!zf      if (lonsiz /= plon) then
!zf         write(6,*)'SULFINI: lonsiz=',lonsiz,' must = ',plon
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_sulf_am, latdimid, latso4siz   )
!zf      call wrap_inq_dimlen( ncid_sulf_am, latdimid, latsiz   )
!zf      if (latsiz /= plat) then
!zf         write(6,*)'SULFINI: latsiz=',latsiz,' must = ',plat
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_sulf_am, levdimid, levsiz   )
!     if (levsiz /= pver) then
      if (levsiz >  pver) then
         write(6,*)'SULFINI: levsiz=',levsiz,' must le= ',pver
         call endrun
      end if

!-----------------zf 2010.11.17------------
      allocate( xsulfi_temp(lonso4siz,pver,latso4siz), stat=istat )
      allocate( so4lon(lonso4siz), stat=istat )  
      allocate( so4lat(latso4siz), stat=istat )

      call wrap_inq_varid( ncid_sulf_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_sulf_am, 'lat', latid   )
      call wrap_get_var_realx( ncid_sulf_am, lonid, so4lon )
      call wrap_get_var_realx( ncid_sulf_am, latid, so4lat )
!-----------------zf---------------------------------------------
      call wrap_inq_dimlen( ncid_sulf_am, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'SULFINI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!
! wzz, 2009.9.28
      call wrap_inq_varid( ncid_sulf_ap, 'lev' , sulf_id_ap )
      call wrap_get_var_realx (ncid_sulf_ap,sulf_id_ap,m_bi)
!
! Obtain sulfate mixing ratio id
!     call wrap_inq_varid( ncid_sulf_am, 'sulfmmrbio' , sulf_id_am )
!     call wrap_inq_varid( ncid_sulf_bm, 'sulfmmrbio' , sulf_id_bm )
!     call wrap_inq_varid( ncid_sulf_ap, 'sulfmmrbio' , sulf_id_ap )
!     call wrap_inq_varid( ncid_sulf_bp, 'sulfmmrbio' , sulf_id_bp )
      call wrap_inq_varid( ncid_sulf_am, 'so4' , sulf_id_am )
      call wrap_inq_varid( ncid_sulf_bm, 'so4' , sulf_id_bm )
      call wrap_inq_varid( ncid_sulf_ap, 'so4' , sulf_id_ap )
      call wrap_inq_varid( ncid_sulf_bp, 'so4' , sulf_id_bp )

      call wrap_inq_vardimid (ncid_sulf_am, sulf_id_am, dimids)
      if (dimids(1) /= londimid .and. dimids(2) /= levdimid .and. dimids(3) /= latdimid) then
         write(6,*)'SULFINI: Data must be ordered lon, lev, lat, time'
         call endrun
      end if
!
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
!-------------------zf 2010.11.18-------------------
      cnt4(1)  = lonso4siz
      cnt4(2)  = levsiz
      cnt4(3)  = latso4siz
!---------------------------------------------------
!zf      cnt4(1)  = lonsiz
!zf      cnt4(2)  = levsiz
!zf      cnt4(3)  = latsiz
      cnt4(4)  = 1
!
! read in the 4 sample of rawdata
!
      strt4(4) = sp_m 
      xsulfi_am = 0.0
!zf      call wrap_get_vara_realx (ncid_sulf_am,sulf_id_am,strt4,cnt4, &
!zf           xsulfi_am(:,1:levsiz,:))
!-------------zf 2010.11.18-----------------
     xsulfi_temp = 0.
     call wrap_get_vara_realx (ncid_sulf_am,sulf_id_am,strt4,cnt4, &
           xsulfi_temp(:,1:levsiz,:))
     if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
      xsulfi_am(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
     else

      call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                 latso4siz,xsulfi_am,londeg,latdeg,plond,nlon,pver,plat)
     end if
!-----------------------------------------------
      xsulfi_bm = 0.0
!zf      call wrap_get_vara_realx (ncid_sulf_bm,sulf_id_bm,strt4,cnt4, &
!zf           xsulfi_bm(:,1:levsiz,:))
!-------------zf 2010.11.18-----------------
      xsulfi_temp = 0.
      call wrap_get_vara_realx (ncid_sulf_bm,sulf_id_bm,strt4,cnt4, &
           xsulfi_temp(:,1:levsiz,:))
     if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
      xsulfi_bm(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
     else

      call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                 latso4siz,xsulfi_bm,londeg,latdeg,plond,nlon,pver,plat)
     end if
!-----------------------------------------------
!
      strt4(4) = sp_p 
      xsulfi_ap = 0.0
!zf      call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
!zf           xsulfi_ap(:,1:levsiz,:))
!-------------zf 2010.11.18-----------------
      xsulfi_temp = 0.
      call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
           xsulfi_temp(:,1:levsiz,:))
     if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
      xsulfi_ap(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
     else

      call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                 latso4siz,xsulfi_ap,londeg,latdeg,plond,nlon,pver,plat)
     end if
!-----------------------------------------------
      xsulfi_bp = 0.0
!zf      call wrap_get_vara_realx (ncid_sulf_bp,sulf_id_bp,strt4,cnt4, &
!zf           xsulfi_bp(:,1:levsiz,:))
!-------------zf 2010.11.18-----------------
      xsulfi_temp = 0.
      call wrap_get_vara_realx (ncid_sulf_bp,sulf_id_bp,strt4,cnt4, &
           xsulfi_temp(:,1:levsiz,:))
     if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
      xsulfi_bp(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
     else

      call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                 latso4siz,xsulfi_bp,londeg,latdeg,plond,nlon,pver,plat)
     end if
!-----------------------------------------------

 
! Normal interpolation between consecutive time slices.

      write(6,*) "yr_bm",yr_bm, "yr_am=",yr_am, "yrm=",yrm
      deltat = dble( yr_bm - yr_am )
      fact1  = dble( yr_bm - yrm  )/deltat
      fact2  = dble( yrm   - yr_am)/deltat
      write(6,*) "xinxg"
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'SULFINI--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,pver
        do i = 1,plon
           xsulfi(i,k,lat,nmsul)= fact1*xsulfi_am(i,k,lat)+fact2*xsulfi_bm(i,k,lat)
        enddo
       enddo
      enddo
!
      deltat = dble( yr_bp - yr_ap )
      fact1  = dble( yr_bp - yrp  )/deltat
      fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'SULFINI--2: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,pver
        do i = 1,plon
           xsulfi(i,k,lat,npsul)= fact1*xsulfi_ap(i,k,lat)+fact2*xsulfi_bp(i,k,lat)
        enddo
       enddo
      enddo

    endif    ! end if (.not.outofbndry)
    deallocate (xsulfi_temp)   !zf
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif
!------------------------------------------------------------------------------


   call scatter_field_to_chunk(1,pver,2,plon,xsulfi,sulfi)

   if (outofbndry) then
!zf      call scatter_field_to_chunk(1,pver,1,plon,xsulfi_ap,sulf)
      call scatter_field_to_chunk(1,pver,1,plond,xsulfi_ap(1:plon,:,:),sulf)  !zf 2010.11.18
      sulf(:,:,:) = sulf(:,:,:)*1.e-9    !  ug/m3  -> kg/m3
   endif

   return
end subroutine sulfini_IPCC

!###############################################################################
!
subroutine sulfint_IPCC
!----------------------------------------------------------------------- 
!
! Purpose: Interpolate sulfate mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
!-----------------------------------------------------------------------
   use ioFileMod,    only: getfil
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use timeinterp,   only: getfactors
!----------------------zf 2010.11.08
   use history,        only: outfld
!---------------------

   implicit none

!     integer itsst             ! Sea surf. temp. update freq. (iters)
!     integer nsrest            ! Restart flag
!     integer iradsw            ! Iteration freq. for shortwave radiation
!     integer iradlw            ! Iteration freq. for longwave radiation
!     integer iradae            ! Iteration freq. for absorptivity/emissivity
!     integer nrefrq            ! Restart write freq.

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer cnt4(4)            ! array of counts for each dimension
   integer strt4(4)           ! array of starting indices
   integer i, k, lchnk,lat    ! column, level, chunk, latitude indices
   integer ncol               ! number of columns in current chunk
   integer ntmp               ! temporary
   integer yr, mon, day       ! components of a date
   integer ncsec              ! current time of day [seconds]

   character(len=256)  locfn_bp     ! local filename

   real(r8) calday            ! current calendar day
   real(r8) caldayloc         ! calendar day (includes yr if no cycling)
   real(r8) deltat            ! time (days) between interpolating sulfate data
   real(r8) fact1, fact2      ! time interpolation factors

   real(r8) xsulfi   (plon,pver,plat)  ! input sulfate bio mixing ratios
!zf   real(r8) xsulfi_ap(plon,pver,plat)  ! input sulfate bio mixing ratios
!zf   real(r8) xsulfi_bp(plon,pver,plat)  ! input sulfate ant mixing ratios
!-----------------------zf 1010.11.18----------------------------------------
   integer  istat
   real(r8) xsulfi_ap(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xsulfi_bp(plond,pver,plat)  ! input sulfate ant mixing ratios
   real(r8), allocatable :: xsulfi_temp(:,:,:)
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)
   if ( rampyear_prescribed_sulfur .ne. bigint ) then
      yr = rampyear_prescribed_sulfur        ! wzz, 2009.10.15
   endif
   caldayloc = calday + yr*365.

   if ((caldayloc.lt.bndry_m).or.(caldayloc.ge.bndry_p)) then
        outofbndry = .true.
   else
        outofbndry = .false.
   endif
!-------------------------------------------------------------------------------
!  there is no need to do anything when current model time is
!   out of the time-boundary of input data files
!-------------------------------------------------------------------------------
 if (.not.outofbndry) then
!
! Initialize hyperslab corners
!
   if (masterproc) then

      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
      cnt4(1) = lonso4siz   !zf
      cnt4(3) = latso4siz   !zf
!zf      cnt4(1)  = plon
      cnt4(2)  = nsullev    ! wzz, 2009.9.28
!zf      cnt4(3)  = plat
      cnt4(4)  = 1

   endif
!
! If model time is past current forward sulfate timeslice, read in the next
! timeslice for time interpolation. 
!
!  write(0,*)'wzz: caldayloc=', caldayloc, cdaysulfp, sp_m, sp_p, yrp, yr_bp, indxbp, &
!     nmsul, npsul
!  if ( caldayloc .gt. cdaysulfp ) then
   if ( (caldayloc .gt. cdaysulfp .and. rampyear_prescribed_sulfur == bigint) .or. &
        (caldayloc .gt. cdaysulfp .and. .not. (sp_p == 1 .and. caldayloc > cdaysulfm))) then
      sp_m = sp_p
      sp_p = sp_p+1

      if ( sp_p.gt.nsample ) then     ! a new year
          sp_p = 1
          yrp  = yrp+1
          if ( rampyear_prescribed_sulfur .ne. bigint ) then
             yrp = rampyear_prescribed_sulfur        ! wzz, 2009.10.15
          endif

          if (masterproc) then

            if ( yrp.gt.yr_bp ) then    ! need to locate the new year
             indxbp = indxbp+1
             yr_ap = yr_bp
             yr_bp = yr_sulf(indxbp)

             ncid_sulf_ap = ncid_sulf_bp
             sulf_id_ap = sulf_id_bp

             sulfdatafile_bp = trim(sulfdata)//trim(file_sulf(indxbp))
             call getfil(sulfdatafile_bp, locfn_bp)
             call wrap_open(locfn_bp, 0, ncid_sulf_bp)
!            call wrap_inq_varid( ncid_sulf_bp, 'sulfmmrbio' , sulf_id_bp )
             call wrap_inq_varid( ncid_sulf_bp, 'so4' , sulf_id_bp )
             write(6,*)"Opening new sulfate data file:  ", sulfdatafile_bp 
            endif

          endif

      endif
      
      cdaysulfm = cdaysulfp
      cdaysulfp = yrp*365.0+sample_sulf(sp_p)

      ntmp = nmsul
      nmsul   = npsul
      npsul   = ntmp

      if (masterproc) then
        allocate( xsulfi_temp(lonso4siz,pver,latso4siz), stat=istat )   !zf2010.11.18
!       write(*,*)'wzz: new month= ', sp_p, npsul, indxbp, yr_ap,  yr_bp
        strt4(4) = sp_p 
        xsulfi_ap = 0.0
!zf        call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
!zf             xsulfi_ap(:,1:nsullev,:))
!-------------------------zf 2010.11.18---------------------------------------
        xsulfi_temp = 0.
        call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
             xsulfi_temp(:,1:nsullev,:))
        if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
          xsulfi_ap(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
        else

          call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                     latso4siz,xsulfi_ap,londeg,latdeg,plond,nlon,pver,plat)
        end if
!-----------------------------------------------

!
        xsulfi_bp = 0.0
!zf        call wrap_get_vara_realx (ncid_sulf_bp,sulf_id_bp,strt4,cnt4, &
!zf             xsulfi_bp(:,1:nsullev,:))
!-------------------------zf 2010.11.18---------------------------------------
        xsulfi_temp = 0.
        call wrap_get_vara_realx (ncid_sulf_ap,sulf_id_ap,strt4,cnt4, &
             xsulfi_temp(:,1:nsullev,:))
        if(lonso4siz .eq. plon .and. latso4siz .eq. plat) then
          xsulfi_bp(1:plon,:,:) = xsulfi_temp(1:plon,:,:)
        else

          call bilin(xsulfi_temp,so4lon,so4lat,lonso4siz,lonso4siz,pver,pver,   &
                     latso4siz,xsulfi_bp,londeg,latdeg,plond,nlon,pver,plat)
        end if
!-----------------------------------------------
!
! Normal interpolation between consecutive time slices.
!
        deltat = dble( yr_bp - yr_ap )
        fact1  = dble( yr_bp - yrp  )/deltat
        fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'SULFINT--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
        do lat = 1,plat
         do k  = 1,pver
          do i = 1,plon
             xsulfi(i,k,lat)= fact1*xsulfi_ap(i,k,lat)+fact2*xsulfi_bp(i,k,lat)
          enddo
         enddo
        enddo
!
! master does the work above
!
      deallocate (xsulfi_temp)   !zf2010.11.18
      endif  

      call scatter_field_to_chunk(1,pver,1,plon,xsulfi,sulfi(1,1,begchunk,npsul))

   end if
!
! Determine time interpolation factor.   
!
   if ( rampyear_prescribed_sulfur .ne. bigint ) then   ! wzz, 2009.10.20
!   if (masterproc) then
!    write(*,*)'wzz: calday=', sample_sulf(sp_m), sample_sulf(sp_p), calday,sp_m,sp_p
!   endif
      call getfactors (.true., sp_p, sample_sulf(sp_m), sample_sulf(sp_p), calday, &
                       fact1, fact2, 'sulfint_IPCC:')
   else
!   if (masterproc) then
!    write(*,*)'wzz: calday=', cdaysulfm, cdaysulfp, caldayloc,sp_m,sp_p
!   endif
      call getfactors (.false., sp_p, cdaysulfm, cdaysulfp, caldayloc, &
                       fact1, fact2, 'sulfint_IPCC:')
   endif
!
! Time interpolation.
!
!   if (masterproc) then
!    write(*,*)'wzz: fact1, fact2= ', fact1, fact2, nmsul, npsul
!   endif
   do lchnk=begchunk,endchunk
      ncol = get_ncols_p(lchnk)
      do k=1,pver
         do i=1,ncol
            sulf(i,k,lchnk) = sulfi(i,k,lchnk,nmsul)*fact1 +  &
                              sulfi(i,k,lchnk,npsul)*fact2
         end do
      end do
   end do
!
   sulf(:,:,:) = sulf(:,:,:)*1.e-9    ! ug/m3  -> kg/m3

!----------------------------------------------------------------------------
 endif
!----------------------------------------------------------------------------

   return
end subroutine sulfint_IPCC

subroutine aeroini_IPCC(levsiz)
!----------------------------------------------------------------------- 
! 
! Purpose: Do initial read of time-variant aerosol dataset, containing
!          aerosol mixing ratios as a function of time.  It is currently
!          required that the aerosol dataset have the *SAME* horizontal
!          and vertical resolution as the model. Therefore, ONLY a time
!          interpolation of the dataset is currently performed.
! 
!-----------------------------------------------------------------------
   use ioFileMod,      only: getfil
   use error_messages, only: alloc_err, handle_ncerr
   use time_manager,   only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use dycore,         only: get_resolution
   implicit none

   integer,  intent(out) :: levsiz     ! size of level dimension on sulfate dataset
!  real(r8), intent(out) :: m_bi(paerlev+1)       ! vertical level values of sulfate dataset
!
   integer :: ncid_aero_am=-1, ncid_aero_bm  ! sulfate dataset id
   integer :: aero_id_ap,   aero_id_bp
   integer :: aero_id_am,   aero_id_bm 
 
   integer :: sul_id_am,    sul_id_bm    ! netcdf id for sulfate mmr bio variable,idxSUL   =  1
   integer :: sslt_id_am,   sslt_id_bm  ! idxSSLT  =  2
   integer :: dst1_id_am,   dst1_id_bm  ! idxDUSTfirst    = 3, numDUST         = 4
   integer :: dst2_id_am,   dst2_id_bm
   integer :: dst3_id_am,   dst3_id_bm
   integer :: dst4_id_am,   dst4_id_bm
   integer :: ocpho_id_am,  ocpho_id_bm  ! idxCARBONfirst = 7
   integer :: bcpho_id_am,  bcpho_id_bm
   integer :: ocphi_id_am,  ocphi_id_bm
   integer :: bcphi_id_am,  bcphi_id_bm

   integer :: lonsiz     ! size of longitude dimension on sulfate dataset
   integer :: latsiz     ! size of latitude dimension on sulfate dataset
   integer :: timsiz     ! size of time dimension on sulfate dataset

   integer :: idxAERO

   character*16  ::  IPCC_scenario   ! name of the IPCC scenario
   character*256 ::  aerodatafile_am ! full pathname for sulfate dataset
   character*256 ::  aerodatafile_bm ! full pathname for sulfate dataset
   character*256 ::  aerodatafile_ap ! full pathname for sulfate dataset

   character*80, parameter ::  p1t42   ='cmip5_aero_T42_L26_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t42L40='cmip5_aero_T42_L40exp_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t42L46='cmip5_aero_T42_L46exp_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t42L68='cmip5_aero_T42_L68_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t42L56='cmip5_aero_T42_L56_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t42L70='cmip5_aero_T42_L70_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t42L71='cmip5_aero_T42_L71_'   ! part of the name of sulfate data file
   character*80, parameter ::  p1t106  ='cmip5_aero_T106_L26_'   ! part of the name of sulfate data file
   character*80, parameter ::  p2='.nc'   ! part of the name of sulfate data file
   character*80, parameter ::  p22='_0.1mb.nc'   ! part of the name of sulfate data file

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!
! Local workspace
!
   character(len=256) :: locfn_am, locfn_bm, locfn_ap, locfn_bp     ! local filenames

   integer londimid              ! netcdf id for longitude dimension
   integer latdimid              ! netcdf id for latitude dimension
   integer levdimid              ! netcdf id for level dimension
   integer lonid                 ! netcdf id for longitude variable
   integer latid                 ! netcdf id for latitude variable
   integer levid                 ! netcdf id for level variable
   integer timeid                ! netcdf id for time variable
   integer cnt4(4)               ! array of counts for each dimension
   integer strt4(4)              ! array of starting indices
   integer i, k, lat, n          ! longitude, level, latitude, time indices
   integer istat                 ! error return
   integer dimids(nf_max_var_dims) ! netcdf variable shape

   integer yr, mon, day, ncsec ! components of a date
   real(r8) calday                   ! current calendar day
   real(r8) caldayloc                ! calendar day (includes yr if no cycling)
   real(r8) fact1, fact2, deltat

   character*4 yrtmp

!zf   real(r8) xaeroi_am(plon,plat,pver)  ! input sulfate bio mixing ratios
!zf   real(r8) xaeroi_bm(plon,plat,pver)  ! input sulfate ant mixing ratios
!zf   real(r8) xaeroi_ap(plon,plat,pver)  ! input sulfate bio mixing ratios
!zf   real(r8) xaeroi_bp(plon,plat,pver)  ! input sulfate ant mixing ratios

!---------------zf 2010.11.11------------------------
   real(r8) xaeroi_am(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xaeroi_bm(plond,pver,plat)  ! input sulfate ant mixing ratios
   real(r8) xaeroi_ap(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xaeroi_bp(plond,pver,plat)  ! input sulfate ant mixing ratios
!-----------------------------------------------------

   real(r8) xaeroi(plon,pver,plat,2)   ! input aerosol ant mixing ratios

   real(r8), allocatable :: xaeroi_tempa(:,:,:) 
   real(r8), allocatable :: xaeroi_tempb(:,:,:)
!
!-----------------------------------------------------------------------
!
! Allocate space for data.
!
  allocate( aeroi(pcols,pver,begchunk:endchunk,2,naer), stat=istat )
  call alloc_err( istat, 'aeroini', 'aeroi', pcols*pver*(endchunk-begchunk+1)*2*naer )
  allocate( aero(pcols,pver,begchunk:endchunk,naer), stat=istat )
  call alloc_err( istat, 'aeroini', 'aero',  pcols*pver*(endchunk-begchunk+1)*naer )
!
! unit shift from second to day
!
   do n=1,nsample
      sample_aero(n) = sample_aero(n)/86400.0+1.0
   enddo
!
! the names of rawdata files
!
   yrtmp = '1950'
   do n=1,nfile
      write(yrtmp,'(i4.4)') yr_aero(n)
      if( get_resolution() == 'T106' .or. get_resolution() == 'T266' .or. get_resolution() == 'T255') then
         if( pver == 68 ) then
             file_aero(n) = trim(p1t42L68)//yrtmp//trim(p2)
         elseif( pver == 46 ) then
             file_aero(n) = trim(p1t42L46)//yrtmp//trim(p2)
         elseif( pver == 56 ) then
             file_aero(n) = trim(p1t42L56)//yrtmp//trim(p2)
         elseif( pver == 70 ) then
    !        file_aero(n) = trim(p1t42L70)//yrtmp//trim(p2)
             file_aero(n) = trim(p1t42L70)//yrtmp//trim(p22)
         elseif( pver == 71 ) then
             file_aero(n) = trim(p1t42L71)//yrtmp//trim(p2)
         else ! pver == 26
             file_aero(n) = trim(p1t106)//yrtmp//trim(p2)
         endif
      elseif(  get_resolution() == 'T42' ) then
         if ( pver == 68 ) then
             file_aero(n) = trim(p1t42L68)//yrtmp//trim(p2)
         elseif( pver == 46 ) then
             file_aero(n) = trim(p1t42L46)//yrtmp//trim(p2)
         else   ! pver == 26
             file_aero(n) = trim(p1t42)//yrtmp//trim(p2)
         end if
      endif
   enddo

! set the time_boundaries of input dataset
!
   bndry_m = yr_aero(1    )*365.0 + sample_aero(1) 
   bndry_p = yr_aero(nfile)*365.0 + sample_aero(nsample) 
  
!!   if (masterproc) then
!!       write(*,*) 'bndry_m=',bndry_m
!!       write(*,*) 'bndry_p=',bndry_p
!!   endif
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
!  if (masterproc) then

     calday = get_curr_calday()
     call get_curr_date(yr, mon, day, ncsec)

#ifdef MAC_SP

     yr = 1850    ! year 1850 is fixed, only considered to set the background aerosols
     write(*,*) "SXJ rampyear_prescribed_aeros",rampyear_prescribed_aeros 
     caldayloc = calday + yr*365.

#else
     if ( rampyear_prescribed_aeros .ne. bigint ) then
        yr = rampyear_prescribed_aeros        ! wzz, 2009.10.15
        !++sxj
        write(*,*) "SXJ rampyear_prescribed_aeros",rampyear_prescribed_aeros
        !--sxj
     endif
     caldayloc = calday + yr*365.

#endif

!----------------------------------------------------------------
! locate the model time among the samples of sulfate data file      
!---------------------------------------------------------------
    if ( caldayloc.lt.bndry_m ) then

        outofbndry = .true.
        cdayaerop  = bndry_m
        cdayp      = sample_aero(1)    !???
        sp_p   = 1
        mo_nxt = sp_p
        yrp    = yr_aero(1)
        yr_ap  = yr_aero(1)
        yr_bp  = yr_aero(2)
        indxbp = 2
        aerodatafile_ap = trim(aerodata)//file_aero(1)
        aerodatafile_bp = trim(aerodata)//file_aero(2)

    elseif ( caldayloc.ge.bndry_p ) then

        outofbndry = .true.
        sp_p   = nsample 
        yrp    = yr_aero(nfile)
        yr_ap  = yr_aero(nfile)
        yr_bp  = yr_aero(nfile)
        aerodatafile_ap = trim(aerodata)//file_aero(nfile)

    else
        outofbndry = .false.
!
! month
!
       if ( calday.lt.sample_aero(1) ) then
          yrm  = yr-1
          yrp  = yr
          sp_m = nsample 
          sp_p = 1
          goto 100
       elseif (calday.ge.sample_aero(nsample) ) then
          yrm = yr
          yrp = yr+1
          sp_m = nsample
          sp_p = 1 
          goto 100
       else 
         do n=1,nsample-1
            if ( calday.ge.sample_aero(n) .and. calday.lt.sample_aero(n+1) ) then
               yrm  = yr
               yrp  = yr
               sp_m = n
               sp_p = n+1
               goto 100
            endif
         enddo
       endif
       write(*,*) 'AEROINI: Failed to find seconds bracketing model_time =',calday
       call endrun
100    continue

#ifdef MAC_SP
       yrm = 1850   ! 1850 is only the background year
       yrp = 1850     
#else
       if ( rampyear_prescribed_aeros .ne. bigint ) then
          yrm = rampyear_prescribed_aeros        ! wzz, 2009.10.15
          yrp = rampyear_prescribed_aeros        ! wzz, 2009.10.15
       endif
#endif
       cdayaerom = yrm*365.0+sample_aero(sp_m)
       cdayaerop = yrp*365.0+sample_aero(sp_p)
       cdaym = sample_aero(sp_m)
       cdayp = sample_aero(sp_p)
       mo_nxt = sp_p
!
! year ( both yrm & yrp )
!
       do n=1,nfile-1
          yr_am = yr_aero(n)
          yr_bm = yr_aero(n+1)
          if ( yrm.ge.yr_am .and. yrm.lt.yr_bm ) goto 200
       enddo
       write(*,*) 'AEROINI: Failed to find years bracketing year  yrm=',yrm
       call endrun
200    continue
       aerodatafile_am = trim(aerodata)//trim(file_aero(n))
       aerodatafile_bm = trim(aerodata)//trim(file_aero(n+1))
 
       do n=1,nfile-1
          yr_ap = yr_aero(n)
          yr_bp = yr_aero(n+1)
          if ( yrp.ge.yr_ap .and. yrp.lt.yr_bp ) goto 300
       enddo
       write(*,*) 'AEROINI: Failed to find years bracketing year  yrp=',yrp
       call endrun
300    indxbp = n+1 
       aerodatafile_ap = trim(aerodata)//trim(file_aero(n))
       aerodatafile_bp = trim(aerodata)//trim(file_aero(n+1))

    endif

   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    if ( outofbndry ) then

      call getfil(aerodatafile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_aero_ap)

      write(6,*) 'Now open aero file (locfn_ap):', locfn_ap
!
! Obtain dimension id's
!
      call wrap_inq_dimid( ncid_aero_ap, 'lon', londimid)
      call wrap_inq_dimid( ncid_aero_ap, 'lat', latdimid)
      call wrap_inq_dimid( ncid_aero_ap, 'lev', levdimid)
      call wrap_inq_dimid( ncid_aero_ap, 'time',timeid  )
!
! Obtain size of dimensions.
! Check that horizontal and vertical dimensions are same as model's
! (check one file only and assume that all the other files are of the same setting)
!
       call wrap_inq_dimlen( ncid_aero_ap, londimid, lonaerosiz   ) !zf 2010.11.10
!zf      call wrap_inq_dimlen( ncid_aero_ap, londimid, lonsiz   )
!zf      if (lonsiz /= plon) then
!zf         write(6,*)'AEROINI: lonsiz=',lonsiz,' must = ',plon
!zf         call endrun
!zf      end if

       call wrap_inq_dimlen( ncid_aero_ap, latdimid, lataerosiz   )  !zf 2010.11.10
!zf      call wrap_inq_dimlen( ncid_aero_ap, latdimid, latsiz   )
!zf      if (latsiz /= plat) then
!zf         write(6,*)'AEROINI: latsiz=',latsiz,' must = ',plat
!zf         call endrun
!zf      end if

!-----------------zf 2010.11.10------------
      allocate( aerolon(lonaerosiz), stat=istat )  
      allocate( aerolat(lataerosiz), stat=istat )  

      call wrap_inq_varid( ncid_aero_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_aero_am, 'lat', latid   )
      call wrap_get_var_realx( ncid_aero_am, lonid, aerolon )
      call wrap_get_var_realx( ncid_aero_am, latid, aerolat )
!---------------------------------------------------------------
      call wrap_inq_dimlen( ncid_aero_ap, levdimid, levsiz   )
      if (levsiz > pver) then
         write(6,*)'AEROINI: levsiz=',levsiz,' must le ',pver
         call endrun
      end if
      levaerosiz=levsiz   !zf

      call wrap_inq_dimlen( ncid_aero_ap, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'AEROINI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!
! wzz, 2009.9.28
!     call wrap_inq_varid( ncid_aero_ap, 'lev' , aero_id_ap )
!     call wrap_get_var_realx (ncid_aero_ap,aero_id_ap,m_bi)
!
! the variable
!

      call wrap_inq_varid( ncid_aero_ap, 'SO4' , aero_id_ap )

      call wrap_inq_vardimid (ncid_aero_ap, aero_id_ap, dimids)
      if (dimids(1) /= londimid .and. dimids(3) /= levdimid .and. dimids(2) /= latdimid) then
         write(6,*)'AEROINI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
!
!---------------------------------------------------------------
! if not out of boundary
!---------------------------------------------------------------
    else   
  
      call getfil(aerodatafile_am, locfn_am)
      call wrap_open(locfn_am, 0, ncid_aero_am)
      write(6,*) 'Now open aero file (locfn_am):', locfn_am

      call getfil(aerodatafile_bm, locfn_bm)
      call wrap_open(locfn_bm, 0, ncid_aero_bm)
      write(6,*) 'Now open aero file (locfn_bm):', locfn_bm

      write(*,*)'wzz: locfn_ap'
      call getfil(aerodatafile_ap, locfn_ap)
      call wrap_open(locfn_ap, 0, ncid_aero_ap)
      write(6,*) 'Now open aero file (locfn_ap):', locfn_ap

      write(*,*)'wzz: locfn_bp'
      call getfil(aerodatafile_bp, locfn_bp)
      call wrap_open(locfn_bp, 0, ncid_aero_bp)
      write(6,*) 'Now open aero file (locfn_bp):', locfn_bp
!
      call wrap_inq_dimid( ncid_aero_am, 'lon', londimid)
      call wrap_inq_dimid( ncid_aero_am, 'lat', latdimid)
      call wrap_inq_dimid( ncid_aero_am, 'lev', levdimid)
      call wrap_inq_dimid( ncid_aero_am, 'time',timeid  )

      call wrap_inq_dimlen( ncid_aero_am, londimid, lonaerosiz   )
!zf      call wrap_inq_dimlen( ncid_aero_am, londimid, lonsiz   )
!zf      if (lonsiz /= plon) then
!zf         write(6,*)'AEROINI: lonsiz=',lonsiz,' must = ',plon
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_aero_am, latdimid, lataerosiz   )
!zf      call wrap_inq_dimlen( ncid_aero_am, latdimid, latsiz   )
!zf      if (latsiz /= plat) then
!zf         write(6,*)'AEROINI: latsiz=',latsiz,' must = ',plat
!zf         call endrun
!zf      end if

      call wrap_inq_dimlen( ncid_aero_am, levdimid, levsiz   )
      if (levsiz >  pver) then
         write(6,*)'AEROINI: levsiz=',levsiz,' must le ',pver
         call endrun
      end if

      call wrap_inq_dimlen( ncid_aero_am, timeid, timsiz   )
      if (timsiz /= nsample) then
         write(6,*)'AEROINI: timsiz=',timsiz,' must = ', nsample 
         call endrun
      end if
!------------------zf 2010.11.08-----------------------------------
      levaerosiz=levsiz  
      allocate( aerolon(lonaerosiz), stat=istat )   
      allocate( aerolat(lataerosiz), stat=istat )

      call wrap_inq_varid( ncid_aero_am, 'lon', lonid   )
      call wrap_inq_varid( ncid_aero_am, 'lat', latid   )
      call wrap_get_var_realx( ncid_aero_am, lonid, aerolon )
      call wrap_get_var_realx( ncid_aero_am, latid, aerolat )
!--------------------------------------------------------------------
!
! wzz, 2009.9.28
!     call wrap_inq_varid( ncid_aero_ap, 'lev' , aero_id_ap )
!     call wrap_get_var_realx (ncid_aero_ap,aero_id_ap,m_bi)
!
! Obtain sulfate mixing ratio id

      call wrap_inq_varid( ncid_aero_am, 'SO4' , aero_id_am )

      call wrap_inq_vardimid (ncid_aero_am, aero_id_am, dimids)
      if (dimids(1) /= londimid .and. dimids(3) /= levdimid .and. dimids(2) /= latdimid) then
         write(6,*)'AEROINI: Data must be ordered lon, lat, lev, time'
         call endrun
      end if
    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif
!
   allocate( xaeroi_tempa(lonaerosiz,lataerosiz,levsiz), stat=istat )   !zf2010.11.08
   allocate( xaeroi_tempb(lonaerosiz,levsiz,lataerosiz), stat=istat )   !zf2010.11.08

   do idxAERO=1,naer

   if (masterproc) then
!--------------------------------------------------------------------------
! open the right data files and read in raw data
!--------------------------------------------------------------------------
    
    if ( outofbndry ) then
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
!zf      cnt4(1)  = lonsiz
!zf      cnt4(2)  = latsiz
      cnt4(1)  = lonaerosiz  !zf
      cnt4(2)  = lataerosiz  !zf
      cnt4(3)  = levsiz
      cnt4(4)  = 1
!
! need to read in only 1 sample of rawdata
!
      strt4(4) = sp_p 
      xaeroi_ap = 0.0
      xaeroi_tempa = 0.0   !zf
      xaeroi_tempb = 0.0   !zf

      call wrap_inq_varid( ncid_aero_ap, trim(varname(idxAERO)) , aero_id_ap )
      call wrap_get_vara_realx (ncid_aero_ap,aero_id_ap,strt4,cnt4, &
           xaeroi_tempa(:,:,1:levsiz))  !zf
!zf           xaeroi_ap(:,:,1:levsiz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do n = 1, lataerosiz
      do k = 1, levsiz
        xaeroi_tempb(i, k, n) = xaeroi_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_ap = xaeroi_tempa
     else
      
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levsiz,levsiz,   &
                 lataerosiz,xaeroi_ap,londeg,latdeg,plond,nlon,levsiz,plat)
     end if
!-----------------------------------------------

      do n    = 1,2
       do lat = 1,plat
        do k  = 1,pver
         do i = 1,plon
!zf           xaeroi(i,k,lat,n)= xaeroi_ap(i,lat,k)
           xaeroi(i,k,lat,n)= xaeroi_ap(i,k,lat)
         enddo
        enddo
       enddo
      enddo

     else
! Set up hyperslab corners
!
      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
!zf      cnt4(1)  = lonsiz
!zf      cnt4(2)  = latsiz
      cnt4(1)  = lonaerosiz  !zf
      cnt4(2)  = lataerosiz   !zf
      cnt4(3)  = levsiz
      cnt4(4)  = 1
!
! read in the 4 sample of rawdata
!
      strt4(4) = sp_m 
      xaeroi_am = 0.0

      call wrap_inq_varid( ncid_aero_am, trim(varname(idxAERO)) , aero_id_am )
      call wrap_inq_varid( ncid_aero_bm, trim(varname(idxAERO)) , aero_id_bm )
      call wrap_inq_varid( ncid_aero_ap, trim(varname(idxAERO)) , aero_id_ap )
      call wrap_inq_varid( ncid_aero_bp, trim(varname(idxAERO)) , aero_id_bp )

      xaeroi_tempa = 0.0   !zf
      xaeroi_tempb = 0.0   !zf

      call wrap_get_vara_realx (ncid_aero_am,aero_id_am,strt4,cnt4, &
           xaeroi_tempa(:,:,1:levsiz))  !zf
!zf           xaeroi_am(:,:,1:levsiz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do n = 1, lataerosiz
      do k = 1, levsiz
        xaeroi_tempb(i, k, n) = xaeroi_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_am = xaeroi_tempb
     else
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levsiz,levsiz,   &
                 lataerosiz,xaeroi_am,londeg,latdeg,plond,nlon,levsiz,plat)
     endif
!-----------------------------------------------

      xaeroi_bm = 0.0
      xaeroi_tempa = 0.0   !zf
      xaeroi_tempb = 0.0   !zf
      call wrap_get_vara_realx (ncid_aero_bm,aero_id_bm,strt4,cnt4, &
           xaeroi_tempa(:,:,1:levsiz))  !zf
!zf           xaeroi_bm(:,:,1:levsiz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do n = 1, lataerosiz
      do k = 1, levsiz
        xaeroi_tempb(i, k, n) = xaeroi_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_bm = xaeroi_tempb
     else
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levsiz,levsiz,   &
                 lataerosiz,xaeroi_bm,londeg,latdeg,plond,nlon,levsiz,plat)
     endif
!-----------------------------------------------

!
      strt4(4) = sp_p 
      xaeroi_ap = 0.0
      xaeroi_tempa = 0.0   !zf
      xaeroi_tempb = 0.0   !zf
      call wrap_get_vara_realx (ncid_aero_ap,aero_id_ap,strt4,cnt4, &
           xaeroi_tempa(:,:,1:levsiz))  !zf
!zf           xaeroi_ap(:,:,1:levsiz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do n = 1, lataerosiz
      do k = 1, levsiz
        xaeroi_tempb(i, k, n) = xaeroi_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_ap = xaeroi_tempb
     else
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levsiz,levsiz,   &
                 lataerosiz,xaeroi_ap,londeg,latdeg,plond,nlon,levsiz,plat)
     endif
!-----------------------------------------------

      xaeroi_bp = 0.0
      xaeroi_tempa = 0.0   !zf
      xaeroi_tempb = 0.0   !zf
      call wrap_get_vara_realx (ncid_aero_bp,aero_id_bp,strt4,cnt4, &
           xaeroi_tempa(:,:,1:levsiz))  !zf
!zf           xaeroi_bp(:,:,1:levsiz))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do n = 1, lataerosiz
      do k = 1, levsiz
        xaeroi_tempb(i, k, n) = xaeroi_tempa(i, n, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_bp = xaeroi_tempb
     else
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levsiz,levsiz,   &
                 lataerosiz,xaeroi_bp,londeg,latdeg,plond,nlon,levsiz,plat)
     endif
!-----------------------------------------------


 
! Normal interpolation between consecutive time slices.
!
      write(6,*) "yr_bm=",yr_bm,"yr_am=",yr_am, "yrm=",yrm
      deltat = dble( yr_bm - yr_am )
      fact1  = dble( yr_bm - yrm  )/deltat
      fact2  = dble( yrm   - yr_am)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'AEROINI--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,pver
        do i = 1,plon
           xaeroi(i,k,lat,nmsul)= fact1*xaeroi_am(i,k,lat)+fact2*xaeroi_bm(i,k,lat)
        enddo
       enddo
      enddo
!
      deltat = dble( yr_bp - yr_ap )
      fact1  = dble( yr_bp - yrp  )/deltat
      fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'AEROINI--2: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
      do lat = 1,plat
       do k  = 1,pver
        do i = 1,plon
           xaeroi(i,k,lat,npsul)= fact1*xaeroi_ap(i,k,lat)+fact2*xaeroi_bp(i,k,lat)
        enddo
       enddo
      enddo

    endif    ! end if (.not.outofbndry)
!------------------------------------------------------------------------------
! master does the work above
!------------------------------------------------------------------------------
   endif
!------------------------------------------------------------------------------


   call scatter_field_to_chunk(1,pver,2,plon,xaeroi,aeroi(:,:,:,:,idxAERO))

   if (outofbndry) then
      call scatter_field_to_chunk(1,pver,1,plon,xaeroi_ap,aero(:,:,:,idxAERO))  !??
!     sulf(:,:,:) = sulf(:,:,:)*1.e-9    !  ug/m3  -> kg/m3
   endif

   enddo

   deallocate (xaeroi_tempa)   !zf2010.11.08
   deallocate (xaeroi_tempb)   !zf2010.11.08

   return
end subroutine aeroini_IPCC

!###############################################################################
!
subroutine aeroint_IPCC
!----------------------------------------------------------------------- 
!
! Purpose: Interpolate sulfate mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
!-----------------------------------------------------------------------
   use ioFileMod,    only: getfil
   use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
   use timeinterp,   only: getfactors

   implicit none

!-----------------------------------------------------------------------
   include 'netcdf.inc'
!-----------------------------------------------------------------------
!
! Local workspace
!
   integer cnt4(4)            ! array of counts for each dimension
   integer strt4(4)           ! array of starting indices
   integer i, k, lchnk,lat    ! column, level, chunk, latitude indices
   integer ncol               ! number of columns in current chunk
   integer ntmp               ! temporary
   integer yr, mon, day       ! components of a date
   integer ncsec              ! current time of day [seconds]
   integer idxAERO

   character(len=256)  locfn_bp     ! local filename

   real(r8) calday            ! current calendar day
   real(r8) caldayloc         ! calendar day (includes yr if no cycling)
   real(r8) deltat            ! time (days) between interpolating sulfate data
   real(r8) fact1, fact2      ! time interpolation factors

   real(r8) xaeroi   (plon,pver,plat)  ! input sulfate bio mixing ratios
!zf   real(r8) xaeroi_ap(plon,plat,pver)  ! input sulfate bio mixing ratios
!zf   real(r8) xaeroi_bp(plon,plat,pver)  ! input sulfate ant mixing ratios
!--------------------------------------zf---------------------------------
   real(r8) xaeroi_ap(plond,pver,plat)  ! input sulfate bio mixing ratios
   real(r8) xaeroi_bp(plond,pver,plat)  ! input sulfate ant mixing ratios
   real(r8), allocatable :: xaeroi_tempa(:,:,:)   
   real(r8), allocatable :: xaeroi_tempb(:,:,:) 
   integer istat                        !  error return
!--------------------------------------------------------------------------

   integer :: sul_id_ap, sul_id_bp
!-----------------------------------------------------------------------
!
! Use year information only if a multiyear dataset
!
   calday = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)

#ifdef MAC_SP
      yr = 1850   ! 1850 is only the background year
#else
   if ( rampyear_prescribed_aeros .ne. bigint ) then
      yr = rampyear_prescribed_aeros        ! wzz, 2009.10.15
   endif
#endif

   caldayloc = calday + yr*365.

   if ((caldayloc.lt.bndry_m).or.(caldayloc.ge.bndry_p)) then
        outofbndry = .true.
   else
        outofbndry = .false.
   endif
   allocate( xaeroi_tempa(lonaerosiz,lataerosiz,pver), stat=istat )   !zf2010.11.08
   allocate( xaeroi_tempb(lonaerosiz,pver,lataerosiz), stat=istat )   !zf2010.11.08
!-------------------------------------------------------------------------------
!  there is no need to do anything when current model time is
!   out of the time-boundary of input data files
!-------------------------------------------------------------------------------
 if (.not.outofbndry) then
!
! Initialize hyperslab corners
!
   if (masterproc) then

      strt4(1) = 1
      strt4(2) = 1
      strt4(3) = 1
!zf      cnt4(1)  = plon
!zf      cnt4(2)  = plat
      cnt4(1)  = lonaerosiz  !zf
      cnt4(2)  = lataerosiz  !zf
      cnt4(3)  = pver       ! wzz, 2010.4.26
!     cnt4(3)  = nsullev    ! wzz, 2009.9.28
      cnt4(4)  = 1

   endif
!
! If model time is past current forward sulfate timeslice, read in the next
! timeslice for time interpolation. 
!
!  write(0,*)'wzz: caldayloc=', caldayloc, cdaysulfp, sp_m, sp_p, yrp, yr_bp, indxbp, &
!     nmsul, npsul
!-------------------------------
#ifdef MAC_SP

   if ( caldayloc .gt. cdayaerop .and. .not. (sp_p == 1 .and. caldayloc > cdayaerom) ) then
      sp_m = sp_p
      sp_p = sp_p+1

      if ( sp_p.gt.nsample ) then     ! a new year
          sp_p = 1
          yrp = 1850  ! 1850 is only the background year

          if (masterproc) then

            if ( yrp.gt.yr_bp ) then    ! need to locate the new year
             indxbp = indxbp+1
             yr_ap = yr_bp
             yr_bp = yr_aero(indxbp)

             ncid_aero_ap = ncid_aero_bp
             sul_id_ap = sul_id_bp

             aerodatafile_bp = trim(aerodata)//trim(file_aero(indxbp))
             call getfil(aerodatafile_bp, locfn_bp)
             call wrap_open(locfn_bp, 0, ncid_aero_bp)
             write(6,*) 'Now open aero file (locfn_bp):', locfn_bp

             call wrap_inq_varid( ncid_aero_bp, 'SO4' , sul_id_bp )
            endif

          endif

      endif

!------------------------------
#else

   if ( (caldayloc .gt. cdayaerop .and. rampyear_prescribed_aeros == bigint) .or. &
        (caldayloc .gt. cdayaerop .and. .not. (sp_p == 1 .and. caldayloc > cdayaerom))) then

      sp_m = sp_p
      sp_p = sp_p+1

      if ( sp_p.gt.nsample ) then     ! a new year
          sp_p = 1
          yrp  = yrp+1

          if ( rampyear_prescribed_aeros .ne. bigint ) then
             yrp = rampyear_prescribed_aeros        ! wzz, 2009.10.15
          endif
          if (masterproc) then

            if ( yrp.gt.yr_bp ) then    ! need to locate the new year
             indxbp = indxbp+1
             yr_ap = yr_bp
             yr_bp = yr_aero(indxbp)

             ncid_aero_ap = ncid_aero_bp
             sul_id_ap = sul_id_bp

             aerodatafile_bp = trim(aerodata)//trim(file_aero(indxbp))
             call getfil(aerodatafile_bp, locfn_bp)
             call wrap_open(locfn_bp, 0, ncid_aero_bp)
             write(6,*) 'Now open aero file (locfn_bp):', locfn_bp 

             call wrap_inq_varid( ncid_aero_bp, 'SO4' , sul_id_bp )
            endif

          endif

      endif
#endif
!-------------------------------------------------------------------
      
      cdayaerom = cdayaerop
      cdayaerop = yrp*365.0+sample_aero(sp_p)

      cdaym = cdayp
      cdayp = sample_aero(sp_p)
      mo_nxt = sp_p

      ntmp = nmsul
      nmsul   = npsul
      npsul   = ntmp

    do idxAERO = 1, naer
      if (masterproc) then
!       write(*,*)'wzz: new month= ', sp_p, npsul, indxbp, yr_ap,  yr_bp
        strt4(4) = sp_p 
        xaeroi_ap = 0.0
        xaeroi_tempa = 0.0  !zf
        xaeroi_tempb = 0.0  !zf
        call wrap_inq_varid( ncid_aero_ap, trim(varname(idxAERO)) , sul_id_ap )
        call wrap_get_vara_realx (ncid_aero_ap,sul_id_ap,strt4,cnt4, &
             xaeroi_tempa(:,:,1:pver))  !zf
!zf             xaeroi_ap(:,:,1:pver))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do lat = 1, lataerosiz
      do k = 1, levaerosiz
        xaeroi_tempb(i, k, lat) = xaeroi_tempa(i, lat, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_ap = xaeroi_tempb
     else
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levaerosiz,levaerosiz,  &
                 lataerosiz,xaeroi_ap,londeg,latdeg,plond,nlon,levaerosiz,plat)
     endif
!-----------------------------------------------

!
        xaeroi_bp = 0.0
        xaeroi_tempa = 0.0  !zf
        xaeroi_tempb = 0.0  !zf
        call wrap_inq_varid( ncid_aero_bp, trim(varname(idxAERO)) , sul_id_bp )
        call wrap_get_vara_realx (ncid_aero_bp,sul_id_bp,strt4,cnt4, &
             xaeroi_tempa(:,:,1:pver))  !zf
!zf             xaeroi_bp(:,:,1:pver))

!-------------zf 2010.11.10-----------------
      do i = 1, lonaerosiz
      do lat = 1, lataerosiz
      do k = 1, levaerosiz
        xaeroi_tempb(i, k, lat) = xaeroi_tempa(i, lat, k)
      enddo
      enddo
      enddo
       
     if(lonaerosiz .eq. plon .and. lataerosiz .eq. plat) then
      xaeroi_bp = xaeroi_tempb
     else
      call bilin(xaeroi_tempb,aerolon,aerolat,lonaerosiz,lonaerosiz,levaerosiz,levaerosiz,  &
                 lataerosiz,xaeroi_bp,londeg,latdeg,plond,nlon,levaerosiz,plat)
     endif
!-----------------------------------------------

!
! Normal interpolation between consecutive time slices.
!
        write(6,*) "yr_bp,yr_ap,yrp" , yr_bp,yr_ap,yrp
        deltat = dble( yr_bp - yr_ap )
        fact1  = dble( yr_bp - yrp  )/deltat
        fact2  = dble( yrp   - yr_ap)/deltat
!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
        if (abs(fact1+fact2-1.).gt.1.e-6 .or.  &
            fact1.gt.1.000001 .or. fact1.lt.-1.e-6 .or. &
            fact2.gt.1.000001 .or. fact2.lt.-1.e-6) then
            write(6,*)'AEROINT--1: Bad fact1 and/or fact2=',fact1,fact2
            call endrun
        end if
!
! interpolation
!      
        do lat = 1,plat
         do k  = 1,pver
          do i = 1,plon
             xaeroi(i,k,lat)= fact1*xaeroi_ap(i,k,lat)+fact2*xaeroi_bp(i,k,lat)
          enddo
         enddo
        enddo
!
! master does the work above
!
      endif  

      call scatter_field_to_chunk(1,pver,1,plon,xaeroi,aeroi(1,1,begchunk,npsul,idxAERO))

    enddo

   end if
!
! Determine time interpolation factor.   
!

#ifdef MAC_SP
      call getfactors (.true., sp_p, sample_aero(sp_m), sample_aero(sp_p), calday, &
                       fact1, fact2, 'aeroint_IPCC:')
#else
   if ( rampyear_prescribed_aeros .ne. bigint ) then   ! wzz, 2009.10.20
      call getfactors (.true., sp_p, sample_aero(sp_m), sample_aero(sp_p), calday, &
                       fact1, fact2, 'aeroint_IPCC:')
   else
      call getfactors (.false., sp_p, cdayaerom, cdayaerop, caldayloc, &
                       fact1, fact2, 'aeroint_IPCC:')
   endif

#endif

!
! Time interpolation.
!
!   if (masterproc) then
!    write(*,*)'wzz: fact1, fact2= ', fact1, fact2, nmsul, npsul
!   endif
   do lchnk=begchunk,endchunk
      ncol = get_ncols_p(lchnk)
      do k=1,pver
         do i=1,ncol
            aero(i,k,lchnk,:) = aeroi(i,k,lchnk,nmsul,:)*fact1 +  &
                              aeroi(i,k,lchnk,npsul,:)*fact2
         end do
      end do
   end do
!

!----------------------------------------------------------------------------
 endif
!----------------------------------------------------------------------------

   deallocate (xaeroi_tempa)   !zf2010.11.08
   deallocate (xaeroi_tempb)   !zf2010.11.08

   return
end subroutine aeroint_IPCC

subroutine IPCCsul_diagnostics(state, AEROSOL_r, nlev, shybi, MAerosol, sul_ps)
!-----------------------------------------------------------------
! write out the mass of aerosol in each layer
!-----------------------------------------------------------------

  use physics_types, only: physics_state
  use physconst,     only: rga, rair
  use history,       only: outfld

  type(physics_state), intent(in) :: state     ! state from physics code
  integer,  intent(in) :: nlev
  real(r8), intent(in) :: shybi(nlev)          ! pressure coordinate, from top -> bottom
  real(r8), intent(in) :: sul_ps(pcols)          ! pressure coordinate, from top -> bottom
  real(r8), intent(in) :: AEROSOL_r(pcols, nlev) ! one kind aerosol only here, for sulfure.

  real(r8), intent(out) :: MAerosol(pcols, paerlev+1)  !mass of aerosols in each layer
! real(r8), intent(out) :: MAerosol(pcols, nlev)  !mass of aerosols in each layer

  integer lchnk,ncol  ! chunk index and number of columns
  real(r8) :: AEROSOL_mmr(pcols, nlev) ! one kind aerosol only here, for sulfure.
  real(r8) :: tout(pcols)
  real(r8) :: dplevs(nlev)
  real(r8) :: plevs(nlev)          ! pressure coordinate, from top -> bottom
  integer k, i        ! level indices
!
  plevs(:) = sul_ps(1)*shybi(:)     !  wzz, 2009.10.9
  do k = 1, nlev-1
     dplevs(k) = plevs(k+1)-plevs(k)
  enddo
  dplevs(nlev) = 0.0

  ncol = state%ncol
  lchnk = state%lchnk
!
! convert from kg/m3 -> kg/kg (mmr) first
   do k = 1, nlev
     call vertinterp(ncol, pcols, pver, state%pmid, plevs(k), state%t, tout)
     AEROSOL_mmr(:ncol,k) = AEROSOL_r(:ncol,k)*rair*tout(:ncol)/plevs(k)
   enddo
!
! compute column mass of aerosols (and groups)
!
! first, mass in each layer
!
  MAerosol(:,:) = 0.0

   do k = 1, nlev-1
    MAerosol(:ncol,k) = (AEROSOL_mmr(:ncol,k+1)+AEROSOL_mmr(:ncol,k))*dplevs(k)*rga*0.5
   enddo
!  do k = 1, nlev-1
!     do i = 1, ncol
!        if(plevs(k).gt.state%ps(i)) then
!          MAerosol(i,k) = 0.0
!        endif
!        if(plevs(k).le.state%ps(i).and.plevs(k+1).gt.state%ps(i)) then
!           MAerosol(i,k) = (AEROSOL_mmr(i,k+1)+AEROSOL_mmr(i,k))*(state%ps(i)-plevs(k))*rga*0.5
!        endif
!     enddo
!  enddo
!  do i = 1, ncol
!     if(plevs(nlev).le.state%ps(i)) then
!        MAerosol(i,nlev) = AEROSOL_mmr(i,nlev)*(state%ps(i)-plevs(nlev))*rga
!     endif
!  enddo
!   if(lchnk==300) then
!     write(*,*)'wzz:MAerosol=',MAerosol(3,:)
!   endif
!
! now accumulate mass below upper interface bounding each layer
!
  do k = nlev-1,1,-1
    MAerosol(:ncol,k) = MAerosol(:ncol,k+1) + MAerosol(:ncol,k)
  enddo
!
! Now the total mass in column is given by cumulative mass at level = 1
!
end subroutine IPCCsul_diagnostics
!
subroutine getso4bnd_IPCC( lchnk, ncol, sulfdata )

!----------------------------------------------------------------------- 
! 
! Purpose: Return slice of time interpolated sulfate aerosol.
! 
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns

   real(r8), intent(out) :: sulfdata(pcols,pver)  ! biogenic sulfate
!
! Local variables.
!
   integer :: i, k                 ! longitude, level indices
!-----------------------------------------------------------------------
   do k = 1, pver
      do i = 1, ncol
         sulfdata(i,k)  = sulf(i,k,lchnk)
      end do
   end do

end subroutine getso4bnd_IPCC

subroutine getaerbnd_IPCC( lchnk, ncol, aerodata )

!----------------------------------------------------------------------- 
! 
! Purpose: Return slice of time interpolated sulfate aerosol.
! 
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! Arguments
!
  integer , intent(in)   :: lchnk            ! chunk identifier
  integer , intent(in)   :: ncol             ! number of atmospheric columns

   real(r8), intent(out) :: aerodata(pcols,pver,naer)  ! biogenic sulfate
!
! Local variables.
!
   integer :: i, k                 ! longitude, level indices
!-----------------------------------------------------------------------
   do k = 1, pver
      do i = 1, ncol
         aerodata(i,k,:)  = aero(i,k,lchnk,:)
      end do
   end do

end subroutine getaerbnd_IPCC


!=======================================================================
#ifdef MOZART2

      subroutine get_aerosol_mozart(mmr, aerosol)
!-----------------------------------------------------------------------
!     ... Chem_solver advances the volumetric mixing ratio
!         forward one time step via a combination of explicit,
!         ebi, hov, fully implicit, and/or rodas algorithms.
!-----------------------------------------------------------------------

      use ppgrid,        only : pcols, pver
      use mo_chem_utls,  only : get_spc_ndx
      use chemistry,     only : ixchm
      use mo_grid,       only : pcnstm1

!      use prescribed_aerosols, only: naer_all, idxSUL, idxSSLT, idxDUSTfirst,    &
!                                     idxOCPHO, idxOCPHI, idxBCPHO, idxBCPHI

      implicit none

!-----------------------------------------------------------------------
!        ... Dummy arguments
!-----------------------------------------------------------------------
      real, intent(in)    ::  mmr(pcols, pver, pcnstm1)
      real, intent(inout) ::  aerosol(pcols,pver,naer_all)

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer     :: so4_ndx, sslt01_ndx, sslt02_ndx, sslt03_ndx, sslt04_ndx,      &
                     dst01_ndx, dst02_ndx, dst03_ndx, dst04_ndx, oc1_ndx,          &
                     oc2_ndx, cb1_ndx, cb2_ndx
!-----------------------------------------------------------------------

      so4_ndx =  get_spc_ndx( 'SO4' )
      sslt01_ndx =  get_spc_ndx( 'SSLT01' )
      sslt02_ndx =  get_spc_ndx( 'SSLT02' )
      sslt03_ndx =  get_spc_ndx( 'SSLT03' )
      sslt04_ndx =  get_spc_ndx( 'SSLT04' )
      dst01_ndx =  get_spc_ndx( 'DST01' )
      dst02_ndx =  get_spc_ndx( 'DST02' )
      dst03_ndx =  get_spc_ndx( 'DST03' )
      dst04_ndx =  get_spc_ndx( 'DST04' )
      oc1_ndx =  get_spc_ndx( 'OC1' )
      oc2_ndx =  get_spc_ndx( 'OC2' )
      cb1_ndx =  get_spc_ndx( 'CB1' )
      cb2_ndx =  get_spc_ndx( 'CB2' )

      aerosol(:,:,idxSUL) = mmr(:,:,so4_ndx)
      aerosol(:,:,idxSSLT) = mmr(:,:,sslt01_ndx) + mmr(:,:,sslt02_ndx) + mmr(:,:,sslt03_ndx) + mmr(:,:,sslt04_ndx)
      aerosol(:,:,idxDUSTfirst) = mmr(:,:,dst01_ndx)
      aerosol(:,:,idxDUSTfirst+1) = mmr(:,:,dst02_ndx)
      aerosol(:,:,idxDUSTfirst+2) = mmr(:,:,dst03_ndx)
      aerosol(:,:,idxDUSTfirst+3) = mmr(:,:,dst04_ndx)
      aerosol(:,:,idxOCPHO) = mmr(:,:,oc1_ndx)
      aerosol(:,:,idxBCPHO) = mmr(:,:,cb1_ndx)
      aerosol(:,:,idxOCPHI) = mmr(:,:,oc2_ndx)
      aerosol(:,:,idxBCPHI) = mmr(:,:,cb2_ndx)
      end subroutine get_aerosol_mozart

      subroutine IPCC_aerosol_output(aerosol, lchnk)
      use ppgrid,              only: pcols, pver
      use history,             only: outfld
!     use prescribed_aerosols, only: naer_all, idxSUL, idxSSLT, idxDUSTfirst,    &
!                                    idxOCPHO, idxOCPHI, idxBCPHO, idxBCPHI
      implicit none

      integer, intent(in)  ::  lchnk
      real, intent(in)     ::  aerosol(pcols,pver,naer_all)
!---------------------
      call outfld('IPCC_SO4',aerosol(:,:,idxSUL), pcols, lchnk)
      call outfld('IPCC_SLT',aerosol(:,:,idxSSLT), pcols, lchnk)
      call outfld('IPCCDST1',aerosol(:,:,idxDUSTfirst), pcols, lchnk)
      call outfld('IPCCDST2',aerosol(:,:,idxDUSTfirst+1), pcols, lchnk)
      call outfld('IPCCDST3',aerosol(:,:,idxDUSTfirst+2), pcols, lchnk)
      call outfld('IPCCDST4',aerosol(:,:,idxDUSTfirst+3), pcols, lchnk)
      call outfld('IPCC_OC1',aerosol(:,:,idxOCPHO), pcols, lchnk)
      call outfld('IPCC_OC2',aerosol(:,:,idxOCPHI), pcols, lchnk)
      call outfld('IPCC_CB1',aerosol(:,:,idxBCPHO), pcols, lchnk)
      call outfld('IPCC_CB2',aerosol(:,:,idxBCPHI), pcols, lchnk)
      end subroutine IPCC_aerosol_output

#endif

!###############################################################################
end module prescribed_aerosols
