#include <misc.h>
#include <params.h>

#if (defined BCCCHEM) || (defined CO2) || (defined MOZART2) || (defined WACCM)

module prescribed_fco2_data

!----------------------------------------------------------------------- 
! Purpose:
!
! Interpolation of datasets.
!-----------------------------------------------------------------------

   use shr_kind_mod,   only: r8 => shr_kind_r8
   use pmgrid,         only: plon, plat, masterproc
   use ppgrid,         only: begchunk, endchunk, pcols, pver
   use phys_grid,      only: scatter_field_to_chunk
   use time_manager,   only: get_curr_date, get_curr_calday
   use abortutils,     only: endrun
   use filenames,      only: bndco2emis

#if ( defined SPMD )
   use mpishorthand
#endif

   implicit none

   public fco2ini_IPCC
   public fco2int_IPCC
 
   logical, public  :: fco2cyc = .false.

   save

   real(r8), public, allocatable :: fco2(:,:)
   real(r8), public, allocatable :: co2emis(:,:)

! Private data

  real(r8), private :: cdaym     ! dataset calendar day previous month
  real(r8), private :: cdayp     ! dataset calendar day next month

  integer  :: np1       ! current forward time index of ozone dataset
  integer  :: ncid_fco2 ! netcdf ID for ozone dataset 
  integer  :: fco2id    ! netcdf id for ozone variable
  integer  :: co2emisid ! netcdf id for ozone variable
  integer  :: timesiz   ! size of time dimension on ozone dataset

  real(r8), private, allocatable :: time_fco2(:)   ! Date on dataset (days since 0000 yr) 

!------
!
  real(r8), private, allocatable  :: fco2m(:,:,:)    ! (pcols,begchunk:endchunk,2)
  real(r8), private, allocatable  :: co2emism(:,:,:)    ! (pcols,begchunk:endchunk,2)

!===============================================================================
contains
!===============================================================================

   subroutine fco2ini_IPCC

   use iofilemod,          only : getfil
!
! Local workspace
!
   integer dateid                          ! netcdf id for date variable
   integer secid                           ! netcdf id for seconds variable
   integer londimid                        ! netcdf id for longitude dimension
   integer latdimid                        ! netcdf id for latitude dimension
   integer lonid                           ! netcdf id for longitude variable
   integer latid                           ! netcdf id for latitude variable
   integer timeid                          ! netcdf id for time variable
   integer dimids(7)                       ! variable shape
   integer cnt3(3)                         ! array of counts for each dimension
   integer strt3(3)                        ! array of starting indices
   integer i, j, k, lat, n                    ! longitude, level, latitude, time indices
   integer  :: yr, mon, day                ! components of a date
   integer  :: ncdate                      ! current date in integer format [yyyymmdd]
   integer  :: ncsec                       ! current time of day [seconds]
   integer  :: istat
   real(r8) :: calday                      ! current calendar day
   real(r8) :: caldayloc                     ! calendar day (includes yr if no cycling)
   real(r8) :: fco2bdym(plon,plat)  ! ozone data previous time sample
   real(r8) :: fco2bdyp(plon,plat)  ! ozone data next time sample

   real(r8) :: co2emisbdym(plon,plat)  ! ozone data previous time sample
   real(r8) :: co2emisbdyp(plon,plat)  ! ozone data next time sample

   character(len=256) :: locfn             ! netcdf local filename to open 
   character(len=256) :: filename
  
   integer nm, np
   integer latsiz, lonsiz
!-----------------------------------------------------------------------

   allocate (fco2(pcols,begchunk:endchunk), stat=istat)
   allocate (co2emis(pcols,begchunk:endchunk), stat=istat)

! Initialize module data
!
   nm = 1
   np = 2
!
! SPMD: Master does all the work. Sends needed info to slaves
!
   if (masterproc) then

! need declaration for locfn

      call getfil (bndco2emis, locfn, 0)
      call wrap_open (locfn, 0, ncid_fco2)

!
! Use year information only if not cycling ozone dataset
!
      calday = get_curr_calday()
      call get_curr_date(yr, mon, day, ncsec)

!-------zf 2016.06.12
      if (fco2cyc) then
        caldayloc = calday
      else
        caldayloc = calday + yr*365.  
      end if
!
! Get and check dimension info
!
      CALL WRAP_INQ_DIMID( ncid_fco2, 'lon', londimid   )
      CALL WRAP_INQ_DIMID( ncid_fco2, 'lat', latdimid   )
      CALL WRAP_INQ_DIMID( ncid_fco2, 'time', timeid  )

      CALL WRAP_INQ_DIMLEN( ncid_fco2, londimid, lonsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_fco2, latdimid, latsiz   )
      CALL WRAP_INQ_DIMLEN( ncid_fco2, timeid, timesiz   )

      CALL WRAP_INQ_VARID( ncid_fco2, 'co2emis', co2emisid   )
      CALL WRAP_INQ_VARID( ncid_fco2, 'fco2nat', fco2id   )
     
      CALL WRAP_INQ_VARID( ncid_fco2, 'lon', lonid   )
      CALL WRAP_INQ_VARID( ncid_fco2, 'lat', latid   )

      CALL WRAP_INQ_VARDIMID (ncid_fco2, fco2id, dimids)
      if (lonsiz .ne. plon .or. latsiz .ne. plat ) then
         call endrun ('The lonsiz and latsiz are not the same as the model resolution')
      end if

   end if   ! masterproc

#if (defined SPMD )
   call mpibcast (timesiz, 1, mpiint, 0, mpicom)
   call mpibcast (fco2id, 1, mpiint, 0, mpicom)
   call mpibcast (co2emisid, 1, mpiint, 0, mpicom)
#endif
!
! Dynamically allocated memory for module data
!
   allocate( time_fco2(timesiz),                     &
             co2emism(pcols,begchunk:endchunk,2) ,   &
             fco2m(pcols,begchunk:endchunk,2), stat=istat )

   if (masterproc) then
!
! Retrieve entire date and sec variables.
!
      CALL WRAP_INQ_VARID( ncid_fco2, 'time', timeid   )
      CALL WRAP_GET_VAR_REALX (ncid_fco2,timeid,time_fco2)

!zf      do i=1, timesiz
!zf	     time_fco2(i) = time_fco2(i) + 365.0*1850.0    ! Begin date: 19850.1.1  
!zf      enddo

      strt3(1) = 1
      strt3(2) = 1
      cnt3(1)  = lonsiz
      cnt3(2)  = latsiz
      cnt3(3)  = 1

!---------zf 2016.06.12
!
! Special code for interpolation between December and January
!
      if (fco2cyc) then
        n = 12
        np1 = 1
         cdaym = time_fco2(n)
         cdayp = time_fco2(np1)
         write(6,*) 'zftest fco2ini_IPCC: cdaym = ', cdaym, ' cdayp = ', cdayp 
         if (caldayloc <= cdayp .or. caldayloc > cdaym) then
            strt3(3) = n
            call wrap_get_vara_realx (ncid_fco2,fco2id,strt3,cnt3,fco2bdym)
            call wrap_get_vara_realx (ncid_fco2,co2emisid,strt3,cnt3,co2emisbdym)

            strt3(3) = np1
            call wrap_get_vara_realx (ncid_fco2,fco2id,strt3,cnt3,fco2bdyp)
            call wrap_get_vara_realx (ncid_fco2,co2emisid,strt3,cnt3,co2emisbdyp)
            goto 10
         end if
      end if
!
! Normal interpolation between consecutive time slices.
!
      do n=1,timesiz-1
         np1 = n + 1
         cdaym = time_fco2(n)
         cdayp = time_fco2(n+1)
 
!---------zf 2016.06.13
         if (.not.fco2cyc ) then
           cdaym = cdaym + 365.0*1850.0 
           cdayp = cdayp + 365.0*1850.0 
         end if
!---------------
 
         if (caldayloc >= cdaym .and. caldayloc < cdayp) then
            strt3(3) = n
            call wrap_get_vara_realx (ncid_fco2,fco2id,strt3,cnt3,fco2bdym)
            call wrap_get_vara_realx (ncid_fco2,co2emisid,strt3,cnt3,co2emisbdym)

            do i=1, plon
               do j=1, plat
                  if( co2emisbdym(i,j) .gt. 1.0 .or. co2emisbdym(i,j) .lt. 0.0) then
                      co2emisbdym(i,j) = 0.0
                  endif
                  if( abs( fco2bdym(i,j) ) .gt. 1.0 ) then
                      fco2bdym(i,j) = 0.0
                  endif
               enddo
            enddo

            strt3(3) = np1
            call wrap_get_vara_realx (ncid_fco2,fco2id,strt3,cnt3,fco2bdyp)
            call wrap_get_vara_realx (ncid_fco2,co2emisid,strt3,cnt3,co2emisbdyp)

            do i=1, plon
               do j=1, plat
                  if( co2emisbdyp(i,j) .gt. 1.0 .or. co2emisbdyp(i,j) .lt. 0.0) then
                      co2emisbdyp(i,j) = 0.0
                  endif
                  if( abs( fco2bdyp(i,j) ) .gt. 1.0 ) then
                      fco2bdyp(i,j) = 0.0
                  endif
               enddo
            enddo
            goto 10
         end if
      end do

      write(6,*)'fco2_init: Failed to find dates bracketing ncdate, ncsec=', ncdate, ncsec
      call endrun
!------------------------------------------------
10    continue
 
   endif   ! materproc
!
! Spatial interpolation.  If ozone dataset is 2-d (i.e. lonsiz = 1) and 
! thus only latitude interpolation is necessary, expand to 3-d after 
! interpolation.
!
   call scatter_field_to_chunk(1,1,1,plon,fco2bdym,fco2m(1,begchunk,1))
   call scatter_field_to_chunk(1,1,1,plon,fco2bdyp,fco2m(1,begchunk,2))

   call scatter_field_to_chunk(1,1,1,plon,co2emisbdym,co2emism(1,begchunk,1))
   call scatter_field_to_chunk(1,1,1,plon,co2emisbdyp,co2emism(1,begchunk,2))

#if (defined SPMD )
   call mpibcast (np1, 1, mpiint, 0, mpicom)
   call mpibcast (time_fco2, timesiz, mpir8, 0, mpicom )
   call mpibcast (cdaym, 1, mpir8, 0, mpicom)
   call mpibcast (cdayp, 1, mpir8, 0, mpicom)
#endif

   co2emis(:,:) =0.0_r8   
   fco2(:,:)    =0.0_r8

end subroutine fco2ini_IPCC

!===============================================================================

subroutine fco2int_IPCC ( co2emis0 , fco20 )
!----------------------------------------------------------------------- 
! 
! Purpose: Interpolate ozone mixing ratios to current time, reading in new monthly
!          data if necessary, and spatially interpolating it.
! 
! Method: Find next month of ozone data to interpolate.  Linearly interpolate 
!         vertically and horizontally
! 
! Author: CCM Core Group
! 
!-----------------------------------------------------------------------
   use timeinterp,   only: getfactors
   use phys_grid,    only: get_ncols_p
   use time_manager, only: get_curr_date
!
! Local workspace
!
   real(r8),intent(out)     ::  co2emis0(pcols,begchunk:endchunk)
   real(r8),intent(out)     ::  fco20(pcols,begchunk:endchunk)

   integer cnt3(3)                ! array of counts for each dimension
   integer strt3(3)               ! array of starting indices
   integer i, j, k, lat              ! longitude/column, level, latitude indices
   integer c, ncol                ! chunk index, number of columns in chunk
   integer ntmp                   ! temporary
   integer :: yr, mon, day        ! components of a date
   integer :: ncdate              ! current date in integer format [yyyymmdd]
   integer :: ncsec               ! current time of day [seconds]

   real(r8) fact1, fact2          ! time interpolation factors
   real(r8) :: calday             ! current calendar day
   real(r8) caldayloc             ! calendar day (includes yr if no cycling)
   real(r8) deltat                ! time (days) between interpolating ozone data
!zf   logical  :: fco2cyc           

   real(r8) :: fco2bdyp(plon,plat)  ! ozone data from next time sample
   real(r8) :: co2emisbdyp(plon,plat)  ! ozone data from next time sample
!
! Use year information only if a multiyear dataset
!
   calday    = get_curr_calday()
   call get_curr_date(yr, mon, day, ncsec)

!----------zf2016.06.13
   if (fco2cyc) then
     caldayloc = calday
   else
     caldayloc = calday + yr*365.
   end if
!
! If model time is past current forward ozone timeslice, then
! masterproc reads in the next timeslice for time interpolation.  Messy logic is 
! for fco2cyc = .true. interpolation between December and January (np1 == 1).  Note 
! that np1 is never 1 when fco2cyc is .false.
!
   if (caldayloc > cdayp .and. .not. (np1 == 1 .and. caldayloc > cdaym)) then

      fco2m(:,:,1)    =  fco2m(:,:,2) 
      co2emism(:,:,1) =  co2emism(:,:,2) 

      if (masterproc) then
         if (fco2cyc) then
           np1 = mod(np1,12) + 1           !zf2016.05.13
         else
           np1 = np1 + 1
         end if

         if (np1 > timesiz) then
            call endrun ('fco2_timestep_init: Attempt to read past end of dataset')
         end if

         cdaym = cdayp
         cdayp = time_fco2(np1)
         if ( .not.fco2cyc) then
           cdayp = cdayp + 365.0*1850.0           !zf2016.05.13
         end if
         write(6,*) 'zftest fco2int_IPCC: cdaym = ', cdaym, ' cdayp = ', cdayp 

         if (.not. (np1 == 1 .or. caldayloc <= cdayp)) then
            if (masterproc) then
               write(6,*)'fco2_timestep_init: Input fco2 for date',time_fco2(np1), &
                        'does not exceed model date',' Stopping.'
            endif
            call endrun ()
         end if

         strt3(1) = 1
         strt3(2) = 1
         strt3(3) = np1
         cnt3(1)  = plon
         cnt3(2)  = plat
         cnt3(3)  = 1
!
! Allocate memory for more dynamic local workspace
!
         call wrap_get_vara_realx (ncid_fco2,fco2id,strt3,cnt3,fco2bdyp)
         call wrap_get_vara_realx (ncid_fco2,co2emisid,strt3,cnt3,co2emisbdyp)
         write(6,*)'fco2_ini: Read ozone for date (days) ', time_fco2(np1)

         do i=1, plon
            do j=1, plat
               if( co2emisbdyp(i,j) .gt. 1.0 .or. co2emisbdyp(i,j) .lt. 0.0) then
                   co2emisbdyp(i,j) = 0.0
               endif
!               if( abs( fco2bdyp(i,j) ) .gt. 1.0 ) then
!                   fco2bdyp(i,j) = 0.0
!               endif
            enddo
         enddo

      end if  ! masterproc

#if (defined SPMD )
     call mpibcast (np1, 1, mpiint, 0, mpicom)
     call mpibcast (cdaym, 1, mpir8, 0, mpicom)
     call mpibcast (cdayp, 1, mpir8, 0, mpicom)
#endif

      call scatter_field_to_chunk(1,1,1,plon,fco2bdyp,fco2m(1,begchunk,2) )
      call scatter_field_to_chunk(1,1,1,plon,co2emisbdyp,co2emism(1,begchunk,2) )

   end if

!
! Determine time interpolation factors.
!
!zf   fco2cyc = .false.

   call getfactors (fco2cyc, np1, cdaym, cdayp, caldayloc, &
                    fact1, fact2, 'fco2_data_TIMESTEP_INIT:')
!
! Time interpolation.
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do i=1,ncol
            fco2(i,c) = fco2m(i,c,1)*fact1 + fco2m(i,c,2)*fact2
            fco2(i,c) = fco2(i,c)*1.e+3    ! kg CO2/m2/s -> g CO2/m2/s

            co2emis(i,c) = co2emism(i,c,1)*fact1 + co2emism(i,c,2)*fact2  
                                           ! g CO2/m2/s
      end do
   end do

   fco20(:,:) = fco2(:,:)
   co2emis0(:,:) = co2emis(:,:)

end subroutine fco2int_IPCC

end module prescribed_fco2_data

#endif
