#include <misc.h>
#include <params.h>

module radheat
!-----------------------------------------------------------------------
! Purpose: to provide an interface to convert shortwave and longwave 
! radiative heating terms from radctl into net heating. This module exists
! primarily to provide a hook for waccm, to allow incorporating additional
! radiative terms (eUV heating and nonLTE longwave cooling).
! Public functions/subroutines:
!     radheat_net
! 
! Author: B.A. Boville
! 
!-----------------------------------------------------------------------
  use shr_kind_mod, only: r8 => shr_kind_r8
  use ppgrid,        only: pcols, pver
  use physics_types, only: physics_state, physics_ptend

  implicit none
  private          ! Make default type private to the module
!
! Public interfaces
!
  public radheat_net, radheat_tend   ! return net radiative heating

!===============================================================================
contains
!===============================================================================
  subroutine radheat_net(state, ptend, qrl, qrs)
!-----------------------------------------------------------------------
! Compute net radiative heating from qrs and qrl, and return in ptend.
! This routine provides the waccm hook for computing nonLTE cooling and 
! eUV heating. 
!-----------------------------------------------------------------------

!------------------------------Arguments--------------------------------
    real(r8), intent(in) :: qrl(pcols,pver)        ! LTE longwave heating
    real(r8), intent(in) :: qrs(pcols,pver)        ! shortwave heating (>200 nm)

    type(physics_state), intent(in)    :: state   ! Physics state variables
    type(physics_ptend), intent(inout) :: ptend   ! indivdual parameterization tendencie

!---------------------------Local storage-------------------------------
    integer :: lchnk                               ! chunk identifier
    integer :: ncol                                ! number of atmospheric columns
!-----------------------------------------------------------------------
    lchnk = state%lchnk
    ncol = state%ncol

    ptend%name       = 'radheat'
    ptend%s(:ncol,:) = (qrs(:ncol,:) + qrl(:ncol,:))
    ptend%ls         = .TRUE.

    return
  end subroutine radheat_net
!===============================================================================

subroutine radheat_tend(state, ptend, qrl, qrs, fsns, &
                        fsnt, flns, flnt, asdir, net_flx)
!-----------------------------------------------------------------------
! Compute net radiative heating from qrs and qrl, and the associated net
! boundary flux.
!-----------------------------------------------------------------------
  use physics_types, only:  physics_ptend_init   ! wzz, 2008.7.25

! Arguments
   type(physics_state), intent(in)  :: state             ! Physics state variables
   type(physics_ptend), intent(out) :: ptend             ! indivdual parameterization tendencie
   real(r8),            intent(in)  :: qrl(pcols,pver)   ! longwave heating
   real(r8),            intent(in)  :: qrs(pcols,pver)   ! shortwave heating
   real(r8),            intent(in)  :: fsns(pcols)       ! Surface solar absorbed flux
   real(r8),            intent(in)  :: fsnt(pcols)       ! Net column abs solar flux at model top
   real(r8),            intent(in)  :: flns(pcols)       ! Srf longwave cooling (up-down) flux
   real(r8),            intent(in)  :: flnt(pcols)       ! Net outgoing lw flux at model top
   real(r8),            intent(in)  :: asdir(pcols)      ! shortwave, direct albedo
   real(r8),            intent(out) :: net_flx(pcols)  


! Local variables
   integer :: i
   integer :: ncol
!-----------------------------------------------------------------------

   ncol = state%ncol
   call physics_ptend_init(ptend)

   ptend%name       = 'radheat'
   ptend%s(:ncol,:) = (qrs(:ncol,:) + qrl(:ncol,:))
   ptend%ls         = .TRUE.

   do i = 1, ncol
      net_flx(i) = fsnt(i) - fsns(i) - flnt(i) + flns(i)
   end do

end subroutine radheat_tend
end module radheat
