#include <misc.h>
#include <params.h>

module seasalt_intr

!---------------------------------------------------------------------------------
! Module to interface the aerosol parameterizations with CAM
! written by PJR (extensively modified from chemistry module)
!---------------------------------------------------------------------------------

!-----the following 7 lines are added by lqx, 20070906----------------------
  use shr_kind_mod,only: r8 => shr_kind_r8
  use pmgrid,      only: plat, plev, plevp, masterproc
  use ppgrid,      only: pcols, pver
  use shr_const_mod, only: SHR_CONST_G
  use constituents,only: ppcnst, cnst_add, cnst_name, advected, cnst_get_ind, iaero
  use cam_drydep_mod, only: setdvel, ddflux

  use abortutils,  only: endrun
    
  implicit none

  private          ! Make default type private to the module

  save

!------The following 5 lines are added by Liu Qianxia, 20070906------------
  character(len=80) :: srcnam (12)  !names of source/sink tendencies, 12 advected species
  integer ncyear

  integer :: ncnst                                      ! number of constituents
  character(len=8), allocatable :: cnst_names(:)     ! constituent names
  integer seasalt_idx1

! Public interfaces
!
  public seasalt_register_cnst                        ! register consituents
  public seasalt_implements_cnst                      ! returns true if consituent is implemented by this package
  public seasalt_init_cnst                            ! initialize mixing ratios if not read from initial file
  public seasalt_initialize                           ! initialize (history) variables
  public seasalt_srcsnk                               ! production and loss of seasalt
  public ssyc_idx1, ncnst

!-----------defining mw and cp of all the black carbon bins by Liuqx 20070906------------

  real(r8), parameter:: cpSS1 = 666.
  real(r8), parameter:: cpSS2 = 666.
  real(r8), parameter:: cpSS3 = 666.
  real(r8), parameter:: cpSS4 = 666.
  real(r8), parameter:: cpSS5 = 666.
  real(r8), parameter:: cpSS6 = 666.
  real(r8), parameter:: cpSS7 = 666.
  real(r8), parameter:: cpSS8 = 666.
  real(r8), parameter:: cpSS9 = 666.
  real(r8), parameter:: cpSS10 = 666.
  real(r8), parameter:: cpSS11 = 666.
  real(r8), parameter:: cpSS12 = 666.


  real(r8), parameter:: mwSS1  = 67.18
!  real(r8), parameter:: mwSS1  = 9999.
  real(r8), parameter:: mwSS2  = 9999.
  real(r8), parameter:: mwSS3 = 9999.
  real(r8), parameter:: mwSS4  = 9999.
  real(r8), parameter:: mwSS5  = 9999.
  real(r8), parameter:: mwSS6  = 9999.
  real(r8), parameter:: mwSS7 = 9999.
  real(r8), parameter:: mwSS8  = 9999.
  real(r8), parameter:: mwSS9  = 9999.
  real(r8), parameter:: mwSS10  = 9999.
  real(r8), parameter:: mwSS11= 9999.
  real(r8), parameter:: mwSS12 = 9999.

  real(r8), parameter::  gravit = SHR_CONST_G
!------------------------------------------------
contains

!===============================================================================
  subroutine seasalt_register_cnst
!----------------------------------------------------------------------- 
! 
! Purpose: register seasalt
! 
! Method: 
! Adopt the registering method of CUACEAero which defines 12 bins of seasalt species
!       Liu Qianxia 2007.07.06
!-----------------------------------------------------------------------
!---------------------------Local workspace-----------------------------
    integer :: m, i                                ! tracer index
    integer :: istat
    real(r8), parameter :: zero  = 0._r8
    character names*8

! Define constituent names
    ncnst = seasaltcyc_ncomp()  !liuqx 20070906

    if ( ncnst > 0 ) then
    allocate( cnst_names(ncnst), stat=istat )
    if ( istat /= 0 ) then
    call endrun ('SEASALT_REGISTER_CNST: ERROR: allocate of cnst_names failed') !liuqx
    end if
    end if

    select case ( iaero )
    case (0)
       call endrun('SEASALT_REGISTER_CNST:  code under development')
    case (1)
       if ( ncnst < 1 ) then
          call endrun('SEASALT_REGISTER_CNST:  the selected species are not correct')
       end if
       names = 'SS01'
       do i = 1, ncnst
          write(names(3:4),'(i2.2)') i
          cnst_names(i) = names
!         call cnst_add(cnst_names(i), advected, mwSS1, cpSS1, zero, m, mixtype='dry')
          call cnst_add(cnst_names(i), advected, mwSS1, cpSS1, zero, m)
          if ( i == 1 ) then
             call ssyc_idx_set(m)
          endif
       enddo
    case default
       call endrun('SEASALT_REGISTER_CNST:  unknown scheme')
    end select

  end subroutine seasalt_register_cnst


!=======================================================================
  function seasalt_implements_cnst(name)
!----------------------------------------------------------------------- 
! 
! Purpose: return true if specified constituent is implemented by this 
!          package
! 
!-----------------------------------------------------------------------
!-----------------------------Arguments---------------------------------

     character(len=*), intent(in) :: name   ! constituent name
     logical :: seasalt_implements_cnst     ! return value
     integer :: m
!-----------------------------------------------------------------------

    select case ( iaero )
    case (0)
     call endrun('SEASALT_IMPLEMENTS_CNST:  code under development')
    case (1)
        seasalt_implements_cnst = .false.
        do m = 1, ncnst
        if (name == cnst_names(m)) then
           seasalt_implements_cnst = .true.
           exit
        end if
        end do
    case default
     call endrun('SEASALT_IMPLEMENTS_CNST:  unkown scheme')
    end select

  end function seasalt_implements_cnst


!=======================================================================
  subroutine seasalt_init_cnst(name, q)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Set initial mass mixing ratios.
!
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8

!-----------------------------Arguments---------------------------------
    
    character(len=*), intent(in) :: name         ! constituent name
    
    real(r8), intent(out) :: q(:,:,:)            !  mass mixing ratio
!-----------------------------------------------------------------------
    
    select case ( iaero )
    case ( 0 )
       call endrun('SEASALT_INIT_CNST:  code under development')
    case ( 1 )
       q(:,:,:) = 0.0
    case default 
       call endrun('SEASALT_INIT_CNST:  unknown scheme')
    end select

  end subroutine seasalt_init_cnst


!===============================================================================
  subroutine seasalt_initialize 
!----------------------------------------------------------------------- 
! 
! Purpose: initialize parameterization of seasalt 
!          (declare history variables)
! 
! Method: 
!-----------------------------------------------------------------------
    use history,    only: addfld, add_default, phys_decomp  ! wzz, 2008.8.6
!-----------------------------------------------------------------------
    integer i

    select case ( iaero )
    case ( 0 )
    call endrun('SEASALT_INITIALIZE:  code under development')
    case ( 1 )
! wzz, 2008.8.6
       do i = 1, ncnst
          call addfld (trim(cnst_names(i))//'DRY','kg/m2/s ',1, 'A', &
                      'SS dry deposition',phys_decomp)
          call add_default (trim(cnst_names(i))//'DRY', 1, ' ')
!
          call addfld (trim(cnst_names(i))//'WETB','kg/m2/s ',1, 'A', &
                      'SS wet deposition below cloud',phys_decomp)
          call add_default (trim(cnst_names(i))//'WETB', 1, ' ')
!
          call addfld (trim(cnst_names(i))//'WETI','kg/m2/s ',1, 'A', &
                      'SS wet deposition in cloud',phys_decomp)
          call add_default (trim(cnst_names(i))//'WETI', 1, ' ')
       enddo
!   if (masterproc) write(0,*)'wzz: SEASALT_INITIALIZE is under development'
    case default
    call endrun('SEASALT_INITIALIZE:  unknown scheme')
    end select

  end subroutine seasalt_initialize


!===============================================================================
  subroutine seasalt_srcsnk (state, dt, u10, ocnfrac, ptend)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to emission of all seasalts
! 
! Method: 
!-----------------------------------------------------------------------

    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols

!-----------------------------------------------------------------------
!
! Arguments:
!
    type(physics_state), intent(in ) :: state          ! Physics state variables
    real(r8),            intent(in)  :: dt             ! time step
    real(r8),            intent(in)  :: u10(pcols)     ! 10 meter wind (m/s)
    real(r8),            intent(in)  :: ocnfrac(pcols) ! ocean fraction
    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies

    call endrun('SEASALT_SRCSNK:  code under development')

  end subroutine seasalt_srcsnk 
!#######################################################################

   integer function seasaltcyc_ncomp()
#include <aero_chem_order_cam9.cdk>
    character*8 aeroname(aerochem_np)
    COMMON /CAM_C/ AERONAME
    integer i

    seasaltcyc_ncomp = 0

    if ( iaero == 1 ) then
      do i = 1, aerochem_np
         if ( AERONAME(i) == 'SEA-SALT' ) then
            seasaltcyc_ncomp = aero_nsize
            exit
         end if
      enddo
    endif

   end function seasaltcyc_ncomp
!
   subroutine ssyc_idx_set(m)
     ! store the start index of the Sea-Salt species
     integer m
!    write (6,*) ' ssyc: ssyc_idx_set to ', m
     seasalt_idx1 = m
   end subroutine ssyc_idx_set

   integer function ssyc_idx1()
      ssyc_idx1 = seasalt_idx1
   end function ssyc_idx1

end module seasalt_intr
