#include <misc.h>
#include <params.h>

module volcanicradi
!----------------------------------------------------------------------- 
! 
! Purpose: read, store, interpolate, and retrieve mass fields describing
!   volcanic emissions
! 
! Author: A. Conley, but mostly copied from ozone code base.
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  use pmgrid,       only: plat, plon, plond, masterproc, plev, plevp
  use commap,       only: latdeg
  use ppgrid,       only: begchunk, endchunk, pcols, pver, pverp
  use phys_grid,    only: scatter_field_to_chunk, get_ncols_p
  use filenames,    only: bndtvvolcr
  use abortutils,   only: endrun
  use timeinterp,   only: validfactors
  use rgrid,        only: nlon

#if ( defined SPMD )
  use mpishorthand
#endif

  implicit none

  private
  save

  public read_volcanic_radi
  public get_volcanic_radi
  public volcanic_radi_initialize

  real(r8) cdayprev  ! dataset calendar day previous month
  real(r8) cdaynext  ! dataset calendar day next month

  integer massid    ! netcdf id for volcanic variable
  integer levsiz    ! size of level dimension on volcanic dataset
  integer latsiz    ! size of latitude dimension on volcanic dataset
  integer binsiz    ! size of solar/terrestrial bands dimension on volcanic dataset
  integer sbinsiz    ! size of solar/terrestrial bands dimension on volcanic dataset
  integer tbinsiz    ! size of solar/terrestrial bands dimension on volcanic dataset
  integer timesiz   ! size of time dimension on volcanic dataset
  integer np1       ! current forward time index of volcanic dataset
  integer np_rampyear_01
  integer np_rampyear_12
  integer ncid_volc ! netcdf file id for volcanic file

  real(r8), allocatable :: ext_sun_prev(:,:,:,:), ext_sun_next(:,:,:,:)  ! (pcols,pver,lchunk,binsiz)
  real(r8), allocatable :: omega_sun_prev(:,:,:,:), omega_sun_next(:,:,:,:)
  real(r8), allocatable :: g_sun_prev(:,:,:,:), g_sun_next(:,:,:,:)
  real(r8), allocatable :: ext_ter_prev(:,:,:,:), ext_ter_next(:,:,:,:)
  real(r8), allocatable :: omega_ter_prev(:,:,:,:), omega_ter_next(:,:,:,:)
  real(r8), allocatable :: g_ter_prev(:,:,:,:), g_ter_next(:,:,:,:)
  real(r8), allocatable :: pin(:)              ! volcanic pressure level (levsiz)
  real(r8), allocatable :: volclat(:)          ! Latitudes of volcanic dataset (latsiz)
  integer, allocatable  :: date_volc(:)         ! Date on volcanic dataset (YYYYMMDD)
                                                ! (timesiz)
  integer, allocatable  :: sec_volc(:)          ! seconds of date (0-86399)
                                                ! (timesiz)
  integer, parameter :: nem    = 6
  character*10,parameter :: varname2d (nem) = (/ &
                      'ext_sun   ','omega_sun ','g_sun     ','ext_ter   ','omega_ter ','g_ter     '/)

  logical, public :: aerocyc = .false. ! .true. => assume annual cycle volradi data

contains

subroutine volcanic_radi_initialize(rampyear_prescribed_aeros)
!----------------------------------------------------------------------- 
! 
! Purpose: Read appropriate portion of time-variant volcanic boundary 
!          dataset, containing volcanic mixing ratios as a 
!          function of latitude and pressure.  Read two
!          consecutive months between which the current date lies.  
!
!          Another routine
!          then evaluates the two path length integrals (with and without
!          pressure weighting) from zero to the interfaces between the input
!          levels.  It also stores the contribution to the integral from each
!          layer.
! 
! Method: Call appropriate netcdf wrapper routines and interpolate to model grid
! 
! Author: CCM Core Group
! Modified: P. Worley, August 2003, for chunking and performance optimization
! Modified: A. Conley, Sept 2003, for volcanic aerosols
! 
!-----------------------------------------------------------------------
  use commap, only: latdeg, londeg
  use time_manager, only: get_curr_date, get_perp_date, get_curr_calday, &
                           is_perpetual
  use ioFileMod, only: getfil


!-----------------------------------------------------------------------
  include 'netcdf.inc'
!-----------------------------------------------------------------------
!
! Local workspace
!
  integer dateid                          ! netcdf id for date variable
  integer secid                           ! netcdf id for seconds variable
  integer latdimid                        ! netcdf id for latitude dimension
  integer levdimid                        ! netcdf id for level dimension
  integer sbindimid                        ! netcdf id for bin dimension
  integer tbindimid                        ! netcdf id for bin dimension
  integer latid                           ! netcdf id for latitude variable
  integer levid                           ! netcdf id for level variable
  integer sbinid                           ! netcdf id for level variable
  integer tbinid                           ! netcdf id for level variable
  integer dimids(nf_max_var_dims)         ! variable shape
  integer dim_cnt(4)                      ! array of counts for each dimension
  integer dim_strt(4)                     ! array of starting indices
  integer ilon, ilev, ilat, ibin, time, timep1,time0    ! longitude, level, latitude, bin, time indices
  integer ichunk                          ! index into chunk
  integer idx,ndx,n                         ! index into var 
  integer  :: yr, mon, day                ! components of a date
  integer  :: ncdate                      ! current date in integer format [yyyymmdd]
  integer  :: ncsec                       ! current time of day [seconds]
  integer,intent(in)  :: rampyear_prescribed_aeros
  real(r8) :: calday                      ! current calendar day
  real(r8) caldayloc                      ! calendar day (includes yr)
  real(r8), allocatable :: volcbdyprev(:,:,:,:) ! volcanic data previous time sample
  real(r8), allocatable :: volcbdynext(:,:,:,:) ! volcanic data next time sample
  real(r8), allocatable :: var_prev(:,:,:,:) !(plond,plev,plat,binsiz)
  real(r8), allocatable :: var_next(:,:,:,:) !(plond,plev,plat,binsiz)

  real(r8) :: caldayloc_001, caldayloc_340

  character(len=256) :: locfn          ! netcdf local filename to open

!
! find and open file; abort if fail (getfil(,,0)).
!
! SPMD: Master does all the work.  Sends needed info to slaves
!
  if (masterproc) then
    calday = get_curr_calday()
    call get_curr_date(yr, mon, day, ncsec)

    if(aerocyc)then
       ncdate = rampyear_prescribed_aeros*10000 + mon*100 + day
       caldayloc = calday+rampyear_prescribed_aeros*365
    else
       ncdate = yr*10000 + mon*100 + day
       caldayloc = calday + yr*365.
    endif

!
! Get and check dimension info
!

    call getfil(bndtvvolcr, locfn, 0)
    call wrap_open(locfn, 0, ncid_volc)

    CALL WRAP_INQ_DIMID( ncid_volc, 'lev', levdimid   )
    CALL WRAP_INQ_DIMID( ncid_volc, 'lat', latdimid   )
    CALL WRAP_INQ_DIMID( ncid_volc, 'sbin', sbindimid )
    CALL WRAP_INQ_DIMID( ncid_volc, 'tbin', tbindimid )
    CALL WRAP_INQ_DIMID( ncid_volc, 'date', dateid  )

    CALL WRAP_INQ_DIMLEN( ncid_volc, levdimid, levsiz   )
    CALL WRAP_INQ_DIMLEN( ncid_volc, latdimid, latsiz   )
    CALL WRAP_INQ_DIMLEN( ncid_volc, sbindimid, sbinsiz   )
    CALL WRAP_INQ_DIMLEN( ncid_volc, tbindimid, tbinsiz   )
    CALL WRAP_INQ_DIMLEN( ncid_volc, dateid, timesiz   )

!    CALL WRAP_INQ_VARID( ncid_volc, 'date', dateid   )
!    CALL WRAP_INQ_VARID( ncid_volc, 'datesec', secid   )
    CALL WRAP_INQ_VARID( ncid_volc, 'lat', latid   )
    CALL WRAP_INQ_VARID( ncid_volc, 'lev', levid   )
    CALL WRAP_INQ_VARID( ncid_volc, 'sbin', sbinid   )
    CALL WRAP_INQ_VARID( ncid_volc, 'tbin', tbinid   )

  endif !masterproc

#if (defined SPMD )
  call mpibcast( latsiz, 1, mpiint, 0, mpicom )
  call mpibcast( levsiz, 1, mpiint, 0, mpicom )
  call mpibcast( sbinsiz, 1, mpiint, 0, mpicom )
  call mpibcast( tbinsiz, 1, mpiint, 0, mpicom )
  call mpibcast( timesiz, 1, mpiint, 0, mpicom )
#endif


!
! Dynamically allocated memory for module
!
  allocate (date_volc(timesiz))
  allocate (sec_volc(timesiz))
  allocate (pin(levsiz))
  allocate (volclat(latsiz))

  allocate (ext_sun_prev(pcols,plev,begchunk:endchunk,sbinsiz))
  allocate (ext_sun_next(pcols,plev,begchunk:endchunk,sbinsiz))
  allocate (omega_sun_prev(pcols,plev,begchunk:endchunk,sbinsiz))
  allocate (omega_sun_next(pcols,plev,begchunk:endchunk,sbinsiz))
  allocate (g_sun_prev(pcols,plev,begchunk:endchunk,sbinsiz))
  allocate (g_sun_next(pcols,plev,begchunk:endchunk,sbinsiz))
  allocate (ext_ter_prev(pcols,plev,begchunk:endchunk,tbinsiz))
  allocate (ext_ter_next(pcols,plev,begchunk:endchunk,tbinsiz))
  allocate (omega_ter_prev(pcols,plev,begchunk:endchunk,tbinsiz))
  allocate (omega_ter_next(pcols,plev,begchunk:endchunk,tbinsiz))
  allocate (g_ter_prev(pcols,plev,begchunk:endchunk,tbinsiz))
  allocate (g_ter_next(pcols,plev,begchunk:endchunk,tbinsiz))


  do idx = 1,6


    if (idx .le. 3) then
      binsiz = sbinsiz
    else
      binsiz = tbinsiz
    end if
    allocate (var_prev(plond,plev,plat,binsiz))
    allocate (var_next(plond,plev,plat,binsiz))
!
! More dynamically allocated memory for module 
! (for masterproc only)
!
    allocate (volcbdyprev(levsiz,latsiz,binsiz,1))
    allocate (volcbdynext(levsiz,latsiz,binsiz,1))
    volcbdyprev = 0.
    volcbdynext = 0.

  if (masterproc) then
!
! Retrieve longitude, latitude and level arrays for interpolation.
!
    call wrap_get_var_realx (ncid_volc, latid, volclat)
    call wrap_get_var_realx (ncid_volc, levid, pin)
    CALL WRAP_INQ_VARID( ncid_volc, trim(varname2d(idx)), massid   )
    CALL WRAP_INQ_VARDIMID (ncid_volc, massid, dimids)
    
!
! Retrieve entire date and sec variables.
!
    call wrap_get_var_int (ncid_volc,dateid,date_volc)
!    call wrap_get_var_int (ncid_volc,secid,sec_volc)
    sec_volc = 0


    dim_strt(1) = 1
    dim_strt(2) = 1
    dim_strt(3) = 1
    dim_cnt(1)  = levsiz
    dim_cnt(2)  = latsiz
    dim_cnt(3)  = binsiz
    dim_cnt(4)  = 1


  if(aerocyc)then
     caldayloc_340 = 340.0 + rampyear_prescribed_aeros*365.
     caldayloc_001 = 1.0 + rampyear_prescribed_aeros*365.
     do time=1,timesiz-1
        timep1 = time + 1
        call bnddyi(date_volc(time), sec_volc(time), cdayprev)
        call bnddyi(date_volc(timep1 ), sec_volc(timep1 ), cdaynext)

        yr = date_volc(time)/10000
        cdayprev = cdayprev + yr*365.
        yr = date_volc(timep1)/10000
        cdaynext = cdaynext + yr*365.

        if (caldayloc_001 > cdayprev .and. caldayloc_001 <= cdaynext) then
           np_rampyear_01 = timep1
        endif
        if (caldayloc_340 > cdayprev .and. caldayloc_340 <= cdaynext) then
           np_rampyear_12  = timep1
       endif
       if( np_rampyear_12 == 12) np_rampyear_01 = 1
     end do
  endif
!
! Normal interpolation between consecutive time slices.
!
  if( aerocyc .and. np_rampyear_12 == 12 ) then
      time = mon-1
      np1  = time+1
      if( (mon==12 .and. day > 15) .or. (mon==1 .and. day < 15) ) then
          time = 12
          np1  = 1 
      endif
      call bnddyi(date_volc(time), sec_volc(time), cdayprev)
      call bnddyi(date_volc(np1 ), sec_volc(np1 ), cdaynext)

      yr = date_volc(time)/10000
      cdayprev = cdayprev + yr*365.
      yr = date_volc(np1)/10000
      cdaynext = cdaynext + yr*365.

      dim_strt(4) = time
      call wrap_get_vara_realx (ncid_volc,massid,dim_strt,dim_cnt,volcbdyprev)
      dim_strt(4) = np1
      call wrap_get_vara_realx (ncid_volc,massid,dim_strt,dim_cnt,volcbdynext)
      goto 10

  else

    do time=1,timesiz-1
      np1 = time + 1
      call bnddyi(date_volc(time), sec_volc(time), cdayprev)
      call bnddyi(date_volc(np1 ), sec_volc(np1 ), cdaynext)

      yr = date_volc(time)/10000
      cdayprev = cdayprev + yr*365.
      yr = date_volc(np1)/10000
      cdaynext = cdaynext + yr*365.

      if (caldayloc > cdayprev .and. caldayloc <= cdaynext) then
         if( aerocyc .and. mod(np1,12) == 1 ) then
             time0 = np_rampyear_12
             call bnddyi(date_volc(time0), sec_volc(time0), cdayprev)
             yr = date_volc(time0)/10000
             cdayprev = cdayprev + yr*365.
             dim_strt(4) = time0
             call wrap_get_vara_realx (ncid_volc,massid,dim_strt,dim_cnt,volcbdyprev)

             np1 = np_rampyear_01
             call bnddyi(date_volc(np1), sec_volc(np1), cdaynext)
             yr = date_volc(np1)/10000
             cdaynext = cdaynext + yr*365.
             dim_strt(4) = np1
             call wrap_get_vara_realx (ncid_volc,massid,dim_strt,dim_cnt,volcbdynext)

         else
             dim_strt(4) = time
             call wrap_get_vara_realx (ncid_volc,massid,dim_strt,dim_cnt,volcbdyprev)
             dim_strt(4) = np1
             call wrap_get_vara_realx (ncid_volc,massid,dim_strt,dim_cnt,volcbdynext)
         endif 
         goto 10
      endif
    end do
    write(6,*)'volcanic_initialize: Failed to find dates bracketing ncdate, ncsec=', ncdate, ncsec
    call endrun

 endif

10  continue

 end if  ! master  

#if (defined SPMD )
  call mpibcast (np1, 1, mpiint, 0, mpicom)
  call mpibcast (np_rampyear_01, 1, mpiint, 0, mpicom)
  call mpibcast (np_rampyear_12, 1, mpiint, 0, mpicom)
  call mpibcast (pin, levsiz, mpir8, 0, mpicom)
  call mpibcast (date_volc, timesiz, mpiint, 0, mpicom )
  call mpibcast (sec_volc, timesiz, mpiint, 0, mpicom )
  call mpibcast (cdayprev, 1, mpir8, 0, mpicom)
  call mpibcast (cdaynext, 1, mpir8, 0, mpicom)
  call mpibcast (volclat, latsiz, mpir8, 0, mpicom)
#endif

  if (masterproc) then
    do ibin = 1, binsiz
          call aero_interpolate( levsiz, pin, latsiz, volclat, volcbdyprev(1:levsiz,1:latsiz,ibin,1), &
                                 var_prev(1:plond,1:plev,1:plat,ibin))
          call aero_interpolate( levsiz, pin, latsiz, volclat, volcbdynext(1:levsiz,1:latsiz,ibin,1), &
                                 var_next(1:plond,1:plev,1:plat,ibin))
    end do ! binsiz


    do ibin = 1,binsiz
      if( idx==1 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),ext_sun_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_sun_next(:,:,:,ibin))
      end if
      if( idx==2 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),omega_sun_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_sun_next(:,:,:,ibin))
      end if
      if( idx==3 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),g_sun_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_sun_next(:,:,:,ibin))
      end if
      if( idx==4 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),ext_ter_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_ter_next(:,:,:,ibin))
      end if
      if( idx==5 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),omega_ter_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_ter_next(:,:,:,ibin))
      end if
      if( idx==6 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),g_ter_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_ter_next(:,:,:,ibin))
      end if
    end do


#if (defined SPMD )
else
    do ibin = 1,binsiz
      if( idx==1 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),ext_sun_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_sun_next(:,:,:,ibin))
      end if
      if( idx==2 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),omega_sun_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_sun_next(:,:,:,ibin))
      end if
      if( idx==3 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),g_sun_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_sun_next(:,:,:,ibin))
      end if
      if( idx==4 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),ext_ter_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_ter_next(:,:,:,ibin))
      end if
      if( idx==5 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),omega_ter_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_ter_next(:,:,:,ibin))
      end if
      if( idx==6 ) then
          call scatter_field_to_chunk(1,plev,1,plond,var_prev(:,:,:,ibin),g_ter_prev(:,:,:,ibin))
          call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_ter_next(:,:,:,ibin))
      end if
    end do
#endif
endif !master
!    deallocate (volclat)
    deallocate (volcbdyprev)
    deallocate (volcbdynext)
    deallocate(var_prev)
    deallocate(var_next)
end do !idx


return
end subroutine volcanic_radi_initialize


subroutine aero_interpolate( levsiz, pin, latsiz, volclat, volc_levsize, volc3D)

#include <comhyb.h>
   real(r8), intent(in)  :: volc_levsize(levsiz,latsiz)
   integer,  intent(in)  :: levsiz, latsiz
   real(r8), intent(in)  :: pin(levsiz), volclat(latsiz)
   real(r8), intent(out) :: volc3D(plond,plev,plat)

   real(r8)  :: volc2D(levsiz,plat)    ! temporary volcanic arrays
   real(r8)  :: volc_plev(plev,plat)
   real(r8)  :: model_level(pver)
   integer   :: lchnk              ! chunk index
   integer   :: kk,idx     ! boundary data index into file data
   integer   :: ibin,ilat, ilon, ilev, icol ! indices
   real(r8)  :: dpl, dpu   ! delta p across lower and upper parts of interpolant

!
! Latitude interpolation . Expand to 3D after interpolation.
!

   call lininterp (volc_levsize,volclat   ,levsiz  ,latsiz  ,volc2D, &
                   latdeg  ,plat    )

!
! Vertical interpolation
!

   model_level(:) = 1.e3_r8 *(hyam(:) + hybm(:))   ! pin : hPa

      volc_plev(1,1:plat) = volc2D(1,1:plat)
!
! At every pressure level, interpolate onto that pressure level
!
      kk = 2 
      do ilev = 2, pver
!
! If we've fallen through the kk=1,levsiz-1 loop, we cannot interpolate and
! must extrapolate from the bottom or top pressure level for at least some
! of the longitude points.
!

        if (model_level(ilev) .lt. pin(1)) then
             volc_plev(ilev,1:plat) = volc2D(1,1:plat)

        else if (model_level(ilev) .ge. pin(levsiz)) then
             volc_plev(ilev,1:plat) = 0._r8

        else
             do while (model_level(ilev) .ge. pin(kk) )
                kk = kk + 1
             enddo

             dpu = model_level(ilev) - pin(kk-1)
             dpl =           pin(kk) - model_level(ilev)
             volc_plev(ilev,1:plat) = &
                  (volc2D(kk-1,1:plat)*dpl + &
                  volc2D(kk  ,1:plat)*dpu)/(dpl + dpu)

        end if

      end do

   do ilat=1,plat
      do ilev=1,plev
         do ilon=1,nlon(ilat)
            volc3D(ilon,ilev,ilat) = volc_plev(ilev,ilat)
         end do
      end do
   end do

  return

end subroutine aero_interpolate



subroutine read_volcanic_radi(rampyear_prescribed_aeros)
  
  use rgrid,        only: nlon
  use time_manager, only: get_curr_date, get_curr_calday

  integer :: dim_cnt(4)                ! array of counts for each dimension
  integer :: dim_strt(4)               ! array of starting indices
  integer :: ibin,ilon,ilev,ilat,icol,ncols ! indices
  integer :: idx,ndx
  integer :: lchnk                     ! chunk index
  integer :: yr,mon,day                ! components of date
  integer :: ncdate                    ! formated date [yyyymmdd]
  integer :: ncsec                     ! seconds past midnight of current ncdate
  real(r8):: calday                    ! current calendar day
  real(r8):: caldayloc                 ! calendar day (including yr)
  real(r8):: deltat                    ! time (days) between times of dataset
  real(r8), allocatable :: volcbdynext(:,:,:,:) ! volcanic data next time sample
  real(r8), allocatable :: var_next(:,:,:,:) !(plond,plev,plat,binsiz)
  real(r8):: temp_sbin(pcols,plev,begchunk:endchunk,sbinsiz)
  real(r8):: temp_tbin(pcols,plev,begchunk:endchunk,tbinsiz)
  integer,intent(in)  :: rampyear_prescribed_aeros
 
  include 'netcdf.inc'


  call get_curr_date(yr, mon, day, ncsec)
  calday = get_curr_calday()

  if(aerocyc)then
     ncdate = rampyear_prescribed_aeros*10000 + mon*100 + day
     caldayloc = calday + rampyear_prescribed_aeros*365.
  else
     ncdate = yr*10000 + mon*100 + day
     caldayloc = calday + yr*365.
  endif

  if ( ( (.not. aerocyc)  .and. caldayloc > cdaynext .and. .not.(np1 == 1 .and. caldayloc > cdayprev)) .or. &
       (aerocyc .and. caldayloc > cdaynext .and. .not.(np1 == np_rampyear_01 .and. caldayloc > cdayprev)) .or. &
       (aerocyc .and. caldayloc > cdaynext .and. (np1 == np_rampyear_01 .and. cdaynext < cdayprev .and. caldayloc < cdayprev)) ) then
!if (caldayloc > cdaynext .and. .not.(np1 == 1 .and. caldayloc > cdayprev)) then 

      np1 = np1 + 1
      if(aerocyc) then
         if( np1 > np_rampyear_12 )  then
             np1 = np_rampyear_01
         endif
      endif

      if (np1 > timesiz) then
          call endrun ('READ_VOLCANIC_RADI: Attempt to read past end of Volcanic dataset')
      end if
    
      cdayprev = cdaynext
      call bnddyi(date_volc(np1), sec_volc(np1), cdaynext)
      yr = date_volc(np1)/10000
      cdaynext = cdaynext + yr*365.

   if(.not.aerocyc)then
      if (.not. (np1 == 1 .or. caldayloc <= cdaynext)) then
          write(6,*)'caldayloc, cdayprev, cdaynext, np1: ',caldayloc, cdayprev, cdaynext, np1
          if (masterproc) then
              write(6,*)'volcanicradi_timestep_init: Input for date',date_volc(np1), &
                        'does not exceed model date',' Stopping.'
          endif
          call endrun ()
      end if
   end if

         temp_sbin      = ext_sun_prev
         ext_sun_prev   = ext_sun_next
         ext_sun_next   = temp_sbin
         temp_sbin      = omega_sun_prev
         omega_sun_prev = omega_sun_next
         omega_sun_next = temp_sbin
         temp_sbin      = g_sun_prev
         g_sun_prev     = g_sun_next
         g_sun_next     = temp_sbin

         temp_tbin      = ext_ter_prev
         ext_ter_prev   = ext_ter_next
         ext_ter_next   = temp_tbin
         temp_tbin      = omega_ter_prev
         omega_ter_prev = omega_ter_next
         omega_ter_next = temp_tbin
         temp_tbin      = g_ter_prev
         g_ter_prev     = g_ter_next
         g_ter_next     = temp_tbin


!
! Allocate memory for dynamic local workspace
!

  do idx = 1,6
      if (idx .le. 3) then
          binsiz = sbinsiz
      else
          binsiz = tbinsiz
      end if
      allocate (volcbdynext(levsiz,latsiz,binsiz,1))
      allocate (var_next(plond,plev,plat,binsiz))

  if (masterproc) then
!
! Allocate memory for more dynamic local workspace
!
      dim_strt(1) = 1
      dim_strt(2) = 1
      dim_strt(3) = 1
      dim_strt(4) = np1
      dim_cnt(1)  = levsiz
      dim_cnt(2)  = latsiz
      dim_cnt(3)  = binsiz
      dim_cnt(4)  = 1

      call wrap_inq_varid( ncid_volc, trim(varname2d(idx)), massid   )
      call wrap_get_vara_realx(ncid_volc,massid,dim_strt,dim_cnt,volcbdynext)

    do ibin = 1, binsiz
       call aero_interpolate( levsiz, pin, latsiz, volclat, volcbdynext(1:levsiz,1:latsiz,ibin,1), &
                              var_next(1:plond,1:plev,1:plat,ibin))
    end do ! binsiz


    do ibin = 1,binsiz
       if( idx==1 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_sun_next(:,:,:,ibin))
       end if
       if( idx==2 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_sun_next(:,:,:,ibin))
       end if
       if( idx==3 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_sun_next(:,:,:,ibin))
       end if
       if( idx==4 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_ter_next(:,:,:,ibin))
       end if
       if( idx==5 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_ter_next(:,:,:,ibin))
       end if
       if( idx==6 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_ter_next(:,:,:,ibin))
       end if
    end do

#if (defined SPMD )
  else
    do ibin = 1,binsiz
       if( idx==1 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_sun_next(:,:,:,ibin))
       end if
       if( idx==2 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_sun_next(:,:,:,ibin))
       end if
       if( idx==3 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_sun_next(:,:,:,ibin))
       end if
       if( idx==4 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),ext_ter_next(:,:,:,ibin))
       end if
       if( idx==5 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),omega_ter_next(:,:,:,ibin))
       end if
       if( idx==6 ) then
           call scatter_field_to_chunk(1,plev,1,plond,var_next(:,:,:,ibin),g_ter_next(:,:,:,ibin))
       end if
    end do
#endif
  endif !master
    deallocate(volcbdynext)
    deallocate(var_next)
  end do !idx

!    else  ! if(caldayloc <= cdaynext)
!      if (masterproc) then
!        write(6,*)'READ_VOLCANIC_MASS: date from dataset ',date_volc(np1), &
!                  ' does not exceed model date ', n)cdate
!      endif
!      call endrun
!    endif ! if(caldayloc <= cdaynext)

        
  endif ! (caldayloc > cdaynext)

end subroutine read_volcanic_radi


subroutine get_volcanic_radi(lchnk, aerosol_sext,aerosol_somega,aerosol_sg, &
                                    aerosol_text,aerosol_tomega,aerosol_tg, &
                                    rampyear_prescribed_aeros)
 
  use rgrid,        only: nlon
  use time_manager, only: get_curr_date, get_curr_calday
  use timeinterp,   only: getfactors

  integer, intent(in)   :: lchnk           ! chunk id for local aerosol array
  integer, intent(in)   :: rampyear_prescribed_aeros
  real(r8), intent(out) :: aerosol_sext(:,:,:),aerosol_somega(:,:,:),aerosol_sg(:,:,:)
  real(r8), intent(out) :: aerosol_text(:,:,:),aerosol_tomega(:,:,:),aerosol_tg(:,:,:)
  integer :: dim_cnt(4)                ! array of counts for each dimension
  integer :: dim_strt(4)               ! array of starting indices
  integer :: ibin,ilon,ilev,ilat,icol,ncols,idx ! indices
  integer :: yr,mon,day                ! components of date
  integer :: ncdate                    ! formated date [yyyymmdd]
  integer :: ncsec                     ! seconds past midnight of current ncdate
  integer :: aerocyc_np1
  real(r8):: aerocyc_cdayprev
  real(r8):: aerocyc_cdaynext
  real(r8):: fact1, fact2              ! time interpolation factors
  real(r8):: calday                    ! current calendar day
  real(r8):: caldayloc                 ! calendar day (including yr)
  real(r8):: deltat                    ! time (days) between times of dataset
!
! time interpolate
!
  call get_curr_date(yr, mon, day, ncsec)
  calday = get_curr_calday()

  if(aerocyc)then
     caldayloc = calday
     aerocyc_np1 = mod(np1,12)
     aerocyc_cdayprev = mod(cdayprev,365.)
     aerocyc_cdaynext = mod(cdaynext,365.)
     if(aerocyc_np1==0) aerocyc_np1=12
     call getfactors (aerocyc, aerocyc_np1, aerocyc_cdayprev, aerocyc_cdaynext, caldayloc, &
                   fact1, fact2, 'get_volcanic_radi:')

  else
     caldayloc = calday + yr*365.
     call getfactors (aerocyc, np1, cdayprev, cdaynext, caldayloc, &
                   fact1, fact2, 'get_volcanic_radi:')
  endif

!
! Check sanity of time interpolation calculation to within 32-bit roundoff
!
  if (.not. validfactors (fact1, fact2)) then
     write(6,*)'get_volcanic_radi: Bad fact1 and/or fact2=',cdayprev, cdaynext, caldayloc, fact1,fact2
     call endrun ()
  end if
!
! Interpolate (in time)!
!
  ncols = get_ncols_p(lchnk)
    do ibin=1,sbinsiz
      do ilev=1,plev
        do icol=1,ncols
          aerosol_sext(icol,ilev,ibin)   = ext_sun_prev(icol,ilev,lchnk,ibin)*fact1 &
                                          +ext_sun_next(icol,ilev,lchnk,ibin)*fact2
          aerosol_somega(icol,ilev,ibin) = omega_sun_prev(icol,ilev,lchnk,ibin)*fact1 &
                                          +omega_sun_next(icol,ilev,lchnk,ibin)*fact2
          aerosol_sg(icol,ilev,ibin)     = g_sun_prev(icol,ilev,lchnk,ibin)*fact1  &
                                          +g_sun_next(icol,ilev,lchnk,ibin)*fact2
        enddo
      enddo
    enddo



    do ibin=1,tbinsiz
      do ilev=1,plev
        do icol=1,ncols
          aerosol_text(icol,ilev,ibin)   = ext_ter_prev(icol,ilev,lchnk,ibin)*fact1   &
                                          +ext_ter_next(icol,ilev,lchnk,ibin)*fact2
          aerosol_tomega(icol,ilev,ibin) = omega_ter_prev(icol,ilev,lchnk,ibin)*fact1 &
                                          +omega_ter_next(icol,ilev,lchnk,ibin)*fact2
          aerosol_tg(icol,ilev,ibin)     = g_ter_prev(icol,ilev,lchnk,ibin)*fact1     &
                                          +g_ter_next(icol,ilev,lchnk,ibin)*fact2
        enddo
      enddo
    enddo

return

end subroutine get_volcanic_radi

end module volcanicradi

