#include <misc.h>
#include <params.h>
module wu_cdnc

!---------------------------------------------------------------------------------
! Purpose:
!
! Compute cloud cloud optical properties: liquid and ice partical size; emissivity
!
! Author: wutw 
!
!---------------------------------------------------------------------------------

  use shr_kind_mod,  only: r8=>shr_kind_r8
  use ppgrid,        only: pcols, pver, pverp

 save
   private
   public :: get_cdnc

contains

!===============================================================================
  subroutine get_cdnc (lchnk   ,ncol    , state, cloud, rel, rei, CDNC )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Compute liquid cloud droplet number concentration and cloud effective drop size
! 
! Method: 
! use empirical formulas to construct effective radii
! 
! Author: J.T. Kiehl, B. A. Boville, P. Rasch
! 
!-----------------------------------------------------------------------

#ifdef MAC_SP
    use prescribed_aerosols_cdnc, only: get_aerosol_cdnc
    use prescribed_aerosols,      only: naer_all, &
                                        get_int_scales, idxSUL, idxSSLT, idxSSLT01, idxOCPHI, idxNH4NO3
#else
    use prescribed_aerosols, only: get_aerosol, naer_all, &
                                   get_int_scales, idxSUL, idxSSLT, idxSSLT01, idxOCPHI, idxNH4NO3
                                 ! idxSUL   =  1
                                 ! idxSSLT01=  11
                                 ! idxOCPHI =  9
#endif

    use aerosol_intr,        only: set_aerosol_from_prognostics
    use pkg_cldoptics,       only: reitab
    use physics_types,       only: physics_state
    use constituents,        only: cnst_get_ind
    use history,             only: outfld
    use physconst,           only: gravit

    implicit none
!------------------------------Arguments--------------------------------
!
! Input arguments
!
    integer, intent(in) :: lchnk                 ! chunk identifier
    integer, intent(in) :: ncol                  ! number of atmospheric columns
    type(physics_state), intent(in), target :: state
    real(r8), intent(in) :: cloud(pcols,pver) 
!
! Output arguments
!
    real(r8), intent(out) :: rel(pcols,pver)      ! Liquid effective drop size (microns)
    real(r8), intent(out) :: CDNC(pcols,pver)     ! Liquid cloud droplet number concentration  (1/cm3)
    real(r8), intent(out) :: rei(pcols,pver)      ! Ice effective drop size (microns)
!--------------------------------------------------------------------------------------------
    real(r8) aerosol(pcols, pver, naer_all) ! aerosol mass mixing ratios
    real(r8) scales(naer_all)               ! scaling factors for aerosols
    real(r8) dNovrN(pcols)
    real(r8) CDNC1850(pcols,pver)     ! Liquid cloud droplet number concentration  (1/cm3)

!-------------------------------------------------------------------------------
    integer :: i, k
    real(r8) :: sulfate
    real(r8) :: OM
    real(r8) :: OC
    real(r8) :: Seasalt
    real(r8) :: Seasalt01
    real(r8) :: nh4no3
    real(r8) :: airdensity
    real(r8) :: Rhocgs
    real(r8) :: LWC            ! cloud liquid water content 
    real(r8) :: IWC            ! cloud liquid water content 
    real(r8) :: pi
    real(r8) :: rvol           ! volume-weighted mean cloud droplet radius    
    real(r8) :: reivol    
    real(r8) :: CDNC_ice
    real(r8) :: Ricecgs
    real(r8) :: m_to_mmr

    integer  :: ixcldliq
    integer  :: ixcldice
!------------------------------------------------
!#ifdef MAC_SP
!    dNovrN(:) = 1.
!    call get_mac_cdnc (lchnk   ,ncol    , state%phis(:ncol),  state%zm(:ncol,pver) , &
!                       state%zi(:ncol,pverp), dNovrN(:ncol)    )
!#endif

!-------------------------------------------------

       call get_int_scales(scales)   

#ifdef MAC_SP
       call get_aerosol_cdnc(lchnk, state%pint, aerosol, scales, state)  ! aerosol : mmr 
#else
       call get_aerosol(lchnk, state%pint, aerosol, scales, state)  ! aerosol : mmr
#endif
       cdnc(:,:) = 0.0_r8
       rel(:,:)  = 0.0_r8
       rei(:,:)  = 0.0_r8

       call cnst_get_ind('CLDLIQ', ixcldliq)
       call cnst_get_ind('CLDICE', ixcldice)

       do i=1, ncol
          do k=1, pver

             sulfate = aerosol(i,k,idxSUL) 
             OC      = aerosol(i,k,idxOCPHI) 
             Seasalt01 = aerosol(i,k,idxSSLT01) 
             Seasalt = aerosol(i,k,idxSSLT)
             nh4no3  = aerosol(i,k,idxNH4NO3)

             airdensity = state%pmid(i,k)/(287.04*state%t(i,k))   ! kg/m3

             sulfate = sulfate * airdensity * 1.0e+09   ! mmr -> ug.m-3 
             OC      = OC  * airdensity * 1.0e+09   ! mmr -> ug.m-3
             OM      = 1.3*OC
             Seasalt = Seasalt * airdensity * 1.0e+09   ! mmr -> ug.m-3
             Seasalt01 = Seasalt01 * airdensity * 1.0e+09   ! mmr -> ug.m-3
             nh4no3  = nh4no3  * airdensity * 1.0e+09   ! mmr -> ug.m-3

             sulfate = max(sulfate, 1.e-15)
             OC      = max(OC, 1.e-15)
             OM      = max(OM, 1.e-15)
             Seasalt = max(Seasalt, 1.e-15)
             Seasalt01 = max(Seasalt01, 1.e-15)
             nh4no3  = max(nh4no3, 1.e-15)

           ! CDNC(i,k) =  10.0**(2.41+0.50*log10(sulfate)+0.13*log10(OM)+0.05*log10(Seasalt01))   ! cm-3

           ! ---------------QBL06 2006, ACP, scheme-
           !
              CDNC(i,k) =  exp(5.1+0.41*log(sulfate+OC+seasalt01+nh4no3))   ! cm-3 

           ! --------------Hegg 1994, JGR, scheme-
           !
           !  CDNC(i,k) =  90.7*(sulfate+OC+seasalt01+nh4no3)**0.45+23

             CDNC(i,k) = min( max(CDNC(i,k), 5.0),  2000.0) 
           ! CDNC(i,k) = min( max(CDNC(i,k), 30.0), 3500.0) 


!#ifdef MAC_SP
!             CDNC1850(i,k) =  CDNC(i,k) 
!             CDNC(i,k) = CDNC1850(i,k)* dNovrN(i)    ! CDNC is calculated by aerosol in 1850 
!                                                     ! multiply the mac_sp factor (dNovrN) in ramped year 
!             CDNC(i,k) = min( max(CDNC(i,k), 5.0),  2000.0)
!#endif

             Rhocgs    = airdensity * 1.e+3               ! kg m-3 -> g m-3
             LWC       = Rhocgs * state%q(i,k,ixcldliq)/max(cloud(i,k),0.01)   ! cloud liquid water content (gm-3)
             LWC       = max(LWC, 3.0e-15)             ! gm-3
             pi        = 3.1415926

             rvol =  (LWC/(1.3333*pi*1.0*1.e+12*CDNC(i,k)))**0.33333 *1.e+6     ! m -> um (micron)
             rel(i,k) =  min((0.00084*CDNC(i,k)+1.2),1.6)*rvol

       !      Rhocgs    = airdensity * 1.e-3               ! kg m-3 -> g cm-3
       !      LWC       = Rhocgs * state%q(i,k,ixcldliq)/max(cloud(i,k),0.01)   ! cloud liquid water content (g cm-3) 
       !      LWC       = max(LWC, 3.0e-4*1.e-6)             ! g cm-3
       !      pi        = 3.1415926
       !      rvol =  (LWC/(1.3333*pi*1.0*CDNC(i,k)))**0.33333 *1.e+6     ! m -> nm (micron)
       !      rel(i,k) =  1.27*rvol * 1.e+06                              ! m -> nm (micron) 
   
              rel(i,k) = min( max(rel(i,k),3.0), 25.)

       !      Ricecgs  = airdensity * 0.917e-3                ! kg m-3 -> g cm-3
       !      IWC      = Ricecgs * state%q(i,k,ixcldice)/max(cloud(i,k),0.01)
       !      CDNC_ice = 1.0e-4 * exp(0.2*(271.15-state%t(i,k)))   ! cm-3
       !      reivol = (IWC/(1.3333*pi*1.0*CDNC_ice))**0.33333 *1.e-02    ! cm -> m
       !      rei(i,k) = 1.2*reivol * 1.e+09                          ! m -> nm 
       !      rei(i,k) = min( max(rei(i,k),1.0), 25.)
 
          enddo
       enddo

!-------------------------
!
! following Kristjansson and Mitchell

       call reitab(ncol, state%t, rei)

!------------------------------
       call outfld('CDNC', cdnc, pcols, lchnk)
       call outfld('REL',  rel, pcols, lchnk)
       call outfld('REI',  rei, pcols, lchnk)

       return
  end subroutine get_cdnc


#ifdef MAC_SP
  subroutine get_mac_cdnc (lchnk   ,ncol    , phis,  zm,  zi, dNovrN)
   use shr_kind_mod,    only: r8 => shr_kind_r8
   use physconst,       only: gravit
   use time_manager,    only: get_curr_date
   use volcanicradi,    only: aerocyc
   use prescribed_aerosols, only: rampyear_prescribed_aeros
   use mo_simple_plumes, only: sp_aop_profile
   use phys_grid,       only: get_rlat_all_p, get_rlon_all_p

   implicit none

   real(r8) ::  wavmin(19), wavmax(19)
   data wavmin / .200,  .245,  .265,  .275,  .285, &
                 .295,  .305,  .350,  .640,  .700,  .701, &
                 .701,  .701,  .701,  .702,  .702, &
                 2.630, 4.160, 4.160/
   data wavmax / .245,  .265,  .275,  .285,  .295, &
                 .305,  .350,  .640,  .700, 5.000, 5.000, &
                 5.000, 5.000, 5.000, 5.000, 5.000, &
                 2.860, 4.550, 4.550/
!---------------------------------------------
   real(r8), intent(in) :: phis(ncol)    ! m2/s2 
   real(r8), intent(in) :: zm(ncol,pver)  
   real(r8), intent(in) :: zi(ncol,pverp)  
   real(r8), intent(out) :: dNovrN(ncol)  

!--

   integer, intent(in) :: lchnk                 ! chunk identifier
   integer, intent(in) :: ncol                  ! number of atmospheric columns
   real(r8) :: clat_org(ncol),clon_org(ncol) 
   real(r8) :: oro(ncol) 
   real(r8) :: year_fr
   real(r8) :: wavmid
   integer  :: iyear, imon, iday, tod
   integer, parameter :: nmon=12
   integer  :: i, k
   real(r8) :: zdelt(ncol,pver)
   real(r8) :: aod_prof(ncol,pver)
   real(r8) :: ssa_prof(ncol,pver)
   real(r8) :: asy_prof(ncol,pver)

!--------------------------------------------- 
    oro(:)=phis(:)/gravit

    call get_curr_date(iyear,imon,iday,tod)
    if(aerocyc)then
       year_fr = rampyear_prescribed_aeros + (imon-0.5) / nmon
    else
       year_fr = iyear + (imon-0.5) / nmon
    endif

    call get_rlat_all_p(lchnk, ncol, clat_org)
    call get_rlon_all_p(lchnk, ncol, clon_org)
    clat_org(:ncol) = clat_org(:ncol)*57.2958  !radians to degrees
    clon_org(:ncol) = clon_org(:ncol)*57.2958  !radians to degrees

    oro(:ncol) = phis(:ncol)/gravit

    do i=1,ncol
       do k=1, pver
          zdelt(i,k)= zi(i,k)-zi(i,k+1)
       end do
    end do

    wavmid = 0.5_r8*(wavmin(1) + wavmax(1))*1000.

    call sp_aop_profile ( pver,    ncol,         ncol,                                 &
                          wavmid  ,oro(1:ncol),   clon_org(1:ncol)  ,clat_org(1:ncol)   , &
                          year_fr ,zm(1:ncol,pver:1:-1)      ,zdelt(1:ncol, pver:1:-1)  , &
                          dNovrN(1:ncol)               , aod_prof(1:ncol,pver:1:-1)            , &
                          ssa_prof(1:ncol,pver:1:-1)   , asy_prof(1:ncol,pver:1:-1)  )
    return
    end subroutine get_mac_cdnc 

#endif


end module wu_cdnc 

