#include <misc.h>
#include <params.h>

module cloudsimulator
!-----------------------------------------------------------------------
!Purpose: CAM interface to
!         Name:		ISCCP Simulator ICARUS/SCOPS
!         What:		Simulate ISCCP cloud products from GCM inputs
!         Version:	3.4
!         Authors:	Steve Klein (sak@gfdl.noaa.gov)
!         		Mark Webb (Mark.Webb@MetOffice.com)
!
!Author:  W. Lin,       Original version
!         J. Rosinski,  Modifications, March 2003
!         B. Eaton,     Update to simulator version 3.4, March 2004
!-----------------------------------------------------------------------

   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid,       only: pcols, pver
   use history,      only: addfld, add_default, phys_decomp, outfld, fillvalue
   use icarus_scops, only: ntau, npres

   USE mod_cosp_constants      ! wzz for COSP, 2009.4.3
   use comsrf,       only: landfrac
   use wv_saturation,  only: aqsat
   use mod_radar_simulator

   implicit none
   private
   save

!Public functions/subroutines

   public :: &
      cloudsimulator_init,    &
      cloudsimulator_run

   logical, public :: doisccp = .false.    ! whether to do ISCCP calcs and I/O     
   logical, public :: dolidar = .true.     ! whether to do CALIPSO Lidar calcs and I/O     
   logical, public :: doradar = .true.     ! whether to do CALIPSO Radar  calcs and I/O     
   logical, public :: domodis = .true.     ! whether to do MODIS calcs and I/O
   logical, public :: domisr  = .true.     ! whether to do MISR calcs and I/O

   integer,  parameter :: nsubcol = 20      ! # of columns each grid box is subdivided into
!
   integer :: lidar_ice_type = 0 !ice particle shape hypothesis in lidar calculations
                                 !(ice_type=0 for spheres, ice_type=1 for non spherical particles)
   logical :: use_vgrid,csat_vgrid,use_precipitation_fluxes,use_reff

   real(r8) :: radar_freq,k2,ZenAng,co2,ch4,n2o,co  ! from namelis (cosp_input_nl_new.txt) 
!
  integer,dimension(RTTOV_MAX_CHANNELS) :: Channels
  real,dimension(RTTOV_MAX_CHANNELS) :: Surfem
  integer :: overlap   !  overlap type: 1=max, 2=rand, 3=max/rand
  integer :: isccp_topheight,isccp_topheight_direction
  integer :: Ncolumns     ! Number of subcolumns in SCOPS
  integer :: Npoints      ! Number of gridpoints
  integer :: Nlevels      ! Number of levels
  integer :: Nlr=40       ! Number of levels in statistical outputs
  integer :: Npoints_it   ! Max number of gridpoints to be processed in one iteration
  integer :: surface_radar,use_mie_tables,use_gas_abs,do_ray,melt_lay
  integer :: Nprmts_max_hydro,Naero,Nprmts_max_aero
  integer :: platform,satellite,Instrument,Nchannels
!
  ! Summary statistics from radar
  TYPE COSP_VGRID
    logical :: use_vgrid ! Logical flag that indicates change of grid
    logical :: csat_vgrid ! Flag for Cloudsat grid
    integer :: Npoints   ! Number of sampled points
    integer :: Ncolumns  ! Number of subgrid columns
    integer :: Nlevels   ! Number of model levels
    integer :: Nlvgrid   ! Number of levels of new grid
    ! Array with dimensions (Nlvgrid)
    real, dimension(:), pointer :: z,zl,zu ! Height and lower and upper boundaries of new levels
    ! Array with dimensions (Nlevels)
    real, dimension(:), pointer :: mz,mzl,mzu ! Height and lower and upper boundaries of model levels
  END TYPE COSP_VGRID

  type(cosp_vgrid) :: vgrid
!
CONTAINS

subroutine cloudsimulator_init

   use icarus_scops, only: icarus_scops_init

!  character(len=64)  :: cosp_input_nl='cosp_input_nl.txt'
  character(len=64)  :: cosp_input_nl='cosp_input_nl_new.txt'
  character(len=512) :: finput ! Input file name
  character(len=512) :: cmor_nl

  integer :: i, j
!
  namelist /cosp_input/ cmor_nl,overlap,isccp_topheight,isccp_topheight_direction, &
              npoints,npoints_it,ncolumns,nlevels,use_vgrid,nlr,csat_vgrid,finput, &
              radar_freq,surface_radar,use_mie_tables, &
              use_gas_abs,do_ray,melt_lay,k2,Nprmts_max_hydro,Naero,Nprmts_max_aero, &
              lidar_ice_type,use_precipitation_fluxes,use_reff, &
              platform,satellite,Instrument,Nchannels, &
              Channels,Surfem,ZenAng,co2,ch4,n2o,co
   !------------------------------------------------------------------------------

!----------------------------------------------------------
!
! CALIPSO
!
    call addfld ('CLLCALIP','%',1,'A','Lidar Low-level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLMCALIP','%',1,'A','Lidar Mid-level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLHCALIP','%',1,'A','Lidar High-level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLTCALIP','%',1,'A','Lidar Total Cloud Fraction',phys_decomp,flag_xyfill=.true.)

    call addfld ('CLLICE  ','%',1,'A','CALIPSO Ice-Phase Low Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLLLIQ  ','%',1,'A','CALIPSO Liq-Phase Low Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLMICE  ','%',1,'A','CALIPSO Ice-Phase Mid Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLMLIQ  ','%',1,'A','CALIPSO Liq-Phase Mid Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLHICE  ','%',1,'A','CALIPSO Ice-Phase High Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLHLIQ  ','%',1,'A','CALIPSO Liq-Phase High Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLTICE  ','%',1,'A','CALIPSO Ice-Phase Total Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLTLIQ  ','%',1,'A','CALIPSO Liq-Phase Total Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLLUNDEF','%',1,'A','CALIPSO Undefined-Phase Low Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLMUNDEF','%',1,'A','CALIPSO Undefined-Phase Mid Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLHUNDEF','%',1,'A','CALIPSO Undefined-Phase High Level Cloud Fraction',phys_decomp,flag_xyfill=.true.)
    call addfld ('CLTUNDEF','%',1,'A','CALIPSO Undefined-Phase Total Cloud Fraction',phys_decomp,flag_xyfill=.true.)
!
! Calipso + Cloudsat
!
    call addfld ('CLCALIP2','1',pver,'A','CALIPSO Cloud Fraction Undetected by CloudSat',phys_decomp)
    call addfld ('TCLRADLI','%',1,'A','Lidar and Radar Total Cloud Fraction',phys_decomp,flag_xyfill=.true.)
!
! ISCCP
!
   call addfld ('SUNLIT  ','1',1,'A','1 for day points, 0 for nightime',phys_decomp,flag_xyfill=.true.)

   call addfld ('CLISCCP ','unitless',npres*ntau,'A', &
                'grid box fraction covered by each ISCCP D level cloud types',phys_decomp,&
                flag_xyfill=.true., flag_isccplev=.true.)
   call addfld ('TCLDAREA','fraction',1,'A','isccp_total_cloud_area_fraction',phys_decomp,flag_xyfill=.true.)
   call addfld ('MEANPTOP','Pa      ',1,'A','isccp_mean_cloud_top_pressure',phys_decomp,flag_xyfill=.true.)
   call addfld ('MEANTAU ','unitless',1,'A','ISCCP mean optical thickness',phys_decomp,flag_xyfill=.true.)
   call addfld ('MEANTTOP','K       ',1,'A','ISCCP mean cloud top temperature',phys_decomp,flag_xyfill=.true.)
   call addfld ('CLOUDY  ','fraction',1,'A','Binary flag for TCLDAREA > cldmin',phys_decomp,flag_xyfill=.true.)
   call addfld ('MEANALBE','unitless',1,'A','isccp_mean_cloud_albedo',phys_decomp,flag_xyfill=.true.)
   call addfld ('BOXPTOP ','mb      ',1,'A','ISCCP box cloud top pressure',phys_decomp,flag_xyfill=.true.)
   call addfld ('BOXTAU  ','unitless',1,'A','ISCCP box optical thickness',phys_decomp,flag_xyfill=.true.)

   call addfld ('MEANTB  ','K',1,'A','Mean all-sky 10.5 micron brightness temperature as calculated by the ISCCP Simulator', &
                  phys_decomp,flag_xyfill=.true.)
   call addfld ('MEANTBCL','K',1,'A','Mean clear-sky 10.5 micron brightness temperature as calculated by the ISCCP Simulator', &
                  phys_decomp,flag_xyfill=.true.)

   if ( dolidar ) then
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! Read namelist with COSP inputs
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     use_precipitation_fluxes=.true.
!
     open(10,file=cosp_input_nl,status='old')
     read(10,nml=cosp_input)
     close(10)

     use_vgrid = .false.

     call construct_cosp_vgrid(pcols,nsubcol,pver,Nlr,use_vgrid,csat_vgrid,vgrid)

     call addfld ('CLDTAU  ','unitless',pver,'A','BCC optical thickness',phys_decomp,flag_xyfill=.true.)

!
! add outputs from CALIPSO Lidar, wzz, 2009.4.8
!
!   if ( use_vgrid ) then  ! user level
!       call addfld ('CLCALIP ','unitless',Nlr,'A', &   ! 40 height levels ??
!                'grid box cloud fraction simulated by CALIPSO',phys_decomp,&
!                flag_xyfill=.true., namlev='alt')
!       call addfld ('CFADSR  ','1',Nlr,'A','Lidar Scattering Ratio CFAD (532 nm)',phys_decomp, &
!                flag_xyfill=.true., namlev='alt')
!       call addfld ('CLICE   ','%',Nlr,'A','Lidar Ice-Phase Cloud Fraction',phys_decomp,&
!                flag_xyfill=.true., namlev='alt')
!       call addfld ('CLLIQ   ','%',Nlr,'A','Lidar Liq-Phase Cloud Fraction',phys_decomp,&
!                flag_xyfill=.true., namlev='alt')
!       call addfld ('CLUNDEF ','%',Nlr,'A','Lidar Undef-Phase Cloud Fraction',phys_decomp, &
!                flag_xyfill=.true., namlev='alt')
!   else    ! model level
       call addfld ('CLCALIP ','%',pver,'A','Lidar Cloud Fraction (532 nm)',phys_decomp)
       call addfld ('CFADSR01','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 01',phys_decomp)
       call addfld ('CFADSR02','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 02',phys_decomp)
       call addfld ('CFADSR03','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 03',phys_decomp)
       call addfld ('CFADSR04','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 04',phys_decomp)
       call addfld ('CFADSR05','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 05',phys_decomp)
       call addfld ('CFADSR06','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 06',phys_decomp)
       call addfld ('CFADSR07','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 07',phys_decomp)
       call addfld ('CFADSR08','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 08',phys_decomp)
       call addfld ('CFADSR09','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 09',phys_decomp)
       call addfld ('CFADSR10','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 10',phys_decomp)
       call addfld ('CFADSR11','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 11',phys_decomp)
       call addfld ('CFADSR12','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 12',phys_decomp)
       call addfld ('CFADSR13','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 13',phys_decomp)
       call addfld ('CFADSR14','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 14',phys_decomp)
       call addfld ('CFADSR15','1',pver,'A','Lidar Scattering Ratio CFAD (532 nm) - bin 15',phys_decomp)
       call addfld ('CLICE   ','%',pver,'A','Lidar Ice-Phase Cloud Fraction',phys_decomp)
       call addfld ('CLLIQ   ','%',pver,'A','Lidar Liq-Phase Cloud Fraction',phys_decomp)
       call addfld ('CLUNDEF ','%',pver,'A','Lidar Undef-Phase Cloud Fraction',phys_decomp)
!   endif

   call addfld ('PARASOLC','unitless',PARASOL_NREFL,'A','calipso_parasol_reflectance',phys_decomp,  &
                 flag_xyfill=.true., namlev='sza5')
!
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   endif

   call icarus_scops_init
!----------------------------------------------------------------------------
! RADAR initialzation
!
    !
    ! set up Temperature bin structure used for z scaling
    !
    do i=1,cnt_ice
        mt_tti(i)=(i-1)*5-90 + 273.15
    enddo

    do i=1,cnt_liq
        mt_ttl(i)=(i-1)*5-60 + 273.15
    enddo


   ! define array discrete diameters used in mie calculations
   !
    delt = (log(dmax)-log(dmin))/(nd-1)
    deltp = exp(delt)

    D(1) = dmin
    do i=2,nd
       D(i) = D(i-1)*deltp
    enddo

  ! set up Re bin "structure" for z_scaling
  !
    base_list(1)=0;
    do j=1,Re_MAX_BIN
        step_list(j)=0.1+0.1*((j-1)**1.5);
        if(step_list(j)>Re_BIN_LENGTH) then
           step_list(j)=Re_BIN_LENGTH;
        endif
        if(j>1) then
            base_list(j) =base_list(j-1)+floor(Re_BIN_LENGTH/step_list(j-1));
        endif
    enddo

   Z_scale_flag(:,:,:) = .false.
   Ze_scaled(:,:,:)    = 0.0
   Zr_scaled(:,:,:)    = 0.0
   kr_scaled(:,:,:)    = 0.0   
   rho_eff(:,:,:)      = R_UNDEF

!-----------------------------------------------------------------------------
end subroutine cloudsimulator_init

subroutine cloudsimulator_run(state, ts, concld, cld, cliqwp, &
                              cicewp, rel, rei, emis, coszrs, CDNC  ) 

   use physics_types,   only: physics_state
   use constituents,    only: cnst_get_ind
   use icarus_scops,    only: isccp_cloud_types
   use mod_lmd_ipsl_stats,     only: diag_lidar
   use mod_lmd_ipsl_stats_new, only: diag_lidar_new
   use moistconvection,    only: rgrav

   use modis_simulator, only: modis_L2_simulator, modis_L3_simulator
   use modis_simulator, only: numModisTauBins      => numTauHistogramBins,      &
                              numModisPressureBins => numPressureHistogramBins

   type(physics_state), intent(in) :: state

   real(r8), intent(in) :: ts(pcols)             ! skin temperature
   real(r8), intent(in) :: concld(pcols,pver)  ! convective cloud cover
   real(r8), intent(in) :: cld(pcols,pver)     ! cloud cover
   real(r8), intent(in) :: cliqwp(pcols,pver)  ! in-cloud liquid water path
   real(r8), intent(in) :: cicewp(pcols,pver)  ! in-cloud ice water path
   real(r8), intent(in) :: rel(pcols,pver)     ! Liquid cloud particle effective radius
   real(r8), intent(in) :: rei(pcols,pver)     ! Ice effective drop size (microns)
   real(r8), intent(in) :: emis(pcols,pver)    ! Cloud longwave emissivity
   real(r8), intent(in) :: coszrs(pcols)         ! cosine solar zenith angle (to tell if day or night)
   real(r8), intent(in) :: CDNC(pcols,pver)     ! Liquid cloud droplet number concentration  (1/cm3)

! Local variables:

   integer, parameter :: debug = 0     ! set to non-zero value to print out inputs
                                       ! with step debug
   integer, parameter :: debugcol = 0  ! set to non-zero value to print out column
                                       ! decomposition with step debugcol

   integer, parameter :: top_height = 1 ! 1 = adjust top height using both a computed
                                        ! infrared brightness temperature and the visible
					! optical depth to adjust cloud top pressure. Note
					! that this calculation is most appropriate to compare
					! to ISCCP data during sunlit hours.
   INTEGER, parameter :: top_height_direction = 2 ! direction for finding atmosphere pressure level
                                ! with interpolated temperature equal to the radiance
                                ! determined cloud-top temperature
                                !
                                ! 1 = find the *lowest* altitude (highest pressure) level
                                ! with interpolated temperature equal to the radiance
                                ! determined cloud-top temperature
                                !
                                ! 2 = find the *highest* altitude (lowest pressure) level
                                ! with interpolated temperature equal to the radiance
                                ! determined cloud-top temperature
                                !
                                ! ONLY APPLICABLE IF top_height EQUALS 1 or 3
                                !
                                ! 1 = old setting: matches all versions of
                                ! ISCCP simulator with versions numbers 3.5.1 and lower
                                !
                                ! 2 = default setting: for version numbers 4.0 and higher
   integer, parameter :: overlap = 3    ! 3=max/rand

!  integer,  parameter :: nsubcol = 50      ! # of columns each grid box is subdivided into
   real(r8), parameter :: emsfc_lw = 0.99   ! longwave emissivity of surface at 10.5 microns

! constants for optical depth calculation from radcswmx.F90
   real(r8), parameter :: abarl = 2.817e-02    ! A coefficient for extinction optical depth
   real(r8), parameter :: bbarl = 1.305        ! b coefficient for extinction optical depth
   real(r8), parameter :: abari = 3.448e-03    ! A coefficient for extinction optical depth
   real(r8), parameter :: bbari = 2.431        ! b coefficient for extinction optical depth
   real(r8), parameter :: cldmin = 1.0e-80_r8  ! note: cldmin much less than cldmin from cldnrh
   real(r8), parameter :: cldeps = 0.0_r8      ! 

   integer :: lchnk                         ! chunk identifier
   integer :: ncol                          ! number of atmospheric columns

   integer  :: sunlit(pcols)                ! 1 for day points, 0 for night time
   integer  :: seed1(pcols)                  ! seed values for random number generator
   integer  :: seed2(pcols)                  ! seed values for random number generator
   integer  :: seed3(pcols)                  ! seed values for random number generator
   integer  :: seed4(pcols)                  ! seed values for random number generator

   real(r8) :: tau(pcols,pver)              ! optical depth
   real(r8) :: tau_s(pcols,pver)            ! optical depth of stratiform clouds
   real(r8) :: tau_c(pcols,pver)            ! optical depth of convective clouds

   real(r8) :: fq_isccp(pcols,ntau,npres)   ! the fraction of the model grid box covered by
                                            ! each of the 49 ISCCP D level cloud types
   real(r8) :: totalcldarea(pcols)          ! the fraction of model grid box columns
                                            ! with cloud somewhere in them.  This should
					    ! equal the sum over all entries of fq_isccp
   real(r8) :: meanptop(pcols)              ! mean cloud top pressure (mb) - linear averaging
                                            ! in cloud top pressure.
   real(r8) :: meanttop(pcols)              ! mean cloud top temp (k) - linear averaging

   real(r8) :: meantaucld(pcols)            ! mean optical thickness 
                                            ! linear averaging in albedo performed.

   real(r8) :: meanalbedocld(pcols) 

   Real(r8) :: meantb(pcols)                ! mean all-sky 10.5 micron brightness temperature

   Real(r8) :: meantbclr(pcols)             ! mean clear-sky 10.5 micron brightness temperature

   real(r8) :: boxtau(pcols,nsubcol)        ! optical thickness in each column
      
   real(r8) :: boxptop(pcols,nsubcol)       ! cloud top pressure (mb) in each column

   real(r8) :: fq_isccp_s1(pcols,ntau*npres)! accumulated fq_isccp

   integer :: i, k, it, ip, itaunpres       ! indices

   real(r8) :: cloudy(pcols)   ! cloudy flag, which may be used to derived mean values under cloudy
        		       ! conditions. The cloud flag itself is freuency of cloudy condition
	 		       ! when average over an accumulation period.
   real(r8) :: land(pcols)     !Landmask [0 - Ocean, 1 - Land]

   real(r8), target  :: frac_out(pcols,nsubcol,pver)
!
   real(r8),dimension(pcols, pver+1) :: presf
   real(r8),dimension(pcols, pver) :: mr_ll,mr_li,mr_cl,mr_ci
   real(r8),dimension(pcols, pver) :: ybeta_mol,beta_tot,tau_tot,dep_ratio
   real(r8),dimension(pcols, PARASOL_NREFL)  :: refle
   real(r8),dimension(pcols, nsubcol, pver) :: ybeta_tot,ytau_tot,ydep_ratio

   real(r8),dimension(pcols, nsubcol, pver) :: ybetaperp_tot    ! wtw

   real(r8),dimension(pcols, nsubcol, PARASOL_NREFL)  :: yrefl
   real(r8),dimension(pcols, nsubcol, pver, N_hydro)  :: mr_hydro
   real(r8),dimension(pcols, nsubcol, pver, N_hydro)  :: Reff
   real(r8),dimension(SR_BINS)                     ::  srbval
!  real(r8),dimension(pcols, pver)                 ::  lidarcld
   real(r8),dimension(pcols, LIDAR_NCAT)           ::  cldlayer
   real(r8),dimension(pcols, PARASOL_NREFL)        ::  parasolrefl
   real(r8),dimension(pcols, SR_BINS, pver)        ::  cfad_sr
   real(r8),dimension(pcols)                       ::  zs     ! surface height(m)
   real(r8),dimension(pcols, pver)                 ::  zfull, zhalf

   real(r8),dimension(:,:),allocatable             ::  lidarcld
   real(r8),dimension(:,:),allocatable :: frac_ls,frac_cv ! Cloud fraction in each model level
   real(r8),dimension(:,:),pointer :: column_frac_out ! Array with one column of frac_out

   logical :: ok_lidar_cfad = .false.
   integer :: ixcldice, ixcldliq              ! constituent indices for cloud liquid and ice water.
   integer :: Nlr          ! Number of levels in statistical outputs
!  integer :: Nlv
   real,dimension(:,:,:),allocatable :: Ze_out,betatot_out,betamol_in,betamol_out,ph_in,ph_out
   real,dimension(:,:),allocatable :: ph_c,betamol_c

!---------------
! wtw
   real(r8) :: betaperp_tot(pcols,pver)   ! perpendicular lidar backscattered signal power
   integer,parameter :: nphase = 6
             ! nb of cloud layer phase types (ice,liquid,undefined,false ice,false liquid,Percent of ice)
   real lidarcldphase(pcols, pver, nphase)   ! 3D "lidar" phase cloud fraction
   real lidarcldtmp  (pcols,40,5)             ! 3D "lidar" phase cloud fraction as a function of temp
   real cldlayerphase(pcols,LIDAR_NCAT,nphase)    ! "lidar" phase low mid high cloud fraction

   real,dimension(:,:,:),allocatable :: temp_out, temp_in, betaperptot_out    ! wtw
   real,dimension(:,:),allocatable   :: temp_c               ! wtw

   real temp_tot(pcols, pver)
   real xdata_r(pcols,pver)

!------------------------------------
! radar

   real(r8) ::  Ze_non  (pcols,pver)  ! radar reflectivity withOUT attenuation (dBZ)
   real(r8) ::  Ze_ray  (pcols,pver)  ! Rayleigh reflectivity (dBZ)
   real(r8) ::  h_atten_to_vol(pcols,pver)       ! attenuation by hydromets, radar to vol (dB)
   real(r8) ::  g_atten_to_vol(pcols,pver)       ! gaseous atteunation, radar to vol (dB)
   real(r8) ::  dBZe    (pcols,pver)             ! effective radar reflectivity factor (dBZ)
   real(r8) ::  g_to_vol(pcols,pver)             !  Optional argument. It is computed and returned in the first call to
                                                ! radar_simulator, and passed as input in the rest
   real(r8) ::  Ze_tot(pcols,nsubcol,pver)       ! radar reflectivity
   real(r8) ::  es(pcols,pver), qs(pcols,pver)
   real(r8) ::  p_matrix(pcols,pver)
   real(r8) ::  hgt_matrix(pcols,pver)
   real(r8) ::  t_matrix(pcols,pver)
   real(r8) ::  rh_matrix(pcols,pver)

   real(r8) ::  hm_matrix(N_hydro,pcols,pver)
   real(r8) ::  re_matrix(N_hydro,pcols,pver)
   real(r8) ::  Np_matrix(N_hydro,pcols,pver)
   integer  ::  pr
   real(r8) ::  airdensity(pcols,pver)   ! kg/m3 
!---------------------------------
! wtw MODIS   
!
   real(r8) :: pressureLevels(pcols, pver+1)
   real(r8) :: pressureLayers(pcols, pver  )
   real(r8) :: temperature   (pcols, pver  )
   real(r8) :: opticalThickness(pcols, nsubcol, pver)
   real(r8) :: cloudWater      (pcols, nsubcol, pver)
   real(r8) :: cloudIce        (pcols, nsubcol, pver)
   real(r8) :: waterSize       (pcols, nsubcol, pver)
   real(r8) :: iceSize         (pcols, nsubcol, pver)
   real(r8) :: isccpTau        (pcols, nsubcol)
   real(r8) :: isccpCloudTopPressure  (pcols, nsubcol)
   integer  :: retrievedPhase         (pcols,nsubcol)    ! liquid/ice/other - integer, defined in module header
   real(r8) :: retrievedCloudTopPressure(pcols,nsubcol)  ! units of pressureLayers
   real(r8) :: retrievedTau             (pcols,nsubcol)  ! unitless
   real(r8) :: retrievedSize            (pcols,nsubcol)            ! microns

   real(r8) :: cfTotal    (pcols) , Cloud_Fraction_Total_Mean (pcols)
   real(r8) :: cfLiquid   (pcols) , Cloud_Fraction_Water_Mean (pcols)
   real(r8) :: cfIce      (pcols) , Cloud_Fraction_Ice_Mean   (pcols)
   real(r8) :: cfHigh     (pcols) , Cloud_Fraction_High_Mean  (pcols)
   real(r8) :: cfMid      (pcols) , Cloud_Fraction_Mid_Mean   (pcols)
   real(r8) :: cfLow      (pcols) , Cloud_Fraction_Low_Mean   (pcols)
   real(r8) :: meanTauTotal (pcols) , Optical_Thickness_Total_Mean (pcols)
   real(r8) :: meanTauLiquid(pcols) , Optical_Thickness_Water_Mean (pcols)
   real(r8) :: meanTauIce   (pcols) , Optical_Thickness_Ice_Mean   (pcols)
   real(r8) :: meanLogTauTotal (pcols)  , Optical_Thickness_Total_LogMean (pcols)
   real(r8) :: meanLogTauLiquid(pcols)  , Optical_Thickness_Water_LogMean (pcols)
   real(r8) :: meanLogTauIce   (pcols)  , Optical_Thickness_Ice_LogMean   (pcols)
   real(r8) :: meanSizeLiquid  (pcols)  , Cloud_Particle_Size_Water_Mean  (pcols)
   real(r8) :: meanSizeIce     (pcols)  , Cloud_Particle_Size_Ice_Mean    (pcols)
   real(r8) :: meanCloudTopPressure   (pcols) , Cloud_Top_Pressure_Total_Mean (pcols)
   real(r8) :: meanLiquidWaterPath    (pcols) , Liquid_Water_Path_Mean  (pcols)
   real(r8) :: meanIceWaterPath       (pcols) , Ice_Water_Path_Mean     (pcols)
   real(r8) :: jointHistogram  (pcols, numModisTauBins, numModisPressureBins )
                  ! numTauHistogramBins = 6, numPressureHistogramBins = 7

   real(r8) :: Optical_Thickness_vs_Cloud_Top_Pressure (pcols, numModisTauBins+1, numModisPressureBins )

   integer  :: nsunlit, isunlit, jsunlit, j, i1, i2

!------------------
! MISR simulator
!
   real(r8) :: fq_MISR                   (pcols, 7, n_MISR_CTH)
   real(r8) :: MISR_dist_model_layertops (pcols, n_MISR_CTH)
   real(r8) :: MISR_meanztop             (pcols)           
   real(r8) :: MISR_cldarea              (pcols)

!------------------------------------------------------------------------------------------------

   call t_startf ('cloudsimulator_run')

   lchnk = state%lchnk
   ncol  = state%ncol

   ! compute the optical depth (code from radcswmx)
   do k=1,pver
      do i=1,ncol
         if(cld(i,k) >= cldmin .and. cld(i,k) >= cldeps) then
            tau(i,k) = (abarl + bbarl/rel(i,k)) * cliqwp(i,k) + &
                       (abari + bbari/rei(i,k)) * cicewp(i,k)
         else
            tau(i,k) = 0.0
         endif
      end do
   end do

   tau_s(:,:) = tau(:,:)   ! wtw 2017.2.24
   tau_c(:,:) = tau(:,:)   ! wtw 2017.2.24

   call outfld('CLDTAU  ',tau      ,pcols,lchnk)    ! wzz, 2009.6.2

   sunlit = 0
   do i=1,ncol
      if (coszrs(i) > 0.) sunlit(i) = 1
      seed1(i) = (state%pmid(i,pver) - int(state%pmid(i,pver)))      * 1000000000
      seed2(i) = (state%pmid(i,pver-1) - int(state%pmid(i,pver-1)))  * 1000000000
      seed3(i) = (state%pmid(i,pver-2) - int(state%pmid(i,pver-2)))  * 1000000000
      seed4(i) = (state%pmid(i,pver-3) - int(state%pmid(i,pver-3)))  * 1000000000
   end do

   presf(:ncol,:) = state%pint(:ncol,:)
   presf(:,1) = 0.0

   call ISCCP_CLOUD_TYPES( &
      debug,               &
      debugcol,            &
      ncol,                &
      sunlit(:ncol),       &
      pver,                &
      nsubcol,             &
      seed1(:ncol),        &
      seed2(:ncol),        &
      seed3(:ncol),        &
      seed4(:ncol),        &
      state%pmid(:ncol,:), &
      presf(:ncol,:),      &
      state%q(:ncol,:,1),  &
      cld(:ncol,:),        &
      concld(:ncol,:),     &
      tau(:ncol,:),        &
      tau(:ncol,:),        &
      top_height,          &
      top_height_direction,&
      overlap,             &
      frac_out(:ncol,:,:), &
      ts(:ncol),           &
      emsfc_lw,            &
      state%t(:ncol,:),    &
      emis(:ncol,:),       &
      emis(:ncol,:),       &
      fq_isccp(:ncol,:,:), &
      totalcldarea(:ncol), &
      meanptop(:ncol),     &
      meanttop(:ncol),     &
      meantaucld(:ncol),   &
      meanalbedocld(:ncol),&
      meantb(:ncol),       &
      meantbclr(:ncol),    &
      boxtau(:ncol,:),     &
      boxptop(:ncol,:)     )
            
   do i=1,ncol
      if (coszrs(i) > 0.) then
         if (totalcldarea(i) >= cldmin) then
            cloudy(i) = 1.0

            ! save standard ISCCP type of 7x7 clouds
            do ip=1,npres
               do it=1,ntau
                  itaunpres = (ip-1)*ntau+it
                  fq_isccp_s1(i,itaunpres) = fq_isccp(i,it,ip)
               end do
            end do
            
         else                             !cloud free in the (daytime) grid box
            fq_isccp_s1(i,:) = 0.
            totalcldarea(i)  = 0.
            meanptop(i)      = fillvalue
            meantaucld(i)    = fillvalue
            meanttop(i)      = fillvalue
            cloudy(i)        = 0.
         endif
      else                                ! nighttime
         fq_isccp_s1(i,:) = fillvalue
         totalcldarea(i)  = fillvalue
         meanptop(i)      = fillvalue
         meantaucld(i)    = fillvalue
         meanttop(i)      = fillvalue
         cloudy(i)        = fillvalue
         meantbclr(i)     = fillvalue
         meantb(i)        = fillvalue
         meanalbedocld(i) = fillvalue
      end if
   end do
!
! dont need to call outfld if all points are nighttime
!
      call outfld('SUNLIT',coszrs, pcols, lchnk)

      call outfld('CLISCCP ',fq_isccp_s1, pcols,lchnk)
      call outfld('TCLDAREA',totalcldarea,pcols,lchnk)
      call outfld('MEANPTOP',meanptop    ,pcols,lchnk)
      call outfld('MEANTAU ',meantaucld  ,pcols,lchnk)
      call outfld('MEANTTOP',meanttop    ,pcols,lchnk)
      call outfld('CLOUDY  ',cloudy      ,pcols,lchnk)
      call outfld('MEANALBE',meanalbedocld,pcols,lchnk)   ! wzz, 2009.4.2
      call outfld('BOXPTOP ',boxptop     ,pcols,lchnk)    ! wzz, 2009.4.2
      call outfld('BOXTAU  ',boxtau      ,pcols,lchnk)    ! wzz, 2009.4.2

      call outfld('MEANTB',meantb, pcols, lchnk)
      call outfld('MEANTBCL',meantbclr, pcols, lchnk)
!---------------------------------------------------------------------------------
!
! Calipso part
!
     if ( dolidar ) then
!
! Levels from SURFACE to TOA.
!

        zs(:ncol) = state%phis(:ncol)*rgrav
        zfull(:ncol,1:pver) = state%zm(:ncol, pver:1:-1)
        zhalf(:ncol,1:pver) = state%zi(:ncol, pver+1:2:-1)
        do k = 1, pver
           zfull(:ncol,k) = zfull(:ncol,k) + zs(:ncol)
           zhalf(:ncol,k) = zhalf(:ncol,k) + zs(:ncol)
        enddo
         
        frac_out(:ncol,:,1:pver) = frac_out(:ncol,:,pver:1:-1)
!
        allocate(frac_ls(ncol,pver),frac_cv(ncol,pver))
        frac_ls=0.0
        frac_cv=0.0
!
        do k = 1,pver
        do i = 1, ncol
           do it = 1, nsubcol
              if(frac_out(i,it,k)==1) frac_ls(i,k) = frac_ls(i,k) + 1.
              if(frac_out(i,it,k)==2) frac_cv(i,k) = frac_cv(i,k) + 1.
           enddo
           frac_ls(i,k) = frac_ls(i,k)/nsubcol
           frac_cv(i,k) = frac_cv(i,k)/nsubcol
        enddo
        enddo
!
     mr_hydro(:,:,:,:) = 0.0
     reff(:,:,:,:) = 0.0
     presf(:ncol,1:pver) = state%pint(:ncol,pver+1:2:-1)
     presf(:,pver+1) = 0.0
!
     call cnst_get_ind('CLDLIQ', ixcldliq)
     call cnst_get_ind('CLDICE', ixcldice)

     do i = 1, nsubcol 
            !--------- Mixing ratios for clouds and Reff for Clouds and precip -------
            column_frac_out => frac_out(:,i,:)
            where (column_frac_out == 1)     !+++++++++++ LS clouds ++++++++
                mr_hydro(:,i,1:pver,I_LSCLIQ) = state%q(:,pver:1:-1,ixcldliq)
                mr_hydro(:,i,1:pver,I_LSCICE) = state%q(:,pver:1:-1,ixcldice)

                Reff(:,i,:,I_LSCLIQ)     = rel(:,pver:1:-1)*1.0e-6
                Reff(:,i,:,I_LSCICE)     = rei(:,pver:1:-1)*1.0e-6
            elsewhere (column_frac_out == 2) !+++++++++++ CONV clouds ++++++++
                Reff(:,i,:,I_CVCLIQ)     = rel(:,pver:1:-1)*1.0e-6   !??
                Reff(:,i,:,I_CVCICE)     = rei(:,pver:1:-1)*1.0e-6   !??
            end where
            !--------- Precip -------
            if (.not. use_precipitation_fluxes) then
                where (column_frac_out == 1)  !+++++++++++ LS Precipitation ++++++++
!                   mr_hydro(:,i,:,I_LSRAIN) = gbx%mr_hydro(:,:,I_LSRAIN)
!                   mr_hydro(:,i,:,I_LSSNOW) = gbx%mr_hydro(:,:,I_LSSNOW)
!                   mr_hydro(:,i,:,I_LSGRPL) = gbx%mr_hydro(:,:,I_LSGRPL)
                elsewhere (column_frac_out == 2) !+++++++++++ CONV Precipitation ++++++++
!                   mr_hydro(:,i,:,I_CVRAIN) = gbx%mr_hydro(:,:,I_CVRAIN)
!                   mr_hydro(:,i,:,I_CVSNOW) = gbx%mr_hydro(:,:,I_CVSNOW)
                end where
            endif
     enddo
        ! convert the mixing ratio and precipitation flux from gridbox mean to the fraction-based values
     do k=1,pver
        do i = 1,ncol
             !--------- Clouds -------
             if (frac_ls(i,k) .ne. 0.) then
                 mr_hydro(i,:,k,I_LSCLIQ) = mr_hydro(i,:,k,I_LSCLIQ)/frac_ls(i,k)
                 mr_hydro(i,:,k,I_LSCICE) = mr_hydro(i,:,k,I_LSCICE)/frac_ls(i,k)
             endif
             if (frac_cv(i,k) .ne. 0.) then
                 mr_hydro(i,:,k,I_CVCLIQ) = mr_hydro(i,:,k,I_CVCLIQ)/frac_cv(i,k)
                 mr_hydro(i,:,k,I_CVCICE) = mr_hydro(i,:,k,I_CVCICE)/frac_cv(i,k)
             endif
        enddo
     enddo
     deallocate (frac_ls,frac_cv)
!
     do i = 1, nsubcol 
      ! Temporary arrays for simulator call
      mr_ll(:,:) = mr_hydro(:,i,:,I_LSCLIQ)
      mr_li(:,:) = mr_hydro(:,i,:,I_LSCICE)
      mr_cl(:,:) = mr_hydro(:,i,:,I_CVCLIQ)
      mr_ci(:,:) = mr_hydro(:,i,:,I_CVCICE)

      call lidar_simulator_new (ncol, pver, 4 &
            , PARASOL_NREFL, LIDAR_UNDEF  &
            , state%pmid(:ncol,pver:1:-1), presf(:ncol,:), state%t(:ncol,pver:1:-1) &
            , mr_ll(:ncol,:), mr_li(:ncol,:), mr_cl(:ncol,:), mr_ci(:ncol,:) &
            , Reff(:ncol,i,:,I_LSCLIQ), Reff(:ncol,i,:,I_LSCICE)  &
            , Reff(:ncol,i,:,I_CVCLIQ), Reff(:ncol,i,:,I_CVCICE)  &
            , lidar_ice_type, ybeta_mol(:ncol,:), beta_tot(:ncol,:), betaperp_tot(:ncol,:) &
            , tau_tot(:ncol,:), refle(:ncol,:) ) ! reflectance


      ybetaperp_tot(:ncol,i,:) = betaperp_tot(:ncol,:)

      ybeta_tot(:ncol,i,:) = beta_tot(:ncol,:)
      ytau_tot(:ncol,i,:)  = tau_tot(:ncol,:)
      ydep_ratio(:ncol,i,:)= dep_ratio(:ncol,:)
      yrefl(:ncol,i,:)     = refle(:ncol,:)
    enddo
    ok_lidar_cfad=.true.
!
! Statistics in model levels
!
   land(:) = landfrac(:,lchnk)
   where(land>0.5)
      land = 1.0
   elsewhere
      land = 0.0
   end where

   if (vgrid%use_vgrid) then ! Statistics in a different vertical grid
        Nlr = vgrid%Nlvgrid
        allocate(Ze_out(pcols,Nsubcol,Nlr),betatot_out(pcols,Nsubcol,Nlr), &
                 betamol_in(pcols,1,pver),betamol_out(pcols,1,Nlr),betamol_c(pcols,Nlr), &
                 ph_in(pcols,1,pver),ph_out(pcols,1,Nlr),ph_c(pcols,Nlr))

        allocate(betaperptot_out(pcols,Nsubcol,Nlr), temp_in(pcols,1,pver),temp_out(pcols,1,Nlr))      ! wtw
        allocate(temp_c(pcols,Nlr))          ! wtw

        betaperptot_out = 0.0                !wtw
        temp_in  = 0.0                       !wtw
        temp_out = 0.0                       !wtw
        temp_c   = 0.0                       ! wtw


        Ze_out = 0.0
        betatot_out  = 0.0
        betamol_in(:,1,:) = ybeta_mol(:,:)
        betamol_out= 0.0
        betamol_c  = 0.0
        ph_in(:ncol,1,:)  = presf(:ncol,1:pver)    ! from bottom to top 
        ph_out  = 0.0
        ph_c    = 0.0

        allocate(lidarcld(pcols,Nlr))
        lidarcld(:,:) = 0.
!
        call cosp_change_vertical_grid(pcols,1,pver, zfull, zhalf, betamol_in, &
                                       Nlr,vgrid%zl,vgrid%zu,betamol_out)
        call cosp_change_vertical_grid(pcols,Nsubcol,pver, zfull, zhalf,ybeta_tot, &
                                       Nlr,vgrid%zl,vgrid%zu,betatot_out)
        call cosp_change_vertical_grid(pcols,1,pver, zfull, zhalf,ph_in, &
                                       Nlr,vgrid%zl,vgrid%zu,ph_out)
!---------------
! wtw
        temp_in(:ncol,1,:pver) = state%t(:ncol, pver:1:-1)

        call cosp_change_vertical_grid(pcols,Nsubcol,pver, zfull, zhalf,betaperp_tot , &
                                       Nlr,vgrid%zl,vgrid%zu,betaperptot_out)
        call cosp_change_vertical_grid(pcols,1,pver, zfull, zhalf,temp_in , &
                                       Nlr,vgrid%zl,vgrid%zu,temp_out)

        betamol_c(:,:) = betamol_out(:,1,:)
        temp_c(:,:) = temp_out(:,1,:)

!----------------

        ph_c(:,:) = ph_out(:,1,:)
        betamol_c(:,:) = betamol_out(:,1,:)
        ! Stats from lidar_stat_summary

        call diag_lidar_new(ncol,Nsubcol,Nlr,SR_BINS,PARASOL_NREFL &
                        ,temp_c(:ncol,:) &                                          ! wtw
                        ,betatot_out(:ncol,:,:) &
                        ,betaperptot_out(:ncol,:,:) &                               ! wtw
                        ,betamol_c(:ncol,:),yrefl(:ncol,:,:),land(:ncol) &
                        ,ph_c(:ncol,:), LIDAR_UNDEF,ok_lidar_cfad &
                        ,cfad_sr(:ncol,:,:),srbval &
                        ,LIDAR_NCAT,lidarcld(:ncol,:) &
                        ,lidarcldphase(:ncol,:,:) &                                   ! wtw
                        ,cldlayer(:ncol,:)&
                        ,cldlayerphase(:ncol,:,:)  &                                  !wtw
                        ,lidarcldtmp(:ncol,:,:)    &                                  !wtw
                        ,parasolrefl(:ncol,:))

        deallocate(Ze_out,betatot_out,betamol_in,betamol_out,betamol_c,ph_in,ph_out,ph_c)
        deallocate(temp_out, temp_c, betaperptot_out)     ! wtw

   else  ! Statistics in model levels

      allocate(lidarcld(pcols,pver))
      lidarcld(:,:) = 0.

!      call diag_lidar(ncol,nsubcol,pver,SR_BINS,PARASOL_NREFL &
!                        ,ybeta_tot(:ncol,:,:),ybeta_mol(:ncol,:),yrefl(:ncol,:,:),land(:ncol)  &
!                        ,presf(:ncol,1:pver), LIDAR_UNDEF,ok_lidar_cfad &
!                        ,cfad_sr(:ncol,:,:),srbval &
!                        ,LIDAR_NCAT,lidarcld(:ncol,:),cldlayer(:ncol,:),parasolrefl(:ncol,:))
!-----

      temp_tot(:ncol,:pver) = state%t(:ncol, pver:1:-1)
      lidarcldphase(:ncol,:pver,:) = 0.0
      cldlayerphase(:,:,:) = 0.0
      lidarcldtmp(:,:,:)   = 0.0

      call diag_lidar_new(ncol,nsubcol,pver,SR_BINS,PARASOL_NREFL  &
                        ,temp_tot(:ncol,:) &                                          ! wtw
                          ,ybeta_tot(:ncol,:,:) &
                      ,ybetaperp_tot(:ncol,:,:) &                               ! wtw
                        ,ybeta_mol(:ncol,:),yrefl(:ncol,:,:),land(:ncol)  &
                        ,presf(:ncol,1:pver), LIDAR_UNDEF,ok_lidar_cfad &
                        ,cfad_sr(:ncol,:,:),srbval &
                        ,LIDAR_NCAT,lidarcld(:ncol,:)&
                      ,lidarcldphase(:ncol,:,:) &                                   ! wtw
                        ,cldlayer(:ncol,:) &
                      ,cldlayerphase(:ncol,:,:)  &                                  !wtw
                      ,lidarcldtmp(:ncol,:,:)    &                                  !wtw
                        ,parasolrefl(:ncol,:))
   endif

   where (cfad_sr   == LIDAR_UNDEF) cfad_sr   = fillvalue
   where (lidarcld  == LIDAR_UNDEF) lidarcld  = 0.0
!  where (lidarcld  == LIDAR_UNDEF) lidarcld  = fillvalue
   where (cldlayer  == LIDAR_UNDEF) cldlayer  = fillvalue
   where (parasolrefl == LIDAR_UNDEF) parasolrefl = fillvalue

   where (ybeta_mol == LIDAR_UNDEF) ybeta_mol = fillvalue
   where (lidarcldtmp == LIDAR_UNDEF) lidarcldtmp = fillvalue
   where (cldlayerphase == LIDAR_UNDEF) cldlayerphase = fillvalue
   where (lidarcldphase == LIDAR_UNDEF) lidarcldphase = 0.0
   where (lidarcldtmp == LIDAR_UNDEF) lidarcldtmp = fillvalue
   where (cfad_sr == LIDAR_UNDEF) cfad_sr = 0.0

  !+++++++++++ change of units after computation of statistics +++++++++++
   where((ybeta_tot > 0.0) .and. (ybeta_tot /= R_UNDEF))
       ybeta_tot = log10(ybeta_tot)
   elsewhere
       ybeta_tot = fillvalue
   end where

!------------------
   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,1,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR01',xdata_r,pcols,lchnk) 

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,2,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR02',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,3,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR03',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,4,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR04',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,5,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR05',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,6,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR06',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,7,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR07',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,8,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR08',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,9,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR09',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,10,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR10',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,11,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR11',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,12,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR12',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,13,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR13',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,14,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR14',xdata_r,pcols,lchnk)

   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = cfad_sr(i,15,pver-k+1)
      enddo
   enddo
   call outfld('CFADSR15',xdata_r,pcols,lchnk)

!------------------------------------------

   call outfld('CLLCALIP',cldlayer(:,1), pcols,lchnk)
   call outfld('CLMCALIP',cldlayer(:,2), pcols,lchnk)
   call outfld('CLHCALIP',cldlayer(:,3), pcols,lchnk)
   call outfld('CLTCALIP',cldlayer(:,4), pcols,lchnk)
   call outfld('PARASOLC',parasolrefl,   pcols,lchnk)
   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = lidarcld(i,pver-k+1)
      enddo
   enddo
   call outfld('CLCALIP ',xdata_r(:,:), pcols,lchnk)

   call outfld('CLLICE',cldlayerphase(:,1,1), pcols,lchnk)
   call outfld('CLHICE',cldlayerphase(:,3,1), pcols,lchnk)
   call outfld('CLMICE',cldlayerphase(:,2,1), pcols,lchnk)
   call outfld('CLTICE',cldlayerphase(:,4,1), pcols,lchnk)
   call outfld('CLHICE',cldlayerphase(:,3,1), pcols,lchnk)
   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = lidarcldphase(i,pver-k+1,1)
      enddo   
   enddo 
   call outfld('CLICE ',xdata_r, pcols,lchnk)

   call outfld('CLLLIQ',cldlayerphase(:,1,2), pcols,lchnk)
   call outfld('CLHLIQ',cldlayerphase(:,3,2), pcols,lchnk)
   call outfld('CLMLIQ',cldlayerphase(:,2,2), pcols,lchnk)
   call outfld('CLTLIQ',cldlayerphase(:,4,2), pcols,lchnk)
   call outfld('CLHLIQ',cldlayerphase(:,3,2), pcols,lchnk)
   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = lidarcldphase(i,pver-k+1,2)
      enddo
   enddo
   call outfld('CLLIQ ',xdata_r, pcols,lchnk)

   call outfld('CLLUNDEF',cldlayerphase(:,1,3), pcols,lchnk)
   call outfld('CLHUNDEF',cldlayerphase(:,3,3), pcols,lchnk)
   call outfld('CLMUNDEF',cldlayerphase(:,2,3), pcols,lchnk)
   call outfld('CLTUNDEF',cldlayerphase(:,4,3), pcols,lchnk)
   call outfld('CLHUNDEF',cldlayerphase(:,3,3), pcols,lchnk)
   do i=1, ncol
      do k=1, pver
         xdata_r(i,k) = lidarcldphase(i,pver-k+1,3)
      enddo
   enddo
   call outfld('CLUNDEF ',xdata_r, pcols,lchnk)
 
   deallocate(lidarcld)

   endif    !   if(dolidar)
!=============================================================================================
! RADAR

   call aqsat(state%t, state%pmid, es, qs, pcols, ncol, pver, 1, pver)

   if (doradar) then

   call aqsat(state%t, state%pmid, es, qs, pcols, ncol, pver, 1, pver)

   p_matrix(:ncol,1:pver)   = state%pmid(:ncol,pver:1:-1)/100.0     ! From Pa to hPa
   hgt_matrix(:ncol,1:pver) = state%zm(:ncol, pver:1:-1)/1000.0     ! From m to km
   t_matrix(:ncol,1:pver)   = state%t(:ncol, pver:1:-1)
   rh_matrix(:ncol,1:pver)  = state%q(:ncol, pver:1:-1,1)/qs(:ncol,pver:1:-1)*100.  !relative humidity (%)
   re_matrix(:,:,:)  = 0.0
   Np_matrix(:,:,:)  = 0.0

   use_reff   = .true.
   airdensity(1:ncol,1:pver)   = state%pmid(1:ncol,1:pver)/(287.04*state%t(1:ncol,1:pver))   ! kg/m3
   do pr= 1, nsubcol
      do i=1, N_hydro
            hm_matrix(i,1:pcols,1:pver) = mr_hydro(1:pcols,pr,1:pver,i)*1000.0 ! Units from kg/kg to g/kg
                                                                        ! mr_hydro: from bottom to top
            if (use_reff) then
                re_matrix(i,1:ncol,1:pver)  = Reff(1:ncol,pr,1:pver,i)*1.e6   ! Units from m to micron, from bottom to top
                do j=1, ncol
                do k=1, pver
                   Np_matrix(i,j,k)  = CDNC(j,pver-k+1)*1.e-6/airdensity(j,pver-k+1)
                                                  ! Units [#/kg] , from bottom to top
                enddo
                enddo
            endif
      enddo

      if (pr == 1) then ! Compute gaseous attenuation for all profiles
          call radar_simulator(ncol, radar_freq,   k2,     do_ray,  use_gas_abs,     &
                               hgt_matrix(1:ncol,1:pver),            hm_matrix(1:N_hydro,1:ncol,1:pver),  &
                               re_matrix(1:N_hydro, 1:ncol,1:pver),  Np_matrix(1:N_hydro, 1:ncol,1:pver), &
                               p_matrix(1:ncol,1:pver),  t_matrix(1:ncol,1:pver), rh_matrix(1:ncol,1:pver),  & 
                               Ze_non(1:ncol, 1:pver),   Ze_ray(1:ncol, 1:pver),           &
                               h_atten_to_vol(1:ncol, 1:pver), g_atten_to_vol(1:ncol, 1:pver) ,  &
                               dBZe(1:ncol, 1:pver),           g_to_vol_out=g_to_vol )

      else ! Use gaseous atteunuation for pr = 1

          call radar_simulator(ncol, radar_freq,   k2,     do_ray,  use_gas_abs,     &
                               hgt_matrix(1:ncol,1:pver),            hm_matrix(1:N_hydro,1:ncol,1:pver),  &
                               re_matrix(1:N_hydro, 1:ncol,1:pver),  Np_matrix(1:N_hydro, 1:ncol,1:pver), &
                               p_matrix(1:ncol,1:pver),  t_matrix(1:ncol,1:pver), rh_matrix(1:ncol,1:pver),  &
                               Ze_non(1:ncol, 1:pver),   Ze_ray(1:ncol, 1:pver),           &
                               h_atten_to_vol(1:ncol, 1:pver), g_atten_to_vol(1:ncol, 1:pver) ,  &
                               dBZe(1:ncol, 1:pver),           g_to_vol_in=g_to_vol )
      endif

      ! output dBZe   ! Radar reflectivity

      do i=1, ncol
         do k=1, pver
            Ze_tot(i,pr,k) = dBZe(i,pver-k+1)   ! Ze_tot : from top to bottom
         enddo
      enddo
    enddo !pr

   endif   ! end if (doradar)

!=============================================================================================
! MODIS
!
   if( domodis ) then

    !
    ! Are there any sunlit points?
    !

    nSunlit = count(sunlit(1:ncol) > 0) 

    if(nSunlit > 0) then

      iSunlit = 0 
      do i=1, ncol
         if( sunlit(i) > 0 ) then
             iSunlit = iSunlit + 1
 
             pressureLevels(iSunlit, 1)       = 0.0 ! Top of model, following ISCCP sim
             temperature   (iSunlit, 1:pver)  = state%t    (i, 1:pver)
             pressureLayers(iSunlit, 1:pver)  = state%pmid (i, 1:pver)
             pressureLevels(iSunlit, 2:pver+1)= state%q(i, 1:pver,1)/qs(i,1:pver)*100.  !relative humidity (%) ??

             do i1=1, nsubcol
             do k =1, pver
               if( frac_out(i,i1,pver-k+1) == I_LSC ) then
                 cloudWater(iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_LSCLIQ)
                 waterSize (iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_LSCLIQ)
                 cloudIce  (iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_LSCICE)
                 iceSize   (iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_LSCICE)

               elseif ( frac_out(i,i1,pver-k+1) == I_CVC ) then
                 cloudWater(iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_CVCLIQ)
                 waterSize (iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_CVCLIQ)
                 cloudIce  (iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_CVCICE)
                 iceSize   (iSunlit, i1,k) = mr_hydro(i, i1, pver-k+1, I_CVCICE)
               else
                 cloudWater(iSunlit, i1,k) = 0.0
                 waterSize (iSunlit, i1,k) = 0.0
                 cloudIce  (iSunlit, i1,k) = 0.0
                 iceSize   (iSunlit, i1,k) = 0.0
               endif
 
               if( frac_out(i,i1,pver-k+1) == I_LSC ) then
                    opticalThickness(iSunlit, i1, k) = tau_s(i, k)

               elseif ( frac_out(i,i1,pver-k+1) == I_CVC ) then
                    opticalThickness(iSunlit, i1,k) = tau_c(i, k)
               else
                    opticalThickness(iSunlit, i1,k) =  0.0
               endif
             enddo
             enddo

             isccpTau             (iSunlit, 1:nsubcol) =  boxtau(i,1:nsubcol)
             isccpCloudTopPressure(iSunlit, 1:nsubcol) =  boxptop(i,1:nsubcol)

         endif   ! end if( sunlit(i) > 0 )
      enddo

!---
      do i = 1, nSunlit
        call modis_L2_simulator( &
                                temperature(i, :), pressureLayers(i, :), pressureLevels(i, :),     &
                                opticalThickness(i, :, :), cloudWater(i, :, :), cloudIce(i, :, :), &
                                waterSize(i, :, :), iceSize(i, :, :),                       &
                                isccpTau(i, :), isccpCloudTopPressure(i, :),                &
                                retrievedPhase(i, :), retrievedCloudTopPressure(i, :),      &
                                retrievedTau(i, :), retrievedSize(i, :))
        !
        ! output : Properties retrieved by MODIS
        !          retrievedPhase, retrievedCloudTopPressure, retrievedTau, retrievedSize
      end do
      call modis_L3_simulator(  &
                              retrievedPhase(1:nSunlit,1:nsubcol),              &
                              retrievedCloudTopPressure(1:nSunlit,1:nsubcol),   &
                              retrievedTau(1:nSunlit,1:nsubcol), retrievedSize(1:nSunlit,1:nsubcol), &
                              cfTotal(1:nSunlit),         cfLiquid(1:nSunlit),         cfIce(1:nSunlit),         &
                              cfHigh(1:nSunlit),          cfMid(1:nSunlit),            cfLow(1:nSunlit),         &
                              meanTauTotal(1:nSunlit),    meanTauLiquid(1:nSunlit),    meanTauIce(1:nSunlit),    &
                              meanLogTauTotal(1:nSunlit), meanLogTauLiquid(1:nSunlit), meanLogTauIce(1:nSunlit), &
                              meanSizeLiquid(1:nSunlit),   meanSizeIce(1:nSunlit),   &
                              meanCloudTopPressure(1:nSunlit),                             &
                              meanLiquidWaterPath(1:nSunlit), meanIceWaterPath(1:nSunlit), &
                              jointHistogram(1:nSunlit, 1:numModisTauBins, 1:numModisPressureBins) )
     jSunlit = 0
     do j=1, ncol
        if(sunlit(j) > 0 ) then
           jSunlit = jSunlit + 1
           Cloud_Fraction_Total_Mean(j) = cfTotal(jSunlit)  
           Cloud_Fraction_Water_Mean(j) = cfLiquid(jSunlit)  
           Cloud_Fraction_Ice_Mean(j) = cfIce(jSunlit)  
           Cloud_Fraction_High_Mean(j) = cfHigh(jSunlit)  
           Cloud_Fraction_Mid_Mean(j) = cfMid(jSunlit)  
           Cloud_Fraction_Low_Mean(j) = cfLow(jSunlit)  
           Optical_Thickness_Total_Mean(j) = meanTauTotal(jSunlit)  
           Optical_Thickness_Water_Mean(j) = meanTauLiquid(jSunlit)  
           Optical_Thickness_Ice_Mean(j) = meanTauIce(jSunlit)  
           Optical_Thickness_Total_LogMean(j) = meanLogTauTotal(jSunlit)  
           Optical_Thickness_Water_LogMean(j) = meanLogTauLiquid(jSunlit)  
           Optical_Thickness_Ice_LogMean(j) = meanLogTauIce(jSunlit)  
           Cloud_Particle_Size_Water_Mean(j) = meanSizeLiquid(jSunlit)  
           Cloud_Particle_Size_Ice_Mean(j) = meanSizeIce(jSunlit)  
           Cloud_Top_Pressure_Total_Mean(j) = meanCloudTopPressure(jSunlit)  
           Liquid_Water_Path_Mean(j) = meanLiquidWaterPath(jSunlit)  
           Ice_Water_Path_Mean(j) = meanIceWaterPath(jSunlit)  
           do i1 = 1, numModisTauBins
              do i2= 1, numModisPressureBins
                 Optical_Thickness_vs_Cloud_Top_Pressure(j, i1+1, numModisPressureBins-i2+1) = &
                                          jointHistogram(jSunlit, i1, i2)
              enddo
           enddo
         else
           Cloud_Fraction_Total_Mean(j) = R_UNDEF
           Cloud_Fraction_Water_Mean(j) = R_UNDEF
           Cloud_Fraction_Ice_Mean(j) = R_UNDEF
           Cloud_Fraction_High_Mean(j) = R_UNDEF
           Cloud_Fraction_Mid_Mean(j) = R_UNDEF
           Cloud_Fraction_Low_Mean(j) = R_UNDEF
           Optical_Thickness_Total_Mean(j) = R_UNDEF
           Optical_Thickness_Water_Mean(j) = R_UNDEF
           Optical_Thickness_Ice_Mean(j) = R_UNDEF
           Optical_Thickness_Total_LogMean(j) = R_UNDEF
           Optical_Thickness_Water_LogMean(j) = R_UNDEF
           Optical_Thickness_Ice_LogMean(j) = R_UNDEF
           Cloud_Particle_Size_Water_Mean(j) = R_UNDEF
           Cloud_Particle_Size_Ice_Mean(j) = R_UNDEF
           Cloud_Top_Pressure_Total_Mean(j) = R_UNDEF
           Liquid_Water_Path_Mean(j) = R_UNDEF
           Ice_Water_Path_Mean(j) = R_UNDEF
           Optical_Thickness_vs_Cloud_Top_Pressure(j, :, :) = R_UNDEF
         endif
      enddo
 
  else
           Cloud_Fraction_Total_Mean(:) = R_UNDEF
           Cloud_Fraction_Water_Mean(:) = R_UNDEF
           Cloud_Fraction_Ice_Mean(:) = R_UNDEF
           Cloud_Fraction_High_Mean(:) = R_UNDEF
           Cloud_Fraction_Mid_Mean(:) = R_UNDEF
           Cloud_Fraction_Low_Mean(:) = R_UNDEF
           Optical_Thickness_Total_Mean(:) = R_UNDEF
           Optical_Thickness_Water_Mean(:) = R_UNDEF
           Optical_Thickness_Ice_Mean(:) = R_UNDEF
           Optical_Thickness_Total_LogMean(:) = R_UNDEF
           Optical_Thickness_Water_LogMean(:) = R_UNDEF
           Optical_Thickness_Ice_LogMean(:) = R_UNDEF
           Cloud_Particle_Size_Water_Mean(:) = R_UNDEF
           Cloud_Particle_Size_Ice_Mean(:) = R_UNDEF
           Cloud_Top_Pressure_Total_Mean(:) = R_UNDEF
           Liquid_Water_Path_Mean(:) = R_UNDEF
           Ice_Water_Path_Mean(:) = R_UNDEF
           Optical_Thickness_vs_Cloud_Top_Pressure(:, :,:) =R_UNDEF

    end if  ! end if (nSunlit > 0) 

   endif ! end if (domodis)
!=============================================================================================
! MISR simulator
!
   if( domisr) then
       
       frac_out(:ncol,:,1:pver) = frac_out(:ncol,:,pver:1:-1)  ! from upward to downward

       call MISR_simulator( ncol, pver, nsubcol, &
                            sunlit(1:ncol),  state%zm(1:ncol,1:pver), state%t(1:ncol,1:pver), &
                            tau_s(1:ncol,1:pver), tau_c(1:ncol,1:pver),  &
                            frac_out(1:ncol,1:nsubcol,:), R_UNDEF, &
                            fq_MISR(1:ncol,:,:),   MISR_dist_model_layertops(1:ncol,:), &
                            MISR_meanztop(1:ncol), MISR_cldarea(1:ncol) )
   endif
!=============================================================================================
   call t_stopf ('cloudsimulator_run')

end subroutine cloudsimulator_run

!#######################################################################

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!------------- SUBROUTINE CONSTRUCT_COSP_VGRID ------------------
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! SUBROUTINE CONSTRUCT_COSP_VGRID(gbx,Nlvgrid,use_vgrid,cloudsat,x)
  SUBROUTINE CONSTRUCT_COSP_VGRID(Npoints,Ncolumns,Nlevels,Nlvgrid,use_vgrid,cloudsat,x)
!   type(cosp_gridbox),intent(in) :: gbx ! Gridbox information
    integer,intent(in) :: Npoints,Ncolumns,Nlevels  ! Numbers of model grid
    integer,intent(in) :: Nlvgrid  ! Number of new levels    
    logical,intent(in) :: use_vgrid! Logical flag that controls the output on a different grid
    logical,intent(in) :: cloudsat ! TRUE if a CloudSat like grid (480m) is requested
    type(cosp_vgrid),intent(out) :: x
    
    ! Local variables
    integer :: i
    real :: zstep
    
    x%use_vgrid  = use_vgrid
    x%csat_vgrid = cloudsat
    
    ! Dimensions
    x%Npoints  = Npoints
    x%Ncolumns = Ncolumns
    x%Nlevels  = Nlevels
    
    ! --- Allocate arrays ---
    if (use_vgrid) then
      x%Nlvgrid = Nlvgrid
    else 
      x%Nlvgrid = Nlevels
    endif
    allocate(x%z(x%Nlvgrid),x%zl(x%Nlvgrid),x%zu(x%Nlvgrid))
    allocate(x%mz(x%Nlevels),x%mzl(x%Nlevels),x%mzu(x%Nlevels))
    
    ! --- Model vertical levels ---
    ! Use height levels of first model gridbox
!   x%mz  = gbx%zlev(1,:)
!   x%mzl = gbx%zlev_half(1,:)
!   x%mzu(1:x%Nlevels-1) = gbx%zlev_half(1,2:x%Nlevels)
!   x%mzu(x%Nlevels) = gbx%zlev(1,x%Nlevels) + (gbx%zlev(1,x%Nlevels) - x%mzl(x%Nlevels))
      ! --- Initialise to zero ---
      x%mz  = 0.0
      x%mzl = 0.0
      x%mzu = 0.0
    
    if (use_vgrid) then
      ! --- Initialise to zero ---
      x%z  = 0.0
      x%zl = 0.0
      x%zu = 0.0
      if (cloudsat) then ! --- CloudSat grid requested ---
         zstep = 480.0
      else
         ! Other grid requested. Constant vertical spacing with top at 20 km
         zstep = 20000.0/x%Nlvgrid
      endif
      do i=1,x%Nlvgrid
         x%zl(i) = (i-1)*zstep
         x%zu(i) = i*zstep
      enddo
      x%z = (x%zl + x%zu)*0.5
    else
      x%z  = x%mz
      x%zl = x%mzl
      x%zu = x%mzu
    endif
    
  END SUBROUTINE CONSTRUCT_COSP_VGRID

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!---------- SUBROUTINE COSP_CHANGE_VERTICAL_GRID ----------------
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUBROUTINE COSP_CHANGE_VERTICAL_GRID(Npoints,Ncolumns,Nlevels,zfull,zhalf,y,M,zl,zu,r,log_units)
   implicit none
   ! Input arguments
   integer,intent(in) :: Npoints  !# of grid points
   integer,intent(in) :: Nlevels  !# of levels
   integer,intent(in) :: Ncolumns !# of columns
   real,dimension(Npoints,Nlevels),intent(in) :: zfull ! Height at model levels [m] (Bottom of model layer)
   real,dimension(Npoints,Nlevels),intent(in) :: zhalf ! Height at half model levels [m] (Bottom of model layer)
   real,dimension(Npoints,Ncolumns,Nlevels),intent(in) :: y     ! Variable to be changed to a different grid
   integer,intent(in) :: M  !# levels in the new grid
   real,dimension(M),intent(in) :: zl ! Lower boundary of new levels  [m]
   real,dimension(M),intent(in) :: zu ! Upper boundary of new levels  [m]
   logical,optional,intent(in) :: log_units ! log units, need to convert to linear units
   ! Output
   real,dimension(Npoints,Ncolumns,M),intent(out) :: r ! Variable on new grid

   ! Local variables
   integer :: i,j,k
   logical :: lunits
   real :: ws
   real,dimension(Nlevels) :: xl,xu ! Lower and upper boundaries of model grid
   real,dimension(M) :: dz          ! Layer depth
   real,dimension(Nlevels,M) :: w   ! Weights to do the mean at each point
   real,dimension(Ncolumns,Nlevels) :: yp  ! Variable to be changed to a different grid.
                                           ! Local copy at a particular point.
                                           ! This allows for change of units.
   
   lunits=.false.
   if (present(log_units)) lunits=log_units
   
   r = 0.0
   do i=1,Npoints
     ! Vertical grid at that point
     xl = zhalf(i,:)
     xu(1:Nlevels-1) = xl(2:Nlevels)
     xu(Nlevels) = zfull(i,Nlevels) +  zfull(i,Nlevels) - zhalf(i,Nlevels) ! Top level symmetric
     dz = zu - zl
     yp = y(i,:,:) ! Temporary variable to regrid
     ! Find weights
     w = 0.0
     do k=1,M
       do j=1,Nlevels
         if ((xl(j) < zl(k)).and.(xu(j) > zl(k)).and.(xu(j) <= zu(k))) then
           !xl(j)-----------------xu(j)
           !      zl(k)------------------------------zu(k)
           w(j,k) = xu(j) - zl(k)
         else if ((xl(j) >= zl(k)).and.(xu(j) <= zu(k))) then
           !           xl(j)-----------------xu(j)
           !      zl(k)------------------------------zu(k)
           w(j,k) = xu(j) - xl(j)
         else if ((xl(j) >= zl(k)).and.(xl(j) < zu(k)).and.(xu(j) >= zu(k))) then
           !                           xl(j)-----------------xu(j)
           !      zl(k)------------------------------zu(k)
           w(j,k) = zu(k) - xl(j)
         else if ((xl(j) <= zl(k)).and.(xu(j) >= zu(k))) then
           !  xl(j)---------------------------xu(j)
           !        zl(k)--------------zu(k)
           w(j,k) = dz(j)
         endif
       enddo
     enddo
     ! Check for dBZ and change if necessary
     if (lunits) then
        where (yp /= R_UNDEF)
          yp = 10.0**(yp/10.0)
        elsewhere
          yp = 0.0
        end where
     endif
     ! Do the weighted mean
     do j=1,Ncolumns
       do k=1,M
          ws = sum(w(:,k))
          if (ws > 0.0) r(i,j,k) = sum(w(:,k)*yp(j,:))/ws
       enddo
     enddo
     ! Check for dBZ and change if necessary
     if (lunits) then
        where (r(i,:,:) <= 0.0)
          r(i,:,:) = R_UNDEF
        elsewhere
          r(i,:,:) = 10.0*log10(r(i,:,:))
        end where
     endif
   enddo
 
 
END SUBROUTINE COSP_CHANGE_VERTICAL_GRID 

end module cloudsimulator
