#include <misc.h>
#include <params.h>

module mod_radar_simulator

  use shr_kind_mod,    only: r8 => shr_kind_r8
  use ppgrid,          only: pcols, pver
  use m_mrgrnk 
  use array_lib
  use math_lib
  use optics_lib
  use mod_cosp_constants

  implicit none
  private
  save

  integer, parameter, public ::       &
     maxhclass = 20         ,& ! max number of hydrometeor classes
     nRe_types = 550       ! max number or Re size bins allowed in N and Z_scaled look up table

  ! These variables define discrete diameters used to represent the DSDs.
  integer, parameter, public ::       &
     nd = 85               ! number of discrete particles used in construction DSDs
  real(r8), parameter, public ::        &
     dmin = 0.1                 ,& ! min size of discrete particle
     dmax = 10000.                 ! max size of discrete particle

  integer, parameter, public :: &   ! These parameters used to define temperature intervals in mie LUTs
     mt_nfreq = 5              , &
     mt_ntt = 39               , & ! num temperatures in table
     mt_nf = 14            , & ! number of ice fractions in table
     mt_nd = 85

  integer, parameter, public :: &   ! These parameters used to defines Re intervals in scale LUTs
     Re_BIN_LENGTH=10, &
     Re_MAX_BIN=250

  integer, parameter, public :: &   ! These parameters used to define Temperature invervals in scale LUTs
     cnt_liq = 19, &       ! liquid temperature count
     cnt_ice = 20          ! ice temperature countB

   real(r8), public :: &
        mt_ttl(cnt_liq), &  ! liquid temperatures (K)
        mt_tti(cnt_ice)     ! ice temperatures (K)

   real(r8), public  :: D(nd)
   real(r8), public  :: delt, deltp
   real(r8), public  :: step_list(Re_MAX_BIN),base_list(Re_MAX_BIN)

   logical,  dimension(maxhclass, mt_ntt,nRe_types), public :: Z_scale_flag
   real(r8), dimension(maxhclass, mt_ntt,nRe_types), public :: Ze_scaled,      Zr_scaled,   kr_scaled
   real(r8), dimension(maxhclass, nd,    nRe_types), public :: rho_eff

   public radar_simulator

CONTAINS

  subroutine radar_simulator( ncol, radar_freq,   k2,     do_ray,  use_gas_abs, &
                       hgt_matrix,   hm_matrix,   re_matrix,   Np_matrix, &
                       p_matrix,     t_matrix,    rh_matrix, &
                       Ze_non,       Ze_ray,      a_to_vol,    g_to_vol,dBZe, &
                       g_to_vol_in,  g_to_vol_out )
!------------------------------------------------------------
! Purpose:
!
!   Simulates a vertical profile of radar reflectivity
!   Originally Part of QuickBeam v1.04 by John Haynes & Roger Marchand.
!   but has been substantially modified since that time by
!   Laura Fowler and Roger Marchand (see modifications below).
!
! Inputs:
!
!   [hp]              structure that defines hydrometeor types and other radar properties
!
!   [ncol]           number of hydrometeor profiles
!   [pver]           number of vertical layers
!
!   [R_UNDEF]           missing data value
!   (The following 5 arrays must be in order from closest to the radar
!    to farthest...)
!
!   [hgt_matrix]      height of hydrometeors (km)
!   [p_matrix]        pressure profile (hPa)
!   [t_matrix]        temperature profile (K)
!   [rh_matrix]       relative humidity profile (%) -- only needed if gaseous aborption calculated.
!
!   [hm_matrix]       table of hydrometeor mixing rations (g/kg)
!   [re_matrix]       table of hydrometeor effective radii.  0 ==> use defaults. (units=microns)
!   [Np_matrix]       table of hydrometeor number concentration.  0 ==> use defaults. (units = 1/kg)
!
! Outputs:
!
!   [Ze_non]          radar reflectivity without attenuation (dBZ)
!   [Ze_ray]          Rayleigh reflectivity (dBZ)
!   [h_atten_to_vol]  attenuation by hydromets, radar to vol (dB)
!   [g_atten_to_vol]  gaseous atteunation, radar to vol (dB)
!   [dBZe]            effective radar reflectivity factor (dBZ)
!
! Optional:
!   [g_to_vol_in]     integrated atten due to gases, r>v (dB).
!                     If present then is used as gaseous absorption, independently of the
!                     value in use_gas_abs
!   [g_to_vol_out]    integrated atten due to gases, r>v (dB).
!                     If present then gaseous absorption for each profile is returned here.
!
! Created:
!   11/28/2005  John Haynes (haynes@atmos.colostate.edu)
!
! Modified:
!   09/2006  placed into subroutine form (Roger Marchand,JMH)
!   08/2007  added equivalent volume spheres, Z and N scalling most distrubtion types (Roger Marchand)
!   01/2008  'Do while' to determine if hydrometeor(s) present in volume
!             changed for vectorization purposes (A. Bodas-Salcedo)
!
!   07/2010  V3.0 ... Modified to load or save scale factors to disk as a Look-Up Table (LUT)
!  ... All hydrometeor and radar simulator properties now included in hp structure
!  ... hp structure should be initialized by call to radar_simulator_init prior
!  ... to calling this subroutine.
!     Also ... Support of Morrison 2-moment style microphyscis (Np_matrix) added
!  ... Changes implement by Roj Marchand following work by Laura Fowler
!
!   10/2011  Modified pver loop to go in either direction depending on flag
!     hp%radar_at_layer_one.  This affects the direction in which attenuation is summed.
!
!     Also removed called to AVINT for gas and hydrometeor attenuation and replaced with simple
!     summation. (Roger Marchand)
!
! ----- INPUTS -----  
   integer, intent(in) ::  ncol
   real(r8),intent(in) ::  hgt_matrix(pcols,pver)
   real(r8),intent(in) ::  hm_matrix(N_hydro,pcols,pver)
   real(r8),intent(inout) ::  re_matrix(N_hydro,pcols,pver)
   real(r8),intent(in) ::  Np_matrix(N_hydro,pcols,pver)
   real(r8),intent(in) ::  p_matrix(pcols,pver)
   real(r8),intent(in) ::  t_matrix(pcols,pver)
   real(r8),intent(in) ::  rh_matrix(pcols,pver)

   real(r8),intent(out) ::  Ze_non  (pcols,pver)  ! radar reflectivity withOUT attenuation (dBZ)
   real(r8),intent(out) ::  Ze_ray  (pcols,pver)  ! Rayleigh reflectivity (dBZ)
   real(r8),intent(out) ::  g_to_vol(pcols,pver)  ! attenuation by hydromets, radar to vol (dB)
   real(r8),intent(out) ::  a_to_vol(pcols,pver)       ! attenuation by hydromets, radar to vol (dB)
   real(r8),intent(out) ::  dBZe    (pcols,pver)             ! effective radar reflectivity factor (dBZ)

   real*8, optional, dimension(pcols,pver) :: &
           g_to_vol_in,   g_to_vol_out ! integrated atten due to gases, r>v (dB). This allows to output and then input
                                       ! the same gaseous absorption in different calls. Optional to allow compatibility
                                       ! with original version. A. Bodas April 2008.

   real(r8), intent(in) :: radar_freq, k2 
   integer,  intent(in) :: use_gas_abs, do_ray

!-----------------------------------------------------
! local variables
! 
  logical, parameter  ::  DO_LUT_TEST = .false. 
  logical, parameter  ::  DO_NP_TEST  = .false. 
  logical, parameter  ::  radar_at_layer_one= .true.

! ----- INTERNAL -----

  real, parameter :: one_third = 1.0/3.0
  real*8          :: t_kelvin
  integer :: &
             HCLASS_PHASE0, & ! 0=liquid, 1=ice
             ns               ! number of discrete drop sizes

  logical :: hydro	! true=hydrometeor in vol, false=none
  real*8  :: &
             HCLASS_RHO_a, &   ! air density (kg m^-3)
             gases      ! function: 2-way gas atten (dB/km)

  real*8, dimension(:), allocatable :: &
           Di, Deq, &   ! discrete drop sizes (um)
           Ni, &        ! discrete concentrations (cm^-3 um^-1)
           HCLASS_RHOi         ! discrete densities (kg m^-3)

  real*8, dimension(pcols, pver) :: &
           z_vol, &      ! effective reflectivity factor (mm^6/m^3)
           z_ray, &                      ! reflectivity factor, Rayleigh only (mm^6/m^3)
           kr_vol, &     ! attenuation coefficient hydro (dB/km)
           g_vol         ! attenuation coefficient gases (dB/km)


  integer,parameter :: KR8 = selected_real_kind(15,300)
  real*8, parameter :: xx = -1.0_KR8
  real*8,  dimension(:), allocatable :: xxa
  real*8 :: kr, ze, zr, pi, scale_factor, tc, Re, ld, tmHCLASS_P1, &
            ze2, kr2
  real*8 :: half_a_atten_current,half_a_atten_above
  real*8 :: half_g_atten_current,half_g_atten_above
  integer*4 :: tp, i, j, k, pr, itt, iff

  real*8    step,base, Np
  integer*4 iRe_type,n,max_bin

  integer   start_gate,end_gate,d_gate

  logical :: g_to_vol_in_present, g_to_vol_out_present

  ! Logicals to avoid calling present within the loops
  g_to_vol_in_present  = present(g_to_vol_in)
  g_to_vol_out_present = present(g_to_vol_out)

  !
  ! load scaling matricies from disk -- but only the first time this subroutine is called
  !

  pi = acos(-1.0)

!   ----- Initialisation -----
  g_to_vol = 0.0
  a_to_vol = 0.0
  z_vol    = 0.0
  z_ray    = 0.0
  kr_vol   = 0.0

!   // loop over each range gate (pver) ... starting with layer closest to the radar !
  if(radar_at_layer_one) then
    start_gate=1
    end_gate=pver
    d_gate=1
  else
    start_gate=pver
    end_gate=1
    d_gate=-1
  endif

!--------------------------------------------
!wtw  do k=start_gate,end_gate,d_gate
!wtw  ! // loop over each profile ()
!wtw  do pr=1, ncol

  do pr=1, ncol

     do k=start_gate,end_gate,d_gate

      t_kelvin = t_matrix(pr,k)
!     :: determine if hydrometeor(s) present in volume
      hydro = .false.
      do j=1,N_hydro
        if ((hm_matrix(j,pr,k) > 1E-12) .and. (HCLASS_TYPE(j) > 0)) then
          hydro = .true.
          exit
        endif
      enddo

!     :: if there is hydrometeor in the volume
      if (hydro) then

        HCLASS_RHO_a = (p_matrix(pr,k)*100.)/(287.0*(t_kelvin))
!       :: loop over hydrometeor type
        do tp=1,N_hydro
          if (hm_matrix(tp,pr,k) <= 1E-12) cycle
          HCLASS_PHASE0 = HCLASS_PHASE(tp)
          if (HCLASS_PHASE0==0) then
            itt = infind(mt_ttl,t_kelvin)
          else
            itt = infind(mt_tti,t_kelvin)
          endif
          if (re_matrix(tp,pr,k).eq.0) then
            call calc_Re(hm_matrix(tp,pr,k),Np_matrix(tp,pr,k),HCLASS_RHO_a, &
              HCLASS_TYPE(tp),HCLASS_DMIN(tp),HCLASS_DMAX(tp),HCLASS_APM(tp),HCLASS_BPM(tp), &
              HCLASS_RHO(tp),HCLASS_P1(tp),HCLASS_P2(tp),HCLASS_P3(tp),Re)
            re_matrix(tp,pr,k)=Re
          else
            if (Np_matrix(tp,pr,k)>0) then
!               print *, 'Warning: Re and Np set for the same ', &
!                        'volume & hydrometeor type.  Np is being ignored.'
            endif
            Re = re_matrix(tp,pr,k)
          endif

          iRe_type=1
          if(Re.gt.0) then
            ! determine index in to scale LUT
            !
            ! distance between Re points (defined by "base" and "step") for
            ! each interval of size Re_BIN_LENGTH
            ! Integer asignment, avoids calling floor intrinsic
            n=Re/Re_BIN_LENGTH
            if (n>=Re_MAX_BIN) n=Re_MAX_BIN-1
            step=step_list(n+1)
            base=base_list(n+1)
            iRe_type=Re/step
            if (iRe_type.lt.1) iRe_type=1

            Re=step*(iRe_type+0.5)	! set value of Re to closest value allowed in LUT.
            iRe_type=iRe_type+base-int(n*Re_BIN_LENGTH/step)

            ! make sure iRe_type is within bounds
            if (iRe_type.ge.nRe_types) then
!               write(*,*) 'Warning: size of Re exceed value permitted ', &
!                    'in Look-Up Table (LUT).  Will calculate. '
               ! no scaling allowed
               iRe_type=nRe_types
               Z_scale_flag(tp,itt,iRe_type)=.false.
            else
               ! set value in re_matrix to closest values in LUT
              if (.not. DO_LUT_TEST) re_matrix(tp,pr,k)=Re
            endif
          endif
          ! use Ze_scaled, Zr_scaled, and kr_scaled ... if know them
          ! if not we will calculate Ze, Zr, and Kr from the distribution parameters
          if( (.not. Z_scale_flag(tp,itt,iRe_type)) .or. DO_LUT_TEST)  then
            ! :: create a distribution of hydrometeors within volume
            select case(HCLASS_TYPE(tp))
              case(4)
                ns = 1
                allocate(Di(ns),Ni(ns),HCLASS_RHOi(ns),xxa(ns),Deq(ns))
                Di = HCLASS_P1(tp)
                Ni = 0.
              case default
                ns = nd  
                allocate(Di(ns),Ni(ns),HCLASS_RHOi(ns),xxa(ns),Deq(ns))
                Di = D
                Ni = 0.
            end select
            call dsd(hm_matrix(tp,pr,k),re_matrix(tp,pr,k),Np_matrix(tp,pr,k), &
                     Di,Ni,ns,HCLASS_TYPE(tp),HCLASS_RHO_a,t_kelvin, &
                     HCLASS_DMIN(tp),HCLASS_DMAX(tp),HCLASS_APM(tp),HCLASS_BPM(tp), &
                     HCLASS_RHO(tp),HCLASS_P1(tp),HCLASS_P2(tp),HCLASS_P3(tp))

            ! calculate particle density
            if (HCLASS_PHASE0 == 1) then
              if (HCLASS_RHO(tp) < 0) then
                ! Use equivalent volume spheres.
                rho_eff(tp,1:ns,iRe_type) = 917  				! solid ice == equivalent volume approach
                Deq = ( ( 6/pi*HCLASS_APM(tp)/917 ) ** (1.0/3.0) ) * ( (Di*1E-6) ** (HCLASS_BPM(tp)/3.0) )  * 1E6
                ! alternative is to comment out above two lines and use the following block
                ! MG Mie approach - adjust density of sphere with D = D_characteristic to match particle density
                !
                ! hp%rho_eff(tp,1:ns,iRe_type) = (6/pi)*hp%HCLASS_APM(tp)*(Di*1E-6)**(hp%HCLASS_BPM(tp)-3)   !MG Mie approach

                ! as the particle size gets small it is possible that the mass to size relationship of 
                ! (given by power law in hclass.data) can produce impossible results 
                ! where the mass is larger than a solid sphere of ice.  
                ! This loop ensures that no ice particle can have more mass/density larger than an ice sphere.
                ! do i=1,ns
                ! if(hp%rho_eff(tp,i,iRe_type) > 917 ) then
                ! hp%rho_eff(tp,i,iRe_type) = 917
                ! endif
                ! enddo
              else
                ! Equivalent volume sphere (solid ice HCLASS_RHO_ice=917 kg/m^3).
                rho_eff(tp,1:ns,iRe_type) = 917
                Deq=Di * ((HCLASS_RHO(tp)/917)**(1.0/3.0))
                ! alternative ... coment out above two lines and use the following for MG-Mie
                ! hp%rho_eff(tp,1:ns,iRe_type) = hp%HCLASS_RHO(tp)   !MG Mie approach
              endif
            else
              ! I assume here that water HCLASS_PHASE droplets are spheres.
              ! hp%HCLASS_RHO should be ~ 1000  or hp%HCLASS_APM=524 .and. hp%HCLASS_BPM=3
              Deq = Di
            endif

            ! calculate effective reflectivity factor of volume
            xxa = -9.9
            HCLASS_RHOi = rho_eff(tp,1:ns,iRe_type)
            call zeff(radar_freq,Deq,Ni,ns,k2,t_kelvin,HCLASS_PHASE0,do_ray, &
                      ze,zr,kr,xxa,xxa,HCLASS_RHOi)

            ! test code ... compare Np value input to routine with sum of DSD
            ! NOTE: if .not. DO_LUT_TEST, then you are checking the LUT approximation 
            ! not just the DSD representation given by Ni
            if(Np_matrix(tp,pr,k)>0 .and. DO_NP_TEST ) then
              Np = path_integral(Ni,Di,1,ns-1)/HCLASS_RHO_a*1E6
              ! Note: Representation is not great or small Re < 2 
              if( (Np_matrix(tp,pr,k)-Np)/Np_matrix(tp,pr,k)>0.1 ) then
                write(*,*) 'Error: Np input does not match sum(N)'
                write(*,*) tp,pr,k,Re,Ni(1),Ni(ns),10*log10(ze)
                write(*,*) Np_matrix(tp,pr,k),Np,(Np_matrix(tp,pr,k)-Np)/Np_matrix(tp,pr,k)
                write(*,*)
              endif
            endif

            deallocate(Di,Ni,HCLASS_RHOi,xxa,Deq)

            ! LUT test code
            ! This segment of code compares full calculation to scaling result
            if ( Z_scale_flag(tp,itt,iRe_type) .and. DO_LUT_TEST )  then
              scale_factor=HCLASS_RHO_a*hm_matrix(tp,pr,k)
              ! if more than 2 dBZe difference print error message/parameters.
              if ( abs(10*log10(ze) - 10*log10(Ze_scaled(tp,itt,iRe_type) * &
                   scale_factor)) > 2 ) then
                write(*,*) 'Roj Error: ',tp,itt,iRe_type,Z_scale_flag(tp,itt,iRe_type),n,step,base
                write(*,*) 10*log10(ze),10*log10(Ze_scaled(tp,itt,iRe_type) * scale_factor)
                write(*,*) Ze_scaled(tp,itt,iRe_type),scale_factor
                write(*,*) re_matrix(tp,pr,k),Re
                write(*,*)
              endif
            endif

          else ! can use z scaling
            scale_factor=HCLASS_RHO_a*hm_matrix(tp,pr,k)
            zr = Zr_scaled(tp,itt,iRe_type) * scale_factor
            ze = Ze_scaled(tp,itt,iRe_type) * scale_factor
            kr = kr_scaled(tp,itt,iRe_type) * scale_factor
          endif  ! end z_scaling

          kr_vol(pr,k) = kr_vol(pr,k) + kr
          z_vol(pr,k)  = z_vol(pr,k)  + ze
          z_ray(pr,k)  = z_ray(pr,k)  + zr

          ! construct Ze_scaled, Zr_scaled, and kr_scaled ... if we can
          if ( .not. Z_scale_flag(tp,itt,iRe_type) ) then
            if (iRe_type>1) then
              scale_factor=HCLASS_RHO_a*hm_matrix(tp,pr,k)
              Ze_scaled(tp,itt,iRe_type) = ze/ scale_factor
              Zr_scaled(tp,itt,iRe_type) = zr/ scale_factor
              kr_scaled(tp,itt,iRe_type) = kr/ scale_factor
              Z_scale_flag(tp,itt,iRe_type) = .true.
            endif
          endif

        enddo	! end loop of tp (hydrometeor type)

      else
!     :: volume is hydrometeor-free	
        kr_vol(pr,k) = 0
        z_vol(pr,k)  = R_UNDEF
        z_ray(pr,k)  = R_UNDEF
      endif

      !     :: attenuation due to hydrometeors between radar and volume
      !
      ! NOTE old scheme integrates attenuation only for the layers ABOVE
      ! the current layer ... i.e. 1 to k-1 rather than 1 to k ...
      ! which may be a problem.   ROJ
      ! in the new scheme I assign half the attenuation to the current layer
      if(d_gate==1) then
        ! dheight calcuations assumes hgt_matrix points are the cell mid-points.
        if (k>2) then
          ! add to previous value to half of above layer + half of current layer
          a_to_vol(pr,k)=  a_to_vol(pr,k-1) + &
             (kr_vol(pr,k-1)+kr_vol(pr,k))*(hgt_matrix(pr,k-1)-hgt_matrix(pr,k))
        else
          a_to_vol(pr,k)=  kr_vol(pr,k)*(hgt_matrix(pr,k)-hgt_matrix(pr,k+1))
        endif
      else   ! d_gate==-1
        if(k<pver) then
          ! add to previous value half of above layer + half of current layer
          a_to_vol(pr,k) = a_to_vol(pr,k+1) + &
              (kr_vol(pr,k+1)+kr_vol(pr,k))*(hgt_matrix(pr,k+1)-hgt_matrix(pr,k))
        else
          a_to_vol(pr,k)= kr_vol(pr,k)*(hgt_matrix(pr,k)-hgt_matrix(pr,k-1))
        endif
      endif

      !     :: attenuation due to gaseous absorption between radar and volume
      if (g_to_vol_in_present) then
        g_to_vol(pr,k) = g_to_vol_in(pr,k)
      else
        if ( (use_gas_abs == 1) .or. ((use_gas_abs == 2) .and. (pr == 1)) ) then
          g_vol(pr,k) = gases(p_matrix(pr,k),t_kelvin,rh_matrix(pr,k),radar_freq)
          if (d_gate==1) then
            if (k>1) then
              ! add to previous value to half of above layer + half of current layer
              g_to_vol(pr,k) =  g_to_vol(pr,k-1) + &
                  0.5*(g_vol(pr,k-1)+g_vol(pr,k))*(hgt_matrix(pr,k-1)-hgt_matrix(pr,k))
            else
              g_to_vol(pr,k)=  0.5*g_vol(pr,k)*(hgt_matrix(pr,k)-hgt_matrix(pr,k+1))
            endif
          else   ! d_gate==-1
            if (k<pver) then
              ! add to previous value to half of above layer + half of current layer
              g_to_vol(pr,k) = g_to_vol(pr,k+1) + &
                 0.5*(g_vol(pr,k+1)+g_vol(pr,k))*(hgt_matrix(pr,k+1)-hgt_matrix(pr,k))
            else
              g_to_vol(pr,k)= 0.5*g_vol(pr,k)*(hgt_matrix(pr,k)-hgt_matrix(pr,k-1))
            endif
          endif
        elseif(use_gas_abs == 2) then
          ! using value calculated for the first column
          g_to_vol(pr,k) = g_to_vol(1,k)
        elseif (use_gas_abs == 0) then
          g_to_vol(pr,k) = 0
        endif
      endif

      ! Compute Rayleigh reflectivity, and full, attenuated reflectivity
      if ((do_ray == 1) .and. (z_ray(pr,k) > 0)) then
        Ze_ray(pr,k) = 10*log10(z_ray(pr,k))
      else
        Ze_ray(pr,k) = R_UNDEF
      endif
      if (z_vol(pr,k) > 0) then
        Ze_non(pr,k) = 10*log10(z_vol(pr,k))
        dBZe(pr,k) = Ze_non(pr,k)-a_to_vol(pr,k)-g_to_vol(pr,k)
      else
        dBZe(pr,k) = R_UNDEF
        Ze_non(pr,k) = R_UNDEF
      endif

 
! old   enddo   ! end loop over pr (profile)
! old  enddo ! end loop of k (range gate)

     enddo ! end loop of k (range gate)
  enddo   ! end loop over pr (profile)

  ! Output array with gaseous absorption
  if (g_to_vol_out_present) g_to_vol_out = g_to_vol

!wtw  ! save any updates made 
!wtw  if (hp%update_scale_LUTs) call save_scale_LUTs(hp)

  end subroutine radar_simulator

end module mod_radar_simulator
