#include <misc.h>
#include <params.h>

module bcarbon_intr

!---------------------------------------------------------------------------------
! Module to interface the aerosol parameterizations with CAM
! written by PJR (extensively modified from chemistry module)
!---------------------------------------------------------------------------------

!-----the following 7 lines are added by lqx, 20070906----------------------
  use shr_kind_mod,only: r8 => shr_kind_r8
  use pmgrid,      only: plat, plev, plevp, masterproc
  use ppgrid,      only: pcols, pver
  use shr_const_mod, only: SHR_CONST_G
!  use physconst,   only: mwdry, mwh2o
  use constituents,only: ppcnst, cnst_add, cnst_name, advected, cnst_get_ind, iaero
  use cam_drydep_mod, only: setdvel, ddflux
!-------------------------------------------------------------------------
  use abortutils,  only: endrun
    

  implicit none

  private          ! Make default type private to the module

  save

!------The following 5 lines are added by Liu Qianxia, 20070906------------
  integer ncyear

  integer :: ncnst                                   ! number of constituents
  character(len=8), allocatable :: cnst_names(:)     ! constituent names
  integer  bcarbon_idx1                              ! start constituent index for first BC cycle component
!------------------------------------------------------------------------------

!
! Public interfaces
!
  public bcarbon_register_cnst                     ! register consituents, liuqx 20070906
  public bcarbon_implements_cnst   ! returns true if consituent is implemented by this package
  public bcarbon_init_cnst        ! initialize mixing ratios if not read from initial file
  public bcarbon_initialize                           ! initialize (history) variables
  public bcarbon_wet_intr                             ! interface to wet deposition
  public bcarbon_emis_intr                            ! interface to emission
  public bcarbon_drydep_intr                          ! interface to tendency computation
  public bcarbon_time_interp                          ! interpolate oxidants and fluxes to current time
  public bcyc_idx1, ncnst                             ! constituent index of 1st BC cycle component

!-----------defining mw and cp of all the black bcarbon bins by Liuqx 20070906-----------

  real(r8), parameter:: cpBC1 = 666.
  real(r8), parameter:: cpBC2 = 666.
  real(r8), parameter:: cpBC3 = 666.
  real(r8), parameter:: cpBC4 = 666.
  real(r8), parameter:: cpBC5 = 666.
  real(r8), parameter:: cpBC6 = 666.
  real(r8), parameter:: cpBC7 = 666.
  real(r8), parameter:: cpBC8 = 666.
  real(r8), parameter:: cpBC9 = 666.
  real(r8), parameter:: cpBC10 = 666.
  real(r8), parameter:: cpBC11 = 666.
  real(r8), parameter:: cpBC12 = 666.


  real(r8), parameter:: mwBC1  = 12.011
  real(r8), parameter:: mwBC2  = 9999.
  real(r8), parameter:: mwBC3 = 9999.
  real(r8), parameter:: mwBC4  = 9999.
  real(r8), parameter:: mwBC5  = 9999.
  real(r8), parameter:: mwBC6  = 9999.
  real(r8), parameter:: mwBC7 = 9999.
  real(r8), parameter:: mwBC8  = 9999.
  real(r8), parameter:: mwBC9  = 9999.
  real(r8), parameter:: mwBC10  = 9999.
  real(r8), parameter:: mwBC11= 9999.
  real(r8), parameter:: mwBC12 = 9999.

  real(r8), parameter::  gravit = SHR_CONST_G

contains

!===============================================================================
  subroutine bcarbon_register_cnst
!----------------------------------------------------------------------- 
! 
! Purpose: register advected constituents for all aerosols
! 
! Method: 
! Adopt the registering method of CUACEAero which defines 12 bins of black carbon species
!---------------------------Local workspace-----------------------------
    integer :: m, i                                ! tracer index
    integer :: istat
    real(r8), parameter :: zero  = 0._r8
    character names*8
!-----------------------------------------------------------------------

! Define constituent names
    ncnst = bcarboncyc_ncomp()  !liuqx 20070906
 
    if ( ncnst > 0 ) then
    allocate( cnst_names(ncnst), stat=istat )
    if ( istat /= 0 ) then
      call endrun ('BCARBON_REGISTER_CNST: ERROR: allocate of cnst_names failed') !liuqx 20070906
    end if
    end if

    select case ( iaero )
    case (0)
       call endrun('BCARBON_REGISTER_CNST:  code under development')
    case (1)
       if ( ncnst < 1 ) then
       call endrun('BCARBON_REGISTER_CNST:  the selected species are not correct')
       end if
       names = 'BC01'
       do i = 1, ncnst
          write(names(3:4),'(i2.2)') i
          cnst_names(i) = names
!  Set names of variables undergoing evolution
!  returns m as current index for tracer numbers
!         call cnst_add(cnst_names(i), advected, mwBC1, cpBC1, zero, m, mixtype='dry')
          call cnst_add(cnst_names(i), advected, mwBC1, cpBC1, zero, m)
          if ( i == 1 ) then
             call bcyc_idx_set(m)
          endif
       enddo

    case default
       call endrun('BCARBON_REGISTER_CNST:  unknown scheme')
    end select

  end subroutine bcarbon_register_cnst

!=======================================================================
  function bcarbon_implements_cnst(name)
!----------------------------------------------------------------------- 
! 
! Purpose: return true if specified constituent is implemented by this 
!          package
! 
!-----------------------------------------------------------------------
!-----------------------------Arguments---------------------------------

     character(len=*), intent(in) :: name   ! constituent name
     logical :: bcarbon_implements_cnst      ! return value
     integer :: m
!-----------------------------------------------------------------------

        bcarbon_implements_cnst = .false.
        do m = 1, ncnst
        if (name == cnst_names(m)) then
           bcarbon_implements_cnst = .true.
           exit
        end if
        end do

  end function bcarbon_implements_cnst


!=======================================================================
  subroutine bcarbon_init_cnst(name, q)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Set initial mass mixing ratios.
!
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8

!-----------------------------Arguments---------------------------------
    
    character(len=*), intent(in) :: name         ! constituent name
    
    real(r8), intent(out) :: q(:,:,:)            !  mass mixing ratio
!-----------------------------------------------------------------------
    
    q = 0._r8
!    call endrun('BCARBON_INIT_CNST:  code under development')

  end subroutine bcarbon_init_cnst


!===============================================================================
  subroutine bcarbon_initialize 
!----------------------------------------------------------------------- 
! 
! Purpose: initialize parameterization of bcarbon chemistry
!          (declare history variables)
! 
! Method: 
!-----------------------------------------------------------------------
    use history,    only: addfld, add_default, phys_decomp  ! wzz, 2008.8.6
!-----------------------------------------------------------------------
    integer i
!-----------------------------------------------------------------------

!    call endrun('BCARBON_INITIALIZE:  code under development')
    select case ( iaero )
    case (0)
       call endrun('BCARBON_INITIALIZE:  code under development')

    case (1)
       do i = 1, ncnst
! wzz, 2008.8.3
          call addfld (trim(cnst_names(i))//'DRY','kg/m2/s ',1, 'A', &
                      'BC dry deposition',phys_decomp)
          call add_default (trim(cnst_names(i))//'DRY', 1, ' ')
!
          call addfld (trim(cnst_names(i))//'WETB','kg/m2/s ',1, 'A', &
                      'BC wet deposition below cloud',phys_decomp)
          call add_default (trim(cnst_names(i))//'WETB', 1, ' ')
!
          call addfld (trim(cnst_names(i))//'WETI','kg/m2/s ',1, 'A', &
                      'BC wet deposition in cloud',phys_decomp)
          call add_default (trim(cnst_names(i))//'WETI', 1, ' ')
       enddo

    case default
       call endrun('BCARBON_INITIALIZE:  unknown scheme')
    end select
!  if ( masterproc ) write(0,*)'BCARBON_INITIALIZE: code under development'

  end subroutine bcarbon_initialize


!===============================================================================
  subroutine bcarbon_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
       evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to we processing of aerosols (source and sinks).
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols, pver
    use constituents,only: ppcnst

!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer, intent(in) :: nstep
    integer, intent(in) :: lat(pcols)                  ! latitude index for S->N storage
    real(r8), intent(in) :: clat(pcols)                    ! latitude 
    real(r8), intent(in) :: cme(pcols,pver)            ! local condensation of cloud water
    real(r8), intent(in) :: prain(pcols,pver)            ! production of rain
    real(r8), intent(in) :: evapr(pcols,pver)            ! evaporation of rain
    real(r8), intent(in) :: cldn(pcols,pver)            ! cloud fraction
    real(r8), intent(in) :: cldc(pcols,pver)            ! convective cloud fraction
    real(r8), intent(in) :: cldv(pcols,pver)            ! cloudy volume undergoing scavenging

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies
    real(r8), intent(inout)  :: cflx(pcols,ppcnst)       ! Surface constituent flux (kg/m^2/s)

    real(r8), intent(inout) :: fracis(pcols,pver,ppcnst)         ! fraction of transported species that are insoluble

    real(r8), intent(in) :: conicw(pcols, pver)
    real(r8), intent(in) :: cmfdqr(pcols, pver)
    real(r8), intent(in) :: rainmr(pcols, pver) ! rain mixing ratio
    real(r8) :: calday        ! current calendar day

!-----------------------------------------------------------------------

    call endrun('BCARBON_WET_INTR:  code under development')


  end subroutine bcarbon_wet_intr

  subroutine bcarbon_drydep_intr (state, ptend, wvflx, dt, lat, clat, &
       fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, month, landfrac, &
       icefrac, ocnfrac)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to parameterized greenhouse gas chemisty (source/sink).
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols

!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer, intent(in) :: lat(pcols)                  ! latitude index for S->N storage
    real(r8), intent(in) :: clat(pcols)                 ! latitude 
    real(r8), intent(in) :: fsds(pcols)                 ! longwave down at sfc
    real(r8), intent(in) :: obklen(pcols)                 ! longwave down at sfc
    real(r8), intent(in) :: ustar(pcols)                  ! sfc fric vel
    real(r8), intent(in) :: ts(pcols)                     ! sfc temp
    real(r8), intent(in) :: landfrac(pcols)               ! land fraction
    real(r8), intent(in) :: icefrac(pcols)                ! ice fraction
    real(r8), intent(in) :: ocnfrac(pcols)                ! ocean fraction
    real(r8), intent(in) :: hflx(pcols)                  ! sensible heat flux
    real(r8), intent(in) :: prect(pcols)                     ! prect
    real(r8), intent(in) :: snowh(pcols)                     ! snow depth
    real(r8), intent(in) :: pblh(pcols)                     ! pbl height
    integer, intent(in)  :: month
    real(r8), intent(in) :: wvflx(pcols)       ! water vapor flux

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies

!-----------------------------------------------------------------------

    call endrun('BCARBON_DRYDEP_INTR:  code under development')

  end subroutine bcarbon_drydep_intr

  subroutine bcarbon_time_interp

    call endrun('BCARBON_TIME_INTERP:  code under development')

  end subroutine bcarbon_time_interp

  subroutine bcarbon_emis_intr (state, ptend, dt)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to emission of all bcarbons
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend

!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies

    call endrun('BCARBON_EMIS_INTR:  code under development')

  end subroutine bcarbon_emis_intr 

!#######################################################################

   integer function bcarboncyc_ncomp()
#include <aero_chem_order_cam9.cdk>
    character*8 aeroname(aerochem_np)
    COMMON /CAM_C/ AERONAME
    integer i

    bcarboncyc_ncomp = 0

    if ( iaero == 1 ) then
      do i = 1, aerochem_np
         if ( AERONAME(i) == 'BLCARBON' ) then
            bcarboncyc_ncomp = aero_nsize
            exit
         end if
      enddo
    endif

   end function bcarboncyc_ncomp

   subroutine bcyc_idx_set(m)
     ! store the start index of the BC species
     integer m
!     write (6,*) ' bcyc: bcyc_idx_set to ', m
     bcarbon_idx1 = m
   end subroutine bcyc_idx_set

   integer function bcyc_idx1()
      bcyc_idx1 = bcarbon_idx1
   end function bcyc_idx1

end module bcarbon_intr
