      SUBROUTINE AEROPROP (NTR,      NTP,      ILG,     ILEV,      IL1,  &
     &                     IL2,    ISIZE,   RHSIZE, AEROSIZE,     RHOP,  &
     &                   RHOP0,    RHROW,    THROW,    RGRID,  AERONUM,  &
     &                  TOTMAS,     PHIX,     IAE1, AERONAME,  TRWTROW,  &
     &                     FMO,     FMSO,   PHIAW1,      AMW,  SAVERAD,  &
     &                     ANU,        A,     BMIX,      FR1,       FC,  &
     &                     FRC,      AWX,   DELIQS,   RECRYS,   ROAROW,  &
     &                     SHJ,   PRESSG,    PDIFF,    PDEPV,      MAE)
!-----------------------------------------------------------------------
!     PURPOSE:
!     --------
!     AEROSOL PROPERTY CALCULATION                                     
!
!     HISTORY:
!     --------
!     * OCT 21/02 - G. GONG     TAKE THE DRY DENSITY OUT OF THIS ROUTINE TO
!                               BE USED IN COAGD BY RECRYS ARRAY.
!     * SEP 4/99  - G. GONG     ADD THE AEROSOL GRAVITATIONAL SETTLING
!     *                         VELOCITY CALCULATION WHICH WAS DONE
!     *                         SEVERAL TIMES IN OTHER PLACES. 
!     * MAY  5/99 - G. LESINS   ADDED ORGANIC AEROSOL TYPE - TREATED AS SULPHATE
!     * JUN 19/98 - G. LESINS   ADDED NITRATE, BC, DUST
!     *                         ADDED DELIQUESCENCE & RECRYST DATA
!     * MAR 13/98 - G. LESINS   MODIFIED PARAMETERIZATION
!     *                         VALID FOR ALL SIZES AND RH
!     *                         USES EMPIRICAL OSMOTIC COEFFICIENTS
!     * JUN 25/97 - S.L. GONG   COMBINE PREVIOUS AERODEN AND GROWTH
!     *                         ROUTINES TO COMPUTE THE AEROSOL
!     *                         AMBIENT PROPERTIES.
!     *
!     * JAN 19/96 - S.L. GONG   VECTORIZED THE WHOLE PROGRAM AND ADD
!     *                         WORKING SPACES.
!     *
!     * DEC  5/94 - S.L. GONG   FIRST VERSION
!     *
!
!     ARGUMENT LIST
!     NTR - TOTAL NUMBER OF TRACE SUBSTANCES (GASES AND AEROSOLS)
!     NTP - NUMBER OF AEROSOL TYPES
!     ILG - NUMBER OF LONGITUDE GRID POINTS
!     ILEV - NUMBER OF VERTICAL LEVELS
!     IL1 - MINIMUM INDEX FOR ILG
!     IL2 - MAXIMUM INDEX FOR ILG
!     ISIZE - NUMBER OF SIZE BINS
!     RHSIZE - WET RADIUS ***OUTPUT***
!     AEROSIZE - DRY RADIUS
!     RHOP - FINAL WET DENSITY ***OUTPUT***
!     RHOP0 - DENSITY FOR EACH AEROSOL TYPE
!     RHROW - RELATIVE HUMIDITY
!     THROW - TEMP
!     RGRID - MASS MIXING RATIO FOR EACH TRACE SUBSTANCE
!     AERONUM - NUMBER CONCONCENRATION (#/KG)  ***OUTPUT***
!     TOTMAS - TOTAL MASS MIXING RATIO FOR EACH BIN (ALL COMPONENTS)
!     TRWTROW - AEROSOL LIQUID WATER CONTENT FOR EACH BIN. 
!                 ***OUTPUT***
!     IAE1 - INDEX OF FIRST AEROSOL IN TRACE SUBSTANCE LIST (NTR)
!     AERONAME - NAME OF EACH AEROSOL TYPE
!     THE REMAINING ARGUMENTS ARE WORKING ARRAYS ADDED BY LESINS
!     PHIX - OSMOTIC COEFFICIENT
!     FMO - MASS FRACTION OF EACH DRY COMPONENT
!     FMSO - MASS FRACTION OF TOTAL SOLUBLE PART
!     PHIAW1 - OSMOTIC COEFFICIENT AT WATER ACTIVITY=1
!     AMW - AVERAGE MOLECULAR WEIGHT OF SOLUBLE PART
!     ANU - AVERAGE NU (ION NUMBER) OF SOLUBLE PART
!     A - SURFACE TENSION COEFFICIENT IN KOHLER EQUATION
!     BMIX - SOLUTE COEFFICIENT IN KOHLER EQUATION
!     FR1 - ESTIMATED RADIUS RATIO AT RELATIVE HUMIDITY=1
!     FC - CRITICAL RELATIVE HUMIDITY (SUPERSATURATED)
!     FRC - CRITICAL RADIUS RATIO
!     AWX - ITERATED VALUE FOR WATER ACTIVITY
!     DELIQS - AVERAGE DELIQUESCENCE POINTS FOR MIXTURE
!     RECRYS - AVERAGE RECRYSTALLIZATION POINTS FOR MIXTURE
!
!     *
!-----------------------------------------------------------------------     
      implicit none
!
! IN THE PARAMETER STATEMENT:
!   NUMSOL = TOTAL NUMBER OF SOLUBLE AEROSOL TYPES POSSIBLE
!   NUMINS = TOTAL NUMBER OF INSOLUBLE AEROSOL TYPES POSSIBLE
      integer NUMSOL, NUMINS, NUMTYPES
      PARAMETER (NUMSOL=4, NUMINS=2, NUMTYPES=NUMSOL+NUMINS)
!
      integer NTR, NTP, ILG, ILEV, IL1, IL2, ISIZE
      integer IAE1, MAE
      real SAVERAD
      REAL RHROW(ILG,ILEV+1), THROW(ILG,ILEV+1), RHOPD(ILG,ILEV,ISIZE)
      REAL RHSIZE(ILG,ILEV,ISIZE),RHOP(ILG,ILEV,ISIZE), RHOP0(NTP)
      REAL AEROSIZE (2,ISIZE),RGRID(ILG,ILEV,NTR),TOTMAS(ILG,ILEV,ISIZE)
      REAL AERONUM(ILG,ILEV,ISIZE),TRWTROW(ILG,ILEV,ISIZE)
      CHARACTER*8 AERONAME(NTP), SORTNAME(NUMTYPES)
      
! FOLLOWING SECTION CONTAINS DATA ARRAYS FOR ALL AEROSOL TYPES
      REAL NU(NUMTYPES),MW(NUMTYPES),PHIK(NUMTYPES),B(NUMTYPES)
      REAL DELIQ(NUMTYPES),RECRY(NUMTYPES)
      REAL PHIT(11,NUMSOL)
      REAL NU_(NUMTYPES),MW_(NUMTYPES)
      REAL DELIQ_(NUMTYPES),RECRY_(NUMTYPES)
      REAL PHIT_(11,NUMSOL)
      CHARACTER*8 NAMSOL(NUMSOL), NAMINS(NUMINS)

! FOLLOWING SECTION CONTAINS WORKING ARRAYS   
      REAL AVESIZE(100)
      REAL FMO(ILG,ILEV,ISIZE,NTP),FMSO(ILG,ILEV,ISIZE)
      REAL PHIAW1(ILG,ILEV,ISIZE),AMW(ILG,ILEV,ISIZE)
      REAL ANU(ILG,ILEV,ISIZE),A(ILG,ILEV,ISIZE)
      REAL BMIX(ILG,ILEV,ISIZE),FR1(ILG,ILEV,ISIZE)
      REAL FC(ILG,ILEV,ISIZE),FRC(ILG,ILEV,ISIZE)
      REAL AWX(ILG,ILEV,ISIZE,2), PHIX(ILG,ILEV,ISIZE)
      REAL DELIQS(ILG,ILEV,ISIZE),RECRYS(ILG,ILEV,ISIZE)

      REAL PDIFF(ILG,ILEV,ISIZE),PDEPV(ILG,ILEV,ISIZE)
      REAL PRESSG(ILG),SHJ(ILG,ILEV),ROAROW(ILG,ILEV)      
      REAL MWW,RW,DENW,SFCTEN
      REAL*8 R,Q,D
      INTEGER NTPS, NUMITER, SORTNUM(NUMTYPES), LN(NUMTYPES)
      LOGICAL DELIQCRY

      REAL WW,TW,RAYON,ASQ,G,RGAS,RGOCP,  &
     &     RGOASQ,CPRES,RGASV,CPRESV
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV

      real AVNO,    RGASi,     AM,   BOLTZK,  PI,  Aa
      real A0,RSN0,RCUT0,RCG0,P
      COMMON /NARCM/ AVNO,    RGASi,     AM,   BOLTZK,  PI, Aa
      COMMON /NARCM/ A0,RSN0,RCUT0,RCG0,P
!
      integer I, J, L, N, NO, NT, NT0, ITER, ITRx
      integer ILAST, INEXT, IFOUND
      real FRX1, FRX3, BPR, RH, DSR
      real AA1, AA2, AA3, AWX1, AW2, AW3, AW4, AMU
      real dd, FF, vv, POP, WMAS, WATE, PRE
      real PPT, AMFP, FRCTEST, PRII, PRIIV
      real THETA, R3D, RWI, CFAC, TRAMASS
      real TAUREL, AMOB, CUB, OCFACTOR

!xjx-----------20090821
      real CVMGT
      external CVMGT
!xjx-----end

! AEROSOL TYPE DATA ARE IN THE FOLLOWING ORDER:
!  1. SEA SALT
!  2. (NH4)2SO4
!  3. NH4NO3
!  4. ORGANIC
!  5. BLACK CARBON
!  6. SOIL DUST
!
! NAMSOL ARE THE PERMITTED SOLUBLE AEROSOL NAMES
      DATA NAMSOL/'SEA-SALT','SULPHATE','NITRATES','OMCARBON'/
! NAMINS ARE THE PERMITTED INSOLUBLE AEROSOL NAMES
      DATA NAMINS/'BLCARBON','SOILDUST'/
      
! MW ARE THE MOLECULAR WEIGHTS OF THE DRY AEROSOL COMPONENTS      
      DATA MW_ /67.180, 132.1342, 80.0435, 132.1342, 12.011, 60.08/
!      
! NU ARE THE IONS PER SOLUTE MOLECULE
      DATA NU_ /2.165, 3.0, 2.0, 3.0, 0., 0./

! DELIQ IS THE DELIQUESCENCE RELATIVE HUMIDITY
! RECRY IS THE RECRYSTALLIZATION RELATIVE HUMIDITY
      DATA DELIQ_/0.74, 0.80, 0.62, 0.80, 1.1, 1.1/
      DATA RECRY_/0.45, 0.37, 0.25, 0.37, 1.1, 1.1/
!      
! MWW IS THE MOLECULAR WEIGHT OF WATER 
      DATA MWW/18.015/
!      
! RW IS THE GAS CONSTANT FOR WATER VAPOUR ( J/(KG K) )
      DATA RW/461.51/
!     
! DENW IS THE DENSITY OF WATER ( KG / M^3 )
      DATA DENW/1000./
!      
! SFCTEN IS THE SURFACE TENSION BETWEEN WATER AND AIR ( J / M^2 )
      DATA SFCTEN/0.076/
!
! NTPS IS THE INDEX NUMBER OF THE LAST SOLUBLE AEROSOL TYPE (<= NTP)
! IF NTPS=NTP THEN THERE ARE NO INSOLUBLE COMPONENTS
!      
! NUMITER IS THE NUMBER OF ITERATIONS PERFORMED FOR KOHLER EQUATION
! NUMITER=2 SHOULD GIVE BETTER THAN 1% ACCURACY
! NUMITER MUST NOT EXCEED THE DECLARED SIZE OF THE LAST INDEX IN AWX
      DATA NUMITER/2/
!
! OCFACTOR IS A FACTOR USED TO MULTIPLY THE ORGANIC CARBON OSMOTIC
!   COEFFICIENTS TO FORCE A DIFFERENCE FROM SULPHATE
      DATA OCFACTOR/0.54/
!
! PHIT(J,I) is the phi(aw) table of coefficients
! I is the solute index:
!     1 = sea salt
!     2 = (NH4)2SO4
!     3 = NH4NO3
!     4 = organic
! I MUST BE DIMENSIONED TO EQUAL NTPS
! J is the data :
!  1 = minimum aw for polynomial fit
!  2 = aw break point between two polynomials
!  3 = x^3 coef for aw > awbreak
!  4 = x^2 coef
!  5 = x^1 coef
!  6 = x^0 coef
!  7 = x^4 coef for aw <= awbreak
!  8 = x^3 coef
!  9 = x^2 coef
!  10= x^1 coef
!  11= x^0 coef
      DATA PHIT_/0.44,0.92, 410.74729, -1138.2693, 1049.2792,-320.74562,  &
     & -5.79690208, 17.7685336, -22.5253540, 11.8087027, -0.48210984,  &
     & 0.39, 0.92, 457.060777, -1280.47495, 1194.81750, -370.739425,  &
     & -1.62440470, 4.07342346, -5.61205075, 3.873682106, -0.216021389,  &
     & 0.275, 0.81, 7.6174049, -19.354181, 17.103802, -4.5561686,  &
     & -1.1108526, 3.7035588, -5.1408203, 4.0788267, -0.77326108,  &
     & 0.39, 0.92, 457.060777, -1280.47495, 1194.81750, -370.739425,  &
     & -1.62440470, 4.07342346, -5.61205075, 3.873682106, -0.216021389/
!
!  DELIQCRY IS A LOGICAL SWITCH TO HANDLE SIZE IF RH IS BETWEEN
!    CRYSTALLIZATION AND DELIQUESCENCE.  IF =.TRUE. THEN FINAL SIZE IS
!    WEIGHTED AVERAGE BETWEEN DRY AND DELIQUESCED SIZES.  IF =.FALSE.
!    THEN FINAL SIZE IS THE FULLY DELIQUESCED SIZE.
      DATA DELIQCRY/.FALSE./
      DATA AA1/1.257/, AA2/0.4/, AA3/1.1/
!
      CALL PUTZERO(RHOP,   ILG*ILEV*ISIZE)
      CALL PUTZERO(TOTMAS, ILG*ILEV*ISIZE)
      CALL PUTZERO(FMO,    ILG*ILEV*ISIZE*NTP)
      CALL PUTZERO(FMSO,   ILG*ILEV*ISIZE)
      CALL PUTZERO(PHIAW1, ILG*ILEV*ISIZE)
      CALL PUTZERO(AMW,    ILG*ILEV*ISIZE)
      CALL PUTZERO(ANU,    ILG*ILEV*ISIZE)
      CALL PUTZERO(DELIQS, ILG*ILEV*ISIZE)

      CALL PUTZERO(RECRYS, ILG*ILEV*ISIZE)

      CUB=1./3.
      
!        SORT THE AEROSOL NAMES (SOLUBLES FIRST, THEN INSOLUBLES)
      INEXT=1
      ILAST=NTP
      DO NT=1,NTP
         IFOUND=0
         DO J=1,NUMSOL
            IF (AERONAME(NT) .EQ. NAMSOL(J)) THEN
               SORTNAME(INEXT) = AERONAME(NT)
               SORTNUM(INEXT) = J
               LN(INEXT) = NT
               INEXT = INEXT + 1
               IFOUND = 1
            ENDIF
         ENDDO
         IF (IFOUND .EQ. 0) THEN
            DO J=1,NUMINS
               IF (AERONAME(NT) .EQ. NAMINS(J)) THEN
                  SORTNAME(ILAST) = AERONAME(NT)
                  SORTNUM(ILAST) = J + NUMSOL
                  LN(ILAST) = NT
                  ILAST = ILAST - 1
                  IFOUND = 1
               ENDIF
            ENDDO
         ENDIF
         IF (IFOUND .EQ. 0) THEN     
            WRITE(6,*) '***ERROR*** UNKNOWN AEROSOL NAME'
            STOP   
         ENDIF
         IF (NT .EQ. NTP) NTPS = INEXT - 1
      ENDDO
!
!      DO NT=1,NTP
!         WRITE(6,600) AERONAME(NT),LN(NT),SORTNAME(NT),SORTNUM(NT)
! 600     FORMAT(1X,A8,5X,I1,5X,A8,5X,I1)
!      ENDDO
!
!
!  ADJUST OSMOTIC COEFFICIENTS FOR ORGANIC CARBON
!      DO J=3,11
!         PHIT_(J,4)=OCFACTOR*PHIT_(J,4)   !??? wzz
!      ENDDO
!      write(0,*)'wzz: PHIT=', phit_(3,4), phit_(1,4)
        
!  ARRANGE THE DATA ARRAYS IN THE CORRECT ORDER
      DO NT=1,NTP
         MW(NT)=MW_(SORTNUM(NT))
         NU(NT)=NU_(SORTNUM(NT))
         DELIQ(NT)=DELIQ_(SORTNUM(NT))
         RECRY(NT)=RECRY_(SORTNUM(NT))
      ENDDO
      DO NT=1,NTPS
         DO J=1,11
            PHIT(J,NT)=PHIT_(J,SORTNUM(NT))
         ENDDO
         if ( SORTNUM(NT) .eq. 4) then   ! wzz, 2008.5.6
            do j =3,11
               PHIT(J,NT) = OCFACTOR*PHIT(J,NT)
            enddo
         endif
      ENDDO

!
!     * TOTAL DRY MASS MIXING RATIO & DRY AEROSOL COMPOSITE DENSITY
!       OF AEROSOL IN EACH BIN
!
!                    m1+m2+m3  
!       RHO = -------------------------
!              m1/rho1+m2/rho2+m3/rho3
!
!       IN THIS DO LOOP, RHOP(*,*,*) IS ONLY HOLDING THE DENOMINATOR
!       OF THIS EQUATION

      DO NT=1,NTP
         NT0=LN(NT)
         DO N=1,ISIZE
            NO= N+ISIZE*(NT0-1)+(IAE1-1)
            DO L=1+MAE,ILEV
              DO I=IL1,IL2
                 TRAMASS=AMAX1(1.0E-33, RGRID(I,L,NO))
                 TOTMAS(I,L,N)=TOTMAS(I,L,N)+TRAMASS
                 RHOP(I,L,N)=RHOP(I,L,N)+TRAMASS/RHOP0(NT0)
              END DO
            END DO
         END DO
      END DO
!
! COMPUTE THE MASS FRACTION OF EACH DRY AEROSOL COMPONENT, FMO 
!     
      DO NT=1,NTP
         NT0=LN(NT)
         DO N=1,ISIZE
            NO= N+ISIZE*(NT0-1)+(IAE1-1)
            DO L=1+MAE,ILEV
              DO I=IL1,IL2
                 TRAMASS=AMAX1(1.0E-33, RGRID(I,L,NO))
                 FMO(I,L,N,NT)=TRAMASS/TOTMAS(I,L,N)
              END DO
            END DO
         END DO
      END DO
!
!      COMPUTE PHI AT AW=1 FOR EACH SOLUTE
!      COMPUTE KOHLER B FACTOR FOR EACH SOLUTE
!
      DO NT=1,NTPS
         PHIK(NT)=PHIT(3,NT)+PHIT(4,NT)+PHIT(5,NT)+PHIT(6,NT)
         B(NT)=NU(NT)*PHIK(NT)*MWW*RHOP0(LN(NT))/(MW(NT)*DENW)
      END DO
!      
! COMPUTE THE SOLUTE MASS FRACTION, FMSO      
! COMPUTE AVERAGE NU AND AVERAGE MOLECULAR WEIGHT OF SOLUBLE PART
! COMPUTE PHI AT AW=1 FOR THE MIXED AEROSOL
! COMPUTE AVERAGE DELIQUESCENCE AND RECRYSTALLIZATION POINTS
      DO NT=1,NTPS
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
           DO I=IL1,IL2
              FMSO(I,L,N)=FMSO(I,L,N)+FMO(I,L,N,NT)
              AMW(I,L,N)=AMW(I,L,N)+FMO(I,L,N,NT)/MW(NT)
              ANU(I,L,N)=ANU(I,L,N)+NU(NT)*FMO(I,L,N,NT)/MW(NT)
              PHIAW1(I,L,N)=PHIAW1(I,L,N)+PHIK(NT)*NU(NT)*FMO(I,L,N,NT)  &
     &                        /MW(NT)
              DELIQS(I,L,N)=DELIQS(I,L,N)+FMO(I,L,N,NT)*DELIQ(NT)
              RECRYS(I,L,N)=RECRYS(I,L,N)+FMO(I,L,N,NT)*RECRY(NT)
           END DO
         END DO
      END DO
      END DO
!
! COMPUTE THE AVERAGE DRY AEROSOL RADIUS, AVESIZE
!
      DO N=1,ISIZE
         RWI=(AEROSIZE(1,N)+AEROSIZE(2,N))*0.5
         R3D=4.189*RWI*RWI*RWI
         AVESIZE(N)=RWI
         DO L=1+MAE,ILEV
           DO I=IL1,IL2
              AERONUM(I,L,N)=RHOP(I,L,N)/R3D    ! wzz, 2008.8.20
              RHOP(I,L,N)=TOTMAS(I,L,N)/RHOP(I,L,N)
              IF (FMSO(I,L,N) .NE. 0.) THEN
                 PHIAW1(I,L,N)=PHIAW1(I,L,N)/ANU(I,L,N)  ! wzz
                 ANU(I,L,N)=ANU(I,L,N)/AMW(I,L,N)   ! wzz, 2008.8.20
                 AMW(I,L,N)=FMSO(I,L,N)/AMW(I,L,N)
!                ANU(I,L,N)=ANU(I,L,N)*AMW(I,L,N)/FMSO(I,L,N)
!                PHIAW1(I,L,N)=PHIAW1(I,L,N)*AMW(I,L,N)/(FMSO(I,L,N)*
!    1                        ANU(I,L,N))
                 DELIQS(I,L,N)=DELIQS(I,L,N)/FMSO(I,L,N)
                 RECRYS(I,L,N)=RECRYS(I,L,N)/FMSO(I,L,N)
              ELSE
                 DELIQS(I,L,N)=1.1
                 RECRYS(I,L,N)=1.1
              END IF
!
!     * AEROSOL NUMBER CONCENTRATION (#/KG_AIR)
!      NOTE:  4.189 = 4 * PI / 3
!
!             AERONUM(I,L,N)=TOTMAS(I,L,N)/(R3D*RHOP(I,L,N))
           END DO
         END DO
      END DO
      
!      WRITE(6,*) 'FMO:'
!      WRITE(6,*) (FMO(1,1,1,M),M=1,NTP)
!      WRITE(6,*) 'FMSO= ',FMSO(1,1,1)
!      WRITE(6,*) 'MW:'
!      WRITE(6,*) (MW(M),M=1,NTP)
!      WRITE(6,*) 'AMW= ',AMW(1,1,1)
!      WRITE(6,*) 'NU:'
!      WRITE(6,*) (NU(M),M=1,NTP)
!      WRITE(6,*) 'ANU= ',ANU(1,1,1)
!      WRITE(6,*) 'RHOP0:'
!      WRITE(6,*) (RHOP0(LN(M)),M=1,NTP)
      
!
!  COMPUTE KOHLER A' FACTOR AND B FACTOR FOR MIXTURE
!  COMPUTE RADIUS RATIO AT RH=1 (FR1)
!  COMPUTE CRITICAL RADUS RATIO (FRC)
!  COMPUTE CRITICAL RELATIVE HUMIDITY (FC)    
!
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
           DO I=IL1,IL2
              A(I,L,N)=2.0*SFCTEN/(DENW*RW*THROW(I,L+1)*AVESIZE(N))
              IF (AMW(I,L,N) .NE. 0) THEN
                 BMIX(I,L,N)=ANU(I,L,N)*PHIAW1(I,L,N)*MWW*RHOP(I,L,N)*  &
     &                     FMSO(I,L,N)/(AMW(I,L,N)*DENW)
              ELSE
                 BMIX(I,L,N)=0.0
              END IF
              Q=-BMIX(I,L,N)/(3.0*A(I,L,N))
!              R=0.5
              D=Q*Q*Q+0.25
              IF(D .LT. 0.0 .AND. Q .LT. 0.0) THEN
                 THETA=ACOS(0.5/SQRT(-Q*Q*Q))
                 FR1(I,L,N)=2.0*SQRT(-Q)*COS(THETA/3.0)
              ELSE
                 DSR=SQRT(D)
                 FR1(I,L,N)=(0.5+DSR)**CUB+(0.5-DSR)**CUB
              END IF

!*******************
!  THIS SECTION CAN BE COMMENTED OUT IF YOU ARE NOT INTERESTED
!    IN COMPUTING THE CRITICAL RADIUS AND CRITICAL RELATIVE HUMIDITY

              Q=Q*3.0
!              R=1.0
              D=Q*Q*Q+1.0
              IF(D .LT. 0.0 .AND. Q .LT. 0.0) THEN
                 THETA=ACOS(1.0/SQRT(-Q*Q*Q))
                 FRC(I,L,N)=2.0*SQRT(-Q)*COS(THETA/3.0)
              ELSE
                 DSR=SQRT(D)
                 FRC(I,L,N)=(1.0+DSR)**CUB+(1.0-DSR)**CUB
              END IF
!             FRCTEST=FRC(I,L,N)**3.0
              FRCTEST=FRC(I,L,N)
              FRCTEST=FRCTEST*FRCTEST*FRCTEST
              IF (FRCTEST .EQ. 1.0) THEN
                 FC(I,L,N)=1.0
              ELSE
                 FC(I,L,N)=EXP(A(I,L,N)/FRC(I,L,N)-BMIX(I,L,N)/   &
     &                  (FRCTEST-1.0))
              END IF
!*********************
!
           END DO
         END DO
      END DO
!
! SOLVE FOR THE WET RADIUS AFTER WATER UPTAKE
      DO ITER=1,NUMITER
         IF (ITER.EQ.1) THEN
            ITRx=1
         ELSE
            ITRx=ITER-1
         END IF
         CALL PUTZERO(PHIX, ILG*ILEV*ISIZE)
         DO NT=1,NTPS
          PPT=NU(NT)/MW(NT)
          DO N=1,ISIZE
            DO L=1+MAE,ILEV
               DO I=IL1,IL2
                 RH=AMIN1(1.0,AMAX1(0.0,RHROW(I,L+1)))
                 IF(ITER .EQ. 1)    &
     &               AWX(I,L,N,ITER)=MIN(1.0,  &
     &                     RH*EXP(-A(I,L,N)/(0.8*FR1(I,L,N))))
                 AWX1=AWX(I,L,N,ITRx)
                 AW2=AWX1*AWX1
!                AW3=AWX1*AWX1*AWX1
                 AW3=AW2*AWX1
                 AW4=AW3*AWX1
                 POP=CVMGT(0.,1.,AWX1 .GT. PHIT(2,NT))
                 
                 PHIX(I,L,N)=PHIX(I,L,N)+  &
     &                       (1.0-POP)*(PHIT(3,NT)*AW3+  &
     &                     PHIT(4,NT)*AW2 +PHIT(5,NT)*AWX1 +  &
     &                      PHIT(6,NT))*PPT*FMO(I,L,N,NT) +  &
     &                     POP*(PHIT(7,NT)*AW4+PHIT(8,NT)*AW3 +  &
     &                      PHIT(9,NT)*AW2 + PHIT(10,NT)*AWX1 +  &
     &                      PHIT(11,NT))*PPT*FMO(I,L,N,NT)

               END DO
            END DO
          END DO
         END DO

         DO N=1,ISIZE
            DO L=1+MAE,ILEV
              DO I=IL1,IL2
                 RH=AMIN1(1.0,AMAX1(0.0,RHROW(I,L+1)))
                 AWX1=AWX(I,L,N,ITRx)
                 IF (FMSO(I,L,N) .NE. 0. .AND. AWX1 .GT. 0.0) THEN
                    PHIX(I,L,N)=PHIX(I,L,N)*AMW(I,L,N)/(FMSO(I,L,N)  &
     &                                       *ANU(I,L,N))
                    FRX1=(1.0-ANU(I,L,N)*PHIX(I,L,N)*MWW*RHOP(I,L,N)*  &
     &                  FMSO(I,L,N)/(AMW(I,L,N)*DENW*LOG(AWX1)))**CUB
                 ELSE
                    PHIX(I,L,N)=0.
                    FRX1=1.0
                 END IF
!
!*********************
!  THIS SECTION GREATLY IMPROVES ACCURACY FOR RH ~ 1    
!  IT CAN BE COMMENTED OUT IF SPEED IS MORE IMPORTANT THAN ACCURACY
! 
                 IF(RH .GT. 0.98 .AND. FRX1 .GT. 1.0) THEN
                    FRX3=FRX1*FRX1*FRX1
                    BPR=ANU(I,L,N)*PHIX(I,L,N)*MWW*RHOP(I,L,N)*  &
     &                   FMSO(I,L,N)*FRX3/(AMW(I,L,N)*DENW*(FRX3-1.0))
                    Q=-A(I,L,N)/(3.0*BPR)
                    R=-LOG(RH)/(2.0*BPR)
                    D=Q*Q*Q+R*R
                    IF(D .LT. 0.0 .AND. Q .LT. 0.0) THEN
                       THETA=ACOS(R/SQRT(-Q*Q*Q))
                       FRX1=1.0/(2.0*SQRT(-Q)*COS(THETA/3.0))
                    ELSE
                       vv=ABS(R+SQRT(D))  
                       dd=ABS(R-SQRT(D))
                       FRX1=1.0/((vv)**CUB + (dd)**CUB)
                    END IF
                 END IF
!**********************
!                 
                 AWX(I,L,N,ITER)=RH*EXP(-A(I,L,N)/FRX1)
                 RHSIZE(I,L,N)=AVESIZE(N)*FRX1
              END DO
            END DO
         END DO
      END DO
!
! ADJUST SIZE IF RH < DELIQUESCENCE POINT
! COMPUTE THE MEAN DENSITY OF THE WET AEROSOL
!
      DO N=1,ISIZE
         DO L=1+MAE,ILEV
           DO I=IL1,IL2
              RH=RHROW(I,L+1)
              IF( RH .LT. DELIQS(I,L,N)) THEN
                 IF( RH .LT. RECRYS(I,L,N)) THEN
                    RHSIZE(I,L,N)=AVESIZE(N)
                 ELSE
                    IF( DELIQCRY ) RHSIZE(I,L,N)=AVESIZE(N)+  &
     &                 (RHSIZE(I,L,N)-AVESIZE(N))*  &
     &                 (RH-RECRYS(I,L,N))/(DELIQS(I,L,N)-RECRYS(I,L,N))
                 END IF
              END IF
              FF=AVESIZE(N)/RHSIZE(I,L,N)
              
              RECRYS(I,L,N)=RHOP(I,L,N)  !Assign dry rhop the value for coagd use
              RHOP(I,L,N)=DENW+AMAX1(0.,FF*FF*FF*(RHOP(I,L,N)-DENW))

!
! COMPUTE THE AEROSOL LIQUID WATER CONTENT OF EACH SIZE BIN: WATE
! ACCUMULATE IT FOR OUTPUT [KG/KG AIR]
!

              WMAS=4.189*RHSIZE(I,L,N)**3*AERONUM(I,L,N)*RHOP(I,L,N)
              WATE=AMAX1(0.,WMAS - TOTMAS(I,L,N))
              TRWTROW(I,L,N)=WATE
!
!    AEROSOL GRAVITATIONAL SETTLING VELOCITY
!    AND DIFFUSION COEFFICIENT
!
!     * AIR'S DYNAMIC VISCOSITY
!
              AMU=145.8*1.E-8*THROW(I,L+1)**1.5/(THROW(I,L+1)+110.4)
! . . . . MID LAYER PRESSURE IN [PASCAL].
              PRE = PRESSG(I)*SHJ(I,L)
!
!     * MEAN MOLECULAR FREE PATH.
!       K.V. BEARD [1976], J ATM. SCI., 33
!
              AMFP=6.54E-8*(AMU/1.818E-5)*(1.013E5/PRE)*  &
     &                          (THROW(I,L+1)/293.15)**0.5
!    &                          (THROW(I,L+1)/293.15)**(1./2.)
              PRII=2./9.*G/AMU
              PRIIV=PRII*(RHOP(I,L,N)-ROAROW(I,L))
!
!     * CUNNINGHAM SLIP CORRECTION FACTOR AND RELAXATION TIME = vg/Grav.
!
              CFAC=1.+AMFP/RHSIZE(I,L,N)*(AA1+AA2*  &
     &                                EXP(-AA3*RHSIZE(I,L,N)/AMFP))
              TAUREL=AMAX1(PRIIV*RHSIZE(I,L,N)**2*CFAC/G,0.0)
!
!     * STOKES FRICTION AND DIFFUSION COEFFICIENTS.
!
              AMOB=6.*PI*AMU*RHSIZE(I,L,N)/CFAC
              PDIFF(I,L,N)=BOLTZK*THROW(I,L+1)/AMOB
!
!     * GRAVITATIONAL SETTLING VELOCITY.
!
              PDEPV(I,L,N) =TAUREL*G
           END DO
         END DO
      END DO
!
      RETURN
      END
