      SUBROUTINE DRYPAR  (THROW,    LEV,   ILEV,     ILG,  ISIZE,  NTP,  &
     &                      IL1,    IL2,    NTR,    ADT2,   IAE1,  MAE,  &
     &                   PRESSG,    SHJ,  THLEV,  ROAROW,    ANU,PDIFF,  &
     &                     RHOP, RHSIZE,  FLAND,    XROW,   SCHM,  LUC,  &
     &                   SURFDG, SURFWD,  PDEPV,   RTDRY,   DSHJ,  VDP,  &
     &                     JLAT,  SEASN,     RA,     USI,   PLLP, AEST,  &
     &                       NS,  GAMMA)
!-----------------------------------------------------------------------
!
!     PURPOSE
!     -------
!     - CALCULATION OF DRY DEPOSITION OF AEROSOL PARTICLES.
!
!     HISTORY:
!     --------
!     * SEP 04/1999 - S.L. GONG   THE GRAVITATIONAL VELOCITY IS NOW 
!     *                           DONE IN AEROPROP.
!     *
!     * AUG 01/1999 - S.L. GONG   PASS ALL CONTROL VARIABLES FOR 15
!     *                           LAND-USE FROM THE CONTROL FILE.
!     *
!     * MAR 19/1998 - L.M. ZHANG  MODIFY DRY DEPOSITION PARAMETERIZATION
!     *
!     * DEC 19/1996 - S.L. GONG   VECTORIZED THE WHOLE PROGRAM AND ADD
!     *                           WORKING SPACES.
!     *
!     * AUG  4/1994 - S.L. GONG   FIRST VERSION
!     *
!
!     METHOD
!     ---------
!

!-----------------------------------------------------------------------
!        |                                      | T |            |     |
! NAME   |                                      | Y | DIMENSIONS | IN/ |
!        |                                      | P |            | OUT |
!________|______________________________________|_E_|____________|_____|
! IL1    | starting point of x direction -- 1   | I |            |  I  |
! IL2    | ending point of x direction  -- nis  | I |            |  I  |
! ILEV   | Number of model level                | I |            |  I  |
! ILG    | Number of grid points in x-direction | I |            |  I  |
! ISIZE  | Number of aerosol size bins          | I |            |  I  |
! RHOP   | bin-specific wet aerosol density     | R | ILGVSIZ #  |  I  |
!        |      [kg/m^3]                        |   |            |     |
! RHOP0  | mass density of aerosol chemical     |   |            |     |
!        |      components [kg/m^3]             |   |            |     |
! RADCLD | mean radius of cloud droplets  [um]  | R | ILG*ILEV   |  I  |
! RTDRY  | dry deposition tendency              | R |ILG*ILEV*NTR|  O  |
! RHROW  | Slice of relative humidity [%]       | R | ILG*LEV    |  I  |
! RHSIZE | unactivated ambient aerosol wet      | R | ILGVSIZ #  |  O  |
! THLEV  | layer thickness [m]                  | R | ILG*ILEV   |  I  |
! THROW  | Grid slice of mid-layer tempt. [K]   | R | ILG*LEV    |  I  |
! PRESSG | Grid row of surface pressure [Pa]    | R | ILG        |  I  |
! SURFDG | surface drag coefficient CDM=(U*/U)^2| R | ILG        |  I  |
! SURFWD | surface wind                         | R | ILG        |  I  |
! PDIFF  | Particle diffusion coefficient       | R | ILG*ISIZE  |  I  !
! PDEPV  | dry deposition at first level  [m/s] | R | ILGVSIZ #  |  O  |
!        | gravitional velocity at other levels | R |            |  I  |
! ROAROW | AIR DENSITY    [KG M-3]              | R | ILG*ILEV   |  I  |
! SHJ    | Local mid-layer sigma value          | R | ILG*ILEV   |  O  |
! FLAND  | 15 Land use categories               | R | ILG*LUC    |  L  | 
! XROW   | Tracer array              [kg/kg]    | R | ILG*LEV*NTR|  I  |
! DSHJ   | Sigma value difference between layers| R | ILG*ILEV   |  O  |
! RA     ! Aerodynamic resistance from DRYGAS   ! R | ILG        |  I  |
! USI    | Friction velocity from DRYGAS        ! R ! ILG        |  I  |

      implicit none

      REAL WW,TW,RAYON,ASQ,G,RGAS,RGOCP,   &
     &     RGOASQ,CPRES,RGASV,CPRESV
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV

      real AVNO,    RGASi,     AM,   BOLTZK,  PI,  A
      real A0,RSN0,RCUT0,RCG0,P
      COMMON /NARCM/ AVNO,    RGASi,     AM,   BOLTZK,  PI,  A
      COMMON /NARCM/ A0,RSN0,RCUT0,RCG0,P
!
!     * INPUT ARRAYS...
!
      integer LEV, ILEV, ILG, ISIZE, NTP,  &
     &        IL1, IL2,  NTR, IAE1,  MAE, LUC, NS, JLAT
      real ADT2
      REAL RHOP(ILG,ILEV,ISIZE),PDIFF(ILG,ILEV,ISIZE)
      REAL THROW(ILG,LEV), SHJ(ILG,ILEV), THLEV(ILG, ILEV)
      REAL RHSIZE(ILG,ILEV,ISIZE),PDEPV(ILG,ILEV,ISIZE)
      REAL SURFDG(ILG),SURFWD(ILG)
      REAL PRESSG (ILG), DSHJ(ILG,ILEV), FLAND(ILG,LUC)
      REAL XROW(ILG,LEV,NTR), RTDRY(ILG,ILEV,NTR)
      REAL VDP(ILG),PLLP(LUC,NS),SEASN(ILG),RA(ILG,LUC)
      REAL ROAROW(ILG,ILEV)
      REAL ANU(ILG), SCHM(ILG)
      REAL AEST(LUC),USI(ILG,LUC), GAMMA(LUC)
!  local variables
      integer I, L, IC, N, NN, NP, ISEA
      real AMU, R1, RS, VDF, XXX, QCC, ST, EB, EIN, EIM

!xjx-----------20090821
      real CVMGT
      external CVMGT
!xjx-----end

!-----------------------------------------------------------------------
!
!     * LOOP FOR THE NUMBER OF AEROSOLS (SIZE BINS)
!
!-----------------------------------------------------------------------
!--------------------------------------
!     * FOR THE SURFACE LAYER, DRY DEPOSITION IS CALCULATED.
!     * NOTE: THE GRAVITATIONAL VELOCITY WAS COMPUTED IN AEROPROP.
!--------------------------------------
      L=ILEV
      DO 4000 N=1,ISIZE
          DO I=IL1,IL2
            VDP(I)=0.0
          END DO
          DO 500 IC=1,LUC
           DO 400 I=IL1,IL2
              ISEA = INT(SEASN(I))
              IF (FLAND(I,IC).GE.0.005) THEN
!
!     * AIR'S DYNAMIC VISCOSITY
!
                 AMU=145.8*1.E-8*THROW(I,L+1)**1.5/(THROW(I,L+1)+110.4)
!     * AIR'S KINEMATIS VISCOSITY
!
                 ANU(I)=AMU/ROAROW(I,L)
                 SCHM(I)=ANU(I)/PDIFF(I,L,N)
!
!     * CALCULATE middle-variles needed for EB,EIN,EM, etc., then Ra, Rs, VDF
!
                 IF (PLLP(IC,ISEA).LE.0.) THEN
                      ST=PDEPV(I,L,N)/G*USI(I,IC)*USI(I,IC)/ANU(I)
                 ELSE
                      ST=PDEPV(I,L,N)/G*USI(I,IC)/PLLP(IC,ISEA)*1000.
                 END IF
                 EB= SCHM(I)**GAMMA(IC)
!                EIM=( St/(St+AEST(IC)))**2    ! wzz
                 EIM= St/(St+AEST(IC))
                 EIM= EIM*EIM
                 EIM=AMIN1(EIM,0.8)
                 EIN=0.0
                 IF (PLLP(IC,ISEA) .GT. 0.01)  &
     &              EIN=(1000.0*2.*RHSIZE(I,L,N)/PLLP(IC,ISEA))**2*0.5
                 EIN=AMIN1(EIN,0.6)
                 R1= AMAX1(0.5,  EXP(-ST**0.5))
                 RS= 1./3./USI(I,IC)/(EB+EIM+EIN)/R1
                 VDF = PDEPV(I,L,N) + 1./(RA(I,IC)+RS)
                 VDP(I) = VDP(I)+VDF*FLAND(I,IC)
              END IF
400        CONTINUE
500       CONTINUE

          DO I=IL1,IL2
             PDEPV(I,L,N)=VDP(I)
          END DO
4000  CONTINUE

!--------------------------------------
!     * SECTION 3
!     * TENDENCY CALCULATION.
!--------------------------------------
!
!     * CREATE DEPOSITION TENDENCIES FOR EACH LAYER
!
      DO NN=1,NTP
        DO N = 1,ISIZE
          NP=ISIZE*(NN-1)+N+(IAE1-1)
          DO  L = ILEV, 1+MAE, -1
            DO I = IL1, IL2
                QCC = -PDEPV(I,L,N)*ADT2/THLEV(I,L)
                IF (L.EQ.ILEV) QCC = -PDEPV(I,L,N)*ADT2/(THLEV(I,L)*0.5)
                XXX = CVMGT(XROW(I,L+1,NP),0.0,    &
     &                               XROW(I,L+1,NP) .GT. 1.E-20)
                RTDRY(I,L,NP) = XXX*(EXP(QCC)-1.0)/ADT2

            END DO
          END DO
        END DO
      END DO

      RETURN
      END
