      SUBROUTINE SFSS   (TRAC,  DSHJ,  ILG,   LEV,  ILEV,        &
     &                     R1,    R2,  DF0,   DF1,   DF2, I_SS,  &      
     &                      T,   NTR,   NN,   NTP, ISIZE,  LUC,  &         
     &               AEROSIZE,SURFWD,RHOP0,RSFROW,PRESSG,   GC,  &     
     &                  FLAND,FINTROW, IL1,   IL2,  IAE1,IREST, trnch)
!----------------------------------------------------------------------
!     PURPOSE:
!     --------
!     A subroutine to evaluate the generation rate of sea salt aerosols
!     as a function of meteorologic conditions.
!
!     HISTORY:
!
!     * NOV 04/02 - S.L. GONG   A NEW SCHEME - GONG-MONAHAN [Global 
!                               Biogeochemical Cycles,2003]
!     *                         WAS USED RE REPLACE THE OLD MONAHAN'S
!     *                         SCHEME SO THAT THE APPLIABLE RANGE OF
!     *                         SEA-SALT PRODUCTION IS EXPANDED TO LESS
!     *                         THAN 0.1 MIRON.
!     * OCT 31/01 - S.L. GONG   THE PARTICLE MASS OF EACH SUB-DEVIDED
!     *                         BIN IS NOW COMPUTED IN THE DO IK=1,N*35
!     *                         LOOP: PMASS. IT WAS DONE ON AN AVERAGED
!     *                         MASS IN A BIN. VERSION 5C
!     *
!     * AUG 01/01 - S.L. GONG   THE SURF ZONE PRODUCTION OF SEA SPRAY
!     *                         FUNCTION OF de LEEUW ET AL (2001) WAS   
!     *                         ADDED WITH OCEAN COVER LESS THAN 20% GRID.
!     *
!     * SEP 02/99 - S.L. GONG   ADD THE SMITH'S SEA-SALT SURFACE FLUX
!     *                         FUNCTION. 
!     *
!     * JUN 11/98 - S.L. GONG   FLUX INTEGRATION OVER EACH SIZE BIN IS 
!     *                         DONE NOW WHEN MODEL STARTS OR RESTARTS. 
!     *                         THE INTEGRATED EFFICIENT IS SAVED IN  
!     *                         FINTROW.
!     *
!     * DEC 12/97 - S.L. GONG   NEW LANDUSE CATAGORY FLAND ARRAY IS USED
!     *                         TO REPLACE GC. THE FLAND IS BASED UPON
!     *                         EOS 1x1 km RESOLUTION DATA SETS. 
!     *
!     * DEC 19/96 - S.L. GONG   AS IN PREVIOUS SFSALT BUT
!     *                         OPTIMIZED BY VECTORIZATION
!     *
!     * NOV 21/94 - S.L. GONG   AEROSOL SURFACE FLUX COMPUTATION
!     *                         FOR SEA-SALT
!
!     METHOD:
!     -------
!     Monahan et al (Oceanic Whitecaps, E.C. Monahan
!     and G.Mac Niocail (eds), 1986)
!
!     Smith et al.
!     
!     Gong, Global Biogeochemical Cycles 2003
!
!     ARGUMENTS:
!     ----------
!     SUBROUTINE CALLED FROM SFFLUX
!
!----------------------------------------------------------------------
      implicit none

! arguments
      integer, intent(in) :: ILG, LEV, ILEV
      integer, intent(in) :: NTR
      integer, intent(in) :: ISIZE, NTP, LUC, NN, I_SS
      integer, intent(in) :: IL1, IL2, IAE1, IREST
      integer, intent(in) :: trnch

      REAL, intent(in) :: DSHJ(ILG,ILEV), T(ILG,LEV), GC(ILG)   
      REAL, intent(in) :: FLAND(ILG,LUC)
      REAL, intent(in) :: AEROSIZE(2,ISIZE),SURFWD(ILG),RHOP0(NTP),PRESSG(ILG)
      REAL, intent(inout) :: TRAC(ILG,LEV,NTR)
      REAL, intent(out)   :: RSFROW(ILG,NTR), FINTROW(ILG,ISIZE,3)
      REAL, intent(out)   :: R1(ILG), R2(ILG), DF0(ILG), DF1(ILG), DF2(ILG)                               
!
! common blocks
      real WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES
      real RGASV,CPRESV
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,G,RGAS,RGOCP,RGOASQ,CPRES        
      COMMON /PARAMS/ RGASV,CPRESV                                     
      real DELT
      integer IFDIFF,KSTART,KTOTAL,NEWRUN,NDAYS,NSECS,IDAY,LDAY,MDAY,INCD
      COMMON  /TIMESc/ DELT,IFDIFF,KSTART,KTOTAL,NEWRUN,        &
     &                NDAYS,NSECS,IDAY,LDAY,MDAY,INCD                  
!
! local variables
      integer I, IK, N, NP, N35
      real RR1, RR2, PVOL, PMASS, DELR0, A1, A2, CC3, DEL, RWI
      real B, FLX0, FLX1, FLX2, FLX3, X
      REAL CC (4,4)                                                    
      DATA CC /0.7674, 3.079, 2.572E-11, -1.424,   &
     &         0.3926, 3.101, 4.190E-11, -1.404,   &
     &         0.2789, 3.115, 5.415E-11, -1.399,   &
     &         0.4809, 3.082, 3.110E-11, -1.428/                       
!                                                                      
!    STATEMENT FUNCTIONS
!
!     (1) GONG-MONAHAN'S SOURCE                                        
!                                                                      
      B(X)  = (0.433333-LOG10(X))/0.433333                                     

!     FLX0(X)=1.373*X**(-4.7*(1.+35.0*X)**(-0.017*X**(-1.44)))*(1.0+        
      FLX0(X)=X**(-4.7*(1.+35.0*X)**(-0.017*X**(-1.44)))*(1.0+        &
     &                    0.057*X**3.45)*10.0**(1.607*EXP(-B(X)*B(X))) 
!
!     (2) SMITH'S SOURCE
!
      FLX1(X)=EXP(-3.1*(LOG(X/2.1))**2)
      FLX2(X)=EXP(-3.3*(LOG(X/9.2))**2)
!
!     (3)  SURF ZONE PRODUCTION
!
!     FLX3(X)=1.1*X**(-1.65)
      FLX3(X)=X**(-1.65)

!      if(trnch.eq.153) write(0,*)'wzz: NN in SFSS=', NN, IREST,
!     &   AEROSIZE(1,12),AEROSIZE(2,12),RHOP0(NN),T(1,ILEV+1)
!
!     FOR NON-START RUN, USE THE SAVED FINTROW TO COMPUTE FLUX
!
      IF (IREST .EQ. 0) GOTO 100
!
!     OTHERWISE COMPUTE THE INTEGRATION OF SEA-SALT FOR THIS
!     MODEL RUN - FINTROW
!
      CALL PUTZERO (FINTROW, ILG*ISIZE*3)

      DO N = 1,ISIZE 
          RR1 = AEROSIZE(1, N)*100.0     !From m to cm 
          RR2 = AEROSIZE(2, N)*100.0                                   
          DO I=IL1,IL2                                                 
!                                                                      
!     * CONVERT THE DRY SIZE OF SEA-SALT AEROSOLS INTO WET SIZE BY     
!         TAKING RH INTO ACCOUNT. THE WET SIZE (80% RH) IS             
!         USED IN THE SURFACE FLUX CALCULATION.                        
!                                                                      
!     * TEMPERATURE EFFECT                                             
!                                                                      
             CC3 = CC(3,1)*(1.0+0.004*(298.0-T(I,ILEV+1)))             
             R1(I) = (RR1*RR1*RR1+CC(1,1)*RR1**CC(2,1)/     &
     &                (  CC3*RR1**CC(4,1) - LOG10(0.8)  )   &
     &                )**(1./3.)*10000.00 
             R2(I) = (RR2*RR2*RR2+CC(1,1)*RR2**CC(2,1)/     &
     &                (  CC3*RR2**CC(4,1) - LOG10(0.8)  )   &
     &                )**(1./3.)*10000.00  
!                                                                     
!     * DF0 is for the indirect mechanism (via bubbles)               
!                                                                     
             DF0(I)=0.                                                
             DF1(I)=0.
             DF2(I)=0.
          END DO                                                      
!      if(trnch.eq.153) write(0,*)'wzz: N in SFSS=', N, RR1, RR2, R1(1), R2(1)
!
          N35 = N*35
          DO IK=1,N35                                                
!             DO I=IL1,IL2
!
!      GC(ILG) = 1 FOR SEA-ICE EXCLUDED FROM SEA-SALT PRODUCTION
!                                             
                DEL = (RR2-RR1)/FLOAT(N35)        !del in cm
                RWI = (RR1+DEL*FLOAT(IK-1))*1.E-2  !from cm to m                   
!                                                                      
!     * DRY VOLUME AND MASS OF AN AEROSOL PARTICLE OF SIZE RWI             
!                                                                      
                PVOL = 4./3.*3.1415926*RWI*RWI*RWI                                
                PMASS = PVOL*RHOP0(NN)                                       
                
             DO I=IL1,IL2    ! wzz, 2008.2.13
                DELR0 = (R2(I)-R1(I))/FLOAT(N35)

                IF (FLAND(I,14) .GT. 0.0 .AND. GC(I) .NE. 1.0) THEN 
                   IF (I_SS .EQ. 1) THEN      !GONG'S SCHEME
                     DF0(I) = DF0(I)+DELR0*PMASS*            &
     &                     FLX0(R1(I)+DELR0*FLOAT(IK-1))
                     
                   ELSE                       !SMITH'S SCHEME
                     DF0(I) = DF0(I)+DELR0*PMASS*            &
     &                     FLX1(R1(I)+DELR0*FLOAT(IK-1))
                     DF1(I) = DF1(I)+DELR0*PMASS*            &
     &                     FLX2(R1(I)+DELR0*FLOAT(IK-1))
                   END IF
!
!        SURF ZONE PRODUCTION
!
                   IF (FLAND(I,14) .LT. 1.0) THEN
                       DF2(I)=DF2(I)+DELR0*2.0*PMASS*        &
     &                     FLX3(R1(I)*2.0+DELR0*2.0*FLOAT(IK-1)) 
                   END IF            
                 END IF
             END DO                                                   
          END DO                                                      
          DO I=IL1,IL2
            FINTROW(I,N,1) = DF0(I)
            FINTROW(I,N,2) = DF1(I)
            FINTROW(I,N,3) = DF2(I)
          END DO                                                      
      END DO
  100 CONTINUE                                                        

!                                                                     
!     *  INSERTION DU FLUX DE SURFACE                                 
!
!                                                                    
!      if(trnch.eq.153) write(0,*)'wzz: NN=', NN, TRAC(1,lev,12), IAE1, ISIZE, DELT,SURFWD(1),
!     &       GC(1), FLAND(1,14),I_SS
      DO 845 N = 1,ISIZE
         NP=ISIZE*(NN-1)+N+(IAE1-1)
         DO 842 I=IL1,IL2 
           IF (SURFWD(I) .GT. 0.0 .AND. GC(I) .NE. 1.0 .AND.    &
     &         FLAND(I,14) .GT. 0.0) THEN    
              IF (I_SS .EQ. 1) THEN      !GONG-MONAHAN'S SCHEME
                  RSFROW(I,NP) = 1.373*FINTROW(I,N,1)*SURFWD(I)**3.41  &
     &                                                 *FLAND(I,14)
              ELSE                       !SMITH'S SCHEME
                  A1 = 10**(0.0676*SURFWD(I)+2.430)
                  A2 = 10**(0.9590*SQRT(SURFWD(I))-1.476)
                  RSFROW(I,NP) = (A1*FINTROW(I,N,1)+A2*FINTROW(I,N,2))  &
     &                           *FLAND(I,14)
              END IF
!
!     ADDING SURFING PRODUCTION FLUX
!
              RSFROW(I,NP) =RSFROW(I,NP)+ 1.1*FINTROW(I,N,3)       &
     &                            *EXP(0.23*SURFWD(I))*FLAND(I,14)
              TRAC(I,ILEV+1,NP) = TRAC(I,ILEV+1,NP) + RSFROW(I,NP) &
     &                  /(DSHJ(I,ILEV)*PRESSG(I)/G)*2.*DELT      
              
           END IF
  842 CONTINUE                                                        
  845 CONTINUE                                                        
      if(NN/=1)write(*,*)'wzz: NN in SFSS=', NN

      RETURN                                                          
      END SUBROUTINE SFSS
