#include <misc.h>

module ccrnbnd

   !----------------------------------------------------------------------- 
   ! Purpose: 
   ! Interpolate the SOX emissions data from the GEIA-SMITH datasets.  This
   ! dataset contains seasonal average data for various years at roughly 15
   ! year intervals.  The interpolation scheme does linear interpolations to
   ! the requested calendar day in the seasonal cycle for each of
   ! the two years in the dataset that bound the requested year.  Finally a 
   ! linear interpolation is done to the requested year.
   !
   ! It is assumed that the model calling this interface has been
   ! compiled so that 8 byte real data are being used.  On many
   ! machines this implies compiling with a "-r8" flag.
   ! 
   ! Author: B. Eaton
   !
   ! Modified 11 Jan 2000 PJR: work with 4 or 8 byte floats 
   ! Modified 15 Jan 2004 D Bundy: can cycle one year of emission data
   !          using namelist variable rampyear_prognostic_sulfur
   !
   !----------------------------------------------------------------------- 

   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid,       only: masterproc, plon, plat
   use ppgrid,       only: pcols, begchunk, endchunk, pver
   use abortutils,   only: endrun
   use infnan,       only: bigint

!nf90   use netcdf

   implicit none
!
   save
   private
   public :: &
      ccrnbndini,      &! initialize ccrnbnd module
      ccrnbndint,      &! interpolate ccrnbnd data to requested date/time
      ccrnfldget,      &! return latitude slice FLUC  data at current date/time
      ccrninpget,      &! return latitude slice input data at current date/time
      ccrnbndget        ! return latitude slice emission data at current date/time

   ! public module data
   
   !  Variables set by namelist
   character(len=256) :: soxems_data
   character(len=256) :: ccrn_data
!++drb
   integer, public :: rampyear_prognostic_sulfur = bigint  ! if = bigint => unset, else = YYYY to cycle
   character(len=16), public :: scenario_prognostic_sulfur = 'RAMPED' ! only option available

   logical :: doramp_sox   ! at the moment this is the only option
   logical :: fixyear_sox  ! if true then cycle on year from the emissions dataset
!--drb

#include <netcdf.inc>

   integer, parameter ::&
      ndlev=5            ! number of levels in emissions data

   integer, allocatable, dimension(:) :: &
      date,             &! date coordinate (yyyymmdd format)
      yr                 ! years of annual cycle data
   real(r8), allocatable, dimension(:) :: &
      cdays              ! mid-season calendar days (fractional)
   real(r8), allocatable, dimension(:,:,:,:) :: &
      bcfyrlo,          &! BCFLX input data for lower bound year (pcols,begchunk:endchunk,ndlev,ntpy)
      bcfyrhi            ! BCFLX input data for upper bound year
   real(r8), allocatable, dimension(:,:,:,:) :: &
      ocfyrlo,   &       ! OCFLX input data for lower bound year (pcols,begchunk:endchunk,ndlev,ntpy)
      ohyrlo,    &       ! OH    input data for lower bound year (pcols,begchunk:endchunk,pver,ntpy)
      h2o2yrlo,  &       ! H2O2  input data for lower bound year (pcols,begchunk:endchunk,pver,ntpy)
      o3yrlo,    &       ! O3    input data for lower bound year (pcols,begchunk:endchunk,pver,ntpy)
      no3yrlo,   &       ! NO3   input data for lower bound year (pcols,begchunk:endchunk,pver,ntpy)
      hno3yrlo,  &       ! HNO3  input data for lower bound year (pcols,begchunk:endchunk,pver,ntpy)
      nh3yrlo            ! NH3   input data for lower bound year (pcols,begchunk:endchunk,pver,ntpy)
   real(r8), allocatable, dimension(:,:,:) :: &
      dmsoyrlo,  &       ! ODFLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
      dmslyrlo,  &       ! LDFLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
      h2syrlo,   &       ! H2SFLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
      pso2yrlo,  &       ! PSO2FLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
      aso2yrlo,  &       ! ASO2FLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
      pso4yrlo,  &       ! PSO4FLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
      aso4yrlo           ! ASO4FLX input data for lower bound year (pcols,begchunk:endchunk,ntpy)
   real(r8), allocatable, dimension(:,:,:) :: &
      bcf                ! BCFLX interpolated data (pcols,begchunk:endchunk,ndlev)
   real(r8), allocatable, dimension(:,:,:) :: &
      bcfh               ! BCFLXHIT interpolated data (pcols,begchunk:endchunk,ndlev)
   real(r8), allocatable, dimension(:,:,:) :: &
      ocf                ! OCFLX interpolated data (pcols,begchunk:endchunk,ndlev)
   real(r8), allocatable, dimension(:,:,:) :: &
      ocfh               ! OCFLXHIT interpolated data (pcols,begchunk:endchunk,ndlev)
   real(r8), allocatable, dimension(:,:,:) :: &
      fluc               ! FLUC interpolated data (pcols,begchunk:endchunk,15)
   real(r8), allocatable, dimension(:,:,:) :: &
      snd12              ! SAND2 interpolated data (pcols,begchunk:endchunk,12)
   real(r8), allocatable, dimension(:,:,:) :: &
      cly2               ! CLAY2 interpolated data (pcols,begchunk:endchunk,12)
   real(r8), allocatable, dimension(:,:,:) :: &
      chds6              ! CHDS6 interpolated data (pcols,begchunk:endchunk,6)
   real(r8), allocatable, dimension(:,:)   :: &
      dmso, dmsl, &      ! DMS interpolated data (pcols,begchunk:endchunk)
      h2s,        &      ! H2S interpolated data (pcols,begchunk:endchunk)
      pso2, aso2, &      ! SO2 interpolated data (pcols,begchunk:endchunk)
      pso4, aso4         ! SO4 interpolated data (pcols,begchunk:endchunk)
   real(r8), allocatable, dimension(:,:,:) :: &
      oh,   &            ! OH    interpolated data (pcols,begchunk:endchunk,pver)
      h2o2, &            ! H2O2  interpolated data (pcols,begchunk:endchunk,pver)
      o3,   &            ! O3    interpolated data (pcols,begchunk:endchunk,pver)
      no3,  &            ! NO3   interpolated data (pcols,begchunk:endchunk,pver)
      hno3, &            ! HNO3  interpolated data (pcols,begchunk:endchunk,pver)
      nh3                ! NH3   interpolated data (pcols,begchunk:endchunk,pver)

   integer ::&
      ntpy,             &! number of time samples per year in emissions dataset
      nyr,              &! number of years in emissions dataset
      ncid,             &! ID for netCDF emission data file
      nfid,             &! ID for netCDF input data file
      loyri,            &! index in yr array for lower bound year
      start(4),         &! start vector for netCDF hyperslabs
      count(4)           ! count vector for netCDF hyperslabs

!##############################################################################
contains
!##############################################################################


  subroutine ccrnbndini( inyear )

      !----------------------------------------------------------------------- 
      ! Purpose: 
      ! Open netCDF file containing SOX emissions data.  Initialize arrays
      ! with the data to be interpolated to the current time.
      !
      ! It is assumed that each year for which data is available contains
      ! the same number of time samples.
      !
      ! It is assumed that the input data contains a "date" coordinate that is
      ! increasing and represents dates in the format yyyymmdd.
      ! 
      ! Author: B. Eaton
      !----------------------------------------------------------------------- 

      use error_messages, only: alloc_err, handle_ncerr, handle_err
      use phys_grid,      only: scatter_field_to_chunk
      use ioFileMod,      only: getfil
      use time_manager,   only: get_curr_date
      use mpishorthand                     ! wzz, 2008.9.17

#if ( defined SPMD )
      use mpishorthand
#endif

#ifdef MATCH
      use calendar
#endif

!-----------------------------------------------------------------------
#include <netcdf.inc>
!-----------------------------------------------------------------------

      integer, intent(in) ::&
         inyear    ! current year

      ! Local variables:
      integer year  ! year that is used (may be inyear or rampyear_prognostic_sulfur if fixed)
      integer :: i, istat, did, nlon, vid, recid, nrec
      integer :: yrc, mon, day, ncsec          ! wzz, 2008.9.12
!
      real(r8), allocatable :: soxlatlonhi(:,:,:)    ! used for netCDF input
      real(r8), allocatable :: soxlatlonlo(:,:,:,:)  ! used for netCDF input
      character*10 errmsg                            ! error messages
      character*80 :: bndccrn = 'ccrnemis.nc'        ! emission data file name
      character*80 :: filname = 'mozartdat.nc'       ! input data file name

      ! Externals
!      real(r8) ::&
!         caldayr
      !-----------------------------------------------------------------------
! read the emission data first
      ! Get file name.  
      if (masterproc) then
      call getfil(bndccrn, soxems_data, 0)
      endif

      !++drb Set logical flags according to namelist variables
      !    Currently the only scenario available is RAMPED      
      if ( scenario_prognostic_sulfur == 'RAMPED' ) then
         doramp_sox = .true.
         if ( masterproc ) &
              write(6,*)'ccrnbndini: scenario_prognostic_sulfur = RAMPED'
      else if  ( scenario_prognostic_sulfur == 'FIXED' ) then
         call endrun('ccrnbndini: input namelist SCENARIO_SOX must be set to RAMPED')
      else
         call endrun('ccrnbndini: input namelist SCENARIO_SOX must be set to RAMPED')
      end if

      ! Determine if namelist variable to cycle year has been set
      rampyear_prognostic_sulfur = 1870    !  wzz, 2008.3.10
!
      if ( rampyear_prognostic_sulfur .ne. bigint ) then
         fixyear_sox = .true.
         if ( masterproc ) &
              write(6,*) 'ccrnbndini: cycling emissions from year ',rampyear_prognostic_sulfur
      else
         fixyear_sox = .false.
      end if

      ! Set year to get out of file to either year from model (inyear) or fixed (rampyear_prognostic_sulfur)
      if ( .not. fixyear_sox ) then
         year = inyear
      else
         if ( masterproc ) &
              write(6,*)'ccrnbndini: WARNING- overriding model year with rampyear_prognostic_sulfur',rampyear_prognostic_sulfur
         year = rampyear_prognostic_sulfur
      endif
      !--drb fixed year 

      if (masterproc) then

        write(6,*)'ccrnbndini: called for model year:',year

      ! Open file.
        call handle_ncerr( nf_open( soxems_data, NF_NOWRITE, ncid ), &
           'ccrnbndini: error opening file '//trim(soxems_data) )

      ! get the record id
        call handle_ncerr( nf_inq_unlimdim( ncid, recid), &
           'ccrnbndini: no record variables ' )

      ! Check that input data is a right resolution.
        call handle_ncerr( nf_inq_dimid( ncid, 'lon', did ), 'ccrnbndini: ' )
        call handle_ncerr( nf_inq_dimlen( ncid, did, nlon ), 'ccrnbndini: ' )
        if ( nlon .ne. plon ) then
           write(unit=errmsg,fmt='(i10)') nlon
           call endrun('ccrnbndini: number of longitudes ('//trim(errmsg)//') doesn''t match model resolution.')
        end if

      ! Get size of unlimited dimension.
        call handle_ncerr( nf_inq_dimlen( ncid, recid, nrec ), 'ccrnbndini: ' )

      endif  ! end of masterproc

#if ( defined SPMD )
      ! broadcast nrec to nodes
      call mpibcast (nrec, 1, mpiint, 0, mpicom)
#endif

      allocate( date(nrec), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'date', nrec )

      ! Get date coordinate.
      if (masterproc) then

        call handle_ncerr( nf_inq_varid( ncid, 'date', vid ), &
           'ccrnbndini: cannot find date coordinate variable' )
        call handle_ncerr( nf_get_var_int( ncid, vid, date ), &
           'ccrnbndini: error getting date coordinate data' )

      endif  ! end of masterproc

#if ( defined SPMD )
      ! broadcast date to nodes
      call mpibcast (date, nrec, mpiint, 0, mpicom)
#endif

      ! Determine number of time samples per year.
      ntpy = 1
      do i = 2, nrec
         if ( date(i)/10000 .eq. date(1)/10000 ) ntpy = ntpy + 1
      end do

      ! Construct the years array.
      nyr = nrec/ntpy
      allocate( yr(nyr), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'yr', nyr )
      do i = 1, nyr
         yr(i) = date((i-1)*ntpy+1)/10000
      end do
      if ( masterproc ) &
           write(6,*)'ccrnbndini: years in emission dataset:',(yr(i),i=1,nyr)

      ! Construct array of calendar days for the annual cycle.
      allocate( cdays(ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'cdays', ntpy )
      do i = 1, ntpy
#ifdef MATCH         
         cdays(i) = caldayr( date(i), 0 )      ! match version
#else
         call bnddyi( date(i), 0, cdays(i) ) ! ccm version
#endif
      end do
      if ( masterproc ) &
           write(6,*)'ccrnbndini: calendar days in annual cycle:', (cdays(i),i=1,ntpy)

      if ( masterproc ) &
           write (6,*) 'ccrnbndini: searching for emissions for year ', year

      ! Check that requested year is contained in the data.
      if ( year .lt. yr(1) .or. year .gt. yr(nyr) ) then
         call endrun ('CCRNBNDINI: requested year outside data limits in '//trim(soxems_data))
      end if

      ! Find index for the data year that is the lower bound of the
      ! interval containing the input year.
      loyri = 1    ! wzz
      do i = 1, nyr
         if ( yr(i) .gt. year ) then
            loyri = i - 1
            exit
         end if
      end do

      allocate( bcfyrlo(pcols,begchunk:endchunk,ndlev,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'bcfyrlo', pcols*((endchunk-begchunk)+1)*ndlev*ntpy )
      allocate( bcfyrhi(pcols,begchunk:endchunk,ndlev,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'bcfyrhi', pcols*((endchunk-begchunk)+1)*ndlev*ntpy )
      allocate( ocfyrlo(pcols,begchunk:endchunk,ndlev,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'ocfyrlo', pcols*((endchunk-begchunk)+1)*ndlev*ntpy )
      allocate( dmsoyrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'dmsoyrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      allocate( dmslyrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'dmslyrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      allocate( h2syrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'h2syrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      allocate( pso2yrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'pso2yrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      allocate( aso2yrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'aso2yrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      allocate( pso4yrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'pso4yrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      allocate( aso4yrlo(pcols,begchunk:endchunk,ntpy), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'aso4yrlo', pcols*((endchunk-begchunk)+1)*ntpy )
      bcfyrlo(:,:,:,:) = 0._r8
      bcfyrhi(:,:,:,:) = 0._r8
      ocfyrlo(:,:,:,:) = 0._r8
      dmsoyrlo(:,:,:)  = 0._r8
      dmslyrlo(:,:,:)  = 0._r8
      h2syrlo(:,:,:)   = 0._r8
      pso2yrlo(:,:,:)  = 0._r8
      aso2yrlo(:,:,:)  = 0._r8
      pso4yrlo(:,:,:)  = 0._r8
      aso4yrlo(:,:,:)  = 0._r8
!
      allocate( dmso(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'dmso', pcols*(endchunk-begchunk+1) )
      allocate( dmsl(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'dmsl', pcols*(endchunk-begchunk+1) )
      allocate( h2s(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'h2s', pcols*(endchunk-begchunk+1) )
      allocate( pso2(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'pso2', pcols*(endchunk-begchunk+1) )
      allocate( aso2(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'aso2', pcols*(endchunk-begchunk+1) )
      allocate( pso4(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'pso4', pcols*(endchunk-begchunk+1) )
      allocate( aso4(pcols,begchunk:endchunk), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'aso4', pcols*(endchunk-begchunk+1) )
      dmso(:,:) = 0._r8;   dmsl(:,:) = 0._r8
      h2s(:,:)  = 0._r8
      pso2(:,:) = 0._r8;   aso2(:,:) = 0._r8
      pso4(:,:) = 0._r8;   aso4(:,:) = 0._r8
!
      allocate( fluc(pcols,begchunk:endchunk,15), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'fluc', pcols*(endchunk-begchunk+1)*15 )
      allocate( snd12(pcols,begchunk:endchunk,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'snd12', pcols*(endchunk-begchunk+1)*12 )
      allocate( cly2(pcols,begchunk:endchunk,2), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'cly2', pcols*(endchunk-begchunk+1)*2 )
      allocate( chds6(pcols,begchunk:endchunk,6), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'chds6', pcols*(endchunk-begchunk+1)*6 )
      fluc(:,:,:)  = 0._r8
      snd12(:,:,:) = 0._r8
      cly2(:,:,:)  = 0._r8
      chds6(:,:,:) = 0._r8
!
      allocate( bcf(pcols,begchunk:endchunk,ndlev), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'bcf', pcols*((endchunk-begchunk)+1)*ndlev )
      allocate( bcfh(pcols,begchunk:endchunk,ndlev), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'bcfh', pcols*((endchunk-begchunk)+1)*ndlev )
      allocate( ocf(pcols,begchunk:endchunk,ndlev), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'ocf', pcols*((endchunk-begchunk)+1)*ndlev )
      allocate( ocfh(pcols,begchunk:endchunk,ndlev), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'ocfh', pcols*((endchunk-begchunk)+1)*ndlev )
!
      bcf(:,:,:)  = 0._r8
      bcfh(:,:,:) = 0._r8
      ocf(:,:,:)  = 0._r8
      ocfh(:,:,:) = 0._r8
!
      allocate( soxlatlonlo(plon,plat,ndlev,ntpy),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*ndlev*ntpy )

!  Read the monthly data 
      if (masterproc) then

        ! Read BCFLX data for years surrounding initial year.
        start(1) = 1
        start(2) = 1
        start(3) = 1
        start(4) = 1
        count(1) = plon
        count(2) = plat
        count(3) = ndlev
        count(4) = ntpy

        call handle_ncerr( nf_inq_varid( ncid, 'BCFLX', vid ), &
           'ccrnbndini: cannot find variable '//'BCFLX' )

        start(4) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'BCFLX' )

!        start(4) = start(4) + ntpy
!        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonhi ), &
!           'ccrnbndini: cannot read data for '//'SOx' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ndlev*ntpy, plon, soxlatlonlo, &
                                   bcfyrlo(1,begchunk,1,1))

      if (masterproc) then

        ! Read OCFLX data for years surrounding initial year.
        start(1) = 1
        start(2) = 1
        start(3) = 1
        start(4) = 1
        count(1) = plon
        count(2) = plat
        count(3) = ndlev
        count(4) = ntpy

        call handle_ncerr( nf_inq_varid( ncid, 'OCFLX', vid ), &
           'ccrnbndini: cannot find variable '//'OCFLX' )

        start(4) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'OCFLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ndlev*ntpy, plon, soxlatlonlo, &
                                   ocfyrlo(1,begchunk,1,1))
!
      if ( masterproc ) then
        write(6,*)'ccrnbndini: read data for years ', yr(loyri)
!        write(6,*)'ccrnbndini: read data for years; ', yr(loyri),' and ',yr(loyri+1)
      endif  ! end of masterproc

      if (masterproc) then

        ! Read BCHIT data.

        call handle_ncerr( nf_inq_varid( ncid, 'BCHIT', vid ), &
           'ccrnbndini: cannot find variable '//'BCHIT' )

        call handle_ncerr( nf_get_var_double( ncid, vid,  soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'BCHIT' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ndlev, plon, soxlatlonlo, &
                                   bcfh(1,begchunk,1))

      if (masterproc) then

        ! Read OCHIT data.

        call handle_ncerr( nf_inq_varid( ncid, 'OCHIT', vid ), &
           'ccrnbndini: cannot find variable '//'OCHIT' )

        call handle_ncerr( nf_get_var_double( ncid, vid,  soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'OCHIT' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ndlev, plon, soxlatlonlo, &
                                   ocfh(1,begchunk,1))

      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')

! ODFLX data as DMSO for years surrounding initial year.
      allocate( soxlatlonlo(plon,plat,ntpy,1),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*ntpy )
      if (masterproc) then

        start(1) = 1
        start(2) = 1
        start(3) = 1
        start(4) = 1
        count(1) = plon
        count(2) = plat
        count(3) = ntpy
        count(4) = 1

        call handle_ncerr( nf_inq_varid( ncid, 'ODFLX', vid ), &
           'ccrnbndini: cannot find variable '//'ODFLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'OCFLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   dmsoyrlo(1,begchunk,1))
      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')
!
!  Read the seasonly data 
      allocate( soxlatlonlo(plon,plat,ntpy,1),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*ntpy )
! LDMFLX data as DMSL for years surrounding initial year.
      if (masterproc) then

        ! Read DMSL data.

        call handle_ncerr( nf_inq_varid( ncid, 'LDMFLX', vid ), &
           'ccrnbndini: cannot find variable '//'LDMFLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'LDMFLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   dmslyrlo(1,begchunk,1))
! H2SFLX data as H2S for years surrounding initial year.
      if (masterproc) then

        ! Read H2S data.

        call handle_ncerr( nf_inq_varid( ncid, 'H2SFLX', vid ), &
           'ccrnbndini: cannot find variable '//'LDMFLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'H2SFLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   h2syrlo(1,begchunk,1))
!
! PSO2FLX data  for years surrounding initial year.
      if (masterproc) then

        ! Read PSO2 data.

        call handle_ncerr( nf_inq_varid( ncid, 'PSO2FLX', vid ), &
           'ccrnbndini: cannot find variable '//'PSO2FLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'PSO2FLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   pso2yrlo(1,begchunk,1))
!
! ASO2FLX data  for years surrounding initial year.
      if (masterproc) then

        ! Read ASO2 data.

        call handle_ncerr( nf_inq_varid( ncid, 'ASO2FLX', vid ), &
           'ccrnbndini: cannot find variable '//'ASO2FLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'ASO2FLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   aso2yrlo(1,begchunk,1))
!
! PSO4FLX data  for years surrounding initial year.
      if (masterproc) then

        ! Read PSO2 data.

        call handle_ncerr( nf_inq_varid( ncid, 'PSO4FLX', vid ), &
           'ccrnbndini: cannot find variable '//'PSO4FLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'PSO4FLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   pso4yrlo(1,begchunk,1))
!
! ASO4FLX data  for years surrounding initial year.
      if (masterproc) then

        ! Read ASO4 data.

        call handle_ncerr( nf_inq_varid( ncid, 'ASO4FLX', vid ), &
           'ccrnbndini: cannot find variable '//'ASO4FLX' )

        start(3) = (loyri-1)*ntpy + 1
        call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'ASO4FLX' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, ntpy, plon, soxlatlonlo, &
                                   aso4yrlo(1,begchunk,1))
!
      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')
!
!  Read the time-invariant data 
! land surface type
      allocate( soxlatlonlo(plon,plat,15,1),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*15 )

      if (masterproc) then

        ! Read FLUC data.

        call handle_ncerr( nf_inq_varid( ncid, 'FLUC', vid ), &
           'ccrnbndini: cannot find variable '//'FLUC' )

        call handle_ncerr( nf_get_var_double( ncid, vid,  soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'FLUC' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, 15, plon, soxlatlonlo, &
                                   fluc(1,begchunk,1))

      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')

! soil type
      allocate( soxlatlonlo(plon,plat,12,1),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*12 )

      if (masterproc) then

        ! Read SAND2 data.

        call handle_ncerr( nf_inq_varid( ncid, 'SAND2', vid ), &
           'ccrnbndini: cannot find variable '//'SAND2' )

        call handle_ncerr( nf_get_var_double( ncid, vid,  soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'SAND2' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, 12, plon, soxlatlonlo, &
                                   snd12(1,begchunk,1))

      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')

! clay type
      allocate( soxlatlonlo(plon,plat,2,1),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*2 )

      if (masterproc) then

        ! Read CLAY2 data.

        call handle_ncerr( nf_inq_varid( ncid, 'CLAY2', vid ), &
           'ccrnbndini: cannot find variable '//'CLAY2' )

        call handle_ncerr( nf_get_var_double( ncid, vid,  soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'CLAY2' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, 2, plon, soxlatlonlo, &
                                   cly2(1,begchunk,1))

      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')

! sand classification
      allocate( soxlatlonlo(plon,plat,6,1),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*6 )

      if (masterproc) then

        ! Read CHDS6 data.

        call handle_ncerr( nf_inq_varid( ncid, 'CHDS6', vid ), &
           'ccrnbndini: cannot find variable '//'CHDS6' )

        call handle_ncerr( nf_get_var_double( ncid, vid,  soxlatlonlo ), &
           'ccrnbndini: cannot read data for '//'CHDS6' )

      endif  ! end of masterproc

      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, 6, plon, soxlatlonlo, &
                                   chds6(1,begchunk,1))

      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')
!
! open another file and read the monthly input data, wzz, 2008.9.17

      allocate( ohyrlo(pcols,begchunk:endchunk,pver,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'ohyrlo', pcols*((endchunk-begchunk)+1)*pver*12 )
      allocate( h2o2yrlo(pcols,begchunk:endchunk,pver,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'h2o2yrlo', pcols*((endchunk-begchunk)+1)*pver*12 )
      allocate( o3yrlo(pcols,begchunk:endchunk,pver,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'o3yrlo', pcols*((endchunk-begchunk)+1)*pver*12 )
      allocate( no3yrlo(pcols,begchunk:endchunk,pver,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'no3yrlo', pcols*((endchunk-begchunk)+1)*pver*12 )
      allocate( hno3yrlo(pcols,begchunk:endchunk,pver,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'hno3yrlo', pcols*((endchunk-begchunk)+1)*pver*12 )
      allocate( nh3yrlo(pcols,begchunk:endchunk,pver,12), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'nh3yrlo', pcols*((endchunk-begchunk)+1)*pver*12 )
      ohyrlo(:,:,:,:)   = 0._r8
      h2o2yrlo(:,:,:,:) = 0._r8
      o3yrlo(:,:,:,:)   = 0._r8
      no3yrlo(:,:,:,:)  = 0._r8
      hno3yrlo(:,:,:,:) = 0._r8
      nh3yrlo(:,:,:,:)  = 0._r8
!
      allocate( oh(pcols,begchunk:endchunk,pver), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'oh', pcols*((endchunk-begchunk)+1)*pver  )
      allocate( h2o2(pcols,begchunk:endchunk,pver), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'h2o2', pcols*((endchunk-begchunk)+1)*pver  )
      allocate( o3(pcols,begchunk:endchunk,pver), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'o3', pcols*((endchunk-begchunk)+1)*pver  )
      allocate( no3(pcols,begchunk:endchunk,pver), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'no3', pcols*((endchunk-begchunk)+1)*pver  )
      allocate( hno3(pcols,begchunk:endchunk,pver), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'hno3', pcols*((endchunk-begchunk)+1)*pver  )
      allocate( nh3(pcols,begchunk:endchunk,pver), stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'nh3', pcols*((endchunk-begchunk)+1)*pver  )
      oh(:,:,:)   = 0._r8
      h2o2(:,:,:) = 0._r8
      o3(:,:,:)   = 0._r8
      no3(:,:,:)  = 0._r8
      hno3(:,:,:) = 0._r8
      nh3(:,:,:)  = 0._r8
!
      allocate( soxlatlonlo(plon,plat,pver,12),  stat=istat )
      call alloc_err( istat, 'ccrnbndini', 'soxlatlonlo', &
                      plon*plat*pver*12 )
!
! read the data first
      call get_curr_date(yrc, mon, day, ncsec)   !  wzz, 2008.9.12
      if (masterproc) then
      ! Get file name.  
      call getfil(filname, ccrn_data, 0)
      ! Open file.
      call handle_ncerr( nf_open( ccrn_data, NF_NOWRITE, nfid ), &
         'ccrnbndini: error opening file '//trim(ccrn_data) )
      endif
!
        start(1) = 1
        start(2) = 1
        start(3) = 1
        start(4) = 1
        count(1) = plon
        count(2) = plat
        count(3) = pver
        count(4) = 12
!
      if (masterproc) then
! OH
      call handle_ncerr( nf_inq_varid( nfid, 'OH', vid ), &
         'ccrnbndini: cannot find variable '//'OH' )
      call handle_ncerr( nf_get_vara_double( nfid, vid, start, count, soxlatlonlo ), &
         'ccrnbndini: cannot read data for '//'OH' )
      endif
      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, pver*12, plon, soxlatlonlo, &
                                   ohyrlo(1,begchunk,1,1))

      if (masterproc) then
! H2O2
      call handle_ncerr( nf_inq_varid( nfid, 'H2O2', vid ), &
         'ccrnbndini: cannot find variable '//'H2O2' )
      call handle_ncerr( nf_get_vara_double( nfid, vid, start, count, soxlatlonlo ), &
         'ccrnbndini: cannot read data for '//'H2O2' )
      endif
      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, pver*12, plon, soxlatlonlo, &
                                   h2o2yrlo(1,begchunk,1,1))

      if (masterproc) then
! O3
      call handle_ncerr( nf_inq_varid( nfid, 'O3', vid ), &
         'ccrnbndini: cannot find variable '//'O3' )
      call handle_ncerr( nf_get_vara_double( nfid, vid, start, count, soxlatlonlo ), &
         'ccrnbndini: cannot read data for '//'O3' )
      endif
      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, pver*12, plon, soxlatlonlo, &
                                   o3yrlo(1,begchunk,1,1))

      if (masterproc) then
! NO3
      call handle_ncerr( nf_inq_varid( nfid, 'NO3', vid ), &
         'ccrnbndini: cannot find variable '//'NO3' )
      call handle_ncerr( nf_get_vara_double( nfid, vid, start, count, soxlatlonlo ), &
         'ccrnbndini: cannot read data for '//'NO3' )
      endif
      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, pver*12, plon, soxlatlonlo, &
                                   no3yrlo(1,begchunk,1,1))

      if (masterproc) then
! HNO3
      call handle_ncerr( nf_inq_varid( nfid, 'HNO3', vid ), &
         'ccrnbndini: cannot find variable '//'HNO3' )
      call handle_ncerr( nf_get_vara_double( nfid, vid, start, count, soxlatlonlo ), &
         'ccrnbndini: cannot read data for '//'HNO3' )
      endif
      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, pver*12, plon, soxlatlonlo, &
                                   hno3yrlo(1,begchunk,1,1))
!
      if (masterproc) then
! NH3
      call handle_ncerr( nf_inq_varid( nfid, 'NH3', vid ), &
         'ccrnbndini: cannot find variable '//'NH3' )
      call handle_ncerr( nf_get_vara_double( nfid, vid, start, count, soxlatlonlo ), &
         'ccrnbndini: cannot read data for '//'NH3' )
      endif
      ! scatter data to chunked data structures
      call scatter_field_to_chunk (1, 1, pver*12, plon, soxlatlonlo, &
                                   nh3yrlo(1,begchunk,1,1))
!
      deallocate( soxlatlonlo, stat=istat )
      call handle_err( istat, &
         'ERROR deallocating memory for soxlatlonlo in routine ccrnbndini')

      oh(:,:,:)   = ohyrlo(:,:,:,mon)
      h2o2(:,:,:) = h2o2yrlo(:,:,:,mon)
      o3(:,:,:)   = o3yrlo(:,:,:,mon)
      no3(:,:,:)  = no3yrlo(:,:,:,mon)
      hno3(:,:,:) = hno3yrlo(:,:,:,mon)
      nh3(:,:,:)  = nh3yrlo(:,:,:,mon)
!
   end subroutine ccrnbndini

!#######################################################################

    subroutine ccrnbndint( inyear, calday )

      !----------------------------------------------------------------------- 
      ! Purpose: 
      ! Interpolate SOX data to the current time.  Update the input data
      ! as necessary.
      !
      ! Author: B. Eaton
      !----------------------------------------------------------------------- 

      use time_manager,   only: get_curr_date, is_end_curr_month
      use error_messages, only: alloc_err, handle_ncerr, handle_err
      use phys_grid,      only: scatter_field_to_chunk

#if ( defined SPMD )
      use mpishorthand
#endif

!-----------------------------------------------------------------------
#include <netcdf.inc>
!-----------------------------------------------------------------------

      integer, intent(in) ::&
           inyear    ! current year
      real(r8), intent(in) ::&
         calday  ! calendar day (w/ fraction) in current year, range = [1.,366.)

      ! Local variables:
      integer year  ! year that is used (may be inyear or rampyear_prognostic_sulfur if fixed)
      integer ::&
         i, j, k, n,    &
         vid,           &
         lotim, hitim
      real(r8) :: dt, dt1, tint
      real(r8), allocatable :: anncyclo(:,:,:)   ! low year data interpolated to calday
      real(r8), allocatable :: anncychi(:,:,:)   ! high year data interpolated to calday
      ! TBH:  Get rid of all the replicated code for I/O and scattering!  
      real(r8), allocatable :: soxlatlonhi(:,:,:,:)  ! used for netCDF input
      integer :: istat
      integer :: yrc, mon, day, ncsec, iso    ! wzz, 2008.9.12
      !-----------------------------------------------------------------------

      allocate( anncyclo(pcols,begchunk:endchunk,ndlev), &
                anncychi(pcols,begchunk:endchunk,ndlev), stat=istat )
      call alloc_err( istat, 'ccrnbndint', 'anncyclo and anncychi', &
                      pcols*((endchunk-begchunk)+1)*ndlev*2 )
      anncyclo(:,:,:) = 0._r8
      anncychi(:,:,:) = 0._r8

      !++drb fixed year 
      if ( .not. fixyear_sox ) then
         year = inyear
      else
         year = rampyear_prognostic_sulfur
      endif
      !--drb fixed year 


      ! Check to see if model year is still bounded by dataset years.
      if ( year .gt. yr(nyr) ) then
         write(6,*)'ccrnbndint: requested year = ',year, ' last dataset year = ',yr(nyr)
         call endrun
      end if

      call get_curr_date(yrc, mon, day, ncsec)   !  wzz, 2008.9.12

      if ( ntpy .gt. 1 ) then
         call findplb( cdays, ntpy, calday, lotim )
         hitim = mod( lotim, ntpy ) + 1
      endif

!  For BCFLX
!      if ( year .gt. yr(loyri+1) ) then
      if ( year .gt. yr(loyri) ) then    ! wzz, 2008.3.11
         loyri = loyri + 1
         bcfyrlo = bcfyrhi
         allocate( soxlatlonhi(plon,plat,ndlev,ntpy), stat=istat )
         call alloc_err( istat, 'ccrnbndini', 'soxlatlonhi', &
                         plon*plat*ndlev*ntpy )
         ! Read in new bcfyrhi data.  Replace old bcfyrlo data.
         if (masterproc) then
           start(1) = 1
           start(2) = 1
           start(3) = 1
           start(4) = (loyri-1)*ntpy + 1
           count(1) = plon
           count(2) = plat
           count(3) = ndlev
           count(4) = ntpy
!          start(4) = start(4) + ntpy
           call handle_ncerr( nf_inq_varid( ncid, 'BCFLX', vid ), &
              'ccrnbndint: cannot find variable '//'BCFLX' )
           call handle_ncerr( nf_get_vara_double( ncid, vid, start, count, soxlatlonhi ), &
              'ccrnbndint: cannot read data for '//'BCFLX' )
         endif  ! end of masterproc

         ! scatter data to chunked data structures
         call scatter_field_to_chunk (1, 1, ndlev*ntpy, plon, soxlatlonhi, &
                                      bcfyrhi(1,begchunk,1,1))

         if ( masterproc ) then
           write(6,*)'ccrnbndint: read BCFLX data for year; ',yr(loyri+1)
         endif  ! end of masterproc
         deallocate( soxlatlonhi, stat=istat )
         call handle_err( istat, &
             'ERROR deallocating memory for soxlatlonhi in routine ccrnbndint')

      end if

      ! Linear interpolation...  Start by computing the number of days between
      !                          the lower and upper bounds, and days between
      !                          the model time and lower bound.

      if ( ntpy .gt. 1 ) then

!        call findplb( cdays, ntpy, calday, lotim )
!        hitim = mod( lotim, ntpy ) + 1

         if( cdays(hitim) .lt. cdays(lotim) )then
            dt = 365. - cdays(lotim) + cdays(hitim)
            if( calday .le. cdays(hitim) )then
               dt1 = 365. - cdays(lotim) + calday
            else
               dt1 = calday - cdays(lotim)
            end if
         else
            dt = cdays(hitim) - cdays(lotim)
            dt1 = calday - cdays(lotim)
         end if
         tint = dt1/dt
         ! Annual cycle interpolations.
!TBH         call linintp( size(anncyclo), 0._r8, 1._r8, tint, bcfyrlo(1,1,1,lotim), &
!TBH                       bcfyrlo(1,1,1,hitim), anncyclo )
!TBH         call linintp( size(anncychi), 0._r8, 1._r8, tint, bcfyrhi(1,1,1,lotim), &
!TBH                       bcfyrhi(1,1,1,hitim), anncychi )
!         write(0,*)'wzz: lotim=', lotim, hitim, begchunk,endchunk, calday
!        call linintp( size(anncyclo), 0._r8, 1._r8, tint,  &
!                      bcfyrlo(1,begchunk,1,lotim), &
!                      bcfyrlo(1,begchunk,1,hitim), &
!                      anncyclo )
!         call linintp( size(anncychi), 0._r8, 1._r8, tint,  &
!                       bcfyrhi(1,begchunk,1,lotim), &
!                       bcfyrhi(1,begchunk,1,hitim), &
!                       anncychi )
          iso = mon   ! for monthly data
          anncyclo(:,:,:) = bcfyrlo(:,:,:,iso)
      else
         anncyclo(:,:,:) = bcfyrlo(:,:,:,1)
!         anncychi(:,:,:) = bcfyrhi(:,:,:,1)
      end if

      ! Interpolate between years for which annual cycle data is present
!      dt = yr(loyri+1) - yr(loyri)
!      dt1 = year - yr(loyri)
!      tint = dt1/dt
!      call linintp( size(bcf), 0._r8, 1._r8, tint, anncyclo, anncychi, bcf )
!
       bcf = anncyclo

! For OCFLX
! skip the data read now. wzz, 2008.4.21
!
      if ( ntpy .gt. 1 ) then

!        call findplb( cdays, ntpy, calday, lotim )
!        hitim = mod( lotim, ntpy ) + 1

         if( cdays(hitim) .lt. cdays(lotim) )then
            dt = 365. - cdays(lotim) + cdays(hitim)
            if( calday .le. cdays(hitim) )then
               dt1 = 365. - cdays(lotim) + calday
            else
               dt1 = calday - cdays(lotim)
            end if
         else
            dt = cdays(hitim) - cdays(lotim)
            dt1 = calday - cdays(lotim)
         end if
         tint = dt1/dt
         ! Annual cycle interpolations.
!TBH         call linintp( size(anncyclo), 0._r8, 1._r8, tint, ocfyrlo(1,1,1,lotim), &
!TBH                       ocfyrlo(1,1,1,hitim), anncyclo )
!TBH         call linintp( size(anncychi), 0._r8, 1._r8, tint, ocfyrhi(1,1,1,lotim), &
!TBH                       ocfyrhi(1,1,1,hitim), anncychi )
!        call linintp( size(anncyclo), 0._r8, 1._r8, tint,  &
!                      ocfyrlo(1,begchunk,1,lotim), &
!                      ocfyrlo(1,begchunk,1,hitim), &
!                      anncyclo )
!         call linintp( size(anncychi), 0._r8, 1._r8, tint,  &
!                       ocfyrhi(1,begchunk,1,lotim), &
!                       ocfyrhi(1,begchunk,1,hitim), &
!                       anncychi )
          iso = mon   ! for monthly data
          anncyclo(:,:,:) = ocfyrlo(:,:,:,iso)
      else
         anncyclo(:,:,:) = ocfyrlo(:,:,:,1)
!         anncychi(:,:,:) = ocfyrhi(:,:,:,1)
      end if

      ! Interpolate between years for which annual cycle data is present
!      dt = yr(loyri+1) - yr(loyri)
!      dt1 = year - yr(loyri)
!      tint = dt1/dt
!      call linintp( size(ocf), 0._r8, 1._r8, tint, anncyclo, anncychi, ocf )
!
       ocf = anncyclo

! For ODFLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then

         if( cdays(hitim) .lt. cdays(lotim) )then
            dt = 365. - cdays(lotim) + cdays(hitim)
            if( calday .le. cdays(hitim) )then
               dt1 = 365. - cdays(lotim) + calday
            else
               dt1 = calday - cdays(lotim)
            end if
         else
            dt = cdays(hitim) - cdays(lotim)
            dt1 = calday - cdays(lotim)
         end if
         tint = dt1/dt
         ! Annual cycle interpolations.
!        call linintp( size(anncyclo(:,:,1)), 0._r8, 1._r8, tint,  &
!                      dmsoyrlo(1,begchunk,lotim), &
!                      dmsoyrlo(1,begchunk,hitim), &
!                      anncyclo )
          iso = mon   ! for monthly data
          anncyclo(:,:,1) = dmsoyrlo(:,:,iso)
      else
         anncyclo(:,:,1) = dmsoyrlo(:,:,1)
      end if
      dmso = anncyclo(:,:,1)

! For LDMFLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then
          iso = mon   ! for seasonal data
          if ( mon==12 .or. mon<=2 ) iso = 1
          if ( mon>=3 .and. mon<=5 ) iso = 4
          if ( mon>=6 .and. mon<=8 ) iso = 7
          if ( mon>=9 .and. mon<=11) iso = 10
          anncyclo(:,:,1) = dmslyrlo(:,:,iso)
      else
         anncyclo(:,:,1) = dmslyrlo(:,:,1)
      end if
      dmsl = anncyclo(:,:,1)

! For H2SFLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then
          iso = mon   ! for seasonal data
          if ( mon==12 .or. mon<=2 ) iso = 1
          if ( mon>=3 .and. mon<=5 ) iso = 4
          if ( mon>=6 .and. mon<=8 ) iso = 7
          if ( mon>=9 .and. mon<=11) iso = 10
          anncyclo(:,:,1) = h2syrlo(:,:,iso)
      else
         anncyclo(:,:,1) = h2syrlo(:,:,1)
      end if
      h2s = anncyclo(:,:,1)

! For PSO2FLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then
          iso = mon   ! for seasonal data
          if ( mon==12 .or. mon<=2 ) iso = 1
          if ( mon>=3 .and. mon<=5 ) iso = 4
          if ( mon>=6 .and. mon<=8 ) iso = 7
          if ( mon>=9 .and. mon<=11) iso = 10
          anncyclo(:,:,1) = pso2yrlo(:,:,iso)
      else
         anncyclo(:,:,1) = pso2yrlo(:,:,1)
      end if
      pso2 = anncyclo(:,:,1)

! For ASO2FLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then
          iso = mon   ! for seasonal data
          if ( mon==12 .or. mon<=2 ) iso = 1
          if ( mon>=3 .and. mon<=5 ) iso = 4
          if ( mon>=6 .and. mon<=8 ) iso = 7
          if ( mon>=9 .and. mon<=11) iso = 10
          anncyclo(:,:,1) = aso2yrlo(:,:,iso)
      else
         anncyclo(:,:,1) = aso2yrlo(:,:,1)
      end if
      aso2 = anncyclo(:,:,1)

! For PSO4FLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then
          iso = mon   ! for seasonal data
          if ( mon==12 .or. mon<=2 ) iso = 1
          if ( mon>=3 .and. mon<=5 ) iso = 4
          if ( mon>=6 .and. mon<=8 ) iso = 7
          if ( mon>=9 .and. mon<=11) iso = 10
          anncyclo(:,:,1) = pso4yrlo(:,:,iso)
      else
         anncyclo(:,:,1) = pso4yrlo(:,:,1)
      end if
      pso4 = anncyclo(:,:,1)

! For ASO4FLX
! skip the data read now. wzz, 2008.9.11
!
      if ( ntpy .gt. 1 ) then
          iso = mon   ! for seasonal data
          if ( mon==12 .or. mon<=2 ) iso = 1
          if ( mon>=3 .and. mon<=5 ) iso = 4
          if ( mon>=6 .and. mon<=8 ) iso = 7
          if ( mon>=9 .and. mon<=11) iso = 10
          anncyclo(:,:,1) = aso4yrlo(:,:,iso)
      else
         anncyclo(:,:,1) = aso4yrlo(:,:,1)
      end if
      aso4 = anncyclo(:,:,1)

      ! deallocate memory
      deallocate( anncyclo, anncychi, stat=istat )
      call handle_err( istat, &
        'ERROR deallocating memory for anncyclo and anncychi in routine ccrnbndint')

! set the mozart monthly input data
      if ( is_end_curr_month() ) then

      oh(:,:,:)   = ohyrlo(:,:,:,mon)
      h2o2(:,:,:) = h2o2yrlo(:,:,:,mon)
      o3(:,:,:)   = o3yrlo(:,:,:,mon)
      no3(:,:,:)  = no3yrlo(:,:,:,mon)
      hno3(:,:,:) = hno3yrlo(:,:,:,mon)
      nh3(:,:,:)  = nh3yrlo(:,:,:,mon)
!
      endif
!
   end subroutine ccrnbndint

!#######################################################################

   subroutine ccrnbndget( ncol, lchnk, x, xo, sand2, clay2, sand6, &
                          dmsor, dmslr, aso2r, pso2r, aso4r, pso4r, h2sr )

      !----------------------------------------------------------------------- 
      ! Purpose: 
      ! Return ccrn emission data for the requested latitude.
      !
      ! Author: wang zaizhi
      !----------------------------------------------------------------------- 

      integer, intent(in) :: ncol          ! number of columns used
      integer, intent(in) :: lchnk         ! chunk index

      real, intent(out) :: x(pcols,ndlev, 2)  ! BCFLX emissions in Tg S/m2/s
      real, intent(out) :: xo(pcols,ndlev, 2) ! OCFLX emissions in Tg S/m2/s
      real, intent(out) :: sand2(:,:)  !
      real, intent(out) :: clay2(:,:)  !
      real, intent(out) :: sand6(:,:)  !
      real, intent(out) :: dmsor(:), dmslr(:)  !    DMS emissions
      real, intent(out) :: pso2r(:), aso2r(:)  !    SO2 emissions
      real, intent(out) :: pso4r(:), aso4r(:)  !    SO4 emissions
      real, intent(out) :: h2sr(:)             !    H2S emissions

      ! Local variables:
      integer :: i
      !-----------------------------------------------------------------------

!      if (lchnk==139) then
!        write(0,*)'wzz: bcf=', bcf(28,lchnk,1), ocf(28,lchnk,1)
!      endif
      do i = 1, ncol
         x(i,:,1)   = bcf(i,lchnk,:)
         x(i,:,2)   = bcfh(i,lchnk,:)
         xo(i,:,1)  = ocf(i,lchnk,:)
         xo(i,:,2)  = ocfh(i,lchnk,:)
!
         sand2(i,:) = snd12(i,lchnk,:)
         clay2(i,:) = cly2(i,lchnk,:)
         sand6(i,:) = chds6(i,lchnk,:)
!
         dmsor(i)   = dmso(i,lchnk)
         dmslr(i)   = dmsl(i,lchnk)
         pso2r(i)   = pso2(i,lchnk)
         aso2r(i)   = aso2(i,lchnk)
         pso4r(i)   = pso4(i,lchnk)
         aso4r(i)   = aso4(i,lchnk)
         h2sr(i)    = h2s(i,lchnk)
      end do

   end subroutine ccrnbndget

!#######################################################################
   subroutine ccrninpget( ncol, lchnk, ohrow, h2o2row, o3row, no3row, &
                          hno3row, nh3row )

      !----------------------------------------------------------------------- 
      ! Purpose: 
      ! Return ccrn INPUT data for the requested latitude.
      !
      ! Author: wang zaizhi
      !----------------------------------------------------------------------- 

      integer, intent(in) :: ncol          ! number of columns used
      integer, intent(in) :: lchnk         ! chunk index

      real, intent(out) :: ohrow(:, :)  ! OH
      real, intent(out) :: h2o2row(:,:) ! H2O2
      real, intent(out) :: o3row(:,:)   ! O3
      real, intent(out) :: no3row(:,:)  ! NO3
      real, intent(out) :: hno3row(:,:) ! HNO3
      real, intent(out) :: nh3row(:,:)  ! NH3

      ! Local variables:
      integer :: i
      !-----------------------------------------------------------------------

      do i = 1, ncol
         ohrow(i,:)   = oh(i,lchnk,:)
         h2o2row(i,:) = h2o2(i,lchnk,:)
         o3row(i,:)   = o3(i,lchnk,:)
         no3row(i,:)  = no3(i,lchnk,:)
         hno3row(i,:) = hno3(i,lchnk,:)
         nh3row(i,:)  = nh3(i,lchnk,:)
      end do

   end subroutine ccrninpget

!#######################################################################

   subroutine ccrnfldget( ncol, lchnk, fland )

      !----------------------------------------------------------------------- 
      ! Purpose: 
      ! Return ccrn FLAND data for the requested latitude.
      !
      ! Author: wang zaizhi
      !----------------------------------------------------------------------- 

      integer, intent(in) :: ncol          ! number of columns used
      integer, intent(in) :: lchnk         ! chunk index

      real, intent(out) :: fland(pcols, 15)

      ! Local variables:
      integer :: i
      !-----------------------------------------------------------------------

      do i = 1, ncol
!
         fland(i,:) = fluc(i,lchnk,:)
      end do

   end subroutine ccrnfldget

!#######################################################################
end module ccrnbnd
