module cuace_driver

!---------------------------------------------------------------------------------
! Purpose:
!
! Interface from CUACE areosol scheme, includes emission, dry and wet 
! deposition scheme
!
! Author: Wang Zaizhi, from CAMS (Zhou Cunhong)  
!
! $Id: cuace_driver.F90,v 1.12 2010/08/04 06:17:20 wangzzh Exp $
! $Author: wangzzh $
!
!---------------------------------------------------------------------------------

  use shr_kind_mod,  only: r8 => shr_kind_r8
  use pmgrid,        only: masterproc
  use shr_const_mod, only: pi => SHR_CONST_PI
  use ppgrid,        only: ni=>pcols, nk=>pverp, begchunk, endchunk
  use constituents,  only: ppcnst
  use constituents,  only: ixaer, ncnst_aer
  use physics_types, only: physics_state, physics_ptend
  use phys_grid,     only: get_rlon_all_p, get_rlat_all_p
  use abortutils,    only: endrun
  use time_manager,  only: get_curr_calday, is_end_curr_day
  use wv_saturation, only: aqsat
  use seasalt_intr,  only: ssyc_idx1

  implicit none

  public :: intrfc, cam_set_gem
  private

  interface intrfc
    module procedure intrfc2
    module procedure intrfcwet
  end interface ! intrfc

  real, allocatable :: tcsza(:, :)    !  wzz, 2008.10.16
!
contains

  subroutine cam_set_gem
!--------------------------------------------------------------------------          
!                                                                                    
!     PURPOSE                                                                        
!     -------
!     - CONTROL TO
!       - READ IN ALL PROPERTIES OF AEROSOLS AND CONTROLS 
!       - READ IN GAS DRY DEPOSITION PARAMETERS
!       - SET PRE-CALCULATION OF SOME COEFFICIENTS USED IN CAM_V5
!
!     HISTORY:                                                                       
!     --------                                                                       
!     *                                                                              
!     * MAR 18,  2004 - S.L. GONG, P. HUANG FIRST VERSION FOR GEMAQ-CAM
!     *
!------------------------------------------------------------------------------      
!
#include "aero_chem_order_cam9.cdk"
#include "chem_land_bats15.cdk"
#include "cam_set_gem9.cdk"
!
      INTEGER I,N,J,K,L
      INTEGER ITOT,ISIZE,NPAB,NPSC
      REAL CONDNU(15)
      REAL VOIJ, RI, VOKP1, VOK, VOKM1
      real POP1,POP
!
      real CVMGT
      external CVMGT
!
      CALL SPWCON7
!
!     * CALL TO INPUT PARAMETERS FOR AEROSOLS MODEL
!
      CALL INAero(AERONAME, AEROSIZE,aerochem_np,aero_nsize, RHOP0, ISCAM, &
     &                 IDD,      JDD,   ITI,  ITR,   ISLEV,   CAMR,        &
     &                DLEV,    gn_luc,    NG,   NS,     NL,                &
     &                PLLP,    Z0CAM,   SCC,  RCW,     AC, CSPEC,          &
     &              LAICAM,      RM0,   ZPD, FCAP,     WP, RGW,            &
     &                AEST,    GAMMA)

      if ( masterproc ) then
      IF (IDEBUG .GE. 1)     THEN

      WRITE (*,*)                                                     &
     &  'CAM -> IDD,JDD,ITI,ISCAM,INT(CAMR),IDEBUG,ISLEV,ICOB,IAT,MAE'
      WRITE (*,*)                                                     &
     &  '      ',IDD,JDD,ITI,ISCAM,INT(CAMR),IDEBUG,ISLEV,ICOB,        &
     &           aerochem_np,MAE
      WRITE (*,1111)  (AERONAME(N),N=1,aerochem_np)
      WRITE (*,2222)  (RHOP0(N),N=1,aerochem_np)
      WRITE (*,*) '       CAM SIZE BIN CONFIGURATIONS '
      DO N=1,aero_nsize
          WRITE (*,*) AEROSIZE(1,N), '   ', AEROSIZE(2,N)
      END DO
      ENDIF
      endif
 1111 FORMAT (' AEROSOL TYPE(S) -> ', 7('| ',A8,' |',1X))
 2222 FORMAT ('     DENSITY(KG/M3) ', 7('  ',F8.2,'  ',1X))

      ISIZE=aero_nsize
!
!    BIN VOLUME OF DRY AEROSOL
!
      DO I=1,ISIZE
         RI=(AEROSIZE(1,I)+AEROSIZE(2,I))*0.5
         PVOL(I)=4.189*RI*RI*RI
      END DO
!
!     COAGULATION TRANFORMATION RATIO
!
      CALL PUTZERO(COAGFR,ISIZE*ISIZE*ISIZE)
      DO I=1,ICOB
        DO J=I,ICOB
!
!     * NEW VOLUME OF COAGULATED PARTICLE [I,J]
!
           VOIJ=PVOL(I)+PVOL(J)
           DO K=J,ICOB
              VOK=PVOL(K)
              IF (K .EQ. ICOB .AND. VOIJ .GE. VOK) THEN
                 COAGFR(I,J,K)=1.0
              END IF
              IF (K .LT. ICOB) THEN
                 VOKP1=PVOL(K+1)
                 COAGFR(I,J,K)=CVMGT(VOK/VOIJ*(VOKP1-VOIJ)/(VOKP1-VOK),  &
     &              COAGFR(I,J,K), VOIJ .GE. VOK .AND. VOIJ .LT. VOKP1)
              END IF
              IF (K .GT. 1 ) THEN
                 VOKM1=PVOL(K-1)
                 COAGFR(I,J,K)=CVMGT(1.-COAGFR(I,J,K-1), COAGFR(I,J,K),  &
     &                   VOIJ .GT. VOKM1 .AND. VOIJ .LT. VOK)
              END IF
              COAGFR(J,I,K)=COAGFR(I,J,K)
           END DO
        ENDDO
      END DO

      DO K=1,ICOB
        ITOT=0
        DO J=1,ICOB
            DO I=1,ICOB
               IF (COAGFR(I,J,K) .GT. 0.0) THEN
                  ITOT=ITOT+1
                  IGFIJ(K,ITOT,1)=I
                  IGFIJ(K,ITOT,2)=J
               END IF
            END DO
        END DO
        IGF(K)=ITOT
      END DO
!
  end subroutine cam_set_gem
!------------------------------------------------------------------------------


  SUBROUTINE SPWCON7
! 
!     * DEC 04/90 - M.LAZARE. - T2S NOW -40C INSTEAD OF -15C. 
!     * JAN 30/89 - M.LAZARE. - INCLUDE PARAMETERS USED IN ICE-PHASE AND
!     *                         MOISTURE CONSERVATION CALCULATIONS. 
!     * MAR 15/88 - R.LAPRISE.  ORIGINAL VERSION SPWCONH FOR GCM3H. 
  
!     * SETS CONSTANTS FOR THE GCM7.
!     * ILEVM = JUST ILEV-1.
!     *  LEVS = THE NUMBER OF PROGNOSTIC MOISTURE LEVELS. 
!     *    SH = SIGMA VALUE AT MID POINT OF THERMO. LAYERS. 
!     *   SHB = SIGMA VALUE AT BASE OF THERMO. LAYERS.
!     *    WW = EARTH ROTATION RATE (1/SEC).
!     *     A = EARTH RADIUS (M). 
!     *  GRAV = GRAVITY ACCELERATION (M/SEC**2).
!     *  RGAS = DRY AIR GAS CONSTANT (JOULE/(KG*DEG)).
!     * RGOCP = RGAS/(DRY AIR SPECIFIC HEAT)
!     * FVORT = VORTICITY OF EARTH ROTATION.
! 
! 
!     * PARAMETERS USED IN THERMODYNAMICS AND DYNAMICS. 
! 
   use shr_const_mod   !  wzz, 2008.9.9
!
      COMMON /PARAMS/ WW,TW,RAYON,ASQ,GRAV,RGAS,RGOCP,RGOASQ,CPRES
      COMMON /PARAMS/ RGASV,CPRESV
      real WW,TW,RAYON,ASQ,GRAV,RGAS,RGOCP,RGOASQ,CPRES
      real RGASV,CPRESV
! 
!     * PARAMETERS USED BY  FUNCTION  HTVOCP. 
! 
      COMMON /HTCP/   T1S,T2S,AI,BI,AW,BW,SLP 
      real T1S,T2S,AI,BI,AW,BW,SLP
! 
!     * PARAMETERS USED BY  FUNCTIONS  DEWPNT,  SPCHUM,  DELTAQ.
! 
      COMMON /EPS/    A,B,EPS1,EPS2 
      real A,B,EPS1,EPS2
! 
!     * PARAMETERS USED BY  FUNCTION  GAMSAT. 
! 
      COMMON /GAMS/   EPSS,CAPA 
      real EPSS,CAPA
! 
!     * PARAMETERS USED IN MOIST CONVECTIVE ADJUSTMENT. 
! 
      COMMON /ADJPCP/ HC,HF,HM,AA,DEPTH,LHEAT,MOIADJ,MOIFLX 
      real HC,HF,HM,AA,DEPTH
      integer LHEAT,MOIADJ,MOIFLX
! 
!     * PARAMETERS USED IN ICE PHASE PHYSICS CALCULATIONS.
! 
      COMMON /EPSICE/ AICE,BICE,TICE,QMIN 
      real AICE,BICE,TICE,QMIN
      
      COMMON /PARAM1/ PAI, RVORD, TFREZ, HS_CAM, HV_CAM, DAYLNT
      real PAI, RVORD, TFREZ, HS_CAM, HV_CAM, DAYLNT
!
      real FVORT, CP, RAUW
!-----------------------------------------------------------------------
!     WW     =7.292E-5
      WW     = SHR_CONST_OMEGA
      TW     =WW+WW 
      RAYON  = SHR_CONST_REARTH
      ASQ    = RAYON*RAYON 
      GRAV   = SHR_CONST_G
      RGAS   =287.04E0
      RGOCP  =2.E0/7.E0 
      RGOASQ =RGAS/ASQ
      CPRES  =RGAS/RGOCP
      CPRESV =1870.E0 
      RGASV  =461.50E0
      FVORT  =TW*SQRT(2.E0/3.E0)
! 
      RAUW   =1.E+3 
      DEPTH  = 1.E0/(RAUW*GRAV) 
      CP     = SHR_CONST_CPDAIR
      CAPA   =RGOCP 
      T1S    = SHR_CONST_TKFRZ
      T2S    =233.16E0
      AW     =3.15213E+6/CP 
      BW     =2.38E+3/CP
      AI     =2.88053E+6/CP 
      BI     =0.167E+3/CP 
      A      =21.656E0
      B      =5418.E0 
      AICE   =24.292E0
      BICE   =6141.E0 
      TICE   =233.16E0
      SLP    =1.E0/(T1S-T2S)
      QMIN   =1.E-20
      EPS1   =0.622E0 
      EPS2   =0.378E0 
      EPSS   =EPS1
! 
      HC     =0.95E0 
      HF     =0.95E0 
      HM     =0.95E0 
      LHEAT  =1 
      MOIADJ =1 
      MOIFLX =1 
      AA     =0.0E0 
      IF(HM.LT.1.E0)  AA=1.E0/(6.E0*(1.E0-HM)**2) 
! 
      RVORD  = 0.622
      TFREZ  = 273.16
      HS_CAM = 2.834E6
      HV_CAM = 2.501E6
      DAYLNT = 86400.
!
  END SUBROUTINE SPWCON7
!-----------------------------------------------------------------------
!
  SUBROUTINE INAero(AERONAME, AEROSIZE,   IAT,  ISZ,  RHOP0, ISCAM,     &
     &                        ID,       JD,    IT,  ITR,  ISLEV,  CAMR, &
     &                      DLEV,      LUC,    NG,   NS,     ML,        &
     &                      PLLP,       Z0,    SC,  RCW,     AC, CSPEC, &
     &                       LAI,      RM0,   ZPD, FCAP,     WP,   RGW, &
     &                      AEST,      GAMMA)

!--------------------------------------------------------------------------          
!                                                                                    
!     PURPOSE                                                                        
!     -------                                                                        
!     - READ IN ALL PROPERTIES OF AEROSOLS AND CONTROLS                              
!                                                                                    
!     HISTORY:                                                                       
!     --------                                                                       
!     * AUG 15/1999 - S.L. GONG   READ ALL DRY DEPOSITION
!     *                           PARAMETERS FROM CONTROL FILE
!     *                           FOR NEW 15 LAND-USE TYPES.
!     *                                                                     
!     * JUN 24/1999 - S.L. GONG   ADD CONSISTANCE CHECK
!     *                           GAS DRY DEPOSITION PARAMETERS
!     *                                                                              
!     * JAN  4/1997 - S.L. GONG   FIRST VERSION                                        
!     *                                                                              
!                                                                                    
!------------------------------------------------------------------------------      
! **********************************************************************
! 
!    PURPOSE ----ALL Non-change parameters for dry deposition subroutine.
!    
!         LEIMING ZHANG           JULY 20, 1997
!         
!***********************************************************************
!
!
      integer ITR, IT, ISLEV, ID, JD, ISCAM
      integer LUC, NS, NG
      integer IAT, ISZ, ML

      REAL PLLP(LUC,NS),Z0(LUC,NS),SC(NG), RCW(NG), AC(NG)
      REAL LAI(LUC,NS),RM0(NG), ZPD(LUC,NS)
      REAL FCAP(ML),WP(ML),RGW(NG)
      REAL AEST(LUC),GAMMA(LUC)
      real CAMR

      CHARACTER*4 CSPEC(NG)
      CHARACTER AERONAME(1)*8
      CHARACTER*3 LAB                                                                
      CHARACTER*90 LINE, LIN                                                         
      LOGICAL OK                                                                     
                                                                                     
      REAL AEROSIZE(2,ISZ+1), RHOP0(IAT)
      INTEGER DLEV                                                   

      integer IDEBUG,   IMM,    IMA,   INU,  ICOB,IDRYDEP
      integer ITUBMIX, ICOAG,I_SS,IINCLD,IBLCLD,ICCRN
      COMMON /AEROSWTH/ IDEBUG,   IMM,    IMA,   INU,  ICOB,IDRYDEP, &
     &                 ITUBMIX, ICOAG,I_SS,IINCLD,IBLCLD,ICCRN     
      real AVNO,    RGASi,     AM,   BOLTZK,  PI,  A
      real A0,RSN0,RCUT0,RCG0,P
      COMMON /NARCM/ AVNO,    RGASi,     AM,   BOLTZK,  PI,  A
      COMMON /NARCM/ A0,RSN0,RCUT0,RCG0,P
      integer MACHINE,INTSIZE
      COMMON /MACHTYP/ MACHINE,INTSIZE
                                                                                     
!     DATA AVNO/6.022E23/,RGASi/8.314 /                                              
!     DATA   AM/28.97   /,   PI/3.1415926/                                              
!     DATA    A/6371220.0/
!      AVNO     REAL AVOGADROS NUMBER.[MOL-1]                                        
!      RGASi    REAL UNIVERSAL ideal GAS CONSTANT.[J K-1 MOL-1]                      
!      AM       REAL MOLECULAR MASS OF AIR.[-]                                       
!      BOLTZK   REAL BOLTZMANNS CONSTANT.[J K-1]
      integer INPUT
      integer ISIZE, ICOM, IC
      integer NTR, NT, NR, N
      integer IB, IE, IREC
                                                                                     
      AVNO = 6.022E23; RGASi = 8.314
      AM = 28.97;      PI = 3.1415926
      A  = 6371220.0
!                                                                                    
!     * OPEN DATA FILE FOR SIZE AND DENSITY DEFINITIONS                              
!                                                                                    
      BOLTZK=RGASi/AVNO
      INPUT = 30                                                                     

      OPEN (INPUT, FILE='AEROSOL',  FORM='FORMATTED')                                
!      OPEN (INPUT, FILE=AERO_config_file_S,  FORM='FORMATTED')                                
                                                                                     
!                                                                                    
!     * READ MODEL CONTROL                                                           
!                          
      CALL SKIPGET ('cam',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -88)                                          
      READ (LINE(6:9),* ) CAMR                                                   
                                                          
      CALL SKIPGET ('ccr',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -100)                                          
      CALL SWCON (LINE, 4, ICCRN)                                                   

      CALL SKIPGET ('deb',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -1)                                          
      CALL SWCON (LINE, 5, IDEBUG)                                                   
                                                                                     
      CALL SKIPGET ('mmr',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -2)                                          
      CALL SWCON (LINE, 6, IMM)                                                      
                                                                                     
      CALL SKIPGET ('mas',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -3)                                          
      CALL SWCON (LINE, 7, IMA)                                                      
                                                                                     
      CALL SKIPGET ('num',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -4)                                          
      CALL SWCON (LINE, 6, INU)                                                      
                                                                                     
      CALL SKIPGET ('cob',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -5)                                          
      READ (LINE(6:9),'(I4)' ) ICOB                                                    

      CALL SKIPGET ('lat',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -6)                                          
      READ (LINE(6:9),'(I4)' ) JD                                                    
                                                                                     
      CALL SKIPGET ('lon',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -7)                                          
      READ (LINE(6:9),'(I4)' ) ID                                                    

      CALL SKIPGET ('dle',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -67)                                          
      READ (LINE(6:9),'(I4)' ) DLEV                                                    

      CALL SKIPGET ('iko',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -87)                                          
      READ (LINE(9:13),'(I5)' ) IT                                                    

      CALL SKIPGET ('itr',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -89)                                          
      READ (LINE(10:12),'(I3)' ) ITR                                                    
!                                                                                    
!     * READ MODULE CONTROL                                                          
!                                                                                    
      CALL SKIPGET ('inc',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -8)                                          
      CALL SWCON (LINE, 7, IINCLD)                                                   
                                                                                     
      CALL SKIPGET ('bel',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -9)                                          
      CALL SWCON (LINE, 10, IBLCLD)                                                  
                                                                                     
      CALL SKIPGET ('dry',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -14)                                         
      CALL SWCON (LINE, 6, IDRYDEP)                                                  
                                                                                     
      CALL SKIPGET ('coa',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -19)                                         
      READ (LINE(7:10),'(I4)' ) ICOAG                                                    

      CALL SKIPGET ('ica',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -22)                                         
      READ (LINE(7:10),'(I4)' ) ISCAM

      CALL SKIPGET ('ilv',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -22)                                         
      READ (LINE(7:10),'(I4)' ) ISLEV
                                                                                     
      CALL SKIPGET ('tur',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -24)                                         
      CALL SWCON (LINE, 7, ITUBMIX)                                                  
                                                                                     
      CALL SKIPGET ('con',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -29)                                         
      CALL SWCON (LINE, 6, I_SS)                                                  
!                                                                                    
!     * READ IN MODEL PARAMETERS                                                     
!                                                                                    
      CALL SKIPGET ('Bin',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -34)                                         
      READ (LINE(12:15),'(I4)' ) ISIZE                                               
                                                                                     
      CALL SKIPGET ('Com',INPUT, LINE, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -39)                                         
      READ (LINE(12:15),'(I4)') ICOM                                                 
      CALL SKIPGET ('Den',INPUT,  LIN, OK)                                          
      IF (.NOT. OK) CALL XIT(' INAERO', -42)                                         
!
!    CONSISTANCE CHECK
!
      IF (ISIZE .NE. ISZ .OR. ICOM .NE. IAT) THEN
         WRITE (*,*) ' THE VALUE FOR SIZE BIN AND ', ISIZE, ISZ, ICOM, IAT
         WRITE (*,*) ' AEROSOL TYPES ARE NOT AGREED '
         WRITE (*,*) ' BETWEEN AEROSOL AND PARAMETER. '
         WRITE (*,*) ' AEROSOL FILE ', ISIZE,ICOM
         WRITE (*,*) ' PARAMETER    ', ISZ,IAT
         CALL XIT  (' INAERO', -88)
      END IF
                                                                                     
      DO IC = 1, ICOM                                                                
         IB = 22+(IC-1)*12                                                           
         IE = 29+(IC-1)*12                                                           
         AERONAME(IC) = LINE(IB:IE)                                                  
         READ (LIN(IB:IE),*) RHOP0(IC)                                               
      END DO                                                                         
! 
!     * DETERMINE THE MACHINE TYPE FOR NON GCM RUN.
!
      IF (CAMR .GE. 10.0) THEN
!         MACHINE = ME32O64(IDUMMY)        !open it in the future HP
!         INTSIZE = INTEFLT(IDUMMY)
      END IF
!                                                                                    
!     * READ AEROSOL SIZE SPECTRUM SPECIFICATION                                     
!                                                                                    
      IREC = ISIZE/5+1                                                               
      DO N=1, IREC                                                                   
          WRITE(LAB,'(A1,I2.2)')'b',(N-1)*5+1                                        
          CALL SKIPGET (LAB,INPUT, LINE, OK)                                        
          IF (.NOT. OK) CALL XIT(' INAERO', -44)                                     
          IF (N .EQ. IREC) THEN                                                      
            NTR = ISIZE-(IREC-1)*5+1                                                 
          ELSE                                                                       
            NTR = 5                                                                  
          END IF                                                                     
          DO NT=1,NTR
            IB = 6+(NT-1)*16
            IE = 6+(NT-1)*16+7   
           READ(LINE(IB:IE), *) AEROSIZE(1,NT+(N-1)*5)                              
            IF (AEROSIZE(1,NT+(N-1)*5) .EQ. 0.0 ) THEN                               
               WRITE (*,*) ' CAM ERROR: ZERO SIZE'          &
     &           ,' SPECTRUM SPECIFIED IN FILE AEROSOL'                              
                WRITE (*,*) '              BIN NUMBER    : ', ISIZE                  
                WRITE (*,*) '              ZERO SIZE AT  : ', NT+(N-1)*5             
                CALL XIT (' INAERO', -16)                                            
            END IF                                                                   
          END DO                                                                     
      END DO                                                                              
                                                                                     
!
!     READ IN DRY DEPOSITION CONTROL PARAMETERS
!
      CALL SKIPGET ('Sur',INPUT,  LIN, OK)
      READ(INPUT,101) Z0  

      CALL SKIPGET ('Zer',INPUT,  LIN, OK) 
      READ(INPUT,101) ZPD 

      CALL SKIPGET ('Lea',INPUT,  LIN, OK)
      READ(INPUT,101) LAI 

      CALL SKIPGET ('Pll',INPUT,  LIN, OK)
      READ(INPUT,101) PLLP

      CALL SKIPGET ('Aes',INPUT,  LIN, OK)
      READ(INPUT,101) AEST

      CALL SKIPGET ('Gam',INPUT,  LIN, OK)
      READ(INPUT,101) GAMMA

      CALL SKIPGET ('Fie',INPUT,  LIN, OK)   
      READ(INPUT,102) FCAP

      CALL SKIPGET ('Soi',INPUT,  LIN, OK)   
      READ(INPUT,102) WP

      CALL SKIPGET ('Gas',INPUT,  LIN, OK)   
      READ(INPUT,106) CSPEC
      READ(INPUT,103) SC
      READ(INPUT,103) AC
      READ(INPUT,103) RGW  
      READ(INPUT,103) RM0
      READ(INPUT,103) RCW   
      READ(INPUT,105) A0,P
      READ(INPUT,107) RSN0,RCUT0,RCG0

      DO NR=1,ISIZE                                                                  
         AEROSIZE(2,NR) = AEROSIZE(1,NR+1)                                           
         AEROSIZE(1,NR) = AEROSIZE(1,NR)*1.E-6                                       
         AEROSIZE(2,NR) = AEROSIZE(2,NR)*1.E-6                                       
      END DO                                                                         
!                                                                                    
!     * SCREEN OUTPUTS FOR THE INPUTS                                                    
!                                                                                    
      if (masterproc) then
      WRITE(*,700)                                                                   
      REWIND INPUT                                                                   
  100 READ (INPUT,500,END=220) LINE
      WRITE (*,600) LINE                                                             
      GOTO 100                                                                       
      endif
                                                                                     
  220 CONTINUE                                                                       
                                                                                     
      CLOSE (INPUT)                                                                  
101   FORMAT (15F6.2)
102   FORMAT (5x,12F6.2)
103   FORMAT (10x,12F6.0)
104   FORMAT (16x,3E10.2)
105   FORMAT (10x,F10.2,E10.2)
106   FORMAT (11x,12(A4,2x))
107   FORMAT (16x,3F10.0)

  500 FORMAT (A90)                                                                   
  600 FORMAT (1X,A90)                                                                
  700 FORMAT(10X, '++++++++++++++++++++++++++++++++++++++++++++++++++++'   &
     &      /10X, '+                                                  +'   &
     &      /10X, '+             CANADIAN AEROSOL MODULE              +'   &
     &      /10X, '+       -----------    CAM V 5.0    ----------     +'   &
     &      /10X, '+                                                  +'   &
     &      /10X, '++++++++++++++++++++++++++++++++++++++++++++++++++++'   &
     &       )                                                                       
!
  END SUBROUTINE INAero
!-----------------------------------------------------------------------
!
                                                                                   
  SUBROUTINE skipget (ascii,input,Tab, OK)                                       
    integer, intent(in) :: input
    CHARACTER ascii *3, Tab *82                                                    
    LOGICAL OK                                                                     
!
    1 READ(input,500,ERR=200) Tab                                                
      IF (Tab(1:3) .ne. ascii) go to 1                                           
      GO TO 300                                                                          
  500 FORMAT (A82)                                                                   
  200 OK=.FALSE.                                                                     
!
  300 OK=.TRUE.                                                                      
!
  END SUBROUTINE skipget
                                                                                     
  SUBROUTINE SWCON (SWITCH, N, INDEX)                                            
      integer, intent(in)  :: N
      integer, intent(out) :: INDEX
      CHARACTER*3 SWITCH, ON, OFF                                                    
                                                                                     
      DATA   ON/'on '/, OFF/'off'/                                                   
                                                                                     
      IF (SWITCH(N+2:N+5) .EQ. ON) THEN                                              
          INDEX=1                                                                    
      ELSE IF (SWITCH(N+2:N+5) .EQ. OFF) THEN                                        
          INDEX=0                                                                    
      ELSE                                                                           
          WRITE (*,*)  SWITCH(1:N),      &
     &        ' control switch is wrong: ',SWITCH(N+2:N+5)                               
          CALL endrun('SWCON: control switch is wrong')
      END IF                                                                         
                                                                                     
  END SUBROUTINE SWCON 
!
!
  subroutine intrfc2(dt,kount,state, ptend, ftsoil, par, snowhland, &
          landfrac, ice, ocnfrac, grcld_ck, grcld_cs, grcld_fn,    &
          fkt, fhs, PRETROW, cflx, fracis, modx)

!---------------------------------------------------------------------
!    NAME       |          description                         | IN/ |
!               |                                              | OUT |
!_______________|______________________________________________|_____|
!  dt           | timestep       [sec.]                        |  I  |
!  IDAY         | Julian date                                  |  I  |
!  trnch        | slice number                                 |  I  |
!  kount        | timestep number                              |  I  |
!  ni           | horizontal dimension on longitude            |  I  |
!  nj           | horizontal dimension on latitude             |  I  |
!  nk           | vertical dimension                           |  I  |
!  deglat(ni)   | Grid latitude value in degrees               |  I  |
!  deglon(ni)   | Grid longitude value in degrees              |  I  |
!  csza(ni)     | the cos( solar zenith angle )                |  I  |
!  sig (NI,NK)  | sigma value of Local interface (bottom)      |  I  |
!  hght(NI,NK)  | hieght of centre of each layer [m]           |  I  |
!  hght      ???| Height at the each interface level [m]       |  I  |
!  grdz(ni,nk)  | layer thickness [m]                          |  I  |
!  GC1(ni)      | ground cover (land=0.0, sea=-1.0, sea-ice area=+1.0 )
!  fland        | Land use
!  gn_luc       | 15 Land use categories
!
!
!  aero_tr(ni,nk,gn_aerochem) -- tracer array [kg/kg]          | I/O |
!  aero_diags(ni,nk,gn_aerodiags) -- diagnois arry             | I/O |
!  gn_aerochem  | total tracer numbers                         |  P  |
!  gn_aerodiags | tracers number of diagnois arry              |  P  |
!
!  tt(NI,NK)    | Grid slice of mid-layer tempt. [K]           |  I  |
!               | or box centre temperature                    |  I  |
!  hu(NI,NK)    | specific humidity                            |  I  |
!  psfc (NI)    | Grid row of surface pressure [Pa]            |  I  |
!  uu(ni,nk)    | longitude componet of wind speed             |  I  |
!  vv(ni,nk)    | latitude componet of wind speed              |  I  |
!  surfwd(NI)   | surface wind speed (10m hight)   [m/s]       |  I  |
!  RHROW(NI,NK) | Slice of relative humidity [%]               |  I  |
!  snow(ni)     | SNOW COVER (0.0-1.0)                         |  I  |
!  ice(ni)      | ICE COVER OF ENTIRE GRID CELL (0.0-1.0)      |  I  |
!  ftsoil(ni)   | Soil Temperature [K]                         |  I  |
!  fhs(ni)      | Soil moisture                                |  I  |
!  par(ni)      | Ground solar irradiance (W/M**2)             |  I  |
!  grcld_fn     | Cloud fraction of each cell box (3D)         |  I  |
!  flwc(ni,nk)  | Cloud liquid water [kg/kg]                   |  I  |
!  PBLTROW(ni)  | ROW OF PBL TOP INDICES                       |  I  |
!  ftv(ni,nk)   | VIRTUAL TEMP.                                |  I  |
!  fkt(ni,nk)   | VERTICAL DIFF. COEFF.(M2/S)                  |  I  |
!  fws(ni,nk)   | sub-grid vertical velocity                   |  I  |
!
!  grcld_ck(ni,nk) -- Cloud fraction, Convec. [0-1.]           |  I  |
!  grcld_cs(ni,nk) -- Cloud fraction, Strat.  [0-1.]           |  I  |
!  PRETROW(ni,nk,1) -- strat precip rate                       |  I  |
!  PRETROW(ni,nk,2) -- conv precip rate (get from conv6x)      |  I  |
!  ROAROW(ni,nk-1) -- AIR DENSITY    [KG M-3]                  |  I  |
!  fmodv(ni,nk) = sqrt(uu(i,k)**2+vv(i,k)**2) -- wind speed [m/s]

!---------------------------------------------------------------------
  use ccrnbnd,       only: ccrnfldget     ! wzz, 2008.3.11
  use history,       only: outfld

! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    integer,             intent(in)  :: kount
    integer,             intent(in)  :: modx 
    type(physics_state), intent(in ) :: state          ! Physics state variables
    real(r8), intent(in) :: ftsoil(ni) 
    real(r8), intent(in) :: fhs(ni) 
    real(r8), intent(in) :: par(ni) 
    real(r8), intent(in) :: snowhland(ni)              ! snow depth
    real(r8), intent(in) :: landfrac(ni)               ! land fraction
    real(r8), intent(in) :: ice(ni)                    ! ice fraction
    real(r8), intent(in) :: ocnfrac(ni)                ! ocean fraction
    real(r8), intent(in) :: grcld_ck(ni,nk)
    real(r8), intent(in) :: grcld_cs(ni,nk)
    real(r8), intent(in) :: grcld_fn(ni,nk)
    real(r8), intent(in) :: fkt(ni,nk)
!    real(r8), intent(in) :: PBLTROW(ni)
    real(r8), intent(in) :: PRETROW(ni,nk-1,2)

    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies
    real(r8), intent(inout) :: fracis(ni,nk-1,ppcnst)  ! fraction of transported species that are insoluble
    real(r8), intent(out) :: cflx(ni,ppcnst)           ! surface flux

! Local variables
    real mu(ni, nk-1), eu(ni,nk-1), du(ni,nk-1)
    real md(ni, nk-1), mc(ni,nk-1), ed(ni,nk-1)
    real dp(ni, nk-1)
    real zfg(ni,nk)
    real mb(ni), eps0(ni), dsubcld(ni)
    integer ideep(ni), jt(ni), maxg(ni), jd(ni)
    integer lengath
!
#include "aero_chem_order_cam9.cdk"
! add chem_land and gn_luc information, liuqx 20080124
#include "chem_land_bats15.cdk"
    real  :: fland(ni,gn_luc)
    real  :: ZSUB0(ni), HH(gn_luc), Z0(gn_luc), zz(ni,gn_luc)
    real RGAS

      integer ncol
      integer npg    ! the prognostic variable numbers
      integer i, k
      INTEGER IDAY,trnch,nj
      integer ss_indx1                     ! first index of sea-salt

      real(r8) :: zlnd = 0.01              !Roughness length for soil [m]
      real(r8) calday                      ! current calendar day
      real(r8) tmpm(ni,nk)
      REAL GC1(ni),delt
      REAL deglat(ni),deglon(ni),csza(ni),sig(NI,NK),hght(NI,NK),grdz(ni,nk)
      REAL aero_tr(ni,nk,gn_aerochem)
      real cflxx(ni,gn_aerochem)           ! surface flux
!      REAL aero_tr(ni,nk,ppcnst-3), aero_diags(ni,nk,gn_aerodiags)
      REAL tt(NI,NK),hu(NI,NK),psfc (NI),uu(ni,nk),vv(ni,nk),surfwd(NI)
      REAL RHROW(NI,NK),snow(ni)
      REAL flwc(ni,nk)
      REAL ftv(ni,nk),fws(ni,nk)
      REAL ROAROW(ni,nk-1),fmodv(ni,nk)
      real es(ni, nk-1), qs(ni, nk-1)
      real prain(ni,nk-1), nevapr(ni,nk-1), WRSCAV(ni,nk-1)
! for emissions, 2008.02.13, wzz
!#include "aero_emis_order_geia.cdk"
      data Z0/0.90, 2.65, 0.90, 1.05, 1.15, 0.10, 0.10, 0.04, 0.03, &
              0.10, 0.03, 0.01, 0.000001, 0.000001, 1.00 /
!     HH (CM) IS THE PHYSICAL HEIGHT OF ROUGHNESS ELEMENTS
!     OF 15 KINDS OF LUC
      DATA HH/2000.,4000.,2000.,2000.,3000.,20.,20.,0.2, &
     &        0.2,100.,2.,0.01,0.001,0.001,1000./
!---------------------------------------------------------------------
!
      trnch = state%lchnk

    ss_indx1 = ixaer
    npg = ncnst_aer
!   ss_indx1 = ssyc_idx1()
!   npg = ppcnst-ss_indx1+1
!   if ( trnch .eq. 249) then
!    write(0,*)'wzz: ssinx1 =', ss_indx1, ppcnst, gn_aerochem,npg
!   endif
    if ( npg > gn_aerochem ) then
      call endrun ('intrfc: ERROR: gn_aerochem is not correct')
    endif
!
      ncol =state%ncol

!--------------------------------
! calculation of sigma
!--------------------------------
     do k=1, nk
       do i=1,ncol
         sig(i,k) = state%pint(i,k)/state%ps(i)
       enddo
     enddo

!
! Cosine solar zenith angle for current time step
!
   calday = get_curr_calday()
   call get_rlat_all_p(trnch, ni, deglat)    !  in radians
   call get_rlon_all_p(trnch, ni, deglon)    !  in radians
   call zenith (calday, deglat, deglon, csza, ni)
   where ( csza < 0. ) csza = 0.
   IDAY = calday

!
! calculation of snow cover
!
     do i=1, ncol
       snow(i)=snowhland(i)/(10.*zlnd+snowhland(i))
     enddo
!
! calculation of potential temperature
!
     tmpm(:ncol,2:nk)=state%t(:ncol,:)*state%exner(:ncol,:)
     tmpm(:ncol,1)=tmpm(:ncol,2)

!
! calculation of GC1
!
       do i=1,ncol
        if(ice(i)>0.5 .and. landfrac(i)<0.5) then
              GC1(i)=1.0
        else if (ocnfrac(i)>0.5 .and. ice(i)<0.5) then
              GC1(i)=-1.0
        else
             GC1(i)=0.0
        end if
      enddo


      delt=0.5*dt
 
      aero_tr(:,:,:) = 0.0
      aero_tr(:ncol,2:nk,1:npg) = state%q(:ncol,:,ss_indx1:npg+ss_indx1-1)
!      ptend%q(:ncol,:,:) = state%q(:ncol,:,:)
      hu(:ncol,1)           = state%q(:ncol,1,1)
      hu(:ncol,2:nk)        = state%q(:ncol,:,1)
      flwc(:ncol,1)         = state%q(:ncol,1,2)
      flwc(:ncol,2:nk)      = state%q(:ncol,:,2)
      tt(:ncol,1)           = state%t(:ncol,1)
      tt(:ncol,2:nk)        = state%t(:ncol,:)
      uu(:ncol,1)           = state%u(:ncol,1)
      uu(:ncol,2:nk)        = state%u(:ncol,:)
      vv(:ncol,1)           = state%v(:ncol,1)
      vv(:ncol,2:nk)        = state%v(:ncol,:)
      fws(:ncol,2:nk)       = state%omega(:ncol,:)
      hght(:ncol,1)         = state%zi(:ncol,1)
      hght(:ncol,2:nk)      = state%zi(:ncol,1:nk-1)
!     hght(:ncol,:)      = state%zi(:ncol,:)
!     hght(:ncol,nk)     = state%zm(:ncol,nk-1)
!     hght(:ncol,1)         = state%zm(:ncol,1)
!     hght(:ncol,2:nk)      = state%zm(:ncol,:)
!      grdz(:ncol,:)      = state%pdel(:ncol,:)    !  ???
      do k = 2, nk
         grdz(:ncol,k) = state%zi(:ncol, k-1) - state%zi(:ncol, k)
      enddo
      psfc(:ncol)        = state%ps(:ncol)
!
     do k=1,nk
        do i=1,ncol
          fmodv(i,k)=sqrt(uu(i,k)**2+vv(i,k)**2)
        enddo
     enddo

     call virtem(ncol,ni,nk,tmpm,hu,0.61_r8,ftv)

    RGAS   =287.04E0
    ROAROW(:ncol,:) = state%pmid(:ncol,:)/state%t(:ncol,:)/RGAS

    call aqsat(state%t    ,state%pmid    ,es      ,qs      ,ni   , &
              ncol    ,nk-1    ,1       ,nk-1    )
    RHROW(:ncol,2:nk)     = state%q(:ncol,:,1)/qs(:ncol,:)
    RHROW(:ncol,1) = RHROW(:ncol,2)

!
! PASS LAND_USE DATA TO CAM ARRAYS
!
     call ccrnfldget( ncol, trnch, fland )
!
     deglat(:) = deglat(:)*180./pi         ! to degree
     deglon(:) = deglon(:)*180./pi         ! to degree
!
!      SURFFACE ROUGHNESS LENGTH FOR 15 TYPES OF GN_LUC
      DO k=1,gn_luc
        DO i=1,ncol
            IF(FLAND(I,k).LT.0.11) THEN
              ZZ(I,k)=(0.479*FLAND(I,k)-0.001)*HH(k)*0.01
              IF(ZZ(I,k).LE.0.) ZZ(I,k)=MAX(0.000001,Z0(k))
            ELSE
              ZZ(I,k)=0.00005*HH(k)
            ENDIF
        END DO
      END DO
!
     zsub0(:) = 0.0
     do i=1, ncol
        do k = 1, gn_luc
!          zsub0(i)= zsub0(i) + fland(i,k)*log(zz(i,k))
           zsub0(i)= zsub0(i) + fland(i,k)*zz(i,k)
        enddo
!      zsub0(i)=exp(zsub0(i))
     end do
     surfwd(:) = 0.0
     do i=1, ncol
        do k = 1, gn_luc
           surfwd(i)= surfwd(i) + fland(i,k)*alog(10./z0(k))/alog(state%zm(i,nk-1)/z0(k))
!          surfwd(i)= surfwd(i) + fland(i,k)*alog(10./zz(i,k))/alog(hght(i,nk)/zz(i,k))
        enddo
       surfwd(i)=fmodv(i,nk)*surfwd(i)
!      surfwd(i)=fmodv(i,nk)*alog(10./zsub0(i))/alog(hght(i,nk)/zsub0(i))
     end do
    surfwd(:) = surfwd(:)*0.72     ! experiment reduce as the wind is too large, wzz, 2008.6.10
    if ( modx == 0 ) then
       call outfld('U10M' ,surfwd               , ni   ,trnch)
       call outfld('SOILW' ,fhs                  , ni   ,trnch)
!    if ( trnch .eq. 290) then
!    do i = 1, ncol
!       if( fhs(i)>100.or. fhs(i).ne.fhs(i) ) then
!   write(0,*)'wzz: SOILW =', fhs(i), FLAND(i,8),i,trnch
!       endif
!    enddo
!   write(0,*)'wzz: SOILW =', fhs(24)
!   write(0,*)'wzz: FL =', FLAND(24,8),ice(24), landfrac(24)
!    endif
    endif
!   if ( trnch .eq. 193) then
!     write(0,*)'wzz: surfwd =', surfwd(1), fmodv(1,nk), zsub0(1), hght(1,nk)
!     write(0,*)'wzz: z0 =', z0
!     write(0,*)'wzz: HH =', HH*0.00005
!     write(0,*)'wzz: FL =', FLAND(1,:)
!    write(0,*)'wzz: aero 0 =', lengath,aero_tr(2,nk,49),state%q(2,nk-1,12+ss_indx1-1)
!   endif
!
      prain(:,:) = 1.E-9
      nevapr(:,:) = 0.5
      WRSCAV(:,:) = 0.5
!
      call aero_chem_cam(delt,trnch,kount,ni,nk, 1, ncol,   &
     &                   aero_tr(:,:,:),gn_aerochem,npg,    &
     &                   tt,hu,psfc,sig,hght,surfwd,        &
     &              fws,fmodv,flwc,ftsoil,                  &
     &              ice,snow,par,fland,gn_luc,grdz,ftv,deglat,deglon,    &
     &              zsub0, csza,grcld_ck,grcld_cs,          &
!    &              tcsza(:,trnch), csza,grcld_ck,grcld_cs,          &
     &              mu,eu,du,md,mc,ed,                      &
     &              dp,mb,eps0,zfg,dsubcld,ideep,jt,maxg,jd,lengath, &
     &              fkt,fhs,uu,vv,PRETROW,                  &
     &              grcld_fn,GC1,IDAY,ROAROW,RHROW,         &
     &              prain, nevapr, WRSCAV, cflxx, fracis(:,:,ss_indx1),modx)

    ptend%lq(1:ss_indx1-1)    = .false.
!   ptend%q(:,:,1:ss_indx1-1) = 0.0
   ptend%lq(ss_indx1:npg+ss_indx1-1)        = .true.
   ptend%q(:ncol,:,ss_indx1:npg+ss_indx1-1) = (aero_tr(:ncol,2:nk,1:npg)-  &
                                               state%q(:ncol,:,ss_indx1:npg+ss_indx1-1))/dt
!   if ( trnch .eq. 193) then
!    do k = 2, nk  
!    do i = 1, ncol
!       if(aero_tr(i,k,12).ne.aero_tr(i,k,12)) then
!       write(0,*)'wzz: i0=',i,k,aero_tr(i,k,12), trnch
!       endif
!    enddo
!    enddo
!    write(0,*)'wzz: aero 1 =', aero_tr(2,nk,12),aero_tr(2,nk,13),ptend%q(2,nk-1,12+ss_indx1-1)
!   endif
!
  end subroutine intrfc2
!
  subroutine intrfcwet(dt,kount,state, ptend, ftsoil, par, snowhland, &
          landfrac, ice, ocnfrac, grcld_ck, grcld_cs, grcld_fn,    &
          prain, nevapr, WRSCAV,mu,eu,du,md,mc,ed,         &
          dp,mb,eps0,zfg,dsubcld,ideep,jt,maxg,jd,lengath, &
          fkt, fhs, PRETROW, cflx, fracis)

!---------------------------------------------------------------------
!    NAME       |          description                         | IN/ |
!               |                                              | OUT |
!_______________|______________________________________________|_____|
!  dt           | timestep       [sec.]                        |  I  |
!  IDAY         | Julian date                                  |  I  |
!  trnch        | slice number                                 |  I  |
!  kount        | timestep number                              |  I  |
!  ni           | horizontal dimension on longitude            |  I  |
!  nj           | horizontal dimension on latitude             |  I  |
!  nk           | vertical dimension                           |  I  |
!  deglat(ni)   | Grid latitude value in degrees               |  I  |
!  deglon(ni)   | Grid longitude value in degrees              |  I  |
!  csza(ni)     | the cos( solar zenith angle )                |  I  |
!  sig (NI,NK)  | sigma value of Local interface (bottom)      |  I  |
!  hght(NI,NK)  | hieght of centre of each layer [m]           |  I  |
!  hght      ???| Height at the each interface level [m]       |  I  |
!  grdz(ni,nk)  | layer thickness [m]                          |  I  |
!  GC1(ni)      | ground cover (land=0.0, sea=-1.0, sea-ice area=+1.0 )
!  fland        | Land use
!  gn_luc       | 15 Land use categories
!
!
!  aero_tr(ni,nk,gn_aerochem) -- tracer array [kg/kg]          | I/O |
!  aero_diags(ni,nk,gn_aerodiags) -- diagnois arry             | I/O |
!  gn_aerochem  | total tracer numbers                         |  P  |
!  gn_aerodiags | tracers number of diagnois arry              |  P  |
!
!  tt(NI,NK)    | Grid slice of mid-layer tempt. [K]           |  I  |
!               | or box centre temperature                    |  I  |
!  hu(NI,NK)    | specific humidity                            |  I  |
!  psfc (NI)    | Grid row of surface pressure [Pa]            |  I  |
!  uu(ni,nk)    | longitude componet of wind speed             |  I  |
!  vv(ni,nk)    | latitude componet of wind speed              |  I  |
!  surfwd(NI)   | surface wind speed (10m hight)   [m/s]       |  I  |
!  RHROW(NI,NK) | Slice of relative humidity [%]               |  I  |
!  snow(ni)     | SNOW COVER (0.0-1.0)                         |  I  |
!  ice(ni)      | ICE COVER OF ENTIRE GRID CELL (0.0-1.0)      |  I  |
!  ftsoil(ni)   | Soil Temperature [K]                         |  I  |
!  fhs(ni)      | Soil moisture                                |  I  |
!  par(ni)      | Ground solar irradiance (W/M**2)             |  I  |
!  grcld_fn     | Cloud fraction of each cell box (3D)         |  I  |
!  flwc(ni,nk)  | Cloud liquid water [kg/kg]                   |  I  |
!  PBLTROW(ni)  | ROW OF PBL TOP INDICES                       |  I  |
!  ftv(ni,nk)   | VIRTUAL TEMP.                                |  I  |
!  fkt(ni,nk)   | VERTICAL DIFF. COEFF.(M2/S)                  |  I  |
!  fws(ni,nk)   | sub-grid vertical velocity                   |  I  |
!
!  grcld_ck(ni,nk) -- Cloud fraction, Convec. [0-1.]           |  I  |
!  grcld_cs(ni,nk) -- Cloud fraction, Strat.  [0-1.]           |  I  |
!  PRETROW(ni,nk,1) -- strat precip rate                       |  I  |
!  PRETROW(ni,nk,2) -- conv precip rate (get from conv6x)      |  I  |
!  ROAROW(ni,nk-1) -- AIR DENSITY    [KG M-3]                  |  I  |
!  fmodv(ni,nk) = sqrt(uu(i,k)**2+vv(i,k)**2) -- wind speed [m/s]

!---------------------------------------------------------------------
  use ccrnbnd,       only: ccrnfldget     ! wzz, 2008.3.11
  use scyc,          only: scyc_idx1      ! wzz, 2008.9.26

! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    integer,             intent(in)  :: kount
    type(physics_state), intent(in ) :: state          ! Physics state variables
    real(r8), intent(in) :: ftsoil(ni) 
    real(r8), intent(in) :: fhs(ni) 
    real(r8), intent(in) :: par(ni) 
    real(r8), intent(in) :: snowhland(ni)              ! snow depth
    real(r8), intent(in) :: landfrac(ni)               ! land fraction
    real(r8), intent(in) :: ice(ni)                    ! ice fraction
    real(r8), intent(in) :: ocnfrac(ni)                ! ocean fraction
    real(r8), intent(in) :: grcld_ck(ni,nk)
    real(r8), intent(in) :: grcld_cs(ni,nk)
    real(r8), intent(in) :: grcld_fn(ni,nk)
    real(r8), intent(in) :: fkt(ni,nk)
    real(r8), intent(in) :: PRETROW(ni,nk-1,2)
    real(r8), intent(in) :: prain(ni,nk-1), nevapr(ni,nk-1)
    real(r8), intent(inout) :: WRSCAV(ni,nk-1)

    type(physics_ptend), intent(inout) :: ptend        ! indivdual parameterization tendencies
    real(r8), intent(inout) :: fracis(ni,nk-1,ppcnst)  ! fraction of transported species that are insoluble
    real(r8), intent(out) :: cflx(ni,ppcnst)           ! surface flux

    real(r8), intent(in) :: mu(ni, nk-1), eu(ni,nk-1), du(ni,nk-1)
    real(r8), intent(in) :: md(ni, nk-1), mc(ni,nk-1), ed(ni,nk-1)
    real(r8), intent(in) :: dp(ni, nk-1)
    real(r8), intent(in) :: zfg(ni,nk)
    real(r8), intent(in) :: mb(ni), eps0(ni), dsubcld(ni)
    integer ideep(ni), jt(ni), maxg(ni), jd(ni)
    integer lengath
!
! Local variables
    integer :: modx 
!
#include "aero_chem_order_cam9.cdk"
! add chem_land and gn_luc information, liuqx 20080124
#include "chem_land_bats15.cdk"
    real  :: fland(ni,gn_luc)
    real  :: ZSUB0(ni), HH(gn_luc), Z0(gn_luc), zz(ni,gn_luc)
    real RGAS

      integer ncol
      integer npg    ! the prognostic variable numbers
      integer i, k, l
      INTEGER IDAY,trnch,nj
      integer ss_indx1                     ! first index of sea-salt
      integer sc_indx1                     ! first index of sulfur

      real(r8) :: zlnd = 0.01              !Roughness length for soil [m]
      real(r8) calday                      ! current calendar day
      real(r8) tmpm(ni,nk)
      REAL GC1(ni),delt
      REAL deglat(ni),deglon(ni),csza(ni),sig(NI,NK),hght(NI,NK),grdz(ni,nk)
      REAL aero_tr(ni,nk,gn_aerochem)
      real cflxx(ni,gn_aerochem)           ! surface flux
!      REAL aero_tr(ni,nk,ppcnst-3), aero_diags(ni,nk,gn_aerodiags)
      REAL tt(NI,NK),hu(NI,NK),psfc (NI),uu(ni,nk),vv(ni,nk),surfwd(NI)
      REAL RHROW(NI,NK),snow(ni)
      REAL flwc(ni,nk)
      REAL ftv(ni,nk),fws(ni,nk)
      REAL ROAROW(ni,nk-1),fmodv(ni,nk)
      real es(ni, nk-1), qs(ni, nk-1)
! for emissions, 2008.02.13, wzz
!#include "aero_emis_order_geia.cdk"
      data Z0/0.90, 2.65, 0.90, 1.05, 1.15, 0.10, 0.10, 0.04, 0.03, &
              0.10, 0.03, 0.01, 0.000001, 0.000001, 1.00 /
!     HH (CM) IS THE PHYSICAL HEIGHT OF ROUGHNESS ELEMENTS
!     OF 15 KINDS OF LUC
      DATA HH/2000.,4000.,2000.,2000.,3000.,20.,20.,0.2, &
     &        0.2,100.,2.,0.01,0.001,0.001,1000./
!---------------------------------------------------------------------
!
      modx = 1     !  wzz, 2008.9.8
      trnch = state%lchnk

    ss_indx1 = ixaer
    npg = ncnst_aer
!   ss_indx1 = ssyc_idx1()
    sc_indx1 = scyc_idx1()
!   npg = ppcnst-ss_indx1+1
!    if ( trnch .eq. 153) then
!     write(0,*)'wzz: ssinx1 =', ss_indx1, ppcnst, gn_aerochem,npg
!    endif
    if ( npg > gn_aerochem ) then
      call endrun ('intrfc: ERROR: gn_aerochem is not correct')
    endif
!
      ncol =state%ncol

!--------------------------------
! calculation of sigma
!--------------------------------
     do k=1, nk
       do i=1,ncol
         sig(i,k) = state%pint(i,k)/state%ps(i)
       enddo
     enddo

      delt=0.5*dt
 
!
! Cosine solar zenith angle for current time step
!
   calday = get_curr_calday()
   call get_rlat_all_p(trnch, ni, deglat)    !  in radians
   call get_rlon_all_p(trnch, ni, deglon)    !  in radians
   call zenith (calday, deglat, deglon, csza, ni)
   where ( csza < 0. ) csza = 0.
   IDAY = calday
!
   if ( .not.allocated(tcsza) ) then
      allocate ( tcsza(ni, begchunk:endchunk) )
      tcsza(:,:) = 86400./1200./3.5     !  initial value, wzz, 2008.10.16
   endif
! the total Cosine solar zenith angle for current day, 2008.10.16
   if ( is_end_curr_day() ) then
      tcsza(:,trnch) = csza
      do k = 1, int(86400/delt)-1
       calday = calday + delt/86400.
       call zenith (calday, deglat, deglon, snow, ncol)
       where ( snow < 0. ) snow = 0.
       tcsza(:,trnch) = tcsza(:,trnch) + snow(:)
      enddo
      where ( tcsza(:,trnch) <=0. ) tcsza(:,trnch) = 1.e-20
      tcsza(:,trnch) = tcsza(:,trnch)/int(86400/delt)    !  daily mean
   endif

!
! calculation of snow cover
!
     do i=1, ncol
       snow(i)=snowhland(i)/(10.*zlnd+snowhland(i))
     enddo
!
! calculation of potential temperature
!
     tmpm(:ncol,2:nk)=state%t(:ncol,:)*state%exner(:ncol,:)
     tmpm(:ncol,1)=tmpm(:ncol,2)

!
! calculation of GC1
!
       do i=1,ncol
        if(ice(i)>0.5 .and. landfrac(i)<0.5) then
              GC1(i)=1.0
        else if (ocnfrac(i)>0.5 .and. ice(i)<0.5) then
              GC1(i)=-1.0
        else
             GC1(i)=0.0
        end if
      enddo


      aero_tr(:,:,:) = 0.0
      aero_tr(:ncol,2:nk,1:npg) = state%q(:ncol,:,ss_indx1:npg+ss_indx1-1)
!      ptend%q(:ncol,:,:) = state%q(:ncol,:,:)
      hu(:ncol,1)           = state%q(:ncol,1,1)
      hu(:ncol,2:nk)        = state%q(:ncol,:,1)
      flwc(:ncol,1)         = state%q(:ncol,1,2)
      flwc(:ncol,2:nk)      = state%q(:ncol,:,2)
      tt(:ncol,1)           = state%t(:ncol,1)
      tt(:ncol,2:nk)        = state%t(:ncol,:)
      uu(:ncol,1)           = state%u(:ncol,1)
      uu(:ncol,2:nk)        = state%u(:ncol,:)
      vv(:ncol,1)           = state%v(:ncol,1)
      vv(:ncol,2:nk)        = state%v(:ncol,:)
      fws(:ncol,2:nk)       = state%omega(:ncol,:)
      hght(:ncol,1)         = state%zi(:ncol,1)
      hght(:ncol,2:nk)      = state%zi(:ncol,1:nk-1)
!     hght(:ncol,:)      = state%zi(:ncol,:)
!     hght(:ncol,nk)     = state%zm(:ncol,nk-1)
!     hght(:ncol,1)         = state%zm(:ncol,1)
!     hght(:ncol,2:nk)      = state%zm(:ncol,:)
!      grdz(:ncol,:)      = state%pdel(:ncol,:)    !  ???
      do k = 2, nk
         grdz(:ncol,k) = state%zi(:ncol, k-1) - state%zi(:ncol, k)
      enddo
      psfc(:ncol)        = state%ps(:ncol)
!
     do k=1,nk
        do i=1,ncol
          fmodv(i,k)=sqrt(uu(i,k)**2+vv(i,k)**2)
        enddo
     enddo

     call virtem(ncol,ni,nk,tmpm,hu,0.61_r8,ftv)

    RGAS   =287.04E0
    ROAROW(:ncol,:) = state%pmid(:ncol,:)/state%t(:ncol,:)/RGAS

    call aqsat(state%t    ,state%pmid    ,es      ,qs      ,ni   , &
              ncol    ,nk-1    ,1       ,nk-1    )
    RHROW(:ncol,2:nk)     = state%q(:ncol,:,1)/qs(:ncol,:)
    RHROW(:ncol,1) = RHROW(:ncol,2)

!
! PASS EMISSIOM DATA TO CAM EMISSION ARRAYS
!
!
     deglat(:) = deglat(:)*180./pi         ! to degree
     deglon(:) = deglon(:)*180./pi         ! to degree
!
     call ccrnfldget( ncol, trnch, fland )
!
!      SURFFACE ROUGHNESS LENGTH FOR 15 TYPES OF GN_LUC
      DO k=1,gn_luc
        DO i=1,ncol
            IF(FLAND(I,k).LT.0.11) THEN
              ZZ(I,k)=(0.479*FLAND(I,k)-0.001)*HH(k)*0.01
              IF(ZZ(I,k).LE.0.) ZZ(I,k)=MAX(0.000001,Z0(k))
            ELSE
              ZZ(I,k)=0.00005*HH(k)
            ENDIF
        END DO
      END DO
!
     zsub0(:) = 0.0
     do i=1, ncol
        do k = 1, gn_luc
!          zsub0(i)= zsub0(i) + fland(i,k)*log(zz(i,k))
           zsub0(i)= zsub0(i) + fland(i,k)*zz(i,k)
        enddo
!      zsub0(i)=exp(zsub0(i))
     end do
     surfwd(:) = 0.0
     do i=1, ncol
        do k = 1, gn_luc
           surfwd(i)= surfwd(i) + fland(i,k)*alog(10./z0(k))/alog(state%zm(i,nk-1)/z0(k))
!          surfwd(i)= surfwd(i) + fland(i,k)*alog(10./zz(i,k))/alog(hght(i,nk)/zz(i,k))
        enddo
       surfwd(i)=fmodv(i,nk)*surfwd(i)
!      surfwd(i)=fmodv(i,nk)*alog(10./zsub0(i))/alog(hght(i,nk)/zsub0(i))
     end do
    surfwd(:) = surfwd(:)*0.72     ! experiment reduce as the wind is too large, wzz, 2008.6.10
!
!   if ( trnch .eq. 185) then
!    write(0,*)'wzz: aero 2 =', grcld_ck(13,21),grcld_cs(13,21)
!    write(0,*)'wzz: aero 2 =', aero_tr(2,nk,12),aero_tr(2,nk,13),state%q(2,nk-1,12+ss_indx1-1), &
!       mu(2,nk-1)
!   endif
      WRSCAV(:,:) = WRSCAV(:,:)*ROAROW(:,:)     !  wzz, 2008.10.23
      call aero_chem_cam(delt,trnch,kount,ni,nk, 1, ncol,   &
     &                   aero_tr(:,:,:),gn_aerochem,npg,    &
     &                   tt,hu,psfc,sig,hght,surfwd,        &
     &              fws,fmodv,flwc,ftsoil,                  &
     &              ice,snow,par,fland,gn_luc,grdz,ftv,deglat,deglon,    &
     &              tcsza(:,trnch), csza,grcld_ck,grcld_cs,                 &
     &              mu,eu,du,md,mc,ed,                      &
     &              dp,mb,eps0,zfg,dsubcld,ideep,jt,maxg,jd,lengath, &
     &              fkt,fhs,uu,vv,PRETROW,                  &
     &              grcld_fn,GC1,IDAY,ROAROW,RHROW,         &
     &              prain, nevapr, WRSCAV, cflxx, fracis(:,:,ss_indx1),modx)

!
      do l = 1, npg
      do k = 1, nk
         do i = 1, ncol
!           if ( aero_tr(i,k,l).gt.0.1.or.aero_tr(i,k,l).lt.0.) then
            if ( aero_tr(i,k,l).lt.0.) then
                 aero_tr(i,k,l) = 0.0
            endif
         enddo
      enddo
      enddo

   ptend%lq(:)    = .false.
!   ptend%q(:,:,1:ss_indx1-1) = 0.0
   ptend%lq(ss_indx1:npg+ss_indx1-1)        = .true.
!  ptend%lq(ss_indx1:sc_indx1-1)        = .true.
   ptend%q(:ncol,:,ss_indx1:npg+ss_indx1-1) = (aero_tr(:ncol,2:nk,1:npg)-  &
                                               state%q(:ncol,:,ss_indx1:npg+ss_indx1-1))/dt

!   if ( trnch .eq. 249) then
!    write(0,*)'wzz: aero 3 =', aero_tr(2,nk,12), aero_tr(2,nk,13),ptend%q(2,nk-1,12+ss_indx1-1), &
!         mu(2,nk-1)
!    do k = 2, nk  
!    do i = 1, ncol
!       if(aero_tr(i,k,12).ne.aero_tr(i,k,12)) then
!       write(0,*)'wzz: i1=',i,k,aero_tr(i,k,12)
!       endif
!    enddo
!    enddo
!   endif
!
  end subroutine intrfcwet
!
end module cuace_driver
