#include <misc.h>
#include <params.h>

module ocarbon_intr

!---------------------------------------------------------------------------------
! Module to interface the aerosol parameterizations with CAM
! written by PJR (extensively modified from chemistry module)
!---------------------------------------------------------------------------------

!-----the following 7 lines are added by lqx, 20070906----------------------
  use shr_kind_mod,only: r8 => shr_kind_r8
  use pmgrid,      only: plat, plev, plevp, masterproc
  use ppgrid,      only: pcols, pver
  use shr_const_mod, only: SHR_CONST_G
!  use physconst,   only: mwdry, mwh2o
  use constituents,only: ppcnst, cnst_add, cnst_name, advected, cnst_get_ind, iaero
  use cam_drydep_mod, only: setdvel, ddflux
!-------------------------------------------------------------------------
  use abortutils,  only: endrun
    

  implicit none

  private          ! Make default type private to the module

  save

  integer :: ncnst                                   ! number of constituents
  character(len=8), allocatable :: cnst_names(:)     ! constituent names
  integer  ocarbon_idx1                              ! start constituent index for first OC cycle component
!------------------------------------------------------------------------------

!
! Public interfaces
!
  public ocarbon_register_cnst                     ! register consituents, liuqx 20070906
  public ocarbon_implements_cnst                      ! returns true if consituent is implemented by this package
  public ocarbon_init_cnst                            ! initialize mixing ratios if not read from initial file
  public ocarbon_initialize                           ! initialize (history) variables
  public ocarbon_wet_intr                             ! interface to wet deposition
  public ocarbon_emis_intr                            ! interface to emission
  public ocarbon_drydep_intr                          ! interface to tendency computation
  public ocarbon_time_interp                          ! interpolate oxidants and fluxes to current time
  public ocyc_idx1, ncnst                             ! constituent index of 1st OC cycle component

!-----------defining mw and cp of all the black ocarbon bins by Liuqx 20070906--------------------

  real(r8), parameter:: cpOC1 = 666.
  real(r8), parameter:: cpOC2 = 666.
  real(r8), parameter:: cpOC3 = 666.
  real(r8), parameter:: cpOC4 = 666.
  real(r8), parameter:: cpOC5 = 666.
  real(r8), parameter:: cpOC6 = 666.
  real(r8), parameter:: cpOC7 = 666.
  real(r8), parameter:: cpOC8 = 666.
  real(r8), parameter:: cpOC9 = 666.
  real(r8), parameter:: cpOC10 = 666.
  real(r8), parameter:: cpOC11 = 666.
  real(r8), parameter:: cpOC12 = 666.


  real(r8), parameter:: mwOC1  = 132.1342
  real(r8), parameter:: mwOC2  = 9999.
  real(r8), parameter:: mwOC3 = 9999.
  real(r8), parameter:: mwOC4  = 9999.
  real(r8), parameter:: mwOC5  = 9999.
  real(r8), parameter:: mwOC6  = 9999.
  real(r8), parameter:: mwOC7 = 9999.
  real(r8), parameter:: mwOC8  = 9999.
  real(r8), parameter:: mwOC9  = 9999.
  real(r8), parameter:: mwOC10  = 9999.
  real(r8), parameter:: mwOC11= 9999.
  real(r8), parameter:: mwOC12 = 9999.

  real(r8), parameter::  gravit = SHR_CONST_G
!------------------------------------------------

contains

!===============================================================================
  subroutine ocarbon_register_cnst
!----------------------------------------------------------------------- 
! 
! Purpose: register advected constituents for all aerosols
! 
! Method: 
! Adopt the registering method of CUACEAero which defines 12 bins of organic carbon species
!---------------------------Local workspace-----------------------------
!
    integer :: m,i                                 ! tracer index
    integer :: istat
    real(r8), parameter :: zero  = 0._r8
    character names*8
!-----------------------------------------------------------------------

! Define constituent names
    ncnst = ocarboncyc_ncomp()  !liuqx 20070906
 
    if ( ncnst > 0 ) then
    allocate( cnst_names(ncnst), stat=istat )
    if ( istat /= 0 ) then
      call endrun ('OCARBON_REGISTER_CNST: ERROR: allocate of cnst_names failed') !liuqx 20070906
    end if
    end if

    select case ( iaero )
    case (0)
       call endrun('OCARBON_REGISTER_CNST:  code under development')

    case (1)
       if ( ncnst < 1 ) then
       call endrun('OCARBON_REGISTER_CNST:  the selected species are not correct')
       end if
       names = 'OC01'
       do i = 1, ncnst
          write(names(3:4),'(i2.2)') i
          cnst_names(i) = names
          call cnst_add(cnst_names(i), advected, mwOC1, cpOC1, zero, m)
          if ( i == 1 ) then
             call ocyc_idx_set(m)
          endif
       enddo

    case default
       call endrun('OCARBON_REGISTER_CNST:  unknown scheme')
    end select

  end subroutine ocarbon_register_cnst

!=======================================================================
  function ocarbon_implements_cnst(name)
!----------------------------------------------------------------------- 
! 
! Purpose: return true if specified constituent is implemented by this 
!          package
! 
!-----------------------------------------------------------------------
!-----------------------------Arguments---------------------------------

     character(len=*), intent(in) :: name   ! constituent name
     logical :: ocarbon_implements_cnst      ! return value
     integer :: m
!-----------------------------------------------------------------------

        ocarbon_implements_cnst = .false.
        do m = 1, ncnst
        if (name == cnst_names(m)) then
           ocarbon_implements_cnst = .true.
           exit
        end if
        end do

  end function ocarbon_implements_cnst


!=======================================================================
  subroutine ocarbon_init_cnst(name, q)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Set initial mass mixing ratios.
!
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8

!-----------------------------Arguments---------------------------------
    
    character(len=*), intent(in) :: name         ! constituent name
    
    real(r8), intent(out) :: q(:,:,:)            !  mass mixing ratio
!-----------------------------------------------------------------------
    
     q = 0._r8
!     if ( name == cnst_names(1) ) then
!     q = 0._r8
!     endif
!    call endrun('OCARBON_INIT_CNST:  code under development')

  end subroutine ocarbon_init_cnst


!===============================================================================
  subroutine ocarbon_initialize 
!----------------------------------------------------------------------- 
! 
! Purpose: initialize parameterization of ocarbon chemistry
!          (declare history variables)
! 
! Method: 
!-----------------------------------------------------------------------
    use history,    only: addfld, add_default, phys_decomp  ! wzz, 2008.8.3
!-----------------------------------------------------------------------
    integer i
!
!    call endrun('OCARBON_INITIALIZE:  code under development')
    select case ( iaero )
    case (0)
       call endrun('OCARBON_INITIALIZE:  code under development')

    case (1)
! wzz, 2008.8.3
       do i = 1, ncnst
          call addfld (trim(cnst_names(i))//'DRY','kg/m2/s ',1, 'A', &
                      'OC dry deposition',phys_decomp)
          call add_default (trim(cnst_names(i))//'DRY', 1, ' ')
!
          call addfld (trim(cnst_names(i))//'WETB','kg/m2/s ',1, 'A', &
                      'OC wet deposition below cloud',phys_decomp)
          call add_default (trim(cnst_names(i))//'WETB', 1, ' ')
!
          call addfld (trim(cnst_names(i))//'WETI','kg/m2/s ',1, 'A', &
                      'OC wet deposition in cloud',phys_decomp)
          call add_default (trim(cnst_names(i))//'WETI', 1, ' ')
       enddo

    case default
       call endrun('OCARBON_INITIALIZE:  unknown scheme')
    end select
!   if( masterproc ) write(0,*)'OCARBON_INITIALIZE: code under development'

  end subroutine ocarbon_initialize


!===============================================================================
  subroutine ocarbon_wet_intr (state, ptend, cflx, nstep, dt, lat, clat, cme, prain, &
       evapr, cldv, cldc, cldn, fracis, calday, cmfdqr, conicw, rainmr)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to we processing of aerosols (source and sinks).
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols, pver
    use constituents,only: ppcnst

    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer, intent(in) :: nstep
    integer, intent(in) :: lat(pcols)                  ! latitude index for S->N storage
    real(r8), intent(in) :: clat(pcols)                    ! latitude 
    real(r8), intent(in) :: cme(pcols,pver)            ! local condensation of cloud water
    real(r8), intent(in) :: prain(pcols,pver)            ! production of rain
    real(r8), intent(in) :: evapr(pcols,pver)            ! evaporation of rain
    real(r8), intent(in) :: cldn(pcols,pver)            ! cloud fraction
    real(r8), intent(in) :: cldc(pcols,pver)            ! convective cloud fraction
    real(r8), intent(in) :: cldv(pcols,pver)            ! cloudy volume undergoing scavenging

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies
    real(r8), intent(inout)  :: cflx(pcols,ppcnst)       ! Surface constituent flux (kg/m^2/s)

    real(r8), intent(inout) :: fracis(pcols,pver,ppcnst)         ! fraction of transported species that are insoluble

    real(r8), intent(in) :: conicw(pcols, pver)
    real(r8), intent(in) :: cmfdqr(pcols, pver)
    real(r8), intent(in) :: rainmr(pcols, pver) ! rain mixing ratio
    real(r8) :: calday        ! current calendar day

!-----------------------------------------------------------------------

    call endrun('OCARBON_WET_INTR:  code under development')

    return

  end subroutine ocarbon_wet_intr

  subroutine ocarbon_drydep_intr (state, ptend, wvflx, dt, lat, clat, &
       fsds, obklen, ts, ustar, prect, snowh, pblh, hflx, month, landfrac, &
       icefrac, ocnfrac)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to parameterized greenhouse gas chemisty (source/sink).
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend
    use ppgrid,      only: pcols

    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    integer, intent(in) :: lat(pcols)                  ! latitude index for S->N storage
    real(r8), intent(in) :: clat(pcols)                 ! latitude 
    real(r8), intent(in) :: fsds(pcols)                 ! longwave down at sfc
    real(r8), intent(in) :: obklen(pcols)                 ! longwave down at sfc
    real(r8), intent(in) :: ustar(pcols)                  ! sfc fric vel
    real(r8), intent(in) :: ts(pcols)                     ! sfc temp
    real(r8), intent(in) :: landfrac(pcols)               ! land fraction
    real(r8), intent(in) :: icefrac(pcols)                ! ice fraction
    real(r8), intent(in) :: ocnfrac(pcols)                ! ocean fraction
    real(r8), intent(in) :: hflx(pcols)                  ! sensible heat flux
    real(r8), intent(in) :: prect(pcols)                     ! prect
    real(r8), intent(in) :: snowh(pcols)                     ! snow depth
    real(r8), intent(in) :: pblh(pcols)                     ! pbl height
    integer, intent(in)  :: month
    real(r8), intent(in) :: wvflx(pcols)       ! water vapor flux

    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies

!-----------------------------------------------------------------------

    call endrun('OCARBON_DRYDEP_INTR:  code under development')

    return
  end subroutine ocarbon_drydep_intr

  subroutine ocarbon_time_interp

    call endrun('OCARBON_TIME_INTERP:  code under development')

  end subroutine ocarbon_time_interp

  subroutine ocarbon_emis_intr (state, ptend, dt)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Interface to emission of all ocarbons
! 
! Method: 
!-----------------------------------------------------------------------
    use shr_kind_mod,only: r8 => shr_kind_r8
    use physics_types, only: physics_state, physics_ptend

    implicit none
!-----------------------------------------------------------------------
!
! Arguments:
!
    real(r8),            intent(in)  :: dt             ! time step
    type(physics_state), intent(in ) :: state          ! Physics state variables
    type(physics_ptend), intent(inout) :: ptend          ! indivdual parameterization tendencies

    call endrun('OCARBON_EMIS_INTR:  code under development')

    return
  end subroutine ocarbon_emis_intr 

!#######################################################################

   integer function ocarboncyc_ncomp()
#include <aero_chem_order_cam9.cdk>
    character*8 aeroname(aerochem_np)
    COMMON /CAM_C/ AERONAME
    integer i

    ocarboncyc_ncomp = 0

    if ( iaero == 1 ) then
      do i = 1, aerochem_np
         if ( AERONAME(i) == 'OMCARBON' ) then
            ocarboncyc_ncomp = aero_nsize
            exit
         end if
      enddo
    endif

   end function ocarboncyc_ncomp

   subroutine ocyc_idx_set(m)
     ! store the start index of the OC species
     integer m
!    write (6,*) ' ocyc: ocyc_idx_set to ', m
     ocarbon_idx1 = m
   end subroutine ocyc_idx_set

   integer function ocyc_idx1()
      ocyc_idx1 = ocarbon_idx1
   end function ocyc_idx1

end module ocarbon_intr
