
      module gc_emis_utils
!---------------------------------------------------------------
! 	... Interpolation surface emissions
!---------------------------------------------------------------

      use pmgrid,         only: plon, plat, masterproc

      implicit none

      private
      public  :: gc_read_emis

      real    :: factor

      contains

      subroutine gc_read_emis( plonl, platl, pplon, species, mw, units, ncid, emis )
!-----------------------------------------------------------------------
!       ... Read surface emissions from NetCDF file
!-----------------------------------------------------------------------

      use mo_netcdf

      use mo_const_mozart,  only : phi, lam, d2r

      use abortutils,     only: endrun
      use error_messages, only: handle_ncerr

      implicit none
!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: plonl, platl, pplon
      integer, intent(in) :: ncid
      real, intent(in)    :: mw
      real, dimension(plonl,platl,12), intent(out) :: emis
      character(len=*), intent(in) :: species, units

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer :: j, jl, ju, m               ! Indices
      integer :: jlim(2)
      integer :: ierr
      integer :: vid, nlat, nlon, ntimes, nvars, ndims, attlen
      integer :: dimid_lat, dimid_lon, dimid_time
      integer, allocatable :: dimid(:)
      integer :: gndx
      real, allocatable :: lat(:), lon(:)
      real :: total, total_wrk, scale_factor
      real, allocatable :: wrk(:,:,:)
      character(len=NF_MAX_NAME) :: varname
      character(len=100) :: var_longname

!-----------------------------------------------------------------------
!       ... Get grid dimensions from file
!-----------------------------------------------------------------------
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lat', dimid_lat ), &
                         'read_emis: Failed to find dimension lat for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lat, nlat ), &
                         'read_emis: Failed to get length of dimension lat for species ' // &
                         trim(species) )
      allocate( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'read_emis: lat allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lat', vid ), &
                         'read_emis: Failed to find variable lat for species ' // trim(species) )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, lat ), &
                         'read_emis: Failed to read variable lat for species ' // trim(species) )
      lat(:nlat) = lat(:nlat) * d2r
 
      call handle_ncerr( NF_INQ_DIMID( ncid, 'lon', dimid_lon ), &
                         'read_emis: Failed to find dimension lon for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'read_emis: Failed to find dimension lon for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_lon, nlon ), &
                         'read_emis: Failed to get length of dimension lon for species ' // &
                         trim(species) )
      allocate( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'read_emis: lon allocation error = ',ierr
         call endrun
      end if
      call handle_ncerr( NF_INQ_VARID( ncid, 'lon', vid ), &
                         'read_emis: Failed to find variable lon for species ' // trim(species) )
      call handle_ncerr( NF_GET_VAR_DOUBLE( ncid, vid, lon ), &
                         'read_emis: Failed to read variable lon for species ' // trim(species) )
      lon(:nlon) = lon(:nlon) * d2r
 
      call handle_ncerr( NF_INQ_DIMID( ncid, 'time', dimid_time ), &
                         'read_emis: Failed to find dimension month for species ' // trim(species) )
      call handle_ncerr( NF_INQ_DIMLEN( ncid, dimid_time, ntimes ), &
                         'read_emis: Failed to get length of dimension month for species ' // &
                         trim(species) )
!      if( nmonth /= 12 ) then
!         write(*,*) 'read_emis: Error, number of months = ',nmonth,' for species ' // trim(species)
!         call endrun
!      end if

      call handle_ncerr( NF_INQ_NVARS( ncid, nvars ), &
                         'read_emis: Failed to get number of variables for species ' // &
                         trim(species) )
!-----------------------------------------------------------------------
! Allocate working emissions array
!-----------------------------------------------------------------------

      allocate( wrk(plonl, platl, ntimes), stat=ierr)
      if (ierr /= 0) then
	write(*,*) 'read_gc_emis: Failed to allocate working array, wrk'
	call endrun
      endif

!-----------------------------------------------------------------------
!       ... Set total emissions to zero
!-----------------------------------------------------------------------
      emis(:,:,:) = 0.
!-----------------------------------------------------------------------
!       ... Loop over file variables
!-----------------------------------------------------------------------
      write(*,*) 'Annual average ',trim(species),' emissions ',trim(units)
      write(*,*) trim(species),' dimensions: ',platl,' (lat) x ',plonl,    &
	' (lon) x ',ntimes,' (time)'

      do vid = 1,nvars
         ierr = NF_INQ_VARNAME( ncid, vid, varname )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: Failed to get name of variable # ',vid, &
                '; species=' // trim(species)
            call endrun
         end if
         call handle_ncerr( NF_INQ_VARNDIMS( ncid, vid, ndims ), &
                            'gc_read_emis: Failed to get number of dimensions for ' // &
                            'variable ' // trim(varname) // ', species=' // trim(species) )
         if( ndims < 3 ) then
            cycle
         elseif( ndims > 3 ) then
            write(*,*) 'gc_read_emis: Skipping variable ', trim(varname),', ndims = ',ndims, &
                       ' , species=',trim(species)
            cycle
         end if
!-----------------------------------------------------------------------
!       ... Check order of dimensions. Must be (lon, lat, month).
!-----------------------------------------------------------------------
         allocate( dimid( ndims ), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'read_emis: dimid allocation error = ',ierr
            call endrun
         end if
         call handle_ncerr( NF_INQ_VARDIMID( ncid, vid, dimid ), &
                            'read_emis: Failed to get dimension IDs for variable ' // &
                            trim(varname)  // ', species=' // trim(species) )
         if( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. dimid(3) /= dimid_time ) then
            write(*,*) 'gc_read_emis: Dimensions in wrong order for variable ',trim(varname)
            write(*,*) '...      Expecting (lon, lat, time)'
            call endrun
         end if
         deallocate( dimid, stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'gc_read_emis: Failed to deallocate dimid, ierr = ',ierr
            call endrun
         end if
!-----------------------------------------------------------------------
!       ... Check for long_name of variable
!-----------------------------------------------------------------------
         var_longname = ' '
         ierr = NF_GET_ATT_TEXT( ncid, vid, 'long_name', var_longname )
         if( ierr /= 0 ) then
            var_longname = trim( varname )
         end if
!-----------------------------------------------------------------------
!       ... Read data from this variable
!-----------------------------------------------------------------------
         call handle_ncerr( NF_GET_VARA_DOUBLE( ncid, vid, &
                                                (/ 1, 1, 1/), &                 ! start
                                                (/ nlon, nlat, ntimes /), &  ! count
                                                wrk), &
                            'gc_read_emis: Failed to read variable ' // trim( varname ) // &
                            ', species=' // trim(species) )
	 write(*,*) 'gc_read_emis: Successfully read emissions for ', trim(species)
	 write(*,*) 'gc_read_emis: total for time-slab = ', sum(wrk)
!-----------------------------------------------------------------------
!       ... Check for scale_factor
!-----------------------------------------------------------------------
         ierr = NF_GET_ATT_DOUBLE( ncid, vid, 'scale_factor', scale_factor )
         if( ierr /= NF_NOERR  ) then
            scale_factor = 1.
         end if

         total = 0.
         do m = 1,ntimes
           do j = 1,plat
             total = total + SUM( wrk( :plon, j, m ) )
           enddo
           emis (:,:,m) = emis(:,:,m) + wrk(:,:,m)
         enddo
!-----------------------------------------------------------------------
!       ... Get global emission from this source
!-----------------------------------------------------------------------
	 factor = 1.e-6 ! Tg/Kg -> MSL
         total  = total * factor ! convert from Kg/y to Tg/y
 
            write(*,'(2a10,1x,a50,1x,f15.3)') trim(species), trim(varname), trim(var_longname), total
      end do  ! (vid)
!-----------------------------------------------------------------------
!       ... Close NetCDF file
!-----------------------------------------------------------------------
      call handle_ncerr( NF_CLOSE( ncid ), &
                         'read_emis: Failed to close NetCDF file, species=' // trim(species) )

!-----------------------------------------------------------------------
!       ... Get total global emission for this species
!-----------------------------------------------------------------------
      total = 0.
      do j = 1,platl
         total = total + SUM( emis( :plonl, j, :ntimes ) )
      end do
      total = total * factor ! convert from Kg/y to Tg/y
 
      write(*,'(2a10,1x,a50,1x,f10.3)') trim(species), 'TOTAL', 'TOTAL', total

!	write(*,*) 'EMIS TEST: ', wrk !sum(emis)/(plonl*platl)
	
      end subroutine gc_read_emis

! The MOZART equivalent of srf_emis (formerly in this module) is handled in
! GEOS-Chem's 'DO_EMISSIONS' routine. MSL

      end module gc_emis_utils
