#include <misc.h>
#include <params.h>

#ifdef MICROP-MG

  module macrop_driver

  !-------------------------------------------------------------------------------------------------------
  ! Purpose:
  !
  ! Provides the CAM interface to the prognostic cloud macrophysics
  !
  ! Author: Andrew Gettelman, Cheryl Craig October 2010
  ! Origin: modified from stratiform.F90 elements 
  !    (Boville 2002, Coleman 2004, Park 2009, Kay 2010)
  !-------------------------------------------------------------------------------------------------------

  use shr_kind_mod,  only: r8=>shr_kind_r8
  use ppgrid,        only: pcols, pver, pverp
  use physconst,     only: latice
  use constituents,  only: cnst_get_ind

  implicit none
  private
  save

  public :: macrop_driver_tend

  ! ------------------------- !
  ! Private Module Parameters !
  ! ------------------------- !

  ! 'cu_det_st' : If .true. (.false.), detrain cumulus liquid condensate into the pre-existing liquid stratus 
  !               (environment) without (with) macrophysical evaporation. If there is no pre-esisting stratus, 
  !               evaporate cumulus liquid condensate. This option only influences the treatment of cumulus
  !               liquid condensate, not cumulus ice condensate.

  logical,          private, parameter :: cu_det_st  = .false.  

  ! -------------------------------- !
  ! End of Private Module Parameters !
  ! -------------------------------- !

  logical            :: use_shfrc                       ! Local copy of flag from convect_shallow_use_shfrc
  character(len=16)  :: microp_scheme                   ! Microphysics scheme

  integer :: &
    ixcldliq,     &! cloud liquid amount index
    ixcldice,     &! cloud ice amount index
    ixnumliq,     &! cloud liquid number index
    ixnumice,     &! cloud ice water index
    qcwat_idx,    &! qcwat index in physics buffer
    lcwat_idx,    &! lcwat index in physics buffer
    iccwat_idx,   &! iccwat index in physics buffer
    nlwat_idx,    &! nlwat index in physics buffer
    niwat_idx,    &! niwat index in physics buffer
    tcwat_idx,    &! tcwat index in physics buffer
    CC_T_idx,     &!
    CC_qv_idx,    &!
    CC_ql_idx,    &!
    CC_qi_idx,    &!
    CC_nl_idx,    &!
    CC_ni_idx,    &!
    CC_qlst_idx,  &!
    cld_idx,      &! cld index in physics buffer
    ast_idx,      &! stratiform cloud fraction index in physics buffer
    aist_idx,     &! ice stratiform cloud fraction index in physics buffer
    alst_idx,     &! liquid stratiform cloud fraction index in physics buffer
    qist_idx,     &! ice stratiform in-cloud IWC 
    qlst_idx,     &! liquid stratiform in-cloud LWC  
    concld_idx,   &! concld index in physics buffer
    rhdfda_idx,   &! rhdfda index in physics buffer
    fice_idx,     &  
    qme_idx,      & 
    shfrc_idx,    & 
    rhu00_idx,    & 
    concldql_idx, & 
    sh_frac_idx,  &
    dp_frac_idx,  &
    qini_idx,     &
    cldliqini_idx,&
    cldiceini_idx,&
    tini_idx      

  contains

  subroutine macrop_driver_tend(                             &
             state, ptend_all, dtime, landfrac,  &
             ocnfrac,  snowh,            &
             dlf, dlf2, lcl, cmfmc, cmfmc2, ts,          &
             sst, zdu,       &
             pbuf, state_eq, cmeliq )

  !-------------------------------------------------------- !  
  !                                                         ! 
  ! Purpose:                                                !
  !                                                         !
  ! Interface to detrain, cloud fraction and                !
  !     cloud macrophysics subroutines                      !
  !                                                         ! 
  ! Author: A. Gettelman, C. Craig, Oct 2010                !
  ! based on stratiform_tend by D.B. Coleman 4/2010         !
  !                                                         !
  !-------------------------------------------------------- !

  use shr_kind_mod,     only: r8 => shr_kind_r8
  use cloud_fraction,   only: microp_cldfrc
  use physics_types,    only: physics_state, physics_ptend, physics_tend
  use physics_types,    only: physics_ptend_init, physics_update, physics_tend_init
  use physics_types,    only: physics_ptend_sum,  physics_state_copy
  use phys_buffer,      only: pbuf_size_max, pbuf_fld, pbuf_old_tim_idx, pbuf_get_fld_idx
  use constituents,     only: cnst_get_ind,  ppcnst
  use cldwat,           only: pcond, cldwat_fice
  use cldwat2m_macro,   only: mmacro_pcond
  use physconst,        only: cpair
  use time_manager,     only: get_nstep
  use physconst,        only: latice
  use ppgrid,           only: pcols, pver, pverp
  
  use phys_buffer,     only: pbuf_get_fld_idx

!wtw  use cam_history,      only: outfld

  implicit none

  !
  ! Input arguments
  !
  type(physics_state), intent(in)    :: state   ! state variables
  type(physics_ptend), intent(out)   :: ptend_all   ! package tendencies
  type(pbuf_fld),      intent(inout), dimension(pbuf_size_max) :: pbuf

  real(r8), intent(in)  :: dtime                    ! Timestep
  real(r8), intent(in)  :: landfrac(pcols)          ! Land fraction (fraction)
  real(r8), intent(in)  :: ocnfrac (pcols)          ! Ocean fraction (fraction)
  real(r8), intent(in)  :: snowh(pcols)             ! Snow depth over land, water equivalent (m)
  real(r8), intent(in)  :: dlf(pcols,pver)          ! Detrained water from convection schemes
  real(r8), intent(in)  :: dlf2(pcols,pver)         ! Detrained water from shallow convection scheme
  real(r8), intent(in)  :: cmfmc(pcols,pverp)       ! Deep + Shallow Convective mass flux [ kg /s/m^2 ]
  real(r8), intent(in)  :: cmfmc2(pcols,pverp)      ! Shallow convective mass flux [ kg/s/m^2 ]

  real(r8), intent(in)  :: ts(pcols)                ! Surface temperature
  real(r8), intent(in)  :: sst(pcols)               ! Sea surface temperature
  real(r8), intent(in)  :: zdu(pcols,pver)          ! Detrainment rate from deep convection

  integer , intent(in)  :: lcl(pcols)
  ! Equilibrium state variables at the end of macrophysics
  ! Below 'state_eq' is for future use as the input of radiation'PBL scheme

  type(physics_state), intent(out) :: state_eq   ! Equilibrium state variables at the end of macrophysics

  ! for passing to microphysics
  real(r8), intent(out) :: cmeliq(pcols,pver)                      ! Rate of cond-evap of liq within the cloud

!--------------
  !
  ! Local variables
  !
  type(physics_state)   :: state1                   ! Local copy of the state variable
  type(physics_tend )   :: tend                     ! Physics tendencies (empty, needed for physics_update call)
  type(physics_ptend)   :: ptend_loc                ! Package tendencies

  integer i,k
  integer :: lchnk                                  ! Chunk identifier
  integer :: ncol                                   ! Number of atmospheric columns

  ! Physics buffer fields

  integer itim
  real(r8), pointer, dimension(:,:) :: rhdfda       ! calculated here, and out of subroutine cldnrh,dRh/dcloud, old 
  real(r8), pointer, dimension(:,:) :: rhu00        ! out of subroutine cldnrh,and microp_cldfrc,Rh threshold for cloud, old 
  real(r8), pointer, dimension(:,:) :: qcwat        ! in,Cloud water old q
  real(r8), pointer, dimension(:,:) :: tcwat        ! in,Cloud water old temperature
  real(r8), pointer, dimension(:,:) :: lcwat        ! in,calculated here,Cloud liquid water old q
  real(r8), pointer, dimension(:,:) :: iccwat       ! in,calculated here,Cloud ice water old q
  real(r8), pointer, dimension(:,:) :: nlwat        ! calculated here,Cloud liquid droplet number condentration. old.
  real(r8), pointer, dimension(:,:) :: niwat        ! calculated here,Cloud ice    droplet number condentration. old.
  real(r8), pointer, dimension(:,:) :: CC_T         ! calculated here,Grid-mean microphysical tendency
  real(r8), pointer, dimension(:,:) :: CC_qv        ! calculated here,Grid-mean microphysical tendency
  real(r8), pointer, dimension(:,:) :: CC_ql        ! calculated here,Grid-mean microphysical tendency
  real(r8), pointer, dimension(:,:) :: CC_qi        ! calculated here,Grid-mean microphysical tendency
  real(r8), pointer, dimension(:,:) :: CC_nl        ! calculated here,Grid-mean microphysical tendency
  real(r8), pointer, dimension(:,:) :: CC_ni        ! calculated here,Grid-mean microphysical tendency
  real(r8), pointer, dimension(:,:) :: CC_qlst      ! calculated here,In-liquid stratus microphysical tendency
  real(r8), pointer, dimension(:,:) :: cld          ! out of microp_cldfrc,Total cloud fraction
  real(r8), pointer, dimension(:,:) :: ast          ! calculated here,Relative humidity cloud fraction
  real(r8), pointer, dimension(:,:) :: aist         ! out of mmacro_pcond,Physical ice stratus fraction
  real(r8), pointer, dimension(:,:) :: alst         ! out of mmacro_pcond,Physical liquid stratus fraction
  real(r8), pointer, dimension(:,:) :: qist         ! out of mmacro_pcond,Physical in-cloud IWC
  real(r8), pointer, dimension(:,:) :: qlst         ! out of mmacro_pcond,Physical in-cloud LWC
  real(r8), pointer, dimension(:,:) :: concld       ! out of subroutine microp_cldfrc,Convective cloud fraction
  real(r8), pointer, dimension(:,:) :: qme          ! out of mmacro_pcond,Net condensation rate [kg/kg/s]

  real(r8) :: shfrc(pcols,pver)                     ! in,Cloud fraction from shallow convection scheme

  ! Convective cloud to the physics buffer for purposes of ql contrib. to radn.

  real(r8), pointer, dimension(:,:) :: concld_ql    ! =concld,Convective cloud
  real(r8), pointer, dimension(:,:) :: fice_ql      ! =fice, out of cldwat_fice,Cloud ice/water partitioning ratio.

  ! Local variables for cldfrc

  real(r8)  cldst(pcols,pver)                       ! out of subroutine cldnrh,and microp_cldfrc,Stratus cloud fraction
  real(r8)  rhcloud(pcols,pver)                     ! out of microp_cldfrc,Relative humidity cloud (last timestep)
  real(r8)  rhcloud2(pcols,pver)                    ! out of microp_cldfrc,Relative humidity cloud (perturbation)
  real(r8)  clc(pcols)                              ! out of microp_cldfrc,Column convective cloud amount
  real(r8)  relhum(pcols,pver)                      ! out of microp_cldfrc,RH, output to determine drh/da
  real(r8)  rhu002(pcols,pver)                      ! out of microp_cldfrc,Same as rhu00 but for perturbed rh 
  real(r8)  cld2(pcols,pver)                        ! out of microp_cldfrc,Same as cld but for perturbed rh
  real(r8)  concld2(pcols,pver)                     ! out of microp_cldfrc,Same as concld but for perturbed rh 
  real(r8)  cldst2(pcols,pver)                      ! out of microp_cldfrc,Same as cldst but for perturbed rh 
  real(r8)  relhum2(pcols,pver)                     ! out of microp_cldfrc,RH after  perturbation            
  real(r8)  icecldf(pcols,pver)                     ! out of microp_cldfrc,Ice cloud fraction
  real(r8)  liqcldf(pcols,pver)                     ! out of microp_cldfrc,Liquid cloud fraction (combined into cloud)
  real(r8)  icecldf2(pcols,pver)                    ! out of microp_cldfrc,Ice cloud fraction
  real(r8)  liqcldf2(pcols,pver)                    ! out of microp_cldfrc,Liquid cloud fraction (combined into cloud)

  ! Local variables for macrophysics

  real(r8)  rdtime                                  ! here,1./dtime
  real(r8)  qtend(pcols,pver)                       ! here,Moisture tendencies
  real(r8)  ttend(pcols,pver)                       ! here,Temperature tendencies
  real(r8)  ltend(pcols,pver)                       ! here,Cloud liquid water tendencies
  real(r8)  fice(pcols,pver)                        ! out of cldwat_fice,Fractional ice content within cloud
  real(r8)  fsnow(pcols,pver)                       ! out of cldwat_fice,Fractional snow production
  real(r8)  homoo(pcols,pver)                       ! out of mmicro_pcond,homogeneos freezign cloud water
  real(r8)  qcreso(pcols,pver)                      ! out of mmicro_pcond,residual cloud condensation due to removal of excess supersat                                               
  real(r8)  prcio(pcols,pver)                       ! out of mmicro_pcond,autoconversion of cloud ice to snow  
  real(r8)  praio(pcols,pver)                       ! out of mmicro_pcond,accretion of cloud ice by snow  
  real(r8)  qireso(pcols,pver)                      ! out of mmicro_pcond,residual ice deposition due to removal of excess supersat
  real(r8)  ftem(pcols,pver)                        ! temporary workspace
  real(r8)  pracso (pcols,pver)                     ! out of mmicro_pcond,mixing ratio tendency due to accretion of rain by snow (1/s) 
  real(r8)  dpdlfliq(pcols,pver)                    ! here
  real(r8)  dpdlfice(pcols,pver)                    ! here
  real(r8)  shdlfliq(pcols,pver)                    ! here
  real(r8)  shdlfice(pcols,pver)                    ! here
  real(r8)  dpdlft  (pcols,pver)                    ! here
  real(r8)  shdlft  (pcols,pver)                    ! here

  real(r8)  dum1                                   ! here
  real(r8)  qc(pcols,pver)                          !in
  real(r8)  qi(pcols,pver)                          !in
  real(r8)  nc(pcols,pver)                          !in
  real(r8)  ni(pcols,pver)                          !in

  ! Output from mmacro_pcond

  real(r8)  tlat(pcols,pver)
  real(r8)  qvlat(pcols,pver)
  real(r8)  qcten(pcols,pver)
  real(r8)  qiten(pcols,pver)
  real(r8)  ncten(pcols,pver)
  real(r8)  niten(pcols,pver)

  ! Output from mmacro_pcond

  real(r8)  qvadj(pcols,pver)                       ! Macro-physics adjustment tendency from "positive_moisture" call (vapor)
  real(r8)  qladj(pcols,pver)                       ! Macro-physics adjustment tendency from "positive_moisture" call (liquid)
  real(r8)  qiadj(pcols,pver)                       ! Macro-physics adjustment tendency from "positive_moisture" call (ice)
  real(r8)  qllim(pcols,pver)                       ! Macro-physics tendency from "instratus_condensate" call (liquid)
  real(r8)  qilim(pcols,pver)                       ! Macro-physics tendency from "instratus_condensate" call (ice)

  ! For revised macophysics, mmacro_pcond

  real(r8)  itend(pcols,pver)
  real(r8)  lmitend(pcols,pver)
  real(r8)  zeros(pcols,pver)
  real(r8)  t_inout(pcols,pver)
  real(r8)  qv_inout(pcols,pver)
  real(r8)  ql_inout(pcols,pver)
  real(r8)  qi_inout(pcols,pver)
  real(r8)  concld_old(pcols,pver)

  real(r8)  nl_inout(pcols,pver)
  real(r8)  ni_inout(pcols,pver)

  real(r8)  nltend(pcols,pver)
  real(r8)  nitend(pcols,pver)


  ! For detraining cumulus condensate into the 'stratus' without evaporation
  ! This is for use in mmacro_pcond (calculated here)

  real(r8)  dlf_T(pcols,pver)
  real(r8)  dlf_qv(pcols,pver)
  real(r8)  dlf_ql(pcols,pver)
  real(r8)  dlf_qi(pcols,pver)
  real(r8)  dlf_nl(pcols,pver)
  real(r8)  dlf_ni(pcols,pver)

  ! Local variables for CFMIP calculations (calculated here)
  real(r8) :: mr_lsliq(pcols,pver)			! mixing_ratio_large_scale_cloud_liquid (kg/kg)
  real(r8) :: mr_lsice(pcols,pver)			! mixing_ratio_large_scale_cloud_ice (kg/kg)
  real(r8) :: mr_ccliq(pcols,pver)			! mixing_ratio_convective_cloud_liquid (kg/kg)
  real(r8) :: mr_ccice(pcols,pver)			! mixing_ratio_convective_cloud_ice (kg/kg)

  ! ======================================================================

   logical  convect_shallow_use_shfrc
  
  !-----------------------------------------------------------------------
  ! Initialization routine for cloud macrophysics

  ! Find out whether shfrc from convect_shallow will be used in cldfrc

    convect_shallow_use_shfrc = .false.
    
    if( convect_shallow_use_shfrc ) then
        use_shfrc = .true.
        shfrc_idx = pbuf_get_fld_idx('shfrc')
    else 
        use_shfrc = .false.
    endif

    microp_scheme = 'MG'

! Retrieve the indices for pbuf entries

    fice_idx    = pbuf_get_fld_idx('FICE')
    qme_idx     = pbuf_get_fld_idx('QME')

    concld_idx  = pbuf_get_fld_idx('CONCLD')
    qcwat_idx   = pbuf_get_fld_idx('QCWAT')
    lcwat_idx   = pbuf_get_fld_idx('LCWAT')
    tcwat_idx   = pbuf_get_fld_idx('TCWAT')
    cld_idx     = pbuf_get_fld_idx('CLD')
    qini_idx    = pbuf_get_fld_idx('QINI')
    tini_idx    = pbuf_get_fld_idx('TINI')

  !-----------------------------------------------------------------------
    ast_idx       = pbuf_get_fld_idx('AST')
    aist_idx      = pbuf_get_fld_idx('AIST')
    alst_idx      = pbuf_get_fld_idx('ALST')
    qist_idx      = pbuf_get_fld_idx('QIST')
    qlst_idx      = pbuf_get_fld_idx('QLST')
    rhdfda_idx    = pbuf_get_fld_idx('RHDFDA')
    rhu00_idx     = pbuf_get_fld_idx('RHU00')
    iccwat_idx    = pbuf_get_fld_idx('ICCWAT')
    nlwat_idx     = pbuf_get_fld_idx('NLWAT')
    niwat_idx     = pbuf_get_fld_idx('NIWAT')
    CC_T_idx      = pbuf_get_fld_idx('CC_T')
    CC_qv_idx     = pbuf_get_fld_idx('CC_qv')
    CC_ql_idx     = pbuf_get_fld_idx('CC_ql')
    CC_qi_idx     = pbuf_get_fld_idx('CC_qi')
    CC_nl_idx     = pbuf_get_fld_idx('CC_nl')
    CC_ni_idx     = pbuf_get_fld_idx('CC_ni')
    CC_qlst_idx   = pbuf_get_fld_idx('CC_qlst')
    concldql_idx  = pbuf_get_fld_idx('CONCLDQL')
    sh_frac_idx   = pbuf_get_fld_idx('SH_FRAC')
    dp_frac_idx   = pbuf_get_fld_idx('DP_FRAC')
    cldliqini_idx = pbuf_get_fld_idx('CLDLIQINI')
    cldiceini_idx = pbuf_get_fld_idx('CLDICEINI')

!----------------------------------------------------------------------------

  lchnk = state%lchnk
  ncol  = state%ncol

  call cnst_get_ind('CLDLIQ', ixcldliq)
  call cnst_get_ind('CLDICE', ixcldice)
  call cnst_get_ind('NUMLIQ', ixnumliq)
  call cnst_get_ind('NUMICE', ixnumice)

!----------------------------------------------------------------------------------------------------
  call physics_state_copy(state,state1)             ! Copy state to local state1.
  call physics_ptend_init(ptend_loc)                ! Initialize local ptend type
  call physics_ptend_init(ptend_all)                ! Initialize output ptend type
  call physics_tend_init(tend)                      ! tend here is just a null place holder

! Associate pointers with physics buffer fields
  itim = pbuf_old_tim_idx()

  qcwat => pbuf(qcwat_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  rhdfda => pbuf(rhdfda_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  rhu00 => pbuf(rhu00_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  tcwat => pbuf(tcwat_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  lcwat => pbuf(lcwat_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  iccwat => pbuf(iccwat_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  nlwat => pbuf(nlwat_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  niwat => pbuf(niwat_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_T => pbuf(cc_t_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_qv => pbuf(cc_qv_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_ql => pbuf(cc_ql_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_qi => pbuf(cc_qi_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_nl => pbuf(cc_nl_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_ni => pbuf(cc_ni_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  CC_qlst => pbuf(cc_qlst_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  cld => pbuf(cld_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  ast => pbuf(ast_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  aist => pbuf(aist_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  alst => pbuf(alst_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  qist => pbuf(qist_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  qlst => pbuf(qlst_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

  concld => pbuf(concld_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,itim)

! For purposes of convective ql.

  concld_ql => pbuf(concldql_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)                                      

  fice_ql => pbuf(fice_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)

  qme  => pbuf(qme_idx)%fld_ptr(1,1:pcols,1:pver,lchnk, 1)

  ! Initialize convective detrainment tendency

  dlf_T(:,:)  = 0._r8
  dlf_qv(:,:) = 0._r8
  dlf_ql(:,:) = 0._r8
  dlf_qi(:,:) = 0._r8
  dlf_nl(:,:) = 0._r8
  dlf_ni(:,:) = 0._r8

   ! ------------------------------------- !
   ! From here, process computation begins ! 
   ! ------------------------------------- !

   ! ----------------------------------------------------------------------------- !
   ! Detrainment of convective condensate into the environment or stratiform cloud !
   ! ----------------------------------------------------------------------------- !

   ptend_loc%name = 'pcwdetrain'

   ptend_loc%lq(ixcldliq) = .TRUE.
   ptend_loc%lq(ixcldice) = .TRUE.
   ptend_loc%lq(ixnumliq) = .TRUE.
   ptend_loc%lq(ixnumice) = .TRUE.
   ptend_loc%ls           = .TRUE.

     ! Procedures :
     ! (1) Partition detrained convective cloud water into liquid and ice based on T.
     !     This also involves heating.
     !     If convection scheme can handle this internally, this step is not necssary.
     ! (2) Assuming a certain effective droplet radius, computes number concentration
     !     of detrained convective cloud liquid and ice.
     ! (3) If 'cu_det_st = .true' ('false'), detrain convective cloud 'liquid' into 
     !     the pre-existing 'liquid' stratus ( mean environment ).  The former does
     !     not involve any macrophysical evaporation while the latter does. This is
     !     a kind of 'targetted' deposition. Then, force in-stratus LWC to be bounded 
     !     by qcst_min and qcst_max in mmacro_pcond.
     ! (4) In contrast to liquid, convective ice is detrained into the environment 
     !     and involved in the sublimation. Similar bounds as liquid stratus are imposed.
     ! This is the key procesure generating upper-level cirrus clouds.
     ! The unit of dlf : [ kg/kg/s ]

   do k = 1, pver
   do i = 1, state1%ncol
      if( state1%t(i,k) > 268.15_r8 ) then
          dum1 = 0.0_r8
      elseif( state1%t(i,k) < 238.15_r8 ) then
          dum1 = 1.0_r8
      else
          dum1 = ( 268.15_r8 - state1%t(i,k) ) / 30._r8
      endif
      ptend_loc%q(i,k,ixcldliq) = dlf(i,k) * ( 1._r8 - dum1 )
      ptend_loc%q(i,k,ixcldice) = dlf(i,k) * dum1
    ! dum2                      = dlf(i,k) * ( 1._r8 - dum1 )
      ptend_loc%q(i,k,ixnumliq) = 3._r8 * ( max(0._r8, ( dlf(i,k) - dlf2(i,k) )) * ( 1._r8 - dum1 ) ) /  &
	                              (4._r8*3.14_r8* 8.e-6_r8**3*997._r8) + & ! Deep    Convection
                                  3._r8 * (                         dlf2(i,k)    * ( 1._r8 - dum1 ) ) /  &
								  (4._r8*3.14_r8*10.e-6_r8**3*997._r8)     ! Shallow Convection 
    ! dum2                      = dlf(i,k) * dum1
      ptend_loc%q(i,k,ixnumice) = 3._r8 * ( max(0._r8, ( dlf(i,k) - dlf2(i,k) )) *  dum1 ) / &
                                   (4._r8*3.14_r8*25.e-6_r8**3*500._r8) + & ! Deep    Convection
                                  3._r8 * ( dlf2(i,k)    *  dum1 ) / (4._r8*3.14_r8*50.e-6_r8**3*500._r8)     ! Shallow Convection
      ptend_loc%s(i,k)          = dlf(i,k) * dum1 * latice
    ! Targetted detrainment of convective liquid water either directly into the
    ! existing liquid stratus or into the environment. 
      if( cu_det_st ) then
          dlf_T(i,k)  = ptend_loc%s(i,k)/cpair
          dlf_qv(i,k) = 0._r8
          dlf_ql(i,k) = ptend_loc%q(i,k,ixcldliq)
          dlf_qi(i,k) = ptend_loc%q(i,k,ixcldice)
          dlf_nl(i,k) = ptend_loc%q(i,k,ixnumliq)
          dlf_ni(i,k) = ptend_loc%q(i,k,ixnumice)         
          ptend_loc%q(i,k,ixcldliq) = 0._r8
          ptend_loc%q(i,k,ixcldice) = 0._r8
          ptend_loc%q(i,k,ixnumliq) = 0._r8
          ptend_loc%q(i,k,ixnumice) = 0._r8
          ptend_loc%s(i,k)          = 0._r8
          dpdlfliq(i,k)             = 0._r8
          dpdlfice(i,k)             = 0._r8
          shdlfliq(i,k)             = 0._r8
          shdlfice(i,k)             = 0._r8
          dpdlft  (i,k)             = 0._r8
          shdlft  (i,k)             = 0._r8
       else
          dpdlfliq(i,k) = ( dlf(i,k) - dlf2(i,k) ) * ( 1._r8 - dum1 )
          dpdlfice(i,k) = ( dlf(i,k) - dlf2(i,k) ) * ( dum1 )
          shdlfliq(i,k) = dlf2(i,k) * ( 1._r8 - dum1 )
          shdlfice(i,k) = dlf2(i,k) * ( dum1 )
          dpdlft  (i,k) = ( dlf(i,k) - dlf2(i,k) ) * dum1 * latice/cpair
          shdlft  (i,k) = dlf2(i,k) * dum1 * latice/cpair
      endif
   end do
   end do

!   call outfld( 'DPDLFLIQ ', dpdlfliq, pcols, lchnk )
!   call outfld( 'DPDLFICE ', dpdlfice, pcols, lchnk )
!   call outfld( 'SHDLFLIQ ', shdlfliq, pcols, lchnk )
!   call outfld( 'SHDLFICE ', shdlfice, pcols, lchnk )
!   call outfld( 'DPDLFT   ', dpdlft  , pcols, lchnk )
!   call outfld( 'SHDLFT   ', shdlft  , pcols, lchnk )
!
!   call outfld( 'ZMDLF',     dlf     , pcols, state1%lchnk )

 ! Add hie detrainment tendency to tend from the other prior processes

   call physics_ptend_sum( ptend_loc, ptend_all, state )
   call physics_update( state1, tend, ptend_loc, dtime )
   call physics_ptend_init( ptend_loc )

   ! -------------------------------------- !
   ! Computation of Various Cloud Fractions !
   ! -------------------------------------- !

   ! ----------------------------------------------------------------------------- !
   ! Treatment of cloud fraction in CAM4 and CAM5 differs                          !  
   ! (1) CAM4                                                                      !
   !     . Cumulus AMT = Deep    Cumulus AMT ( empirical fcn of mass flux ) +      !
   !                     Shallow Cumulus AMT ( empirical fcn of mass flux )        !
   !     . Stratus AMT = max( RH stratus AMT, Stability Stratus AMT )              !
   !     . Cumulus and Stratus are 'minimally' overlapped without hierarchy.       !
   !     . Cumulus LWC,IWC is assumed to be the same as Stratus LWC,IWC            !
   ! (2) CAM5                                                                      !
   !     . Cumulus AMT = Deep    Cumulus AMT ( empirical fcn of mass flux ) +      !
   !                     Shallow Cumulus AMT ( internally fcn of mass flux and w ) !
   !     . Stratus AMT = fcn of environmental-mean RH ( no Stability Stratus )     !
   !     . Cumulus and Stratus are non-overlapped with higher priority on Cumulus  !
   !     . Cumulus ( both Deep and Shallow ) has its own LWC and IWC.              !
   ! ----------------------------------------------------------------------------- ! 

   concld_old(:ncol,:pver) = concld(:ncol,:pver)

!---------------------
!
   if( use_shfrc ) then     ! Local copy of flag from convect_shallow_use_shfrc
       shfrc(:pcols,:pver) = pbuf(shfrc_idx)%fld_ptr(1,1:pcols,1:pver,lchnk,1)
   else 
       shfrc(:,:) = 0._r8
   endif

   ! CAM5 only uses 'concld' output from the below subroutine. 
   ! Stratus ('ast' = max(alst,aist)) and total cloud fraction ('cld = ast + concld')
   ! will be computed using this updated 'concld' in the stratiform macrophysics 
   ! scheme (mmacro_pcond) later below. 
   ! Note 'shfrc' and' deep convective cloud fraction' will be saved into the 
   ! physical buffer (SH_FRAC,DP_FRAC) within cldfrc.

   call t_startf("cldfrc")

   call microp_cldfrc( lchnk, ncol, pbuf,                                                 &
                state1%pmid, state1%t, state1%q(:,:,1), state1%omega, state1%phis, &
                shfrc, use_shfrc,                                &
                state1%zm , lcl ,                                &
                cld, rhcloud, clc, state1%pdel,                                    &
                cmfmc, cmfmc2, landfrac,snowh, concld, cldst,                      &
                ts, sst, state1%pint(:,pverp), zdu, ocnfrac, rhu00,                &
                state1%q(:,:,ixcldice), icecldf, liqcldf,                          &
                relhum, 0 )    

   ! Re-calculate cloud with perturbed rh add call cldfrc  

   call microp_cldfrc( lchnk, ncol, pbuf,                                                 &
                state1%pmid, state1%t, state1%q(:,:,1), state1%omega, state1%phis, &
                shfrc, use_shfrc,                                                  &
                state1%zm , lcl ,                                &       
                cld2, rhcloud2, clc, state1%pdel,                                  &
                cmfmc, cmfmc2, landfrac, snowh, concld2, cldst2,                   &
                ts, sst, state1%pint(:,pverp), zdu, ocnfrac, rhu002,               &
                state1%q(:,:,ixcldice), icecldf2, liqcldf2,                        &
                relhum2, 1 )              

   call t_stopf("cldfrc")

   ! Add following to estimate rhdfda. Below block is only for CAM4                       

   rhu00(:ncol,1) = 2.0_r8

   do k = 1, pver
   do i = 1, ncol
      if( relhum(i,k) < rhu00(i,k) ) then
          rhdfda(i,k) = 0.0_r8
      elseif( relhum(i,k) >= 1.0_r8 ) then
          rhdfda(i,k) = 0.0_r8
      else
         ! Under certain circumstances, rh+ cause cld not to changed
         ! when at an upper limit, or w/ strong subsidence
         if( ( cld2(i,k) - cld(i,k) ) < 1.e-4_r8 ) then
               rhdfda(i,k) = 0.01_r8*relhum(i,k)*1.e+4_r8  
         else
               rhdfda(i,k) = 0.01_r8*relhum(i,k)/(cld2(i,k)-cld(i,k))
         endif
      endif
   enddo
   enddo
 
   ! ---------------------------------------------- !
   ! Stratiform Cloud Macrophysics and Microphysics !
   ! ---------------------------------------------- !

   call t_startf('microp_driver_microphys')

   lchnk  = state1%lchnk
   ncol   = state1%ncol
   rdtime = 1._r8/dtime

 ! Define fractional amount of stratus condensate and precipitation in ice phase.
 ! This uses a ramp ( -30 ~ -10 for fice, -5 ~ 0 for fsnow ). 
 ! The ramp within convective cloud may be different

   call cldwat_fice( ncol, state1%t, fice, fsnow )

   ptend_loc%name         = 'cldwat'
   ptend_loc%ls           = .true.
   ptend_loc%lq(1)        = .true.
   ptend_loc%lq(ixcldice) = .true.
   ptend_loc%lq(ixcldliq) = .true.

   ptend_loc%lq(ixnumliq) = .true.
   ptend_loc%lq(ixnumice) = .true.

 ! ------------------------------ !
 ! Liquid Microp_Driver Macrophysics !
 ! ------------------------------ !

   call t_startf('mmacro_pcond')

   zeros(:ncol,:pver)  = 0._r8
   qc(:ncol,:pver) = state1%q(:ncol,:pver,ixcldliq)
   qi(:ncol,:pver) = state1%q(:ncol,:pver,ixcldice)
   nc(:ncol,:pver) = state1%q(:ncol,:pver,ixnumliq)
   ni(:ncol,:pver) = state1%q(:ncol,:pver,ixnumice)

 ! In CAM5, 'microphysical forcing' ( CC_... ) and 'the other advective forcings' ( ttend, ... ) 
 ! are separately provided into the prognostic microp_driver macrophysics scheme. This is an
 ! attempt to resolve in-cloud and out-cloud forcings. 

   if( get_nstep() .le. 1 ) then
       tcwat(:ncol,:pver)   = state1%t(:ncol,:pver)
       qcwat(:ncol,:pver)   = state1%q(:ncol,:pver,1)
       lcwat(:ncol,:pver)   = qc(:ncol,:pver) + qi(:ncol,:pver)
       iccwat(:ncol,:pver)  = qi(:ncol,:pver)
       nlwat(:ncol,:pver)   = nc(:ncol,:pver)
       niwat(:ncol,:pver)   = ni(:ncol,:pver)
       ttend(:ncol,:pver)   = 0._r8
       qtend(:ncol,:pver)   = 0._r8
       ltend(:ncol,:pver)   = 0._r8
       itend(:ncol,:pver)   = 0._r8
       nltend(:ncol,:pver)  = 0._r8
       nitend(:ncol,:pver)  = 0._r8
       CC_T(:ncol,:pver)    = 0._r8
       CC_qv(:ncol,:pver)   = 0._r8
       CC_ql(:ncol,:pver)   = 0._r8
       CC_qi(:ncol,:pver)   = 0._r8
       CC_nl(:ncol,:pver)   = 0._r8
       CC_ni(:ncol,:pver)   = 0._r8
       CC_qlst(:ncol,:pver) = 0._r8
   else
       ttend(:ncol,:pver)   = ( state1%t(:ncol,:pver)   -  tcwat(:ncol,:pver)) * rdtime -   CC_T(:ncol,:pver) 
       qtend(:ncol,:pver)   = ( state1%q(:ncol,:pver,1) -  qcwat(:ncol,:pver)) * rdtime -  CC_qv(:ncol,:pver)
       ltend(:ncol,:pver)   = ( qc(:ncol,:pver) + qi(:ncol,:pver)  -  & 
                                     lcwat(:ncol,:pver) ) * rdtime - (CC_ql(:ncol,:pver) + CC_qi(:ncol,:pver))
       itend(:ncol,:pver)   = ( qi(:ncol,:pver)         - iccwat(:ncol,:pver)) * rdtime -  CC_qi(:ncol,:pver)
       nltend(:ncol,:pver)  = ( nc(:ncol,:pver)         -  nlwat(:ncol,:pver)) * rdtime -  CC_nl(:ncol,:pver)
       nitend(:ncol,:pver)  = ( ni(:ncol,:pver)         -  niwat(:ncol,:pver)) * rdtime -  CC_ni(:ncol,:pver)
   endif
   lmitend(:ncol,:pver) = ltend(:ncol,:pver) - itend(:ncol,:pver)

   t_inout(:ncol,:pver)  =  tcwat(:ncol,:pver) 
   qv_inout(:ncol,:pver) =  qcwat(:ncol,:pver)
   ql_inout(:ncol,:pver) =  lcwat(:ncol,:pver) - iccwat(:ncol,:pver)
   qi_inout(:ncol,:pver) = iccwat(:ncol,:pver)
   nl_inout(:ncol,:pver) =  nlwat(:ncol,:pver)
   ni_inout(:ncol,:pver) =  niwat(:ncol,:pver)

 ! Liquid Microp_Driver Macrophysics.
 ! The main roles of this subroutines are
 ! (1) compute net condensation rate of strayiform liquid ( cmeliq )
 ! (2) compute liquid stratus and ice stratus fractions. 
 ! Note 'ttend...' are advective tendencies except microphysical process while
 !      'CC...'    are microphysical tendencies. 

   call mmacro_pcond( lchnk, ncol, dtime, state1%pmid, state1%pdel,              &
                      t_inout, qv_inout, ql_inout, qi_inout, nl_inout, ni_inout, &                  
                      ttend, qtend, lmitend, itend, nltend, nitend,              &
                      CC_T, CC_qv, CC_ql, CC_qi, CC_nl, CC_ni, CC_qlst,          & 
                      dlf_T, dlf_qv, dlf_ql, dlf_qi, dlf_nl, dlf_ni,             &
                      concld_old, concld, landfrac, snowh,                       &
                      tlat, qvlat, qcten, qiten, ncten, niten,                   &
                      cmeliq, qvadj, qladj, qiadj, qllim, qilim,                 &
                      cld, alst, aist, qlst, qist ) 

 ! Copy of concld/fice to put in physics buffer
 ! Below are used only for convective cloud.

   concld_ql(:ncol,:pver) = concld(:ncol,:pver)
   fice_ql(:ncol,:pver)   = fice(:ncol,:pver)


 ! Compute net stratus fraction using maximum over-lapping assumption

   do k = 1, pver
   do i = 1, ncol
      ast(i,k) = max( alst(i,k), aist(i,k) )      
   enddo
   enddo

   call t_stopf('mmacro_pcond')


   do k = 1, pver
   do i = 1, ncol
      ptend_loc%s(i,k)          =  tlat(i,k)
      ptend_loc%q(i,k,1)        = qvlat(i,k)
      ptend_loc%q(i,k,ixcldliq) = qcten(i,k)
      ptend_loc%q(i,k,ixcldice) = qiten(i,k)
      ptend_loc%q(i,k,ixnumliq) = ncten(i,k)
      ptend_loc%q(i,k,ixnumice) = niten(i,k)
   end do
   end do   

!  call outfld( 'MACPDT   ', tlat ,  pcols, lchnk )
!  call outfld( 'MACPDQ   ', qvlat,  pcols, lchnk )
!  call outfld( 'MACPDLIQ ', qcten,  pcols, lchnk )
!  call outfld( 'MACPDICE ', qiten,  pcols, lchnk )
!  call outfld( 'CLDVAPADJ', qvadj,  pcols, lchnk )
!  call outfld( 'CLDLIQADJ', qladj,  pcols, lchnk )
!  call outfld( 'CLDICEADJ', qiadj,  pcols, lchnk )
!  call outfld( 'CLDLIQDET', dlf_ql, pcols, lchnk )
!  call outfld( 'CLDICEDET', dlf_qi, pcols, lchnk )
!  call outfld( 'CLDLIQLIM', qllim,  pcols, lchnk )
!  call outfld( 'CLDICELIM', qilim,  pcols, lchnk )

!  call outfld( 'CONCLD  ', concld, pcols, lchnk )
!  call outfld( 'CLDST   ', cldst,  pcols, lchnk )
!  call outfld( 'CNVCLD  ', clc,    pcols, lchnk )

! calculations and outfld calls for CLDLIQSTR, CLDICESTR, CLDLIQCON, CLDICECON for CFMIP

   ! initialize local variables
   mr_ccliq(1:ncol,1:pver)=0._r8
   mr_ccice(1:ncol,1:pver)=0._r8
   mr_lsliq(1:ncol,1:pver)=0._r8
   mr_lsice(1:ncol,1:pver)=0._r8

   do k=1,pver
   do i=1,ncol
      if (cld(i,k) .gt. 0._r8) then
    	mr_ccliq(i,k) = 0._r8   !! not seen by radiation, so setting to 0 
	mr_ccice(i,k) = 0._r8   !! not seen by radiation, so setting to 0
	mr_lsliq(i,k) = state%q(i,k,ixcldliq)
	mr_lsice(i,k) = state%q(i,k,ixcldice)
      else
	mr_ccliq(i,k) = 0._r8
	mr_ccice(i,k) = 0._r8
	mr_lsliq(i,k) = 0._r8
	mr_lsice(i,k) = 0._r8
      end if
   end do
   end do

!  call outfld( 'CLDLIQSTR  ', mr_lsliq,    pcols, lchnk )
!  call outfld( 'CLDICESTR  ', mr_lsice,    pcols, lchnk )
!  call outfld( 'CLDLIQCON  ', mr_ccliq,    pcols, lchnk )
!  call outfld( 'CLDICECON  ', mr_ccice,    pcols, lchnk )

 ! Here 'state_eq' is the equlibrium state after macrophysics for potential
 ! use in the radiation scheme later.

   call physics_ptend_sum( ptend_loc, ptend_all, state )
   call physics_update( state1, tend, ptend_loc, dtime )
   call physics_state_copy( state1, state_eq )

end subroutine macrop_driver_tend

end module macrop_driver

#endif
