#include <misc.h>

module chem_surfvals

!-----------------------------------------------------------------------------------
! Purpose: Provides greenhouse gas (ghg) and other species values at the 
!          model level nearest the surface.
!          These values may be time dependent.
!
! Author: Brian Eaton (assembled module from existing scattered code pieces)
!-----------------------------------------------------------------------------------

   use shr_kind_mod, only: r8 => shr_kind_r8
   use pmgrid,       only: masterproc
   use physconst,    only: mwdry, mwco2
   use constituents, only: ppcnst, cnst_mw

   use abortutils, only: endrun
   use dycore, only: get_resolution
 
   implicit none
   private
   save

! Public methods

   public :: &
      chem_surfvals_defaultopts, &! set default values of namelist variables
      chem_surfvals_setopts,     &! get namelist input
      chem_surfvals_init,        &! initialize options that depend on namelist input
      chem_surfvals_set,         &! set surface values
      chem_surfvals_get           ! return surface values for: CO2VMR, CO2MMR, CH4VMR
                                  ! N2OVMR, F11VMR, and F12VMR
 
! Private module data

   ! Default values for namelist variables
   real(r8) :: co2vmr = 3.550e-4         ! co2   volume mixing ratio 
   real(r8) :: n2ovmr = 0.311e-6         ! n2o   volume mixing ratio 
   real(r8) :: ch4vmr = 1.714e-6         ! ch4   volume mixing ratio 
   real(r8) :: f11vmr = 0.280e-9         ! cfc11 volume mixing ratio 
   real(r8) :: f12vmr = 0.503e-9         ! cfc12 volume mixing ratio 
   character(len=16) :: lbc_scenario = 'RAMPED' ! 'FIXED' or 'RAMPED'
   integer  :: lbc_fixed_date = 0        ! ramped gases fixed at this date (if > 0)
   character(len=256) :: lbc_file        ! filepath for lbc data

   integer, parameter :: time_span = 12
   integer :: ncid
   integer :: ntimes
   integer :: nlons, nlats
   integer :: nlbc_spc = 0
   integer :: ndx_ch4, ndx_n2o, ndx_co2, ndx_f11, ndx_f12
   integer :: tim_ndx(2) = 0
   integer :: lbc_map(ppcnst) = 0
   integer, allocatable  :: dates(:)
   real(r8) :: mw_fac(ppcnst)
   real(r8), allocatable :: times(:)
   real(r8), allocatable :: co2_mean(:)
   real(r8), allocatable :: wlbc_conc(:,:,:,:)
   real(r8), allocatable :: lbc_conc(:,:,:,:)
   logical :: doRamp_lbc    ! true => turn on ramping for lbc

!=========================================================================================
contains
!=========================================================================================


subroutine chem_surfvals_defaultopts(co2vmr_out, n2ovmr_out, ch4vmr_out, f11vmr_out, f12vmr_out, &
                                     scenario_ghg_out, rampyear_ghg_out, bndtvghg_out, &
                                     ramp_co2_start_ymd_out, ramp_co2_annual_rate_out, &
                                     ramp_co2_cap_out, lbc_scenario_out, lbc_fixed_date_out, &
                                     lbc_file_out, &
                                     ghg_yearStart_model_out, ghg_yearStart_data_out)
!----------------------------------------------------------------------- 
! Purpose: Return default runtime options
!-----------------------------------------------------------------------

   real(r8),           intent(out), optional :: co2vmr_out
   real(r8),           intent(out), optional :: n2ovmr_out
   real(r8),           intent(out), optional :: ch4vmr_out
   real(r8),           intent(out), optional :: f11vmr_out
   real(r8),           intent(out), optional :: f12vmr_out
   character(len=16),  intent(out), optional :: scenario_ghg_out
   integer,            intent(out), optional :: rampYear_ghg_out
   character(len=256), intent(out), optional :: bndtvghg_out
   integer,            intent(out), optional :: ramp_co2_start_ymd_out
   real(r8),           intent(out), optional :: ramp_co2_annual_rate_out
   real(r8),           intent(out), optional :: ramp_co2_cap_out
   character(len=16),  intent(out), optional :: lbc_scenario_out
   integer,            intent(out), optional :: lbc_fixed_date_out
   character(len=256), intent(out), optional :: lbc_file_out
   integer,            intent(out), optional :: ghg_yearStart_model_out
   integer,            intent(out), optional :: ghg_yearStart_data_out   
   !-----------------------------------------------------------------------

   if ( present(co2vmr_out) ) then
      co2vmr_out = co2vmr
   endif
   if ( present(n2ovmr_out) ) then
      n2ovmr_out = n2ovmr
   endif
   if ( present(ch4vmr_out) ) then
      ch4vmr_out = ch4vmr
   endif
   if ( present(f11vmr_out) ) then
      f11vmr_out = f11vmr
   endif
   if ( present(f12vmr_out) ) then
      f12vmr_out = f12vmr
   endif
   if ( present(lbc_scenario_out) ) then
      lbc_scenario_out = lbc_scenario
   endif
   if ( present(lbc_fixed_date_out) ) then
      lbc_fixed_date_out = lbc_fixed_date
   endif
   if ( present(lbc_file_out) ) then
      lbc_file_out = lbc_file
   endif

end subroutine chem_surfvals_defaultopts

!================================================================================================

subroutine chem_surfvals_setopts(co2vmr_in, n2ovmr_in, ch4vmr_in, f11vmr_in, f12vmr_in, &
                                 scenario_ghg_in, rampyear_ghg_in, bndtvghg_in, &
                                 ramp_co2_start_ymd_in, ramp_co2_annual_rate_in, &
                                 ramp_co2_cap_in, lbc_scenario_in, lbc_fixed_date_in, &
                                 lbc_file_in, &
                                 ghg_yearStart_model_in, ghg_yearStart_data_in)
!----------------------------------------------------------------------- 
! Purpose: Set runtime options
!-----------------------------------------------------------------------

   real(r8),           intent(in), optional :: co2vmr_in
   real(r8),           intent(in), optional :: n2ovmr_in
   real(r8),           intent(in), optional :: ch4vmr_in
   real(r8),           intent(in), optional :: f11vmr_in
   real(r8),           intent(in), optional :: f12vmr_in
   character(len=16),  intent(in), optional :: scenario_ghg_in
   integer,            intent(in), optional :: rampYear_ghg_in
   character(len=256), intent(in), optional :: bndtvghg_in
   integer,            intent(in), optional :: ramp_co2_start_ymd_in
   real(r8),           intent(in), optional :: ramp_co2_annual_rate_in
   real(r8),           intent(in), optional :: ramp_co2_cap_in
   character(len=16),  intent(in), optional :: lbc_scenario_in
   integer,            intent(in), optional :: lbc_fixed_date_in
   character(len=256), intent(in), optional :: lbc_file_in
   integer,            intent(in), optional :: ghg_yearStart_model_in
   integer,            intent(in), optional :: ghg_yearStart_data_in   
!-----------------------------------------------------------------------

   if ( present(co2vmr_in) ) then
      co2vmr = co2vmr_in
   endif
   if ( present(n2ovmr_in) ) then
      n2ovmr = n2ovmr_in
   endif
   if ( present(ch4vmr_in) ) then
      ch4vmr = ch4vmr_in
   endif
   if ( present(f11vmr_in) ) then
      f11vmr = f11vmr_in
   endif
   if ( present(f12vmr_in) ) then
      f12vmr = f12vmr_in
   endif
   if ( present(lbc_scenario_in) ) then
      lbc_scenario = lbc_scenario_in
   endif
   if ( present(lbc_fixed_date_in) ) then
      lbc_fixed_date = lbc_fixed_date_in
   endif
   if ( present(lbc_file_in) ) then
      lbc_file = lbc_file_in
   endif

end subroutine chem_surfvals_setopts

!=========================================================================================

subroutine chem_surfvals_init
!----------------------------------------------------------------------- 
! Purpose: 
! Initialize the ramp options that are controlled by namelist input.
! Set surface values at initial time.
! N.B. This routine must be called after the time manager has been initialized
!      since ghg_chem_surfvals_set calls time manager methods.
! 
! Author: B. Eaton - merged code from parse_namelist and rampnl_ghg.
!-----------------------------------------------------------------------

   use pmgrid,         only : plon, plat
   use ioFileMod,      only : getfil
   use time_manager,   only : get_curr_date
   use constituents,   only : cnst_name, cnst_get_ind
   use string_utils,   only : to_upper
   use error_messages, only : alloc_err
#ifdef SPMD
   use mpishorthand,  only : mpiint, mpir8, mpicom
#endif

   include 'netcdf.inc'

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
   integer  :: m, n
   integer  :: dimid
   integer  :: varid
   integer  :: ndims
   integer  :: nvars
   integer  :: ierr, astat
   integer  :: wrk_date
   integer  :: yr, mon, day, ncsec
   real(r8) :: wrk_time
   character(len=8)  :: cnst, table_cnst
   character(len=32) :: varname
   character(len=256) :: locfn          ! netcdf local filename to open
!-----------------------------------------------------------------------

   if( lbc_scenario == 'FIXED' ) then
      if( lbc_fixed_date == 0 ) then
         write(*,*)'chem_surfvals_init: FIXED lbc requires namelist lbc_fixed_date setting'
         call endrun
      end if
      doRamp_lbc = .false.
      write(*,*) 'chem_surfvals_init: lbc surface values are fixed as follows'
      write(*,*) 'FIXED values at date ',lbc_fixed_date
      wrk_date = lbc_fixed_date
   else if( lbc_scenario == 'RAMPED' ) then
      doRamp_lbc  = .true.
      call get_curr_date( yr, mon, day, ncsec )
      wrk_date = 10000*yr + 100*mon + day
      write(*,*) 'RAMPED values from date ',wrk_date
   else
      write(*,*)'chem_surfvals_init: input namelist SCENARIO_LBC must be set to either FIXED or RAMPED'
      call endrun
   end if

   call cnst_get_ind( 'CH4', ndx_ch4, abort=.false. )
   call cnst_get_ind( 'N2O', ndx_n2o, abort=.false. )
   call cnst_get_ind( 'CO2', ndx_co2, abort=.false. )
   call cnst_get_ind( 'CFC11', ndx_f11, abort=.false. )
   call cnst_get_ind( 'CFC12', ndx_f12, abort=.false. )

!-----------------------------------------------------------------------
!	... readin the lb dataset
!-----------------------------------------------------------------------
masterproc_only : &
   if( masterproc ) then
 
      write(*,*) 'chem_surfvals_init: lbc dataset = ', lbc_file
 
      call getfil (lbc_file, locfn, 0)
      call wrap_open (trim(locfn), NF_NOWRITE, ncid)
      call wrap_inq_dimid( ncid, 'time', dimid )
      call wrap_inq_dimlen( ncid, dimid, ntimes )
      allocate( dates(ntimes), times(ntimes),stat=astat )
      if( astat /= 0 ) then
	 call alloc_err( astat, 'chem_surfvals_init', 'dates,times', ntimes )
      end if
      call wrap_inq_varid( ncid, 'date', varid )
      call wrap_get_var_int( ncid, varid, dates )

      do n = 1,ntimes
         call days0( dates(n), 0, times(n) )
      end do
      call days0( wrk_date, 0, wrk_time )
      if( wrk_time < times(1) .or. wrk_time > times(ntimes) ) then
         write(*,*) 'chem_surfvals_init: initial time is out of range of lbc times'
         call endrun
      end if
      do n = 2,ntimes
         if( wrk_time <= times(n) ) then
            exit
         end if
      end do
      tim_ndx(1) = n - 1
      if( doRamp_lbc ) then
         tim_ndx(2) = min( ntimes,tim_ndx(1) + time_span )
      else
         tim_ndx(2) = n
      end if
#ifdef SURF_DIAGS
      write(*,*) '-------------------------------'
      write(*,*) 'chem_surfvals_init: diagnostics'
      write(*,*) '               file dates'
      write(*,'(10i10)') dates(:)
      write(*,*) '               file times'
      write(*,'(1p,10g12.4)') times(:)
      write(*,*) 'chem_surfvals_init: wrk_date = ',wrk_date
      write(*,*) 'chem_surfvals_init: wrk_time = ',wrk_time
      write(*,*) 'chem_surfvals_init: ntimes   = ',ntimes
      write(*,*) 'chem_surfvals_init: tim_ndx = ',tim_ndx(:)
      write(*,*) '-------------------------------'
#endif
      call wrap_inq_dimid( ncid, 'lon', dimid )
      call wrap_inq_dimlen( ncid, dimid, nlons )
      call wrap_inq_dimid( ncid, 'lat', dimid )
      call wrap_inq_dimlen( ncid, dimid, nlats )
      ierr = nf_inq_nvars( ncid, nvars )
      if( ierr /= nf_noerr ) then
         write(*,*) 'chem_surfvals_init: failed to get variable count; error = ',ierr
         call endrun
      end if

!-----------------------------------------------------------------------
!	... check variables in netcdf file against simulation for match
!-----------------------------------------------------------------------
      do n = 1,nvars
         ierr = nf_inq_varndims( ncid, n, ndims )
         if( ierr /= nf_noerr ) then
            write(*,*) 'chem_surfvals_init: failed to get variable dim count; error = ',ierr
            call endrun
         end if
         if( ndims == 3 ) then
            varname = ' '
            ierr = nf_inq_varname( ncid, n, varname )
            if( ierr /= nf_noerr ) then
               write(*,*) 'chem_surfvals_init: failed to get variable name; error = ',ierr
               call endrun
            end if
            m = index( varname, '_' )
            if( m == 0 ) then
               cycle
            end if
            table_cnst = varname(:m-1)
            do m = 1,ppcnst
               cnst = to_upper( cnst_name(m) )
               if( trim(cnst) == 'CFC11' ) then
                  cnst = 'CFCL3'
               else if( trim(cnst) == 'CFC12' ) then
                  cnst = 'CF2CL2'
               end if
               if( trim(cnst) == trim(table_cnst) ) then
                  lbc_map(m) = n
                  mw_fac(m)  = cnst_mw(m)/mwdry
                  exit
               end if
            end do
         end if
      end do

      nlbc_spc = count( lbc_map(:) > 0 )
      if( nlbc_spc > 0 ) then
!-----------------------------------------------------------------------
!	... special section for co2 mean values
!-----------------------------------------------------------------------
         call wrap_inq_varid( ncid, 'CO2_LBC_mean', varid )
         allocate( co2_mean(ntimes),stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'chem_surfvals_init', 'co2_mean', ntimes )
         end if
         call wrap_get_var_realx( ncid, varid, co2_mean )
         call surfvals_get
      end if
      call wrap_close( ncid )
      write(*,*) '-------------------------------'
      write(*,*) 'chem_surfvals_init: diagnostics'
      write(*,*) 'chem_surfvals_init: mean co2'
      write(*,'(1p,10g12.4)') co2_mean(tim_ndx(1):tim_ndx(2))
      write(*,*) 'chem_surfvals_init: lbc concentrations'
      write(*,'(1p,10g12.4)') wlbc_conc(1,1,1,:)
      write(*,*) ' '
      write(*,'(1p,10g12.4)') wlbc_conc(1,1,2,:)
      write(*,*) '-------------------------------'
   end if masterproc_only

#ifdef SPMD
   call mpibcast( ntimes, 1, mpiint, 0, mpicom )
   call mpibcast( nlbc_spc, 1, mpiint, 0, mpicom )
   call mpibcast( nlons, 1, mpiint, 0, mpicom )
   call mpibcast( nlats, 1, mpiint, 0, mpicom )
   call mpibcast( tim_ndx, 2, mpiint, 0, mpicom )
   call mpibcast( mw_fac, ppcnst, mpir8, 0, mpicom )
   if( .not. masterproc ) then
      allocate( dates(ntimes), times(ntimes),co2_mean(ntimes),stat=astat )
      if( astat /= 0 ) then
	 call alloc_err( astat, 'chem_surfvals_init', 'dates,times,co2_mean', ntimes )
      end if
      if( nlbc_spc > 0 ) then
         allocate( wlbc_conc(nlons,nlats,tim_ndx(2)-tim_ndx(1)+1,nlbc_spc) )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'chem_surfvals_init', 'wlbc_conc', nlons*nlats*nlbc_spc*(tim_ndx(2)-tim_ndx(1)+1) )
         end if
      end if
   end if
   call mpibcast( lbc_map, ppcnst, mpiint, 0, mpicom )
   call mpibcast( dates, ntimes, mpiint, 0, mpicom )
   call mpibcast( times, ntimes, mpir8, 0, mpicom )
   call mpibcast( co2_mean, ntimes, mpir8, 0, mpicom )
   if( nlbc_spc > 0 ) then
      n = nlons*nlats*(tim_ndx(2)-tim_ndx(1)+1)*nlbc_spc
      call mpibcast( wlbc_conc, n, mpir8, 0, mpicom )
      call surfvals_dist
      deallocate( wlbc_conc )
   end if
#else
   if( nlbc_spc > 0 ) then
      call surfvals_dist
      deallocate( wlbc_conc )
   end if
#endif


   if( masterproc ) then
      write(6,*) 'co2 volume mixing ratio = ',co2vmr
      write(6,*) 'ch4 volume mixing ratio = ',ch4vmr
      write(6,*) 'n2o volume mixing ratio = ',n2ovmr
      write(6,*) 'f11 volume mixing ratio = ',f11vmr
      write(6,*) 'f12 volume mixing ratio = ',f12vmr
   end if

#ifdef SURF_DIAGS
   write(*,*) '-------------------------------'
   write(*,*) 'chem_surfvals_init: diagnostics'
   write(*,*) '               lbc_map'
   if( nlbc_spc > 0 ) then
      write(*,*) '               following species have set lbc'
      do m = 1,ppcnst
	 if( lbc_map(m) > 0 ) then
            write(*,*) trim(cnst_name(m))
	 end if
      end do
   else
      write(*,*) '               no species with set lbc'
   end if
   write(*,*) '-------------------------------'
#endif

end subroutine chem_surfvals_init

subroutine surfvals_get
!----------------------------------------------------------------------- 
! Purpose: 
! reads the surfvals from primary netcdf file
!----------------------------------------------------------------------- 

   use error_messages, only : alloc_err
#ifdef SPMD
   use mpishorthand,   only : mpiint, mpir8, mpicom
#endif

   implicit none

!----------------------------------------------------------------------- 
!	... local variables
!----------------------------------------------------------------------- 
   integer :: m, n, tcnt, t
   integer :: astat
   integer :: start(3)
   integer :: cnt(3)

   tcnt     = tim_ndx(2) - tim_ndx(1) + 1
   cnt(:)   = (/ nlons,nlats,tcnt /)
   start(:) = (/ 1,1,tim_ndx(1) /)
   allocate( wlbc_conc(nlons,nlats,cnt(3),nlbc_spc),stat=astat )
   if( astat /= 0 ) then
      call alloc_err( astat, 'surfvals_get', 'wlbc_conc', nlons*nlats*nlbc_spc*tcnt )
   end if
   n = 0
   do m = 1,ppcnst
      if( lbc_map(m) > 0 ) then
         n = n + 1
         call wrap_get_vara_realx( ncid, lbc_map(m), start, cnt, wlbc_conc(:,:,:,n) )
      end if
   end do

end subroutine surfvals_get

subroutine surfvals_dist
!----------------------------------------------------------------------- 
! Purpose: 
! "chunk" the surfvals
!----------------------------------------------------------------------- 

   use ppgrid,         only : pcols, begchunk, endchunk
   use phys_grid,      only : get_lat_p, get_lon_p, get_ncols_p
   use error_messages, only : alloc_err

   implicit none

!----------------------------------------------------------------------- 
!	... local variables
!----------------------------------------------------------------------- 
   integer :: c, ncols
   integer :: i, ic, jc
   integer :: astat

   if( allocated( lbc_conc ) ) then
      deallocate( lbc_conc,stat=astat )
      if( astat /= 0 ) then
         write(*,*) 'surfvals_dist: failed to deallocate lbc_conc; error = ',astat
         call endrun
      end if
   end if
   allocate( lbc_conc(pcols,begchunk:endchunk,tim_ndx(1):tim_ndx(2),nlbc_spc),stat=astat )
   if( astat /= 0 ) then
      c = pcols*(endchunk-begchunk+1)*nlbc_spc*(tim_ndx(2)-tim_ndx(1)+1)
      call alloc_err( astat, 'surfvals_dist', 'lbc_conc', c )
   end if

   do c = begchunk,endchunk
      ncols = get_ncols_p( c )
      do i = 1,ncols
         ic = get_lon_p( c,i )
         jc = get_lat_p( c,i )
         lbc_conc(i,c,tim_ndx(1):tim_ndx(2),:) = wlbc_conc(ic,jc,:tim_ndx(2)-tim_ndx(1)+1,:)
      end do
   end do
#ifdef SURF_DIAGS
      write(*,*) '-------------------------------'
      write(*,*) 'surfvals_dist: diagnostics'
      write(*,*) 'surfvals_dist: mean ch4'
      write(*,'(1p,10g12.4)') ch4_mean(tim_ndx(1):tim_ndx(2))
      ic = get_lon_p( begchunk,1 )
      jc = get_lat_p( begchunk,1 )
      write(*,*) 'surfvals_dist: ic,jc for begchunk,1 = ', ic,jc
      write(*,*) 'surfvals_dist: wlbc concentrations'
      write(*,'(1p,10g12.4)') wlbc_conc(ic,jc,1,:)
      write(*,*) 'surfvals_dist: lbc concentrations'
      write(*,'(1p,10g12.4)') lbc_conc(1,begchunk,tim_ndx(1),:)
      write(*,*) '-------------------------------'
#endif

end subroutine surfvals_dist

subroutine chem_surfvals_set( phys_state )
!----------------------------------------------------------------------- 
! Purpose: 
! Sets volume mixing ratios via interpolation of netcdf input data
!-----------------------------------------------------------------------

   use time_manager,   only : get_curr_date
   use ppgrid,         only : pcols, pver, begchunk, endchunk
   use phys_grid,      only : get_ncols_p
   use physics_types,  only : physics_state
   use constituents,   only : cnst_get_ind
   use ioFileMod,      only : getfil
   use error_messages, only : alloc_err
#ifdef SPMD
   use mpishorthand,   only : mpiint, mpir8, mpicom
#endif

   include 'netcdf.inc'

!-----------------------------------------------------------------------
!	... dummy arguments
!-----------------------------------------------------------------------
   type(physics_state), intent(inout), dimension(begchunk:endchunk) :: phys_state

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
   integer  :: m, n, nm1, ndx    ! indicies
   integer  :: c, ncols, s       ! indicies
   integer  :: yr, mon, day      ! components of a date
   integer  :: ncdate            ! current date in integer format [yyyymmdd]
   integer  :: ncsec             ! current seconds in ncdate
   integer  :: astat

   real(r8) :: wrk_time          ! wrk time
   real(r8) :: fac1              ! time interpolation factor

   character(len=256) :: locfn   ! netcdf local filename to open
!-----------------------------------------------------------------------
!	... anything to do ?
!-----------------------------------------------------------------------
   if( nlbc_spc == 0 ) then
      return
   end if

ramping : &
   if( doRamp_lbc ) then
      call get_curr_date(yr, mon, day, ncsec)
      ncdate = yr*10000 + mon*100 + day
      call days0( ncdate, ncsec, wrk_time )
!-----------------------------------------------------------------------
!	... if ramping read next set of data ?
!-----------------------------------------------------------------------
      if( wrk_time > times(tim_ndx(2)) ) then
         tim_ndx(1) = tim_ndx(2)
         tim_ndx(2) = min( ntimes,tim_ndx(1) + time_span )
         if( masterproc ) then
            call getfil (lbc_file, locfn, 0)
            call wrap_open (trim(locfn), NF_NOWRITE, ncid)
            call surfvals_get
            call wrap_close( ncid )
         end if
#ifdef SPMD
         call mpibcast( tim_ndx, 2, mpiint, 0, mpicom )
         if( .not. masterproc ) then
            allocate( wlbc_conc(nlons,nlats,tim_ndx(2)-tim_ndx(1)+1,nlbc_spc),stat=astat )
            if( astat /= 0 ) then
               n = nlons*nlats*(tim_ndx(2)-tim_ndx(1)+1)*nlbc_spc
               call alloc_err( astat, 'surfvals_get', 'wlbc_conc', n )
            end if
         end if
         n = nlons*nlats*(tim_ndx(2)-tim_ndx(1)+1)*nlbc_spc
         call mpibcast( wlbc_conc, n, mpir8, 0, mpicom )
         call surfvals_dist
         deallocate( wlbc_conc )
#else
         call surfvals_dist
         deallocate( wlbc_conc )
#endif
      end if
   else
      call days0( lbc_fixed_date, 0, wrk_time )
   end if ramping

!-----------------------------------------------------------------------
! determine index into input data
!-----------------------------------------------------------------------
   do n = tim_ndx(1)+1,tim_ndx(2)
      if( wrk_time <= times(n) ) then
         exit
      end if
   end do

!-----------------------------------------------------------------------
! check index
!-----------------------------------------------------------------------
   if( n > tim_ndx(2) ) then
      write(*,*) 'chem_surfvals_set: failed to find present time in lbc array'
      call endrun
   end if

!-----------------------------------------------------------------------
! set timing factor and time interpolate the surface concentrations
!-----------------------------------------------------------------------
   nm1  = n - 1
   fac1 = (wrk_time - times(nm1))/(times(n) - times(nm1))
#ifdef SURF_DIAGS
   write(*,*) '-------------------------------'
   write(*,*) 'chem_surfvals_set: diagnostics'
   write(*,*) 'chem_surfvals_set: nm1,n,fac1 = ',nm1,n,fac1
   write(*,*) 'chem_surfvals_set: wrk_time,times(nm1:n) = ',wrk_time,times(nm1:n)
   write(*,*) '              lbc_map'
   write(*,'(10i10)') lbc_map(:)
   write(*,*) 'size of lbc_conc = ',size(lbc_conc,dim=1),size(lbc_conc,dim=2), &
                                    size(lbc_conc,dim=3),size(lbc_conc,dim=4)
   write(*,*) 'lbnds of lbc_conc = ',lbound(lbc_conc,dim=2),lbound(lbc_conc,dim=3), &
                                     lbound(lbc_conc,dim=4)
   write(*,*) 'ubnds of lbc_conc = ',ubound(lbc_conc,dim=2),ubound(lbc_conc,dim=3), &
                                     ubound(lbc_conc,dim=4)
   write(*,*) '-------------------------------'
#endif

   s = 0
   do m = 1,ppcnst
      if( lbc_map(m) > 0 ) then
         s = s + 1
         do c = begchunk,endchunk
            ncols = get_ncols_p( c )
            phys_state(c)%q(:ncols,pver,m) = lbc_conc(:ncols,c,nm1,s) &
                                           + fac1*(lbc_conc(:ncols,c,n,s) - lbc_conc(:ncols,c,nm1,s))
         end do
      end if
   end do

!-----------------------------------------------------------------------
! set ghg values; note these are in mol/mol
!-----------------------------------------------------------------------
   if( ndx_co2 > 0 ) then
      co2vmr = co2_mean(nm1) + fac1*(co2_mean(n) - co2_mean(nm1))
   end if
   if( ndx_ch4 > 0 ) then
      ch4vmr = phys_state(begchunk)%q(1,pver,ndx_ch4)
   end if
   if( ndx_n2o > 0 ) then
      n2ovmr = phys_state(begchunk)%q(1,pver,ndx_n2o)
   end if
   if( ndx_f11 > 0 ) then
      f11vmr = phys_state(begchunk)%q(1,pver,ndx_f11)
   end if
   if( ndx_f12 > 0 ) then
      f12vmr = phys_state(begchunk)%q(1,pver,ndx_f12)
   end if

#ifdef SURF_DIAGS
   if( masterproc ) then
      write(*,*) '-------------------------------'
      write(*,*) 'chem_surfvals_set: diagnostics'
      write(*,*) 'chem_surfvals_set: indicies = ',ndx_co2,ndx_ch4,ndx_n2o,ndx_f11,ndx_f12
      write(*,*) 'chem_surfvals_set: co2vmr,ch4vmr,n2ovmr,f11vmr,f12vmr'
      write(*,'(1p,5g13.5)') co2vmr,ch4vmr,n2ovmr,f11vmr,f12vmr
      write(*,*) '-------------------------------'
   end if
#endif

!-----------------------------------------------------------------------
! convert surface concentrations from volume to mass mixing ratio
!-----------------------------------------------------------------------
   do m = 1,ppcnst
      if( lbc_map(m) > 0 ) then
         do c = begchunk,endchunk
            ncols = get_ncols_p( c )
            phys_state(c)%q(:ncols,pver,m) = mw_fac(m)*phys_state(c)%q(:ncols,pver,m)
         end do
      end if
   end do

!
! output statements of ramping
!
!  if ( masterproc )then
!     write(6,'(a,f8.2,6(1pe22.14))') 'calday1 = ',calday,co2vmr/1.e-06,ch4vmr/1.e-09, &
!                                     n2ovmr/1.e-09
!     write(6,'(a,f8.2,6(1pe22.14))') 'calday2 = ',calday,cfcscl, &
!                                     (f11(nyrm)*fact1 + f11(nyrp)*fact2),f12vmr/1.e-12
!  end if

end subroutine chem_surfvals_set

subroutine days0( ncdate, ncsec, days )
!----------------------------------------------------------------------- 
! Purpose: Convert date and seconds of day to floating point days since
!          00/01/01
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
!-----------------------------------------------------------------------


   implicit none

!-----------------------------------------------------------------------
!	... dummy arguments
!-----------------------------------------------------------------------
   integer, intent(in)   :: ncdate      ! Current date as yymmdd or yyyymmdd
   integer, intent(in)   :: ncsec       ! Seconds of day for current date
   real(r8), intent(out) :: days        ! Day of year

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
   integer  :: year        ! year
   integer  :: mnth        ! Month number
   integer  :: mday        ! Day number of month
   integer  :: jdcon(12) & ! Starting day number for each month
                    = (/ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /)
   real(r8) :: doy

!-----------------------------------------------------------------------
! Decode month and day
!-----------------------------------------------------------------------
   year = ncdate/10000
   mnth = mod( ncdate,10000 )/100
   if( mnth < 1 .or. mnth > 12) then
      write(*,*) 'days0: Bad month index=', mnth
      call endrun
   end if
   mday = mod(ncdate,100)
   doy  = jdcon(mnth) + mday + ncsec/86400._r8

   if( doy < 1._r8 .or. doy > 366._r8) then
      write(*,*) 'days0: bad day of year = ',doy
      call endrun
   end if

   days = 365._r8*year + doy

end subroutine days0

!=========================================================================================

function chem_surfvals_get(name)

  character(len=*), intent(in) :: name

  real(r8), parameter :: rmwco2 = mwco2/mwdry    ! ratio of molecular weights of co2 to dry air
  real(r8) :: chem_surfvals_get

  select case (name)
  case ('CO2VMR')
     chem_surfvals_get = co2vmr
  case ('CO2MMR')
     chem_surfvals_get = rmwco2 * co2vmr
  case ('N2OVMR')
     chem_surfvals_get = n2ovmr
  case ('CH4VMR')
     chem_surfvals_get = ch4vmr
  case ('F11VMR')
     chem_surfvals_get = f11vmr
  case ('F12VMR')
     chem_surfvals_get = f12vmr
  end select

end function chem_surfvals_get

!=========================================================================================

end module chem_surfvals
