 
module iondrag
!-------------------------------------------------------------------------------
! Purpose:
!   Calculate ion drag tendency and apply to horizontal velocities.
!   Also calculate joule heating tendency and apply to neutral temperature. 
! 
! Subroutines:
!   iondrag_init (initialize module)
!   iondrag_calc (calculate ion drag tensors)
!   iondrag_tend (ion drag tendency)   
!   qjoule_tend (joule heating)
!
! Calling sequence:
!   inti
!     iondrag_init
!   tphysac
!     iondrag_calc
!       iondrag_tend
!       qjoule_tend
!
! Dependencies:
!   Magnetic field from apex module
!   ExB ion drifts from exbdrift module
!
! Author:
!   B. Foster Feb, 2004
!
!-------------------------------------------------------------------------------

  use shr_kind_mod ,only: r8 => shr_kind_r8
  use ppgrid       ,only: pcols, pver
  use history      ,only: addfld, phys_decomp, add_default, outfld
  use physics_types,only: physics_state, physics_ptend
  use phys_buffer  ,only: pbuf_size_max, pbuf_fld, pbuf_add, pbuf_get_fld_idx

  implicit none

  save

  private                         ! Make default type private to the module


!-------------------------------------------------------------------------------
! Public interfaces:
!-------------------------------------------------------------------------------
  public iondrag_init             ! Initialization
  public iondrag_calc             ! ion drag tensors lxx,lyy,lxy,lyx

!-------------------------------------------------------------------------------
! Private data
!-------------------------------------------------------------------------------

! Namelist variables

  real(r8),parameter :: amu    = 1.6605387e-27_r8  ! atomic mass unit (kg)

  integer  :: ntop_lev = 1
  integer  :: nbot_lev = 0
  integer  :: id_xo2, id_xo1             ! indices to tn and major sp
  integer  :: id_o2p, id_op, id_nop      ! indices to ions
  integer  :: id_elec
  real(r8) :: rmass_elec                 ! mass of electron (g/mole)
  real(r8) :: rmass_op                   ! mass of O+ (g/mole)
  real(r8) :: rmass_o2p                  ! mass of O2+ (g/mole)
  real(r8) :: rmass_nop                  ! mass of NO+ (g/mole)
  real(r8) :: rmass_o1                   ! mass of O (g/mole)
  real(r8) :: rmass_o2                   ! mass of O2 (g/mole)
  real(r8) :: rmass_n2                   ! mass of N2 (g/mole)

!-------------------------------------------------------------------------------
! Inverted masses (for multiply in loops rather than divide):
!-------------------------------------------------------------------------------
  real(r8) :: rmi_o1
  real(r8) :: rmi_o2
  real(r8) :: rmi_n2
  real(r8) :: rmi_op
  real(r8) :: rmi_o2p
  real(r8) :: rmi_nop
  real(r8) :: rmi_elec
  real(r8) :: rmi_op_kg
  real(r8) :: rmi_o2p_kg
  real(r8) :: rmi_nop_kg

  contains

!================================================================================================

  subroutine iondrag_calc( lchnk, ncol, state, ptend, pbuf, delt )
!-------------------------------------------------------------------------------
! Calculate ion drag tensors lxx,lyy,lxy,lyx.
! Also calculate Pedersen and Hall conductivities.
! This is called from tphysac.
!-------------------------------------------------------------------------------

  use constituents ,only: cnst_get_ind      ! get constituent index function
  use mo_apex,only: & ! (pcols,begchunk:endchunk)
    bnorth,      & ! northward component of magnetic field (nT)
    beast,       & ! eastward component of magnetic field (nT)
    bdown,       & ! downward component of magnetic field (nT)
    bmag           ! magnetic field magnitude (nT)
  use shr_const_mod, only: shr_const_avogad, shr_const_boltz
  use chem_mods,     only: imozart
  use mo_mean_mass,  only: set_mean_mass
  use exbdrift,      only: get_exbdrift
  use phys_grid,     only: get_lon_all_p, get_lat_all_p

!-------------------------------------------------------------------------------
! dummy arguments
!-------------------------------------------------------------------------------
  integer,intent(in)   :: lchnk               ! current chunk index
  integer,intent(in)   :: ncol                ! number of atmospheric columns
  real(r8), intent(in) :: delt                ! time step (s)
  type(physics_state), intent(in)    :: state ! Physics state variables
  type(physics_ptend), intent(inout) :: ptend ! Physics tendencies
  type(pbuf_fld)     , intent(inout) :: pbuf(pbuf_size_max) ! physics buffer

!-------------------------------------------------------------------------------
! Local variables
!-------------------------------------------------------------------------------
  integer :: i,k ! loop indices
  integer :: lonndx(ncol)
  integer :: latndx(ncol)
  real(r8) :: sqrt_te                              ! sqrt(te)
  real(r8) :: sqrt_tnti                            ! sqrt(tnti)
  real(r8) :: wrk
  real(r8),parameter :: dipmin = 0.17_r8           ! minimum dip angle (tuneable)
  real(r8),parameter :: emass  = 9.1093819e-31_r8  ! electron mass (kg)
  real(r8),parameter :: qe     = 1.6021765e-19_r8  ! electronic charge (coulombs)
  real(r8),parameter :: colfac = 1.5_r8            ! collision factor (tuneable)
  real(r8),parameter :: boltz  = 1.e7_r8 * shr_const_boltz
  real(r8),parameter :: avo    = shr_const_avogad*1.e-3_r8 ! (molecules/mole)
! real(r8),parameter :: rmass_op  = 15.9989_r8     ! mass of O+
! real(r8),parameter :: rmass_o2p = 31.9983_r8     ! mass of O2+
! real(r8),parameter :: rmass_nop = 30.0056_r8     ! mass of NO+
! real(r8),parameter :: rmass_o1  = 16._r8         ! mass of O
! real(r8),parameter :: rmass_o2  = 32._r8         ! mass of O2
! real(r8),parameter :: rmass_n2  = 28._r8         ! mass of N2

!-------------------------------------------------------------------------------
! Inverted masses (for multiply in loops rather than divide):
!-------------------------------------------------------------------------------
! real(r8),parameter :: rmi_o1     = 1._r8/rmass_o1
! real(r8),parameter :: rmi_o2     = 1._r8/rmass_o2
! real(r8),parameter :: rmi_n2     = 1._r8/rmass_n2
! real(r8),parameter :: rmi_op     = 1._r8/rmass_op
! real(r8),parameter :: rmi_o2p    = 1._r8/rmass_o2p
! real(r8),parameter :: rmi_nop    = 1._r8/rmass_nop
! real(r8),parameter :: rmi_op_kg  = 1._r8/(rmass_op *amu)
! real(r8),parameter :: rmi_o2p_kg = 1._r8/(rmass_o2p*amu)  
! real(r8),parameter :: rmi_nop_kg = 1._r8/(rmass_nop*amu)

  real(r8) :: tn      (pcols,pver)   ! neutral gas temperature (deg K)
  real(r8) :: ti      (pcols,pver)   ! ion temperature (deg K)
  real(r8) :: te      (pcols,pver)   ! electron temperature (deg K)
  real(r8) :: xo2     (pcols,pver)   ! O2  (mmr)
  real(r8) :: xo1     (pcols,pver)   ! O   (mmr)
  real(r8) :: xn2     (pcols,pver)   ! N2  (mmr)
  real(r8) :: o2p     (pcols,pver)   ! O2+ (mmr)
  real(r8) :: op      (pcols,pver)   ! O+  (mmr)
  real(r8) :: nop     (pcols,pver)   ! NO+ (mmr)
  real(r8) :: barm    (pcols,pver)   ! mean molecular weight (g/mole)
  real(r8) :: xnmbar  (pcols,pver)   ! for unit conversion to volume density
  real(r8) :: tnti    (pcols,pver)   ! average of tn and ti
  real(r8) :: o2_cm3  (pcols,pver)   ! o2 volume density (cm-3)
  real(r8) :: o1_cm3  (pcols,pver)   ! o  volume density (cm-3)
  real(r8) :: n2_cm3  (pcols,pver)   ! n2 volume density (cm-3)
  real(r8) :: o2p_cm3 (pcols,pver)   ! O2+ (cm-3)
  real(r8) :: op_cm3  (pcols,pver)   ! O+  (cm-3)
  real(r8) :: nop_cm3 (pcols,pver)   ! NO+ (cm-3)
  real(r8) :: ne      (pcols,pver)   ! electron density (assume o2p+op+nop)
  real(r8) :: lamda1  (pcols,pver)   ! sigped*b**2/rho
  real(r8) :: lamda2  (pcols,pver)   ! sighal*b**2/rho
  real(r8) :: lxxnorot(pcols,pver)   ! XX before rotation
  real(r8) :: lyynorot(pcols,pver)   ! YY before rotation
  real(r8) :: lxynorot(pcols,pver)   ! XY before rotation
  real(r8) :: lyxnorot(pcols,pver)   ! YX before rotation

!-------------------------------------------------------------------------------
! Ion-neutral momentum transfer collision frequencies.
! rnu_xxx_xx = ratio of collision to gyro-frequences for O2+, O+, NO+.
!-------------------------------------------------------------------------------
  real(r8) :: rnu_o2p_o2(pcols,pver)   ! O2+ ~ O2 collision freq (resonant, T dependent)
  real(r8) :: rnu_op_o2 (pcols,pver)   ! O+  ~ O2 collision freq (non-resonant)
  real(r8) :: rnu_nop_o2(pcols,pver)   ! NO+ ~ O2 collision freq (non-resonant)
  real(r8) :: rnu_o2p_o (pcols,pver)   ! O2+ ~ O  collision freq (non-resonant)
  real(r8) :: rnu_op_o  (pcols,pver)   ! O+  ~ O  collision freq (resonant, T dependent)
  real(r8) :: rnu_nop_o (pcols,pver)   ! NO+ ~ O  collision freq (non-resonant)
  real(r8) :: rnu_o2p_n2(pcols,pver)   ! O2+ ~ N2 collision freq (non-resonant)
  real(r8) :: rnu_op_n2 (pcols,pver)   ! O+  ~ N2 collision freq (non-resonant)
  real(r8) :: rnu_nop_n2(pcols,pver)   ! NO+ ~ N2 collision freq (non-resonant)
  real(r8) :: rnu_o2p   (pcols,pver)   ! [[o2p~o2]n(o2)+[o2p~o]n(o)+[o2p~n2]n(n2)]/w(o2p)
  real(r8) :: rnu_op    (pcols,pver)   ! [[op ~o2]n(o2)+[op ~o]n(o)+[op ~n2]n(n2)]/w(op )
  real(r8) :: rnu_nop   (pcols,pver)   ! [[nop~o2]n(o2)+[nop~o]n(o)+[nop~n2]n(n2)]/w(nop)
  real(r8) :: rnu_ne    (pcols,pver)   ! electron ~ neutral collision frequency (s-1)

  real(r8) :: press        (pcols)     ! pressure at interface levels (dyne/cm^2)
  real(r8) :: qe_fac       (pcols)     ! unit conversion factor for conductivities
  real(r8) :: dipmag       (pcols)     ! magnetic dip angle 
  real(r8) :: decmag       (pcols)     ! magnetic declination
  real(r8) :: btesla       (pcols)     ! magnetic field (teslas)
  real(r8) :: sindip       (pcols)     ! sin(dipmag)
  real(r8) :: cosdip       (pcols)     ! cos(dipmag)
  real(r8) :: sin2dip      (pcols)     ! sindip^2
  real(r8) :: cos2dip      (pcols)     ! cosdip^2
  real(r8) :: sindec       (pcols)     ! sin(decmag)
  real(r8) :: cosdec       (pcols)     ! cos(decmag)
  real(r8) :: sin2dec      (pcols)     ! sindec^2
  real(r8) :: cos2dec      (pcols)     ! cosdec^2
  real(r8) :: omega_o2p    (pcols)     ! angular gyrofrequency for o2+ (s-1)
  real(r8) :: omega_op     (pcols)     ! angular gyrofrequency for o+ (s-1)
  real(r8) :: omega_nop    (pcols)     ! angular gyrofrequency for no+ (s-1)
  real(r8) :: omega_e      (pcols)     ! electron angular gyrofrequency (s-1)
  real(r8) :: omega_o2p_inv(pcols)     ! inverse of o2+ gyrofrequency
  real(r8) :: omega_op_inv (pcols)     ! inverse of o+  gyrofrequency
  real(r8) :: omega_nop_inv(pcols)     ! inverse of no+ gyrofrequency
  real(r8) :: omega_e_inv  (pcols)     ! inverse of electron gyrofrequency

!-------------------------------------------------------------------------------
! Ion drag coefficients output:
!-------------------------------------------------------------------------------
  real(r8) :: lxx(pcols,pver)  ! lambda XX coefficients (s-1)
  real(r8) :: lyy(pcols,pver)  ! lambda YY coefficients (s-1)
  real(r8) :: lxy(pcols,pver)  ! lambda XY coefficients (s-1)
  real(r8) :: lyx(pcols,pver)  ! lambda YX coefficients (s-1)

!-------------------------------------------------------------------------------
! Conductivities output:
!-------------------------------------------------------------------------------
  real(r8) :: sigma_ped (pcols,pver)   ! pedersen conductivity (siemens/m)
  real(r8) :: sigma_hall(pcols,pver)   ! hall conductivity (siemens/m)

  real(r8) :: qout(pcols,pver)         ! temp for outfld

!-------------------------------------------------------------------------------
! Define local tn and major species from state (mmr): 
!-------------------------------------------------------------------------------
  do k = 1,pver
    do i = 1,ncol
      tn (i,k) = state%t(i,k)
      ti (i,k) = tn(i,k)                 ! assume ti==tn for now (waccm2, 1/04)
      te (i,k) = tn(i,k)                 ! assume te==tn for now (waccm2, 1/04)
      xo2(i,k) = state%q(i,k,id_xo2)     ! o2  (mmr)
      xo1(i,k) = state%q(i,k,id_xo1)     ! o   (mmr)
      xn2(i,k) = 1._r8 - (xo2(i,k) + xo1(i,k)) ! n2  (mmr)
      xn2(i,k) = max( 1.e-20_r8,xn2(i,k) )
      o2p(i,k) = state%q(i,k,id_o2p)     ! o2+ (mmr)
      op (i,k) = state%q(i,k,id_op )     ! o+  (mmr)
      nop(i,k) = state%q(i,k,id_nop)     ! no+ (mmr)
    end do
  end do

! call outfld ('TN      ',tn  ,pcols,lchnk)
! call outfld ('XO2     ',xo2 ,pcols,lchnk)
! call outfld ('XO      ',xo1 ,pcols,lchnk)
! call outfld ('XN2     ',xn2 ,pcols,lchnk)
! call outfld ('O2P     ',o2p ,pcols,lchnk)
! call outfld ('OP      ',op  ,pcols,lchnk)
! call outfld ('NOP     ',nop ,pcols,lchnk)
  qout(:ncol,:) = o2p(:ncol,:) + op(:ncol,:) + nop(:ncol,:)
  call outfld ('QIONSUM ', qout, pcols, lchnk)
 
!-------------------------------------------------------------------------------
! calculate ExB drift velocities
!-------------------------------------------------------------------------------
   call t_startf ( 'exbdrift' )
   call get_exbdrift( lchnk, ncol, pbuf )
   call t_stopf  ( 'exbdrift' )
!-------------------------------------------------------------------------------

  do i = 1,ncol
    btesla(i) = bmag(i,lchnk)*1.e-9_r8 ! nT to teslas (see bmag in apex module)
!-------------------------------------------------------------------------------
! Angular gyrofrequency of O+, O2+ and NO+ (s-1):
!-------------------------------------------------------------------------------
    omega_op (i) = qe*btesla(i)*rmi_op_kg
    omega_o2p(i) = qe*btesla(i)*rmi_o2p_kg
    omega_nop(i) = qe*btesla(i)*rmi_nop_kg
!-------------------------------------------------------------------------------
! Electron angular gyrofrequency (s-1):
!-------------------------------------------------------------------------------
    omega_e(i) = qe*btesla(i)/emass 
!-------------------------------------------------------------------------------
! Invert now, so we can multiply rather than divide in loops below:
!-------------------------------------------------------------------------------
    omega_op_inv (i) = 1._r8/omega_op(i)
    omega_o2p_inv(i) = 1._r8/omega_o2p(i)
    omega_nop_inv(i) = 1._r8/omega_nop(i)
    omega_e_inv(i)   = 1._r8/omega_e(i)
!-------------------------------------------------------------------------------
! Magnetic field geometry (used below in rotation of lambdas):
!-------------------------------------------------------------------------------
    dipmag(i) = atan( bdown(i,lchnk)/sqrt(bnorth(i,lchnk)**2+beast(i,lchnk)**2) )
    decmag(i) = -atan2( beast(i,lchnk),bnorth(i,lchnk) )
    cosdec(i) = cos( decmag(i) )
    sindec(i) = sin( decmag(i) )
    if( abs(dipmag(i)) >= dipmin ) then
      sindip(i) = sin(dipmag(i))
      cosdip(i) = cos(dipmag(i))
    else
      if( dipmag(i) >= 0._r8 ) then
        sindip(i) = sin( dipmin )
        cosdip(i) = cos( dipmin )
      else
        sindip(i) = sin( -dipmin )
        cosdip(i) = cos( -dipmin )
      end if
    end if
    sin2dip(i) = sindip(i)**2
    cos2dip(i) = cosdip(i)**2
    sin2dec(i) = sindec(i)**2
    cos2dec(i) = cosdec(i)**2
  end do

! write(6,"('iondrag: btesla=',   /,(6e12.4))") btesla
! write(6,"('iondrag: bdown=',    /,(6e12.4))") bdown(:,lchnk)
! write(6,"('iondrag: beast=',    /,(6e12.4))") beast(:,lchnk)
! write(6,"('iondrag: bnorth=',   /,(6e12.4))") bnorth(:,lchnk)
! write(6,"('iondrag: omega_o2p=',/,(6e12.4))") omega_o2p
! write(6,"('iondrag: omega_op=' ,/,(6e12.4))") omega_op
! write(6,"('iondrag: omega_nop=',/,(6e12.4))") omega_nop

!-------------------------------------------------------------------------------
! Ion-neutral momentum transfer collision frequency coefficients:
!-------------------------------------------------------------------------------
  do k = 1,pver
    do i = 1,ncol
      tnti(i,k) = 0.5_r8*(ti(i,k) + tn(i,k))           ! ave of tn & ti
      sqrt_tnti = sqrt( tnti(i,k) )
      wrk       = log10( tnti(i,k) )
!-------------------------------------------------------------------------------
! Collision frequency coefficients with O2 (cm3/s):
!-------------------------------------------------------------------------------
      rnu_o2p_o2(i,k) = 2.59e-11_r8*sqrt_tnti &        ! O2+ ~ O2 (resonant)
                        *(1._r8 - .073_r8*wrk)**2
      rnu_op_o2 (i,k) = 6.64e-10_r8                    ! O+  ~ O2
      rnu_nop_o2(i,k) = 4.27e-10_r8                    ! NO+ ~ O2
!-------------------------------------------------------------------------------
! Collision frequency coefficients with O (cm3/s):
!-------------------------------------------------------------------------------
      rnu_o2p_o(i,k) = 2.31e-10_r8                     ! O2+ ~ O
      rnu_op_o (i,k) = 3.67e-11_r8*sqrt_tnti  &        ! O+  ~ O (resonant)
                       *(1._r8 - .064_r8*wrk)**2*colfac     
      rnu_nop_o(i,k) = 2.44e-10_r8                     ! NO+ ~ O
!-------------------------------------------------------------------------------
! Collision frequency coefficients with N2 (cm3/s):
!-------------------------------------------------------------------------------
      rnu_o2p_n2(i,k) = 4.13e-10_r8                    ! O2+ ~ N2
      rnu_op_n2 (i,k) = 6.82e-10_r8                    ! O+  ~ N2
      rnu_nop_n2(i,k) = 4.34e-10_r8                    ! NO+ ~ N2
    end do
  end do

! call outfld ('N_O2P_O2',rnu_o2p_o2,pcols,lchnk)
! call outfld ('N_OP_O2 ',rnu_op_o2 ,pcols,lchnk)
! call outfld ('N_NOP_O2',rnu_nop_o2,pcols,lchnk)

! call outfld ('N_O2P_O ',rnu_o2p_o ,pcols,lchnk)
! call outfld ('N_OP_O  ',rnu_op_o  ,pcols,lchnk)
! call outfld ('N_NOP_O ',rnu_nop_o ,pcols,lchnk)

! call outfld ('N_O2P_N2',rnu_o2p_n2,pcols,lchnk)
! call outfld ('N_OP_N2 ',rnu_op_n2 ,pcols,lchnk)
! call outfld ('N_NOP_N2',rnu_nop_n2,pcols,lchnk)
!
!-------------------------------------------------------------------------------
! Sub set_mean_mass (mo_mean_mass.F90) returns barm(ncol,pver) in g/mole,
!   however, set_mean_mass sometimes returns zero in top(?) four values 
!   of the column, so barm is calculated here, see below.
!
! call set_mean_mass(ncol, state%q(1,1,imozart), barm)
!
! Major species and ion number densities (mmr to cm-3):
!-------------------------------------------------------------------------------

  call set_mean_mass( ncol, state%q(:,:,imozart:), barm )

  do k = 1,pver
    do i = 1,ncol
      press(i)     = 10._r8*state%pmid(i,k) ! from Pa to dyne/cm^2
!     barm(i,k)   = 1._r8 / (xo2(i,k)*rmi_o2 + xo1(i,k)*rmi_o1 + xn2(i,k)*rmi_n2)
      xnmbar(i,k)  = press(i)*barm(i,k)/(boltz*tn(i,k)) 
      o2_cm3(i,k)  = xo2(i,k)*xnmbar(i,k)*rmi_o2    ! o2 (cm-3)
      o1_cm3(i,k)  = xo1(i,k)*xnmbar(i,k)*rmi_o1    ! o  (cm-3)
      n2_cm3(i,k)  = xn2(i,k)*xnmbar(i,k)*rmi_n2    ! n2 (cm-3)
      o2p_cm3(i,k) = o2p(i,k)*xnmbar(i,k)*rmi_o2p  ! o2+ (cm-3)
      op_cm3 (i,k) = op (i,k)*xnmbar(i,k)*rmi_op   ! o+  (cm-3)
      nop_cm3(i,k) = nop(i,k)*xnmbar(i,k)*rmi_nop  ! no+ (cm-3)
      ne(i,k)      = state%q(i,k,id_elec)*xnmbar(i,k)*rmi_elec  ! e (cm-3)
    end do
  end do

! call outfld ('O2_CM3  ',o2_cm3 ,pcols,lchnk)
! call outfld ('O1_CM3  ',o1_cm3 ,pcols,lchnk)
! call outfld ('N2_CM3  ',n2_cm3 ,pcols,lchnk)
! call outfld ('O2P_CM3 ',o2p_cm3,pcols,lchnk)
! call outfld ('OP_CM3  ',op_cm3 ,pcols,lchnk)
! call outfld ('NOP_CM3 ',nop_cm3,pcols,lchnk)

!-------------------------------------------------------------------------------
! Multiply collision freq by neutral number density and sum for each ion:
!
! rnu_o2p = [[o2p~o2]n(o2)+[o2p~o]n(o)+[o2p~n2]n(n2)]/w(o2p)
! rnu_op  = [[op ~o2]n(o2)+[op ~o]n(o)+[op ~n2]n(n2)]/w(op )
! rnu_nop = [[nop~o2]n(o2)+[nop~o]n(o)+[nop~n2]n(n2)]/w(nop)
!-------------------------------------------------------------------------------
  do k = 1,pver
    do i = 1,ncol
      rnu_o2p(i,k) = rnu_o2p_o2(i,k)*o2_cm3(i,k) &
                     + rnu_o2p_o (i,k)*o1_cm3(i,k) &
                     + rnu_o2p_n2(i,k)*n2_cm3(i,k)
      rnu_op (i,k) = rnu_op_o2 (i,k)*o2_cm3(i,k) &
                     + rnu_op_o  (i,k)*o1_cm3(i,k) &
                     + rnu_op_n2 (i,k)*n2_cm3(i,k)
      rnu_nop(i,k) = rnu_nop_o2(i,k)*o2_cm3(i,k) &
                     + rnu_nop_o (i,k)*o1_cm3(i,k) &
                     + rnu_nop_n2(i,k)*n2_cm3(i,k)
!-------------------------------------------------------------------------------
! Electron collision frequency (s-1):
!-------------------------------------------------------------------------------
      sqrt_te = sqrt(te(i,k))
      rnu_ne(i,k) = & 
        2.33e-11_r8*n2_cm3(i,k)*te(i,k)*(1._r8 - 1.21e-4_r8*te(i,k)) &
        + 1.82e-10_r8*o2_cm3(i,k)*sqrt_te*(1._r8 + 3.60e-2_r8*sqrt_te) &
        + 8.90e-11_r8*o1_cm3(i,k)*sqrt_te*(1._r8 + 5.70e-4_r8*te(i,k))
    end do
  end do

! call outfld ('COLL_O2P',rnu_o2p,pcols,lchnk)
! call outfld ('COLL_OP ',rnu_op ,pcols,lchnk)
! call outfld ('COLL_NOP',rnu_nop,pcols,lchnk)
! call outfld ('COLL_NE ',rnu_ne ,pcols,lchnk)

!-------------------------------------------------------------------------------
! Ratio of collision to gyro frequencies for o2+, o+, no+, ne:
!-------------------------------------------------------------------------------
  do k = 1,pver
    do i = 1,ncol
      rnu_o2p(i,k) = rnu_o2p(i,k)*omega_o2p_inv(i)
      rnu_op (i,k) = rnu_op (i,k)*omega_op_inv (i)
      rnu_nop(i,k) = rnu_nop(i,k)*omega_nop_inv(i)
      rnu_ne (i,k) = rnu_ne (i,k)*omega_e_inv  (i)
    end do
  end do

! call outfld ('RNU_O2P ',rnu_o2p,pcols,lchnk)
! call outfld ('RNU_OP  ',rnu_op ,pcols,lchnk)
! call outfld ('RNU_NOP ',rnu_nop,pcols,lchnk)
! call outfld ('RNU_NE  ',rnu_ne ,pcols,lchnk)

!-------------------------------------------------------------------------------
! Calculate pedersen and Hall conductivities (siemens/m):
!
! Qe_fac: 1.e6 to convert number densities from cm-3 to m-3:
!-------------------------------------------------------------------------------
  qe_fac(:ncol) = qe*1.e6_r8/btesla(:ncol)

  do k = 1,pver
    do i = 1,ncol
!-------------------------------------------------------------------------------
! ne = electron density assuming charge neutrality (cm-3):
!-------------------------------------------------------------------------------
!     ne(i,k) = op_cm3(i,k) + o2p_cm3(i,k) + nop_cm3(i,k)
!-------------------------------------------------------------------------------
! Pedersen conductivity (siemens/m):
!-------------------------------------------------------------------------------
      sigma_ped(i,k) = qe_fac(i)                           &
        *((op_cm3   (i,k)*rnu_op (i,k)/(1._r8 + rnu_op (i,k)**2)) &
          + (o2p_cm3(i,k)*rnu_o2p(i,k)/(1._r8 + rnu_o2p(i,k)**2)) &
          + (nop_cm3(i,k)*rnu_nop(i,k)/(1._r8 + rnu_nop(i,k)**2)) &
          + (ne     (i,k)*rnu_ne (i,k)/(1._r8 + rnu_ne (i,k)**2)))
!-------------------------------------------------------------------------------
! Hall conductivity (siemens/m):
!-------------------------------------------------------------------------------
      sigma_hall(i,k) = qe_fac(i)           &
        *(ne       (i,k)/(1._r8 + rnu_ne (i,k)**2) &
          - op_cm3 (i,k)/(1._r8 + rnu_op (i,k)**2) &
          - o2p_cm3(i,k)/(1._r8 + rnu_o2p(i,k)**2) &
          - nop_cm3(i,k)/(1._r8 + rnu_nop(i,k)**2))
    end do
  end do

  call outfld ('ELECDEN ',ne        ,pcols,lchnk)
  call outfld ('SIGMAPED',sigma_ped ,pcols,lchnk)
  call outfld ('SIGMAHAL',sigma_hall,pcols,lchnk)

  do k = 1,pver
    do i = 1,ncol
      wrk         = btesla(i)**2*avo*1.e-3_r8/xnmbar(i,k)
      lamda1(i,k) = sigma_ped(i,k)*wrk
      lamda2(i,k) = sigma_hall(i,k)*wrk
    end do
  end do

  call outfld ('LAMDA1',lamda1,pcols,lchnk)
  call outfld ('LAMDA2',lamda2,pcols,lchnk)
 
  do k = 1,pver
    do i = 1,ncol
      lxxnorot(i,k) = lamda1(i,k)
      lyynorot(i,k) = lamda1(i,k)*sin2dip(i)
      lxynorot(i,k) = lamda2(i,k)*sindip(i)
      lyxnorot(i,k) = lxynorot(i,k)
    end do
  end do

! call outfld ('LXXNOROT',lxxnorot,pcols,lchnk)
! call outfld ('LYYNOROT',lyynorot,pcols,lchnk)
! call outfld ('LXYNOROT',lxynorot,pcols,lchnk)
! call outfld ('LYXNOROT',lyxnorot,pcols,lchnk)

!-------------------------------------------------------------------------------
! Rotate lambdas from local magnetic to geographic coordinates:
!-------------------------------------------------------------------------------
  do k = 1,pver
    do i = 1,ncol
      lxx(i,k) = lxxnorot(i,k)*cos2dec(i) + lyynorot(i,k)*sin2dec(i)
      lyy(i,k) = lyynorot(i,k)*cos2dec(i) + lxxnorot(i,k)*sin2dec(i)
      wrk      = (lyynorot(i,k) - lxxnorot(i,k))*sindec(i)*cosdec(i)
      lyx(i,k) = lxynorot(i,k) - wrk
      lxy(i,k) = lxynorot(i,k) + wrk
    end do
  end do

  call outfld ('LXX     ',lxx,pcols,lchnk)
  call outfld ('LYY     ',lyy,pcols,lchnk)
  call outfld ('LXY     ',lxy,pcols,lchnk)
  call outfld ('LYX     ',lyx,pcols,lchnk)

  call get_lon_all_p( lchnk, ncol, lonndx )
  call get_lat_all_p( lchnk, ncol, latndx )

!-------------------------------------------------------------------------------
! Calculate ion drag tendencies and apply to neutral velocities:
!-------------------------------------------------------------------------------
    call iondrag_tend( lchnk, ncol, state, ptend, pbuf, &
                       lxx, lyy, lxy, lyx, delt )

!-------------------------------------------------------------------------------
! Calculate joule heating tendency and apply to temperature:
!-------------------------------------------------------------------------------
    call jouleheat_tend( lchnk, ncol, state, ptend, pbuf, &
                         lxx, lyy, lxy, lyx )

  end subroutine iondrag_calc

  subroutine iondrag_init( hypm , efield_lflux_file, efield_hflux_file, efield_wei96_file )
!-------------------------------------------------------------------------------
! Iondrag initialization, called from inti.F90.
!-------------------------------------------------------------------------------

  use pmgrid,       only: masterproc
  use constituents, only: cnst_get_ind, cnst_mw
  use efield,       only: efield_init
  use exbdrift,     only: exbdrift_init
  use mo_apex,      only: apexmag

  implicit none

!-------------------------------------------------------------------------------
! dummy arguments
!-------------------------------------------------------------------------------
  real(r8), intent(in) :: hypm(pver)

  character(len=256) :: efield_lflux_file 
  character(len=256) :: efield_hflux_file
  character(len=256) :: efield_wei96_file 

!-------------------------------------------------------------------------------
! local variables
!-------------------------------------------------------------------------------
  integer  :: m, k
  real(r8) :: rpsh

!-------------------------------------------------------------------------------
! initialize related packages: electric field
!-------------------------------------------------------------------------------
  call apexmag
  
  call efield_init (efield_lflux_file, efield_hflux_file, efield_wei96_file)
  call exbdrift_init

!-------------------------------------------------------------------------------
! find lower bnd for iondrag
!-------------------------------------------------------------------------------
  if (hypm(1) .lt. 0.1) then
     do k = 1, pver
        if (hypm(k) .lt. 50.) nbot_lev  = k
     end do
  end if
 
!  nbot_lev = pver
  if( masterproc ) then
     write(*,*) ' '
     write(*,*) 'iondrag_init: nbot_lev,press = ',nbot_lev,hypm(nbot_lev)
     write(*,*) ' '
  end if

  call cnst_get_ind( 'e', id_elec )
  rmass_elec = cnst_mw(id_elec)
  call cnst_get_ind( 'Op', id_op )
  rmass_op  = cnst_mw(id_op)
  call cnst_get_ind( 'O2p', id_o2p )
  rmass_o2p = cnst_mw(id_o2p)
  call cnst_get_ind( 'NOp', id_nop )
  rmass_nop = cnst_mw(id_nop)
  call cnst_get_ind( 'O', id_xo1 )
  rmass_o1  = cnst_mw(id_xo1)
  call cnst_get_ind( 'O2', id_xo2 )
  rmass_o2 = cnst_mw(id_xo2)
  call cnst_get_ind( 'N', m )
  rmass_n2 = 2._r8*cnst_mw(m)

  rmi_elec   = 1._r8/rmass_elec
  rmi_o1     = 1._r8/rmass_o1
  rmi_o2     = 1._r8/rmass_o2
  rmi_n2     = 1._r8/rmass_n2
  rmi_op     = 1._r8/rmass_op
  rmi_o2p    = 1._r8/rmass_o2p
  rmi_nop    = 1._r8/rmass_nop
  rmi_op_kg  = 1._r8/(rmass_op *amu)
  rmi_o2p_kg = 1._r8/(rmass_o2p*amu)  
  rmi_nop_kg = 1._r8/(rmass_nop*amu)

!-------------------------------------------------------------------------------
! Set up fields to history files.
!-------------------------------------------------------------------------------
! call addfld('TN      ','K  ',pver ,'I','TN      ',phys_decomp)
! call addfld('XO2     ','MMR',pver ,'I','O2      ',phys_decomp)
! call addfld('XO      ','MMR',pver ,'I','O       ',phys_decomp)
! call addfld('XN2     ','MMR',pver ,'I','N2      ',phys_decomp)
! call addfld('O2P     ','MMR',pver ,'I','O2+     ',phys_decomp)
! call addfld('OP      ','MMR',pver ,'I','O+      ',phys_decomp)
! call addfld('NOP     ','MMR',pver ,'I','NO+     ',phys_decomp)

! call addfld('N_O2P_O2','S-1',pver ,'I','RNU_O2P_O2',phys_decomp)
! call addfld('N_OP_O2 ','S-1',pver ,'I','RNU_OP_O2 ',phys_decomp)
! call addfld('N_NOP_O2','S-1',pver ,'I','RNU_NOP_O2',phys_decomp)

! call addfld('N_O2P_O ','S-1',pver ,'I','RNU_O2P_O ',phys_decomp)
! call addfld('N_OP_O  ','S-1',pver ,'I','RNU_OP_O  ',phys_decomp)
! call addfld('N_NOP_O ','S-1',pver ,'I','RNU_NOP_O ',phys_decomp)

! call addfld('N_O2P_N2','S-1',pver ,'I','RNU_O2P_N2',phys_decomp)
! call addfld('N_OP_N2 ','S-1',pver ,'I','RNU_OP_N2 ',phys_decomp)
! call addfld('N_NOP_N2','S-1',pver ,'I','RNU_NOP_N2',phys_decomp)

! call addfld('O2_CM3  ','CM-3',pver,'I','O2_CM3  ',phys_decomp)
! call addfld('O1_CM3  ','CM-3',pver,'I','O1_CM3  ',phys_decomp)
! call addfld('N2_CM3  ','CM-3',pver,'I','N2_CM3  ',phys_decomp)

! call addfld('O2P_CM3 ','CM-3',pver,'I','O2P_CM3 ',phys_decomp)
! call addfld('OP_CM3  ','CM-3',pver,'I','OP_CM3  ',phys_decomp)
! call addfld('NOP_CM3 ','CM-3',pver,'I','NOP_CM3 ',phys_decomp)

! call addfld('COLL_O2P','S-1',pver,'I','COLL_O2P',phys_decomp)
! call addfld('COLL_OP ','S-1',pver,'I','COLL_OP ',phys_decomp)
! call addfld('COLL_NOP','S-1',pver,'I','COLL_NOP',phys_decomp)
! call addfld('COLL_NE ','S-1',pver,'I','COLL_NE ',phys_decomp)

! call addfld('RNU_O2P ',' ',pver,'I','RNU_O2P ',phys_decomp)
! call addfld('RNU_OP  ',' ',pver,'I','RNU_OP  ',phys_decomp)
! call addfld('RNU_NOP ',' ',pver,'I','RNU_NOP ',phys_decomp)
! call addfld('RNU_NE  ',' ',pver,'I','RNU_NE  ',phys_decomp)

  call addfld('QIONSUM ','S-1' ,pver,'I','Ion prod sum',phys_decomp)
  call addfld('ELECDEN ','CM-3',pver,'I','NE (ion sum)',phys_decomp)
  call addfld( 'SIGMAPED', 'siemens/m', pver, 'I', 'Pederson conductivity', phys_decomp )
  call addfld( 'SIGMAHAL', 'siemens/m', pver, 'I', 'Hall conductivity', phys_decomp )
  call addfld('LAMDA1'  ,'S-1',pver,'I' ,'LAMDA PED',phys_decomp)
  call addfld('LAMDA2'  ,'S-1',pver,'I' ,'LAMDA HALL',phys_decomp)

! call addfld('LXXNOROT','S-1',pver,'I','LXX prerot',phys_decomp)
! call addfld('LYYNOROT','S-1',pver,'I','LYY prerot',phys_decomp)
! call addfld('LXYNOROT','S-1',pver,'I','LXY prerot',phys_decomp)
! call addfld('LYXNOROT','S-1',pver,'I','LYX prerot',phys_decomp)
  call addfld('LXX','S-1',pver,'I','LXX',phys_decomp)
  call addfld('LYY','S-1',pver,'I','LYY',phys_decomp)
  call addfld('LXY','S-1',pver,'I','LXY',phys_decomp)
  call addfld('LYX','S-1',pver,'I','LYX',phys_decomp)
 
! call add_default ('TN      ', 1, ' ')
! call add_default ('XO2     ', 1, ' ')
! call add_default ('XO      ', 1, ' ')
! call add_default ('XN2     ', 1, ' ')
! call add_default ('O2P     ', 1, ' ')
! call add_default ('OP      ', 1, ' ')
! call add_default ('NOP     ', 1, ' ')

! call add_default ('N_O2P_O2', 1, ' ')
! call add_default ('N_OP_O2 ', 1, ' ')
! call add_default ('N_NOP_O2', 1, ' ')

! call add_default ('N_O2P_O ', 1, ' ')
! call add_default ('N_OP_O  ', 1, ' ')
! call add_default ('N_NOP_O ', 1, ' ')

! call add_default ('N_O2P_N2', 1, ' ')
! call add_default ('N_OP_N2 ', 1, ' ')
! call add_default ('N_NOP_N2', 1, ' ')

! call add_default ('O2_CM3  ', 1, ' ')
! call add_default ('O1_CM3  ', 1, ' ')
! call add_default ('N2_CM3  ', 1, ' ')
! call add_default ('O2P_CM3 ', 1, ' ')
! call add_default ('OP_CM3  ', 1, ' ')
! call add_default ('NOP_CM3 ', 1, ' ')

! call add_default ('COLL_O2P', 1, ' ')
! call add_default ('COLL_OP ', 1, ' ')
! call add_default ('COLL_NOP', 1, ' ')
! call add_default ('COLL_NE ', 1, ' ')

! call add_default ('RNU_O2P ', 1, ' ')
! call add_default ('RNU_OP  ', 1, ' ')
! call add_default ('RNU_NOP ', 1, ' ')
! call add_default ('RNU_NE  ', 1, ' ')
! call add_default ('ELECDEN ', 1, ' ')
! call add_default ('QIONSUM ', 1, ' ')

! call add_default ('SIGMAPED', 1, ' ')
! call add_default ('SIGMAHAL', 1, ' ')
! call add_default ('LAMDA1  ', 1, ' ')
! call add_default ('LAMDA2  ', 1, ' ')

! call add_default ('LXXNOROT', 1, ' ')
! call add_default ('LYYNOROT', 1, ' ')
! call add_default ('LXYNOROT', 1, ' ')
! call add_default ('LYXNOROT', 1, ' ')
! call add_default ('LXX     ', 1, ' ')
! call add_default ('LYY     ', 1, ' ')
! call add_default ('LXY     ', 1, ' ')
! call add_default ('LYX     ', 1, ' ')
!
! U,V ion drag tendencies:
! (outfld called from iondrag_tend):
!
  call addfld('UIONTEND','M/S2',pver,'I','u-tendency due to ion drag',phys_decomp)
  call addfld('VIONTEND','M/S2',pver,'I','v-tendency due to ion drag',phys_decomp)
! call add_default ('UIONTEND', 1, ' ')
! call add_default ('VIONTEND', 1, ' ')
!
! Joule heating, and tn before and after joule heating tendencies are applied:
!
  call addfld( 'QJOULE  ', 'K/s' , pver, 'I', 'Joule Heat', phys_decomp )  ! joule heating
  call add_default( 'QJOULE  ', 1, ' ' )                                   ! joule heating (K/s)

  end subroutine iondrag_init

  subroutine iondrag_tend( lchnk, ncol, state, ptend, pbuf, &
                           lxx, lyy, lxy, lyx, delt )
!-------------------------------------------------------------------------------
! Calculate tendencies in U and V from ion drag tensors, which were
!   calculated by sub iondrag_calc (module data lxx,lyy,lxy,lyx).
! This is called from sub iondrag_calc.
!-------------------------------------------------------------------------------

  use phys_grid,     only: get_lon_all_p, get_lat_all_p

!-------------------------------------------------------------------------------
! dummy arguments
!-------------------------------------------------------------------------------
  integer,intent(in)    :: lchnk                 ! current chunk index
  integer,intent(in)    :: ncol                  ! number of atmospheric columns
  real(r8), intent(in)  :: delt                  ! time step (s)
  real(r8), intent(in)  :: lxx(pcols,pver)       ! ion drag tensor
  real(r8), intent(in)  :: lyy(pcols,pver)       ! ion drag tensor
  real(r8), intent(in)  :: lxy(pcols,pver)       ! ion drag tensor
  real(r8), intent(in)  :: lyx(pcols,pver)       ! ion drag tensor
  type(physics_state), intent(in)    :: state ! Physics state variables
  type(physics_ptend), intent(inout) :: ptend ! Physics tendencies
  type(pbuf_fld), intent(in) :: pbuf(pbuf_size_max) ! physics buffer

!-------------------------------------------------------------------------------
! Local variables
!-------------------------------------------------------------------------------
  integer  :: i, k
  integer  :: ndx_ue, ndx_ve             ! indices to ion velocities in pbuf
  integer  :: lonndx(ncol)
  integer  :: latndx(ncol)
  real(r8) :: dti
  real(r8) :: detr
  real(r8) :: us, vs
  real(r8) :: l11, l12, l21, l22
  real(r8) :: du(pcols,pver)             ! zonal ion drag tendency
  real(r8) :: dv(pcols,pver)             ! meridional ion drag tendency
  real(r8) :: dui(pcols,pver)            ! zonal ion drag tendency
  real(r8) :: dvi(pcols,pver)            ! meridional ion drag tendency
  real(r8), pointer :: ue(:)             ! pointer to pbuf
  real(r8), pointer :: ve(:)             ! pointer to pbuf

!-------------------------------------------------------------------------------
! Get ion ExB drift from physics buffer (they were defined by exbdrift module)
! Ion drifts are 2d arrays, i.e., no vertical dimension.
!-------------------------------------------------------------------------------
  ndx_ue = pbuf_get_fld_idx( 'UE' )
  ndx_ve = pbuf_get_fld_idx( 'VE' )
  ue     => pbuf(ndx_ue)%fld_ptr(1,1:pcols,1,lchnk,1)
  ve     => pbuf(ndx_ve)%fld_ptr(1,1:pcols,1,lchnk,1)

! write(6,"('iondrag_tend: ue=',/,(6e12.4))") ue(:)
! write(6,"('iondrag_tend: ve=',/,(6e12.4))") ve(:)

  call get_lon_all_p( lchnk, ncol, lonndx )
  call get_lat_all_p( lchnk, ncol, latndx )

  dti = 1._r8/delt


!-------------------------------------------------------------------------------
! Zonal (du) and meridional (dv) wind drag, using ExB drift velocities
! from exbdrift module (pbuf):
!-------------------------------------------------------------------------------
  do k = ntop_lev,nbot_lev
    do i = 1,ncol
!-------------------------------------------------------------------------------
! 2/28/04 btf:
! Full ion-drag, using lambdas and ExB drifts. 
!   This should succeed with bz = 0 (efield module)
! Runs:
!   bz=-5, nstep=24 min, nsplit=4 (6 min dynamics): crashed after 2 days.
!   bz=-5, nstep=24 min, nsplit=6 (4 min dynamics): 5 day run succeeded.
! See comments in efield module re bz < 0 (efield.F90).
!-------------------------------------------------------------------------------
      us      = ue(i) - state%u(i,k)
      vs      = ve(i) - state%v(i,k)
!     du(i,k) = lxy(i,k)*vs + lxx(i,k)*us
!     dv(i,k) = lyy(i,k)*vs - lyx(i,k)*us
!-------------------------------------------------------------------------------
! Exclude ue,ve drift momentum source to avoid crashes when bz < 0 and 
! full 30 min timestep (partial ion-drag):
!-------------------------------------------------------------------------------
!     du(i,k) = -(lxy(i,k)*state%v(i,k) + lxx(i,k)*state%u(i,k))
!     dv(i,k) =  lyx(i,k)*state%u(i,k) - lyy(i,k)*state%v(i,k)
      l11     = dti + lxx(i,k)
      l12     = lxy(i,k)
      l21     = -lyx(i,k)
      l22     = dti + lyy(i,k)
      detr    = dti/(l11*l22 - l12*l21)
      dui(i,k) = dti*(detr*(l12*vs - l22*us) + us)
      dvi(i,k) = dti*(detr*(l21*us - l11*vs) + vs)
!      write (*,*) '**********************'
!      write (*,*) 'l12,l21,detr=',l12,l21,detr
!      write (*,*) '**********************'
    end do
  end do

!-------------------------------------------------------------------------------
! Apply to model tendencies:
!-------------------------------------------------------------------------------
  do k = ntop_lev,nbot_lev
!-------------------------------------------------------------------------------
! Ion drag tendencies:
!-------------------------------------------------------------------------------
!     ptend%u(:ncol,k) = du(:ncol,k)
!     ptend%v(:ncol,k) = dv(:ncol,k)
      ptend%u(:ncol,k) = dui(:ncol,k)
      ptend%v(:ncol,k) = dvi(:ncol,k)
!-------------------------------------------------------------------------------
! Turn off ion drag tendency:
!-------------------------------------------------------------------------------
!     ptend%u(:ncol,k) = 0._r8
!     ptend%v(:ncol,k) = 0._r8
  end do
  do k = nbot_lev+1,pver
      dui(:ncol,k)      = 0._r8
      dvi(:ncol,k)      = 0._r8
      ptend%u(:ncol,k) = 0._r8
      ptend%v(:ncol,k) = 0._r8
  end do

  ptend%name = "ion drag"
  ptend%lu = .true.
  ptend%lv = .true.
 
  call outfld ( 'UIONTEND', dui, pcols, lchnk )      ! u ion drag tendency
  call outfld ( 'VIONTEND', dvi, pcols, lchnk )      ! v ion drag tendency

  end subroutine iondrag_tend

  subroutine jouleheat_tend( lchnk, ncol, state, ptend, pbuf, &
                             lxx, lyy, lxy, lyx )
!-------------------------------------------------------------------------------
! Calculate tendencies in T due to joule heating.
! This is called from sub iondrag_calc.
!-------------------------------------------------------------------------------

  use physconst, only: cpair
  use shr_const_mod, only: pi => shr_const_pi
  use phys_grid, only: get_rlon_p, get_rlat_p
  use phys_grid, only: get_lon_all_p, get_lat_all_p

!-------------------------------------------------------------------------------
! dummy arguments
!-------------------------------------------------------------------------------
  integer,intent(in)    :: lchnk                    ! current chunk index
  integer,intent(in)    :: ncol                     ! number of atmospheric columns
  real(r8), intent(in)  :: lxx(pcols,pver)          ! ion drag tensor
  real(r8), intent(in)  :: lyy(pcols,pver)          ! ion drag tensor
  real(r8), intent(in)  :: lxy(pcols,pver)          ! ion drag tensor
  real(r8), intent(in)  :: lyx(pcols,pver)          ! ion drag tensor
  type(physics_state), intent(in)    :: state       ! Physics state variables
  type(physics_ptend), intent(inout) :: ptend       ! Physics tendencies (inout)
  type(pbuf_fld), intent(in) :: pbuf(pbuf_size_max) ! physics buffer

!-------------------------------------------------------------------------------
! Local variables
!-------------------------------------------------------------------------------
  integer  :: k, i
  integer  :: ndx_ue, ndx_ve             ! indices to ion velocities in pbuf
  integer  :: max_ind(2)
  integer :: lonndx(ncol)
  integer :: latndx(ncol)
  real(r8) :: us, vs
  real(r8) :: max_q
  real(r8) :: qjoule(pcols,pver)         ! joule heating
  real(r8) :: qout(pcols,pver)           ! temp for outfld
  real(r8), pointer :: ue(:)             ! pointer to pbuf
  real(r8), pointer :: ve(:)             ! pointer to pbuf

!-------------------------------------------------------------------------------
! Get ion velocities from physics buffer (they were defined by exbdrift module)
! Ion velocities are 2d arrays, i.e., no vertical dimension.
!-------------------------------------------------------------------------------
  ndx_ue = pbuf_get_fld_idx( 'UE' )
  ndx_ve = pbuf_get_fld_idx( 'VE' )
  ue     => pbuf(ndx_ue)%fld_ptr(1,1:pcols,1,lchnk,1)
  ve     => pbuf(ndx_ve)%fld_ptr(1,1:pcols,1,lchnk,1)

! write(6,"(/,'qjoule: lchnk=',i5,' pcols=',i4,' ncol=',i4)") lchnk,pcols,ncol
! write(6,"('qjoule: ue=',/,(6e12.4))") ue
! write(6,"('qjoule: ve=',/,(6e12.4))") ve

  do k = ntop_lev,nbot_lev
!   write(6,"('qjoule: k=',i3,' u=',/,(6e12.4))") k,state%u(:,k)
!   write(6,"('qjoule: k=',i3,' v=',/,(6e12.4))") k,state%v(:,k)
!   write(6,"('qjoule: k=',i3,' lxx=',/,(6e12.4))") k,lxx(:,k)
!   write(6,"('qjoule: k=',i3,' lxy=',/,(6e12.4))") k,lxy(:,k)
!   write(6,"('qjoule: k=',i3,' lyx=',/,(6e12.4))") k,lyx(:,k)
!   write(6,"('qjoule: k=',i3,' lyy=',/,(6e12.4))") k,lyy(:,k)
    do i = 1,ncol
      us           = ue(i) - state%u(i,k)
      vs           = ve(i) - state%v(i,k)
      qjoule(i,k)  = us*us*lxx(i,k) + us*vs*(lxy(i,k) - lyx(i,k)) + vs*vs*lyy(i,k)
      ptend%s(i,k) = qjoule(i,k)        ! joule heating tendency
!     ptend%s(i,k) = 0._r8              ! no joule heating tendency
    end do
!   write(6,"('qjoule: k=',i3,' qjoule(:,k)=',/,(6e12.4))") k,qjoule(:,k)
  end do
  do k = nbot_lev+1,pver
     qjoule(:ncol,k)  = 0._r8
     ptend%s(:ncol,k) = 0._r8        ! no joule heating tendency
  end do

  call get_lon_all_p( lchnk, ncol, lonndx )
  call get_lat_all_p( lchnk, ncol, latndx )


#ifdef SW_DEBUG
  max_q      = 100._r8*maxval( abs( qjoule(:ncol,ntop_lev:nbot_lev) )/state%t(:ncol,ntop_lev:nbot_lev) )
  max_ind(:) = maxloc( abs( qjoule(:ncol,ntop_lev:nbot_lev) )/state%t(:ncol,ntop_lev:nbot_lev) )
  i = max_ind(1)
  k = max_ind(2)
  if( lchnk == 25 ) then
  i = 14
  k = 3
  write(*,*) ' '
  write(*,*) '-------------------------------------------------------'
  write(*,*) 'jouleheat_tend: lon,lat = ',get_rlon_p(lchnk,14)*180._r8/pi, get_rlat_p(lchnk,14)*180._r8/pi
  write(*,*) 'jouleheat_tend: dt,t,max% dt/t = ',qjoule(i,k)/cpair,state%t(i,k),max_q,' @ lchnk,i,k = ',lchnk,max_ind(:)
  write(*,*) 'jouleheat_tend: lxx,xy,yx,yy   = ',lxx(i,k),lxy(i,k),lyx(i,k),lyy(i,k)
  write(*,*) 'jouleheat_tend: u,ue,v,ve      = ',state%u(i,k),ue(i),state%v(i,k),ve(i)
  write(*,*) 'jouleheat_tend: us,vs          = ',ue(i) - state%u(i,k),ve(i) - state%v(i,k)
  write(*,*) 'jouleheat_tend: du,dv          = ',ptend%u(i,k),ptend%v(i,k)
  write(*,*) 'jouleheat_tend: dt'
  write(*,'(1p,5g15.7)') qjoule(max_ind(1),ntop_lev:nbot_lev)/cpair
  write(*,*) '-------------------------------------------------------'
  write(*,*) ' '
! stop 'diagnostics'
  end if
#endif

  ptend%name = "joule heating"
  ptend%ls   = .true.

  qout(:ncol,:) = qjoule(:ncol,:)/cpair
  call outfld ( 'QJOULE', qout, pcols, lchnk )

  end subroutine jouleheat_tend

  end module iondrag
