
#include <misc.h>

      module mo_airplane
!--------------------------------------------------------------------
!	... Airplane insitu emission sources
!--------------------------------------------------------------------

      use shr_kind_mod, only : r8 => shr_kind_r8
      use abortutils,   only : endrun

      implicit none

      private
      public  :: airpl_src
      public  :: air_altitude, pno, pco


      save 

      real(r8), allocatable :: pno(:,:,:)
      real(r8), allocatable :: pco(:,:,:)
      real(r8), allocatable :: air_altitude(:)

      contains

      subroutine airpl_src( filename )
!-----------------------------------------------------------------------
! 	... Initialize airplane emissions
!	    Note: The emissions are read in in units of molecules/cm**2/s
!	          on a vertically resolved grid.
!		  Conversion to units of molec/cm**3/s is done in SETEXT
!-----------------------------------------------------------------------

      use pmgrid,        only : masterproc, plat, plon
      use phys_grid,     only : clat_p, clon_p
      use mo_regrider,   only : regrid_inti, regrid_2d, regrid_lat_limits
      use ioFileMod,     only : getfil
      use chem_mods,     only : adv_mass
      use m_spc_id,      only : id_co
      use dynconst,      only : rearth
      use commap,        only : latwts => w
      use shr_const_mod, only : pi => shr_const_pi
#ifdef SPMD
      use mpishorthand,  only : mpicom, mpir8, mpiint
#endif

!-----------------------------------------------------------------------
! 	... Dummy args
!-----------------------------------------------------------------------
      character(len=*), intent(in) :: filename

!-----------------------------------------------------------------------
!	... Local variables
!-----------------------------------------------------------------------
      real(r8), parameter :: msq2cmsq = 1.e4_r8
      integer  :: ios, k, jl, ju, j
      integer  :: jlim(2)
      integer  :: nlat, nlon, nlev, ndims
      integer  :: ncid, vid, ierr
      integer  :: dimid_lat, dimid_lon, dimid_lev
      integer  :: gndx
      integer  :: dimid(3)
      real(r8) :: d2r
      real(r8), allocatable :: lat(:), lon(:)
      real(r8), allocatable :: pno_in(:,:,:), pco_in(:,:,:)
      real(r8)  :: total(3)
      real(r8)  :: seq, sf(plat), factor
      character(len=256) :: locfn

#include <netcdf.inc>


#ifdef SPMD
      write(*,*) 'airpl_src: mpicom,mpir8,mpiint = ',mpicom,mpir8,mpiint
#endif

Masterproc_only : &
      if( masterproc ) then
!-----------------------------------------------------------------------
!	... Open NetCDF file
!-----------------------------------------------------------------------
      call getfil (filename, locfn, 0)
      call wrap_open (trim(locfn), NF_NOWRITE, ncid)
      d2r = pi/180._r8

!-----------------------------------------------------------------------
!       ... Get grid dimensions from file
!-----------------------------------------------------------------------
      call wrap_inq_dimid( ncid, 'lat', dimid_lat )
      call wrap_inq_dimlen( ncid, dimid_lat, nlat )
      allocate( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: lat allocation error = ',ierr
         call endrun
      end if
      call wrap_inq_varid( ncid, 'lat', vid )
      call wrap_get_var_realx( ncid, vid, lat )
      lat(:nlat) = lat(:nlat) * d2r
 
      call wrap_inq_dimid( ncid, 'lon', dimid_lon )
      call wrap_inq_dimlen( ncid, dimid_lon, nlon )
      allocate( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: lon allocation error = ',ierr
         call endrun
      end if
      call wrap_inq_varid( ncid, 'lon', vid )
      call wrap_get_var_realx( ncid, vid, lon )
      lon(:nlon) = lon(:nlon) * d2r
 
      call wrap_inq_dimid( ncid, 'altitude', dimid_lev )
      call wrap_inq_dimlen( ncid, dimid_lev, nlev )
      allocate( air_altitude(nlev+1), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: air_altitude allocation error = ',ierr
         call endrun
      end if
      call wrap_inq_varid( ncid, 'altitude', vid )
      call wrap_get_var_realx( ncid, vid, air_altitude(1:nlev) )
      air_altitude(nlev+1) = air_altitude(nlev) + (air_altitude(nlev) - air_altitude(nlev-1))

!-----------------------------------------------------------------------
!       ... Set up regridding
!-----------------------------------------------------------------------
      gndx = regrid_inti( nlat, plat, &
                          nlon, plon, &
                          lon,  clon_p(:,1), &
                          lat,  clat_p, &
                          0, &
                          do_lons=.true.,do_lats=.true. )
      deallocate( lat, lon, stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: Failed to deallocate lat,lon; ierr = ',ierr
         call endrun
      end if
      jl   = 1
      ju   = plat
      if( gndx /= 0 )then
         jlim = regrid_lat_limits( gndx )
      else
         jlim = (/ jl,ju /)
      end if
      allocate( pno_in(nlon,jlim(1):jlim(2),nlev), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: pno_in allocation error = ',ierr
         call endrun
      end if
      allocate( pco_in(nlon,jlim(1):jlim(2),nlev), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: pco_in allocation error = ',ierr
         call endrun
      end if
      allocate(pno(plon,plat,nlev), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: pno allocation error = ',ierr
         call endrun
      end if
      allocate( pco(plon,plat,nlev), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: pco allocation error = ',ierr
         call endrun
      end if

!-----------------------------------------------------------------------
!	... Read emissions
!-----------------------------------------------------------------------
      call wrap_inq_varid( ncid, 'nox', vid )
      ierr = nf_inq_varndims( ncid, vid, ndims )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'airpl_src: Failed to get number of dimensions for variable nox'
         call endrun
      end if
      if( ndims /= 3 ) then
         write(*,*) 'airpl_src: variable nox has ndims = ',ndims,', expecting 3'
         call endrun
      end if
      ierr = nf_inq_vardimid( ncid, vid, dimid )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'airpl_src: Failed to get dimension IDs for variable nox'
         call endrun
      end if
      if( dimid(1) /= dimid_lon  .or. dimid(2) /= dimid_lat .or.  dimid(3) /= dimid_lev ) then
            write(*,*) 'airpl_src: Dimensions in wrong order for variable nox'
            write(*,*) '...      Expecting (lon, lat, lev)'
            call endrun
      end if
      call wrap_get_vara_realx( ncid, vid, &
                                (/ 1, jlim(1), 1/), &                    ! start
                                (/ nlon, jlim(2)-jlim(1)+1, nlev /), &   ! count
                                pno_in )

      call wrap_inq_varid( ncid, 'co', vid )
      ierr = nf_inq_varndims( ncid, vid, ndims )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'airpl_src: Failed to get number of dimensions for variable co'
         call endrun
      end if
      if( ndims /= 3 ) then
         write(*,*) 'READ_SFLX: variable co has ndims = ',ndims,', expecting 3'
         call endrun
      end if
      ierr = nf_inq_vardimid( ncid, vid, dimid )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'airpl_src: Failed to get dimension IDs for variable co'
         call endrun
      end if
      if( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. dimid(3) /= dimid_lev ) then
            write(*,*) 'airpl_src: Dimensions in wrong order for variable co'
            write(*,*) '...      Expecting (lon, lat, lev)'
            call endrun
      end if
      call wrap_get_vara_realx( ncid, vid, &
                                (/ 1, jlim(1), 1/), &                    ! start
                                (/ nlon, jlim(2)-jlim(1)+1, nlev /), &   ! count
                                pco_in )
      call wrap_close( ncid )

!-----------------------------------------------------------------------
!	... Regrid emissions
!-----------------------------------------------------------------------
      do k = 1,nlev
         call regrid_2d( pno_in(:,jlim(1):jlim(2),k), pno(:,:,k), gndx, jl, ju, do_poles=.true. )
         call regrid_2d( pco_in(:,jlim(1):jlim(2),k), pco(:,:,k), gndx, jl, ju, do_poles=.true. )
      end do
      deallocate( pno_in, pco_in, stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'airpl_src: Failed to deallocate pno_in,pco_in; ierr = ',ierr
         call endrun
      end if
!-----------------------------------------------------------------------
!       ... Get global emission from this source
!-----------------------------------------------------------------------
      seq = 2.*pi*msq2cmsq*rearth**2/real(plon)
      do j = 1,plat
         sf(j) = seq*latwts(j)
      end do

      factor = 86400._r8 * 365._r8 &   ! sec / year
             / 6.022e23_r8 &           ! molec / mole
             * 1.e-12_r8               ! Tg / g

      total(:) = 0._r8
      do j = 1,plat
         total(1) = total(1) + sum( pno(:plon,j,:nlev ) ) * sf(j)
         total(2) = total(2) + sum( pco(:plon,j,:nlev ) ) * sf(j)
      end do
!-----------------------------------------------------------------------
!       ... Convert totals from molec cm^-2 s^-1 to Tg y^-1
!-----------------------------------------------------------------------
      total(1) = total(1) * 14.00674_r8 * factor
      total(2) = total(2) * adv_mass(id_co) * factor

      write(*,*) 'airpl_src: nlev = ',nlev
      write(*,'('' airpl_src Aircraft emissions: '',a6,'' = '',f10.3,1X,a6)') 'NO',total(1),'TgN/y'
      write(*,'('' airpl_src Aircraft emissions: '',a6,'' = '',f10.3,1X,a6)') 'CO',total(2),'Tg/y'

      end if Masterproc_only

#ifdef SPMD
      call mpibarrier( mpicom )
      call mpibcast( nlev, 1, mpiint, 0, mpicom )
      write(*,*) 'airpl_src: nlev = ',nlev
      if( .not. masterproc ) then
         allocate( air_altitude(nlev+1), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'airpl_src: air_altitude allocation error = ',ierr
            call endrun
         end if
         allocate( pno(plon,plat,nlev), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'airpl_src: pno allocation error = ',ierr
            call endrun
         end if
         allocate( pco(plon,plat,nlev), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'airpl_src: pco allocation error = ',ierr
            call endrun
         end if
      end if
      call mpibarrier( mpicom )
      call mpibcast( air_altitude, nlev+1, mpir8, 0, mpicom )
      write(*,*) 'airpl_src: bcast air_altitude'
      call mpibcast( pno, plon*plat*nlev, mpir8, 0, mpicom )
      write(*,*) 'airpl_src: bcast pno'
      call mpibcast( pco, plon*plat*nlev, mpir8, 0, mpicom )
      write(*,*) 'airpl_src: bcast pco'
#endif

      end subroutine airpl_src

      end module mo_airplane
