
      module mo_cph
!---------------------------------------------------------------------
!	... compute chemical potential heating
!---------------------------------------------------------------------

      use shr_kind_mod,  only : r8 => shr_kind_r8

      implicit none

      save

      integer , parameter :: ncph    = 41
      real(r8), parameter :: secpday = 86400._r8
      real(r8), parameter :: daypsec = 1._r8/secpday
!==============================================================
!... Doug Kinnison, dkin@ucar.edu
!
!... Enthalpy Data are taken from Atkinson et al., 
!    Evaluated kinetic and photochemical data for atmospheric
!    chemistry: Volume I, Atmos. Chem. Phys., 4, 1461-1738.
!... Heats of formation at 0K.
!... Units: kJ mol-1
!
!... Exception to the Atkinson et al. reference  (@0K unless noted)
!    (4), (5), (8), (9), (10, (11), (14), (15), (27), (28) at 298K  
!    (7)  h + o2 -> oh + o2 is multiplied by 0.6 (Mlynczak) to represent
!         AG loss of excited OH.
!    (25) n2d + o2 -> no + o1d taken from Roble, UMLT, Johnson and Killeen.
!    (26) n2d + o  -> n  + o   taken from Roble, UMLT, Johnson and Killeen.
!    (30-41) Taken from Roble, UMLT, Johnson and Killeen Ed., Geophys. Mono. 87
!==============================================================
      real(r8) :: exotherm(ncph) = (/ &
         392.19_r8, &                       !(1)  o + o3 -> 2o2
         493.58_r8, &                       !(2)  o + o + m -> o2 + m
         67.67_r8,  &                       !(3)  o + oh -> h + o2
         226.58_r8, &                       !(4)  o + ho2 -> oh + o2
         203.40_r8, &                       !(5)  h + o2 + m -> ho2 + m
         101.39_r8, &                       !(6)  o + o2 + m -> o3 + m
         194.71_r8, &                       !(7)  h + o3 -> oh + o2 (*0.6)
         34.47_r8,  &                       !(8)  ho2 + no -> no2 + oh
         120.10_r8, &                       !(9)  ho2 + o3 -> oh + 2o2
         165.51_r8, &                       !(10) ho2 + ho2 -> h2o2 + o2
         165.30_r8, &                       !(11) oh + o3 -> ho2 + o2
         199.17_r8, &                       !(12) no + o3 -> no2 + o2
         193.02_r8, &                       !(13) no2 + o -> no + o2
         293.62_r8, &                       !(14) oh + ho2 -> h2o + o2
         232.59_r8, &                       !(15) h + ho2 -> h2 + o2
         32.91_r8,  &                       !(16) o1d + o2 -> o + o2_1s
         189.91_r8, &                       !(17) o1d + n2 -> o + n2
         62.60_r8,  &                       !(18) o2_1s + o -> o2_1d + o
         62.60_r8,  &                       !(19) o2_1s + o2 -> o2_1d + o2
         62.60_r8,  &                       !(20) o2_1s + n2 -> o2_1d + n2
         62.60_r8,  &                       !(21) o2_1s + o3 -> o2_1d + o3
         94.30_r8,  &                       !(22) o2_1d + o -> o2 + o
         94.30_r8,  &                       !(23) o2_1d + o2 -> 2o2
         94.30_r8,  &                       !(24) o2_1d + n2 -> o2 + n2
         177.51_r8, &                       !(25) n2d + o2 -> no + o1d
         229.61_r8, &                       !(26) n2d + o -> n + o
         133.75_r8, &                       !(27) n + o2 -> no + o
         313.75_r8, &                       !(28) n + no -> n2 + o
         189.81_r8, &                       !(29) o1d + o2 -> o + o2
         150.11_r8, &                       !(30) Op + o2 -> O2p + o
         105.04_r8, &                       !(31) Op + n2 -> NOp + n
         67.53_r8,  &                       !(32) N2p + o -> NOp + n2d
         406.16_r8, &                       !(33) O2p + n -> NOp + o
         271.38_r8, &                       !(34) O2p + no -> NOp + o2
         239.84_r8, &                       !(35) Np + o2 -> O2p + n
         646.28_r8, &                       !(36) Np + o2 -> NOp + o
         95.55_r8,  &                       !(37) Np + o -> Op + n
         339.59_r8, &                       !(38) N2p + o2 -> O2p + n2
         82.389_r8, &                       !(39) NOp + e -> 
         508.95_r8, &                       !(40) O2p + e -> 
         354.83_r8  &                       !(41) N2p + e -> 
         /)

      private
      public :: cph

      contains

      subroutine cph( cph_tot, vmr, rxt, cp, mbar, &
                      kbot, ncol, lchnk )
!-----------------------------------------------------------------------
!      	... forms the chemical potential heating rates
!-----------------------------------------------------------------------

      use m_spc_id
      use m_rxt_id
      use chem_mods,     only : gas_pcnst, rxntot
      use ppgrid,        only : pver
      use history,       only : outfld

      implicit none

!-----------------------------------------------------------------------
!     	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)   ::  ncol                                ! columns in chunck
      integer, intent(in)   ::  lchnk                               ! chunk index
      integer, intent(in)   ::  kbot                                ! bottom vert index
      real(r8), intent(in)  ::  rxt(ncol,pver,rxntot)               ! rxt rates (1/cm^3/s)
      real(r8), intent(in)  ::  cp(ncol,pver)                       ! specific heat capacity (J/K/kg)
      real(r8), intent(in)  ::  mbar(ncol,pver)                     ! atm mean mass (g/mole)
      real(r8), intent(in)  ::  vmr(ncol,pver,gas_pcnst)            ! concentrations (mol/mol)
      real(r8), intent(out) ::  cph_tot(ncol,pver)                  ! total heating (K/s)

!-----------------------------------------------------------------------
!     	... local variables
!-----------------------------------------------------------------------
      integer  ::  i, k
      real(r8) ::  tmp(ncol)
      real(r8) ::  cph_rate(ncol,pver,ncph)

      cph_rate(:,:,:) = 0._r8
      do k = 1,kbot
         tmp(:)           = 1._r8 / (1.e-6_r8*cp(:,k)*mbar(:,k))
         cph_rate(:,k,1)  = tmp(:)*rxt(:,k,rid_cph1)*vmr(:,k,id_o)*vmr(:,k,id_o3)*exotherm(1)
         cph_rate(:,k,2)  = tmp(:)*rxt(:,k,rid_usr2)*vmr(:,k,id_o)*vmr(:,k,id_o)*exotherm(2)
         cph_rate(:,k,3)  = tmp(:)*rxt(:,k,rid_cph3)*vmr(:,k,id_o)*vmr(:,k,id_oh)*exotherm(3)
         cph_rate(:,k,4)  = tmp(:)*rxt(:,k,rid_cph4)*vmr(:,k,id_o)*vmr(:,k,id_ho2)*exotherm(4)
         cph_rate(:,k,5)  = tmp(:)*rxt(:,k,rid_cph5)*vmr(:,k,id_h)*vmr(:,k,id_o2)*exotherm(5)
         cph_rate(:,k,6)  = tmp(:)*rxt(:,k,rid_usr1)*vmr(:,k,id_o)*vmr(:,k,id_o2)*exotherm(6)
         cph_rate(:,k,7)  = tmp(:)*rxt(:,k,rid_cph7)*vmr(:,k,id_h)*vmr(:,k,id_o3)*exotherm(7)
         cph_rate(:,k,8)  = tmp(:)*rxt(:,k,rid_cph8)*vmr(:,k,id_ho2)*vmr(:,k,id_no)*exotherm(8)
         cph_rate(:,k,9)  = tmp(:)*rxt(:,k,rid_cph9)*vmr(:,k,id_ho2)*vmr(:,k,id_o3)*exotherm(9)
         cph_rate(:,k,10) = tmp(:)*rxt(:,k,rid_usr7)*vmr(:,k,id_ho2)*vmr(:,k,id_ho2)*exotherm(10)
         cph_rate(:,k,11) = tmp(:)*rxt(:,k,rid_cph11)*vmr(:,k,id_oh)*vmr(:,k,id_o3)*exotherm(11)
         cph_rate(:,k,12) = tmp(:)*rxt(:,k,rid_cph12)*vmr(:,k,id_no)*vmr(:,k,id_o3)*exotherm(12)
         cph_rate(:,k,13) = tmp(:)*rxt(:,k,rid_cph13)*vmr(:,k,id_no2)*vmr(:,k,id_o)*exotherm(13)
         cph_rate(:,k,14) = tmp(:)*rxt(:,k,rid_cph14)*vmr(:,k,id_oh)*vmr(:,k,id_ho2)*exotherm(14)
         cph_rate(:,k,15) = tmp(:)*rxt(:,k,rid_cph15)*vmr(:,k,id_h)*vmr(:,k,id_ho2)*exotherm(15)
         cph_rate(:,k,16) = tmp(:)*rxt(:,k,rid_cph16)*vmr(:,k,id_o1d)*vmr(:,k,id_o2)*exotherm(16)
         cph_rate(:,k,17) = tmp(:)*rxt(:,k,rid_cph17)*vmr(:,k,id_o1d)*exotherm(17)
         cph_rate(:,k,18) = tmp(:)*rxt(:,k,rid_cph18)*vmr(:,k,id_o2_1s)*vmr(:,k,id_o)*exotherm(18)
         cph_rate(:,k,19) = tmp(:)*rxt(:,k,rid_cph19)*vmr(:,k,id_o2_1s)*vmr(:,k,id_o2)*exotherm(19)
         cph_rate(:,k,20) = tmp(:)*rxt(:,k,rid_cph20)*vmr(:,k,id_o2_1s)*exotherm(20)
         cph_rate(:,k,21) = tmp(:)*rxt(:,k,rid_cph21)*vmr(:,k,id_o2_1s)*vmr(:,k,id_o3)*exotherm(21)
         cph_rate(:,k,22) = tmp(:)*rxt(:,k,rid_cph22)*vmr(:,k,id_o2_1d)*vmr(:,k,id_o)*exotherm(22)
         cph_rate(:,k,23) = tmp(:)*rxt(:,k,rid_cph23)*vmr(:,k,id_o2_1d)*vmr(:,k,id_o2)*exotherm(23)
         cph_rate(:,k,24) = tmp(:)*rxt(:,k,rid_cph24)*vmr(:,k,id_o2_1d)*exotherm(24)
         cph_rate(:,k,25) = tmp(:)*rxt(:,k,rid_cph25)*vmr(:,k,id_n2d)*vmr(:,k,id_o2)*exotherm(25)
         cph_rate(:,k,26) = tmp(:)*rxt(:,k,rid_cph26)*vmr(:,k,id_n2d)*vmr(:,k,id_o)*exotherm(26)
         cph_rate(:,k,27) = tmp(:)*rxt(:,k,rid_cph27)*vmr(:,k,id_n)*vmr(:,k,id_o2)*exotherm(27)
         cph_rate(:,k,28) = tmp(:)*rxt(:,k,rid_cph28)*vmr(:,k,id_n)*vmr(:,k,id_no)*exotherm(28)
         cph_rate(:,k,29) = tmp(:)*rxt(:,k,rid_cph29)*vmr(:,k,id_o1d)*vmr(:,k,id_o2)*exotherm(29)
         cph_rate(:,k,30) = tmp(:)*rxt(:,k,rid_ion1)*vmr(:,k,id_op)*vmr(:,k,id_o2)*exotherm(30)
         cph_rate(:,k,31) = tmp(:)*rxt(:,k,rid_ion2)*vmr(:,k,id_op)*exotherm(31)
         cph_rate(:,k,32) = tmp(:)*rxt(:,k,rid_ion3)*vmr(:,k,id_n2p)*vmr(:,k,id_o)*exotherm(32)
         cph_rate(:,k,33) = tmp(:)*rxt(:,k,rid_ion4)*vmr(:,k,id_o2p)*vmr(:,k,id_n)*exotherm(33)
         cph_rate(:,k,34) = tmp(:)*rxt(:,k,rid_ion5)*vmr(:,k,id_o2p)*vmr(:,k,id_no)*exotherm(34)
         cph_rate(:,k,35) = tmp(:)*rxt(:,k,rid_ion6)*vmr(:,k,id_o2)*vmr(:,k,id_np)*exotherm(35)
         cph_rate(:,k,36) = tmp(:)*rxt(:,k,rid_ion7)*vmr(:,k,id_o2)*vmr(:,k,id_np)*exotherm(36)
         cph_rate(:,k,37) = tmp(:)*rxt(:,k,rid_ion8)*vmr(:,k,id_np)*vmr(:,k,id_o)*exotherm(37)
         cph_rate(:,k,38) = tmp(:)*rxt(:,k,rid_ion9)*vmr(:,k,id_n2p)*vmr(:,k,id_o2)*exotherm(38)
         cph_rate(:,k,39) = tmp(:)*rxt(:,k,rid_elec1)*vmr(:,k,id_e)*vmr(:,k,id_nop)*exotherm(39)
         cph_rate(:,k,40) = tmp(:)*rxt(:,k,rid_elec2)*vmr(:,k,id_e)*vmr(:,k,id_o2p)*exotherm(40)
         cph_rate(:,k,41) = tmp(:)*rxt(:,k,rid_elec3)*vmr(:,k,id_e)*vmr(:,k,id_n2p)*exotherm(41)
      end do

      cph_tot(:,:) = 0._r8
      do k = 1,kbot
         do i = 1,ncol
            cph_tot(i,k) = sum( cph_rate(i,k,:) )
         end do
      end do

!-----------------------------------------------------------------------
!     	... output the rates
!-----------------------------------------------------------------------
      call outfld( 'CPH1', cph_rate(:,:,1), ncol, lchnk )
      call outfld( 'CPH2', cph_rate(:,:,2), ncol, lchnk )
      call outfld( 'CPH3', cph_rate(:,:,3), ncol, lchnk )
      call outfld( 'CPH4', cph_rate(:,:,4), ncol, lchnk )
      call outfld( 'CPH5', cph_rate(:,:,5), ncol, lchnk )
      call outfld( 'CPH6', cph_rate(:,:,6), ncol, lchnk )
      call outfld( 'CPH7', cph_rate(:,:,7), ncol, lchnk )
      call outfld( 'CPH8', cph_rate(:,:,8), ncol, lchnk )
      call outfld( 'CPH9', cph_rate(:,:,9), ncol, lchnk )
      call outfld( 'CPH10', cph_rate(:,:,10), ncol, lchnk )
      call outfld( 'CPH11', cph_rate(:,:,11), ncol, lchnk )
      call outfld( 'CPH12', cph_rate(:,:,12), ncol, lchnk )
      call outfld( 'CPH13', cph_rate(:,:,13), ncol, lchnk )
      call outfld( 'CPH14', cph_rate(:,:,14), ncol, lchnk )
      call outfld( 'CPH15', cph_rate(:,:,15), ncol, lchnk )
      call outfld( 'CPH16', cph_rate(:,:,16), ncol, lchnk )
      call outfld( 'CPH17', cph_rate(:,:,17), ncol, lchnk )
      call outfld( 'CPH18', cph_rate(:,:,18), ncol, lchnk )
      call outfld( 'CPH19', cph_rate(:,:,19), ncol, lchnk )
      call outfld( 'CPH20', cph_rate(:,:,20), ncol, lchnk )
      call outfld( 'CPH21', cph_rate(:,:,21), ncol, lchnk )
      call outfld( 'CPH22', cph_rate(:,:,22), ncol, lchnk )
      call outfld( 'CPH23', cph_rate(:,:,23), ncol, lchnk )
      call outfld( 'CPH24', cph_rate(:,:,24), ncol, lchnk )
      call outfld( 'CPH25', cph_rate(:,:,25), ncol, lchnk )
      call outfld( 'CPH26', cph_rate(:,:,26), ncol, lchnk )
      call outfld( 'CPH27', cph_rate(:,:,27), ncol, lchnk )
      call outfld( 'CPH28', cph_rate(:,:,28), ncol, lchnk )
      call outfld( 'CPH29', cph_rate(:,:,29), ncol, lchnk )
      call outfld( 'CPH30', cph_rate(:,:,30), ncol, lchnk )
      call outfld( 'CPH31', cph_rate(:,:,31), ncol, lchnk )
      call outfld( 'CPH32', cph_rate(:,:,32), ncol, lchnk )
      call outfld( 'CPH33', cph_rate(:,:,33), ncol, lchnk )
      call outfld( 'CPH34', cph_rate(:,:,34), ncol, lchnk )
      call outfld( 'CPH35', cph_rate(:,:,35), ncol, lchnk )
      call outfld( 'CPH36', cph_rate(:,:,36), ncol, lchnk )
      call outfld( 'CPH37', cph_rate(:,:,37), ncol, lchnk )
      call outfld( 'CPH38', cph_rate(:,:,38), ncol, lchnk )
      call outfld( 'CPH39', cph_rate(:,:,39), ncol, lchnk )
      call outfld( 'CPH40', cph_rate(:,:,40), ncol, lchnk )
      call outfld( 'CPH41', cph_rate(:,:,41), ncol, lchnk )
      call outfld( 'QCP', cph_tot(:,:), ncol, lchnk )

      end subroutine cph

      end module mo_cph
