#include <misc.h>
#include <params.h>

      module mo_drydep
!---------------------------------------------------------------------
!       ... Dry deposition velocity input data and code for netcdf input
!---------------------------------------------------------------------

      use shr_kind_mod, only : r8 => shr_kind_r8
      use chem_mods,    only : gas_pcnst
      use pmgrid,       only : plon, plat
      use abortutils,   only : endrun

      implicit none
      private
      save

      public  :: dvel_inti, interpdvel, mkdvel, drydep

      integer  :: map(gas_pcnst)                        ! indices for drydep species
      integer  :: nspecies                              ! number of depvel species in input file
      real(r8) :: pid2
      real(r8), allocatable :: dvel(:,:,:,:)            ! depvel array interpolated to model grid
      real(r8), allocatable :: dvel_interp(:,:,:)       ! depvel array interpolated to grid and time
      real(r8) :: days(12)

      contains

      subroutine dvel_inti(depvel_file)
!---------------------------------------------------------------------------
!       ... Initialize module, depvel arrays, and a few other variables.
!           The depvel fields will be linearly interpolated to the correct time
!---------------------------------------------------------------------------

      use pmgrid,        only : masterproc
      use phys_grid,     only : clat_p, clon_p
      use ioFileMod,     only : getfil
      use mo_regrider,   only : regrid_inti, regrid_lat_limits, regrid_2d
      use mo_tracname,   only : solsym
      use time_manager,  only : get_calday
      use shr_const_mod, only : pi => shr_const_pi
#ifdef SPMD
      use mpishorthand,  only : mpicom, mpir8, mpiint
#endif

      implicit none

      character(len=*), intent(in) :: depvel_file

!---------------------------------------------------------------------------
!       ... Local variables
!---------------------------------------------------------------------------
      integer  :: ncid, vid, vid_dvel, nlat, nlon, nmonth, ndims
      integer  :: dimid_lat, dimid_lon, dimid_species, dimid_time
      integer  :: dimid(4), count(4), start(4)
      integer  :: jlim(2), jl, ju
      integer  :: gndx, m, ispecies, nchar, ierr
      integer  :: dates(12) = (/ 116, 214, 316, 415,  516,  615, 716, 816, 915, 1016, 1115, 1216 /)
      real(r8) :: scale_factor, r2d, d2r
      real(r8), allocatable :: dvel_lats(:), dvel_lons(:)
      real(r8), allocatable :: dvel_in(:,:,:,:)                          ! input depvel array
      character(len=50)              :: units
      character(len=256)             :: locfn
      character(len=20), allocatable :: species_names(:)             ! names of depvel species
      logical  :: found

#include <netcdf.inc>

      pid2 = .5_r8 * pi

Masterproc_only : &
      if( masterproc ) then
!---------------------------------------------------------------------------
!       ... Open NetCDF file
!---------------------------------------------------------------------------
      d2r  = pi/180._r8
      r2d  = 1._r8/d2r

      call getfil (depvel_file, locfn, 0)
      call wrap_open (trim(locfn), NF_NOWRITE, ncid)

!---------------------------------------------------------------------------
!       ... Get variable ID for dep vel array
!---------------------------------------------------------------------------
      call wrap_inq_varid( ncid, 'dvel', vid_dvel )

!---------------------------------------------------------------------------
!       ... Inquire about dimensions
!---------------------------------------------------------------------------
      call wrap_inq_dimid( ncid, 'lon', dimid_lon )
      call wrap_inq_dimlen( ncid, dimid_lon, nlon )
      call wrap_inq_dimid( ncid, 'lat', dimid_lat )
      call wrap_inq_dimlen( ncid, dimid_lat, nlat )
      call wrap_inq_dimid( ncid, 'species', dimid_species )
      call wrap_inq_dimlen( ncid, dimid_species, nspecies )
      call wrap_inq_dimid( ncid, 'time', dimid_time )
      call wrap_inq_dimlen( ncid, dimid_time, nmonth )

      write(*,*) 'dvel_inti: dimensions (nlon,nlat,nspecies,nmonth) = ',nlon,nlat,nspecies,nmonth

!---------------------------------------------------------------------------
!       ... Check dimensions of dvel variable. Must be (lon, lat, species, month).
!---------------------------------------------------------------------------
      ierr = nf_inq_varndims( ncid, vid_dvel, ndims )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'dvel_inti: Failed to get ndims for dvel'
	 call endrun
      end if
      if( ndims /= 4 ) then
         write(*,*) 'dvel_inti: dvel has ',ndims,' dimensions. Expecting 4.'
         call endrun
      end if
      ierr = nf_inq_vardimid( ncid, vid_dvel, dimid )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'dvel_inti: Failed to get dimension IDs for dvel'
	 call endrun
      end if
      if( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. &
          dimid(3) /= dimid_species .or. dimid(4) /= dimid_time ) then
         write(*,*) 'dvel_inti: Dimensions in wrong order for dvel'
         write(*,*) '...      Expecting (lon, lat, species, month)'
         call endrun
      end if

!---------------------------------------------------------------------------
!       ... Allocate depvel lats, lons and read
!---------------------------------------------------------------------------
      allocate( dvel_lats(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel_lats vector'
         call endrun
      end if
      allocate( dvel_lons(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel_lons vector'
         call endrun
      end if
      call wrap_inq_varid( ncid, 'lat', vid )
      call wrap_get_var_realx( ncid, vid, dvel_lats )
      call wrap_inq_varid( ncid, 'lon', vid )
      call wrap_get_var_realx( ncid, vid, dvel_lons )

!---------------------------------------------------------------------------
!       ... Set the transform from inputs lats to simulation lats
!---------------------------------------------------------------------------
      write(*,*) 'dvel_inti: source lats (deg)'
      write(*,'(10f8.2)') dvel_lats
      write(*,*) 'dvel_inti: target lats (deg)'
      write(*,'(10f8.2)') clat_p(:)*r2d
      write(*,*) 'dvel_inti: source lons (deg)'
      write(*,'(10f8.2)') dvel_lons
      write(*,*) 'dvel_inti: target lons (deg)'
      write(*,'(10f8.2)') clon_p(:,1)*r2d

      dvel_lats(:nlat) = d2r * dvel_lats(:nlat)
      dvel_lons(:nlon) = d2r * dvel_lons(:nlon)
      gndx = regrid_inti( nlat, plat, nlon, plon, dvel_lons, clon_p(:,1), dvel_lats, clat_p, 0 )
      write(*,*) 'dvel_inti: REGRID_INTI using grid number ',gndx
      jl = 1
      ju = plat
      if( gndx /= 0 ) then
         jlim(:) = regrid_lat_limits( gndx )
      else
         jlim(:) = (/ jl,ju /)
      end if
      write(*,*) 'dvel_inti: jlim',jlim(:)
      deallocate( dvel_lats, dvel_lons )

!---------------------------------------------------------------------------
!     	... Allocate dvel and read data from file
!---------------------------------------------------------------------------
      allocate( dvel_in(nlon,jlim(1):jlim(2),nspecies,nmonth), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel_in'
         call endrun
      end if
      start = (/ 1, jlim(1), 1, 1 /)
      count = (/ nlon, jlim(2)-jlim(1)+1, nspecies, nmonth /)
      call wrap_get_vara_realx( ncid, vid_dvel, start, count, dvel_in )

!---------------------------------------------------------------------------
!     	... Check units of deposition velocity. If necessary, convert to cm/s.
!---------------------------------------------------------------------------
      units(:) = ' '
      call wrap_get_att_text( ncid, vid_dvel, 'units', units )
      write(*,*) 'dvel_inti: depvel len, units = ',len_trim(units),units
      if( units(1:3) == 'm/s' ) then
#ifdef DEBUG
         write(*,*) 'dvel_inti: depvel units = m/s. Converting to cm/s'
#endif
         scale_factor = 100._r8
      else if( units(1:4) == 'cm/s' ) then
#ifdef DEBUG
         write(*,*) 'dvel_inti: depvel units = cm/s'
#endif
         scale_factor = 1._r8
      else
#ifdef DEBUG
         write(*,*) 'dvel_inti: Warning! depvel units unknown = ',trim(units)
         write(*,*) '           ...      proceeding with scale_factor=1'
#endif
         scale_factor = 1._r8
      end if

!---------------------------------------------------------------------------
!     	... Regrid deposition velocities
!---------------------------------------------------------------------------
      allocate( dvel(plon,plat,nspecies,nmonth),stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: Failed to allocate dvel'
         call endrun
      end if
      do nmonth = 1,12
         do m = 1,nspecies
            call regrid_2d( dvel_in(:,jlim(1):jlim(2),m,nmonth), dvel(:,:,m,nmonth), gndx, jl, ju, &
                            do_poles=.true., scaling=scale_factor )
         end do
      end do
      deallocate( dvel_in )

!---------------------------------------------------------------------------
!     	... Read in species names and determine mapping to tracer numbers
!---------------------------------------------------------------------------
      allocate( species_names(nspecies), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: species_names allocation error = ',ierr
         call endrun
      end if
      call wrap_inq_varid( ncid, 'species_name', vid )
      ierr = nf_inq_varndims( ncid, vid, ndims )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'dvel_inti: Getting number of dimensions for species_name'
	 call endrun
      end if
      ierr = nf_inq_vardimid( ncid, vid, dimid )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'dvel_inti: Getting dimensions for species_name'
	 call endrun
      end if
      call wrap_inq_dimlen( ncid, dimid(1), nchar )
      map(:) = 0
      do ispecies = 1,nspecies
         start(:2) = (/ 1, ispecies /)
         count(:2) = (/ nchar, 1 /)
         species_names(ispecies)(:) = ' '
         ierr = nf_get_vara_text( ncid, vid, start(:2), count(:2), species_names(ispecies) )
         if( ierr /= NF_NOERR ) then
            write(*,*) 'dvel_inti: Getting species names'
	    call endrun
         end if
         found = .false.
         do m = 1,gas_pcnst
            if( species_names(ispecies) == solsym(m) .or. &
                (species_names(ispecies) == 'HNO4' .and. solsym(m) == 'HO2NO2') ) then
               found = .true.
	       if( solsym(m) /= 'CH4' ) then
                  map(m) = ispecies
#ifdef DEBUG
                  if( masterproc ) then
                     write(*,*) 'dvel_inti: ispecies, m, solsym = ',ispecies,m,trim(solsym(m))
                  end if
#endif
                  exit
               end if
            end if
         end do
         if( .not. found ) then
            write(*,*) 'dvel_inti: Warning! DVEL species ',TRIM(species_names(ispecies)),' not found'
         endif
      end do
      deallocate( species_names )
      call wrap_close( ncid )
      end if Masterproc_only

#ifdef SPMD
      call mpibarrier( mpicom )
      call mpibcast( nspecies, 1, mpiint, 0, mpicom )
      if( .not. masterproc ) then
         allocate( dvel(plon,plat,nspecies,12),stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'dvel_inti: Failed to allocate dvel'
            call endrun
         end if
      end if
      call mpibcast( dvel, plon*plat*nspecies*12, mpir8, 0, mpicom )
      call mpibcast( map, gas_pcnst, mpiint, 0, mpicom )
#endif

!---------------------------------------------------------------------------
!     	... Setup timing dates
!---------------------------------------------------------------------------
      do m = 1,12
         days(m) = get_calday( dates(m), 0 )
      end do
      write(*,*) 'dvel_inti : days'
      write(*,'(1p,5g15.8)') days(:)

!---------------------------------------------------------------------------
!     	... Allocate dvel_interp array
!---------------------------------------------------------------------------
      allocate( dvel_interp(plon,plat,nspecies), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'dvel_inti: dvel_interp allocation error = ',ierr
         call endrun
      end if
      write(*,*) 'dvel_inti : size of dvel = ',size(dvel,dim=1),size(dvel,dim=2),size(dvel,dim=3),size(dvel,dim=4)

      end subroutine dvel_inti

      subroutine interpdvel( calday )
!---------------------------------------------------------------------------
! 	... Interpolate the fields whose values are required at the
!           begining of a timestep.
!---------------------------------------------------------------------------

      implicit none

!---------------------------------------------------------------------------
! 	... Dummy arguments
!---------------------------------------------------------------------------
      real(r8), intent(in) :: &
        calday                            ! Interpolate the input data to calday

!---------------------------------------------------------------------------
! 	... Local variables
!---------------------------------------------------------------------------
      integer  :: &
        m, last, next
      real(r8) :: calday_loc, last_days, next_days

      if( calday < days(1) ) then
         next = 1
         last = 12
      else if( calday >= days(12) ) then
         next = 1
         last = 12
      else
         do m = 11,1,-1
            if( calday >= days(m) ) then
               exit
            end if
         end do
         last = m
         next = m + 1
      end if

      last_days  = days( last )
      next_days  = days( next )
      calday_loc = calday

      if( next_days < last_days ) then
         next_days = next_days + 365._r8
      end if
      if( calday_loc < last_days ) then
         calday_loc = calday_loc + 365._r8
      end if

      do m = 1,nspecies
         call intp2d( last_days, next_days, calday_loc, &
                      dvel(1,1,m,last), &
                      dvel(1,1,m,next), &
                      dvel_interp(1,1,m) )
      end do

      end subroutine interpdvel

      subroutine intp2d( t1, t2, tint, f1, f2, fint )
!-----------------------------------------------------------------------
! 	... Linearly interpolate between f1(t1) and f2(t2) to fint(tint).
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
! 	... Dummy arguments
!-----------------------------------------------------------------------
      real(r8), intent(in) :: &
        t1, &            ! time level of f1
        t2, &            ! time level of f2
        tint             ! interpolant time
      real(r8), dimension(plon,plat), intent(in) :: &
        f1, &            ! field at time t1
        f2               ! field at time t2

      real(r8), intent(out) :: &
        fint(plon,plat) ! field at time tint

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      integer  :: j
      real(r8) :: factor

      factor = (tint - t1)/(t2 - t1)
      do j = 1,plat
         fint(:,j) = f1(:,j) + (f2(:,j) - f1(:,j))*factor
      end do

      end subroutine intp2d

      subroutine mkdvel( latndx, lonndx, tsurf, zen_angle, depvel, &
			 ncol, lchnk )
!--------------------------------------------------------
!       ... Form the deposition velocities for this
!           latitude slice
!--------------------------------------------------------

      use m_spc_id
      use chem_mods, only : gas_pcnst
      use ppgrid,    only : pcols
      use comsrf,    only : landfrac, ocnfrac, icefrac

      implicit none

!--------------------------------------------------------
!       ... Dummy arguments
!--------------------------------------------------------
      integer, intent(in)     ::   ncol                    ! columns in chunk
      integer, intent(in)     ::   lchnk                   ! chunk id
      integer, intent(in)     ::   latndx(pcols)           ! chunk latitude indicies
      integer, intent(in)     ::   lonndx(pcols)           ! chunk longitude indicies
      real(r8), intent(in)    ::   tsurf(pcols)            ! surface temperature
      real(r8), intent(in)    ::   zen_angle(ncol)         ! zenith angle
      real(r8), intent(out)   ::   depvel(ncol,gas_pcnst)  ! deposition vel (cm/s)

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      integer :: i, ii, jj, m
      real(r8), dimension(ncol) :: vel

!--------------------------------------------------------
!       ... Initialize all deposition velocities to zero
!--------------------------------------------------------
      depvel(:,:) = 0._r8

!--------------------------------------------------------
!       ... Set deposition velocities
!--------------------------------------------------------
      do m = 1,gas_pcnst
         if( map(m) /= 0 ) then
	    do i = 1,ncol
	       ii = lonndx(i)
	       jj = latndx(i)
               depvel(i,m) = dvel_interp(ii,jj,map(m))
	    end do
         end if
      end do

!--------------------------------------------------------
!       ... Set no2 dvel
!--------------------------------------------------------
      if( map(id_no2) == 0 ) then
         depvel(:,id_no2) = (.6_r8*depvel(:,id_o3) + .055_r8*ocnfrac(:ncol,lchnk)) * .9_r8
      end if

!--------------------------------------------------------
!       ... Set hno3 dvel
!--------------------------------------------------------
       if( map(id_hno3) == 0 ) then
          depvel(:,id_hno3) = 2._r8*landfrac(:ncol,lchnk) + ocnfrac(:ncol,lchnk) + .05_r8 * icefrac(:ncol,lchnk)
       endif

!--------------------------------------------------------
!       ... Set h2o2 dvel
!--------------------------------------------------------
      if( map(id_h2o2) == 0 ) then
          depvel(:,id_h2o2) = .05_r8*icefrac(:ncol,lchnk) + ocnfrac(:ncol,lchnk) &
                            + landfrac(:ncol,lchnk) &
                              *max( 1._r8,1._r8/(.5_r8 + 1._r8/(6._r8*depvel(:,id_o3))) )
      end if

!--------------------------------------------------------
!       ... Set ch2o, ch3ooh, pooh, ch3coooh, & c2h5ooh etc ...
!--------------------------------------------------------
      if( map(id_ch2o) == 0 ) then
         depvel(:,id_ch2o) = depvel(:,id_o3)
      end if
      if( map(id_ch3ooh) == 0 ) then
         depvel(:,id_ch3ooh) = depvel(:,id_h2o2) * .5_r8
      end if

!--------------------------------------------------------
!       ... Set diurnal dependence for O3 dvel
!--------------------------------------------------------
      vel(:) = max( 1.,sqrt( (depvel(:,id_o3) - .2_r8)**3/.27_r8 + 4._r8*depvel(:,id_o3) + .67_r8 ) )
      where( abs( zen_angle(:) ) > pid2 )
         depvel(:,id_o3) = depvel(:,id_o3) / vel(:)
      elsewhere
         depvel(:,id_o3) = depvel(:,id_o3) * vel(:)
      endwhere

      end subroutine mkdvel

      subroutine drydep( vg, q, p, ts, flux, ncol )
!-----------------------------------------------------------------------
!       ... Compute surface flux due to dry deposition processes
!-----------------------------------------------------------------------

      use m_spc_id
      use ppgrid,    only : pcols, pver
      use chem_mods, only : gas_pcnst
      use physconst, only : rair

      implicit none

!-----------------------------------------------------------------------
!       ... Dummy args
!-----------------------------------------------------------------------
      integer, intent(in)  :: &
        ncol                        ! columns in chunk
      real(r8), intent(in) :: &
        vg(ncol,gas_pcnst), &       ! dry deposition velocity in cm/s
        q(pcols,pver,gas_pcnst), &  ! tracer mmr (kg tracer/kg moist air)
        p(pcols), &                 ! midpoint pressure in surface layer (Pa)
        ts(pcols)                   ! surface temperature (K)
      real(r8), intent(inout) :: &
        flux(pcols,gas_pcnst)       ! flux due to dry deposition and emission (kg/m^s/sec)

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      real(r8), parameter :: cm2m = 1.e-2_r8
      real(r8)    :: wrk(ncol)
      real(r8)    :: dflx(ncol)

!-----------------------------------------------------------------------
!       ... Note the factor 1.e-2 in the wrk array calculation is
!           to transform the incoming dep vel from cm/s to m/s
!-----------------------------------------------------------------------
      wrk(:)                = cm2m * p(:ncol) / (rair * ts(:ncol))
      dflx(:)               = wrk(:) * vg(:,id_no) * q(:ncol,pver,id_no)
      flux(:ncol,id_no)     = flux(:ncol,id_no) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_no2) * q(:ncol,pver,id_no2)
      flux(:ncol,id_no2)    = flux(:ncol,id_no2) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_hno3) * q(:ncol,pver,id_hno3)
      flux(:ncol,id_hno3)   = flux(:ncol,id_hno3) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_ho2no2) * q(:ncol,pver,id_ho2no2)
      flux(:ncol,id_ho2no2) = flux(:ncol,id_ho2no2) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_o3) * q(:ncol,pver,id_o3)
      flux(:ncol,id_o3)     = flux(:ncol,id_o3) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_h2o2) * q(:ncol,pver,id_h2o2)
      flux(:ncol,id_h2o2)   = flux(:ncol,id_h2o2) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_ch2o) * q(:ncol,pver,id_ch2o)
      flux(:ncol,id_ch2o)   = flux(:ncol,id_ch2o) - dflx(:)
      dflx(:)               = wrk(:) * vg(:,id_ch3ooh) * q(:ncol,pver,id_ch3ooh)
      flux(:ncol,id_ch3ooh) = flux(:ncol,id_ch3ooh) - dflx(:)
      dflx(:)               = wrk(:) * vg(:ncol,id_co) * q(:ncol,pver,id_co)
      flux(:ncol,id_co)     = flux(:ncol,id_co) - dflx(:)

      end subroutine drydep

      end module mo_drydep
