      module mo_imp_sol

      use shr_kind_mod, only : r8 => shr_kind_r8
      use chem_mods, only : clscnt4, gas_pcnst, clsmap

      implicit none

!-----------------------------------------------------------------------
! Newton-Raphson iteration limits
!-----------------------------------------------------------------------
      integer, parameter :: itermax = 11
      integer, parameter :: cut_limit = 5

      save

      real(r8) :: small
      real(r8) :: epsilon(clscnt4)
      logical :: factor(itermax)

      private
      public :: imp_slv_inti, imp_sol

      contains

      subroutine imp_slv_inti
!-----------------------------------------------------------------------
! ... Initialize the implict solver
!-----------------------------------------------------------------------

      use m_spc_id

      implicit none

!-----------------------------------------------------------------------
! ... Local variables
!-----------------------------------------------------------------------
      integer :: m
      real(r8) :: eps(gas_pcnst)

! small = 1.e6_r8 * tiny( small )
      small = 1.e-40_r8
      factor(:) = .true.
      eps(:) = .001_r8
      eps((/id_o3,id_no,id_no2,id_no3,id_hno3,id_ho2no2,id_n2o5,id_oh,id_ho2/)) = .0001_r8
      do m = 1,clscnt4
         epsilon(m) = eps(clsmap(m,4))
      end do

      end subroutine imp_slv_inti

      subroutine imp_sol( vmr, base_sol, reaction_rates, het_rates, extfrc, delt, &
                          ncol, lchnk )
!-----------------------------------------------------------------------
! ... imp_sol advances the volumetric mixing ratio
! forward one time step via the fully implicit euler scheme.
! this source is meant for small l1 cache machines such as
! the intel pentium and itanium cpus
!-----------------------------------------------------------------------

      use chem_mods, only : rxntot, extcnt, nzcnt, diag_map, permute, hetcnt, cls_rxt_cnt
      use mo_tracname, only : solsym
      use ppgrid, only : pver
      use pmgrid, only : iam
      use mo_lin_matrix, only : linmat
      use mo_nln_matrix, only : nlnmat
      use mo_lu_factor, only : lu_fac
      use mo_lu_solve, only : lu_slv
      use mo_prod_loss, only : imp_prod_loss
      use mo_indprd, only : indprd
      use time_manager, only : get_nstep

      implicit none

!-----------------------------------------------------------------------
! ... dummy args
!-----------------------------------------------------------------------
      integer, intent(in) :: ncol ! columns in chunck
      integer, intent(in) :: lchnk ! chunk id
      real(r8), intent(in) :: delt ! time step (s)
      real(r8), intent(in) :: reaction_rates(ncol,pver,max(1,rxntot)), & ! rxt rates (1/cm^3/s)
                                 extfrc(ncol,pver,max(1,extcnt)), & ! external in-situ forcing (1/cm^3/s)
                                 het_rates(ncol,pver,max(1,hetcnt)) ! washout rates (1/s)
      real(r8), intent(inout) :: base_sol(ncol,pver,gas_pcnst) ! species mixing ratios (vmr)
      real(r8), intent(inout) :: vmr(ncol,pver,gas_pcnst) ! species mixing ratios (vmr)

!-----------------------------------------------------------------------
! ... local variables
!-----------------------------------------------------------------------
      integer :: nr_iter, &
                   lev, &
                   i, &
                   j, &
                   k, l, &
                   m
      integer :: fail_cnt, cut_cnt, stp_con_cnt
      integer :: nstep
      real(r8) :: interval_done, dt, dti, wrk
      real(r8) :: max_delta(max(1,clscnt4))
      real(r8) :: sys_jac(max(1,nzcnt))
      real(r8) :: lin_jac(max(1,nzcnt))
      real(r8), dimension(max(1,clscnt4)) :: &
                   solution, &
                   forcing, &
                   iter_invariant, &
                   prod, &
                   loss
      real :: lrxt(max(1,rxntot))
      real :: lsol(max(1,gas_pcnst))
      real :: lhet(max(1,hetcnt))
      real, dimension(ncol,pver,max(1,clscnt4)) :: &
                   ind_prd
      logical :: convergence
      logical :: frc_mask, iter_conv
      logical :: converged(max(1,clscnt4))

!-----------------------------------------------------------------------
! ... class independent forcing
!-----------------------------------------------------------------------
      base_sol(:,:,:) = vmr(:,:,:)  

      if( cls_rxt_cnt(1,4) > 0 .or. extcnt > 0 ) then
         call indprd( 4, ind_prd, clscnt4, base_sol, extfrc, &
                      reaction_rates, ncol )
      else
         do m = 1,max(1,clscnt4)
            ind_prd(:,:,m) = 0._r8
         end do
      end if
level_loop : &
      do lev = 1,pver
column_loop : &
  do i = 1,ncol
!-----------------------------------------------------------------------
! ... transfer from base to local work arrays
!-----------------------------------------------------------------------
            do m = 1,rxntot
        lrxt(m) = reaction_rates(i,lev,m)
     end do
            if( hetcnt > 0 ) then
               do m = 1,hetcnt
           lhet(m) = het_rates(i,lev,m)
        end do
            end if
!-----------------------------------------------------------------------
! ... time step loop
!-----------------------------------------------------------------------
     dt = delt
     cut_cnt = 0
     fail_cnt = 0
     stp_con_cnt = 0
     interval_done = 0.
time_step_loop : &
     do
        dti = 1. / dt
!-----------------------------------------------------------------------
! ... transfer from base to local work arrays
!-----------------------------------------------------------------------
               do m = 1,gas_pcnst
           lsol(m) = base_sol(i,lev,m)
        end do
!-----------------------------------------------------------------------
! ... transfer from base to class array
!-----------------------------------------------------------------------
               do k = 1,clscnt4
                  j = clsmap(k,4)
                  m = permute(k,4)
                  solution(m) = lsol(j)
               end do
!-----------------------------------------------------------------------
! ... set the iteration invariant part of the function f(y)
!-----------------------------------------------------------------------
               if( cls_rxt_cnt(1,4) > 0 .or. extcnt > 0 ) then
                  do m = 1,clscnt4
                     iter_invariant(m) = dti * solution(m) + ind_prd(i,lev,m)
                  end do
               else
                  do m = 1,clscnt4
                     iter_invariant(m) = dti * solution(m)
                  end do
               end if
!-----------------------------------------------------------------------
! ... the linear component
!-----------------------------------------------------------------------
        if( cls_rxt_cnt(2,4) > 0 ) then
                  call linmat( lin_jac, lsol, lrxt, lhet )
               end if

!=======================================================================
! the newton-raphson iteration for f(y) = 0
!=======================================================================
iter_loop : &
               do nr_iter = 1,itermax
!-----------------------------------------------------------------------
! ... the non-linear component
!-----------------------------------------------------------------------
                  if( factor(nr_iter) ) then
                     call nlnmat( sys_jac, lsol, lrxt, lin_jac, dti )
!-----------------------------------------------------------------------
! ... factor the "system" matrix
!-----------------------------------------------------------------------
              call lu_fac( sys_jac )
                  end if
!-----------------------------------------------------------------------
! ... form f(y)
!-----------------------------------------------------------------------
                  call imp_prod_loss( prod, loss, lsol, lrxt, lhet )
           do m = 1,clscnt4
                     forcing(m) = solution(m)*dti - (iter_invariant(m) + prod(m) - loss(m))
           end do
!-----------------------------------------------------------------------
! ... solve for the mixing ratio at t(n+1)
!-----------------------------------------------------------------------
           call lu_slv( sys_jac, forcing )
           do m = 1,clscnt4
                     solution(m) = solution(m) + forcing(m)
           end do
!-----------------------------------------------------------------------
! ... convergence measures
!-----------------------------------------------------------------------
                  if( nr_iter > 1 ) then
              do k = 1,clscnt4
          m = permute(k,4)
          if( abs(solution(m)) > 1.e-20_r8 ) then
             max_delta(k) = abs( forcing(m)/solution(m) )
          else
             max_delta(k) = 0._r8
          end if
              end do
           end if
!-----------------------------------------------------------------------
! ... limit iterate
!-----------------------------------------------------------------------
                  where( solution(:) < 0._r8 )
              solution(:) = 0._r8
           endwhere
!-----------------------------------------------------------------------
! ... transfer latest solution back to work array
!-----------------------------------------------------------------------
                  do k = 1,clscnt4
                     j = clsmap(k,4)
                     m = permute(k,4)
                     lsol(j) = solution(m)
                  end do
!-----------------------------------------------------------------------
! ... check for convergence
!-----------------------------------------------------------------------
           if( nr_iter > 1 ) then
              do k = 1,clscnt4
          m = permute(k,4)
   frc_mask = abs( forcing(m) ) > small
   if( frc_mask ) then
      converged(k) = abs(forcing(m)) <= epsilon(k)*abs(solution(m))
   else
      converged(k) = .true.
   end if
              end do
              convergence = all( converged(:) )
              if( convergence ) then
                 exit
              end if
           end if
               end do iter_loop

!-----------------------------------------------------------------------
! ... check for newton-raphson convergence
!-----------------------------------------------------------------------
               if( .not. convergence ) then
!-----------------------------------------------------------------------
! ... non-convergence
!-----------------------------------------------------------------------
    fail_cnt = fail_cnt + 1
    nstep = get_nstep()
!wtw            write(*,'('' imp_sol: Time step '',1p,e21.13,'' failed to converge @ (lchnk,lev,col,nstep) = '',4i6)') &
!wtw               dt,lchnk,lev,i,nstep
    stp_con_cnt = 0
                  if( cut_cnt < cut_limit ) then
       cut_cnt = cut_cnt + 1
       if( cut_cnt < cut_limit ) then
          dt = .5_r8 * dt
       else
          dt = .1_r8 * dt
       end if
       cycle time_step_loop
    else
!wtw              write(*,'('' imp_sol: Failed to converge @ (lchnk,lev,col,nstep,dt,time) = '',4i6,1p,2e21.13)') &
!wtw           lchnk,lev,i,nstep,dt,interval_done+dt
              do m = 1,clscnt4
                 if( .not. converged(m) ) then
!wtw                    write(*,'(1x,a8,1x,1pe10.3)') solsym(clsmap(m,4)), max_delta(m)
                 end if
              end do
    end if
        end if
!-----------------------------------------------------------------------
! ... check for interval done
!-----------------------------------------------------------------------
        interval_done = interval_done + dt
        if( abs( delt - interval_done ) <= .0001_r8 ) then
    if( fail_cnt > 0 ) then
!wtw              write(*,*) 'imp_sol : @ (lchnk,lev,col) = ',lchnk,lev,i,' failed ',fail_cnt,' times'
    end if
    exit time_step_loop
        else
!-----------------------------------------------------------------------
! ... transfer latest solution back to base array
!-----------------------------------------------------------------------
    if( convergence ) then
       stp_con_cnt = stp_con_cnt + 1
    end if
                  do m = 1,gas_pcnst
                     base_sol(i,lev,m) = lsol(m)
                  end do
    if( stp_con_cnt >= 2 ) then
       dt = 2._r8*dt
       stp_con_cnt = 0
    end if
    dt = min( dt,delt-interval_done )
! write(*,'('' imp_sol: New time step '',1p,e21.13)') dt
               end if
            end do time_step_loop
!-----------------------------------------------------------------------
! ... Transfer latest solution back to base array
!-----------------------------------------------------------------------
            do k = 1,clscnt4
               j = clsmap(k,4)
               m = permute(k,4)
               base_sol(i,lev,j) = solution(m)
            end do
         end do column_loop
      end do level_loop

      end subroutine imp_sol

      end module mo_imp_sol
