
      module mo_setext

      use shr_kind_mod, only : r8 => shr_kind_r8

      private
      public :: setext

      contains

      subroutine setext( lonndx, latndx, extfrc, zint, cldtop, &
                         zmid, lchnk, tfld, o2mmr, ommr, &
                         pmid, mbar, rlats, calday, ncol, rlons )
!--------------------------------------------------------
!     ... For this latitude slice:
!         - Form the no production from lighting
!         - Form the no production from airplanes
!         - Form the co production from airplanes
!--------------------------------------------------------

      use ppgrid,       only : pver, pcols
      use mo_airplane,  only : pno, pco, air_altitude
      use chem_mods,    only : extcnt
      use mo_lightning, only : prod_no
      use mo_aurora,    only : aurora
      use history,      only : outfld

      implicit none

!--------------------------------------------------------
!     ... dummy arguments
!--------------------------------------------------------
      integer, intent(in)   ::   lchnk                       ! chunk id
      integer, intent(in)   ::   ncol                        ! columns in chunk
      integer, intent(in)   ::   lonndx(pcols)               ! long indicies in chunk
      integer, intent(in)   ::   latndx(pcols)               ! lat  indicies in chunk
      real(r8), intent(in)  ::   calday                      ! calendar day of year
      real(r8), intent(in)  ::   rlats(ncol)                 ! column latitudes (radians)
      real(r8), intent(in)  ::   rlons(ncol)                 ! column longitudes (radians)
      real(r8), intent(in)  ::   zint(ncol,pver+1)           ! interface geopot height ( km )
      real(r8), intent(in)  ::   zmid(ncol,pver)             ! midpoint geopot height ( km )
      real(r8), intent(in)  ::   pmid(pcols,pver)            ! midpoint pressure (Pa)
      real(r8), intent(in)  ::   tfld(pcols,pver)            ! midpoint temperature (K)
      real(r8), intent(in)  ::   mbar(ncol,pver)             ! mean molecular mass (g/mole)
      real(r8), intent(in)  ::   o2mmr(ncol,pver)            ! o2 concentration (kg/kg)
      real(r8), intent(in)  ::   ommr(ncol,pver)             ! o concentration (kg/kg)
      real(r8), intent(in)  ::   cldtop(ncol)                ! cloud top index
      real(r8), intent(out) ::   extfrc(ncol,pver,extcnt)    ! the "extraneous" forcing

!--------------------------------------------------------
!     ... local variables
!--------------------------------------------------------
      real(r8), parameter :: km2cm = 1.e5_r8
      real(r8), parameter :: z0    = 110._r8
      real(r8), parameter :: alpha = -.04_r8
      real(r8), parameter :: p0    = 1.e4_r8
      integer     :: i, ii, jj, k, k1, kk, n, cldind
      real(r8)    :: frac
      real(r8)    :: zlev_top, zlev_bot, zdel, zdeli
      real(r8)    :: ztab_top, ztab_bot
      real(r8)    :: ion_prod(ncol,pver,4)
      real(r8)    :: aircraft_no(ncol,pver)

!--------------------------------------------------------
!     ... set nox production from lighting
!         note: from ground to cloud top production is C shaped
!--------------------------------------------------------
      extfrc(:,:,:) = 0._r8
      do i = 1,ncol
         cldind = nint( cldtop(i) )
         if( cldind < pver .and. cldind > 0  ) then
            extfrc(i,cldind:pver,1) = prod_no(i,cldind:pver,lchnk)
         end if
      end do

      call outfld( 'NO_Lightning', extfrc(:,:,1), ncol, lchnk )

!---------------------------------------------------------------------
!     ... add the airplane emissions; must do vertical interpolation
!---------------------------------------------------------------------
      ztab_top = maxval( air_altitude )
      ztab_bot = minval( air_altitude )
      n        = size(air_altitude) - 1
!---------------------------------------------------------------------
!     ... temp fix to avoid the zint(pver+1) problem
!---------------------------------------------------------------------
      do k = 1,pver
         aircraft_no(:,k) = 0._r8
         do i = 1,ncol
            ii = lonndx(i)
            jj = latndx(i)
            zlev_top = zint(i,k)                 ! altitude at top of model level (km)
            zlev_bot = zint(i,k+1)               ! altitude at bottom of model level (km)
            zdel = (zlev_top - zlev_bot) * km2cm ! model level thickness (cm)
            zdeli = 1._r8/zdel
            if( zlev_bot <= ztab_top .and. zlev_top >= ztab_bot ) then
               do kk = 1,n
                  if( zlev_bot <= air_altitude(kk+1) .and. zlev_top >= air_altitude(kk) ) then
                     frac = (min( zlev_top, air_altitude(kk+1) ) - max( zlev_bot, air_altitude(kk) )) &
                            /(air_altitude(kk+1) - air_altitude(kk))
                     aircraft_no(i,k) = frac * pno(ii,jj,kk) * zdeli
                     extfrc(i,k,1) = extfrc(i,k,1) + aircraft_no(i,k)
                     extfrc(i,k,2) = extfrc(i,k,2) + frac * pco(ii,jj,kk) * zdeli
                  end if
               end do
            end if
         end do
      end do

      call outfld( 'NO_Aircraft', aircraft_no, ncol, lchnk )

!---------------------------------------------------------------------
!     ... set ion auroral production
!---------------------------------------------------------------------
      call aurora( tfld, o2mmr, ommr, mbar, rlats, &
                   extfrc(:,:,4), extfrc(:,:,3), extfrc(:,:,6), extfrc(:,:,5), pmid, &
                   lchnk, calday, ncol, rlons )
!---------------------------------------------------------------------
!     ... set n(2d) and n(4s) auroral production
!---------------------------------------------------------------------
      do k = 1,pver
         extfrc(:,k,7) = 1.57_r8*.6_r8*extfrc(:,k,6)
         extfrc(:,k,8) = 1.57_r8*.4_r8*extfrc(:,k,6)
      end do
!---------------------------------------------------------------------
!     ... set electron auroral production
!---------------------------------------------------------------------
      do k = 1,pver
         extfrc(:,k,extcnt) = extfrc(:,k,3) + extfrc(:,k,4) &
                            + extfrc(:,k,5) + extfrc(:,k,6)
      end do

      end subroutine setext

      end module mo_setext
