
#include <misc.h>

      module mo_solar_parms

      use shr_kind_mod, only : r8 => shr_kind_r8
      use abortutils,   only : endrun

      implicit none

      private
      public :: solar_parms_init
      public :: solar_parms_timestep_init
      public :: get_solar_parms
      public :: rebin

      save

      integer               :: ntimes
      integer               :: tim_ndx
      integer,  allocatable :: dates(:)
      real(r8)              :: dels
      real(r8), allocatable :: times(:)
      real(r8), allocatable :: f107(:)
      real(r8), allocatable :: f107a(:)
      real(r8), allocatable :: kp(:)
      real(r8), allocatable :: ap(:)

      contains

      subroutine solar_parms_init (solar_parms_file)
!---------------------------------------------------------------
!	... initialize solar parmaters
!---------------------------------------------------------------

      use pmgrid,         only : masterproc
      use ioFileMod
      use time_manager,   only : get_curr_date
      use error_messages, only : alloc_err
#ifdef SPMD
      use mpishorthand,   only : mpiint, mpir8, mpicom
#endif

      implicit none

      character(len=*), intent(in) :: solar_parms_file

!---------------------------------------------------------------
!	... local variables
!---------------------------------------------------------------
      integer  :: ncid
      integer  :: n
      integer  :: dimid
      integer  :: varid
      integer  :: astat
      integer  :: wrk_date
      integer  :: yr, mon, day, ncsec
      real(r8) :: wrk_time
      real(r8), allocatable :: bz(:)
      integer  :: ndx(1)
      character(len=256) :: locfn

#include <netcdf.inc>

!-----------------------------------------------------------------------
!	... readin the solar parms dataset
!-----------------------------------------------------------------------
masterproc_only : &
      if( masterproc ) then
         write (6,*) 'SOLAR_PARMS: getting file ', trim(solar_parms_file)
         call getfil(solar_parms_file,  locfn, 0)
         write (6,*) 'SOLAR_PARMS: opening file ', trim(locfn)
         call wrap_open (locfn, 0, ncid)
         call wrap_inq_dimid( ncid, 'time', dimid )
         call wrap_inq_dimlen( ncid, dimid, ntimes )
         allocate( dates(ntimes), times(ntimes),stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'solar_parms_init', 'dates,times', ntimes )
         end if
         call wrap_inq_varid( ncid, 'date', varid )
         call wrap_get_var_int( ncid, varid, dates )

         do n = 1,ntimes
            call days0( dates(n), 0, times(n) )
         end do
         call get_curr_date( yr, mon, day, ncsec )
         wrk_date = 10000*yr + 100*mon + day
         write(*,*) ' '
         write(*,*) '--------------------------------------------------'
         write(*,*) 'solar_parms_init: values for date = ',wrk_date
         call days0( wrk_date, 0, wrk_time )
         if( wrk_time < times(1) .or. wrk_time > times(ntimes) ) then
            write(*,*) 'solar_parms_init: initial time is out of range of solar parm times'
            call endrun
         end if
         do n = 2,ntimes
            if( wrk_time <= times(n) ) then
               exit
            end if
         end do
         tim_ndx = n - 1
         dels    = (wrk_time - times(tim_ndx))/(times(tim_ndx+1) - times(tim_ndx))
         write(*,*) 'solar_parms_init: tim_ndx, dels, times(tim_ndx:tim_ndx+1) = ', &
                    tim_ndx, dels, dates(tim_ndx:tim_ndx+1)
         write(*,*) '--------------------------------------------------'
         write(*,*) ' '
!---------------------------------------------------------------
!	... allocate and read solar parms
!---------------------------------------------------------------
         allocate( f107(ntimes), f107a(ntimes), &
                   kp(ntimes), ap(ntimes), stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'solar_parms_init', 'f107 ... ap', ntimes )
         end if
         call wrap_inq_varid( ncid, 'f107', varid )
         call wrap_get_var_realx( ncid, varid, f107 )
         call wrap_inq_varid( ncid, 'f107a', varid )
         call wrap_get_var_realx( ncid, varid, f107a )
         call wrap_inq_varid( ncid, 'kp', varid )
         call wrap_get_var_realx( ncid, varid, kp )
         call wrap_inq_varid( ncid, 'ap', varid )
         call wrap_get_var_realx( ncid, varid, ap )

         call wrap_close( ncid )

      end if masterproc_only

#ifdef SPMD
      call mpibcast( ntimes, 1, mpiint, 0, mpicom )
      call mpibcast( tim_ndx, 1, mpiint, 0, mpicom )
      call mpibcast( dels, 1, mpir8, 0, mpicom )
      if( .not. masterproc ) then
         allocate( dates(ntimes), times(ntimes),stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'solar_parms_init', 'dates,times', ntimes )
         end if
         allocate( f107(ntimes), f107a(ntimes), &
                   kp(ntimes), ap(ntimes), stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'solar_parms_init', 'f107 ... ap', ntimes )
         end if
      end if
      call mpibcast( dates, ntimes, mpiint, 0, mpicom )
      call mpibcast( times, ntimes, mpir8, 0, mpicom )
      call mpibcast( f107, ntimes, mpir8, 0, mpicom )
      call mpibcast( f107a, ntimes, mpir8, 0, mpicom )
      call mpibcast( kp, ntimes, mpir8, 0, mpicom )
      call mpibcast( ap, ntimes, mpir8, 0, mpicom )
#endif

      end subroutine solar_parms_init

      subroutine solar_parms_timestep_init
!---------------------------------------------------------------
!	... set solar parameters timing
!---------------------------------------------------------------

      use time_manager,   only : get_curr_date, is_end_curr_day


      implicit none

!---------------------------------------------------------------
!	... local variables
!---------------------------------------------------------------
      integer  :: n
      integer  :: wrk_date
      integer  :: yr, mon, day, ncsec
      real(r8) :: wrk_time

      if( is_end_curr_day() ) then
         call get_curr_date( yr, mon, day, ncsec )
         wrk_date = 10000*yr + 100*mon + day
         write(*,*) 'solar_parms_timestep_init: values for date = ',wrk_date
         call days0( wrk_date, 0, wrk_time )
         if( wrk_time < times(1) .or. wrk_time > times(ntimes) ) then
            write(*,*) 'solar_parms_timestep_init: time is out of range of solar parm times'
            call endrun
         end if
         do n = 2,ntimes
            if( wrk_time <= times(n) ) then
               exit
            end if
         end do
         tim_ndx = n - 1
         dels    = (wrk_time - times(tim_ndx))/(times(tim_ndx+1) - times(tim_ndx))
      end if

      
    end subroutine solar_parms_timestep_init

    subroutine get_solar_parms( f107_s, f107a_s, ap_s, kp_s, hp_s )
!---------------------------------------------------------------
!	... set,retrieve solar parmaters
!---------------------------------------------------------------

      implicit none

!---------------------------------------------------------------
!	... dummy arguments
!---------------------------------------------------------------
      real(r8), optional, intent(out) :: f107_s                   ! solar euv factor
      real(r8), optional, intent(out) :: f107a_s                  ! averaged solar euv factor
      real(r8), optional, intent(out) :: ap_s                     ! solar mag factor
      real(r8), optional, intent(out) :: kp_s                     ! solar mag factor
      real(r8), optional, intent(out) :: hp_s                     ! hemispheric power

!---------------------------------------------------------------
!	... local variables
!---------------------------------------------------------------
      integer  :: tnp
      real(r8) :: wkp                                             ! wrk solar mag factor

      tnp = tim_ndx + 1
      if( present( f107_s ) ) then
         f107_s  =  f107(tim_ndx) + dels*(f107(tnp) - f107(tim_ndx))
      end if
      if( present( f107a_s ) ) then
         f107a_s  =  f107a(tim_ndx) + dels*(f107a(tnp) - f107a(tim_ndx))
      end if
      if( present( kp_s ) ) then
         kp_s  =  kp(tim_ndx) + dels*(kp(tnp) - kp(tim_ndx))
      end if
      if( present( ap_s ) ) then
         ap_s  =  ap(tim_ndx) + dels*(ap(tnp) - ap(tim_ndx))
      end if
      if( present( hp_s ) ) then
         wkp  =  kp(tim_ndx) + dels*(kp(tnp) - kp(tim_ndx))
	 hp_s = max( 3._r8,-2.78_r8 + 9.39_r8*wkp )
      end if

      end subroutine get_solar_parms

      subroutine rebin( nsrc, ntrg, src_x, trg_x, src, trg )
!---------------------------------------------------------------
!	... rebin src to trg
!---------------------------------------------------------------

      implicit none

!---------------------------------------------------------------
!	... dummy arguments
!---------------------------------------------------------------
      integer, intent(in)   :: nsrc                  ! dimension source array
      integer, intent(in)   :: ntrg                  ! dimension target array
      real(r8), intent(in)  :: src_x(nsrc+1)         ! source coordinates
      real(r8), intent(in)  :: trg_x(ntrg+1)         ! target coordinates
      real(r8), intent(in)  :: src(nsrc)             ! source array
      real(r8), intent(out) :: trg(ntrg)             ! target array

!---------------------------------------------------------------
!	... local variables
!---------------------------------------------------------------
      integer  :: i, l
      integer  :: si, si1
      integer  :: sil, siu
      real(r8) :: y
      real(r8) :: sl, su
      real(r8) :: tl, tu

!---------------------------------------------------------------
!	... check interval overlap
!---------------------------------------------------------------
      if( trg_x(1) < src_x(1) .or. trg_x(ntrg+1) > src_x(nsrc+1) ) then
         write(*,*) 'rebin: target grid is outside source grid'
         write(*,*) '       target grid from ',trg_x(1),' to ',trg_x(ntrg+1)
         write(*,*) '       source grid from ',src_x(1),' to ',src_x(nsrc+1)
         call endrun
      end if

      !      write(*,*) ' '
      !      write(*,*) '------------------------------------------------------'
      !      write(*,*) 'rebin: diagnostics'
      !      write(*,*) 'rebin: nsrc, ntrg = ',nsrc,ntrg
      !      write(*,*) 'rebin: src_x'
      !      write(*,'(1p,5g15.7)') src_x(:)
      !      write(*,*) 'rebin: trg_x'
      !      write(*,'(1p,5g15.7)') trg_x(:)
      !      write(*,*) '------------------------------------------------------'
      !      write(*,*) ' '
      !      write(*,*) ' '
      !      write(*,*) 'rebin: individual bins'
      do i = 1,ntrg
        tl     = trg_x(i)
        do sil = 1,nsrc+1
           if( tl <= src_x(sil) ) then
              exit
           end if
        end do
        tu     = trg_x(i+1)
        do siu = 1,nsrc+1
           if( tu <= src_x(siu) ) then
              exit
           end if
        end do
        y   = 0._r8
        sil = max( sil,2 )
        siu = min( siu,nsrc+1 )
        do si = sil,siu
           si1 = si - 1
           sl  = max( tl,src_x(si1) )
           su  = min( tu,src_x(si) )
           y   = y + (su - sl)*src(si1)
        end do
        trg(i) = y/(trg_x(i+1) - trg_x(i))
      end do

      end subroutine rebin

      subroutine days0( ncdate, ncsec, days )
!----------------------------------------------------------------------- 
! Purpose: Convert date and seconds of day to floating point days since
!          00/01/01
! 
! Method: Use table of days per month to do conversion
! 
! Author: CCM Core Group
!-----------------------------------------------------------------------


      implicit none

!-----------------------------------------------------------------------
!	... dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in)   :: ncdate      ! Current date as yymmdd or yyyymmdd
      integer, intent(in)   :: ncsec       ! Seconds of day for current date
      real(r8), intent(out) :: days        ! Day of year

!-----------------------------------------------------------------------
!	... local variables
!-----------------------------------------------------------------------
      integer  :: year        ! year
      integer  :: mnth        ! Month number
      integer  :: mday        ! Day number of month
      integer  :: jdcon(12) & ! Starting day number for each month
                       = (/ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /)
      real(r8) :: doy

!-----------------------------------------------------------------------
! Decode month and day
!-----------------------------------------------------------------------
      year = ncdate/10000
      mnth = mod( ncdate,10000 )/100
      if( mnth < 1 .or. mnth > 12) then
         write(*,*) 'days0: Bad month index=', mnth
         call endrun
      end if
      mday = mod(ncdate,100)
      doy  = jdcon(mnth) + mday + ncsec/86400._r8

      if( doy < 1._r8 .or. doy > 366._r8) then
         write(*,*) 'days0: bad day of year = ',doy
         call endrun
      end if

      days = 365._r8*year + doy

      end subroutine days0

      end module mo_solar_parms
