
#include <misc.h>

      module mo_srf_emissions
!---------------------------------------------------------------
! 	... Set surface emissions
!---------------------------------------------------------------

      use shr_kind_mod, only : r8 => shr_kind_r8
      use pmgrid,        only : plon, plat
      use abortutils,    only : endrun

      implicit none

      private
      public  :: srf_emissions_inti, set_srf_emissions, set_srf_emissions_time 

      save

      integer :: last, next
      real(r8)    :: sf(plat)
      real(r8)    :: factor
      real(r8)    :: dels
      real(r8)    :: days(12)
      real(r8)    :: noxem(plon,plat,12)  = 0._r8
      real(r8)    :: ch2oem(plon,plat,12) = 0._r8
      real(r8)    :: coem(plon,plat,12)   = 0._r8

      contains

      subroutine srf_emissions_inti(nox_emis_file, co_emis_file, ch2o_emis_file)
!-----------------------------------------------------------------------
! 	... Read the primary emission flux datasets 
!-----------------------------------------------------------------------

      use m_spc_id
      use chem_mods,     only : adv_mass
      use dynconst,      only : rearth
      use ioFileMod,     only : getfil
      use pmgrid,        only : masterproc
      use commap,        only : latwts => w
      use time_manager,  only : get_calday
      use shr_const_mod, only : pi => shr_const_pi
#ifdef SPMD
      use mpishorthand,  only : mpicom, mpir8
#endif

#include <netcdf.inc>

      character(len=*), intent(in) :: nox_emis_file
      character(len=*), intent(in) :: co_emis_file
      character(len=*), intent(in) :: ch2o_emis_file

!-----------------------------------------------------------------------
! 	... Local variables
!-----------------------------------------------------------------------
      real(r8), parameter :: amufac = 1.65979e-23_r8          ! 1.e4* kg / amu
      integer     :: j, ncid, m                               ! Indices
      integer     :: dates(12) = (/ 116, 214, 316, 415,  516,  615, 716, 816, 915, 1016, 1115, 1216 /)
      real(r8)    :: mw
      real(r8)    :: seq
      real(r8)    :: d2r
      character(len=10) :: species
      character(len=256) :: locfn

Masterproc_only : &
      if( masterproc ) then
         d2r = pi/180._r8
!-----------------------------------------------------------------------
! 	... Diagnostics setup
!-----------------------------------------------------------------------
         seq = 2._r8*pi*1.e4_r8*rearth**2/real(plon)
         sf(:plat) = seq*latwts(:plat)
         factor = 86400._r8 * 365._r8 &      ! sec / year
                / 6.022e23_r8 &              ! molec / mole 
                * 1.e-12_r8                  ! Tg / g

!-----------------------------------------------------------------------
! 	... NO
!-----------------------------------------------------------------------
         species  = 'NO'
         mw       = adv_mass(id_n)             ! g N / mole
         call getfil (nox_emis_file, locfn, 0)
         call wrap_open (trim(locfn), NF_NOWRITE, ncid)
         call read_emissions( species, mw, 'TgN/y', ncid, noxem, d2r )

!-----------------------------------------------------------------------
! 	... CO
!-----------------------------------------------------------------------
         species  = 'CO'
         mw       = adv_mass(id_co)               ! g / mole
         call getfil (co_emis_file, locfn, 0)
         call wrap_open (trim(locfn), NF_NOWRITE, ncid)
         call read_emissions( species, mw, 'Tg/y', ncid, coem, d2r )

!-----------------------------------------------------------------------
! 	... CH2O
!-----------------------------------------------------------------------
         species  = 'CH2O'
         mw       = adv_mass(id_ch2o)               ! g / mole
         call getfil (ch2o_emis_file, locfn, 0)
         call wrap_open (trim(locfn), NF_NOWRITE, ncid)
         call read_emissions( species, mw, 'Tg/y', ncid, ch2oem, d2r )
      end if Masterproc_only

#ifdef SPMD
!-----------------------------------------------------------------------
! 	... Wait for masterproc to read surface emissions
!-----------------------------------------------------------------------
      call mpibarrier( mpicom )
      m = plon*plat*12
      call mpibcast( noxem, m, mpir8, 0, mpicom )
      call mpibcast( ch2oem, m, mpir8, 0, mpicom )
      call mpibcast( coem, m, mpir8, 0, mpicom )
#endif

!--------------------------------------------------------------------------
! 	... Transform from molecules/cm^2/s to kg/m^2/s 
!--------------------------------------------------------------------------
      noxem(:,:,:)  = amufac * adv_mass(id_no) * noxem(:,:,:)
      ch2oem(:,:,:) = amufac * adv_mass(id_ch2o) * ch2oem(:,:,:)
      coem(:,:,:)   = amufac * adv_mass(id_co) * coem(:,:,:)

!--------------------------------------------------------------------------
!	... Initialize the monthly day of year times
!--------------------------------------------------------------------------
      do m = 1,12
         days(m) = get_calday( dates(m), 0 )
      end do

      end subroutine srf_emissions_inti

      subroutine set_srf_emissions_time( calday )
!--------------------------------------------------------------------
!	... Check and set time interpolation indicies
!--------------------------------------------------------------------

      implicit none

!--------------------------------------------------------------------
!	... Dummy args
!--------------------------------------------------------------------
      real(r8), intent(in)    :: calday

!--------------------------------------------------------------------
!	... Local variables
!--------------------------------------------------------------------
      real(r8), parameter :: dayspy = 365._r8
      integer  ::  m, upper
      real(r8) ::  numer, denom

!--------------------------------------------------------------------
!	... Setup the time interpolation
!--------------------------------------------------------------------
      if( calday < days(1) ) then
	 next = 1
	 last = 12
      else
	 if( days(12) < dayspy ) then
	    upper = 12
	 else
	    upper = 11
	 end if
         do m = upper,1,-1
	    if( calday >= days(m) ) then
	       exit
	    end if
         end do
	 last = m
	 next = mod( m,12 ) + 1
      end if
      numer = calday - days(last)
      denom = days(next) - days(last)
      if( numer < 0._r8 ) then
	    numer = dayspy + numer
      end if
      if( denom < 0._r8 ) then
	    denom = dayspy + denom
      end if
      dels = max( min( 1._r8,numer/denom ),0._r8 )

      end subroutine set_srf_emissions_time

      subroutine read_emissions( species, mw, units, ncid, emis, d2r )
!-----------------------------------------------------------------------
!       ... Read surface emissions from NetCDF file
!-----------------------------------------------------------------------

      use phys_grid,     only : clat_p, clon_p
      use mo_regrider,   only : regrid_inti, regrid_2d, regrid_lat_limits, regrid_diagnostics

      implicit none

!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      character(len=*), intent(in) :: species, units                 ! species and units names
      integer, intent(in)  :: ncid                                   ! netcdf file id
      real(r8), intent(in) :: mw                                     ! species moleuclar weight (amu)
      real(r8), intent(in) :: d2r                                    ! degrees to radians
      real(r8), dimension(plon,plat,12), intent(out) :: emis         ! surface emission (kg/m^2/s)

#include <netcdf.inc>

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      integer :: j, jl, ju, m               ! Indices
      integer :: jlim(2)
      integer :: ierr
      integer :: gndx
      integer :: vid, nlat, nlon, nmonth, nvars, ndims, attlen
      integer :: dimid_lat, dimid_lon, dimid_month
      integer, allocatable       :: dimid(:)
      real(r8)                   :: total, scale_factor
      real(r8)                   :: lam(plon)
      real(r8)                   :: tmp2d(plon,plat)
      real(r8), allocatable      :: lat(:), lon(:)
      real(r8), allocatable      :: wrk(:,:,:)
      character(len=NF_MAX_NAME) :: varname
      character(len=100)         :: var_longname


!-----------------------------------------------------------------------
!       ... Get grid dimensions from file
!-----------------------------------------------------------------------
      call wrap_inq_dimid( ncid, 'lat', dimid_lat )
      call wrap_inq_dimlen( ncid, dimid_lat, nlat )
      allocate( lat(nlat), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'READ_SFLX: lat allocation error = ',ierr
         call endrun
      end if
      call wrap_inq_varid( ncid, 'lat', vid )
      call wrap_get_var_realx( ncid, vid, lat )
      lat(:nlat) = lat(:nlat) * d2r
 
      call wrap_inq_dimid( ncid, 'lon', dimid_lon )
      call wrap_inq_dimlen( ncid, dimid_lon, nlon )
      allocate( lon(nlon), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'READ_SFLX: lon allocation error = ',ierr
         call endrun
      end if
      call wrap_inq_varid( ncid, 'lon', vid )
      call wrap_get_var_realx( ncid, vid, lon )
      lon(:nlon) = lon(:nlon) * d2r
 
      call wrap_inq_dimid( ncid, 'month', dimid_month )
      call wrap_inq_dimlen( ncid, dimid_month, nmonth )
      if( nmonth /= 12 ) then
         write(*,*) 'READ_SFLX: Error, number of months = ',nmonth,' for species ' // trim(species)
         call endrun
      end if

      ierr = nf_inq_nvars( ncid, nvars )
      if( ierr /= NF_NOERR ) then
         write(*,*) 'READ_SFLX: Failed to get number of variables for species ' // trim(species)
         call endrun
      end if

!-----------------------------------------------------------------------
!       ... Set up regridding
!-----------------------------------------------------------------------
      lam(:) = clon_p(:,1)
      gndx = regrid_inti( nlat, plat, &
                          nlon, plon, &
                          lon,  lam, &
                          lat,  clat_p, &
                          0, &
                          do_lons=.true.,do_lats=.true. )
      deallocate( lat, lon, stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'READ_SFLX: Failed to deallocate lat,lon; ierr = ',ierr
         call endrun
      end if
      jl   = 1
      ju   = plat
      if( gndx /= 0 ) then
         jlim = regrid_lat_limits( gndx )
      else
         jlim = (/ jl,ju /)
      end if
      write(*,'(1x,''READ_SFLX: gndx='',I2,'', grid limits = '',2i4,'', jl,ju='',2i4)') gndx,jlim,jl,ju
      allocate( wrk(nlon,nlat,nmonth), stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'READ_SFLX: wrk allocation error = ',ierr
         call endrun
      end if

      if( gndx > 0 ) then
	 write(*,*) 'read_emissions : regrid diagnostic'
         call regrid_diagnostics( gndx )
      end if

!-----------------------------------------------------------------------
!       ... Set total emissions to zero
!-----------------------------------------------------------------------
      emis(:,:,:) = 0.

      write(*,*) 'Annual average ',trim(species),' emissions ',trim(units)
!-----------------------------------------------------------------------
!       ... Loop over file variables
!-----------------------------------------------------------------------
Variables_loop : &
      do vid = 1,nvars
         ierr = nf_inq_varname( ncid, vid, varname )
         if( ierr /= NF_NOERR ) then
            write(*,*) 'READ_SFLX: Failed to get name of variable # ', vid, &
                ', species=' // trim(species)
            call endrun
         end if
         ierr = nf_inq_varndims( ncid, vid, ndims )
         if( ierr /= 0 ) then
            write(*,*) 'READ_SFLX: Failed to get number of dimensions for ' // &
                            'variable ' // trim(varname) // ', species=' // trim(species)
            call endrun
         end if
         if( ndims < 3 ) then
            cycle
         else if( ndims > 3 ) then
            write(*,*) 'READ_SFLX: Skipping variable ', trim(varname),', ndims = ',ndims, &
                       ' , species=',trim(species)
            cycle
         end if
!-----------------------------------------------------------------------
!       ... Check order of dimensions. Must be (lon, lat, month).
!-----------------------------------------------------------------------
         allocate( dimid(ndims), stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'READ_SFLX: dimid allocation error = ',ierr
            call endrun
         end if
         ierr = nf_inq_vardimid( ncid, vid, dimid )
         if( ierr /= NF_NOERR ) then
            write(*,*) 'READ_SFLX: Failed to get dimension IDs for variable ' // &
                            trim(varname)  // ', species=' // trim(species)
            call endrun
         end if
         if( dimid(1) /= dimid_lon .or. dimid(2) /= dimid_lat .or. dimid(3) /= dimid_month ) then
            write(*,*) 'READ_SFLX: Dimensions in wrong order for variable ',trim(varname)
            write(*,*) '...      Expecting (lon, lat, month)'
            call endrun
         end if
         deallocate( dimid,stat=ierr )
         if( ierr /= 0 ) then
            write(*,*) 'READ_SFLX: Failed to deallocate dimid, ierr = ',ierr
            call endrun
         end if
!-----------------------------------------------------------------------
!       ... Check for long_name of variable
!-----------------------------------------------------------------------
         var_longname = ' '
         ierr = nf_get_att_text( ncid, vid, 'long_name', var_longname )
         if( ierr /= NF_NOERR ) then
            var_longname = trim( varname )
         end if
!-----------------------------------------------------------------------
!       ... Read data for this variable
!-----------------------------------------------------------------------
         ierr = nf_get_var_double( ncid, vid, wrk )
         if( ierr /= NF_NOERR ) then
            write(*,*) 'READ_SFLX: Failed to read variable ' // trim( varname ) // &
                            ', species=' // trim(species)
            call endrun
         end if
!-----------------------------------------------------------------------
!       ... Check for scale_factor
!-----------------------------------------------------------------------
         ierr = nf_get_att_double( ncid, vid, 'scale_factor', scale_factor )
         if( ierr /= NF_NOERR  ) then
            scale_factor = 1._r8
         end if

         total = 0._r8
         do m = 1,12
            call regrid_2d( wrk(:,:,m), tmp2d(:,:), gndx, jl, ju, &
                            do_poles=.true., scaling=scale_factor )
            do j = 1,plat
               emis(:,j,m) = emis(:,j,m) + tmp2d(:,j)
            end do
         end do
      end do Variables_loop
!-----------------------------------------------------------------------
!       ... Close NetCDF file
!-----------------------------------------------------------------------
      call wrap_close( ncid )

      deallocate( wrk,stat=ierr )
      if( ierr /= 0 ) then
         write(*,*) 'READ_SFLX: Failed to deallocate wrk, ierr = ',ierr
         call endrun
      end if

      end subroutine read_emissions

      subroutine set_srf_emissions( latndx, lonndx, cflx, ncol )
!--------------------------------------------------------
!	... Form the surface fluxes for this
!	    latitude slice
!--------------------------------------------------------

      use m_spc_id
      use ppgrid,       only : pcols
      use chem_mods,    only : gas_pcnst

      implicit none

!--------------------------------------------------------
!	... Dummy arguments
!--------------------------------------------------------
      integer, intent(in)     ::   ncol                        ! columns in chunk
      integer, intent(in)     ::   latndx(pcols)               ! chunk latitude indicies
      integer, intent(in)     ::   lonndx(pcols)               ! chunk longitude indicies
      real(r8), intent(inout) ::   cflx(pcols,gas_pcnst)       ! surface emissions ( kg/m^2/s )

!--------------------------------------------------------
!	... Local variables
!--------------------------------------------------------
      integer :: i, ii, jj, m

!--------------------------------------------------------
!	... Initialize
!--------------------------------------------------------
      do m = 1,gas_pcnst-1
         cflx(:ncol,m) = 0._r8
      end do

!--------------------------------------------------------
!	... Set non-zero emissions
!--------------------------------------------------------
      do i = 1,ncol
	 ii = lonndx(i)
	 jj = latndx(i)
         cflx(i,id_no)   = noxem(ii,jj,last) + dels * (noxem(ii,jj,next) - noxem(ii,jj,last))
         cflx(i,id_ch2o) = ch2oem(ii,jj,last) + dels * (ch2oem(ii,jj,next) - ch2oem(ii,jj,last))
         cflx(i,id_co)   = coem(ii,jj,last) + dels * (coem(ii,jj,next) - coem(ii,jj,last))
      end do

      end subroutine set_srf_emissions

      end module mo_srf_emissions
