
      module mo_wetdep

      private
      public  :: wetdep

      contains

      subroutine wetdep( het_rates, pmid, zmid, phis, &
			 tfld, cmfdqr, nrain, nevapr, delt, &
			 xhnm, vmr, ncol )
!-----------------------------------------------------------------------      
!       ... Compute rainout loss rates
!-----------------------------------------------------------------------      

      use shr_kind_mod, only : r8 => shr_kind_r8
      use m_spc_id,     only : id_hno3, id_h2o2
      use physconst,    only : rga
      use chem_mods,    only : hetcnt, gas_pcnst
      use ppgrid,       only : pver, pcols

      implicit none
!-----------------------------------------------------------------------      
!       ... Dummy arguments
!-----------------------------------------------------------------------      
      integer, intent(in)   ::    ncol                         ! columns in chunk
      real(r8), intent(in)  ::    delt                         ! time step ( s )
      real(r8), intent(in)  ::    pmid(pcols,pver)             ! midpoint pressure (Pa)
      real(r8), intent(in)  ::    cmfdqr(ncol,pver)            ! dq/dt for convection
      real(r8), intent(in)  ::    nrain(ncol,pver)             ! stratoform precip
      real(r8), intent(in)  ::    nevapr(ncol,pver)            ! evaporation
      real(r8), intent(in)  ::    vmr(ncol,pver,gas_pcnst)     ! xported species ( vmr )
      real(r8), intent(in)  ::    zmid(ncol,pver)              ! midpoint geopot (km)
      real(r8), intent(in)  ::    phis(pcols)                  ! surf geopot
      real(r8), intent(in)  ::    tfld(pcols,pver)             ! temperature
      real(r8), intent(in)  ::    xhnm(ncol,pver)              ! total atms density ( /cm**3)
      real(r8), intent(out) ::    het_rates(ncol,pver,hetcnt)  ! rainout loss rates (1/s)

!-----------------------------------------------------------------------      
!       ... Local variables
!-----------------------------------------------------------------------      
      real(r8), parameter ::  boltz = 1.38044e-16_r8     ! erg/K
      real(r8), parameter ::  avo   = 6.023e23_r8        ! 1/mol
      real(r8), parameter ::  xrm   = .189_r8            ! mean diameter of rain drop (cm)
      real(r8), parameter ::  xum   = 748._r8            ! mean rain drop terminal velocity (cm/s)
      real(r8), parameter ::  xvv   = 6.18e-2_r8         ! kinetic viscosity (cm^2/s)
      real(r8), parameter ::  xdg   = .112_r8            ! mass transport coefficient (cm/s)
      real(r8), parameter ::  t0    = 298._r8            ! reference temperature (K)
      real(r8), parameter ::  xph0  = 1.e-5_r8           ! Cloud [H+]
      real(r8), parameter ::  satf_hno3  = .016_r8       ! saturation factor for HNO3 in clouds 
      real(r8), parameter ::  satf_h2o2  = .016_r8       ! saturation factor for HNO3 in clouds 
      real(r8), parameter ::  satf_ch2o  = .1_r8         ! saturation factor for HNO3 in clouds 
      real(r8), parameter ::  const0   = boltz * 1.e-6_r8! (atmospheres/deg K/cm^3)
      real(r8), parameter ::  hno3_diss = 15.4_r8        ! hno3 dissociation constant
      real(r8), parameter ::  geo_fac  = 6._r8           ! geometry factor (surf area/volume = geo_fac/diameter)
      real(r8), parameter ::  mass_air = 29._r8          ! mass of background atmosphere (amu)
      real(r8), parameter ::  mass_h2o = 18._r8          ! mass of water vapor (amu)
      real(r8), parameter ::  h2o_mol  = 1.e3_r8/mass_h2o   ! (gm/mol water)
      real(r8), parameter ::  km2cm    = 1.e5_r8         ! convert km to cm
      real(r8), parameter ::  m2km     = 1.e-3_r8        ! convert m to km
      real(r8), parameter ::  cm3_2_m3 = 1.e-6_r8        ! convert cm^3 to m^3
      real(r8), parameter ::  m3_2_cm3 = 1.e6_r8         ! convert m^3 to cm^3
      real(r8), parameter ::  liter_per_gram = 1.e-3_r8
      real(r8), parameter ::  avo2  = avo * liter_per_gram * cm3_2_m3 ! (liter/gm/mol*(m/cm)^3)

      integer    ::      i, m, k, kk, ktop_all       ! indicies
      integer    ::      ktop(ncol)                  ! 100 mb level
      real(r8)   ::      xkgm                        ! mass flux on rain drop
      real(r8)   ::      all1, all2                  ! work variables
      real(r8)   ::      stay                        ! fraction of layer traversed by falling drop in timestep delt
      real(r8)   ::      xeqca1, xeqca2, xca1, xca2, xdtm
      real(r8)   ::      xxx1, xxx2, yhno3, yh2o2
      
      real(r8), dimension(ncol)  :: &
                         xk0, work1, work2, work3, zsurf
      real(r8), dimension(pver)  :: &
                         xgas1, xgas2
      real(r8), dimension(ncol,pver)  :: &
                         delz, &              ! layer depth about interfaces (cm)
                         xhno3, &             ! hno3 concentration (molecules/cm^3)
                         xh2o2, &             ! h2o2 concentration (molecules/cm^3)
                         xliq, &              ! liquid rain water content in a grid cell (gm/m^3)
                         rain                 ! conversion rate of water vapor into rain water (molecules/cm^3/s)
      real(r8), dimension(ncol,pver)  :: &
                         xhen_hno3, xhen_h2o2, xhen_ch2o, xhen_ch3ooh, xhen_ho2no2
      real(r8), dimension(ncol,pver)  :: precip

!-----------------------------------------------------------------
!        NOTE: The pmid array is in pascals and must be
!              mutiplied by 10 to yield dynes/cm**2.
!-----------------------------------------------------------------
!       ... Set wet deposition for
!           1. h2o2         2. hno3
!           3. ch2o         4. ch3ooh
!           5. ho2no2       6. clono2
!           7. brono2       8. hcl
!           9. n2o5        10. hocl
!          11. hobr        12. hbr
!-----------------------------------------------------------------

!-----------------------------------------------------------------
!	... The 2 and .6 multipliers are from a formula by Frossling (1938)
!-----------------------------------------------------------------
      xkgm = xdg/xrm * 2._r8 + xdg/xrm * .6_r8 * SQRT( xrm*xum/xvv ) * (xvv/xdg)**(1._r8/3._r8) 

!-----------------------------------------------------------------
!	... Find 100 mb level
!-----------------------------------------------------------------
      do i = 1,ncol
         do k = pver,1,-1
	    if( pmid(i,k) < 100.e2_r8 ) then
	       ktop(i) = k
	       exit
	    end if
	 end do
      end do
      ktop_all = minval( ktop(:) )

!     do k = ktop_all,pver
      do k = 1,pver
	 do i = 1,ncol
            precip(i,k) = cmfdqr(i,k) + nrain(i,k) - nevapr(i,k)
            rain(i,k)   = mass_air*precip(i,k)*xhnm(i,k) / mass_h2o
            xliq(i,k)   = precip(i,k) * delt * xhnm(i,k) / avo*mass_air * m3_2_cm3
            xhno3(i,k)  = vmr(i,k,id_hno3) * xhnm(i,k)
            xh2o2(i,k)  = vmr(i,k,id_h2o2) * xhnm(i,k)
         end do 
      end do 
   
      zsurf(:ncol) = m2km * phis(:ncol) * rga
      do k = ktop_all,pver-1
         delz(:,k) = abs( (zmid(:,k) - zmid(:,k+1))*km2cm ) 
      end do 
      delz(:,pver) = abs( (zmid(:,pver) - zsurf(:) )*km2cm ) 

!-----------------------------------------------------------------
!       ... PART 0b,  for temperature dependent of Henrys
!                     xxhe1 = henry con for hno3
!                     xxhe2 = henry con for h2o2
!lwh 10/00 -- Take Henry''s law constants from Brasseur et al. [1999],
!             Appendix J. For HNO3, also consider dissociation to
!             get effective Henry''s law constant; equilibrium
!             constant for dissociation from Brasseur et al. [1999],
!             Appendix K. Assume pH=5 (set as xph0 above).
!             Heff = H*K/[H+] for HNO3 (complete dissociation)
!             Heff = H for H2O2 (no dissociation)
!             Heff = H * (1 + K/[H+]) (in general)
!-----------------------------------------------------------------
      do k = ktop_all,pver
!-----------------------------------------------------------------
! 	... Effective Henry''s law constants:
!	    HNO3, H2O2, CH2O, CH3OOH, CH3COOOH (Brasseur et al., 1999)
!           HO2NO2 (MOZART-1)
!-----------------------------------------------------------------
	  do i = 1,ncol
             work1(i)           = (t0 - tfld(i,k))/(t0*tfld(i,k))
             xk0(i)             = 2.1e5_r8 *exp( 8700._r8*work1(i) )
             xhen_hno3(i,k)     = xk0(i) * (1._r8 + hno3_diss / xph0 )
             xhen_h2o2(i,k)     = 7.45e4_r8 * exp( 6620._r8 * work1(i) )
             xhen_ch2o(i,k)     = 6.3e3_r8 * exp( 6460._r8 * work1(i) )
             xhen_ch3ooh(i,k)   = 2.27e2_r8 * exp( 5610._r8 * work1(i) )
             xhen_ho2no2(i,k)   = 2.e4_r8
	  end do
      end do

!-----------------------------------------------------------------
!       ... Initialize all rates to zero
!-----------------------------------------------------------------
      do m = 1,hetcnt
	 do i = 1,ncol
	    do k = 1,pver
               het_rates(i,k,m) = 0._r8
	    end do
	 end do
      end do
!-----------------------------------------------------------------
!       ... PART 1, solve for high henry constant ( HNO3, H2O2)
!-----------------------------------------------------------------
Column_loop : &
      do i = 1,ncol
         xgas1(:) = xhno3(i,:)                     ! xgas will change during 
         xgas2(:) = xh2o2(i,:)                     ! different levels wash 
Level_loop : &
         do kk = ktop(i),pver
            stay = 1.
            if( rain(i,kk) /= 0._r8 ) then            ! finding rain cloud           
               all1 = 0._r8                           ! accumulation to justisfy saturation
               all2 = 0._r8
               stay = (zmid(i,kk)*km2cm)/(xum*delt)
               stay = MIN( stay,1. )
!-----------------------------------------------------------------
!       ... Calculate the saturation concentration eqca
!-----------------------------------------------------------------
               do k = kk,pver                      ! cal washout below cloud
                  xeqca1 =  xgas1(k) &
                            / (xliq(i,kk)*avo2 + 1._r8/(xhen_hno3(i,k)*const0*tfld(i,k))) &
                            *  xliq(i,kk)*avo2
                  xeqca2 =  xgas2(k) &
                            / (xliq(i,kk)*avo2 + 1._r8/(xhen_h2o2(i,k)*const0*tfld(i,k))) &
                            *  xliq(i,kk)*avo2
!-----------------------------------------------------------------
!       ... Calculate ca; inside cloud concentration in #/cm3(air)
!-----------------------------------------------------------------
                  xca1 = geo_fac*xkgm*xgas1(k)/(xrm*xum)*delz(i,k) * xliq(i,kk) * cm3_2_m3
                  xca2 = geo_fac*xkgm*xgas2(k)/(xrm*xum)*delz(i,k) * xliq(i,kk) * cm3_2_m3
!-----------------------------------------------------------------
!       ... If is not saturated
!               hno3(gas)_new = hno3(gas)_old - hno3(h2o)
!           otherwise
!               hno3(gas)_new = hno3(gas)_old
!-----------------------------------------------------------------
                   all1 = all1 + xca1
                   all2 = all2 + xca2
                   if( all1 < xeqca1 ) then
                      xgas1(k) = max( xgas1(k) - xca1,0._r8 )
                   end if
                   if( all2 < xeqca2 ) then
                      xgas2(k) = max( xgas2(k) - xca2,0._r8 )
                   end if
               end do
            end if
!-----------------------------------------------------------------
!       ... Calculate the lifetime of washout (second)
!             after all layers washout 
!             the concentration of hno3 is reduced 
!             then the lifetime xtt is calculated by
!
!                  xtt = (xhno3(ini) - xgas1(new))/(dt*xhno3(ini))
!                  where dt = passing time (s) in vertical
!                             path below the cloud
!                        dt = dz(cm)/um(cm/s)
!-----------------------------------------------------------------
            xdtm = delz(i,kk) / xum                     ! the traveling time in each dz
            xxx1 = xhno3(i,kk) - xgas1(kk)
            xxx2 = xh2o2(i,kk) - xgas2(kk)
            if( xxx1 /= 0._r8 ) then                       ! if no washout lifetime = 1.e29
               yhno3  = xhno3(i,kk)/xxx1 * xdtm    
            else
               yhno3  = 1.e29_r8
            end if
            if( xxx2 /= 0._r8 ) then                       ! if no washout lifetime = 1.e29
               yh2o2  = xh2o2(i,kk)/xxx2 * xdtm     
            else
               yh2o2  = 1.e29_r8
            end if 
            het_rates(i,kk,1) = max( 1._r8 / yh2o2,0._r8 ) * stay
            het_rates(i,kk,2) = max( 1._r8 / yhno3,0._r8 ) * stay
         end do Level_loop
      end do Column_loop

!-----------------------------------------------------------------
!       ... PART 2, in-cloud solve for low henry constant
!                   HNO3 and H2O2 have both in and under cloud
!-----------------------------------------------------------------
      do k = ktop_all,pver
	 do i = 1,ncol
            work1(i)          = avo2 * xliq(i,k)
            work2(i)          = const0 * tfld(i,k)
            het_rates(i,k,3)  = max( rain(i,k) / (h2o_mol*(work1(i) + 1._r8/(xhen_ch2o(i,k)*work2(i)))),0._r8 ) &    ! ch2o
                                * satf_ch2o
            het_rates(i,k,4)  = max( rain(i,k) / (h2o_mol*(work1(i) + 1._r8/(xhen_ch3ooh(i,k)*work2(i)))),0._r8 )    ! ch3ooh
            het_rates(i,k,5)  = max( rain(i,k) / (h2o_mol*(work1(i) + 1._r8/(xhen_ho2no2(i,k)*work2(i)))),0._r8 )    ! ho2no2
            work3(i)          = satf_h2o2 * max( rain(i,k) / (h2o_mol*(work1(i) + 1._r8/(xhen_h2o2(i,k)*work2(i)))),0._r8 )    
            het_rates(i,k,1)  = work3(i) + het_rates(i,k,1)                                                    ! h2o2
            work3(i)          = satf_hno3 * max( rain(i,k) / (h2o_mol*(work1(i) + 1._r8/(xhen_hno3(i,k)*work2(i)))),0._r8 )    
            het_rates(i,k,2)  = work3(i) + het_rates(i,k,2)                                                    ! hno3
	 end do
      end do

!-----------------------------------------------------------------
!	... Set rates for clono2, brono2, hcl, n2o5, hocl, hobr, and hbr = hno3
!-----------------------------------------------------------------
      do m = 6,hetcnt
	 do k = ktop_all,pver
            het_rates(:,k,m) = het_rates(:,k,2)
	 end do
      end do

!-----------------------------------------------------------------
!	... Set rates above tropopause = 0.
!-----------------------------------------------------------------
      do m = 1,hetcnt
         do i = 1,ncol
            do k = 1,ktop(i)
               het_rates(i,k,m) = 0._r8
            end do
         end do
      end do

      end subroutine wetdep

      end module mo_wetdep
