
      module neckel

      use shr_kind_mod, only : r8 => shr_kind_r8
      use abortutils,   only : endrun
      use pmgrid,       only : masterproc

      implicit none

      private
      public :: neckel_init, neckel_scale_etf
      public :: nw, we, etf

      save

      integer               :: nw
      real(r8), allocatable :: we(:)
      real(r8), allocatable :: base_etf(:)
      real(r8), allocatable :: etf(:)

      contains

      subroutine neckel_init (neckel_file)
!-------------------------------------------------------------
!	... initialize neckel module
!-------------------------------------------------------------

      use units,          only : getunit, freeunit
      use error_messages, only : alloc_err
      use ioFileMod,      only : getfil

      character(len=*), intent(in) :: neckel_file

      include 'netcdf.inc'

      integer               :: wn
      integer               :: unit
      integer               :: astat
      integer               :: istat
      real(r8)              :: wcen, flux
      real(r8), allocatable :: wc(:)
      character(len=256)    :: locfn

      unit     = getunit()
      call getfil (neckel_file, locfn, 0)
      open( unit, file = trim(locfn), status='UNKNOWN', iostat=istat )
      if( istat /= 0 ) then
         write(*,*) 'neckel_init: failed to open ',trim(locfn),'; error = ',istat
         call endrun
      end if
      read(unit,'(////////)',iostat=istat)
      if( istat /= 0 ) then
         write(*,*) 'euv_init: failed to read ',trim(locfn),'; error = ',istat
         call endrun
      end if
!-------------------------------------------------------------
!	... read lines to determine size of data arrays
!-------------------------------------------------------------
      nw = 0
      do
         read(unit,*,iostat=istat) wcen, flux
         if( istat == 0 ) then
            nw = nw + 1
         else
            exit
         end if
      end do
      rewind( unit )

!-------------------------------------------------------------
!	... now read each line for wave interval center (nm)
!           and etf (photons/cm^2/s/nm)
!-------------------------------------------------------------
      read(unit,'(////////)',iostat=istat)
      if( istat /= 0 ) then
         write(*,*) 'euv_init: failed to read ',trim(locfn),'; error = ',istat
         call endrun
      end if
      allocate( wc(nw), we(nw+1), etf(nw), base_etf(nw), stat=astat )
      if( astat /= 0 ) then
         call alloc_err( astat, 'neckel_init', 'wc ... etf', nw )
      end if
      do wn = 1,nw
         read(unit,*,iostat=istat) wc(wn), base_etf(wn)
         if( istat /= 0 ) then
            write(*,*) 'neckel_init: failed to read ',trim(locfn),'; error = ',istat
            call endrun
         end if
      end do

!-------------------------------------------------------------
!	... set wave interval edges
!-------------------------------------------------------------
      do wn = 1,nw
         if( wc(wn) <= 629.5_r8 ) then
            we(wn) = wc(wn) - .5_r8
         else if( wc(wn) <= 869._r8 ) then
            we(wn) = wc(wn) - 1._r8
         else
            we(wn) = wc(wn) - 2.5_r8
         end if
      end do
      we(nw+1)    = we(nw) + 5._r8

!-------------------------------------------------------------
!	... change etf from per nm to per wave interval
!-------------------------------------------------------------
      base_etf(:nw) = (we(2:nw+1) - we(:nw))*base_etf(:nw)

#ifdef NECKEL_DIAGS
      if( masterproc ) then
         write(*,*) ' '
         write(*,*) '========================================================='
         write(*,*) 'neckel_init: wn,we,wc,etf'
         do wn = 1,nw
            write(*,'(i4,1p,3g15.7)') wn,we(wn),wc(wn),base_etf(wn)
         end do
         write(*,'(i4,1p,g15.7)') nw+1,we(nw+1)
         write(*,*) '========================================================='
         write(*,*) ' '
      end if
#endif
      
      deallocate( wc )

      close( unit )
      call freeunit( unit )

      end subroutine neckel_init

      subroutine neckel_scale_etf( f107, f107a )
!-------------------------------------------------------------
!	... adjust neckel with woods 350 nm value
!-------------------------------------------------------------

!-------------------------------------------------------------
!	... dummy arguments
!-------------------------------------------------------------
      real(r8), intent(in) :: f107
      real(r8), intent(in) :: f107a

!-------------------------------------------------------------
!	... local variables
!-------------------------------------------------------------
      real(r8), parameter :: neckel_350 = 1.97811e14_r8
      real(r8), parameter :: f7min    = 71._r8
      real(r8), parameter :: f7max    = 212._r8
      real(r8), parameter :: f7deli   = 1._r8/(f7max - f7min)
      real(r8), parameter :: f7roti   = 1._r8/73._r8
      real(r8), parameter :: rt_min   = 2.0124e14_r8
!     real(r8), parameter :: rt_27day = .99807_r8
      real(r8), parameter :: rt_27day = 1._r8
!     real(r8), parameter :: rt_11yr  = 1.00251_r8
      real(r8), parameter :: rt_11yr  = 1.001_r8
      real(r8), parameter :: ref_27day = rt_min * (rt_27day - 1._r8)
      real(r8), parameter :: ref_11yr  = rt_min * (rt_11yr - 1._r8)

      integer  :: wn
      real(r8) :: neckel_scale
      real(r8) :: woods_etf

      woods_etf = rt_min + ref_27day * (f107 - f107a)*f7roti &
                         + ref_11yr * (f107a - f7min)*f7deli
      neckel_scale = woods_etf / neckel_350
      etf(:)       = neckel_scale * base_etf(:)

#ifdef NECKEL_DIAGS
      if( masterproc ) then
         write(*,*) '-----------------------------------------------'
         write(*,*) 'neckel_scale_etf: scale factor = ',neckel_scale
!        write(*,'(1p,5g15.7)') base_etf(:)
!        write(*,'(1p,5g15.7)') etf(:)
         write(*,*) '-----------------------------------------------'
      end if
#endif

      end subroutine neckel_scale_etf

      end module neckel
