#include <misc.h>
#include <params.h>

module solvar_interface

!
! This module provides an interface between WACCM heating rates and CAM calculations
!
! Author: F. Sassi
! Date: Nov 2004
!

  use shr_kind_mod,      only : r8 => shr_kind_r8
  use abortutils, only: endrun
  use mo_solar_parms,   only: rebin, get_solar_parms
  use woods,            only: nbins, woods_etf, we, woods_set_etf

#include <comsol.h>

  private
  save

  real(r8), parameter   :: a_coef = 1307.810791
  real(r8), parameter   :: b_coef = 0.00499603
  real(r8), parameter   :: c_coef = -0.00976559

  integer :: nspcam

  real(r8) :: f107_0                                ! reference F107 corresponding to the CAM TSI (1367)
  real(r8)              :: res350_0                 ! residual longwards of 350 nm at reference F107
  

  real(r8), allocatable, dimension(:) :: wavbne     ! CAM wave bins edges shortward of 350 nm
  integer nbin350                                   ! no. of bins in cam shortward of 350 nm

  real(r8), allocatable, dimension(:) :: woods_etf_ref  ! Woods&Rotmann ETF for the reference F10.7
  real(r8), allocatable, dimension(:) :: woodsi_ref     ! Woods&Rotmann ETF rebinned at the CAM sw bins
                                                        ! and calculated for the reference F10.7
  public solvar_init, solvar_cambins

  contains

!=======================================================================================

    subroutine solvar_long350_scale (fac350)


!
!      Bi-linear regression for the spectrum longwards of 350 nm
!

! In/out vars
      real(r8), intent(out) :: fac350                                    ! scaling factor for the CAM bins longwards of 350 nm
      

! local vars
      real(r8)              :: res350             ! residual longwards of 350nm at current F107
      real(r8)              :: f107
      real(r8)              :: f107a

!---------------------------------------------------------------------------------------

      call get_solar_parms ( f107_s = f107, f107a_s = f107a )

      res350 = a_coef + b_coef * f107a + c_coef * ( f107 - f107a)

      fac350 = res350 / res350_0

      return

    end subroutine solvar_long350_scale

!=======================================================================================

    subroutine solvar_cambins (sfac)


! In/out vars
      real(r8), intent(out) :: sfac(nspcam)       ! scaling factors for CAM heating

! local vars
      real(r8) :: woodsi(nbin350)   ! Woods&Rotmann ETF re-binned on the CAM bins
      real(r8) :: fac350                              ! scaling factor for the CAM bins longwards of 350 nm
      
      integer i
      integer ic
      integer ierr

      real(r8) :: f107

!----------------------------------------------------------------------------------------

      ! Re-bin the Woods&Rotmann spectrum for the current F107
      call rebin (nbins, nbin350, we, wavbne, woods_etf, woodsi)
      
      sfac(:) = 1.
      do i=1, nbin350
         sfac(i) = woodsi(i) / woodsi_ref(i)
!        if (dof107 == 1) write (6,*) 'bin, sfac = ', wavbne(i), wavbne(i+1), sfac(i)
      end do

      call solvar_long350_scale (fac350)
      do i=nbin350+1, nspcam
         sfac(i) = fac350
      end do

      return

    end subroutine solvar_cambins

!================================================================================================

    subroutine solvar_init (wavmin, wavmax, nspint)

!  initialization routine

! in/out vars
      integer, intent(in) :: nspint
      real(r8), intent(in) :: wavmin(nspint)
      real(r8), intent(in) :: wavmax(nspint)


! local vars
      integer i
      integer ic
      integer ierr
      
      real(r8), parameter :: refmin = 57.3624
      real(r8), parameter :: ref11yr1 = -0.114690
      real(r8), parameter :: ref11yr2 =  0.00161535

!------------------------------------------------------------------------------------------------

      nspcam = nspint

      ! Evaluate reference F107
      f107_0 = ( scon*1e-3 - a_coef - refmin - ref11yr1 ) /( b_coef  + ref11yr2 )
      write (6,*) 'Reference F107 =',f107_0
      write (6,*) scon, a_coef, refmin, ref11yr1, b_coef, ref11yr2

      ! Evaluate residual
      res350_0 = a_coef + b_coef * f107_0


      ! Construct an array with the spectral coordinates of the CAM bins
      ! Limit to wavelength less or equal to 350 nm
      ! input fields are in micro-meters

      ic = 0
      do i=1, nspint
         if (wavmax(i)*1e3 <= 350.) ic = ic + 1
      end do

      nbin350 = ic 

      allocate (wavbne(nbin350+1),stat=ierr)
      if (ierr /= 0) then
         write (6,*) 'SOLVAR_INTERFACE: Error allocating space for wavbne'
         call endrun
      end if

      allocate (woodsi_ref(nbin350), stat=ierr)
      if (ierr /= 0) then
         write (6,*) 'SOLVAR_INTERFACE: Error allocating space for woodsi_ref'
         call endrun
      end if

      allocate (woods_etf_ref(nbins), stat=ierr)
      if (ierr /= 0) then
         write (6,*) 'SOLVAR_INTERFACE: Error allocating space for woods_etf_ref'
         call endrun
      end if

      do i=1, nbin350,2
         wavbne(i)   = wavmin(i) * 1e3
         wavbne(i+1) = wavmax(i) * 1e3
      end do

      ! Calculate Woods&Rotmann ETF corresponding to the reference F107
      ! (this part of the code can be moved to an initialization phase)
      call woods_set_etf ( f107_0 , f107_0, etf_temp=woods_etf_ref)
      ! Re-bin
      call rebin (nbins, nbin350, we, wavbne, woods_etf_ref, woodsi_ref)

      return

    end subroutine solvar_init

  end module solvar_interface
