
#include <misc.h>

      module woods

      use shr_kind_mod, only : r8 => shr_kind_r8
      use abortutils,   only : endrun

      implicit none

      private
      public :: woods_init
      public :: woods_set_etf
      public :: nbins, woods_etf, we

      save

      integer               :: nstruct
      integer               :: nbins
      real(r8), allocatable :: wc(:)                ! wave interval center (nm)
      real(r8), allocatable :: we(:)                ! wave interval edges (nm)
      real(r8), allocatable :: wlintv(:)            ! wave interval (nm)
      real(r8), allocatable :: wlintvi(:)           ! inverse wave interval (nm)
      real(r8), allocatable :: refmin(:)
      real(r8), allocatable :: rt27day(:)
      real(r8), allocatable :: rt11yr(:)
      real(r8), allocatable :: woods_etf(:)

      contains

      subroutine woods_init (woods_etf_file)
!---------------------------------------------------------------
!	... initialize woods etf module
!---------------------------------------------------------------

      use pmgrid,         only : masterproc
      use error_messages, only : alloc_err
      use ioFileMod,      only : getfil
#ifdef SPMD
      use mpishorthand,   only : mpiint, mpir8, mpicom
#endif

      implicit none

      character(len=*), intent(in) :: woods_etf_file

!---------------------------------------------------------------
!	... local variables
!---------------------------------------------------------------
      integer  :: ncid
      integer  :: n
      integer  :: dimid
      integer  :: varid
      integer  :: astat
      character(len=256) :: locfn

      include 'netcdf.inc'

!-----------------------------------------------------------------------
!	... readin the etf data
!-----------------------------------------------------------------------
masterproc_only : &
      if( masterproc ) then
         call getfil( woods_etf_file, locfn, 0 )
         call wrap_open (trim(locfn), NF_NOWRITE, ncid)
!-----------------------------------------------------------------------
!	... check primary dimension consistency
!-----------------------------------------------------------------------
         call wrap_inq_dimid( ncid, 'dim1_WC', dimid )
         call wrap_inq_dimlen( ncid, dimid, nbins )
         call wrap_inq_dimid( ncid, 'dim1_WLINT', dimid )
         call wrap_inq_dimlen( ncid, dimid, n )
         if( n /= nbins ) then
            write(*,*) 'woods_init: WLINT dimension(',n,') does not match bin count ',nbins
            call endrun
         end if
         call wrap_inq_dimid( ncid, 'dim1_REFMIN', dimid )
         call wrap_inq_dimlen( ncid, dimid, n )
         if( n /= nbins ) then
            write(*,*) 'woods_init: REFMIN dimension(',n,') does not match bin count ',nbins
            call endrun
         end if
         call wrap_inq_dimid( ncid, 'dim1_RT27DAY', dimid )
         call wrap_inq_dimlen( ncid, dimid, n )
         if( n /= nbins ) then
            write(*,*) 'woods_init: RT27DAY dimension(',n,') does not match bin count ',nbins
            call endrun
         end if
         call wrap_inq_dimid( ncid, 'dim1_RT11YR', dimid )
         call wrap_inq_dimlen( ncid, dimid, n )
         if( n /= nbins ) then
            write(*,*) 'woods_init: RT11YR dimension(',n,') does not match bin count ',nbins
            call endrun
         end if

!-----------------------------------------------------------------------
!	... allocate primary arrays
!-----------------------------------------------------------------------
         allocate( wc(nbins), we(nbins+1), wlintv(nbins), wlintvi(nbins), &
                   refmin(nbins), rt27day(nbins), rt11yr(nbins), woods_etf(nbins), stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'woods_init', 'wc ... woods_etf', nbins )
         end if
!-----------------------------------------------------------------------
!	... read primary arrays
!-----------------------------------------------------------------------
         call wrap_inq_varid( ncid, 'WC', varid )
         call wrap_get_var_realx( ncid, varid, wc )
         call wrap_inq_varid( ncid, 'WLINT', varid )
         call wrap_get_var_realx( ncid, varid, wlintv )
         call wrap_inq_varid( ncid, 'REFMIN', varid )
         call wrap_get_var_realx( ncid, varid, refmin )
         call wrap_inq_varid( ncid, 'RT27DAY', varid )
         call wrap_get_var_realx( ncid, varid, rt27day )
         call wrap_inq_varid( ncid, 'RT11YR', varid )
         call wrap_get_var_realx( ncid, varid, rt11yr )

         call wrap_close( ncid )
      end if masterproc_only

#ifdef SPMD
      call mpibcast( nbins, 1, mpiint, 0, mpicom )
      if( .not. masterproc ) then
         allocate( wc(nbins), we(nbins+1), wlintv(nbins), wlintvi(nbins), &
                   refmin(nbins), rt27day(nbins), rt11yr(nbins), woods_etf(nbins), stat=astat )
         if( astat /= 0 ) then
	    call alloc_err( astat, 'woods_init', 'wc ... woods_etf', nbins )
         end if
      end if
      call mpibcast( wc, nbins, mpir8, 0, mpicom )
      call mpibcast( wlintv, nbins, mpir8, 0, mpicom )
      call mpibcast( refmin, nbins, mpir8, 0, mpicom )
      call mpibcast( rt27day, nbins, mpir8, 0, mpicom )
      call mpibcast( rt11yr, nbins, mpir8, 0, mpicom )
#endif

      wlintvi(:)  = 1._r8/wlintv(:)
      we(:nbins)  = wc(:nbins) - .5_r8*wlintv(:nbins)
      we(nbins+1) = wc(nbins) + .5_r8*wlintv(nbins)

      end subroutine woods_init

      subroutine woods_set_etf( f107, f107a , etf_temp )
!---------------------------------------------------------------
!	... set woods etf
!---------------------------------------------------------------

      implicit none

!---------------------------------------------------------------
!	... dummy arguments
!---------------------------------------------------------------
      real(r8), intent(in) :: f107
      real(r8), intent(in) :: f107a
      real(r8), optional, intent(out) :: etf_temp(nbins)

!---------------------------------------------------------------
!	... local variables
!---------------------------------------------------------------
      real(r8), parameter :: f7min  = 71._r8
      real(r8), parameter :: f7max  = 212._r8
      real(r8), parameter :: f7deli = 1._r8/(f7max - f7min)
      real(r8), parameter :: f7roti = 1._r8/73._r8

      real(r8) :: ref_27day(nbins)
      real(r8) :: ref_11yr(nbins)

      ref_27day(:) = refmin(:) * (rt27day(:) - 1._r8)
      ref_11yr(:)  = refmin(:) * (rt11yr(:) - 1._r8)

      if ( present( etf_temp ) ) then
         etf_temp(:) = refmin(:) + ref_27day(:) * (f107 - f107a)*f7roti &
              + ref_11yr(:) * (f107a - f7min)*f7deli
      else
         woods_etf(:) = refmin(:) + ref_27day(:) * (f107 - f107a)*f7roti &
              + ref_11yr(:) * (f107a - f7min)*f7deli
      end if

      !      write(*,*) ' '
      !      write(*,*) '----------------------------------------------'
      !      write(*,*) 'woods_set_etf: diagnostics for woods_etf'
      !      write(*,*) 'woods_set_etf: nbins = ',nbins
      !      write(*,'(1p,5g15.7)') woods_etf(:)
      !      write(*,*) '----------------------------------------------'
      !      write(*,*) ' '

      end subroutine woods_set_etf

      end module woods
