!===============================================================================
! CVS $Id: diag_mod.F90,v 1.14 2001/12/07 22:13:35 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/diag_mod.F90,v $
! CVS $Name: ccsm2_0_beta58 $
!===============================================================================

MODULE diag_mod

   !----------------------------------------------------------------------------
   !  This module computes spatial & time averages of fluxed quatities
   !
   !  Sign convention:
   !    positive value <=> the model is gaining water, heat, momentum, etc.
   !
   !  Unit convention:
   !    heat flux     ~ W/m^2 
   !    momentum flux ~ N/m^2
   !    water flux    ~ (kg/s)/m^2
   !    salt  flux    ~ (kg/s)/m^2
   !
   !----------------------------------------------------------------------------

   use map_mod       ! map data
   use buf_mod       ! model data buffers
   use control_mod   ! control flags
   use shr_sys_mod   ! shared code: system calls
   use shr_timer_mod ! timing routines
   use const_mod     ! physical constants

#ifdef MPI_P
   use shr_kind_mod, only: SHR_KIND_IN, SHR_KIND_R8
#endif

   implicit none

   !--- local constants ---
   real,parameter,private :: So = OCN_REF_SAL          ! salinity of ocn (ppt)
   real,parameter,private :: Si = ICE_REF_SAL          ! salinity of ice (ppt)
   real,parameter :: HFLXtoWFLX = -(So-Si)/(So*latice) ! convert heat to water

   !----- diagnostic data array -----
   real        :: diag_eday0        ! partial sum: start date
   real        :: diag_eday1        ! partial sum: end   date
   real        :: diag_ns           ! partial sum: number of samples
   real,target :: diag_datai(8,6,3) ! data: instantaneous (n,m,p)
   real,target :: diag_datas(8,6,3) ! data: partial-sum   
   real,target :: diag_dataa(8,6,3) ! data: annual-avg    

   !--- indicies into data array ---
   integer,parameter :: p_heat    = 1 ! group index: heat fluxes
   integer,parameter :: p_water   = 2 ! group index: water fluxes
   integer,parameter :: p_area    = 3 ! group index: area

   integer,parameter :: m_atm     = 1 ! model index: atm
   integer,parameter :: m_ice_nh  = 2 ! model index: ice, northern
   integer,parameter :: m_ice_sh  = 3 ! model index: ice, southern
   integer,parameter :: m_lnd     = 4 ! model index: lnd
   integer,parameter :: m_ocn     = 5 ! model index: ocn
   integer,parameter :: m_sum     = 6 ! model index: sum of all

   integer,parameter :: n_area    = 1 ! area (wrt to unit sphere)

   integer,parameter :: n_hfrz    = 1 ! heat : latent, freezing 
   integer,parameter :: n_hmelt   = 2 ! heat : latent, melting  
   integer,parameter :: n_hswnet  = 3 ! heat : short wave, net
   integer,parameter :: n_hlwdn   = 4 ! heat : longwave down
   integer,parameter :: n_hlwup   = 5 ! heat : longwave up
   integer,parameter :: n_hlat    = 6 ! heat : latent, vaporization
   integer,parameter :: n_hsen    = 7 ! heat : sensible
   integer,parameter :: n_hnet    = 8 ! heat : sum of all heat

   integer,parameter :: n_wfrz    = 1 ! water: freezing
   integer,parameter :: n_wmelt   = 2 ! water: melting
   integer,parameter :: n_wrain   = 3 ! water: precip, liquid
   integer,parameter :: n_wsnow   = 4 ! water: precip, frozen
   integer,parameter :: n_wevap   = 5 ! water: evaporation
   integer,parameter :: n_wroff   = 6 ! water: runoff
   integer,parameter :: n_wnet    = 7 ! water: sum of all water

   !----- used for short-wave net verification -----
   real,save   :: swnet_atm           ! cpl's expected swnet from atm
   real,save   :: swnet_lnd           ! cpl's expected swnet from lnd
   real,save   :: swnet_ice_nh        ! cpl's expected swnet from ice, northern
   real,save   :: swnet_ice_sh        ! cpl's expected swnet from ice, southern
   real,save   :: swnet_ocn           ! cpl's expected swnet from ocn

!===============================================================================
CONTAINS
!===============================================================================

SUBROUTINE diagnos()

   logical,save :: first_call = .true.  ! do 1st call setup stuff
   integer,save :: timer_n     ! system-clock-timer number

   !---------------------------------------------------------------------------
   ! PURPOSE:
   !   compute global-average diagnostics
   !
   ! NOTE: 
   !   area averages are relative to the entire unit sphere, area = 4*pi rad^2
   !----------------------------------------------------------------------------

   if (first_call) call shr_timer_get(timer_n,'diagnostics')
   call shr_timer_start(timer_n)

   !--- start of initial run => init time-avg & zero-out partial sum ---
   if (rest_type(1:4)=='init' .and. first_call) then
     diag_dataa = 0.0  ! time average
     diag_datai = 0.0  ! instantanious
     diag_datas = 0.0  ! partial sum
     diag_eday0 = eday ! 1st  day in partial sum
     diag_eday1 = -1   ! last day in partial sum
     diag_ns    =  0   ! number of samples in partial sum
   end if

   !--- start of year => compute time-avg & zero-out partial sum ---
   if ((month+day+sec) == 2 ) then
     if (diag_ns /= 0.0) diag_dataa = diag_datas/diag_ns
     diag_datas = 0.0  ! partial sum
     diag_eday0 = eday ! 1st  day in partial sum
     diag_eday1 = -1   ! last day in partial sum
     diag_ns    =  0   ! number of samples in partial sum
   end if

!!!!!--- compute instantaneous data ---

!!!---$OMP PARALLEL SECTIONS
   call diag_atm()
!!!---$OMP SECTION
   call diag_lnd()
!!!!---$OMP SECTION
   call diag_ocn()
!!!!---$OMP SECTION
   call diag_ice_nh()
!!!!---$OMP SECTION
   call diag_ice_sh()

!!!---$OMP END PARALLEL SECTIONS

   !--- sum of instantaneous, inter-model data ---
   diag_datai(:,m_sum,:) = 0.0
   diag_datai(:,m_sum,p_heat ) = sum(diag_datai(:,:,p_heat ),DIM=2)
   diag_datai(:,m_sum,p_water) = sum(diag_datai(:,:,p_water),DIM=2)
   diag_datai(:,m_sum,p_area ) = sum(diag_datai(:,:,p_area ),DIM=2)

   !--- add instantaneous data to partial sum ---
   diag_datas = diag_datas + diag_datai
   diag_ns    = diag_ns + 1
   diag_eday1 = eday + sec/86400.0

   !--- print data to stdout ---
   if (diag_now /= 0) call diag_print

   call shr_timer_stop(timer_n)

   first_call = .false.

   RETURN
END SUBROUTINE diagnos

!===============================================================================

SUBROUTINE diag_atm()

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer      :: n                 ! generic loop indices
   real,pointer :: h(:),w(:),a(:)    ! heat/water/area  data for one model only 
   real         :: da                ! area of one grid cell
#ifdef MPI_P
   integer ierr
   REAL (SHR_KIND_R8), allocatable :: redu1(:,:),redu2(:,:) 
#endif
   !----------------------------------------------------------------------------
   ! PURPOSE:
   !   compute diagnostics (instantaneous global averages)
   !
   ! NOTE: 
   !   area averages are relative to the entire unit sphere, area = 4*pi rad^2
   !----------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! area integral of instantaneous data
   !-----------------------------------------------------------------
   h => diag_datai(:,m_atm,p_heat )
   w => diag_datai(:,m_atm,p_water)
   a => diag_datai(:,m_atm,p_area )
   h = 0.0
   w = 0.0
   a = 0.0
#ifdef MPI_P
   do n=IST_a,IEND_a
#else
   do n=1,n_a
#endif
      !--- area -----------------------
      da  = domain_a%area(n)
      a(n_area  ) = a(n_area  ) + da 
      !--- heat flux ------------------
      h(n_hfrz  ) =  0.0
      h(n_hmelt ) =  0.0
!!!   h(n_hswnet) = h(n_hswnet) - da * Faxa_a(n,kFaxa_swnet)
      h(n_hswnet) = h(n_hswnet) - da *                        &
     &       ( (1.0-Ss_a(n,kSs_avsdr))*Fala_a(n,kFala_swvdr)  &
     &       + (1.0-Ss_a(n,kSs_anidr))*Fala_a(n,kFala_swndr)  &
     &       + (1.0-Ss_a(n,kSs_avsdf))*Fala_a(n,kFala_swvdf)  &
     &       + (1.0-Ss_a(n,kSs_anidf))*Fala_a(n,kFala_swndf)  )
      h(n_hlwdn ) = h(n_hlwdn ) - da * Fala_a(n,kFala_lwdn )
      h(n_hlwup ) = h(n_hlwup ) - da * Faxx_a(n,kFaxx_lwup )
      h(n_hlat  ) = h(n_hlat  ) - da * Faxx_a(n,kFaxx_lat  )
      h(n_hsen  ) = h(n_hsen  ) - da * Faxx_a(n,kFaxx_sen  )
      !--- water flux -----------------
      w(n_wfrz  ) =  0.0
      w(n_wmelt ) =  0.0
      w(n_wrain ) = w(n_wrain ) - da * Fala_a(n,kFala_rainc)
      w(n_wrain ) = w(n_wrain ) - da * Fala_a(n,kFala_rainl)
      w(n_wsnow ) = w(n_wsnow ) - da * Fala_a(n,kFala_snowc)
      w(n_wsnow ) = w(n_wsnow ) - da * Fala_a(n,kFala_snowl)
      w(n_wevap ) = w(n_wevap ) - da * Faxx_a(n,kFaxx_evap )
      w(n_wroff ) =  0.0
      !--------------------------------
   end do

#ifdef MPI_P
   allocate(redu1(size(diag_datai,1),3))
   allocate(redu2(size(diag_datai,1),3))
   redu1 = 0.
   redu2 = 0.
   redu1(:,1) = a(:) 
   redu1(:,2) = h(:) 
   redu1(:,3) = w(:) 
   call MPI_ALLREDUCE(redu1,redu2,size(diag_datai,1)*3,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
   a(:) = redu2(:,1)
   h(:) = redu2(:,2)  
   w(:) = redu2(:,3)  
   deallocate(redu1)
   deallocate(redu2)
#endif
   h = h/(4*PI)
   w = w/(4*PI)
   a = a/(4*PI)
   h(n_hnet) = sum(h)
   w(n_wnet) = sum(w)

   swnet_atm = h(n_hswnet)

   return
END SUBROUTINE diag_atm

!===============================================================================

SUBROUTINE diag_lnd()

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer      :: n,k               ! generic loop indices
   real,pointer :: h(:),w(:),a(:)    ! heat/water/area  data for one model only 
   real         :: da                ! area of one grid cell
   real         :: daa,dai,dal,dao   ! area     of atm/ice/lnd/ocn in a cell
#ifdef MPI_P
   integer ierr
   REAL (SHR_KIND_R8), allocatable :: redu1(:,:),redu2(:,:) 
#endif

   !----------------------------------------------------------------------------
   ! PURPOSE:
   !   compute diagnostics (instantaneous global averages)
   !
   ! NOTE: 
   !   area averages are relative to the entire unit sphere, area = 4*pi rad^2
   !----------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! area integral of instantaneous data
   !-----------------------------------------------------------------
   h => diag_datai(:,m_lnd,p_heat )
   w => diag_datai(:,m_lnd,p_water)
   a => diag_datai(:,m_lnd,p_area )
   h = 0.0
   w = 0.0
   a = 0.0
#ifdef MPI_P
   do n=IST_l,IEND_l
#else
   do n=1,n_l
#endif
      if (mask_l(n) /= 0) then
         !--- area -----------------------
         dal = domain_l%area(n)*fl_l(n)
         a(n_area  ) = a(n_area  ) + dal
         !--- heat flux ------------------
         h(n_hfrz  ) = 0.0
         h(n_hmelt ) = 0.0
!!!      h(n_hswnet) = h(n_hswnet) + dal*Fall_l(n,kFall_swnet)
         h(n_hswnet) = h(n_hswnet) + dal*                       &
     &         ( (1.0-Sl_l(n,kSl_avsdr))*Fala_l(n,kFala_swvdr)  &
     &         + (1.0-Sl_l(n,kSl_anidr))*Fala_l(n,kFala_swndr)  &
     &         + (1.0-Sl_l(n,kSl_avsdf))*Fala_l(n,kFala_swvdf)  &
     &         + (1.0-Sl_l(n,kSl_anidf))*Fala_l(n,kFala_swndf)  )
         h(n_hlwdn ) = h(n_hlwdn ) + dal*Fala_l(n,kFala_lwdn )
         h(n_hlwup ) = h(n_hlwup ) + dal*Fall_l(n,kFall_lwup )
         h(n_hlat  ) = h(n_hlat  ) + dal*Fall_l(n,kFall_lat  )
         h(n_hsen  ) = h(n_hsen  ) + dal*Fall_l(n,kFall_sen  )
         !--- water flux -----------------
         w(n_wfrz  ) =  0.0
         w(n_wmelt ) =  0.0
         w(n_wrain ) = w(n_wrain ) + dal*Fala_l(n,kFala_rainc)
         w(n_wrain ) = w(n_wrain ) + dal*Fala_l(n,kFala_rainl)
         w(n_wsnow ) = w(n_wsnow ) + dal*Fala_l(n,kFala_snowc)
         w(n_wsnow ) = w(n_wsnow ) + dal*Fala_l(n,kFala_snowl)
         w(n_wevap ) = w(n_wevap ) + dal*Fall_l(n,kFall_evap )
         !--------------------------------
      end if
   end do

#ifdef MPI_P
   allocate(redu1(size(diag_datai,1),3))
   allocate(redu2(size(diag_datai,1),3))
   redu1 = 0.
   redu2 = 0.
   redu1(:,1) = a(:) 
   redu1(:,2) = h(:) 
   redu1(:,3) = w(:) 
   call MPI_ALLREDUCE(redu1,redu2,size(diag_datai,1)*3,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)

   a(:) = redu2(:,1)
   h(:) = redu2(:,2)  
   w(:) = redu2(:,3)  
   deallocate(redu1)
   deallocate(redu2)
#endif

   do k=1,domain_r%nc                          ! index into compressed vector
     n = domain_r%iunc(k)                      ! index into uncompressed vector
     if (domain_r%mask(n) /= 0) then
       da = domain_r%area(n)                     ! cell area in rad^2
       w(n_wroff) = w(n_wroff) - roff_rc(k,1)*da ! runoff in (kg/s)*rad^2
     end if
   end do
!
   h = h/(4*PI)
   w = w/(4*PI)
   a = a/(4*PI)
   h(n_hnet) = sum(h)
   w(n_wnet) = sum(w)

   swnet_lnd = h(n_hswnet)

   return
END SUBROUTINE diag_lnd

!===============================================================================

SUBROUTINE diag_ice_nh

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer      :: n                 ! generic loop indices
   real,pointer :: h(:),w(:),a(:)    ! heat/water/area  data for one model only 
   real         :: da                ! area of one grid cell
   real         :: daa,dai,dal,dao   ! area     of atm/ice/lnd/ocn in a cell
#ifdef MPI_P
   integer ierr
   REAL (SHR_KIND_R8), allocatable :: redu1(:,:),redu2(:,:) 
#endif

   !----------------------------------------------------------------------------
   ! PURPOSE:
   !   compute diagnostics (instantaneous global averages)
   !
   ! NOTE: 
   !   area averages are relative to the entire unit sphere, area = 4*pi rad^2
   !----------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! area integral of instantaneous data
   !-----------------------------------------------------------------
   h => diag_datai(:,m_ice_nh,p_heat )
   w => diag_datai(:,m_ice_nh,p_water)
   a => diag_datai(:,m_ice_nh,p_area )
   h = 0.0
   w = 0.0
   a = 0.0
   swnet_ice_nh = 0.0 ! need swnet w/o penetrating sw for solar verification
#ifdef MPI_P
   do n=IST_i,IEND_i
#else
   do n=1,n_i
#endif
      if (mask_i(n) /= 0 .and. yc_i(n) > 0.0) then
         !--- area -----------------------
         da  = domain_i%area(n)
         dai = domain_i%area(n)*fi_i(n)
         dao = domain_i%area(n)*fo_i(n)
         a(n_area  ) = a(n_area  ) + dai
         !--- heat flux ------------------
         h(n_hfrz  ) = h(n_hfrz  ) - da *max(Fioo_i(n,kFioo_q    ),0.0)
         h(n_hmelt ) = h(n_hmelt ) - dai*    Fioi_i(n,kFioi_melth)
!!!      h(n_hswnet) = h(n_hswnet) + dai*    Faii_i(n,kFaii_swnet)
         swnet_ice_nh = swnet_ice_nh + dai*                         &
     &             ( (1.0-Si_i(n,kSi_avsdr))*Faia_i(n,kFaia_swvdr)  &
     &             + (1.0-Si_i(n,kSi_anidr))*Faia_i(n,kFaia_swndr)  &
     &             + (1.0-Si_i(n,kSi_avsdf))*Faia_i(n,kFaia_swvdf)  &
     &             + (1.0-Si_i(n,kSi_anidf))*Faia_i(n,kFaia_swndf)  )
         h(n_hswnet) = h(n_hswnet) - dai*    Fioi_i(n,kFioi_swnet)
         h(n_hlwdn ) = h(n_hlwdn ) + dai*    Faia_i(n,kFaia_lwdn )
         h(n_hlwup ) = h(n_hlwup ) + dai*    Faii_i(n,kFaii_lwup )
         h(n_hlat  ) = h(n_hlat  ) + dai*    Faii_i(n,kFaii_lat  )
         h(n_hsen  ) = h(n_hsen  ) + dai*    Faii_i(n,kFaii_sen  )
         !--- water flux -----------------
         w(n_wfrz  ) = h(n_hfrz )*HFLXtoWFLX ! implied water flux
         w(n_wmelt ) = w(n_wmelt ) - dai*    Fioi_i(n,kFioi_meltw)
         w(n_wrain ) = w(n_wrain ) + dai*    Faia_i(n,kFaia_rain )
         w(n_wsnow ) = w(n_wsnow ) + dai*    Faia_i(n,kFaia_snow )
         w(n_wevap ) = w(n_wevap ) + dai*    Faii_i(n,kFaii_evap )
         w(n_wroff ) =  0.0
      end if
   end do
#ifdef MPI_P
   allocate(redu1(size(diag_datai,1),4))
   allocate(redu2(size(diag_datai,1),4))
   redu1 = 0.
   redu2 = 0.
   redu1(:,1) = a(:) 
   redu1(:,2) = h(:) 
   redu1(:,3) = w(:) 
   redu1(1,4) = swnet_ice_nh
   call MPI_ALLREDUCE(redu1,redu2,size(diag_datai,1)*3+1,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
   a(:) = redu2(:,1)
   h(:) = redu2(:,2)  
   w(:) = redu2(:,3)  
   swnet_ice_nh = redu2(1,4)
   deallocate(redu1)
   deallocate(redu2)
#endif
   h(n_hswnet) = h(n_hswnet) + swnet_ice_nh 
   swnet_ice_nh = swnet_ice_nh /(4*PI)
   h = h/(4*PI)
   w = w/(4*PI)
   a = a/(4*PI)
   h(n_hnet) = sum(h)
   w(n_wnet) = sum(w)

   return
END SUBROUTINE diag_ice_nh

!===============================================================================

SUBROUTINE diag_ice_sh

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer      :: n                 ! generic loop indices
   real,pointer :: h(:),w(:),a(:)    ! heat/water/area  data for one model only 
   real         :: da                ! area of one grid cell
   real         :: daa,dai,dal,dao   ! area     of atm/ice/lnd/ocn in a cell
#ifdef MPI_P
   integer ierr
   REAL (SHR_KIND_R8), allocatable :: redu1(:,:),redu2(:,:) 
#endif

   !----------------------------------------------------------------------------
   ! PURPOSE:
   !   compute diagnostics (instantaneous global averages)
   !
   ! NOTE: 
   !   area averages are relative to the entire unit sphere, area = 4*pi rad^2
   !----------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! area integral of instantaneous data
   !-----------------------------------------------------------------
   h => diag_datai(:,m_ice_sh,p_heat )
   w => diag_datai(:,m_ice_sh,p_water)
   a => diag_datai(:,m_ice_sh,p_area )
   h = 0.0
   w = 0.0
   a = 0.0
   swnet_ice_sh = 0.0 ! need swnet w/o penetrating sw for solar verification
#ifdef MPI_P
   do n=IST_i,IEND_i
#else
   do n=1,n_i
#endif
      if (mask_i(n) /= 0 .and. yc_i(n) < 0.0) then
         !--- area -----------------------
         da  = domain_i%area(n)
         dai = domain_i%area(n)*fi_i(n)
         dao = domain_i%area(n)*fo_i(n)
         a(n_area  ) = a(n_area  ) + dai
         !--- heat flux ------------------
         h(n_hfrz  ) = h(n_hfrz  ) - da *max(Fioo_i(n,kFioo_q    ),0.0)
         h(n_hmelt ) = h(n_hmelt ) - dai*    Fioi_i(n,kFioi_melth)
!!!      h(n_hswnet) = h(n_hswnet) + dai*    Faii_i(n,kFaii_swnet)
         swnet_ice_sh = swnet_ice_sh + dai*                         &
     &             ( (1.0-Si_i(n,kSi_avsdr))*Faia_i(n,kFaia_swvdr)  &
     &             + (1.0-Si_i(n,kSi_anidr))*Faia_i(n,kFaia_swndr)  &
     &             + (1.0-Si_i(n,kSi_avsdf))*Faia_i(n,kFaia_swvdf)  &
     &             + (1.0-Si_i(n,kSi_anidf))*Faia_i(n,kFaia_swndf)  )
         h(n_hswnet) = h(n_hswnet) - dai*    Fioi_i(n,kFioi_swnet)
         h(n_hlwdn ) = h(n_hlwdn ) + dai*    Faia_i(n,kFaia_lwdn )
         h(n_hlwup ) = h(n_hlwup ) + dai*    Faii_i(n,kFaii_lwup )
         h(n_hlat  ) = h(n_hlat  ) + dai*    Faii_i(n,kFaii_lat  )
         h(n_hsen  ) = h(n_hsen  ) + dai*    Faii_i(n,kFaii_sen  )
         !--- water flux -----------------
         w(n_wfrz  ) = h(n_hfrz )*HFLXtoWFLX ! implied water flux
         w(n_wmelt ) = w(n_wmelt ) - dai*    Fioi_i(n,kFioi_meltw)
         w(n_wrain ) = w(n_wrain ) + dai*    Faia_i(n,kFaia_rain )
         w(n_wsnow ) = w(n_wsnow ) + dai*    Faia_i(n,kFaia_snow )
         w(n_wevap ) = w(n_wevap ) + dai*    Faii_i(n,kFaii_evap )
         w(n_wroff ) =  0.0
      end if
   end do

#ifdef MPI_P
   allocate(redu1(size(diag_datai,1),4))
   allocate(redu2(size(diag_datai,1),4))
   redu1 = 0.
   redu2 = 0.
   redu1(:,1) = a(:) 
   redu1(:,2) = h(:) 
   redu1(:,3) = w(:) 
   redu1(1,4) = swnet_ice_sh
   call MPI_ALLREDUCE(redu1,redu2,size(diag_datai,1)*3+1,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
   a(:) = redu2(:,1)
   h(:) = redu2(:,2)  
   w(:) = redu2(:,3)  
   swnet_ice_sh = redu2(1,4)
   deallocate(redu1)
   deallocate(redu2)
#endif
   h(n_hswnet) = h(n_hswnet) + swnet_ice_sh 
   swnet_ice_sh = swnet_ice_sh /(4*PI)
   h = h/(4*PI)
   w = w/(4*PI)
   a = a/(4*PI)
   h(n_hnet) = sum(h)
   w(n_wnet) = sum(w)

   return
END SUBROUTINE diag_ice_sh

!===============================================================================

SUBROUTINE diag_ocn()

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer      :: n                 ! generic loop indices
   real,pointer :: h(:),w(:),a(:)    ! heat/water/area  data for one model only 
   real         :: da                ! area of one grid cell
   real         :: daa,dai,dal,dao   ! area     of atm/ice/lnd/ocn in a cell
   real         :: fa ,fi ,fl ,fo    ! fraction of atm/ice/lnd/ocn in a cell
#ifdef MPI_P
   integer ierr
   REAL (SHR_KIND_R8), allocatable :: redu1(:,:),redu2(:,:) 
#endif

   !----------------------------------------------------------------------------
   ! PURPOSE:
   !   compute diagnostics (instantaneous global averages)
   !
   ! NOTE: 
   !   area averages are relative to the entire unit sphere, area = 4*pi rad^2
   !----------------------------------------------------------------------------

   !-----------------------------------------------------------------
   ! area integral of instantaneous data
   !-----------------------------------------------------------------
   h => diag_datai(:,m_ocn,p_heat )
   w => diag_datai(:,m_ocn,p_water)
   a => diag_datai(:,m_ocn,p_area )
   h = 0.0
   w = 0.0
   a = 0.0
   swnet_ocn = 0.0 ! need swnet w/o penetrating sw for solar verification
#ifdef MPI_P
   do n=IST_o,IEND_o
#else
   do n=1,n_o
#endif
      if (mask_o(n) /= 0) then
         da  = domain_o%area(n)
         daa = da*fa_o(n)
         dai = da*fi_o(n)
         !--- area -----------------------
         a(n_area  ) = a(n_area  ) + daa
         !--- heat flux ------------------
         h(n_hfrz  ) = h(n_hfrz  ) + da *max(Fioo_o(n,kFioo_q    ),0.0)
         h(n_hmelt ) = h(n_hmelt ) + da *    Foxx_o(n,kFoxx_melth)
         h(n_hswnet) = h(n_hswnet) + dai*    Fioi_i(n,kFioi_swnet)
         swnet_ocn   = swnet_ocn   + daa*                           &
     &             ( (1.0-So_o(n,kSo_avsdr))*Faoa_o(n,kFaoa_swvdr)  &
     &             + (1.0-So_o(n,kSo_anidr))*Faoa_o(n,kFaoa_swndr)  &
     &             + (1.0-So_o(n,kSo_avsdf))*Faoa_o(n,kFaoa_swvdf)  &
     &             + (1.0-So_o(n,kSo_anidf))*Faoa_o(n,kFaoa_swndf)  )
         h(n_hlwdn ) = h(n_hlwdn ) + da *    Foxx_o(n,kFoxx_lwdn )
         h(n_hlwup ) = h(n_hlwup ) + da *    Foxx_o(n,kFoxx_lwup )
         h(n_hlat  ) = h(n_hlat  ) + da *    Foxx_o(n,kFoxx_lat  )
         !! h(n_hsen  ) = h(n_hsen  ) + da *    Foxx_o(n,kFoxx_sen  )
         !--- water flux -----------------
         w(n_wfrz  ) = h(n_hfrz )*HFLXtoWFLX ! implied water flux
         w(n_wmelt ) = w(n_wmelt ) + da *    Foxx_o(n,kFoxx_meltw)
         w(n_wrain ) = w(n_wrain ) + da *    Foxx_o(n,kFoxx_rain )
         w(n_wsnow ) = w(n_wsnow ) + da *    Foxx_o(n,kFoxx_snow )
         !! w(n_wevap ) = w(n_wevap ) + da *    Foxx_o(n,kFoxx_evap )
         w(n_wroff ) = w(n_wroff ) + da *    Foxx_o(n,kFoxx_roff )
         !----------------)----------------
         if ( ocn_type(1:3) == 'POP' ) then
            w(n_wevap ) = w(n_wevap ) + da *    Foxx_o(n,kFoxx_evap )
            h(n_hsen  ) = h(n_hsen  ) + da *    Foxx_o(n,kFoxx_sen  )
         else if ( ocn_type(1:3) == 'MOM' ) then
!--20081225            w(n_wevap ) = w(n_wevap ) - da *    Foxx_o(n,kFoxx_evap )
!--20081225            h(n_hsen  ) = h(n_hsen  ) - da *    Foxx_o(n,kFoxx_sen  )
            w(n_wevap ) = w(n_wevap ) + daa *    Faoc_o(n,kFaoc_evap )
            h(n_hsen  ) = h(n_hsen  ) + daa *    Faoc_o(n,kFaoc_sen  )
         endif

       end if
   end do
#ifdef MPI_P
   allocate(redu1(size(diag_datai,1),4))
   allocate(redu2(size(diag_datai,1),4))
   redu1 = 0.
   redu2 = 0.
   redu1(:,1) = a(:) 
   redu1(:,2) = h(:) 
   redu1(:,3) = w(:) 
   redu1(1,4) = swnet_ocn
   call MPI_ALLREDUCE(redu1,redu2,size(diag_datai,1)*3+1,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
   a(:) = redu2(:,1)
   h(:) = redu2(:,2)  
   w(:) = redu2(:,3)  
   swnet_ocn = redu2(1,4)
   deallocate(redu1)
   deallocate(redu2)
#endif
   h(n_hswnet) = h(n_hswnet) + swnet_ocn 
   swnet_ocn = swnet_ocn /(4*PI)
   h = h/(4*PI)
   w = w/(4*PI)
   a = a/(4*PI)
   h(n_hnet) = sum(h)
   w(n_wnet) = sum(w)

   return
END SUBROUTINE diag_ocn

!===============================================================================

SUBROUTINE diag_print

   implicit none

   !--- local ---
   integer :: m           ! data array indicies
   integer :: nday        ! number of days in time avg
   real    :: area (  6)  ! area data
   real    :: heat (8,6)  ! heat data 
   real    :: water(8,6)  ! water data 

   !--- formats ---
   character(len=*),parameter :: F10="('(diag) ',a17,'    date    sec    area',&
   &       '    freeze      melt     netsw      lwdn',                         &
   &       '      lwup       lat       sen       net W/m^2')"
   character(len=*),parameter :: F11="('(diag) ',a17,' -------- ----- -------',&
   &       8(' ---------'))"
   character(len=*),parameter :: F12="('(diag) ',a17,i9.8,i6,f8.5,8(f10.4))"

   character(len=*),parameter :: F20="('(diag) ',a17,'    date    sec    area',&
   &       '    freeze      melt      rain      snow',                         &
   &       '      evap   run-off       net 10^-6 (kg/s)/m^2')"
   character(len=*),parameter :: F21="('(diag) ',a17,' -------- ----- -------',&
   &       7(' ---------'))"
   character(len=*),parameter :: F22="('(diag) ',a17,i9.8,i6,f8.5,7(f11.5))"

   character(len=*),parameter :: F30="('(diag) ',a17,'    date  ndays    area',&
   &       '    freeze      melt     netsw      lwdn',                         &
   &       '      lwup       lat       sen       net W/m^2')"
   character(len=*),parameter :: F40="('(diag) ',a17,'    date  ndays    area',&
   &       '    freeze      melt      rain      snow',                         &
   &       '      evap   run-off       net 10^-6 (kg/s)/m^2')"

!!----------------wangln 2008-08-07 for MOM Output---------------------------------
   character(len=*),parameter :: F50="('(diag) ',a17,'    date    sec    area',&
   &       '    frazil      melt     netsw      lwdn',                         &
   &       '      lwup       lat       sen       net W/m^2')"
   character(len=*),parameter :: F60="('(diag) ',a17,'    date    sec    area',&
   &       '    frazil      melt      rain      snow',                         &
   &       '      evap   run-off       net 10^-6 (kg/s)/m^2')"
   character(len=*),parameter :: F70="('(diag) ',a17,'    date  ndays    area',&
   &       '    freeze      melt     netsw      lwdn',                         &
   &       '      lwup       lat       sen       net W/m^2')"
   character(len=*),parameter :: F80="('(diag) ',a17,'    date  ndays    area',&
   &       '    frazil      melt      rain      snow',                         &
   &       '      evap   run-off       net 10^-6 (kg/s)/m^2')"
!!----------------wangln 2008-08-07 for MOM Output---------------------------------

   !----------------------------------------------------------------------------
   ! PURPOSE:
   !   print global-average diagnostics
   !----------------------------------------------------------------------------

   !---------------------------------------------------------------
   ! instantaneous data
   !---------------------------------------------------------------
   area  = diag_datai(n_area,:,p_area )
   heat  = diag_datai(   :  ,:,p_heat )
   water = diag_datai(   :  ,:,p_water)

   water = water*1.0e6 ! scale data for printout

   if ( ocn_type(1:3) == 'POP' ) then
        write(6,F10)'heat  inst       '
   else if ( ocn_type(1:3) == 'MOM' ) then
        write(6,F50)'heat  inst       '
   endif
   write(6,F11)'heat  inst       '
   write(6,F12)'heat  inst atm   ',cdate, sec,area(m_atm   ) ,heat(1:8,m_atm   )
   write(6,F12)'heat  inst lnd   ',cdate, sec,area(m_lnd   ) ,heat(1:8,m_lnd   )
   write(6,F12)'heat  inst ice_nh',cdate, sec,area(m_ice_nh) ,heat(1:8,m_ice_nh)
   write(6,F12)'heat  inst ice_sh',cdate, sec,area(m_ice_sh) ,heat(1:8,m_ice_sh)
   write(6,F12)'heat  inst ocn   ',cdate, sec,area(m_ocn   ) ,heat(1:8,m_ocn   )
   write(6,F11)'heat  inst       '
   write(6,F12)'heat  inst sum   ',cdate, sec,area(m_sum   ) ,heat(1:8,m_sum   )
   write(6,*)

   if ( ocn_type(1:3) == 'POP' ) then
        write(6,F20)'water  inst      '
   else if ( ocn_type(1:3) == 'MOM' ) then
        write(6,F60)'water  inst      '
   endif
   write(6,F21)'water inst       '
   write(6,F22)'water inst atm   ',cdate, sec,area(m_atm   ),water(1:7,m_atm   )
   write(6,F22)'water inst lnd   ',cdate, sec,area(m_lnd   ),water(1:7,m_lnd   )
   write(6,F22)'water inst ice_nh',cdate, sec,area(m_ice_nh),water(1:7,m_ice_nh)
   write(6,F22)'water inst ice_sh',cdate, sec,area(m_ice_sh),water(1:7,m_ice_sh)
   write(6,F22)'water inst ocn   ',cdate, sec,area(m_ocn   ),water(1:7,m_ocn   )
   write(6,F21)'water inst       '
   write(6,F22)'water inst sum   ',cdate, sec,area(m_sum   ),water(1:7,m_sum   )
   write(6,*)

   !---------------------------------------------------------------
   ! time-avg data
   !---------------------------------------------------------------
   if (diag_now < 2) return
   if (diag_ns <= 0) return ! this shouldn't happen

   area  = diag_datas(n_area,:,p_area )
   heat  = diag_datas(   :  ,:,p_heat )
   water = diag_datas(   :  ,:,p_water)

   water = water*1.0e6 ! scale data for printout

   area  =  area/diag_ns
   heat  =  heat/diag_ns
   water = water/diag_ns
   nday  = nint(diag_eday1 - diag_eday0)

   if ( ocn_type(1:3) == 'POP' ) then
        write(6,F30)'heat  tavg       '
   else if ( ocn_type(1:3) == 'MOM' ) then
        write(6,F70)'heat  tavg       '
   endif
   write(6,F11)'heat  tavg       '
   write(6,F12)'heat  tavg atm   ',cdate,nday,area(m_atm   ) ,heat(1:8,m_atm   )
   write(6,F12)'heat  tavg lnd   ',cdate,nday,area(m_lnd   ) ,heat(1:8,m_lnd   )
   write(6,F12)'heat  tavg ice_nh',cdate,nday,area(m_ice_nh) ,heat(1:8,m_ice_nh)
   write(6,F12)'heat  tavg ice_sh',cdate,nday,area(m_ice_sh) ,heat(1:8,m_ice_sh)
   write(6,F12)'heat  tavg ocn   ',cdate,nday,area(m_ocn   ) ,heat(1:8,m_ocn   )
   write(6,F11)'heat  tavg       '
   write(6,F12)'heat  tavg sum   ',cdate,nday,area(m_sum   ) ,heat(1:8,m_sum   )
   write(6,*)

   if ( ocn_type(1:3) == 'POP' ) then
        write(6,F40)'water tavg       '
   else if ( ocn_type(1:3) == 'MOM' ) then
        write(6,F40)'water tavg       '
   endif
   write(6,F21)'water tavg       '
   write(6,F22)'water tavg atm   ',cdate,nday,area(m_atm   ),water(1:7,m_atm   )
   write(6,F22)'water tavg lnd   ',cdate,nday,area(m_lnd   ),water(1:7,m_lnd   )
   write(6,F22)'water tavg ice_nh',cdate,nday,area(m_ice_nh),water(1:7,m_ice_nh)
   write(6,F22)'water tavg ice_sh',cdate,nday,area(m_ice_sh),water(1:7,m_ice_sh)
   write(6,F22)'water tavg ocn   ',cdate,nday,area(m_ocn   ),water(1:7,m_ocn   )
   write(6,F21)'water tavg       '
   write(6,F22)'water tavg sum   ',cdate,nday,area(m_sum   ),water(1:7,m_sum   )
   write(6,*)

   call shr_sys_flush(6)

   RETURN
END SUBROUTINE diag_print

!===============================================================================

SUBROUTINE diag_solar()

   implicit none
#ifdef MPI_P
   include 'mpif.h'
#endif

   !----- local -----
   integer :: n                     ! generic loop index
   real    :: sa1,sl1,sin1,sis1,so1 ! net solar computed in cpl diagnostics
   real    :: sa2,sl2,sin2,sis2,so2 ! net solar sent to cpl from components

   !----- formats -----
   character(len=*),parameter :: F00="('(diag_solar) ',a,2f16.6)"
   character(len=*),parameter :: F05="('(diag_solar) ', &
   &       'swnet        atm         lnd      ice-nh      ice-sh         ocn ')"
   character(len=*),parameter :: F06="('(diag_solar) ', &
   &       '      ----------- ----------- ----------- ----------- -----------')"
   character(len=*),parameter :: F07="('(diag_solar)   cpl',5f12.6)"
   character(len=*),parameter :: F08="('(diag_solar) model',5f12.6)"
#ifdef MPI_P
   integer ierr
   real (SHR_KIND_R8) :: redu1(4),redu2(4)
#endif

!-------------------------------------------------------------------------------
! PURPOSE:
!   compare expected vs. actual short-wave net (absorbed solar)
!-------------------------------------------------------------------------------

   if (diag_now == 0) return

   !--- atm --------------------------------------------------------------
   sa1 = swnet_atm ! previously computed value
   sa2 = 0.0
#ifdef MPI_P
!---!$OMP PARALLEL DO PRIVATE(n)
   do n=IST_a,IEND_a
#else
   do n=1,n_a
#endif
      sa2 = sa2 - domain_a%area(n)*Faxa_a(n,kFaxa_swnet)
   end do
#ifdef MPI_P
   redu1(1) = sa2
#else
   sa2 = sa2/(4.0*PI)
#endif

   !--- lnd --------------------------------------------------------------
   sl1 = swnet_lnd
   sl2 = 0.0
#ifdef MPI_P
!---!$OMP PARALLEL DO PRIVATE(n)
   do n=IST_l,IEND_l
#else
   do n=1,n_l
#endif
      if ( domain_l%mask(n) /= 0 ) then
        sl2 = sl2 + domain_l%area(n)*fl_l(n)*Fall_l(n,kFall_swnet)
      end if
   end do
#ifdef MPI_P
   redu1(2) = sl2
#else
   sl2 = sl2/(4.0*PI)
#endif

   !--- ice-nh -----------------------------------------------------------
   sin1 = swnet_ice_nh ! previously computed value
   sin2 = 0.0
#ifdef MPI_P
!---!$OMP PARALLEL DO PRIVATE(n)
   do n=IST_i,IEND_i
#else
   do n=1,n_i
#endif
      if ( domain_i%mask(n) /= 0 .and. domain_i%yc(n) > 0.0) then
        sin2 = sin2 + domain_i%area(n)*fi_i(n)*Faii_i(n,kFaii_swnet)
      end if
   end do

#ifdef MPI_P
   redu1(3) = sin2
#else
   sin2 = sin2/(4.0*PI)
#endif

   !--- ice-sh  ----------------------------------------------------------
   sis1 = swnet_ice_sh ! previously computed value
   sis2 = 0.0
#ifdef MPI_P
!---!$OMP PARALLEL DO PRIVATE(n)
   do n=IST_i,IEND_i
#else
   do n=1,n_i
#endif
      if ( domain_i%mask(n) /= 0 .and. domain_i%yc(n) < 0.0) then
        sis2 = sis2 + domain_i%area(n)*fi_i(n)*Faii_i(n,kFaii_swnet)
      end if
   end do

#ifdef MPI_P
   redu1(4) = sis2
#else
   sis2 = sis2/(4.0*PI)
#endif

#ifdef MPI_P
   call MPI_ALLREDUCE(redu1,redu2,4,&
                 & MPI_REAL8,MPI_SUM,MPI_COMM_CPL,ierr)
   sa2  = redu2(1)/(4.0*PI)
   sl2  = redu2(2)/(4.0*PI)
   sin2 = redu2(3)/(4.0*PI)
   sis2 = redu2(4)/(4.0*PI) 
#endif
   !--- ocn --------------------------------------------------------------
   ! note: ocn does not provide cpl with any sw-net info
   !----------------------------------------------------------------------
   so1 = swnet_ocn  ! previously computed value
   so2 = -999.0     ! flags invalid/missing data

   !--- print ------------------------------------------------------------
   write(6,F05)
   write(6,F06)
   write(6,F07) sa1,sl1,sin1,sis1,so1
   write(6,F08) sa2,sl2,sin2,sis2

   call shr_sys_flush(6)

   RETURN
END SUBROUTINE diag_solar

!===============================================================================
END MODULE diag_mod
!===============================================================================
