!! Copyright (C) 2005 Alexander Barth <barth.alexander@gmail.com>
!! Copyright (C) 2006 David Saunders (NASA Ames Research Center)
!! Copyright (C) 2008 Gian Franco Marras (CINECA) <g.marras@cineca.it>
!!
!! This program is free software; you can redistribute it and/or modify it under
!! the terms of the GNU General Public License as published by the Free Software
!! Foundation; either version 3 of the License, or (at your option) any later
!! version.
!!
!! This program is distributed in the hope that it will be useful, but WITHOUT
!! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
!! FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
!! details.
!!
!! You should have received a copy of the GNU General Public License along with
!! this program; if not, see <http://www.gnu.org/licenses/>.

!! Fortran 90 module for n-dimensional optimal interpolation.
!! Dependencies: LAPACK (dsyev)

#define DIAG_OBS_COVAR
      module ice_assimil_oi

!     working precision 
!     4 = simple precision, 8 is double precision, dlb_kind is double precision
      integer, parameter :: wp = kind(real)


      contains

!     --------------------------------------------------------------------------

      subroutine select_nearest(x,ox,param,m,index,distance)

!     Select the m observations from ox(1:nd,1:on) closest to point x(1:nd).

!     Arguments:

      implicit none
      real(wp),intent(in)  :: x(:),ox(:,:),param(:)
      integer, intent(in)  :: m
      real(wp),intent(out) :: distance(m)
      integer, intent(out) :: index(m)

!     Local variables:

      real(wp) :: d(size(ox,2))
      integer  :: i

!     Execution:

!     Calculate a measure of (squared) distance to each observation:

      do i=1,size(ox,2)
        d(i) = sum(((x - ox(:,i)) * param)**2)
      end do

      call sort(d,m,index)

      distance = d(index)

      end subroutine select_nearest


!     --------------------------------------------------------------------------

      subroutine sort(d,m,pannier)

!     Return the indices of the m smallest elements in d(:).
!     The algorithm is succinctly coded, but would a heap sort be faster?

!     Arguments:

      implicit none
      real(wp), intent(in)  :: d(:)
      integer,  intent(in)  :: m
      integer,  intent(out) :: pannier(m)

      integer :: i,max_pannier(1)

      do i=1,m
        pannier(i) = i
      end do

      max_pannier = maxloc(d(pannier))

      do i=m+1,size(d)
        if (d(i) .lt. d(pannier(max_pannier(1)))) then
          pannier(max_pannier(1)) = i
          max_pannier = maxloc(d(pannier))
        end if
      end do

      end subroutine sort

!     --------------------------------------------------------------------------

      subroutine observation_covariance(ovar,index,R)
      implicit none
      real(wp),intent(in) ::ovar(:)
      integer,intent(in) :: index(:)
#     ifdef DIAG_OBS_COVAR
      real(wp),    intent (out) :: R(size(index))
#     else
      real(wp),    intent (out) :: R(size(index),size(index))
      integer :: i
#     endif


#     ifdef DIAG_OBS_COVAR
      R = ovar(index)
#     else
      R = 0
      do i=1,size(index)
        R(i,i) = ovar(index(i))
      end do
#     endif

      end subroutine observation_covariance

!     --------------------------------------------------------------------------
!     Modified Bessel functions of 2nd kind (order 1)

      function mod_bessel_K1(x)       
      implicit none
      real(wp), intent(in) :: x
      real(wp) :: mod_bessel_K1

      real(8) :: y
      real(8), parameter :: & 
           P1 = 1.0D0, &
           P2 = 0.15443144D0, &
           P3 = -0.67278579D0, &
           P4 = -0.18156897D0, &
           P5 = -0.1919402D-1, &
           P6 = -0.110404D-2, &
           P7 = -0.4686D-4, &
           Q1 = 1.25331414D0, &
           Q2 = 0.23498619D0, &
           Q3 = -0.3655620D-1, &
           Q4 = 0.1504268D-1, &
           Q5 = -0.780353D-2, &
           Q6 = 0.325614D-2, &
           Q7 = -0.68245D-3

      if(x <= 0.) stop 'error x <= 0' 

      if(x <= 2.0) then
         y = x * x * 0.25
         mod_bessel_K1 = (log(x/2.0)*mod_bessel_I1(x))+(1.0/x)*(P1+y*(P2+y*(P3+ &
              y*(P4+y*(P5+y*(P6+y*P7))))))
      else
         y = 2.0 / x
         mod_bessel_K1 = (exp(-x)/sqrt(x))*(Q1+y*(Q2+y*(Q3+   &
              y*(Q4+y*(Q5+y*(Q6+y*Q7))))))
      endif

      end function

!     --------------------------------------------------------------------------
!     Modified Bessel functions of 1st kind (order 1)

      function mod_bessel_I1(x)
      implicit none
      real(wp) :: x
      real(wp) :: mod_bessel_I1

      real(8) :: y, ax
      real(8), parameter :: & 
           P1 = 0.5D0, &
           P2 = 0.87890594D0, &
           P3 = 0.51498869D0, & 
           P4 = 0.15084934D0, &
           P5 = 0.2658733D-1, &
           P6 = 0.301532D-2, &
           P7 = 0.32411D-3, &
           Q1 = 0.39894228D0, &
           Q2 = -0.3988024D-1, &
           Q3 = -0.362018D-2, &
           Q4 = 0.163801D-2, &
           Q5 = -0.1031555D-1, &
           Q6 = 0.2282967D-1, &
           Q7 = -0.2895312D-1, &
           Q8 = 0.1787654D-1, &
           Q9 = -0.420059D-2

      if(abs(x) < 3.75) then
         y = x*x / (3.75*3.75)
         mod_bessel_I1 = x*(P1+y*(P2+y*(P3+y*(P4+y*(P5+y*(P6+y*P7))))))
      else
         ax = abs(x)
         y = 3.75 / ax
         mod_bessel_I1 = (exp(ax)/sqrt(ax))*(Q1+y*(Q2+y*(Q3+         &
              y*(Q4+y*(Q5+y*(Q6+y*(Q7+y*(Q8+y*Q9))))))))
         if (x < 0.) mod_bessel_I1 = - mod_bessel_I1
      endif

      end function

!     --------------------------------------------------------------------------

      function  background_covariance_diva(x1,x2,param) result(c)
      implicit none
      real(wp),intent(in) :: x1(:),x2(:),param(:)
      real(wp) :: c

      real(wp) :: d(size(x1))
      real(wp) :: rn 

      d = (x1 - x2)*param
      rn = sqrt(sum(d**2))

      if (rn == 0) then
        c = 1
      else
        c = rn * mod_bessel_K1(rn)
      end if

      end function background_covariance_diva


!     --------------------------------------------------------------------------

      function  background_covariance_gaussian(x1,x2,param) result(c)
      implicit none
      real(wp),intent(in) :: x1(:),x2(:),param(:)
      real(wp) :: c

      real(wp) :: d(size(x1))

      d = (x1 - x2)*param
      c = exp(-sum(d**2))

      end function background_covariance_gaussian

!     --------------------------------------------------------------------------


      function  background_covariance(x1,x2,param) result(c)
      implicit none
      real(wp),intent(in) :: x1(:),x2(:),param(:)
      real(wp) :: c

#     if defined(BACKGROUND_COVARIANCE_DIVA)
      c = background_covariance_diva(x1,x2,param)
#     else
      c = background_covariance_gaussian(x1,x2,param)
#     endif

      end function background_covariance




!     --------------------------------------------------------------------------

      subroutine pinv (A, tolerance, work, D)

!     Compute pseudo-inverse A+ of symmetric A in factored form U D+ U', where
!     U overwrites A and D is diagonal matrix D+.

!     Saunders notes: Working with the factors of the pseudo-inverse is
!                     preferable to multiplying them together (more stable,
!                     less arithmetic).  Also, the choice of tolerance is
!                     not straightforward.  If A is noisy, try 1.e-2 * || A ||,
!                     else machine-eps. * || A ||  (1 norms).
!     Arguments:

      real(wp), intent (inout) :: A(:,:)  ! Upper triangle input; orthogonal U out
      real(wp), intent (in)    :: tolerance
      real(wp), intent (out)   :: work(:), D(:)

!     Local variables:

      integer :: i, info, N

!     Execution:

      N = size (A,1)

!     Eigendecomposition/SVD of symmetric A:

      call dsyev ('V', 'U', N, A, N, D, work, size (work), info)

!     Diagonal factor D+ of pseudo-inverse:

      do i = 1, N
         if (D(i) > tolerance) then
            D(i) = 1. / D(i)
         else
            D(i) = 0.
         end if
      end do


      end subroutine pinv

!     --------------------------------------------------------------------------

      function pinv_workspace(N) result(lwork)

!     Determine the workspace needed for dsyev.

      implicit none
      integer,intent(in) :: N
      integer :: lwork

      integer :: info
      real(wp) :: dummy,rwork

      call dsyev('V','U', N, dummy,N, dummy, rwork, -1, info )
      lwork = ceiling(rwork)

      end function

!     --------------------------------------------------------------------------

      subroutine optiminterp(ox,of,ovar,param,m,gx,gf,gvar)

!     Main optimal interpolation routine

      implicit none
      real(wp), intent(in)  :: ox(:,:), of(:,:)  ! Observations
      real(wp), intent(in)  :: ovar(:)           ! Observation error variances
      real(wp), intent(in)  :: param(:)          ! inverse of correlation lengths
      integer,  intent(in)  :: m                 ! # nearest observations used
      real(wp), intent(in)  :: gx(:,:)           ! Target grid coords.
      real(wp), intent(out) :: gf(:,:), gvar(:)  ! Interpolated fields.
                                                 ! and error variances
!     Local variables:

      real(wp) :: PH(m), PHiA(m),A(m,m),D(m)

#ifdef DIAG_OBS_COVAR
      real(wp) :: R(m)
#else
      real(wp) :: R(m,m)
#endif

      integer  :: gn,nf,index(m)
      real(wp) :: distance(m)

      integer  :: i,j1,j2,j,lwork
      real(wp) :: tolerance = 1e-5

#     ifdef VERBOSE
      integer  :: percentage_done
#     endif
#     ifdef STATIC_WORKSPACE
      real(wp) :: work((m+2)*m)
#     else
      real(wp), allocatable :: work(:)
#     endif

!     Execution:

      gn = size(gx,2)
      nf = size (of, 1)  ! # functions at each observation point

#     ifndef STATIC_WORKSPACE
!     query and allocate workspace for pseudo-inverse
      lwork = pinv_workspace(m)
#     endif

!$omp parallel  default(none) private(work,i,PH,PHiA,index,distance,j1,j2,R,D,A) &
!$omp shared(gf,gn,gvar,gx,lwork,m,of,ovar,ox,param,tolerance)

#     ifndef STATIC_WORKSPACE
      allocate(work(lwork))
#     endif
      
#     ifdef VERBOSE
      percentage_done = 0
#     endif

!$omp do 
      do i=1,gn
#       ifdef VERBOSE
        if (percentage_done .ne. int(100.*real(i)/real(gn))) then
          percentage_done = int(100.*real(i)/real(gn))
          write(6,*) 'done ',percentage_done
        end if
#       endif

!       get the indexes of the nearest observations

        call select_nearest(gx(:,i),ox,param,m,index,distance)
     
!       form compute the error covariance matrix of the observation 

        call observation_covariance(ovar,index,R)

!       form the error covariance matrix background field

        do j2 = 1, m
          ! Upper triangle only

           do j1 = 1, j2  
             A(j1,j2) = &
                 background_covariance(ox(:,index(j1)),ox(:,index(j2)),param)
           end do

           PH(j2) = background_covariance(gx(:,i),ox(:,index(j2)),param)
        end do

!       covariance matrix of the innovation

#ifdef DIAG_OBS_COVAR
        do j = 1, m
           A(j,j) = A(j,j) + R(j)
        end do
#else
        A  = A + R
#endif

!       pseudo inverse of the covariance matrix of the innovation

        call pinv(A,tolerance,work,D)

        PHiA = matmul (A, D * matmul (PH, A))

!       compute the analysis for all fields

        gf(:,i) = matmul(of(:,index),PHiA)

!       compute the error variance of the analysis

        gvar(i) = 1. - dot_product(PHiA,PH)

      end do
!$omp end do

#     ifndef STATIC_WORKSPACE
      deallocate(work)
#     endif
 
!$omp end parallel 


      end subroutine optiminterp

#ifdef FORTRAN_2003_INTEROP

      subroutine optiminterp_gw(n,gn,on,nparam,ox,of,ovar,
     &                           param,m,gx,gf,gvar) bind(c)
      USE ISO_C_BINDING
      implicit none
      integer(C_INT) :: m,n,gn,on,nparam
      real(C_DOUBLE) :: gx(n,gn),ox(n,on),of(on),ovar(on),param(nparam)
      real(C_DOUBLE) :: gf(gn),gvar(gn)


      call optiminterp(ox,of,ovar,param,m,gx,gf,gvar)
      
      end subroutine optiminterp_gw


#endif

! ----------------------------------------------------------- 
! nclfortstart
      subroutine drcm2points(ngrd,nyi,nxi,yi,xi,fi,nxyo,yo,xo,fo   &
                            ,xmsg,opt,ncrit,kval,ier)

      implicit none

      integer :: ngrd,nxi,nyi,nxyo,opt,ncrit,kval,ier
      real(wp) :: xi(nxi,nyi),yi(nxi,nyi),fi(nxi,nyi,ngrd)
      real(wp) :: xo(nxyo),yo(nxyo),fo(nxyo,ngrd),xmsg
! nclend

! This is written  with GNU f77 acceptable extensions
! .   This could be improved considerably with f90

! nomenclature:
! .   nxi,nyi - lengths of xi,yi and dimensions of fi (must be >= 2)
! .   xi      - coordinates of fi (eg, lon [2D] )
! .   yi      - coordinates of fi (eg, lat [2D] )
! .   fi      - functional input values [2D]
! .   nxyo    - number of output points
! .   xo      - lon coordinates of fo (eg, lon [1D])
! .   yo      - lat coordinates of fo (eg, lat [1D])
! .   fo      - functional output values [interpolated]
! .   xmsg    - missing code
! .   opt     - 0/1 = inv distance, 2 = bilinear
! .   ier     - error code
! .             =0;   no error
! .             =1;   not enough points in input/output array
! .             =2/3; xi or yi are not monotonically increasing
! .             =4/5; xo or yo are not monotonically increasing
!
!                              local
      integer :: ng,nx,ny,nxy,nexact,ix,iy,m,n,nw,ner,k
      real(wp) :: fw(2,2),w(2,2),sumf,sumw,chklat(nyi),chklon(nxi)
      real(wp) :: wx, wy
      real(wp) :: rearth, dlat, pi, rad, dkm, dist 
!                              error checking
      ier = 0
      if (nxi.le.1 .or. nyi.le.1 .or. nxyo.le.0) then
          ier = 1
          return
      end if
      if (ier.ne.0) return

      do ny = 1,nyi
          chklat(ny) = yi(1,ny)
!          print *,"chklat: ny=",ny,"  chklat=",chklat(ny)
      end do
      call dmonoinc(chklat,nyi,ier,ner)
      if (ier.ne.0) return

      do nx = 1,nxi
          chklon(nx) = xi(nx,1)
!          print *,"chklon: nx=",nx,"  chklon=",chklon(nx)
      end do
      call dmonoinc(chklat,nyi,ier,ner)
      if (ier.ne.0) return

! original  (k = op, never implemented)
      if (kval.le.0) then
         k = 1
      else
         k = kval
      end if
      do ng = 1,ngrd
        do nxy = 1,nxyo
           fo(nxy,ng) = xmsg
        end do
      end do
!                              main loop [exact matches]
      nexact = 0
      do nxy = 1,nxyo

          do iy = 1,nyi
              do ix = 1,nxi
                  if (xo(nxy).eq.xi(ix,iy) .and.   &
                      yo(nxy).eq.yi(ix,iy)) then
                      do ng = 1,ngrd
                         fo(nxy,ng) = fi(ix,iy,ng)
                         nexact     = nexact + 1
                      end do
                      go to 10
                  end if
              end do
          end do

   10     continue
      end do

!      print *, "nexact=",nexact
!                              main loop [interpolation]
      do nxy = 1,nxyo

              do iy = 1,nyi - k
                do ix = 1,nxi - k
                   if (xo(nxy).ge.xi(ix,iy) .and.   &
                       xo(nxy).le.xi(ix+k,iy) .and.   &
                       yo(nxy).ge.yi(ix,iy) .and.   &
                       yo(nxy).le.yi(ix,iy+k)) then

                   if (abs(opt).eq.2) then
                       wx = (xo(nxy)-xi(ix,iy))/   &
                            (xi(ix+k,iy)-xi(ix,iy))
                       wy = (yo(nxy)-yi(ix,iy))/   &
                            (yi(ix,iy+k)-yi(ix,iy))
                       w(1,1) = (1.d0-wx)*(1.d0-wy)
                       w(2,1) = wx*(1.d0-wy)
                       w(1,2) = (1.d0-wx)*wy
                       w(2,2) = wx*wy
                   else
                       w(1,1) = (1.d0/dgcdist(yo(nxy),xo(nxy),   &
                                 yi(ix,iy),xi(ix,iy),2))**2
                       w(2,1) = (1.d0/dgcdist(yo(nxy),xo(nxy),   &
                                 yi(ix+k,iy),xi(ix+k,iy),2))**2
                       w(1,2) = (1.d0/dgcdist(yo(nxy),xo(nxy),   &
                                 yi(ix,iy+k),xi(ix,iy+k),2))**2
                       w(2,2) = (1.d0/dgcdist(yo(nxy),xo(nxy),   &
                                 yi(ix+k,iy+k),xi(ix+k,iy+k),2))**2
                   end if

                   do ng = 1,ngrd
                      if (fo(nxy,ng).eq.xmsg) then
                          
                          fw(1,1) = fi(ix,iy,ng)
                          fw(2,1) = fi(ix+k,iy,ng)
                          fw(1,2) = fi(ix,iy+k,ng)
                          fw(2,2) = fi(ix+k,iy+k,ng)

                          nw = 0
                          sumf = 0.0d0
                          sumw = 0.0d0
                          do n = 1,2
                              do m = 1,2
                                  if (fw(m,n).ne.xmsg) then
                                      sumf = sumf + fw(m,n)*w(m,n)
                                      sumw = sumw + w(m,n)
                                      nw = nw + 1
                                  end if
                              end do
                          end do
!                                             nw >=3 arbitrary
                          if (nw.ge.ncrit .and. sumw.gt.0.d0) then
                              fo(nxy,ng) = sumf/sumw
                          end if
                      end if
                    end do
                    go to 20
                  end if
                end do
              end do

   20         continue
      end do

! Are all the output points filled in? Check the 1st grid
! If so, return

      do ng = 1,ngrd   
        do nxy = 1,nxyo
           if (fo(nxy,ng).eq.xmsg) go to 30
        end do
      end do
      return

! only enter if some points are not interpolated to
! dlat is arbitrary.  It ould be made an option.
! dlat is expressed in terms of degrees of latitude.
! dkm  is dlat in kilometers

   30 rearth= 6371d0
      dlat  = 5  
      pi    = 4d0*atan(1.0d0)
      rad   = pi/180d0
      dkm   = dlat*(2d0*pi*rearth)/360d0

! loop over each grid ... inefficient 
! the rub is that some levels could have xmsg.

      do ng = 1,ngrd   

        do nxy = 1,nxyo
           if(fo(nxy,ng).eq.xmsg) then

! find all grid points within 'dkm' kilometers of pt 

              nw   = 0
              sumf = 0.0d0
              sumw = 0.0d0

              do iy = 1,nyi
                do ix = 1,nxi
                   if ((yi(ix,iy).ge.yo(nxy)-dlat)  .and.   &
                       (yi(ix,iy).le.yo(nxy)+dlat)) then      
                        dist = dgcdist(yo(nxy),xo(nxy)   &
                                      ,yi(ix,iy),xi(ix,iy),2)
                        if (dist.le.dkm .and. dist.gt.0.0d0 .and.   &
                            fi(ix,iy,ng).ne.xmsg) then
                            dist = 1.0d0/dist**2
                            sumf = sumf + fi(ix,iy,ng)*dist
                            sumw = sumw + dist
                            nw   = nw + 1
                        end if
                   end if
                end do
              end do

!              if (nw.ge.ncrit .and. sumw.gt. 0.0d0) then
              if (sumw.gt.0.0d0) then
                  fo(nxy,ng) = sumf/sumw
              end if
           end if
        end do
      end do

      return
      end subroutine drcm2points

! ---------------------------------------------
      subroutine dmonoinc(x,nx,ner,ier)

      implicit none

! chk to make sure that x is monotonically increasing

      integer :: nx,ner,ier
      real(wp) :: x(nx)
!                          local
      integer :: n

      ier = 0
      if (nx.le.1) return

      do n = 1,nx - 1
          if (x(n+1).le.x(n)) then
              ier = ner
              return
          end if
      end do

      return
      end subroutine dmonoinc

! ---------------------------------------------
      function dgcdist(rlat1,rlon1,rlat2,rlon2,iu)

      implicit none
!     
! calculate the great circle distance between two points
!        
! usage: dist = gcdist (rlat1,rlon1,rlat2,rlon2,iu)
!     
! nomenclature :
! .   rlat1,rlon1 - latitude and longtitude of the first point
! .   rlat2,rlon2 - latitude and longtitude of the second point
! .   iu          - code for the type units gcdist is to return
! .               = 1 : gcdist returned in radians
! .               = 2 : gcdist returned in degrees
! .               = 3 : gcdist returned in meters
! .               = 4 : gcdist returned in kilometers
! .               = 5 : gcdist returned in *not used*
!
! input
      integer iu
! input types
      real(wp) :: rlat1,rlon1,rlat2,rlon2
! function value
      real(wp) :: dgcdist

! local stuff
      real(wp) :: units(5),rad,dlonr,rlat1r,rlat2r
      data units/1.0,57.29577995691645,6371220.,6371.2200,0./
! change as required
      data rad/0.01745329238474369/

! special test if RLAT1=RLAT2 and RLON1=RLON2
      if(rlat1.eq.rlat2.and.rlon1.eq.rlon2) then
         dgcdist = 0.
         return
      end if
      rlat1r = rlat1*rad
      rlat2r = rlat2*rad
      dlonr = min(abs(rlon1-rlon2),abs(360.-rlon1+rlon2),   &
              abs(360.-rlon2+rlon1))*rad

      dgcdist = atan2(sqrt((cos(rlat2r) * sin(dlonr)) ** 2 +   &
                           (cos(rlat1r) * sin(rlat2r) -   &
                            sin(rlat1r) * cos(rlat2r) * cos(dlonr)) ** 2   &
                          ),   &
                      sin(rlat1r)*sin(rlat2r)+   &
                      cos(rlat1r)*cos(rlat2r)*cos(dlonr)   &
                     ) * units(iu)

      return

      end function dgcdist

! ---------------------------------------------

      end module ice_assimil_oi
