module sat_vapor_pres_mod


use fms_mod,          only : stdout, mpp_pe, mpp_root_pe

implicit none
private

 public :: escomp, descomp, sat_vapor_pres_init
 public :: compute_es
 
 real(kind=8) , parameter :: TFREEZE = 273.16
 integer, parameter :: tcmin = -260  ! minimum temperature (degC) in lookup table
 integer, parameter :: tcmax =  100  ! maximum temperature (degC) in lookup table
 integer, parameter :: esres =  10   ! table resolution (increments per degree)
 integer, parameter :: nsize = (tcmax-tcmin)*esres+1    !  lookup table size
 integer, parameter :: nlim  = nsize-1

 real(kind=8)    :: tmin, tmax          !  lookup table limits in degK
 real(kind=8)    :: dtres, dtinv, teps

 real(kind=8)  ::   TABLE(nsize)    !  sat vapor pres (es)
 real(kind=8)  ::  DTABLE(nsize)    !  first derivative of es
 real(kind=8)  :: D2TABLE(nsize)    ! second derivative of es

 logical :: module_is_initialized = .FALSE.

!-----------------------------------------------------------------------

contains

!#######################################################################

 subroutine escomp ( temp, esat )
!   <IN NAME="temp" UNIT="degrees Kelvin" TYPE="real" DIM="(scalar)"></IN>
!   <OUT NAME="esat" UNITS="pascal" TYPE="real" DIM="(scalar)"></OUT>

 real(kind=8), intent(in)  :: temp
 real(kind=8), intent(out) :: esat

 real(kind=8)    :: tmp, del
 integer :: ind

   if (.not.module_is_initialized) call sat_vapor_pres_init

   tmp = temp-tmin
   ind = int(dtinv*(tmp+teps))
   del = tmp-dtres*real(ind)
   esat = TABLE(ind+1) + del*(DTABLE(ind+1) + del*D2TABLE(ind+1))

 end subroutine escomp

!#######################################################################

 subroutine descomp ( temp, desat )
!   <IN NAME="temp" UNIT="degrees Kelvin" TYPE="real" DIM="(scalar)"></IN>
!   <OUT NAME="desat" UNITS="pascal" TYPE="real" DIM="(scalar)"></OUT>

 real(kind=8), intent(in)  :: temp
 real(kind=8), intent(out) :: desat

 real(kind=8)    :: tmp, del
 integer :: ind

   if (.not.module_is_initialized) call sat_vapor_pres_init

   tmp = temp-tmin
   ind = int(dtinv*(tmp+teps))
   del = tmp-dtres*real(ind)
   desat = DTABLE(ind+1) + 2.*del*D2TABLE(ind+1)

 end subroutine descomp

!#######################################################################
 subroutine sat_vapor_pres_init

 real(kind=8)    :: tem(3), es(3), hdtinv
 integer :: i, n

! increment used to generate derivative table
  real(kind=8), parameter :: tinrc = .01           
  real(kind=8), parameter :: tfact = 1./(2.*tinrc)

     if (module_is_initialized) return   !!! return silently if this routine has already been called


! global variables
      tmin = real(tcmin)+TFREEZE   ! minimum valid temp in table
      tmax = real(tcmax)+TFREEZE   ! maximum valid temp in table
      dtinv = real(esres)
      dtres = 1./dtinv
      teps = 1./real(2*esres)
! local variables
      hdtinv = dtinv*0.5

! compute es tables from tcmin to tcmax
! estimate es derivative with small +/- difference

      do i = 1, nsize
         tem(1) = tmin + dtres*real(i-1)
         tem(2) = tem(1)-tinrc
         tem(3) = tem(1)+tinrc
         es = compute_es (tem)
          TABLE(i) = es(1)
         DTABLE(i) = (es(3)-es(2))*tfact
      enddo

! compute one-half second derivative using centered differences
! differencing des values in the table

      do i = 2, nsize-1
         D2TABLE(i) = 0.25*dtinv*(DTABLE(i+1)-DTABLE(i-1))
      enddo
    ! one-sided derivatives at boundaries
         D2TABLE(1)     = 0.50*dtinv*(DTABLE(2)    -DTABLE(1))
         D2TABLE(nsize) = 0.50*dtinv*(DTABLE(nsize)-DTABLE(nsize-1))

    module_is_initialized = .true.

    !!if ( mpp_pe().EQ.mpp_root_pe() )then
    !! do i = 1, nsize
    !!    write(stdout(),'(i6,f10.3,3f20.10)') i, 113.16+i*0.1, table(i), dtable(i), d2table(i)
    !! enddo
    !!endif  
 end subroutine sat_vapor_pres_init

!#######################################################################

 function compute_es (tem) result (es)
!   <IN NAME="temp" UNIT="degrees Kelvin" TYPE="real" DIM="(:)"></IN>
!   <OUT NAME="es" UNITS="pascal" TYPE="real" DIM="(:)"></OUT>
 real(kind=8), intent(in) :: tem(:)
 real(kind=8) :: es(size(tem,1))

 real(kind=8), parameter :: TBASW = TFREEZE+100.
 real(kind=8), parameter :: TBASI = TFREEZE
 real(kind=8), parameter :: ESBASW = 101324.60
 real(kind=8), parameter :: ESBASI =    610.71

 real(kind=8)    :: x, esice, esh2o
 integer :: i

   do i = 1, size(tem(:))

!  compute es over ice 

     if (tem(i) < TBASI) then
         x = -9.09718*(TBASI/tem(i)-1.0) - 3.56654*log10(TBASI/tem(i)) &
             +0.876793*(1.0-tem(i)/TBASI) + log10(ESBASI)
         esice =10.**(x)
     else
         esice = 0.
     endif

!  compute es over water greater than -20 c.
!  values over 100 c may not be valid
!  see smithsonian meteorological tables page 350.

     if (tem(i) > -20.+TBASI) then
         x = -7.90298*(TBASW/tem(i)-1) + 5.02808*log10(TBASW/tem(i)) &
             -1.3816e-07*(10**((1-tem(i)/TBASW)*11.344)-1)        &
             +8.1328e-03*(10**((TBASW/tem(i)-1)*(-3.49149))-1)    &
             +log10(ESBASW)
         esh2o = 10.**(x)
     else
         esh2o = 0.
     endif

!  derive blended es over ice and supercooled water between -20c and 0c

     if (tem(i) <= -20.+TBASI) then
         es(i) = esice
     else if (tem(i) >= TBASI) then
         es(i) = esh2o
     else
         es(i) = 0.05*((TBASI-tem(i))*esice + (tem(i)-TBASI+20.)*esh2o)
     endif

   enddo
   
 end function compute_es

!#######################################################################

end module sat_vapor_pres_mod
