#include <misc.h>
#include <preproc.h>

#if (defined MOZART2)

module VOC_GEOS_EmissionMod

!-----------------------------------------------------------------------
! Volatile organic compound emission
!
  use abortutils,           only : endrun
  use shr_kind_mod ,        only : r8 => shr_kind_r8
  use VOCemiss_initialize,  only : NORM_FAC, DAYS_BTW_M
  use clm_varcon,           only : istsoil, istice, istdlak, istwet, isturb

  implicit none

  private
  save
!
! Public interfaces 
!
  public :: VOC_GEOS_Emission
!
! Private data
!
    real(r8), pointer :: AEF_ISOP(:)      
    real(r8), pointer :: AEF_MBOX(:)      
    real(r8), pointer :: AEF_MYRC(:)       
    real(r8), pointer :: AEF_SABI(:)       
    real(r8), pointer :: AEF_APIN(:)       
    real(r8), pointer :: AEF_LIMO(:)       
    real(r8), pointer :: AEF_CARE(:)       
    real(r8), pointer :: AEF_BPIN(:)       
    real(r8), pointer :: AEF_OCIM(:)       
    real(r8), pointer :: AEF_OMON(:)       
    real(r8), pointer :: AEF_MOHX(:)       
    real(r8), pointer :: AEF_ACET(:)       
    real(r8), pointer :: AEF_ETOH(:)       
    real(r8), pointer :: AEF_CH2O(:)       
    real(r8), pointer :: AEF_ALD2(:)       
    real(r8), pointer :: AEF_FAXX(:)       
    real(r8), pointer :: AEF_AAXX(:)       
    real(r8), pointer :: AEF_C2H4(:)       
    real(r8), pointer :: AEF_TOLU(:)       
    real(r8), pointer :: AEF_HCNX(:)       
    real(r8), pointer :: AEF_PRPE(:)       
    real(r8), pointer :: AEF_FARN(:)       
    real(r8), pointer :: AEF_BCAR(:)       
    real(r8), pointer :: AEF_OSQT(:)       

    real(r8), pointer :: vocflx_ISOP(:)
    real(r8), pointer :: vocflx_MBOX(:)
    real(r8), pointer :: vocflx_MYRC(:)
    real(r8), pointer :: vocflx_SABI(:)
    real(r8), pointer :: vocflx_APIN(:)
    real(r8), pointer :: vocflx_LIMO(:)
    real(r8), pointer :: vocflx_CARE(:)
    real(r8), pointer :: vocflx_BPIN(:)
    real(r8), pointer :: vocflx_OCIM(:)
    real(r8), pointer :: vocflx_OMON(:)
    real(r8), pointer :: vocflx_MOHX(:)
    real(r8), pointer :: vocflx_ACET(:)
    real(r8), pointer :: vocflx_ETOH(:)
    real(r8), pointer :: vocflx_CH2O(:)
    real(r8), pointer :: vocflx_ALD2(:)
    real(r8), pointer :: vocflx_FAXX(:)
    real(r8), pointer :: vocflx_AAXX(:)
    real(r8), pointer :: vocflx_C2H4(:)
    real(r8), pointer :: vocflx_TOLU(:)
    real(r8), pointer :: vocflx_HCNX(:)
    real(r8), pointer :: vocflx_PRPE(:)
    real(r8), pointer :: vocflx_FARN(:)
    real(r8), pointer :: vocflx_BCAR(:)
    real(r8), pointer :: vocflx_OSQT(:)
    real(r8), pointer :: vocflx_OCPI(:)
    real(r8), pointer :: vocflx_MONO(:)

    real(r8), PARAMETER   :: WM2_TO_UMOLM2S = 4.766_r8 ! W/m2 -> umol/m2/s

contains

!**********************************************************************************

  subroutine VOC_GEOS_Emission (lbg, ubg, num_nolakep, filter_nolakep, &
                                 caldayp1, eccen, obliqr, lambm0, mvelpp)
 
!-----------------------------------------------------------------------
! Volatile organic compound emission
! This code simulates volatile organic compound emissions
! following the algorithm presented in Guenther, A., 1999: Modeling
! Biogenic Volatile Organic Compound Emissions to the Atmosphere. In
! Reactive Hydrocarbons in the Atmosphere, Ch. 3
! This model relies on the assumption that 90% of isoprene and monoterpene
! emissions originate from canopy foliage:
!    E = epsilon * gamma * density * delta
! The factor delta (longterm activity factor) applies to isoprene emission
! from deciduous plants only. We neglect this factor at the present time.
! This factor is discussed in Guenther (1997).
! Subroutine written to operate at the patch level.
! IN FINAL IMPLEMENTATION, REMEMBER:
! 1. may wish to call this routine only as freq. as rad. calculations
! 2. may wish to place epsilon values directly in pft-physiology file
! Output: vocflx(nvoc) !VOC flux [ug C m-2 h-1]
!
! !REVISION HISTORY:
! Author: Sam Levis
! 2/1/02, Peter Thornton: migration to new data structure
!-----------------------------------------------------------------------
    use shr_orb_mod,   only : shr_orb_cosz, shr_orb_decl
    use clmtype
    use time_manager,  only : get_curr_calday

!------------------------------Arguments--------------------------------
    integer, intent(in) :: lbg, ubg                    ! pft bounds
    integer, intent(in) :: num_nolakep                 ! number of column non-lake points in pft filter
    integer, intent(in) :: filter_nolakep(num_nolakep) ! pft filter for non-lake points

!---------------------------Local workspace-----------------------------

    integer , pointer :: pgridcell(:)     ! gridcell index of corresponding pft
    real(r8), pointer :: t_ref2m(:)       ! 2 m height surface air temperature (Kelvin)
    real(r8), pointer :: fsd24(:)         ! 24hr average of direct beam radiation
    real(r8), pointer :: fsd240(:)        ! 240hr average of direct beam radiation
    real(r8), pointer :: fsi24(:)         ! 24hr average of diffuse beam radiation
    real(r8), pointer :: fsi240(:)        ! 240hr average of diffuse beam radiation
    real(r8), pointer :: t_veg24(:)       ! 24hr average vegetation temperature (K)
    real(r8), pointer :: t_veg240(:)      ! 240hr average vegetation temperature (K)
    real(r8), pointer :: elai_p(:)        ! 24hr average leaf area index 
    real(r8), pointer :: elai_p2(:)       ! 48hr average leaf area index 
    real(r8), pointer :: lat(:)           ! gridcell latitude (radians)
    real(r8), pointer :: lon(:)           ! gridcell longitude (radians)
    real(r8), pointer :: h2osoi(:,:)      ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
    real(r8), pointer :: wiltsm(:,:)      ! wilting point soil moisture (nlevsoi)
    real(r8), pointer :: fieldsm(:,:)      ! wilting point soil moisture (nlevsoi)
    integer , pointer :: ityplun(:)       ! landunit type
    integer , pointer :: ivt(:)           ! pft vegetation type
    integer , pointer :: plandunit(:)     ! landunit of corresponding pft
    real(r8), pointer :: frac_sno(:)      ! fraction of ground covered by snow (0 to 1)
    integer , pointer :: pcolumn(:)       ! column of corresponding pft

! Conversion factors for OC/BC
    REAL(r8), PARAMETER   :: FC1 = 136.2364_r8 / 120.11_r8
! Fraction of yield of OC (hydrophilic) from terpene emissions
    REAL(r8), PARAMETER   :: EMMT2OC = 1.0e-1_r8

    REAL(r8), PARAMETER   :: ISOP_SCALING = 1.0  !?? not find in GEOSCHEM -- zhanglin
!
! !OTHER LOCAL VARIABLES:
!
    integer  :: fp,p,g,c         ! indices

! Divide by 2 in the equation to get dry weight foliar mass from grams carbon

    real(r8) :: delta, eccf 
    real(r8) :: coszen(lbg:ubg)      ! cosine solar zenith angle for next time step (gridcell level)
    integer  :: RC
    real(r8), intent(in) :: caldayp1 ! calendar day at Greenwich (1.00, ..., 365.99)
    real(r8), intent(in) :: eccen    ! Earth's orbital eccentricity
    real(r8), intent(in) :: obliqr   ! Earth's obliquity in radians
    real(r8), intent(in) :: lambm0   ! Mean longitude of perihelion at the vernal equin
    real(r8), intent(in) :: mvelpp   ! Earth's moving vernal equinox long. of perihelio
    real(r8) :: lai, plai
    integer  :: l, itype, itype_veg
!-----------------------------------------------------------------------

    ! Assign local pointers to derived type members (gridcell-level)


    ! Assign local pointers to derived subtypes components (pft-level)

    AEF_ISOP   => clm3%g%l%c%p%pef%AEF_ISOP
    AEF_MBOX   => clm3%g%l%c%p%pef%AEF_MBOX
    AEF_MYRC   => clm3%g%l%c%p%pef%AEF_MYRC
    AEF_SABI   => clm3%g%l%c%p%pef%AEF_SABI
    AEF_APIN   => clm3%g%l%c%p%pef%AEF_APIN
    AEF_LIMO   => clm3%g%l%c%p%pef%AEF_LIMO
    AEF_CARE   => clm3%g%l%c%p%pef%AEF_CARE
    AEF_BPIN   => clm3%g%l%c%p%pef%AEF_BPIN
    AEF_OCIM   => clm3%g%l%c%p%pef%AEF_OCIM
    AEF_OMON   => clm3%g%l%c%p%pef%AEF_OMON
    AEF_MOHX   => clm3%g%l%c%p%pef%AEF_MOHX
    AEF_ACET   => clm3%g%l%c%p%pef%AEF_ACET
    AEF_ETOH   => clm3%g%l%c%p%pef%AEF_ETOH
    AEF_CH2O   => clm3%g%l%c%p%pef%AEF_CH2O
    AEF_ALD2   => clm3%g%l%c%p%pef%AEF_ALD2
    AEF_FAXX   => clm3%g%l%c%p%pef%AEF_FAXX
    AEF_AAXX   => clm3%g%l%c%p%pef%AEF_AAXX
    AEF_C2H4   => clm3%g%l%c%p%pef%AEF_C2H4
    AEF_TOLU   => clm3%g%l%c%p%pef%AEF_TOLU
    AEF_HCNX   => clm3%g%l%c%p%pef%AEF_HCNX
    AEF_PRPE   => clm3%g%l%c%p%pef%AEF_PRPE
    AEF_FARN   => clm3%g%l%c%p%pef%AEF_FARN
    AEF_BCAR   => clm3%g%l%c%p%pef%AEF_BCAR
    AEF_OSQT   => clm3%g%l%c%p%pef%AEF_OSQT

    vocflx_ISOP=> clm3%g%l%c%p%pvf%vocflx_ISOP
    vocflx_MBOX=> clm3%g%l%c%p%pvf%vocflx_MBOX
    vocflx_MYRC=> clm3%g%l%c%p%pvf%vocflx_MYRC
    vocflx_SABI=> clm3%g%l%c%p%pvf%vocflx_SABI
    vocflx_APIN=> clm3%g%l%c%p%pvf%vocflx_APIN
    vocflx_LIMO=> clm3%g%l%c%p%pvf%vocflx_LIMO
    vocflx_CARE=> clm3%g%l%c%p%pvf%vocflx_CARE
    vocflx_BPIN=> clm3%g%l%c%p%pvf%vocflx_BPIN
    vocflx_OCIM=> clm3%g%l%c%p%pvf%vocflx_OCIM
    vocflx_OMON=> clm3%g%l%c%p%pvf%vocflx_OMON
    vocflx_MOHX=> clm3%g%l%c%p%pvf%vocflx_MOHX
    vocflx_ACET=> clm3%g%l%c%p%pvf%vocflx_ACET
    vocflx_ETOH=> clm3%g%l%c%p%pvf%vocflx_ETOH
    vocflx_CH2O=> clm3%g%l%c%p%pvf%vocflx_CH2O
    vocflx_ALD2=> clm3%g%l%c%p%pvf%vocflx_ALD2
    vocflx_FAXX=> clm3%g%l%c%p%pvf%vocflx_FAXX
    vocflx_AAXX=> clm3%g%l%c%p%pvf%vocflx_AAXX
    vocflx_C2H4=> clm3%g%l%c%p%pvf%vocflx_C2H4
    vocflx_TOLU=> clm3%g%l%c%p%pvf%vocflx_TOLU
    vocflx_HCNX=> clm3%g%l%c%p%pvf%vocflx_HCNX
    vocflx_PRPE=> clm3%g%l%c%p%pvf%vocflx_PRPE
    vocflx_FARN=> clm3%g%l%c%p%pvf%vocflx_FARN
    vocflx_BCAR=> clm3%g%l%c%p%pvf%vocflx_BCAR
    vocflx_OSQT=> clm3%g%l%c%p%pvf%vocflx_OSQT
    vocflx_OCPI=> clm3%g%l%c%p%pvf%vocflx_OCPI
    vocflx_MONO=> clm3%g%l%c%p%pvf%vocflx_MONO

    pgridcell  => clm3%g%l%c%p%gridcell
    lat        => clm3%g%lat
    lon        => clm3%g%lon
    ityplun    => clm3%g%l%itype
    ivt        => clm3%g%l%c%p%itype
    plandunit  => clm3%g%l%c%p%landunit
    frac_sno   => clm3%g%l%c%cps%frac_sno
    pcolumn    => clm3%g%l%c%p%column

!zf    t_ref2m        => clm3%g%l%c%p%pes%t_ref2m
    t_ref2m        => clm3%g%l%c%p%pes%t_veg      !zf 2016.05.20
    fsd24          => clm3%g%l%c%p%pvs%fsd24
    fsd240         => clm3%g%l%c%p%pvs%fsd240
    fsi24          => clm3%g%l%c%p%pvs%fsi24 
    fsi240         => clm3%g%l%c%p%pvs%fsi240
    t_veg24        => clm3%g%l%c%p%pvs%t_veg24
    t_veg240       => clm3%g%l%c%p%pvs%t_veg240
    elai_p         => clm3%g%l%c%p%pvs%elai_p
    elai_p2        => clm3%g%l%c%p%pvs%elai_p2
    h2osoi         => clm3%g%l%c%cws%h2osoi_vol
    wiltsm         => clm3%g%l%c%cps%wiltsm
    fieldsm        => clm3%g%l%c%cps%fieldsm

!--------

    call shr_orb_decl (caldayp1, eccen, mvelpp, lambm0, obliqr, delta, eccf)

!dir$ concurrent
!cdir nodep
    do g = lbg, ubg
       coszen(g) = shr_orb_cosz (caldayp1, lat(g), lon(g), delta)
    end do

    do fp = 1,num_nolakep
       p = filter_nolakep(fp)
       g = pgridcell(p)
       c = pcolumn(p)
       l = plandunit(p)
       itype     = ityplun(l)
       itype_veg = ivt(p) 
       lai  = elai_p(p)                            ! 24hr mean 
       plai = elai_p2(p)*2 - elai_p(p)             ! 24hr mean in the day before  

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'ISOP',      vocflx_ISOP(p), fieldsm(c,5),RC )
       vocflx_ISOP(p) = vocflx_ISOP(p) * ISOP_SCALING   ! ISOP (kgC/m2/s)

!       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
!                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
!                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
!                                 plai,      'MBOX',      vocflx_MBOX(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'APIN',      vocflx_APIN(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'BPIN',      vocflx_BPIN(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'LIMO',      vocflx_LIMO(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'SABI',      vocflx_SABI(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'MYRC',      vocflx_MYRC(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'CARE',      vocflx_CARE(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'OCIM',      vocflx_OCIM(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'OMON',      vocflx_OMON(p), fieldsm(c,5),RC )

      ! ---------------------------------------------------
      ! Total monoterpenes = sum of individual
      ! dbm, now add other lumped monoterpenes (11/2012)
      !  [kg C/m2/s]

       vocflx_MONO(p) = vocflx_APIN(p) + vocflx_BPIN(p) + vocflx_LIMO(p) + vocflx_SABI(p)    &
                      + vocflx_MYRC(p) + vocflx_CARE(p) + vocflx_OCIM(p) + vocflx_OMON(p)

       vocflx_LIMO(p) = vocflx_LIMO(p) * FC1    ![kg C/m2/s]


!       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
!                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
!                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
!                                 plai,      'ALD2',      vocflx_ALD2(p), fieldsm(c,5),RC )

!       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
!                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
!                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
!                                 plai,      'MOHX',      vocflx_MOHX(p), fieldsm(c,5),RC )

      ! (3) BIOGENIC ACETONE -- DIRECT EMISSION 
      ! evf, removed obsolete code, replaced with MEGAN acetone
      ! emissions (5/25/2011) Direct Emission now includes emission
      ! from grasses and emission from dry leaf matter

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'ACET',      vocflx_ACET(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'PRPE',      vocflx_PRPE(p), fieldsm(c,5),RC )

       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
                                 plai,      'C2H4',      vocflx_C2H4(p), fieldsm(c,5),RC )

     ! Convert monoterpenes into hydrophilic OC and pass to
     ! OC emissions array 
     ! (kg/m2/s)

       vocflx_OCPI(p) = vocflx_MONO(p) * EMMT2OC

!       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
!                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
!                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
!                                 plai,      'FARN',      vocflx_FARN(p), fieldsm(c,5),RC )

!       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
!                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
!                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
!                                 plai,      'BCAR',      vocflx_BCAR(p), fieldsm(c,5),RC )

!       call get_megan_emissions( p,          itype,      itype_veg,      frac_sno(c), wiltsm(c,5),   &
!                                 coszen(g),  lai,        t_ref2m(p),     fsd24(p),    h2osoi(c,5),   &
!                                 fsi24(p),   fsd240(p),  fsi240(p),      t_veg24(p),  t_veg240(p),   &
!                                 plai,      'OSQT',      vocflx_OSQT(p), fieldsm(c,5),RC )

    end do   ! end pft loop

  end subroutine VOC_GEOS_Emission

!********************************************************************************

  subroutine get_megan_emissions( fp,     itype,  itype_veg,  frac_sno, wiltsm,     &
                                  coszen, alai,   t_ref2m,    fsd24,    h2osoi,     &
                                  fsi24,  fsd240, fsi240,     t_veg24,  t_veg240,   &
                                  elai_p, CMPD,   MEGAN_EMIS, fieldsm,  RC )

!--------------------------------------------------------------------------------
!  (1 ) Guenther et al, 1995, 1999, 2000, 2004, 2006
!  (2 ) Wang,    et al, 1998
!  (3 ) Guenther et al, 2007, MEGAN v2.1 User mannual
!  (4 ) Guenther et al, 2012 GMD MEGANv2.1 description and associated code at
!                                http://acd.ucar.edu/~guenther/MEGAN/
!
! !REVISION HISTORY:
!  (1 ) Original code by Dorian Abbot (9/2003).  Updated to the latest
!        algorithm and modified for the standard code by May Fu (11/20/04)
!  (2 ) All MEGAN biogenic emission are currently calculated using TS from DAO
!        met field. TS is the surface air temperature, which should be
!        carefully distinguished from TSKIN. (tmf, 11/20/04)
!  (3 ) Restructing of function & implementation of activity factors (mpb,2009)
!  17 Dec 2009 - R. Yantosca - Added ProTeX headers
!  11 Apr 2012 - R. Yantosca - Now use data from modis_lai_mod.F90
!  11 Apr 2012 - R. Yantosca - Cosmetic changes
!  26 Jan 2015 - M. Sulprizio- Update from D. Millet (21 Jun 2012): New driver
!                              routine for all MEGAN compounds
!--------------------------------------------------------------------------------

!------------------------------Arguments-----------------------------------------
    integer, intent(in)          :: fp
    character(len=*), intent(in) :: CMPD
    real(r8), intent(in)         :: frac_sno
    real(r8), intent(in)         :: alai
    real(r8), intent(in)         :: elai_p
    real(r8), intent(in)         :: t_ref2m
    real(r8), intent(in)         :: fsd24
    real(r8), intent(in)         :: fsi24
    real(r8), intent(in)         :: h2osoi
    real(r8), intent(in)         :: wiltsm
    real(r8), intent(in)         :: fieldsm
    real(r8), intent(in)         :: coszen  
    real(r8), intent(in)         :: fsd240 
    real(r8), intent(in)         :: fsi240
    real(r8), intent(in)         :: t_veg24
    real(r8), intent(in)         :: t_veg240
    integer, intent(in)          :: itype
    integer, intent(in)          :: itype_veg

    real(r8), intent(out)   :: MEGAN_EMIS ! VOC emission in kgC/m2/s or kg/m2/s, depending on
                                          ! units the compound is carried in
    integer, INTENT(INOUT)  :: RC

!------------------------------Local workspace-----------------------------------
 
!zf     real(r8), parameter   :: t_standard = 303.d0
      real(r8), parameter   :: t_standard = 297._r8    !zf
      REAL(r8)  :: GAMMA_LAI
      REAL(r8)  :: GAMMA_AGE
      REAL(r8)  :: GAMMA_PAR
      REAL(r8)  :: GAMMA_T_LD
      REAL(r8)  :: GAMMA_T_LI
      REAL(r8)  :: GAMMA_SM
      REAL(r8)  :: AEF
      REAL(r8)  :: D_BTW_M
      REAL(r8)  :: TS, SUNCOS
      REAL(r8)  :: Q_DIR_2, Q_DIFF_2
      REAL(r8)  :: BETA, LDF, CT1, CEO
      REAL(r8)  :: ANEW, AGRO, AMAT, AOLD
      REAL(r8)  :: ISOLAI, PMISOLAI, MISOLAI
      LOGICAL   :: BIDIR

      real(r8)  :: HCOT_15
      real(r8)  :: HCOT_15_AVG
      real(r8)  :: HCO_SUCCESS   !??????????? 
      real(r8)  :: HCOPARDR_15_AVG
      real(r8)  :: HCOPARDF_15_AVG

!--------------------------------------------------------------------------------

      if( (itype == istsoil .and. itype_veg == 0)   .or. &
          (itype == istice) .or. (itype == istdlak) .or. &
          (itype == istwet) .or. (itype == isturb ) )    then
          MEGAN_EMIS = 0.0 
          return
      endif

! Initialize parameters, gamma values, and return value

      MEGAN_EMIS = 0.0_r8
      GAMMA_LAI  = 0.0_r8
      GAMMA_AGE  = 0.0_r8
      GAMMA_T_LD = 0.0_r8
      GAMMA_T_LI = 0.0_r8
      GAMMA_PAR  = 0.0_r8
      GAMMA_SM   = 0.0_r8
      BETA       = 0.0_r8
      AEF        = 0.0_r8
      LDF        = 0.0_r8
      CT1        = 0.0_r8
      CEO        = 0.0_r8
      ANEW       = 0.0_r8
      AGRO       = 0.0_r8
      AMAT       = 0.0_r8
      AOLD       = 0.0_r8
      BIDIR      = .FALSE.

      HCOT_15         = t_veg24
      HCOPARDR_15_AVG = fsd240
      HCOPARDF_15_AVG = fsi240
      HCOT_15_AVG     = t_veg240

! Number of days between MISOLAI and PMISOLAI
      D_BTW_M  =  real(DAYS_BTW_M) 

! Pass met variables. Now use only LAI (ckeller, 10/9/2014)
      ISOLAI   = alai      !ExtState%LAI%Arr%Val(I,J)
      PMISOLAI = elai_p    !LAI_PREVDAY(I,J)  ?????????????
      MISOLAI  = ISOLAI
      TS       = t_ref2m !ExtState%T2M%Arr%Val(I,J)
      SUNCOS   = coszen  !ExtState%SUNCOS%Arr%Val(I,J)

! Convert Q_DIR and Q_DIFF from (W/m2) to (micromol/m2/s)
!      Q_DIR_2  = ExtState%PARDR%Arr%Val(I,J) * WM2_TO_UMOLM2S
!      Q_DIFF_2 = ExtState%PARDF%Arr%Val(I,J) * WM2_TO_UMOLM2S
      Q_DIR_2  = fsd24 * WM2_TO_UMOLM2S
      Q_DIFF_2 = fsi24 * WM2_TO_UMOLM2S

! --------------------------------------------
! Get MEGAN parameters for this compound
! --------------------------------------------
      CALL GET_MEGAN_PARAMS ( CMPD, BETA, LDF,  CT1,  CEO,            &
                              ANEW, AGRO, AMAT, AOLD, BIDIR )

! ------------------------------------------------------------
! Get base emission factor for this compound and grid square 
! Units: kgC/m2/s or kg/m2/s
! ------------------------------------------------------------
      CALL GET_MEGAN_AEF ( CMPD, fp, AEF )

!-----------------------------------------------------
! Only interested in terrestrial biosphere
! If (local LAI != 0 .AND. baseline emission !=0 )
!-----------------------------------------------------
    IF ( ISOLAI * AEF > 0.0_r8 ) THEN

! --------------------------------------------------
! GAMMA_par (light activity factor)
! --------------------------------------------------

! Calculate GAMMA PAR only during day
      IF ( SUNCOS > 0.0_r8 ) THEN

        GAMMA_PAR = GET_GAMMA_PAR_PCEEA( SUNCOS, Q_DIR_2,  Q_DIFF_2,     &
                                        HCOPARDR_15_AVG, HCOPARDF_15_AVG )

      ELSE

        ! If night
        GAMMA_PAR = 0.0_r8

      ENDIF

! ------------------------------------------------------------------------
! GAMMA_T_LI (temperature activity factor for light-independent fraction)
! ------------------------------------------------------------------------
!zf         GAMMA_T_LI = GET_GAMMA_T_LI( TS, BETA )   
      GAMMA_T_LI = exp(beta*(ts - t_standard))         !zf 

! ------------------------------------------------------------------------
! GAMMA_T_LD (temperature activity factor for light-dependent fraction)
! ------------------------------------------------------------------------
         GAMMA_T_LD = GET_GAMMA_T_LD( TS, HCOT_15_AVG,     &
              HCOT_15, CT1, CEO )

! ------------------------------------------------------------------------
! GAMMA_LAI (leaf area index activity factor)
! ------------------------------------------------------------------------
         GAMMA_LAI = GET_GAMMA_LAI( MISOLAI, BIDIR )

! --------------------------------------------------
! GAMMA_AGE (leaf age activity factor)
! --------------------------------------------------
         GAMMA_AGE = GET_GAMMA_AGE( MISOLAI, PMISOLAI,         &
              D_BTW_M, HCOT_15_AVG, ANEW, AGRO, AMAT, AOLD )

! --------------------------------------------------
! GAMMA_SM (soil moisture activity factor)
! --------------------------------------------------
!  GAMMA_SM = GET_GAMMA_SM( ExtState, I, J, CMPD )

!zf      if ( trim(CMPD) == 'ISOP' ) then
        gamma_sm = get_gamma_sm(h2osoi, wiltsm, fieldsm)
!zf      else
!zf        GAMMA_SM = 1.0
!zf      endif

    ELSE

! set activity factors to zero
      GAMMA_PAR  = 0.0_r8
      GAMMA_T_LI = 0.0_r8
      GAMMA_T_LD = 0.0_r8
      GAMMA_LAI  = 0.0_r8
      GAMMA_AGE  = 0.0_r8
      GAMMA_SM   = 0.0_r8

    END IF

!-----------------------------------------------------------------------------
! Emission is the product of all of these. Units here are kgC/m2/s or kg/m2/s
! as appropriate for the compound. Normalization factor ensures product of 
! GAMMA values is 1.0 under standard conditions.
!-----------------------------------------------------------------------------

    MEGAN_EMIS = NORM_FAC * AEF * GAMMA_AGE * GAMMA_SM * GAMMA_LAI     &
                 * ((1.0_r8 - LDF) * GAMMA_T_LI +                      &
                   (LDF * GAMMA_PAR * GAMMA_T_LD))

    MEGAN_EMIS = MEGAN_EMIS*(1.0-frac_sno)

! Leave w/ success
    RC = HCO_SUCCESS

 END SUBROUTINE GET_MEGAN_EMISSIONS

!******************************************************************

      SUBROUTINE GET_MEGAN_PARAMS( CPD,   BTA,   LIDF,  C_T1,  C_EO,        &
                                   A_NEW, A_GRO, A_MAT, A_OLD, BI_DIR )

!----------------------------------------------------------------------------
! Returns the emission parameters for each MEGAN compound needed
! to compute emissions. Called from get_megan_emissions
! References (see above for full citations):
!  (1 ) Guenther et al, (GMD 2012) and associated MEGANv2.1 source code
!
! !REVISION HISTORY:
!  (1 ) Created by dbm 07/2012
!----------------------------------------------------------------------------
 
!------------------------------Arguments-------------------------------------
      CHARACTER(LEN=*), INTENT(IN) :: CPD    ! Compound name
 
      REAL(r8), INTENT(INOUT) :: BTA    ! Beta coefficient for temperature activity
                                        ! factor for light-independent fraction
      REAL(r8), INTENT(INOUT) :: LIDF   ! Light-dependent fraction of emissions
      REAL(r8), INTENT(INOUT) :: C_T1   ! CT1 parameter for temperature activity
                                        ! factor for light-dependent fraction
      REAL(r8), INTENT(INOUT) :: C_EO   ! Ceo parameter for temperature activity
                                        ! factor for light-dependent fraction
      REAL(r8), INTENT(INOUT) :: A_NEW  ! Relative emission factor (new leaves)
      REAL(r8), INTENT(INOUT) :: A_GRO  ! Relative emission factor (growing leaves)
      REAL(r8), INTENT(INOUT) :: A_MAT  ! Relative emission factor (mature leaves)
      REAL(r8), INTENT(INOUT) :: A_OLD  ! Relative emission factor (old leaves)
      LOGICAL,  INTENT(INOUT) :: BI_DIR ! Logical flag to indicate bidirectional exchange
 
!---------------------------Local workspace-----------------------------------------
      CHARACTER(LEN=255):: MSG

!------------------------------------------------------------------------------
! Initialize values

      BTA    = 0.0_r8
      LIDF   = 0.0_r8
      C_T1   = 0.0_r8
      C_EO   = 0.0_r8
      A_NEW  = 0.0_r8
      A_GRO  = 0.0_r8
      A_MAT  = 0.0_r8
      A_OLD  = 0.0_r8
      BI_DIR = .FALSE.

      ! ----------------------------------------------------------------
      ! Note that not all the above compounds are used in standard chemistry
      ! simulations, but they are provided here for future incorporation or
      ! specialized applications. More compounds can be added as needed
      ! by adding the corresponding CPD name and the appropriate paramaters.
      ! (dbm, 01/2013)
      !
      ! Values are from Table 4 in Guenther et al., 2012
      ! ----------------------------------------------------------------

      ! Isoprene, MBO
      IF ( TRIM(CPD) == 'ISOP' .OR.                              &
           TRIM(CPD) == 'MBOX' ) THEN
         BTA    = 0.13_r8  ! Not actually used for ISOP, MBO
         LIDF   = 1.0_r8
         C_T1   = 95.0_r8
         C_EO   = 2.0_r8
         A_NEW  = 0.05_r8
         A_GRO  = 0.6_r8
         A_MAT  = 1.0_r8
         A_OLD  = 0.9_r8
         BI_DIR = .FALSE.

      ! Myrcene, sabinene, alpha-pinene
      ELSE IF ( TRIM(CPD) == 'MYRC' .OR.          &
                TRIM(CPD) == 'SABI' .OR.          &
                TRIM(CPD) == 'APIN' ) THEN
         BTA    = 0.10_r8
         LIDF   = 0.6_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 2.0_r8
         A_GRO  = 1.8_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.05_r8
         BI_DIR = .FALSE.

      ! Limonene, 3-carene, beta-pinene
      ELSE IF ( TRIM(CPD) == 'LIMO' .OR.              &
                TRIM(CPD) == 'CARE' .OR.              &
                TRIM(CPD) == 'BPIN' ) THEN
         BTA    = 0.10_r8
         LIDF   = 0.2_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 2.0_r8
         A_GRO  = 1.8_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.05_r8
         BI_DIR = .FALSE.

      ! t-beta-ocimene
      ELSE IF ( TRIM(CPD) == 'OCIM' ) THEN
         BTA    = 0.10_r8
         LIDF   = 0.8_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 2.0_r8
         A_GRO  = 1.8_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.05_r8
         BI_DIR = .FALSE.

      ! Other monoterpenes (lumped)
      ELSE IF ( TRIM(CPD) == 'OMON' ) THEN
         BTA    = 0.10_r8
         LIDF   = 0.4_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 2.0_r8
         A_GRO  = 1.8_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.05_r8
         BI_DIR = .FALSE.

      ! Methanol
      ELSE IF ( TRIM(CPD) == 'MOHX' ) THEN
         BTA    = 0.08_r8
         LIDF   = 0.8_r8
         C_T1   = 60.0_r8
         C_EO   = 1.6_r8
         A_NEW  = 3.5_r8
         A_GRO  = 3.0_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.2_r8
         BI_DIR = .FALSE.

      ! Acetone
      ELSE IF ( TRIM(CPD) == 'ACET' ) THEN
         BTA    = 0.1_r8
         LIDF   = 0.2_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 1.0_r8
         A_GRO  = 1.0_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.0_r8
         BI_DIR = .FALSE.

      ! Bidirectional VOC: Ethanol, formaldehyde, acetaldehyde, formic acid,
      ! acetic acid
      ELSE IF ( TRIM(CPD) == 'ETOH' .OR.        &
                TRIM(CPD) == 'CH2O' .OR.        &
                TRIM(CPD) == 'ALD2' .OR.        &
                TRIM(CPD) == 'FAXX' .OR.        &
                TRIM(CPD) == 'AAXX' ) THEN
         BTA    = 0.13_r8
         LIDF   = 0.8_r8
         C_T1   = 95.0_r8
         C_EO   = 2.0_r8
         A_NEW  = 1.0_r8
         A_GRO  = 1.0_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.0_r8
         BI_DIR = .TRUE.

      ! Stress VOCs: ethene, toluene, HCN
      ! There are others species in this category but none are currently
      ! used in GEOS-Chem
      ELSE IF ( TRIM(CPD) == 'C2H4' .OR.       &
                TRIM(CPD) == 'TOLU' .OR.       &
                TRIM(CPD) == 'HCNX' ) THEN
         BTA    = 0.1_r8
         LIDF   = 0.8_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 1.0_r8
         A_GRO  = 1.0_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.0_r8
         BI_DIR = .FALSE.

      ! Other VOCs: >C2 alkenes
      ! This includes propene, butene and very minor contribution from
      ! larger alkenes
      ELSE IF ( TRIM(CPD) == 'PRPE' ) THEN
         BTA    = 0.1_r8
         LIDF   = 0.2_r8
         C_T1   = 80.0_r8
         C_EO   = 1.83_r8
         A_NEW  = 1.0_r8
         A_GRO  = 1.0_r8
         A_MAT  = 1.0_r8
         A_OLD  = 1.0_r8
         BI_DIR = .FALSE.

      ! SOAupdate: Sesquiterpenes hotp 3/2/10
      ! alpha-Farnesene, beta-Caryophyllene, other sesquiterpenes
      ELSE IF ( TRIM(CPD) == 'FARN' .OR.          &
                TRIM(CPD) == 'BCAR' .OR.          &
                TRIM(CPD) == 'OSQT' ) THEN
         BTA    = 0.17_r8
         LIDF   = 0.5_r8
         C_T1   = 130.0_r8
         C_EO   = 2.37_r8
         A_NEW  = 0.4_r8
         A_GRO  = 0.6_r8
         A_MAT  = 1.0_r8
         A_OLD  = 0.95_r8
         BI_DIR = .FALSE.

      ! Calls for any other MEGAN compounds (e.g. sesquiterpenes, CO, etc.) can
      ! be added following the above format based on the parameters in
      ! Guenther 2012 or the MEGAN source code (dbm, 6/21/2012).
      ELSE

         MSG = 'Invalid compound name'
         call endrun()

      ENDIF
      END SUBROUTINE GET_MEGAN_PARAMS

!******************************************************************

      SUBROUTINE GET_MEGAN_AEF( CPD, fp, EMFAC )

!------------------------------------------------------------------------------
! Returns the appropriate AEF value for a given compound and grid square.
! References (see above for full citations):
!  (1 ) Guenther et al, 2012, MEGANv2.1 source code
! REVISION HISTORY:
!  (1 ) Created 11/2012 by dbm
!------------------------------------------------------------------------------

!------------------------------Arguments---------------------------------------
      CHARACTER(LEN=*),  INTENT(IN)  :: CPD         ! Compound name
      integer, intent(in)            :: fp
      REAL(r8),          INTENT(OUT) :: EMFAC       ! MEGAN base emission factor for grid cell 

!-------------------------------Local workspace--------------------------------
      CHARACTER(LEN=255):: MSG
!------------------------------------------------------------------------------
! Find appropriate tracer

      SELECT CASE ( TRIM( CPD ) )
         CASE ( 'ISOP' )
            EMFAC = AEF_ISOP(fp)
         CASE ( 'MBOX' )
            EMFAC = AEF_MBOX(fp)
         CASE ( 'MYRC' )
            EMFAC = AEF_MYRC(fp)
         CASE ( 'SABI' )
            EMFAC = AEF_SABI(fp)
         CASE ( 'APIN' )
            EMFAC = AEF_APIN(fp)
         CASE ( 'LIMO' )
            EMFAC = AEF_LIMO(fp)
         CASE ( 'CARE' )
            EMFAC = AEF_CARE(fp)
         CASE ( 'BPIN' )
            EMFAC = AEF_BPIN(fp)
         CASE ( 'OCIM' )
            EMFAC = AEF_OCIM(fp)
         CASE ( 'OMON' )
            EMFAC = AEF_OMON(fp)
         CASE ( 'MOHX' )
            EMFAC = AEF_MOHX(fp)
         CASE ( 'ACET' )
            EMFAC = AEF_ACET(fp)
         CASE ( 'ETOH' )
            EMFAC = AEF_ETOH(fp)
         CASE ( 'CH2O' )
            EMFAC = AEF_CH2O(fp)
         CASE ( 'ALD2' )
            EMFAC = AEF_ALD2(fp)
         CASE ( 'FAXX' )
            EMFAC = AEF_FAXX(fp)
         CASE ( 'AAXX' )
            EMFAC = AEF_AAXX(fp)
         CASE ( 'C2H4' )
            EMFAC = AEF_C2H4(fp)
         CASE ( 'TOLU' )
            EMFAC = AEF_TOLU(fp)
         CASE ( 'HCNX' )
            EMFAC = AEF_HCNX(fp)
         CASE ( 'PRPE' )
            EMFAC = AEF_PRPE(fp)
         CASE ( 'FARN' )
            EMFAC = AEF_FARN(fp)
         CASE ( 'BCAR' )
            EMFAC = AEF_BCAR(fp)
         CASE ( 'OSQT' )
            EMFAC = AEF_OSQT(fp)
         CASE DEFAULT
            MSG = 'Invalid compound name'
            call endrun()
         END SELECT

      END SUBROUTINE GET_MEGAN_AEF

!************************************************

 real function GET_GAMMA_PAR_PCEEA( SUNCOS, Q_DIR_2, Q_DIFF_2, HCOPARDR_AVG_SIM, HCOPARDF_AVG_SIM )

!----------------------------------------------------------------
! Computes the PCEEA gamma activity factor with sensitivity to LIGHT.
! REMARKS:
! References (see above for full citations):
!  (1 ) Guenther et al, 2006
!  (2 ) Guenther et al, 2007, MEGAN v2.1 user guide
!
! REVISION HISTORY:
!  (1 ) Here PAR*_AVG_SIM is the average light conditions over the simulation
!       period. I've set this = 10 days to be consistent with temperature & as
!       outlined in Guenther et al, 2006. (mpb,2009)
!  (2 ) Code was taken & adapted directly from the MEGAN v2.1 source code.
!       (mpb,2009)
!  17 Dec 2009 - R. Yantosca - Added ProTeX headers
!  01 Mar 2012 - R. Yantosca - Now use GET_YMID(I,J,L) from grid_mod.F90
!  01 Mar 2012 - R. Yantosca - Now use GET_LOCALTIME(I,J,L) from time_mod.F90
!----------------------------------------------------------------
!zf      USE HCO_CLOCK_MOD, ONLY : HcoClock_Get, HcoClock_GetLocal
    use clmtype
    use time_manager,  only : get_curr_calday
!------------------------------Arguments--------------------------------
!zf      TYPE(HCO_State), POINTER    :: HcoState
!zf      TYPE(Ext_State), POINTER    :: ExtState
      REAL(r8),        INTENT(IN) :: HCOPARDR_AVG_SIM  ! Avg direct PAR [W/m2]
      REAL(r8),        INTENT(IN) :: HCOPARDF_AVG_SIM  ! Avg diffuse PAR [W/m2]
      REAL(r8),        INTENT(IN) :: Q_DIR_2           ! Direct PAR [umol/m2/s]
      REAL(r8),        INTENT(IN) :: Q_DIFF_2          ! Diffuse PAR [umol/m2/s]
      real(r8),        intent(in) :: SUNCOS

      REAL(r8)                    :: GAMMA_P_PCEEA     ! GAMMA factor for light
!---------------------------Local workspace-----------------------------
      REAL(r8)   :: mmHCOPARDR_DAILY
      REAL(r8)   :: mmHCOPARDF_DAILY
      REAL(r8)   :: PAC_DAILY, PAC_INSTANT
      REAL(r8)   :: PTOA, PHI
      REAL(r8)   :: BETA,   SINbeta
      INTEGER    :: DOY
      REAL(r8)   :: AAA, BBB

      real(r8)   :: RAD2D 
      real(r8)   :: pi
!----------------------------------------------------------------
! Initialize

      pi    = 4.0*atan(1.)
      RAD2D = 180./pi
      PTOA  = 0.0_r8

! Convert past light conditions to micromol/m2/s
      mmHCOPARDR_DAILY   = HCOPARDR_AVG_SIM  * WM2_TO_UMOLM2S
      mmHCOPARDF_DAILY   = HCOPARDF_AVG_SIM  * WM2_TO_UMOLM2S

! Work out the light at the top of the canopy.
      PAC_DAILY    = mmHCOPARDR_DAILY + mmHCOPARDF_DAILY
      PAC_INSTANT  = Q_DIR_2       +  Q_DIFF_2

!zf      ! Get latitude
!zf      lat       => clm3%g%lat

! Get day of year, local-time and latitude
! TODO: Evaluate RC?
!zf      CALL HcoClock_Get ( cDOY = DOY, RC=RC )
!zf      CALL HcoClock_GetLocal ( HcoState, I, J, cH = LocalHour, RC=RC )
!zf      DOY = get_curr_calday(offset=0)                !zf

! Get solar elevation angle
!zf      SINbeta      =  SOLAR_ANGLE( HcoState, DOY, LocalHour, lat )
!zf      BETA         =  ASIN( SINbeta ) * RAD2D
!wtw      BETA = (pi/2. - acos(SUNCOS) )            !zf
!wtw      SINbeta = sin(BETA)                       !zf
!wtw      BETA = BETA * RAD2D  

      SINbeta = SUNCOS
      BETA    = asin(SINbeta) * RAD2D

      IF ( SINbeta < 0.0_r8 ) THEN

         GAMMA_P_PCEEA = 0.0_r8

      ELSEIF ( SINbeta > 0.0_r8 ) THEN

         ! PPFD at top of atmosphere
         PTOA    = 3000.0_r8 + 99.0_r8 *                                    &
                   COS( 2._r8 * 3.14_r8 *( DOY - 10.0_r8 ) / 365.0_r8 )

         ! Above canopy transmission
         PHI     = PAC_INSTANT / ( SINbeta * PTOA )

         ! Work out gamma P
         BBB     = 1.0_r8 + 0.0005_r8 *( PAC_DAILY - 400.0_r8 )
         AAA     = ( 2.46_r8 * BBB * PHI ) - ( 0.9_r8 * PHI**2 )

         GAMMA_P_PCEEA = SINbeta * AAA

      ENDIF

! Screen unforced errors. IF solar elevation angle is
! less than 1 THEN gamma_p can not be greater than 0.1.
       IF ( BETA < 1.0_r8 .AND. GAMMA_P_PCEEA > 0.1_r8 ) THEN
          GAMMA_P_PCEEA  = 0.0_r8
       ENDIF
  
      ! Prevent negative values
      GAMMA_P_PCEEA = MAX( GAMMA_P_PCEEA , 0.0_r8 )
      GET_GAMMA_PAR_PCEEA = GAMMA_P_PCEEA 

      END FUNCTION GET_GAMMA_PAR_PCEEA

!************************************************
 
 real function get_gamma_sm(h2osoi, wiltsm, fieldsm)
 
 real(r8),intent(in)  ::  h2osoi, wiltsm, fieldsm
 real(r8)             ::  delt_h2osoi_1 = 0.04
 real(r8)             ::  h2osoi_1

!----------------------------------------
 h2osoi_1 = wiltsm + delt_h2osoi_1

 if (h2osoi .gt. fieldsm) then
   get_gamma_sm = 1.
 else if (h2osoi .lt. wiltsm) then
   get_gamma_sm = 0.3
 else
   get_gamma_sm = 0.3 + (1-0.3)/(fieldsm - wiltsm)*(h2osoi - wiltsm)
 endif

 end function get_gamma_sm
!************************************************
 
 real function GET_GAMMA_T_LD( T, PT_15, PT_1, CT1, CEO )

!----------------------------------------------------------------------------
! computes the temperature sensitivity for the light-dependent fraction of emissions. 
! References (see above for full citations):
!  (1 ) Guenther et al, 1995
!  (2 ) Guenther et al, 2006
!  (3 ) Guenther et al, MEGAN v2.1 user mannual 2007-08
!  (4 ) Guenther et al., GMD 2012 and MEGANv2.1 source code.
!
! REVISION HISTORY:
!  (1 ) Includes the latest MEGAN v2.1 temperature algorithm (mpb, 2009).
!       Note, this temp-dependence is the same for the PCEEA & hybrid models.
!  17 Dec 2009 - R. Yantosca - Added ProTeX headers
!  (2 ) Modified to gamma_t_ld and to permit compound specific parameters
!       CT1 and Ceo (dbm, 6/21/2012)
!----------------------------------------------------------------------------
!------------------------------Arguments-------------------------------------
     REAL(r8), INTENT(IN) :: T         ! Current leaf temperature [K], the surface air temperature field (TS) 
                                       ! is assumed equivalent to the leaf temperature over forests.
     REAL(r8), INTENT(IN) :: PT_15     ! Average leaf temperature over the past 15 days
     REAL(r8), INTENT(IN) :: PT_1      ! Average leaf temperature over the past arbitray day(s). This is not used at present
     REAL(r8), INTENT(IN) :: CT1, CEO  ! Compound-specific parameters for light-dependent temperature 
                                       ! activity factor (dbm, 6/21/2012)
 
!      REAL(r8)    :: GAMMA_T_LD   ! Temperature activity factor for the light-dependent fraction of emissions
!--------------------------------Local workspace-------------------------------
      REAL(r8), PARAMETER   :: R   = 8.314e-3_r8          ! Ideal gas constant [J/mol/K]
      REAL(r8)              :: C_T,   CT2
      REAL(r8)              :: E_OPT, T_OPT, X
!-----------------------------------------------------------------------------

!zf      E_OPT = CEO * EXP( 0.08_r8 * ( PT_15  - 2.97e2_r8 ) )
      E_OPT = CEO * EXP( 0.05_r8 * ( PT_1  - 2.97e2_r8 ) ) * EXP( 0.05_r8 * ( PT_15  - 2.97e2_r8 ) )       !zf 2016.05.20
      T_OPT = 3.13e2_r8 + ( 6.0e-1_r8 * ( PT_15 - 2.97e2_r8 ) )
      CT2   = 200.0_r8

! Variable related to temperature
      X     = ( 1.0_r8/T_OPT - 1.0_r8/T ) / R

!--------------------------------------------------------------------------------------
! C_T: Effect of temperature on leaf BVOC emission, including effect of average 
! temperature over previous 15 days, based on Eq 5a, 5b, 5c from Guenther et al, 1999.
!--------------------------------------------------------------------------------------
      C_T   = E_OPT * CT2 * EXP( CT1 * X ) /                    &
             ( CT2 - CT1 * ( 1.0_r8 - EXP( CT2 * X ) ) )

! Hourly emission activity = C_T. Prevent negative values
      GET_GAMMA_T_LD = MAX( C_T, 0.0_r8 )

      END FUNCTION GET_GAMMA_T_LD

!************************************************

 real FUNCTION GET_GAMMA_LAI( CMLAI, BIDIREXCH )

!----------------------------------------------------------------------------
! computes the gamma exchange activity factor which is sensitive to leaf area (= GAMMA\_LAI).
! References (see above for full citations):
!  ============================================================================
!  (1 ) Guenther et al, 2006
!  (2 ) Guenther et al, MEGAN user mannual 2007-08
!  (3 ) Guenther et al., GMD 2012 and MEGANv2.1 source code.
!
! !REVISION HISTORY:
!  (1 ) Original code by Dorian Abbot (9/2003).  Modified for the standard
!        code by May Fu (11/2004)
!  (2 ) Update to publically released (as of 11/2004) MEGAN algorithm and
!        modified for the standard code by May Fu (11/2004).
!  (3 ) Algorithm is based on the latest MEGAN v2.1 User's Guide (mpb,2009)
!  (4 ) Updated to treat bidirectional exchange compounds appropriately (dbm, 6/2012)
!  17 Dec 2009 - R. Yantosca - Added ProTeX headers
!----------------------------------------------------------------------------
!------------------------------Arguments-------------------------------------
   REAL(r8), INTENT(IN) :: CMLAI       ! Current month's LAI [cm2/cm2]
   LOGICAL,  INTENT(IN) :: BIDIREXCH   ! Logical flag indicating whether the
                                       ! compound undergoes bidirectional exchange
!--------------------------------Local workspace-------------------------------

!----------------------------------------------------------------------------
! Formulation for birectional compounds is as described for ALD2 in Millet et al., ACP 2010

      IF ( BIDIREXCH ) THEN

         IF ( CMLAI <= 6.0_r8) THEN

            ! if lai less than 2:
            IF ( CMLAI <= 2.0_r8 ) THEN
               GET_GAMMA_LAI = 0.5_r8 * CMLAI

            ! if between 2 and 6:
            ELSE
               GET_GAMMA_LAI = 1.0_r8 - 0.0625_r8 * ( CMLAI - 2.0_r8 )
            END IF

         ELSE
            ! keep at 0.75 for LAI > 6
            GET_GAMMA_LAI = 0.75_r8
         END IF

      ! For all other compounds use the standard gamma_lai formulation
      ELSE
         GET_GAMMA_LAI = 0.49_r8 * CMLAI / SQRT( 1.0_r8 + 0.2_r8 * CMLAI*CMLAI)
      ENDIF

      END FUNCTION GET_GAMMA_LAI

!************************************************

 real FUNCTION GET_GAMMA_AGE( CMLAI, PMLAI, DBTWN, TT, AN, AG, AM, AO )

!----------------------------------------------------------------------------
! computes the gamma exchange activity factor which is sensitive to leaf age (= Gamma_Age).
! References (see above for full citations):
!  (1 ) Guenther et al, 2006
!  (2 ) Guenther et al, MEGAN user mannual 2007-08
!  (3 ) Guenther et al., GMD 2012 and MEGANv2.1 source code
!
! REVISION HISTORY:
!  (1 ) Original code by Dorian Abbot (9/2003). Modified for the standard
!        code by May Fu (11/2004)
!  (2 ) Update to publically released (as of 11/2004) MEGAN algorithm and
!        modified for the standard code by May Fu (11/2004).
!  (3 ) Algorithm is based on the latest User's Guide (tmf, 11/19/04)
!  (4 ) Renamed & now includes specific relative emission activity factors for
!       each BVOC based on MEGAN v2.1 algorithm (mpb,2008)
!  (5 ) Now calculate TI (number of days after budbreak required to induce
!       iso. em.) and TM (number of days after budbreak required to reach
!       peak iso. em. rates) using the daily average temperature, instead
!       of using fixed values (mpb,2008)
!       NOTE: Can create 20% increases in tropics (Guenther et al 2006)
!  (6 ) Implemented change for the calculation of FGRO if ( CMLAI > PMLAI ),
!       i.e. if LAI has increased with time, and used new values for
!       all foilage fractions if ( CMLAI = PMLAI ). Also removed TG variable
!       as not now needed. (mpb,2000)
!  (7 ) Updated to pass leaf age activity factors as arguments (dbm, 6/2012)
!  17 Dec 2009 - R. Yantosca - Added ProTeX headers
!  13 Aug 2013 - M. Sulprizio- Updated for sesquiterpenes (H. Pye)
!----------------------------------------------------------------------------
!------------------------------Arguments-------------------------------------
      REAL(r8), INTENT(IN) :: CMLAI     ! Current month's LAI [cm2/cm2]
      REAL(r8), INTENT(IN) :: PMLAI     ! Previous months LAI [cm2/cm2]
      REAL(r8), INTENT(IN) :: DBTWN     ! Number of days between
      REAL(r8), INTENT(IN) :: TT        ! Daily average temperature [K]
      REAL(r8), INTENT(IN) :: AN        ! Relative emiss factor (new leaves)
      REAL(r8), INTENT(IN) :: AG        ! Relative emiss factor (growing leaves)
      REAL(r8), INTENT(IN) :: AM        ! Relative emiss factor (mature leaves)
      REAL(r8), INTENT(IN) :: AO        ! Relative emiss factor (old leaves)

      REAL(r8)             :: FNEW  ! Fraction of new leaves in canopy
      REAL(r8)             :: FGRO  ! Fraction of growing leaves
      REAL(r8)             :: FMAT  ! Fraction of mature leaves
      REAL(r8)             :: FOLD  ! Fraction of old leaves

      REAL(r8)             :: TI !number of days after budbreak required to induce emissions
      REAL(r8)             :: TM !number of days after budbreak required to reach peak emissions
!----------------------------------------------------------------------------
! 
! Compute TI and TM (mpb,2009)
!
      IF ( TT <= 303.0_r8 ) THEN
         TI = 5.0_r8 + 0.7_r8 * ( 300.0_r8 - TT )
      ELSEIF ( TT >  303.0_r8 ) THEN
         TI = 2.9_r8
      ENDIF
      TM = 2.3_r8 * TI
!
! Compute GAMMA_AGE
!
      IF ( CMLAI == PMLAI ) THEN !(i.e. LAI stays the same)

         FNEW = 0.0_r8
         FGRO = 0.1_r8
         FMAT = 0.8_r8
         FOLD = 0.1_r8

      ELSE IF ( CMLAI > PMLAI ) THEN !(i.e. LAI has INcreased)
!
! Calculate Fnew
!
         IF ( DBTWN > TI ) THEN
            FNEW = ( TI / DBTWN ) * ( 1.0_r8 -  PMLAI / CMLAI )
         ELSE
            FNEW = 1.0_r8 - ( PMLAI / CMLAI )
         ENDIF
!
! Calculate FMAT
!
         IF ( DBTWN > TM ) THEN
            FMAT = ( PMLAI / CMLAI ) +   &
                  (( DBTWN - TM ) / DBTWN )*( 1.0_r8 -  PMLAI / CMLAI )
         ELSE
            FMAT = ( PMLAI / CMLAI )
         ENDIF
!
! Calculate Fgro and Fold
!
         FGRO = 1.0_r8 - FNEW - FMAT
         FOLD = 0.0_r8

      ELSE           ! This is the case if  PMLAI > CMLAI (i.e. LAI has DEcreased)

         FNEW = 0.0_r8
         FGRO = 0.0_r8
         FOLD = ( PMLAI - CMLAI ) / PMLAI
         FMAT = 1.0_r8 - FOLD

      ENDIF
!
! Age factor
!
      GET_GAMMA_AGE = FNEW * AN + FGRO * AG + FMAT * AM + FOLD * AO
!
! Prevent negative values
!
      GET_GAMMA_AGE = MAX( GET_GAMMA_AGE , 0.0_r8 )

      END FUNCTION GET_GAMMA_AGE

!------------------------------------------------------------------------------

end module VOC_GEOS_EmissionMod

#endif
