#include <misc.h>
#include <preproc.h>

#if (defined PHENO1)
module CanopyFluxes1Mod

!------------------------------------------------------------------------------
!BOP
!
! !MODULE: CanopyFluxesMod
!
! !DESCRIPTION:
! Calculates the leaf temperature and the leaf fluxes,
! transpiration, photosynthesis and  updates the dew
! accumulation due to evaporation.
!
! !USES:
   use abortutils,   only: endrun
   use time_manager       , only : get_step_size , get_nstep
!! lwp 2008-5-23 add the definition of r8
   use shr_kind_mod, only: r8 => shr_kind_r8
   use clm_varsur, only : sand3d, clay3d
!
! !PUBLIC TYPES:
   implicit none
   save
!
! !PUBLIC MEMBER FUNCTIONS:
   public :: CanopyFluxes !Calculates the leaf temperature and leaf fluxes
!
! !PRIVATE MEMBER FUNCTIONS:
   private :: Stomata     !Leaf stomatal resistance and leaf photosynthesis
#ifdef AVIM
!  private :: Phosyn      !AVIM conductance-photosyn,zhangyw 20071218 
   private :: NPPRmRg     !AVIM conductance-photosyn,zhangyw 20071218 
   private :: Dislos
   private :: Bmlai
   private :: SoilCarbon 
#endif
!
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!------------------------------------------------------------------------------

contains

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: CanopyFluxes
!
! !INTERFACE:
  subroutine CanopyFluxes(lbg, ubg, lbc, ubc, lbp, ubp, &
                          num_nolakep, filter_nolakep, pco2)
!
! !DESCRIPTION:
! 1. Calculates the leaf temperature:
! 2. Calculates the leaf fluxes, transpiration, photosynthesis and
!    updates the dew accumulation due to evaporation.
!
! Method:
! Use the Newton-Raphson iteration to solve for the foliage
! temperature that balances the surface energy budget:
!
! f(t_veg) = Net radiation - Sensible - Latent = 0
! f(t_veg) + d(f)/d(t_veg) * dt_veg = 0     (*)
!
! Note:
! (1) In solving for t_veg, t_grnd is given from the previous timestep.
! (2) The partial derivatives of aerodynamical resistances, which cannot
!     be determined analytically, are ignored for d(H)/dT and d(LE)/dT
! (3) The weighted stomatal resistance of sunlit and shaded foliage is used
! (4) Canopy air temperature and humidity are derived from => Hc + Hg = Ha
!                                                          => Ec + Eg = Ea
! (5) Energy loss is due to: numerical truncation of energy budget equation
!     (*); and "ecidif" (see the code) which is dropped into the sensible
!     heat
! (6) The convergence criteria: the difference, del = t_veg(n+1)-t_veg(n)
!     and del2 = t_veg(n)-t_veg(n-1) less than 0.01 K, and the difference
!     of water flux from the leaf between the iteration step (n+1) and (n)
!     less than 0.1 W/m2; or the iterative steps over 40.
!
! !USES:
    use shr_kind_mod       , only: r8 => shr_kind_r8
    use clmtype
    use time_manager       , only : get_step_size
    use clm_varpar         , only : nlevsoi, nlevsno
    use clm_varctl         , only : finidat
    use clm_varcon         , only : sb, cpair, hvap, vkc, grav, denice, &
                                    denh2o, tfrz, csoilc, dzsoi, zsoi
    use QSatMod            , only : QSat
    use FrictionVelocityMod, only : FrictionVelocity, MoninObukIni
!! lwp 2008-5-23
    use spmdMod, only: masterproc
   use clm_varsur, only : sand3d, clay3d
!
! !ARGUMENTS:
    implicit none
    integer, intent(in) :: lbg, ubg                    ! gridcell bounds
    integer, intent(in) :: lbc, ubc                    ! column bounds
    integer, intent(in) :: lbp, ubp                    ! pft bounds
    integer, intent(in) :: num_nolakep                 ! number of column non-lake points in pft filter
    integer, intent(in) :: filter_nolakep(ubp-lbp+1)   ! pft filter for non-lake points
!
! !CALLED FROM:
! subroutine Biogeophysics1 in module Biogeophysics1Mod
!
! !REVISION HISTORY:
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! 12/19/01, Peter Thornton
! Changed tg to t_grnd for consistency with other routines
! 1/29/02, Peter Thornton
! Migrate to new data structures, new calling protocol. For now co2 and
! o2 partial pressures are hardwired, but they should be coming in from
! forc_co2 and forc_o2. Keeping the same hardwired values as in CLM2 to
! assure bit-for-bit results in the first comparisons.
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
   integer , pointer :: ixy(:)             ! xy lon index (column-level)
   integer , pointer :: jxy(:)             ! xy lat index (column_level)

   integer , pointer :: ivt(:)         ! pft vegetation type
   integer , pointer :: pcolumn(:)     ! pft's column index
   integer , pointer :: plandunit(:)   ! pft's landunit index
   integer , pointer :: pgridcell(:)   ! pft's gridcell index
   real(r8), pointer :: forc_th(:)     ! atmospheric potential temperature (Kelvin)
   real(r8), pointer :: t_grnd(:)      ! ground surface temperature [K]
   real(r8), pointer :: thm(:)         ! intermediate variable (forc_t+0.0098*forc_hgt_t)
   real(r8), pointer :: qg(:)          ! specific humidity at ground surface [kg/kg]
   real(r8), pointer :: thv(:)         ! virtual potential temperature (kelvin)
   real(r8), pointer :: z0mv(:)        ! roughness length over vegetation, momentum [m]
   real(r8), pointer :: z0hv(:)        ! roughness length over vegetation, sensible heat [m]
   real(r8), pointer :: z0qv(:)        ! roughness length over vegetation, latent heat [m]
   real(r8), pointer :: z0mg(:)        ! roughness length of ground, momentum [m]
   real(r8), pointer :: dqgdT(:)       ! temperature derivative of "qg"
   real(r8), pointer :: htvp(:)        ! latent heat of evaporation (/sublimation) [J/kg]
   real(r8), pointer :: emv(:)         ! ground emissivity
   real(r8), pointer :: emg(:)         ! vegetation emissivity
   real(r8), pointer :: forc_pbot(:)   ! atmospheric pressure (Pa)
   real(r8), pointer :: forc_q(:)      ! atmospheric specific humidity (kg/kg)
   real(r8), pointer :: forc_u(:)      ! atmospheric wind speed in east direction (m/s)
   real(r8), pointer :: forc_v(:)      ! atmospheric wind speed in north direction (m/s)
   real(r8), pointer :: forc_hgt_u(:)  ! observational height of wind [m]
   real(r8), pointer :: forc_rho(:)    ! density (kg/m**3)
! wtw
   real(r8), pointer :: forc_co2(:)    ! atmospheric CO2 concentration (vmr)
   real(r8), pointer :: forc_o2(:)     ! atmospheric O2 concentration (vmr)
! wtw
   real(r8), pointer :: londeg(:)     ! atmospheric O2 concentration (Pa)
   real(r8), pointer :: latdeg(:)     ! atmospheric O2 concentration (Pa)
! lwp 2008-6-6 add solar forcing
   real(r8), pointer :: forc_solad(:,:)  ! direct beam radiation (W/m**2)
   real(r8), pointer :: forc_solai(:,:)  ! diffuse radiation (W/m**2)

   real(r8), pointer :: forc_lwrad(:)  ! downward infrared (longwave) radiation (W/m**2)
   real(r8), pointer :: displa(:)      ! displacement height (m)
   real(r8), pointer :: parsun(:)      ! average absorbed PAR for sunlit leaves (W/m**2)
   real(r8), pointer :: parsha(:)      ! average absorbed PAR for shaded leaves (W/m**2)
   real(r8), pointer :: elai(:)        ! one-sided leaf area index with burying by snow
   real(r8), pointer :: esai(:)        ! one-sided stem area index with burying by snow
   real(r8), pointer :: fdry(:)        ! fraction of foliage that is green and dry [-]
   real(r8), pointer :: fwet(:)        ! fraction of canopy that is wet (0 to 1)
   real(r8), pointer :: laisun(:)      ! sunlit leaf area
   real(r8), pointer :: laisha(:)      ! shaded leaf area
   integer , pointer :: frac_veg_nosno(:)  ! frac of veg not covered by snow (0 OR 1 now) [-]
   real(r8), pointer :: sabv(:)        ! solar radiation absorbed by vegetation (W/m**2)
   real(r8), pointer :: watsat(:,:)    ! volumetric soil water at saturation (porosity)
   real(r8), pointer :: h2osoi_ice(:,:)! ice lens (kg/m2)
   real(r8), pointer :: h2osoi_liq(:,:)! liquid water (kg/m2)
   real(r8), pointer :: dz(:,:)        ! layer depth (m)
   real(r8), pointer :: t_soisno(:,:)  ! soil temperature (Kelvin)
   real(r8), pointer :: sucsat(:,:)    ! minimum soil suction (mm)
   real(r8), pointer :: bsw(:,:)       ! Clapp and Hornberger "b"
   real(r8), pointer :: rootfr(:,:)    ! fraction of roots in each soil layer
   real(r8), pointer :: smpmax(:)      ! wilting point potential in mm
   real(r8), pointer :: dleaf(:)       ! characteristic leaf dimension (m)
!
! local pointers to implicit inout arguments
!
   real(r8), pointer :: cgrnds(:)      ! deriv. of soil sensible heat flux wrt soil temp [w/m2/k]
   real(r8), pointer :: cgrndl(:)      ! deriv. of soil latent heat flux wrt soil temp [w/m**2/k]
   real(r8), pointer :: t_veg(:)       ! vegetation temperature (Kelvin)
!zhangyw
   real(r8), pointer :: t_zyw(:)       ! vegetation temperature (Kelvin)
   real(r8), pointer :: t_ref2m(:)     ! 2 m height surface air temperature (Kelvin)
   real(r8), pointer :: q_ref2m(:)     ! 2 m height surface specific humidity (kg/kg)
   real(r8), pointer :: h2ocan(:)      ! canopy water (mm H2O)
#ifdef DGVM
   real(r8), pointer :: annpsnpot(:)   ! annual potential photosynthesis (umol CO2 /m**2)
   real(r8), pointer :: annpsn(:)      ! annual photosynthesis (umol CO2 /m**2)
#else
   real(r8) :: annpsnpot(lbp:ubp)      ! temporary, set to zer0
   real(r8) :: annpsn(lbp:ubp)         ! temporary, set to zer0
#endif
!
! local pointers to implicit out arguments
!
   real(r8), pointer :: cgrnd(:)           ! deriv. of soil energy flux wrt to soil temp [w/m2/k]
   real(r8), pointer :: dlrad(:)           ! downward longwave radiation below the canopy [W/m2]
   real(r8), pointer :: ulrad(:)           ! upward longwave radiation above the canopy [W/m2]
   real(r8), pointer :: ram1(:)            ! aerodynamical resistance (s/m)
   real(r8), pointer :: btran(:)           ! transpiration wetness factor (0 to 1)
   real(r8), pointer :: rssun(:)           ! sunlit stomatal resistance (s/m)
   real(r8), pointer :: rssha(:)           ! shaded stomatal resistance (s/m)
   real(r8), pointer :: psnsun(:)          ! sunlit leaf photosynthesis (umol CO2 /m**2/ s)
   real(r8), pointer :: psnsha(:)          ! shaded leaf photosynthesis (umol CO2 /m**2/ s)
   real(r8), pointer :: rf(:)              ! leaf photosynthesis (umol CO2 /m**2/ s)  !zhangyw
   real(r8), pointer :: qflx_tran_veg(:)   ! vegetation transpiration (mm H2O/s) (+ = to atm)
   real(r8), pointer :: dt_veg(:)          ! change in t_veg, last iteration (Kelvin)
   real(r8), pointer :: qflx_evap_veg(:)   ! vegetation evaporation (mm H2O/s) (+ = to atm)
   real(r8), pointer :: eflx_sh_veg(:)     ! sensible heat flux from leaves (W/m**2) [+ to atm]
   real(r8), pointer :: taux(:)            ! wind (shear) stress: e-w (kg/m/s**2)
   real(r8), pointer :: tauy(:)            ! wind (shear) stress: n-s (kg/m/s**2)
   real(r8), pointer :: eflx_sh_grnd(:)    ! sensible heat flux from ground (W/m**2) [+ to atm]
   real(r8), pointer :: qflx_evap_soi(:)   ! soil evaporation (mm H2O/s) (+ = to atm)
   real(r8), pointer :: fpsn(:)            ! photosynthesis (umol CO2 /m**2 /s)
   real(r8), pointer :: rootr(:,:)         ! effective fraction of roots in each soil layer
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   real(r8), pointer :: rb1(:)             ! boundary layer resistance (s/m)
#endif

!zhangyw
#ifdef AVIM
   real(r8), pointer :: phd(:)             !  !zhangyw
   real(r8), pointer :: npd(:)             !  !zhangyw
   real(r8), pointer :: nep(:)             !  lwp net earth primary productivity
   real(r8), pointer :: dco2(:)            !  lwp soil respiration of CO2
   real(r8), pointer :: dsmc(:)            !  lwp total soil Carbon
   real(r8), pointer :: nmin(:)            !  lwp total soil Nitrogen
   real(r8), pointer :: polc1(:)           !  lwp carbon pool 1
   real(r8), pointer :: polc2(:)           !  lwp carbon pool 2
   real(r8), pointer :: polc3(:)           !  lwp carbon pool 3
   real(r8), pointer :: polc4(:)           !  lwp carbon pool 4
   real(r8), pointer :: polc5(:)           !  lwp carbon pool 5
   real(r8), pointer :: polc6(:)           !  lwp carbon pool 6
   real(r8), pointer :: polc7(:)           !  lwp carbon pool 7
   real(r8), pointer :: polc8(:)           !  lwp carbon pool 8
!  real(r8), pointer :: ded(:)             !  !zhangyw
   real(r8), pointer :: alai(:)            !  !liwp
   real(r8), pointer :: vegtype(:)            !  !liwp
   real(r8), pointer :: pht(:)             ! leaf photosynthesis (umol CO2 /m**2/ s)  !zhangyw
   real(r8), pointer :: npp(:)             !  !zhangyw
   real(r8), pointer :: rmp(:)             !  !zhangyw
   real(r8), pointer :: rgp(:)             !  !zhangyw
   real(r8), pointer :: dis1(:)            !  !zhangyw
   real(r8), pointer :: dis2(:)            !  !zhangyw
   real(r8), pointer :: dis3(:)            !  !zhangyw
   real(r8), pointer :: dis4(:)            !  !zhangyw
   real(r8), pointer :: los1(:)            !  !zhangyw
   real(r8), pointer :: los2(:)            !  !zhangyw
   real(r8), pointer :: los3(:)            !  !zhangyw
   real(r8), pointer :: los4(:)            !  !zhangyw
   real(r8), pointer :: bm1(:)             !  !zhangyw
   real(r8), pointer :: bm2(:)             !  !zhangyw
   real(r8), pointer :: bm3(:)             !  !zhangyw
   real(r8), pointer :: bm4(:)             !  !zhangyw
   real(r8), pointer :: surl(:)            !  !liwp for surface carbon input
   real(r8), pointer :: rotl(:)            !  !liwp for root carbon input
   real(r8), pointer :: ts1d(:)  !  daily 1stlayer soil T (depend on the definition of 1st layer, top 10cm)
   real(r8), pointer :: ts2d(:)  !  daily 2ndlayer soil T (depend on the definition of 1st layer, top170cm)
   real(r8), pointer :: ts3d(:)  !  daily 3rdlayer soil T (depend on the definition of 1st layer)
   real(r8), pointer :: sm1d(:)  !  daily 1stlayer soil mositure (depend on the definition of 1st layer)
   real(r8), pointer :: sm2d(:)  !  daily 2ndlayer soil mositure (depend on the definition of 2nd layer)
!! lwp 2008-7-9 now AVIM use only 12 types of vegetation
!  real(r8) :: glai(13)
   real(r8) :: gbm  (4,16)
   real(r8) :: glos (4,16)
   real(r8) :: gdis (4,16)
   real(r8) :: gpolc(8,16)
!! lwp 2008-6-6 solar forcing
   real(r8) :: avmpar(lbg:ubg)
   real(r8) :: pk, fl 
#endif
   real(r8), pointer :: vcmxout(:)             ! lwp output actual vcmx for vegetation photosynthesis 
   real(r8), pointer :: ftout(:)             ! lwp output actual vcmx for vegetation photosynthesis 
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
   real(r8), parameter :: btran0 = 1.0e-10  ! initial value
   real(r8), parameter :: zii = 1000.0      ! convective boundary layer height [m]
   real(r8), parameter :: beta = 1.0        ! coefficient of conective velocity [-]
   real(r8), parameter :: delmax = 1.0      ! maxchange in  leaf temperature [K]
   real(r8), parameter :: dlemin = 0.1      ! max limit for energy flux convergence [w/m2]
   real(r8), parameter :: dtmin = 0.01      ! max limit for temperature convergence [K]
   real(r8), parameter :: po2 = 0.209       ! partial pressure  o2 (mol/mol)
!! lwp 2008-12-08 pco2 is controled by clm_varcon.F90 and transfered in from driver.F90 
   real(r8) :: pco2                         ! partial pressure co2 (mol/mol)
!  real(r8), parameter :: pco2 = 335.e-06   ! partial pressure co2 (mol/mol)
!! lwp 2008-11-22 the original pco2 is 355
!  real(r8), parameter :: pco2 = 355.e-06   ! partial pressure co2 (mol/mol)
!! lwp 2008-11-22 use higher level of pco2 to test its sensitivity 
!  real(r8), parameter :: pco2 = 375.e-06   ! partial pressure co2 (mol/mol)
   integer , parameter :: itmax = 40        ! maximum number of iteration [-]
   integer , parameter :: itmin = 2         ! minimum number of iteration [-]
   real(r8) :: dtime                 ! land model time step (sec)
   real(r8) :: zldis(lbp:ubp)        ! reference height "minus" zero displacement height [m]
   real(r8) :: zeta                  ! dimensionless height used in Monin-Obukhov theory
   real(r8) :: wc                    ! convective velocity [m/s]
   real(r8) :: dth(lbp:ubp)          ! diff of virtual temp. between ref. height and surface
   real(r8) :: dthv                  ! diff of vir. poten. temp. between ref. height and surface
   real(r8) :: dqh(lbp:ubp)          ! diff of humidity between ref. height and surface
   real(r8) :: obu(lbp:ubp)          ! Monin-Obukhov length (m)
   real(r8) :: um(lbp:ubp)           ! wind speed including the stablity effect [m/s]
   real(r8) :: ur(lbp:ubp)           ! wind speed at reference height [m/s]
   real(r8) :: uaf                   ! velocity of air within foliage [m/s]
   real(r8) :: temp1(lbp:ubp)        ! relation for potential temperature profile
   real(r8) :: temp12m(lbp:ubp)      ! relation for potential temperature profile applied at 2-m
   real(r8) :: temp2(lbp:ubp)        ! relation for specific humidity profile
   real(r8) :: temp22m(lbp:ubp)      ! relation for specific humidity profile applied at 2-m
   real(r8) :: ustar(lbp:ubp)        ! friction velocity [m/s]
   real(r8) :: tstar                 ! temperature scaling parameter
   real(r8) :: qstar                 ! moisture scaling parameter
   real(r8) :: thvstar               ! virtual potential temperature scaling parameter
   real(r8) :: taf(lbp:ubp)          ! air temperature within canopy space [K]
   real(r8) :: qaf(lbp:ubp)          ! humidity of canopy air [kg/kg]
   real(r8) :: rpp                   ! fraction of potential evaporation from leaf [-]
   real(r8) :: rppdry                ! fraction of potential evaporation through transp [-]
   real(r8) :: cf                    ! heat transfer coefficient from leaves [-]
   real(r8) :: rb(lbp:ubp)           ! leaf boundary layer resistance [s/m]
   real(r8) :: rah(lbp:ubp,2)        ! thermal resistance [s/m]
   real(r8) :: raw(lbp:ubp,2)        ! moisture resistance [s/m]
   real(r8) :: wta                   ! heat conductance for air [m/s]
   real(r8) :: wtg(lbp:ubp)          ! heat conductance for ground [m/s]
   real(r8) :: wtl                   ! heat conductance for leaf [m/s]
   real(r8) :: wta0(lbp:ubp)         ! normalized heat conductance for air [-]
   real(r8) :: wtl0(lbp:ubp)         ! normalized heat conductance for leaf [-]
   real(r8) :: wtg0                  ! normalized heat conductance for ground [-]
   real(r8) :: wtal(lbp:ubp)         ! normalized heat conductance for air and leaf [-]
   real(r8) :: wtga                  ! normalized heat cond. for air and ground  [-]
   real(r8) :: wtaq                  ! latent heat conductance for air [m/s]
   real(r8) :: wtlq                  ! latent heat conductance for leaf [m/s]
   real(r8) :: wtgq(lbp:ubp)         ! latent heat conductance for ground [m/s]
   real(r8) :: wtaq0(lbp:ubp)        ! normalized latent heat conductance for air [-]
   real(r8) :: wtlq0(lbp:ubp)        ! normalized latent heat conductance for leaf [-]
   real(r8) :: wtgq0                 ! normalized heat conductance for ground [-]
   real(r8) :: wtalq(lbp:ubp)        ! normalized latent heat cond. for air and leaf [-]
   real(r8) :: wtgaq                 ! normalized latent heat cond. for air and ground [-]
   real(r8) :: el(lbp:ubp)           ! vapor pressure on leaf surface [pa]
   real(r8) :: deldT                 ! derivative of "el" on "t_veg" [pa/K]
   real(r8) :: qsatl(lbp:ubp)        ! leaf specific humidity [kg/kg]
   real(r8) :: qsatldT(lbp:ubp)      ! derivative of "qsatl" on "t_veg"
   real(r8) :: air(lbp:ubp),bir(lbp:ubp),cir(lbp:ubp)  ! atmos. radiation temporay set
   real(r8) :: dc1,dc2               ! derivative of energy flux [W/m2/K]
   real(r8) :: delt                  ! temporary
   real(r8) :: delq                  ! temporary
   real(r8) :: del(lbp:ubp)          ! absolute change in leaf temp in current iteration [K]
   real(r8) :: del2(lbp:ubp)         ! change in leaf temperature in previous iteration [K]
   real(r8) :: dele(lbp:ubp)         ! change in latent heat flux from leaf [K]
   real(r8) :: dels                  ! change in leaf temperature in current iteration [K]
   real(r8) :: det(lbp:ubp)          ! maximum leaf temp. change in two consecutive iter [K]
   real(r8) :: efeb(lbp:ubp)         ! latent heat flux from leaf (previous iter) [mm/s]
   real(r8) :: efeold                ! latent heat flux from leaf (previous iter) [mm/s]
   real(r8) :: efpot                 ! potential latent energy flux [kg/m2/s]
   real(r8) :: efe(lbp:ubp)          ! water flux from leaf [mm/s]
   real(r8) :: efsh                  ! sensible heat from leaf [mm/s]
   real(r8) :: obuold(lbp:ubp)       ! monin-obukhov length from previous iteration
   real(r8) :: tlbef(lbp:ubp)        ! leaf temperature from previous iteration [K]
   real(r8) :: ecidif                ! excess energies [W/m2]
   real(r8) :: err(lbp:ubp)          ! balance error
   real(r8) :: erre                  ! balance error
   real(r8) :: co2(lbp:ubp)          ! atmospheric co2 concentration (pa)
   real(r8) :: o2(lbp:ubp)           ! atmospheric o2 concentration (pa)
   real(r8) :: svpts(lbp:ubp)        ! saturation vapor pressure at t_veg (pa)
   real(r8) :: eah(lbp:ubp)          ! canopy air vapor pressure (pa)
   real(r8) :: s_node                ! vol_liq/eff_porosity
   real(r8) :: smp_node              ! matrix potential
   real(r8) :: vol_ice               ! partial volume of ice lens in layer
   real(r8) :: eff_porosity          ! effective porosity in layer
   real(r8) :: vol_liq               ! partial volume of liquid water in layer
   real(r8) :: rresis                ! soil water contribution to root resistance
   integer  :: itlef                 ! counter for leaf temperature iteration [-]
   integer  :: nmozsgn(lbp:ubp)      ! number of times stability changes sign
   real(r8) :: w                     ! exp(-LSAI)
   real(r8) :: csoilcn               ! interpolated csoilc for less than dense canopies
   real(r8) :: fm(lbp:ubp)           ! needed for BGC only to diagnose 10m wind speed
   real(r8) :: wtshi                 ! sensible heat resistance for air, grnd and leaf [-]
   real(r8) :: wtsqi                 ! latent heat resistance for air, grnd and leaf [-]
   integer  :: j                     ! soil/snow level index
   integer  :: p                     ! pft index
   integer  :: c                     ! column index
   integer  :: l                     ! landunit index
   integer  :: g                     ! gridcell index
   integer  :: fp                    ! lake filter pft index
   integer  :: fn                    ! number of values in pft filter
   integer  :: fnorig                ! number of values in pft filter copy
   integer  :: fnold                 ! temporary copy of pft count
   integer  :: f                     ! filter index
   integer  :: filterp(ubp-lbp+1)
   integer  :: fporig(ubp-lbp+1)

  integer  :: nstep         ! time step number
  integer  :: nstep_day  !! step numbers per day 
  integer  :: year_number !! numbers of model year 
  integer  :: kkk         !! number of DO LOOP 
  integer  :: ix, jy 
!------------------------------------------------------------------------------

   ! Assign local pointers to derived type members (gridcell-level)

!! lwp 2008-6-6 add solar forcing
     forc_solad    => clm3%g%a2lf%forc_solad
     forc_solai    => clm3%g%a2lf%forc_solai

   forc_lwrad     => clm3%g%a2lf%forc_lwrad
   forc_q         => clm3%g%a2ls%forc_q
   forc_pbot      => clm3%g%a2ls%forc_pbot
   forc_u         => clm3%g%a2ls%forc_u
   forc_v         => clm3%g%a2ls%forc_v
   forc_th        => clm3%g%a2ls%forc_th
   forc_hgt_u     => clm3%g%a2ls%forc_hgt_u
   forc_rho       => clm3%g%a2ls%forc_rho

   ! Assign local pointers to derived type members (column-level)
!----
! wtw
!
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   rb1            => clm3%g%l%c%p%pps%rb1
#endif

   forc_co2       => clm3%g%a2ls%forc_co2
   forc_o2        => clm3%g%a2ls%forc_o2
!
!-----

   t_soisno       => clm3%g%l%c%ces%t_soisno
   watsat         => clm3%g%l%c%cps%watsat
   h2osoi_ice     => clm3%g%l%c%cws%h2osoi_ice
   dz             => clm3%g%l%c%cps%dz
   h2osoi_liq     => clm3%g%l%c%cws%h2osoi_liq
   sucsat         => clm3%g%l%c%cps%sucsat
   bsw            => clm3%g%l%c%cps%bsw
   emg            => clm3%g%l%c%cps%emg
   t_grnd         => clm3%g%l%c%ces%t_grnd
   thm            => clm3%g%l%c%ces%thm
   qg             => clm3%g%l%c%cws%qg
   thv            => clm3%g%l%c%ces%thv
   dqgdT          => clm3%g%l%c%cws%dqgdT
   htvp           => clm3%g%l%c%cps%htvp
   z0mg           => clm3%g%l%c%cps%z0mg
  ixy             => clm3%g%l%c%ixy
  jxy             => clm3%g%l%c%jxy

   ! Assign local pointers to derived type members (pft-level)

   ivt            => clm3%g%l%c%p%itype
   pcolumn        => clm3%g%l%c%p%column
   plandunit      => clm3%g%l%c%p%landunit
   pgridcell      => clm3%g%l%c%p%gridcell
   frac_veg_nosno => clm3%g%l%c%p%pps%frac_veg_nosno
   btran          => clm3%g%l%c%p%pps%btran
   rootr          => clm3%g%l%c%p%pps%rootr
   rootfr         => clm3%g%l%c%p%pps%rootfr
   emv            => clm3%g%l%c%p%pps%emv
   t_veg          => clm3%g%l%c%p%pes%t_veg
   displa         => clm3%g%l%c%p%pps%displa
   z0mv           => clm3%g%l%c%p%pps%z0mv
   z0hv           => clm3%g%l%c%p%pps%z0hv
   z0qv           => clm3%g%l%c%p%pps%z0qv
   ram1           => clm3%g%l%c%p%pps%ram1
   parsun         => clm3%g%l%c%p%pef%parsun
   rssun          => clm3%g%l%c%p%pps%rssun
!  rf             => clm3%g%l%c%p%pcf%rf 
   psnsun         => clm3%g%l%c%p%pcf%psnsun
   parsha         => clm3%g%l%c%p%pef%parsha
   rssha          => clm3%g%l%c%p%pps%rssha
   psnsha         => clm3%g%l%c%p%pcf%psnsha
   elai           => clm3%g%l%c%p%pps%elai
   esai           => clm3%g%l%c%p%pps%esai
   fdry           => clm3%g%l%c%p%pps%fdry
   laisun         => clm3%g%l%c%p%pps%laisun
   laisha         => clm3%g%l%c%p%pps%laisha
   qflx_tran_veg  => clm3%g%l%c%p%pwf%qflx_tran_veg
   fwet           => clm3%g%l%c%p%pps%fwet
   h2ocan         => clm3%g%l%c%p%pws%h2ocan
   dt_veg         => clm3%g%l%c%p%pps%dt_veg
   sabv           => clm3%g%l%c%p%pef%sabv
   qflx_evap_veg  => clm3%g%l%c%p%pwf%qflx_evap_veg
   eflx_sh_veg    => clm3%g%l%c%p%pef%eflx_sh_veg
   taux           => clm3%g%l%c%p%pmf%taux
   tauy           => clm3%g%l%c%p%pmf%tauy
   eflx_sh_grnd   => clm3%g%l%c%p%pef%eflx_sh_grnd
   qflx_evap_soi  => clm3%g%l%c%p%pwf%qflx_evap_soi
   t_ref2m        => clm3%g%l%c%p%pes%t_ref2m
   q_ref2m        => clm3%g%l%c%p%pes%q_ref2m
   dlrad          => clm3%g%l%c%p%pef%dlrad
   ulrad          => clm3%g%l%c%p%pef%ulrad
   cgrnds         => clm3%g%l%c%p%pef%cgrnds
   cgrndl         => clm3%g%l%c%p%pef%cgrndl
   cgrnd          => clm3%g%l%c%p%pef%cgrnd
   fpsn           => clm3%g%l%c%p%pcf%fpsn
#ifdef DGVM
   annpsnpot      => clm3%g%l%c%p%pdgvs%annpsnpot
   annpsn         => clm3%g%l%c%p%pdgvs%annpsn
#endif
   londeg => clm3%g%londeg
   latdeg => clm3%g%latdeg

!zhangyw
#if (defined AVIM)
!! 2008-5-23 add three pointers
   t_zyw          => clm3%g%l%c%p%pes%t_zyw
   phd            => clm3%g%l%c%p%pcf%phd
   npd            => clm3%g%l%c%p%pcf%npd
   nep            => clm3%g%l%c%p%pcf%nep
   dsmc           => clm3%g%l%c%p%pcf%dsmc
   dco2           => clm3%g%l%c%p%pcf%dco2
   nmin           => clm3%g%l%c%p%pcf%nmin
   ts1d           => clm3%g%l%c%p%pcf%ts1d
   ts2d           => clm3%g%l%c%p%pcf%ts2d
   ts3d           => clm3%g%l%c%p%pcf%ts3d
   sm1d           => clm3%g%l%c%p%pcf%sm1d
   sm2d           => clm3%g%l%c%p%pcf%sm2d

!  ded            => clm3%g%l%c%p%pcf%ded
   alai           => clm3%g%l%c%p%pcf%alai
   vegtype        => clm3%g%l%c%p%pcf%vegtype

   pht            => clm3%g%l%c%p%pcf%pht
   npp            => clm3%g%l%c%p%pcf%npp
   rmp            => clm3%g%l%c%p%pcf%rmp
   rgp            => clm3%g%l%c%p%pcf%rgp
   dis1           => clm3%g%l%c%p%pcf%dis1
   dis2           => clm3%g%l%c%p%pcf%dis2
   dis3           => clm3%g%l%c%p%pcf%dis3
   dis4           => clm3%g%l%c%p%pcf%dis4
   los1           => clm3%g%l%c%p%pcf%los1
   los2           => clm3%g%l%c%p%pcf%los2
   los3           => clm3%g%l%c%p%pcf%los3
   los4           => clm3%g%l%c%p%pcf%los4
    bm1           => clm3%g%l%c%p%pcf%bm1 
    bm2           => clm3%g%l%c%p%pcf%bm2 
    bm3           => clm3%g%l%c%p%pcf%bm3 
    bm4           => clm3%g%l%c%p%pcf%bm4 
   surl           => clm3%g%l%c%p%pcf%surl
   rotl           => clm3%g%l%c%p%pcf%rotl
   polc1          => clm3%g%l%c%p%pcf%polc1
   polc2          => clm3%g%l%c%p%pcf%polc2
   polc3          => clm3%g%l%c%p%pcf%polc3
   polc4          => clm3%g%l%c%p%pcf%polc4
   polc5          => clm3%g%l%c%p%pcf%polc5
   polc6          => clm3%g%l%c%p%pcf%polc6
   polc7          => clm3%g%l%c%p%pcf%polc7
   polc8          => clm3%g%l%c%p%pcf%polc8
#endif
   vcmxout        => clm3%g%l%c%p%pcf%vcmxout
   ftout        => clm3%g%l%c%p%pcf%ftout

   ! Assign local pointers to derived type members (ecophysiological)

   smpmax         => pftcon%smpmax
   dleaf          => pftcon%dleaf

! lwp 2008-5-27 add the following initials for dis, los and bm from zyw

#if (defined AVIM)
!    data glai /5.0, 6.0, 2.0, 3.0, 4.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 1.5,0.1/
        data gdis/0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.4,    0.1,    0.5,    0, &
                  0.5,    0,      0.5,    0, &
                  0.5,    0,      0.5,    0, &
                  0.5,    0,      0.5,    0, &
                  0.5,    0,      0.5,    0, &
                  0.5,    0,      0.5,    0  /

!       data glos/0.006,  0.00015 ,0.0004 ,0.00005, &
!       data glos/0.005,  0.00012 ,0.0004 ,0.00005, &  !! type 1 evergreen needleleaf
!                 0.005 , 0.00014 ,0.0005 ,0.00005, &  !! type 2 evergreen broadleaf
!                 0.003,  0.00011 ,0.0002 ,0.00005, &  !! type 3 deciduous needleleaf
!                 0.005 , 0.00017 ,0.00045,0.00005, &  !! type 4 deciduous broadleaf
!                 0.004 , 0.00015 ,0.0004 ,0.00005, &  !! type 5 mixed forest
!                 0.003,  0.00045 ,0.0004 ,0.00005, &  !! type 6 closed shrubland
!                 0.010,  0.0001  ,0.0001 ,0.00005, &  !! type 7 open shrubland
!                 0.0055, 0.0001  ,0.0014 ,0.00005, &  !! type 8 woody savana 
!                 0.01,   0       ,0.0015 ,0.002,   &  !! type 9 savana
!                 0.01,   0       ,0.0016 ,0.002,   &  !! type 10 grassland
!                 0.01,   0       ,0.0015 ,0.002,   &  !! type 11 crops
!                 0.0056, 0.00016 ,0.0003 ,0.002,   &  !! type 12 crops and natural vegetation
!                 0.0001, 0       ,0.0001 ,0.002    /  !! type 13 barren soil and sparse vegetation

        data glos/0.004,  0.00010 ,0.0003 ,0.00005, &  !! type 1 NET temperate  Needleleaf Evergreen Tree
                  0.003,  0.00007 ,0.00022,0.00005, &  !! type 2 NET boreal     Needleleaf Evergreen Tree
                  0.003,  0.00006 ,0.0001 ,0.00005, &  !! type 3 NDT boreal     Needleleaf Deciduous Tree
!                 0.0015, 0.00014 ,0.0005 ,0.00005, &  !! type 4 BET tropical   Broadleaf  Evergreen Tree
                  0.0003, 0.00013 ,0.0005 ,0.00005, &  !! type 4 BET tropical   Broadleaf  Evergreen Tree
                  0.0020, 0.00010 ,0.0003 ,0.00005, &  !! type 5 BET temperate  Broadleaf  Evergreen Tree 
!                 0.0036, 0.00017 ,0.0004 ,0.00005, &  !! type 6 BDT tropical   Broadleaf  Deciduous Tree 
                  0.0030, 0.00017 ,0.0004 ,0.00005, &  !! type 6 BDT tropical   Broadleaf  Deciduous Tree 
                  0.004,  0.00015 ,0.0003 ,0.00005, &  !! type 7 BDT temperate  Broadleaf  Deciduous Tree 
                  0.005,  0.00015 ,0.0002 ,0.00005, &  !! type 8 BDT boreal     Broadleaf  Deciduous Tree 
                  0.005,  0.00020 ,0.0003 ,0.00005, &  !! type 9 BES temperate  Broadleaf Evergreen Shrub 
                  0.003,  0.00005 ,0.00003,0.00005, &  !! type 10 BDS temperate Broadleaf Deciduous Shrub 
                  0.003,  0.00015 ,0.0001 ,0.00005, &  !! type 11 BDS boreal    Broadleaf Deciduous Shrub 
                  0.001,  0       ,0.0009 ,0.002,   &  !! type 12 C3 grass arctic 
!                 0.007,  0       ,0.0010 ,0.002,   &  !! type 13 C3 grass grassland
                  0.005,  0       ,0.0010 ,0.002,   &  !! type 13 C3 grass grassland
!                 0.015,  0       ,0.0018 ,0.002,   &  !! type 14 C4 grass grassland
                  0.006,  0       ,0.0018 ,0.002,   &  !! type 14 C4 grass grassland
                  0.015,  0       ,0.0015 ,0.002,   &  !! type 15 crop 1
                  0.015,  0       ,0.0015 ,0.002    /  !! type 16 crop 2 

        data  gbm/0.45,    8,  2.5,  0.1 , &   !! type 1 evergreen needleleaf temperate 
                  0.45,    8,  2.5,  0.1 , &   !! type 2 evergreen needleleaf boreal
                  0.1 ,    8,  2.5,  0.1 , &   !! type 3 deciduous needleleaf
                  0.35,   12,  4.5,  0.1 , &   !! type 4 evergreen broadleaf tropical
                  0.3,     8,  2.0,  0.1 , &   !! type 5 evergreen broadleaf temperate
                  0.2,     5,  2.0,  0.01, &   !! type 6 deciduous broadleaf tropical
                  0.1,     5,  2.0,  0.01, &   !! type 7 deciduous broadleaf temperate 
                  0.05,    5,  2.0,  0.01, &   !! type 8 deciduous broadleaf boreal 
                  0.05,  1.0,  1.0,  0.01, &   !! type 9 broadleaf evergreen shrubland temperate
                  0.05,  0.8,  1.0,  0.01, &   !! type 10 broadleaf deciduous shrubland temperate 
                  0.05,  0.4,  1.0,  0.01, &   !! type 11 broadleaf deciduous shrubland boreal
                  0.05,  0.0,  1.0,  0.01, &   !! type 12 C3 arctic grassland 
                  0.05,  0.0,  1.0,  0.01, &   !! type 13 C3 grassland 
                  0.05,  0.0,  1.0,  0.01, &   !! type 14 C4 grassland 
                  0.05,  0.0,  0.5,  0.01, &   !! type 15 crop 1 
                  0.05,  0.0,  0.5,  0.01 /    !! type 16 crop 2

      data  gpolc/ 400, 800 ,  500, 600, 550, 280,  5000, 3000,  & ! type 1
                   500, 900 ,  600, 700, 650, 380,  7000, 3700,  & ! type 2
                   500, 900 ,  600, 700, 650, 380,  9000, 3700,  & ! type 3
                   400, 800 ,  200, 400, 350, 220, 4500 , 2000,  & ! type 4
                   400, 800 ,  200, 400, 350, 220, 6500 , 2000,  & ! type 5
                   500, 600 ,  900, 500, 500, 200,  4000, 2520,  & ! type 6
                   500, 900 ,  500, 680, 550, 250,  5000, 2700,  & ! type 7
                   500, 600 ,  500, 950, 950, 180,  5000, 2600,  & ! type 8
                   900, 200 ,  500, 200, 200, 100, 5000 , 2200,  & ! type 9
                   500, 100 ,  300, 200, 200, 100, 2000 , 1200,  & ! type 10
                  500 , 300 ,  600, 350, 240, 280, 7000 , 3200,  & ! type 11
                  500 , 300 ,  600, 300, 300, 300, 7000 , 3000,  & ! type 12
                  300 , 250 ,  600, 200, 300, 200, 7000 , 2500,  & ! type 13
                  300 , 320 ,  900, 300, 400, 250, 7000 , 2500,  & ! type 14
                  300 , 300 ,  500, 500, 300, 200, 4000 , 2000,  & ! type 15
                  300 , 200 ,  400, 200, 200, 100, 3000 , 2000 /   ! type 16
                  
#endif

   ! Determine step size

   dtime = get_step_size()
!! lwp 2008-8-29 to modify the step control by "dtime(second)" rather than manually fixed
       nstep_day = 24* 3600/int(dtime)
       nstep = get_nstep()
       year_number = int(nstep/(365*nstep_day)) + 1
!      if (nstep==1) write(101,*) 'dtime, nstep_day= ', dtime, nstep_day
!     write(101,*) 'year_number= ', year_number 

   ! Filter pfts where frac_veg_nosno is non-zero

   fn = 0
   do fp = 1,num_nolakep
      p = filter_nolakep(fp)
!! lwp 2008-10-30 to add land points of Grass or Crop with very small LAI in winter
      if (frac_veg_nosno(p) /= 0) then
         fn = fn + 1
         filterp(fn) = p
      end if
   end do

   ! Initialize

   do f = 1, fn
      p = filterp(f)
      del(p)    = 0._r8  ! change in leaf temperature from previous iteration
      efeb(p)   = 0._r8  ! latent head flux from leaf for previous iteration
      wtlq0(p)  = 0._r8
      wtalq(p)  = 0._r8
      wtgq(p)   = 0._r8
      wtaq0(p)  = 0._r8
      obuold(p) = 0._r8
      btran(p)  = btran0
#ifndef DGVM
      annpsnpot(p) = 0._r8
      annpsn(p)    = 0._r8
#endif

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
      rb1(p) = 0._r8
#endif

   end do

   do j = 1,nlevsoi

      ! Effective porosity of soil, partial volume of ice and liquid (needed
      ! for btran)

!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         c = pcolumn(p)
         l = plandunit(p)

         ! Root resistance factors

         if (t_soisno(c,j) > tfrz) then
            vol_ice = min(watsat(c,j), h2osoi_ice(c,j)/(dz(c,j)*denice))
            eff_porosity = watsat(c,j)-vol_ice
            vol_liq = min(eff_porosity, h2osoi_liq(c,j)/(dz(c,j)*denh2o))
            s_node = max(vol_liq/eff_porosity,0.01_r8)
            smp_node = max(smpmax(ivt(p)), -sucsat(c,j)*s_node**(-bsw(c,j)))
            rresis = (1.-smp_node/smpmax(ivt(p)))/(1.+sucsat(c,j)/smpmax(ivt(p)))
            rootr(p,j) = rootfr(p,j)*rresis
            btran(p) = btran(p) + rootr(p,j)
         else
            rootr(p,j) = 0._r8
         end if
      end do

   end do

   ! Normalize root resistances to get layer contribution to ET

   do j = 1,nlevsoi
!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         rootr(p,j) = rootr(p,j)/btran(p)
      end do
   end do

!      nstep = get_nstep()

!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      c = pcolumn(p)
      g = pgridcell(p)

        if(nstep <= 1.and.finidat == ' ')then
           dis1(p)=gdis(1,ivt(p))
           dis2(p)=gdis(2,ivt(p))
           dis3(p)=gdis(3,ivt(p))
           dis4(p)=gdis(4,ivt(p))
           los1(p)=glos(1,ivt(p))
           los2(p)=glos(2,ivt(p))
           los3(p)=glos(3,ivt(p))
           los4(p)=glos(4,ivt(p))
           bm1(p) =gbm(1,ivt(p))
           bm2(p) =gbm(2,ivt(p))
           bm3(p) =gbm(3,ivt(p))
           bm4(p) =gbm(4,ivt(p))
           polc1(p) = gpolc(1,ivt(p)) 
           polc2(p) = gpolc(2,ivt(p)) 
           polc3(p) = gpolc(3,ivt(p)) 
           polc4(p) = gpolc(4,ivt(p)) 
           polc5(p) = gpolc(5,ivt(p)) 
           polc6(p) = gpolc(6,ivt(p)) 
           polc7(p) = gpolc(7,ivt(p)) 
           polc8(p) = gpolc(8,ivt(p)) 
           alai(:)   =  elai(:)
        endif


      ! Net absorbed longwave radiation by canopy and ground
      ! =air+bir*t_veg**4+cir*t_grnd(c)**4

      air(p) =   emv(p) * (1.+(1.-emv(p))*(1.-emg(c))) * forc_lwrad(g)
      bir(p) = - (2.-emv(p)*(1.-emg(c))) * emv(p) * sb
      cir(p) =   emv(p)*emg(c)*sb

      ! Saturated vapor pressure, specific humidity, and their derivatives
      ! at the leaf surface

      call QSat (t_veg(p), forc_pbot(g), el(p), deldT, qsatl(p), qsatldT(p))

      ! Determine atmospheric co2 and o2
      ! for bit-for-bit results with clm2, using the hardwired local co2 and o2

      co2(p) = pco2 * forc_pbot(g)
      o2(p)  = po2 * forc_pbot(g)

#if (defined OFFLINE)
      co2(p) = pco2 * forc_pbot(g)
      o2(p)  = po2 * forc_pbot(g)

#elif (defined COUP_CSM)
      co2(p)  = forc_co2(g) * forc_pbot(g)            ! CO2 vmr to Pa
      o2(p)   = forc_o2(g)  * forc_pbot(g)            !  O2 vmr to Pa

#elif (defined COUP_CAM)
      co2(p)  = forc_co2(g) * forc_pbot(g)            ! CO2 vmr to Pa
      o2(p)   = forc_o2(g)  * forc_pbot(g)            !  O2 vmr to Pa

#endif

      ! Initialize flux profile

      nmozsgn(p) = 0

      taf(p) = (t_grnd(c) + thm(c))/2.
      qaf(p) = (forc_q(g)+qg(c))/2.

      ur(p) = max(1.0_r8,sqrt(forc_u(g)*forc_u(g)+forc_v(g)*forc_v(g)))
      dth(p) = thm(c)-taf(p)
      dqh(p) = forc_q(g)-qaf(p)
      dthv = dth(p)*(1.+0.61*forc_q(g))+0.61*forc_th(g)*dqh(p)
      zldis(p) = forc_hgt_u(g) - displa(p)

      ! Initialize Monin-Obukhov length and wind speed

      call MoninObukIni(ur(p), thv(c), dthv, zldis(p), z0mv(p), um(p), obu(p))

   end do

   itlef = 0    ! counter for leaf temperature iteration
   fnorig = fn
   fporig(1:fn) = filterp(1:fn)


   ! Begin stability iteration
   ITERATION : do while (itlef <= itmax .and. fn > 0)

      ! Determine friction velocity, and potential temperature and humidity
      ! profiles of the surface boundary layer

      call FrictionVelocity (lbp, ubp, fn, filterp, &
           displa(lbp:ubp), z0mv(lbp:ubp), z0hv(lbp:ubp), z0qv(lbp:ubp), &
           obu(lbp:ubp), itlef+1, ur(lbp:ubp), um(lbp:ubp), ustar(lbp:ubp), &
           temp1(lbp:ubp), temp2(lbp:ubp), temp12m(lbp:ubp), temp22m(lbp:ubp), fm(lbp:ubp))

!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         c = pcolumn(p)
         g = pgridcell(p)

         tlbef(p) = t_veg(p)
         del2(p) = del(p)

         ! Determine aerodynamic resistances

         ram1(p)  = 1./(ustar(p)*ustar(p)/um(p))
         rah(p,1) = 1./(temp1(p)*ustar(p))
         raw(p,1) = 1./(temp2(p)*ustar(p))

         ! Bulk boundary layer resistance of leaves

         uaf = um(p)*sqrt( 1./(ram1(p)*um(p)) )
         cf  = 0.01/(sqrt(uaf)*sqrt(dleaf(ivt(p))))
         rb(p)  = 1./(cf*uaf)
#if (defined BCCCHEM) || (defined MOZART2) || ( defined WACCM)
         rb1(p) = rb(p)
#endif
         ! Parameterization for variation of csoilc with canopy density from
         ! X. Zeng, University of Arizona

         w = exp(-(elai(p)+esai(p)))
         csoilcn = (vkc/(0.13*(z0mg(c)*uaf/1.5e-5)**0.45))*w + csoilc*(1.-w)
         rah(p,2) = 1./(csoilcn*uaf)
         raw(p,2) = rah(p,2)

         ! Stomatal resistances for sunlit and shaded fractions of canopy.
         ! Done each iteration to account for differences in eah, tv.

         svpts(p) = el(p)                         ! pa
         eah(p) = forc_pbot(g) * qaf(p) / 0.622   ! pa
      end do

      call Stomata (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
           parsun(lbp:ubp), svpts(lbp:ubp), eah(lbp:ubp), thm(lbc:ubc), forc_pbot(lbg:ubg),  &
           t_veg(lbp:ubp), o2(lbp:ubp), co2(lbp:ubp), btran(lbp:ubp), rb(lbp:ubp), &
           rssun(lbp:ubp), psnsun(lbp:ubp), annpsnpot(lbp:ubp), annpsn(lbp:ubp),vcmxout(lbp:ubp),ftout(lbp:ubp) )

      call Stomata (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
           parsha(lbp:ubp), svpts(lbp:ubp), eah(lbp:ubp), thm(lbc:ubc), forc_pbot(lbg:ubg), &
           t_veg(lbp:ubp), o2(lbp:ubp), co2(lbp:ubp), btran(lbp:ubp), rb(lbp:ubp), &
           rssha(lbp:ubp), psnsha(lbp:ubp), annpsnpot(lbp:ubp), annpsn(lbp:ubp),vcmxout(lbp:ubp),ftout(lbp:ubp) )

!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         c = pcolumn(p)
         g = pgridcell(p)

         ! Sensible heat conductance for air, leaf and ground
         ! Moved the original subroutine in-line...

         wta    = 1./rah(p,1)                ! air
         wtl    = (elai(p)+esai(p))/rb(p)    ! leaf
         wtg(p) = 1./rah(p,2)                ! ground
         wtshi  = 1./(wta+wtl+wtg(p))

         wtl0(p) = wtl*wtshi         ! leaf
         wtg0    = wtg(p)*wtshi      ! ground
         wta0(p) = wta*wtshi         ! air

         wtga    = wta0(p)+wtg0      ! ground + air
         wtal(p) = wta0(p)+wtl0(p)   ! air + leaf

         ! Fraction of potential evaporation from leaf

         if (fdry(p) .gt. 0._r8) then
            rppdry  = fdry(p)*rb(p)*(laisun(p)/(rb(p)+rssun(p)) + &
                                     laisha(p)/(rb(p)+rssha(p)))/elai(p)
         else
            rppdry = 0._r8
         end if
         efpot = forc_rho(g)*wtl*(qsatl(p)-qaf(p))

         if (efpot > 0._r8) then
            if (btran(p) > btran0) then
               qflx_tran_veg(p) = efpot*rppdry
               rpp = rppdry + fwet(p)
            else
               !No transpiration if btran below 1.e-10
               rpp = fwet(p)
               qflx_tran_veg(p) = 0._r8
            end if
            !Check total evapotranspiration from leaves
            rpp = min(rpp, (qflx_tran_veg(p)+h2ocan(p)/dtime)/efpot)
         else
            !No transpiration if potential evaporation less than zero
            rpp = 1._r8
            qflx_tran_veg(p) = 0._r8
         end if

         ! Update conductances for changes in rpp
         ! Latent heat conductances for ground and leaf.
         ! Air has same conductance for both sensible and latent heat.
         ! Moved the original subroutine in-line...

         wtaq    = frac_veg_nosno(p)/raw(p,1)                        ! air
         wtlq    = frac_veg_nosno(p)*(elai(p)+esai(p))/rb(p) * rpp   ! leaf
         wtgq(p) = frac_veg_nosno(p)/raw(p,2)                        ! ground
         wtsqi   = 1./(wtaq+wtlq+wtgq(p))

         wtgq0    = wtgq(p)*wtsqi      ! ground
         wtlq0(p) = wtlq*wtsqi         ! leaf
         wtaq0(p) = wtaq*wtsqi         ! air

         wtgaq    = wtaq0(p)+wtgq0     ! air + ground
         wtalq(p) = wtaq0(p)+wtlq0(p)  ! air + leaf

         dc1 = forc_rho(g)*cpair*wtl
         dc2 = hvap*forc_rho(g)*wtlq

         efsh   = dc1*(wtga*t_veg(p)-wtg0*t_grnd(c)-wta0(p)*thm(c))
         efe(p) = dc2*(wtgaq*qsatl(p)-wtgq0*qg(c)-wtaq0(p)*forc_q(g))

         ! Evaporation flux from foliage

         erre = 0._r8
         if (efe(p)*efeb(p) < 0._r8) then
            efeold = efe(p)
            efe(p)  = 0.1*efeold
            erre = efe(p) - efeold
         end if
         dt_veg(p) = (sabv(p) + air(p) + bir(p)*t_veg(p)**4 + &
              cir(p)*t_grnd(c)**4 - efsh - efe(p)) / &
              (- 4.*bir(p)*t_veg(p)**3 +dc1*wtga +dc2*wtgaq*qsatldT(p))
         t_veg(p) = tlbef(p) + dt_veg(p)
         dels = dt_veg(p)
         del(p)  = abs(dels)
         err(p) = 0._r8
         if (del(p) > delmax) then
            dt_veg(p) = delmax*dels/del(p)
            t_veg(p) = tlbef(p) + dt_veg(p)
            err(p) = sabv(p) + air(p) + bir(p)*tlbef(p)**3*(tlbef(p) + &
                 4.*dt_veg(p)) + cir(p)*t_grnd(c)**4 - &
                 (efsh + dc1*wtga*dt_veg(p)) - (efe(p) + &
                 dc2*wtgaq*qsatldT(p)*dt_veg(p))
         end if

         ! Fluxes from leaves to canopy space
         ! "efe" was limited as its sign changes frequently.  This limit may
         ! result in an imbalance in "hvap*qflx_evap_veg" and
         ! "efe + dc2*wtgaq*qsatdt_veg"

         efpot = forc_rho(g)*wtl*(wtgaq*(qsatl(p)+qsatldT(p)*dt_veg(p)) &
            -wtgq0*qg(c)-wtaq0(p)*forc_q(g))
         qflx_evap_veg(p) = rpp*efpot

         ! Calculation of evaporative potentials (efpot) and
         ! interception losses; flux in kg m**-2 s-1.  ecidif
         ! holds the excess energy if all intercepted water is evaporated
         ! during the timestep.  This energy is later added to the
         ! sensible heat flux.

         ecidif = 0._r8
         if (efpot > 0._r8 .and. btran(p) > btran0) then
            qflx_tran_veg(p) = efpot*rppdry
         else
            qflx_tran_veg(p) = 0._r8
         end if
         ecidif = max(0._r8, qflx_evap_veg(p)-qflx_tran_veg(p)-h2ocan(p)/dtime)
         qflx_evap_veg(p) = min(qflx_evap_veg(p),qflx_tran_veg(p)+h2ocan(p)/dtime)

         ! The energy loss due to above two limits is added to
         ! the sensible heat flux.

         eflx_sh_veg(p) = efsh + dc1*wtga*dt_veg(p) + err(p) + erre + hvap*ecidif

         ! Re-calculate saturated vapor pressure, specific humidity, and their
         ! derivatives at the leaf surface

         call QSat(t_veg(p), forc_pbot(g), el(p), deldT, qsatl(p), qsatldT(p))

         ! Update vegetation/ground surface temperature, canopy air
         ! temperature, canopy vapor pressure, aerodynamic temperature, and
         ! Monin-Obukhov stability parameter for next iteration.

         taf(p) = wtg0*t_grnd(c) + wta0(p)*thm(c) + wtl0(p)*t_veg(p)
         qaf(p) = wtlq0(p)*qsatl(p) + wtgq0*qg(c) + forc_q(g)*wtaq0(p)

         ! Update Monin-Obukhov length and wind speed including the
         ! stability effect

         dth(p) = thm(c)-taf(p)
         dqh(p) = forc_q(g)-qaf(p)

         tstar = temp1(p)*dth(p)
         qstar = temp2(p)*dqh(p)

         thvstar = tstar*(1.+0.61*forc_q(g)) + 0.61*forc_th(g)*qstar
         zeta = zldis(p)*vkc*grav*thvstar/(ustar(p)**2*thv(c))

         if (zeta >= 0.) then     !stable
            zeta = min(2._r8,max(zeta,0.01_r8))
            um(p) = max(ur(p),0.1_r8)
         else                     !unstable
            zeta = max(-100._r8,min(zeta,-0.01_r8))
            wc = beta*(-grav*ustar(p)*thvstar*zii/thv(c))**0.333
            um(p) = sqrt(ur(p)*ur(p)+wc*wc)
         end if
         obu(p) = zldis(p)/zeta

         if (obuold(p)*obu(p) < 0.) nmozsgn(p) = nmozsgn(p)+1
         if (nmozsgn(p) >= 4) obu(p) = zldis(p)/(-0.01)
         obuold(p) = obu(p)

      end do   ! end of filtered pft loop

      ! Test for convergence

      itlef = itlef+1
      if (itlef > itmin) then
!dir$ concurrent
!cdir nodep
         do f = 1, fn
            p = filterp(f)
            dele(p) = abs(efe(p)-efeb(p))
            efeb(p) = efe(p)
            det(p)  = max(del(p),del2(p))
         end do
         fnold = fn
         fn = 0
         do f = 1, fnold
            p = filterp(f)
            if (.not. (det(p) < dtmin .and. dele(p) < dlemin)) then
               fn = fn + 1
               filterp(fn) = p
            end if
         end do
      end if

   end do ITERATION     ! End stability iteration

   fn = fnorig
   filterp(1:fn) = fporig(1:fn)

!! lwp 2008-5-23 to move the following AVIM part outside the ITERATION

!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      c = pcolumn(p)
      g = pgridcell(p)

      ! Energy balance check in canopy

      err(p) = sabv(p) + air(p) + bir(p)*tlbef(p)**3*(tlbef(p) + 4.*dt_veg(p)) &
         + cir(p)*t_grnd(c)**4 - eflx_sh_veg(p) - hvap*qflx_evap_veg(p)

      ! Fluxes from ground to canopy space

      delt    = wtal(p)*t_grnd(c)-wtl0(p)*t_veg(p)-wta0(p)*thm(c)
      delq    = wtalq(p)*qg(c)-wtlq0(p)*qsatl(p)-wtaq0(p)*forc_q(g)
      taux(p) = -forc_rho(g)*forc_u(g)/ram1(p)
      tauy(p) = -forc_rho(g)*forc_v(g)/ram1(p)
      eflx_sh_grnd(p) = cpair*forc_rho(g)*wtg(p)*delt
      qflx_evap_soi(p) = forc_rho(g)*wtgq(p)*delq

      ! 2 m height air temperature

      t_ref2m(p) = thm(c) + temp1(p)*dth(p)*(1./temp12m(p) - 1./temp1(p))

      ! 2 m height specific humidity

      q_ref2m(p) = forc_q(g) + temp2(p)*dqh(p)*(1./temp22m(p) - 1./temp2(p))

      ! Downward longwave radiation below the canopy

      dlrad(p) = (1.-emv(p))*emg(c)*forc_lwrad(g) + &
         emv(p)*emg(c)*sb*tlbef(p)**3*(tlbef(p) + 4.*dt_veg(p))

      ! Upward longwave radiation above the canopy

      ulrad(p) = ((1.-emg(c))*(1.-emv(p))*(1.-emv(p))*forc_lwrad(g) &
         + emv(p)*(1.+(1.-emg(c))*(1.-emv(p)))*sb*tlbef(p)**3*(tlbef(p) + &
         4.*dt_veg(p)) + emg(c)*(1.-emv(p))*sb*t_grnd(c)**4)

      ! Derivative of soil energy flux with respect to soil temperature

      cgrnds(p) = cgrnds(p) + cpair*forc_rho(g)*wtg(p)*wtal(p)
      cgrndl(p) = cgrndl(p) + forc_rho(g)*wtgq(p)*wtalq(p)*dqgdT(c)
      cgrnd(p)  = cgrnds(p) + cgrndl(p)*htvp(c)

      ! Update dew accumulation (kg/m2)

      h2ocan(p) = max(0._r8,h2ocan(p)+(qflx_tran_veg(p)-qflx_evap_veg(p))*dtime)

      ! total photosynthesis

      laisun(p)=max(laisun(p),0.2)
      fpsn(p) = psnsun(p)*laisun(p) + psnsha(p)*laisha(p)
!! lwp 2008-7-1 to change the unit of CLM3 fpsn to AVIM pht (umol/m**2/s CO2 to Kg/m**2/day biomass)
!! 0.0024624=28.5*1.0e-9*3600*24 
!     pk =0.5
!       fl=(1.-exp(-pk*(elai(p)+esai(p))))/pk
!     fl =1.0
!     pht(p)  = fpsn(p)*0.0024624*fl
#if (defined AVIM)
      pht(p)  = fpsn(p)*0.0024624
#endif

    enddo         

!! lwp 2008-7-1 to move AVIM part here to use CLM3's fpsn as total photosynthesis

#if (defined AVIM)

!! lwp 2008-8-29 to initialize daily mean variables at the beginning of each day
#ifdef OFFLINE
  if (mod(nstep-1,nstep_day)==0 ) then  !! nstep=1 for first step offline run
#else
  if (mod(nstep,nstep_day)==0 ) then    !! nstep=0 for first step CAM-CLM3 coupled runs
#endif

        phd(:)  = 0.
        npd(:)  = 0.
!       ded(:)  = 0.
        ts1d(:) = 0.
        ts2d(:) = 0.
        ts3d(:) = 0.
        sm1d(:) = 0.
        sm2d(:) = 0.
       endif

      call NPPRmRg (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg,  &
           thm(lbc:ubc),taf(lbp:ubp),rssun(lbp:ubp), &
           pht(lbp:ubp),npp(lbp:ubp),rmp(lbp:ubp),rgp(lbp:ubp), &
           phd(lbp:ubp),npd(lbp:ubp), &
           ts1d(lbp:ubp),ts2d(lbp:ubp),sm1d(lbp:ubp),sm2d(lbp:ubp), &
           dis1(lbp:ubp),dis2(lbp:ubp),dis3(lbp:ubp),dis4(lbp:ubp), &
           bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp),bm4(lbp:ubp) )
!          los1(lbp:ubp),los2(lbp:ubp),los3(lbp:ubp),los4(lbp:ubp), &

!    if(mod(nstep,48)==0) then    ! 48 steps per-day when the time step is 1800 seconds
#ifdef OFFLINE
  if (mod(nstep,nstep_day)==0 ) then  !! nstep=1 for first step offline run
#else
  if (mod(nstep+1,nstep_day)==0 ) then    !! nstep=0 for first step CAM-CLM3 coupled runs
#endif
     call  Dislos (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
           npd(lbp:ubp), alai(lbp:ubp), &
           dis1(lbp:ubp),dis2(lbp:ubp),dis3(lbp:ubp),dis4(lbp:ubp), &
           los1(lbp:ubp),los2(lbp:ubp),los3(lbp:ubp),los4(lbp:ubp),glos )
     call  Bmlai  (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
           npd(lbp:ubp), alai(lbp:ubp), &
           dis1(lbp:ubp),dis2(lbp:ubp),dis3(lbp:ubp),dis4(lbp:ubp), &
           los1(lbp:ubp),los2(lbp:ubp),los3(lbp:ubp),los4(lbp:ubp),&
           bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp),bm4(lbp:ubp),surl(lbp:ubp),rotl(lbp:ubp) )
!    write(111,*) 'OKKKK, nstep, nstep_day, t_zyw=', nstep, nstep_day, t_zyw
!! lwp 2008-11-7 to speedup soil carbon cycling during the first 100 years, then 20 years, then 30years
!    if ( year_number<=100 ) then 
!       surl(:) = surl(:)/20
!       rotl(:) = rotl(:)/20
!      do kkk=1,20
!        call SoilCarbon (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
!             ts1d(lbp:ubp),ts2d(lbp:ubp),sm1d(lbp:ubp),sm2d(lbp:ubp), &
!             los1(lbp:ubp),los2(lbp:ubp),los3(lbp:ubp), &
!             bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp), &
!             polc1(lbp:ubp),polc2(lbp:ubp),polc3(lbp:ubp),polc4(lbp:ubp), &
!             polc5(lbp:ubp),polc6(lbp:ubp),polc7(lbp:ubp),polc8(lbp:ubp), &
!             dsmc(lbp:ubp),dco2(lbp:ubp),nmin(lbp:ubp), 1.0, &
!             npd(lbp:ubp),nep(lbp:ubp)  )
!      enddo
!    else if ( year_number<=120 ) then 
!      do kkk=1,121-year_number
!        call SoilCarbon (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
!             ts1d(lbp:ubp),ts2d(lbp:ubp),sm1d(lbp:ubp),sm2d(lbp:ubp), &
!             los1(lbp:ubp),los2(lbp:ubp),los3(lbp:ubp), &
!             bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp), &
!             polc1(lbp:ubp),polc2(lbp:ubp),polc3(lbp:ubp),polc4(lbp:ubp), &
!             polc5(lbp:ubp),polc6(lbp:ubp),polc7(lbp:ubp),polc8(lbp:ubp), &
!             dsmc(lbp:ubp),dco2(lbp:ubp),nmin(lbp:ubp), 1.0, &
!             npd(lbp:ubp),nep(lbp:ubp)  )
!      enddo
!    else 
         call SoilCarbon (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
              surl(lbp:ubp),rotl(lbp:ubp), &
              ts1d(lbp:ubp),ts2d(lbp:ubp),sm1d(lbp:ubp),sm2d(lbp:ubp), &
              polc1(lbp:ubp),polc2(lbp:ubp),polc3(lbp:ubp),polc4(lbp:ubp), &
              polc5(lbp:ubp),polc6(lbp:ubp),polc7(lbp:ubp),polc8(lbp:ubp), &
              dsmc(lbp:ubp),dco2(lbp:ubp),nmin(lbp:ubp), 1.0, &
              npd(lbp:ubp),nep(lbp:ubp)  )
!    endif
   endif
#endif

   ! filter out pfts which have small energy balance errors; report others
   fnold = fn
   fn = 0
   do f = 1, fnold
      p = filterp(f)
      if (abs(err(p)) > 0.1) then
         fn = fn + 1
         filterp(fn) = p
      end if
   end do

   do f = 1, fn
      p = filterp(f)
      write(6,*) 'energy balance in canopy ',p,', err=',err(p)
   end do

   end subroutine CanopyFluxes

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: Stomata
!
! !INTERFACE:
   subroutine Stomata (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                       apar, ei, ea, tgcm, forc_pbot, &
                       tl, o2, co2, btran, rb, &
                       rs, psn, annpsnpot, annpsn,vcmxout,ftout)
!
! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_RGAS
     use clmtype
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in)    :: apar(lbp:ubp)      ! par absorbed per unit lai (w/m**2)
     real(r8), intent(in)    :: ei(lbp:ubp)        ! vapor pressure inside leaf (sat vapor press at tl) (pa)
     real(r8), intent(in)    :: ea(lbp:ubp)        ! vapor pressure of canopy air (pa)
     real(r8), intent(in)    :: tgcm(lbc:ubc)      ! air temperature at agcm reference height (kelvin)
     real(r8), intent(in)    :: forc_pbot(lbg:ubg) ! atmospheric pressure (Pa)
     real(r8), intent(in)    :: tl(lbp:ubp)        ! leaf temperature (Kelvin)
     real(r8), intent(in)    :: o2(lbp:ubp)        ! atmospheric o2 concentration (pa)
     real(r8), intent(in)    :: co2(lbp:ubp)       ! atmospheric co2 concentration (pa)
     real(r8), intent(in)    :: btran(lbp:ubp)     ! soil water transpiration factor (0 to 1)
     real(r8), intent(inout) :: rb(lbp:ubp)        ! boundary layer resistance (s/m)
     real(r8), intent(inout) :: rs(lbp:ubp)        ! leaf stomatal resistance (s/m)
     real(r8), intent(inout) :: psn(lbp:ubp)       ! foliage photosynthesis (umol co2 /m**2/ s) [always +]
     real(r8), intent(inout) :: annpsnpot(lbp:ubp) ! annual potential photosynthesis (umol CO2 /m**2)
     real(r8), intent(inout) :: annpsn(lbp:ubp)    ! annual photosynthesis (umol CO2 /m**2)
#if (defined AVIM)
     real(r8), pointer :: tmax(:)          !
     real(r8), pointer :: tmin(:)          !
     real(r8), pointer :: topt(:)
#endif
!
! !CALLED FROM:
! subroutine CanopyFluxes in this modeul
!
! !REVISION HISTORY:
! author:            Gordon Bonan
! standardized:      J. Truesdale, Feb. 1996
! reviewed:          G. Bonan, Feb. 1996
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! 1/30/02, PET  Made all input and output parameters explicit (removed reference to clm)
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     real(r8), pointer :: qe25(:)        ! quantum efficiency at 25C (umol CO2 / umol photon)
     real(r8), pointer :: vcmx25(:)      ! max rate of carboxylation at 25C (umol CO2/m**2/s)
     real(r8), pointer :: c3psn(:)       ! photosynthetic pathway: 0. = c4, 1. = c3
     real(r8), pointer :: mp(:)          ! slope of conductance-to-photosynthesis relationship
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
     real(r8), parameter :: mpe = 1.e-6   ! prevents overflow error if division by zero
     integer , parameter :: niter = 3     ! number of iterations
     integer  :: f,p,c,g ! indices
     integer  :: iter    ! iteration index
     real(r8) :: ab      ! used in statement functions
     real(r8) :: bc      ! used in statement functions
     real(r8) :: f1      ! generic temperature response (statement function)
     real(r8) :: f2      ! generic temperature inhibition (statement function)
     real(r8) :: tc      ! leaf temperature (degree celsius)
     real(r8) :: cs      ! co2 concentration at leaf surface (pa)
     real(r8) :: kc      ! co2 michaelis-menten constant (pa)
     real(r8) :: ko      ! o2 michaelis-menten constant (pa)
     real(r8) :: atmp    ! intermediate calculations for rs
     real(r8) :: btmp    ! intermediate calculations for rs
     real(r8) :: ctmp    ! intermediate calculations for rs
     real(r8) :: q       ! intermediate calculations for rs
     real(r8) :: r1,r2   ! roots for rs
     real(r8) :: ppf     ! absorb photosynthetic photon flux (umol photons/m**2/s)
     real(r8) :: wc      ! rubisco limited photosynthesis (umol co2/m**2/s)
     real(r8) :: wj      ! light limited photosynthesis (umol co2/m**2/s)
     real(r8) :: we      ! export limited photosynthesis (umol co2/m**2/s)
     real(r8) :: cp      ! co2 compensation point (pa)
     real(r8) :: ci      ! internal co2 (pa)
     real(r8) :: awc     ! intermediate calcuation for wc
     real(r8) :: vcmx    ! maximum rate of carboxylation (umol co2/m**2/s)
#if (defined DGVM)
     real(r8) :: vcmxpot ! potential maximum rate of carboxylation (umol co2/m**2/s)
#endif
     real(r8) :: j       ! electron transport (umol co2/m**2/s)
     real(r8) :: cea     ! constrain ea or else model blows up
     real(r8) :: cf      ! s m**2/umol -> s/m
     real(r8) :: rsmax0  ! maximum stomatal resistance [s/m]

     real(r8) :: kc25    ! co2 michaelis-menten constant at 25c (pa)
     real(r8) :: akc     ! q10 for kc25
     real(r8) :: ko25    ! o2 michaelis-menten constant at 25c (pa)
     real(r8) :: ako     ! q10 for ko25
     real(r8) :: avcmx   ! q10 for vcmx25
     real(r8) :: bp      ! minimum leaf conductance (umol/m**2/s)
        real(r8) :: ftemp ! temperature restriction parameter 
!! lwp 2008-8-28 to change the intent attribute of btran to set >=0.1 restriction on btran
     real(r8) :: btran1     ! soil water transpiration factor (0 to 1)
     real(r8) :: vcmxout(lbp:ubp)     ! actual vcmx 
     real(r8) :: ftout(lbp:ubp)     ! actual vcmx 
!------------------------------------------------------------------------------

     ! Set statement functions

     f1(ab,bc) = ab**((bc-25.)/10.)
     f2(ab) = 1. + exp((-2.2e05+710.*(ab+SHR_CONST_TKFRZ))/(SHR_CONST_RGAS*0.001*(ab+SHR_CONST_TKFRZ)))

     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     qe25      => pftcon%qe25
     vcmx25    => pftcon%vcmx25
     c3psn     => pftcon%c3psn
     mp        => pftcon%mp

     ! Set constant values

     kc25  = 30.
     akc   = 2.1
     ko25  = 30000.
     ako   = 1.2
     avcmx = 2.4
     bp    = 2000.
#if (defined AVIM)
     tmax      => pftcon%tmax
     tmin      => pftcon%tmin
     topt      => pftcon%topt
#endif

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)

        ! Initialize rs=rsmax and psn=0 because calculations are performed only
        ! when apar > 0, in which case rs <= rsmax and psn >= 0
        ! Set constants

        rsmax0 = 2.e4
        cf = forc_pbot(g)/(SHR_CONST_RGAS*0.001*tgcm(c))*1.e06

        if (apar(p) <= 0.) then          ! night time
           rs(p) = min(rsmax0, 1./bp * cf)
           psn(p) = 0._r8
        else                             ! day time
           tc = tl(p) - SHR_CONST_TKFRZ
           ppf = 4.6 * apar(p)
           j = ppf * qe25(ivt(p))
           kc = kc25 * f1(akc,tc)
           ko = ko25 * f1(ako,tc)
           awc = kc * (1.+o2(p)/ko)
           cp = 0.5*kc/ko*o2(p)*0.21

!! lwp 2008-7-22 use CLM3 temperature & moisture restriction on vcmx 
!! lwp 2008-8-28 set a lower limit for btran (>=0.20) 
#if(defined AVIM)
!          btran1 = max (0.20, btran(p) )
           btran1 = max (0.30, btran(p) )
           if(ivt(p)==15)btran1 = max (0.40, btran(p) )
!! lwp 2008-8-28 test the temperature restriction on vcmax according to AVIM(not good, comment out)
!! lwp 2008-10-12 using AVIM f(T)
 !  if(tl(p).le.topt(ivt(p))) ftemp=1.-((tl(p)-topt(ivt(p)))/(tmin(ivt(p))-topt(ivt(p))))**2
 !  if(tl(p).gt.topt(ivt(p))) ftemp=1.-((tl(p)-topt(ivt(p)))/(tmax(ivt(p))-topt(ivt(p))))**2
 !          if(tl(p).lt.tmin(ivt(p)))  ftemp=0.1e-5
 !          if(tl(p).gt.tmax(ivt(p)))  ftemp=0.1e-5
 !          vcmx = vcmx25(ivt(p)) * ftemp * btran1
!! lwp 2008-10-12 using CLM3 f(T)
           vcmx = vcmx25(ivt(p)) * f1(avcmx,tc) / f2(tc) * btran1
#else
           vcmx = vcmx25(ivt(p)) * f1(avcmx,tc) / f2(tc) * btran(p)
#endif
!! lwp 2008-9-2 to output vcmx 
           vcmxout(p) = vcmx
!! use CLM3 f(T)
           ftout(p) = f1(avcmx,tc) / f2(tc) 
!! use AVIM f(T)
!          ftout(p) = ftemp 

#if (defined DGVM)
           vcmxpot = vcmx25(ivt(p)) * f1(avcmx,tc) / f2(tc)
#endif

           ! First guess ci

           ci = 0.7*co2(p)*c3psn(ivt(p)) + 0.4*co2(p)*(1.-c3psn(ivt(p)))

           ! rb: s/m -> s m**2 / umol

           rb(p) = rb(p)/cf

           ! Constrain ea

           cea = max(0.25*ei(p)*c3psn(ivt(p))+0.40*ei(p)*(1.-c3psn(ivt(p))), min(ea(p),ei(p)) )

#if (defined DGVM)
           ! ci iteration for 'potential' photosynthesis

           do iter = 1, niter
              wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
              wc = max(ci-cp,0._r8)*vcmxpot/(ci+awc)*c3psn(ivt(p)) + vcmxpot*(1.-c3psn(ivt(p)))
              we = 0.5*vcmxpot*c3psn(ivt(p)) + 4000.*vcmxpot*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
              psn(p) = min(wj,wc,we)
              cs = max( co2(p) - 1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
              atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
              btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
              ctmp = -rb(p)
              if (btmp >= 0.) then
                 q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
              else
                 q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
              end if
              r1 = q/atmp
              r2 = ctmp/q
              rs(p) = max(r1,r2)
              ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           end do

           annpsnpot(p) = annpsnpot(p) + psn(p)

           ! First guess ci

           ci = 0.7*co2(p)*c3psn(ivt(p)) + 0.4*co2(p)*(1.-c3psn(ivt(p)))
#endif

           ! ci iteration for 'actual' photosynthesis

#if (defined NEC_SX)
!NEC NEC NEC
           if ( niter /= 3 ) then
              write(6,*) 'Stomata: niter /= 3';  call endrun()
           end if
           !
           ! ITER = 1
           !
           wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
           wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
           we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
           psn(p) = min(wj,wc,we)
           cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
           atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
           btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
           ctmp = -rb(p)
           if (btmp >= 0.) then
              q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
           else
              q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
           end if
           r1 = q/atmp
           r2 = ctmp/q
           rs(p) = max(r1,r2)
           ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           !
           ! ITER = 2
           !
           wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
           wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
           we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
           psn(p) = min(wj,wc,we)
           cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
           atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
           btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
           ctmp = -rb(p)
           if (btmp >= 0.) then
              q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
           else
              q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
           end if
           r1 = q/atmp
           r2 = ctmp/q
           rs(p) = max(r1,r2)
           ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           !
           ! ITER = 3
           !
           wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
           wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
           we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
           psn(p) = min(wj,wc,we)
           cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
           atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
           btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
           ctmp = -rb(p)
           if (btmp >= 0.) then
              q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
           else
              q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
           end if
           r1 = q/atmp
           r2 = ctmp/q
           rs(p) = max(r1,r2)
           ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
!NEC NEC NEC
#else
           do iter = 1, niter
              wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
              wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
              we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
              psn(p) = min(wj,wc,we)
              cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
              atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
              btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
              ctmp = -rb(p)
              if (btmp >= 0.) then
                 q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
              else
                 q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
              end if
              r1 = q/atmp
              r2 = ctmp/q
              rs(p) = max(r1,r2)
              ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           end do
#endif

#if (defined DGVM)
           annpsn(p) = annpsn(p) + psn(p)
#endif

           ! rs, rb:  s m**2 / umol -> s/m

           rs(p) = min(rsmax0, rs(p)*cf)
           rb(p) = rb(p) * cf

        end if

     end do

  end subroutine Stomata

!------------------------------------------------------------------------------
!! lwp 2008-9-1 the following are source codes of AVIM

#if (defined AVIM)

!BOP
!
! !IROUTINE: NPPRmRg
!
! !INTERFACE:
   subroutine NPPRmRg (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                       tgcm, taf,rs, &
                       pht,npp,rmp,rgp , &
                       phd,npd, &
                       ts1d,ts2d,sm1d,sm2d,&
                       dis1,dis2,dis3,dis4,&
!                      los1,los2,los3,los4,&
                       bm1,bm2,bm3,bm4)

!
! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_RGAS
     use time_manager       , only : get_step_size , get_nstep
     use clmtype
!! lwp 2008-5-20
     use spmdMod, only: masterproc
!! lwp 2008-8-20  to use dzsoil for weighted soil moisture and temperature
    use clm_varpar   , only: numcol, numrad, nlevsoi, nlevlak
    use clm_varcon   , only: dzsoi, zsoi
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in)    :: tgcm(lbc:ubc)      ! air temperature at agcm reference height (kelvin)
!    real(r8), intent(in)    :: tl(lbp:ubp)        ! leaf temperature (Kelvin)
     real(r8), intent(in)    :: taf(lbp:ubp)       ! within canopy air temperature (Kelvin)
     real(r8), intent(inout) :: rs(lbp:ubp)        ! leaf stomatal resistance (s/m)
     real(r8), intent(inout) :: pht(lbp:ubp)       ! foliage photosynthesis (umol co2 /m**2/ s) [always +]

     real(r8), intent(  out) :: npp(lbp:ubp)       !
     real(r8), intent(  out) :: rmp(lbp:ubp)       !
     real(r8), intent(  out) :: rgp(lbp:ubp)       !

!!   lwp 2008-5-22 to change the intent characteristics of the following three from 'out' to 'inout'
     real(r8), intent(inout) :: phd(lbp:ubp)       !
     real(r8), intent(inout) :: npd(lbp:ubp)       !
     real(r8), intent(inout) :: ts1d(lbp:ubp)      ! daily mean 1rd-layer soil temperature
     real(r8), intent(inout) :: ts2d(lbp:ubp)      ! daily mean 2rd-layer soil temperature
!    real(r8), intent(inout) :: ts3d(lbp:ubp)      ! daily mean 3rd-layer soil temperature
     real(r8), intent(inout) :: sm1d(lbp:ubp)      ! daily mean 1rd-layer soil moisture 
     real(r8), intent(inout) :: sm2d(lbp:ubp)      ! daily mean 2rd-layer soil moisture
!    real(r8), intent(inout) :: sm3d(lbp:ubp)      ! daily mean 3rd-layer soil moisture
!    real(r8), intent(inout) :: ded(lbp:ubp)       !
!    real(r8) :: phdo(lbp:ubp)
!    real(r8) :: npdo(lbp:ubp)
!    real(r8) :: dedo(lbp:ubp)

     real(r8), intent(in   )                :: dis1(lbp:ubp)       !
     real(r8), intent(in   )                :: dis2(lbp:ubp)       !
     real(r8), intent(in   )                :: dis3(lbp:ubp)       !
     real(r8), intent(in   )                :: dis4(lbp:ubp)       !
     real(r8), intent(in   )                ::  bm1(lbp:ubp)       !
     real(r8), intent(in   )                ::  bm2(lbp:ubp)       !
     real(r8), intent(in   )                ::  bm3(lbp:ubp)       !
     real(r8), intent(in   )                ::  bm4(lbp:ubp)       !
!
! !CALLED FROM:
! subroutine CanopyFluxes in this modeul
!
! !REVISION HISTORY:
! author:            JJJ         
! standardized:      JJJ and zhangyw 2007 
! reviewed:          zhangyw 2007 
! revised:           Li Weiping, 2008-07 
! 12/18/07, PET  Made all input and output parameters explicit (removed reference to clm)
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     real(r8), pointer :: londeg(:)        ! latitude (degrees)
     real(r8), pointer :: latdeg(:)        ! latitude (degrees)

     real(r8), pointer :: h2osoi_vol(:,:)  ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
     real(r8), pointer :: watsat(:,:)       ! volumetric soil water at saturation (porosity) (nlevsoi)
     real(r8), pointer :: forc_q(:)       !atmospheric specific humidity (kg/kg)
     real(r8), pointer :: t_grnd(:)       !ground temperature (Kelvin)          
     real(r8), pointer :: t_soisno(:,:)    ! soil temperature (Kelvin)  (-nlevsno+1:nlevsoi)

     real(r8), pointer :: elai(:)        ! one-sided leaf area index
     real(r8), pointer :: esai(:)        ! one-sided leaf area index

     real(r8), pointer :: crg1(:)        ! growth respiration factor for leaf 
     real(r8), pointer :: crg2(:)        ! growth respiration factor for stem 
     real(r8), pointer :: crg3(:)        ! growth respiration factor for root 
     real(r8), pointer :: crg4(:)        ! growth respiration factor for root 
     real(r8), pointer :: crm1(:)        ! maintain respiration factor for leaf 
     real(r8), pointer :: crm2(:)        ! maintain respiration factor for stem 
     real(r8), pointer :: crm3(:)        ! maintain respiration factor for root 
     real(r8), pointer :: crm4(:)        ! maintain respiration factor for root 
!    real(r8), pointer :: dis1(:)        ! fractional distribution of NPP to leaf 
!    real(r8), pointer :: dis2(:)        ! fractional distribution of NPP to stem 
!    real(r8), pointer :: dis3(:)        ! fractional distribution of NPP to root 
!    real(r8), pointer :: dis4(:)        ! fractional distribution of NPP to 
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
!    real(r8), parameter :: mpe = 1.e-6   ! prevents overflow error if division by zero
!    integer , parameter :: niter = 3     ! number of iterations
     integer  :: f,p,c,g ! indices
!    integer  :: iter    ! iteration index
!------------------------------------------------------------------------------

!------------------------------Arguments--------------------------------
!input
        real(r8) ::  pk    ! canopy extinction coefficient
        real(r8) ::  lai   ! leaf area index
        real(r8) ::  dthd  ! dth=dtd/24 dtd=84000s   or =1  ?
        real(r8) ::  dtd  ! 
!       real(r8) ::  crg(4)
!       real(r8) ::  crm(4)
!! 2008-5-27  add the following three variables for initialization
!! itype has one more(type=0) than the 13 kinds of vegetation type of AVIM
        real(r8) :: gbm (4,16)
        real(r8) :: glos(4,16)
        real(r8) :: gdis(4,16)

        real(r8) ::  t25
        real(r8) ::  ts1    ! first soil temperature
        real(r8) ::  ts2    !  2nd soil temperature
        real(r8) ::  ts3    !  3rd soil temperature
        real(r8) ::  ws    ! ws=0.5*(w1+w2)
        real(r8) ::  wi    ! wilting point
        real(r8) ::  sm1    ! first soil water content
        real(r8) ::  sm2    ! second soil water content

!       real(r8) ::  rf    ! leaf transmit
!       real(r8) ::  pht0  ! photosynthate

!output
        real(r8) ::  rm1d,rm2d,rm3d
        real(r8) ::  rg1d,rg2d,rg3d
        real(r8) ::  npo
        real(r8) ::  pht1 
        real(r8) ::  rmd,deg
        real(r8) ::  rmr
        real(r8) ::  rgr
        real(r8) ::  npr
! ---!LOCAL VARIABLES
!
        real(r8) :: qtaf
        real(r8) :: qstem
        real(r8) :: degom
        real(r8) :: fl
        real(r8) :: fw2
        real(r8) :: qt1
        real(r8) :: fdw
        real(r8) :: decom
        real(r8) :: rm1,rm2,rm3
        real(r8) :: rg1,rg2,rg3
        real(r8) :: ph
        real(r8) :: rf   ! leaf transmit
        real(r8) :: dtime                 ! land model time step (sec)
        real(r8) :: height  ! height of top 5 soil layer to calculate the weighted average 
        integer  :: nstep         ! time step number
!! lwp 2008-8-25 to calculate weighted soil temp and moisture for top 10cm, top 170cm
        real(r8) :: dz4, dz9 
!---------------zhangyw------------------------------------------------

     ! Set statement functions

     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     londeg         => clm3%g%londeg
     latdeg         => clm3%g%latdeg
     
     h2osoi_vol     => clm3%g%l%c%cws%h2osoi_vol
     watsat         => clm3%g%l%c%cps%watsat
     t_soisno       => clm3%g%l%c%ces%t_soisno
     forc_q         => clm3%g%a2ls%forc_q
     
     elai           => clm3%g%l%c%p%pps%elai
     esai           => clm3%g%l%c%p%pps%esai
     t_grnd     => clm3%g%l%c%ces%t_grnd
!  dis1           => clm3%g%l%c%p%pcf%dis1
!  dis2           => clm3%g%l%c%p%pcf%dis2
!  dis3           => clm3%g%l%c%p%pcf%dis3
!  dis4           => clm3%g%l%c%p%pcf%dis4

     crg1     =>  pftcon%crg1
     crg2     =>  pftcon%crg2
     crg3     =>  pftcon%crg3
     crg4     =>  pftcon%crg4
     crm1     =>  pftcon%crm1
     crm2     =>  pftcon%crm2
     crm3     =>  pftcon%crm3
     crm4     =>  pftcon%crm4

     ! Set constant values

!zhangyw

      pk=0.5
      t25=298.0
    
! lwp 2008-5-20 add the following definition of wi
      wi = 0.12
      
      dtd=1.
!      dthd=dtd/24.
      
      dtime = get_step_size()
      dthd=dtime/3600./24.

      nstep = get_nstep()      

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)

!      write(222,*) '11p,ivt,dis1,los1= ', p, ivt(p),londeg(g),latdeg(g),dis1(p),los1(p)

!! lwp 2008-5-22
 !      phdo(p) = phd(p)
 !      npdo(p) = npd(p)
 !      dedo(p) = ded(p)
!! lwp 2008-7-30 use CLM3 soil data as input for AVIM surface(top 10cm) and root zone(top 100cm) 
!! lwp 2008-8-20  to use dzsoi for vertically weighted soil moisture and temperature

        dz4= dzsoi(1)+dzsoi(2)+dzsoi(3)+dzsoi(4)
        dz9= dz4+dzsoi(5)+dzsoi(6)+dzsoi(7)+dzsoi(8)+dzsoi(9) 
        sm1=(h2osoi_vol(c,1)/watsat(c,1) * dzsoi(1) &
            +h2osoi_vol(c,2)/watsat(c,2) * dzsoi(2) &
            +h2osoi_vol(c,3)/watsat(c,3) * dzsoi(3) &
            +h2osoi_vol(c,4)/watsat(c,4) * dzsoi(4) ) / dz4
        sm2=(h2osoi_vol(c,1)/watsat(c,1) * dzsoi(1) &
            +h2osoi_vol(c,2)/watsat(c,2) * dzsoi(2) &
            +h2osoi_vol(c,3)/watsat(c,3) * dzsoi(3) &
            +h2osoi_vol(c,4)/watsat(c,4) * dzsoi(4) &
            +h2osoi_vol(c,5)/watsat(c,5) * dzsoi(5) &
            +h2osoi_vol(c,6)/watsat(c,6) * dzsoi(6) &
            +h2osoi_vol(c,7)/watsat(c,7) * dzsoi(7) &
            +h2osoi_vol(c,8)/watsat(c,8) * dzsoi(8) & 
            +h2osoi_vol(c,9)/watsat(c,9) * dzsoi(9) ) / dz9
        
        ts1 = (t_soisno(c,1) * dzsoi(1) &
              +t_soisno(c,2) * dzsoi(2) &
              +t_soisno(c,3) * dzsoi(3) &
              +t_soisno(c,4) * dzsoi(4) ) / dz4
        ts2 = (t_soisno(c,1) * dzsoi(1) &
              +t_soisno(c,2) * dzsoi(2) &
              +t_soisno(c,3) * dzsoi(3) &
              +t_soisno(c,4) * dzsoi(4) &
              +t_soisno(c,5) * dzsoi(5) &
              +t_soisno(c,6) * dzsoi(6) &
              +t_soisno(c,7) * dzsoi(7) &
              +t_soisno(c,8) * dzsoi(8) &
              +t_soisno(c,9 )* dzsoi(9) ) / dz9
        ts3 = (t_soisno(c,9)  * dzsoi(9) &
              +t_soisno(c,10) * dzsoi(10) )&
              /(dzsoi(9)+dzsoi(10))
!       write(111,*) 'ts1,2, sm1,2= ', p, ts1-273, ts2-273,sm1,sm2 

!           rf=rs(p)/(elai(p)+esai(p))
! lwp 2008-5-29 to replace CLM3 LAI with AVIM LAI
!           rf=rs(p)/alai(p)
!! lwp 2008-8-14 replace following tl(vegetation temperature) with taf(temperature with the canopy) 
!! lwp 2008-8-14 replace following t_grnd(Ts) with Tsoil(weighted average of top 25cm soil temp)
!           qtaf=1.8**((tl(p)-t25)/10.)
!           qt1 =1.8**((t_grnd(c)-t25)/10.)
!           if(tl(p).le.273.0) qtaf=0.0
!           if(t_grnd(c).le.273.0) qt1=0.0

!! lwp 2008-9-3 to reduce respiration when T is higher than 25 C
!! lwp 2008-10-30 to reduce stem respiration by decreasing stem temperature (assume: Tstem=taf-1)
!         if (taf(p).le.t25) then
            qtaf =1.8**((taf(p)-t25)/10.)
            qstem=1.8**((taf(p)-t25-1)/10.)
!         else
!           qtaf=1.4**((taf(p)-t25)/10.)
!         endif

!         if (ts1.le.t25) then
            qt1 =1.8**((ts1   -t25)/10.)
!         else
!           qt1 =1.4**((ts1   -t25)/10.)
!         endif

!! lwp 2008-10-16 to restrain the following restrictions at low temperatures
!           if(taf(p).le.273.0) qtaf=0.0
!           if(ts1   .le.273.0) qt1=0.0
            if(taf(p).le.270.0) qtaf=0.0
            if(ts1   .le.270.0) qt1=0.0

! lwp 2008-5-28 use vegetation type-dependent crgs and crms
            rm1=crm1(ivt(p))*qtaf *bm1(p)
            rm2=crm2(ivt(p))*qstem*bm2(p)
            rm3=crm3(ivt(p))*qt1  *bm3(p)

            rmp(p)=rm1+rm2+rm3

!! lwp 2008-9-3 to reduce respiration when it exceeds photosynthesis
  !        if (rmp(p).gt.pht(p)) then
  !         rmp(p) = rmp(p) *0.5
  !        endif

            npo = pht(p) - rmp(p)
!           if(npo.le. 0.0) npo=0.0

! lwp 2008-5-28 use vegetation type-dependent crgs and crms
! lwp 2008-5-29 rewrite the following part 
          if (npo.le.0.0) then
            rg1=0.0
            rg2=0.0
            rg3=0.0
          else
            rg1=dis1(p)*crg1(ivt(p))*npo
            rg2=dis2(p)*crg2(ivt(p))*npo
            rg3=dis3(p)*crg3(ivt(p))*npo
          endif

            rgp(p)=rg1+rg2+rg3
            npp(p)=npo-rgp(p)

!           rg =    rg*dthd
! lwp 2008-5-19
            npd(p) =npd(p) + npp(p)*dthd
            phd(p) =phd(p) + pht(p)*dthd
            ts1d(p)=ts1d(p)+ ts1 *  dthd
            ts2d(p)=ts2d(p)+ ts2 *  dthd
!           ts3d(p)=ts3d(p)+ ts3 *  dthd
            sm1d(p)=sm1d(p)+ sm1 *  dthd
            sm2d(p)=sm2d(p)+ sm2 *  dthd
!           rm =    rm*dthd

!! lwp 2008-8-20 to comment out the following lines, for "ded" is useless in nowadays Soil Carbon

     end do

  end subroutine NPPRmRg

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: Dislos
!
! !INTERFACE:
   subroutine  Dislos (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                       npd, alai, &
                       dis1,dis2,dis3,dis4, &
                       los1,los2,los3,los4,glos  )
!
! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_RGAS
     use clmtype
     use time_manager, only : get_curr_date
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds

!    real(r8)                :: phd(lbp:ubp)       ! foliage photosynthesis (umol co2 /m**2/ s) [always +]
     real(r8)                :: npd(lbp:ubp)       ! foliage net primary production (umol co2 /m**2/ s) 
     real(r8), intent(in)    :: alai(lbp:ubp)      ! AVIM LAI (m^2/m^2) [always +]
!    real(r8)                :: glai(0:13)
!output 
     real(r8)                :: laim(lbp:ubp)       ! 

     real(r8), intent(  out) :: dis1(lbp:ubp)      ! 
     real(r8), intent(  out) :: dis2(lbp:ubp)      ! 
     real(r8), intent(  out) :: dis3(lbp:ubp)      ! 
     real(r8), intent(  out) :: dis4(lbp:ubp)      ! 
     real(r8), intent(  out) :: los1(lbp:ubp)      ! 
     real(r8), intent(  out) :: los2(lbp:ubp)      ! 
     real(r8), intent(  out) :: los3(lbp:ubp)      ! 
     real(r8), intent(  out) :: los4(lbp:ubp)      ! 
     real(r8), intent(  in ) :: glos (4,16)

!
! !CALLED FROM:
! subroutine CanopyFluxes in this modeul
!
! !REVISION HISTORY:
! author:            JJJ         
! standardized:      JJJ and zhangyw 2007 
! reviewed:          zhangyw 2007 
! revised:           Li Weiping, 2008-07 
! 12/18/07, PET  Made all input and output parameters explicit (removed reference to clm)
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     
     real(r8), pointer :: londeg(:)        ! latitude (degrees)     
     real(r8), pointer :: latdeg(:)        ! latitude (degrees)     
     real(r8), pointer :: elai(:)        ! one-sided leaf area index
     real(r8), pointer :: esai(:)        ! one-sided leaf area index
!EOP
!
! !OTHER LOCAL VARIABLES:
!
!    real(r8), parameter :: mpe = 1.e-6   ! prevents overflow error if division by zero
!    integer , parameter :: niter = 3     ! number of iterations
     integer  :: f,p,c,g ! indices
!    integer  :: iter    ! iteration index

    integer :: cday                   !day (1 -> 365)
    integer :: day                    !day (1 -> 31)
    integer :: mon                    !month (1 -> 12)
    integer :: yr                     !year (0 -> ...)
    integer :: sec                    !seconds into current day
!! lwp 2008-5-20 to add nstep as screen output
    integer :: nstep                  ! step time manager
    real(r8) :: dis23                 ! fractional distribution of NPP to stem and root
    real(r8) :: los1max, los1default 

!------------------------------------------------------------------------------

     ! Set statement functions

     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     
     londeg         => clm3%g%londeg 
     latdeg         => clm3%g%latdeg 
     elai           => clm3%g%l%c%p%pps%elai
     esai           => clm3%g%l%c%p%pps%esai

! lwp 2008-5-27 
!    alai           => clm3%g%l%c%p%pcf%alai
!    data glai /0.1,5.0, 6.0, 2.0, 3.0, 4.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 1.5,0.1/

     ! Set constant values

!zhangyw
!     laim(:) = 7.
!! lwp 2008-8-1 to adjust the reference LAI in each vegetation type
      
      nstep = get_nstep()
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)

          los1(p)=glos(1,ivt(p))
          los2(p)=glos(2,ivt(p))
          los3(p)=glos(3,ivt(p))
          los4(p)=glos(4,ivt(p))
!------------------------------------------------------------------------
! lwp 2008-5-28 to move the adjustment of laim before the calculation of dis1
! lwp 2008-6-4 not adjust for Vegetation 1 and 2 
        if(ivt(p).eq.1) then
          laim(p)=6.0
        endif

        if(ivt(p).eq.2) then
          laim(p)=7.0
        endif

        if(ivt(p).eq.3) then
          laim(p)=6.5
        endif

        if(ivt(p).eq.4) then
          laim(p)=7.2
        endif

        if(ivt(p).eq.5) then
          laim(p)=6.6
        endif

        if(ivt(p).eq.6.or.ivt(p).eq.7.or.ivt(p).eq.8) then
          laim(p)=6.5
        endif

        if(ivt(p).eq.9.or.ivt(p).eq.10) then
          laim(p)=6.0
        endif

        if(ivt(p).ge.11.and.ivt(p).le.12) then
          laim(p)=4.0
        endif

        if(ivt(p).le.13) then
          laim(p)=4.5
        endif

        if(ivt(p).eq.14) then
            laim(p)=3.5
        endif

#if (defined RICEWATER)
        if(ivt(p).eq.15.or.ivt(p).eq.16) then
#else
        if(ivt(p).eq.15) then
#endif
          if (latdeg(g).ge.-30.0.and.latdeg(g).le.30.0) then
            laim(p)=5.0
          else
            laim(p)=4.5
          endif
        endif
! lwp 2008-5-28 to move the adjustment of laim before the calculation of dis1

         call get_curr_date(yr, mon, day, sec)
         cday = mon*100 + day 

!------------------------------------------------------------------------
!         if(phd(p).gt.0.0) then
!! lwp 2008-7-8 use npd rather than phd as the criteria of dry material distribution
          if(npd(p).gt.0.0) then
! lwp 2008-5-27 use AVIM lai
           dis1(p)=(1-0.1)*exp(-2*(alai(p)/laim(p))**2)
!          dis1(p)=(1-0.1)*exp(-2*(elai(p)/laim(p))**2)
!          dis2(p)=0.5*(1-dis1(p))
!! lwp 2008-7-15 to increase the part of stem (dis2)
!! lwp 2008-8-29 adjust dis2,3 PFT-dependent rather than PFT-independent
           dis23 = 1-dis1(p)
!          if (ivt(p).eq.1) then
!           dis3(p)=min(0.40, dis23 )
!          else if (ivt(p).ge.2.and.ivt(p).le.5) then
!           dis3(p)=min(0.25, dis23 )
!          else if (ivt(p).eq.6.or.ivt(p).eq.7) then
!           dis3(p)=min(0.35, dis23 )
!          else if (ivt(p).eq.8) then
!           if (ivt(p).eq.8) dis3(p)=0.60*dis23 
!           endif
            dis3(p)=0.50*dis23
            dis2(p)=  dis23-dis3(p) 
          else
           dis1(p)=0.0
           dis2(p)=0.0
           dis3(p)=0.0
          end if

! lwp 2008-6-4 not adjust for Vegetation 1
!!!----the die down of veg1 ----------------------
       if(ivt(p).eq.1) then
         if (latdeg(g).ge.12) then
           if(cday.ge.201.and.cday.le.901)then
            los1(p)=0.0007
            los3(p)=0.0002
           else
            los1(p)=0.002 
            los3(p)=0.0004 
           endif
         else if (latdeg(g).le.-12) then
           if(cday.ge.801.or.cday.le.301)then
            los1(p)=0.0007
            los3(p)=0.0002
           else
            los1(p)=0.002 
            los3(p)=0.0004
           endif
         endif
       end if

!!!----the die down of veg2 ----------------------
       if(ivt(p).eq.2) then
         if(cday.ge.301.and.cday.le.1001)then
            los1(p)=0.0003
            los3(p)=0.0001
         else
            los1(p)=0.0012
            los3(p)=0.0002
         endif
       end if

!!!----the die down of veg3-------------------------------
       if(ivt(p).eq.3) then
          if(cday.ge.401.and.cday.lt.901)then
            los1(p)=0.0001
          else if (cday.ge.901.and.cday.le.1101)then
            los1(p)=0.020
          else
            los1(p)=0.001 
          endif
       end if

!!!----the die down of veg5-------------------------------
       if(ivt(p).eq.5) then
         if (latdeg(g).ge.12) then
           if(cday.ge.401.and.cday.le.901)then
            los1(p)=0.0004
            los3(p)=0.0002
           else
            los1(p)=0.0010
            los3(p)=0.0003
           endif
         else if (latdeg(g).le.-12) then
           if(cday.ge.1001.or.cday.le.301)then
            los1(p)=0.0004
            los3(p)=0.0002
           else
            los1(p)=0.0010
            los3(p)=0.0003
           endif
         endif
       end if

!!!----the die down of veg6-------------------------------
        if(ivt(p).eq.6) then
          if (latdeg(g).ge.12) then
            if(cday.ge.301.and.cday.lt.901) then
              los1(p)=0.0005
              los3(p)=0.0003
            else if(cday.ge.1001.and.cday.le.1201) then
!             los1(p)=0.015
!20091013 
              los1(p)=0.005
              los3(p)=0.0006
            else
              los1(p)=0.002
              los3(p)=0.0005
            endif
          elseif (latdeg(g).le.-12) then
            if(cday.ge.901.or.cday.le.301) then
   !          los1(p)=0.0005
              los1(p)=0.005
              los3(p)=0.0003
            else if(cday.ge.401.and.cday.le.601) then
   !          los1(p)=0.015
              los1(p)=0.020
              los3(p)=0.0006
            else
    !         los1(p)=0.002
              los1(p)=0.004
              los3(p)=0.0005
            endif
          end if
        end if

!!!----the die down of veg7 ------------------------------
       if(ivt(p).eq.7) then
          if (latdeg(g).ge.12) then
            if(cday.ge.1001.and.cday.le.1201)then
              los1(p)=0.018
            else
              los1(p)=0.003
            endif
          elseif (latdeg(g).le.-12) then
            if(cday.ge.401.and.cday.le.601)then
              los1(p)=0.018
            else
              los1(p)=0.003
            endif
          end if
       end if

!!!----the die down of veg8 (BDT boreal) -----------------
       if(ivt(p).eq.8) then
            if(cday.ge.920.and.cday.le.1101) then
              los1(p)=0.050
            else
              los1(p)=0.003
            endif
       end if

!!!----the die down of veg9 ------------------------------

      if(ivt(p).eq.9) then      !! BES temperate 
        if (latdeg(g).ge.12) then
          if(cday.ge.501.and.cday.le.901)then
           los1(p)=0.002
          else
           los1(p)=0.007
          endif
        elseif (latdeg(g).le.-12) then
          if(cday.ge.1101.or.cday.le.301)then
           los1(p)=0.002
          else
           los1(p)=0.007
          endif
        end if
      end if
!!!----the die down of veg7 ------------------------------
       if(ivt(p).eq.10) then    !! BDS temperate 
         if (latdeg(g).ge.12) then
            if(cday.ge.1001.and.cday.le.1201)then
             los1(p)=0.012
            else
             los1(p)=0.0014
            endif
         elseif (latdeg(g).le.-12) then
            if(cday.ge.401.and.cday.le.601)then
             los1(p)=0.012
            else
             los1(p)=0.0014
            endif
         endif
       endif

       if(ivt(p).eq.11) then    !! BDS boreal
            if(cday.ge.901.and.cday.le.1101)then
             los1(p)=0.02 
             los3(p)=0.0010
            else
             los1(p)=0.003
             los3(p)=0.0007
            endif
       endif

       if(ivt(p).eq.12) then    !! C3 Arctic grass 
            if(cday.ge.901.and.cday.le.1001)then
             los1(p)=0.030
             los3(p)=0.0015
            else
             los1(p)=0.003
             los3(p)=0.0009
            endif
            dis1(p)=0.50
            dis2(p)=0
            dis3(p)=0.50
            dis4(p)=0
       endif

!!!----the die down of veg13 (C3 grass), veg14 (C4 grass) ---------------------
       if(ivt(p).eq.13) then
         if (latdeg(g).ge.0.0) then
             los1default = glos(1,ivt(p))    ! = 0.005 
             los1max    = los1default + (0.08-los1default)*min(latdeg(g)/15.0, 1.0)
             los1(p)= los1default
             los3(p)=0.0020
            if(cday.ge.901.and.cday.le.1101)then
             los1(p)= los1max 
             los3(p)=0.0009
            endif
            if(cday.ge.801.and.cday.le.901)then
             los1(p)=los1default + (day/31)*(los1max-los1default) 
             los3(p)=0.002+(day/31)*(0.0009-0.002) 
            endif
            if(cday.ge.1101.and.cday.le.1201)then
             los1(p)= los1default 
             los3(p)=0.0009+(day/31)*(0.002-0.0009) 
            endif
         endif
          
         if (latdeg(g).le.0.0) then
             los1default = glos(1,ivt(p))    ! = 0.005 
             los1max    = los1default + (0.08-los1default)*min(abs(latdeg(g))/15.0, 1.0)
             los1(p)= los1default
             los3(p)=0.0020
            if(cday.ge.301.and.cday.le.501)then
             los1(p)= los1max 
             los3(p)=0.0009
            endif
            if(cday.ge.201.and.cday.le.301)then
             los1(p)=los1default + (day/31)*(los1max-los1default) 
             los3(p)=0.002+(day/31)*(0.0009-0.002) 
            endif
            if(cday.ge.501.and.cday.le.601)then
             los1(p)=los1default 
             los3(p)=0.0009+(day/31)*(0.002-0.0009) 
            endif
         endif
          
            dis1(p)=0.5
            dis2(p)=0
            dis3(p)=0.5
            dis4(p)=0
       end if
!      if(ivt(p).eq.13) then
!        if (latdeg(g).ge.15) then
!           if(cday.ge.901.and.cday.le.1101)then
!            los1(p)=0.06
!            los3(p)=0.0020
!           else
!            los1(p)=0.004 
!            los3(p)=0.0009
!           endif
!        elseif (latdeg(g).le.-15) then
!           if(cday.ge.301.and.cday.le.501) then
!            los1(p)=0.06
!            los3(p)=0.0020
!           else
!            los1(p)=0.004 
!            los3(p)=0.0009
!           endif
!        end if
!           dis1(p)=0.5
!           dis2(p)=0
!           dis3(p)=0.5
!           dis4(p)=0
!      end if

       if(ivt(p).eq.14) then
         if (latdeg(g).ge.0.0) then
             los1default = glos(1,ivt(p))    ! = 0.006 
             los1max    = los1default + (0.06-los1default)*min(latdeg(g)/12.0, 1.0)
             los1(p)= los1default
             los3(p)=0.0010
            if(cday.ge.901.and.cday.le.1101)then
             los1(p)= los1max 
             los3(p)=0.003
            endif
            if(cday.ge.801.and.cday.le.901)then
             los1(p)=los1default + (day/31)*(los1max-los1default) 
             los3(p)=0.001+(day/31)*(0.003-0.001) 
            endif
            if(cday.ge.1101.and.cday.le.1201)then
             los1(p)=los1default 
             los3(p)=0.003+(day/31)*(0.001-0.003) 
            endif
         endif
          
         if (latdeg(g).le.0.0) then
             los1default = glos(1,ivt(p))    ! = 0.006 
             los1max    = los1default + (0.06-los1default)*min(abs(latdeg(g))/12.0, 1.0)
             los1(p)= los1default
             los3(p)=0.0010
            if(cday.ge.301.and.cday.le.501)then
             los1(p)= los1max 
             los3(p)=0.003
            endif
            if(cday.ge.201.and.cday.le.301)then
             los1(p)=los1default + (day/31)*(los1max-los1default) 
             los3(p)=0.001+(day/31)*(0.003-0.001) 
            endif
            if(cday.ge.501.and.cday.le.601)then
             los1(p)=los1default 
             los3(p)=0.003+(day/31)*(0.001-0.003) 
            endif
         endif
          

      !  elseif (latdeg(g).le.-12) then
      !     if(cday.ge.301.and.cday.le.501) then
      !      los1(p)=0.04
      !      los3(p)=0.003
      !     else
      !      los1(p)=0.006
      !      los3(p)=0.0010
      !     endif
      !  end if
            dis1(p)=0.5
            dis2(p)=0
            dis3(p)=0.5
            dis4(p)=0
       end if

!!!----the die down of veg11------------------------------
       if (ivt(p).eq.15) then
         if (latdeg(g).ge.5) then
            if(cday.ge.901.and.cday.le.1101) then
             los1(p)=0.065
             los3(p)=0.0030
            else
             los1(p)=0.002
             los3(p)=0.0012
            endif
         elseif (latdeg(g).le.-5) then
            if(cday.ge.301.and.cday.le.501) then
             los1(p)=0.065
             los3(p)=0.0030
            else
             los1(p)=0.002
             los3(p)=0.0012
            endif
         end if
            dis1(p)=0.50
            dis2(p)=0
            dis3(p)=0.50
            dis4(p)=0
       end if
#if (defined RICEWATER)
       if (ivt(p).eq.16) then
         if((cday.ge.710.and.cday.le.720).or.(cday.ge.1015.and.cday.le.1025))then
           los1(p)=0.50
           los3(p)=0.50
         else if(cday.ge.401.and.cday.le.406)then
           los1(p)=0.1
           los3(p)=0.1
         else if(cday.ge.1101.or.cday.le.330)then
           los1(p)=0.02
           los3(p)=0.02
         else
           los1(p)=0.002
           los3(p)=0.0012
         endif
       end if
#endif
!!!----the die down of veg13------------------------------
!         if(ivt(p).eq.13)then
!           if(cday.ge.901)then
!            los1(p)=0.0029*12
!          los3(p)=0.0009*2.3
!         else
!          los1(p)=0.0029
!            los3(p)=0.0009
!         endif
!           dis1(p)=0.5
!           dis2(p)=0
!           dis3(p)=0.5
!           dis4(p)=0
!         end if
!!!----the die down of veg12 (Crop and natural vegetation)------------------------------
   !      if(ivt(p).eq.12) then
   !        los1(p)=0.0002
   !        if(cday.ge.901)then
   !         los1(p)=0.033
   !        end if
   !        if(cday.ge.1201) then
   !         los1(p)=0.8
!! !       los1(p)=0.03
   !        endif
   !      end if
!!!----the die down of veg15------------------------------
!         if(ivt(p).eq.15)then
!           if(cday.ge.901)then
!            los1(p)=0.0029*12
!            los3(p)=0.0009*2.5
!           else
!            los1(p)=0.0029*0.1
!            los3(p)=0.0009*0.1
!           endif
!           dis1(p)=0.5
!           dis2(p)=0
!           dis3(p)=0.5
!           dis4(p)=0
!         end if
!!!----the die down of veg16(now type 13 barren or sparsely vegetated)--------
   !      if(ivt(p).eq.13)then
   !        if(cday.ge.901.and.npd(p).gt.0)then
   !         los1(p)=0.00565*4
   !         los3(p)=0.00076
   !        else
   !         los1(p)=0.0001
   !         los3(p)=0.0001
   !        endif
   !         dis1(p)=0.5
   !         dis2(p)=0
   !         dis3(p)=0.5
   !      end if

!! lwp 2008-5-28 for southern hemisphere

     end do

  end subroutine Dislos 

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: Bmlai
!
! !INTERFACE:
   subroutine  Bmlai  (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                       npd,alai,  &
                       dis1,dis2,dis3,dis4, &
                       los1,los2,los3,los4,&
                       bm1,bm2,bm3,bm4,surl,rotl )
!
! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_RGAS
     use clmtype
     use time_manager, only : get_curr_date
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds

     real(r8), intent(in   ) :: npd(lbp:ubp)
!    real(r8), intent(inout) :: ded(lbp:ubp)
!output
! lwp 2008-5-27 
     real(r8), intent(inout) :: alai(lbp:ubp)       ! 
     real(r8)                :: laim(lbp:ubp)       ! 

     real(r8), intent(in   ) :: dis1(lbp:ubp)      ! 
     real(r8), intent(in   ) :: dis2(lbp:ubp)      ! 
     real(r8), intent(in   ) :: dis3(lbp:ubp)      ! 
     real(r8), intent(in   ) :: dis4(lbp:ubp)      ! 
     real(r8), intent(in   ) :: los1(lbp:ubp)      ! 
     real(r8), intent(in   ) :: los2(lbp:ubp)      ! 
     real(r8), intent(in   ) :: los3(lbp:ubp)      ! 
     real(r8), intent(in   ) :: los4(lbp:ubp)      ! 
     real(r8), intent(inout) ::  bm1(lbp:ubp)      ! 
     real(r8), intent(inout) ::  bm2(lbp:ubp)      ! 
     real(r8), intent(inout) ::  bm3(lbp:ubp)      ! 
     real(r8), intent(inout) ::  bm4(lbp:ubp)      ! 
!! lwp 2008-11-10 add two outputs for surface and root carbon input to soil
     real(r8), intent(inout) :: surl(lbp:ubp)      ! 
     real(r8), intent(inout) :: rotl(lbp:ubp)      ! 

!
! !CALLED FROM:
! subroutine CanopyFluxes in this modeul
!
! !REVISION HISTORY:
! author:            JJJ         
! standardized:      JJJ and zhangyw 2007 
! reviewed:          zhangyw 2007 
! revised:           Li Weiping, 2008-07 
! 12/18/07, PET  Made all input and output parameters explicit (removed reference to clm)
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     
!    real(r8), pointer :: npp(:)

     real(r8), pointer :: londeg(:)        ! longitude (degrees)     
     real(r8), pointer :: latdeg(:)        ! latitude (degrees)     
     real(r8), pointer :: elai(:)        ! one-sided leaf area index
     real(r8), pointer :: esai(:)        ! one-sided leaf area index
!    real(r8), pointer :: alai(:)        ! AVIM one-sided leaf area index
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
!    real(r8), parameter :: mpe = 1.e-6   ! prevents overflow error if division by zero
!    integer , parameter :: niter = 3     ! number of iterations
     integer  :: f,p,c,g ! indices
!    integer  :: iter    ! iteration index

    integer :: cday                   !day (1 -> 365)
    integer :: day                    !day (1 -> 31)
    integer :: mon                    !month (1 -> 12)
    integer :: yr                     !year (0 -> ...)
    integer :: sec                    !seconds into current day
!! lwp 2008-5-20 to add nstep as screen output
    integer :: nstep                  ! time step number

     real(r8) :: bmo1(lbp:ubp)      ! 
     real(r8) :: bmo2(lbp:ubp)      ! 
     real(r8) :: bmo3(lbp:ubp)      ! 
     real(r8) :: bmo4(lbp:ubp)      ! 
     real(r8) :: alaio(lbp:ubp)      ! 
!! lwp 2008-10-15 to use CLM3 16 PFTs rather than AVIM 13 biome types
!   real(r8) :: gdef(13)
    real(r8) :: gdef(16)
    real(r8) :: dtd
!------------------------------------------------------------------------------

     ! Set statement functions

     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     
     londeg         => clm3%g%londeg 
     latdeg         => clm3%g%latdeg 
     elai           => clm3%g%l%c%p%pps%elai
     esai           => clm3%g%l%c%p%pps%esai

     ! Set constant values
!! lwp 2008-7-31 to decrease gdef to increase LAI
!    data gdef/0.10,0.05,0.08,0.06,0.065,0.09,0.09,0.067,0.09,0.09,0.1,0.1,0.15 /
!! lwp 2008-10-15 to use CLM3 16 PFTs rather than AVIM 13 biome types
   data gdef/0.09, 0.09, 0.08, 0.05, 0.05, 0.055, 0.07, 0.08, 0.09, 0.09, 0.09, 0.1, 0.09, 0.09, 0.1, 0.1/
      dtd=1.0
      nstep = get_nstep()
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)

!------------------------------------------------------------------------
         call get_curr_date(yr, mon, day, sec)
         cday = mon*100 + day 
         
            bmo1(p)=bm1(p)
            bmo2(p)=bm2(p)
            bmo3(p)=bm3(p)
!           bmo4(p)=bm4(p)
           alaio(p)=alai(p)

         if(npd(p).gt.0.0) then
         bm1(p)=dtd*dis1(p)*npd(p)+bmo1(p)*(1.-dtd*los1(p))
         bm2(p)=dtd*dis2(p)*npd(p)+bmo2(p)*(1.-dtd*los2(p))
         bm3(p)=dtd*dis3(p)*npd(p)+bmo3(p)*(1.-dtd*los3(p))

         else
         bm1(p)=bmo1(p)*(1.-dtd*los1(p))
         bm2(p)=bmo2(p)*(1.-dtd*los2(p))
         bm3(p)=bmo3(p)*(1.-dtd*los3(p))

         endif

!! lwp 2008-10-17 to comment out the adjustment of Biome MAss 
           if(ivt(p).eq.1) then         !! NET temperate
            if (latdeg(g).ge.10) then
             if(cday.eq.310) then
               bm1(p)=bm1(p)+0.08
               bm3(p)=bm3(p)-0.08
             end if
            else if (latdeg(g).le.-10) then
             if(cday.eq.910) then
               bm1(p)=bm1(p)+0.08
               bm3(p)=bm3(p)-0.08
             end if
            end if
           end if

           if(ivt(p).eq.2) then
             if(cday.eq.320) then
               bm1(p)=bm1(p)+0.12
               bm3(p)=bm3(p)-0.12
             end if
           end if

           if(ivt(p).eq.3) then
             if(cday.eq.320) then
               bm1(p)=bm1(p)+0.12
               bm3(p)=bm3(p)-0.12
             end if
           end if

           if(ivt(p).eq.5) then         !! BET temperate
            if (latdeg(g).ge.5) then
             if(cday.eq.320) then
               bm1(p)=bm1(p)+0.06
               bm3(p)=bm3(p)-0.06
             end if
            else if (latdeg(g).le.-5) then
             if(cday.eq.920) then
               bm1(p)=bm1(p)+0.06
               bm3(p)=bm3(p)-0.06
             end if
            end if
           end if

!! lwp 2008-10-15 broadleaf deciduous trees(BDT, type 6,7,8 of CLM3, also type 4 of AVIM)
           if(ivt(p).eq.6) then         !! BDT tropical
            if (latdeg(g).ge.1) then
             if(cday.eq.320) then
               bm1(p)=bm1(p)+0.07
               bm3(p)=bm3(p)-0.07
             end if
            else if (latdeg(g).le.-1) then
             if(cday.eq.920) then
               bm1(p)=bm1(p)+0.07
               bm3(p)=bm3(p)-0.07
             end if
            end if
           end if

           if(ivt(p).eq.7) then         !! BDT temperate
            if (latdeg(g).ge.5.0) then
             if(cday.eq.320) then
               bm1(p)=bm1(p)+0.06
               bm3(p)=bm3(p)-0.06
             end if
            else if (latdeg(g).le.-5.0) then
             if(cday.eq.920) then
               bm1(p)=bm1(p)+0.06
               bm3(p)=bm3(p)-0.06
             end if
            end if
           end if

           if(ivt(p).eq.8) then         !! BDT boreal
             if(cday.eq.401) then
               bm1(p)=bm1(p)+0.10
               bm3(p)=bm3(p)-0.10
             end if
           end if

           if(ivt(p).eq.10) then         !! BDS temperate 
            if (latdeg(g).ge.5.0) then
             if(cday.eq.320) then
               bm1(p)=bm1(p)+0.03
               bm3(p)=bm3(p)-0.03
             end if
            else if (latdeg(g).le.-5.0) then
             if(cday.eq.920) then
               bm1(p)=bm1(p)+0.03
               bm3(p)=bm3(p)-0.03
             end if
            end if
           end if

           if(ivt(p).eq.11) then         !! BDS boreal
             if(cday.eq.420) then
               bm1(p)=bm1(p)+0.04
               bm3(p)=bm3(p)-0.04
             end if
           end if

           if(ivt(p).eq.12) then       !! C3 Arctic grass 
             if(cday.eq.420) then
               bm1(p)=bm1(p)+0.06
               bm3(p)=bm3(p)-0.06
             end if
           end if

!          if(ivt(p).eq.13.or.ivt(p).eq.14) then       !! C3 and C4 grass 
           if(ivt(p).eq.13) then       !! C3 grass 
            if (latdeg(g).ge.5.0) then
             if(cday.eq.420) then
               bm1(p)=bm1(p)+0.08
               bm3(p)=bm3(p)-0.08
             end if
            else if (latdeg(g).le.-5.0) then
             if(cday.eq.1020) then
               bm1(p)=bm1(p)+0.08
               bm3(p)=bm3(p)-0.08
             end if
            end if
           end if

           if(ivt(p).eq.15) then      !! Crop1 and Crop2 
            if (latdeg(g).ge.5.0) then
             if(cday.eq.420) then
               bm1(p)=bm1(p)+0.04
               bm3(p)=bm3(p)-0.04
             end if
            else if (latdeg(g).le.-5.0) then
             if(cday.eq.1020) then
               bm1(p)=bm1(p)+0.04
               bm3(p)=bm3(p)-0.04
             end if
            end if
           end if
#if (defined RICEWATER)
           if(ivt(p).eq.16) then      !! for rice
             if(cday.eq.408.or.cday.eq.725)then
               bm1(p)=bm1(p)+0.04
               bm3(p)=bm3(p)-0.04
             end if
             if(cday.eq.1101)then
               bm1(p)=bm1(p)+0.012
               bm3(p)=bm3(p)-0.01
             end if
           end if
#endif

!! lwp 2008-10-17 to comment out the adjustment of Biome MAss 

!! lwp 2008-10-15 comment out th efollowing lines, for CLM3 doesn't has "mixed forest"
!         if(ivt(p).eq.5) then          !! mixed forest for AVIM
!           if (latdeg(g).ge.10.0) then
!! lwp 2008-5-19 401 hereby means April 01 ( 4x100+1=401 )
!!           if(cday.eq.90) then
!           if(cday.eq.401) then
!              bm1(p)=bm1(p)+0.12
!              bm3(p)=bm3(p)-0.12
!           end if

!---------------------------------------------------------

 !        bm4(p)=dtd*(los1(p)*bmo1(p)+los2(p)*bmo2(p)+los3(p)*bmo3(p))-ded(p)*dtd
!! lwp 2008-10-28 move the adjustment of Biomass from the beginning of this subroutine here
!! lwp 2008-10-24 to set the bottom line for Biomass, bm2 for grasses and crops are 0.0 for AVIM
          bm1(p) = max(bm1(p), 0.0)
          bm2(p) = max(bm2(p), 0.0)
          bm3(p) = max(bm3(p), 0.05)

!! lwp 2008-5-27 to calculate lai according to AVIM
          alai(p)=alaio(p)+(bm1(p)-bmo1(p))/gdef(ivt(p))

           alai(p) = min(alai(p), 12.0)
!! lwp 2008-11-6 to seperate Crop whose LAI can be zero afer reap from other natural vegetation
          if (ivt(p).lt.15) then
           alai(p) = max(alai(p), 0.2)
          else
!! lwp 2008-11-6 make sure Vegetation LAI >= 0.05, so that these land points are calculated in CLM3 
           alai(p) = max(alai(p), 0.06)  
          endif

          elai(p) = alai(p)

!! lwp 2008-7-31 to increase lai to enhance GPP
!! lwp 2008-7-7 to feedback AVIM LAI to CLM3
!          if (ivt(p).le.5) then
!            elai(p) = alai(p) * 0.75
!          else if (ivt(p).gt.5.and.ivt(p).lt.8) then
!            elai(p) = alai(p) * 0.85
!          else if (ivt(p).eq.10) then
!            elai(p) = alai(p) * 0.95
!          else
!            elai(p) = alai(p) * 0.90
!          endif
            
! lwp 2008-7-7 CLM3 sai are calculated as the remaining part of AVIM lai, which includes lai and sai
!! lwp 2008-7-31 to comment out the following line, esai in CLM3 is constant as original 0.1*(elai+esai)

!!           esai(p) = alai(p) - elai(p)

! lwp 2008-7-7  climatic LAI must be turned off in "STATICEcosysDynMod.F90" under directory Biogeochem
!! lwp 2008-11-10 calculate surface and root los for carbon input to soil (unit in g C/m**2)
!! 2009-10-9 reduce nep
!      surl(p) = (bm1(p)*los1(p)+bm2(p)*los2(p))*1000.*0.48
!      rotl(p) = bm3(p)*los3(p)*1000.*0.48

       surl(p) = (bm1(p)*los1(p)+bm2(p)*los2(p))*1000.*0.474   ! 0.474
       rotl(p) = bm3(p)*los3(p)*1000.*0.474                    ! 0.474

!-----------------

!! lwp 2008-11-3 to reduce Carbon input to soil from los of Crops which are mostly reaped by human 
#if (defined RICEWATER)
       if (ivt(p).eq.15.or.ivt(p).eq.16) then    !! Crops
#else
       if (ivt(p).eq.15) then    !! Crops
#endif
!! 2009-10-9 reduce nep
!        surl(p) = 0.5 * surl(p)                 !! assume 50% stems+leaves reaped, 50% left in the soil
!        rotl(p) = 0.8 * rotl(p)                 !! assume 20% roots were reaped, 80% left in the soil

#if (defined OFFLINE)
         surl(p) = 0.474 * surl(p)   ! 0.475              !! assume 50% stems+leaves reaped, 50% left in the soil
         rotl(p) = 0.774 * rotl(p)   ! 0.775              !! assume 20% roots were reaped, 80% left in the soil
#endif

#if (defined COUP_CSM)
     if( lsmlon == 128 .or. lsmlon == 800 ) then          ! T42 or T266

         surl(p) = 0.45 * surl(p)   ! 0.475              !! assume 50% stems+leaves reaped, 50% left in the soil
         rotl(p) = 0.65 * rotl(p)   ! 0.775              !! assume 20% roots were reaped, 80% left in the soil

     elseif ( lsmlon == 320 ) then                        ! T106
! wtw
         surl(p) = 0.45 * surl(p)   ! 0.40              !! assume 50% stems+leaves reaped, 50% left in the soil
         rotl(p) = 0.65 * rotl(p)   ! 0.60              !! assume 20% roots were reaped, 80% left in the soil

     else
         surl(p) = 0.474 * surl(p)   ! 0.475              !! assume 50% stems+leaves reaped, 50% left in the soil
         rotl(p) = 0.774 * rotl(p)   ! 0.775              !! assume 20% roots were reaped, 80% left in the soil
     endif
#endif

#if (defined COUP_CAM)
         surl(p) = 0.475 * surl(p)   ! 0.475              !! assume 50% stems+leaves reaped, 50% left in the soil
         rotl(p) = 0.775 * rotl(p)   ! 0.775              !! assume 20% roots were reaped, 80% left in the soil
#endif

       endif

     end do

  end subroutine Bmlai

!--------------------------------------------------------------------72
!BOP
!
! !IROUTINE: SoilCarbon 
!
! !INTERFACE:

! ===================================================================
!      SCNM is a program to simulate soil carbon & nitrogen dynamics
!      which is coupled to DOLY vegetation model.
! ====================================================================       
    subroutine  SoilCarbon(fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                           surl, rotl, ts1d,ts2d,sm1d,sm2d, &
                           polc1, polc2, polc3, polc4, &
                           polc5, polc6, polc7, polc8, &
                           dsmc, dco2, nmin, dth, &
                           npd, nep) 
!dth is the time step: if time step is half hour than dth=1/48

!
! Interface routine for AVIM.
!  1995  JJJ
! 29 Aug 2007 zhangyw Revision
! 26 July 2008 LiWeiping Revised
!-------------------------------Commons---------------------------------
  ! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_RGAS
     use clmtype
     use clm_varsur, only : sand3d, clay3d
     use time_manager, only : get_curr_date, get_nstep
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds

     real(r8), intent(in   ) :: ts1d(lbp:ubp)      !
     real(r8), intent(in   ) :: ts2d(lbp:ubp)      !
!    real(r8), intent(in   ) :: ts3d(lbp:ubp)      !
     real(r8), intent(in   ) :: sm1d(lbp:ubp)
     real(r8), intent(in   ) :: sm2d(lbp:ubp)

!    real(r8), intent(in   ) ::  bm1(lbp:ubp)      !
!    real(r8), intent(in   ) ::  bm2(lbp:ubp)      !
!    real(r8), intent(in   ) ::  bm3(lbp:ubp)      !
     real(r8), intent(in   ) :: surl(lbp:ubp)      !
     real(r8), intent(in   ) :: rotl(lbp:ubp)      !
     real(r8), intent(inout) :: polc1(lbp:ubp)     !
     real(r8), intent(inout) :: polc2(lbp:ubp)     !
     real(r8), intent(inout) :: polc3(lbp:ubp)     !
     real(r8), intent(inout) :: polc4(lbp:ubp)     !
     real(r8), intent(inout) :: polc5(lbp:ubp)     !
     real(r8), intent(inout) :: polc6(lbp:ubp)     !
     real(r8), intent(inout) :: polc7(lbp:ubp)     !
     real(r8), intent(inout) :: polc8(lbp:ubp)     !
     real(r8), intent(inout) :: dsmc(lbp:ubp)     !
     real(r8), intent(inout) :: dco2(lbp:ubp)     !
     real(r8), intent(inout) :: nmin(lbp:ubp)     !
     real(r8), intent(in   ) :: npd(lbp:ubp)      !
     real(r8), intent(out  ) :: nep(lbp:ubp)      !

!----------------------------------------------------------------------

      real(r8),intent(in)  :: dth
!     real(r8),intent(inout)  :: POLC(1:8) 
!     real(r8),intent(in)  :: dth,ts1,ts2,ts3

! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)       ! pft's column index
     integer , pointer :: clandunit(:)     ! landunit index of corresponding column
     integer , pointer :: pgridcell(:)     ! pft's gridcell index
     integer , pointer :: ivt(:)           ! pft vegetation type
     integer , pointer :: ixy(:)           ! xy lon index (column-level)
     integer , pointer :: jxy(:)           ! xy lat index (column_level)

     real(r8), pointer :: londeg(:)        ! latitude (degrees)
     real(r8), pointer :: latdeg(:)        ! latitude (degrees)

!    real(r8), pointer :: h2osoi_vol(:,:)  ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
!    real(r8), pointer :: watsat(:,:)      ! volumetric soil water at saturation (porosity) (nlevsoi)
!    real(r8), pointer :: t_soisno(:,:)    ! soil temperature (Kelvin)  (-nlevsno+1:nlevsoi)
   real(r8), pointer :: carbon(:)    !zhangyw 20081121

!local  
      integer  :: f,p,c,g, i, j, l            ! indices
      integer  :: Pc
      real(r8) :: LIGN,NITG
      real(r8) :: TX,WTL,LC
      real(r8) :: LNSC
      real(r8) :: CAL,CAP,CAS,CSP,CSA,FM
      real(r8) :: IFSC,IFMC,IRSC,IRMC
      real(r8) :: POC,PON,TNF1,TNF2,TNF,CIN3,CIN7
!!lwp 2008-7-26 change "F" to "XF" to avoid confusion when use CLM3 structure(f,p,c,g)
      real(r8) :: K(1:8),XF(1:8),CN(1:8),CF(1:8)
      real(r8) :: NF(1:8),ND(1:8),NLIM(1:8)
      real(r8) :: NLMT,NLM
      real(r8) :: TEM
      real(r8) :: FTEM,FMOI
      real(r8) :: surABF,rootABF
!     real(r8) :: LTF,ROTF,ABF
      real(r8) :: ts1, ts2, ts3
      real(r8) :: w1, w2, wm12
      real(r8) :: sand,silt,clay
      integer  :: nstep

!----------------------------------------------------------------------
     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     ixy       => clm3%g%l%c%ixy
     jxy       => clm3%g%l%c%jxy
     clandunit       => clm3%g%l%c%landunit
       carbon         => clm3%g%l%c%p%pef%carbon    !!zhangyw 20081121

      K(1)=0.12
      K(2)=0.14
      K(3)=0.18
      K(4)=0.16
      K(5)=0.25
      K(6)=0.28
      K(7)=0.018
!! lwp 2008-12-1 to accelerate soil decomposition of pool 8 by doubling its decomposition rate
!     K(8)=0.0004      !! original is 0.0004
      K(8)=0.0004*2

      K = 1.50 * K/30. * dth      !! original is 1.5*K

      XF(1)=0.6
      XF(2)=0.55
      XF(3)=0.55
      XF(4)=0.6
      XF(5)=0.6
      XF(6)=0.55
      XF(7)=0.55
      XF(8)=0.55

!! lwp 2008-7-28 to move the following three lines from the end of this subroutine here
      DSMC = 0.0
      DCO2 = 0.0
      NMIN = 0.0

      nstep = get_nstep()

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
  !     g = pgridcell(p)
        l = clandunit(c)
        i = ixy(c)
        j = jxy(c)

!! lwp 2008-7-26 use soil parameters of the 6th layer(36cm deep from surface??)
!! lwp 2008-7-29 the unit of sand3d is percent, therefore should be divided by 100. 
            clay = clay3d(i,j,5)/100.
            sand = sand3d(i,j,5)/100.
            silt = 1.0 - sand - clay

!! lwp 2008-8-20 use daily mean soil temperature and moisture rather than instant values 
!! lwp 2008-7-25 to add soil moisture here

!       wm12=(w1+w2)/2.
        wm12 = sm1d(p)
        if(wm12.le.0.1) FMOI=0.2
!! lwp 2008-10-16 the following lines is original
        if(wm12.gt.0.1.and.wm12.le.0.4) FMOI=0.2+2.*(wm12-0.1)
        if(wm12.gt.0.4.and.wm12.le.0.8) FMOI=0.8+0.5*(wm12-0.4)
        if(wm12.ge.0.8) FMOI=1.0
!! lwp 2008-10-16 adjust to reduce respiration in moist conditions 
!       if(wm12.gt.0.1.and.wm12.le.0.5) FMOI=0.2+1.5*(wm12-0.1)
!       if(wm12.gt.0.5.and.wm12.le.0.9) FMOI=0.8+0.4*(wm12-0.4)
!       if(wm12.ge.0.9) FMOI=1.0

!! lwp 2008-7-28 to revised the original F77 arrays to F90, and move it outside of spacial do loop
!     do  pc = 1, 8
!       K(pc) = 1.50*K(pc)/30.*dth
!     enddo 
!     K = 1.50 * K/30. * dth

      LNSC = 0.3
      LC = EXP (-3.0 * LNSC)
      WTL = 0.2
!--------add by hm------
!	if (vgty.eq.1) then
!	 do p=1,8
!	 k(p)=8.5*k(p)/30.*dth
!   	end do
!       end if	
!       if(vgty.eq.2) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.78
!        end do
!        end if
!         if(vgty.eq.3) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.3
!        end do
!        end if
!        if(vgty.eq.4.) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.8
!        end do
!        end if
!       if(vgty.eq.5.) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.2
!        end do
!         end if
!      if(vgty.eq.6.) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.2
!        end do
!        end if
!       if(vgty.eq.7.) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.05
!        lc=lc*0.05
!        end do
!        end if
!        if(vgty.eq.10.) then
!        do p=1,8
!        k(p)=1.5*k(p)/30.*dth
!        f(p)=f(p)*0.5
!        lc=lc*0.5
!        end do
!        end if
!--------end of add----------
 
! lwp 2008-7-24 vegetation 12 is now type 11 (crops), no type 13 and 15 in this version
!       if (ivt(p).eq.7.or.ivt(p).eq.9.or.ivt(p).eq.10.or.ivt(p).eq.11.or.ivt(p).eq.13) then
!! lwp 2008-11-3 for all grasses and crops that LIGN is small but LIGN/NITG is high
        if (ivt(p).ge.12) then
         LIGN = 0.12
         NITG = 0.012
        else 
         LIGN = 0.2
         NITG = 0.008
        endif

      CN(1) = 150.0
      CN(2) = 150.0
      CN(3) = 8.0
      CN(4) = 8.0
      CN(5) = 0.18/NITG
      CN(6) = 0.18/NITG
      CN(7) = 20.0-400.0*NITG
      CN(8) = 16.0-600.0*NITG

      FM = 0.85-0.018*LIGN/NITG  
      TX = 1.0 - 0.75 * (SILT + CLAY)
      CAL = (WTL/18.0) * (0.01 + .04 * SAND)
      CAP = 0.003 + 0.032 * CLAY
      XF(3) = 0.85 - 0.68 * (SILT + CLAY)
      CAS = 1.0 - CAL - CAP - XF(3)
      CSP = 0.003 - 0.009 * CLAY
      if (CSP .le. 0.0)  CSP = 0.0
      CSA = (1.0-CSP-0.55)
      if (CSA .le. 0.0)  CSA = 0.0 

!     The effect of temperature and moisture on decomposition
!         ROTL = LTF * ROTF
!         SURL = LTF *(1.0-ROTF)
!! lwp 2008-8-20 replace the following instant soil temperature (ts1) with daily mean(ts1d)
!       TEM=ts1-273.16
        TEM=ts1d(p)-273.16

!! lwp 2008-11-11 to increase Soil respiration at low temperatures <=-5 ( original is zero)
!     if  (TEM.le.(-5.0)) FTEM = 0.0
      if  (TEM.le.(-10.0)) FTEM = 0.0
      if  (TEM.gt.(-10.0).and.TEM.le.(-5.0)) FTEM = 0.02
      if ((TEM.gt.(-5.0)).and.(TEM.le.0.0))       &
!! lwp 2008-9-5 to increase soil respiration at low T but reduce soil respiration at high T
!! lwp 2008-10-16 to go back to the original code of T adjustment 
!         FTEM = 0.04
          FTEM = 0.08
      if ((TEM .gt.0.0) .and. (TEM .le.  5.0))    & 
!! lwp 2008-7-24 the following lines add one zero (0.06 to 0.006) according to Prof Ji.
!         FTEM=0.04+0.006*TEM
          FTEM=0.08+0.008*TEM
      if ((TEM .gt.5.0) .and. (TEM .le. 10.0))    & 
!         FTEM=0.07+0.016*(TEM-5.0) 
          FTEM=0.12+0.016*(TEM-5.0)   
      if ((TEM .gt.10.0) .and. (TEM .le. 35.0))   & 
!! lwp 2008-7-29 to make FTEM "smoothly" connected from 0.15 to 0.95
!         FTEM=0.15+0.032*(TEM-10.0)
          FTEM=0.20+0.030*(TEM-10.0)
      if ((TEM.gt.35.0) .and. (TEM.le.40.0))      & 
          FTEM=0.95
!         FTEM=0.95+0.008*(TEM-35.0)
      if ((TEM .gt. 40.0) .and. (TEM .le. 47.0))  & 
          FTEM=0.95-0.135*(TEM-40.0)
      if  (TEM.gt.47.0) FTEM = 0.0

      surABF=FTEM*FMOI*0.9

!! lwp 2008-8-25 use daily top 1.7m soil moisture for root zone rather than the same wm12 as top 10cm
        wm12 = sm2d(p)
        if(wm12.le.0.1) FMOI=0.2
        if(wm12.gt.0.1.and.wm12.le.0.4) FMOI=0.2+2.*(wm12-0.1)
        if(wm12.gt.0.4.and.wm12.le.0.8) FMOI=0.8+0.5*(wm12-0.4)
        if(wm12.ge.0.8) FMOI=1.0
!! lwp 2008-10-16 adjust to reduce respiration in moist conditions 
!       if(wm12.gt.0.1.and.wm12.le.0.5) FMOI=0.2+1.5*(wm12-0.1)
!       if(wm12.gt.0.5.and.wm12.le.0.9) FMOI=0.8+0.4*(wm12-0.4)
!       if(wm12.ge.0.9) FMOI=1.0

!! lwp 2008-8-20 replace the following instant soil temperature (ts2) with daily mean(ts2d)
!       TEM=(ts2-273.16+ts3-273.16)/2.
        TEM=ts2d(p)-273.16

!! lwp 2008-11-11 to increase Soil respiration at low temperatures <=-5 ( original is zero)
!         if  (TEM.le.(-5.0)) FTEM = 0.0
      if  (TEM.le.(-10.0)) FTEM = 0.0
      if  (TEM.gt.(-10.0).and.TEM.le.(-5.0)) FTEM = 0.02
      if ((TEM.gt.(-5.0)).and.(TEM.le.0.0))       &
!! lwp 2008-10-16 to go back to the original code of T adjustment 
 !        FTEM = 0.04
          FTEM = 0.08
      if ((TEM .gt.0.0) .and. (TEM .le.  5.0))    & 
!! lwp 2008-7-24 the following lines add one zero (0.06 to 0.006) according to Prof Ji.
 !        FTEM=0.04+0.006*TEM
          FTEM=0.08+0.008*TEM
      if ((TEM .gt.5.0) .and. (TEM .le. 10.0))    & 
 !        FTEM=0.07+0.016*(TEM-5.0) 
          FTEM=0.12+0.016*(TEM-5.0)   
      if ((TEM .gt.10.0) .and. (TEM .le. 35.0))   & 
!! lwp 2008-7-29 to make FTEM "smoothly" connected from 0.15 to 0.95
 !        FTEM=0.15+0.032*(TEM-10.0)
          FTEM=0.20+0.030*(TEM-10.0)
      if ((TEM.gt.35.0) .and. (TEM.le.40.0))      & 
          FTEM=0.95
!!        FTEM=0.91+0.008*(TEM-35.0)
      if ((TEM .gt. 40.0) .and. (TEM .le. 47.0))  & 
          FTEM=0.95-0.135*(TEM-40.0)
      if  (TEM.gt.47.0) FTEM = 0.0

      rootABF=FTEM*FMOI*0.9

100   continue

!     Limitations of available nitrogen
!     on carbon mineralization
		
        POC = 0.0
        PON = 0.0

!! lwp 2008-7-26 revise this do loop using polc1~polc8 rather than polc(1:8)
  !     do  Pc = 1, 8    ! 1001
!       NLIM(Pc) = 1.0  ! LWP this line no use here

  !     if (pc.eq.1.or.pc.eq.4.or.pc.eq.5) then
  !       CF(Pc) = POLC(Pc) * K(Pc) * surABF
! !       CF(1)  = polc1(p) * K(1) * surABF
  !     else 
  !       CF(Pc) = POLC(Pc) * K(Pc) * rootABF
  !     end if

!        CF(Pc) = POLC(Pc) * K(Pc) * ABF

  !     if ((Pc .eq. 1) .or. (Pc .eq. 2)) CF(Pc)=CF(Pc)*LC
  !     if (Pc .eq. 3) CF(Pc) = CF(Pc) * TX

  !     NF(Pc) = CF(Pc)/CN(Pc)

  !     if ((Pc .eq. 1) .or. (Pc .eq. 2)) then
  !        POC=POC+2.0*CF(Pc)
  !        PON=PON+2.0*NF(Pc) 
  !     else
  !        POC=POC+CF(Pc)
  !        PON=PON+NF(Pc)
  !     endif
  !     enddo    !  1001
!! for surface pool
      CF(1) = polc1(p) * K(1) * surABF * LC
      CF(4) = polc4(p) * K(4) * surABF
      CF(5) = polc5(p) * K(5) * surABF
!! for root pool
      CF(2) = polc2(p) * K(2) * rootABF * LC
      CF(3) = polc3(p) * K(3) * rootABF * TX
      CF(6) = polc6(p) * K(6) * rootABF
      CF(7) = polc7(p) * K(7) * rootABF
      CF(8) = polc8(p) * K(8) * rootABF

!     write(111,*) 'CF= ', p, CF
!! lwp 2008-7-28 to substitute the following line in FORTRAN 90 for the above do loop 
        NF = CF/CN

        POC = 2.0*(CF(1)+CF(2)) + CF(3)+CF(4)+CF(5)+CF(6)+CF(7)+CF(8)
        PON = 2.0*(NF(1)+NF(2)) + NF(3)+NF(4)+NF(5)+NF(6)+NF(7)+NF(8)

        ND(1) = CF(1)*(0.4/CN(4)+0.7/CN(7))
        ND(2) = CF(2)*(0.45/CN(3)+0.7/CN(7))
        ND(3) = CF(3)*(CAS/CN(7)+CAP/CN(8)+CAL/CN(3))
        ND(4) = CF(4)*0.4/CN(7)
        ND(5) = CF(5)*0.4/CN(4)
        ND(6) = CF(6)*0.45/CN(3)
        ND(7) = CF(7)*(CSA/CN(3)+CSP/CN(8))
        ND(8) = CF(8)*0.45/CN(3)

        TNF1 = 2.0*NF(1) + NF(5) + NF(4)
        TNF2 = PON -TNF1

        TNF = TNF1 * CN(4) + TNF2 * CN(3)

        if (TNF .le. POC) then     !! lwp123
        NLM = 0.0

!       do  Pc = 1, 8  ! 1002
!        if (abs(POC) .lt. 1.0E-20) then
!          NLIM(Pc)=1.0 
!        else
!          NLIM(Pc)=TNF/POC
!        endif
!       enddo   !  1002
   	
!! lwp 2008-7-28 to revised the original F77 arrays to F90, delete do loop
         if (abs(POC) .lt. 1.0E-20) then
           NLIM = 1.0 
         else
           NLIM = TNF/POC
         endif
        
!       ND(Pc) = ND(Pc) * NLIM(Pc)
!       NF(Pc) = NF(Pc) * NLIM(Pc)
!       CF(Pc) = CF(Pc) * NLIM(Pc)
!       NLM = NLM + NLIM(Pc)/8.0
        ND = ND * NLIM
        NF = NF * NLIM
        CF = CF * NLIM
        NLM =(NLIM(1)+NLIM(2)+NLIM(3)+NLIM(4)   &
             +NLIM(5)+NLIM(6)+NLIM(7)+NLIM(8) )/8.0

        else               !! lwp123 TNF > POC
!! lwp 2008-7-28 the following line for the definition of NLMT is useless, for NLMT is never used thereafter.
!       NLMT = 1.0
        NLM  = 1.0
        endif
  
!     Carbon and nitrogen dynamics

      IFSC = SURL(p) * (1.0 - FM)
      IFMC = SURL(p) * FM
      IRSC = ROTL(p) * (1.0 - FM)
      IRMC = ROTL(p) * FM

      CIN3 = CF(2)*(1.0- XF(2)) + CF(6)*(1.0 - XF(6)) +   &
             CF(7)*CSA + CF(8)*(1.0 - XF(8))
      CIN7 = CF(1)*0.7 + CF(4)*(1.0 - XF(4)) +           &
             CF(2)*0.7 + CF(3)*CAS

!! lwp 2008-7-26 revise this do loop using polc1~polc8 rather than polc(1:8)
!     write(111,*) 'cin3,cf3,cin7,cf7= ', cin3, cf(3), cin7,cf(7)
      POLC1(p) = POLC1(p) + IFSC - 2.0 * CF(1)
      POLC2(p) = POLC2(p) + IRSC - 2.0 * CF(2)
      POLC3(p) = POLC3(p) + (CIN3 - CF(3))
      POLC4(p) = POLC4(p) + CF(1)*(1.0-XF(1)) + CF(5)*(1.0-XF(5)) - CF(4)
      POLC5(p) = POLC5(p) + IFMC - CF(5)
      POLC6(p) = POLC6(p) + IRMC - CF(6)
      POLC7(p) = POLC7(p) + (CIN7 - CF(7))
      POLC8(p) = POLC8(p) + (CF(7)*CSP + CF(3)*CAP - CF(8))
   	   
!     DSMC(p)=0.0
!     DCO2(p)=0.0
!     NMIN(p)=0.0

!     do  Pc = 1, 8 ! 1003
!        NMIN(p) = NMIN(p) + NF(Pc)-ND(Pc)
!        DCO2(p) = DCO2(p) + CF(Pc)*XF(Pc)
!!       DSMC(p) = DSMC(p) + POLC(Pc)
!     enddo    ! 1003

!     DCO2(p) = DCO2(p) + (CF(1) + CF(2))*0.3

!! lwp 2008-7-28 to revised the original F77 arrays to F90, delete do loop
      NMIN(p) = NF(1) + NF(2) + NF(3) + NF(4) + NF(5) + NF(6) + NF(7) + NF(8)  & 
              -(ND(1) + ND(2) + ND(3) + ND(4) + ND(5) + ND(6) + ND(7) + ND(8) )
      DCO2(p) = CF(1)*XF(1) + CF(2)*XF(2) + CF(3)*XF(3) + CF(4)*XF(4)  &
               +CF(5)*XF(5) + CF(6)*XF(6) + CF(7)*XF(7) + CF(8)*XF(8)  &
               +(CF(1)+CF(2))*0.3 
      DSMC(p) = POLC1(p) + POLC2(p) + POLC3(p) + POLC4(p)  &
              + POLC5(p) + POLC6(p) + POLC7(p) + POLC8(p)
      
!! lwp 2008-7-26 unit change, from Kg (DM) to g (C) --- (1g DM ~ 0.45 g C)
!! lwp 2008-8-14 unit change, from Kg (DM) to g (C) --- (1g DM ~ 0.48 g C)

! org      nep(p) = npd(p)*1000.*0.48 - DCO2(p)
!--------
! wtw

#ifdef CO2
      nep(p) = npd(p)*1000.*0.46 - DCO2(p)
#elif (defined HAMOCC) 
      nep(p) = npd(p)*1000.*0.48 - DCO2(p)    ! T42ESM, HAMOCC
#else
      nep(p) = npd(p)*1000.*0.465- DCO2(p)
#endif 

!----------------------
!! zhangyw 20081121 add carbon for BCC_CSM (nep(  C g/m**2/day ---> C g/m**2/s ,+ up to atm ,- to lnd)
       carbon(p)=-1.0*nep(p)/86400.

    enddo
  end subroutine SoilCarbon 

#endif

end module CanopyFluxes1Mod
#endif
