#include <misc.h>
#include <preproc.h>

module FrictionVelocityMod

!------------------------------------------------------------------------------
!BOP
!
! !MODULE: FrictionVelocityMod
!
! !DESCRIPTION:
! Calculation of the friction velocity, relation for potential
! temperature and humidity profiles of surface boundary layer.
!
! !USES:
  use shr_kind_mod, only: r8 => shr_kind_r8
!
! !PUBLIC TYPES:
  implicit none
  save
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: FrictionVelocity       ! Calculate friction velocity
  public :: MoninObukIni           ! Initialization of the Monin-Obukhov length
!
! !PRIVATE MEMBER FUNCTIONS:
  private :: StabilityFunc1        ! Stability function for rib < 0.
  private :: StabilityFunc2        ! Stability function for rib < 0.
!
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!------------------------------------------------------------------------------

contains

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: FrictionVelocity
!
! !INTERFACE:
  subroutine FrictionVelocity(lbp, ubp, fn, filterp, &
                              displa, z0m, z0h, z0q, &
                              obu, iter, ur, um, ustar, &
                              temp1, temp2, temp12m, temp22m, fm)
!
! !DESCRIPTION:
! Calculation of the friction velocity, relation for potential
! temperature and humidity profiles of surface boundary layer.
! The scheme is based on the work of Zeng et al. (1998):
! Intercomparison of bulk aerodynamic algorithms for the computation
! of sea surface fluxes using TOGA CORE and TAO data. J. Climate,
! Vol. 11, 2628-2644.
!
! !USES:
   use clmtype
   use clm_varcon, only : vkc
!
! !ARGUMENTS:
   implicit none
   integer , intent(in)  :: lbp, ubp         ! pft array bounds
   integer , intent(in)  :: fn               ! number of filtered pft elements
   integer , intent(in)  :: filterp(fn)      ! pft filter
   real(r8), intent(in)  :: displa(lbp:ubp)  ! displacement height (m)
   real(r8), intent(in)  :: z0m(lbp:ubp)     ! roughness length over vegetation, momentum [m]
   real(r8), intent(in)  :: z0h(lbp:ubp)     ! roughness length over vegetation, sensible heat [m]
   real(r8), intent(in)  :: z0q(lbp:ubp)     ! roughness length over vegetation, latent heat [m]
   real(r8), intent(in)  :: obu(lbp:ubp)     ! monin-obukhov length (m)
   integer,  intent(in)  :: iter             ! iteration number
   real(r8), intent(in)  :: ur(lbp:ubp)      ! wind speed at reference height [m/s]
   real(r8), intent(in)  :: um(lbp:ubp)      ! wind speed including the stablity effect [m/s]
   real(r8), intent(out) :: ustar(lbp:ubp)   ! friction velocity [m/s]
   real(r8), intent(out) :: temp1(lbp:ubp)   ! relation for potential temperature profile
   real(r8), intent(out) :: temp12m(lbp:ubp) ! relation for potential temperature profile applied at 2-m
   real(r8), intent(out) :: temp2(lbp:ubp)   ! relation for specific humidity profile
   real(r8), intent(out) :: temp22m(lbp:ubp) ! relation for specific humidity profile applied at 2-m
   real(r8), intent(inout) :: fm(lbp:ubp)    ! needed for DGVM only to diagnose 10m wind
!
! !CALLED FROM:
!
! !REVISION HISTORY:
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! 12/19/01, Peter Thornton
! Added arguments to eliminate passing clm derived type into this function.
! Created by Mariana Vertenstein
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in arguments
!
   integer , pointer :: pgridcell(:)   ! pft's gridcell index
   real(r8), pointer :: forc_hgt(:)    ! atmospheric reference height (m)
   real(r8), pointer :: forc_hgt_u(:)  ! observational height of wind [m]
   real(r8), pointer :: forc_hgt_t(:)  ! observational height of temperature [m]
   real(r8), pointer :: forc_hgt_q(:)  ! observational height of humidity [m]
!
! local pointers to implicit out arguments
!
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   real(r8), pointer :: rds(:)         ! rds=1/vds  vds: dry deposition velocity term (m/s) (for SO4 NH4NO3)
#endif
   real(r8), pointer :: u10(:)         ! 10-m wind (m/s) (for dust model)
   real(r8), pointer :: fv(:)          ! friction velocity (m/s) (for dust model)
!!!------------------zhangyw 10m wind 20090814-------------------------
   real(r8), pointer :: u_10m(:)       ! 10m height u-wind (m/s)
   real(r8), pointer :: v_10m(:)       ! 10m height v-wind (m/s)
   real(r8), pointer :: forc_u(:)         ! atmospheric wind speed in east direction (m/s)
   real(r8), pointer :: forc_v(:)         ! atmospheric wind speed in north direction (m/s)
!!!------------------zhangyw 10m wind 20090814-------------------------
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
   real(r8), parameter :: zetam = 1.574 ! transition point of flux-gradient relation (wind profile)
   real(r8), parameter :: zetat = 0.465 ! transition point of flux-gradient relation (temp. profile)
   integer :: f                         ! pft-filter index
   integer :: p                         ! pft index
   integer :: g                         ! gridcell index
   real(r8):: zldis(lbp:ubp)            ! reference height "minus" zero displacement heght [m]
   real(r8):: zeta(lbp:ubp)             ! dimensionless height used in Monin-Obukhov theory
!!!------------------zhangyw 10m wind 20090814-------------------------
!#if (defined DGVM) 
   real(r8) :: tmp1,tmp2,tmp3,tmp4      ! Used to diagnose the 10 meter wind
   real(r8) :: fmnew                    ! Used to diagnose the 10 meter wind
   real(r8) :: fm10                     ! Used to diagnose the 10 meter wind
   real(r8) :: zeta10                   ! Used to diagnose the 10 meter wind
!#endif
!!!------------------zhangyw 10m wind 20090814-------------------------
!------------------------------------------------------------------------------

   ! Assign local pointers to derived type members (gridcell-level)

   pgridcell  => clm3%g%l%c%p%gridcell
   forc_hgt   => clm3%g%a2ls%forc_hgt
   forc_hgt_u => clm3%g%a2ls%forc_hgt_u
   forc_hgt_t => clm3%g%a2ls%forc_hgt_t
   forc_hgt_q => clm3%g%a2ls%forc_hgt_q
!!!------------------zhangyw 10m wind 20090814-------------------------
   u_10m => clm3%g%l%c%p%pes%u_10m  ! 10m height wind (m/s)
   v_10m => clm3%g%l%c%p%pes%v_10m  ! 10m height wind (m/s)
   forc_u => clm3%g%a2ls%forc_u ! 10m height wind (m/s)
   forc_v => clm3%g%a2ls%forc_v ! 10m height wind (m/s)
!!!------------------zhangyw 10m wind 20090814-------------------------

   ! Assign local pointers to derived type members (pft-level)

   u10        => clm3%g%l%c%p%pps%u10
   fv         => clm3%g%l%c%p%pps%fv

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   rds        => clm3%g%l%c%p%pps%rds
#endif

   ! Adjustment factors for unstable (moz < 0) or stable (moz > 0) conditions.

#if (!defined PERGRO)

!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      g = pgridcell(p)

      ! Wind profile

      zldis(p) = forc_hgt_u(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetam) then
         ustar(p) = vkc*um(p)/(log(-zetam*obu(p)/z0m(p))&
              - StabilityFunc1(-zetam) &
              + StabilityFunc1(z0m(p)/obu(p)) &
              + 1.14*((-zeta(p))**0.333-(zetam)**0.333))
      else if (zeta(p) < 0.) then
         ustar(p) = vkc*um(p)/(log(zldis(p)/z0m(p))&
              - StabilityFunc1(zeta(p))&
              + StabilityFunc1(z0m(p)/obu(p)))
      else if (zeta(p) <=  1.) then
         ustar(p) = vkc*um(p)/(log(zldis(p)/z0m(p)) + 5.*zeta(p) -5.*z0m(p)/obu(p))
      else
         ustar(p) = vkc*um(p)/(log(obu(p)/z0m(p))+5.-5.*z0m(p)/obu(p) &
              +(5.*log(zeta(p))+zeta(p)-1.))
      end if

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
      if (zeta(p) < 0._r8) then
         rds(p) = 1.0_r8 / (2.e-3_r8*ustar(p) * ( 1._r8 + (300._r8/(-obu(p)))**0.666_r8) )
      else
         rds(p) = 1.0_r8 / (2.e-3_r8*ustar(p) )
      endif
#endif
      ! Temperature profile

      zldis(p) = forc_hgt_t(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetat) then
         temp1(p) = vkc/(log(-zetat*obu(p)/z0h(p))&
              - StabilityFunc2(-zetat) &
              + StabilityFunc2(z0h(p)/obu(p)) &
              + 0.8*((zetat)**(-0.333)-(-zeta(p))**(-0.333)))
      else if (zeta(p) < 0.) then
         temp1(p) = vkc/(log(zldis(p)/z0h(p)) &
              - StabilityFunc2(zeta(p)) &
              + StabilityFunc2(z0h(p)/obu(p)))
      else if (zeta(p) <=  1.) then
         temp1(p) = vkc/(log(zldis(p)/z0h(p)) + 5.*zeta(p) - 5.*z0h(p)/obu(p))
      else
         temp1(p) = vkc/(log(obu(p)/z0h(p)) + 5. - 5.*z0h(p)/obu(p) &
              + (5.*log(zeta(p))+zeta(p)-1.))
      end if

      ! Humidity profile

      if (forc_hgt_q(g) == forc_hgt_t(g) .and. z0q(p) == z0h(p)) then
         temp2(p) = temp1(p)
      else
         zldis(p) = forc_hgt_q(g)-displa(p)
         zeta(p) = zldis(p)/obu(p)
         if (zeta(p) < -zetat) then
            temp2(p) = vkc/(log(-zetat*obu(p)/z0q(p)) &
                 - StabilityFunc2(-zetat) &
                 + StabilityFunc2(z0q(p)/obu(p)) &
                 + 0.8*((zetat)**(-0.333)-(-zeta(p))**(-0.333)))
         else if (zeta(p) < 0.) then
            temp2(p) = vkc/(log(zldis(p)/z0q(p)) &
                 - StabilityFunc2(zeta(p)) &
                 + StabilityFunc2(z0q(p)/obu(p)))
         else if (zeta(p) <=  1.) then
            temp2(p) = vkc/(log(zldis(p)/z0q(p)) + 5.*zeta(p)-5.*z0q(p)/obu(p))
         else
            temp2(p) = vkc/(log(obu(p)/z0q(p)) + 5. - 5.*z0q(p)/obu(p) &
                 + (5.*log(zeta(p))+zeta(p)-1.))
         end if
      endif

      ! Temperature profile applied at 2-m

      zldis(p) = 2.0 + z0h(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetat) then
         temp12m(p) = vkc/(log(-zetat*obu(p)/z0h(p))&
              - StabilityFunc2(-zetat) &
              + StabilityFunc2(z0h(p)/obu(p)) &
              + 0.8*((zetat)**(-0.333)-(-zeta(p))**(-0.333)))
      else if (zeta(p) < 0.) then
         temp12m(p) = vkc/(log(zldis(p)/z0h(p)) &
              - StabilityFunc2(zeta(p))  &
              + StabilityFunc2(z0h(p)/obu(p)))
      else if (zeta(p) <=  1.) then
         temp12m(p) = vkc/(log(zldis(p)/z0h(p)) + 5.*zeta(p) - 5.*z0h(p)/obu(p))
      else
         temp12m(p) = vkc/(log(obu(p)/z0h(p)) + 5. - 5.*z0h(p)/obu(p) &
              + (5.*log(zeta(p))+zeta(p)-1.))
      end if

      ! Humidity profile applied at 2-m

      if (z0q(p) == z0h(p)) then
         temp22m(p) = temp12m(p)
      else
         zldis(p) = 2.0 + z0q(p)
         zeta(p) = zldis(p)/obu(p)
         if (zeta(p) < -zetat) then
            temp22m(p) = vkc/(log(-zetat*obu(p)/z0q(p)) - &
                 StabilityFunc2(-zetat) + StabilityFunc2(z0q(p)/obu(p)) &
                 + 0.8*((zetat)**(-0.333)-(-zeta(p))**(-0.333)))
         else if (zeta(p) < 0.) then
            temp22m(p) = vkc/(log(zldis(p)/z0q(p)) - &
                 StabilityFunc2(zeta(p))+StabilityFunc2(z0q(p)/obu(p)))
         else if (zeta(p) <=  1.) then
            temp22m(p) = vkc/(log(zldis(p)/z0q(p)) + 5.*zeta(p)-5.*z0q(p)/obu(p))
         else
            temp22m(p) = vkc/(log(obu(p)/z0q(p)) + 5. - 5.*z0q(p)/obu(p) &
                 + (5.*log(zeta(p))+zeta(p)-1.))
         end if
      end if

!pp!!------------------zhangyw 10m wind 20090814-------------------------
!#if (defined DGVM) 
      ! diagnose 10-m wind for dust model (dstmbl.F)
      ! Notes from C. Zender's dst.F:
      ! According to Bon96 p. 62, the displacement height d (here displa) is
      ! 0.0 <= d <= 0.34 m in dust source regions (i.e., regions w/o trees).
      ! Therefore d <= 0.034*z1 and may safely be neglected.
      ! Code from LSM routine SurfaceTemperature was used to obtain u10

      zldis(p) = forc_hgt_u(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (min(zeta(p), 1.) < 0.) then
         tmp1 = (1. - 16.*min(zeta(p),1.))**0.25
         tmp2 = log((1.+tmp1*tmp1)/2.)
         tmp3 = log((1.+tmp1)/2.)
         fmnew = 2.*tmp3 + tmp2 - 2.*atan(tmp1) + 1.5707963
      else
         fmnew = -5.*min(zeta(p),1.)
      endif
      if (iter == 1) then
         fm(p) = fmnew
      else
         fm(p) = 0.5 * (fm(p)+fmnew)
      end if
      zeta10 = min(10./obu(p), 1.)
      if (zeta(p) == 0.) zeta10 = 0.
      if (zeta10 < 0.) then
         tmp1 = (1.0 - 16.0 * zeta10)**0.25
         tmp2 = log((1.0 + tmp1*tmp1)/2.0)
         tmp3 = log((1.0 + tmp1)/2.0)
         fm10 = 2.0*tmp3 + tmp2 - 2.0*atan(tmp1) + 1.5707963
      else                ! not stable
         fm10 = -5.0 * zeta10
      end if
      tmp4 = log(forc_hgt(g) / 10.)
      u10(p) = ur(p) - ustar(p)/vkc * (tmp4 - fm(p) + fm10)
      fv(p)  = ustar(p)
!!!------------------zhangyw 10m wind 20090814-------------------------
      u_10m(p)= u10(p)*forc_u(g)/sqrt(forc_u(g)*forc_u(g)+forc_v(g)*forc_v(g))
      v_10m(p)= u10(p)*forc_v(g)/sqrt(forc_u(g)*forc_u(g)+forc_v(g)*forc_v(g))
!#endif
!!!------------------zhangyw 10m wind 20090814-------------------------

   end do
#endif


#if (defined PERGRO)

   !===============================================================================
   ! The following only applies when PERGRO is defined
   !===============================================================================

!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      g = pgridcell(p)

      zldis(p) = forc_hgt_u(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetam) then           ! zeta < -1
         ustar(p) = vkc * um(p) / log(-zetam*obu(p)/z0m(p))
      else if (zeta(p) < 0.) then         ! -1 <= zeta < 0
         ustar(p) = vkc * um(p) / log(zldis(p)/z0m(p))
      else if (zeta(p) <= 1.) then        !  0 <= ztea <= 1
         ustar(p)=vkc * um(p)/log(zldis(p)/z0m(p))
      else                             !  1 < zeta, phi=5+zeta
         ustar(p)=vkc * um(p)/log(obu(p)/z0m(p))
      endif

      zldis(p) = forc_hgt_t(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetat) then
         temp1(p)=vkc/log(-zetat*obu(p)/z0h(p))
      else if (zeta(p) < 0.) then
         temp1(p)=vkc/log(zldis(p)/z0h(p))
      else if (zeta(p) <= 1.) then
         temp1(p)=vkc/log(zldis(p)/z0h(p))
      else
         temp1(p)=vkc/log(obu(p)/z0h(p))
      end if

      zldis(p) = forc_hgt_q(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetat) then
         temp2(p)=vkc/log(-zetat*obu(p)/z0q(p))
      else if (zeta(p) < 0.) then
         temp2(p)=vkc/log(zldis(p)/z0q(p))
      else if (zeta(p) <= 1.) then
         temp2(p)=vkc/log(zldis(p)/z0q(p))
      else
         temp2(p)=vkc/log(obu(p)/z0q(p))
      end if

      zldis(p) = 2.0 + z0h(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetat) then
         temp12m(p)=vkc/log(-zetat*obu(p)/z0h(p))
      else if (zeta(p) < 0.) then
         temp12m(p)=vkc/log(zldis(p)/z0h(p))
      else if (zeta(p) <= 1.) then
         temp12m(p)=vkc/log(zldis(p)/z0h(p))
      else
         temp12m(p)=vkc/log(obu(p)/z0h(p))
      end if

      zldis(p) = 2.0 + z0q(p)
      zeta(p) = zldis(p)/obu(p)
      if (zeta(p) < -zetat) then
         temp22m(p)=vkc/log(-zetat*obu(p)/z0q(p))
      else if (zeta(p) < 0.) then
         temp22m(p)=vkc/log(zldis(p)/z0q(p))
      else if (zeta(p) <= 1.) then
         temp22m(p)=vkc/log(zldis(p)/z0q(p))
      else
         temp22m(p)=vkc/log(obu(p)/z0q(p))
      end if
#if (defined DGVM) 
      ! diagnose 10-m wind for dust model (dstmbl.F)
      ! Notes from C. Zender's dst.F:
      ! According to Bon96 p. 62, the displacement height d (here displa) is
      ! 0.0 <= d <= 0.34 m in dust source regions (i.e., regions w/o trees).
      ! Therefore d <= 0.034*z1 and may safely be neglected.
      ! Code from LSM routine SurfaceTemperature was used to obtain u10

      zldis(p) = forc_hgt_u(g)-displa(p)
      zeta(p) = zldis(p)/obu(p)
      if (min(zeta(p), 1.) < 0.) then
         tmp1 = (1. - 16.*min(zeta(p),1.))**0.25
         tmp2 = log((1.+tmp1*tmp1)/2.)
         tmp3 = log((1.+tmp1)/2.)
         fmnew = 2.*tmp3 + tmp2 - 2.*atan(tmp1) + 1.5707963
      else
         fmnew = -5.*min(zeta(p),1.)
      endif
      if (iter == 1) then
         fm(p) = fmnew
      else
         fm(p) = 0.5 * (fm(p)+fmnew)
      end if
      zeta10 = min(10./obu(p), 1.)
      if (zeta(p) == 0.) zeta10 = 0.
      if (zeta10 < 0.) then
         tmp1 = (1.0 - 16.0 * zeta10)**0.25
         tmp2 = log((1.0 + tmp1*tmp1)/2.0)
         tmp3 = log((1.0 + tmp1)/2.0)
         fm10 = 2.0*tmp3 + tmp2 - 2.0*atan(tmp1) + 1.5707963
      else                ! not stable
         fm10 = -5.0 * zeta10
      end if
      tmp4 = log(forc_hgt(g) / 10.)
      u10(p) = ur(p) - ustar(p)/vkc * (tmp4 - fm(p) + fm10)
      fv(p)  = ustar(p)
#endif
   end do

#endif

   end subroutine FrictionVelocity

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: StabilityFunc
!
! !INTERFACE:
   real(r8) function StabilityFunc1(zeta)
!
! !DESCRIPTION:
! Stability function for rib < 0.
!
! !USES:
      use shr_const_mod, only: SHR_CONST_PI
!
! !ARGUMENTS:
      implicit none
      real(r8), intent(in) :: zeta  ! dimensionless height used in Monin-Obukhov theory
!
! !CALLED FROM:
! subroutine FrictionVelocity in this module
!
! !REVISION HISTORY:
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
!
!EOP
!
! !LOCAL VARIABLES:
      real(r8) :: chik, chik2
!------------------------------------------------------------------------------

      chik2 = sqrt(1.-16.*zeta)
      chik = sqrt(chik2)
      StabilityFunc1 = 2.*log((1.+chik)*0.5) &
           + log((1.+chik2)*0.5)-2.*atan(chik)+SHR_CONST_PI*0.5

    end function StabilityFunc1

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: StabilityFunc2
!
! !INTERFACE:
   real(r8) function StabilityFunc2(zeta)
!
! !DESCRIPTION:
! Stability function for rib < 0.
!
! !USES:
     use shr_const_mod, only: SHR_CONST_PI
!
! !ARGUMENTS:
     implicit none
     real(r8), intent(in) :: zeta  ! dimensionless height used in Monin-Obukhov theory
!
! !CALLED FROM:
! subroutine FrictionVelocity in this module
!
! !REVISION HISTORY:
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
!
!EOP
!
! !LOCAL VARIABLES:
     real(r8) :: chik2
!------------------------------------------------------------------------------

     chik2 = sqrt(1.-16.*zeta)
     StabilityFunc2 = 2.*log((1.+chik2)*0.5)

   end function StabilityFunc2

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: MoninObukIni
!
! !INTERFACE:
  subroutine MoninObukIni (ur, thv, dthv, zldis, z0m, um, obu)
!
! !DESCRIPTION:
! Initialization of the Monin-Obukhov length.
! The scheme is based on the work of Zeng et al. (1998):
! Intercomparison of bulk aerodynamic algorithms for the computation
! of sea surface fluxes using TOGA CORE and TAO data. J. Climate,
! Vol. 11, 2628-2644.
!
! !USES:
    use clm_varcon, only : grav
!
! !ARGUMENTS:
    implicit none
    real(r8), intent(in)  :: ur    ! wind speed at reference height [m/s]
    real(r8), intent(in)  :: thv   ! virtual potential temperature (kelvin)
    real(r8), intent(in)  :: dthv  ! diff of vir. poten. temp. between ref. height and surface
    real(r8), intent(in)  :: zldis ! reference height "minus" zero displacement heght [m]
    real(r8), intent(in)  :: z0m   ! roughness length, momentum [m]
    real(r8), intent(out) :: um    ! wind speed including the stability effect [m/s]
    real(r8), intent(out) :: obu   ! monin-obukhov length (m)
!
! !CALLED FROM:
! subroutine BareGroundFluxes in module BareGroundFluxesMod.F90
! subroutine BiogeophysicsLake in module BiogeophysicsLakeMod.F90
! subroutine CanopyFluxes in module CanopyFluxesMod.F90
!
! !REVISION HISTORY:
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
!
!EOP
!
! !LOCAL VARIABLES:
!
    real(r8) :: wc    ! convective velocity [m/s]
    real(r8) :: rib   ! bulk Richardson number
    real(r8) :: zeta  ! dimensionless height used in Monin-Obukhov theory
    real(r8) :: ustar ! friction velocity [m/s]
!-----------------------------------------------------------------------

    ! Initial values of u* and convective velocity

    ustar=0.06
    wc=0.5
    if (dthv >= 0.) then
       um=max(ur,0.1_r8)
    else
       um=sqrt(ur*ur+wc*wc)
    endif

    rib=grav*zldis*dthv/(thv*um*um)
#if (defined PERGRO)
    rib = 0.
#endif

    if (rib >= 0.) then      ! neutral or stable
       zeta = rib*log(zldis/z0m)/(1.-5.*min(rib,0.19_r8))
       zeta = min(2._r8,max(zeta,0.01_r8 ))
    else                     ! unstable
       zeta=rib*log(zldis/z0m)
       zeta = max(-100._r8,min(zeta,-0.01_r8 ))
    endif

    obu=zldis/zeta

  end subroutine MoninObukIni

end module FrictionVelocityMod
