#include <misc.h>
#include <preproc.h>

module SurfaceAlbedoMod

!-----------------------------------------------------------------------
!BOP
!
! !MODULE: SurfaceAlbedoMod
!
! !DESCRIPTION:
! Performs surface albedo calculations
!
! !PUBLIC TYPES:
  use shr_sys_mod, only : shr_sys_flush
  implicit none
  save
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: SurfaceAlbedo  ! Surface albedo and two-stream fluxes
  public :: SnowAge        ! Update snow age
!
! !PRIVATE MEMBER FUNCTIONS:
  private :: SnowAlbedo    ! Determine snow albedos
  private :: SoilAlbedo    ! Determine ground surface albedo
#if (defined FOURSTRM)
  private :: FourStream    ! Four-stream fluxes for canopy radiative transfer
#else
  private :: TwoStream     ! Two-stream fluxes for canopy radiative transfer
#endif
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!-----------------------------------------------------------------------

contains

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: SurfaceAlbedo
!
! !INTERFACE:
  subroutine SurfaceAlbedo(lbg, ubg, lbc, ubc, lbp, ubp, &
       caldayp1, eccen, obliqr, lambm0, mvelpp)
!
! !DESCRIPTION:
! Surface albedo and two-stream fluxes
! Surface albedos. Also fluxes (per unit incoming direct and diffuse
! radiation) reflected, transmitted, and absorbed by vegetation.
! Also sunlit fraction of the canopy.
! The calling sequence is:
! -> SurfaceAlbedo:   albedos for next time step
!    -> SnowAlbedo:   snow albedos: direct beam
!    -> SnowAlbedo:   snow albedos: diffuse
!    -> SoilAlbedo:   soil/lake/glacier/wetland albedos
!    -> TwoStream:    absorbed, reflected, transmitted solar fluxes (vis dir,vis dif, nir dir, nir dif)
 ! -> FourStream
! !USES:
    use shr_kind_mod, only: r8 => shr_kind_r8
    use clmtype
    use shr_orb_mod
!
! !ARGUMENTS:
    implicit none
    integer , intent(in) :: lbg, ubg ! gridcell bounds
    integer , intent(in) :: lbc, ubc ! column bounds
    integer , intent(in) :: lbp, ubp ! pft bounds
    real(r8), intent(in) :: caldayp1 ! calendar day at Greenwich (1.00, ..., 365.99)
    real(r8), intent(in) :: eccen    ! Earth's orbital eccentricity
    real(r8), intent(in) :: obliqr   ! Earth's obliquity in radians
    real(r8), intent(in) :: lambm0   ! Mean longitude of perihelion at the vernal equinox (radians)
    real(r8), intent(in) :: mvelpp   ! Earth's moving vernal equinox long. of perihelion
!
! !CALLED FROM:
! subroutine lpjreset1 in module DGVMMod (only applicable when cpp token DGVM is defined)
! subroutine driver
! subroutine iniTimeVar
!
! !REVISION HISTORY:
! Author: Gordon Bonan
! 2/1/02, Peter Thornton: Migrate to new data structures
! 8/20/03, Mariana Vertenstein: Vectorized routine
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in arguments
!
    integer , pointer :: pgridcell(:) ! gridcell of corresponding pft
    integer , pointer :: pcolumn(:)   ! column of corresponding pft
    integer , pointer :: cgridcell(:) ! gridcell of corresponding column
    real(r8), pointer :: lat(:)       ! gridcell latitude (radians)
    real(r8), pointer :: lon(:)       ! gridcell longitude (radians)
    real(r8), pointer :: elai(:)      ! one-sided leaf area index with burying by snow
    real(r8), pointer :: esai(:)      ! one-sided stem area index with burying by snow
    real(r8), pointer :: h2osno(:)    ! snow water (mm H2O)
    real(r8), pointer :: snowage(:)   ! non dimensional snow age [-]
    real(r8), pointer :: rhol(:,:)    ! leaf reflectance: 1=vis, 2=nir
    real(r8), pointer :: rhos(:,:)    ! stem reflectance: 1=vis, 2=nir
    real(r8), pointer :: taul(:,:)    ! leaf transmittance: 1=vis, 2=nir
    real(r8), pointer :: taus(:,:)    ! stem transmittance: 1=vis, 2=nir
    integer , pointer :: ivt(:)       ! pft vegetation type
!
! local pointers toimplicit out arguments
!
    real(r8), pointer :: fsun(:)      ! sunlit fraction of canopy
    real(r8), pointer :: albgrd(:,:)  ! ground albedo (direct)
    real(r8), pointer :: albgri(:,:)  ! ground albedo (diffuse)
    real(r8), pointer :: albd(:,:)    ! surface albedo (direct)
    real(r8), pointer :: albi(:,:)    ! surface albedo (diffuse)
    real(r8), pointer :: fabd(:,:)    ! flux absorbed by veg per unit direct flux
    real(r8), pointer :: fabi(:,:)    ! flux absorbed by veg per unit diffuse flux
    real(r8), pointer :: ftdd(:,:)    ! down direct flux below veg per unit dir flx
    real(r8), pointer :: ftid(:,:)    ! down diffuse flux below veg per unit dir flx
    real(r8), pointer :: ftii(:,:)    ! down diffuse flux below veg per unit dif flx
!--------S2S for snow albedo output(zhangyw20140306)--------------
    real(r8), pointer :: fsds_vis_d(:)        ! incident direct beam vis solar radiation (W/m**2)
    real(r8), pointer :: fsds_nir_d(:)        ! incident direct beam nir solar radiation (W/m**2)
    real(r8), pointer :: fsds_vis_i(:)        ! incident diffuse vis solar radiation (W/m**2)
    real(r8), pointer :: fsds_nir_i(:)        ! incident diffuse nir solar radiation (W/m**2)
    real(r8), pointer :: fsr_vis_d_sno(:)     ! snow reflected direct beam vis solar radiation (W/m**2)
    real(r8), pointer :: fsr_nir_d_sno(:)     ! snow reflected direct beam nir solar radiation (W/m**2)
    real(r8), pointer :: fsr_vis_i_sno(:)     ! snow reflected diffuse vis solar radiation (W/m**2)
    real(r8), pointer :: fsr_nir_i_sno(:)     ! snow reflected diffuse nir solar radiation (W/m**2)
    real(r8), pointer :: fsr_sno(:)     ! snow reflected diffuse nir solar radiation (W/m**2)
    real(r8), pointer :: fsds_sno(:)     ! snow reflected diffuse nir solar radiation (W/m**2)
!-----------------------------------------------------------------
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
    real(r8), parameter :: mpe = 1.e-06    ! prevents overflow for division by zero
    integer  :: fp,g,c,p                   ! indices
    integer  :: ib                         ! band index
    integer  :: ic                         ! 0=unit incoming direct; 1=unit incoming diffuse
    real(r8) :: delta                      ! solar declination angle in radians
    real(r8) :: eccf                       ! earth orbit eccentricity factor
    real(r8) :: wl(lbp:ubp)                ! fraction of LAI+SAI that is LAI
    real(r8) :: ws(lbp:ubp)                ! fraction of LAI+SAI that is SAI
    real(r8) :: vai(lbp:ubp)               ! elai+esai
    real(r8) :: rho(lbp:ubp,numrad)        ! leaf/stem refl weighted by fraction LAI and SAI
    real(r8) :: tau(lbp:ubp,numrad)        ! leaf/stem tran weighted by fraction LAI and SAI
    real(r8) :: ftdi(lbp:ubp,numrad)       ! down direct flux below veg per unit dif flux = 0
    real(r8) :: albsnd(lbc:ubc,numrad)     ! snow albedo (direct)
    real(r8) :: albsni(lbc:ubc,numrad)     ! snow albedo (diffuse)
    real(r8) :: gdir(lbp:ubp)              ! aver projected leaf/stem area in solar direction
    real(r8) :: ext(lbp:ubp)               ! optical depth direct beam per unit LAI+SAI
    real(r8) :: coszen_gcell(lbg:ubg)      ! cosine solar zenith angle for next time step (gridcell level)
    real(r8) :: coszen_col(lbc:ubc)        ! cosine solar zenith angle for next time step (column level)
    real(r8) :: coszen_pft(lbp:ubp)        ! cosine solar zenith angle for next time step (pft level)
    integer  :: num_vegsol                 ! number of vegetated pfts where coszen>0
    integer  :: filter_vegsol(ubp-lbp+1)   ! pft filter where vegetated and coszen>0
    integer  :: num_novegsol               ! number of vegetated pfts where coszen>0
    integer  :: filter_novegsol(ubp-lbp+1) ! pft filter where vegetated and coszen>0
    integer  :: num_solar                  ! number of gridcells where coszen>0
  !-----------------------------------------------------------------------

    ! Assign local pointers to derived subtypes components (gridcell-level)

    lat       => clm3%g%lat
    lon       => clm3%g%lon

    ! Assign local pointers to derived subtypes components (column-level)

    cgridcell => clm3%g%l%c%gridcell
    h2osno    => clm3%g%l%c%cws%h2osno
    snowage   => clm3%g%l%c%cps%snowage
    albgrd    => clm3%g%l%c%cps%albgrd
    albgri    => clm3%g%l%c%cps%albgri

    ! Assign local pointers to derived subtypes components (pft-level)

    pgridcell => clm3%g%l%c%p%gridcell
    pcolumn   => clm3%g%l%c%p%column
    albd      => clm3%g%l%c%p%pps%albd
    albi      => clm3%g%l%c%p%pps%albi
    fabd      => clm3%g%l%c%p%pps%fabd
    fabi      => clm3%g%l%c%p%pps%fabi
    ftdd      => clm3%g%l%c%p%pps%ftdd
    ftid      => clm3%g%l%c%p%pps%ftid
    ftii      => clm3%g%l%c%p%pps%ftii
    fsun      => clm3%g%l%c%p%pps%fsun
    elai      => clm3%g%l%c%p%pps%elai
    esai      => clm3%g%l%c%p%pps%esai
    ivt       => clm3%g%l%c%p%itype
    rhol      => pftcon%rhol
    rhos      => pftcon%rhos
    taul      => pftcon%taul
    taus      => pftcon%taus
!--------S2S for snow albedo output(zhangyw20140306)--------------
    fsds_vis_d        => clm3%g%l%c%p%pef%fsds_vis_d
    fsds_nir_d        => clm3%g%l%c%p%pef%fsds_nir_d
    fsds_vis_i        => clm3%g%l%c%p%pef%fsds_vis_i
    fsds_nir_i        => clm3%g%l%c%p%pef%fsds_nir_i
    fsr_vis_d_sno     => clm3%g%l%c%p%pef%fsr_vis_d_sno
    fsr_nir_d_sno     => clm3%g%l%c%p%pef%fsr_nir_d_sno
    fsr_vis_i_sno     => clm3%g%l%c%p%pef%fsr_vis_i_sno
    fsr_nir_i_sno     => clm3%g%l%c%p%pef%fsr_nir_i_sno
    fsr_sno           => clm3%g%l%c%p%pef%fsr_sno
    fsds_sno          => clm3%g%l%c%p%pef%fsds_sno
!-----------------------------------------------------------------

    ! Solar declination for next time step

    call shr_orb_decl (caldayp1, eccen, mvelpp, lambm0, obliqr, delta, eccf)

    ! Cosine solar zenith angle for next time step

!dir$ concurrent
!cdir nodep
    do g = lbg, ubg
       coszen_gcell(g) = shr_orb_cosz (caldayp1, lat(g), lon(g), delta)
    end do

!dir$ concurrent
!cdir nodep
    do c = lbc, ubc
       g = cgridcell(c)
       coszen_col(c) = coszen_gcell(g)
    end do

!dir$ concurrent
!cdir nodep
    do p = lbp, ubp
       g = pgridcell(p)
       coszen_pft(p) = coszen_gcell(g)
    end do

    ! Initialize output because solar radiation only done if coszen > 0

    do ib = 1, numrad
!dir$ concurrent
!cdir nodep
       do c = lbc,ubc
          albgrd(c,ib) = 0._r8
          albgri(c,ib) = 0._r8
       end do
!dir$ concurrent
!cdir nodep
       do p = lbp,ubp
          albd(p,ib) = 1.
          albi(p,ib) = 1.
          fabd(p,ib) = 0._r8
          fabi(p,ib) = 0._r8
          ftdd(p,ib) = 0._r8
          ftid(p,ib) = 0._r8
          ftii(p,ib) = 0._r8
          if (ib==1) fsun(p) = 0.
       end do
    end do

    ! Index points with positive coszen for subsequent calculations
    ! Return if all coszen are not positive

    num_solar = 0
!dir$ concurrent
!cdir nodep
    do g = lbg,ubg
       if (coszen_gcell(g) > 0._r8) num_solar = num_solar + 1
    end do
    if (num_solar <= 0._r8) return

    ! Snow albedos
    ! Note that snow albedo routine will only compute nonzero snow albedos
    ! where h2osno> 0 and coszen > 0

    ic = 0; call SnowAlbedo(lbc, ubc, coszen_col, ic, albsnd)
    ic = 1; call SnowAlbedo(lbc, ubc, coszen_col, ic, albsni)

!--------S2S for snow albedo output(zhangyw20140306)--------------
!dir$ concurrent
!cdir nodep
     do p = lbp,ubp
         c = pcolumn(p)
         if (coszen_col(c) > 0._r8 .and. h2osno(c) > 0._r8) then
           fsr_sno(p)  = albsnd(c,1)*fsds_vis_d(p) + albsnd(c,2)*fsds_nir_d(p) +  &
                         albsni(c,1)*fsds_vis_i(p) + albsni(c,2)*fsds_nir_i(p)
           fsds_sno(p) = fsds_vis_d(p)+fsds_nir_d(p)+fsds_vis_i(p)+fsds_nir_i(p)
         else
           fsr_sno(p)  = 0.0
           fsds_sno(p) = 0.0
         end if

     end do
!-----------------------------------------------------------------

    ! Ground surface albedos
    ! Note that ground albedo routine will only compute nonzero snow albedos
    ! where coszen > 0

    call SoilAlbedo(lbc, ubc, coszen_col, albsnd, albsni)

    ! Creat solar-vegetated filter for the following calculations

    num_vegsol = 0
    num_novegsol = 0
    do p = lbp,ubp
       if (coszen_pft(p) > 0._r8 .and. (elai(p) + esai(p)) > 0._r8) then
          num_vegsol = num_vegsol + 1
          filter_vegsol(num_vegsol) = p
       else if (coszen_pft(p) > 0._r8 .and. (elai(p) + esai(p)) == 0._r8) then
          num_novegsol = num_novegsol + 1
          filter_novegsol(num_novegsol) = p
       end if
    end do

    ! Weight reflectance/transmittance by lai and sai
    ! Only perform on vegetated pfts where coszen > 0

!dir$ concurrent
!cdir nodep
    do fp = 1,num_vegsol
       p = filter_vegsol(fp)
       vai(p) = elai(p) + esai(p)
       wl(p) = elai(p) / max( vai(p), mpe )
       ws(p) = esai(p) / max( vai(p), mpe )
    end do

    do ib = 1, numrad
!dir$ concurrent
!cdir nodep
       do fp = 1,num_vegsol
          p = filter_vegsol(fp)
          rho(p,ib) = max( rhol(ivt(p),ib)*wl(p) + rhos(ivt(p),ib)*ws(p), mpe )
          tau(p,ib) = max( taul(ivt(p),ib)*wl(p) + taus(ivt(p),ib)*ws(p), mpe )
       end do
    end do

    ! Calculate surface albedos and fluxes
    ! Only perform on vegetated pfts where coszen > 0


#if (defined FOURSTRM)
      call FourStream(lbc, ubc, lbp, ubp, filter_vegsol, num_vegsol, &
                    coszen_pft, vai, rho, tau, ftdi, gdir)
#else
     call TwoStream (lbc, ubc, lbp, ubp, filter_vegsol, num_vegsol, &
                     coszen_pft, vai, rho, tau, ftdi, gdir)
#endif

    ! Calculate sunlit fraction of canopy. Set fsun = 0 if fsun < 0.01.
    ! Only perform on vegetated pfts where coszen > 0.

!dir$ concurrent
!cdir nodep
    do fp = 1,num_vegsol
       p = filter_vegsol(fp)
       ext(p) = gdir(p)/coszen_pft(p) * sqrt(1.-rho(p,1)-tau(p,1))
       fsun(p) = (1.-exp(-ext(p)*vai(p))) / max(ext(p)*vai(p),mpe)
       ext(p) = fsun(p)                    !temporary fsun
       if (ext(p) < 0.01) then
          wl(p) = 0._r8                    !temporary fsun
       else
          wl(p) = ext(p)                   !temporary fsun
       end if
       fsun(p) = wl(p)
    end do

    ! Determine values for non-vegetated pfts where coszen > 0

    do ib = 1,numrad
!dir$ concurrent
!cdir nodep
       do fp = 1,num_novegsol
          p = filter_novegsol(fp)
          c = pcolumn(p)
          fabd(p,ib) = 0.
          fabi(p,ib) = 0.
          ftdd(p,ib) = 1.
          ftid(p,ib) = 0.
          ftii(p,ib) = 1.
          albd(p,ib) = albgrd(c,ib)
          albi(p,ib) = albgri(c,ib)
          fsun(p) = 0.
       end do
    end do

  end subroutine SurfaceAlbedo

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: SnowAlbedo
!
! !INTERFACE:
  subroutine SnowAlbedo (lbc, ubc, coszen, ind, alb)
!
! !DESCRIPTION:
! Determine snow albedos
!
! !USES:
    use shr_kind_mod, only: r8 => shr_kind_r8
    use clmtype
!
! !ARGUMENTS:
    implicit none
    integer , intent(in) :: lbc, ubc                 ! column bounds
    real(r8), intent(in) :: coszen(lbc:ubc)          ! cosine solar zenith angle for next time step
    integer , intent(in) :: ind                      ! 0=direct beam, 1=diffuse radiation
    real(r8), intent(out):: alb(lbc:ubc,2)           ! snow albedo by waveband (assume 2 wavebands)
!
! !CALLED FROM:
! subroutine SurfaceAlbedo in this module
!
! !REVISION HISTORY:
! Author: Gordon Bonan
! 2/5/02, Peter Thornton: Migrated to new data structures. Eliminated
! reference to derived types in this subroutine, and made consistent use
! of the assumption that numrad = 2, with index values: 1=visible,2=NIR
! 8/20/03, Mariana Vertenstein: Vectorized routine
!
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in arguments
!
    real(r8), pointer :: h2osno(:)    ! snow water (mm H2O)
    real(r8), pointer :: snowage(:)   ! non dimensional snow age [-]
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
! variables and constants for snow albedo calculation
!
    real(r8), parameter :: snal0 = 0.95 ! vis albedo of new snow for sza<60
    real(r8), parameter :: snal1 = 0.65 ! nir albedo of new snow for sza<60
    real(r8), parameter :: cons  = 0.2  ! constant for visible snow albedo calculation [-]
    real(r8), parameter :: conn  = 0.5  ! constant for nir snow albedo calculation [-]
    real(r8), parameter :: sl    = 2.0  ! factor that helps control alb zenith dependence [-]
    integer  :: c                       ! index
    real(r8) :: age                     ! factor to reduce visible snow alb due to snow age [-]
    real(r8) :: albs                    ! temporary vis snow albedo
    real(r8) :: albl                    ! temporary nir snow albedo
    real(r8) :: cff                     ! snow alb correction factor for zenith angle > 60 [-]
    real(r8) :: czf                     ! solar zenith correction for new snow albedo [-]
!-----------------------------------------------------------------------

    ! Assign local pointers to derived subtypes components (column-level)

    h2osno  => clm3%g%l%c%cws%h2osno
    snowage => clm3%g%l%c%cps%snowage

    ! this code assumes that numrad = 2 , with the following
    ! index values: 1 = visible, 2 = NIR

    ! Albedo for snow cover.
    ! Snow albedo depends on snow-age, zenith angle, and thickness of snow,
    ! age gives reduction of visible radiation
    ! age below is correction for snow age
    ! czf below corrects albedo of new snow for solar zenith

!dir$ concurrent
!cdir nodep
    do c = lbc, ubc
       if (coszen(c) > 0._r8 .and. h2osno(c) > 0._r8) then
          age = 1.-1./(1.+snowage(c))
          albs = snal0*(1.-cons*age)
          albl = snal1*(1.-conn*age)
          if (ind == 0) then
             cff  = ((1.+1./sl)/(1.+max(0.001_r8, coszen(c))*2.*sl )- 1./sl)
             cff  = max(cff, 0._r8)
             czf  = 0.4 * cff * (1.-albs)
             albs = albs + czf
             czf  = 0.4 *cff * (1.-albl)
             albl = albl + czf
          end if
          alb(c,1) = albs
          alb(c,2) = albl
       else
          alb(c,1) = 0._r8
          alb(c,2) = 0._r8
       end if
    end do

  end subroutine SnowAlbedo

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: SoilAlbedo
!
! !INTERFACE:
  subroutine SoilAlbedo (lbc, ubc, coszen, albsnd, albsni)
!
! !DESCRIPTION:
! Determine ground surface albedo, accounting for snow
!
! !USES:
    use shr_kind_mod, only: r8 => shr_kind_r8
    use clmtype
    use clm_varpar, only : numrad
    use clm_varcon, only : albsat, albdry, alblak, albice, tfrz, istice, istsoil
!
! !ARGUMENTS:
    implicit none
    integer , intent(in) :: lbc, ubc                ! column bounds
    real(r8), intent(in) :: coszen(lbc:ubc)         ! cos solar zenith angle next time step (column-level)
    real(r8), intent(in) :: albsnd(lbc:ubc,numrad)  ! snow albedo (direct)
    real(r8), intent(in) :: albsni(lbc:ubc,numrad)  ! snow albedo (diffuse)
!
! !CALLED FROM:
! subroutine SurfaceAlbedo in this module
!
! !REVISION HISTORY:
! Author: Gordon Bonan
! 2/5/02, Peter Thornton: Migrated to new data structures.
! 8/20/03, Mariana Vertenstein: Vectorized routine
!
! !LOCAL VARIABLES:
!
! local pointers to original implicit in arguments
!
    integer , pointer :: clandunit(:)    ! landunit of corresponding column
    integer , pointer :: ltype(:)        ! landunit type
    integer , pointer :: isoicol(:)      ! soil color class
    real(r8), pointer :: t_grnd(:)       ! ground temperature (Kelvin)
    real(r8), pointer :: frac_sno(:)     ! fraction of ground covered by snow (0 to 1)
    real(r8), pointer :: h2osoi_vol(:,:) ! volumetric soil water [m3/m3]
#if (defined RICEWATER)
    integer , pointer :: ivt(:)          ! pft vegetation type
    integer , pointer :: pfti(:)         ! beginning pft index for each column
#endif
!
! local pointers to original implicit out arguments
!
    real(r8), pointer:: albgrd(:,:)      ! ground albedo (direct)
    real(r8), pointer:: albgri(:,:)      ! ground albedo (diffuse)
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
    integer, parameter :: nband =numrad ! number of solar radiation waveband classes
    integer  :: c,l           ! indices
    integer  :: ib            ! waveband number (1=vis, 2=nir)
    real(r8) :: inc           ! soil water correction factor for soil albedo
    real(r8) :: albsod        ! soil albedo (direct)
    real(r8) :: albsoi        ! soil albedo (diffuse)
    integer  :: soilcol       ! soilcolor
!-----------------------------------------------------------------------
!dir$ inlinenever SoilAlbedo

    ! Assign local pointers to derived subtypes components (column-level)

    clandunit  => clm3%g%l%c%landunit
    isoicol    => clm3%g%l%c%cps%isoicol
    t_grnd     => clm3%g%l%c%ces%t_grnd
    frac_sno   => clm3%g%l%c%cps%frac_sno
    h2osoi_vol => clm3%g%l%c%cws%h2osoi_vol
    albgrd     => clm3%g%l%c%cps%albgrd
    albgri     => clm3%g%l%c%cps%albgri
#if (defined RICEWATER)
    ivt        => clm3%g%l%c%p%itype
    pfti       => clm3%g%l%c%pfti
#endif

    ! Assign local pointers to derived subtypes components (landunit-level)

    ltype      => clm3%g%l%itype

    ! Compute soil albedos

    do ib = 1, nband
!dir$ concurrent
!cdir nodep
       do c = lbc, ubc
          if (coszen(c) > 0._r8) then
             l = clandunit(c)

             if (ltype(l) == istsoil)  then              ! soil
                inc    = max(0.11-0.40*h2osoi_vol(c,1), 0._r8)
                soilcol = isoicol(c)
                albsod = min(albsat(soilcol,ib)+inc, albdry(soilcol,ib))
                albsoi = albsod
             else if (ltype(l) == istice)  then          ! land ice
                albsod = albice(ib)
                albsoi = albsod
             else if (t_grnd(c) > tfrz) then             ! unfrozen lake, wetland
                albsod = 0.05/(max(0.001_r8,coszen(c)) + 0.15)
                albsoi = albsod
             else                                        ! frozen lake, wetland
                albsod = alblak(ib)
                albsoi = albsod
             end if

#if (defined RICEWATER)
             if (ivt(pfti(c)).eq.17)then
               if (t_grnd(c) > tfrz) then
                 albsod = 0.05/(max(0.001_r8,coszen(c)) + 0.15)
                 albsoi = albsod
               else
                 albsod = alblak(ib)
                 albsoi = albsod
               end if
             end if
#endif
             albgrd(c,ib) = albsod*(1.-frac_sno(c)) + albsnd(c,ib)*frac_sno(c)
             albgri(c,ib) = albsoi*(1.-frac_sno(c)) + albsni(c,ib)*frac_sno(c)
          end if
       end do
    end do

  end subroutine SoilAlbedo

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: TwoStream
!
! !INTERFACE:

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: SnowAge
!
! !INTERFACE:
  subroutine SnowAge (lbc, ubc)
!
! !DESCRIPTION:
! Updates snow age Based on BATS code.
!
! !USES:
    use shr_kind_mod, only: r8 => shr_kind_r8
    use clmtype
    use clm_varcon, only : tfrz
    use time_manager, only : get_step_size
!
! !ARGUMENTS:
    implicit none
    integer , intent(in) :: lbc, ubc ! column bounds
!
! !CALLED FROM:
!
! !REVISION HISTORY:
! Original Code:  Robert Dickinson
! 15 September 1999: Yongjiu Dai; Integration of code into CLM
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! 3/4/02, Peter Thornton: Migrated to new data structures.
! 8/20/03, Mariana Vertenstein: Vectorized routine
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in arguments
!
    real(r8), pointer :: t_grnd(:)     ! ground temperature (Kelvin)
    real(r8), pointer :: h2osno(:)     ! snow water (mm H2O)
    real(r8), pointer :: h2osno_old(:) ! snow mass for previous time step (kg/m2)
!
! local pointers to implicit inout arguments
!
    real(r8), pointer :: snow_age(:)   ! non dimensional snow age [-]
#if (defined RICEWATER)
    integer , pointer :: ivt(:)        ! pft vegetation type
    integer , pointer :: pfti(:)       ! column's beginning pft index - ADD
#endif
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
    integer  :: c     ! index
    real(r8) :: age1  ! snow aging factor due to crystal growth [-]
    real(r8) :: age2  ! snow aging factor due to surface growth [-]
    real(r8) :: age3  ! snow aging factor due to accum of other particles [-]
    real(r8) :: arg   ! temporary variable used in snow age calculation [-]
    real(r8) :: arg2  ! temporary variable used in snow age calculation [-]
    real(r8) :: dela  ! temporary variable used in snow age calculation [-]
    real(r8) :: dels  ! temporary variable used in snow age calculation [-]
    real(r8) :: sge   ! temporary variable used in snow age calculation [-]
    real(r8) :: dtime ! land model time step (sec)
!-----------------------------------------------------------------------

   ! Assign local pointers to derived type members (column-level)

    snow_age   => clm3%g%l%c%cps%snowage
    t_grnd     => clm3%g%l%c%ces%t_grnd
    h2osno     => clm3%g%l%c%cws%h2osno
    h2osno_old => clm3%g%l%c%cws%h2osno_old
#if (defined RICEWATER)
    ivt        => clm3%g%l%c%p%itype
    pfti       => clm3%g%l%c%pfti
#endif

    ! Determine snow age

    dtime = get_step_size()

!dir$ concurrent
!cdir nodep
    do c = lbc, ubc
       if (h2osno(c) <= 0.) then
          snow_age(c) = 0._r8
       else if (h2osno(c) > 800.) then       ! Over Antarctica
          snow_age(c) = 0._r8
       else                                  ! Away from Antarctica
          age3 = 0.3
          arg  = 5.e3*(1./tfrz-1./t_grnd(c))
          arg2 = min(0._r8, 10.*arg)
          age2 = exp(arg2)
          age1 = exp(arg)
          dela = 1.e-6 * dtime * (age1+age2+age3)
          dels = 0.1*max(0.0_r8,  h2osno(c)-h2osno_old(c))
          sge  = (snow_age(c)+dela) * (1.0-dels)
          snow_age(c) = max(0.0_r8, sge)
       end if
#if (defined RICEWATER)
       if(ivt(pfti(c)).eq.16.or.ivt(pfti(c)).eq.17)then
        if (h2osno(c) >= 0.and.t_grnd(c)>=tfrz)then
          snow_age(c) = 0._r8
        end if
       end if
#endif
    end do

 end subroutine SnowAge


  subroutine TwoStream (lbc, ubc, lbp, ubp, filter_vegsol, num_vegsol, &
                        coszen, vai, rho, tau, ftdi, gdir)
!
! !DESCRIPTION:
! Two-stream fluxes for canopy radiative transfer
! Use two-stream approximation of Dickinson (1983) Adv Geophysics
! 25:305-353 and Sellers (1985) Int J Remote Sensing 6:1335-1372
! to calculate fluxes absorbed by vegetation, reflected by vegetation,
! and transmitted through vegetation for unit incoming direct or diffuse
! flux given an underlying surface with known albedo.
!
! !USES:
    use shr_kind_mod, only: r8 => shr_kind_r8
    use clmtype
    use clm_varpar, only : numrad
    use clm_varcon, only : omegas, tfrz, betads, betais
!
! !ARGUMENTS:
    implicit none
    integer , intent(in)  :: lbc, ubc                 ! column bounds
    integer , intent(in)  :: lbp, ubp                 ! pft bounds
    integer , intent(in)  :: filter_vegsol(ubp-lbp+1) ! filter for vegetated pfts with coszen>0
    integer , intent(in)  :: num_vegsol               ! number of vegetated pfts where coszen>0
    real(r8), intent(in)  :: coszen(lbp:ubp)          ! cosine solar zenith angle for next time step
    real(r8), intent(in)  :: vai(lbp:ubp)             ! elai+esai
    real(r8), intent(in)  :: rho(lbp:ubp,numrad)      ! leaf/stem refl weighted by fraction LAI and SAI
    real(r8), intent(in)  :: tau(lbp:ubp,numrad)      ! leaf/stem tran weighted by fraction LAI and SAI
    real(r8), intent(out) :: ftdi(lbp:ubp,numrad)     ! down direct flux below veg per unit dif flux
    real(r8), intent(out) :: gdir(lbp:ubp)            ! aver projected leaf/stem area in solar direction
!
! !CALLED FROM:
! subroutine SurfaceAlbedo in this module
!
! !REVISION HISTORY:
! Author: Gordon Bonan
! Modified for speedup: Mariana Vertenstein, 8/26/02
! Vectorized routine: Mariana Vertenstein:  8/20/03
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in scalars
!
    integer , pointer :: pcolumn(:)    ! column of corresponding pft
    real(r8), pointer :: albgrd(:,:)   ! ground albedo (direct) (column-level)
    real(r8), pointer :: albgri(:,:)   ! ground albedo (diffuse)(column-level)
    real(r8), pointer :: t_veg(:)      ! vegetation temperature (Kelvin)
    real(r8), pointer :: fwet(:)       ! fraction of canopy that is wet (0 to 1)
    integer , pointer :: ivt(:)        ! pft vegetation type
    real(r8), pointer :: xl(:)         ! ecophys const - leaf/stem orientation index
!
! local pointers to implicit out scalars
!
    real(r8), pointer :: albd(:,:)     ! surface albedo (direct)
    real(r8), pointer :: albi(:,:)     ! surface albedo (diffuse)
    real(r8), pointer :: fabd(:,:)     ! flux absorbed by veg per unit direct flux
    real(r8), pointer :: fabi(:,:)     ! flux absorbed by veg per unit diffuse flux
    real(r8), pointer :: ftdd(:,:)     ! down direct flux below veg per unit dir flx
    real(r8), pointer :: ftid(:,:)     ! down diffuse flux below veg per unit dir flx
    real(r8), pointer :: ftii(:,:)     ! down diffuse flux below veg per unit dif flx
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
    integer  :: fp,p,c           ! array indices
    !integer  :: ic               ! 0=unit incoming direct; 1=unit incoming diffuse
    integer  :: ib               ! waveband number
    real(r8) :: cosz             ! 0.001 <= coszen <= 1.000
    real(r8) :: asu              ! single scattering albedo
    real(r8) :: chil(lbp:ubp)    ! -0.4 <= xl <= 0.6
    real(r8) :: twostext(lbp:ubp)! optical depth of direct beam per unit leaf area
    real(r8) :: avmu(lbp:ubp)    ! average diffuse optical depth
    real(r8) :: omega            ! fraction of intercepted radiation that is scattered
    real(r8) :: omegal           ! omega for leaves
    real(r8) :: betai            ! upscatter parameter for diffuse radiation
    real(r8) :: betail           ! betai for leaves
    real(r8) :: betad            ! upscatter parameter for direct beam radiation
    real(r8) :: betadl           ! betad for leaves
    real(r8) :: tmp0,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,tmp9 ! temporary
    real(r8) :: p1,p2,p3,p4,s1,s2,u1,u2,u3                        ! temporary
    real(r8) :: b,c1,d,d1,d2,f,h,h1,h2,h3,h4,h5,h6,h7,h8,h9,h10   ! temporary
    real(r8) :: phi1,phi2,sigma                                   ! temporary
    real(r8) :: temp0(lbp:ubp),temp1,temp2(lbp:ubp)               ! temporary
!-----------------------------------------------------------------------

    ! Assign local pointers to derived subtypes components (column-level)

    albgrd  => clm3%g%l%c%cps%albgrd
    albgri  => clm3%g%l%c%cps%albgri

    ! Assign local pointers to derived subtypes components (pft-level)

    pcolumn => clm3%g%l%c%p%column
    fwet    => clm3%g%l%c%p%pps%fwet
    t_veg   => clm3%g%l%c%p%pes%t_veg
    ivt     => clm3%g%l%c%p%itype
    albd    => clm3%g%l%c%p%pps%albd
    albi    => clm3%g%l%c%p%pps%albi
    fabd    => clm3%g%l%c%p%pps%fabd
    fabi    => clm3%g%l%c%p%pps%fabi
    ftdd    => clm3%g%l%c%p%pps%ftdd
    ftid    => clm3%g%l%c%p%pps%ftid
    ftii    => clm3%g%l%c%p%pps%ftii
    xl      => pftcon%xl

    ! Calculate two-stream parameters omega, betad, betai, avmu, gdir, twostext.
    ! Omega, betad, betai are adjusted for snow. Values for omega*betad
    ! and omega*betai are calculated and then divided by the new omega
    ! because the product omega*betai, omega*betad is used in solution.
    ! Also, the transmittances and reflectances (tau, rho) are linear
    ! weights of leaf and stem values.

!dir$ concurrent
!cdir nodep
    do fp = 1,num_vegsol
       p = filter_vegsol(fp)
       cosz = max(0.001_r8, coszen(p))
       chil(p) = min( max(xl(ivt(p)), -0.4_r8), 0.6_r8 )
       if (abs(chil(p)) <= 0.01) chil(p) = 0.01
       phi1 = 0.5 - 0.633*chil(p) - 0.330*chil(p)*chil(p)
       phi2 = 0.877 * (1.-2.*phi1)
       gdir(p) = phi1 + phi2*cosz
       twostext(p) = gdir(p)/cosz
       avmu(p) = ( 1. - phi1/phi2 * log((phi1+phi2)/phi1) ) / phi2
       temp0(p) = gdir(p) + phi2*cosz
       temp1 = phi1*cosz
       temp2(p) = ( 1. - temp1/temp0(p) * log((temp1+temp0(p))/temp1) )
    end do

    do ib = 1, numrad
!dir$ concurrent
!cdir nodep
       do fp = 1,num_vegsol
          p = filter_vegsol(fp)
          c = pcolumn(p)

          omegal = rho(p,ib) + tau(p,ib)
          asu = 0.5*omegal*gdir(p)/temp0(p) *temp2(p)
          betadl = (1.+avmu(p)*twostext(p))/(omegal*avmu(p)*twostext(p))*asu
          betail = 0.5 * ((rho(p,ib)+tau(p,ib)) + (rho(p,ib)-tau(p,ib)) &
               * ((1.+chil(p))/2.)**2) / omegal

          ! Adjust omega, betad, and betai for intercepted snow

          if (t_veg(p) > tfrz) then                             !no snow
             tmp0 = omegal
             tmp1 = betadl
             tmp2 = betail
          else
             tmp0 =   (1.-fwet(p))*omegal        + fwet(p)*omegas(ib)
             tmp1 = ( (1.-fwet(p))*omegal*betadl + fwet(p)*omegas(ib)*betads ) / tmp0
             tmp2 = ( (1.-fwet(p))*omegal*betail + fwet(p)*omegas(ib)*betais ) / tmp0
          end if
          omega = tmp0
          betad = tmp1
          betai = tmp2

          ! Absorbed, reflected, transmitted fluxes per unit incoming radiation
          b = 1. - omega + omega*betai
          c1 = omega*betai
          tmp0 = avmu(p)*twostext(p)
          d = tmp0 * omega*betad
          f = tmp0 * omega*(1.-betad)
          tmp1 = b*b - c1*c1
          h = sqrt(tmp1) / avmu(p)
          sigma = tmp0*tmp0 - tmp1
          p1 = b + avmu(p)*h
          p2 = b - avmu(p)*h
          p3 = b + tmp0
          p4 = b - tmp0
          s1 = exp(-h*vai(p))
          s2 = exp(-twostext(p)*vai(p))

          ! Determine fluxes for vegetated pft for unit incoming direct
          ! Loop over incoming direct and incoming diffuse
          ! 0=unit incoming direct; 1=unit incoming diffuse

          ! ic = 0 unit incoming direct flux
          ! ========================================

          u1 = b - c1/albgrd(c,ib)
          u2 = b - c1*albgrd(c,ib)
          u3 = f + c1*albgrd(c,ib)

          tmp2 = u1 - avmu(p)*h
          tmp3 = u1 + avmu(p)*h
          d1 = p1*tmp2/s1 - p2*tmp3*s1
          tmp4 = u2 + avmu(p)*h
          tmp5 = u2 - avmu(p)*h
          d2 = tmp4/s1 - tmp5*s1
          h1 = -d*p4 - c1*f
          tmp6 = d - h1*p3/sigma
          tmp7 = ( d - c1 - h1/sigma*(u1+tmp0) ) * s2
          h2 = ( tmp6*tmp2/s1 - p2*tmp7 ) / d1
          h3 = - ( tmp6*tmp3*s1 - p1*tmp7 ) / d1
          h4 = -f*p3 - c1*d
          tmp8 = h4/sigma
          tmp9 = ( u3 - tmp8*(u2-tmp0) ) * s2
          h5 = - ( tmp8*tmp4/s1 + tmp9 ) / d2
          h6 = ( tmp8*tmp5*s1 + tmp9 ) / d2
          h7 = (c1*tmp2) / (d1*s1)
          h8 = (-c1*tmp3*s1) / d1
          h9 = tmp4 / (d2*s1)
          h10 = (-tmp5*s1) / d2

          ! Downward direct and diffuse fluxes below vegetation (ic = 0)

          ftdd(p,ib) = s2
          ftid(p,ib) = h4*s2/sigma + h5*s1 + h6/s1

          ! Flux reflected by vegetation (ic = 0)

          albd(p,ib) = h1/sigma + h2 + h3

          ! Flux absorbed by vegetation (ic = 0)

          fabd(p,ib) = 1. - albd(p,ib) &
               - (1.-albgrd(c,ib))*ftdd(p,ib) - (1.-albgri(c,ib))*ftid(p,ib)

          ! ic = 1 unit incoming diffuse
          ! ========================================

          u1 = b - c1/albgri(c,ib)
          u2 = b - c1*albgri(c,ib)
          u3 = f + c1*albgri(c,ib)

          tmp2 = u1 - avmu(p)*h
          tmp3 = u1 + avmu(p)*h
          d1 = p1*tmp2/s1 - p2*tmp3*s1
          tmp4 = u2 + avmu(p)*h
          tmp5 = u2 - avmu(p)*h
          d2 = tmp4/s1 - tmp5*s1
          h1 = -d*p4 - c1*f
          tmp6 = d - h1*p3/sigma
          tmp7 = ( d - c1 - h1/sigma*(u1+tmp0) ) * s2
          h2 = ( tmp6*tmp2/s1 - p2*tmp7 ) / d1
          h3 = - ( tmp6*tmp3*s1 - p1*tmp7 ) / d1
          h4 = -f*p3 - c1*d
          tmp8 = h4/sigma
          tmp9 = ( u3 - tmp8*(u2-tmp0) ) * s2
          h5 = - ( tmp8*tmp4/s1 + tmp9 ) / d2
          h6 = ( tmp8*tmp5*s1 + tmp9 ) / d2
          h7 = (c1*tmp2) / (d1*s1)
          h8 = (-c1*tmp3*s1) / d1
          h9 = tmp4 / (d2*s1)
          h10 = (-tmp5*s1) / d2

          ! Downward direct and diffuse fluxes below vegetation

          ftdi(p,ib) = 0._r8
          ftii(p,ib) = h9*s1 + h10/s1

          ! Flux reflected by vegetation

          albi(p,ib) = h7 + h8    !zwy cancel 08/08/22

          ! Flux absorbed by vegetation

          fabi(p,ib) = 1. - albi(p,ib) &
               - (1.-albgrd(c,ib))*ftdi(p,ib) - (1.-albgri(c,ib))*ftii(p,ib)
       end do   ! end of pft loop
    end do   ! end of radiation band loop

  end subroutine TwoStream

!-----------------------------------------------------------------------

#if (defined FOURSTRM)
!-----------------------------------------------------------------------
   subroutine Fourstream(lbc,ubc,lbp,ubp,filter_vegsol,&
                        num_vegsol,coszen,vai,rho,tau,ftdi,gdir)
!--------------------------------Commons--------------------------------
!--------------------------------Arguments-----------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use clmtype
   use clm_varpar, only : numrad
   use clm_varcon, only : omegas, tfrz, betads, betais
   implicit none
   integer , intent(in)  :: lbc, ubc  ! column bounds
   integer , intent(in)  :: lbp, ubp   ! pft bounds
   integer , intent(in)  :: filter_vegsol(ubp-lbp+1)
   integer , intent(in)  :: num_vegsol
   real(r8), intent(in)  :: coszen(lbp:ubp)
   real(r8), intent(in)  :: vai(lbp:ubp)
   real(r8), intent(in)  :: rho(lbp:ubp,numrad)!zwy_in
   real(r8), intent(in)  :: tau(lbp:ubp,numrad)!zwy_in
   real(r8), intent(out) :: ftdi(lbp:ubp,numrad)
   real(r8), intent(out) :: gdir(lbp:ubp)
   integer , pointer :: pcolumn(:)    ! column of corresponding pft
   real(r8), pointer :: albgrd(:,:)   ! ground albedo (direct) (column-level)
   real(r8), pointer :: albgri(:,:)   ! ground albedo (diffuse)(column-level)
   real(r8), pointer :: t_veg(:)      ! vegetation temperature (Kelvin)
   real(r8), pointer :: fwet(:)       ! fraction of canopy that is wet (0 to 1)
   integer , pointer :: ivt(:)        ! pft vegetation type
   real(r8), pointer :: xl(:)         ! ecophys const - leaf/stem orientation index!! 
   real(r8), pointer :: albd(:,:)     ! surface albedo (direct)
   real(r8), pointer :: albi(:,:)     ! surface albedo (diffuse)
   real(r8), pointer :: fabd(:,:)   ! flux absorbed by veg per unit d
   real(r8), pointer :: fabi(:,:)
   real(r8), pointer :: ftdd(:,:)     ! down direct flux below veg per unit dir
   real(r8), pointer :: ftid(:,:)     ! down diffuse flux below veg per unit dir
   real(r8), pointer :: ftii(:,:)     ! down diffuse flux below veg per unit dif
   real  :: yy,yy1,yy2,yy3,yy4,yy5,yy6,yy7,yy8
   real  ::  ww,ww1,ww2,ww3,ww4
   real  :: ee,ee1,ee2,ee3,ee4,ee5,ee6,ag(4),u(4)
   real  :: xib,w,w1,w2,w3,t0,t1,u0,f0
   
   integer :: i,mbs
   real:: leafxl,angle,g1
   real:: bo1,bo2,to1,to2,abr1,abr2,GK,ps,ps1,G0
   real:: albgs,coszrs,xlsai
   real:: abo1,abo2
   real:: rhos,rhos1
   integer :: fp,p,c
   integer :: ib
   real::cosz
   real::asu
   real :: chil(lbp:ubp)
   real :: twostext(lbp:ubp)
   real :: omega,wrt1,wrt2
   real :: omegal
   real :: betai
   real :: betail
   real :: betad
   real :: betadl
   real :: phi1,phi2
   real :: coszi
   real :: fI1,fI2,fI3,fI4,fI5,fI6,fLa,fLb,f0a
   real :: fII1,fII2,fII3,fII4,fII5,fII6,FfLa,FfLb,Ff0a
   real :: s2,x1
   real :: fk1,fk2,z(4),CC(4,4),b(4,3),bb(4,4),a1(4,4),zz(4,2),aa(4,4,2),z1(4)
   Integer :: IS(4),JS(4),ie
   real :: G(4)
   real :: p0d(4),p1d(4),p2d(4),p3d(4)
   real :: p11d(4,4),p22d(4,4),p33d(4,4),A(4,4),CCC(4,4)
   real :: BBB(4,4)
   real :: bx(4,3),ax(2,2,2),dx(4),b1x,c1x,zx(4),aax(4,4,2),zzx(4,2),a1x(4,4),z1x(4)
   real(r8) :: tmp0,tmp2 ! temporary

     
   albgrd  => clm3%g%l%c%cps%albgrd
   albgri  => clm3%g%l%c%cps%albgri
   pcolumn => clm3%g%l%c%p%column
   fwet    => clm3%g%l%c%p%pps%fwet
   t_veg   => clm3%g%l%c%p%pes%t_veg
   ivt     => clm3%g%l%c%p%itype
   albd    => clm3%g%l%c%p%pps%albd
   albi    => clm3%g%l%c%p%pps%albi
   fabd    => clm3%g%l%c%p%pps%fabd
   fabi    => clm3%g%l%c%p%pps%fabi
   ftdd    => clm3%g%l%c%p%pps%ftdd
   ftid    => clm3%g%l%c%p%pps%ftid
   ftii    => clm3%g%l%c%p%pps%ftii
   xl      => pftcon%xl

    data ag / 0.3478548, 0.6521452, 0.6521452, 0.3478548 /
    data u / -0.8611363, -0.3399810, 0.3399810, 0.8611363 /
    data p0d /.100000E+01,.100000E+01,.100000E+01,.100000E+01 /
  
    data p1d / -.861136E+00, -.339981E+00,  .339981E+00, .861136E+00 /
    data p2d /  0.6123337,-0.3266194,-0.3266194,0.6123337 /
    data p3d /-0.3047470, 0.4117280,-0.4117280,0.3047470/      


     data p11d / 0.3707779,0.1463850,-0.1463850,-0.3707779, &
                 0.1463850,5.7793535E-02,-5.7793535E-02,-0.1463850, &
               -0.1463850,-5.7793535E-02,5.7793535E-02,0.1463850, &
               -0.3707779,-0.1463850,0.1463850,0.3707779 /
     data p22d /0.1874762,-0.1000000,-0.1000000,0.1874762, &
                -0.1000000,5.3340111E-02,5.3340111E-02,-0.1000000, &
               -0.1000000,5.3340111E-02,5.3340111E-02,-0.1000000, &
                0.1874762,-0.1000000,-0.1000000,0.1874762 /
     data p33d /4.6435371E-02,-6.2736437E-02,6.2736437E-02, &
     	-4.6435371E-02,  &
       -6.2736437E-02,8.4759958E-02,-8.4759958E-02,6.2736437E-02, &
       6.2736437E-02,-8.4759958E-02,8.4759958E-02,-6.2736437E-02, &
      -4.6435371E-02,6.2736437E-02,-6.2736437E-02,4.6435371E-02 /
          
        xib=2
        mbs=6

        do ib = 1, numrad
!dir$ concurrent
!cdir nodep
          do fp=1,num_vegsol
         p=filter_vegsol(fp)
          cosz = max(0.001, coszen(p))
        chil(p) =xl(ivt(p)) 
        chil(p) = min( max(xl(ivt(p)), -0.4), 0.6 )
        if (abs(chil(p)) <= 0.01) chil(p) = 0.01
!___________________________________________________________________
        phi1 = 0.5 - 0.633*chil(p) - 0.330*chil(p)*chil(p)
        phi2 = 0.877 * (1.-2.*phi1)
        gdir(p) = phi1 + phi2*cosz
         twostext(p) = gdir(p)/cosz
          leafxl=chil(p)

          c = pcolumn(p)
          omegal = rho(p,ib) + tau(p,ib)

         betail = 0.5 * ((rho(p,ib)+tau(p,ib)) + (rho(p,ib)-tau(p,ib)) &
          * ((1.+chil(p))/2.)**2) / omegal
          
          if (t_veg(p) > tfrz) then            !no snow
             tmp0 = omegal
             tmp2 = betail
          else
             tmp0 =   (1.-fwet(p))*omegal        + fwet(p)*omegas(ib)
             tmp2 = ( (1.-fwet(p))*omegal*betail + fwet(p)*omegas(ib)*betais ) / tmp0
          end if

          omegal = tmp0
          betail = tmp2

          rhos=albgrd(c,ib)
          rhos1=albgri(c,ib)

       s2 = exp(-twostext(p)*vai(p))
       u0=cosz  
       t0=0.0  
       t1=vai(p)   
       if(betail.eq.0.5.or.rho(p,ib).eq.tau(p,ib))then
       betail=0.499
       endif        
       g1=3.0-6.0*betail
       if(g1.eq.1.0)then
       g1=0.999
       endif
       if(g1.eq.-1.0)then
       g1=-0.999
       endif     
       w1=0    
       w2=2.5
       w3=0
       w=omegal
       PS=rhos 
       PS1=rhos1
       leafxl=chil(p)
       f0=1 
      if (leafxl.eq.0.0) then
          G0=0.5
      else if (leafxl.eq.1.0) then
          G0=abs(u0)
      else if (leafxl.eq.-1.) then
          G0=2.*sqrt(1-u0**2)/3.1415926
      else
          G0=0.5-0.633*leafxl-0.33*leafxl**2+&
            0.877*(1-2.0*(0.5-0.633*leafxl-0.33*leafxl**2))*abs(u0)
        endif
        GK=G0/u0   
      call coefft(u0,p0d,p1d,p2d,p3d,p11d,p22d,p33d,xib,w,w1,&
       w2,w3,t0,t1,f0,mbs,leafxl,ag,u,GK,PS,G0,bx,ax,dx,b1x&
       ,c1x,zx,aax,zzx,a1x,z1x,fk1,fk2)

       call conmatrix(A,CC,IS,JS,aax,zzx,a1x,z1x,fk1,fk2,BBB)
       call matrixp1(A,BB,CC,CCC,BBB,aax,zzx,a1x,z1x,fk1,fk2)
       call updown1(BBB,CCC,xib,w,w1,w2,w3,t0,t1,u0,f0,aax,zzx,a1x,&
        z1x,fk1,fk2,fI1,fI2,fI3,fI4,fI5,fI6,fLa,fLb,f0a,GK,ps,G0,x1,ag,u)
       call updown2(BBB,CCC,xib,w,w1,w2,w3,t0,t1,u0,f0,aax,zzx,a1x,z1x,&
        fk1,fk2,fII1,fII2,fII3,fII4,fII5,fII6,&
        FfLa,FfLb,Ff0a,GK,PS1,G0,ag,u)
       call albedo(fI1,fI2,fI3,fI4,fI5,fI6,fLa,fLb,f0a,fII1,fII2,&
        fII3,fII4,fII5,fII6,FfLa,FfLb,Ff0a,&
        GK,ps,G0,xib,w,w1,w2,w3,t0,t1,u0,f0,ag,u,bo1,bo2,to1,to2)

         s2=exp(-twostext(p)*vai(P))
         ftdd(p,ib)=s2
         ftid(p,ib)=to2
         albd(p,ib)=bo2
         ftdi(p,ib) = 0.
         ftii(p,ib)=to1
         albi(p,ib)=bo1
         fabd(p,ib) = 1. - albd(p,ib) &
            - (1.-albgrd(c,ib))*ftdd(p,ib) - (1.-albgri(c,ib))*ftid(p,ib)
         fabi(p,ib) = 1. - albi(p,ib) &
           - (1.-albgrd(c,ib))*ftdi(p,ib) - (1.-albgri(c,ib))*ftii(p,ib)
                
        enddo
		enddo
        end subroutine Fourstream
   
       ! **********************************************************************
! coefficient calculations for four first-order differential equations.
! **********************************************************************
       subroutine coeff1(u0,p0d,p1d,p2d,p3d,p11d,p22d,p33d,xib,w,w1,&
       w2,w3,t0,t1,f0,mbs,leafxl,ag,u,GK,PS,G0,b)
! include "para.file"
 
    implicit none 
      real:: p0d(4), p1d(4), p2d(4), p3d(4), p11d(4,4), p22d(4,4),&
       p33d(4,4), xib, w, w1, w2, w3, t0, t1,&
       u0, f0,b(4,3),GK,ps,G0,ag(4),u(4)
      real::leafxl
      integer::mbs
      real::c(4,5)
      real::G(4)
      real::w0w,w1w,w2w,w3w,fw,q1,q2,q3,fq,x
      integer::i,j
  
      do i=1,4
      if (leafxl.eq.0.) then
        G(i)=0.5
      else if (leafxl.eq.1.) then
        G(i)=abs(u(i))
      else if (leafxl.eq.-1.) then
        G(i)=2./(3.1415926*Sqrt(1.-u(i)**2))
      else
        G(i)=0.5-0.633*leafxl-0.33*leafxl**2+&
        0.877*(1-2.0*(0.5-0.633*leafxl-0.33*leafxl**2))*abs(u(i))
       endif
       enddo
         x =  w  !changed by zhouwenyan
         w0w = x
         w1w = x * w1
         w2w = x * w2
         w3w = x * w3
      if ( xib .le. mbs ) then
         fw =u0*u0    
         q1 = - w1w * u0
         q2 = w2w * ( 1.5 * fw - 0.5 )
         q3 = - w3w * ( 2.5 * fw - 1.5 ) * u0
         endif
         fq = 0.5 * w0w
      do 10 i = 1, 4    
         do 20 j = 1, 4
         c(i,j) =( fq + w1w * p11d(i,j) +&
         w2w * p22d(i,j) + w3w * p33d(i,j))*G(j)*ag(j)
      if ( i .eq. j ) then
         c(i,j) = ( c(i,j) - G(i) ) / u(i)
         else
         c(i,j) = c(i,j) / u(i)
      endif
20    continue
10    continue




!******************************************************
!     edited by zwy
!***********************************************************

          do 30 i = 1,4   !changed by zwy
           if ( xib .le. mbs ) then
              c(i,5) = ((w0w + q1 * p1d(i) + &
           q2  * p2d(i) + q3 * p3d(i)))*0.5*G0 !changed by zwy
	 
             else
               c(i,5) = 1.0
           endif
               c(i,5) = c(i,5) / u(i)
30           continue


!************************************************************
           b(1,1) = c(4,4) - c(4,1)
           b(1,2) = c(4,4) + c(4,1)
           b(2,1) = c(4,3) - c(4,2)
           b(2,2) = c(4,3) + c(4,2)
           b(3,1) = c(3,4) - c(3,1)
           b(3,2) = c(3,4) + c(3,1)
           b(4,1) = c(3,3) - c(3,2)
           b(4,2) = c(3,3) + c(3,2)
           b(1,3) = c(4,5) - c(1,5)
           b(2,3) = c(3,5) - c(2,5)
           b(3,3) = c(3,5) + c(2,5)
           b(4,3) = c(4,5) + c(1,5)
           return
           end subroutine coeff1
! **********************************************************************
! coefficient calculations for second order differential equations.
! **********************************************************************
           subroutine coeff2(xib,w,w1,w2,w3,t0,t1,u0,f0,b,a,d,mbs,GK,PS,G0)
 
       implicit none
       real:: xib, w, w1, w2, w3, t0, t1, u0, f0
       real:: b(4,3), a(2,2,2), d(4),GK,ps,G0
       integer::mbs
       real::fw1,fw2,fw3,fw4 
         fw1 = b(1,1) * b(1,2)
         fw2 = b(2,1) * b(3,2)
         fw3 = b(3,1) * b(2,2)
         fw4 = b(4,1) * b(4,2)
         a(2,2,1) = fw1 + fw2
         a(2,1,1) = b(1,1) * b(2,2) + b(2,1) * b(4,2)
         a(1,2,1) = b(3,1) * b(1,2) + b(4,1) * b(3,2)
         a(1,1,1) = fw3 + fw4
         a(2,2,2) = fw1 + fw3
         a(2,1,2) = b(1,2) * b(2,1) + b(2,2) * b(4,1)
         a(1,2,2) = b(3,2) * b(1,1) + b(4,2) * b(3,1)
         a(1,1,2) = fw2 + fw4
         d(1) = b(3,2) * b(4,3) + b(4,2) * b(3,3) + b(2,3)*G0 / u0  !2013
         d(2) = b(1,2) * b(4,3) + b(2,2) * b(3,3) + b(1,3)*G0  / u0   !2013
         d(3) = b(3,1) * b(1,3) + b(4,1) * b(2,3) + b(3,3)*G0  / u0   !2013
         d(4) = b(1,1) * b(1,3) + b(2,1) * b(2,3) + b(4,3)*G0  / u0   !2013
         return
         end subroutine coeff2


! **********************************************************************
       subroutine coeff4(xib,w,w1,w2,w3,t0,t1,u0,f0,b,a,d,b1,c1,z,&
     aa,zz,a1,z1,fk1,fk2,mbs,GK,PS,G0)
 
       implicit none      
       real:: xib, w, w1, w2, w3, t0, t1, u0, f0
       real::a(2,2,2), d(4), b1, c1, z(4),GK,ps,G0
       real::x,a1(4,4),zz(4,2),aa(4,4,2),fk1,fk2,z1(4)    
       real::b(4,3)
       integer::mbs  
 
          x = U0**2/(G0**2)
          b1 = a(2,2,1) + a(1,1,1)
          c1 = a(2,1,1) * a(1,2,1) - a(1,1,1) * a(2,2,1)
          z(1) = a(2,1,1) * d(3) + d(4) / x - a(1,1,1) * d(4)
          z(2) = a(1,2,1) * d(4) - a(2,2,1) *d(3) + d(3) / x
          z(3) = a(2,1,2) * d(1) + d(2) / x - a(1,1,2) * d(2)
          z(4) = a(1,2,2) * d(2) - a(2,2,2) * d(1) + d(1) / x
          return
          end subroutine coeff4


! **********************************************************************
! fk1 and fk2 are the eigenvalues.
! **********************************************************************
         subroutine coeffl(xib,w,w1,w2,w3,t0,t1,u0,f0,b,a,&
        d,b1,c1,z,aa,zz,a1,z1,fk1,fk2,mbs,GK,PS,G0)
!### include "para.file"
 
       implicit none
       real::b(4,3),a(2,2,2),d(4),b1,c1,z(4)
       real::aa(4,4,2), zz(4,2), a1(4,4), z1(4), fk1, fk2
       real::GK,ps,G0
       integer::mbs    
       real::u0,f0,t1,w,w1,w2,t0,w3,xib
       real::a2,b2   
       real::dt,x,fw,fw1,fw2,y,zx,fq0,fq1
       integer::i,j,l
          dt = t1 - t0
          x = sqrt ( b1 * b1 + 4.0 * c1 )
          fk1 = sqrt ( ( b1 + x ) * 0.5 )
          fk2 = sqrt ( ( b1 - x ) * 0.5 )
          fw =U0**2/(G0**2)
          x = 1.0 / ( fw * fw ) - b1 / fw - c1

         if(abs(x).lt.1.0e-16)then
         if(x.lt.0.0)then
         x=-1.0e-6
         else
         x=1.0e-6
         endif
         endif

          fw = 0.5 * f0 / x
          z(1) = fw * z(1)
          z(2) = fw * z(2)
          z(3) = fw * z(3)
          z(4) = fw * z(4)
          z1(1) = 0.5 * ( z(1) + z(3) )
          z1(2) = 0.5 * ( z(2) + z(4) )
          z1(3) = 0.5 * ( z(2) - z(4) )
          z1(4) = 0.5 * ( z(1) - z(3) )
          a2 = ( fk1 * fk1 - a(2,2,1) ) / a(2,1,1)
          b2 = ( fk2 * fk2 - a(2,2,1) ) / a(2,1,1)
          x = b(1,1) * b(4,1) - b(3,1) * b(2,1)
          fw1 = fk1 / x
          fw2 = fk2 / x
          y = fw2 * ( b2 * b(2,1) - b(4,1) )
          zx = fw1 * ( a2 * b(2,1) - b(4,1) )
          a1(1,1) = 0.5 * ( 1 - y )
          a1(1,2) = 0.5 * ( 1 - zx )
          a1(1,3) = 0.5 * ( 1 + zx )
          a1(1,4) = 0.5 * ( 1 + y )
          y = fw2 * ( b(3,1) - b2 * b(1,1) )
          zx = fw1 * ( b(3,1) - a2 * b(1,1) )
          a1(2,1) = 0.5 * ( b2 - y )
          a1(2,2) = 0.5 * ( a2 - zx )
          a1(2,3) = 0.5 * ( a2 + zx )
          a1(2,4) = 0.5 * ( b2 + y )
          a1(3,1) = a1(2,4)
          a1(3,2) = a1(2,3)
          a1(3,3) = a1(2,2)
          a1(3,4) = a1(2,1)
          a1(4,1) = a1(1,4)
          a1(4,2) = a1(1,3)
          a1(4,3) = a1(1,2)
          a1(4,4) = a1(1,1)
          if ( xib .le. mbs ) then
            fq0 = exp ( - t0*G0 / u0 ) !changed by zwy  2013
            fq1 = exp (  -t1*G0 / u0 )  !changed by zwy  2013
          else
            fq0 = 1.0
            fq1 = exp ( - dt*G0 / u0 )  !2013
          endif
            x = exp ( - fk1 * t1 )  !changed by zwy
            y = exp ( - fk2 * t1 ) !changed by zwy
          do 40 i = 1, 4
            zz(i,1) = z1(i) * fq0
            zz(i,2) = z1(i) * fq1

            aa(i,1,1) = a1(i,1)
            aa(i,2,1) = a1(i,2)
            aa(i,3,1) = a1(i,3) * x
            aa(i,4,1) = a1(i,4) * y
            aa(i,3,2) = a1(i,3)
            aa(i,4,2) = a1(i,4)
            aa(i,1,2) = a1(i,1) * y
            aa(i,2,2) = a1(i,2) * x
40          continue
       
            return
            end subroutine coeffl

! **********************************************************************
! See the paper by Liou, Fu and Ackerman (1988) for the formulation of
! the delta-four-stream approximation in a homogeneous layer.
! **********************************************************************
          subroutine coefft(u0,p0d,p1d,p2d,p3d,p11d,p22d,p33d,xib,w,w1,&
          w2,w3,t0,t1,f0,mbs,leafxl,ag,u,GK,PS,G0,b,a,d,b1&
          ,c1,z,aa,zz,a1,z1,fk1,fk2)

      implicit none 
      real::u0,p0d(4),p1d(4),p2d(4),p3d(4),xib,w,w1,w2,w3,t0,t1,f0
      integer::mbs
      real::GK,PS,G0
      real::p11d(4,4),p22d(4,4),p33d(4,4)
      real::b(4,3),leafxl,ag(4),u(4),a(2,2,2),d(4),b1,c1
      real::z(4),aa(4,4,2),zz(4,2),a1(4,4),z1(4),fk1,fk2 
      call coeff1(u0,p0d,p1d,p2d,p3d,p11d,p22d,p33d,&
          xib,w,w1,w2,w3,t0,t1,f0,&
          mbs,leafxl,ag,u,GK,PS,G0,b)
      call coeff2(xib,w,w1,w2,w3,t0,t1,u0,f0,b,a,d,mbs,GK,PS,G0)

      call coeff4(xib,w,w1,w2,w3,t0,t1,u0,f0,b,a,d,b1,c1,z,aa,&
          zz,a1,z1,fk1,fk2,mbs,GK,PS,G0)
      call coeffl(xib,w,w1,w2,w3,t0,t1,u0,f0,b,a,d,b1,c1,z,&
          aa,zz,a1,z1,fk1,fk2,mbs,GK,PS,G0)
       return
       end subroutine coefft


!     calculate contrary matrix

       subroutine conmatrix(A,C,IS,JS,aa,zz,a1,z1,fk1,fk2,BB)
    
      implicit none 
      real:: aa(4,4,2), zz(4,2), a1(4,4), z1(4), fk1, fk2,BB(4,4)
      real::GK,PS,G0 
      integer::IS(4),JS(4)
      real::A(4,4),C(4,4),ag(4),u(4)
      real::u0,p0d(4),p1d(4),p2d(4),p3d(4),xib,w,w1, w2,w3
      real::p11d(4,4),p22d(4,4),p33d(4,4)
      real::t0,t1,f0,leafxl,b(4,3)
      integer::i,j,l,mbs             
      call coeff1(u0,p0d,p1d,p2d,p3d,p11d,p22d,p33d,xib,w,w1,&
        w2,w3,t0,t1,f0,mbs,leafxl,ag,u,GK,PS,G0,b)

      do j=1,4
         A(1,j)=aa(1,j,2)
         A(2,j)=aa(2,j,2)

         A(3,j)=aa(3,j,1)
         A(4,j)=aa(4,j,1)

      enddo
        CALL BRINV(A,4,L,IS,JS)
        IF(L.NE.0)THEN

        END IF
        DO 5 I=1,4
        DO 5 J=1,4
5        BB(I,J)=A(I,J)
        END subroutine conmatrix

!     calculate real contrary matrix
         subroutine BRINV(A,N,L,IS,JS)
        implicit none 
        real:: A(N,N)
        integer::IS(N),JS(N)
        real::D,T
        integer::i,j,k,m,n,l
         L=1
         DO 100 K=1,N
         D=0.0
         DO 10 I=K,N
         DO 10 J=K,N
         IF(ABS(A(I,J)).GT.D)THEN
         D=ABS(A(I,J))
         IS(K)=I
         JS(K)=J
         END IF
10       CONTINUE
         IF(D+1.0.EQ.1.0)THEN
         L=0
         RETURN
          END IF
20       FORMAT(1X,'ERR**NOT INV')
         DO 30 J=1,N
         T=A(K,J)
         A(K,J)=A(IS(K),J)
          A(IS(K),J)=T
30        CONTINUE
          DO 40 I=1,N
          T=A(I,K)
          A(I,K)=A(I,JS(K))
           A(I,JS(K))=T
40         CONTINUE
          A(K,K)=1/A(K,K)
          DO 50 J=1,N
          IF(J.NE.K)THEN
           A(K,J)=A(K,J)*A(K,K)
           END IF
50         CONTINUE
           DO 70 I=1,N
           IF(I.NE.K)THEN
           DO 60 J=1,N
           IF(J.NE.K)THEN
           A(I,J)=A(I,J)-A(I,K)*A(K,J)
           END IF
60          CONTINUE
            END IF
70           CONTINUE

           DO 80 I=1,N
           IF(I.NE.K)THEN
            A(I,K)=-A(I,K)*A(K,K)
            END IF
80          CONTINUE
100         CONTINUE
            DO 130 K=N,1,-1
            DO 110 J=1,N
            T=A(K,J)
             A(K,J)=A(JS(K),J)
             A(JS(K),J)=T
110          CONTINUE
              DO 120 I=1,N
              T=A(I,K)
               A(I,K)=A(I,IS(K))
              A(I,IS(K))=T
120           CONTINUE
130           CONTINUE
   RETURN

END subroutine BRINV

!     matrix multiply
!     #calculate matrix product
            subroutine matrixp1(A,B,C,CC,BB,aa,zz,a1,z1,fk1,fk2)
 
        implicit none
 
          real:: CC(4,4), BB(4,4), aa(4,4,2), zz(4,2), a1(4,4),&
               z1(4), fk1, fk2
         integer::i,j
         real::A(4,4),B(4,4),C(4,4)
             do j=1,4
             A(1,j)=aa(1,j,2)
             A(2,j)=aa(2,j,2)
             A(3,j)=aa(3,j,2)
             A(4,j)=aa(4,j,2)
            enddo

           do i=1,4
           do j=1,4
           B(i,j)=BB(i,j)
          enddo;enddo

           CALL BRMUL(A,B,4,4,4,C)
           do i=1,4
           do j=1,4

           CC(i,j)=C(i,j)
          enddo;enddo
          END subroutine matrixp1

          SUBROUTINE BRMUL(A,B,M,N,K,C)
          real:: A(M,N),B(N,K),C(M,K)
          integer::i,j,m,k,n,l
          DO 50 I=1,M
          DO 50 J=1,K
          C(I,J)=0.0
          DO 10 L=1,N
          C(I,J)=C(I,J)+A(I,L)*B(L,J)
10        CONTINUE
50        CONTINUE
          RETURN
          END SUBROUTINE BRMUL


!    #calculate radiative upward-downward fluxes-Edited by zhouwy-direct
       subroutine updown1(BB,CC,xib,w,w1,w2,w3,t0,t1,u0,f0,aa,zz,&
       a1,z1,fk1,fk2,fI1,fI2,fI3,fI4,&
      fI5,fI6,fLa,fLb,f0a,GK,ps,G0,x1,ag,u)
    
     implicit none
       real:: fI1,fI2,fI3,fI4,fI5,fI6,fLa,fLb,f0a 
       real:: BB(4,4), CC(4,4), xib, w, w1, w2, w3, t0, t1, u0, f0,&
            aa(4,4,2), zz(4,2), a1(4,4), z1(4), fk1, fk2,ag(4),u(4)
       real::ps,GK,x1,G0   
       real::x0,f1,f2,f3,f4,g1,g2,g3,g4,au1,au2,au3,xx1,xx2
       real::fJ1,fJ2,fJ3,fJ4,fJX1,fJX2,fJX3,fJX4,fJY1,fJY2,fJY3,fJY4
       real::fJZ1,fJZ2,fJZ3,fJZ4,fI7,fI8
            x1=3.1415927*u0*exp(-GK*t1*1.0)
            x0=exp(-GK*t0*1.0)

            f1=CC(3,1)*(-zz(1,2))
            f2=CC(3,2)*(-zz(2,2))
            f3=CC(3,3)*(-zz(3,1))
	        f4=CC(3,4)*(-zz(4,1))

            g1=CC(4,1)*(-zz(1,2))
            g2=CC(4,2)*(-zz(2,2))
            g3=CC(4,3)*(-zz(3,1))
	        g4=CC(4,4)*(-zz(4,1))
            au1=ag(2)*u(3)
            au2=ag(1)*u(4)
            au3=au1+au2

        xx1=1-cc(4,1)*ps*au2/au3-cc(4,2)*ps*au2/au3
        xx2=1-(cc(3,1)*ps*au1/au3+cc(3,2)*ps*au1/au3)&
          -(cc(3,1)*ps*au2/au3+cc(3,2)*ps*au2/au3)&
          *(cc(4,1)*ps*au1/au3+cc(4,2)*ps*au1/au3)/xx1

        fI3=(f1+f2+f3+f4+cc(3,1)*ps*X1/(2.0*3.1415927*au3)  &
          +cc(3,2)*ps*X1/(2.0*3.1415927*au3) &
		  +(cc(3,1)*ps*au2/au3+cc(3,2)*ps*au2/au3)&
          *(g1+g2+g3+g4+cc(4,1)*ps*X1/(2.0*3.1415927*au3)& 
		  +cc(4,2)*ps*X1/(2.0*3.1415927*au3)+zz(4,2))/xx1+zz(3,2))/xx2  !downward radiative fluxes

        fI4=(g1+g2+g3+g4+cc(4,1)*ps*(2.0*3.1415927*au1*abs(fI3)+X1)  &
           /(2.0*3.1415927*au3)&
	       +cc(4,2)*ps*(2.0*3.1415927*au1*abs(fI3)+X1)&
           /(2.0*3.1415927*au3)+zz(4,2))/xx1

        fI2=PS*(2*3.1415927*(au1*ABS(fI3)+au2*ABS(fI4))+X1)&
           /(2.0*3.1415927*au3)
	    fI1=PS*(2*3.1415927*(au1*ABS(fI3)+au2*ABS(fI4))+X1)&
           /(2.0*3.1415927*au3)

         fI1=PS*(2*3.1415927*(au1*ABS(fI3)+au2*ABS(fI4))+X1)&
           /(2.0*3.1415927*au3)



        fJ1=aa(1,1,1)*BB(1,1)+aa(1,2,1)*BB(2,1)&
          +aa(1,3,1)*BB(3,1)+aa(1,4,1)*BB(4,1)
        fJ2=aa(1,1,1)*BB(1,2)+aa(1,2,1)*BB(2,2)&
          +aa(1,3,1)*BB(3,2)+aa(1,4,1)*BB(4,2)
        fJ3=aa(1,1,1)*BB(1,3)+aa(1,2,1)*BB(2,3)&
          +aa(1,3,1)*BB(3,3)+aa(1,4,1)*BB(4,3)
        fJ4=aa(1,1,1)*BB(1,4)+aa(1,2,1)*BB(2,4)&
          +aa(1,3,1)*BB(3,4)+aa(1,4,1)*BB(4,4)
        fJX1=aa(2,1,1)*BB(1,1)+aa(2,2,1)*BB(2,1)&
          +aa(2,3,1)*BB(3,1)+aa(2,4,1)*BB(4,1)
        fJX2=aa(2,1,1)*BB(1,2)+aa(2,2,1)*BB(2,2)&
          +aa(2,3,1)*BB(3,2)+aa(2,4,1)*BB(4,2)
        fJX3=aa(2,1,1)*BB(1,3)+aa(2,2,1)*BB(2,3)&
          +aa(2,3,1)*BB(3,3)+aa(2,4,1)*BB(4,3)
        fJX4=aa(2,1,1)*BB(1,4)+aa(2,2,1)*BB(2,4)&
          +aa(2,3,1)*BB(3,4)+aa(2,4,1)*BB(4,4)

        fJY1=aa(3,1,1)*BB(1,1)+aa(3,2,1)*BB(2,1)&
          +aa(3,3,1)*BB(3,1)+aa(3,4,1)*BB(4,1)
        fJY2=aa(3,1,1)*BB(1,2)+aa(3,2,1)*BB(2,2)&
          +aa(3,3,1)*BB(3,2)+aa(3,4,1)*BB(4,2)
        fJY3=aa(3,1,1)*BB(1,3)+aa(3,2,1)*BB(2,3)&
          +aa(3,3,1)*BB(3,3)+aa(3,4,1)*BB(4,3)
        fJY4=aa(3,1,1)*BB(1,4)+aa(3,2,1)*BB(2,4)&
          +aa(3,3,1)*BB(3,4)+aa(3,4,1)*BB(4,4)

        fJZ1=aa(4,1,1)*BB(1,1)+aa(4,2,1)*BB(2,1)&
          +aa(4,3,1)*BB(3,1)+aa(4,4,1)*BB(4,1)
        fJZ2=aa(4,1,1)*BB(1,2)+aa(4,2,1)*BB(2,2)&
          +aa(4,3,1)*BB(3,2)+aa(4,4,1)*BB(4,2)
        fJZ3=aa(4,1,1)*BB(1,3)+aa(4,2,1)*BB(2,3)&
          +aa(4,3,1)*BB(3,3)+aa(4,4,1)*BB(4,3)
        fJZ4=aa(4,1,1)*BB(1,4)+aa(4,2,1)*BB(2,4)&
          +aa(4,3,1)*BB(3,4)+aa(4,4,1)*BB(4,4)


     fI5=fJX1*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
     au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(1,2))&
     +fJX2*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
     au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(2,2))&
     +fJX3*(-zz(3,1))+fJX4*(-zz(4,1))+zz(2,1)

      fI6=fJ1*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
      au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(1,2))&
      +fJ2*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
      au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(2,2))&
      +fJ3*(-zz(3,1))+fJ4*(-zz(4,1))+zz(1,1)
            

      fI7=fJY1*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
      au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(1,2))&
      +fJY2*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
      au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(2,2))&
      +fJY3*(-zz(3,1))+fJY4*(-zz(4,1))+zz(3,1)


      fI8=fJZ1*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
      au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(1,2))&
      +fJZ2*((ps*(2.0*3.1415927*(au1*abs(fI3)+&
      au2*abs(fI4))+X1)/(2.0*3.1415927*au3))-zz(2,2))&
      +fJZ3*(-zz(3,1))+fJZ4*(-zz(4,1))+zz(4,1)


        fLa=2.0*3.1415927*(au1*abs(fI1)+au2*abs(fI2))
	    fLb=2.0*3.1415927*(au1*abs(fI3)+au2*abs(fI4))

        f0a=2.0*3.1415927*(au1*abs(fI5)+au2*abs(fI6))

          end subroutine updown1

!   #calculate radiative upward-downward fluxes-Edited by zhouwy-diffuse

         subroutine updown2(BB,CC,xib,w,w1,w2,w3,t0,t1,u0,f0,aa,&
       zz,a1,z1,fk1,fk2,fII1,fII2,fII3,fII4,fII5,fII6,&
        FfLa,FfLb,Ff0a,GK,ps1,G0,ag,u)

       implicit none
       real:: fII1,fII2,fII3,fII4,fII5,fII6,FfLa,FfLb,Ff0a
       real::BB(4,4), CC(4,4), xib, w, w1, w2, w3, t0, t1, u0, f0,&
           aa(4,4,2), zz(4,2), a1(4,4), z1(4), fk1, fk2,GK,ps,ps1,G0,ag(4),u(4)
       real::x0,au1,au2,au3,f1,f2,f3,f4,g1,g2,g3,g4,xx1,xx2
       real::fJ1,fJ2,fJ3,fJ4,fJX1,fJX2,fJX3,fJX4,fJY1,fJY2,fJY3,fJY4
       REAL::fJZ1,fJZ2,fJZ3,fJZ4,fII7,fII8
       integer::i,j
	      x0=3.1415927*exp(-GK*t0*1.0)
          au1=ag(2)*u(3)
          au2=ag(1)*u(4)
          au3=au1+au2
          f1=CC(3,1)
          f2=CC(3,2)
          f3=CC(3,3)
          f4=CC(3,4)
          g1=CC(4,1)
          g2=CC(4,2)
          g3=CC(4,3)
          g4=CC(4,4)

        xx1=1-cc(4,1)*ps1*au2/au3-cc(4,2)*ps1*au2/au3
        xx2=1-(cc(3,1)*ps1*au1/au3+cc(3,2)*ps1*au1/au3)&
            -(cc(3,1)*ps1*au2/au3+cc(3,2)*ps1*au2/au3)&
            *(cc(4,1)*ps1*au1/au3+cc(4,2)*ps1*au1/au3)/xx1

        fII3=(f3+f4+(cc(3,1)*ps1*au2/au3+cc(3,2)*ps1*au2/au3)&
            *(g3+g4)/xx1)/xx2 !downward radiative fluxes
        fII4=(g3+g4+CC(4,1)*ps1*au1*abs(fII3)/au3&
            +cc(4,2)*ps1*au1*abs(fII3)/au3)/xx1

        fII2=PS1*(au1*ABS(fII3)+au2*ABS(fII4))/au3
        fII1=PS1*(au1*ABS(fII3)+au2*ABS(fII4))/au3

        fJ1=aa(1,1,1)*BB(1,1)+aa(1,2,1)*BB(2,1)&
             +aa(1,3,1)*BB(3,1)+aa(1,4,1)*BB(4,1)
        fJ2=aa(1,1,1)*BB(1,2)+aa(1,2,1)*BB(2,2)&
            +aa(1,3,1)*BB(3,2)+aa(1,4,1)*BB(4,2)
        fJ3=aa(1,1,1)*BB(1,3)+aa(1,2,1)*BB(2,3)&
            +aa(1,3,1)*BB(3,3)+aa(1,4,1)*BB(4,3)
        fJ4=aa(1,1,1)*BB(1,4)+aa(1,2,1)*BB(2,4)&
             +aa(1,3,1)*BB(3,4)+aa(1,4,1)*BB(4,4)

        fJX1=aa(2,1,1)*BB(1,1)+aa(2,2,1)*BB(2,1)&
             +aa(2,3,1)*BB(3,1)+aa(2,4,1)*BB(4,1)
        fJX2=aa(2,1,1)*BB(1,2)+aa(2,2,1)*BB(2,2)&
            +aa(2,3,1)*BB(3,2)+aa(2,4,1)*BB(4,2)
        fJX3=aa(2,1,1)*BB(1,3)+aa(2,2,1)*BB(2,3)&
            +aa(2,3,1)*BB(3,3)+aa(2,4,1)*BB(4,3)
        fJX4=aa(2,1,1)*BB(1,4)+aa(2,2,1)*BB(2,4)&
             +aa(2,3,1)*BB(3,4)+aa(2,4,1)*BB(4,4)


        fJY1=aa(3,1,1)*BB(1,1)+aa(3,2,1)*BB(2,1)&
             +aa(3,3,1)*BB(3,1)+aa(3,4,1)*BB(4,1)
        fJY2=aa(3,1,1)*BB(1,2)+aa(3,2,1)*BB(2,2)&
             +aa(3,3,1)*BB(3,2)+aa(3,4,1)*BB(4,2)
        fJY3=aa(3,1,1)*BB(1,3)+aa(3,2,1)*BB(2,3)&
             +aa(3,3,1)*BB(3,3)+aa(3,4,1)*BB(4,3)
        fJY4=aa(3,1,1)*BB(1,4)+aa(3,2,1)*BB(2,4)&
             +aa(3,3,1)*BB(3,4)+aa(3,4,1)*BB(4,4)
        fJZ1=aa(4,1,1)*BB(1,1)+aa(4,2,1)*BB(2,1)&
             +aa(4,3,1)*BB(3,1)+aa(4,4,1)*BB(4,1)
        fJZ2=aa(4,1,1)*BB(1,2)+aa(4,2,1)*BB(2,2)&
            +aa(4,3,1)*BB(3,2)+aa(4,4,1)*BB(4,2)
        fJZ3=aa(4,1,1)*BB(1,3)+aa(4,2,1)*BB(2,3)&
             +aa(4,3,1)*BB(3,3)+aa(4,4,1)*BB(4,3)
        fJZ4=aa(4,1,1)*BB(1,4)+aa(4,2,1)*BB(2,4)&
            +aa(4,3,1)*BB(3,4)+aa(4,4,1)*BB(4,4)

!******************************************
        fII5=fJX1*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJX2*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJX3+fJX4

        fII6=fJ1*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJ2*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJ3+fJ4

        fII7=fJY1*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJY2*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJY3+fJY4
        fII8=fJZ1*PS1*(au1*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJZ2*PS1*(au2*abs(fII3)&
              +au2*abs(fII4))/au3&
              +fJZ3+fJZ4

         FfLa=2.0*3.1415927*(ag(2)*u(3)*abs(fII1)+ag(1)*u(4)*abs(fII2))
         FfLb=2.0*3.1415927*(ag(2)*u(3)*abs(fII3)+ag(1)*u(4)*abs(fII4))

         Ff0a=2.0*3.1415927*(au1*abs(fII5)+au2*abs(fII6))
          end subroutine updown2

!     #calculate diffuse solar albedo: Edited by zhouwy
         subroutine albedo(fI1,fI2,fI3,fI4,fI5,fI6,fLa,fLb,&
          f0a,fII1,fII2,fII3,fII4,fII5,fII6,FfLa,FfLb,&
          Ff0a,GK,ps,G0,xib,w,w1,w2,w3,t0,t1,u0,f0,ag,u,bo1,bo2,to1,to2)
         REAL:: f0a2
 
      real:: fI1,fI2,fI3,fI4,fI5,fI6,fLa,fLb,f0a,&
          fII1,fII2,fII3,fII4,fII5,fII6,FfLa,FfLb,Ff0a,&
          GK,ps,G0,xib, w, w1, w2, w3, t0, t1, u0, f0,ag(4),u(4)
      real::fLa2,fLb2,bo1,bo2,to1,to2,abr1,abr2
      integer::i,j
      
          fLa2=fLa+FfLa
          fLb2=fLb+FfLb
          bo1=Ff0a/(2.0*3.1415927*(ag(2)*u(3)+ag(1)*u(4))) !diffuse
          bo2=f0a/(3.1415927*u0)
          to1=FfLb/(2.0*3.1415927*(ag(2)*u(3)+ag(1)*u(4)))
          to2=fLb/(3.1415927*u0) 
!          abr1=(2.0*3.1415927*(ag(2)*u(3)+ag(1)*u(4))-(Ff0a+FfLb))/&
!          (2.0*3.1415927*(ag(2)*u(3)+ag(1)*u(4)))       
!          abr2=(3.1415927*u0-(f0a+flb))/(3.1415927*u0)
          abr1=1-bo1-to1*(1-ps)
          abr2=1-bo2-(1-ps)*exp(-GK*t1*1.0)-(1-ps)*to2
!zhouwy   
        
     end subroutine albedo
!-----------------------------------------------------------------------
#endif
    end module SurfaceAlbedoMod
