#include <misc.h>
#include <preproc.h>

module clm_csmMod

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Set of routines that define communication between the land model
! and the flux coupler. The order of sends/receives is as follows:
!  - receive orbital data from coupler (csm_recvorb)
!  - send control data (grids and masks) to coupler (csm_sendcontrol)
!    land grid does not have valid data, runoff grid does 
!  - receive valid land grid from flux coupler (csm_recvgrid)
!  - send compressed runoff information to flux coupler (csm_sendrunoff)
!  - send first land model data to flux coupler (csm_send_alb)
!  - start normal send/recv communication pattern   
!      => csm_dosndrcv
!      => csm_recv
!      => csm_flxave
!      => csm_send   
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------
! $Id: clm_csmMod.F90,v 1.11.4.9.2.3 2002/07/24 22:10:52 mvertens Exp $
!-----------------------------------------------------------------------

#if (defined COUP_CSM)

  USE shr_kind_mod, only: r8 => shr_kind_r8
  USE infnan
!xjx  USE mpishorthand
  USE clm_varpar         !parameters
  USE shr_msg_mod        !csm_share message passing routines

!==================wangln 2007-07-22==================================
#if (defined SPMD)
  USE spmdMod        , only : masterproc, mpicom     !spmd routines and variables
  USE spmdGathScatMod, only : gather_data_to_master, scatter_data_from_master
!xjx  use mpishorthand, only : mpir8  !------wangln 2007-07-23-------------
#else
!xjx-----------20090821
  USE mpishorthand
!xjx-----------20090821 end
  use spmdMod        , only : masterproc
#endif
!xjx  use mpiinc
  use RunoffMod        , only : runoff
  use abortutils       , only : endrun
  use shr_sys_mod      , only : shr_sys_flush
!==================wangln 2007-07-22==================================

  USE shr_sys_mod, only: shr_sys_irtc !csm_share system utility routines
  

  implicit none 

! Buffer information 

  integer, parameter :: nibuff = 100    ! cpl ->atm msg, initial
  integer, parameter :: ncbuff_max=2000 ! Max size of character data from cpl

  integer   :: ncbuff             !Size of character data from cpl
  integer   :: ibuffr(nibuff)     !Integer buffer from cpl
  integer   :: ibuffs(nibuff)     !Integer buffer to   cpl
  real(r8)  :: rbuff(nibuff)      !Floating pt buffer from cpl
  character :: cbuff(ncbuff_max)  !Character data recieved from cpl

! Timing information 

  logical :: csm_timing
  integer, public :: csm_dtime   ! value passed to coupler on initialization set to namelist input
  integer :: irtc_w               !rc ticks when waiting for msg
  integer :: irtc_r               !rtc tics when msg recved
  integer :: irtc_s               !rtc tics when msg sent

! Send/recv buffers 
!!------------------wangln 2008-09-03 add the carbon flux-----------------
  !!integer, parameter :: nsnd = 15
  !!integer, parameter :: nrcv = 16

  !!real(r8) :: send2d(lsmlon,lsmlat,nsnd)  !2d send buffer
  !!real(r8) :: recv2d(lsmlon,lsmlat,nrcv)  !2d recv buffer

  integer  :: nsnd 
  integer  :: nrcv 
  logical  :: carbon_switch

  real(r8), allocatable :: send2d(:,:,:)  !2d send buffer
  real(r8), allocatable :: recv2d(:,:,:)  !2d recv buffer
!!------------------wangln 2008-09-03 add the carbon flux-----------------
  
  real(r8), private, pointer :: send1d(:,:)    !1d send buffer
  real(r8), private, pointer :: gather1d(:,:)  
  real(r8), private, pointer :: recv1d(:,:)    !1d recv buffer
  real(r8), private, pointer :: scatter1d(:,:)

!===============wangln 2007-07-21==============================
  real(r8), private, pointer ::  loc_roff(:)
  real(r8), private, pointer :: glob_roff(:)
  
  real(r8), allocatable      :: ocnrof_vec(:)
!===============wangln 2007-07-21==============================


  logical :: debug_flag   !received from coupler

! Flux averaging arrays and counters

  integer  :: icnt  !step counter for flux averager 
  integer  :: ncnt  !number of steps over which to average output fluxes 
  real(r8) :: rncnt !reciprocal of ncnt

  real(r8), allocatable :: taux_ave(:)     !averaged array
  real(r8), allocatable :: tauy_ave(:)     !averaged array
  real(r8), allocatable :: lhflx_ave(:)    !averaged array
  real(r8), allocatable :: shflx_ave(:)    !averaged array
  real(r8), allocatable :: lwup_ave(:)     !averaged array
  real(r8), allocatable :: qflx_ave(:)     !averaged array
  real(r8), allocatable :: swabs_ave(:)    !averaged array
!-------------------------------wangln 2008-09-03 add carbon flux--------------------------
  real(r8), allocatable :: carbon_ave(:)   !averaged array
!-------------------------------wangln 2008-09-03 add carbon flux--------------------------
#ifdef MOZART2
  real(r8), allocatable :: flx_mss_vrt_dst01_ave(:)     !averaged array
  real(r8), allocatable :: flx_mss_vrt_dst02_ave(:)     !averaged array
  real(r8), allocatable :: flx_mss_vrt_dst03_ave(:)     !averaged array
  real(r8), allocatable :: flx_mss_vrt_dst04_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_dst01_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_dst02_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_dst03_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_dst04_ave(:)     !averaged array
  real(r8), allocatable :: lnd_ram1_ave(:)      !averaged array
  real(r8), allocatable :: lnd_fv_ave(:)        !averaged array
  real(r8), allocatable :: vlc_trb_SO4_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_NH4_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_CB2_ave(:)     !averaged array
  real(r8), allocatable :: vlc_trb_SO2_ave(:)     !averaged array

  real(r8), allocatable :: flux_ISOP_ave(:)     !averaged array
  real(r8), allocatable :: flux_ACET_ave(:)     !averaged array
  real(r8), allocatable :: flux_C3H6_ave(:)     !averaged array
  real(r8), allocatable :: flux_C2H4_ave(:)     !averaged array
  real(r8), allocatable :: flux_OC2_ave(:)       !averaged array
  real(r8), allocatable :: flux_C10H16_ave(:)     !averaged array
#endif
! When to send/receive messages to coupler and when to make restart and stop

  integer :: ncpday         !number of send/recv calls per day
  logical :: dorecv         !receive data from coupler this step
  logical :: dosend         !send data to coupler this step
  logical :: csmstop_next   !received stop at eod signal and will stop on next ts
  logical :: csmstop_now    !received stop now signal from coupler
  logical :: csmrstrt       !restart write signal received from coupler

! Indices for send/recv fields

  integer, parameter :: irecv_hgt    = 1    !zgcmxy       Atm state m    
  integer, parameter :: irecv_u      = 2    !forc_uxy     Atm state m/s  
  integer, parameter :: irecv_v      = 3    !forc_vxy     Atm state m/s  
  integer, parameter :: irecv_th     = 4    !forc_thxy    Atm state K    
  integer, parameter :: irecv_q      = 5    !forc_qxy     Atm state kg/kg
  integer, parameter :: irecv_pbot   = 6    !ptcmxy       Atm state Pa   
  integer, parameter :: irecv_t      = 7    !forc_txy     Atm state K    
  integer, parameter :: irecv_lwrad  = 8    !flwdsxy      Atm flux  W/m^2
  integer, parameter :: irecv_rainc  = 9    !rainxy       Atm flux  mm/s 
  integer, parameter :: irecv_rainl  = 10   !rainxy       Atm flux  mm/s 
  integer, parameter :: irecv_snowc  = 11   !snowfxy      Atm flux  mm/s 
  integer, parameter :: irecv_snowl  = 12   !snowfxl      Atm flux  mm/s 
  integer, parameter :: irecv_soll   = 13   !forc_sollxy  Atm flux  W/m^2
  integer, parameter :: irecv_sols   = 14   !forc_solsxy  Atm flux  W/m^2
  integer, parameter :: irecv_solld  = 15   !forc_solldxy Atm flux  W/m^2
  integer, parameter :: irecv_solsd  = 16   !forc_solsdxy Atm flux  W/m^2
  integer, parameter :: irecv_co2 = 17      ! Atm CO2 (ppm) 
!---------------------------------------------------------------------
  integer, parameter :: isend_trad   = 1
  integer, parameter :: isend_asdir  = 2
  integer, parameter :: isend_aldir  = 3
  integer, parameter :: isend_asdif  = 4
  integer, parameter :: isend_aldif  = 5 
  integer, parameter :: isend_sno    = 6 
  integer, parameter :: isend_taux   = 7 
  integer, parameter :: isend_tauy   = 8 
  integer, parameter :: isend_lhflx  = 9 
  integer, parameter :: isend_shflx  = 10
  integer, parameter :: isend_lwup   = 11
  integer, parameter :: isend_qflx   = 12
  integer, parameter :: isend_tref2m = 13
  integer, parameter :: isend_swabs  = 14
  integer, parameter :: isend_qref2m = 15

!-------------------------------wangln 2008-09-03 add carbon flux--------------------------
  integer, parameter :: isend_carbon = 16   !carbon Lnd flux  Kg/m^2s
!-------------------------------wangln 2008-09-03 add carbon flux--------------------------

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
  integer, parameter :: isend_u10m = 17   ! 10m U wind speed m/s
  integer, parameter :: isend_v10m = 18   ! 10m V wind speed m/s
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

#ifdef MOZART2

  integer, parameter :: isend_flx_mss_vrt_dst01 = 19
  integer, parameter :: isend_flx_mss_vrt_dst02 = 20
  integer, parameter :: isend_flx_mss_vrt_dst03 = 21
  integer, parameter :: isend_flx_mss_vrt_dst04 = 22
  integer, parameter :: isend_vlc_trb_dst01 = 23
  integer, parameter :: isend_vlc_trb_dst02 = 24
  integer, parameter :: isend_vlc_trb_dst03 = 25
  integer, parameter :: isend_vlc_trb_dst04 = 26
  integer, parameter :: isend_lnd_ram1 = 27
  integer, parameter :: isend_lnd_fv   = 28
  integer, parameter :: isend_vlc_trb_SO4 = 29
  integer, parameter :: isend_vlc_trb_NH4 = 30
  integer, parameter :: isend_vlc_trb_CB2 = 31
  integer, parameter :: isend_vlc_trb_SO2 = 32

  integer, parameter :: isend_flux_ISOP = 33
  integer, parameter :: isend_flux_ACET = 34
  integer, parameter :: isend_flux_C3H6 = 35
  integer, parameter :: isend_flux_C2H4 = 36
  integer, parameter :: isend_flux_OC2 = 37
  integer, parameter :: isend_flux_C10H16 = 38


#endif

! csm timers

  logical  :: timer_lnd_sendrecv = .false. !true => timer is on
  logical  :: timer_lnd_recvsend = .false. !true => timer is on

!==================wangln 2007-07-19=====================================
  integer :: csm_nptg                         ! Loc sizes, grid coupling buffers
  integer :: csm_nptr                         ! Loc sizes, roff coupling buffers

  !!!!!integer :: begp, endp                       ! per-proc beginning and ending pft indices
  integer :: begpatch, endpatch                       ! per-proc gridcell ending gridcell indices

  integer :: begc, endc                       ! per-proc beginning and ending column indices
  integer :: begl, endl                       ! per-proc beginning and ending landunit indices
  integer :: begg, endg                       ! per-proc gridcell ending gridcell indices
  integer :: numg                             ! total number of gridcells across all processors
  integer :: numl                             ! total number of landunits across all processors
  integer :: numc                             ! total number of columns across all processors

  !!!integer :: nump                             ! total number of pfts across all processors
  integer :: numpatch    !numpatch of CLM2 == nump of CLM3       ! total number of gridcells across all processors

!=================wangln 2007-07-21======================================================
  integer :: num_lnd                          ! RTM  land runoff points accross all processors
  integer :: num_ocn                          ! RTM  Ocn  runoff points accross all processors
!=================wangln 2007-07-21======================================================

  integer :: beg_lnd_rof,end_lnd_rof          ! beginning,ending landrunoff points
  integer :: beg_ocn_rof,end_ocn_rof          ! beginning,ending oceaa 
!==================wangln 2007-07-19=====================================

  SAVE

!===============================================================================
CONTAINS
!===============================================================================

  SUBROUTINE csm_recvorb (eccen, obliqr, lambm0, mvelpp)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! receive the initial integer, real and character control data from 
! the flux coupler.  Then parse it out into the variables used by the 
! land model.
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

!xjx----20090821   use mpishorthand, only : mpir8  !------wangln 2007-07-23-------------

! ---------------------- arguments------- -------------------------
    real(r8), intent(out) :: eccen  !Earth's eccentricity of orbit
    real(r8), intent(out) :: obliqr !Earth's obliquity in radians
    real(r8), intent(out) :: lambm0 !Mean longitude of perihelion at the vernal equinox (radians)
    real(r8), intent(out) :: mvelpp !Earth's moving vernal equinox long of perihelion plus pi (radians)
! -----------------------------------------------------------------

! ---------------------- Local variables --------------------------
    integer  :: i,j,k                !indices
    integer  :: ierr                 !error code                     
    integer  :: info_time            !T => turn on msg-passing timing
    integer  :: maj_vers             !Major version of message passed from the cpl
    integer  :: min_vers             !Minor version of message passed from the cpl
    real(r8) :: spval                !float-pt buffer special value from coupler

! -----------------------------------------------------------------

    if (masterproc) then

! Receive control message from coupler

       ibuffr(:) = 0    

       call shr_msg_recv_i (ibuffr, size(ibuffr), SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)

       ierr       = ibuffr( 1)  !error code
       info_time  = ibuffr(11)  !T => turn on msg-passing timing
       maj_vers   = ibuffr(40)  !Coupler message major version
       min_vers   = ibuffr(41)  !Coupler message minor version
       ncbuff     = ibuffr(42)  !Size of character data to receive

!!------------------wangln 2008-09-03 add the carbon flux-----------------
       if( ibuffr(77) == 1 ) then
          carbon_switch = .true.
          write(6,*) '---------Carbon flux switch On------------'
       else
          carbon_switch = .false.
          write(6,*) '---------Carbon flux switch Off------------'
       endif
       if( carbon_switch ) then

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

#ifdef MOZART2
! wtw
! MOZART2  added 14 variables and 6 flux voc 
!
          nsnd = 18 + 14 + 6
#else
          nsnd = 18
#endif 
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

           nrcv = 17
       else
           nsnd = 15
           nrcv = 16
       endif 
!!------------------wangln 2008-09-03 add the carbon flux-----------------

       write(6,*) '(CSM_RECVORB): recd d->l initial ibuffr msg_id = ',SHR_MSG_TAG_C2LI
       write(6,*) 'Nsnd & Nrcv=', nsnd, nrcv

! Determine debug flag

       if (ibuffr(12) >= 2) then	
          debug_flag = .true.
       else
          debug_flag = .false.
       endif

! Check that the version of the message from the coupler is valid

       call csm_compat(maj_vers, min_vers,SHR_MSG_L_MAJ_V04, SHR_MSG_L_MIN_V00)

! Receive orbital parameters from coupler

       rbuff(:) = 0.0

       call shr_msg_recv_r (rbuff, size(rbuff), SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)

       spval  = rbuff(1)      !Coupler float-pt special data flag
       eccen  = rbuff(2)      !Earth's eccentricity of orbit
       obliqr = rbuff(3)      !Earth's Obliquity radians
       lambm0 = rbuff(4)      !Earth's Long. of prehelian at v-equinox
       mvelpp = rbuff(5)      !Earth's Moving vernal equinox of orbit + pi
       write(6,*)'(CSM_RECVORB): recd d->l initial real buff msg_id = ',SHR_MSG_TAG_C2LI
 
! Check that data received is good data and not the special value
 
       call compat_check_spval(spval, eccen ,'Eccentricity'     )
       call compat_check_spval(spval, obliqr,'Obliquity'        )
       call compat_check_spval(spval, lambm0,'Long of perhelion')
       call compat_check_spval(spval, mvelpp,'Move long of perh')
       
       write(6,*)'(CSM_RECVORB): eccen:  ', eccen
       write(6,*)'(CSM_RECVORB): obliqr: ', obliqr
       write(6,*)'(CSM_RECVORB): lambm0: ', lambm0
       write(6,*)'(CSM_RECVORB): mvelpp: ', mvelpp
       
! Receive character data from coupler and determine if will output csm timing info

       if (ncbuff > 0) then
          call shr_msg_recv_c (cbuff, ncbuff, SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
          write(6,*)'(CSM_RECVORB): recd d->a initial char. buf msg_id = ',SHR_MSG_TAG_C2LI
          write(6,*)'(CSM_RECVORB): Char: ',(cbuff(i), i = 1,ncbuff)
       end if

       if (info_time == 0) then
          csm_timing = .false.
       else
          csm_timing = .true.
       endif

    end if
!!------------------wangln 2008-09-03 add the carbon flux-----------------
#if ( defined SPMD )
    call mpi_bcast( nsnd, 1, MPI_INTEGER, 0, mpicom, ierr)
    if( ierr /= MPI_SUCCESS) then
        write(6,*) 'CSM_SENDCONTROL Error : nsnd '
    endif
    call mpi_bcast( nrcv, 1, MPI_INTEGER, 0, mpicom, ierr)
    if( ierr /= MPI_SUCCESS) then
        write(6,*) 'CSM_SENDCONTROL Error : nrcv '
    endif
    call mpi_bcast( carbon_switch, 1, MPI_LOGICAL, 0, mpicom, ierr)
    if( ierr /= MPI_SUCCESS) then
        write(6,*) 'CSM_SENDCONTROL Error : carbon_switch '
    endif
#endif
       allocate( send2d(lsmlon, lsmlat, nsnd), stat = ierr )
       if( ierr /= 0 ) write(6,*) 'CLM3 applied Send2d Failed'
       allocate( recv2d(lsmlon, lsmlat, nrcv), stat = ierr )
       if( ierr /= 0 ) write(6,*) 'CLM3 applied Recv2d Failed'
       !!real(r8) :: send2d(lsmlon,lsmlat,nsnd)  !2d send buffer
       !!real(r8) :: recv2d(lsmlon,lsmlat,nrcv)  !2d recv buffer
!!------------------wangln 2008-09-03 add the carbon flux-----------------
#if ( defined SPMD )
   call mpi_bcast(spval , 1, MPI_REAL8, 0, mpicom, ierr)
   call mpi_bcast(eccen , 1, MPI_REAL8, 0, mpicom, ierr)
   call mpi_bcast(obliqr, 1, MPI_REAL8, 0, mpicom, ierr)
   call mpi_bcast(lambm0, 1, MPI_REAL8, 0, mpicom, ierr)
   call mpi_bcast(mvelpp, 1, MPI_REAL8, 0, mpicom, ierr)
#endif

 END SUBROUTINE csm_recvorb 

!===============================================================================

 SUBROUTINE csm_sendcontrol(irad)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! send first control data to flux coupler and "invalid" grid
! containing special value data
! 
! Method: 
! The coupler treats points where the mask is nonzero as points where 
! you could possibly do a calculation (in the case of the runoff, this 
! corresponds to all RTM ocean points). The coupler then defines a "key"
! as points where the model can give you valid data (in the case of runoff, 
! this corresponds to points where the land model will give you valid 
! compressed data points). The key can be 0 where the mask is 1. However,
! the key cannot be 1 where the mask is 0 unless the data is also zero. 
! In the case of runoff, the key the coupler builds is time invariant.
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

    use clm_varctl   , only : csm_doflxave, nsrest
    use RtmMod       , only : area_r, longxy_r, latixy_r, mask_r
    use clm_varcon   , only : re	
    use time_manager , only : get_step_size
    use shr_const_mod, only : SHR_CONST_CDAY

! ---------------------- arguments--------------------------------------
    integer , intent(in) :: irad  !frequency of radiation computation 
!-----------------------------------------------------------------------

! ---------------------- Local variables --------------------------
    real(r8) rtemp_lnd(lsmlon*lsmlat*4) !temporary vector
    integer  itemp_lnd(lsmlon*lsmlat)   !temporary vector 
    real(r8) rtemp_rtm(rtmlon*rtmlat*4) !temporary vector
    real(r8) temp_area(rtmlon,rtmlat)   !temporary area 
    integer  temp_mask(rtmlon,rtmlat)   !temporary mask
    real(r8) dtime                      !step size
! -----------------------------------------------------------------

    if (masterproc) then

       rtemp_lnd(:) = 1.e30
       itemp_lnd(:) = 999
       rtemp_rtm(:) = 1.e30

! Determine number of send/recv calls steps per day to flux coupler
    
       if (nsrest == 0) then
          dtime = get_step_size()
       else
          dtime = csm_dtime
       endif

!--------------------------
!fyj       if (csm_doflxave) then
!fyj          ncpday = nint(SHR_CONST_CDAY/dtime)/irad
!fyj       else
!fyj          ncpday = nint(SHR_CONST_CDAY/dtime)
!fyj       endif
!------------------
        ncpday = nint(SHR_CONST_CDAY/dtime)/irad   ! for T106, T42
!-----------------------------------------
       
! Send integer control information 

       ibuffs(:)  = 0                !initialize ibuffs
       ibuffs(7)  = lsmlon           !number of land longitudes
       ibuffs(8)  = lsmlat           !number of land latitudes
       ibuffs(9)  = ncpday           !number of land send/recv calls per day
       ibuffs(34) = 1                !T(or 1) => requests cpl to send valid land domain info back
       ibuffs(36) = 0                !size of compressed runoff vector, if zero then not sending compressed info 
       ibuffs(37) = rtmlon           !number of longitudes in uncompressed 2d runoff array
       ibuffs(38) = rtmlat           !number of latitudes  in uncompressed 2d runoff array

       call shr_msg_send_i (ibuffs ,nibuff, SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)

! Send "invalid" land model grid and mask data

       call shr_msg_send_r (rtemp_lnd(1:lsmlon*lsmlat  ), lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (rtemp_lnd(1:lsmlon*lsmlat  ), lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (rtemp_lnd(1:lsmlon*lsmlat*4), lsmlon*lsmlat*4, SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (rtemp_lnd(1:lsmlon*lsmlat*4), lsmlon*lsmlat*4, SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (rtemp_lnd(1:lsmlon*lsmlat  ), lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_i (itemp_lnd(1:lsmlon*lsmlat  ), lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)

! Send "valid" RTM grid and mask data 
      
       temp_area(:,:) = area_r(:,:)/(re*re)  !convert from km^2 to radians^2 before sending to coupler
       temp_mask(:,:) = 1 - mask_r(:,:)      !make coupler runoff mask 1 over ocean and 0 over land 

       call shr_msg_send_r (longxy_r , rtmlon*rtmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (latixy_r , rtmlon*rtmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (rtemp_rtm, rtmlon*rtmlat*4, SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (rtemp_rtm, rtmlon*rtmlat*4, SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_r (temp_area, rtmlon*rtmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_i (temp_mask, rtmlon*rtmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)

       write(6,*)'(CSM_SENDCONTROL): there will be ',ncpday, &
            ' send/recv calls per day from the land model to the flux coupler'
       write(6,*)'(CSM_SENDCONTROL):sent l->d control data msg_id = ',SHR_MSG_TAG_L2CI

    endif

  END SUBROUTINE csm_sendcontrol

!===============================================================================

  SUBROUTINE csm_recvgrid (cam_longxy, cam_latixy, cam_numlon, cam_landfrac, cam_landmask) 

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Receive valid land grid and land mask from coupler
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

! ---------------------- arguments--------------------------------------
    integer , intent(out) :: cam_numlon(lsmlat)           !cam number of longitudes 
    real(r8), intent(out) :: cam_longxy(lsmlon,lsmlat)    !cam lon values
    real(r8), intent(out) :: cam_latixy(lsmlon,lsmlat)    !cam lat values 
    real(r8), intent(out) :: cam_landfrac(lsmlon,lsmlat)  !cam fractional land
    integer , intent(out) :: cam_landmask(lsmlon,lsmlat)  !cam land mask
!-----------------------------------------------------------------------

! ---------------------- Local variables -------------------------------
    integer  i,j,k                       !loop indices
    real(r8) xe(4,lsmlon,lsmlat)      !coupler land grid edges 
    real(r8) ye(4,lsmlon,lsmlat)      !coupler land grid edges
    real(r8) area_a(lsmlon,lsmlat)    !coupler atm grid areas
    integer  mask_a(lsmlon,lsmlat)    !coupler atm valid grid mask 
!-----------------------------------------------------------------------

    if (masterproc) then

       ibuffr(:) = 0
       
       call shr_msg_recv_i (ibuffr       ,nibuff         , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_r (cam_longxy   ,lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_r (cam_latixy   ,lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_r (xe           ,lsmlon*lsmlat*4, SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_r (ye           ,lsmlon*lsmlat*4, SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_r (area_a       ,lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_r (cam_landfrac ,lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_i (cam_landmask ,lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       call shr_msg_recv_i (mask_a       ,lsmlon*lsmlat  , SHR_MSG_TID_CPL, SHR_MSG_TAG_C2LI)
       
       write(6,*)'(CSM_SENDGRID):recd d->l land grid, msg_id= ',SHR_MSG_TAG_C2L
       
! USE mask_a to determine number of valid longitudes for each latitude band 
! this is the only use for mask_a
       
       cam_numlon(:) = 0
       do j = 1,lsmlat
          do i= 1,lsmlon
! and by wutw 2010-1009
!             write(100,*) cam_longxy(i,j),cam_latixy(i,j),(xe(k,i,j),k=1,4),&
!                          (ye(k,i,j),k=1,4),area_a(i,j),cam_landfrac(i,j),&
!                          cam_landmask(i,j),mask_a(i,j)
!            
             if (mask_a(i,j) /= 0) cam_numlon(j) = cam_numlon(j)+1
          end do
       end do
       
    endif  !end of if-masterproc block
    
    return
  END SUBROUTINE csm_recvgrid

!===============================================================================

  SUBROUTINE csm_sendrunoff()

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Send valid runoff information back to flux coupler
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------
    use decompMod    , only : get_proc_bounds, get_proc_global
    use RunoffMod    , only : get_proc_rof_bounds, get_proc_rof_global

    integer, allocatable :: ocnrof_iindx(:), ocnrof_jindx(:)
    integer :: ier, n

    call get_proc_bounds(begg, endg, begl, endl, begc, endc, begpatch, endpatch)
    call get_proc_global(numg, numl, numc, numpatch)
    call get_proc_rof_global(num_lnd, num_ocn)
    call get_proc_rof_bounds(beg_lnd_rof, end_lnd_rof, beg_ocn_rof, end_ocn_rof)

    write(6,*) 'CSM_SENDRUNOFF : numg, numl, numc, numpatch=', numg, numl, numc, numpatch
    write(6,*) 'CSM_SENDRUNOFF : begg, endg begpatch, endpatch beg_ocn_rof, end_ocn_rof=', &
                                 begg, endg, begpatch, endpatch, beg_ocn_rof, end_ocn_rof   
    call shr_sys_flush(6)
    if (masterproc) then
        allocate( ocnrof_vec( num_ocn ), stat=ier )
    endif
  
    call shr_sys_flush(6)


    if (masterproc) then
   
!===============wangln 2007-07-19================================
       allocate( ocnrof_iindx( num_ocn), stat = ier )
       allocate( ocnrof_jindx( num_ocn), stat = ier )
       do n = 1, num_ocn
          ocnrof_iindx( n ) = runoff%ocn_ixy(n)
          ocnrof_jindx( n ) = runoff%ocn_jxy(n)
       enddo
!===============wangln 2007-07-19================================


! Send integer buffer control info

       ibuffs(7)  = lsmlon           !number of model longitudes
       ibuffs(8)  = lsmlat           !number of model latitudes
       ibuffs(36) = num_ocn          !number of data points in compressed runoff data 
       ibuffs(37) = rtmlon           !number of longitudes in uncompressed 2d runoff array
       ibuffs(38) = rtmlat           !number of latitudes  in uncompressed 2d runoff array

       call shr_msg_send_i(ibuffs       ,nibuff          , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)

! Send runoff vector compression info

       call shr_msg_send_i(ocnrof_iindx, size(ocnrof_iindx), SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)
       call shr_msg_send_i(ocnrof_jindx, size(ocnrof_jindx), SHR_MSG_TID_CPL, SHR_MSG_TAG_L2CI)

       deallocate ( ocnrof_iindx)
       deallocate ( ocnrof_jindx) 

       write(6,*) '(CSM_SENDROF):sent l->d valid initial runoff info msg_id = ',SHR_MSG_TAG_L2CI

    endif

  END SUBROUTINE csm_sendrunoff

!===============================================================================

  SUBROUTINE csm_sendalb
 
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Send initial albedos, surface temperature and snow data to the 
! flux coupler
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

!xjx-------20090821    use clm_varsur                    
    use clm_varctl  , only : csm_doflxave, nsrest
    use time_manager, only : get_curr_date, get_prev_date

 !==============wangln 2007-07-21=========================
    use clmtype
    use clm_varsur  , only : landmask
    use lnd2atmMod  , only : lnd2atm
    use clm_varcon  , only : sb
    type(gridcell_type), pointer :: gptr  ! pointer to gridcell derived subtype
 !==============wangln 2007-07-21=========================

	
! --------------------------- Local variables ---------------------
    integer  :: i,j,k,l,m,n !loop indices
    real(r8) :: wt          !weight  
    integer  :: yr          !current year 
    integer  :: mon         !current month 
    integer  :: day         !current day (0, 1, ...)
    integer  :: ncsec       !current seconds of current date (0, ..., 86400)
    integer  :: ncdate      !current date (yymmdd format) (e.g., 021105)
    integer  :: ier                !return error code
! -----------------------------------------------------------------

! Allocate dynamic memory 

    gptr => clm3%g
 
    allocate (send1d(nsnd,begg:endg), STAT=ier) 
    allocate (loc_roff(beg_ocn_rof:end_ocn_rof), STAT=ier) 

    if (ier /= 0) then
       write(6,*)'CSM_SENDALB error: send1d allocation error'
       call shr_sys_flush(6)
       call endrun
    endif
    send1d(:,:)=inf

#if (defined SPMD)
    if (masterproc) then
       allocate (gather1d(nsnd,numg), STAT=ier)
       allocate (glob_roff(num_ocn), STAT=ier) 
       if (ier /= 0) then
          write(6,*)'CSM_SENDALB error: gather1d allocation error'
          call shr_sys_flush(6)
          call endrun
       endif
       gather1d(:,:) = inf
    endif
#else
    gather1d => send1d
    glob_roff => loc_roff
#endif

    if (masterproc) then
       allocate (recv1d(nrcv,numg), STAT=ier) 
       if (ier /= 0) then
          write(6,*)'CSM_SENDALB error: recv1d allocation error'
          call shr_sys_flush(6)
          call endrun
       endif
       recv1d(:,:)=inf
    endif

#if (defined SPMD)
    allocate (scatter1d(nrcv,numg), STAT=ier) !-----wangln 2007-07-25 
    if (ier /= 0) then
       write(6,*)'CSM_SENDALB error: scatter1d allocation error'
       call endrun
    endif
    scatter1d(:,:) = inf
#else
    scatter1d => recv1d
#endif

! Send first data to coupler

    if (nsrest == 0) then   !initial run

! On initial timestep ONLY: determine 1d vector of states that will be sent
! to coupler and map fields from 1d subgrid vector to 2d [lsmlon]x[lsmlat] grid.
       call lnd2atm(init=.true.)
       write(6,*) ' --------CSM_SENDALB call lnd2atm Ok'
       write(6,*) '---------CSM_SENDALB begg & endg=', begg, endg

       do k = begg, endg
          !send1d(:,k)            = 1.e30                !don't want to send NaN 
          send1d(isend_trad ,k)  = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(k)/sb))        !tsxy
          send1d(isend_sno  ,k)  = gptr%l2as%h2osno(k)  !snow (convert from mm to m)
          send1d(isend_asdir,k)  = gptr%l2as%albd(k,1)       !asdir
          send1d(isend_aldir,k)  = gptr%l2as%albd(k,2)       !aldir
          send1d(isend_asdif,k)  = gptr%l2as%albi(k,1)       !asdif
          send1d(isend_aldif,k)  = gptr%l2as%albi(k,2)       !aldif

!------------------------
! wtw 2015.06.18
!          send1d(isend_u10m ,k)  = gptr%l2as%u_10m(k)         ! 10m U wind
!          send1d(isend_v10m ,k)  = gptr%l2as%v_10m(k)         ! 10m V wind
!
!          send1d(isend_tref2m,k) = gptr%l2as%t_ref2m(k)            !tref
!          send1d(isend_qref2m,k) = gptr%l2as%q_ref2m(k)            !tref
!------------
       end do
       write(6,*) 'CSM_Sendalb Step 1'; call shr_sys_flush(6)

        loc_roff = 0.0
       glob_roff = 0.0
       do k = beg_ocn_rof, end_ocn_rof
          loc_roff( k ) = runoff%ocn(k) / (runoff%ocn_area(k)*1000. ) ! Get Loca OCN Roff Data
       enddo 
       write(6,*) 'CSM_Sendalb Step 2'; call shr_sys_flush(6)

#if (defined SPMD)
       call gather_data_to_master( send1d,    gather1d, clmlevel=nameg )
       call gather_data_to_master( loc_roff, glob_roff, clmlevel=ocnrof )
#else
       gather1d  => send1d
       glob_roff => loc_roff
#endif

       if (masterproc ) then
          do n=1,nsnd
             where (landmask(:,:) > 0) 
                send2d(:,:,n) = 0.
             elsewhere
                send2d(:,:,n) = 1.e30
             end where
          end do
          send2d(:,:,isend_sno) = 0. ! snow initialized to 0 everywhere

          do k = 1, numg
             i = gptr%ixy(k)   
             j = gptr%jxy(k)   
             do n = 1, nsnd
                send2d(i,j,n) = gather1d(n,k)
             enddo
          enddo 

          write(6,*) 'CSM_Sendalb Step 3'; call shr_sys_flush(6)
          do k = 1, num_ocn
             ocnrof_vec( k ) = glob_roff(k)
          enddo
          write(6,*) 'CSM_Sendalb Step 4'; call shr_sys_flush(6)

       endif

    else  ! restart run

! On a restart run, no meaningful data is sent to the flux coupler - 
! this includes the ocean runoff vector (which should only contain zero values)
! since the runoff code (riverfluxrtm) has not been called yet

       if (masterproc) send2d(:,:,:) = 1.e30  ! this will be sent on a restart timestep

    endif  
      
! Send data to coupler
! Determine time index to send to coupler. Note that for a restart run,
! the next time step is nstep+1. But must send current time step to 
! flux coupler here. 

    if (masterproc) then

       if (nsrest == 0) then
          call get_curr_date (yr, mon, day, ncsec) 
       else
          call get_prev_date (yr, mon, day, ncsec)
       endif
       ncdate = yr*10000 + mon*100 + day

       ibuffs(4) = ncdate  !model date (yyyymmdd)
       ibuffs(5) = ncsec   !elapsed seconds in model date

       call shr_msg_send_i (ibuffs    , size(ibuffs)    , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2C)
       call shr_msg_send_r (send2d    , size(send2d)    , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2C)
       call shr_msg_send_r (ocnrof_vec, size(ocnrof_vec), SHR_MSG_TID_CPL, SHR_MSG_TAG_L2C)

       if (csm_timing) then
          irtc_s = shr_sys_irtc()
          write(6,9099) irtc_s,'l->d sending'
9099      format('[mp timing]  irtc = ',i20,' ',a)
       end if

    endif  ! end of if_masterproc

    return
  END SUBROUTINE csm_sendalb

!===============================================================================

  SUBROUTINE csm_dosndrcv (doalb)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Determine when to send and receive messages to/from the
! flux coupler on this time-step.
! 
! Method: 
! Determine if send/receive information to/from flux coupler
! Send msgs (land state and fluxes) to the flux coupler only when 
! doalb is true (i.e. on time steps before the atm does a solar
! radiation computation). Receive msgs (atm state) from the
! flux coupler only when dorad is true (i.e. on time steps 
! when the atm does a solar radiation computation). 
! The fluxes are then averaged between the send and receive calls. 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

    use clm_varctl   , only : csm_doflxave
    use time_manager , only : get_step_size, get_nstep
    use shr_const_mod, only : SHR_CONST_CDAY

!----------------------------- Arguments --------------------------
    logical, intent(in) :: doalb  !true=>next timestep a radiation time step
!-----------------------------------------------------------------

! ---------------------- Local variables --------------------------
    integer  :: ntspday           !model steps per day
    real(r8) :: dtime             !step size (seconds)   
    integer  :: nstep             !time step 
!-----------------------------------------------------------------

! -----------------------------------------------------------------

! Determine if send/receive information to/from flux coupler

    nstep = get_nstep()
    if (csm_doflxave) then
       if (nstep == 0) then
          dorecv = .true.
          dosend = .false.
       else if (nstep == 1) then
          dorecv = .false.
          dosend = doalb
       else
          dorecv = dosend
          dosend = doalb
       endif
    else
       if (nstep == 0) then
          dorecv = .true.
          dosend = .false.
       else if (nstep == 1) then
          dorecv = .false.
          dosend = .true.
       else
          dorecv = .true.
          dosend = .true.
       endif
    endif

! If at end of day: check if should write restart file or stop at next time step
! Note these statements must appear here since ibuffr is not received at every time 
! step when flux averaging occurs.

    csmstop_next = .false.
    csmrstrt     = .false.
    dtime        = get_step_size()
    ntspday      = nint(SHR_CONST_CDAY/dtime)
    if (mod(nstep,ntspday) == 0) then
       if (ibuffr(2) /= 0) then  !stop at end of day
          csmstop_next = .true.  !will stop on next time step
          write(6,*)'(CSM_DOSNDRCV) output restart and history files at nstep = ',nstep
       endif
       if (ibuffr(21) /= 0) then !write restart at end of day
          csmrstrt = .true.      !will write restart now
          write(6,*)'(CSM_DOSNDRCV) output restart and history files at nstep = ',nstep
       endif
    endif

    return
  END SUBROUTINE csm_dosndrcv

!===============================================================================

  SUBROUTINE csm_recv()

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Receive data from the flux coupler
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

    use clm_varcon, only : rair, po2, pco2
    use clm_varsur  , only : landmask

    use clmtype
    type(gridcell_type), pointer :: gptr  ! pointer to gridcell derived subtype

! --------------------------- Local variables ---------------------
    integer :: i,j,k,n             !indices 
    real(r8):: forc_rainc          !rainxy Atm flux mm/s   
    real(r8):: forc_rainl          !rainxy Atm flux mm/s   
    real(r8):: forc_snowc          !snowfxy Atm flux  mm/s 
    real(r8):: forc_snowl          !snowfxl Atm flux  mm/s 
    integer :: ier                 !return error code	
! -----------------------------------------------------------------

! Start timers

     gptr => clm3%g
     if (timer_lnd_sendrecv) then
        call t_stopf ('lnd_sendrecv') ; timer_lnd_sendrecv = .false.
     endif

     call t_startf('lnd_recv')

! Receive message from flux coupler

    if (masterproc) then
       ibuffr(:)     = 0            
       recv2d(:,:,:) = 1.e30
       if (csm_timing) irtc_w = shr_sys_irtc()
       call shr_msg_recv_i (ibuffr, size(ibuffr), SHR_MSG_TID_CPL, SHR_MSG_TAG_C2L)
       call shr_msg_recv_r (recv2d, size(recv2d), SHR_MSG_TID_CPL, SHR_MSG_TAG_C2L)
       if (csm_timing) then
          irtc_r = shr_sys_irtc()
          write(6,9099) irtc_w,'d->l waiting'
9099      format('[mp timing]  irtc = ',i20,' ',a)
          write(6,9099) irtc_r,'d->l received'
       end if

! Do global integrals of fluxes if flagged

       if (debug_flag) then	
          write(6,*)
          write(6,100) 'lnd','recv', irecv_lwrad, global_sum(recv2d(1,1,irecv_lwrad),1.e30), ' lwrad'
          write(6,100) 'lnd','recv', irecv_rainc, global_sum(recv2d(1,1,irecv_rainc),1.e30), ' rainc'
          write(6,100) 'lnd','recv', irecv_rainl, global_sum(recv2d(1,1,irecv_rainl),1.e30), ' rainl'
          write(6,100) 'lnd','recv', irecv_snowc, global_sum(recv2d(1,1,irecv_snowc),1.e30), ' snowc'
          write(6,100) 'lnd','recv', irecv_snowl, global_sum(recv2d(1,1,irecv_snowl),1.e30), ' snowl'
          write(6,100) 'lnd','recv', irecv_soll , global_sum(recv2d(1,1,irecv_soll ),1.e30), ' soll '
          write(6,100) 'lnd','recv', irecv_sols , global_sum(recv2d(1,1,irecv_sols ),1.e30), ' sols '
          write(6,100) 'lnd','recv', irecv_solld, global_sum(recv2d(1,1,irecv_solld),1.e30), ' solld'
          write(6,100) 'lnd','recv', irecv_solsd, global_sum(recv2d(1,1,irecv_solsd),1.e30), ' solsd'
100       format('comm_diag ',a3,1x,a4,1x,i3,es26.19,a)
          write(6,*)
       endif
    endif  ! end of if-masteproc
#if (defined SPMD)
    call mpi_bcast (ibuffr, size(ibuffr), MPI_INTEGER, 0, mpicom, ier)    
#endif

! Stop timer

     call t_stopf('lnd_recv') 

! Check if end of run now, if so stop (each processor does this) 

    csmstop_now = .false.
    if (ibuffr(3) /= 0) then 
       csmstop_now = .true.
       if (timer_lnd_recvsend) call t_stopf('lnd_recvsend')
       if (timer_lnd_sendrecv) call t_stopf('lnd_sendrecv')
       write(6,*)'(CSM_RECV) stop now signal from flux coupler'
       write(6,*)'(CSM_RECV) ibuffr(3) = ',ibuffr(3)
       if (masterproc) then
          write(6,9001)
          write(6,9002) ibuffr(4)
          write(6,9003)
9001      format(/////' ===========> Terminating CLM Model')
9002      format(     '      Date: ',i8)
9003      format(/////' <=========== CLM Model Terminated')
!-------------------------------wangln 2008-09-03 add carbon flux--------------------------
    if (masterproc) then
          deallocate( send2d )
          deallocate( recv2d )
    endif
!-------------------------------wangln 2008-09-03 add carbon flux--------------------------

       endif
       RETURN
    endif

! More timer logic
 
     if (.not. timer_lnd_recvsend) then
        call t_startf('lnd_recvsend') ; timer_lnd_recvsend = .true.
     endif

! Map 2d received fields on [lsmlon]x[lsmlat] grid to subgrid vectors

    if (masterproc) then
          do k = 1, numg
             i = gptr%ixy(k)   
             j = gptr%jxy(k)   
             do n = 1, nrcv
                recv1d(n,k) = recv2d(i,j,n)
             enddo
          enddo 
    end if

#if (defined SPMD)
    call scatter_data_from_master( scatter1d, recv1d, clmlevel = nameg ) 
#else
    scatter1d => recv1d
#endif

! Split data from coupler into component arrays. Note that the precipitation fluxes received 
! from the coupler are in units of kg/s/m^2. To convert these precipitation rates in units of 
! mm/sec, one must divide by 1000 kg/m^3 and multiply by 1000 mm/m resulting in an overall 
! factor of unity. Below the units are therefore given in mm/s.

    do k = begg, endg
       gptr%a2ls%forc_hgt(k)      = scatter1d(irecv_hgt  ,k) !zgcmxy  Atm state m
       gptr%a2ls%forc_u(k)        = scatter1d(irecv_u    ,k) !forc_uxy  Atm state m/s
       gptr%a2ls%forc_v(k)        = scatter1d(irecv_v    ,k) !forc_vxy  Atm state m/s
       gptr%a2ls%forc_th(k)       = scatter1d(irecv_th   ,k) !forc_thxy Atm state K
       gptr%a2ls%forc_q(k)        = scatter1d(irecv_q    ,k) !forc_qxy  Atm state kg/kg
       gptr%a2ls%forc_pbot(k)     = scatter1d(irecv_pbot ,k) !ptcmxy  Atm state Pa
       gptr%a2ls%forc_t(k)        = scatter1d(irecv_t    ,k) !forc_txy  Atm state K
       gptr%a2lf%forc_lwrad(k)    = scatter1d(irecv_lwrad,k) !flwdsxy Atm flux  W/m^2 
       forc_rainc                 = scatter1d(irecv_rainc,k) !mm/s 
       forc_rainl                 = scatter1d(irecv_rainl,k) !mm/s 
       forc_snowc                 = scatter1d(irecv_snowc,k) !mm/s
       forc_snowl                 = scatter1d(irecv_snowl,k) !mm/s
       gptr%a2lf%forc_solad(k,2)  = scatter1d(irecv_soll ,k) !forc_sollxy  Atm flux  W/m^2 
       gptr%a2lf%forc_solad(k,1)  = scatter1d(irecv_sols ,k) !forc_solsxy  Atm flux  W/m^2
       gptr%a2lf%forc_solai(k,2)  = scatter1d(irecv_solld,k) !forc_solldxy Atm flux  W/m^2
       gptr%a2lf%forc_solai(k,1)  = scatter1d(irecv_solsd,k) !forc_solsdxy Atm flux  W/m^2

       ! determine derived quantities

       gptr%a2ls%forc_hgt_u(k) = gptr%a2ls%forc_hgt(k)    !observational height of wind [m]
       gptr%a2ls%forc_hgt_t(k) = gptr%a2ls%forc_hgt(k)    !observational height of temperature [m]
       gptr%a2ls%forc_hgt_q(k) = gptr%a2ls%forc_hgt(k)    !observational height of humidity [m]
       gptr%a2ls%forc_vp(k)    = gptr%a2ls%forc_q(k) * gptr%a2ls%forc_pbot(k) &
                                 / (0.622 + 0.378 * gptr%a2ls%forc_q(k))
       gptr%a2ls%forc_rho(k)   = (gptr%a2ls%forc_pbot(k) - 0.378 * gptr%a2ls%forc_vp(k)) &
                                 / (rair * gptr%a2ls%forc_t(k))

!------------------------------------------
       if( carbon_switch ) then
           gptr%a2ls%forc_co2(k)   = scatter1d(irecv_co2,k) !forc_co2:  Co2 concentration (ppm)
       else
           gptr%a2ls%forc_co2(k)   = pco2                   ! CO2 vmr
       endif
       gptr%a2ls%forc_o2(k)    = po2
!---------------------------------------
       gptr%a2ls%forc_wind(k)  = sqrt(gptr%a2ls%forc_u(k)**2 + gptr%a2ls%forc_v(k)**2)
       gptr%a2lf%forc_solar(k) = gptr%a2lf%forc_solad(k,1) + gptr%a2lf%forc_solai(k,1) + &
                                 gptr%a2lf%forc_solad(k,2) + gptr%a2lf%forc_solai(k,2)

       ! Determine precipitation needed by clm
       
       gptr%a2lf%forc_rain(k) = forc_rainc + forc_rainl
       gptr%a2lf%forc_snow(k) = forc_snowc + forc_snowl

    end do

    return
  END SUBROUTINE csm_recv

!===============================================================================

  SUBROUTINE csm_send()

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Send data to the flux coupler
!
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

    use clm_varctl  , only : csm_doflxave
    use clm_varsur  , only : landmask
    use clm_varcon  , only : sb

    use time_manager, only : get_curr_date, get_nstep
    use clmtype
    use lnd2atmMod  , only : lnd2atm
    type(gridcell_type), pointer :: gptr  ! pointer to gridcell derived type
    type(pft_type)     , pointer :: pptr  ! pointer to column   derived type

! --------------------------- Local variables ---------------------
    integer :: i,j,k,l,m,n, g, p !loop indices
    integer :: yr                !current year 
    integer :: mon               !current month 
    integer :: day               !current day (0, 1, ...)
    integer :: ncsec             !current seconds of current date (0, ..., 86400)
    integer :: ncdate            !current date (yymmdd format) (e.g., 021105)
    real(r8):: wt              ! weight
    integer :: ier                  !error return status
! -----------------------------------------------------------------
    
! Send data to the flux coupler
      ! Set pointers into derived type
    
      gptr => clm3%g
      pptr => clm3%g%l%c%p

    if (timer_lnd_recvsend) then
       call t_stopf ('lnd_recvsend') ; timer_lnd_recvsend = .false.
    endif

! Start timer

    call t_startf('lnd_send')

! Determine 1d vector of fields that will be sent to coupler.
! Coupler has convention that fluxes are positive downward.

    send1d = 0.0
    call lnd2atm()  ! Calculate fluxes and states to send to atm
    if (csm_doflxave) then
       do g = begg,endg
          gptr%l2af%taux(g) = 0.
          gptr%l2af%tauy(g) = 0.
          gptr%l2af%eflx_lh_tot(g) = 0.
          gptr%l2af%eflx_sh_tot(g) = 0.
          gptr%l2af%eflx_lwrad_out(g) = 0.
          gptr%l2af%qflx_evap_tot(g) = 0.
          gptr%l2af%fsa(g) = 0.

          if( carbon_switch ) gptr%l2af%carbon(g) = 0.   ! CO2 flux from land

#ifdef MOZART2
          gptr%l2af%flx_mss_vrt_dst01(g) = 0.0
          gptr%l2af%flx_mss_vrt_dst02(g) = 0.0
          gptr%l2af%flx_mss_vrt_dst03(g) = 0.0
          gptr%l2af%flx_mss_vrt_dst04(g) = 0.0
          gptr%l2af%vlc_trb_dst01(g) = 0.0
          gptr%l2af%vlc_trb_dst02(g) = 0.0
          gptr%l2af%vlc_trb_dst03(g) = 0.0
          gptr%l2af%vlc_trb_dst04(g) = 0.0
          gptr%l2af%lnd_ram1(g) = 0.0
          gptr%l2af%lnd_fv(g) = 0.0
          gptr%l2af%vlc_trb_SO4(g) = 0.0
          gptr%l2af%vlc_trb_NH4(g) = 0.0
          gptr%l2af%vlc_trb_CB2(g) = 0.0
          gptr%l2af%vlc_trb_SO2(g) = 0.0

          gptr%l2af%flux_ISOP(g) = 0.0
          gptr%l2af%flux_ACET(g) = 0.0
          gptr%l2af%flux_C3H6(g) = 0.0
          gptr%l2af%flux_C2H4(g) = 0.0
          gptr%l2af%flux_OC2(g) = 0.0
          gptr%l2af%flux_C10H16(g) = 0.0
#endif
       enddo
       do p = begpatch, endpatch
          g  = pptr%gridcell(p)
          wt = pptr%wtgcell(p)
          gptr%l2af%taux(g)           = gptr%l2af%taux(g)           + taux_ave(p)  * wt
          gptr%l2af%tauy(g)           = gptr%l2af%tauy(g)           + tauy_ave(p)  * wt
          gptr%l2af%eflx_lh_tot(g)    = gptr%l2af%eflx_lh_tot(g)    + lhflx_ave(p) * wt
          gptr%l2af%eflx_sh_tot(g)    = gptr%l2af%eflx_sh_tot(g)    + shflx_ave(p) * wt
          gptr%l2af%eflx_lwrad_out(g) = gptr%l2af%eflx_lwrad_out(g) + lwup_ave(p)  * wt
          gptr%l2af%qflx_evap_tot(g)  = gptr%l2af%qflx_evap_tot(g)  + qflx_ave(p)  * wt
          gptr%l2af%fsa(g)            = gptr%l2af%fsa(g)            + swabs_ave(p) * wt

          if( carbon_switch ) gptr%l2af%carbon(g) = gptr%l2af%carbon(g) + carbon_ave(p) * wt 

!-------------------------------wangln 2008-09-03 add carbon flux--------------------------
#ifdef MOZART2
          gptr%l2af%flx_mss_vrt_dst01(g) = gptr%l2af%flx_mss_vrt_dst01(g)  + flx_mss_vrt_dst01_ave(p) * wt
          gptr%l2af%flx_mss_vrt_dst02(g) = gptr%l2af%flx_mss_vrt_dst02(g)  + flx_mss_vrt_dst02_ave(p) * wt
          gptr%l2af%flx_mss_vrt_dst03(g) = gptr%l2af%flx_mss_vrt_dst03(g)  + flx_mss_vrt_dst03_ave(p) * wt
          gptr%l2af%flx_mss_vrt_dst04(g) = gptr%l2af%flx_mss_vrt_dst04(g)  + flx_mss_vrt_dst04_ave(p) * wt
          gptr%l2af%vlc_trb_dst01(g) = gptr%l2af%vlc_trb_dst01(g)  + vlc_trb_dst01_ave(p) * wt
          gptr%l2af%vlc_trb_dst02(g) = gptr%l2af%vlc_trb_dst02(g)  + vlc_trb_dst02_ave(p) * wt
          gptr%l2af%vlc_trb_dst03(g) = gptr%l2af%vlc_trb_dst03(g)  + vlc_trb_dst03_ave(p) * wt
          gptr%l2af%vlc_trb_dst04(g) = gptr%l2af%vlc_trb_dst04(g)  + vlc_trb_dst04_ave(p) * wt
          gptr%l2af%lnd_ram1(g)  = gptr%l2af%lnd_ram1(g)   + lnd_ram1_ave(p) * wt
          gptr%l2af%lnd_fv(g)    = gptr%l2af%lnd_fv(g)     + lnd_fv_ave(p) * wt
          gptr%l2af%vlc_trb_SO4(g) = gptr%l2af%vlc_trb_SO4(g)  + vlc_trb_SO4_ave(p) * wt
          gptr%l2af%vlc_trb_NH4(g) = gptr%l2af%vlc_trb_NH4(g)  + vlc_trb_NH4_ave(p) * wt
          gptr%l2af%vlc_trb_CB2(g) = gptr%l2af%vlc_trb_CB2(g)  + vlc_trb_CB2_ave(p) * wt
          gptr%l2af%vlc_trb_SO2(g) = gptr%l2af%vlc_trb_SO2(g)  + vlc_trb_SO2_ave(p) * wt

          gptr%l2af%flux_ISOP(g) = gptr%l2af%flux_ISOP(g)  + flux_ISOP_ave(p) * wt
          gptr%l2af%flux_ACET(g) = gptr%l2af%flux_ACET(g)  + flux_ACET_ave(p) * wt
          gptr%l2af%flux_C3H6(g) = gptr%l2af%flux_C3H6(g)  + flux_C3H6_ave(p) * wt
          gptr%l2af%flux_C2H4(g) = gptr%l2af%flux_C2H4(g)  + flux_C2H4_ave(p) * wt
          gptr%l2af%flux_OC2(g) = gptr%l2af%flux_OC2(g)  + flux_OC2_ave(p) * wt
          gptr%l2af%flux_C10H16(g) = gptr%l2af%flux_C10H16(g)  + flux_C10H16_ave(p) * wt
#endif
       enddo
       do g = begg,endg
          gptr%l2as%t_rad(g) = (abs(gptr%l2af%eflx_lwrad_out(g)/sb))**0.25
       enddo  
    endif !ENDIF of csm_doflxave
    do k = begg, endg
       send1d(isend_trad ,k)  = gptr%l2as%t_rad(k)         !tsxy
       send1d(isend_asdir,k)  = gptr%l2as%albd(k,1)        !asdir
       send1d(isend_aldir,k)  = gptr%l2as%albd(k,2)        !aldir
       send1d(isend_asdif,k)  = gptr%l2as%albi(k,1)        !asdif
       send1d(isend_aldif,k)  = gptr%l2as%albi(k,2)        !aldif
       send1d(isend_sno  ,k)  = gptr%l2as%h2osno(k)        !snow (convert from mm to m)

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
       send1d(isend_u10m ,k)  = gptr%l2as%u_10m(k)         ! 10m U wind 
       send1d(isend_v10m ,k)  = gptr%l2as%v_10m(k)         ! 10m V wind 
!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

       send1d(isend_taux ,k)  = -gptr%l2af%taux(k)
       send1d(isend_tauy ,k)  = -gptr%l2af%tauy(k)
       send1d(isend_lhflx,k)  = -gptr%l2af%eflx_lh_tot(k) 
       send1d(isend_shflx,k)  = -gptr%l2af%eflx_sh_tot(k)
       send1d(isend_lwup ,k)  = -gptr%l2af%eflx_lwrad_out(k)
       send1d(isend_qflx ,k)  = -gptr%l2af%qflx_evap_tot(k)
       send1d(isend_swabs,k)  = -gptr%l2af%fsa(k)
       send1d(isend_tref2m,k) = gptr%l2as%t_ref2m(k)            !tref
       send1d(isend_qref2m,k) = gptr%l2as%q_ref2m(k)            !tref

       if( carbon_switch ) then
          send1d(isend_carbon,k) = gptr%l2af%carbon(k)          
       endif

#ifdef MOZART2
       send1d(isend_flx_mss_vrt_dst01,k) = gptr%l2af%flx_mss_vrt_dst01(k)
       send1d(isend_flx_mss_vrt_dst02,k) = gptr%l2af%flx_mss_vrt_dst02(k)
       send1d(isend_flx_mss_vrt_dst03,k) = gptr%l2af%flx_mss_vrt_dst03(k)
       send1d(isend_flx_mss_vrt_dst04,k) = gptr%l2af%flx_mss_vrt_dst04(k)
       send1d(isend_vlc_trb_dst01,k) = gptr%l2af%vlc_trb_dst01(k)
       send1d(isend_vlc_trb_dst02,k) = gptr%l2af%vlc_trb_dst02(k)
       send1d(isend_vlc_trb_dst03,k) = gptr%l2af%vlc_trb_dst03(k)
       send1d(isend_vlc_trb_dst04,k) = gptr%l2af%vlc_trb_dst04(k)
       send1d(isend_lnd_ram1,k) = gptr%l2af%lnd_ram1(k)
       send1d(isend_lnd_fv,k)  = gptr%l2af%lnd_fv(k)
       send1d(isend_vlc_trb_SO4,k) = gptr%l2af%vlc_trb_SO4(k)
       send1d(isend_vlc_trb_NH4,k) = gptr%l2af%vlc_trb_NH4(k)
       send1d(isend_vlc_trb_CB2,k) = gptr%l2af%vlc_trb_CB2(k)
       send1d(isend_vlc_trb_SO2,k) = gptr%l2af%vlc_trb_SO2(k)

       send1d(isend_flux_ISOP,k) = gptr%l2af%flux_ISOP(k)
       send1d(isend_flux_ACET,k) = gptr%l2af%flux_ACET(k)
       send1d(isend_flux_C3H6,k) = gptr%l2af%flux_C3H6(k)
       send1d(isend_flux_C2H4,k) = gptr%l2af%flux_C2H4(k)
       send1d(isend_flux_OC2,k) = gptr%l2af%flux_OC2(k)
       send1d(isend_flux_C10H16,k) = gptr%l2af%flux_C10H16(k)

#endif
    enddo 
    
     loc_roff = 0.0
    glob_roff = 0.0 
    do n = beg_ocn_rof, end_ocn_rof
       loc_roff( n ) = runoff%ocn(n) / (runoff%ocn_area(n)*1000.)
    enddo
   

#if (defined SPMD)
       call gather_data_to_master( send1d,   gather1d,  clmlevel=nameg )
       call gather_data_to_master( loc_roff, glob_roff, clmlevel=ocnrof )
#else
       gather1d  => send1d
       glob_roff => loc_roff
#endif
    
! Send data to flux coupler
! First, map fields from 1d subgrid vector to 2d [lsmlon]x[lsmlat] grid, weighting
! by subgrid fraction. Use only points with wt > 0 so SPMD code will not use 
! uninitialized stack memory values for arrays like taux. NOTE: snow is sent as 
! zero over non-land because currently the ocn and sea-ice send no snow cover 
! to coupler and so the coupler sends back zero snow over non-land to 
! the atm (atm and land grid are currently assumed to be identical)

    if (masterproc) then

       do n = 1,nsnd
          where( landmask(:,:) > 0 ) 
             send2d(:,:,n) = 0.
          elsewhere
             send2d(:,:,n) = 1.e30
          endwhere
       end do
       send2d(:,:,isend_sno) = 0.     !reset snow to 0 everywhere

       do k = 1, numg
          i = gptr%ixy(k)   
          j = gptr%jxy(k)   
          do n = 1, nsnd
             send2d(i,j,n) = gather1d(n,k)
          enddo
       enddo 

       ocnrof_vec = 0.0
       do k = 1, num_ocn
          ocnrof_vec( k ) = glob_roff(k)
       enddo

       call get_curr_date (yr, mon, day, ncsec) 
       ncdate = yr*10000 + mon*100 + day

       ibuffs(4)  = ncdate           !model date (yyyymmdd)
       ibuffs(5)  = ncsec            !elapsed seconds in current date
       
       do j = 1, lsmlat
          do i = 1, lsmlon
             if( send2d(i,j, isend_carbon ) /= send2d(i,j, isend_carbon ) ) then
                 write(6,*) ' I, J, Carbon =', i,j,send2d(i,j, isend_carbon )
             endif
          enddo
       enddo 
          
       call shr_msg_send_i (ibuffs    , size(ibuffs)    , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2C)
       call shr_msg_send_r (send2d    , size(send2d)    , SHR_MSG_TID_CPL, SHR_MSG_TAG_L2C)
       call shr_msg_send_r (ocnrof_vec, size(ocnrof_vec), SHR_MSG_TID_CPL, SHR_MSG_TAG_L2C)

       if (csm_timing) then
          irtc_s = shr_sys_irtc()
          write(6,9099) irtc_s,'l->d sending'
9099      format('[mp timing]  irtc = ',i20,' ',a)
       end if
       
! Do global integrals if flag is set

       if (debug_flag) then	
          write(6,*)
          write(6,100) 'lnd','send', isend_taux , global_sum(send2d(1,1,isend_taux ),1.e30), ' taux'
          write(6,100) 'lnd','send', isend_tauy , global_sum(send2d(1,1,isend_tauy ),1.e30), ' tauy'
          write(6,100) 'lnd','send', isend_lhflx, global_sum(send2d(1,1,isend_lhflx),1.e30), ' lhflx'
          write(6,100) 'lnd','send', isend_shflx, global_sum(send2d(1,1,isend_shflx),1.e30), ' shflx'
          write(6,100) 'lnd','send', isend_lwup , global_sum(send2d(1,1,isend_lwup ),1.e30), ' lwup'
          write(6,100) 'lnd','send', isend_qflx , global_sum(send2d(1,1,isend_qflx ),1.e30), ' qflx'
          write(6,100) 'lnd','send', isend_swabs, global_sum(send2d(1,1,isend_swabs),1.e30), ' swabs'
          write(6,*)
100       format('comm_diag ',a3,1x,a4,1x,i3,es26.19,a)
       endif

    endif  ! end of if_masterproc

! Stop timers

    call t_stopf('lnd_send')

    if (.not. timer_lnd_recvsend) then
       call t_startf('lnd_sendrecv') ; timer_lnd_sendrecv = .true.
    endif

    return
  END SUBROUTINE csm_send

!===============================================================================

  SUBROUTINE csm_flxave() 

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Average output fluxes for flux coupler
! 
! Method: 
! Add land surface model output fluxes to accumulators every time step.
! When icnt==ncnt, compute the average flux over the time interval.
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

    use clm_varctl  , only : irad
    use time_manager, only : get_nstep
    use clmtype
    type(pft_type), pointer :: pptr  ! pointer to pft derived subtype

! ------------------------ local variables -----------------------------
    integer :: i,k,lat,n, p          !indices
    integer :: nstep              !model time step
! ----------------------------------------------------------------------

   ! Set pointers into derived type

     pptr => clm3%g%l%c%p

! Allocate dynamic memory if necessary

    if (.not. allocated(taux_ave)) then
       allocate (taux_ave( begpatch : endpatch ) ) ; taux_ave(:) = inf 
    endif
    if (.not. allocated(tauy_ave)) then
       allocate (tauy_ave( begpatch : endpatch ) ) ; tauy_ave(:) = inf 
    endif
    if (.not. allocated(lhflx_ave)) then
       allocate (lhflx_ave( begpatch : endpatch ) ); lhflx_ave(:) = inf 
    endif
    if (.not. allocated(shflx_ave)) then
       allocate (shflx_ave( begpatch : endpatch ) ); shflx_ave(:) = inf 
    endif
    if (.not. allocated(lwup_ave)) then
       allocate (lwup_ave( begpatch : endpatch ) ) ; lwup_ave(:) = inf 
    endif
    if (.not. allocated(qflx_ave)) then
       allocate (qflx_ave( begpatch : endpatch ) ) ; qflx_ave(:) = inf 
    endif
    if (.not. allocated(swabs_ave)) then
       allocate (swabs_ave( begpatch : endpatch ) ) ; swabs_ave(:) = inf 
    endif
!--------------------------wangln 2008-09-03 add carbon flux---------------------
    if( carbon_switch ) then
        if (.not. allocated(carbon_ave)) then
           allocate (carbon_ave( begpatch : endpatch ) ) ; carbon_ave(:) = 0.0 
        endif
    endif
!--------------------------wangln 2008-09-03 add carbon flux---------------------
#ifdef MOZART2

    if (.not. allocated(flx_mss_vrt_dst01_ave)) then
       allocate (flx_mss_vrt_dst01_ave( begpatch : endpatch ) ); flx_mss_vrt_dst01_ave(:) = 0.0
    endif
    if (.not. allocated(flx_mss_vrt_dst02_ave)) then
       allocate (flx_mss_vrt_dst02_ave( begpatch : endpatch ) ); flx_mss_vrt_dst02_ave(:) = 0.0
    endif
    if (.not. allocated(flx_mss_vrt_dst03_ave)) then
       allocate (flx_mss_vrt_dst03_ave( begpatch : endpatch ) ); flx_mss_vrt_dst03_ave(:) = 0.0
    endif
    if (.not. allocated(flx_mss_vrt_dst04_ave)) then
       allocate (flx_mss_vrt_dst04_ave( begpatch : endpatch ) ); flx_mss_vrt_dst04_ave(:) = 0.0
    endif

    if (.not. allocated(vlc_trb_dst01_ave)) then
       allocate (vlc_trb_dst01_ave( begpatch : endpatch ) ); vlc_trb_dst01_ave(:) = 0.0
    endif
    if (.not. allocated(vlc_trb_dst02_ave)) then
       allocate (vlc_trb_dst02_ave( begpatch : endpatch ) ); vlc_trb_dst02_ave(:) = 0.0
    endif
    if (.not. allocated(vlc_trb_dst03_ave)) then
       allocate (vlc_trb_dst03_ave( begpatch : endpatch ) ); vlc_trb_dst03_ave(:) = 0.0
    endif
    if (.not. allocated(vlc_trb_dst04_ave)) then
       allocate (vlc_trb_dst04_ave( begpatch : endpatch ) ); vlc_trb_dst04_ave(:) = 0.0
    endif

    if (.not. allocated(lnd_ram1_ave)) then
       allocate (lnd_ram1_ave( begpatch : endpatch ) ); lnd_ram1_ave(:) = 0.0
    endif

    if (.not. allocated(lnd_fv_ave)) then
       allocate (lnd_fv_ave( begpatch : endpatch ) ); lnd_fv_ave(:) = 0.0
    endif

    if (.not. allocated(vlc_trb_SO4_ave)) then
       allocate (vlc_trb_SO4_ave( begpatch : endpatch ) ); vlc_trb_SO4_ave(:) = 0.0
    endif

    if (.not. allocated(vlc_trb_NH4_ave)) then
       allocate (vlc_trb_NH4_ave( begpatch : endpatch ) ); vlc_trb_NH4_ave(:) = 0.0
    endif

    if (.not. allocated(vlc_trb_CB2_ave)) then
       allocate (vlc_trb_CB2_ave( begpatch : endpatch ) ); vlc_trb_CB2_ave(:) = 0.0
    endif

    if (.not. allocated(vlc_trb_SO2_ave)) then
       allocate (vlc_trb_SO2_ave( begpatch : endpatch ) ); vlc_trb_SO2_ave(:) = 0.0
    endif

    if (.not. allocated(flux_ISOP_ave)) then
       allocate (flux_ISOP_ave( begpatch : endpatch ) ); flux_ISOP_ave(:) = 0.0
    endif

    if (.not. allocated(flux_ACET_ave)) then
       allocate (flux_ACET_ave( begpatch : endpatch ) ); flux_ACET_ave(:) = 0.0
    endif

    if (.not. allocated(flux_C3H6_ave)) then
       allocate (flux_C3H6_ave( begpatch : endpatch ) ); flux_C3H6_ave(:) = 0.0
    endif

    if (.not. allocated(flux_C2H4_ave)) then
       allocate (flux_C2H4_ave( begpatch : endpatch ) ); flux_C2H4_ave(:) = 0.0
    endif

    if (.not. allocated(flux_OC2_ave)) then
       allocate (flux_OC2_ave( begpatch : endpatch ) ); flux_OC2_ave(:) = 0.0
    endif

    if (.not. allocated(flux_C10H16_ave)) then
       allocate (flux_C10H16_ave( begpatch : endpatch ) ); flux_C10H16_ave(:) = 0.0
    endif

#endif
! Determine output flux averaging interval

    nstep = get_nstep()
    if (dorecv) then
       icnt = 1
       if ( nstep==0 ) then
          ncnt = irad + 1
       else
          ncnt = irad
       endif
       rncnt = 1./ncnt
    endif

! Initial call of averaging interval, copy data to accumulators

    if (icnt == 1) then
       do k = begpatch,endpatch
          taux_ave(k)  = pptr%pmf%taux(k) 
          tauy_ave(k)  = pptr%pmf%tauy(k)
          lhflx_ave(k) = pptr%pef%eflx_lh_tot(k)
          shflx_ave(k) = pptr%pef%eflx_sh_tot(k)
          lwup_ave(k)  = pptr%pef%eflx_lwrad_out(k)
          qflx_ave(k)  = pptr%pwf%qflx_evap_tot(k)
          swabs_ave(k) = pptr%pef%fsa(k)

          if( carbon_switch ) carbon_ave(k) = pptr%pef%carbon(k)

#ifdef MOZART2
          flx_mss_vrt_dst01_ave(k) =  pptr%pdf%flx_mss_vrt_dst(k,1)
          flx_mss_vrt_dst02_ave(k) =  pptr%pdf%flx_mss_vrt_dst(k,2)
          flx_mss_vrt_dst03_ave(k) =  pptr%pdf%flx_mss_vrt_dst(k,3)
          flx_mss_vrt_dst04_ave(k) =  pptr%pdf%flx_mss_vrt_dst(k,4)
          vlc_trb_dst01_ave(k) =  pptr%pdf%vlc_trb_1(k)
          vlc_trb_dst02_ave(k) =  pptr%pdf%vlc_trb_2(k)
          vlc_trb_dst03_ave(k) =  pptr%pdf%vlc_trb_3(k)
          vlc_trb_dst04_ave(k) =  pptr%pdf%vlc_trb_4(k)
          lnd_ram1_ave(k)  =  pptr%pps%ram1(k)
          lnd_fv_ave(k)  =  pptr%pps%fv(k)
          vlc_trb_SO4_ave(k)  =  pptr%pdf%velocity_SO4(k)
          vlc_trb_NH4_ave(k)  =  pptr%pdf%velocity_NH4(k)
          vlc_trb_CB2_ave(k)  =  pptr%pdf%velocity_CB2(k)
          vlc_trb_SO2_ave(k)  =  pptr%pdf%velocity_SO2(k)

          flux_ISOP_ave(k)  =  pptr%pvf%vocflx_ISOP(k)
          flux_ACET_ave(k)  =  pptr%pvf%vocflx_ACET(k)
          flux_C3H6_ave(k)  =  pptr%pvf%vocflx_PRPE(k)
          flux_C2H4_ave(k)  =  pptr%pvf%vocflx_C2H4(k)
          flux_OC2_ave(k)  =  pptr%pvf%vocflx_OCPI(k)
          flux_C10H16_ave(k)  =  pptr%pvf%vocflx_MONO(k)

#endif
       end do
       
! Final call of averaging interval, complete averaging 

    else if (icnt == ncnt) then
       do k = begpatch,endpatch
          taux_ave (k) = rncnt * (taux_ave(k)  + pptr%pmf%taux(k)) 
          tauy_ave (k) = rncnt * (tauy_ave(k)  + pptr%pmf%tauy(k))
          lhflx_ave(k) = rncnt * (lhflx_ave(k) + pptr%pef%eflx_lh_tot(k))
          shflx_ave(k) = rncnt * (shflx_ave(k) + pptr%pef%eflx_sh_tot(k))
          lwup_ave (k) = rncnt * (lwup_ave(k)  + pptr%pef%eflx_lwrad_out(k))
          qflx_ave (k) = rncnt * (qflx_ave(k)  + pptr%pwf%qflx_evap_tot(k))
          swabs_ave(k) = rncnt * (swabs_ave(k) + pptr%pef%fsa(k))

          if( carbon_switch ) carbon_ave(k) = rncnt * ( carbon_ave(k) + pptr%pef%carbon(k) )

#ifdef MOZART2
          flx_mss_vrt_dst01_ave(k) = rncnt * (flx_mss_vrt_dst01_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,1) )
          flx_mss_vrt_dst02_ave(k) = rncnt * (flx_mss_vrt_dst02_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,2) )
          flx_mss_vrt_dst03_ave(k) = rncnt * (flx_mss_vrt_dst03_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,3) )
          flx_mss_vrt_dst04_ave(k) = rncnt * (flx_mss_vrt_dst04_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,4) )
          vlc_trb_dst01_ave(k) = rncnt * (vlc_trb_dst01_ave(k) + pptr%pdf%vlc_trb_1(k) )
          vlc_trb_dst02_ave(k) = rncnt * (vlc_trb_dst02_ave(k) + pptr%pdf%vlc_trb_2(k) )
          vlc_trb_dst03_ave(k) = rncnt * (vlc_trb_dst03_ave(k) + pptr%pdf%vlc_trb_3(k) )
          vlc_trb_dst04_ave(k) = rncnt * (vlc_trb_dst04_ave(k) + pptr%pdf%vlc_trb_4(k) )
          lnd_ram1_ave(k)  = rncnt * (lnd_ram1_ave(k)  + pptr%pps%ram1(k) )
          lnd_fv_ave(k)    = rncnt * (lnd_fv_ave(k)  + pptr%pps%fv(k) )
          vlc_trb_SO4_ave(k)  =  rncnt * (vlc_trb_SO4_ave(k) + pptr%pdf%velocity_SO4(k) )
          vlc_trb_NH4_ave(k)  =  rncnt * (vlc_trb_NH4_ave(k) + pptr%pdf%velocity_NH4(k) )
          vlc_trb_CB2_ave(k)  =  rncnt * (vlc_trb_CB2_ave(k) + pptr%pdf%velocity_CB2(k) )
          vlc_trb_SO2_ave(k)  =  rncnt * (vlc_trb_SO2_ave(k) + pptr%pdf%velocity_SO2(k) )

          flux_ISOP_ave(k)  =  rncnt * (flux_ISOP_ave(k) + pptr%pvf%vocflx_ISOP(k) )
          flux_ACET_ave(k)  =  rncnt * (flux_ACET_ave(k) + pptr%pvf%vocflx_ACET(k) )
          flux_C3H6_ave(k)  =  rncnt * (flux_C3H6_ave(k) + pptr%pvf%vocflx_PRPE(k) )
          flux_C2H4_ave(k)  =  rncnt * (flux_C2H4_ave(k) + pptr%pvf%vocflx_C2H4(k) )
          flux_OC2_ave(k)  =  rncnt * (flux_OC2_ave(k) + pptr%pvf%vocflx_OCPI(k) )
          flux_C10H16_ave(k)  =  rncnt * (flux_C10H16_ave(k) + pptr%pvf%vocflx_MONO(k) )

#endif

       end do

! Intermediate call, add data to accumulators

    else
       do k = begpatch,endpatch
          taux_ave (k) = taux_ave(k)  + pptr%pmf%taux(k) 
          tauy_ave(k)  = tauy_ave(k)  + pptr%pmf%tauy(k)
          lhflx_ave(k) = lhflx_ave(k) + pptr%pef%eflx_lh_tot(k)
          shflx_ave(k) = shflx_ave(k) + pptr%pef%eflx_sh_tot(k)
          lwup_ave(k)  = lwup_ave(k)  + pptr%pef%eflx_lwrad_out(k)
          qflx_ave(k)  = qflx_ave(k)  + pptr%pwf%qflx_evap_tot(k)
          swabs_ave(k) = swabs_ave(k) + pptr%pef%fsa(k)

          if( carbon_switch ) carbon_ave(k) = carbon_ave(k) + pptr%pef%carbon(k) 

#ifdef MOZART2
          flx_mss_vrt_dst01_ave(k) = flx_mss_vrt_dst01_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,1)
          flx_mss_vrt_dst02_ave(k) = flx_mss_vrt_dst02_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,2)
          flx_mss_vrt_dst03_ave(k) = flx_mss_vrt_dst03_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,3)
          flx_mss_vrt_dst04_ave(k) = flx_mss_vrt_dst04_ave(k) + pptr%pdf%flx_mss_vrt_dst(k,4)
          vlc_trb_dst01_ave(k) = vlc_trb_dst01_ave(k) + pptr%pdf%vlc_trb_1(k)
          vlc_trb_dst02_ave(k) = vlc_trb_dst02_ave(k) + pptr%pdf%vlc_trb_2(k)
          vlc_trb_dst03_ave(k) = vlc_trb_dst03_ave(k) + pptr%pdf%vlc_trb_3(k)
          vlc_trb_dst04_ave(k) = vlc_trb_dst04_ave(k) + pptr%pdf%vlc_trb_4(k)
          lnd_ram1_ave(k)  = lnd_ram1_ave(k)  + pptr%pps%ram1(k)
          lnd_fv_ave(k)    =  lnd_fv_ave(k)  + pptr%pps%fv(k)
          vlc_trb_SO4_ave(k)  =  vlc_trb_SO4_ave(k) + pptr%pdf%velocity_SO4(k)
          vlc_trb_NH4_ave(k)  =  vlc_trb_NH4_ave(k) + pptr%pdf%velocity_NH4(k)
          vlc_trb_CB2_ave(k)  =  vlc_trb_CB2_ave(k) + pptr%pdf%velocity_CB2(k)
          vlc_trb_SO2_ave(k)  =  vlc_trb_SO2_ave(k) + pptr%pdf%velocity_SO2(k)

          flux_ISOP_ave(k)  =  flux_ISOP_ave(k) + pptr%pvf%vocflx_ISOP(k)
          flux_ACET_ave(k)  =  flux_ACET_ave(k) + pptr%pvf%vocflx_ACET(k)
          flux_C3H6_ave(k)  =  flux_C3H6_ave(k) + pptr%pvf%vocflx_PRPE(k)
          flux_C2H4_ave(k)  =  flux_C2H4_ave(k) + pptr%pvf%vocflx_C2H4(k)
          flux_OC2_ave(k)  =  flux_OC2_ave(k) + pptr%pvf%vocflx_OCPI(k)
          flux_C10H16_ave(k)  =  flux_C10H16_ave(k) + pptr%pvf%vocflx_MONO(k)

#endif
       end do
    end if
    
! Increment counter

    icnt = icnt + 1
    
    return
  END SUBROUTINE csm_flxave

!===============================================================================

  SUBROUTINE compat_check_spval( spval, data, string )

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Check that the given piece of real data sent from the coupler is valid
! data and not the special data flag set by the coupler. This ensures that 
! the expected data is actually being sent by the coupler.
! 
! Method: 
! 
! Author: Mariana Vertenstein
! 
!-----------------------------------------------------------------------

!------------------ Arguments ------------------------------------------
    real(r8), intent(in) :: spval
    real(r8), intent(in) :: data
    character(len=*)     :: string
!-----------------------------------------------------------------------

    if ( spval == data )then
       write(6,*)'ERROR:(compat_check_spval) msg incompatibility'
       write(6,*)'ERROR: I expect to recieve the data type: ',string
       write(6,*)'from CPL, but all I got was the special data flag'
       write(6,*)'coupler must not be sending this data, you are'
       write(6,*)'running with an incompatable version of the coupler'
       call endrun
    end if
    return
  END SUBROUTINE compat_check_spval
  
!===============================================================================

  SUBROUTINE csm_compat(cpl_maj_vers, cpl_min_vers, expect_maj_vers, expect_min_vers)

!-----------------------------------------------------------------------
! Checks that the message recieved from the coupler is compatable
! with the type of message that I expect to recieve.  If the minor
! version numbers differ I print a warning message.  If the major
! numbers differ I abort since that means that the change is
! drastic enough that I can't run with the differences.
! Original Author: Erik Kluzek Dec/97
!-----------------------------------------------------------------------

!----------------------- Arguments -------------------------------------
    integer, intent(in) :: cpl_maj_vers    ! major version from coupler initial ibuffr array
    integer, intent(in) :: cpl_min_vers    ! minor version from coupler initial ibuffr array
    integer, intent(in) :: expect_maj_vers ! major version of the coupler I'm expecting
    integer, intent(in) :: expect_min_vers ! minor version of the coupler I'm expecting
!-----------------------------------------------------------------------

    write(6,*)'(cpl_COMPAT): This is revision: $Revision: 1.11.4.9.2.3 $'
    write(6,*)'              Tag: $Name: ccsm2_0_beta58 $'
    write(6,*)'              of the message compatability interface:'

    if ( cpl_min_vers /= expect_min_vers )then
       write(6,*) 'WARNING(cpl_compat):: Minor version of coupler ', &
            'messages different than expected: '
       write(6,*) 'The version of the coupler being used is: ',&
            cpl_min_vers
       write(6,*) 'The version I expect is:                  ',&
            expect_min_vers
    end if
    
    if ( cpl_maj_vers /= expect_maj_vers )then
       write(6,*) 'ERROR(cpl_compat):: Major version of coupler ', &
            'messages different than expected: '
       write(6,*) 'The version of the coupler being used is: ',&
            cpl_maj_vers
       write(6,*) 'The version I expect is:                  ',&
            expect_maj_vers
       call endrun
    end if
    
    return
  END SUBROUTINE csm_compat

!===============================================================================
!===============================================================================
  subroutine restart_coupler(nio, flag)

! !DESCRIPTION:
!  Read/write restart data needed for running in flux coupled mode

   use clm_varctl, only : csm_doflxave
   
! !ARGUMENTS:
   implicit none
   integer, intent(in) :: nio             !restart unit
   character(len=*), intent(in) :: flag   !"read" or "write"

! !LOCAL VARIABLES:
   logical :: flxave_res   !flux averaging flag read from restart file
   integer :: ier          !mpi return error code

!=-----------------------------------------------------------------------

   if (flag == 'read') then
      if (masterproc) then   
         read(nio) flxave_res
         read(nio) dosend
         if ((flxave_res .and. .not.csm_doflxave).or.(.not.flxave_res .and. csm_doflxave)) then
            write(6,*)'(RESTART_COUPLER): flxave value from namelist ',csm_doflxave, &
                   ' must be the same as flxave value from restart dataset ',flxave_res
            call endrun
         endif
         if (flxave_res .and. .not. dosend) then
            write(6,*)'(RESTART_COUPLER): assume that current flux coupled model ', &
                 'with flux averaging must stop on a time step where dosend (doalb) is true'
            call endrun
         endif
      endif
#if ( defined SPMD )
       call mpi_bcast(dosend    , 1, MPI_LOGICAL, 0, mpicom, ier)
       if (ier /= MPI_SUCCESS) then
           write(6,*) 'MPI BCAST ERROR: for dosend in restart_coupler'
           call endrun
       endif
       call mpi_bcast(flxave_res, 1, MPI_INTEGER, 0, mpicom, ier)
       if (ier /= MPI_SUCCESS) then
           write(6,*) 'MPI BCAST ERROR: for flxave_res in restart_coupler'
           call endrun
       endif
#endif
   endif

   if (flag == 'write') then
       if (masterproc) then
          write(nio) csm_doflxave
          write(nio) dosend
       endif
   endif

 end subroutine restart_coupler          

!===============================================================================

  real(r8) function global_sum(flux, spval)

!-----------------------------------------------------------------------
! Perform global integral
!-----------------------------------------------------------------------

    use clm_varsur, only : area                 !km^2

    real(r8), intent(in) :: flux(lsmlon,lsmlat) !W/m2, Kg/m2-s or N/m2
    real(r8), intent(in) :: spval               !points to not include in global sum

    integer :: i,j                              !indices

    global_sum = 0.
    do j = 1,lsmlat
       do i = 1,lsmlon
          if (flux(i,j) /= spval) then
             global_sum = global_sum + flux(i,j)*area(i,j)*1.e6
          endif
       end do
    end do

    return
  end function global_sum

!===============================================================================

#endif

end module clm_csmMod
