#include <misc.h>
#include <preproc.h>

module clm_varsur

!-----------------------------------------------------------------------
!BOP
!
! !MODULE: clm_varsur
!
! !DESCRIPTION:
! Module containing 2-d surface boundary data information
!
! !USES:
  use shr_kind_mod, only: r8 => shr_kind_r8
  use clm_varpar, only : lsmlon, lsmlat, nlevsoi
  use abortutils, only : endrun
  use clm_varpar, only : maxpatch_pft
!
! !PUBLIC TYPES:
  implicit none
  save
!
! land model grid
!
  integer :: numlon(lsmlat)             !longitude points for each latitude strip
  real(r8):: latixy(lsmlon,lsmlat)      !latitude of grid cell (degrees)
  real(r8):: longxy(lsmlon,lsmlat)      !longitude of grid cell (degrees)
  real(r8):: area(lsmlon,lsmlat)        !grid cell area (km**2)
  real(r8):: landarea                   !total land area for all gridcells (km^2)
  real(r8):: lats(lsmlat+1)             !grid cell latitude, southern edge (degrees)
  real(r8):: lonw(lsmlon+1,lsmlat)      !grid cell longitude, western edge (degrees)
  real(r8):: lsmedge(4)                 !North,East,South,West edges of grid (deg)
  logical :: pole_points                !true => grid has pole points
  logical :: fullgrid  = .true.         !true => no grid reduction towards poles
  logical :: offline_rdgrid             !true => read offline grid rather than creating it
!
! fractional land and mask
!
  integer  landmask(lsmlon,lsmlat)      !land mask: 1 = land. 0 = ocean
  real(r8) landfrac(lsmlon,lsmlat)      !fractional land
!
!-------------------------pft zhangyw 20090604--------------------
  integer  pft(lsmlon,lsmlat,maxpatch_pft)            !land mask: 1 = land. 0 = ocean
  real(r8) pctpft_lunit(lsmlon,lsmlat,maxpatch_pft)      !fractional land
#if (defined LANDUSE)
  real(r8) pctpft_lunit0(lsmlon,lsmlat,maxpatch_pft)      !fractional land
#endif

#if (defined MOZART2)
  real*8 :: AEF_SABINENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_OCIMENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_MYRCENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_MBO(lsmlon, lsmlat, 1)
  real*8 :: AEF_LIMONENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_ISOPRENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_CARENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_BETA_PINENE(lsmlon, lsmlat, 1)
  real*8 :: AEF_ALPHA_PINENE(lsmlon, lsmlat, 1)
#endif
!-------------------------pft zhangyw 20090604--------------------
!
! surface boundary data
!
  integer , allocatable :: soic2d(:,:)   !soil color
  real(r8), allocatable :: sand3d(:,:,:) !soil texture: percent sand
  real(r8), allocatable :: clay3d(:,:,:) !soil texture: percent clay
  real(r8), allocatable :: pctgla(:,:)   !percent of grid cell that is glacier
  real(r8), allocatable :: pctlak(:,:)   !percent of grid cell that is lake
  real(r8), allocatable :: pctwet(:,:)   !percent of grid cell that is wetland
  real(r8), allocatable :: pcturb(:,:)   !percent of grid cell that is urbanized
!! liwp 2017-02-09 added climate mean rainfall
#if (defined AVIM)
  real(r8), allocatable :: rain_climate(:,:,:)  ! climate 12 monthly rainfall
  real(r8), allocatable :: zstd(:,:)            ! standard deviation of subgrid topography 
#endif

!shixl, 20161117
#ifdef LAKE_DAI
! real(r8), allocatable :: lakedp(:,:)   !depth of lake
#endif
!
  logical, public :: all_pfts_on_srfdat = .false.  ! old format dataset is used by default
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: varsur_alloc    !allocates 2d surface data needed for initialization
  public :: varsur_dealloc  !deallocates 2d surface data needed for initialization
!
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!-----------------------------------------------------------------------

contains

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: varsur_alloc
!
! !INTERFACE:
  subroutine varsur_alloc
!
! !DESCRIPTION:
! Allocate dynamic memory for module variables
!
! !ARGUMENTS:
    implicit none
!
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!
! LOCAL VARIABLES:
    integer :: ier                    !error code
!-----------------------------------------------------------------------

    allocate (soic2d(lsmlon,lsmlat), &
!! liwp 2017-03-07 added the following lines for climate 12 monthly rainfall
#if (defined AVIM)
              rain_climate(lsmlon,lsmlat,12), &
              zstd(lsmlon,lsmlat), &
#endif

!shixl, 20161117
#ifdef LAKE_DAI
!             lakedp(lsmlon,lsmlat), &
#endif
!
              sand3d(lsmlon,lsmlat,nlevsoi), &
              clay3d(lsmlon,lsmlat,nlevsoi), &
              pctgla(lsmlon,lsmlat), &
              pctlak(lsmlon,lsmlat), &
              pctwet(lsmlon,lsmlat), &
              pcturb(lsmlon,lsmlat), stat=ier)
    if (ier /= 0) then
       write(6,*)'varsur_alloc(): allocation error'
       call endrun
    endif

  end subroutine varsur_alloc

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: varsur_dealloc
!
! !INTERFACE:
  subroutine varsur_dealloc
!
! !DESCRIPTION:
! Deallocate dynamic memory for module variables
!
! !ARGUMENTS:
    implicit none
!
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!-----------------------------------------------------------------------

!   deallocate (soic2d, &
!               sand3d, &
!               clay3d, &
!               pctgla, &
!               pctlak, &
!               pctwet, &
!               pcturb)

  end subroutine varsur_dealloc

end module clm_varsur
