#include <misc.h>
#include <preproc.h>

module lnd2atmMod

!-----------------------------------------------------------------------
!BOP
!
! !MODULE: lnd2atmMod
!
! !DESCRIPTION:
! Compute l2a component of gridcell derived type
!
! !USES:
  use shr_kind_mod, only: r8 => shr_kind_r8
!
! !PUBLIC TYPES:
  implicit none
  save
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: lnd2atm
!
! !REVISION HISTORY:
! 03-04-27 : Created by Mariana Vertenstein
!
!EOP
!
! !PRIVATE MEMBER FUNCTIONS:
  private :: makel2a
!-----------------------------------------------------------------------

contains

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: lnd2atm
!
! !INTERFACE: subroutine lnd2atm(init)
  subroutine lnd2atm(init)
!
! !DESCRIPTION:
! Compute l2a component of gridcell derived type
!
! !USES:
    use decompMod, only : get_proc_clumps, get_clump_bounds
!
! !ARGUMENTS:
    implicit none
    logical, optional, intent(in) :: init  ! if true=>only set a subset of arguments
!
! !REVISION HISTORY:
! Mariana Vertenstein: created 03/10-25
!
!EOP
!
! !LOCAL VARIABLES:
    integer :: nc              ! clump index
    integer :: nclumps         ! number of clumps on this processor
    integer :: begp, endp      ! per-proc beginning and ending pft indices
    integer :: begc, endc      ! per-proc beginning and ending column indices
    integer :: begl, endl      ! per-proc beginning and ending landunit indices
    integer :: begg, endg      ! per-proc gridcell ending gridcell indices
!------------------------------------------------------------------------

    ! Determine clump bounds for this processor

    nclumps = get_proc_clumps()

    ! Loop over clumps on this processor

!$OMP PARALLEL DO PRIVATE (nc,begg,endg,begl,endl,begc,endc,begp,endp)
!CSD$ PARALLEL DO PRIVATE (nc,begg,endg,begl,endl,begc,endc,begp,endp)
    do nc = 1,nclumps

       call get_clump_bounds(nc, begg, endg, begl, endl, begc, endc, begp, endp)

       if (present(init)) then
          call makel2a(begp, endp, begc, endc, begg, endg, init=.true.)
       else
          call makel2a(begp, endp, begc, endc, begg, endg)
       end if

    end do
!CSD$ END PARALLEL DO
!$OMP END PARALLEL DO

  end subroutine lnd2atm

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: makel2a
!
! !INTERFACE: subroutine lnd2atm(init)
  subroutine makel2a(lbp, ubp, lbc, ubc, lbg, ubg, init)
!
! !DESCRIPTION:
! Compute l2a component of gridcell derived type
!
! !USES:
    use shr_kind_mod, only: r8 => shr_kind_r8
    use clmtype
    use clm_varcon  , only : sb
!
! !ARGUMENTS:
    implicit none
    integer, intent(in) :: lbp, ubp   ! per-proc beginning and ending pft indices
    integer, intent(in) :: lbc, ubc   ! per-proc beginning and ending column indices
    integer, intent(in) :: lbg, ubg   ! per-proc gridcell ending gridcell indices
    logical, optional, intent(in) :: init  ! if true=>only set a subset of arguments
!
! !REVISION HISTORY:
! 03-04-27 : Created by Mariana Vertenstein
! 03-08-25 : Updated to vector data structure (Mariana Vertenstein)
!
!EOP
!
! !LOCAL VARIABLES:
    integer :: pi,p,c,l,g   ! indices
    real(r8):: wt           ! temporary weight
    type(gridcell_type), pointer :: gptr  ! pointer to gridcell derived subtype
    type(landunit_type), pointer :: lptr  ! pointer to landunit derived subtype
    type(column_type)  , pointer :: cptr  ! pointer to column derived subtype
    type(pft_type)     , pointer :: pptr  ! pointer to pft derived subtype
!------------------------------------------------------------------------

    ! Set pointers into derived type

    gptr => clm3%g
    lptr => clm3%g%l
    cptr => clm3%g%l%c
    pptr => clm3%g%l%c%p

    ! Initialize gridcell land->atm components before grid averaging

    gptr%l2as%albd(lbg:ubg,:) = 0.
    gptr%l2as%albi(lbg:ubg,:) = 0.
    gptr%l2as%t_ref2m(lbg:ubg) = 0.
    gptr%l2as%q_ref2m(lbg:ubg) = 0.
    gptr%l2as%h2osno(lbg:ubg) = 0.
    gptr%l2af%taux(lbg:ubg) = 0.
    gptr%l2af%tauy(lbg:ubg) = 0.
    gptr%l2af%eflx_lh_tot(lbg:ubg) = 0.
    gptr%l2af%eflx_sh_tot(lbg:ubg) = 0.
    gptr%l2af%eflx_lwrad_out(lbg:ubg) = 0.
    gptr%l2af%qflx_evap_tot(lbg:ubg) = 0.
    gptr%l2af%fsa(lbg:ubg) = 0.
    gptr%l2as%t_rad(lbg:ubg) = 0.

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------
    gptr%l2as%u_10m(lbg:ubg) = 0.
    gptr%l2as%v_10m(lbg:ubg) = 0.

!-----------------wangln 2009-08-15 add 5 variables for IPCC AR5----------------------------

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)

    gptr%l2af%carbon(lbg:ubg) = 0.

    gptr%l2af%flx_mss_vrt_dst01(lbg:ubg) = 0.
    gptr%l2af%flx_mss_vrt_dst02(lbg:ubg) = 0.
    gptr%l2af%flx_mss_vrt_dst03(lbg:ubg) = 0.
    gptr%l2af%flx_mss_vrt_dst04(lbg:ubg) = 0.

    gptr%l2af%vlc_trb_dst01(lbg:ubg) = 0.
    gptr%l2af%vlc_trb_dst02(lbg:ubg) = 0.
    gptr%l2af%vlc_trb_dst03(lbg:ubg) = 0.
    gptr%l2af%vlc_trb_dst04(lbg:ubg) = 0.

    gptr%l2af%lnd_ram1(lbg:ubg) = 0.
    gptr%l2af%lnd_fv(lbg:ubg) = 0.

    gptr%l2af%vlc_trb_SO4(lbg:ubg) = 0.
    gptr%l2af%vlc_trb_NH4(lbg:ubg) = 0.
    gptr%l2af%vlc_trb_CB2(lbg:ubg) = 0.
    gptr%l2af%vlc_trb_SO2(lbg:ubg) = 0.

    gptr%l2af%flux_ISOP(lbg:ubg) = 0.
    gptr%l2af%flux_ACET(lbg:ubg) = 0.
    gptr%l2af%flux_C3H6(lbg:ubg) = 0.
    gptr%l2af%flux_C2H4(lbg:ubg) = 0.
    gptr%l2af%flux_OC2(lbg:ubg) = 0.
    gptr%l2af%flux_C10H16(lbg:ubg) = 0.

#endif

!--------------------------------------------
    ! Compute gridcell averages. Note that gridcell value for the
    ! radiative temperature (l2as%t_rad) must be computed after the
    ! gridcell average of eflx_lwrad_out is computed.

   if (present(init)) then

      if (init) then

         do c = lbc,ubc
            g = cptr%gridcell(c)
            wt = cptr%wtgcell(c)
            gptr%l2as%h2osno(g) = gptr%l2as%h2osno(g) + cptr%cws%h2osno(c)/1000. * wt
         end do

         do pi = 1,maxpatch
!dir$ concurrent
!cdir nodep
            do g = lbg,ubg
               if ( pi <=  gptr%npfts(g) ) then
                  p = gptr%pfti(g) + pi - 1
                  wt = pptr%wtgcell(p)
                  gptr%l2as%albd(g,1)         = gptr%l2as%albd(g,1)         + pptr%pps%albd(p,1) * wt
                  gptr%l2as%albd(g,2)         = gptr%l2as%albd(g,2)         + pptr%pps%albd(p,2) * wt
                  gptr%l2as%albi(g,1)         = gptr%l2as%albi(g,1)         + pptr%pps%albi(p,1) * wt
                  gptr%l2as%albi(g,2)         = gptr%l2as%albi(g,2)         + pptr%pps%albi(p,2) * wt
                  gptr%l2af%eflx_lwrad_out(g) = gptr%l2af%eflx_lwrad_out(g) + pptr%pef%eflx_lwrad_out(p) * wt

#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
               gptr%l2af%carbon(g)   = gptr%l2af%carbon(g)  + pptr%pef%carbon(p) * wt
                                                                         ! gC/m**2/s
               gptr%l2af%flx_mss_vrt_dst01(g)    = gptr%l2af%flx_mss_vrt_dst01(g)  + pptr%pdf%flx_mss_vrt_dst(p,1) * wt
               gptr%l2af%flx_mss_vrt_dst02(g)    = gptr%l2af%flx_mss_vrt_dst02(g)  + pptr%pdf%flx_mss_vrt_dst(p,2) * wt
               gptr%l2af%flx_mss_vrt_dst03(g)    = gptr%l2af%flx_mss_vrt_dst03(g)  + pptr%pdf%flx_mss_vrt_dst(p,3) * wt
               gptr%l2af%flx_mss_vrt_dst04(g)    = gptr%l2af%flx_mss_vrt_dst04(g)  + pptr%pdf%flx_mss_vrt_dst(p,4) * wt

               gptr%l2af%vlc_trb_dst01(g)    = gptr%l2af%vlc_trb_dst01(g)  + pptr%pdf%vlc_trb_1(p) * wt
               gptr%l2af%vlc_trb_dst02(g)    = gptr%l2af%vlc_trb_dst02(g)  + pptr%pdf%vlc_trb_2(p) * wt
               gptr%l2af%vlc_trb_dst03(g)    = gptr%l2af%vlc_trb_dst03(g)  + pptr%pdf%vlc_trb_3(p) * wt
               gptr%l2af%vlc_trb_dst04(g)    = gptr%l2af%vlc_trb_dst04(g)  + pptr%pdf%vlc_trb_4(p) * wt

               gptr%l2af%lnd_ram1(g)    = gptr%l2af%lnd_ram1(g)  + pptr%pps%ram1(p) * wt
               gptr%l2af%lnd_fv(g)      = gptr%l2af%lnd_fv(g)    + pptr%pps%fv(p) * wt

               gptr%l2af%vlc_trb_SO4(g)    = gptr%l2af%vlc_trb_SO4(g)  + pptr%pdf%velocity_SO4(p) * wt
               gptr%l2af%vlc_trb_NH4(g)    = gptr%l2af%vlc_trb_NH4(g)  + pptr%pdf%velocity_NH4(p) * wt
               gptr%l2af%vlc_trb_CB2(g)    = gptr%l2af%vlc_trb_CB2(g)  + pptr%pdf%velocity_CB2(p) * wt
               gptr%l2af%vlc_trb_SO2(g)    = gptr%l2af%vlc_trb_SO2(g)  + pptr%pdf%velocity_SO2(p) * wt

               gptr%l2af%flux_ISOP(g)    = gptr%l2af%flux_ISOP(g)  + pptr%pvf%vocflx_ISOP(p) * wt
               gptr%l2af%flux_ACET(g)    = gptr%l2af%flux_ACET(g)  + pptr%pvf%vocflx_ACET(p) * wt
               gptr%l2af%flux_C3H6(g)    = gptr%l2af%flux_C3H6(g)  + pptr%pvf%vocflx_PRPE(p) * wt
               gptr%l2af%flux_C2H4(g)    = gptr%l2af%flux_C2H4(g)  + pptr%pvf%vocflx_C2H4(p) * wt
               gptr%l2af%flux_OC2(g)    = gptr%l2af%flux_OC2(g)  + pptr%pvf%vocflx_OCPI(p) * wt
               gptr%l2af%flux_C10H16(g)    = gptr%l2af%flux_C10H16(g)  + pptr%pvf%vocflx_MONO(p) * wt
#endif

               end if
            end do
         end do

!dir$ concurrent
!cdir nodep
         do g = lbg,ubg
            gptr%l2as%t_rad(g) = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(g)/sb))
         end do
      end if

   else

      do c = lbc,ubc
         g = cptr%gridcell(c)
         wt = cptr%wtgcell(c)
         gptr%l2as%h2osno(g) = gptr%l2as%h2osno(g) + cptr%cws%h2osno(c)/1000. * wt
      end do

      do pi = 1,maxpatch
!dir$ concurrent
!cdir nodep
         do g = lbg,ubg
            if ( pi <=  gptr%npfts(g) ) then
               p = gptr%pfti(g) + pi - 1
               wt = pptr%wtgcell(p)
               gptr%l2as%albd(g,1)         = gptr%l2as%albd(g,1)         + pptr%pps%albd(p,1) * wt
               gptr%l2as%albd(g,2)         = gptr%l2as%albd(g,2)         + pptr%pps%albd(p,2) * wt
               gptr%l2as%albi(g,1)         = gptr%l2as%albi(g,1)         + pptr%pps%albi(p,1) * wt
               gptr%l2as%albi(g,2)         = gptr%l2as%albi(g,2)         + pptr%pps%albi(p,2) * wt
               gptr%l2as%t_ref2m(g)        = gptr%l2as%t_ref2m(g)        + pptr%pes%t_ref2m(p) * wt
               gptr%l2as%q_ref2m(g)        = gptr%l2as%q_ref2m(g)        + pptr%pes%q_ref2m(p) * wt
               gptr%l2af%taux(g)           = gptr%l2af%taux(g)           + pptr%pmf%taux(p) * wt
               gptr%l2af%tauy(g)           = gptr%l2af%tauy(g)           + pptr%pmf%tauy(p) * wt
               gptr%l2af%eflx_lh_tot(g)    = gptr%l2af%eflx_lh_tot(g)    + pptr%pef%eflx_lh_tot(p) * wt
               gptr%l2af%eflx_sh_tot(g)    = gptr%l2af%eflx_sh_tot(g)    + pptr%pef%eflx_sh_tot(p) * wt
               gptr%l2af%eflx_lwrad_out(g) = gptr%l2af%eflx_lwrad_out(g) + pptr%pef%eflx_lwrad_out(p) * wt
               gptr%l2af%qflx_evap_tot(g)  = gptr%l2af%qflx_evap_tot(g)  + pptr%pwf%qflx_evap_tot(p) * wt
               gptr%l2af%fsa(g)            = gptr%l2af%fsa(g)            + pptr%pef%fsa(p) * wt
!!!------------------zhangyw 10m wind 20090814---------------------------------------------------------
               gptr%l2as%u_10m(g)          = gptr%l2as%u_10m(g)          + pptr%pes%u_10m(p) * wt
               gptr%l2as%v_10m(g)          = gptr%l2as%v_10m(g)          + pptr%pes%v_10m(p) * wt
!!!------------------zhangyw 10m wind 20090814---------------------------------------------------------
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
               gptr%l2af%carbon(g)   = gptr%l2af%carbon(g)  + pptr%pef%carbon(p) * wt
                                                                                       ! gC/m**2/s 
               gptr%l2af%flx_mss_vrt_dst01(g)    = gptr%l2af%flx_mss_vrt_dst01(g)  + pptr%pdf%flx_mss_vrt_dst(p,1) * wt
               gptr%l2af%flx_mss_vrt_dst02(g)    = gptr%l2af%flx_mss_vrt_dst02(g)  + pptr%pdf%flx_mss_vrt_dst(p,2) * wt
               gptr%l2af%flx_mss_vrt_dst03(g)    = gptr%l2af%flx_mss_vrt_dst03(g)  + pptr%pdf%flx_mss_vrt_dst(p,3) * wt
               gptr%l2af%flx_mss_vrt_dst04(g)    = gptr%l2af%flx_mss_vrt_dst04(g)  + pptr%pdf%flx_mss_vrt_dst(p,4) * wt

               gptr%l2af%vlc_trb_dst01(g)    = gptr%l2af%vlc_trb_dst01(g)  + pptr%pdf%vlc_trb_1(p) * wt
               gptr%l2af%vlc_trb_dst02(g)    = gptr%l2af%vlc_trb_dst02(g)  + pptr%pdf%vlc_trb_2(p) * wt
               gptr%l2af%vlc_trb_dst03(g)    = gptr%l2af%vlc_trb_dst03(g)  + pptr%pdf%vlc_trb_3(p) * wt
               gptr%l2af%vlc_trb_dst04(g)    = gptr%l2af%vlc_trb_dst04(g)  + pptr%pdf%vlc_trb_4(p) * wt

               gptr%l2af%lnd_ram1(g)    = gptr%l2af%lnd_ram1(g)  + pptr%pps%ram1(p) * wt
               gptr%l2af%lnd_fv(g)      = gptr%l2af%lnd_fv(g)    + pptr%pps%fv(p) * wt

               gptr%l2af%vlc_trb_SO4(g)    = gptr%l2af%vlc_trb_SO4(g)  + pptr%pdf%velocity_SO4(p) * wt
               gptr%l2af%vlc_trb_NH4(g)    = gptr%l2af%vlc_trb_NH4(g)  + pptr%pdf%velocity_NH4(p) * wt
               gptr%l2af%vlc_trb_CB2(g)    = gptr%l2af%vlc_trb_CB2(g)  + pptr%pdf%velocity_CB2(p) * wt
               gptr%l2af%vlc_trb_SO2(g)    = gptr%l2af%vlc_trb_SO2(g)  + pptr%pdf%velocity_SO2(p) * wt

               gptr%l2af%flux_ISOP(g)    = gptr%l2af%flux_ISOP(g)  + pptr%pvf%vocflx_ISOP(p) * wt
               gptr%l2af%flux_ACET(g)    = gptr%l2af%flux_ACET(g)  + pptr%pvf%vocflx_ACET(p) * wt
               gptr%l2af%flux_C3H6(g)    = gptr%l2af%flux_C3H6(g)  + pptr%pvf%vocflx_PRPE(p) * wt
               gptr%l2af%flux_C2H4(g)    = gptr%l2af%flux_C2H4(g)  + pptr%pvf%vocflx_C2H4(p) * wt
               gptr%l2af%flux_OC2(g)    = gptr%l2af%flux_OC2(g)  + pptr%pvf%vocflx_OCPI(p) * wt
               gptr%l2af%flux_C10H16(g)    = gptr%l2af%flux_C10H16(g)  + pptr%pvf%vocflx_MONO(p) * wt

#endif
            end if
         end do
      end do

!dir$ concurrent
!cdir nodep
      do g = lbg,ubg
#if (defined NIE_BLDAS_LST) || (defined NIE_BLDAS_SM)
		 if(gptr%l2af%eflx_lwrad_out(g).gt.0.)then
		   gptr%l2as%t_rad(g) = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(g)/sb))
         else
		   if(gptr%l2af%eflx_lwrad_out(g-1).gt.0.)then
		     gptr%l2as%t_rad(g) = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(g-1)/sb))
		   elseif(gptr%l2af%eflx_lwrad_out(g-2).gt.0.)then
		     gptr%l2as%t_rad(g) = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(g-2)/sb))
		   elseif(gptr%l2af%eflx_lwrad_out(g-3).gt.0.)then
		     gptr%l2as%t_rad(g) = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(g-3)/sb))
		   else
		   endif
		 endif
#else
         gptr%l2as%t_rad(g) = sqrt(sqrt(gptr%l2af%eflx_lwrad_out(g)/sb))
#endif
      end do

   end if

 end subroutine makel2a

end module lnd2atmMod
