#include <misc.h>
#include <preproc.h>

#if (defined NIE_BLDAS_LST)

module obsdrvMod_grndt

!-----------------------------------------------------------------------
!BOP
!
! !MODULE: obsdrvMod_grndt
!
! !DESCRIPTION:
! Read and generate land LST grid data at model resolution
!
! !USES:
  use shr_kind_mod , only : r8 => shr_kind_r8
  use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_PSTD
  use clm_varpar   , only : lsmlon, lsmlat
  use abortutils   , only: endrun
#if (defined SPMD)
  use spmdMod      , only : masterproc, mpicom, MPI_REAL8, MPI_INTEGER
#else
  use spmdMod      , only : masterproc
#endif
!
! !PUBLIC TYPES:
  implicit none
  save
!
! !PUBLIC MEMBER FUNCTIONS:
  public :: obsdrv_grndt       ! read observation data  
  public :: obs_getgrid_grndt  ! read observation grid  
!
! !REVISION HISTORY:
! Created by Suping Nie
!
!EOP
!
! PRIVATE MEMBER FUNCTIONS:
  private :: obs_openfile  ! open observation forcing netCDF file
  private :: obs_readdata  ! read observation forcing data
  private :: obsinterpa2si ! initialize obs->land model interpolation
  private :: obsinterpa2s     ! area average fields from observation to surface grid
!
! PRIVATE TYPES:
  private
!
! logical variables for file manipuation
!
  logical :: open_data=.true.             !true => open data file (first tstep of the run or month)
  logical :: allocated_data=.false.       !true => allocate dynamic data
!
! atmospheric grid data
!
  integer  :: atmlon                      !number of atm longitudes
  integer  :: atmlat                      !number of atm latitudes
  real(r8) :: edge_a(4)                   !N,E,S,W edges of atm grid
  integer , allocatable :: numlon_a(:)    !number of lon points at each lat
  real(r8), allocatable :: latixy_a(:,:)  !latitude of grid cell (degrees)
  real(r8), allocatable :: longxy_a(:,:)  !longitude of grid cell (degrees)

  real(r8), allocatable :: x(:,:,:)            !temp. array in which atm data is stored
  real(r8), allocatable :: a_grndtxy_a (:,:)   !FY3 ground temperate data in ascending orbit (K)
  real(r8), allocatable :: d_grndtxy_a (:,:)   !FY3 ground temperate data in decending orbit (K)
  real(r8), allocatable :: at_grndtxy_a (:,:)  !UTC time for FY3 ground temperate data in ascending orbit (hour)
  real(r8), allocatable :: dt_grndtxy_a (:,:)  !UTC time for FY3 ground temperate data in decending orbit (hour)
!
! atmospheric forcing variables on land model grid
!

  real(r8) :: a_grndtxy (lsmlon,lsmlat)         !FY3 ground temperate data in ascending orbit (K)
  real(r8) :: d_grndtxy (lsmlon,lsmlat)         !FY3 ground temperate data in ascending orbit (K)
  real(r8) :: at_grndtxy (lsmlon,lsmlat)        !UTC time for FY3 ground temperate data in ascending orbit (hour)
  real(r8) :: dt_grndtxy (lsmlon,lsmlat)        !UTC time for FY3 ground temperate data in ascending orbit (hour)


! atmosphere grid to land model surface grid mapping for each land grid cell:

  integer, parameter :: mxovr =40          !maximum number of overlapping cells
  integer :: novr_a2s(lsmlon,lsmlat)       !number    of overlapping atm cells
  integer :: iovr_a2s(lsmlon,lsmlat,mxovr) !lon index of overlapping atm cells
  integer :: jovr_a2s(lsmlon,lsmlat,mxovr) !lat index of overlapping atm cells
  real(r8):: wovr_a2s(lsmlon,lsmlat,mxovr) !weight    of overlapping atm cells
!
! file netCDF id's
!
  integer :: ncid                !netCDF dataset id
  integer :: nvar                !number of variables in the data file
  integer :: nlon                !number of atm longitude points
  integer :: nlat                !number of atm latitude points
  integer :: ntim                !number of atm time slices per data file
  character(len=8) :: varnam(99) !variable names of atm. fields

!2017.9.1#if (defined PERGRO)
!2017.9.1  logical :: do_perturb = .true. !if true, perturb tbot
!2017.9.1#endif
!-----------------------------------------------------------------------

contains

!=======================================================================

!------------------------------------------------------------------------
!BOP
!
! !IROUTINE: obsdrv
!
! !INTERFACE:
  subroutine obsdrv_grndt(nstep)    
!
! !DESCRIPTION:
! This code reads in atmospheric fields from an input file and generates
! the required atmospheric forcing. These data files have [atmmin] minute
! average data for each month. Input data files are named in month-year
! format (e.g., 09-0001 contains 240 3-hour time slices of data, 30*8, for
! September of year one). The model will cycle through however many full
! years of data are available [pyr]. At least one full year of data is
! necessary for cycling. The model may start on any base date, as long as
! this date corresponds to an existing data file. A run need not be an
! exact multiple of a year.
!
! ============
! Data format:
! ============
! Data format is netCDF with dimensions longitude x latitude
! for each time slice and field. Variable names can be as in above list
! or can be reset to desired names using [fldlst] in code below.
!
! ===============
! Namelist input:
! ===============
! character*256 landobsdir = directory for input land obs data files (can be Mass Store)
!
! !USES:
    use nanMod
    use clmtype
    use decompMod   , only : get_proc_bounds
!--
    use clm_varctl  , only : landobsdir, pertlim
!--
    use clm_varcon  , only : rair, cpair, po2, pco2, tcrit
    use time_manager, only : get_step_size, get_curr_calday, get_curr_date
    use fileutils   , only : getfil
!
! !ARGUMENTS:
    implicit none
    integer, intent(in) :: nstep    !current time step
!
! !REVISION HISTORY:
! Created by Suping Nie
!
!EOP
!
! LOCAL VARIABLES:
    integer :: i,j,k,g                !indices
    integer :: itimlast               !last time index used in atmrd
    real(r8):: calday                 !calendar day at Greenwich (1.00 -> 365.99)
    integer :: kda                    !day (1 -> 31)
    integer :: kmo                    !month (1 -> 12)
    integer :: kyr                    !year (0 -> ...)
    integer :: ksec                   !current seconds of current date (0 -> 86400)
    integer :: mcdate                 !current date in integer format [yyyymmdd]
    integer :: mcsec                  !current time of day [seconds]
    integer :: dtime                  !time step size
    integer :: minpday = 1440         !minutes per day
    integer :: secpmin = 60           !seconds per minute
    integer, SAVE :: itim             !time index used in atmrd
    integer, SAVE :: atmmin           !temporal resolution of atm data (in minutes)
    character(len=256), SAVE :: locfn !full file name in case atmdir is in MSS
!2017.9.1#if (defined PERGRO)
!2017.9.1    real(r8) :: pertval
!2017.9.1#endif
    real(r8):: coefb        ! Slope of "Alta" expression for dependence of flfall on temp
    real(r8):: coefa        ! Offset of  of "Alta" expression for dependence of flfall on temp
    integer :: begp, endp   ! per-proc beginning and ending pft indices
    integer :: begc, endc   ! per-proc beginning and ending column indices
    integer :: begl, endl   ! per-proc beginning and ending landunit indices
    integer :: begg, endg   ! per-proc gridcell ending gridcell indices
    type(gridcell_type), pointer :: gptr  ! pointer to gridcell derived subtype
    real(r8), pointer :: forc_obs_grndt(:,:)     !observed ground temperature (K) 
!------------------------------------------------------------------------

    ! Determine necessary indices

    call get_proc_bounds(begg, endg, begl, endl, begc, endc, begp, endp)

    ! Set pointers into derived type

    gptr => clm3%g
    forc_obs_grndt         => clm3%g%a2ls%forc_obs_grndt   


    ! -----------------------------------------------------------------
    ! Open netcdf file and read data every [atmmin] minutes
    ! -----------------------------------------------------------------

    ! Calendar information for current [nstep]

    dtime = get_step_size()
    calday = get_curr_calday()
    call get_curr_date(kyr, kmo, kda, mcsec)
    mcdate = kyr*10000 + kmo*100 + kda

    ! If 1st tstep of the run or of the month, then enter next if-block
    ! Rest flag to open file and set flag to read data

    if ( kda == 1 .and. mcsec == 300) then
write(*,*)'in obsdrv_grndt.f90 before if (kda == 1 .and. mcsec == 300 open_data=',open_data
       open_data = .true.    
    endif

    locfn = " "
    if (open_data) then
       call obs_openfile (kda, kmo, kyr, locfn, itim, atmmin)
    endif

    ! Calculate time index

    itimlast = itim
    itim = 1 + ((kda - 1)*minpday + (mcsec - dtime)/secpmin)/atmmin

    if (dtime == int(atmmin*secpmin)) then
       if (kda == 1 .and. mcsec == 0) itim = itimlast+1
    else
       if (kda == 1 .and. mcsec == 0) itim = itimlast
    endif

    ! Determine if new data is to be read

    if (open_data .or. mod(nstep-1,atmmin*secpmin/dtime) ==0 ) then

       ! Read data for current time slice

       if (masterproc) then

          write (6,*)
          write (6,'(72a1)') ("-",i=1,60)
          write (6,*)'nstep= ',nstep,' date= ',mcdate,' sec= ',mcsec
          if ( len_trim(locfn) > 0 )then
             write (6,*)'OBS: attempting to read obs data from ',trim(locfn)
          end if
          write (6,'(72a1)') ("-",i=1,60)
          write (6,*)
       endif

       call obs_readdata (locfn, kmo, itim)

       ! Map 2d atmospheric fields from atmospheric grid to land model surface grid.
       ! Area-average absolute value of winds (i.e., regardless of
       ! direction) since land model cares about magnitude not direction.
       ! Then need to adjust resultant stresses for direction of wind.

       call obsinterpa2s (a_grndtxy_a   , a_grndtxy   , d_grndtxy_a   , d_grndtxy , & 
	                      at_grndtxy_a   , at_grndtxy   , dt_grndtxy_a   , dt_grndtxy )

       ! Map atmospheric fields to force land model: [lsmlon] x [lsmlat] grid ->
       ! [numland] vector of land points -> [numpatch] vector of subgrid patches

!--------------------------------------------------------------------------------------------

!dir$ concurrent
!cdir nodep
       do g = begg, endg
          i = gptr%ixy(g)
          j = gptr%jxy(g)
          forc_obs_grndt(g,1) = a_grndtxy(i,j) 
          forc_obs_grndt(g,2) = d_grndtxy(i,j)
          forc_obs_grndt(g,3) = at_grndtxy(i,j) 
          forc_obs_grndt(g,4) = dt_grndtxy(i,j)
       end do

    end if

    ! Reset open_data

!---  2018.1.31 пǳĹؼ
    if (open_data) then
       open_data = .false.    !reset to false
!    elseif (kda == 1 .and. mcsec == 0) then
    elseif (kda == 1 .and. mcsec == 300) then
       open_data = .true.     !for next time step
    endif
!---  2018.1.31 пǳĹؼ

  end subroutine obsdrv_grndt

!------------------------------------------------------------------------
!BOP
!
! !IROUTINE: obs_getgrid
!
! !INTERFACE:
  subroutine obs_getgrid_grndt()
!
! !DESCRIPTION:
! Read atmospheric grid
!
! !USES:
    use nanMod
    use clm_varctl  , only : landobsdir
    use clm_varcon  , only : rair, cpair, po2, pco2
    use clm_varsur  , only : numlon, longxy, latixy, lsmedge
    use fileutils   , only : getfil
    use time_manager, only : get_curr_date
    use ncdio
!
! !ARGUMENTS:
    implicit none
!
! !REVISION HISTORY:
! Created by Suping Nie
!
!EOP
!
! LOCAL VARIABLES:
    integer :: kda                !day (1 -> 31)
    integer :: kmo                !month (1 -> 12)
    integer :: kyr                !year (0 -> ...)
    integer :: ksec               !current seconds of current date (0 -> 86400)
    integer :: mcsec              !current time of day [seconds]
    character(len=  7) :: ext     !month-year extension, e.g., 01-0005
    character(len=256) :: filenam !full file name, atmdir + ext
    character(len=256) :: locfn   !full file name in case atmdir is in MSS
    logical :: lexist             !true => file exists, used when looking for a file
    integer :: dimid              !netCDF dimension id
    integer :: varid              !netCDF variable id
    integer :: ier                !error status
    character(len=32) :: subname = 'obs_getgrid_grndt'
!------------------------------------------------------------------------

    ! ----------------------------------------------------------------------
    ! Read offline grid data and allocate dynamic memory
    ! ----------------------------------------------------------------------

    if (masterproc) then

       ! Build [month]-[year] extension for file name to be read
       ! append extension to path name to get full file name

       call get_curr_date(kyr, kmo, kda, mcsec)
       write (ext,'(i4.4,"-",i2.2)') kyr,kmo

       filenam = trim(landobsdir) // '/n2007-01_FY3C_LST_obs.nc'

       call getfil(filenam, locfn, 1)
       inquire (file = locfn, exist = lexist)
       if (.not. lexist) then
          write(6,*) 'OBS_GETGRID error: could not find land obs datafile' 
          call endrun
       endif

       ! Open netCDF data file and get lengths of lat,lon,time dimensions

       call check_ret(nf_open (locfn, nf_nowrite, ncid), subname)

       call check_ret(nf_inq_dimid  (ncid, 'lon', dimid), subname)
       call check_ret(nf_inq_dimlen (ncid, dimid, atmlon), subname)

       call check_ret(nf_inq_dimid  (ncid, 'lat', dimid), subname)
       call check_ret(nf_inq_dimlen (ncid, dimid, atmlat), subname)

       call check_ret(nf_inq_dimid  (ncid, 'time', dimid), subname)
       call check_ret(nf_inq_dimlen (ncid, dimid, ntim), subname)

       if (ntim == 0) then
          write (6,*) 'OBS_GETGRID error: zero input time slices' 
          call endrun
       end if

    endif  !end of if-masterproc block
#if (defined SPMD)
    call mpi_bcast (atmlon, 1, MPI_INTEGER, 0, mpicom, ier)
    call mpi_bcast (atmlat, 1, MPI_INTEGER, 0, mpicom, ier)
#endif

    ! Allocate space for dynamic variables

    if (.not. allocated_data) then
       if (masterproc) write(6,*)' OBS_GETRID: allocating dynamic space' 

       allocate (numlon_a(atmlat), &
                 latixy_a(atmlon,atmlat), &
                 longxy_a(atmlon,atmlat), &
                 a_grndtxy_a(atmlon,atmlat), &
                 d_grndtxy_a(atmlon,atmlat), &
                 at_grndtxy_a(atmlon,atmlat), &
                 dt_grndtxy_a(atmlon,atmlat), &
                 x(atmlon,atmlat,4), stat=ier) 

       if (ier /= 0) then
          write (6,*) 'obs_getgrid(): allocation error'
          call endrun
       end if
       allocated_data = .true.
    endif

    ! Extract atmospheric data grid information and close file

    if (masterproc) then

       call check_ret(nf_inq_varid(ncid, 'EDGEN', varid), subname)
       call check_ret(nf_get_var_double(ncid, varid, edge_a(1)), subname)

       call check_ret(nf_inq_varid(ncid, 'EDGEE', varid), subname)
       call check_ret(nf_get_var_double(ncid, varid, edge_a(2)), subname)

       call check_ret(nf_inq_varid(ncid, 'EDGES', varid), subname)
       call check_ret(nf_get_var_double(ncid, varid, edge_a(3)), subname)

       call check_ret(nf_inq_varid(ncid, 'EDGEW', varid), subname)
       call check_ret(nf_get_var_double(ncid, varid, edge_a(4)), subname)

       call check_ret(nf_inq_varid(ncid, 'LONGXY', varid), subname)
       call check_ret(nf_get_var_double(ncid, varid, longxy_a), subname)

       call check_ret(nf_inq_varid(ncid,'LATXY', varid), subname)  
       call check_ret(nf_get_var_double(ncid, varid, latixy_a), subname)

       call check_ret(nf_close (ncid), subname)
       write (6,*) 'OBS_GETGRID: closing data for ',trim(locfn) 

    end if !end of if-masterproc block

#if (defined SPMD)
    call mpi_bcast (edge_a  , size(edge_a)  , MPI_REAL8 , 0, mpicom, ier)
    call mpi_bcast (longxy_a, size(longxy_a), MPI_REAL8 , 0, mpicom, ier)
    call mpi_bcast (latixy_a, size(latixy_a), MPI_REAL8 , 0, mpicom, ier)
#endif

  end subroutine obs_getgrid_grndt

!------------------------------------------------------------------------
!BOP
!
! !IROUTINE: obs_openfile
!
! !INTERFACE:
  subroutine obs_openfile (kda, kmo, kyr, locfn, itim, atmmin)
!
! !DESCRIPTION:
! Open atmospheric forcing netCDF file
!
! !USES:

    use clm_varctl, only : landobsdir

    use fileutils , only : getfil
    use ncdio
!
! !ARGUMENTS:
    implicit none
    integer, intent(in)  :: kda              !day (1 -> 31)
    integer, intent(in)  :: kmo              !month (1 -> 12)
    integer, intent(in)  :: kyr              !year (0 -> ...)
    character(len=*), intent(inout) :: locfn !history file to open and read
    integer, intent(out) :: itim             !time index used in atmrd
    integer, intent(out) :: atmmin           !temporal resolution of atm data (in minutes)
!
! !REVISION HISTORY:
! Created by Suping Nie
!
!EOP
!
! LOCAL VARIABLES:
    integer :: i,j,k,n                !do loop indices
    integer :: dimid                  !netCDF dimension id
    integer :: status                 !netCDF error status
    integer :: cyc                    !current cycle
    integer :: nyr                    !year extension to data file name, e.g., 05
    integer :: pyr = 0                !complete years of data since basedate
    integer :: nmo = 0                !number of months past basedate
    integer :: mmo = -1               !number of months past end of data
    integer :: ier                    !error status
    character(len=256) :: filenam     !full file name, atmdir + ext
    character(len=256) :: locfnlast   !full file name saved from last call
    character(len=  7) :: ext         !month-year extension, e.g., 01-0005
    logical :: lexist                 !true => file exists, used when looking for a file
    integer :: ndaypm(12) =      &    !number of days per month
             (/ 31,28,31,30,31,30,31,31,30,31,30,31 /)
    integer :: minpday = 1440         !minutes per day

    character(len=32) :: subname = 'obs_openfil_grndt'  

!------------------------------------------------------------------------

    if (masterproc) then

       ! Build [month]-[year] extension for file name to be read
       ! append extension to path name to get full file name
       ! first save old locfn, then obtain the file if it exists, and
       ! finally check if file to be opened exists (e.g. if are at the
       ! end of the set of files to be cycled, then the next file will
       ! not exists and will have to rewind to beginning of data set

       write (ext,'(i4.4,"-",i2.2)') kyr,kmo

       filenam = trim(landobsdir) // '/n' // ext // '_FY3C_LST_obs.nc'

       locfnlast = locfn
       call getfil(filenam, locfn, 1)
       inquire (file = locfn, exist = lexist)

       ! If file exists...
       !   makes sure that if run was restarted within the same month
       !   that the original 'initial' run was started, then nmo will
       !   not count that month twice
       ! If file doesn't exist...
       !   rewind to beginning of data set (by repeating the process of
       !   creating extension, appending to path name and looking for
       !   the file

       if (lexist) then
          if (locfn /= locfnlast) nmo = nmo + 1 !months past base date
          pyr = nmo / 12      !years past base date
       elseif (nmo == 0) then
          write(6,*) 'obs_OPENFILE error: could not find initial land obs datafile' 
          call endrun
       else
          mmo = mmo + 1       !(months-1) past end of data
          cyc = mmo / (pyr*12) !cycles since end of data
          nyr = kyr - pyr * (cyc + 1) !rewind to beginning of dataset
          write (ext,'(i4.4,"-",i2.2)') kyr,kmo

          filenam = trim(landobsdir) // '/n' // ext // '_FY3C_LST_obs.nc'

          call getfil(filenam, locfn, 1)
          inquire (file = locfn, exist = lexist)
          if (.not. lexist) then
             if (nmo < 12) then
                write(6,*) 'ATM_OPENFILE error: You must supply at least a'
                write(6,*) 'year of input data if you wish to'
                write(6,*) 'cycle through it more than once'
             else
                write(6,*) 'LAND_LST_OPENFILE error: Not finding ',locfn
             end if
             call endrun
          end if
       end if

       ! Open netCDF data file and get lengths of lat,lon,time dimensions
       ! Do this only at the first timestep of the run or of the month

       call check_ret(nf_open (locfn, nf_nowrite, ncid), subname)

       call check_ret(nf_inq_dimid  (ncid, 'lon', dimid), subname)
       call check_ret(nf_inq_dimlen (ncid, dimid, nlon), subname)

       call check_ret(nf_inq_dimid  (ncid, 'lat', dimid), subname)
       call check_ret(nf_inq_dimlen (ncid, dimid, nlat), subname)

       call check_ret(nf_inq_dimid  (ncid, 'time', dimid), subname)
       call check_ret(nf_inq_dimlen (ncid, dimid, ntim), subname)

       if (ntim == 0) then
          write (6,*) 'OBS_OPENFILE error: zero input time slices in land obs data' 
          call endrun
       end if
       if (nlon /= atmlon) then

          write (6,*) 'OBS_OPENFILE error: nlon = ',nlon, &
               ' in data file not equal to obslon = ',atmlon,' first read in'  
          call endrun
       end if
       if (nlat /= atmlat) then
          write (6,*) 'OBS error: nlat = ',nlat, &
               ' in data file not equal to obslat = ',atmlat,' first read in' 
          call endrun
       end if

       ! Get variable names

       status = nf_inq_nvars(ncid, nvar)
       if (status /= nf_noerr) then
          write (6,*) ' ATM_OPENFILE netcdf error = ',nf_strerror(status)
          call endrun
       end if
       do i = 1, nvar
          call check_ret(nf_inq_varname (ncid, i, varnam(i)), subname)
       end do

    endif     !end of if-masterproc block

    ! Calculate temporal resolution of the dataset in minutes and
    ! reset time index

#if (defined SPMD)
    call mpi_bcast (ntim, 1, MPI_INTEGER, 0, mpicom, ier)
#endif
    atmmin = ndaypm(kmo) * minpday / ntim
    itim = 0

  end subroutine obs_openfile

!------------------------------------------------------------------------
!BOP
!
! !IROUTINE: obs_readdata
!
! !INTERFACE:
  subroutine obs_readdata (fname, kmo, itim)
!
! !DESCRIPTION:
! read atmospheric forcing single level fields from netCDF file
! this is done at every subsequent call to atmrd
! this includes a second call at the 1st tstep of the run or of the month
! Use nf_get_vara_real to read data for variable referenced by
! variable id = [i]
! nf_get_vara_real (ncid, i, beg, len, varval)
! integer beg - a vector of integers specifying the index in the
!               variable where the first data value is read.
!               must be dimensioned same as the variable's dimension
!               and a starting value must be given for each dimension
! integer len - a vector of integers specifying the number of data
!               values, for each of the variable's dimensions, to read.
!               must be dimensioned same as the variable's dimension
!               and a length must be given for each dimension
!
! !USES:
    use clm_varcon, only : sb
    use fileutils , only : getfil
    use ncdio
!
! !ARGUMENTS:
    implicit none
    character(len=*), intent(in) :: fname           !history file to open and read
    integer, intent(in)  :: kmo, itim               !current month and time index
!
! !REVISION HISTORY:
! Created by Suping Nie
!
!EOP
!
! LOCAL VARIABLES:
    integer :: i,j,k,n                  !do loop indices
    integer :: ier                      !error status
    integer :: varid                    !netCDF variable id
    integer :: status                   !netCDF error status
    integer :: beg3d(3)                 !netCDF 3-d start index (where to read first value)
    integer :: len3d(3)                 !netCDF 3-d count index (number of values to read)
    character(len=32) :: subname = 'obs_readdata_grndt'

    character(len=8) :: fldlst(4)      !name of possible atm fields in input file

    data fldlst( 1) /'A_LST '/
    data fldlst( 2) /'D_LST'/
    data fldlst( 3) /'A_LST_time '/
    data fldlst( 4) /'D_LST_time'/

    real(r8) ea                    !atmospheric emissivity

    logical atmread_err

    ! use polynomials to calculate saturation vapor pressure and derivative with
    ! respect to temperature: over water when t > 0 c and over ice when t <= 0 c
    ! required to convert relative humidity to specific humidity

    real(r8) esatw                 !saturation vapor pressure over water (Pa)
    real(r8) esati                 !saturation vapor pressure over ice (Pa)
    real(r8) e                     !vapor pressure (Pa)
    real(r8) qsat                  !saturation specific humidity (kg/kg)
    real(r8) a0,a1,a2,a3,a4,a5,a6  !coefficients for esat over water
    real(r8) b0,b1,b2,b3,b4,b5,b6  !coefficients for esat over ice
    real(r8) tdc, t                !Kelvins to Celcius function and its input


    integer :: temp_index,temp_index2,temp_index3  


    parameter (a0=6.107799961    , a1=4.436518521e-01, &
               a2=1.428945805e-02, a3=2.650648471e-04, &
               a4=3.031240396e-06, a5=2.034080948e-08, &
               a6=6.136820929e-11)

    parameter (b0=6.109177956    , b1=5.034698970e-01, &
               b2=1.886013408e-02, b3=4.176223716e-04, &
               b4=5.824720280e-06, b5=4.838803174e-08, &
               b6=1.838826904e-10)
!
! function declarations
!
    tdc(t) = min( 50., max(-50.,(t-SHR_CONST_TKFRZ)) )
    esatw(t) = 100.*(a0+t*(a1+t*(a2+t*(a3+t*(a4+t*(a5+t*a6))))))
    esati(t) = 100.*(b0+t*(b1+t*(b2+t*(b3+t*(b4+t*(b5+t*b6))))))
!------------------------------------------------------------------------


    ! Read single level fields

    if (masterproc) then

       ! initialize fields to the flag value

       x(:,:,:) = -1.

       ! read input data single-level fields

       beg3d(1) = 1     ;  len3d(1) = atmlon
       beg3d(2) = 1     ;  len3d(2) = atmlat
       beg3d(3) = itim  ;  len3d(3) = 1

       do k = 1, 4  
          do n = 1, nvar
             if (varnam(n) == fldlst(k)) then
                call check_ret(nf_get_vara_double(ncid,n,beg3d,len3d,x(1,1,k)), subname)
             end if
          end do              !end loop of fields in input file
       end do                 !end loop of fields expected in input file

       ! Close file at the end of the month
       ! NOTE: as written will not close file if run ends mid-month

       if (itim == ntim) then
          call check_ret(nf_close (ncid), subname)
          write (6,*) '---------------------------------------'
          write (6,*) 'OBSRD: closing obs data for ',trim(fname)
          write (6,*) '---------------------------------------'
          write (6,*)
       end if

    endif     !end of if-masterproc block

#if (defined SPMD)
    call mpi_bcast (x, size(x), MPI_REAL8, 0, mpicom, ier)
#endif

    ! ----------------------------------------------------------------------
    ! Determine 2d atmospheric fields
    ! Follow order in fldlst(14) to determine what was read and what was not
    ! ----------------------------------------------------------------------

    ! Loop over atmospheric longitudes and latitudes

    atmread_err = .false.
!$OMP PARALLEL DO PRIVATE (i,j,e,ea,qsat)
!CSD$ PARALLEL DO PRIVATE (i,j,e,ea,qsat)
    do j = 1, atmlat
       do i = 1, atmlon
           a_grndtxy_a(i,j) = x(i,j,1)
           d_grndtxy_a(i,j) = x(i,j,2)
           at_grndtxy_a(i,j) = x(i,j,3)
           dt_grndtxy_a(i,j) = x(i,j,4)
       end do                 !end loop of latitudes
    end do                    !end loop of longitudes
!CSD$ END PARALLEL DO
!$OMP END PARALLEL DO

    if (atmread_err) then
       write(6,*) 'obs_readdata: error reading land FY3 LST data'
       call endrun
    end if

  end subroutine obs_readdata

!------------------------------------------------------------------------
!BOP
!
! !IROUTINE: obsinterpa2si
!
! !INTERFACE:
  subroutine obsinterpa2si()
!
! !DESCRIPTION:
! Initialize variables for atm->land model surface interp
!
! !USES:
    use clm_varsur, only : numlon, longxy, latixy, lsmedge, lonw, lats, area
    use areaMod
!
! !ARGUMENTS:
    implicit none
!
! !REVISION HISTORY:
! Created by Suping Nie
!
!EOP
!
! LOCAL VARIABLES:
    integer :: i,j                       !indices
    integer :: ier                       !error status
    real(r8), allocatable :: lon_a(:,:)  !atm grid longitude cell edges
    real(r8), allocatable :: lat_a(:)    !atm grid latitude cell edges
    real(r8), allocatable :: area_a(:,:) !atm grid grid cell areas
    real(r8), allocatable :: mask_a(:,:) !dummy field: atm grid mask
    real(r8), allocatable :: mask_s(:,:) !dummy field: land model grid mask
!------------------------------------------------------------------------

    ! Dynamically allocate memory

    allocate (lon_a(atmlon+1,atmlat), &
              lat_a(atmlat+1), &
              area_a(atmlon,atmlat), &
              mask_a(atmlon,atmlat), &
              mask_s(lsmlon,lsmlat), stat=ier)
    if (ier /= 0) then
       write (6,*) 'interpa2si(): allocation error'
       call endrun
    end if

    if ( masterproc )then
       write (6,*) 'Attempting to initialize obs->land LST model grid interpolation .....'   
       write (6,*) 'Initializing obs -> srf interpolation .....'                         
    end if

    ! --------------------------------------------------------------------
    ! Map from atmosphere grid to surface grid
    ! --------------------------------------------------------------------

    ! determine numlon for atmosphere grid

    numlon_a(:) = 0
    do j = 1, atmlat
       do i = 1, atmlon
          if (longxy_a(i,j) /= 1.e36) numlon_a(j) = numlon_a(j) + 1
       end do
    end do

    ! [mask_a] = 1 means all grid cells on atm grid, regardless of whether
    ! land or ocean, will contribute to surface grid.

    do j = 1, atmlat
       do i = 1, numlon_a(j)
          mask_a(i,j) = 1.
       end do
    end do

    ! [mask_s] = 1 means all the surface grid is land. Used as dummy
    ! variable so code will not abort with false, non-valid error check

    do j = 1, lsmlat
       do i = 1, numlon(j)
          mask_s(i,j) = 1.
       end do
    end do

    ! For each surface grid cell: get lat [jovr_a2s] and lon [iovr_a2s] indices
    ! and weights [wovr_a2s] of overlapping atm grid cells

    call celledge (atmlat    , atmlon    , numlon_a  , longxy_a  , &
                   latixy_a  , edge_a(1) , edge_a(2) , edge_a(3) , &
                   edge_a(4) , lat_a     , lon_a     )

    call cellarea (atmlat    , atmlon    , numlon_a  , lat_a     , &
                   lon_a     , edge_a(1) , edge_a(2) , edge_a(3) , &
                   edge_a(4) , area_a    )

    call areaini (atmlon, atmlat, numlon_a, lon_a, lat_a, area_a, mask_a, &
                  lsmlon, lsmlat, numlon  , lonw , lats , area  , mask_s, &
                  mxovr , novr_a2s, iovr_a2s, jovr_a2s, wovr_a2s )

    deallocate (lon_a, lat_a, area_a, mask_a, mask_s)

    if ( masterproc )then
       write (6,*) 'Successfully made land LST obs -> srf interpolation'
       write (6,*) 'Successfully initialized area-averaging interpolation'
       write (6,*)
    end if

  end subroutine obsinterpa2si

!------------------------------------------------------------------------
!BOP
!
! !IROUTINE: obsinterpa2s
!
! !INTERFACE:

  subroutine obsinterpa2s (a_grndt_a  , a_grndt_s  , d_grndt_a  , d_grndt_s , &
                           at_grndt_a  , at_grndt_s  , dt_grndt_a  , dt_grndt_s )
! Niesp 

! !DESCRIPTION:
! Area average fields from observation grid to surface grid   
!
! !USES:
    use clm_varsur, only : numlon, longxy, latixy, lsmedge
    use areaMod
!
! !ARGUMENTS:
    implicit none

    real(r8), intent(in)  ::  a_grndt_a(atmlon,atmlat)     !FY3 ground temperature data in ascending orbit (K)
    real(r8), intent(in)  ::  at_grndt_a(atmlon,atmlat)    !UTC time for FY3 ground temperature data in ascending orbit (K)
    real(r8), intent(out) ::  a_grndt_s(lsmlon,lsmlat)     !FY3 ground temperature data in ascending orbit (K)
    real(r8), intent(out) ::  at_grndt_s(lsmlon,lsmlat)    !UTC time for FY3 ground temperature data in ascending orbit (K)

    real(r8), intent(in)  ::  d_grndt_a(atmlon,atmlat)     !MODIS ground temperature data in decending orbit (K)
    real(r8), intent(in)  ::  dt_grndt_a(atmlon,atmlat)    !UTC time for MODIS ground temperature data in decending orbit (K)
    real(r8), intent(out) ::  d_grndt_s(lsmlon,lsmlat)     !MODIS ground temperature data in decending orbit (K)
    real(r8), intent(out) ::  dt_grndt_s(lsmlon,lsmlat)    !UTC time for MODIS ground temperature data in decending orbit (K)

!
!EOP
!
! LOCAL VARIABLES:
    integer  :: i,j                  !longitude,latitude loop indices
    logical  :: initinterp = .false. !interpolation initialization flag
!------------------------------------------------------------------------

    ! Initialize

    if (.not. initinterp) then
       call obsinterpa2si
       initinterp = .true.
    endif

    ! area-average absolute value of winds (i.e., regardless of
    ! direction) since land model cares about magnitude not direction.
    ! then need to adjust resultant stresses for direction of wind.

    call obs_areaave (atmlat   , atmlon   , numlon_a , a_grndt_a  , &
                  lsmlat   , lsmlon   , numlon   , a_grndt_s  , &
                  iovr_a2s , jovr_a2s , wovr_a2s , mxovr   )

    call obs_areaave (atmlat   , atmlon   , numlon_a , at_grndt_a  , &
                  lsmlat   , lsmlon   , numlon   , at_grndt_s  , &
                  iovr_a2s , jovr_a2s , wovr_a2s , mxovr   )

    call areaave (atmlat   , atmlon   , numlon_a , d_grndt_a  , &
                  lsmlat   , lsmlon   , numlon   , d_grndt_s  , &
                  iovr_a2s , jovr_a2s , wovr_a2s , mxovr   )
    call areaave (atmlat   , atmlon   , numlon_a , dt_grndt_a  , &
                  lsmlat   , lsmlon   , numlon   , dt_grndt_s  , &
                  iovr_a2s , jovr_a2s , wovr_a2s , mxovr   )

  end subroutine obsinterpa2s


end module obsdrvMod_grndt

#endif


