#include <misc.h>
#include <params.h>

module wu_conv

!---------------------------------------------------------------------------------
! Purpose:
!
! Interface from Zhang-McFarlane convection scheme, includes evaporation of convective 
! precip from the ZM scheme
!
! Author: Byron Boville, from code in tphysbc
!
! Interface from Wu Tongwen convection scheme
!
! Author: Wu Tongwen
!
!---------------------------------------------------------------------------------
  use shr_kind_mod,    only: r8 => shr_kind_r8
  use ppgrid,          only: pcols, pver, pverp
  use moistconvection, only: cp, grav, rgrav, rgas, limcnv
  use cldwat,          only: cldwat_fice, conke
  use history,         only: outfld, addfld, add_default, phys_decomp
  use physconst,       only: cpair, epsilo, gravit, latice, latvap, tmelt,cappa

  implicit none

  save
  private                         ! Make default type private to the module
!
! PUBLIC: interfaces
!
  public wu_convi                 ! ZM schemea
  public wu_convr                 ! ZM schemea
  public wu_conv_evap             ! evaporation of precip from ZM schemea
  public microp_wu_conv_evap             ! evaporation of precip from ZM schemea
!
! Private data
!
   public rl, cpres, capelmt
   real(r8) rl         ! wg latent heat of vaporization.
   real(r8) cpres      ! specific heat at constant pressure in j/kg-degk.
   real(r8), parameter :: capelmt = 70.  ! threshold value for cape for deep convection.
   real(r8) :: ke                     ! Tunable evaporation efficiency
   real(r8) c0
   real(r8) upconv_alfa  
   real(r8) tau   ! convective time scale
   real(r8),parameter :: a = 21.656
   real(r8),parameter :: b = 5418.
   real(r8),parameter :: c1 = 6.112
   real(r8),parameter :: c2 = 17.67
   real(r8),parameter :: c3 = 243.5
   real(r8) :: tfreez
   real(r8) :: eps1
contains

subroutine wu_convi( x1, x2, x3, x4 )

  use dycore,       only: dycore_is, get_resolution
  use pmgrid,       only: masterproc

!
! Initialization of ZM constants
!
   real(r8), intent(in) :: x1      ! unused
   real(r8), intent(in) :: x2      ! unused
   real(r8), intent(in) :: x3      ! unused
   real(r8), intent(in) :: x4      ! unused

   logical :: exists
   character(len=16) :: varname
   real :: varqi, varke, varp
   integer ioerr
!
   tfreez = tmelt
   eps1   = epsilo
   rl     = latvap
   cpres  = cpair

! read the tunable parameters from file 'parameters', wzz, 2007.5.23
   varqi = -1.
   varke = -1.
   inquire(file='parameters',exist=exists)
   if (exists) then
      open(10,file='parameters',form='formatted')
      do
         read(10,fmt="(a16,4x,e14.7)",iostat=ioerr) varname, varp
         if (ioerr < 0 ) exit
         if (ioerr /= 0 ) cycle
         if (trim(varname).eq.'C0deep_Value') then
            varqi = varp
         endif
         if (trim(varname).eq.'Keconv_Value') then
            varke = varp
         endif
      enddo
      close(10)
   endif
   if( masterproc ) write(6,*)'wzz: C0deep =', varqi, varke
!
! tau=4800. were used in canadian climate center. however, in echam3 t42, 
! convection is too weak, thus adjusted to 2400.
   if ( dycore_is ('LR') ) then
      if ( get_resolution() == '1x1.25' ) then
         if(masterproc) write(6,*) 'ZM: Found LR dycore at 1x1.25'
         tau = 3600.
         c0 = 3.5E-3
         ke = 1.0E-6
      elseif ( get_resolution() == '4x5' ) then
         if(masterproc) write(6,*) 'ZM: Found LR dycore at 4x5'
         tau = 3600.
         c0 = 3.5E-3
         ke = 1.0E-6
      else
         if(masterproc) write(6,*) 'ZM: Found LR dycore at default resolution'
         tau = 3600.
         c0 = 3.5E-3
         ke = 1.0E-6
      endif
   else
      if(get_resolution() == 'T106')then         ! wzz, 2006.04.24
         tau = 3600.
         c0 = 2.0E-3   
         ke = 1.0E-6
         upconv_alfa = 0.0     ! 1e-4 to 3e-4   

      elseif(get_resolution() == 'T266')then   
         tau = 3600.
         c0 = 3.0E-3   
         ke = 1.0E-6             ! 2.0E-6
         upconv_alfa = 0.0    ! 1e-4 to 3e-4

      else              ! T42
         tau  = 3600.
         c0 = 1.0E-3   !  0.8E-3   ! wutw
         ke = 5.0E-6   !  5.0E-6
         upconv_alfa = 0.0     ! 1e-4 to 3e-4

      endif
   endif
         if (varqi > 0) c0 = varqi     ! wzz, 2007.05.23
         if (varke > 0) ke = varke     ! wzz, 2007.05.23
   !-----------------------------------------------------------------------

   call addfld ('ZMFLXPRC','kg/m2/s ',pverp, 'A','Flux of precipitation from ZM convection'       ,phys_decomp)
   call addfld ('ZMFLXSNW','kg/m2/s ',pverp, 'A','Flux of snow from ZM convection'                ,phys_decomp)
   call addfld ('ZMNTPRPD','kg/kg/s ',pver , 'A','Net precipitation production from ZM convection',phys_decomp)
   call addfld ('ZMNTSNPD','kg/kg/s ',pver , 'A','Net snow production from ZM convection'         ,phys_decomp)
   call addfld ('ZMEIHEAT','W/kg'    ,pver , 'A','Heating by ice and evaporation in ZM convection',phys_decomp)

   call addfld ('HKFLXPRC','kg/m2/s ',pverp, 'A','Flux of precipitation from HK convection'       ,phys_decomp)
   call addfld ('HKFLXSNW','kg/m2/s ',pverp, 'A','Flux of snow from HK convection'                ,phys_decomp)
   call addfld ('HKNTPRPD','kg/kg/s ',pver , 'A','Net precipitation production from HK convection',phys_decomp)
   call addfld ('HKNTSNPD','kg/kg/s ',pver , 'A','Net snow production from HK convection'         ,phys_decomp)
   call addfld ('HKEIHEAT','W/kg'    ,pver , 'A','Heating by ice and evaporation in HK convection',phys_decomp)

end subroutine wu_convi

#ifdef WU_CONV
subroutine wu_convr(lchnk   ,ncol    , &
                    t       ,qh      ,prec    ,jctop   ,jcbot   , &
                    pblh    ,kbfs    ,zm      ,geos    ,zi      ,qtnd    , &
                    heat    ,pap     ,paph    ,dpp        , &
                    delt    ,mcon    ,cme     ,cape       ,    &
                    tpert   ,dlf     ,pflx    ,zdu     ,rprd    , &
                    mu      ,md      ,du      ,eu      ,ed      , &
                    mb      ,mc      ,zfg     ,eps0    ,jd      , &   ! wzz, 2008.9.9
                    dp      ,dsubcld ,jt      ,maxg    ,ideep   , &
                    lengath ,ql      ,rliq    ,ptendq  ,ptendt  , &
                    omega   ,ptendps ,landfrac,u       ,v       , &
                    ptendu  ,ptendv , cnst, ncnst , jlcl )
#else
subroutine wu_convr(lchnk   ,ncol    , &
                    t       ,qh      ,prec    ,jctop   ,jcbot   , &
                    pblh    ,zm      ,geos    ,zi      ,qtnd    , &
                    heat    ,pap     ,paph    ,dpp     ,          &
                    delt    ,mcon    ,cme     ,                   &
                    tpert   ,dlf     ,pflx    ,zdu     ,rprd    , &
                    mu      ,md      ,du      ,eu      ,ed      , &
                    mb      ,mc      ,zfg     ,eps0    ,jd      , &   ! wzz, 2008.9.9
                    dp      ,dsubcld ,jt      ,maxg    ,ideep   , &
                    lengath ,ql      ,rliq   )
#endif

!----------------------------------------------------------------------- 
! 
! Purpose: 
! Main driver for zhang-mcfarlane convection scheme 
! 
! Method: 
! performs deep convective adjustment based on mass-flux closure
! algorithm.
! 
! Author:guang jun zhang, m.lazare, n.mcfarlane. CAM Contact: P. Rasch
!
! This is contributed code not fully standardized by the CAM core group.
! All variables have been typed, where most are identified in comments
! The current procedure will be reimplemented in a subsequent version
! of the CAM where it will include a more straightforward formulation
! and will make use of the standard CAM nomenclature
!
! Author: Wu Tongwen
! 
!-----------------------------------------------------------------------
   use constituents, only: pcnst, pnats

!
! ************************ index of variables **********************
!
!  wg * alpha    array of vertical differencing used (=1. for upstream).
!  w  * cape     convective available potential energy.
!  wg * capeg    gathered convective available potential energy.
!  c  * capelmt  threshold value for cape for deep convection.
!  ic  * cpres    specific heat at constant pressure in j/kg-degk.
!  i  * dpp      
!  ic  * delt     length of model time-step in seconds.
!  wg * dp       layer thickness in mbs (between upper/lower interface).
!  wg * dqdt     mixing ratio tendency at gathered points.
!  wg * dsdt     dry static energy ("temp") tendency at gathered points.
!  wg * dudt     u-wind tendency at gathered points.
!  wg * dvdt     v-wind tendency at gathered points.
!  wg * dsubcld  layer thickness in mbs between lcl and maxi.
!  ic  * grav     acceleration due to gravity in m/sec2.
!  wg * du       detrainment in updraft. specified in mid-layer
!  wg * ed       entrainment in downdraft.
!  wg * eu       entrainment in updraft.
!  wg * hmn      moist static energy.
!  wg * hsat     saturated moist static energy.
!  w  * ideep    holds position of gathered points vs longitude index.
!  ic  * pver     number of model levels.
!  wg * j0       detrainment initiation level index.
!  wg * jd       downdraft   initiation level index.
!  ic  * jlatpr   gaussian latitude index for printing grids (if needed).
!  wg * jt       top  level index of deep cumulus convection.
!  w  * lcl      base level index of deep cumulus convection.
!  wg * lclg     gathered values of lcl.
!  w  * lel      index of highest theoretical convective plume.
!  wg * lelg     gathered values of lel.
!  w  * lon      index of onset level for deep convection.
!  w  * maxi     index of level with largest moist static energy.
!  wg * maxg     gathered values of maxi.
!  wg * mb       cloud base mass flux.
!  wg * mc       net upward (scaled by mb) cloud mass flux.
!  wg * md       downward cloud mass flux (positive up).
!  wg * mu       upward   cloud mass flux (positive up). specified
!                at interface
!  ic  * msg      number of missing moisture levels at the top of model.
!  w  * p        grid slice of ambient mid-layer pressure in mbs.
!  i  * pblt     row of pbl top indices.
!  w  * pcpdh    scaled surface pressure.
!  w  * pf       grid slice of ambient interface pressure in mbs.
!  wg * pg       grid slice of gathered values of p.
!  w  * q        grid slice of mixing ratio.
!  wg * qd       grid slice of mixing ratio in downdraft.
!  wg * qg       grid slice of gathered values of q.
!  i/o * qh       grid slice of specific humidity.
!  w  * qh0      grid slice of initial specific humidity.
!  wg * qhat     grid slice of upper interface mixing ratio.
!  wg * ql       grid slice of cloud liquid water.
!  wg * qs       grid slice of saturation mixing ratio.
!  w  * qstp     grid slice of parcel temp. saturation mixing ratio.
!  wg * qstpg    grid slice of gathered values of qstp.
!  wg * qu       grid slice of mixing ratio in updraft.
!  ic  * rgas     dry air gas constant.
!  wg * rl       latent heat of vaporization.
!  w  * s        grid slice of scaled dry static energy (t+gz/cp).
!  wg * sd       grid slice of dry static energy in downdraft.
!  wg * sg       grid slice of gathered values of s.
!  wg * shat     grid slice of upper interface dry static energy.
!  wg * su       grid slice of dry static energy in updraft.
!  i/o * t       
!  o  * jctop    row of top-of-deep-convection indices passed out.
!  O  * jcbot    row of base of cloud indices passed out.
!  wg * tg       grid slice of gathered values of t.
!  w  * tl       row of parcel temperature at lcl.
!  wg * tlg      grid slice of gathered values of tl.
!  w  * tp       grid slice of parcel temperatures.
!  wg * tpg      grid slice of gathered values of tp.
!  i/o * u        grid slice of u-wind (real).
!  wg * ug       grid slice of gathered values of u.
!  i/o * utg      grid slice of u-wind tendency (real).
!  i/o * v        grid slice of v-wind (real).
!  w  * va       work array re-used by called subroutines.
!  wg * vg       grid slice of gathered values of v.
!  i/o * vtg      grid slice of v-wind tendency (real).
!  i  * w        grid slice of diagnosed large-scale vertical velocity.
!  w  * z        grid slice of ambient mid-layer height in metres.
!  w  * zf       grid slice of ambient interface height in metres.
!  wg * zfg      grid slice of gathered values of zf.
!  wg * zg       grid slice of gathered values of z.
!
!-----------------------------------------------------------------------
!
! multi-level i/o fields:
!  i      => input arrays.
!  i/o    => input/output arrays.
!  w      => work arrays.
!  wg     => work arrays operating only on gathered points.
!  ic     => input data constants.
!  c      => data constants pertaining to subroutine itself.
!
! input arguments
!
   integer, intent(in) :: lchnk                   ! chunk identifier
   integer, intent(in) :: ncol                    ! number of atmospheric columns

   real(r8), intent(in) :: t(pcols,pver)              ! grid slice of temperature at mid-layer.
   real(r8), intent(in) :: qh(pcols,pver,pcnst+pnats) ! grid slice of specific humidity.
   real(r8), intent(in) :: pap(pcols,pver)     
   real(r8), intent(in) :: paph(pcols,pver+1)
   real(r8), intent(in) :: dpp(pcols,pver)        ! local sigma half-level thickness (i.e. dshj).
   real(r8), intent(in) :: zm(pcols,pver)

#ifdef WU_CONV
   real(r8), intent(in) :: ptendq(pcols,pver)
   real(r8), intent(in) :: ptendt(pcols,pver)
   real(r8)                omega(pcols,pver)
   real(r8)                ptendps(pcols)
   real(r8), intent(in) :: landfrac(pcols)
#endif
   real(r8), intent(in) :: geos(pcols)
   real(r8), intent(in) :: zi(pcols,pver+1)
   real(r8), intent(in) :: pblh(pcols)
   real(r8), intent(in) :: tpert(pcols)
   real(r8), intent(in) :: kbfs(pcols)                     ! surface turbance flux

#ifdef WU_CONV
   real(r8) negcape(pcols)             ! w  negative convective available potential energy.
   real(r8) ptendtg(pcols,pver)        ! w  grid slice of temp. tend. at interface.
   real(r8) ptendqg(pcols,pver)        ! w  grid slice of moist. tend. at interface.
   logical  deepconv(pcols)

   real(r8), intent(inout) :: u(pcols,pver)
   real(r8), intent(inout) :: v(pcols,pver)
   real(r8), intent(out) :: ptendu(pcols,pver)
   real(r8), intent(out) :: ptendv(pcols,pver)
   integer,  intent(in) :: ncnst
   real, intent(inout) :: cnst(pcols,pver,ncnst)  ! chem constiuents
   real(r8)            :: cnstg(pcols,pver,ncnst)

   real(r8) :: mu2(pcols,pver)
   real(r8) :: eu2(pcols,pver)
   real(r8) :: du2(pcols,pver)
   real(r8) :: md2(pcols,pver)
   real(r8) :: ed2(pcols,pver)

   real(r8) :: cape(pcols)     ! Convective available potential energy (CAPE)
                               ! J kg-1
#endif
!
! output arguments
!
   real(r8), intent(out) :: qtnd(pcols,pver)           ! specific humidity tendency (kg/kg/s)
   real(r8), intent(out) :: heat(pcols,pver)           ! heating rate (dry static energy tendency, W/kg)
   real(r8), intent(out) :: mcon(pcols,pverp)
   real(r8), intent(out) :: dlf(pcols,pver)    ! scattrd version of the detraining cld h2o tend
   real(r8), intent(out) :: pflx(pcols,pverp)  ! scattered precip flux at each level
   real(r8), intent(out) :: cme(pcols,pver)
   real(r8), intent(out) :: zdu(pcols,pver)
   real(r8), intent(out) :: rprd(pcols,pver)     ! rain production rate
! move these vars from local storage to output so that convective
! transports can be done in outside of conv_cam.
   real(r8), intent(out) :: mu(pcols,pver)
   real(r8), intent(out) :: eu(pcols,pver)
   real(r8), intent(out) :: du(pcols,pver)
   real(r8), intent(out) :: md(pcols,pver)
   real(r8), intent(out) :: ed(pcols,pver)
   real(r8), intent(out) :: dp(pcols,pver)       ! wg layer thickness in mbs (between upper/lower interface).
   real(r8), intent(out) :: dsubcld(pcols)       ! wg layer thickness in mbs between lcl and maxi.
   real(r8), intent(out) :: jctop(pcols)  ! o row of top-of-deep-convection indices passed out.
   real(r8), intent(out) :: jcbot(pcols)  ! o row of base of cloud indices passed out.
   real(r8), intent(out) :: prec(pcols)
   real(r8), intent(out) :: rliq(pcols) ! reserved liquid (not yet in cldliq) for energy integrals

   real(r8) zs(pcols)
   real(r8) dlg(pcols,pver)    ! gathrd version of the detraining cld h2o tend
   real(r8) pflxg(pcols,pverp) ! gather precip flux at each level
   real(r8) cug(pcols,pver)    ! gathered condensation rate
   real(r8) evpg(pcols,pver)   ! gathered evap rate of rain in downdraft
   real(r8) mumax(pcols)
   integer jt(pcols)                          ! wg top  level index of deep cumulus convection.
   integer maxg(pcols)                        ! wg gathered values of maxi.
   integer ideep(pcols)                       ! w holds position of gathered points vs longitude index.
   integer lengath
!     diagnostic field used by chem/wetdep codes
   real(r8) ql(pcols,pver)                    ! wg grid slice of cloud liquid water.
!
   real(r8) pblt(pcols)           ! i row of pbl top indices.
   real(r8) eps0(pcols)
!  
!-----------------------------------------------------------------------
!
! general work fields (local variables):
!
   real(r8) q(pcols,pver)              ! w  grid slice of mixing ratio.
   real(r8) p(pcols,pver)              ! w  grid slice of ambient mid-layer pressure in mbs.
   real(r8) z(pcols,pver)              ! w  grid slice of ambient mid-layer height in metres.
   real(r8) s(pcols,pver)              ! w  grid slice of scaled dry static energy (t+gz/cp).
   real(r8) tp(pcols,pver)             ! w  grid slice of parcel temperatures.
   real(r8) zf(pcols,pver+1)           ! w  grid slice of ambient interface height in metres.
   real(r8) pf(pcols,pver+1)           ! w  grid slice of ambient interface pressure in mbs.
   real(r8) qstp(pcols,pver)           ! w  grid slice of parcel temp. saturation mixing ratio.

   real(r8) pgf(pcols,pverp)  

   real(r8) tl(pcols)                  ! w  row of parcel temperature at lcl.

   integer lcl(pcols)                  ! w  base level index of deep cumulus convection.
   integer lel(pcols)                  ! w  index of highest theoretical convective plume.
   integer lon(pcols)                  ! w  index of onset level for deep convection.
   integer maxi(pcols)                 ! w  index of level with largest moist static energy.
   integer index(pcols)
   real(r8) precip
!
! gathered work fields:
!
   real(r8) qg(pcols,pver)             ! wg grid slice of gathered values of q.
   real(r8) tg(pcols,pver)             ! w  grid slice of temperature at interface.
   real(r8) pg(pcols,pver)             ! wg grid slice of gathered values of p.
   real(r8) zg(pcols,pver)             ! wg grid slice of gathered values of z.
   real(r8) sg(pcols,pver)             ! wg grid slice of gathered values of s.
   real(r8) tpg(pcols,pver)            ! wg grid slice of gathered values of tp.
   real(r8) zfg(pcols,pver+1)          ! wg grid slice of gathered values of zf.
   real(r8) qstpg(pcols,pver)          ! wg grid slice of gathered values of qstp.
   real(r8) ug(pcols,pver)             ! wg grid slice of gathered values of u.
   real(r8) vg(pcols,pver)             ! wg grid slice of gathered values of v.
   real(r8) cmeg(pcols,pver)

   real(r8) omegag (pcols,pver)        ! w  grid slice of omega  tend. at interface.
   real(r8) ptendpsg(pcols)
   real(r8) duqlg(pcols,pver)

   real(r8) rprdg(pcols,pver)           ! wg gathered rain production rate
   real(r8) capeg(pcols)               ! wg gathered convective available potential energy.
   real(r8) tlg(pcols)                 ! wg grid slice of gathered values of tl.
   integer lclg(pcols)                 ! wg gathered values of lcl.
   integer lelg(pcols)

   real(r8) ptendug(pcols,pver)
   real(r8) ptendvg(pcols,pver)
   real(r8) tv(pcols,pver)
   real(r8) tpv(pcols,pver)
   real(r8) tvg(pcols,pver)
   real(r8) tpvg(pcols,pver)
   real(r8) qpg(pcols,pver)
   real(r8) uhat(pcols,pver)
   real(r8) vhat(pcols,pver)
   real(r8) upg(pcols,pver)
   real(r8) vpg(pcols,pver)
   real(r8) udg(pcols,pver)
   real(r8) vdg(pcols,pver)
!
! work fields arising from gathered calculations.
!
   real(r8) dqdt(pcols,pver)           ! wg mixing ratio tendency at gathered points.
   real(r8) dsdt(pcols,pver)           ! wg dry static energy ("temp") tendency at gathered points.
!      real(r8) alpha(pcols,pver)      ! array of vertical differencing used (=1. for upstream).
   real(r8) sd(pcols,pver)             ! wg grid slice of dry static energy in downdraft.
   real(r8) qd(pcols,pver)             ! wg grid slice of mixing ratio in downdraft.
   real(r8) mc(pcols,pver)             ! wg net upward (scaled by mb) cloud mass flux.
   real(r8) qhat(pcols,pver)           ! wg grid slice of upper interface mixing ratio.
   real(r8) qu(pcols,pver)             ! wg grid slice of mixing ratio in updraft.
   real(r8) su(pcols,pver)             ! wg grid slice of dry static energy in updraft.
   real(r8) qs(pcols,pver)             ! wg grid slice of saturation mixing ratio.
   real(r8) shat(pcols,pver)           ! wg grid slice of upper interface dry static energy.
   real(r8) hmn(pcols,pver)            ! wg moist static energy.
   real(r8) hsat(pcols,pver)           ! wg saturated moist static energy.
   real(r8) qlg(pcols,pver)
   real(r8) dudt(pcols,pver)           ! wg u-wind tendency at gathered points.
   real(r8) dvdt(pcols,pver)           ! wg v-wind tendency at gathered points.
!      real(r8) ud(pcols,pver)
!      real(r8) vd(pcols,pver)

   real(r8) mb(pcols)                  ! wg cloud base mass flux.
   real(r8) plcl(pcols)                  ! wg cloud base mass flux.

   integer jlcl(pcols)
   integer j0(pcols)                 ! wg detrainment initiation level index.
   integer jd(pcols)                 ! wg downdraft initiation level index.

   real(r8) delt                     ! length of model time-step in seconds.

   integer i
   integer ii
   integer k
   integer msg                      !  ic number of missing moisture levels at the top of model.
   real(r8) qdifr
   real(r8) sdifr
   real(r8) rh, est, qst
!  real(r8) tplcl, estplcl, qstplcl, rh, est, qst
   logical  logicalconv
!
!--------------------------Data statements------------------------------
!
! Set internal variable "msg" (convection limit) to "limcnv-1"
!
   msg = limcnv - 1
!
! initialize necessary arrays.
! zero out variables not used in cam
!
   qtnd(:,:) = 0.
   heat(:,:) = 0.
   mcon(:,:) = 0.
   rliq(:ncol)   = 0.
!
! initialize convective tendencies
!
   prec(:ncol) = 0.
   do k = 1,pver
      do i = 1,ncol
         dqdt(i,k)  = 0.
         dsdt(i,k)  = 0.
         dudt(i,k)  = 0.
         dvdt(i,k)  = 0.
         pflx(i,k)  = 0.
         pflxg(i,k) = 0.
         cme(i,k)   = 0.
         rprd(i,k)  = 0.
         zdu(i,k)   = 0.
         ql(i,k)    = 0.
         qlg(i,k)   = 0.
         dlf(i,k)   = 0.
         dlg(i,k)   = 0.

         mu2(i,k)   = 0.
         md2(i,k)   = 0.
         du2(i,k)   = 0.
         eu2(i,k)   = 0.
         ed2(i,k)   = 0.

      end do
   end do
   do i = 1,ncol
      pflx(i,pverp) = 0
      pflxg(i,pverp) = 0
   end do
!
   do i = 1,ncol
      pblt(i) = pver
      dsubcld(i) = 0.

      jctop(i) = pver
      jcbot(i) = 1
      cape(i) = 0.0
   end do
!
! calculate local pressure (mbs) and height (m) for both interface
! and mid-layer locations.
!
   do i = 1,ncol
      zs(i) = geos(i)*rgrav
      pf(i,pver+1) = paph(i,pver+1)*0.01
      zf(i,pver+1) = zi(i,pver+1) + zs(i)
   end do
   do k = 1,pver
      do i = 1,ncol
         p(i,k) = pap(i,k)*0.01
         pf(i,k) = paph(i,k)*0.01
         z(i,k) = zm(i,k) + zs(i)
         zf(i,k) = zi(i,k) + zs(i)
      end do
   end do
!
   do k = pver - 1,msg + 1,-1
      do i = 1,ncol
         if (abs(z(i,k)-zs(i)-pblh(i)) < (zf(i,k)-zf(i,k+1))*0.5) pblt(i) = k
      end do
   end do
!
! store incoming specific humidity field for subsequent calculation
! of precipitation (through change in storage).
! define dry static energy (normalized by cp).
!
   do k = 1,pver
      do i = 1,ncol
!------
!         est = c1*exp((c2* (t(i,k)-tmelt))/((t(i,k)-tmelt)+c3))
!         qst = eps1*est/(p(i,k)-est)
!         q(i,k) = qh(i,k,1)
!         if(q(i,k) > qst) q(i,k) = qst
!--
         q(i,k) = qh(i,k,1)
!--
         s(i,k) = t(i,k) + (grav/cpres)*z(i,k)
         tp(i,k) = 0.0
         shat(i,k) = s(i,k)
         qhat(i,k) = q(i,k)
      end do
   end do
   do i = 1,ncol
      capeg(i) = 0.
      lclg(i) = 1
      lelg(i) = pver
      maxg(i) = 1
      tlg(i) = 400.
      dsubcld(i) = 0.
   end do
!
! evaluate covective available potential energy (cape).
!

#ifdef WU_CONV
   call buoyan_wu (lchnk   ,ncol    , &
               q       ,t       ,p       ,z       ,pf       , &
               tp      ,qstp    ,tl      ,plcl    ,rl      ,cape     , &
               pblt    ,lcl     ,lel     ,lon     ,maxi     , &
               rgas    ,grav    ,cpres   ,msg     ,deepconv  , &
               tpert   ,negcape , omega  ,tv,     tpv,  kbfs ,landfrac )
#else
   call buoyan(lchnk   ,ncol    , &
               q       ,t       ,p       ,z       ,pf       , &
               tp      ,qstp    ,tl      ,rl      ,cape     , &
               pblt    ,lcl     ,lel     ,lon     ,maxi     , &
               rgas    ,grav    ,cpres   ,msg     , &
               tpert   )
#endif

!
! determine whether grid points will undergo some deep convection
! (ideep=1) or not (ideep=0), based on values of cape,lcl,lel
! (require cape.gt. 0 and lel<lcl as minimum conditions).
!
   lengath = 0
   do i=1,ncol
!#ifdef WU_REFATM
!     if (deepconv(i) .and. cape(i)> capelmt ) then
#ifdef WU_CONV
!--
      est = c1*exp((c2* (t(i,maxi(i))-tmelt))/((t(i,maxi(i))-tmelt)+c3))
      qst = eps1*est/(p(i,maxi(i))-est)
      rh = q(i,maxi(i))/qst
!---
      logicalconv =   rh > 0.70 .and. ((cape(i) > 30.0 .and. kbfs(i) > 0.0 ) &
                     .or. (cape(i) >30.0 .and. omega(i,maxi(i)) < -0.0001)  )
!-- 
      if( logicalconv ) then
!
!                       write(*,*) '===================================================='
!                       write(*,*) '===================================================='
!                       write(*,*) '===================================================='
!
#else
      if (cape(i) > capelmt) then
#endif
         lengath = lengath + 1
         index(lengath) = i
      end if
   end do

   if (lengath.eq.0) return
   do ii=1,lengath
      i=index(ii)
      ideep(ii)=i
   end do
!
! obtain gathered arrays necessary for ensuing calculations.
!
   do k = 1,pver
      do i = 1,lengath
         dp(i,k) = 0.01*dpp(ideep(i),k)
         qg(i,k) = q(ideep(i),k)
         tg(i,k) = t(ideep(i),k)

         cnstg(i,k,1:ncnst) = cnst(ideep(i),k,1:ncnst)

#ifdef WU_CONV
         ptendtg(i,k) = ptendt(ideep(i),k)
         ptendqg(i,k) = ptendq(ideep(i),k)
         omegag (i,k) = omega (ideep(i),k)
#endif
!#ifdef WU_REFATM
         pgf(i,k) = pf(ideep(i),k)
!#endif
         pg(i,k) = p(ideep(i),k)
         zg(i,k) = z(ideep(i),k)
         sg(i,k) = s(ideep(i),k)
         tpg(i,k) = tp(ideep(i),k)
         zfg(i,k) = zf(ideep(i),k)
         qstpg(i,k) = qstp(ideep(i),k)
         ug(i,k) = u(ideep(i),k)
         vg(i,k) = v(ideep(i),k)

         tpg(i,k) = tp(ideep(i),k)
         qpg(i,k) = qstp(ideep(i),k)

         tvg(i,k) = tv(ideep(i),k)
         tpvg(i,k)= tpv(ideep(i),k)
      end do
   end do
!
   do i = 1,lengath
      zfg(i,pver+1) = zf(ideep(i),pver+1)
!#ifdef WU_REFATM
#ifdef WU_CONV
      pgf(i,pver+1) = pf(ideep(i),pver+1)   ! wtw
      ptendpsg(i) = ptendps(ideep(i))
#endif
   end do
   do i = 1,lengath
      capeg(i) = cape(ideep(i))
      lclg(i) = lcl(ideep(i))
      lelg(i) = lel(ideep(i))
      maxg(i) = maxi(ideep(i))
      tlg(i) = tl(ideep(i))
   end do
!
! calculate sub-cloud layer pressure "thickness" for use in
! closure and tendency routines.
!
   do k = msg + 1,pver
      do i = 1,lengath
         if (k >= maxg(i)) then
            dsubcld(i) = dsubcld(i) + dp(i,k)
         end if
      end do
   end do
!
! define array of factors (alpha) which defines interfacial
! values, as well as interfacial values for (q,s) used in
! subsequent routines.
!
   do k = msg + 2,pver
      do i = 1,lengath

!#ifdef WU_REFATM
#ifdef WU_CONV
!---------------------
! wtw 2005.12
!
         sdifr = log(pgf(i,k)/pg(i,k))/log(pg(i,k-1)/pg(i,k))
         shat(i,k) = sg(i,k) + (sg(i,k-1)-sg(i,k))*sdifr 
         qhat(i,k) = qg(i,k) + (qg(i,k-1)-qg(i,k))*sdifr

         uhat(i,k) = ug(i,k) + (ug(i,k-1)-ug(i,k))*sdifr
         vhat(i,k) = vg(i,k) + (vg(i,k-1)-vg(i,k))*sdifr
!----------------
#else

         sdifr = 0.
         qdifr = 0.
         if (sg(i,k) > 0. .or. sg(i,k-1) > 0.) &
            sdifr = abs((sg(i,k)-sg(i,k-1))/max(sg(i,k-1),sg(i,k)))
         if (qg(i,k) > 0. .or. qg(i,k-1) > 0.) &
            qdifr = abs((qg(i,k)-qg(i,k-1))/max(qg(i,k-1),qg(i,k)))
         if (sdifr > 1.E-6) then
            shat(i,k) = log(sg(i,k-1)/sg(i,k))*sg(i,k-1)*sg(i,k)/(sg(i,k-1)-sg(i,k))
         else
            shat(i,k) = 0.5* (sg(i,k)+sg(i,k-1))
         end if
         if (qdifr > 1.E-6) then
            qhat(i,k) = log(qg(i,k-1)/qg(i,k))*qg(i,k-1)*qg(i,k)/(qg(i,k-1)-qg(i,k))
         else
            qhat(i,k) = 0.5* (qg(i,k)+qg(i,k-1))
         end if
#endif
      end do
   end do
!
! obtain cloud properties.
!
#ifdef WU_CONV
   call cldprp_wu(lchnk   , &
               qg      ,tg      ,ug      ,vg      ,pg      , &
               zg      ,sg      ,mu      ,eu      ,du      , &
               md      ,ed      ,sd      ,qd      ,mc      , &
               qu      ,su      ,zfg     ,qs      ,hmn     , &
               hsat    ,shat    ,qlg     , &
               cmeg    ,maxg    ,lelg    ,jt      ,jlcl    , &
               maxg    ,j0      ,jd      ,rl      ,lengath , &
               rgas    ,grav    ,cpres   ,msg     ,duqlg   , &
               pflxg   ,evpg    ,cug     ,rprdg   ,limcnv  ,&
               qpg     ,tpg     ,pgf     ,lclg    ,omegag  ,&
               ptendtg ,ptendqg ,uhat    ,vhat    , &
               upg     ,vpg     ,udg     ,vdg     ,tvg,  tpvg )
#else
   call cldprp(lchnk   , &
               qg      ,tg      ,ug      ,vg      ,pg      , &
               zg      ,sg      ,mu      ,eu      ,du      , &
               md      ,ed      ,sd      ,qd      ,mc      , &
               qu      ,su      ,zfg     ,qs      ,hmn     , &
               hsat    ,shat    ,qlg     , &
               cmeg    ,maxg    ,lelg    ,jt      ,jlcl    , &
               maxg    ,j0      ,jd      ,rl      ,lengath , &
               rgas    ,grav    ,cpres   ,msg     , &
               pflxg   ,evpg    ,cug     ,rprdg   ,limcnv  ,&
               eps0    , pgf )
#endif
!
! convert detrainment from units of "1/m" to "1/mb".
!
   do k = msg + 1,pver
      do i = 1,lengath
#ifdef WU_CONV
         du2(i,k) = du(i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         eu2(i,k) = eu(i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         ed2(i,k) = ed(i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
#endif
         du   (i,k) = du   (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         eu   (i,k) = eu   (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         ed   (i,k) = ed   (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         cug  (i,k) = cug  (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         cmeg (i,k) = cmeg (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         rprdg(i,k) = rprdg(i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         evpg (i,k) = evpg (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
         duqlg (i,k)= duqlg (i,k)* (zfg(i,k)-zfg(i,k+1))/dp(i,k)
      end do
   end do

#ifdef WU_CONV

   call closure_wu(lchnk        ,cug     ,evpg    , &
                qg      ,tg      ,pg      ,zg      ,sg      , &
                tpg     ,qs      ,qu      ,su      ,mc      , &
                du      ,mu      ,md      ,qd      ,sd      , &
                qhat    ,shat    ,dp      ,qstpg   ,zfg     , &
                qlg     ,dsubcld ,mb      ,capeg   ,tlg     , &
                jlcl    ,jt      ,maxg    ,1       , &
                lengath ,rgas    ,grav    ,cpres   ,rl      , &
                msg     ,capelmt ,ptendqg ,ptendtg ,negcape , duqlg ,delt)

#else
   call closure(lchnk   , &
                qg      ,tg      ,pg      ,zg      ,sg      , &
                tpg     ,qs      ,qu      ,su      ,mc      , &
                du      ,mu      ,md      ,qd      ,sd      , &
                qhat    ,shat    ,dp      ,qstpg   ,zfg     , &
                qlg     ,dsubcld ,mb      ,capeg   ,tlg     , &
                lclg    ,lelg    ,jt      ,maxg    ,1       , &
                lengath ,rgas    ,grav    ,cpres   ,rl      , &
                msg     ,capelmt    )
#endif
!
! limit cloud base mass flux to theoretical upper bound.
!
   do i=1,lengath
      mumax(i) = 0
   end do
   do k=msg + 2,pver
      do i=1,lengath
         mumax(i) = max(mumax(i), mu(i,k)/dp(i,k))
      end do
   end do
   do i=1,lengath
      if (mumax(i) > 0.) then
         mb(i) = min(mb(i),0.5/(delt*mumax(i)))
      else
         mb(i) = 0.
      endif
   end do
   do k=msg+1,pver
      do i=1,lengath
         mu   (i,k)  = mu   (i,k)*mb(i)
         md   (i,k)  = md   (i,k)*mb(i)
         mc   (i,k)  = mc   (i,k)*mb(i)
         du   (i,k)  = du   (i,k)*mb(i)
         eu   (i,k)  = eu   (i,k)*mb(i)
         ed   (i,k)  = ed   (i,k)*mb(i)
         cmeg (i,k)  = cmeg (i,k)*mb(i)
         rprdg(i,k)  = rprdg(i,k)*mb(i)
         cug  (i,k)  = cug  (i,k)*mb(i)
         evpg (i,k)  = evpg (i,k)*mb(i)
         pflxg(i,k+1)= pflxg(i,k+1)*mb(i)*100./grav
#ifdef WU_CONV
         mu2(i,k) = mu(i,k)
         md2(i,k) = md(i,k)
         du2(i,k) = du(i,k)
         eu2(i,k) = eu(i,k)
         ed2(i,k) = ed(i,k)

         duqlg (i,k) = duqlg (i,k)*mb(i)
#endif
      end do
   end do
!
! compute temperature and moisture changes due to convection.
!
#ifdef WU_CONV
   call q1q2_wu(lchnk   , &
                 dqdt    ,dsdt    ,qg      ,qs      ,qu      , &
                 su      ,du      ,qhat    ,shat    ,dp      , &
                 mu      ,md      ,sd      ,qd      ,qlg     , &
                 dsubcld ,jt      ,jlcl    ,maxg    ,1       ,lengath , &
                 cpres   ,rl      ,msg     ,  &
                 dlg     ,evpg    ,cug     ,duqlg   ,  &
                 ptendug ,ptendvg ,uhat    ,vhat     , &
                 upg     ,vpg     ,udg     ,vdg     , &
                 delt    ,zg      ,zfg     ,eu      , ed     , &
                 ncnst   ,cnstg    )

#else
   call q1q2_pjr(lchnk   , &
                 dqdt    ,dsdt    ,qg      ,qs      ,qu      , &
                 su      ,du      ,qhat    ,shat    ,dp      , &
                 mu      ,md      ,sd      ,qd      ,qlg     , &
                 dsubcld ,jt      ,maxg    ,1       ,lengath , &
                 cpres   ,rl      ,msg     ,          &
                 dlg     ,evpg    ,cug     )
#endif
!-----------------------------------------------------------------------
!       ... Compute momentum changes due to convection, if desired (i.e
!           if logical switch set).
!-----------------------------------------------------------------------
!#ifdef WU_CONV
      if( ncnst > 0 ) then
         call CONVTRAN_WU( cnst, ncnst, mu2, md2, du2, &
                        eu2, ed2, dp, dsubcld, jt, &
                        maxg, ideep, 1, lengath, &
                        delt, pcols )
      end if
!#endif
!---------------------------------------------
! gather back temperature and mixing ratio.
!
   do k = msg + 1,pver
      do i = 1,lengath
!
! q is updated to compute net precip.
!
!
         ptendu(ideep(i),k) = ptendug(i,k)
         ptendv(ideep(i),k) = ptendvg(i,k)
!----
!
!        cnst(ideep(i),k,1:ncnst) = cnstg(i,k,1:ncnst)   ! not include the impact of deep convection
!------------
         q(ideep(i),k) = qh(ideep(i),k,1) + 2.*delt*dqdt(i,k)
         qtnd(ideep(i),k) = dqdt (i,k)
         cme (ideep(i),k) = cmeg (i,k)
         rprd(ideep(i),k) = rprdg(i,k)
         zdu (ideep(i),k) = du   (i,k)
         mcon(ideep(i),k) = mc   (i,k)
         heat(ideep(i),k) = dsdt (i,k)*cpres
         dlf (ideep(i),k) = dlg  (i,k)
         pflx(ideep(i),k) = pflxg(i,k)
         ql  (ideep(i),k) = qlg  (i,k)
      end do
   end do
!
   do i = 1,lengath
      jctop(ideep(i)) = jt(i)
      jcbot(ideep(i)) = jlcl(i)
      pflx(ideep(i),pverp) = pflxg(i,pverp)
   end do

! Compute precip by integrating change in water vapor minus detrained cloud water
   do k = pver,msg + 1,-1
      do i = 1,ncol
         prec(i) = prec(i) - dpp(i,k)* (q(i,k)-qh(i,k,1)) - dpp(i,k)*dlf(i,k)*2*delt
      end do
   end do

! obtain final precipitation rate in m/s.
   do i = 1,ncol
!     prec(i) = rgrav*max(prec(i),0._r8)/ (2.*delt)/1000.
      prec(i) = rgrav*max(prec(i),0._r8)/ (2.*delt)*1.e-3
   end do

! Compute reserved liquid (not yet in cldliq) for energy integrals.
! Treat rliq as flux out bottom, to be added back later.
   do k = 1, pver
      do i = 1, ncol
         rliq(i) = rliq(i) + dlf(i,k)*dpp(i,k)/gravit
      end do
   end do
   rliq(:ncol) = rliq(:ncol) *1.e-3
!  rliq(:ncol) = rliq(:ncol) /1000.

   return
end subroutine wu_convr


!===============================================================================
  subroutine microp_wu_conv_evap(state, ptend, prdprec, cldfrc, deltat, prec, snow, hackflag, &
                                 flxprec, flxsnow, evapr)


!-----------------------------------------------------------------------
! Compute tendencies due to evaporation of rain from ZM scheme
!--
! Compute the total precipitation and snow fluxes at the surface.
! Add in the latent heat of fusion for snow formation and melt, since it not dealt with
! in the Zhang-MacFarlane parameterization.
! Evaporate some of the precip directly into the environment using a Sundqvist type algorithm
!-----------------------------------------------------------------------

    use physics_types,  only: physics_state, physics_ptend
    use wv_saturation,  only: aqsat
    use phys_grid, only: get_rlat_all_p

!------------------------------Arguments--------------------------------
    type(physics_state), intent(in)    :: state   ! Physics state variables
    type(physics_ptend), intent(inout) :: ptend   ! indivdual parameterization tendencies
    real(r8), intent(in   ) :: prdprec(pcols,pver)! precipitation production (kg/kg/s)
    real(r8), intent(in   ) :: cldfrc(pcols,pver) ! cloud fraction
    real(r8), intent(in   ) :: deltat             ! time step

    logical,  intent(in   ) :: hackflag           ! input is from Hack instead of ZM scheme
                                                  ! only used to specify output variables

    real(r8), intent(inout) :: prec(pcols)        ! Convective-scale preciptn rate
    real(r8), intent(out)   :: snow(pcols)        ! Convective-scale snowfall rate
    real(r8), optional, intent(out) :: evapr(pcols,pver)         ! evaporation of precipitation (kg/kg/s)
!
!---------------------------Local storage-------------------------------

    real(r8) :: est    (pcols,pver)    ! Saturation vapor pressure
    real(r8) :: fice   (pcols,pver)    ! ice fraction in precip production
    real(r8) :: fsnow_conv(pcols,pver) ! snow fraction in precip production
    real(r8) :: qsat   (pcols,pver)    ! saturation specific humidity
    real(r8) :: flxprec(pcols,pverp)   ! Convective-scale flux of precip at interfaces (kg/m2/s)
    real(r8) :: flxsnow(pcols,pverp)   ! Convective-scale flux of snow   at interfaces (kg/m2/s)
    real(r8) :: ntprprd(pcols,pver)    ! net precip production in layer
    real(r8) :: ntsnprd(pcols,pver)    ! net snow production in layer
    real(r8) :: work1                  ! temp variable (pjr)
    real(r8) :: work2                  ! temp variable (pjr)

    real(r8) :: evpvint(pcols)         ! vertical integral of evaporation
    real(r8) :: evpprec(pcols)         ! evaporation of precipitation (kg/kg/s)
    real(r8) :: evpsnow(pcols)         ! evaporation of snowfall (kg/kg/s)
    real(r8) :: snowmlt(pcols)         ! snow melt tendency in layer
    real(r8) :: flxsntm(pcols)         ! flux of snow into layer, after melting

    real(r8) :: evplimit               ! temp variable for evaporation limits
    real(r8) :: rlat(pcols)

    integer :: i,k                     ! longitude,level indices
    integer :: ncol, lchnk             ! number of columns and chunk index

!-----------------------------------------------------------------------
!!$    ke = conke
!!$    ke = 2.0E-6

    ncol  = state%ncol
    lchnk = state%lchnk

! heating and specific humidity tendencies produced
    ptend%name  = 'wu_conv_evap'
    ptend%ls    = .TRUE.
    ptend%lq(1) = .TRUE.

! convert input precip to kg/m2/s
    prec(:ncol) = prec(:ncol)*1000.

! determine saturation vapor pressure
    call aqsat (state%t    ,state%pmid  ,est    ,qsat    ,pcols   , &
         ncol ,pver  ,1       ,pver    )

! determine ice fraction in rain production (use cloud water parameterization fraction at present)
    call cldwat_fice(ncol, state%t, fice, fsnow_conv)

! zero the flux integrals on the top boundary
    flxprec(:ncol,1) = 0.
    flxsnow(:ncol,1) = 0.
    evpvint(:ncol)   = 0.

    do k = 1, pver
       do i = 1, ncol

! Melt snow falling into layer, if necessary.
          if (state%t(i,k) > tmelt) then
             flxsntm(i) = 0.
             snowmlt(i) = flxsnow(i,k) * gravit/ state%pdel(i,k)
          else
             flxsntm(i) = flxsnow(i,k)
             snowmlt(i) = 0.
          end if

! relative humidity depression must be > 0 for evaporation
          evplimit = max(1. - state%q(i,k,1)/qsat(i,k), 0._r8)

! total evaporation depends on flux in the top of the layer
! flux prec is the net production above layer minus evaporation into environmet
          evpprec(i) = ke * (1. - cldfrc(i,k)) * evplimit * sqrt(flxprec(i,k))
!**********************************************************
!!$           evpprec(i) = 0.    ! turn off evaporation for now
!**********************************************************

! Don't let evaporation supersaturate layer (approx). Layer may already be saturated.
! Currently does not include heating/cooling change to qsat
          evplimit   = max(0._r8, (qsat(i,k)-state%q(i,k,1)) / deltat)

! Don't evaporate more than is falling into the layer - do not evaporate rain formed
! in this layer but if precip production is negative, remove from the available precip
! Negative precip production occurs because of evaporation in downdrafts.
!!$          evplimit   = flxprec(i,k) * gravit / state%pdel(i,k) + min(prdprec(i,k), 0.)
          evplimit   = min(evplimit, flxprec(i,k) * gravit / state%pdel(i,k))

! Total evaporation cannot exceed input precipitation
          evplimit   = min(evplimit, (prec(i) - evpvint(i)) * gravit / state%pdel(i,k))

          evpprec(i) = min(evplimit, evpprec(i))

! evaporation of snow depends on snow fraction of total precipitation in the top after melting
          if (flxprec(i,k) > 0.) then
!            evpsnow(i) = evpprec(i) * flxsntm(i) / flxprec(i,k)
!            prevent roundoff problems
             work1 = min(max(0._r8,flxsntm(i)/flxprec(i,k)),1._r8)
             evpsnow(i) = evpprec(i) * work1
          else
             evpsnow(i) = 0.
          end if

! vertically integrated evaporation
          evpvint(i) = evpvint(i) + evpprec(i) * state%pdel(i,k)/gravit

! net precip production is production - evaporation
          ntprprd(i,k) = prdprec(i,k) - evpprec(i)
! net snow production is precip production * ice fraction - evaporation - melting
!pjrworks ntsnprd(i,k) = prdprec(i,k)*fice(i,k) - evpsnow(i) - snowmlt(i)
!pjrwrks2 ntsnprd(i,k) = prdprec(i,k)*fsnow_conv(i,k) - evpsnow(i) - snowmlt(i)
! the small amount added to flxprec in the work1 expression has been increased from
! 1e-36 to 8.64e-11 (1e-5 mm/day).  This causes the temperature based partitioning
! scheme to be used for small flxprec amounts.  This is to address error growth problems.
#ifdef PERGRO
          work1 = min(max(0._r8,flxsnow(i,k)/(flxprec(i,k)+8.64e-11_r8)),1._r8)
#else
          if (flxprec(i,k).gt.0.) then
             work1 = min(max(0._r8,flxsnow(i,k)/flxprec(i,k)),1._r8)
          else
             work1 = 0.
          endif
#endif
          work2 = max(fsnow_conv(i,k), work1)
          if (snowmlt(i).gt.0.) work2 = 0.
!         work2 = fsnow_conv(i,k)
          ntsnprd(i,k) = prdprec(i,k)*work2 - evpsnow(i) - snowmlt(i)

! precipitation fluxes
          flxprec(i,k+1) = flxprec(i,k) + ntprprd(i,k) * state%pdel(i,k)/gravit
          flxsnow(i,k+1) = flxsnow(i,k) + ntsnprd(i,k) * state%pdel(i,k)/gravit

! protect against rounding error
          flxprec(i,k+1) = max(flxprec(i,k+1), 0._r8)
          flxsnow(i,k+1) = max(flxsnow(i,k+1), 0._r8)
! more protection (pjr)
!         flxsnow(i,k+1) = min(flxsnow(i,k+1), flxprec(i,k+1))

! heating (cooling) and moistening due to evaporation
! - latent heat of vaporization for precip production has already been accounted for
! - snow is contained in prec
          ptend%s(i,k)   =-evpprec(i)*latvap + ntsnprd(i,k)*latice
          ptend%q(i,k,1) = evpprec(i)
       end do
    end do

! set output precipitation rates (m/s)
    prec(:ncol) = flxprec(:ncol,pver+1) / 1000.
    snow(:ncol) = flxsnow(:ncol,pver+1) / 1000.
    if( present(evapr) ) then  ! wzz, 2008.2.29
      evapr(:ncol,:) = prdprec(:ncol,:) - ntprprd(:ncol,:)
    endif

! record history variables
    if (hackflag) then
       call outfld('HKFLXPRC', flxprec, pcols, lchnk)
       call outfld('HKFLXSNW', flxsnow, pcols, lchnk)
       call outfld('HKNTPRPD', ntprprd, pcols, lchnk)
       call outfld('HKNTSNPD', ntsnprd, pcols, lchnk)
       call outfld('HKEIHEAT', ptend%s, pcols, lchnk)
    else
       call outfld('ZMFLXPRC', flxprec, pcols, lchnk)
       call outfld('ZMFLXSNW', flxsnow, pcols, lchnk)
       call outfld('ZMNTPRPD', ntprprd, pcols, lchnk)
       call outfld('ZMNTSNPD', ntsnprd, pcols, lchnk)
       call outfld('ZMEIHEAT', ptend%s, pcols, lchnk)
    end if
!**********************************************************
!!$    ptend%s(:ncol,:)   = 0.      ! turn heating off
!**********************************************************

end subroutine microp_wu_conv_evap


!===============================================================================
  subroutine wu_conv_evap(state, ptend, prdprec, cldfrc, deltat, prec, snow, hackflag, evapr)

!-----------------------------------------------------------------------
! Compute tendencies due to evaporation of rain from ZM scheme
!--
! Compute the total precipitation and snow fluxes at the surface.
! Add in the latent heat of fusion for snow formation and melt, since it not dealt with
! in the Zhang-MacFarlane parameterization.
! Evaporate some of the precip directly into the environment using a Sundqvist type algorithm
!-----------------------------------------------------------------------

    use physics_types,  only: physics_state, physics_ptend
    use wv_saturation,  only: aqsat
    use phys_grid, only: get_rlat_all_p

!------------------------------Arguments--------------------------------
    type(physics_state), intent(in)    :: state   ! Physics state variables
    type(physics_ptend), intent(inout) :: ptend   ! indivdual parameterization tendencies
    real(r8), intent(in   ) :: prdprec(pcols,pver)! precipitation production (kg/kg/s)
    real(r8), intent(in   ) :: cldfrc(pcols,pver) ! cloud fraction
    real(r8), intent(in   ) :: deltat             ! time step

    logical,  intent(in   ) :: hackflag           ! input is from Hack instead of ZM scheme
                                                  ! only used to specify output variables

    real(r8), intent(inout) :: prec(pcols)        ! Convective-scale preciptn rate
    real(r8), intent(out)   :: snow(pcols)        ! Convective-scale snowfall rate
    real(r8), optional, intent(out) :: evapr(pcols,pver)         ! evaporation of precipitation (kg/kg/s)
!
!---------------------------Local storage-------------------------------

    real(r8) :: est    (pcols,pver)    ! Saturation vapor pressure
    real(r8) :: fice   (pcols,pver)    ! ice fraction in precip production
    real(r8) :: fsnow_conv(pcols,pver) ! snow fraction in precip production
    real(r8) :: qsat   (pcols,pver)    ! saturation specific humidity
    real(r8) :: flxprec(pcols,pverp)   ! Convective-scale flux of precip at interfaces (kg/m2/s)
    real(r8) :: flxsnow(pcols,pverp)   ! Convective-scale flux of snow   at interfaces (kg/m2/s)
    real(r8) :: ntprprd(pcols,pver)    ! net precip production in layer
    real(r8) :: ntsnprd(pcols,pver)    ! net snow production in layer
    real(r8) :: work1                  ! temp variable (pjr)
    real(r8) :: work2                  ! temp variable (pjr)

    real(r8) :: evpvint(pcols)         ! vertical integral of evaporation
    real(r8) :: evpprec(pcols)         ! evaporation of precipitation (kg/kg/s)
    real(r8) :: evpsnow(pcols)         ! evaporation of snowfall (kg/kg/s)
    real(r8) :: snowmlt(pcols)         ! snow melt tendency in layer
    real(r8) :: flxsntm(pcols)         ! flux of snow into layer, after melting

    real(r8) :: evplimit               ! temp variable for evaporation limits
    real(r8) :: rlat(pcols)

    integer :: i,k                     ! longitude,level indices
    integer :: ncol, lchnk             ! number of columns and chunk index

!-----------------------------------------------------------------------
!!$    ke = conke
!!$    ke = 2.0E-6

    ncol  = state%ncol
    lchnk = state%lchnk

! heating and specific humidity tendencies produced
    ptend%name  = 'wu_conv_evap'
    ptend%ls    = .TRUE.
    ptend%lq(1) = .TRUE.

! convert input precip to kg/m2/s
    prec(:ncol) = prec(:ncol)*1000.

! determine saturation vapor pressure
    call aqsat (state%t    ,state%pmid  ,est    ,qsat    ,pcols   , &
         ncol ,pver  ,1       ,pver    )

! determine ice fraction in rain production (use cloud water parameterization fraction at present)
    call cldwat_fice(ncol, state%t, fice, fsnow_conv)

! zero the flux integrals on the top boundary
    flxprec(:ncol,1) = 0.
    flxsnow(:ncol,1) = 0.
    evpvint(:ncol)   = 0.

    do k = 1, pver
       do i = 1, ncol

! Melt snow falling into layer, if necessary. 
          if (state%t(i,k) > tmelt) then
             flxsntm(i) = 0.
             snowmlt(i) = flxsnow(i,k) * gravit/ state%pdel(i,k)
          else
             flxsntm(i) = flxsnow(i,k)
             snowmlt(i) = 0.
          end if

! relative humidity depression must be > 0 for evaporation
          evplimit = max(1. - state%q(i,k,1)/qsat(i,k), 0._r8)

! total evaporation depends on flux in the top of the layer
! flux prec is the net production above layer minus evaporation into environmet
          evpprec(i) = ke * (1. - cldfrc(i,k)) * evplimit * sqrt(flxprec(i,k))
!**********************************************************
!!$           evpprec(i) = 0.    ! turn off evaporation for now
!**********************************************************

! Don't let evaporation supersaturate layer (approx). Layer may already be saturated.
! Currently does not include heating/cooling change to qsat
          evplimit   = max(0._r8, (qsat(i,k)-state%q(i,k,1)) / deltat)

! Don't evaporate more than is falling into the layer - do not evaporate rain formed
! in this layer but if precip production is negative, remove from the available precip
! Negative precip production occurs because of evaporation in downdrafts.
!!$          evplimit   = flxprec(i,k) * gravit / state%pdel(i,k) + min(prdprec(i,k), 0.)
          evplimit   = min(evplimit, flxprec(i,k) * gravit / state%pdel(i,k))

! Total evaporation cannot exceed input precipitation
          evplimit   = min(evplimit, (prec(i) - evpvint(i)) * gravit / state%pdel(i,k))

          evpprec(i) = min(evplimit, evpprec(i))

! evaporation of snow depends on snow fraction of total precipitation in the top after melting
          if (flxprec(i,k) > 0.) then
!            evpsnow(i) = evpprec(i) * flxsntm(i) / flxprec(i,k)
!            prevent roundoff problems
             work1 = min(max(0._r8,flxsntm(i)/flxprec(i,k)),1._r8)
             evpsnow(i) = evpprec(i) * work1
          else
             evpsnow(i) = 0.
          end if

! vertically integrated evaporation
          evpvint(i) = evpvint(i) + evpprec(i) * state%pdel(i,k)/gravit

! net precip production is production - evaporation
          ntprprd(i,k) = prdprec(i,k) - evpprec(i)
! net snow production is precip production * ice fraction - evaporation - melting
!pjrworks ntsnprd(i,k) = prdprec(i,k)*fice(i,k) - evpsnow(i) - snowmlt(i)
!pjrwrks2 ntsnprd(i,k) = prdprec(i,k)*fsnow_conv(i,k) - evpsnow(i) - snowmlt(i)
! the small amount added to flxprec in the work1 expression has been increased from 
! 1e-36 to 8.64e-11 (1e-5 mm/day).  This causes the temperature based partitioning
! scheme to be used for small flxprec amounts.  This is to address error growth problems.
#ifdef PERGRO
          work1 = min(max(0._r8,flxsnow(i,k)/(flxprec(i,k)+8.64e-11_r8)),1._r8)
#else
          if (flxprec(i,k).gt.0.) then
             work1 = min(max(0._r8,flxsnow(i,k)/flxprec(i,k)),1._r8)
          else
             work1 = 0.
          endif
#endif
          work2 = max(fsnow_conv(i,k), work1)
          if (snowmlt(i).gt.0.) work2 = 0.
!         work2 = fsnow_conv(i,k)
          ntsnprd(i,k) = prdprec(i,k)*work2 - evpsnow(i) - snowmlt(i)

! precipitation fluxes
          flxprec(i,k+1) = flxprec(i,k) + ntprprd(i,k) * state%pdel(i,k)/gravit
          flxsnow(i,k+1) = flxsnow(i,k) + ntsnprd(i,k) * state%pdel(i,k)/gravit

! protect against rounding error
          flxprec(i,k+1) = max(flxprec(i,k+1), 0._r8)
          flxsnow(i,k+1) = max(flxsnow(i,k+1), 0._r8)
! more protection (pjr)
!         flxsnow(i,k+1) = min(flxsnow(i,k+1), flxprec(i,k+1))

! heating (cooling) and moistening due to evaporation 
! - latent heat of vaporization for precip production has already been accounted for
! - snow is contained in prec
          ptend%s(i,k)   =-evpprec(i)*latvap + ntsnprd(i,k)*latice
          ptend%q(i,k,1) = evpprec(i)
       end do
    end do

! set output precipitation rates (m/s)
    prec(:ncol) = flxprec(:ncol,pver+1) / 1000.
    snow(:ncol) = flxsnow(:ncol,pver+1) / 1000.
    if( present(evapr) ) then  ! wzz, 2008.2.29
      evapr(:ncol,:) = prdprec(:ncol,:) - ntprprd(:ncol,:)
    endif

! record history variables
    if (hackflag) then
       call outfld('HKFLXPRC', flxprec, pcols, lchnk)
       call outfld('HKFLXSNW', flxsnow, pcols, lchnk)
       call outfld('HKNTPRPD', ntprprd, pcols, lchnk)
       call outfld('HKNTSNPD', ntsnprd, pcols, lchnk)
       call outfld('HKEIHEAT', ptend%s, pcols, lchnk)
    else
       call outfld('ZMFLXPRC', flxprec, pcols, lchnk)
       call outfld('ZMFLXSNW', flxsnow, pcols, lchnk)
       call outfld('ZMNTPRPD', ntprprd, pcols, lchnk)
       call outfld('ZMNTSNPD', ntsnprd, pcols, lchnk)
       call outfld('ZMEIHEAT', ptend%s, pcols, lchnk)
    end if
!**********************************************************
!!$    ptend%s(:ncol,:)   = 0.      ! turn heating off
!**********************************************************

end subroutine wu_conv_evap


subroutine buoyan_wu (lchnk   ,ncol    , &
                  q       ,t       ,p       ,z       ,pf      , &
                  tp      ,qstp    ,tl      ,pl      ,rl      ,cape    , &
                  pblt    ,lcl     ,lel     ,lon     ,mx      , &
                  rd      ,grav    ,cp      ,msg     , deepconv  , &
                  tpert   ,negcape , omega  ,tv      , tpv    ,kbfs ,landfrac )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author:
! This is contributed code not fully standardized by the CCM core group.
! The documentation has been enhanced to the degree that we are able.
! Reviewed:          P. Rasch, April 1996
! 
!-----------------------------------------------------------------------
   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid,          only: pcols, pver, pverp
   use moistconvection, only: rgas
   use physconst,       only: cpair, epsilo, latvap, tmelt,cappa

   implicit none
!-----------------------------------------------------------------------
!
! input arguments
!
   integer, intent(in) :: lchnk                 ! chunk identifier
   integer, intent(in) :: ncol                  ! number of atmospheric columns

   real(r8), intent(in) :: q(pcols,pver)        ! spec. humidity
   real(r8), intent(in) :: t(pcols,pver)        ! temperature
   real(r8), intent(in) :: p(pcols,pver)        ! pressure
   real(r8), intent(in) :: z(pcols,pver)        ! height
   real(r8), intent(in) :: pf(pcols,pver+1)     ! pressure at interfaces
   real(r8), intent(in) :: pblt(pcols)          ! index of pbl depth
   real(r8), intent(in) :: kbfs(pcols)          ! surface turbance flux
   real(r8), intent(in) :: tpert(pcols)         ! perturbation temperature by pbl processes
   real(r8), intent(in) :: omega(pcols,pver) 
   real(r8), intent(in) :: landfrac(pcols) 
!
! output arguments
!
   real(r8), intent(out) :: tp(pcols,pver)       ! parcel temperature
   real(r8), intent(out) :: qstp(pcols,pver)     ! saturation mixing ratio of parcel
   real(r8), intent(out) :: tl(pcols)            ! parcel temperature at lcl
   real(r8), intent(inout) :: cape(pcols)          ! convective aval. pot. energy.
   real(r8), intent(out) :: negcape(pcols)       ! negative convective aval. pot. energy.
   logical  deepconv(pcols)     

   integer lcl(pcols)        !
   integer lel(pcols)        !
   integer lon(pcols)        ! level of onset of deep convection
   integer mx(pcols)         ! level of max moist static energy
!
!--------------------------Local Variables------------------------------
!
   real(r8) qstpmix (pcols,pver)      !
   real(r8) qmix    (pcols,pver)      !

   real(r8) capeten(pcols,5)     ! provisional value of cape
   real(r8), intent(out) ::  tv(pcols,pver)       !
   real(r8), intent(out) ::  tpv(pcols,pver)      !
   real(r8) buoy(pcols,pver)
   real(r8) qs(pcols,pver)       ! saturation specific humidity

   real(r8) a1(pcols)
   real(r8) a2(pcols)
   real(r8) estp(pcols)
   real(r8) pl(pcols)
   real(r8) plexp(pcols)
   real(r8) hmax(pcols)
   real(r8) hmin(pcols)
   real(r8) hmn(pcols,pver)
   real(r8) y(pcols)

   logical plge600(pcols)
   integer knt(pcols)
   integer lelten(pcols,5)
   integer khmin(pcols)         ! level of min moist static energy

   real(r8) cp
   real(r8) e
   real(r8) grav

   integer i
   integer k
   integer msg
   integer n
   integer idx0 

   real(r8) rd
   real(r8) rl
   real(r8) esttmp 
#ifdef PERGRO
   real(r8) rhd
#endif
   real(r8) theta
   real(r8) thetase(pcols, pver)
   real(r8) hp(pcols, pver)
   real(r8) h(pcols, pver)
   real(r8) dhp
   real(r8) qsl(pcols)

   real(r8) thetaselcl(pcols)
   real(r8) est(pcols)
   real(r8) thetasep, erdt, er, gammap, aa1
   integer  kn
   integer mx1(pcols), mx2(pcols)
   real(r8) tl1, tl2, pl1, pl2
   real(r8) dpcloud
   integer  scheme
!
!-----------------------------------------------------------------------
!
   do n = 1,5
      do i = 1,ncol
         lelten(i,n) = pver
         capeten(i,n) = 0.
      end do
   end do
!
   do i = 1,ncol
      lon(i) = pver
      knt(i) = 0
      lel(i) = pver
      mx(i) = 1
      khmin(i)=pver
      cape(i) = 0.
      negcape(i) = 0.
      hmax(i) = 0.
      hmin(i) = 1.e10 
      deepconv(i) = .false.
   end do
!
      do i = 1, pcols
        do k=pver, 1, -1
           buoy(i,k) = 0.
        end do
      end do

   tp(:ncol,:) = t(:ncol,:)
   qstp(:ncol,:) = q(:ncol,:)
!
! set "launching" level(mx) to be at maximum moist static energy.
! search for this level stops at planetary boundary layer top.

! Find level of max static energy below h_min level 
!
   do k = pver,msg + 1,-1
      do i = 1,ncol
         hmn(i,k) = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
         esttmp = c1*exp((c2* (t(i,k)-tfreez))/((t(i,k)-tfreez)+c3))
         if ( p(i,k)-esttmp > 0. ) then
            qs(i,k) = eps1*esttmp/ (p(i,k)-esttmp)
         else
            qs(i,k) = 1.0
         end if
      end do
   end do

! find min hmn level above 700 mb
!
      do i = 1, ncol
        do k = msg+1, pver
!          if( (pf(i,pverp)-p(i,k)) >300.0 .and. hmin(i) .ge. hmn(i,k) ) then
           if( p(i,k) .le. 700. .and. hmin(i) .ge. hmn(i,k) ) then
               hmin(i) = hmn(i,k)
               khmin(i) = k
          endif
        enddo
      enddo
!
     do i = 1,ncol
       do k = msg + 1, pver

! rh threshold does not apply to midlevel convection
!
        if ( k.gt. khmin(i) .and. k .le. nint(pblt(i)) .and. &
             hmn(i,k).gt.hmax(i) .and. k <= pver-1 ) then
              hmax(i) = hmn(i,k)
              mx(i) = k
              deepconv(i) = .true.
         end if
      enddo
      
   enddo

!------------------------------
   do i = 1,ncol
      do k = pver,1,-1
         theta  = t(i,k)*(1000.0/p(i,k))**cappa
         thetase(i,k) = theta*exp(latvap/cpair*q(i,k)/t(i,k))
      enddo
   enddo
!---
   do i = 1,ncol

   tl(i) = t(i,1)
   if( deepconv(i) ) then

      call condenst_lcl( p(i,mx(i)), t(i,mx(i)), q(i,mx(i)), tl(i),  pl(i))
      if( tl(i) >= t(i,mx(i))) then
              tl(i) = t(i,mx(i))
	      pl(i) = p(i,mx(i))
      endif
!--
!      e = p(i,mx(i))*q(i,mx(i))/ (eps1+q(i,mx(i)))
!      tl(i) = 2840./ (3.5*log(t(i,mx(i)))-log(e)-4.805) + 55.
!      if (tl(i) < t(i,mx(i))) then
!         plexp(i) = (1./ (0.2854* (1.-0.28*q(i,mx(i)))))
!         pl(i) = p(i,mx(i))* (tl(i)/t(i,mx(i)))**plexp(i)
!      else
!         tl(i) = t(i,mx(i))
!         pl(i) = p(i,mx(i))     
!      end if
!
!--------------------------------			      
      lcl(i) = mx(i)
      estp(i) = c1*exp((c2* (tl(i)-tfreez))/((tl(i)-tfreez)+c3))
      if(pl(i)-estp(i)>0.) then
         qsl(i)  = eps1*estp(i)/ (pl(i)-estp(i))
      else
         qsl(i)  = 1.0
      endif
      thetaselcl(i) =  tl(i)*(1000.0/pl(i))**cappa &
                      *exp(latvap/cpair*qsl(i)/tl(i))
!
! calculate lifting condensation level (lcl).
!
     do k = pver,msg + 2,-1
         if (k <= mx(i) .and. (pf(i,k+1) > pl(i) .and. pf(i,k) <= pl(i))) then
            lcl(i) = k 
         end if
     end do
!---------------------------------------------------------------------------
!
! if lcl is above the nominal level of non-divergence (600 mbs),
! no deep convection is permitted (ensuing calculations
! skipped and cape retains initialized value of zero).
!
!     plge600(i) = (pf(i,pverp)-pl(i)) .lt. 400.0 .and. deepconv(i)

!    plge600(i) = pl(i) .gt. 600.0/1013.*pf(i,pverp) .and. deepconv(i)
     plge600(i) = pl(i) .gt. 650.0/1013.*pf(i,pverp) .and. deepconv(i)

    endif

  enddo
!
! initialize parcel properties in sub-cloud layer below lcl.
!
   do k = pver,msg + 1,-1
      do i=1,ncol
         if (k > lcl(i) .and. k <= mx(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/(1.+q(i,k))
            qstp(i,k) = q(i,mx(i))

            tp(i,k) = t(i,mx(i))* (p(i,k)/p(i,mx(i)))**(0.2854* (1.-0.28*q(i,mx(i))))
!
! buoyancy is increased by 0.5 k as in tiedtke
!
            tpv(i,k) = (tp(i,k)+tpert(i))*(1.+1.608*qstp(i,k))/(1.0+q(i,mx(i)))
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5 
         end if
      end do
   end do
!
! define parcel properties at lcl (i.e. level immediately above pl).
!

   do i=1,ncol
      do k = pver,msg + 1,-1

         if (k <= lcl(i) .and. plge600(i)) then

         scheme = 0   ! scheme = 0 for Wu'scheme
         if(scheme .eq. 0) then
!---
            tp(i,k) = tp(i,k+1)* (p(i,k)/p(i,k+1))**cappa
            est(i)    = c1*exp((c2* (tp(i,k)-tmelt))/((tp(i,k)-tmelt)+c3))
            if(p(i,k)-est(i)>0) then
               qstp(i,k) = eps1*est(i)/(p(i,k)-est(i))
  	    else
	       qstp(i,k) = 1.0
            endif
            kn = 1
21	    thetasep =  tp(i,k)*(1000.0/p(i,k))**cappa &
		             *exp(latvap/cpair*qstp(i,k)/tp(i,k))
!           gammap = qstp(i,k)*(1. + qstp(i,k)/eps1)*eps1*rl/(rd*tp(i,k)**2)*rl/cp
            gammap = qstp(i,k)**2*p(i,k)/(eps1*est(i))*c2*c3/((tp(i,k)-tmelt+c3)**2)*rl/cp
            erdt = thetasep/tp(i,k)*(1.0 + gammap -rl/cp*qstp(i,k)/tp(i,k))
            er   = thetasep - thetaselcl(i)
            aa1   = tp(i,k) - er/erdt
	    if( abs(aa1-tp(i,k)).le.0.01 .or. kn.gt.20) then
		      tp(i, k) = aa1
                  est(i)= c1*exp((c2* (tp(i, k)-tmelt))/ ((tp(i, k)-tmelt)+c3))
                  if(p(i,k)-est(i)>0) then
                     qstp(i,k) = eps1*est(i)/(p(i,k)-est(i))
   	          else
		             qstp(i,k) = 1.0
                  endif
	     else
			      tp(i, k) = aa1
                  est(i)= c1*exp((c2* (tp(i, k)-tmelt))/ ((tp(i, k)-tmelt)+c3))
                  if(p(i,k)-est(i)>0) then
                     qstp(i,k) = eps1*est(i)/(p(i,k)-est(i))
   	          else
		             qstp(i,k) = 1.0
                  endif
                  kn = kn +1
                  goto 21  
             endif
!-----------------
         else

            call parcel_property(pl(i), qsl(i), tl(i), p(i,k), qstp(i,k), tp(i,k))   ! according to Zhang's

         endif
!
!--------------------------------------------
! buoyancy is increased by 0.5 k in cape calculation.
! dec. 9, 1994
!
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/(1.0+q(i,k))
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+1.608*qstp(i,k)) / (1.+qstp(i,k))
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5 
        end if
      end do
   end do
!------------------------------------
   do i = 1,ncol
      lel(i) = lcl(i)
      do k = pver, msg + 1, -1
         if( k < lcl(i) .and. buoy(i,k)>0 .and. buoy(i,k-1)<=0 .and. plge600(i)) then
  	         lel(i) = k-1
	         goto 15
         endif
	  enddo
15 enddo
!----
   do i = 1,ncol
      cape(i) = 0.0 
      do k = pver,msg + 1,-1
         if (k >= lel(i) .and. k < mx(i) .and. plge600(i) ) then
             cape(i) = cape(i) + buoy(i,k)*rgas*log(pf(i,k+1)/pf(i,k))
         endif 
       end do
       cape(i) = min(max(cape(i), 0._r8), 1.e+10)
   end do
!
   return
end subroutine buoyan_wu


subroutine cldprp_wu(lchnk   , &
                  q       ,t       ,u       ,v       ,p       , &
                  z       ,s       ,mu      ,eu      ,du      , &
                  md      ,ed      ,sd      ,qd      ,mc      , &
                  qu      ,su      ,zf      ,qst     ,hmn     , &
                  hsat    ,shat    ,ql      , &
                  cmeg    ,jb      ,lel     ,jt      ,jlcl    , &
                  mx      ,j0      ,jd      ,rl      ,il2g    , &
                  rd      ,grav    ,cp      ,msg     ,duql    , &
                  pflx    ,evp     ,cu      ,rprd    ,limcnv  , &
		  qp      ,tp      ,pf      ,lcl     ,omega   , &
		  ptendt  ,ptendq  ,uhat    ,vhat    ,  &
		  up      ,vp      ,ud      ,vd      ,tv    ,tpv )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! may 09/91 - guang jun zhang, m.lazare, n.mcfarlane.
!             original version cldprop.
! 
! Author: See above, modified by P. Rasch
! This is contributed code not fully standardized by the CCM core group.
!
! this code is very much rougher than virtually anything else in the CCM
! there are debug statements left strewn about and code segments disabled
! these are to facilitate future development. We expect to release a
! cleaner code in a future release
!
! the documentation has been enhanced to the degree that we are able
!
!-----------------------------------------------------------------------

   implicit none

!------------------------------------------------------------------------------
!
! Input arguments
!
   integer, intent(in) :: lchnk                  ! chunk identifier

   real(r8), intent(in) :: q(pcols,pver)         ! spec. humidity of env
   real(r8), intent(in) :: t(pcols,pver)         ! temp of env
   real(r8), intent(in) :: p(pcols,pver)         ! pressure of env
   real(r8), intent(in) :: z(pcols,pver)         ! height of env
   real(r8), intent(in) :: s(pcols,pver)         ! normalized dry static energy of env
   real(r8), intent(in) :: zf(pcols,pverp)       ! height of interfaces
   real(r8), intent(in) :: u(pcols,pver)         ! zonal velocity of env
   real(r8), intent(in) :: v(pcols,pver)         ! merid. velocity of env

   integer, intent(in) :: jb(pcols)              ! updraft base level
   integer, intent(inout) :: lel(pcols)             ! updraft launch level
   integer, intent(out) :: jt(pcols)              ! updraft plume top
   integer, intent(out) :: jlcl(pcols)            ! updraft lifting cond level
   integer, intent(in) :: mx(pcols)              ! updraft base level (same is jb)
   integer, intent(out) :: j0(pcols)              ! level where updraft begins detraining
   integer, intent(out) :: jd(pcols)              ! level of downdraft
   integer, intent(in) :: limcnv                 ! convection limiting level
   integer, intent(in) :: il2g                   !CORE GROUP REMOVE
   integer, intent(in) :: msg                    ! missing moisture vals (always 0)
   real(r8), intent(in) :: rl                    ! latent heat of vap
   real(r8), intent(in) :: shat(pcols,pver)      ! interface values of dry stat energy
   real(r8), intent(in) :: uhat(pcols,pver)  
   real(r8), intent(in) :: vhat(pcols,pver)  
!
! output
!
   real(r8), intent(out) :: rprd(pcols,pver)     ! rate of production of precip at that layer
   real(r8), intent(out) :: du(pcols,pver)       ! detrainement rate of updraft
   real(r8), intent(out) :: ed(pcols,pver)       ! entrainment rate of downdraft
   real(r8), intent(out) :: eu(pcols,pver)       ! entrainment rate of updraft
   real(r8), intent(out) :: hmn(pcols,pver)      ! moist stat energy of env
   real(r8), intent(out) :: hsat(pcols,pver)     ! sat moist stat energy of env
   real(r8), intent(out) :: mc(pcols,pver)       ! net mass flux
   real(r8), intent(out) :: md(pcols,pver)       ! downdraft mass flux
   real(r8), intent(out) :: mu(pcols,pver)       ! updraft mass flux
   real(r8), intent(out) :: pflx(pcols,pverp)    ! precipitation flux thru layer
   real(r8), intent(out) :: qd(pcols,pver)       ! spec humidity of downdraft
   real(r8), intent(out) :: ql(pcols,pver)       ! liq water of updraft
   real(r8), intent(out) :: qst(pcols,pver)      ! saturation spec humidity of env.
   real(r8), intent(out) :: qu(pcols,pver)       ! spec hum of updraft
   real(r8), intent(out) :: sd(pcols,pver)       ! normalized dry stat energy of downdraft
   real(r8), intent(out) :: su(pcols,pver)       ! normalized dry stat energy of updraft

   real(r8), intent(out) :: up(pcols,pver)   
   real(r8), intent(out) :: vp(pcols,pver)   
   real(r8), intent(out) :: ud(pcols,pver)   
   real(r8), intent(out) :: vd(pcols,pver)   

   integer  lcl(pcols)
   real(r8), intent(inout) :: qp(pcols,pver)       
   real(r8), intent(inout) :: tp(pcols,pver)        
   real(r8), intent(in) :: pf(pcols,pverp) 
   real(r8), intent(in) :: omega(pcols,pver)   
   real(r8), intent(in) :: ptendt(pcols,pver)   
   real(r8), intent(in) :: ptendq(pcols,pver)   

   real(r8), intent(inout) ::  tpv(pcols,pver) 
   real(r8), intent(inout) ::  tv (pcols,pver) 
   real(r8)  h  (pcols,pver) 
   real(r8)  hp (pcols,pver) 
   real(r8)  coef, dhp, dtp, est0, qst0, dqsdtemp, aleft, aright,dqpdtemp
   real(r8)  tpf (pcols,pverp) 
   real(r8)  qpf (pcols,pverp) 
   real(r8)  dtpdz (pcols,pver) 
   real(r8)  dqpdz (pcols,pver) 
   real(r8)  epss  (pcols,pver) 
   real(r8)  sp   (pcols,pver) 
   real(r8)  Mass (pcols,pver) 
   real(r8)  dqpdt(pcols,pver) 
   real(r8)  dtpdt(pcols,pver) 

   real(r8) rd                   ! gas constant for dry air
   real(r8) grav                 ! gravity
   real(r8) cp                   ! heat capacity of dry air
!
! Local workspace
!
   real(r8) thetasemin(pcols)
   real(r8) thetase(pcols, pver)
   real(r8) theta
   real(r8) kl(pcols,pver)
   real(r8) tf(pcols,pver)
   real(r8) qf(pcols,pver)
   real(r8) uf(pcols,pver)
   real(r8) vf(pcols,pver)

   real(r8) gamma(pcols,pver)
   real(r8) dz(pcols,pver)
   real(r8) iprm(pcols,pver)
   real(r8) hu(pcols,pver)
   real(r8) hd(pcols,pver)
   real(r8) dd(pcols,pver)
   real(r8) eps(pcols,pver)
   real(r8) qsthat(pcols,pver)
   real(r8) hsthat(pcols,pver)
   real(r8) gamhat(pcols,pver)
   real(r8) hmnhat(pcols,pver)
   real(r8) cu(pcols,pver)
   real(r8) duql(pcols,pver)
   real(r8) evp(pcols,pver)
   real(r8) cmeg(pcols,pver)
   real(r8) qds(pcols,pver)
   real(r8) mass_bot(pcols,pver)
   real(r8) hmin(pcols)
   real(r8) eps0(pcols)
   real(r8) rmue(pcols)
   real(r8) zuef(pcols)
   real(r8) zdef(pcols)
   real(r8) epsm(pcols)
   real(r8) ratmjb(pcols)
   real(r8) est(pcols)
   real(r8) totpcp(pcols)
   real(r8) totevp(pcols)
   real(r8) alfa(pcols)
   real(r8) ql1, ql2
   real(r8) tu
   real(r8) estu
   real(r8) qstu

   real(r8) small
   real(r8) mdt
   real(r8) espk,qspk,tpk, hspk
   real(r8) dtempera
   real(r8) gammap, wktmp, thetaseu, thetasep, erdt, er, a1

   integer khighest
   integer klowest
   integer kount, kn
   integer i,k, kk

   logical doit(pcols)
   logical doitt(pcols)
   logical done(pcols)

   real(r8), parameter :: tmax_fsnow = 273.16 - 0.0     ! max temperature for cloud ice formation
   real(r8), parameter :: tmin_fsnow = 273.16 - 10.0     ! max temperature for cloud ice formation
   real(r8)  fice(pcols, pver)
   real(r8)  kcoef
   real(r8)  downdraft
   integer Markreset
   
!
!------------------------------------------------------------------------------

   do k = 1,pver
      do i = 1,il2g
         dz(i,k) = zf(i,k) - zf(i,k+1)
      end do
   end do
!
! initialize many output and work variables to zero
!
   pflx(:il2g,1) = 0

   do k = 1,pver
      do i = 1,il2g
         mu(i,k) = 0.
         eps(i,k) = 0.
         eu(i,k) = 0.
         du(i,k) = 0.
         ql(i,k) = 0.
         cu(i,k) = 0.
         evp(i,k) = 0.
         duql(i,k) = 0.
         cmeg(i,k) = 0.
         md(i,k) = 0.
         ed(i,k) = 0.
	 dd(i,k) = 0.
         mc(i,k) = 0.

         est(i) = c1*exp((c2* (t(i,k)-tfreez))/((t(i,k)-tfreez)+c3))
         if(p(i,k)-est(i)>0) then
            qst(i,k) = eps1*est(i)/(p(i,k)-est(i))
		 else
	    qst(i,k) = 1.0
         endif
!        gamma(i,k) = qst(i,k)*(1. + qst(i,k)/eps1)*eps1*rl/(rd*t(i,k)**2)*rl/cp
         gamma(i,k) = qst(i,k)**2*p(i,k)/(eps1*est(i))*c2*c3/((t(i,k)-tfreez+c3)**2)*rl/cp
!------------------
         hmn(i,k)  = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
         hsat(i,k) = cp*t(i,k) + grav*z(i,k) + rl*qst(i,k)
         rprd(i,k) = 0.

      end do
   end do
!
!jr Set to zero things which make this routine blow up
!
   do k=1,msg
      do i=1,il2g
         rprd(i,k) = 0.
      end do
   end do
!
! interpolate the layer values of qst, hsat and gamma to
! layer interfaces
!
   do i = 1,il2g
      totpcp(i) = 0.
      totevp(i) = 0.
   end do

   do i = 1, il2g
      hmnhat(i,msg+1) = hmn(i,msg+1)
      hsthat(i,msg+1) = hsat(i,msg+1)
      qsthat(i,msg+1) = qst (i,msg+1)
      gamhat(i,msg+1) = gamma(i,msg+1)
      tf(i,msg+1)     = t(i,msg+1)
      qf(i,msg+1)     = q(i,msg+1)
      uf(i,msg+1)     = u(i,msg+1)
      vf(i,msg+1)     = v(i,msg+1)
   enddo
   do k = msg+2, pver 
      do i = 1,il2g
         coef = log(pf(i,k)/p(i,k))/log(p(i,k-1)/p(i,k))
         tf(i,k) = t(i,k)       + (t(i,k-1)-t(i,k))*coef
         qf(i,k) = q(i,k)       + (q(i,k-1)-q(i,k))*coef
         uf(i,k) = u(i,k)       + (u(i,k-1)-u(i,k))*coef
         vf(i,k) = v(i,k)       + (v(i,k-1)-v(i,k))*coef

         est(i) = c1*exp((c2* (tf(i,k)-tfreez))/((tf(i,k)-tfreez)+c3))
         if(pf(i,k)-est(i)>0) then
            qsthat(i,k) = eps1*est(i)/(pf(i,k)-est(i))
	 else
	    qsthat(i,k) = 1.0
         endif

         gamhat(i,k) = gamma(i,k)     + (gamma(i,k-1)-gamma(i,k))*coef
         hsthat(i,k) = cpair*tf(i,k) + gravit*zf(i,k) + latvap*qsthat(i,k)
         hmnhat(i,k) = cpair*tf(i,k) + gravit*zf(i,k) + latvap*qf(i,k)
      enddo
   enddo     

   do k = 1,pver
      do i = 1,il2g
         qu(i,k) = qf(i,k)
         su(i,k) = tf(i,k) + gravit/cp*zf(i,k)
         hu(i,k) = su(i,k)*cp + rl*qu(i,k)
         up(i,k) = uf(i,k)
         vp(i,k) = vf(i,k)

         qds(i,k)= qsthat(i,k)
         sd(i,k) = tf(i,k) + gravit/cp*zf(i,k)
         qd(i,k) = qf(i,k)
         hd(i,k) = su(i,k)*cp + rl*qu(i,k)
         ud(i,k) = uf(i,k)
         vd(i,k) = vf(i,k)
      enddo
   enddo
!
! initialize cloud top to highest plume top.
!jr changed hard-wired 4 to limcnv+1 (not to exceed pver)
!
   do i = 1,il2g
      jt(i) = max(lel(i),limcnv+1)
      jt(i) = min(jt(i),pver)
      jd(i) = pver
      jlcl(i) = lcl(i)
      hmin(i) = 1.E6
   end do

!=========================================================================
   do i = 1,il2g
      do k =  2, pver
!        tv(i,k)  = t(i,k)* (1.+0.608*q(i,k))
!        tpv(i,k) = tp(i,k)* (1.+0.608*qp(i,k))
         hp(i,k)  = cp*tp(i,k) + grav*z(i,k) + rl*qp(i,k)
         sp(i,k)  = tp(i,k) + (gravit/cpair)*z(i,k)
!--
         coef=log(pf(i,k)/p(i,k))/log(p(i,k-1)/p(i,k))
         tpf(i,k) = tp(i,k) +(tp(i,k-1)-tp(i,k))*coef
         qpf(i,k) = qp(i,k) +(qp(i,k-1)-qp(i,k))*coef
      enddo
      coef=log(pf(i,pverp)/p(i,pver))/log(p(i,pver-1)/p(i,pver))
      tpf(i,pverp) = tp(i,pver) + (tp(i,pver-1)-tp(i,pver))*coef
      qpf(i,pverp) = qp(i,pver) + (qp(i,pver-1)-qp(i,pver))*coef
      tpf(i,1)  = tpf(i,2)
      qpf(i,1)  = qpf(i,2)
   enddo
   do i = 1,il2g
      do k = pver, 2, -1
         dtpdz(i,k) = (tpf(i,k+1)-tpf(i,k))/(zf(i,k+1)-zf(i,k))    ! K/m
         dqpdz(i,k) = (qpf(i,k+1)-qpf(i,k))/(zf(i,k+1)-zf(i,k))    ! 
      end do
      dtpdz(i,1) = dtpdz(i,2)  
      dqpdz(i,1) = dqpdz(i,2)  
   end do
!
! compute the entrainnment rate: eps
!
   do i = 1,il2g
      do k=pver, 1, -1
         Mu(i,k) =0.0
	 Eu(i,k) =0.0
         Du(i,k) =0.0
         Mass(i,k) = 0.0
	 epss(i,k) = 0.0
         if (k >= jt(i) .and. k <= jb(i)) then
             su(i,k) = tpf(i,k) + gravit/cp*zf(i,k)
             qu(i,k) = qpf(i,k)
             hu(i,k) = su(i,k)*cp + rl*qu(i,k)
         end if
      enddo
   enddo
   do i=1,il2g
      doit(i) = .true. 
      done(i) = .false.
      doitt(i)= .true.
   end do
!---
   do i = 1,il2g
      do k=pver, msg, -1
          if( k > lcl(i) .and. k <= jb(i) ) then
             dhp = tp(i,k) -t(i,k)  
             if(dhp <= 1.e-01) then
                epss(i,k) = 0.0
	     else
                epss(i,k) = - (grav/cpair*tpv(i,k)/tv(i,k) + dtpdz(i,k))/dhp
                if(epss(i,k) < 0.0) epss(i,k) = 0.0
	     endif 
          elseif(k >= jt(i) .and. k <= lcl(i)) then
    	     dhp = rl*(qp(i,k)-q(i,k))/cpair + (tp(i,k)-t(i,k))
             if(dhp.le.1.e-01) then
                epss(i,k) = 0.0
  	     else
                epss(i,k) = - (grav/cpair*tpv(i,k)/tv(i,k) + &
				              rl/cpair*dqpdz(i,k) + dtpdz(i,k) )/dhp
	     endif
          endif
!
! specify the updraft mass flux mu, entrainment eu, detrainment du
! and moist static energy hu.
! here and below mu, eu,du, md and ed are all normalized by mb
!
          if(k == jb(i)) then
	      Mass(i,k) = 1.0
              Mu(i,k)   = Mass(i,k)
          endif
          if(k == jt(i)) then
	      Mass(i,k) = 0.0
              Mu(i,k)   = 0.0
          endif
          if( k > jt(i) .and. k < jb(i) ) then
              Mass(i,k) = Mass(i,k+1)*exp(epss(i,k)*(zf(i,k)-zf(i,k+1))) 
              Mu(i,k)  = Mass(i,k)
          endif
!-----------
!
          if( k >= jt(i) .and. k <= jb(i) ) then
              if(k ==jb(i)) then
                  Eu(i,k) = (mu(i,jb(i))-0.0)/(zf(i,jb(i))-zf(i,pverp))
                  Du(i,k) = 0.0
	      elseif(k == jt(i)) then
	          Eu(i,k) = 0.0
	          Du(i,k) = -(0.0-Mu(i,k+1))/(zf(i,k) - zf(i,k+1))
	      elseif(k > lcl(i)) then
	          Eu(i,k) = (Mu(i,k)-Mu(i,k+1))/(zf(i,k)-zf(i,k+1))
                  Du(i,k) = 0.0
	      else
                 Eu(i,k) = (Mu(i,k)-Mu(i,k+1))/(zf(i,k)-zf(i,k+1))
                 if(Eu(i,k) >0) then
	            Du(i,k) = 0.0
 	         else
		    Du(i,k) = -Eu(i,k)
		    Eu(i,k) = 0.0
 	         endif
              endif
         endif

         if( k >= jt(i) .and. k <= lcl(i) ) then 
             Eu(i,k) = Eu(i,k) + upconv_alfa* Mu(i,k+1)
             Du(i,k) = Du(i,k) + upconv_alfa* Mu(i,k+1) 
         endif
!---
         if (k < jb(i) .and. k >= jt(i) ) then
            if ( Mu(i,k) < 0.01  ) then
               up(i,k) = uf(i,k)
               vp(i,k) = vf(i,k)
            else
               wktmp = Mu(i,k) - Du(i,k)*0.5*(zf(i,k+1)-zf(i,k)) 
               up(i,k) = (Mu(i,k+1)*up(i,k+1) - (zf(i,k+1)-zf(i,k))* &
 			       (Eu(i,k)*u(i,k) -0.5*Du(i,k)*up(i,k+1)) )/ wktmp
               vp(i,k) = (Mu(i,k+1)*vp(i,k+1) - (zf(i,k+1)-zf(i,k))* &
 			       (Eu(i,k)*v(i,k) -0.5*Du(i,k)*vp(i,k+1)) )/ wktmp
            end if
         endif
!
         if (k <= jb(i)-1 .and. k >= lel(i) ) then
            if (mu(i,k) < 0.001) then
               wktmp = 0.001 - Du(i,k)*0.5*(zf(i,k+1)-zf(i,k)) 
            else
               wktmp = Mu(i,k) - Du(i,k)*0.5*(zf(i,k+1)-zf(i,k)) 
            endif
            hu(i,k) = (Mu(i,k+1)*hu(i,k+1) - (zf(i,k+1)-zf(i,k))* &
 			       (Eu(i,k)*hmn(i,k) -0.5*Du(i,k)*hu(i,k+1)) )/ wktmp
         end if
!
! reset cloud top index beginning from two layers above the
! cloud base (i.e. if cloud is only one layer thick, top is not reset
!
         if (doit(i) .and. k <= jb(i)-2 .and. k >= lel(i)-1) then
            if (hu(i,k) <= hmnhat(i,k) .and. hu(i,k+1) > hmnhat(i,k+1) ) then
               if (hu(i,k)-hmnhat(i,k) < -2000.) then
                  jt(i) = k + 1
                  doit(i) = .false.
               else
                  jt(i) = k
                  doit(i) = .false.
               end if
            end if
         end if
!----
!         if (doit(i) .and. k <= jb(i)-2 .and. k >= lel(i)-1) then
!            if (hu(i,k) <= hsthat(i,k) .and. hu(i,k+1) > hsthat(i,k+1) &
!             .and. mu(i,k) >= 0.002) then                                ! 0.02
!               if (hu(i,k)-hsthat(i,k) < -2000.) then
!                  jt(i) = k + 1
!                  doit(i) = .false.
!               else
!                  jt(i) = k
!                  doit(i) = .false.
!               end if
!            end if
!         end if
!---
         if (k >= lel(i) .and. k < jt(i) ) then
            mu(i,k) = 0.
            eu(i,k) = 0.
            du(i,k) = 0.
            hu(i,k) = cp*tf(i,k) + gravit*zf(i,k) + rl*qf(i,k)
         end if
         if (k == jt(i)) then
            du(i,k) = mu(i,k+1)/dz(i,k)
            mu(i,k) = 0.
	        Eu(i,k) = 0.0
		    Du(i,k) = -(0.0-Mu(i,k+1))/(zf(i,k) - zf(i,k+1))
            wktmp = Du(i,k)*0.5*(zf(i,k+1)-zf(i,k)) 
            hu(i,k) = (Mu(i,k+1)*hu(i,k+1) - (zf(i,k+1)-zf(i,k))* &
 			       (Eu(i,k)*hmn(i,k) -0.5*Du(i,k)*hu(i,k+1)) )/ wktmp
         end if
!
! re-calculate the tp and qp
!
         if (( .not. done(i) .and. k > jt(i) .and. k < jb(i))) then
               if (mu(i,k) < 0.01) then
                  wktmp = 0.01 - Du(i,k)*0.5*(zf(i,k+1)-zf(i,k)) 
               else
                  wktmp = Mu(i,k) - Du(i,k)*0.5*(zf(i,k+1)-zf(i,k)) 
               endif
               su(i,k) = (Mu(i,k+1)*su(i,k+1) - (zf(i,k+1)-zf(i,k))* &
 			       (Eu(i,k)*s(i,k) -0.5*Du(i,k)*su(i,k+1)) )/ wktmp
               qu(i,k) = (Mu(i,k+1)*qu(i,k+1) - (zf(i,k+1)-zf(i,k))* &
 			       (Eu(i,k)*q(i,k) -0.5*Du(i,k)*qu(i,k+1)) )/ wktmp

               tu = su(i,k) - grav/cp*zf(i,k)
               estu = c1*exp((c2* (tu-tfreez))/ ((tu-tfreez)+c3))
               qstu = 0.622*estu/ (pf(i,k)-0.378*estu)
               if (qu(i,k) >= qstu) then
                  jlcl(i) = k
                  done(i) = .true.
              end if
         end if

!------------------------------------
         if ( k > jt(i) .and. k <= jlcl(i) ) then
             tpk = tpf(i,k) 
  	    kn =1
11           espk = c1*exp((c2* (tpk-tmelt))/((tpk-tmelt)+c3))
             if( pf(i,k)-espk>0 ) then
                qspk = eps1*espk/(pf(i,k)-espk)
		     else
		        qspk = 1.0
             endif
             hspk   = cpair*tpk + gravit*zf(i,k) + latvap*qspk
!            gammap = qspk*(1. + qspk/eps1)*eps1*rl/(rd*tpk**2)*rl/cp
             gammap = qspk**2*pf(i,k)/(eps1*espk)*c2*c3/((tpk-tmelt+c3)**2)*rl/cp
             dtempera = (hu(i,k) - hspk) /(1.0+gammap)/ cpair   
             tpk = tpk + dtempera
             if(dtempera >= 0.01 .and. kn < 30) then
	        kn= kn + 1
                goto 11
             endif
             su(i,k) = tpk + (gravit/cpair)*zf(i,k)
             qu(i,k) = qspk
         end if
!------------------------------------------------------------------------------------------
! re-calculate the temperature and humidity of the parcel
!
        if (k > lcl(i) .and. k < jb(i) ) then    
            tu  =  su(i,k) - gravit/cp*zf(i,k)
            tp(i,k-1) = tu* (p(i,k-1)/pf(i,k))**(0.2854* (1.-0.28*qu(i,k)))
            qp(i,k-1) = qu(i,k)
        endif
        if ( k > jt(i) .and. k <= jlcl(i) ) then
!----
         Markreset = 0    ! 0 for wtw's method   
         if( Markreset .eq. 0) then
            tu  =  su(i,k) - gravit/cp*zf(i,k)
			thetaseu =  tu*(1000.0/pf(i,k))**cappa &
			              *exp(latvap/cpair*qu(i,k)/tu)
            tp(i,k-1) = tu* (p(i,k-1)/pf(i,k))**cappa
            est(i)    = c1*exp((c2* (tp(i,k-1)-tmelt))/((tp(i,k-1)-tmelt)+c3))
            if(p(i,k-1)-est(i)>0) then
               qp(i,k-1) = eps1*est(i)/(p(i,k-1)-est(i))
	   	    else
		       qp(i,k-1) = 1.0
            endif
            kn = 1
21	        thetasep =  tp(i,k-1)*(1000.0/p(i,k-1))**cappa &
			              *exp(latvap/cpair*qp(i,k-1)/tp(i,k-1))
!           gammap = qp(i,k-1)*(1. + qp(i,k-1)/eps1)*eps1*rl/(rd*tp(i,k-1)**2)*rl/cp
            gammap = qp(i,k-1)**2*p(i,k-1)/(eps1*est(i))*c2*c3/((tp(i,k-1)-tmelt+c3)**2)*rl/cp
            erdt = thetasep/tp(i,k-1)*(1.0 + gammap -rl/cp*qp(i,k-1)/tp(i,k-1))
			er   = thetasep - thetaseu
            a1   = tp(i,k-1) - er/erdt
		    if( abs(a1-tp(i,k-1)).le.0.01 .or. kn.gt.10) then
			      tp(i, k-1) = a1
                  est(i)= c1*exp((c2* (tp(i, k-1)-tmelt))/ ((tp(i, k-1)-tmelt)+c3))
                  if(p(i,k-1)-est(i)>0) then
                     qp(i,k-1) = eps1*est(i)/(p(i,k-1)-est(i))
	   	          else
		             qp(i,k-1) = 1.0
                  endif
		    else
			      tp(i, k-1) = a1
                  est(i)= c1*exp((c2* (tp(i, k-1)-tmelt))/ ((tp(i, k-1)-tmelt)+c3))
                  if(p(i,k-1)-est(i)>0) then
                     qp(i,k-1) = eps1*est(i)/(p(i,k-1)-est(i))
	   	          else
		             qp(i,k-1) = 1.0
                  endif
                  kn = kn +1
                  goto 21  
            endif

          else
!---------------------
!  Zhang's air parcel
!
            tu  =  su(i,k) - gravit/cp*zf(i,k)
            call parcel_property(pf(i,k), qu(i,k), tu, p(i,k-1), qp(i,k-1), tp(i,k-1))   ! according to ZHang's
!----------------------------------------------
          endif
         endif 
!----------------------
         if (k > jt(i) .and. k < jb(i) ) then    
            tpv(i,k-1) = tp(i,k-1)* (1.+1.608*qp(i,k-1))/(1.0+qp(i,k-1))
            hp(i,k-1)  = cp*tp(i,k-1) + grav*z(i,k-1) + rl*qp(i,k-1)
            sp(i,k-1)  = tp(i,k-1) + (gravit/cpair)*z(i,k-1)
            coef=log(pf(i,k-1)/p(i,k-1))/log(p(i,k)/p(i,k-1))
            tpf(i,k-1) = tp(i,k-1) +(tp(i,k)-tp(i,k-1))*coef
            qpf(i,k-1) = qp(i,k-1) +(qp(i,k)-qp(i,k-1))*coef
            dtpdz(i,k-1) = (tpf(i,k)-tpf(i,k-1))/(zf(i,k)-zf(i,k-1))    ! K/m
            dqpdz(i,k-1) = (qpf(i,k)-qpf(i,k-1))/(zf(i,k)-zf(i,k-1))    ! 

            if (doitt(i) .and. tpv(i,k-1) <= tv(i,k-1) .and. tpv(i,k) > tv(i,k)) then
		    jt(i) = k-1
                    doitt(i) = .false.
            endif
         endif
      end do
   end do

  do i= 1,il2g
     do k = pver, 1,-1
        if( k < jt(i) ) then
            tp(i,k) = t(i,k) 
            qp(i,k) = q(i,k)
            qu(i,k) = qf(i,k)
            su(i,k) = tf(i,k) + gravit/cp*zf(i,k)
        endif
     enddo
  enddo   
!-------------------------------------------------------------
!
! compute condensation in updraft
   do i = 1,il2g
      do k = pver, msg + 2,-1
         if (k >= jt(i) .and. k <= jlcl(i) ) then
            cu(i,k) = - eu(i,k)*s(i,k) + du(i,k)*0.5*(su(i,k)+su(i,k+1)) &
       	               + (su(i,k+1)*mu(i,k+1) - su(i,k)*mu(i,k))/(zf(i,k+1)-zf(i,k)) 
            cu(i,k) = cu(i,k)/(rl/cp)

            if (k == jt(i)) cu(i,k) = 0.
            cu(i,k) = max(0._r8,cu(i,k))
         end if
      end do
   end do
!
! compute condensed liquid, rain production rate
! accumulate total precipitation (condensation - detrainment of liquid)
! Note ql1 = ql(k) + rprd(k)*dz(k)/mu(k)
!
   do k = pver,msg + 2,-1
      do i = 1,il2g
         rprd(i,k) = 0.
!---
         if (k >= jt(i) .and. k <= jlcl(i) .and. mu(i,k) >= 0.0) then
            if (mu(i,k) > 1.e-4) then
               ql1 = 1./mu(i,k)* (mu(i,k+1)*ql(i,k+1)- &
                     dz(i,k)*du(i,k)*ql(i,k+1)+dz(i,k)*cu(i,k))
               ql(i,k) = ql1/ (1.+dz(i,k)*c0)
            else
               ql(i,k) = 0.
            end if
            totpcp(i) = totpcp(i) + dz(i,k)*(cu(i,k)-du(i,k)*ql(i,k+1))
            rprd(i,k) = c0*mu(i,k)*ql(i,k)
            duql(i,k) = du(i,k)*ql(i,k+1)  
         end if
!
      end do
   end do
!-------------
!
! find the level of minimum hs, where detrainment starts
!
   do i = 1,il2g
         j0(i) = jt(i)
         thetasemin(i) = 1.e+08
      do k = pver, msg + 1, -1
         theta  = t(i,k)*(1000.0/p(i,k))**cappa
         thetase(i,k) = theta*exp(latvap/cpair*q(i,k)/t(i,k))
         if (thetase(i,k) <= thetasemin(i) .and. k >= jt(i) .and. k <= jlcl(i)) then
             thetasemin(i) = thetase(i,k)
             j0(i) = k
         end if
      enddo
   end do

!=========================================================================
!
! specify downdraft properties (no downdrafts if jd.ge.jb).
! scale down downward mass flux profile so that net flux
! (up-down) at cloud base in not negative.
!
   do i=1,il2g
!
!    eps0(i) = 2.5e-4   ! too large downdraft 
!
     eps0(i) = 2.0e-5
   end do

   do i = 1,il2g
!
! in normal downdraft strength run alfa=0.2.  In test4 alfa=0.1
!
      alfa(i) = 0.1
      jt(i) = min(jt(i),jb(i)-1)
      jd(i) = max(j0(i),jt(i)+1)
      jd(i) = min(jd(i),jb(i))

      k = jd(i)
      if (jd(i) < jb(i) .and. eps0(i) > 0.) then
          epsm(i) = eps0(i)
          md(i,jd(i)) = -2.0*(1.0- q(i,k)/qst(i,k))*mu(i,k)
  		  md(i,jd(i)) = -min(abs(md(i,k)), mu(i,k)*0.3) 

      end if
   end do
   do k = msg + 1,pver
      do i = 1,il2g
         if ((k > jd(i) .and. k <= jb(i)) .and. eps0(i) > 0.) then
            zdef(i) = zf(i,jd(i)) - zf(i,k)
            md(i,k) = md(i,jd(i))/(2.*eps0(i))*(exp(2.*epsm(i)*zdef(i))-1.)/zdef(i)
        end if
      end do
   end do
   do k = msg + 1,pver
      do i = 1,il2g
         if ((k >= jt(i) .and. k <= jb(i)) .and. eps0(i) > 0. .and. jd(i) < jb(i)) then
            ratmjb(i) = min(abs(mu(i,jb(i))/md(i,jb(i))),1._r8)
            md(i,k) = md(i,k)*ratmjb(i)
         end if
      end do
   end do

   small = 1.e-20
   do k = msg + 1,pver
      do i = 1,il2g
         if ((k >= jt(i) .and. k <= pver) .and. eps0(i) > 0.) then
            ed(i,k-1) = (md(i,k-1)-md(i,k))/dz(i,k-1)
            mdt = min(md(i,k),-small)
            hd(i,k) = (md(i,k-1)*hd(i,k-1) - dz(i,k-1)*ed(i,k-1)*hmn(i,k-1))/mdt

            ud(i,k) = (md(i,k-1)*ud(i,k-1) - dz(i,k-1)*ed(i,k-1)*u(i,k-1))/mdt 
            vd(i,k) = (md(i,k-1)*vd(i,k-1) - dz(i,k-1)*ed(i,k-1)*v(i,k-1))/mdt 
         end if
      end do
   end do
!
! calculate updraft and downdraft properties.
!
   do k = msg + 2,pver
      do i = 1,il2g
         if ((k >= jd(i) .and. k <= jb(i)) .and. eps0(i) > 0. .and. jd(i) < jb(i)) then
            qds(i,k) = qsthat(i,k) + gamhat(i,k)*(hd(i,k)-hsthat(i,k))/ &
               (rl*(1. + gamhat(i,k)))
         end if
      end do
   end do
!
   do i = 1,il2g
      qd(i,jd(i)) = qds(i,jd(i))
      sd(i,jd(i)) = (hd(i,jd(i)) - rl*qd(i,jd(i)))/cp
   end do
!
   do k = msg + 2,pver
      do i = 1,il2g
         if (k >= jd(i) .and. k < jb(i) .and. eps0(i) > 0.) then
            qd(i,k+1) = qds(i,k+1)
            evp(i,k) = -ed(i,k)*q(i,k) + (md(i,k)*qd(i,k)-md(i,k+1)*qd(i,k+1))/dz(i,k)
            evp(i,k) = max(evp(i,k),0._r8)
            mdt = min(md(i,k+1),-small)
            sd(i,k+1) = ((rl/cp*evp(i,k)-ed(i,k)*s(i,k))*dz(i,k) + md(i,k)*sd(i,k))/mdt
            totevp(i) = totevp(i) - dz(i,k)*ed(i,k)*q(i,k)
         end if
      end do
   end do

!---
   do i = 1,il2g
      totevp(i) = totevp(i) + md(i,jd(i))*qd(i,jd(i)) - md(i,jb(i))*qd(i,jb(i))
   end do
   do i = 1,il2g
      totpcp(i) = max(totpcp(i),0._r8)
      totevp(i) = max(totevp(i),0._r8)
   end do
!
   do k = msg + 2,pver
      do i = 1,il2g
         if (totevp(i) > 0. .and. totpcp(i) > 0.) then
            md(i,k)  = md (i,k)*min(1._r8, totpcp(i)/(totevp(i)+totpcp(i)))
            ed(i,k)  = ed (i,k)*min(1._r8, totpcp(i)/(totevp(i)+totpcp(i)))
            evp(i,k) = evp(i,k)*min(1._r8, totpcp(i)/(totevp(i)+totpcp(i)))
         else
            md(i,k)  = 0.
            ed(i,k)  = 0.
            evp(i,k) = 0.
         end if
! cmeg is the cloud water condensed - rain water evaporated
! rprd is the cloud water converted to rain - (rain evaporated)
         cmeg(i,k) = cu(i,k) - evp(i,k)
         rprd(i,k) = rprd(i,k)-evp(i,k)
      end do
   end do

! compute the net precipitation flux across interfaces

   pflx(:il2g,1) = 0.
   do k = 2,pverp
      do i = 1,il2g
         pflx(i,k) = pflx(i,k-1) + rprd(i,k-1)*dz(i,k-1)
      end do
   end do
!
   do k = msg + 1,pver
      do i = 1,il2g
         mc(i,k) = mu(i,k) + md(i,k)
      end do
   end do
!
   return
end subroutine cldprp_wu


subroutine buoyan_new(lchnk   ,ncol    , &
                  q       ,t       ,p       ,z       ,pf      , &
                  tp      ,qstp    ,tl      ,rl      ,cape    , &
                  pblt    ,lcl     ,lel     ,lon     ,mx      , &
                  rd      ,grav    ,cp      ,msg     , &
                  tpert   ,negcape ,omega   ,deepconv)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author:
! This is contributed code not fully standardized by the CCM core group.
! The documentation has been enhanced to the degree that we are able.
! Reviewed:          P. Rasch, April 1996
! 
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! input arguments
!
   integer, intent(in) :: lchnk                 ! chunk identifier
   integer, intent(in) :: ncol                  ! number of atmospheric columns

   real(r8), intent(in) :: q(pcols,pver)        ! spec. humidity
   real(r8), intent(in) :: t(pcols,pver)        ! temperature
   real(r8), intent(in) :: p(pcols,pver)        ! pressure
   real(r8), intent(in) :: z(pcols,pver)        ! height
   real(r8), intent(in) :: pf(pcols,pver+1)     ! pressure at interfaces
   real(r8), intent(in) :: pblt(pcols)          ! index of pbl depth
   real(r8), intent(in) :: tpert(pcols)         ! perturbation temperature by pbl processes

!--
   real(r8), intent(in) :: omega(pcols,pver)   
   logical , intent(out):: deepconv(pcols) 

!
! output arguments
!
   real(r8), intent(out) :: tp(pcols,pver)       ! parcel temperature
   real(r8), intent(out) :: qstp(pcols,pver)     ! saturation mixing ratio of parcel
   real(r8), intent(out) :: tl(pcols)            ! parcel temperature at lcl
   real(r8), intent(out) :: cape(pcols)          ! convective aval. pot. energy.
   real(r8), intent(out) :: negcape(pcols)       ! negative convective aval. pot. energy.
   integer lcl(pcols)        !
   integer lel(pcols)        !
   integer lon(pcols)        ! level of onset of deep convection
   integer mx(pcols)         ! level of max moist static energy
!------------------------------
!
   integer   lel0(pcols)           
   real(r8)  theta
   real(r8)  thetase(pcols,pver)
   real(r8)  thetase_kp1
!
!--------------------------Local Variables------------------------------
!
   real(r8) capeten(pcols,5)     ! provisional value of cape
   real(r8) tv(pcols,pver)       !
   real(r8) tpv(pcols,pver)      !
   real(r8) buoy(pcols,pver)
   real(r8) qs(pcols,pver)       ! saturation specific humidity

   real(r8) a1(pcols)
   real(r8) a2(pcols)
   real(r8) estp(pcols)
   real(r8) pl(pcols)
   real(r8) plexp(pcols)
   real(r8) hmax(pcols)
   real(r8) hmin(pcols)
   real(r8) hmn(pcols,pver)
   real(r8) y(pcols)

   logical plge600(pcols)
   integer knt(pcols)
   integer lelten(pcols,5)
   integer khmin(pcols)         ! level of min moist static energy

   real(r8) cp
   real(r8) e
   real(r8) grav

   integer i
   integer k
   integer msg
   integer n
   integer idx0 

   real(r8) rd
   real(r8) rl
   real(r8) esttmp 
#ifdef PERGRO
   real(r8) rhd
#endif
!
!-----------------------------------------------------------------------
!
   do n = 1,5
      do i = 1,ncol
         lelten(i,n) = pver
         capeten(i,n) = 0.
      end do
   end do
!
   do i = 1,ncol
      lon(i) = pver
      knt(i) = 0
      lel(i) = pver
      mx(i) = lon(i)
      khmin(i)=pver
      cape(i) = 0.
      negcape(i) = 0.
      hmax(i) = 0.
      hmin(i) = 1.e10 
   end do
!
      do i = 1, pcols
        do k=pver, 1, -1
           buoy(i,k) = 0.
        end do
      end do

   tp(:ncol,:) = t(:ncol,:)
   qstp(:ncol,:) = q(:ncol,:)
!
! set "launching" level(mx) to be at maximum moist static energy.
! search for this level stops at planetary boundary layer top.
!

! Find level of max static energy below h_min level 
#ifdef PERGRO
   do k = pver,msg + 1,-1
      do i = 1,ncol
         hmn(i,k) = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
!
! Reset max moist static energy level when relative difference exceeds 1.e-4
!
         rhd = (hmn(i,k) - hmax(i))/(hmn(i,k) + hmax(i))
         if (k >= nint(pblt(i)) .and. k <= lon(i) .and. rhd > -1.e-4) then
            hmax(i) = hmn(i,k)
            mx(i) = k
         end if
      end do
   end do
#else
   do k = pver,msg + 1,-1
      do i = 1,ncol
         hmn(i,k) = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
         if (k >= nint(pblt(i)) .and. k <= lon(i) .and. hmn(i,k) > hmax(i)) then
            hmax(i) = hmn(i,k)
            mx(i) = k
         end if
      end do
   end do
#endif
! find min hmn level above 700 mb
      do i = 1, ncol
        do k = msg+1, pver
          if( p(i,k) .le. 700. .and. hmin(i) .ge. hmn(i,k) ) then
               hmin(i) = hmn(i,k)
               khmin(i) = k
          endif
        enddo
      enddo
!
      do i = 1,ncol
        do k = pver,msg + 1,-1
         esttmp = c1*exp((c2* (t(i,k)-tfreez))/((t(i,k)-tfreez)+c3))
         if ( p(i,k)-esttmp > 0. ) then
            qs(i,k) = eps1*esttmp/ (p(i,k)-esttmp)
         else
            qs(i,k) = 1.0
         end if
! rh threshold does not apply to midlevel convection
!
#ifdef WU_REFATM
          if ( k.gt. khmin(i) .and. k .lt. nint(pblt(i)) .and.   &
               omega(i,k) .lt. 0.0 .and.  &
  			   q(i,k)/qs(i,k) .ge. 0.6 .and. &
               hmn(i,k).gt.hmax(i)) then
#else
          if ( k.gt. khmin(i) .and. k .lt. nint(pblt(i)) .and.   &
            q(i,k)/qs(i,k) .ge. 0.6 .and. &
            hmn(i,k).gt.hmax(i)) then
#endif
            hmax(i) = hmn(i,k)
            mx(i) = k
          end if
        end do
      end do
!------------------------------------------------------------------------
!#ifdef WU_REFATM
!
!      mx(i) = pver
!      do i = 1,ncol
!        do k = pver,msg + 1,-1
!
!            theta  = t(i,k)*(1000.0/p(i,k))**cappa
!            thetase(i,k) = theta*exp(rl/cp*q(i,k)/t(i,k))
!
!			if(k.lt.pver .and. thetase_kp1 .gt. thetase(i,k) .and. mx(i).eq.pver ) then
!			   mx(i) = k+1
!			endif      
!            thetase_kp1 = thetase(i,k)
!		enddo
!      enddo
!
!#endif
!---------------------------------------------------------------------------
!
!#ifdef WU_REFATM
!   do i = 1,ncol
!      lcl(i) = mx(i)
!      call condenst_lcl( p(i,mx(i)), t(i,mx(i)), q(i,mx(i)), tl(i),  pl(i))
!   enddo
!#else
   do i = 1,ncol
      lcl(i) = mx(i)
      e = p(i,mx(i))*q(i,mx(i))/ (eps1+q(i,mx(i)))
      tl(i) = 2840./ (3.5*log(t(i,mx(i)))-log(e)-4.805) + 55.
      if (tl(i) < t(i,mx(i))) then
         plexp(i) = (1./ (0.2854* (1.-0.28*q(i,mx(i)))))
         pl(i) = p(i,mx(i))* (tl(i)/t(i,mx(i)))**plexp(i)
      else
         tl(i) = t(i,mx(i))
         pl(i) = p(i,mx(i))
      end if
   end do
!#endif
!
! calculate lifting condensation level (lcl).
!
   do k = pver,msg + 2,-1
      do i = 1,ncol
         if (k <= mx(i) .and. (p(i,k) > pl(i) .and. p(i,k-1) <= pl(i))) then
            lcl(i) = k - 1
         end if
      end do
   end do
!
! if lcl is above the nominal level of non-divergence (600 mbs),
! no deep convection is permitted (ensuing calculations
! skipped and cape retains initialized value of zero).
!
   do i = 1,ncol
!wtw       plge600(i) = pl(i).ge.600.
      plge600(i) = pl(i).ge.(600.0/1000.0*pf(i,pver+1))
   end do
!
! initialize parcel properties in sub-cloud layer below lcl.
!
   do k = pver,msg + 1,-1
      do i=1,ncol
         if (k > lcl(i) .and. k <= mx(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/ (1.+q(i,k))
            qstp(i,k) = q(i,mx(i))
            tp(i,k) = t(i,mx(i))* (p(i,k)/p(i,mx(i)))**(0.2854* (1.-0.28*q(i,mx(i))))
!
! buoyancy is increased by 0.5 k as in tiedtke
!
!-jjh          tpv (i,k)=tp(i,k)*(1.+1.608*q(i,mx(i)))/
!-jjh     1                     (1.+q(i,mx(i)))
#ifdef WU_REFATM
            tpv(i,k) = (tp(i,k)+tpert(i))*(1.+1.608*q(i,mx(i)))/ (1.+q(i,mx(i)))
#else
            tpv(i,k) = (tp(i,k)+tpert(i))*(1.+1.608*q(i,mx(i)))/ (1.+q(i,mx(i)))
#endif
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5
         end if
      end do
   end do
!
! define parcel properties at lcl (i.e. level immediately above pl).
!
   do k = pver,msg + 1,-1
      do i=1,ncol
         if (k == lcl(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/ (1.+q(i,k))
            qstp(i,k) = q(i,mx(i))
            tp(i,k) = tl(i)* (p(i,k)/pl(i))**(0.2854* (1.-0.28*qstp(i,k)))

!              estp(i)  =exp(a-b/tp(i,k))
! use of different formulas for est has about 1 g/kg difference
! in qs at t= 300k, and 0.02 g/kg at t=263k, with the formula
! above giving larger qs.
!
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i)/ (p(i,k)-estp(i))
            a1(i) = cp / rl + qstp(i,k) * (1.+ qstp(i,k) / eps1) * rl * eps1 / &
                    (rd * tp(i,k) ** 2)
            a2(i) = .5* (qstp(i,k)* (1.+2./eps1*qstp(i,k))* &
                    (1.+qstp(i,k)/eps1)*eps1**2*rl*rl/ &
                    (rd**2*tp(i,k)**4)-qstp(i,k)* &
                    (1.+qstp(i,k)/eps1)*2.*eps1*rl/ &
                    (rd*tp(i,k)**3))
            a1(i) = 1./a1(i)
            a2(i) = -a2(i)*a1(i)**3
            y(i) = q(i,mx(i)) - qstp(i,k)
            tp(i,k) = tp(i,k) + a1(i)*y(i) + a2(i)*y(i)**2
!          estp(i)  =exp(a-b/tp(i,k))
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/ ((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i) / (p(i,k)-estp(i))
!
! buoyancy is increased by 0.5 k in cape calculation.
! dec. 9, 1994
!-jjh          tpv(i,k) =tp(i,k)*(1.+1.608*qstp(i,k))/(1.+q(i,mx(i)))
!
#ifdef WU_REFATM
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+1.608*qstp(i,k)) / (1.+q(i,mx(i)))
#else
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+0.608*qstp(i,k))
#endif
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5
         end if
      end do
   end do
!
! main buoyancy calculation.
!
   do k = pver - 1,msg + 1,-1
      do i=1,ncol
         if (k < lcl(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/ (1.+q(i,k))
            qstp(i,k) = qstp(i,k+1)
            tp(i,k) = tp(i,k+1)* (p(i,k)/p(i,k+1))**(0.2854* (1.-0.28*qstp(i,k)))
!          estp(i) = exp(a-b/tp(i,k))
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i)/ (p(i,k)-estp(i))
            a1(i) = cp/rl + qstp(i,k)* (1.+qstp(i,k)/eps1)*rl*eps1/ (rd*tp(i,k)**2)
            a2(i) = .5* (qstp(i,k)* (1.+2./eps1*qstp(i,k))* &
                    (1.+qstp(i,k)/eps1)*eps1**2*rl*rl/ &
                    (rd**2*tp(i,k)**4)-qstp(i,k)* &
                    (1.+qstp(i,k)/eps1)*2.*eps1*rl/ &
                    (rd*tp(i,k)**3))
            a1(i) = 1./a1(i)
            a2(i) = -a2(i)*a1(i)**3
            y(i) = qstp(i,k+1) - qstp(i,k)
            tp(i,k) = tp(i,k) + a1(i)*y(i) + a2(i)*y(i)**2
!          estp(i)  =exp(a-b/tp(i,k))
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/ ((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i)/ (p(i,k)-estp(i))
!-jjh          tpv(i,k) =tp(i,k)*(1.+1.608*qstp(i,k))/
!jt            (1.+q(i,mx(i)))
#ifdef WU_REFATM
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+1.608*qstp(i,k))/(1.+q(i,mx(i)))
#else
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+0.608*qstp(i,k))
#endif
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5
         end if
      end do
   end do
!
#ifdef WU_REFATM
   do k = pver, msg + 2, -1
#else
   do k = msg + 2,pver
#endif
      do i = 1,ncol
         if (k < lcl(i) .and. plge600(i)) then
            if (buoy(i,k+1) > 0. .and. buoy(i,k) <= 0.) then
               knt(i) = min(5,knt(i) + 1)
               lelten(i,knt(i)) = k
            end if
         end if
      end do
   end do
!
! Negative CAPE*******************************************************
!
      do i = 1, pcols 
        negcape(i)=0.
        idx0= pver 
        do k=pver-1, 10, -1
           if (plge600(i) &
        .and. (buoy(i,k+1).le.0. .and. buoy(i,k).gt.0. &
        .and. buoy(i,k-1).gt.0. .and. buoy(i,k-2).gt.0. ) )  then
                idx0= k + 1
                goto 5
           endif
        end do
5       continue
        do k=pver-1, idx0, -1
          if (plge600(i)) then
              negcape(i) = negcape(i) + &
                             rd*buoy(i,k)*log(pf(i,k+1)/pf(i,k))
           endif
        end do
       end do
!
! calculate convective available potential energy (cape).
!
   do n = 1,5
      do k = msg + 1,pver
         do i = 1,ncol
            if (plge600(i) .and. k <= mx(i) .and. k > lelten(i,n)) then
               capeten(i,n) = capeten(i,n) + rd*buoy(i,k)*log(pf(i,k+1)/pf(i,k))
            end if
         end do
      end do
   end do
!
! find maximum cape from all possible tentative capes from
! one sounding,
! and use it as the final cape, april 26, 1995
!
!#ifdef WU_REFATM
!	  lel(i) = pver
!      do i = 1,ncol
!        do k = pver,msg + 1,-1
!			if( k<mx(i) .and. thetase(i,k)  >= thetase(i,mx(i)) .and. &
!			                     thetase(i,k+1) < thetase(i,mx(i))  ) then
!               lel(i) = k
!			endif      
!		enddo
!      enddo
!
!      do i = 1,ncol
!         cape(i) = 0.0
!         do k = msg + 1,pver
!            if (plge600(i) .and. k <= mx(i) .and. k > lel(i)) then
!               cape(i) = cape(i) + rd*buoy(i,k)*log(pf(i,k+1)/pf(i,k))
!            end if
!         end do
!      end do
!#else
   do n = 1,5
      do i = 1,ncol
         if (capeten(i,n) > cape(i)) then
            cape(i) = capeten(i,n)
            lel(i) = lelten(i,n)
         end if
      end do
   end do
!#endif
!-----------------------------------------------------
#ifdef WU_REFATM
   do i = 1,ncol
       deepconv(i) = .true.
!      deepconv(i) = .false.
!      if (plge600(i) .and. (omega(i,mx(i)).lt.0.0 .or. buoy(i,mx(i)).gt.0.0) ) then  ! snow2
!	      deepconv(i) = .true.
!      endif
   enddo  
#endif
!--------------------------------------------------------------
!
! put lower bound on cape for diagnostic purposes.
!
   do i = 1,ncol
      cape(i) = max(cape(i), 0._r8)
   end do
!
   return
end subroutine buoyan_new

SUBROUTINE condenst_lcl(Pmx, Tmx, Qmx, TL,  PL)
   implicit none
!-----------------------------------------------------------------------
!
! input arguments
!
   real(r8), intent(in) :: Qmx        ! spec. humidity
   real(r8), intent(in) :: Tmx        ! temperature
   real(r8), intent(in) :: Pmx        ! pressure (hPa)

   real(r8), intent(out) :: PL        ! pressure (hPa)
   real(r8), intent(out) :: TL        ! 
!------------------
!
   real(r8),parameter :: B1  = 4.036304e-2   
   real(r8),parameter :: alf = 0.340198873
   real(r8),parameter :: bk1 = 0.141277704
   real(r8),parameter :: ZE0 = 2.188997847
   real(r8),parameter :: T0 = 273.15

   real(r8) y,	QLB, TLB, PLB, PCB, TCB, TC0, ZQX, X1, X2
   integer  i, k
!------------------------------------------------------
!     QLB = qmx
      QLB = qmx/(1+qmx)         ! mix --> specific humidity
      TLB = Tmx
	  PLB = Pmx   ! hPa--> hPa
!
      Y=B1*LOG((0.378+0.622/QLB)/PLB)+BK1*LOG(TLB)                 
      TCB=TLB                                                        
      do k=1, 1000
	     TC0=TCB                                               
         ZQX=-ALF*LOG(TC0)+Y+ZE0                              
         X1=T0/ZQX                                             
         X2=ALF*X1/ZQX                                         
         TCB=TC0*(1.0-(TC0-X1)/(TC0-X2))                       
         IF(ABS(TCB-TC0).LT.0.01) then
 	        PCB=PLB*(TCB/TLB)**3.5 
            PL = PCB   ! hPa --> hPa
			TL = TCB 
		    return
		 endif
	   enddo
   return
end subroutine condenst_lcl


subroutine parcel_property(p1, qstp1, tp1, p2, qstp2, tp2)
   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid,          only: pcols, pver, pverp
   use moistconvection, only: rgas
   implicit none
!--------------------------------------------
! input variables
!
   real(r8) p1                ! hPa
   real(r8) tp1, qstp1

! output variables
!
   real(r8) p2                ! hPa
   real(r8) tp2, qstp2

!---
! local variables
!
   real(r8) a1, a2, y, estp
!------------------------------------------------------------------------
   qstp2 = qstp1
   tp2   = tp1* (p2/p1)**(0.2854* (1.-0.28*qstp2))
   estp  = c1*exp((c2* (tp2-tfreez))/((tp2-tfreez)+c3))
   qstp2 = eps1*estp/ (p2-estp)
   a1 = cp/rl + qstp2* (1.+qstp2/eps1)*rl*eps1/ (rgas*tp2**2)
   a2 = .5* (qstp2* (1.+2./eps1*qstp2)* &
                    (1.+qstp2/eps1)*eps1**2*rl*rl/ &
                    (rgas**2*tp2**4)-qstp2* &
                    (1.+qstp2/eps1)*2.*eps1*rl/ &
                    (rgas*tp2**3))
   a1 = 1./a1
   a2 = -a2*a1**3
   y  = qstp1 - qstp2
   tp2 = tp2 + a1*y + a2*y**2
   estp = c1*exp((c2* (tp2-tfreez))/ ((tp2-tfreez)+c3))
   qstp2 = eps1*estp/ (p2-estp)
!
   return
end subroutine parcel_property

subroutine buoyan(lchnk   ,ncol    , &
                  q       ,t       ,p       ,z       ,pf      , &
                  tp      ,qstp    ,tl      ,rl      ,cape    , &
                  pblt    ,lcl     ,lel     ,lon     ,mx      , &
                  rd      ,grav    ,cp      ,msg     , &
                  tpert   )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author:
! This is contributed code not fully standardized by the CCM core group.
! The documentation has been enhanced to the degree that we are able.
! Reviewed:          P. Rasch, April 1996
! 
!-----------------------------------------------------------------------
   implicit none
!-----------------------------------------------------------------------
!
! input arguments
!
   integer, intent(in) :: lchnk                 ! chunk identifier
   integer, intent(in) :: ncol                  ! number of atmospheric columns

   real(r8), intent(in) :: q(pcols,pver)        ! spec. humidity
   real(r8), intent(in) :: t(pcols,pver)        ! temperature
   real(r8), intent(in) :: p(pcols,pver)        ! pressure
   real(r8), intent(in) :: z(pcols,pver)        ! height
   real(r8), intent(in) :: pf(pcols,pver+1)     ! pressure at interfaces
   real(r8), intent(in) :: pblt(pcols)          ! index of pbl depth
   real(r8), intent(in) :: tpert(pcols)         ! perturbation temperature by pbl processes

!
! output arguments
!
   real(r8), intent(out) :: tp(pcols,pver)       ! parcel temperature
   real(r8), intent(out) :: qstp(pcols,pver)     ! saturation mixing ratio of parcel
   real(r8), intent(out) :: tl(pcols)            ! parcel temperature at lcl
   real(r8), intent(out) :: cape(pcols)          ! convective aval. pot. energy.
   integer lcl(pcols)        !
   integer lel(pcols)        !
   integer lon(pcols)        ! level of onset of deep convection
   integer mx(pcols)         ! level of max moist static energy
!
!--------------------------Local Variables------------------------------
!
   real(r8) capeten(pcols,5)     ! provisional value of cape
   real(r8) tv(pcols,pver)       !
   real(r8) tpv(pcols,pver)      !
   real(r8) buoy(pcols,pver)

   real(r8) a1(pcols)
   real(r8) a2(pcols)
   real(r8) estp(pcols)
   real(r8) pl(pcols)
   real(r8) plexp(pcols)
   real(r8) hmax(pcols)
   real(r8) hmn(pcols)
   real(r8) y(pcols)

   logical plge600(pcols)
   integer knt(pcols)
   integer lelten(pcols,5)

   real(r8) cp
   real(r8) e
   real(r8) grav

   integer i
   integer k
   integer msg
   integer n

   real(r8) rd
   real(r8) rl
#ifdef PERGRO
   real(r8) rhd
#endif
!
!-----------------------------------------------------------------------
!
   do n = 1,5
      do i = 1,ncol
         lelten(i,n) = pver
         capeten(i,n) = 0.
      end do
   end do
!
   do i = 1,ncol
      lon(i) = pver
      knt(i) = 0
      lel(i) = pver
      mx(i) = lon(i)
      cape(i) = 0.
      hmax(i) = 0.
   end do

   tp(:ncol,:) = t(:ncol,:)
   qstp(:ncol,:) = q(:ncol,:)
!
! set "launching" level(mx) to be at maximum moist static energy.
! search for this level stops at planetary boundary layer top.
!
#ifdef PERGRO
   do k = pver,msg + 1,-1
      do i = 1,ncol
         hmn(i) = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
!
! Reset max moist static energy level when relative difference exceeds 1.e-4
!
         rhd = (hmn(i) - hmax(i))/(hmn(i) + hmax(i))
         if (k >= nint(pblt(i)) .and. k <= lon(i) .and. rhd > -1.e-4) then
            hmax(i) = hmn(i)
            mx(i) = k
         end if
      end do
   end do
#else
   do k = pver,msg + 1,-1
      do i = 1,ncol
         hmn(i) = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
         if (k >= nint(pblt(i)) .and. k <= lon(i) .and. hmn(i) > hmax(i)) then
            hmax(i) = hmn(i)
            mx(i) = k
         end if
      end do
   end do
#endif
!
   do i = 1,ncol
      lcl(i) = mx(i)
      e = p(i,mx(i))*q(i,mx(i))/ (eps1+q(i,mx(i)))
      tl(i) = 2840./ (3.5*log(t(i,mx(i)))-log(e)-4.805) + 55.
      if (tl(i) < t(i,mx(i))) then
         plexp(i) = (1./ (0.2854* (1.-0.28*q(i,mx(i)))))
         pl(i) = p(i,mx(i))* (tl(i)/t(i,mx(i)))**plexp(i)
      else
         tl(i) = t(i,mx(i))
         pl(i) = p(i,mx(i))
      end if
   end do
!
! calculate lifting condensation level (lcl).
!
   do k = pver,msg + 2,-1
      do i = 1,ncol
         if (k <= mx(i) .and. (p(i,k) > pl(i) .and. p(i,k-1) <= pl(i))) then
            lcl(i) = k - 1
         end if
      end do
   end do
!
! if lcl is above the nominal level of non-divergence (600 mbs),
! no deep convection is permitted (ensuing calculations
! skipped and cape retains initialized value of zero).
!
   do i = 1,ncol
      plge600(i) = pl(i).ge.600.
   end do
!
! initialize parcel properties in sub-cloud layer below lcl.
!
   do k = pver,msg + 1,-1
      do i=1,ncol
         if (k > lcl(i) .and. k <= mx(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/ (1.+q(i,k))
            qstp(i,k) = q(i,mx(i))
            tp(i,k) = t(i,mx(i))* (p(i,k)/p(i,mx(i)))**(0.2854* (1.-0.28*q(i,mx(i))))
!
! buoyancy is increased by 0.5 k as in tiedtke
!
!-jjh          tpv (i,k)=tp(i,k)*(1.+1.608*q(i,mx(i)))/
!-jjh     1                     (1.+q(i,mx(i)))
            tpv(i,k) = (tp(i,k)+tpert(i))*(1.+1.608*q(i,mx(i)))/ (1.+q(i,mx(i)))
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5
         end if
      end do
   end do
!
! define parcel properties at lcl (i.e. level immediately above pl).
!
   do k = pver,msg + 1,-1
      do i=1,ncol
         if (k == lcl(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/ (1.+q(i,k))
            qstp(i,k) = q(i,mx(i))
            tp(i,k) = tl(i)* (p(i,k)/pl(i))**(0.2854* (1.-0.28*qstp(i,k)))
!              estp(i)  =exp(a-b/tp(i,k))
! use of different formulas for est has about 1 g/kg difference
! in qs at t= 300k, and 0.02 g/kg at t=263k, with the formula
! above giving larger qs.
!
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i)/ (p(i,k)-estp(i))
            a1(i) = cp / rl + qstp(i,k) * (1.+ qstp(i,k) / eps1) * rl * eps1 / &
                    (rd * tp(i,k) ** 2)
            a2(i) = .5* (qstp(i,k)* (1.+2./eps1*qstp(i,k))* &
                    (1.+qstp(i,k)/eps1)*eps1**2*rl*rl/ &
                    (rd**2*tp(i,k)**4)-qstp(i,k)* &
                    (1.+qstp(i,k)/eps1)*2.*eps1*rl/ &
                    (rd*tp(i,k)**3))
            a1(i) = 1./a1(i)
            a2(i) = -a2(i)*a1(i)**3
            y(i) = q(i,mx(i)) - qstp(i,k)
            tp(i,k) = tp(i,k) + a1(i)*y(i) + a2(i)*y(i)**2
!          estp(i)  =exp(a-b/tp(i,k))
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/ ((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i) / (p(i,k)-estp(i))
!
! buoyancy is increased by 0.5 k in cape calculation.
! dec. 9, 1994
!-jjh          tpv(i,k) =tp(i,k)*(1.+1.608*qstp(i,k))/(1.+q(i,mx(i)))
!
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+1.608*qstp(i,k)) / (1.+q(i,mx(i)))
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5
         end if
      end do
   end do
!
! main buoyancy calculation.
!
   do k = pver - 1,msg + 1,-1
      do i=1,ncol
         if (k < lcl(i) .and. plge600(i)) then
            tv(i,k) = t(i,k)* (1.+1.608*q(i,k))/ (1.+q(i,k))
            qstp(i,k) = qstp(i,k+1)
            tp(i,k) = tp(i,k+1)* (p(i,k)/p(i,k+1))**(0.2854* (1.-0.28*qstp(i,k)))
!          estp(i) = exp(a-b/tp(i,k))
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i)/ (p(i,k)-estp(i))
            a1(i) = cp/rl + qstp(i,k)* (1.+qstp(i,k)/eps1)*rl*eps1/ (rd*tp(i,k)**2)
            a2(i) = .5* (qstp(i,k)* (1.+2./eps1*qstp(i,k))* &
                    (1.+qstp(i,k)/eps1)*eps1**2*rl*rl/ &
                    (rd**2*tp(i,k)**4)-qstp(i,k)* &
                    (1.+qstp(i,k)/eps1)*2.*eps1*rl/ &
                    (rd*tp(i,k)**3))
            a1(i) = 1./a1(i)
            a2(i) = -a2(i)*a1(i)**3
            y(i) = qstp(i,k+1) - qstp(i,k)
            tp(i,k) = tp(i,k) + a1(i)*y(i) + a2(i)*y(i)**2
!          estp(i)  =exp(a-b/tp(i,k))
            estp(i) = c1*exp((c2* (tp(i,k)-tfreez))/ ((tp(i,k)-tfreez)+c3))

            qstp(i,k) = eps1*estp(i)/ (p(i,k)-estp(i))
!-jjh          tpv(i,k) =tp(i,k)*(1.+1.608*qstp(i,k))/
!jt            (1.+q(i,mx(i)))
            tpv(i,k) = (tp(i,k)+tpert(i))* (1.+1.608*qstp(i,k))/(1.+q(i,mx(i)))
            buoy(i,k) = tpv(i,k) - tv(i,k) + 0.5
         end if
      end do
   end do
!
   do k = msg + 2,pver
      do i = 1,ncol
         if (k < lcl(i) .and. plge600(i)) then
            if (buoy(i,k+1) > 0. .and. buoy(i,k) <= 0.) then
               knt(i) = min(5,knt(i) + 1)
               lelten(i,knt(i)) = k
            end if
         end if
      end do
   end do
!
! calculate convective available potential energy (cape).
!
   do n = 1,5
      do k = msg + 1,pver
         do i = 1,ncol
            if (plge600(i) .and. k <= mx(i) .and. k > lelten(i,n)) then
               capeten(i,n) = capeten(i,n) + rd*buoy(i,k)*log(pf(i,k+1)/pf(i,k))
            end if
         end do
      end do
   end do
!
! find maximum cape from all possible tentative capes from
! one sounding,
! and use it as the final cape, april 26, 1995
!
   do n = 1,5
      do i = 1,ncol
         if (capeten(i,n) > cape(i)) then
            cape(i) = capeten(i,n)
            lel(i) = lelten(i,n)
         end if
      end do
   end do
!
! put lower bound on cape for diagnostic purposes.
!
   do i = 1,ncol
      cape(i) = max(cape(i), 0._r8)
   end do
!
   return
end subroutine buoyan


subroutine cldprp(lchnk   , &
                  q       ,t       ,u       ,v       ,p       , &
                  z       ,s       ,mu      ,eu      ,du      , &
                  md      ,ed      ,sd      ,qd      ,mc      , &
                  qu      ,su      ,zf      ,qst     ,hmn     , &
                  hsat    ,shat    ,ql      , &
                  cmeg    ,jb      ,lel     ,jt      ,jlcl    , &
                  mx      ,j0      ,jd      ,rl      ,il2g    , &
                  rd      ,grav    ,cp      ,msg     , &
                  pflx    ,evp     ,cu      ,rprd    ,limcnv  ,&
	          eps0    ,  pint)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! may 09/91 - guang jun zhang, m.lazare, n.mcfarlane.
!             original version cldprop.
! 
! Author: See above, modified by P. Rasch
! This is contributed code not fully standardized by the CCM core group.
!
! this code is very much rougher than virtually anything else in the CCM
! there are debug statements left strewn about and code segments disabled
! these are to facilitate future development. We expect to release a
! cleaner code in a future release
!
! the documentation has been enhanced to the degree that we are able
!
!-----------------------------------------------------------------------

   implicit none

!------------------------------------------------------------------------------
!
! Input arguments
!
   integer, intent(in) :: lchnk                  ! chunk identifier

   real(r8), intent(in) :: q(pcols,pver)         ! spec. humidity of env
   real(r8), intent(in) :: t(pcols,pver)         ! temp of env
   real(r8), intent(in) :: p(pcols,pver)         ! pressure of env
   real(r8), intent(in) :: pint(pcols,pverp)         ! pressure of env
   real(r8)             :: coef
   real(r8), intent(in) :: z(pcols,pver)         ! height of env
   real(r8), intent(in) :: s(pcols,pver)         ! normalized dry static energy of env
   real(r8), intent(in) :: zf(pcols,pverp)       ! height of interfaces
   real(r8), intent(in) :: u(pcols,pver)         ! zonal velocity of env
   real(r8), intent(in) :: v(pcols,pver)         ! merid. velocity of env

   integer, intent(in) :: jb(pcols)              ! updraft base level
   integer, intent(in) :: lel(pcols)             ! updraft launch level
   integer, intent(out) :: jt(pcols)              ! updraft plume top
   integer, intent(out) :: jlcl(pcols)            ! updraft lifting cond level
   integer, intent(in) :: mx(pcols)              ! updraft base level (same is jb)
   integer, intent(out) :: j0(pcols)              ! level where updraft begins detraining
   integer, intent(out) :: jd(pcols)              ! level of downdraft
   integer, intent(in) :: limcnv                 ! convection limiting level
   integer, intent(in) :: il2g                   !CORE GROUP REMOVE
   integer, intent(in) :: msg                    ! missing moisture vals (always 0)
   real(r8), intent(in) :: rl                    ! latent heat of vap
   real(r8), intent(in) :: shat(pcols,pver)      ! interface values of dry stat energy
!
! output
!
   real(r8), intent(out) :: rprd(pcols,pver)     ! rate of production of precip at that layer
   real(r8), intent(out) :: du(pcols,pver)       ! detrainement rate of updraft
   real(r8), intent(out) :: ed(pcols,pver)       ! entrainment rate of downdraft
   real(r8), intent(out) :: eu(pcols,pver)       ! entrainment rate of updraft
   real(r8), intent(out) :: hmn(pcols,pver)      ! moist stat energy of env
   real(r8), intent(out) :: hsat(pcols,pver)     ! sat moist stat energy of env
   real(r8), intent(out) :: mc(pcols,pver)       ! net mass flux
   real(r8), intent(out) :: md(pcols,pver)       ! downdraft mass flux
   real(r8), intent(out) :: mu(pcols,pver)       ! updraft mass flux
   real(r8), intent(out) :: pflx(pcols,pverp)    ! precipitation flux thru layer
   real(r8), intent(out) :: qd(pcols,pver)       ! spec humidity of downdraft
   real(r8), intent(out) :: ql(pcols,pver)       ! liq water of updraft
   real(r8), intent(out) :: qst(pcols,pver)      ! saturation spec humidity of env.
   real(r8), intent(out) :: qu(pcols,pver)       ! spec hum of updraft
   real(r8), intent(out) :: sd(pcols,pver)       ! normalized dry stat energy of downdraft
   real(r8), intent(out) :: su(pcols,pver)       ! normalized dry stat energy of updraft

   real(r8) rd                   ! gas constant for dry air
   real(r8) grav                 ! gravity
   real(r8) cp                   ! heat capacity of dry air

!
! Local workspace
!
   real(r8) gamma(pcols,pver)
   real(r8) dz(pcols,pver)
   real(r8) iprm(pcols,pver)
   real(r8) hu(pcols,pver)
   real(r8) hd(pcols,pver)
   real(r8) eps(pcols,pver)
   real(r8) f(pcols,pver)
   real(r8) k1(pcols,pver)
   real(r8) i2(pcols,pver)
   real(r8) ihat(pcols,pver)
   real(r8) i3(pcols,pver)
   real(r8) idag(pcols,pver)
   real(r8) i4(pcols,pver)
   real(r8) qsthat(pcols,pver)
   real(r8) hsthat(pcols,pver)
   real(r8) gamhat(pcols,pver)
   real(r8) cu(pcols,pver)
   real(r8) evp(pcols,pver)
   real(r8) cmeg(pcols,pver)
   real(r8) qds(pcols,pver)
   real(r8) hmin(pcols)
   real(r8) expdif(pcols)
   real(r8) expnum(pcols)
   real(r8) ftemp(pcols)
   real(r8) eps0(pcols)
   real(r8) rmue(pcols)
   real(r8) zuef(pcols)
   real(r8) zdef(pcols)
   real(r8) epsm(pcols)
   real(r8) ratmjb(pcols)
   real(r8) est(pcols)
   real(r8) totpcp(pcols)
   real(r8) totevp(pcols)
   real(r8) alfa(pcols)
   real(r8) ql1
   real(r8) tu
   real(r8) estu
   real(r8) qstu

   real(r8) small
   real(r8) mdt

   integer khighest
   integer klowest
   integer kount
   integer i,k

   logical doit(pcols)
   logical done(pcols)
!
!------------------------------------------------------------------------------
!
   do i = 1,il2g
      ftemp(i) = 0.
      expnum(i) = 0.
      expdif(i) = 0.
   end do
!
!jr Change from msg+1 to 1 to prevent blowup
!
   do k = 1,pver
      do i = 1,il2g
         dz(i,k) = zf(i,k) - zf(i,k+1)
      end do
   end do

!
! initialize many output and work variables to zero
!
   pflx(:il2g,1) = 0

   do k = 1,pver
      do i = 1,il2g
         k1(i,k) = 0.
         i2(i,k) = 0.
         i3(i,k) = 0.
         i4(i,k) = 0.
         mu(i,k) = 0.
         f(i,k) = 0.
         eps(i,k) = 0.
         eu(i,k) = 0.
         du(i,k) = 0.
         ql(i,k) = 0.
         cu(i,k) = 0.
         evp(i,k) = 0.
         cmeg(i,k) = 0.
         qds(i,k) = q(i,k)
         md(i,k) = 0.
         ed(i,k) = 0.
         sd(i,k) = s(i,k)
         qd(i,k) = q(i,k)
         mc(i,k) = 0.
         qu(i,k) = q(i,k)
         su(i,k) = s(i,k)
!        est(i)=exp(a-b/t(i,k))
         est(i) = c1*exp((c2* (t(i,k)-tfreez))/((t(i,k)-tfreez)+c3))
!++bee
         if ( p(i,k)-est(i) > 0. ) then
            qst(i,k) = eps1*est(i)/ (p(i,k)-est(i))
         else
            qst(i,k) = 1.0
         end if
!--bee
         gamma(i,k) = qst(i,k)*(1. + qst(i,k)/eps1)*eps1*rl/(rd*t(i,k)**2)*rl/cp
         hmn(i,k) = cp*t(i,k) + grav*z(i,k) + rl*q(i,k)
         hsat(i,k) = cp*t(i,k) + grav*z(i,k) + rl*qst(i,k)
         hu(i,k) = hmn(i,k)
         hd(i,k) = hmn(i,k)
         rprd(i,k) = 0.
      end do
   end do
!
!jr Set to zero things which make this routine blow up
!
   do k=1,msg
      do i=1,il2g
         rprd(i,k) = 0.
      end do
   end do
!
! interpolate the layer values of qst, hsat and gamma to
! layer interfaces
!
   do i = 1,il2g
      hsthat(i,msg+1) = hsat(i,msg+1)
      qsthat(i,msg+1) = qst(i,msg+1)
      gamhat(i,msg+1) = gamma(i,msg+1)
      totpcp(i) = 0.
      totevp(i) = 0.
   end do
   do k = msg + 2,pver
      do i = 1,il2g
         if (abs(qst(i,k-1)-qst(i,k)) > 1.E-6) then
            qsthat(i,k) = log(qst(i,k-1)/qst(i,k))*qst(i,k-1)*qst(i,k)/ (qst(i,k-1)-qst(i,k))
         else
            qsthat(i,k) = qst(i,k)
         end if

#ifdef WU_REFATM
!
! wtw 2005.12
!
         coef = log(pint(i,k)/p(i,k))/log(p(i,k-1)/p(i,k))
         qsthat(i,k) = qst(i,k) + (qst(i,k-1)-qst(i,k))*coef
#endif
         hsthat(i,k) = cp*shat(i,k) + rl*qsthat(i,k)
!==============================================================
         if (abs(gamma(i,k-1)-gamma(i,k)) > 1.E-6) then
            gamhat(i,k) = log(gamma(i,k-1)/gamma(i,k))*gamma(i,k-1)*gamma(i,k)/ &
                                (gamma(i,k-1)-gamma(i,k))
         else
            gamhat(i,k) = gamma(i,k) 
         end if

#ifdef WU_REFATM
!
! wtw 2005.12
!
         coef = log(pint(i,k)/p(i,k))/log(p(i,k-1)/p(i,k))
         gamhat(i,k) = gamma(i,k) + (gamma(i,k-1)-gamma(i,k))*coef
#endif

      end do
   end do
!
! initialize cloud top to highest plume top.
!jr changed hard-wired 4 to limcnv+1 (not to exceed pver)
!
   do i = 1,il2g
      jt(i) = max(lel(i),limcnv+1)
      jt(i) = min(jt(i),pver)
      jd(i) = pver
      jlcl(i) = lel(i)
      hmin(i) = 1.E6
   end do
!
! find the level of minimum hsat, where detrainment starts
!
   do k = msg + 1,pver
      do i = 1,il2g
         if (hsat(i,k) <= hmin(i) .and. k >= jt(i) .and. k <= jb(i)) then
            hmin(i) = hsat(i,k)
            j0(i) = k
         end if
      end do
   end do
   do i = 1,il2g
      j0(i) = min(j0(i),jb(i)-2)
      j0(i) = max(j0(i),jt(i)+2)
!
! Fix from Guang Zhang to address out of bounds array reference
!
      j0(i) = min(j0(i),pver)
   end do
!
! Initialize certain arrays inside cloud
!
   do k = msg + 1,pver
      do i = 1,il2g
         if (k >= jt(i) .and. k <= jb(i)) then
            hu(i,k) = hmn(i,mx(i)) + cp*0.5
            su(i,k) = s(i,mx(i)) + 0.5
         end if
      end do
   end do
!
! *********************************************************
! compute taylor series for approximate eps(z) below
! *********************************************************
!
   do k = pver - 1,msg + 1,-1
      do i = 1,il2g
         if (k < jb(i) .and. k >= jt(i)) then
            k1(i,k) = k1(i,k+1) + (hmn(i,mx(i))-hmn(i,k))*dz(i,k)
            ihat(i,k) = 0.5* (k1(i,k+1)+k1(i,k))
            i2(i,k) = i2(i,k+1) + ihat(i,k)*dz(i,k)
            idag(i,k) = 0.5* (i2(i,k+1)+i2(i,k))
            i3(i,k) = i3(i,k+1) + idag(i,k)*dz(i,k)
            iprm(i,k) = 0.5* (i3(i,k+1)+i3(i,k))
            i4(i,k) = i4(i,k+1) + iprm(i,k)*dz(i,k)
         end if
      end do
   end do
!
! re-initialize hmin array for ensuing calculation.
!
   do i = 1,il2g
      hmin(i) = 1.E6
   end do
   do k = msg + 1,pver
      do i = 1,il2g
         if (k >= j0(i) .and. k <= jb(i) .and. hmn(i,k) <= hmin(i)) then
            hmin(i) = hmn(i,k)
            expdif(i) = hmn(i,mx(i)) - hmin(i)
         end if
      end do
   end do
!
! *********************************************************
! compute approximate eps(z) using above taylor series
! *********************************************************
!
   do k = msg + 2,pver
      do i = 1,il2g
         expnum(i) = 0.
         ftemp(i) = 0.
         if (k < jt(i) .or. k >= jb(i)) then
            k1(i,k) = 0.
            expnum(i) = 0.
         else
            expnum(i) = hmn(i,mx(i)) - (hsat(i,k-1)*(zf(i,k)-z(i,k)) + &
                        hsat(i,k)* (z(i,k-1)-zf(i,k)))/(z(i,k-1)-z(i,k))
         end if
         if ((expdif(i) > 100. .and. expnum(i) > 0.) .and. &
	     k1(i,k) > expnum(i)*dz(i,k)) then
            ftemp(i) = expnum(i)/k1(i,k)
            f(i,k) = ftemp(i) + i2(i,k)/k1(i,k)*ftemp(i)**2 + &
                     (2.*i2(i,k)**2-k1(i,k)*i3(i,k))/k1(i,k)**2* &
                     ftemp(i)**3 + (-5.*k1(i,k)*i2(i,k)*i3(i,k)+ &
                     5.*i2(i,k)**3+k1(i,k)**2*i4(i,k))/ &
                     k1(i,k)**3*ftemp(i)**4
            f(i,k) = max(f(i,k),0._r8)
#ifdef WU_CONV
            f(i,k) = min(f(i,k),0.002_r8)
#else
            f(i,k) = min(f(i,k),0.0002_r8)
#endif
         end if
      end do
   end do
   do i = 1,il2g
      if (j0(i) < jb(i)) then
         if (f(i,j0(i)) < 1.E-6 .and. f(i,j0(i)+1) > f(i,j0(i))) j0(i) = j0(i) + 1
      end if
   end do
   do k = msg + 2,pver
      do i = 1,il2g
         if (k >= jt(i) .and. k <= j0(i)) then
            f(i,k) = max(f(i,k),f(i,k-1))
         end if
      end do
   end do
   do i = 1,il2g
      eps0(i) = f(i,j0(i))
      eps(i,jb(i)) = eps0(i)
   end do
!
! This is set to match the Rasch and Kristjansson paper
!
   do k = pver,msg + 1,-1
      do i = 1,il2g
         if (k >= j0(i) .and. k <= jb(i)) then
            eps(i,k) = f(i,j0(i))
         end if
      end do
   end do
   do k = pver,msg + 1,-1
      do i = 1,il2g
         if (k < j0(i) .and. k >= jt(i)) eps(i,k) = f(i,k)
      end do
   end do
!
! specify the updraft mass flux mu, entrainment eu, detrainment du
! and moist static energy hu.
! here and below mu, eu,du, md and ed are all normalized by mb
!
   do i = 1,il2g
      if (eps0(i) > 0.) then
         mu(i,jb(i)) = 1.
         eu(i,jb(i)) = mu(i,jb(i))/dz(i,jb(i))
      end if
   end do
   do k = pver,msg + 1,-1
      do i = 1,il2g
         if (eps0(i) > 0. .and. (k >= jt(i) .and. k < jb(i))) then
            zuef(i) = zf(i,k) - zf(i,jb(i))
            rmue(i) = (1./eps0(i))* (exp(eps(i,k+1)*zuef(i))-1.)/zuef(i)
            mu(i,k) = (1./eps0(i))* (exp(eps(i,k  )*zuef(i))-1.)/zuef(i)
            eu(i,k) = (rmue(i)-mu(i,k+1))/dz(i,k)
            du(i,k) = (rmue(i)-mu(i,k))/dz(i,k)
         end if
      end do
   end do
!
   khighest = pverp
   klowest = 1
   do i=1,il2g
      khighest = min(khighest,lel(i))
      klowest = max(klowest,jb(i))
   end do
   do k = klowest-1,khighest,-1
!cdir$ ivdep
      do i = 1,il2g
         if (k <= jb(i)-1 .and. k >= lel(i) .and. eps0(i) > 0.) then
            if (mu(i,k) < 0.01) then
               hu(i,k) = hu(i,jb(i))
               mu(i,k) = 0.
               eu(i,k) = 0.
               du(i,k) = mu(i,k+1)/dz(i,k)
            else
               hu(i,k) = mu(i,k+1)/mu(i,k)*hu(i,k+1) + &
                         dz(i,k)/mu(i,k)* (eu(i,k)*hmn(i,k)- du(i,k)*hsat(i,k))
            end if
         end if
      end do
   end do
!
! reset cloud top index beginning from two layers above the
! cloud base (i.e. if cloud is only one layer thick, top is not reset
!
   do i=1,il2g
      doit(i) = .true.
   end do
   do k=klowest-2,khighest-1,-1
      do i=1,il2g
         if (doit(i) .and. k <= jb(i)-2 .and. k >= lel(i)-1) then
            if (hu(i,k) <= hsthat(i,k) .and. hu(i,k+1) > hsthat(i,k+1) &
             .and. mu(i,k) >= 0.02) then
               if (hu(i,k)-hsthat(i,k) < -2000.) then
                  jt(i) = k + 1
                  doit(i) = .false.
               else
                  jt(i) = k
                  if (eps0(i) <= 0.) doit(i) = .false.
               end if
#ifdef WU_CONV
            else if (hu(i,k) > hu(i,jb(i)) .or. mu(i,k) < 0.02) then
#else
            else if (hu(i,k) > hu(i,jb(i)) .or. mu(i,k) < 0.01) then
#endif
               jt(i) = k + 1
               doit(i) = .false.
            end if
         end if
      end do
   end do
   do k = pver,msg + 1,-1
!cdir$ ivdep
      do i = 1,il2g
         if (k >= lel(i) .and. k <= jt(i) .and. eps0(i) > 0.) then
            mu(i,k) = 0.
            eu(i,k) = 0.
            du(i,k) = 0.
            hu(i,k) = hu(i,jb(i))
         end if
         if (k == jt(i) .and. eps0(i) > 0.) then
            du(i,k) = mu(i,k+1)/dz(i,k)
            eu(i,k) = 0.
            mu(i,k) = 0.
         end if
      end do
   end do
!
! specify downdraft properties (no downdrafts if jd.ge.jb).
! scale down downward mass flux profile so that net flux
! (up-down) at cloud base in not negative.
!
   do i = 1,il2g
!
! in normal downdraft strength run alfa=0.2.  In test4 alfa=0.1
!
      alfa(i) = 0.1
      jt(i) = min(jt(i),jb(i)-1)
      jd(i) = max(j0(i),jt(i)+1)
      jd(i) = min(jd(i),jb(i))
      hd(i,jd(i)) = hmn(i,jd(i)-1)
      if (jd(i) < jb(i) .and. eps0(i) > 0.) then
         epsm(i) = eps0(i)
         md(i,jd(i)) = -alfa(i)*epsm(i)/eps0(i)
      end if
   end do
   do k = msg + 1,pver
      do i = 1,il2g
         if ((k > jd(i) .and. k <= jb(i)) .and. eps0(i) > 0.) then
            zdef(i) = zf(i,jd(i)) - zf(i,k)
            md(i,k) = -alfa(i)/ (2.*eps0(i))*(exp(2.*epsm(i)*zdef(i))-1.)/zdef(i)
         end if
      end do
   end do
   do k = msg + 1,pver
      do i = 1,il2g
         if ((k >= jt(i) .and. k <= jb(i)) .and. eps0(i) > 0. .and. jd(i) < jb(i)) then
            ratmjb(i) = min(abs(mu(i,jb(i))/md(i,jb(i))),1._r8)
            md(i,k) = md(i,k)*ratmjb(i)
         end if
      end do
   end do

   small = 1.e-20
   do k = msg + 1,pver
      do i = 1,il2g
         if ((k >= jt(i) .and. k <= pver) .and. eps0(i) > 0.) then
            ed(i,k-1) = (md(i,k-1)-md(i,k))/dz(i,k-1)
            mdt = min(md(i,k),-small)
            hd(i,k) = (md(i,k-1)*hd(i,k-1) - dz(i,k-1)*ed(i,k-1)*hmn(i,k-1))/mdt
         end if
      end do
   end do
!
! calculate updraft and downdraft properties.
!
   do k = msg + 2,pver
      do i = 1,il2g
         if ((k >= jd(i) .and. k <= jb(i)) .and. eps0(i) > 0. .and. jd(i) < jb(i)) then
            qds(i,k) = qsthat(i,k) + gamhat(i,k)*(hd(i,k)-hsthat(i,k))/ &
               (rl*(1. + gamhat(i,k)))
         end if
      end do
   end do
!
   do i = 1,il2g
      done(i) = .false.
   end do
   kount = 0
   do k = pver,msg + 2,-1
      do i = 1,il2g
         if (( .not. done(i) .and. k > jt(i) .and. k < jb(i)) .and. eps0(i) > 0.) then
            su(i,k) = mu(i,k+1)/mu(i,k)*su(i,k+1) + &
                      dz(i,k)/mu(i,k)* (eu(i,k)-du(i,k))*s(i,k)
            qu(i,k) = mu(i,k+1)/mu(i,k)*qu(i,k+1) + dz(i,k)/mu(i,k)* (eu(i,k)*q(i,k)- &
                            du(i,k)*qst(i,k))
            tu = su(i,k) - grav/cp*zf(i,k)
            estu = c1*exp((c2* (tu-tfreez))/ ((tu-tfreez)+c3))
            qstu = eps1*estu/ ((p(i,k)+p(i,k-1))/2.-estu)
            if (qu(i,k) >= qstu) then
               jlcl(i) = k
               kount = kount + 1
               done(i) = .true.
            end if
         end if
      end do
      if (kount >= il2g) goto 690
   end do
690 continue
   do k = msg + 2,pver
      do i = 1,il2g
         if (k == jb(i) .and. eps0(i) > 0.) then
            qu(i,k) = q(i,mx(i))
            su(i,k) = (hu(i,k)-rl*qu(i,k))/cp
         end if
         if ((k > jt(i) .and. k <= jlcl(i)) .and. eps0(i) > 0.) then
            su(i,k) = shat(i,k) + (hu(i,k)-hsthat(i,k))/(cp* (1.+gamhat(i,k)))
            qu(i,k) = qsthat(i,k) + gamhat(i,k)*(hu(i,k)-hsthat(i,k))/ &
                     (rl* (1.+gamhat(i,k)))
         end if
      end do
   end do

! compute condensation in updraft
   do k = pver,msg + 2,-1
      do i = 1,il2g
         if (k >= jt(i) .and. k < jb(i) .and. eps0(i) > 0.) then
            cu(i,k) = ((mu(i,k)*su(i,k)-mu(i,k+1)*su(i,k+1))/ &
                      dz(i,k)- (eu(i,k)-du(i,k))*s(i,k))/(rl/cp)
            if (k == jt(i)) cu(i,k) = 0.
            cu(i,k) = max(0._r8,cu(i,k))
         end if
      end do
   end do

! compute condensed liquid, rain production rate
! accumulate total precipitation (condensation - detrainment of liquid)
! Note ql1 = ql(k) + rprd(k)*dz(k)/mu(k)
! The differencing is somewhat strange (e.g. du(i,k)*ql(i,k+1)) but is
! consistently applied.
!    mu, ql are interface quantities
!    cu, du, eu, rprd are midpoint quantites
!!$   c0 = 1.E-3
!!$   c0 = 0.25E-3
!!$   c0 = 0.5E-3
   do k = pver,msg + 2,-1
      do i = 1,il2g
         rprd(i,k) = 0.
         if (k >= jt(i) .and. k < jb(i) .and. eps0(i) > 0. .and. mu(i,k) >= 0.0) then
            if (mu(i,k) > 0.) then
               ql1 = 1./mu(i,k)* (mu(i,k+1)*ql(i,k+1)- &
                     dz(i,k)*du(i,k)*ql(i,k+1)+dz(i,k)*cu(i,k))
               ql(i,k) = ql1/ (1.+dz(i,k)*c0)
            else
               ql(i,k) = 0.
            end if
            totpcp(i) = totpcp(i) + dz(i,k)*(cu(i,k)-du(i,k)*ql(i,k+1))
            rprd(i,k) = c0*mu(i,k)*ql(i,k)
         end if
      end do
   end do
!
   do i = 1,il2g
      qd(i,jd(i)) = qds(i,jd(i))
      sd(i,jd(i)) = (hd(i,jd(i)) - rl*qd(i,jd(i)))/cp
   end do
!
   do k = msg + 2,pver
      do i = 1,il2g
         if (k >= jd(i) .and. k < jb(i) .and. eps0(i) > 0.) then
            qd(i,k+1) = qds(i,k+1)
            evp(i,k) = -ed(i,k)*q(i,k) + (md(i,k)*qd(i,k)-md(i,k+1)*qd(i,k+1))/dz(i,k)
            evp(i,k) = max(evp(i,k),0._r8)
            mdt = min(md(i,k+1),-small)
            sd(i,k+1) = ((rl/cp*evp(i,k)-ed(i,k)*s(i,k))*dz(i,k) + md(i,k)*sd(i,k))/mdt
            totevp(i) = totevp(i) - dz(i,k)*ed(i,k)*q(i,k)
         end if
      end do
   end do
   do i = 1,il2g
!*guang         totevp(i) = totevp(i) + md(i,jd(i))*q(i,jd(i)-1) -
      totevp(i) = totevp(i) + md(i,jd(i))*qd(i,jd(i)) - md(i,jb(i))*qd(i,jb(i))
   end do
!!$   if (.true.) then
   if (.false.) then
      do i = 1,il2g
         k = jb(i)
         if (eps0(i) > 0.) then
            evp(i,k) = -ed(i,k)*q(i,k) + (md(i,k)*qd(i,k))/dz(i,k)
            evp(i,k) = max(evp(i,k),0._r8)
            totevp(i) = totevp(i) - dz(i,k)*ed(i,k)*q(i,k)
         end if
      end do
   endif

   do i = 1,il2g
      totpcp(i) = max(totpcp(i),0._r8)
      totevp(i) = max(totevp(i),0._r8)
   end do
!
   do k = msg + 2,pver
      do i = 1,il2g
         if (totevp(i) > 0. .and. totpcp(i) > 0.) then
            md(i,k)  = md (i,k)*min(1._r8, totpcp(i)/(totevp(i)+totpcp(i)))
            ed(i,k)  = ed (i,k)*min(1._r8, totpcp(i)/(totevp(i)+totpcp(i)))
            evp(i,k) = evp(i,k)*min(1._r8, totpcp(i)/(totevp(i)+totpcp(i)))
         else
            md(i,k) = 0.
            ed(i,k) = 0.
            evp(i,k) = 0.
         end if
! cmeg is the cloud water condensed - rain water evaporated
! rprd is the cloud water converted to rain - (rain evaporated)
         cmeg(i,k) = cu(i,k) - evp(i,k)
         rprd(i,k) = rprd(i,k)-evp(i,k)
      end do
   end do

! compute the net precipitation flux across interfaces
   pflx(:il2g,1) = 0.
   do k = 2,pverp
      do i = 1,il2g
         pflx(i,k) = pflx(i,k-1) + rprd(i,k-1)*dz(i,k-1)
      end do
   end do
!
   do k = msg + 1,pver
      do i = 1,il2g
         mc(i,k) = mu(i,k) + md(i,k)
      end do
   end do
!
   return
end subroutine cldprp

subroutine q1q2_wu( lchnk   , &
                    dqdt    ,dsdt    ,q       ,qs      ,qu      , &
                    su      ,du      ,qhat    ,shat    ,dp      , &
                    mu      ,md      ,sd      ,qd      ,ql      , &
                    dsubcld ,lel     ,lcl     ,mx      ,il1g    ,il2g    , &
                    cp      ,rl      ,msg     ,     &
                    dl      ,evp     ,cu      ,duql    ,      &
                    ptendu  ,ptendv  ,uhat    ,vhat    , &
                    up      ,vp      ,ud      ,vd      , &
                    delt    ,z       ,zf      ,eu      ,ed    , &
                    ncnst   ,cnst  )
  
   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid,          only: pcols, pver, pverp
   use moistconvection, only: rgas
   use physconst,       only: cpair, epsilo, latvap, tmelt,cappa
   use dycore,    only: dycore_is, get_resolution
  
   implicit none

!-----------------------------------------------------------------------
! 
! Purpose: 
! <Say what the routine does>
! 
! Method:
! <Describe the algorithm(s) used in the routine.>
! <Also include any applicable external references.>
! 
! Author: phil rasch dec 19 1995
! 
!-----------------------------------------------------------------------


   real(r8), intent(in) :: cp

   integer, intent(in) :: lchnk             ! chunk identifier
   integer, intent(in) :: il1g
   integer, intent(in) :: il2g
   integer, intent(in) :: msg

   real(r8), intent(in) :: q(pcols,pver)
   real(r8), intent(in) :: qs(pcols,pver)
   real(r8), intent(in) :: qu(pcols,pver)
   real(r8), intent(in) :: su(pcols,pver)
   real(r8), intent(in) :: du(pcols,pver)
   real(r8), intent(in) :: qhat(pcols,pver)
   real(r8), intent(in) :: shat(pcols,pver)
   real(r8), intent(in) :: dp(pcols,pver)
   real(r8), intent(in) :: mu(pcols,pver)
   real(r8), intent(in) :: md(pcols,pver)
   real(r8), intent(in) :: sd(pcols,pver)
   real(r8), intent(in) :: qd(pcols,pver)
   real(r8), intent(in) :: ql(pcols,pver)
   real(r8), intent(in) :: evp(pcols,pver)
   real(r8), intent(in) :: cu(pcols,pver)
   real(r8), intent(in) :: dsubcld(pcols)
   real(r8), intent(in) :: duql(pcols,pver)

   real(r8), intent(in) :: eu(pcols,pver)
   real(r8), intent(in) :: ed(pcols,pver)
   real(r8), intent(in) :: z(pcols,pver)
   real(r8), intent(in) :: zf(pcols,pver+1)
   real(r8) :: delt
   integer  :: ncnst
   real(r8), intent(inout) :: cnst(pcols,pver,ncnst)

   real(r8), intent(in) :: uhat(pcols,pver)
   real(r8), intent(in) :: vhat(pcols,pver)
   real(r8), intent(in) :: up(pcols,pver)
   real(r8), intent(in) :: vp(pcols,pver)
   real(r8), intent(in) :: ud(pcols,pver)
   real(r8), intent(in) :: vd(pcols,pver)
   real(r8), intent(out) :: ptendu(pcols,pver)
   real(r8), intent(out) :: ptendv(pcols,pver)

   real(r8) :: ptends(pcols,pver)
   real(r8) :: sp(pcols,pver)
   real(r8) :: sdn(pcols,pver)
   real(r8) :: s(pcols,pver)
   real(r8) :: sf(pcols,pver+1)
   real(r8) :: coef, wktmp, mdt, small
   integer  :: m

   real(r8),intent(out) :: dqdt(pcols,pver),dsdt(pcols,pver)
   real(r8),intent(out) :: dl(pcols,pver)
   integer kbm
   integer ktm
   integer lel(pcols)
   integer lcl(pcols)
   integer mx(pcols)
!
! work fields:
!
   integer i
   integer k

   real(r8) emc
   real(r8) rl
   real(r8) c_deep
!-------------------------------------------------------------------
   do k = 1,pver
      do i = il1g,il2g
         dsdt(i,k) = 0.
         dqdt(i,k) = 0.
         ptendu(i,k) = 0.
         ptendv(i,k) = 0.
         dl(i,k) = 0.
      end do
   end do
!--------------------------------------------------------------------------

do m= 1, ncnst
   s(1:pcols,1:pver) = cnst(1:pcols,1:pver,m)
   ptends(:,:) = 0.0

   do i = il1g,il2g
      sf(i,1)     = s(i,1)
   enddo
   do k = 2, pver
      do i = il1g,il2g
         coef    = (zf(i,k)-z(i,k))/(z(i,k-1)-z(i,k))
         sf(i,k) = s(i,k) + (s(i,k-1)-s(i,k))*coef
      enddo
   enddo

   do k = 1, pver
      do i = il1g,il2g
         sp(i,k) = sf(i,k)
         sdn(i,k)= sf(i,k)
      enddo
   enddo
!
   do k = 1,pver
      do i = il1g,il2g
         if (k <= mx(i) .and. k > lel(i) ) then
            wktmp = mu(i,k) - du(i,k)*0.5*(zf(i,k+1)-zf(i,k))
            if( abs(wktmp) > 1.e-20 ) then
                 sp(i,k) = (mu(i,k+1)*sp(i,k+1) - (zf(i,k+1)-zf(i,k))* &
                         (eu(i,k)*s(i,k) -0.5*du(i,k)*sp(i,k+1)) )/ wktmp
            else
                 sp(i,k) = sf(i,k)
            end if
         endif

         small = 1.e-20
         if ( k > lel(i) .and. md(i,k) < 0.0 .and. k <=mx(i) ) then
            mdt = min(md(i,k),-small)
            sdn(i,k) = (md(i,k-1)*sdn(i,k-1) - (zf(i,k-1)-zf(i,k))*ed(i,k-1)*s(i,k-1))/mdt
         end if
      end do
   end do

   do k = 1,pver
      do i = il1g,il2g
         if (k <mx(i) .and. k >= lel(i)) then
             ptends(i,k) =    &
                       (+mu(i,k+1)* (sp(i,k+1)-sf(i,k+1)) &
                        -mu(i,k)*   (sp(i,k)  -sf(i,k))   &
                        +md(i,k+1)* (sdn(i,k+1)-sf(i,k+1)) &
                        -md(i,k)*   (sdn(i,k)  -sf(i,k))   &
                                                )/dp(i,k)    &
                        +du(i,k)*( (sp(i,k+1)-sf(i,k+1))+(sp(i,k)-sf(i,k)))/2.0
         elseif (k >= mx(i)) then
             ptends(i,k) = 1.0/dsubcld(i)* &
                        (-mu(i,mx(i))* (sp(i,k)-sf(i,k)) &
                         -md(i,mx(i))* (sdn(i,k)-sf(i,k)) &
                                                 )
         end if

         cnst(i,k,m) = cnst(i,k,m) + ptends(i,k)*2.*delt
         if( cnst(i,k,m) < 1.e-20) cnst(i,k,m) = 1.e-20
      enddo
   enddo
enddo
!---------------------------------------------------------------
!
! find the highest level top and bottom levels of convection
!
   c_deep= 0.55
!
   do k = 1, pver
      do i = il1g,il2g
         if( k >=lel(i) .and. k < mx(i)) then
             emc = -cu (i,k)  + evp(i,k)
             dsdt(i,k) = -rl/cp*emc &
                     + (+mu(i,k+1)* (su(i,k+1)-shat(i,k+1)) &
                        -mu(i,k)*   (su(i,k)-shat(i,k)) &
                        +md(i,k+1)* (sd(i,k+1)-shat(i,k+1)) &
                        -md(i,k)*   (sd(i,k)-shat(i,k)) &
                       )/dp(i,k)

             dqdt(i,k) = emc + &
                    (+mu(i,k+1)* (qu(i,k+1)-qhat(i,k+1)) &
                     -mu(i,k)*   (qu(i,k)-qhat(i,k)) &
                     +md(i,k+1)* (qd(i,k+1)-qhat(i,k+1)) &
                     -md(i,k)*   (qd(i,k)-qhat(i,k)) &
                    )/dp(i,k)

            dl(i,k) = duql(i,k)


         elseif (k >= mx(i)) then
            dsdt(i,k) = (1./dsubcld(i))* &
                        (-mu(i,mx(i))* (su(i,mx(i))-shat(i,mx(i))) &
                         -md(i,mx(i))* (sd(i,mx(i))-shat(i,mx(i))) &
                        )
            dqdt(i,k) = (1./dsubcld(i))* &
                        (-mu(i,mx(i))*(qu(i,mx(i))-qhat(i,mx(i))) &
                         -md(i,mx(i))*(qd(i,mx(i))-qhat(i,mx(i))) &
                        )
         end if
      end do
   end do
!
   do k = 1,pver
      do i = il1g,il2g
         if (k <mx(i) .and. k >= lel(i)) then
             ptendu(i,k) = (1.0-c_deep)*    &
                       (+mu(i,k+1)* (up(i,k+1)-uhat(i,k+1)) &
                        -mu(i,k)*   (up(i,k)  -uhat(i,k))   &
                                                +md(i,k+1)* (ud(i,k+1)-uhat(i,k+1)) &
                        -md(i,k)*   (ud(i,k)  -uhat(i,k))   &
                                                )/dp(i,k)    &
                        + Du(i,k)*( (up(i,k+1)-uhat(i,k+1))+(up(i,k)-uhat(i,k)))/2.0

             ptendv(i,k) = (1.0-c_deep)*    &
                                   (+mu(i,k+1)* (vp(i,k+1)-vhat(i,k+1)) &
                        -mu(i,k)*   (vp(i,k)  -vhat(i,k))   &
                        +md(i,k+1)* (vd(i,k+1)-vhat(i,k+1)) &
                        -md(i,k)*   (vd(i,k)  -vhat(i,k))   &
                       )/dp(i,k)    &
                        + Du(i,k)*( (vp(i,k+1)-vhat(i,k+1)) + (vp(i,k)-vhat(i,k)))/2.0

         elseif (k >= mx(i)) then
            ptendu(i,k) = (1.0-c_deep)/dsubcld(i)* &
                        (-mu(i,mx(i))* (up(i,mx(i))-uhat(i,mx(i))) &
                         -md(i,mx(i))* (ud(i,mx(i))-uhat(i,mx(i))) &
                                                 )
            ptendv(i,k) = (1.0-c_deep)/dsubcld(i)* &
                        (-mu(i,mx(i))* (vp(i,mx(i))-vhat(i,mx(i))) &
                         -md(i,mx(i))* (vd(i,mx(i))-vhat(i,mx(i))) &
                                                 )

        end if
     enddo
   enddo
!
   return
end subroutine q1q2_wu

      subroutine wetairparcel(thtu, p, tbar, tp, qp)
      use shr_kind_mod,    only: r8 => shr_kind_r8
      implicit none
!----
	  real(r8), intent(in)    :: p, thtu, tbar
      real(r8), intent(inout) :: tp, qp
!
!--------------------------Local Variables------------------------------
!
      real(r8) es, qs, pi, thtgs, f0, t1, t0, f1, dt
	  integer  ITCNT
!                                                                
!   ITERATE TO FIND WET BULB TEMPERATURE AS A FUNCTION OF EQUIVALENT POT 
!   TEMP AND PRS, ASSUMING SATURATION VAPOR PRESSURE...RATIO2 IS THE DEG
!   OF GLACIATION...                                                   
!-----------------------------------------
!     tp =  tt0* (p/pp0)**(0.2854* (1.-0.28*qq0))
      tp = tbar
!-
      es = c1*exp((c2* (tp-273.16))/((tp-273.16)+c3))
	  if(p-es>0.) then
         qs  = eps1*es/ (p-es)
	  else
	     qs  = 1.0
	  endif

      pi=(1000.0/p)**(0.2854*(1.-0.28*qs))                        
      THTGS= tp*pi*EXP((3374.6525/tp-2.5403)*qs*(1.+0.81*qs))   

      F0=THTGS-THTU                                             
      T1=tp-0.5*F0                                             
      T0=tp                                                   
      ITCNT=0                                                
!--
 90   es = c1*exp((c2* (t1-273.16))/((t1-273.16)+c3))
	  if(p-es>0.) then
         qs  = eps1*es/ (p-es)
	  else
	     qs  = 1.0
	  endif
      pi=(1000.0/p)**(0.2854*(1.-0.28*qs))                 
      THTGS=t1*pi*EXP((3374.6525/T1-2.5403)*QS*(1.+0.81*QS))    
      F1=THTGS-THTU                                        
      IF(ABS(F1).LT.0.01)GOTO 50         
      ITCNT=ITCNT+1                                               
      IF(ITCNT.GT.10)GOTO 50                                     
      DT=F1*(T1-T0)/(F1-F0)                                     
      T0=T1                                                   
      F0=F1                                                  
      T1=T1-DT                                              
      GOTO 90                                              
50    tp = T1
      qp = qs
	  return
  end subroutine wetairparcel


   subroutine check_triger(lchnk   ,ncol    ,omega, p, t, q, z, &
                           LCL,   PLCL,   TLCL, deepconv)
   use shr_kind_mod,    only: r8 => shr_kind_r8
   use ppgrid,          only: pcols, pver

   implicit none
!-----------------------------------------------------------------------

   integer, intent(in) :: lchnk             ! chunk identifier
   integer, intent(in) :: ncol                  ! number of atmospheric columns

   real(r8), intent(in) :: q(pcols,pver)
   real(r8), intent(in) :: t(pcols,pver)        ! temperature
   real(r8), intent(in) :: p(pcols,pver)        ! pressure
   real(r8), intent(in) :: omega(pcols,pver) 
   real(r8), intent(in) :: z(pcols,pver)        ! height

   integer,  intent(in) :: LCL(pcols)
   real(r8), intent(in) :: plcl(pcols)
   real(r8), intent(in) :: tlcl(pcols)
   logical,  intent(inout) :: deepconv(pcols)     

! ---
! local variables
!
   integer i, k, KLCL
   real(r8)  tv(pver)
   real(r8)  rhoe, w, scr1, DLP
   real(r8)  W0AVG(pver)
   real(r8)  TENV, ZLCL, DX, WKLCL, WKL, WABS, WSIGNE, DTLCL
!-------------------------------------------------------------
   do i = 1,ncol

     DO K=pver, 2, -1
         tv(k)  = t(i,k)*(1.+0.608*q(i,K))                                 
!                                                                         
         rhoe= (p(i,K)*100.0)/(287.04*Tv(k))
		 w   = - omega(i,k)/(rhoe*9.80616)      ! w(i,k) vertical velocity (m/s)
         scr1=-1.0E-3*9.80616* rhoe*w
         W0AVG(k) =-101.9368*scr1/rhoe
         W0AVG(k) = w
!
     enddo

!                                                                   
!...ESTIMATE ENVIRONMENTAL TEMPERATURE AND MIXING RATIO AT THE LCL...
!         
        KLCL = lcl(i)                                                          
        K= KLCL + 1
        DLP=LOG(PLCL(i)/p(i,K))/LOG(p(i,KLCL)/p(i,K))     
        TENV=t(i,K)+(t(i,KLCL)-t(i,K))*DLP                            
        ZLCL=z(i,K)+(Z(i,KLCL)-Z(i,K))*DLP - Z(i,pver) 
!                                                                      
!...CHECK TO SEE IF CLOUD IS BUOYANT USING FRITSCH-CHAPPELL TRIGGER  
!...FUNCTION DESCRIBED IN KAIN AND FRITSCH (1992)...W0AVG IS AN     
!...APROXIMATE VALUE FOR THE RUNNING-MEAN GRID-SCALE VERTICAL      
!...VELOCITY, WHICH GIVES SMOOTHER FIELDS OF CONVECTIVE INITIATION 
!...THAN THE INSTANTANEOUS VALUE...FORMULA RELATING TEMPERATURE            
!...PERTURBATION TO VERTICAL VELOCITY HAS BEEN USED WITH THE MOST         
!...SUCCESS AT GRID LENGTHS NEAR 25 km.  FOR DIFFERENT GRID-LENGTHS,     
!...ADJUST VERTICAL VELOCITY TO EQUIVALENT VALUE FOR 25 KM GRID         
!...LENGTH, ASSUMING LINEAR DEPENDENCE OF W ON GRID LENGTH...          
!             
        DX= 25.0E3                 ! wtw    50 km                                                    
        WKLCL=0.02*ZLCL/2.5E3      ! ZLCL: m   WKLCL: m/s   
        WKL=(W0AVG(K)+(W0AVG(KLCL)-W0AVG(K))*DLP)*DX/25.E3- & 
            WKLCL                                                           
        WABS=ABS(WKL)+1.E-10                                               
        WSIGNE=WKL/WABS                                                   
        DTLCL=4.64*WSIGNE*WABS**0.33                                     
!
!       write(*,*)'====================== p(i,klcl), zlcl, w0avg, omega=', p(i,klcl),zlcl,w0avg(klcl), omega(i,klcl), DTLCL,TLCL(i) 
!                          
        IF(TLCL(i)+DTLCL.GT.TENV) then
		   deepconv(i) = .true.
        else
		   deepconv(i) = .false.
        endif
    enddo
end subroutine check_triger


subroutine closure_wu(lchnk        ,cu      ,evp     , &
                   q       ,t       ,p       ,z       ,s       , &
                   tp      ,qs      ,qu      ,su      ,mc      , &
                   du      ,mu      ,md      ,qd      ,sd      , &
                   qhat    ,shat    ,dp      ,qstp    ,zf      , &
                   ql      ,dsubcld ,mb      ,cape    ,tl      , &
                   lcl     ,lel     ,mx      ,il1g    , &
                   il2g    ,rd      ,grav    ,cp      ,rl      , &
                   msg     ,capelmt ,ptendq  ,ptendt  ,negcape , duql, delt)

!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author: G. Zhang and collaborators. CCM contact:P. Rasch
! This is contributed code not fully standardized by the CCM core group.
!
! this code is very much rougher than virtually anything else in the CCM
! We expect to release cleaner code in a future release
!
! the documentation has been enhanced to the degree that we are able
! 
!-----------------------------------------------------------------------
   use dycore,    only: dycore_is, get_resolution
   use constituents,    only: qmin

   implicit none

!
!-----------------------------Arguments---------------------------------
!
   integer, intent(in) :: lchnk                 ! chunk identifier

   real(r8), intent(inout) :: q(pcols,pver)        ! spec humidity
   real(r8), intent(inout) :: t(pcols,pver)        ! temperature
   real(r8), intent(inout) :: p(pcols,pver)        ! pressure (mb)
   real(r8), intent(inout) :: mb(pcols)            ! cloud base mass flux
   real(r8), intent(in) :: z(pcols,pver)        ! height (m)
   real(r8), intent(in) :: s(pcols,pver)        ! normalized dry static energy
   real(r8), intent(in) :: tp(pcols,pver)       ! parcel temp
   real(r8), intent(in) :: qs(pcols,pver)       ! sat spec humidity
   real(r8), intent(in) :: qu(pcols,pver)       ! updraft spec. humidity
   real(r8), intent(in) :: su(pcols,pver)       ! normalized dry stat energy of updraft
   real(r8), intent(in) :: mc(pcols,pver)       ! net convective mass flux
   real(r8), intent(in) :: du(pcols,pver)       ! detrainment from updraft
   real(r8), intent(in) :: mu(pcols,pver)       ! mass flux of updraft
   real(r8), intent(in) :: md(pcols,pver)       ! mass flux of downdraft
   real(r8), intent(in) :: qd(pcols,pver)       ! spec. humidity of downdraft
   real(r8), intent(in) :: sd(pcols,pver)       ! dry static energy of downdraft
   real(r8), intent(in) :: qhat(pcols,pver)     ! environment spec humidity at interfaces
   real(r8), intent(in) :: shat(pcols,pver)     ! env. normalized dry static energy at intrfcs
   real(r8), intent(in) :: dp(pcols,pver)       ! pressure thickness of layers
   real(r8), intent(in) :: qstp(pcols,pver)     ! spec humidity of parcel
   real(r8), intent(in) :: zf(pcols,pver+1)     ! height of interface levels
   real(r8), intent(in) :: ql(pcols,pver)       ! liquid water mixing ratio
   real(r8), intent(in) :: ptendq(pcols,pver)   ! q tendency
   real(r8), intent(in) :: ptendt(pcols,pver)   ! t tendency
   real(r8), intent(in) :: duql(pcols,pver)   
   real(r8), intent(in) :: delt   

   real(r8), intent(in) :: cu(pcols,pver) 
   real(r8), intent(in) :: evp(pcols,pver)

   real(r8), intent(in) :: cape(pcols)          ! available pot. energy of column
   real(r8), intent(in) :: negcape(pcols)       ! negative available pot. energy of column
   real(r8), intent(in) :: tl(pcols)
   real(r8), intent(in) :: dsubcld(pcols)       ! thickness of subcloud layer

   integer, intent(in) :: lcl(pcols)        ! index of lcl
   integer, intent(in) :: lel(pcols)        ! index of launch leve
   integer, intent(in) :: mx(pcols)         ! base of updraft
!
!--------------------------Local variables------------------------------
!
   real(r8) dtpdt(pcols,pver)
   real(r8) dqsdtp(pcols,pver)
   real(r8) dtmdt(pcols,pver)
   real(r8) dqmdt(pcols,pver)
   real(r8) dboydt(pcols,pver)
   real(r8) thetavp(pcols,pver)
   real(r8) thetavm(pcols,pver)
   real(r8) dbdtls(pcols,pver)

   real(r8) dtbdt(pcols),dqbdt(pcols),dtldt(pcols),rhmx(pcols)
   real(r8) beta, mbmax(pcols)
   real(r8) capelmt
   real(r8) cp
   real(r8) dadt(pcols)
   real(r8) debdt
   real(r8) dltaa
   real(r8) eb, emc
   real(r8) grav
   real(r8) rhcrit 
   real(r8) dadtls
   logical mbmax_log(pcols)

   integer i
   integer il1g
   integer il2g
   integer k, kmin, kmax
   integer msg

   real(r8) rd
   real(r8) rl
! change of subcloud layer properties due to convection is
! related to cumulus updrafts and downdrafts.
! mc(z)=f(z)*mb, mub=betau*mb, mdb=betad*mb are used
! to define betau, betad and f(z).
! note that this implies all time derivatives are in effect
! time derivatives per unit cloud-base mass flux, i.e. they
! have units of 1/mb instead of 1/sec.
!
   do i = il1g,il2g
      mb(i) = 0.
      eb = p(i,mx(i))*q(i,mx(i))/ (eps1+q(i,mx(i)))
      dtbdt(i) = 0.*(1./dsubcld(i))* (mu(i,mx(i))*(shat(i,mx(i))-su(i,mx(i)))+ &
                  md(i,mx(i))* (shat(i,mx(i))-sd(i,mx(i))))
      dqbdt(i) = 0.*(1./dsubcld(i))* (mu(i,mx(i))*(qhat(i,mx(i))-qu(i,mx(i)))+ &
                 md(i,mx(i))* (qhat(i,mx(i))-qd(i,mx(i))))
      debdt = eps1*p(i,mx(i))/ (eps1+q(i,mx(i)))**2*dqbdt(i)
      dtldt(i) = -2840.* (3.5/t(i,mx(i))*dtbdt(i)-debdt/eb)/ &
                 (3.5*log(t(i,mx(i)))-log(eb)-4.805)**2
   end do
!
!   dtmdt and dqmdt are cumulus heating and drying.
!
   do k = msg + 1,pver
      do i = il1g,il2g
         dtmdt(i,k) = 0.
         dqmdt(i,k) = 0.
         dbdtls(i,k) = 0.
      end do
   end do
!
   do k = msg + 1,pver
      do i = il1g,il2g
         if( k >=lel(i) .and. k < mx(i)) then
             emc = -cu (i,k)  + evp(i,k)  + duql(i,k)             
!            emc = -cu (i,k)  + evp(i,k)  
             dtmdt(i,k) = -rl/cp*emc &
                     + (+mu(i,k+1)* (su(i,k+1)-shat(i,k+1)) &
                        -mu(i,k)*   (su(i,k)-shat(i,k)) &
                        +md(i,k+1)* (sd(i,k+1)-shat(i,k+1)) &
                        -md(i,k)*   (sd(i,k)-shat(i,k)) &
                       )/dp(i,k)

             dqmdt(i,k) = emc + &
                    (+mu(i,k+1)* (qu(i,k+1)-qhat(i,k+1)) &
                     -mu(i,k)*   (qu(i,k)-qhat(i,k)) &
                     +md(i,k+1)* (qd(i,k+1)-qhat(i,k+1)) &
                     -md(i,k)*   (qd(i,k)-qhat(i,k)) &
                    )/dp(i,k)

          elseif( k == mx(i) ) then
             dtmdt(i,k) = (1./dsubcld(i))* &
                        (-mu(i,k)* (su(i,k)-shat(i,k)) &
                         -md(i,k)* (sd(i,k)-shat(i,k)) &
                        )
             dqmdt(i,k) = (1./dsubcld(i))* &
                        (-mu(i,k)*(qu(i,k)-qhat(i,k)) &
                         -md(i,k)*(qd(i,k)-qhat(i,k)) &
                        )
         else if (k > mx(i)) then
             dtmdt(i,k) = dtmdt(i,k-1)
             dqmdt(i,k) = dqmdt(i,k-1)
!--
         end if
      end do
   end do

!=========================================
   do k = msg + 1,pver
      do i = il1g,il2g
         if (k >= lel(i) .and. k <= lcl(i)) then
            thetavp(i,k) = tp(i,k)* (1000./p(i,k))** (rd/cp)*(1.+1.608*qstp(i,k)-q(i,mx(i)))
            thetavm(i,k) = t(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,k))
            dqsdtp(i,k) = qstp(i,k)* (1.+qstp(i,k)/eps1)*eps1*rl/(rd*tp(i,k)**2)
!
! dtpdt is the parcel temperature change due to change of
! subcloud layer properties during convection.
!
            dtpdt(i,k) = tp(i,k)/ (1.+rl/cp* (dqsdtp(i,k)-qstp(i,k)/tp(i,k)))* &
                        (dtbdt(i)/t(i,mx(i))+rl/cp* (dqbdt(i)/tl(i)-q(i,mx(i))/ &
                         tl(i)**2*dtldt(i)))
!
! dboydt is the integrand of cape change.
!
            dboydt(i,k) = ((dtpdt(i,k)/tp(i,k)+1./(1.+1.608*qstp(i,k)-q(i,mx(i)))* &
                          (1.608 * dqsdtp(i,k) * dtpdt(i,k) -dqbdt(i))) - (dtmdt(i,k)/t(i,k)+0.608/ &
                          (1.+0.608*q(i,k))*dqmdt(i,k)))*grav*thetavp(i,k)/thetavm(i,k)
         end if
      end do
   end do
!
   do k = msg + 1,pver
      do i = il1g,il2g
         if (k > lcl(i) .and. k < mx(i)) then
            thetavp(i,k) = tp(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,mx(i)))
            thetavm(i,k) = t(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,k))
!
! dboydt is the integrand of cape change.
!
            dboydt(i,k) = (dtbdt(i)/t(i,mx(i))+0.608/ (1.+0.608*q(i,mx(i)))*dqbdt(i)- &
                          dtmdt(i,k)/t(i,k)-0.608/ (1.+0.608*q(i,k))*dqmdt(i,k))* &
                          grav*thetavp(i,k)/thetavm(i,k)
         end if
      end do
   end do

!
! buoyant energy change is set to 2/3*excess cape per 3 hours
!
   dadt(il1g:il2g)  = 0.
   kmin = minval(lel(il1g:il2g))
   kmax = maxval(mx(il1g:il2g)) - 1
   do k = kmin, kmax
      do i = il1g,il2g
         if ( k >= lel(i) .and. k <= mx(i) - 1) then
            dadt(i) = dadt(i) + dboydt(i,k)* (zf(i,k)-zf(i,k+1))
         endif
      end do
   end do
!
!  do i = il1g,il2g
!     dltaa = -1.* (cape(i)-0.0)
!     if (dadt(i) /= 0.) mb(i) = max(dltaa/tau/dadt(i),0._r8)
!  end do
!  do i = il1g,il2g
!     mbmax(i)  = 0.0
!     mbmax_log(i) = .false.
!     do k = msg+1, pver
!        if( dqmdt(i,k) < -1.e-10 ) then
!            mbmax(i) = max(mbmax(i), (qmin(1)-q(i,k))/(2.*delt*dqmdt(i,k)) )
!            mbmax_log(i) = .true.
!        endif
!     enddo
!     if( mbmax_log(i) ) then
!         mb(i) = min(mb(i), mbmax(i))
!     endif
!  enddo 

!***********************************
! test a new closure 
!***********************************

      do k = msg + 1,pver
         do i = il1g,il2g
            if (k.ge.lel(i) .and. k.le.lcl(i)) then
               thetavp(i,k) = tp(i,k)* (1000./p(i,k))** (rd/cp)* &
                               (1.+1.608*qstp(i,k)-q(i,mx(i)))
               thetavm(i,k) = t(i,k)* (1000./p(i,k))** (rd/cp)* &
                               (1.+0.608*q(i,k))
               dbdtls(i,k) = &
                             -( 1/t(i,k) * ptendt(i,k) +  &
                              0.608 / (1+0.608*q(i,k))* ptendq(i,k)) &
                            *grav* thetavp(i,k)/thetavm(i,k)
            end if

            if (k > lcl(i) .and. k < mx(i)) then
               thetavp(i,k) = tp(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,mx(i)))
               thetavm(i,k) = t(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,k))
               dbdtls(i,k) = -(ptendt(i,k)/t(i,k)+0.608/ (1.+0.608*q(i,k))*ptendq(i,k))* &
                             grav*thetavp(i,k)/thetavm(i,k)
            end if

         end do
      end do

   do i = il1g,il2g
         dadtls = 0.
         do k = lel(i),mx(i) - 1
            dadtls = dadtls + dbdtls(i,k)* (zf(i,k)-zf(i,k+1))
         end do

!--------
!         dltaa = -cape(i)
!         if (dadt(i) /= 0.) then
!		      mb(i) = max( (dltaa/tau-dadtls)/dadt(i),0._r8)   
!		      mb(i) = max( (dltaa/tau)/dadt(i),0._r8)   
!		      mb(i) = max( (-dadtls)/dadt(i),0._r8)     ! new2
!         endif
!!
         if (dadtls > 0. .and. dadt(i) < 0. ) then
              mb(i) = max(-dadtls/dadt(i),0.)
         endif
!--------------
   enddo

     do i = il1g,il2g
        mbmax(i)  = 0.0
        mbmax_log(i) = .false.
        do k = msg+1, pver
           if( dqmdt(i,k) < -1.e-10 ) then
               mbmax(i) = max(mbmax(i), (qmin(1)-q(i,k))/(2.*delt*dqmdt(i,k)) )
               mbmax_log(i) = .true.
           endif
        enddo
        if( mbmax_log(i) ) then
            mb(i) = min(mb(i), mbmax(i))
        endif
     enddo
   
   return
end subroutine closure_wu

subroutine closure(lchnk   , &
                   q       ,t       ,p       ,z       ,s       , &
                   tp      ,qs      ,qu      ,su      ,mc      , &
                   du      ,mu      ,md      ,qd      ,sd      , &
                   qhat    ,shat    ,dp      ,qstp    ,zf      , &
                   ql      ,dsubcld ,mb      ,cape    ,tl      , &
                   lcl     ,lel     ,jt      ,mx      ,il1g    , &
                   il2g    ,rd      ,grav    ,cp      ,rl      , &
                   msg     ,capelmt )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author: G. Zhang and collaborators. CCM contact:P. Rasch
! This is contributed code not fully standardized by the CCM core group.
!
! this code is very much rougher than virtually anything else in the CCM
! We expect to release cleaner code in a future release
!
! the documentation has been enhanced to the degree that we are able
! 
!-----------------------------------------------------------------------
   use dycore,    only: dycore_is, get_resolution

   implicit none

!
!-----------------------------Arguments---------------------------------
!
   integer, intent(in) :: lchnk                 ! chunk identifier

   real(r8), intent(inout) :: q(pcols,pver)        ! spec humidity
   real(r8), intent(inout) :: t(pcols,pver)        ! temperature
   real(r8), intent(inout) :: p(pcols,pver)        ! pressure (mb)
   real(r8), intent(inout) :: mb(pcols)            ! cloud base mass flux
   real(r8), intent(in) :: z(pcols,pver)        ! height (m)
   real(r8), intent(in) :: s(pcols,pver)        ! normalized dry static energy
   real(r8), intent(in) :: tp(pcols,pver)       ! parcel temp
   real(r8), intent(in) :: qs(pcols,pver)       ! sat spec humidity
   real(r8), intent(in) :: qu(pcols,pver)       ! updraft spec. humidity
   real(r8), intent(in) :: su(pcols,pver)       ! normalized dry stat energy of updraft
   real(r8), intent(in) :: mc(pcols,pver)       ! net convective mass flux
   real(r8), intent(in) :: du(pcols,pver)       ! detrainment from updraft
   real(r8), intent(in) :: mu(pcols,pver)       ! mass flux of updraft
   real(r8), intent(in) :: md(pcols,pver)       ! mass flux of downdraft
   real(r8), intent(in) :: qd(pcols,pver)       ! spec. humidity of downdraft
   real(r8), intent(in) :: sd(pcols,pver)       ! dry static energy of downdraft
   real(r8), intent(in) :: qhat(pcols,pver)     ! environment spec humidity at interfaces
   real(r8), intent(in) :: shat(pcols,pver)     ! env. normalized dry static energy at intrfcs
   real(r8), intent(in) :: dp(pcols,pver)       ! pressure thickness of layers
   real(r8), intent(in) :: qstp(pcols,pver)     ! spec humidity of parcel
   real(r8), intent(in) :: zf(pcols,pver+1)     ! height of interface levels
   real(r8), intent(in) :: ql(pcols,pver)       ! liquid water mixing ratio

   real(r8), intent(in) :: cape(pcols)          ! available pot. energy of column
   real(r8), intent(in) :: tl(pcols)
   real(r8), intent(in) :: dsubcld(pcols)       ! thickness of subcloud layer

   integer, intent(in) :: lcl(pcols)        ! index of lcl
   integer, intent(in) :: lel(pcols)        ! index of launch leve
   integer, intent(in) :: jt(pcols)         ! top of updraft
   integer, intent(in) :: mx(pcols)         ! base of updraft
!
!--------------------------Local variables------------------------------
!
   real(r8) dtpdt(pcols,pver)
   real(r8) dqsdtp(pcols,pver)
   real(r8) dtmdt(pcols,pver)
   real(r8) dqmdt(pcols,pver)
   real(r8) dboydt(pcols,pver)
   real(r8) thetavp(pcols,pver)
   real(r8) thetavm(pcols,pver)

   real(r8) dtbdt(pcols),dqbdt(pcols),dtldt(pcols)
   real(r8) beta
   real(r8) capelmt
   real(r8) cp
   real(r8) dadt(pcols)
   real(r8) debdt
   real(r8) dltaa
   real(r8) eb
   real(r8) grav

   integer i
   integer il1g
   integer il2g
   integer k, kmin, kmax
   integer msg

   real(r8) rd
   real(r8) rl
! change of subcloud layer properties due to convection is
! related to cumulus updrafts and downdrafts.
! mc(z)=f(z)*mb, mub=betau*mb, mdb=betad*mb are used
! to define betau, betad and f(z).
! note that this implies all time derivatives are in effect
! time derivatives per unit cloud-base mass flux, i.e. they
! have units of 1/mb instead of 1/sec.
!
   do i = il1g,il2g
      mb(i) = 0.
      eb = p(i,mx(i))*q(i,mx(i))/ (eps1+q(i,mx(i)))
      dtbdt(i) = (1./dsubcld(i))* (mu(i,mx(i))*(shat(i,mx(i))-su(i,mx(i)))+ &
                  md(i,mx(i))* (shat(i,mx(i))-sd(i,mx(i))))
      dqbdt(i) = (1./dsubcld(i))* (mu(i,mx(i))*(qhat(i,mx(i))-qu(i,mx(i)))+ &
                 md(i,mx(i))* (qhat(i,mx(i))-qd(i,mx(i))))
      debdt = eps1*p(i,mx(i))/ (eps1+q(i,mx(i)))**2*dqbdt(i)
      dtldt(i) = -2840.* (3.5/t(i,mx(i))*dtbdt(i)-debdt/eb)/ &
                 (3.5*log(t(i,mx(i)))-log(eb)-4.805)**2
   end do
!
!   dtmdt and dqmdt are cumulus heating and drying.
!
   do k = msg + 1,pver
      do i = il1g,il2g
         dtmdt(i,k) = 0.
         dqmdt(i,k) = 0.
      end do
   end do
!
   do k = msg + 1,pver - 1
      do i = il1g,il2g
         if (k == jt(i)) then
            dtmdt(i,k) = (1./dp(i,k))*(mu(i,k+1)* (su(i,k+1)-shat(i,k+1)- &
                          rl/cp*ql(i,k+1))+md(i,k+1)* (sd(i,k+1)-shat(i,k+1)))
            dqmdt(i,k) = (1./dp(i,k))*(mu(i,k+1)* (qu(i,k+1)- &
                         qhat(i,k+1)+ql(i,k+1))+md(i,k+1)*(qd(i,k+1)-qhat(i,k+1)))
         end if
      end do
   end do
!
   beta = 0.
   do k = msg + 1,pver - 1
      do i = il1g,il2g
         if (k > jt(i) .and. k < mx(i)) then
            dtmdt(i,k) = (mc(i,k)* (shat(i,k)-s(i,k))+mc(i,k+1)* (s(i,k)-shat(i,k+1)))/ &
                         dp(i,k) - rl/cp*du(i,k)*(beta*ql(i,k)+ (1-beta)*ql(i,k+1))
!          dqmdt(i,k)=(mc(i,k)*(qhat(i,k)-q(i,k))
!     1                +mc(i,k+1)*(q(i,k)-qhat(i,k+1)))/dp(i,k)
!     2                +du(i,k)*(qs(i,k)-q(i,k))
!     3                +du(i,k)*(beta*ql(i,k)+(1-beta)*ql(i,k+1))

            dqmdt(i,k) = (mu(i,k+1)* (qu(i,k+1)-qhat(i,k+1)+cp/rl* (su(i,k+1)-s(i,k)))- &
                          mu(i,k)* (qu(i,k)-qhat(i,k)+cp/rl*(su(i,k)-s(i,k)))+md(i,k+1)* &
                         (qd(i,k+1)-qhat(i,k+1)+cp/rl*(sd(i,k+1)-s(i,k)))-md(i,k)* &
                         (qd(i,k)-qhat(i,k)+cp/rl*(sd(i,k)-s(i,k))))/dp(i,k) + &
                          du(i,k)* (beta*ql(i,k)+(1-beta)*ql(i,k+1))
         end if
      end do
   end do
!
   do k = msg + 1,pver
      do i = il1g,il2g
         if (k >= lel(i) .and. k <= lcl(i)) then
            thetavp(i,k) = tp(i,k)* (1000./p(i,k))** (rd/cp)*(1.+1.608*qstp(i,k)-q(i,mx(i)))
            thetavm(i,k) = t(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,k))
            dqsdtp(i,k) = qstp(i,k)* (1.+qstp(i,k)/eps1)*eps1*rl/(rd*tp(i,k)**2)
!
! dtpdt is the parcel temperature change due to change of
! subcloud layer properties during convection.
!
            dtpdt(i,k) = tp(i,k)/ (1.+rl/cp* (dqsdtp(i,k)-qstp(i,k)/tp(i,k)))* &
                        (dtbdt(i)/t(i,mx(i))+rl/cp* (dqbdt(i)/tl(i)-q(i,mx(i))/ &
                         tl(i)**2*dtldt(i)))
!
! dboydt is the integrand of cape change.
!
            dboydt(i,k) = ((dtpdt(i,k)/tp(i,k)+1./(1.+1.608*qstp(i,k)-q(i,mx(i)))* &
                          (1.608 * dqsdtp(i,k) * dtpdt(i,k) -dqbdt(i))) - (dtmdt(i,k)/t(i,k)+0.608/ &
                          (1.+0.608*q(i,k))*dqmdt(i,k)))*grav*thetavp(i,k)/thetavm(i,k)
         end if
      end do
   end do
!
   do k = msg + 1,pver
      do i = il1g,il2g
         if (k > lcl(i) .and. k < mx(i)) then
            thetavp(i,k) = tp(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,mx(i)))
            thetavm(i,k) = t(i,k)* (1000./p(i,k))** (rd/cp)*(1.+0.608*q(i,k))
!
! dboydt is the integrand of cape change.
!
            dboydt(i,k) = (dtbdt(i)/t(i,mx(i))+0.608/ (1.+0.608*q(i,mx(i)))*dqbdt(i)- &
                          dtmdt(i,k)/t(i,k)-0.608/ (1.+0.608*q(i,k))*dqmdt(i,k))* &
                          grav*thetavp(i,k)/thetavm(i,k)
         end if
      end do
   end do

!
! buoyant energy change is set to 2/3*excess cape per 3 hours
!
   dadt(il1g:il2g)  = 0.
   kmin = minval(lel(il1g:il2g))
   kmax = maxval(mx(il1g:il2g)) - 1
   do k = kmin, kmax
      do i = il1g,il2g
         if ( k >= lel(i) .and. k <= mx(i) - 1) then
            dadt(i) = dadt(i) + dboydt(i,k)* (zf(i,k)-zf(i,k+1))
         endif
      end do
   end do
   do i = il1g,il2g
      dltaa = -1.* (cape(i)-capelmt)
      if (dadt(i) /= 0.) mb(i) = max(dltaa/tau/dadt(i),0._r8)
   end do
!
   return
end subroutine closure


subroutine q1q2_pjr(lchnk   , &
                    dqdt    ,dsdt    ,q       ,qs      ,qu      , &
                    su      ,du      ,qhat    ,shat    ,dp      , &
                    mu      ,md      ,sd      ,qd      ,ql      , &
                    dsubcld ,jt      ,mx      ,il1g    ,il2g    , &
                    cp      ,rl      ,msg     ,          &
                    dl      ,evp     ,cu      )


   implicit none

!----------------------------------------------------------------------- 
! 
! Purpose: 
! <Say what the routine does> 
! 
! Method: 
! <Describe the algorithm(s) used in the routine.> 
! <Also include any applicable external references.> 
! 
! Author: phil rasch dec 19 1995
! 
!-----------------------------------------------------------------------


   real(r8), intent(in) :: cp

   integer, intent(in) :: lchnk             ! chunk identifier
   integer, intent(in) :: il1g
   integer, intent(in) :: il2g
   integer, intent(in) :: msg

   real(r8), intent(in) :: q(pcols,pver)
   real(r8), intent(in) :: qs(pcols,pver)
   real(r8), intent(in) :: qu(pcols,pver)
   real(r8), intent(in) :: su(pcols,pver)
   real(r8), intent(in) :: du(pcols,pver)
   real(r8), intent(in) :: qhat(pcols,pver)
   real(r8), intent(in) :: shat(pcols,pver)
   real(r8), intent(in) :: dp(pcols,pver)
   real(r8), intent(in) :: mu(pcols,pver)
   real(r8), intent(in) :: md(pcols,pver)
   real(r8), intent(in) :: sd(pcols,pver)
   real(r8), intent(in) :: qd(pcols,pver)
   real(r8), intent(in) :: ql(pcols,pver)
   real(r8), intent(in) :: evp(pcols,pver)
   real(r8), intent(in) :: cu(pcols,pver)
   real(r8), intent(in) :: dsubcld(pcols)

   real(r8),intent(out) :: dqdt(pcols,pver),dsdt(pcols,pver)
   real(r8),intent(out) :: dl(pcols,pver)
   integer kbm
   integer ktm
   integer jt(pcols)
   integer mx(pcols)
!
! work fields:
!
   integer i
   integer k

   real(r8) emc
   real(r8) rl
!-------------------------------------------------------------------
   do k = msg + 1,pver
      do i = il1g,il2g
         dsdt(i,k) = 0.
         dqdt(i,k) = 0.
         dl(i,k) = 0.
      end do
   end do
!
! find the highest level top and bottom levels of convection
!
   ktm = pver
   kbm = pver
   do i = il1g, il2g
      ktm = min(ktm,jt(i))
      kbm = min(kbm,mx(i))
   end do

   do k = ktm,pver-1
      do i = il1g,il2g
         emc = -cu (i,k)               &         ! condensation in updraft
               +evp(i,k)                         ! evaporating rain in downdraft

         dsdt(i,k) = -rl/cp*emc &
                     + (+mu(i,k+1)* (su(i,k+1)-shat(i,k+1)) &
                        -mu(i,k)*   (su(i,k)-shat(i,k)) &
                        +md(i,k+1)* (sd(i,k+1)-shat(i,k+1)) &
                        -md(i,k)*   (sd(i,k)-shat(i,k)) &
                       )/dp(i,k)

         dqdt(i,k) = emc + &
                    (+mu(i,k+1)* (qu(i,k+1)-qhat(i,k+1)) &
                     -mu(i,k)*   (qu(i,k)-qhat(i,k)) &
                     +md(i,k+1)* (qd(i,k+1)-qhat(i,k+1)) &
                     -md(i,k)*   (qd(i,k)-qhat(i,k)) &
                    )/dp(i,k)

         dl(i,k) = du(i,k)*ql(i,k+1)

      end do
   end do

!
   do k = kbm,pver
      do i = il1g,il2g
         if (k == mx(i)) then
            dsdt(i,k) = (1./dsubcld(i))* &
                        (-mu(i,k)* (su(i,k)-shat(i,k)) &
                         -md(i,k)* (sd(i,k)-shat(i,k)) &
                        )
            dqdt(i,k) = (1./dsubcld(i))* &
                        (-mu(i,k)*(qu(i,k)-qhat(i,k)) &
                         -md(i,k)*(qd(i,k)-qhat(i,k)) &
                        )
         else if (k > mx(i)) then
            dsdt(i,k) = dsdt(i,k-1)
            dqdt(i,k) = dqdt(i,k-1)
         end if
      end do
   end do
!
   return
end subroutine q1q2_pjr

      subroutine CONVTRAN_WU( q, ncnst, mu, md, du, &
                           eu, ed, dp, dsubcld, jt, &
                           mx, ideep, il1g, il2g, &
                           delt, plonl )
!-----------------------------------------------------------------------
!       ... Convective transport of trace species
!           Note that we are still assuming that the tracers are in
!           a moist mixing ratio this will change soon
!-----------------------------------------------------------------------

      use ppgrid,  only:  pver
      implicit none

!-----------------------------------------------------------------------
!       ... Dummy arguments
!-----------------------------------------------------------------------
      integer, intent(in) :: &
             ncnst, &          ! number of passive tracers
             il1g, &           ! Gathered min lon indices over which to operate
             il2g, &           ! Gathered max lon indices over which to operate
             plonl             ! lon tile dim
      integer, dimension(plonl), intent(in) :: &
             jt, &             ! Index of cloud top for each column
             mx, &             ! Index of cloud top for each column
             ideep             ! Gathering array
      real, intent(in) :: &
             delt                 ! Time step
      real, dimension(plonl,pver), intent(in) :: &
             mu, &                ! Mass flux up
             md, &                ! Mass flux down
             du, &                ! Mass detraining from updraft
             eu, &                ! Mass entraining from updraft
             ed, &                ! Mass entraining from downdraft
             dp                   ! Delta pressure between interfaces
      real, dimension(plonl), intent(in) :: &
           dsubcld              ! Delta pressure from cloud base to sfc

      real, intent(inout) :: q(plonl,pver,ncnst)  ! Tracer array including moisture

!-----------------------------------------------------------------------
!       ... Local variables
!-----------------------------------------------------------------------
      real, parameter :: small = 1.e-36                ! A small number
      real, parameter :: mbsth = 1.e-15                ! mass flux zero threshold

      integer :: i                 ! Work index
      integer :: k                 ! Work index
      integer :: kbm               ! Highest altitude index of cloud base
      integer :: kk                ! Work index
      integer :: kkp1              ! Work index
      integer :: km1               ! Work index
      integer :: kp1               ! Work index
      integer :: ktm               ! Highest altitude index of cloud top
      integer :: m                 ! Work index

      real :: cabv                 ! Mix ratio of constituent above
      real :: cbel                 ! Mix ratio of constituent below
      real :: cdifr                ! Normalized diff between cabv and cbel
      real :: mupdudp              ! A work variable
      real :: minc                 ! A work variable
      real :: maxc                 ! A work variable
      real :: twodelt              ! A work variable
      real :: fluxin               ! A work variable
      real :: fluxout              ! A work variable
      real :: netflux              ! A work variable
      real, dimension(plonl,pver) :: &
          chat, &              ! Mix ratio in env at interfaces
          cond, &              ! Mix ratio in downdraft at interfaces
          const, &             ! Gathered tracer array
          conu, &              ! Mix ratio in updraft at interfaces
          dcondt               ! Gathered tend array

!-----------------------------------------------------------------------
!       ... Find the highest level top and bottom levels of convection
!-----------------------------------------------------------------------
      ktm = pver
      kbm = pver
      do i = il1g,il2g
         ktm = MIN( ktm,jt(i) )
         kbm = MIN( kbm,mx(i) )
      end do

!-----------------------------------------------------------------------
!       ... Loop ever each constituent
!-----------------------------------------------------------------------
      do m = 1,ncnst
!-----------------------------------------------------------------------
!       ... Gather up the constituent and set tend to zero
!-----------------------------------------------------------------------
         do k = 1,pver
            do i = il1g,il2g
               const(i,k)  = q(ideep(i),k,m)
               dcondt(i,k) = 0.
            end do
         end do

!-----------------------------------------------------------------------
!       ... From now on work only with gathered data
!-----------------------------------------------------------------------
!       ... Interpolate environment tracer values to interfaces
!-----------------------------------------------------------------------
         do k = 1,pver
            km1 = MAX( 1,k-1 )
            do i = il1g,il2g
               minc = MIN( const(i,km1),const(i,k) )
               maxc = MAX( const(i,km1),const(i,k) )
               if( minc < 0 ) then
                  cdifr = 0.
               else
                  cdifr = ABS( const(i,k) - const(i,km1) )/MAX( maxc,small )
               end if
!-----------------------------------------------------------------------
!       ... If the two layers differ significantly use a geometric averaging procedure
!-----------------------------------------------------------------------
               if( cdifr > 1.e-6 ) then
                  cabv = MAX( const(i,km1),maxc*1.e-12 )
                  cbel = MAX( const(i,k),maxc*1.e-12 )
                  chat(i,k) = LOG( cabv/cbel )/(cabv - cbel)*cabv*cbel
               else             ! Small diff, so just arithmetic mean
                  chat(i,k) = .5*(const(i,k) + const(i,km1))
               end if
!-----------------------------------------------------------------------
!       ... Provisional up and down draft values
!-----------------------------------------------------------------------
               conu(i,k) = chat(i,k)
               cond(i,k) = chat(i,k)
            end do
         end do



!-----------------------------------------------------------------------
!       ... Do levels adjacent to top and bottom
!-----------------------------------------------------------------------
         k = 2
         km1 = 1
         kk = pver
         do i = il1g,il2g
            mupdudp = mu(i,kk) + du(i,kk)*dp(i,kk)
            if( mupdudp > mbsth ) then
               conu(i,kk) = (eu(i,kk)*const(i,kk)*dp(i,kk))/mupdudp
            end if
            if( md(i,k) < -mbsth ) then
               cond(i,k) = (-ed(i,km1)*const(i,km1)*dp(i,km1))/md(i,k)
            end if
         end do

!-----------------------------------------------------------------------
!       ... Updraft from bottom to top
!-----------------------------------------------------------------------
         do kk = pver-1,1,-1
            kkp1 = kk+1
            kkp1 = MIN( real(pver), real(kkp1) )
            do i = il1g,il2g
               mupdudp = mu(i,kk) + du(i,kk)*dp(i,kk)
               if( mupdudp > mbsth ) then
                  conu(i,kk) = (mu(i,kkp1)*conu(i,kkp1) + eu(i,kk)*const(i,kk)*dp(i,kk))/mupdudp
               end if
            end do
         end do

!-----------------------------------------------------------------------
!       ... Downdraft from top to bottom
!-----------------------------------------------------------------------
         do k = 3,pver
            km1 = MAX( 1,k-1 )
            do i = il1g,il2g
               if( md(i,k) < -mbsth ) then
                  cond(i,k) = (md(i,km1)*cond(i,km1) - ed(i,km1)*const(i,km1)*dp(i,km1))/md(i,k)
               end if
            end do
         end do

         do i = il1g,il2g
            do k = jt(i),mx(i)-1
               km1 = MAX( 1,k-1 )
               kp1 = MIN( real(pver),real(k+1) )
!-----------------------------------------------------------------------
!       ... Version 3 limit fluxes outside convection to mass in appropriate layer
!           these limiters are probably only safe for positive definite quantitities
!           it assumes that mu and md already satify a courant number limit of 1
!-----------------------------------------------------------------------
               fluxin = mu(i,kp1)*conu(i,kp1)  &
                        + mu(i,k)*MIN( chat(i,k),const(i,km1) )  &
                        -(md(i,k)*cond(i,k) + md(i,kp1)*MIN( chat(i,kp1),const(i,kp1) ))
               fluxout = mu(i,k)*conu(i,k) + mu(i,kp1)*MIN( chat(i,kp1),const(i,k) ) &
                         -(md(i,kp1)*cond(i,kp1) + md(i,k)*MIN( chat(i,k),const(i,k) ))
               netflux = fluxin - fluxout
               if( ABS(netflux) >= MAX( fluxin,fluxout )*1.e-12 ) then
                  dcondt(i,k) = netflux/dp(i,k)
               end if
            end do
         end do

         do i = il1g,il2g
            k = mx(i)
            km1 = MAX( 1,k-1 )
!-----------------------------------------------------------------------
!       ... Version 3
!-----------------------------------------------------------------------
            fluxin  = mu(i,k)*MIN( chat(i,k),const(i,km1) ) - md(i,k)*cond(i,k)
            fluxout = mu(i,k)*conu(i,k) - md(i,k)*MIN( chat(i,k),const(i,k) )
            netflux = fluxin - fluxout
            if( ABS(netflux) >= MAX( fluxin,fluxout )*1.e-12 ) then
               dcondt(i,k) = netflux/dp(i,k)
            end if
         end do

!-----------------------------------------------------------------------
!       ... Update and scatter data back to full arrays
!-----------------------------------------------------------------------
         twodelt = 2.*delt
         do k = 1,pver
            do i = il1g,il2g
               q(ideep(i),k,m) = const(i,k) + dcondt(i,k)*twodelt
            end do
         end do
      end do

      end subroutine CONVTRAN_WU


end module wu_conv

