#include <misc.h>
#include <preproc.h>

#if (defined PHENO2)
module CanopyFluxes2Mod

!------------------------------------------------------------------------------
!BOP
!
! !MODULE: CanopyFluxesMod
!
! !DESCRIPTION:
! Calculates the leaf temperature and the leaf fluxes,
! transpiration, photosynthesis and  updates the dew
! accumulation due to evaporation.
!
! !USES:
   use abortutils,   only: endrun
   use time_manager, only : get_step_size , get_nstep, get_curr_calday, get_curr_date
!! lwp 2008-5-23 add the definition of r8
   use shr_kind_mod, only: r8 => shr_kind_r8
   use clm_varsur, only : sand3d, clay3d, pctwet
!
! !PUBLIC TYPES:
   implicit none
   save
!
! !PUBLIC MEMBER FUNCTIONS:
   public :: CanopyFluxes !Calculates the leaf temperature and leaf fluxes
!
! !PRIVATE MEMBER FUNCTIONS:
   private :: Stomata     !Leaf stomatal resistance and leaf photosynthesis
   private :: NPPRmRg     !AVIM conductance-photosyn,zhangyw 20071218 
   private :: Allocate    !AVIM Allocation of photosynthesis assimilated Co2 
   private :: Phenology   !AVIM Phenology of leaf (max growth, normal growth, leaf fall, no leaf) 
!  private :: Turnover    !AVIM turnover of stem and root
   private :: SoilCarbon 
!
! !REVISION HISTORY:
! Created by Mariana Vertenstein
!
!EOP
!------------------------------------------------------------------------------

contains

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: CanopyFluxes
!
! !INTERFACE:
  subroutine CanopyFluxes(lbg, ubg, lbc, ubc, lbp, ubp,      &
                          num_nolakep, filter_nolakep, pco2, &
                          eccen, obliqr, lambm0, mvelpp)
!
! !DESCRIPTION:
! 1. Calculates the leaf temperature:
! 2. Calculates the leaf fluxes, transpiration, photosynthesis and
!    updates the dew accumulation due to evaporation.
!
! Method:
! Use the Newton-Raphson iteration to solve for the foliage
! temperature that balances the surface energy budget:
!
! f(t_veg) = Net radiation - Sensible - Latent = 0
! f(t_veg) + d(f)/d(t_veg) * dt_veg = 0     (*)
!
! Note:
! (1) In solving for t_veg, t_grnd is given from the previous timestep.
! (2) The partial derivatives of aerodynamical resistances, which cannot
!     be determined analytically, are ignored for d(H)/dT and d(LE)/dT
! (3) The weighted stomatal resistance of sunlit and shaded foliage is used
! (4) Canopy air temperature and humidity are derived from => Hc + Hg = Ha
!                                                          => Ec + Eg = Ea
! (5) Energy loss is due to: numerical truncation of energy budget equation
!     (*); and "ecidif" (see the code) which is dropped into the sensible
!     heat
! (6) The convergence criteria: the difference, del = t_veg(n+1)-t_veg(n)
!     and del2 = t_veg(n)-t_veg(n-1) less than 0.01 K, and the difference
!     of water flux from the leaf between the iteration step (n+1) and (n)
!     less than 0.1 W/m2; or the iterative steps over 40.
!
! !USES:
    use shr_kind_mod       , only: r8 => shr_kind_r8
    use clmtype
    use time_manager       , only : get_step_size
    use clm_varpar         , only : nlevsoi, nlevsno
    use clm_varcon         , only : sb, cpair, hvap, vkc, grav, denice, &
                                    denh2o, tfrz, csoilc, dzsoi, zsoi
    use QSatMod            , only : QSat
    use FrictionVelocityMod, only : FrictionVelocity, MoninObukIni
    use clm_varctl         , only : finidat
!   use spmdMod, only: masterproc
#ifdef SPMD
  use spmdMod, only : masterproc, mpicom, MPI_REAL8, MPI_INTEGER, MPI_LOGICAL
#else
  use spmdMod, only : masterproc
#endif

    use clm_varsur, only : sand3d, clay3d, pctwet
    use shr_orb_mod
!
! !ARGUMENTS:
    implicit none
    integer, intent(in) :: lbg, ubg              ! gridcell bounds
    integer, intent(in) :: lbc, ubc              ! column bounds
    integer, intent(in) :: lbp, ubp              ! pft bounds
    integer, intent(in) :: num_nolakep           ! number of column non-lake points in pft filter
    integer, intent(in) :: filter_nolakep(ubp-lbp+1)   ! pft filter for non-lake points
!! liwp 2015-02-26 added for Fire module
!   integer, intent(in) :: num_nolakec           ! number of column non-lake points 
!   integer, intent(in) :: filter_nolakec(ubc-lbc+1)   ! filter for non-lake points at column level
!   integer, intent(in) :: num_nowetlandp        ! number of column non-wetland points in pft filter
!   integer, intent(in) :: filter_nowetlandp(ubp-lbp+1)   ! pft filter for non-wetland points
    real(r8), intent(in) :: eccen    ! Earth's orbital eccentricity
    real(r8), intent(in) :: obliqr   ! Earth's obliquity in radians
    real(r8), intent(in) :: lambm0   ! Mean longitude of perihelion at the vernal equinox (radians)
    real(r8), intent(in) :: mvelpp   ! Earth's moving vernal equinox long. of perihelion
    real(r8), intent(inout) :: pco2                         ! partial pressure co2 (mol/mol)

!
! !CALLED FROM:
! subroutine Biogeophysics1 in module Biogeophysics1Mod
!
! !REVISION HISTORY:
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! 12/19/01, Peter Thornton
! Changed tg to t_grnd for consistency with other routines
! 1/29/02, Peter Thornton
! Migrate to new data structures, new calling protocol. For now co2 and
! o2 partial pressures are hardwired, but they should be coming in from
! forc_co2 and forc_o2. Keeping the same hardwired values as in CLM2 to
! assure bit-for-bit results in the first comparisons.
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
   integer , pointer :: ixy(:)             ! xy lon index (column-level)
   integer , pointer :: jxy(:)             ! xy lat index (column_level)

   integer , pointer :: ivt(:)         ! pft vegetation type
   integer , pointer :: pcolumn(:)     ! pft's column index
   integer , pointer :: plandunit(:)   ! pft's landunit index
   integer , pointer :: pgridcell(:)   ! pft's gridcell index
   real(r8), pointer :: wtcol(:)       ! pft weight relative to column
   real(r8), pointer :: forc_th(:)     ! atmospheric potential temperature (Kelvin)
   real(r8), pointer :: t_grnd(:)      ! ground surface temperature [K]
   real(r8), pointer :: thm(:)         ! intermediate variable (forc_t+0.0098*forc_hgt_t)
   real(r8), pointer :: qg(:)          ! specific humidity at ground surface [kg/kg]
   real(r8), pointer :: thv(:)         ! virtual potential temperature (kelvin)
   real(r8), pointer :: z0mv(:)        ! roughness length over vegetation, momentum [m]
   real(r8), pointer :: z0hv(:)        ! roughness length over vegetation, sensible heat [m]
   real(r8), pointer :: z0qv(:)        ! roughness length over vegetation, latent heat [m]
   real(r8), pointer :: z0mg(:)        ! roughness length of ground, momentum [m]
   real(r8), pointer :: dqgdT(:)       ! temperature derivative of "qg"
   real(r8), pointer :: htvp(:)        ! latent heat of evaporation (/sublimation) [J/kg]
   real(r8), pointer :: emv(:)         ! vegetation emissivity
   real(r8), pointer :: emg(:)         ! ground emissivity 
   real(r8), pointer :: forc_pbot(:)   ! atmospheric pressure (Pa)
   real(r8), pointer :: forc_q(:)      ! atmospheric specific humidity (kg/kg)
   real(r8), pointer :: forc_u(:)      ! atmospheric wind speed in east direction (m/s)
   real(r8), pointer :: forc_v(:)      ! atmospheric wind speed in north direction (m/s)
   real(r8), pointer :: forc_hgt_u(:)  ! observational height of wind [m]
   real(r8), pointer :: forc_rho(:)    ! density (kg/m**3)
   real(r8), pointer :: forc_co2(:)    ! atmospheric CO2 concentration (Pa)
   real(r8), pointer :: forc_o2(:)     ! atmospheric O2 concentration (vmr)
   real(r8), pointer :: londeg(:)      ! longitude in degrees 
   real(r8), pointer :: latdeg(:)      ! latitude in degrees 
   real(r8), pointer :: lat(:)         ! gridcell latitude (radians)
   real(r8), pointer :: lon(:)         ! gridcell longitude (radians)
! lwp 2008-6-6 add solar forcing
   real(r8), pointer :: forc_solad(:,:)  ! direct beam radiation (W/m**2)
   real(r8), pointer :: forc_solai(:,:)  ! diffuse radiation (W/m**2)

   real(r8), pointer :: forc_lwrad(:)  ! downward infrared (longwave) radiation (W/m**2)
   real(r8), pointer :: displa(:)      ! displacement height (m)
   real(r8), pointer :: parsun(:)      ! average absorbed PAR for sunlit leaves (W/m**2)
   real(r8), pointer :: parsha(:)      ! average absorbed PAR for shaded leaves (W/m**2)
   real(r8), pointer :: elai(:)        ! one-sided leaf area index with burying by snow
   real(r8), pointer :: esai(:)        ! one-sided stem area index with burying by snow
   real(r8), pointer :: fdry(:)        ! fraction of foliage that is green and dry [-]
   real(r8), pointer :: fwet(:)        ! fraction of canopy that is wet (0 to 1)
   real(r8), pointer :: laisun(:)      ! sunlit leaf area
   real(r8), pointer :: laisha(:)      ! shaded leaf area
   integer , pointer :: frac_veg_nosno(:)  ! frac of veg not covered by snow (0 OR 1 now) [-]
!! lwp 2011-01-18 add wetland points to increase soil carbon storage
!  logical , pointer :: wetpoi(:)         ! true => landunit is a wetland point
   real(r8), pointer :: sabv(:)        ! solar radiation absorbed by vegetation (W/m**2)
   real(r8), pointer :: watsat(:,:)    ! volumetric soil water at saturation (porosity)
   real(r8), pointer :: h2osoi_ice(:,:)! ice lens (kg/m2)
   real(r8), pointer :: h2osoi_liq(:,:)! liquid water (kg/m2)
   real(r8), pointer :: h2osoi_vol(:,:) ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
   real(r8), pointer :: dz(:,:)        ! layer depth (m)
   real(r8), pointer :: t_soisno(:,:)  ! soil temperature (Kelvin)
   real(r8), pointer :: sucsat(:,:)    ! minimum soil suction (mm)
   real(r8), pointer :: bsw(:,:)       ! Clapp and Hornberger "b"
   real(r8), pointer :: rootfr(:,:)    ! fraction of roots in each soil layer
   real(r8), pointer :: smpmax(:)      ! wilting point potential in mm
   real(r8), pointer :: smpso(:)       ! soil water potential at full stomatal opening (mm)
   real(r8), pointer :: smpsc(:)       ! soil water potential at full stomatal closure (mm)
   real(r8), pointer :: dleaf(:)       ! characteristic leaf dimension (m)
!
! local pointers to implicit inout arguments
!
   real(r8), pointer :: cgrnds(:)      ! deriv. of soil sensible heat flux wrt soil temp [w/m2/k]
   real(r8), pointer :: cgrndl(:)      ! deriv. of soil latent heat flux wrt soil temp [w/m**2/k]
   real(r8), pointer :: t_veg(:)       ! vegetation temperature (Kelvin)
!zhangyw
   real(r8), pointer :: t_zyw(:)       ! vegetation temperature (Kelvin)
   real(r8), pointer :: t_ref2m(:)     ! 2 m height surface air temperature (Kelvin)
   real(r8), pointer :: q_ref2m(:)     ! 2 m height surface specific humidity (kg/kg)
   real(r8), pointer :: h2ocan(:)      ! canopy water (mm H2O)
#ifdef DGVM
   real(r8), pointer :: annpsnpot(:)   ! annual potential photosynthesis (umol CO2 /m**2)
   real(r8), pointer :: annpsn(:)      ! annual photosynthesis (umol CO2 /m**2)
#else
   real(r8) :: annpsnpot(lbp:ubp)      ! temporary, set to zer0
   real(r8) :: annpsn(lbp:ubp)         ! temporary, set to zer0
#endif
!
! local pointers to implicit out arguments
!
   real(r8), pointer :: cgrnd(:)           ! deriv. of soil energy flux wrt to soil temp [w/m2/k]
   real(r8), pointer :: dlrad(:)           ! downward longwave radiation below the canopy [W/m2]
   real(r8), pointer :: ulrad(:)           ! upward longwave radiation above the canopy [W/m2]
   real(r8), pointer :: ram1(:)            ! aerodynamical resistance (s/m)
   real(r8), pointer :: btran(:)           ! transpiration wetness factor (0 to 1)
!  real(r8), pointer :: btran2(:)          ! root zone soil wetness for fire module (0 to 1)
   real(r8), pointer :: rssun(:)           ! sunlit stomatal resistance (s/m)
   real(r8), pointer :: rssha(:)           ! shaded stomatal resistance (s/m)
   real(r8), pointer :: psnsun(:)          ! sunlit leaf photosynthesis (umol CO2 /m**2/ s)
   real(r8), pointer :: psnsha(:)          ! shaded leaf photosynthesis (umol CO2 /m**2/ s)
   real(r8), pointer :: rf(:)              ! leaf photosynthesis (umol CO2 /m**2/ s)  !zhangyw
   real(r8), pointer :: qflx_tran_veg(:)   ! vegetation transpiration (mm H2O/s) (+ = to atm)
   real(r8), pointer :: dt_veg(:)          ! change in t_veg, last iteration (Kelvin)
   real(r8), pointer :: qflx_evap_veg(:)   ! vegetation evaporation (mm H2O/s) (+ = to atm)
   real(r8), pointer :: eflx_sh_veg(:)     ! sensible heat flux from leaves (W/m**2) [+ to atm]
   real(r8), pointer :: taux(:)            ! wind (shear) stress: e-w (kg/m/s**2)
   real(r8), pointer :: tauy(:)            ! wind (shear) stress: n-s (kg/m/s**2)
   real(r8), pointer :: eflx_sh_grnd(:)    ! sensible heat flux from ground (W/m**2) [+ to atm]
   real(r8), pointer :: qflx_evap_soi(:)   ! soil evaporation (mm H2O/s) (+ = to atm)
   real(r8), pointer :: fpsn(:)            ! photosynthesis (umol CO2 /m**2 /s)
   real(r8), pointer :: rootr(:,:)         ! effective fraction of roots in each soil layer
   real(r8), pointer :: rresis(:,:)        ! root resistance by layer (0-1)  (nlevsoi)
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   real(r8), pointer :: rb1(:)             ! boundary layer resistance (s/m)
#endif
   real(r8), pointer :: phd(:)             ! !zhangyw
   real(r8), pointer :: npd(:)             ! !zhangyw
   real(r8), pointer :: tmin(:)            ! liwp minimum temperature for vegetation photosynthesis
   real(r8), pointer :: topt(:)            ! liwp optimal temperature for vegetation photosynthesis
   real(r8), pointer :: tmax(:)            ! liwp maximum temperature for vegetation photosynthesis
   real(r8), pointer :: leafstatus(:)    ! (1:max growth,2:normal,3:leaffall,4:no leaf)
   real(r8), pointer :: normaldays(:)    !  days of leafstatus=2 (normal growth state) 
   real(r8), pointer :: maxdays(:)       !  days of leafstatus=1 (maximum growth state) 
   real(r8), pointer :: maturedays(:)    !  days when crop LAI >= a threshold (say 5.5)
   real(r8), pointer :: pandays(:)         !  positive Gn days (GPP-Rml) 
   real(r8), pointer :: colddays(:)        !  cold days 
   real(r8), pointer :: dailytair(:)  ! daily mean vegetation temperature for phenology
!  real(r8), pointer :: dailytveg(:)  ! daily mean vegetation temperature for phenology
   real(r8), pointer :: dailytroot(:) ! daily mean root temperature for phenology
   real(r8), pointer :: gnd(:)
   real(r8), pointer :: rm1d(:)            ! daily mean maintenance respiration of leaf
   real(r8), pointer :: rm2d(:)            ! daily mean maintenance respiration of stem
   real(r8), pointer :: rm3d(:)            ! daily mean maintenance respiration of root

   real(r8), pointer :: nep(:)             !  lwp net earth primary productivity
   real(r8), pointer :: dco2(:)            !  lwp soil respiration of CO2
   real(r8), pointer :: dsmc(:)            !  lwp total soil Carbon
   real(r8), pointer :: nmin(:)            !  lwp total soil Nitrogen
   real(r8), pointer :: polc1(:)           !  lwp carbon pool 1
   real(r8), pointer :: polc2(:)           !  lwp carbon pool 2
   real(r8), pointer :: polc3(:)           !  lwp carbon pool 3
   real(r8), pointer :: polc4(:)           !  lwp carbon pool 4
   real(r8), pointer :: polc5(:)           !  lwp carbon pool 5
   real(r8), pointer :: polc6(:)           !  lwp carbon pool 6
   real(r8), pointer :: polc7(:)           !  lwp carbon pool 7
   real(r8), pointer :: polc8(:)           !  lwp carbon pool 8
   real(r8), pointer :: alai(:)            !  !liwp
   real(r8), pointer :: vegtype(:)         !  !liwp
   real(r8), pointer :: pht(:)             ! leaf photosynthesis (umol CO2 /m**2/ s)  !zhangyw
   real(r8), pointer :: npp(:)             !! net primary production 
   real(r8), pointer :: rmp(:)             !! maintenance respiration
   real(r8), pointer :: rgp(:)             !! growth respiration
   real(r8), pointer :: dis1(:)            !! allocation of npp to leaf
   real(r8), pointer :: dis2(:)            !! allocation of npp to stem 
   real(r8), pointer :: dis3(:)            !! allocation of npp to root
   real(r8), pointer :: los1(:)            !! loss of leaf
   real(r8), pointer :: los2(:)            !! loss of stem
   real(r8), pointer :: los3(:)            !! loss of root
   real(r8), pointer :: los4(:)            !!zhangyw
   real(r8), pointer :: bm1(:)             !! leaf mass (Kg C)
   real(r8), pointer :: bm2(:)             !! stem mass (Kg C)
   real(r8), pointer :: bm3(:)             !! root mass (Kg C)
   real(r8), pointer :: bm11(:)        ! crop leaf mass when harvest begins 
   real(r8), pointer :: bm21(:)        ! crop stem mass when harvest begins 
   real(r8), pointer :: bm31(:)        ! crop root mass when harvest begins 
   real(r8), pointer :: bm4(:)             !! brown leaf mass of grass (Kg C)
   real(r8), pointer :: los21(:)           !  ! lwp
   real(r8), pointer :: los31(:)           !  ! lwp
   real(r8), pointer :: livefr_stem(:)     !!liwp
   real(r8), pointer :: livefr_root(:)     !!liwp
   real(r8), pointer :: surl(:)            !!surface carbon input(above ground litter)
   real(r8), pointer :: rotl(:)            !!root carbon input(below ground litter)
   real(r8), pointer :: ts1d(:)  !  daily 1stlayer soil T (rely on definition, top 10cm)
   real(r8), pointer :: ts2d(:)  !  daily 2ndlayer soil T (rely on definition, top170cm)
!  real(r8), pointer :: ts3d(:)  !  daily 3rdlayer soil T (rely on definition of 1st layer)
   real(r8), pointer :: sm1d(:)  ! daily 1stlayer soil mositure (depend on the definition )
   real(r8), pointer :: sm2d(:)  ! daily 2ndlayer soil mositure (depend on the definition )
   real(r8) :: gbm  (4,16)
   real(r8) :: gpolc(8,16)
!! lwp 2008-6-6 solar forcing
   real(r8) :: avmpar(lbg:ubg)
   real(r8) :: pk, fl 
!! liwp 2016-11 end of AVIM
   real(r8), pointer :: vcmxout(:)  ! lwp output actual vcmx for vegetation photosynthesis 
   real(r8), pointer :: ftout(:)    ! lwp output actual vcmx for vegetation photosynthesis 
!  real(r8), pointer :: fqout(:)    ! lwp output actual vcmx for vegetation photosynthesis 
!
! !OTHER LOCAL VARIABLES:
!
  real(r8) :: sumwt(lbc:ubc)         ! sum of weights
  real(r8) :: coszen_gcell(lbg:ubg) ! cosine solar zenith angle for next time step (gridcell level)
  real(r8) :: coszen_col(lbc:ubc)   ! cosine solar zenith angle for next time step (column level)
  real(r8) :: coszen_pft(lbp:ubp)   ! cosine solar zenith angle for next time step (pft level)

   real(r8), parameter :: btran0 = 1.0e-10  ! initial value
   real(r8), parameter :: zii = 1000.0      ! convective boundary layer height [m]
   real(r8), parameter :: beta = 1.0        ! coefficient of conective velocity [-]
   real(r8), parameter :: delmax = 1.0      ! maxchange in  leaf temperature [K]
   real(r8), parameter :: dlemin = 0.1      ! max limit for energy flux convergence [w/m2]
   real(r8), parameter :: dtmin = 0.01      ! max limit for temperature convergence [K]
   real(r8), parameter :: po2 = 0.209       ! partial pressure  o2 (mol/mol)
   integer , parameter :: itmax = 40        ! maximum number of iteration [-]
   integer , parameter :: itmin = 2         ! minimum number of iteration [-]
    real(r8) :: delta                      ! solar declination angle in radians
    real(r8) :: eccf                       ! earth orbit eccentricity factor
    real(r8) :: calday                     ! calendar day of current time step
   real(r8) :: dtime                 ! land model time step (sec)
   real(r8) ::  dthd  ! dthd= 1/steps of one day (for dtime=1800s, dthd=1/48) 
   real(r8) :: zldis(lbp:ubp)        ! reference height "minus" zero displacement height [m]
   real(r8) :: zeta                  ! dimensionless height used in Monin-Obukhov theory
   real(r8) :: wc                    ! convective velocity [m/s]
   real(r8) :: dth(lbp:ubp)          ! diff of virtual temp. between ref. height and surface
   real(r8) :: dthv                  ! diff of vir. poten. temp. between ref. height and surface
   real(r8) :: dqh(lbp:ubp)          ! diff of humidity between ref. height and surface
   real(r8) :: obu(lbp:ubp)          ! Monin-Obukhov length (m)
   real(r8) :: um(lbp:ubp)           ! wind speed including the stablity effect [m/s]
   real(r8) :: ur(lbp:ubp)           ! wind speed at reference height [m/s]
   real(r8) :: uaf                   ! velocity of air within foliage [m/s]
   real(r8) :: temp1(lbp:ubp)        ! relation for potential temperature profile
   real(r8) :: temp12m(lbp:ubp)      ! relation for potential temperature profile applied at 2-m
   real(r8) :: temp2(lbp:ubp)        ! relation for specific humidity profile
   real(r8) :: temp22m(lbp:ubp)      ! relation for specific humidity profile applied at 2-m
   real(r8) :: ustar(lbp:ubp)        ! friction velocity [m/s]
   real(r8) :: tstar                 ! temperature scaling parameter
   real(r8) :: qstar                 ! moisture scaling parameter
   real(r8) :: thvstar               ! virtual potential temperature scaling parameter
   real(r8) :: taf(lbp:ubp)          ! air temperature within canopy space [K]
   real(r8) :: qaf(lbp:ubp)          ! humidity of canopy air [kg/kg]
   real(r8) :: rpp                   ! fraction of potential evaporation from leaf [-]
   real(r8) :: rppdry                ! fraction of potential evaporation through transp [-]
   real(r8) :: cf                    ! heat transfer coefficient from leaves [-]
   real(r8) :: rb(lbp:ubp)           ! leaf boundary layer resistance [s/m]
   real(r8) :: rah(lbp:ubp,2)        ! thermal resistance [s/m]
   real(r8) :: raw(lbp:ubp,2)        ! moisture resistance [s/m]
   real(r8) :: wta                   ! heat conductance for air [m/s]
   real(r8) :: wtg(lbp:ubp)          ! heat conductance for ground [m/s]
   real(r8) :: wtl                   ! heat conductance for leaf [m/s]
   real(r8) :: wta0(lbp:ubp)         ! normalized heat conductance for air [-]
   real(r8) :: wtl0(lbp:ubp)         ! normalized heat conductance for leaf [-]
   real(r8) :: wtg0                  ! normalized heat conductance for ground [-]
   real(r8) :: wtal(lbp:ubp)         ! normalized heat conductance for air and leaf [-]
   real(r8) :: wtga                  ! normalized heat cond. for air and ground  [-]
   real(r8) :: wtaq                  ! latent heat conductance for air [m/s]
   real(r8) :: wtlq                  ! latent heat conductance for leaf [m/s]
   real(r8) :: wtgq(lbp:ubp)         ! latent heat conductance for ground [m/s]
   real(r8) :: wtaq0(lbp:ubp)        ! normalized latent heat conductance for air [-]
   real(r8) :: wtlq0(lbp:ubp)        ! normalized latent heat conductance for leaf [-]
   real(r8) :: wtgq0                 ! normalized heat conductance for ground [-]
   real(r8) :: wtalq(lbp:ubp)        ! normalized latent heat cond. for air and leaf [-]
   real(r8) :: wtgaq                 ! normalized latent heat cond. for air and ground [-]
   real(r8) :: el(lbp:ubp)           ! vapor pressure on leaf surface [pa]
   real(r8) :: deldT                 ! derivative of "el" on "t_veg" [pa/K]
   real(r8) :: qsatl(lbp:ubp)        ! leaf specific humidity [kg/kg]
   real(r8) :: qsatldT(lbp:ubp)      ! derivative of "qsatl" on "t_veg"
   real(r8) :: air(lbp:ubp),bir(lbp:ubp),cir(lbp:ubp)  ! atmos. radiation temporay set
   real(r8) :: dc1,dc2               ! derivative of energy flux [W/m2/K]
   real(r8) :: delt                  ! temporary
   real(r8) :: delq                  ! temporary
   real(r8) :: del(lbp:ubp)          ! absolute change in leaf temp in current iteration [K]
   real(r8) :: del2(lbp:ubp)         ! change in leaf temperature in previous iteration [K]
   real(r8) :: dele(lbp:ubp)         ! change in latent heat flux from leaf [K]
   real(r8) :: dels                  ! change in leaf temperature in current iteration [K]
   real(r8) :: det(lbp:ubp)          ! maximum leaf temp. change in two consecutive iter [K]
   real(r8) :: efeb(lbp:ubp)         ! latent heat flux from leaf (previous iter) [mm/s]
   real(r8) :: efeold                ! latent heat flux from leaf (previous iter) [mm/s]
   real(r8) :: efpot                 ! potential latent energy flux [kg/m2/s]
   real(r8) :: efe(lbp:ubp)          ! water flux from leaf [mm/s]
   real(r8) :: efsh                  ! sensible heat from leaf [mm/s]
   real(r8) :: obuold(lbp:ubp)       ! monin-obukhov length from previous iteration
   real(r8) :: tlbef(lbp:ubp)        ! leaf temperature from previous iteration [K]
!! liwp 2015-03-24 add daily mean baf (burned area fraction)
!  real(r8) :: daybaf(lbp:ubp)       ! temporary variable for daily mean baf
!  real(r8) :: daytair(lbp:ubp)      ! temporary variable for daily mean air temp 
!  real(r8) :: daytveg(lbp:ubp)      ! temporary variable for daily mean veg temp 
!  real(r8) :: daytroot(lbp:ubp)     ! temporary variable for daily mean root temp 
   real(r8) :: ecidif                ! excess energies [W/m2]
   real(r8) :: err(lbp:ubp)          ! balance error
   real(r8) :: erre                  ! balance error
   real(r8) :: co2(lbp:ubp)          ! atmospheric co2 concentration (pa)
   real(r8) :: o2(lbp:ubp)           ! atmospheric o2 concentration (pa)
   real(r8) :: svpts(lbp:ubp)        ! saturation vapor pressure at t_veg (pa)
   real(r8) :: eah(lbp:ubp)          ! canopy air vapor pressure (pa)
   real(r8) :: s_node                ! vol_liq/eff_porosity
   real(r8) :: smp_node              ! matrix potential
   real(r8) :: smp_node2             ! matrix potential revised for fire module
   real(r8) :: vol_ice               ! partial volume of ice lens in layer
   real(r8) :: eff_porosity          ! effective porosity in layer
   real(r8) :: vol_liq               ! partial volume of liquid water in layer
   real(r8) :: rresis_clm3           ! soil water contribution to root resistance
   integer  :: itlef                 ! counter for leaf temperature iteration [-]
   integer  :: nmozsgn(lbp:ubp)      ! number of times stability changes sign
   real(r8) :: w                     ! exp(-LSAI)
   real(r8) :: csoilcn               ! interpolated csoilc for less than dense canopies
   real(r8) :: fm(lbp:ubp)           ! needed for BGC only to diagnose 10m wind speed
   real(r8) :: wtshi                 ! sensible heat resistance for air, grnd and leaf [-]
   real(r8) :: wtsqi                 ! latent heat resistance for air, grnd and leaf [-]
   integer  :: j                     ! soil/snow level index
   integer  :: p                     ! pft index
   integer  :: c                     ! column index
   integer  :: l                     ! landunit index
   integer  :: g                     ! gridcell index
   integer  :: iv                    ! vegetation type (1 ~15)
   integer  :: fp                    ! lake filter pft index
   integer  :: fn                    ! number of values in pft filter
   integer  :: fnorig                ! number of values in pft filter copy
   integer  :: fnold                 ! temporary copy of pft count
   integer  :: f                     ! filter index
   integer  :: filterp(ubp-lbp+1)
   integer  :: fporig(ubp-lbp+1)
!! zhangyw 2015-05-20
   real(r8) :: qsatg(lbc:ubc)          ! yanwu  saturated humidity [kg/kg]
   real(r8) :: qsatgdT(lbc:ubc)        ! d(qsatg)/dT
   real(r8) :: eg                      ! water vapor pressure at temperature T [pa]
   real(r8) :: degdT                   ! yanwu d(eg)/dT

!! liwp 2015-02-27
   integer  :: fcn                   ! number of values in pft filter at column level
   integer  :: filterc(ubc-lbc+1)
   integer  :: pi                    ! index for pft numbering

  integer  :: nstep         ! time step number
  integer  :: nstep_day  !! step numbers per day 
  integer  :: year_number !! numbers of model year 
  integer  :: ix, jy 
!! lwp 2011-01-18 add wetland points to increase soil carbon storage
  integer  :: ii, jj 
!! Zhang Yanwu 2010-08-13 add CO2 ppmv from 1948 to 2008
  real  :: pp_co2(1948:2020)
  integer :: year,mon,day,sec
!------------------------------------------------------------------------------

   ! Assign local pointers to derived type members (gridcell-level)
!! lwp 2008-6-6 add solar forcing
    forc_solad    => clm3%g%a2lf%forc_solad
    forc_solai    => clm3%g%a2lf%forc_solai
   forc_lwrad     => clm3%g%a2lf%forc_lwrad
   forc_q         => clm3%g%a2ls%forc_q
   forc_pbot      => clm3%g%a2ls%forc_pbot
   forc_u         => clm3%g%a2ls%forc_u
   forc_v         => clm3%g%a2ls%forc_v
   forc_th        => clm3%g%a2ls%forc_th
   forc_hgt_u     => clm3%g%a2ls%forc_hgt_u
   forc_rho       => clm3%g%a2ls%forc_rho
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
   rb1            => clm3%g%l%c%p%pps%rb1
#endif

   forc_co2       => clm3%g%a2ls%forc_co2
   forc_o2        => clm3%g%a2ls%forc_o2

   ! Assign local pointers to derived type members (column-level)
   t_soisno       => clm3%g%l%c%ces%t_soisno
   watsat         => clm3%g%l%c%cps%watsat
   h2osoi_ice     => clm3%g%l%c%cws%h2osoi_ice
   dz             => clm3%g%l%c%cps%dz
   h2osoi_liq     => clm3%g%l%c%cws%h2osoi_liq
   sucsat         => clm3%g%l%c%cps%sucsat
   bsw            => clm3%g%l%c%cps%bsw
   emg            => clm3%g%l%c%cps%emg
   t_grnd         => clm3%g%l%c%ces%t_grnd
   thm            => clm3%g%l%c%ces%thm
   qg             => clm3%g%l%c%cws%qg
   thv            => clm3%g%l%c%ces%thv
   dqgdT          => clm3%g%l%c%cws%dqgdT
   htvp           => clm3%g%l%c%cps%htvp
   z0mg           => clm3%g%l%c%cps%z0mg
  ixy             => clm3%g%l%c%ixy
  jxy             => clm3%g%l%c%jxy

   ! Assign local pointers to derived type members (pft-level)

!! liwp 2015-02-27
   ivt            => clm3%g%l%c%p%itype
   wtcol          => clm3%g%l%c%p%wtcol
   pcolumn        => clm3%g%l%c%p%column
   plandunit      => clm3%g%l%c%p%landunit
   pgridcell      => clm3%g%l%c%p%gridcell
   frac_veg_nosno => clm3%g%l%c%p%pps%frac_veg_nosno
   btran          => clm3%g%l%c%p%pps%btran
   rootr          => clm3%g%l%c%p%pps%rootr
   rootfr         => clm3%g%l%c%p%pps%rootfr
   rresis         => clm3%g%l%c%p%pps%rresis
   emv            => clm3%g%l%c%p%pps%emv
   t_veg          => clm3%g%l%c%p%pes%t_veg
   displa         => clm3%g%l%c%p%pps%displa
   z0mv           => clm3%g%l%c%p%pps%z0mv
   z0hv           => clm3%g%l%c%p%pps%z0hv
   z0qv           => clm3%g%l%c%p%pps%z0qv
   ram1           => clm3%g%l%c%p%pps%ram1
   parsun         => clm3%g%l%c%p%pef%parsun
   rssun          => clm3%g%l%c%p%pps%rssun
   psnsun         => clm3%g%l%c%p%pcf%psnsun
   parsha         => clm3%g%l%c%p%pef%parsha
   rssha          => clm3%g%l%c%p%pps%rssha
   psnsha         => clm3%g%l%c%p%pcf%psnsha
   elai           => clm3%g%l%c%p%pps%elai
   esai           => clm3%g%l%c%p%pps%esai
   fdry           => clm3%g%l%c%p%pps%fdry
   laisun         => clm3%g%l%c%p%pps%laisun
   laisha         => clm3%g%l%c%p%pps%laisha
   qflx_tran_veg  => clm3%g%l%c%p%pwf%qflx_tran_veg
   fwet           => clm3%g%l%c%p%pps%fwet
   h2ocan         => clm3%g%l%c%p%pws%h2ocan
   dt_veg         => clm3%g%l%c%p%pps%dt_veg
   sabv           => clm3%g%l%c%p%pef%sabv
   qflx_evap_veg  => clm3%g%l%c%p%pwf%qflx_evap_veg
   eflx_sh_veg    => clm3%g%l%c%p%pef%eflx_sh_veg
   taux           => clm3%g%l%c%p%pmf%taux
   tauy           => clm3%g%l%c%p%pmf%tauy
   eflx_sh_grnd   => clm3%g%l%c%p%pef%eflx_sh_grnd
   qflx_evap_soi  => clm3%g%l%c%p%pwf%qflx_evap_soi
   t_ref2m        => clm3%g%l%c%p%pes%t_ref2m
   q_ref2m        => clm3%g%l%c%p%pes%q_ref2m
   dlrad          => clm3%g%l%c%p%pef%dlrad
   ulrad          => clm3%g%l%c%p%pef%ulrad
   cgrnds         => clm3%g%l%c%p%pef%cgrnds
   cgrndl         => clm3%g%l%c%p%pef%cgrndl
   cgrnd          => clm3%g%l%c%p%pef%cgrnd
   fpsn           => clm3%g%l%c%p%pcf%fpsn
#ifdef DGVM
   annpsnpot      => clm3%g%l%c%p%pdgvs%annpsnpot
   annpsn         => clm3%g%l%c%p%pdgvs%annpsn
#endif
   londeg         => clm3%g%londeg
   latdeg         => clm3%g%latdeg
   h2osoi_vol     => clm3%g%l%c%cws%h2osoi_vol
    lat       => clm3%g%lat
    lon       => clm3%g%lon
!zhangyw
   phd            => clm3%g%l%c%p%pcf%phd
   npd            => clm3%g%l%c%p%pcf%npd
   gnd            => clm3%g%l%c%p%pcf%gnd
   rm1d           => clm3%g%l%c%p%pcf%rm1d
   rm2d           => clm3%g%l%c%p%pcf%rm2d
   rm3d           => clm3%g%l%c%p%pcf%rm3d
   leafstatus     => clm3%g%l%c%p%pcf%leafstatus
   normaldays     => clm3%g%l%c%p%pcf%normaldays
   maturedays     => clm3%g%l%c%p%pcf%maturedays
   maxdays        => clm3%g%l%c%p%pcf%maxdays
   pandays        => clm3%g%l%c%p%pcf%pandays
   colddays       => clm3%g%l%c%p%pcf%colddays
   dailytair      => clm3%g%l%c%p%pcf%dailytair
!  dailytveg      => clm3%g%l%c%p%pcf%dailytveg
   dailytroot     => clm3%g%l%c%p%pcf%dailytroot
   nep            => clm3%g%l%c%p%pcf%nep
   dsmc           => clm3%g%l%c%p%pcf%dsmc
   dco2           => clm3%g%l%c%p%pcf%dco2
   nmin           => clm3%g%l%c%p%pcf%nmin
   ts1d           => clm3%g%l%c%p%pcf%ts1d
   ts2d           => clm3%g%l%c%p%pcf%ts2d
   sm1d           => clm3%g%l%c%p%pcf%sm1d
   sm2d           => clm3%g%l%c%p%pcf%sm2d

   alai           => clm3%g%l%c%p%pcf%alai
   vegtype        => clm3%g%l%c%p%pcf%vegtype
   pht            => clm3%g%l%c%p%pcf%pht
   npp            => clm3%g%l%c%p%pcf%npp
   rmp            => clm3%g%l%c%p%pcf%rmp
   rgp            => clm3%g%l%c%p%pcf%rgp
   dis1           => clm3%g%l%c%p%pcf%dis1
   dis2           => clm3%g%l%c%p%pcf%dis2
   dis3           => clm3%g%l%c%p%pcf%dis3
   los1           => clm3%g%l%c%p%pcf%los1
   los2           => clm3%g%l%c%p%pcf%los2
   los3           => clm3%g%l%c%p%pcf%los3
   los4           => clm3%g%l%c%p%pcf%los4

    bm1           => clm3%g%l%c%p%pcf%bm1 
    bm2           => clm3%g%l%c%p%pcf%bm2 
    bm3           => clm3%g%l%c%p%pcf%bm3 
    bm4           => clm3%g%l%c%p%pcf%bm4 
    bm11          => clm3%g%l%c%p%pcf%bm11
    bm21          => clm3%g%l%c%p%pcf%bm21
    bm31          => clm3%g%l%c%p%pcf%bm31
   los21          => clm3%g%l%c%p%pcf%los21
   los31          => clm3%g%l%c%p%pcf%los31
   livefr_stem    => clm3%g%l%c%p%pcf%livefr_stem
   livefr_root    => clm3%g%l%c%p%pcf%livefr_root
   surl           => clm3%g%l%c%p%pcf%surl
   rotl           => clm3%g%l%c%p%pcf%rotl
   polc1          => clm3%g%l%c%p%pcf%polc1
   polc2          => clm3%g%l%c%p%pcf%polc2
   polc3          => clm3%g%l%c%p%pcf%polc3
   polc4          => clm3%g%l%c%p%pcf%polc4
   polc5          => clm3%g%l%c%p%pcf%polc5
   polc6          => clm3%g%l%c%p%pcf%polc6
   polc7          => clm3%g%l%c%p%pcf%polc7
   polc8          => clm3%g%l%c%p%pcf%polc8

   vcmxout        => clm3%g%l%c%p%pcf%vcmxout
   ftout          => clm3%g%l%c%p%pcf%ftout

   ! Assign local pointers to derived type members (ecophysiological)

   smpso          => pftcon%smpso 
   smpsc          => pftcon%smpsc 
   smpmax         => pftcon%smpmax
   dleaf          => pftcon%dleaf

!     data glos/0.004, 0.00010 ,0.0003 ,0.00005, & ! 1 NET temperate 
!               0.003, 0.00007 ,0.00022,0.00005, & ! 2 NET boreal   
!               0.003, 0.00006 ,0.0001 ,0.00005, & ! 3 NDT boreal Needleleaf Deciduous
!               0.0015,0.00014 ,0.0005 ,0.00005, & ! 4 BET tropical Brdlf Evergreen Tree
!               0.0020,0.00010 ,0.0003 ,0.00005, & ! 5 BET temperate 
!               0.0030,0.00017 ,0.0004 ,0.00005, & ! 6 BDT tropical Brdlf Deciduous Tree
!               0.004, 0.00015 ,0.0003 ,0.00005, & ! 7 BDT temperate BrdlfDeciduous Tree
!               0.005, 0.00015 ,0.0002 ,0.00005, & ! 8 BDT boreal Brdlf Deciduous Tree 
!               0.005, 0.00020 ,0.0003 ,0.00005, & ! 9 BES temperate Brdlf EvergreenShrub
!               0.003, 0.00005 ,0.00003,0.00005, & !10 BDS temperate Brdlf DeciduousShrub
!               0.003, 0.00015 ,0.0001 ,0.00005, & !11 BDS boreal Brdlf Deciduous Shrub 
!               0.001,  0       ,0.0009 ,0.0002,   & ! type 12 C3 grass arctic 
!               0.007,  0       ,0.0010 ,0.0002,   & ! type 13 C3 grass grassland
!               0.015,  0       ,0.0018 ,0.0002,   & ! type 14 C4 grass grassland
!               0.015,  0       ,0.0015 ,0.0002,   & ! type 15 crop 1
!               0.015,  0       ,0.0015 ,0.0002    / ! type 16 crop 2 

      data  gbm/0.20,   13,  2.2,  0.0 , &  ! 1 evergreen needleleaf temperate 
                0.20,   13,  2.2,  0.0 , &  ! 2 evergreen needleleaf boreal
                0.1 ,    7,  1.2,  0.0 , &  ! 3 deciduous needleleaf boreal 
                0.45, 16.0,  2.5,  0.0 , &  ! 4 evergreen broadleaf tropical
                0.35,   13,  2.0,  0.0 , &  ! 5 evergreen broadleaf temperate
                0.20,    5,  1.5,  0.0 , &  ! 6 deciduous broadleaf tropical
                0.10,    8,  1.8,  0.0 , &  ! 7 deciduous broadleaf temperate 
                0.10,    6,  2.0,  0.0 , &  ! 8 deciduous broadleaf boreal 
                0.05,  1.0,  0.6,  0.0 , &  ! 9 broadleaf evergreen shrubland temperate
                0.05,  1.5,  0.6,  0.0 , &  !10 broadleaf deciduous shrubland temperate
                0.05,  0.4,  0.6,  0.0 , &  !11 broadleaf deciduous shrubland boreal
                0.05,  0.0,  0.3,  0.01, &  !12 C3 arctic grassland 
                0.05,  0.0,  0.3,  0.01, &  !13 C3 grassland 
                0.05,  0.0,  0.3,  0.01, &  !14 C4 grassland 
                0.005, 0.005, 0.001, 0.0 , & !! 15 crop 1(BM be small at first) 
                0.001, 0.001, 0.001, 0.0  /  !! 16 crop 2

      data  gpolc/800,1200,1000, 800, 650, 280,14500, 3500,  & ! type 1
                 1200,2000,2500, 800, 600, 700,17000, 3600,  & ! type 2
                  900,1200,1100, 600, 550, 360,14000, 3680,  & ! type 3
                  400, 420, 650, 200, 200, 200, 7500, 1450,  & ! type 4
                  700, 800, 900, 390, 450, 220, 9000, 2480,  & ! type 5
                  500, 600, 600, 500, 500, 200, 4600, 2120,  & ! type 6
                  500, 900, 500, 680, 550, 250, 5500, 2700,  & ! type 7
                  500, 600, 500, 950, 950, 180, 5500, 2600,  & ! type 8
                  900, 200, 500, 200, 200, 100, 5000, 2200,  & ! type 9
                  600, 300, 400, 200, 200, 100, 5000, 1200,  & ! type 10
                  500, 400, 600, 350, 240, 280, 9000, 3200,  & ! type 11
                  450, 300, 700, 300, 300, 300, 8400, 3000,  & ! type 12
                  300, 250, 600, 200, 300, 200, 4000, 1500,  & ! type 13
                  300, 350, 500, 300, 400, 250, 3000, 1000,  & ! type 14
                  300, 300, 400, 400, 300, 200, 3100, 1000,  & ! type 15
                  300, 200, 400, 200, 200, 100, 3000, 1000 /   ! type 16

  data pp_co2(1948:2020)/ 310.30, 310.50, 310.70,                                      & ! 48-50
       311.10, 311.50, 311.90, 312.40, 313.00, 313.60, 314.20, 315.62, 316.46, 317.32, & ! 51-60
       318.09, 318.75, 319.07, 319.68, 320.75, 321.78, 322.61, 323.89, 325.21, 326.01, & ! 61-70
       326.90, 328.65, 330.04, 330.74, 331.67, 333.09, 334.80, 336.24, 337.85, 339.36, & ! 71-80
       340.53, 342.03, 343.71, 345.23, 346.61, 348.15, 350.31, 352.23, 353.55, 354.92, & ! 81-90
       356.02, 356.72, 358.00, 359.94, 361.79, 363.22, 365.29, 367.53, 368.89, 370.28, & ! 91-00
       372.10, 374.45, 376.58, 378.61, 380.82, 382.75, 384.49, 385.33, 386.33, 387.33, & ! 01-10
       388.33, 389.33, 390.33, 391.33, 392.33, 393.33, 394.33, 395.33, 396.33, 397.33 /  ! 11-20

   ! Determine step size

   ! Zhang Yanwu 2010-08-13 add CO2 ppmv from 1948 to 2008, rather than constant from dummy variable
     call get_curr_date(year, mon, day, sec)
!    pco2 = pp_co2(year)*1.0e-6

!! lwp 2011-01-14 use constant CO2
!    pco2 = pp_co2(1949)*1.0e-6    !! use co2 concentration of 1949
!    pco2 = 355.0*1.0e-6    !! (CLM3 original, near 1990 level) 
     pco2 = 285.0*1.0e-6    !! (near 1850 level) 

      dtime = get_step_size()
      dthd = dtime/3600./24.
!! lwp 2008-8-29 to modify the step control by "dtime(second)" rather than manually fixed
       nstep_day = 24* 3600/int(dtime)
       nstep = get_nstep()
!! lwp 2009-09 added the following lines to calculate cosine of solar zenith angle
     calday = get_curr_calday(offset=0)

   ! Solar declination for current time step

     call shr_orb_decl (calday, eccen, mvelpp, lambm0, obliqr, delta, eccf)

 ! Cosine solar zenith angle for current time step

!dir$ concurrent
!cdir nodep
    do g = lbg, ubg
       coszen_gcell(g) = shr_orb_cosz (calday, lat(g), lon(g), delta)
    end do
!      year_number = int(nstep/(365*nstep_day)) + 1

   ! Filter pfts where frac_veg_nosno is non-zero

   fn = 0
   do fp = 1,num_nolakep
      p = filter_nolakep(fp)
      c = pcolumn(p)
      ii = ixy(c)
      jj = jxy(c)

!! lwp 2008-10-30 to add land points of Grass or Crop with very small LAI in winter
!! lwp 2009-10-29 not update LAI at wetland points
!     if (frac_veg_nosno(p) /= 0 .and. .not. clm3%g%l%wetpoi(p) ) then

      if (frac_veg_nosno(p) /= 0) then  
         fn = fn + 1
         filterp(fn) = p
      end if
   end do

   ! Initialize

   do f = 1, fn
      p = filterp(f)
      del(p)    = 0._r8  ! change in leaf temperature from previous iteration
      efeb(p)   = 0._r8  ! latent head flux from leaf for previous iteration
      wtlq0(p)  = 0._r8
      wtalq(p)  = 0._r8
      wtgq(p)   = 0._r8
      wtaq0(p)  = 0._r8
      obuold(p) = 0._r8
      btran(p)  = btran0
!     btran2(p) = btran0
#ifndef DGVM
      annpsnpot(p) = 0._r8
      annpsn(p)    = 0._r8
#endif
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
      rb1(p) = 0._r8
#endif

   end do

!! Effective porosity of soil, partial volume of ice and liquid (needed for btran)
   do j = 1,nlevsoi

!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         c = pcolumn(p)
         l = plandunit(p)
         g = pgridcell(p)

          iv =ivt(p)
         ! Root resistance factors
!! liwp 2015-05-19 smpmax=-1.5e5, original s_node >= 0.01 in CLM3 
!! liwp     sucsat(c,lev) = 10.*(10.**(1.88-0.0131*sand))  >=0.0
!! liwp 2015-05-19 modify this part according to CLM4
            vol_ice = min(watsat(c,j), h2osoi_ice(c,j)/(dz(c,j)*denice))
            eff_porosity = watsat(c,j)-vol_ice
            vol_liq = min(eff_porosity,h2osoi_liq(c,j)/(dz(c,j)*denh2o))
         if (vol_liq .le. 0._r8 .or. t_soisno(c,j) .le. tfrz-3._r8) then
            rootr(p,j) = 0._r8
         else
!     s_node = max(vol_liq/eff_porosity,0.01_r8)    !! CLM3 original
!! liwp 2015-05-29 increase the minimum s_node
      s_node = max(vol_liq/eff_porosity,0.01_r8)
!     smp_node = max(smpmax(iv), -sucsat(c,j)*s_node**(-bsw(c,j)))
      smp_node = max(smpsc(iv), -sucsat(c,j)*s_node**(-bsw(c,j)))
      rresis(p,j)=max(0.18,(smp_node -smpsc(iv))/(smpso(iv) -smpsc(iv)) ) 
           if (iv.eq.1 .or. iv.eq.2 .or. iv.eq.6 ) then
               rresis(p,j) = rresis(p,j)**0.5
           else if (iv.eq.3 .or. iv.eq.5 ) then
               rresis(p,j) = rresis(p,j)**0.30
           else if (iv.eq.4 ) then
               rresis(p,j) = rresis(p,j)**0.25
           else if (iv.eq.7 ) then
               rresis(p,j) = rresis(p,j)**0.5
           else if (iv.eq.10 .or. iv.ge.13 ) then
               rresis(p,j) = rresis(p,j)**0.5
           endif
      rresis(p,j)= min ( 0.999_r8, (eff_porosity/watsat(c,j))*rresis(p,j) )
      rootr(p,j) = rootfr(p,j)*rresis(p,j)
!           rresis_clm3 = (1.-smp_node/smpmax(iv))/(1.+sucsat(c,j)/smpmax(iv))
!           rootr(p,j) = rootfr(p,j)*rresis_clm3
            btran(p) = btran(p) + rootr(p,j)

         end if    !! end vol_liq or T criterion
      end do       !! end of p loop
   end do          !! end of j loop (vertical soil layers)

   ! Normalize root resistances to get layer contribution to ET

   do j = 1,nlevsoi
!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
          if (btran(p) .gt. btran0 ) then
           rootr(p,j) = rootr(p,j)/btran(p)
          else
           rootr(p,j) = 0._r8
          end if
      end do
   end do

!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      c = pcolumn(p)
      g = pgridcell(p)
!! lwp 2009-09-09 add cosine of solar zenith angle for each pft
       coszen_pft(p) = coszen_gcell(g)

!! lwp 2009-10-09 "INITIAL RUN" use the following lines, otherwise read from "initial surfdata"
             if(nstep <= 1 .and. finidat == ' ')then
                bm1(p) =gbm(1,ivt(p))
                bm2(p) =gbm(2,ivt(p))
                bm3(p) =gbm(3,ivt(p))
                bm4(p) =gbm(4,ivt(p))
!!!lwp 2007-7-26 to add the initial value of soil Carbon pool
                polc1(p) = gpolc(1,ivt(p)) 
                polc2(p) = gpolc(2,ivt(p)) 
                polc3(p) = gpolc(3,ivt(p)) 
                polc4(p) = gpolc(4,ivt(p)) 
                polc5(p) = gpolc(5,ivt(p)) 
                polc6(p) = gpolc(6,ivt(p)) 
                polc7(p) = gpolc(7,ivt(p)) 
                polc8(p) = gpolc(8,ivt(p)) 
                alai(p)  =  elai(p)
                leafstatus(p) = 2
                los1(p)  = 0.0
                los2(p)  = 0.0
                los3(p)  = 0.0
                los21(p) = 0.0
                los31(p) = 0.0
!! liwp 2017-03-25 added the following lines
                bm11(p)  = 0.0
                bm21(p)  = 0.0
                bm31(p)  = 0.0
                livefr_stem(p) = 1.0
                livefr_root(p) = 1.0
             endif

      ! Net absorbed longwave radiation by canopy and ground
      ! =air+bir*t_veg**4+cir*t_grnd(c)**4

      air(p) =   emv(p) * (1.+(1.-emv(p))*(1.-emg(c))) * forc_lwrad(g)
      bir(p) = - (2.-emv(p)*(1.-emg(c))) * emv(p) * sb
      cir(p) =   emv(p)*emg(c)*sb

      ! Saturated vapor pressure, specific humidity, and their derivatives
      ! at the leaf surface

      call QSat (t_veg(p), forc_pbot(g), el(p), deldT, qsatl(p), qsatldT(p))

      ! Determine atmospheric co2 and o2
      ! for bit-for-bit results with clm2, using the hardwired local co2 and o2

!! liwp 2014-06-17 the following two lines are for OFFLINE only, so modified for ONLINE too

#if (defined OFFLINE)
      co2(p) = pco2 * forc_pbot(g)
      o2(p)  = po2  * forc_pbot(g)
      forc_co2(g) = pco2

#elif (defined COUP_CSM)
      co2(p)  = forc_co2(g) * forc_pbot(g)            ! CO2 vmr to Pa
      o2(p)   = forc_o2(g)  * forc_pbot(g)            !  O2 vmr to Pa

#elif (defined COUP_CAM)
      co2(p)  = forc_co2(g) * forc_pbot(g)            ! CO2 vmr to Pa
      o2(p)   = forc_o2(g)  * forc_pbot(g)            !  O2 vmr to Pa

#endif

      ! Initialize flux profile

      nmozsgn(p) = 0

      taf(p) = (t_grnd(c) + thm(c))/2.
      qaf(p) = (forc_q(g)+qg(c))/2.

      ur(p) = max(1.0_r8,sqrt(forc_u(g)*forc_u(g)+forc_v(g)*forc_v(g)))
      dth(p) = thm(c)-taf(p)
      dqh(p) = forc_q(g)-qaf(p)
      dthv = dth(p)*(1.+0.61*forc_q(g))+0.61*forc_th(g)*dqh(p)
      zldis(p) = forc_hgt_u(g) - displa(p)

      ! Initialize Monin-Obukhov length and wind speed

      call MoninObukIni(ur(p), thv(c), dthv, zldis(p), z0mv(p), um(p), obu(p))

   end do

   itlef = 0    ! counter for leaf temperature iteration
   fnorig = fn
   fporig(1:fn) = filterp(1:fn)

   ! Begin stability iteration
   ITERATION : do while (itlef <= itmax .and. fn > 0)

      ! Determine friction velocity, and potential temperature and humidity
      ! profiles of the surface boundary layer

      call FrictionVelocity (lbp, ubp, fn, filterp, &
           displa(lbp:ubp), z0mv(lbp:ubp), z0hv(lbp:ubp), z0qv(lbp:ubp), &
           obu(lbp:ubp), itlef+1, ur(lbp:ubp), um(lbp:ubp), ustar(lbp:ubp), &
           temp1(lbp:ubp), temp2(lbp:ubp), temp12m(lbp:ubp), temp22m(lbp:ubp), fm(lbp:ubp))

!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         c = pcolumn(p)
         g = pgridcell(p)

         tlbef(p) = t_veg(p)
         del2(p) = del(p)

         ! Determine aerodynamic resistances

         ram1(p)  = 1./(ustar(p)*ustar(p)/um(p))
         rah(p,1) = 1./(temp1(p)*ustar(p))
         raw(p,1) = 1./(temp2(p)*ustar(p))

         ! Bulk boundary layer resistance of leaves

         uaf = um(p)*sqrt( 1./(ram1(p)*um(p)) )
         cf  = 0.01/(sqrt(uaf)*sqrt(dleaf(ivt(p))))
         rb(p)  = 1./(cf*uaf)
#if (defined BCCCHEM) || (defined MOZART2) || (defined WACCM)
         rb1(p) = rb(p)
#endif

         ! Parameterization for variation of csoilc with canopy density from
         ! X. Zeng, University of Arizona

         w = exp(-(elai(p)+esai(p)))
         csoilcn = (vkc/(0.13*(z0mg(c)*uaf/1.5e-5)**0.45))*w + csoilc*(1.-w)
         rah(p,2) = 1./(csoilcn*uaf)
         raw(p,2) = rah(p,2)

         ! Stomatal resistances for sunlit and shaded fractions of canopy.
         ! Done each iteration to account for differences in eah, tv.

         svpts(p) = el(p)                         ! pa
         eah(p) = forc_pbot(g) * qaf(p) / 0.622   ! pa
      end do

      call Stomata (fn, filterp, lbp, ubp,lbc, ubc, lbg, ubg, &
           parsun(lbp:ubp),svpts(lbp:ubp),eah(lbp:ubp),thm(lbc:ubc),forc_pbot(lbg:ubg), &
           t_veg(lbp:ubp), o2(lbp:ubp), co2(lbp:ubp), btran(lbp:ubp), rb(lbp:ubp), &
           rssun(lbp:ubp), psnsun(lbp:ubp), annpsnpot(lbp:ubp), annpsn(lbp:ubp),   &
           vcmxout(lbp:ubp),ftout(lbp:ubp) )

      call Stomata (fn, filterp, lbp, ubp,lbc, ubc, lbg, ubg, &
           parsha(lbp:ubp),svpts(lbp:ubp),eah(lbp:ubp),thm(lbc:ubc),forc_pbot(lbg:ubg), &
           t_veg(lbp:ubp), o2(lbp:ubp), co2(lbp:ubp), btran(lbp:ubp), rb(lbp:ubp), &
           rssha(lbp:ubp), psnsha(lbp:ubp), annpsnpot(lbp:ubp), annpsn(lbp:ubp),   &
           vcmxout(lbp:ubp),ftout(lbp:ubp) )

!dir$ concurrent
!cdir nodep
      do f = 1, fn
         p = filterp(f)
         c = pcolumn(p)
         g = pgridcell(p)

         ! Sensible heat conductance for air, leaf and ground
         ! Moved the original subroutine in-line...

         wta    = 1./rah(p,1)                ! air
         wtl    = (elai(p)+esai(p))/rb(p)    ! leaf
         wtg(p) = 1./rah(p,2)                ! ground
         wtshi  = 1./(wta+wtl+wtg(p))

         wtl0(p) = wtl*wtshi         ! leaf
         wtg0    = wtg(p)*wtshi      ! ground
         wta0(p) = wta*wtshi         ! air

         wtga    = wta0(p)+wtg0      ! ground + air
         wtal(p) = wta0(p)+wtl0(p)   ! air + leaf

         ! Fraction of potential evaporation from leaf

         if (fdry(p) .gt. 0._r8) then
            rppdry  = fdry(p)*rb(p)*(laisun(p)/(rb(p)+rssun(p)) + &
                                     laisha(p)/(rb(p)+rssha(p)))/elai(p)
         else
            rppdry = 0._r8
         end if
         efpot = forc_rho(g)*wtl*(qsatl(p)-qaf(p))

         if (efpot > 0._r8) then
            if (btran(p) > btran0) then
               qflx_tran_veg(p) = efpot*rppdry
               rpp = rppdry + fwet(p)
            else
               !No transpiration if btran below 1.e-10
               rpp = fwet(p)
               qflx_tran_veg(p) = 0._r8
            end if
            !Check total evapotranspiration from leaves
            rpp = min(rpp, (qflx_tran_veg(p)+h2ocan(p)/dtime)/efpot)
         else
            !No transpiration if potential evaporation less than zero
            rpp = 1._r8
            qflx_tran_veg(p) = 0._r8
         end if

         ! Update conductances for changes in rpp
         ! Latent heat conductances for ground and leaf.
         ! Air has same conductance for both sensible and latent heat.
         ! Moved the original subroutine in-line...

         wtaq    = frac_veg_nosno(p)/raw(p,1)                        ! air
         wtlq    = frac_veg_nosno(p)*(elai(p)+esai(p))/rb(p) * rpp   ! leaf
         wtgq(p) = frac_veg_nosno(p)/raw(p,2)                        ! ground
         wtsqi   = 1./(wtaq+wtlq+wtgq(p))

         wtgq0    = wtgq(p)*wtsqi      ! ground
         wtlq0(p) = wtlq*wtsqi         ! leaf
         wtaq0(p) = wtaq*wtsqi         ! air

         wtgaq    = wtaq0(p)+wtgq0     ! air + ground
         wtalq(p) = wtaq0(p)+wtlq0(p)  ! air + leaf

         dc1 = forc_rho(g)*cpair*wtl
         dc2 = hvap*forc_rho(g)*wtlq

         efsh   = dc1*(wtga*t_veg(p)-wtg0*t_grnd(c)-wta0(p)*thm(c))
         efe(p) = dc2*(wtgaq*qsatl(p)-wtgq0*qg(c)-wtaq0(p)*forc_q(g))

         ! Evaporation flux from foliage

         erre = 0._r8
         if (efe(p)*efeb(p) < 0._r8) then
            efeold = efe(p)
            efe(p)  = 0.1*efeold
            erre = efe(p) - efeold
         end if
         dt_veg(p) = (sabv(p) + air(p) + bir(p)*t_veg(p)**4 + &
              cir(p)*t_grnd(c)**4 - efsh - efe(p)) / &
              (- 4.*bir(p)*t_veg(p)**3 +dc1*wtga +dc2*wtgaq*qsatldT(p))
         t_veg(p) = tlbef(p) + dt_veg(p)
         dels = dt_veg(p)
         del(p)  = abs(dels)
         err(p) = 0._r8
         if (del(p) > delmax) then
            dt_veg(p) = delmax*dels/del(p)
            t_veg(p) = tlbef(p) + dt_veg(p)
            err(p) = sabv(p) + air(p) + bir(p)*tlbef(p)**3*(tlbef(p) + &
                 4.*dt_veg(p)) + cir(p)*t_grnd(c)**4 - &
                 (efsh + dc1*wtga*dt_veg(p)) - (efe(p) + &
                 dc2*wtgaq*qsatldT(p)*dt_veg(p))
         end if

         ! Fluxes from leaves to canopy space
         ! "efe" was limited as its sign changes frequently.  This limit may
         ! result in an imbalance in "hvap*qflx_evap_veg" and
         ! "efe + dc2*wtgaq*qsatdt_veg"

         efpot = forc_rho(g)*wtl*(wtgaq*(qsatl(p)+qsatldT(p)*dt_veg(p)) &
            -wtgq0*qg(c)-wtaq0(p)*forc_q(g))
         qflx_evap_veg(p) = rpp*efpot

         ! Calculation of evaporative potentials (efpot) and
         ! interception losses; flux in kg m**-2 s-1.  ecidif
         ! holds the excess energy if all intercepted water is evaporated
         ! during the timestep.  This energy is later added to the
         ! sensible heat flux.

         ecidif = 0._r8
         if (efpot > 0._r8 .and. btran(p) > btran0) then
            qflx_tran_veg(p) = efpot*rppdry
         else
            qflx_tran_veg(p) = 0._r8
         end if
         ecidif = max(0._r8, qflx_evap_veg(p)-qflx_tran_veg(p)-h2ocan(p)/dtime)
         qflx_evap_veg(p) = min(qflx_evap_veg(p),qflx_tran_veg(p)+h2ocan(p)/dtime)

         ! The energy loss due to above two limits is added to
         ! the sensible heat flux.

         eflx_sh_veg(p) = efsh + dc1*wtga*dt_veg(p) + err(p) + erre + hvap*ecidif

         ! Re-calculate saturated vapor pressure, specific humidity, and their
         ! derivatives at the leaf surface

         call QSat(t_veg(p), forc_pbot(g), el(p), deldT, qsatl(p), qsatldT(p))

         ! Update vegetation/ground surface temperature, canopy air
         ! temperature, canopy vapor pressure, aerodynamic temperature, and
         ! Monin-Obukhov stability parameter for next iteration.

         taf(p) = wtg0*t_grnd(c) + wta0(p)*thm(c) + wtl0(p)*t_veg(p)
         qaf(p) = wtlq0(p)*qsatl(p) + wtgq0*qg(c) + forc_q(g)*wtaq0(p)

         ! Update Monin-Obukhov length and wind speed including the
         ! stability effect

         dth(p) = thm(c)-taf(p)
         dqh(p) = forc_q(g)-qaf(p)

         tstar = temp1(p)*dth(p)
         qstar = temp2(p)*dqh(p)

         thvstar = tstar*(1.+0.61*forc_q(g)) + 0.61*forc_th(g)*qstar
         zeta = zldis(p)*vkc*grav*thvstar/(ustar(p)**2*thv(c))

         if (zeta >= 0.) then     !stable
            zeta = min(2._r8,max(zeta,0.01_r8))
            um(p) = max(ur(p),0.1_r8)
         else                     !unstable
            zeta = max(-100._r8,min(zeta,-0.01_r8))
            wc = beta*(-grav*ustar(p)*thvstar*zii/thv(c))**0.333
            um(p) = sqrt(ur(p)*ur(p)+wc*wc)
         end if
         obu(p) = zldis(p)/zeta

         if (obuold(p)*obu(p) < 0.) nmozsgn(p) = nmozsgn(p)+1
         if (nmozsgn(p) >= 4) obu(p) = zldis(p)/(-0.01)
         obuold(p) = obu(p)

      end do   ! end of filtered pft loop

      ! Test for convergence

      itlef = itlef+1
      if (itlef > itmin) then
!dir$ concurrent
!cdir nodep
         do f = 1, fn
            p = filterp(f)
            dele(p) = abs(efe(p)-efeb(p))
            efeb(p) = efe(p)
            det(p)  = max(del(p),del2(p))
         end do
         fnold = fn
         fn = 0
         do f = 1, fnold
            p = filterp(f)
            if (.not. (det(p) < dtmin .and. dele(p) < dlemin)) then
               fn = fn + 1
               filterp(fn) = p
            end if
         end do
      end if

   end do ITERATION     ! End stability iteration

   fn = fnorig
   filterp(1:fn) = fporig(1:fn)

!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      c = pcolumn(p)
      g = pgridcell(p)

      ! Energy balance check in canopy

      err(p) = sabv(p) + air(p) + bir(p)*tlbef(p)**3*(tlbef(p) + 4.*dt_veg(p)) &
         + cir(p)*t_grnd(c)**4 - eflx_sh_veg(p) - hvap*qflx_evap_veg(p)

      ! Fluxes from ground to canopy space

      delt    = wtal(p)*t_grnd(c)-wtl0(p)*t_veg(p)-wta0(p)*thm(c)
      delq    = wtalq(p)*qg(c)-wtlq0(p)*qsatl(p)-wtaq0(p)*forc_q(g)
      taux(p) = -forc_rho(g)*forc_u(g)/ram1(p)
      tauy(p) = -forc_rho(g)*forc_v(g)/ram1(p)
      eflx_sh_grnd(p) = cpair*forc_rho(g)*wtg(p)*delt
      qflx_evap_soi(p) = forc_rho(g)*wtgq(p)*delq

      ! 2 m height air temperature

      t_ref2m(p) = thm(c) + temp1(p)*dth(p)*(1./temp12m(p) - 1./temp1(p))

      ! 2 m height specific humidity

      q_ref2m(p) = forc_q(g) + temp2(p)*dqh(p)*(1./temp22m(p) - 1./temp2(p))

      ! Downward longwave radiation below the canopy

      dlrad(p) = (1.-emv(p))*emg(c)*forc_lwrad(g) + &
         emv(p)*emg(c)*sb*tlbef(p)**3*(tlbef(p) + 4.*dt_veg(p))

      ! Upward longwave radiation above the canopy

      ulrad(p) = ((1.-emg(c))*(1.-emv(p))*(1.-emv(p))*forc_lwrad(g) &
         + emv(p)*(1.+(1.-emg(c))*(1.-emv(p)))*sb*tlbef(p)**3*(tlbef(p) + &
         4.*dt_veg(p)) + emg(c)*(1.-emv(p))*sb*t_grnd(c)**4)

      ! Derivative of soil energy flux with respect to soil temperature

      cgrnds(p) = cgrnds(p) + cpair*forc_rho(g)*wtg(p)*wtal(p)
      cgrndl(p) = cgrndl(p) + forc_rho(g)*wtgq(p)*wtalq(p)*dqgdT(c)
      cgrnd(p)  = cgrnds(p) + cgrndl(p)*htvp(c)

      ! Update dew accumulation (kg/m2)

      h2ocan(p) = max(0._r8,h2ocan(p)+(qflx_tran_veg(p)-qflx_evap_veg(p))*dtime)

      ! total photosynthesis

!! lwp 2009-07 use "virtual LAI (0.1)" when LAI is too small(according to CTEM)
      laisun(p) = max (0.10, laisun(p) )
!! liwp 2016-01-20
       if (ivt(p).eq.14 ) then
           laisun(p) = max(0.15, laisun(p) )
       endif

#if(defined RICEWATER)
         if (ivt(p).eq.16 ) then
           laisun(p) = max(0.25, laisun(p) )
         endif
#endif
      fpsn(p) = psnsun(p)*laisun(p) + psnsha(p)*laisha(p)

!! lwp 2008-7-1 change unit of CLM3 fpsn to AVIM pht (umol/m**2/s CO2 to Kg/m**2/day biomass)
!! 0.0024624=28.5*1.0e-9*3600*24 
!! lwp 2009-07 change unit of CLM3 fpsn to AVIM_CTEM pht (umol/m**2/s CO2 to Kg C /m**2/day)
!! 0.0012312=28.5*1.0e-9*3600*24*0.5 

!     pht(p)  = fpsn(p)*0.0024624
      pht(p)  = fpsn(p)*0.0012312

    enddo         

!! lwp 2009-6-24 to initialize daily mean variables at the beginning of each day
#ifdef OFFLINE
   if (mod(nstep-1, nstep_day)==0 ) then  !! nstep=1 for first step offline run
#else
   if (mod(nstep  , nstep_day)==0 ) then  !! nstep=0 for first step CAM-CLM3 coupled runs
#endif

      do p=lbp,ubp
        phd(p)  = 0. 
        npd(p)  = 0.   !! (original is 0.) 
        gnd(p)  = 0. 
        ts1d(p) = 0.
        ts2d(p) = 0.
        sm1d(p) = 0.
        sm2d(p) = 0.
        rm1d(p) = 0.
        rm2d(p) = 0.
        rm3d(p) = 0.
        dailytair(p) = 0.
!       dailytveg(p) = 0.
        dailytroot(p) = 0.
      enddo

   endif      !! end of daily loop
!! lwp 2011-5-11 end of initializations for daily mean variables

      call NPPRmRg (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg,  &
           leafstatus(lbp:ubp),bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp), & 
           livefr_stem(lbp:ubp), livefr_root(lbp:ubp), &
           coszen_pft(lbp:ubp), taf(lbp:ubp), pht(lbp:ubp), &
           npp(lbp:ubp),rmp(lbp:ubp),rgp(lbp:ubp), &
           phd(lbp:ubp),gnd(lbp:ubp),npd(lbp:ubp), &
           rm1d(lbp:ubp),rm2d(lbp:ubp),rm3d(lbp:ubp), &
           ts1d(lbp:ubp),ts2d(lbp:ubp),sm1d(lbp:ubp),sm2d(lbp:ubp), &
           dailytair(lbp:ubp),dailytroot(lbp:ubp) )

!! lwp 2009-07 NPP allocation at the end of each day, ('nstep+1' relative to the above 'nstep')
#ifdef OFFLINE
   if (mod(nstep  , nstep_day)==0 ) then  !! nstep=1 for first step offline run
#else
   if (mod(nstep+1, nstep_day)==0 ) then  !! nstep=0 for first step CAM-CLM3 coupled runs
#endif

!! lwp 2011-5-11 to calculate daily mean outside of NPPRmRg  
      do p=lbp,ubp
        phd(p)  = phd(p)  * dthd 
        npd(p)  = npd(p)  * dthd
        gnd(p)  = gnd(p)  * dthd
        ts1d(p) = ts1d(p) * dthd
        ts2d(p) = ts2d(p) * dthd 
        sm1d(p) = sm1d(p) * dthd 
        sm2d(p) = sm2d(p) * dthd 
        rm1d(p) = rm1d(p) * dthd 
        rm2d(p) = rm2d(p) * dthd 
        rm3d(p) = rm3d(p) * dthd 
        dailytair(p) = dailytair(p) * dthd 
        dailytroot(p)= dailytroot(p)* dthd 
      enddo

     call Allocate (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                    leafstatus(lbp:ubp), phd(lbp:ubp), npd(lbp:ubp), &
                    rm1d(lbp:ubp),rm2d(lbp:ubp),rm3d(lbp:ubp), &
                    bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp), &
                    dis1(lbp:ubp),dis2(lbp:ubp),dis3(lbp:ubp) )

     call Phenology (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
              gnd(lbp:ubp),dailytair(lbp:ubp),dailytroot(lbp:ubp),btran(lbp:ubp), & 
              bm1(lbp:ubp),bm2(lbp:ubp),bm3(lbp:ubp),bm4(lbp:ubp), &
              leafstatus(lbp:ubp),los1(lbp:ubp),los2(lbp:ubp),los3(lbp:ubp),los4(lbp:ubp), &
              pandays(lbp:ubp), colddays(lbp:ubp),maturedays(lbp:ubp), &
              normaldays(lbp:ubp), maxdays(lbp:ubp), &
              livefr_stem(lbp:ubp),livefr_root(lbp:ubp), &
              surl(lbp:ubp), rotl(lbp:ubp) )

     call SoilCarbon (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
              npd(lbp:ubp), surl(lbp:ubp), rotl(lbp:ubp), &
              ts1d(lbp:ubp),ts2d(lbp:ubp),sm1d(lbp:ubp),sm2d(lbp:ubp), &
              polc1(lbp:ubp),polc2(lbp:ubp),polc3(lbp:ubp),polc4(lbp:ubp), &
              polc5(lbp:ubp),polc6(lbp:ubp),polc7(lbp:ubp),polc8(lbp:ubp), &
              dsmc(lbp:ubp),dco2(lbp:ubp),nmin(lbp:ubp), nep(lbp:ubp) ) 

   endif      !! end of daily loop

   ! filter out pfts which have small energy balance errors; report others
   fnold = fn
   fn = 0
   do f = 1, fnold
      p = filterp(f)
      if (abs(err(p)) > 0.1) then
         fn = fn + 1
         filterp(fn) = p
      end if
   end do

   do f = 1, fn
      p = filterp(f)
      write(6,*) 'energy balance in canopy ',p,', err=',err(p)
   end do
  end subroutine CanopyFluxes

!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: Stomata
!
! !INTERFACE:
   subroutine Stomata (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                       apar, ei, ea, tgcm, forc_pbot, &
                       tl, o2, co2, btran, rb, &
                       rs, psn, annpsnpot, annpsn,vcmxout,ftout)
!
! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use shr_const_mod, only : SHR_CONST_TKFRZ, SHR_CONST_RGAS
     use clmtype
!! lwp 2011-7-15
#ifdef SPMD
  use spmdMod, only : masterproc, mpicom, MPI_REAL8, MPI_INTEGER, MPI_LOGICAL
#else
  use spmdMod, only : masterproc
#endif
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in)    :: apar(lbp:ubp)      ! par absorbed per unit lai (w/m**2)
     real(r8), intent(in)    :: ei(lbp:ubp)        ! vapor pressure inside leaf (sat vapor press at tl) (pa)
     real(r8), intent(in)    :: ea(lbp:ubp)        ! vapor pressure of canopy air (pa)
     real(r8), intent(in)    :: tgcm(lbc:ubc)      ! air temperature at agcm reference height (kelvin)
     real(r8), intent(in)    :: forc_pbot(lbg:ubg) ! atmospheric pressure (Pa)
     real(r8), intent(in)    :: tl(lbp:ubp)        ! leaf temperature (Kelvin)
     real(r8), intent(in)    :: o2(lbp:ubp)        ! atmospheric o2 concentration (pa)
     real(r8), intent(in)    :: co2(lbp:ubp)       ! atmospheric co2 concentration (pa)
     real(r8), intent(in)    :: btran(lbp:ubp)     ! soil water transpiration factor (0 to 1)
     real(r8), intent(inout) :: rb(lbp:ubp)        ! boundary layer resistance (s/m)
     real(r8), intent(inout) :: rs(lbp:ubp)        ! leaf stomatal resistance (s/m)
     real(r8), intent(inout) :: psn(lbp:ubp)       ! foliage photosynthesis (umol co2 /m**2/ s) [always +]
     real(r8), intent(inout) :: annpsnpot(lbp:ubp) ! annual potential photosynthesis (umol CO2 /m**2)
     real(r8), intent(inout) :: annpsn(lbp:ubp)    ! annual photosynthesis (umol CO2 /m**2)
     real(r8), intent(  out) :: vcmxout(lbp:ubp)   ! actual vcmx 
     real(r8), intent(  out) :: ftout(lbp:ubp)     ! actual vcmx 
!
! !CALLED FROM:
! subroutine CanopyFluxes in this modeul
!
! !REVISION HISTORY:
! author:            Gordon Bonan
! standardized:      J. Truesdale, Feb. 1996
! reviewed:          G. Bonan, Feb. 1996
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! 1/30/02, PET  Made all input and output parameters explicit (removed reference to clm)
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     real(r8), pointer :: qe25(:)        ! quantum efficiency at 25C (umol CO2 / umol photon)
     real(r8), pointer :: vcmx25(:)      ! max rate of carboxylation at 25C (umol CO2/m**2/s)
     real(r8), pointer :: c3psn(:)       ! photosynthetic pathway: 0. = c4, 1. = c3
     real(r8), pointer :: mp(:)          ! slope of conductance-to-photosynthesis relationship
!! lwp 2013-04-15 add topt, tmax, tmin
     real(r8), pointer :: topt(:)        ! optimal temperature for PFT growth
     real(r8), pointer :: tmax(:)        ! maximun temperature for PFT growth
     real(r8), pointer :: tmin(:)        ! minimum temperature for PFT growth
!! lwp 2011-7-5
   real(r8), pointer :: londeg(:)     ! longitude in degrees 
   real(r8), pointer :: latdeg(:)     ! latitude in degrees 
!
!EOP
!
! !OTHER LOCAL VARIABLES:
!
     real(r8), parameter :: mpe = 1.e-6   ! prevents overflow error if division by zero
     integer , parameter :: niter = 3     ! number of iterations
     integer  :: f,p,c,g ! indices
     integer  :: iter    ! iteration index
     real(r8) :: ab      ! used in statement functions
     real(r8) :: bc      ! used in statement functions
     real(r8) :: f1      ! generic temperature response (statement function)
     real(r8) :: f2      ! generic temperature inhibition (statement function)
     real(r8) :: tc      ! leaf temperature (degree celsius)
     real(r8) :: cs      ! co2 concentration at leaf surface (pa)
     real(r8) :: kc      ! co2 michaelis-menten constant (pa)
     real(r8) :: ko      ! o2 michaelis-menten constant (pa)
     real(r8) :: atmp    ! intermediate calculations for rs
     real(r8) :: btmp    ! intermediate calculations for rs
     real(r8) :: ctmp    ! intermediate calculations for rs
     real(r8) :: q       ! intermediate calculations for rs
     real(r8) :: r1,r2   ! roots for rs
     real(r8) :: ppf     ! absorb photosynthetic photon flux (umol photons/m**2/s)
     real(r8) :: wc      ! rubisco limited photosynthesis (umol co2/m**2/s)
     real(r8) :: wj      ! light limited photosynthesis (umol co2/m**2/s)
     real(r8) :: we      ! export limited photosynthesis (umol co2/m**2/s)
     real(r8) :: cp      ! co2 compensation point (pa)
     real(r8) :: ci      ! internal co2 (pa)
     real(r8) :: awc     ! intermediate calcuation for wc
     real(r8) :: vcmx    ! maximum rate of carboxylation (umol co2/m**2/s)
#if (defined DGVM)
     real(r8) :: vcmxpot ! potential maximum rate of carboxylation (umol co2/m**2/s)
#endif
     real(r8) :: j       ! electron transport (umol co2/m**2/s)
     real(r8) :: cea     ! constrain ea or else model blows up
     real(r8) :: cf      ! s m**2/umol -> s/m
     real(r8) :: rsmax0  ! maximum stomatal resistance [s/m]

     real(r8) :: kc25    ! co2 michaelis-menten constant at 25c (pa)
     real(r8) :: akc     ! q10 for kc25
     real(r8) :: ko25    ! o2 michaelis-menten constant at 25c (pa)
     real(r8) :: ako     ! q10 for ko25
     real(r8) :: avcmx   ! q10 for vcmx25
     real(r8) :: bp      ! minimum leaf conductance (umol/m**2/s)
!! lwp 2008-8-28 to change the intent attribute of btran to set >=0.1 restriction on btran
     real(r8) :: btran1  ! soil water transpiration factor (0 to 1)
     real(r8) :: ftemp   ! temperature restriction parameter 
     real(r8) :: topt_tp ! optimal temperature for Tibetan vegetation
     integer  :: ivp     ! short for ivt(p)
!------------------------------------------------------------------------------

     ! Set statement functions

     f1(ab,bc) = ab**((bc-25.)/10.)
     f2(ab) = 1. + exp((-2.2e05+710.*(ab+SHR_CONST_TKFRZ))/(SHR_CONST_RGAS*0.001*(ab+SHR_CONST_TKFRZ)))

     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     qe25      => pftcon%qe25
     vcmx25    => pftcon%vcmx25
     c3psn     => pftcon%c3psn
     mp        => pftcon%mp
     tmin      => pftcon%tmin
     topt      => pftcon%topt
     tmax      => pftcon%tmax
   londeg      => clm3%g%londeg
   latdeg      => clm3%g%latdeg

     ! Set constant values

     kc25  = 30.
     akc   = 2.1
     ko25  = 30000.
     ako   = 1.2
     avcmx = 2.4
     bp    = 2000.

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
           
           ivp = ivt(p)

        ! Initialize rs=rsmax and psn=0 because calculations are performed only
        ! when apar > 0, in which case rs <= rsmax and psn >= 0
        ! Set constants

        rsmax0 = 2.e4
        cf = forc_pbot(g)/(SHR_CONST_RGAS*0.001*tgcm(c))*1.e06

        if (apar(p) <= 0.) then          ! night time
           rs(p) = min(rsmax0, 1./bp * cf)
           psn(p) = 0._r8
        else                             ! day time
           tc = tl(p) - SHR_CONST_TKFRZ
           ppf = 4.6 * apar(p)
           j = ppf * qe25(ivt(p))
           kc = kc25 * f1(akc,tc)
           ko = ko25 * f1(ako,tc)
           awc = kc * (1.+o2(p)/ko)
           cp = 0.5*kc/ko*o2(p)*0.21

!! lwp 2008-8-28 set a lower limit for btran (>=0.20) 

           btran1 = max (0.28, btran(p) )
!! 2009-10 increase Crop NPP due to irrigation which increase soil moisture
!! 2016-12 increase PHT, hence positive NPP & +NEP(land carbon sink) for main tree types 
#ifdef OFFLINE
            if (ivt(p).eq.1 .or. ivt(p).eq.2 .or. ivt(p).eq.4  &
                 .or. ivt(p).eq.5 .or. ivt(p).eq.7 .or. ivt(p).eq.15 ) then
                btran1 = max (0.50, btran(p) )
            endif
#else
            if (ivt(p).le.2  ) then
                btran1 = max (0.55, btran(p) )
            elseif (ivt(p).eq.4  ) then
                btran1 = max (0.88, btran(p) )    !! increased from 0.60
            elseif (ivt(p).eq.5 .or. ivt(p).eq.7 ) then 
                btran1 = max (0.65, btran(p) )
            elseif (ivt(p).eq.13 ) then
                btran1 = max (0.45, btran(p) )
            elseif (ivt(p).eq.14 ) then
                btran1 = max (0.40, btran(p) )
            elseif (ivt(p).eq.15 ) then
                btran1 = max (0.72, btran(p) )
            elseif (ivt(p).eq.16) then
                btran1 = max (0.60, btran(p) )
            endif
#endif

!! lwp 2013-04-15 to use optimal temperature (Kelvin) to increase LAI at high latitudes
       if ( tl(p) .gt. tmin(ivp) .and. tl(p) .le. topt(ivp) ) then
        ftemp=max(0.,1.2-1.5*((tl(p)-topt(ivp))/(tmin(ivp)-topt(ivp)))**1.2)
       elseif ( tl(p) .gt. topt(ivp) .and. tl(p) .le. tmax(ivp) ) then
        ftemp=1.2-((tl(p)-topt(ivp))/(tmax(ivp)-topt(ivp)))**2.0
       elseif ( tl(p) .gt. tmax(ivp) .and. tl(p) .le. (tmax(ivp)+2) ) then
        ftemp = 0.2-0.1*(tl(p)-tmax(ivp))
       else
        ftemp = 0.0
       endif

!! liwp 2016-11-02 added the following lines for ivt=10, ivt=13 over the Tibet (80~102E, 29~38N)
      if (londeg(g).gt.80.and.londeg(g).lt.102.and.latdeg(g).gt.29.and.latdeg(g).lt.38 ) then
          if (ivp.eq.10) then
             topt_tp = 309._r8      !! 310K outside Tibet
           if ( tl(p) .gt. tmin(ivp) .and. tl(p) .le. topt_tp ) then
            ftemp=max(0.,1.2-1.5*((tl(p)-topt_tp)/(tmin(ivp)-topt_tp))**1.2)
           elseif ( tl(p) .gt. topt_tp .and. tl(p) .le. tmax(ivp) ) then
            ftemp=1.2-((tl(p)-topt_tp)/(tmax(ivp)-topt_tp))**2.0
           elseif ( tl(p) .gt. tmax(ivp) .and. tl(p) .le. (tmax(ivp)+2) ) then
            ftemp = 0.2-0.1*(tl(p)-tmax(ivp))
           else
            ftemp = 0.0
           endif
          endif     !! end of ivt=10 adjustment

!         if (ivp.eq.13) then
!            topt_tp = 296._r8      !! 298K outside Tibet
!          if ( tl(p) .gt. tmin(ivp) .and. tl(p) .le. topt_tp ) then
!           ftemp=max(0.,1.2-1.5*((tl(p)-topt_tp)/(tmin(ivp)-topt_tp))**1.2)
!          elseif ( tl(p) .gt. topt_tp .and. tl(p) .le. tmax(ivp) ) then
!           ftemp=1.2-((tl(p)-topt_tp)/(tmax(ivp)-topt_tp))**2.0
!          elseif ( tl(p) .gt. tmax(ivp) .and. tl(p) .le. (tmax(ivp)+2) ) then
!           ftemp = 0.2-0.1*(tl(p)-tmax(ivp))
!          else
!           ftemp = 0.0
!          endif
!         endif     !! end of ivt=13 adjustment
      endif         !! end of Tibet  adjustment

!! liwp 2017-04-17 adjust vcmx by times factor to increase(>1) or decrease(<1) PHT, NPP, NEP
!       vcmx = vcmx25(ivp) * ftemp * btran1 * 0.95
        vcmx = vcmx25(ivp) * ftemp * btran1

!! lwp 2008-9-2 to output vcmx and temperature sensitivity of photosynthesis 
          ftout(p) = ftemp 
        vcmxout(p) = ftemp *btran1 

#if (defined CLM3)
!! (the following two lines are CLM3 original)
        ftemp = f1(avcmx,tc) / f2(tc)
        vcmx = vcmx25(ivt(p)) * ftemp * btran(p) 
#endif   
!! end of CLM3 or AVIM ftemp adjustment 

#if (defined DGVM)
           vcmxpot = vcmx25(ivt(p)) * f1(avcmx,tc) / f2(tc)
#endif

           ! First guess ci
           ci = 0.7*co2(p)*c3psn(ivt(p)) + 0.4*co2(p)*(1.-c3psn(ivt(p)))

           ! rb: s/m -> s m**2 / umol
           rb(p) = rb(p)/cf

           ! Constrain ea
           cea = max(0.25*ei(p)*c3psn(ivt(p))+0.40*ei(p)*(1.-c3psn(ivt(p))), min(ea(p),ei(p)) )

#if (defined DGVM)
           ! ci iteration for 'potential' photosynthesis
           do iter = 1, niter
              wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
              wc = max(ci-cp,0._r8)*vcmxpot/(ci+awc)*c3psn(ivt(p)) + vcmxpot*(1.-c3psn(ivt(p)))
              we = 0.5*vcmxpot*c3psn(ivt(p)) + 4000.*vcmxpot*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
              psn(p) = min(wj,wc,we)
              cs = max( co2(p) - 1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
              atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
              btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
              ctmp = -rb(p)
              if (btmp >= 0.) then
                 q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
              else
                 q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
              end if
              r1 = q/atmp
              r2 = ctmp/q
              rs(p) = max(r1,r2)
              ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           end do

           annpsnpot(p) = annpsnpot(p) + psn(p)

           ! First guess ci
           ci = 0.7*co2(p)*c3psn(ivt(p)) + 0.4*co2(p)*(1.-c3psn(ivt(p)))
#endif

           ! ci iteration for 'actual' photosynthesis

#if (defined NEC_SX)
!NEC NEC NEC
           if ( niter /= 3 ) then
              write(6,*) 'Stomata: niter /= 3';  call endrun()
           end if
           !
           ! ITER = 1
           !
           wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
           wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
           we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
           psn(p) = min(wj,wc,we)
           cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
           atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
           btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
           ctmp = -rb(p)
           if (btmp >= 0.) then
              q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
           else
              q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
           end if
           r1 = q/atmp
           r2 = ctmp/q
           rs(p) = max(r1,r2)
           ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           !
           ! ITER = 2
           !
           wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
           wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
           we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
           psn(p) = min(wj,wc,we)
           cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
           atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
           btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
           ctmp = -rb(p)
           if (btmp >= 0.) then
              q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
           else
              q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
           end if
           r1 = q/atmp
           r2 = ctmp/q
           rs(p) = max(r1,r2)
           ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           !
           ! ITER = 3
           !
           wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
           wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
           we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
           psn(p) = min(wj,wc,we)
           cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
           atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
           btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
           ctmp = -rb(p)
           if (btmp >= 0.) then
              q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
           else
              q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
           end if
           r1 = q/atmp
           r2 = ctmp/q
           rs(p) = max(r1,r2)
           ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
!NEC NEC NEC
#else
           do iter = 1, niter
              wj = max(ci-cp,0._r8)*j/(ci+2.*cp)*c3psn(ivt(p)) + j*(1.-c3psn(ivt(p)))
              wc = max(ci-cp,0._r8)*vcmx/(ci+awc)*c3psn(ivt(p)) + vcmx*(1.-c3psn(ivt(p)))
              we = 0.5*vcmx*c3psn(ivt(p)) + 4000.*vcmx*ci/forc_pbot(g)*(1.-c3psn(ivt(p)))
              psn(p) = min(wj,wc,we)
              cs = max( co2(p)-1.37*rb(p)*forc_pbot(g)*psn(p), mpe )
              atmp = mp(ivt(p))*psn(p)*forc_pbot(g)*cea / (cs*ei(p)) + bp
              btmp = ( mp(ivt(p))*psn(p)*forc_pbot(g)/cs + bp ) * rb(p) - 1.
              ctmp = -rb(p)
              if (btmp >= 0.) then
                 q = -0.5*( btmp + sqrt(btmp*btmp-4.*atmp*ctmp) )
              else
                 q = -0.5*( btmp - sqrt(btmp*btmp-4.*atmp*ctmp) )
              end if
              r1 = q/atmp
              r2 = ctmp/q
              rs(p) = max(r1,r2)
              ci = max( cs-psn(p)*forc_pbot(g)*1.65*rs(p), 0._r8 )
           end do
#endif

#if (defined DGVM)
           annpsn(p) = annpsn(p) + psn(p)
#endif

           ! rs, rb:  s m**2 / umol -> s/m

           rs(p) = min(rsmax0, rs(p)*cf)
           rb(p) = rb(p) * cf

        end if
     end do
  end subroutine Stomata

!------------------------------------------------------------------------------
!! lwp 2008-9-1 the following are source codes of AVIM

!BOP
!
! !IROUTINE: NPPRmRg
!
! !INTERFACE:
   subroutine NPPRmRg (fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                       leafstatus,bm1,bm2,bm3, livefr_stem, livefr_root, &
                       cosz, taf, pht,npp,rmp,rgp , &
                       phd,gnd,npd, rm1d, rm2d, rm3d, &
                       ts1d,ts2d,sm1d,sm2d, &
                       daytair, daytroot)
!
! !DESCRIPTION:
! Leaf stomatal resistance and leaf photosynthesis.
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use time_manager , only : get_step_size , get_nstep, get_curr_calday
     use clmtype
!! lwp 2008-5-20
     use spmdMod, only: masterproc
!! lwp 2008-8-20  to use dzsoil for weighted soil moisture and temperature
    use clm_varpar   , only: numcol, numrad, nlevsoi, nlevlak
    use clm_varcon   , only: dzsoi, zsoi
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in   ) :: leafstatus(lbp:ubp) !
     real(r8), intent(in   ) :: bm1(lbp:ubp)       !
     real(r8), intent(in   ) :: bm2(lbp:ubp)       !
     real(r8), intent(in   ) :: bm3(lbp:ubp)       !
     real(r8), intent(in   ) :: livefr_stem(lbp:ubp)       !
     real(r8), intent(in   ) :: livefr_root(lbp:ubp)       !
     real(r8), intent(in   ) :: cosz(lbp:ubp) 
!
     real(r8), intent(in)    :: taf(lbp:ubp)       ! within canopy air temperature (Kelvin)
     real(r8), intent(in)    :: pht(lbp:ubp)       ! foliage photosynthesis (umol co2 /m**2/ s) [always +]
     real(r8), intent(  out) :: npp(lbp:ubp)       !
     real(r8), intent(  out) :: rmp(lbp:ubp)       !
     real(r8), intent(  out) :: rgp(lbp:ubp)       !

!!   lwp 2008-5-22 to change the intent characteristics of the following three from 'out' to 'inout'
     real(r8), intent(inout) :: phd(lbp:ubp)       ! daily mean Gross Photosynthesis rate
     real(r8), intent(inout) :: gnd(lbp:ubp)       ! daily mean net Photosynthesis rate (GPP- Rm_leaf)
     real(r8), intent(inout) :: npd(lbp:ubp)       ! daily mean NPP (GPP - Rm -Rg)
     real(r8), intent(inout) :: rm1d(lbp:ubp)      ! daily mean maintenance respiration of leaf 
     real(r8), intent(inout) :: rm2d(lbp:ubp)      ! daily mean maintenance respiration of stem 
     real(r8), intent(inout) :: rm3d(lbp:ubp)      ! daily mean maintenance respiration of root 
     real(r8), intent(inout) :: ts1d(lbp:ubp)      ! daily mean 1rd-layer soil temperature
     real(r8), intent(inout) :: ts2d(lbp:ubp)      ! daily mean 2rd-layer soil temperature
     real(r8), intent(inout) :: sm1d(lbp:ubp)      ! daily mean 1rd-layer soil moisture 
     real(r8), intent(inout) :: sm2d(lbp:ubp)      ! daily mean 2rd-layer soil moisture
     real(r8), intent(inout) :: daytair(lbp:ubp) ! daily mean vegetation temperature 
!    real(r8), intent(inout) :: daytveg(lbp:ubp) ! daily mean vegetation temperature 
     real(r8), intent(inout) :: daytroot(lbp:ubp) ! daily mean root temperature 
! !CALLED FROM:
! subroutine CanopyFluxes in this module
!
! !REVISION HISTORY:
! author:            JJJ         
! standardized:      JJJ and zhangyw 2007 
! revised:           Li Weiping, 2008-07, 2009-07, 2016-11 
! 12/18/07, PET  Made all input and output parameters explicit (removed reference to clm)
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     real(r8), pointer :: rootfr(:,:)    ! fraction of roots in each soil layer
     real(r8), pointer :: forc_t(:)      ! atmospheric temperature (Kelvin) 
     real(r8), pointer :: t_veg(:)       ! vegetation temperature (Kelvin)

     real(r8), pointer :: h2osoi_vol(:,:) ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
     real(r8), pointer :: watsat(:,:)    ! volumetric soil water at saturation (porosity) (nlevsoi)
!    real(r8), pointer :: forc_q(:)      ! atmospheric specific humidity (kg/kg)
     real(r8), pointer :: t_soisno(:,:)  ! soil temperature (Kelvin)  (-nlevsno+1:nlevsoi)
!    real(r8), pointer :: sm11 (:)       ! top layer soil moisture
!    real(r8), pointer :: sm22 (:)       ! second layer soil moisture

     real(r8), pointer :: elai(:)        ! one-sided leaf area index
     real(r8), pointer :: londeg(:)      ! longitude (degrees)
     real(r8), pointer :: latdeg(:)      ! latitude (degrees)
!
! !OTHER LOCAL VARIABLES:
!
     integer  :: f,p,c,g,iv ! indices
        real(r8) ::  dtd  ! 
        real(r8) :: gnp  (lbp:ubp)
        real(r8) ::  t25
        real(r8) ::  ts1    ! first soil temperature
        real(r8) ::  ts2    !  2nd soil temperature
        real(r8) ::  ts3    !  3rd soil temperature
        real(r8) ::  troot  !  3rd soil temperature
        real(r8) ::  sm1    ! first soil water content
        real(r8) ::  sm2    ! second soil water content
!! lwp 2008-8-25 to calculate weighted soil temp and moisture for top 10cm, top 170cm
        real(r8) :: dz4, dz8, dz9, root8 
!! lwp 2009-07 add the following constants for maintenance and growth respiration
        real(r8) :: cmain1(16), cmain2(16), cmain3(16)
        real(r8) :: cgrowth(16)
        real(r8) :: npo
        real(r8) :: qtaf
        real(r8) :: qstem
        real(r8) :: qroot
        real(r8) :: rm1,rm2,rm3
        real(r8) :: rg1,rg2,rg3
        real(r8) :: dtime         ! land model time step (sec)
        integer  :: nstep         ! time step number
        real(r8) ::  dthd  ! dthd= 1/steps of one day (for dtime=1800s, dthd=1/48) 
!! lwp 2009-07 maintenance respiration rate for leaf(1), stem(2), root(3) (16 PFT)
!! lwp 2009-07 CTEM use 0.015/day for C3, 0.025/day for C4 crop & C4 grass
     data cmain1/0.0070, 0.0090, 0.0045, &
                 0.0050, 0.0090, 0.0040, 0.0050, 0.0045, &
                 0.0006, 0.0008, 0.0060, &
                 0.0090, 0.0080, 0.0050, &
                 0.0200, 0.0020/

     data cmain2/0.0010, 0.0012, 0.0007, &
                 0.0004, 0.0010, 0.0007, 0.0007, 0.0008, &
                 0.0006, 0.0004, 0.0009, &
                 0.000 , 0.00  , 0.00  , &     !! no stems for grasses
                 0.0030, 0.0012/

     data cmain3/0.0020, 0.0035, 0.0015, &
                 0.0010, 0.0035, 0.0009, 0.0015, 0.0008, &
                 0.0007, 0.0006, 0.0030, &
                 0.0040, 0.0040, 0.0040, &
                 0.0100, 0.0015/

     data cgrowth/0.25, 0.25, 0.25, &
                  0.25, 0.25, 0.22, 0.25, 0.25, &
                  0.25, 0.25, 0.25, &
                  0.25, 0.25, 0.25, &
                  0.25, 0.24 /

     ! Set statement functions

     ! Assign local pointers to derived type members (pft-level)

     pcolumn       => clm3%g%l%c%p%column
     pgridcell     => clm3%g%l%c%p%gridcell
     ivt           => clm3%g%l%c%p%itype
     h2osoi_vol    => clm3%g%l%c%cws%h2osoi_vol
     watsat        => clm3%g%l%c%cps%watsat
     t_soisno      => clm3%g%l%c%ces%t_soisno
     elai          => clm3%g%l%c%p%pps%elai
     rootfr        => clm3%g%l%c%p%pps%rootfr
     t_veg         => clm3%g%l%c%p%pes%t_veg
     forc_t        => clm3%g%a2ls%forc_t
     londeg        => clm3%g%londeg
     latdeg        => clm3%g%latdeg
!    sm11          => clm3%g%l%c%p%pcf%sm11
!    sm22          => clm3%g%l%c%p%pcf%sm22

     ! Set constant values

      t25= 273.0 + 25.
      dtd=1.
      dtime = get_step_size()
!! lwp 2011-7-31
      dthd = dtime/3600./24.

      nstep = get_nstep()
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)

!! liwp 2017-03-31
        iv = ivt(p)

!! lwp 2008-7-30 use CLM3 soil data as input for AVIM surface(top 10cm) and root zone(top 100cm) 
!! lwp 2008-8-20  to use dzsoi for vertically weighted soil moisture and temperature

        dz4= dzsoi(1)+dzsoi(2)+dzsoi(3)+dzsoi(4)
        dz8= dz4+dzsoi(5)+dzsoi(6)+dzsoi(7)+dzsoi(8)
        root8= rootfr(p,1)+rootfr(p,2)+rootfr(p,3)+rootfr(p,4) &
              +rootfr(p,5)+rootfr(p,6)+rootfr(p,7)+rootfr(p,8)
!!      dz9= dz4+dzsoi(5)+dzsoi(6)+dzsoi(7)+dzsoi(8)+dzsoi(9) 
        sm1=(h2osoi_vol(c,1)/watsat(c,1) * dzsoi(1) &
            +h2osoi_vol(c,2)/watsat(c,2) * dzsoi(2) &
            +h2osoi_vol(c,3)/watsat(c,3) * dzsoi(3) &
            +h2osoi_vol(c,4)/watsat(c,4) * dzsoi(4) ) 
        sm2=(sm1+h2osoi_vol(c,5)/watsat(c,5) *dzsoi(5) &
                +h2osoi_vol(c,6)/watsat(c,6) *dzsoi(6) &
                +h2osoi_vol(c,7)/watsat(c,7) *dzsoi(7) &
                +h2osoi_vol(c,8)/watsat(c,8) *dzsoi(8) ) 
!!          +h2osoi_vol(c,9)/watsat(c,9) * dzsoi(9) ) / dz9
        sm1= min(1.0, sm1/dz4)
        sm2= min(1.0, sm2/dz8)
!       sm11(p) = sm1
!       sm22(p) = sm2
        
        ts1 = (t_soisno(c,1) * dzsoi(1) &
              +t_soisno(c,2) * dzsoi(2) &
              +t_soisno(c,3) * dzsoi(3) &
              +t_soisno(c,4) * dzsoi(4) ) 
        ts2 = (ts1 +t_soisno(c,5) * dzsoi(5) &
                   +t_soisno(c,6) * dzsoi(6) &
                   +t_soisno(c,7) * dzsoi(7) &
                   +t_soisno(c,8) * dzsoi(8) ) /dz8
        ts1 = ts1 / dz4
        troot=(t_soisno(c,1 ) * rootfr(p,1 ) &
              +t_soisno(c,2 ) * rootfr(p,2 ) &
              +t_soisno(c,3 ) * rootfr(p,3 ) &
              +t_soisno(c,4 ) * rootfr(p,4 ) &
              +t_soisno(c,5 ) * rootfr(p,5 ) &
              +t_soisno(c,6 ) * rootfr(p,6 ) &
              +t_soisno(c,7 ) * rootfr(p,7 ) &
              +t_soisno(c,8 ) * rootfr(p,8 ) ) / root8

!! lwp 2009-07 calculate leaf RMP at daytime(less sensitive to T) or night
       if (cosz(p).gt.0.0) then          !! daytime
            qtaf =1.6**(0.1*(taf(p)-t25))
       else                              !! nighttime
!! lwp 2009-07 *0.4 is used to reduce maintenance respiration at night
            qtaf =1.6**(0.1*(taf(p)-t25)) * 0.4 
       endif
!! lwp 2010-12-31 reduce respiration at high temperature(latitude lower than 8N or 8S)
       if (abs(latdeg(g)).lt.10) then
            qstem=1.6**(0.1*(taf(p)-t25)) 
            qroot=1.6**(0.1*(troot -t25)) 
       else
            qstem=1.9**(0.1*(taf(p)-t25))
            qroot=1.9**(0.1*(troot -t25))
       endif
       if (cosz(p).lt.0.0) then
            qstem= qstem*0.40
            qroot= qroot*0.40 
       endif
!! lwp 2011-07-10 to reduce respiration at high temperatures
       if (taf(p).gt.308.0) then
            qtaf = qtaf *0.40
            qstem= qstem*0.40
            qroot= qroot*0.40
       endif
       if(taf(p).le.268.0 .or. taf(p).gt. 316.0) then
!! lwp 2008-10-16 to restrain the following restrictions at low temperatures
               qtaf = 0.0
               qstem= 0.0
               qroot= 0.0                 
       endif

! lwp 2008-5-28 use vegetation type-dependent crgs and crms
            rm1=cmain1(iv)*qtaf *bm1(p)
            rm2=cmain2(iv)*qstem*bm2(p) *livefr_stem(p)
            rm3=cmain3(iv)*qroot*bm3(p) *livefr_root(p)
         rmp(p)=rm1+rm2+rm3

!! liwp 2015-09-18 reduce maintenance respiration during leaf fall and NO leaf period
         if ( leafstatus(p).gt. 3.5 ) then       !! no leaf mode
              rmp(p)= 0.25* rmp(p)
         else if ( leafstatus(p).gt. 2.5 ) then  !! leaf fall mode
              rmp(p)= 0.65* rmp(p)
         endif

            gnp(p) = pht(p) - rm1    !! net photosynthesis uptake of CO2
            npo = pht(p) - rmp(p)       
! lwp 2008-5-28 use vegetation type-dependent crgs and crms
          if (npo.le.0.0) then
            rgp(p) = 0.0
          else
            rgp(p) = cgrowth(iv) * npo
          endif

            npp(p) = npo - rgp(p)        !! net primary productivity

!! liwp 2017-03-30 calculate daily mean within NPPRmRg by multiplying dthd
!           npd(p) = npd(p)  + npp(p) *dthd
!           gnd(p) = gnd(p)  + gnp(p) *dthd
!           phd(p) = phd(p)  + pht(p) *dthd
!           ts1d(p)= ts1d(p) + ts1  *dthd
!           ts2d(p)= ts2d(p) + ts2  *dthd
!           sm1d(p)= sm1d(p) + sm1  *dthd
!           sm2d(p)= sm2d(p) + sm2  *dthd
!           rm1d(p)= rm1d(p) + rm1  *dthd
!           rm2d(p)= rm2d(p) + rm2  *dthd
!           rm3d(p)= rm3d(p) + rm3  *dthd
            npd(p) = npd(p)  + npp(p)
            gnd(p) = gnd(p)  + gnp(p)
            phd(p) = phd(p)  + pht(p)
            ts1d(p)= ts1d(p) + ts1  
            ts2d(p)= ts2d(p) + ts2 
            sm1d(p)= sm1d(p) + sm1
            sm2d(p)= sm2d(p) + sm2
            rm1d(p)= rm1d(p) + rm1
            rm2d(p)= rm2d(p) + rm2
            rm3d(p)= rm3d(p) + rm3
!! lwp 2009-11-28 use atmospheric temperature as criterion
#if (defined OFFLINE)
!           daytair (p)= daytair (p) + forc_t(g)*dthd
            daytair (p)= daytair (p) + forc_t(g)
#else
!           daytair (p)= daytair (p) + taf(p)   *dthd
            daytair (p)= daytair (p) + taf(p)
#endif
!           daytroot(p)= daytroot(p) + troot    *dthd
            daytroot(p)= daytroot(p) + troot

     end do
  end subroutine NPPRmRg

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: Allocate 
!
! !INTERFACE:
 
      subroutine Allocate(fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                          LFSTATUS, phd , npd , & 
                          rm1d    , rm2d, rm3d, &
                          GLEAFMAS, STEMMASS, ROOTMASS, & 
                          AFRLEAF , AFRSTEM , AFRROOT )
!
!     05  MAY 2003  - THIS SUBROUTINE CALCULATES THE ALLOCATION FRACTIONS
!     V. ARORA        FOR LEAF, STEM, AND ROOT COMPONENTS FOR CTEM's PFTs 
!
!     INPUTS 
!
!     LFSTATUS  - LEAF STATUS. AN INTEGER INDICATING IF LEAVES ARE  
!                 IN "MAX. GROWTH", "NORMAL GROWTH", "FALL/HARVEST",
!                 OR "NO LEAVES" MODE. SEE PHENOLGY SUBROUTINE FOR 
!                 MORE DETAILS.
!     THLIQ     - LIQUID SOIL MOISTURE CONTENT IN 3 SOIL LAYERS
!     AILCG     - GREEN OR LIVE LEAF AREA INDEX
!     AILCB     - BROWN OR DEAD LEAF AREA INDEX
!     SAND      - PERCENTAGE SAND
!     CLAY      - PERCENTAGE CLAY
!     RMATCTEM  - FRACTION OF ROOTS IN EACH SOIL LAYER FOR EACH PFT
!     GLEAFMAS  - GREEN OR LIVE LEAF MASS IN KG C/M2, FOR THE 9 PFTs
!     STEMMASS  - STEM MASS FOR EACH OF THE 9 CTEM PFTs, Kg C/M2
!     ROOTMASS  - ROOT MASS FOR EACH OF THE 9 CTEM PFTs, Kg C/M2
!
!     OUTPUTS
!
!     AFRLEAF   - ALLOCATION FRACTION FOR LEAVES
!     AFRSTEM   - ALLOCATION FRACTION FOR STEM
!     AFRROOT   - ALLOCATION FRACTION FOR ROOT
!     WILTSM    - WILTING POINT SOIL MOISTURE CONTENT
!     FIELDSM   - FIELD CAPACITY SOIL MOISTURE CONTENT
!     WTSTATUS  - SOIL WATER STATUS (0 DRY -> 1 WET)
!     LTSTATUS  - LIGHT STATUS
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use clmtype
     use clm_varpar   , only: numcol, numrad, nlevsoi, nlevlak
  use spmdMod             , only : masterproc
#if (defined COUP_CAM)
  use time_manager        , only : get_step_size, get_curr_calday, &
                                   get_curr_date, get_ref_date, get_nstep, &
                                   is_perpetual
#else
  use time_manager        , only : get_step_size, get_curr_calday, &
                                   get_curr_date, get_ref_date, get_nstep
#endif

! !ARGUMENTS:
      IMPLICIT NONE
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in)    :: LFSTATUS(lbp:ubp)  ! integer indicating leaf phenology(1-4) 
     real(r8), intent(in)    :: phd     (lbp:ubp)  ! photosynthesis rate 
     real(r8), intent(in)    :: npd     (lbp:ubp)  ! daily mean NPP 
     real(r8), intent(in)    :: rm1d    (lbp:ubp)  ! leaf maintenance respiration
     real(r8), intent(in)    :: rm2d    (lbp:ubp)  ! leaf maintenance respiration
     real(r8), intent(in)    :: rm3d    (lbp:ubp)  ! leaf maintenance respiration
     real(r8), intent(inout) :: GLEAFMAS(lbp:ubp)  ! Leaf mass 
     real(r8), intent(inout) :: STEMMASS(lbp:ubp)  ! Stem mass 
     real(r8), intent(inout) :: ROOTMASS(lbp:ubp)  ! Root mass 
     real(r8), intent(out)   :: AFRLEAF (lbp:ubp)  ! Allocation to leaf 
     real(r8), intent(out)   :: AFRSTEM (lbp:ubp)  ! Allocation to stem 
     real(r8), intent(out)   :: AFRROOT (lbp:ubp)  ! Allocation to root 
!
! !CALLED FROM:
! subroutine CanopyFluxes in this module
!
! !REVISION HISTORY:
! author:            Vivek Arora
! revised:           Weiping Li, 2009-07

! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     integer , pointer :: ixy(:)         ! xy lon index (column-level)
     integer , pointer :: jxy(:)         ! xy lat index (column_level)
     real(r8), pointer :: rootfr(:,:)    ! fraction of roots in each soil layer
!!! lwp 2009-07-30 to add wiltsm and fieldsm
  real(r8), pointer :: wiltsm (:,:)        ! wilting point soil moisture (nlevsoi)
  real(r8), pointer :: fieldsm(:,:)        ! field capacity soil moisture (nlevsoi)
  real(r8), pointer :: londeg(:)        ! latitude (degrees)
  real(r8), pointer :: latdeg(:)        ! latitude (degrees)
  real(r8), pointer :: lat(:)           ! latitude (radians)
  real(r8), pointer :: h2osoi_vol(:,:)  ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
  real(r8), pointer :: watsat(:,:)      ! volumetric soil water at saturation (porosity) (nlevsoi)
  real(r8), pointer :: elai(:)          ! one-sided leaf area index
  real(r8), pointer :: alai(:)          ! real LAI calculated by AVIM 
  real(r8), pointer :: pandays(:)       ! positive Gn days (GPP-Rml) 
! !OTHER LOCAL VARIABLES:
     integer  :: f,p,c,g,nstep ! indices
     real(r8) :: THLIQ    (lbc:ubc,1:nlevsoi)
     real(r8) :: AILCG    (lbp:ubp)
     real(r8) :: ALEAF    (lbp:ubp)
     real(r8) :: ASTEM    (lbp:ubp)
     real(r8) :: AROOT    (lbp:ubp)
     real(r8) :: AVTHLIQ  (lbp:ubp)
     real(r8) :: AVWILTSM (lbp:ubp)
     real(r8) :: AFIELDSM (lbp:ubp)
     real(r8) :: WTSTATUS (lbp:ubp)
     real(r8) :: LTSTATUS (lbp:ubp)
     real(r8) :: NSTATUS  (lbp:ubp)
     real(r8) :: WNSTATUS (lbp:ubp)
     real(r8) :: MNSTRTMS (lbp:ubp)
     real(r8) :: DAYLNGTH (lbp:ubp)
     real(r8) :: NTCHLVEG (lbp:ubp)  !! net change of leaf
     real(r8) :: NTCHSVEG (lbp:ubp)  !! net change of stem
     real(r8) :: NTCHRVEG (lbp:ubp)  !! net change of root
     real(r8) :: DENOM, DIFF,TERM1,TERM2
     real(r8) :: ROOTPERCENT, ZERO 

      INTEGER ::    K,  KK, N
!
      PARAMETER (KK=16) ! PRODUCT OF CTEM  PFTs 
!
      REAL(r8) :: OMEGA(KK), EPSILONL(KK), EPSILONS(KK), EPSILONR(KK), &
                  KN(KK), ETA(KK), KAPPA(KK), ALDRLFON(KK), RTSRMIN(KK)
!! lwp 2009-12-1 add the following arguments to calculate day length
      REAL(r8),parameter :: PI=3.1415926535898d0
     real(r8) :: TERM, DECLI, DAY, CALDAY, RADL
     real(r8) :: THETA
!
!     ------------------------------------------------------------------
!                     CONSTANTS AND PARAMETERS
!
!     NOTE THE STRUCTURE OF VECTORS WHICH CLEARLY SHOWS THE CLASS
!     PFTs (ALONG ROWS) AND CTEM SUB-PFTs (ALONG COLUMNS)
!
!     NEEDLE LEAF |  EVG(temp) EVGboreal DCDboreal
!     BROAD LEAF  |  EVG(trop) EVGtemp   DCD-DRY  DCD-temp DCD-COLD
!     BROAD SHRUB |  EVG       DCDtemp   DCDboreal
!     GRASSES     |   C3arctic  C3       C4
!     CROPS       |   C3        C4       ---
!
!     ------------------------------------------------------------------
!
!     OMEGA, PARAMETER USED IN ALLOCATION FORMULAE
      DATA  OMEGA/0.70, 0.70, 0.50,   &
                  0.60, 0.60, 0.70, 0.70, 0.70, &
                  0.70, 0.70, 0.70,  &
                  0.90, 0.90, 0.90,  &
                  0.06, 0.06  /
!     EPSILON LEAF, PARAMETER USED IN ALLOCATION FORMULAE
      DATA EPSILONL/0.21, 0.23, 0.24,  &
                    0.33, 0.35, 0.33, 0.30, 0.35, &
                    0.30, 0.34, 0.30,  &
                    0.36, 0.42, 0.40,  &
                    0.80, 0.80/
!     EPSILON STEM, PARAMETER USED IN ALLOCATION FORMULAE
      DATA EPSILONS/0.03 , 0.04, 0.11,  &
                    0.005, 0.01, 0.09, 0.09, 0.10, &
                    0.10 , 0.10, 0.10,  &
                    0.00 , 0.00, 0.00,  &
                    0.15 , 0.15/
!     EPSILON ROOT, PARAMETER USED IN ALLOCATION FORMULAE
      DATA EPSILONR/0.76 , 0.73, 0.65,  &
                    0.665, 0.64, 0.58, 0.61, 0.55, & 
                    0.60 , 0.56, 0.60,  &
                    0.64 , 0.58, 0.60,  &
                    0.05 , 0.05/
!     MINIMUM ROOT:SHOOT RATIO MOSTLY FOR SUPPORT AND STABILITY
      DATA RTSRMIN /0.16, 0.17, 0.16, &
                    0.16, 0.16, 0.22, 0.18, 0.18, &
                    0.18, 0.18, 0.18, &
                    0.50, 0.50, 0.50, &
                    0.15, 0.15/
!
!     ALLOCATION TO LEAVES DURING LEAF ONSET
!! lwp 2013-05-29 reduce the allocation to leaves of evergreen plants
      DATA ALDRLFON/0.90, 0.90, 0.90, &
                    0.80, 0.75, 0.85, 0.85, 0.90, &
                    0.85, 0.90, 0.90, &
                    0.50, 0.50, 0.50, &
                    0.95, 0.95/

      DATA KN /0.50, 0.50, 0.50,             &
               0.50, 0.50, 0.50, 0.50, 0.50, &
               0.50, 0.50, 0.50,             &
               0.46, 0.46, 0.44,             &  ! no use for grasses
               0.40, 0.48/

      DATA ETA/15.0, 20.0, 30.0,             &
               30.0, 31.0, 30.0, 35.0, 35.0, &
                8.0,  8.0,  8.0,             &
                3.0,  3.0,  3.0,             &
                5.0,  5.0/

      DATA KAPPA/1.6, 1.6, 1.6,           &
                 1.6, 1.6, 1.6, 1.6, 1.6, &
                 1.5, 1.5, 1.5,           &
                 1.2, 1.2, 1.2,           &
                 1.6, 1.6/
      DATA ZERO/1E-5/
!
!     ---------------------------------------------------------------
 ! Assign local pointers to derived type members (pft-level)

     pcolumn        => clm3%g%l%c%p%column
     pgridcell      => clm3%g%l%c%p%gridcell
     ivt            => clm3%g%l%c%p%itype
     londeg         => clm3%g%londeg
     latdeg         => clm3%g%latdeg
     lat            => clm3%g%lat   
     rootfr         => clm3%g%l%c%p%pps%rootfr
     wiltsm         => clm3%g%l%c%cps%wiltsm
     fieldsm        => clm3%g%l%c%cps%fieldsm
     h2osoi_vol     => clm3%g%l%c%cws%h2osoi_vol
     watsat         => clm3%g%l%c%cps%watsat
     elai           => clm3%g%l%c%p%pps%elai
     alai           => clm3%g%l%c%p%pcf%alai
     pandays        => clm3%g%l%c%p%pcf%pandays
!
!     INITIALIZE REQUIRED ARRAYS TO ZERO
!
!     ------------------------------------------------------------------
      calday = get_curr_calday(offset=0)
  nstep = get_nstep()

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
          AFRLEAF(p)=0.0    !ALLOCATION FRACTION FOR LEAVES
          AFRSTEM(p)=0.0    !ALLOCATION FRACTION FOR STEM
          AFRROOT(p)=0.0    !ALLOCATION FRACTION FOR ROOT
            ALEAF(p)=0.0    !TEMPORARY VARIABLE
            ASTEM(p)=0.0    !TEMPORARY VARIABLE
            AROOT(p)=0.0    !TEMPORARY VARIABLE
          AVWILTSM(p)=0.0   !WILTING POINT SOIL MOISTURE
          AFIELDSM(p)=0.0   !FIELD CAPACITY SOIL MOISTURE
           AVTHLIQ(p)=0.0   !LIQUID SOIL MOISTURE CONTENT
          WTSTATUS(p)=0.0   !WATER STATUS
          LTSTATUS(p)=0.0   !LIGHT STATUS
           NSTATUS(p)=0.0   !NITROGEN STATUS, IF AND WHEN WE WILL HAVE N CYCLE IN THE MODEL
          WNSTATUS(p)=0.0   !MIN. OF WATER & N STATUS
          MNSTRTMS(p)=0.0   !MIN. (STEM+ROOT) BIOMASS NEEDED TO SUPPORT LEAVES
!     INITIALIZATION ENDS    
        
!! lwp 2009-10-20 alai can be zero
        AILCG(p) = elai(p)

!     CALCULATE LIQUID SOIL MOISTURE CONTENT, AND WILTING AND FIELD CAPACITY 
!     SOIL MOISTURE CONTENTS AVERAGED OVER THE ROOT ZONE. NOTE THAT WHILE
!     THE SOIL MOISTURE CONTENT IS SAME UNDER THE ENTIRE GCM GRID CELL,
!     SOIL MOISTURE AVERAGED OVER THE ROOTING DEPTH IS DIFFERENT FOR EACH
!     PFT BECAUSE OF DIFFERENT FRACTION OF ROOTS PRESENT IN EACH SOIL LAYER.

! AVERAGED OVER THE ROOT ZONE
         AVWILTSM(p) =  wiltsm(c,1)*rootfr(p,1) + &
                        wiltsm(c,2)*rootfr(p,2) + &
                        wiltsm(c,3)*rootfr(p,3) + &
                        wiltsm(c,4)*rootfr(p,4) + &
                        wiltsm(c,5)*rootfr(p,5) + &
                        wiltsm(c,6)*rootfr(p,6) + &
                        wiltsm(c,7)*rootfr(p,7) + &
                        wiltsm(c,8)*rootfr(p,8) 
     !                  wiltsm(c,9)*rootfr(p,9) 
     !                  wiltsm(c,10)*rootfr(p,10)
!! lwp 2009-07 rootpercent should equal to 1.0
!! lwp 2013-06 use the upper 8 layers (about 1 meters from the surface) 
         ROOTPERCENT = rootfr(p,1) + rootfr(p,2) + rootfr(p,3) + &
                       rootfr(p,4) + rootfr(p,5) + rootfr(p,6) + &
                       rootfr(p,7) + rootfr(p,8)
     !                 rootfr(p,9) + rootfr(p,10)

         AVWILTSM(p) = AVWILTSM(p) / ROOTPERCENT
          AVWILTSM(p)= max(0.0, min(1.0, AVWILTSM(p)) )
 
         AFIELDSM(p) = fieldsm(c,1)*rootfr(p,1) + &
                       fieldsm(c,2)*rootfr(p,2) + &
                       fieldsm(c,3)*rootfr(p,3) + &
                       fieldsm(c,4)*rootfr(p,4) + &
                       fieldsm(c,5)*rootfr(p,5) + &
                       fieldsm(c,6)*rootfr(p,6) + &
                       fieldsm(c,7)*rootfr(p,7) + &
                       fieldsm(c,8)*rootfr(p,8)
     !                 fieldsm(c,9)*rootfr(p,9)
     !                 fieldsm(c,10)*rootfr(p,10) 
         AFIELDSM(p) = AFIELDSM(p) / ROOTPERCENT
          AFIELDSM(p)= max(0.0, min(1.0, AFIELDSM(p)) )
 
         AVTHLIQ(p)  = h2osoi_vol(c,1)*rootfr(p,1) + &
                       h2osoi_vol(c,2)*rootfr(p,2) + &
                       h2osoi_vol(c,3)*rootfr(p,3) + &
                       h2osoi_vol(c,4)*rootfr(p,4) + &
                       h2osoi_vol(c,5)*rootfr(p,5) + &
                       h2osoi_vol(c,6)*rootfr(p,6) + &
                       h2osoi_vol(c,7)*rootfr(p,7) + &
                       h2osoi_vol(c,8)*rootfr(p,8)
     !                 h2osoi_vol(c,9)*rootfr(p,9)
     !                 h2osoi_vol(c,10)*rootfr(p,10)
         AVTHLIQ(p)  = AVTHLIQ(p) / ROOTPERCENT
          AVTHLIQ(p) = max(0.10, min(1.0, AVTHLIQ(p)) )
 
!     USING LIQUID SOIL MOISTURE CONTENT TOGETHER WITH WILTING AND FIELD 
!     CAPACITY SOIL MOISTURE CONTENTS AVERAGED OVER THE ROOT ZONE, FIND
!     SOIL WATER STATUS.
!
       IF(AVTHLIQ(p).LE.AVWILTSM(p)) THEN
         WTSTATUS(p)= 0.001_r8         !! original is 0
!! liwp 2015-08-20 to increase this variable ( <1 ) and decrease allocation to root
       ELSE IF(AVTHLIQ(p).GT.AVWILTSM(p).AND.AVTHLIQ(p).LT.AFIELDSM(p)) THEN
         WTSTATUS(p)=(AVTHLIQ(p)-AVWILTSM(p))/(AFIELDSM(p)-AVWILTSM(p))
         WTSTATUS(p)= WTSTATUS(p)**0.25 + 0.001_r8
       ELSE
         WTSTATUS(p)=1.0
       ENDIF
!
!     CALCULATE LIGHT STATUS AS A FUNCTION OF LAI AND LIGHT EXTINCTION
!     PARAMETER. FOR NOW SET NITROGEN STATUS EQUAL TO 1, WHICH MEANS 
!     NITROGEN IS NON-LIMITING.
!
          IF(ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14) THEN  ! GRASSES
            LTSTATUS(p)=MAX(0.0, (1.0-(AILCG(p)/4.5)) )
          ELSE             ! TREES AND CROPS
            LTSTATUS(p)=EXP(-KN(ivt(p))*AILCG(p))
          ENDIF

          NSTATUS(p) =1.0       ! which may be modified in Nitrogen cycling
!
!     ALLOCATION TO ROOTS IS DETERMINED BY MIN. OF WATER AND NITROGEN
!     STATUS
!
          WNSTATUS(p)=MIN(NSTATUS(p),WTSTATUS(p))
!
!     NOW THAT WE KNOW WATER, LIGHT, AND NITROGEN STATUS WE CAN FIND
!     ALLOCATION FRACTIONS FOR LEAVES, STEM, AND ROOT COMPONENTS. NOTE
!     THAT ALLOCATION FORMULAE FOR GRASSES ARE DIFFERENT FROM THOSE
!     FOR TREES AND CROPS, SINCE THERE IS NO STEM COMPONENT IN GRASSES. 
!
    IF(ivt(p).le.11.or.ivt(p).eq.15 .or. ivt(p).eq.16 ) THEN    ! TREES AND CROPS
      DENOM = 1.0 + OMEGA(ivt(p))*( 2.0-LTSTATUS(p)-WNSTATUS(p) )    
      AFRSTEM(p)=( EPSILONS(ivt(p))+OMEGA(ivt(p))*(1.0-LTSTATUS(p)) )/DENOM
      AFRROOT(p)=( EPSILONR(ivt(p))+OMEGA(ivt(p))*(1.0-WNSTATUS(p)) )/DENOM
      AFRLEAF(p)=  EPSILONL(ivt(p))/DENOM  
    ELSE IF(ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14) THEN  ! GRASSES
      DENOM = 1.0 + OMEGA(ivt(p))*( 1.0+LTSTATUS(p)-WNSTATUS(p) )
      AFRLEAF(p)=( EPSILONL(ivt(p))+OMEGA(ivt(p))*LTSTATUS(p) ) /DENOM  
      AFRROOT(p)=( EPSILONR(ivt(p))+OMEGA(ivt(p))*(1.0-WNSTATUS(p)) )/DENOM
      AFRSTEM(p)= 0.0
    ENDIF
!
!     IF USING CONSTANT ALLOCATION FACTORS THEN REPLACE THE DYNAMICALLY
!     CALCULATED ALLOCATION FRACTIONS.
!
!     MAKE SURE ALLOCATION FRACTIONS ADD TO ONE
!
      IF( ABS(AFRSTEM(p)+AFRROOT(p)+AFRLEAF(p)-1.0).GT.ZERO ) then 
         if (masterproc) then
        WRITE(6,2000)  AFRSTEM(p)+AFRROOT(p)+AFRLEAF(p), ivt(p) 
2000  FORMAT(' ALLOCATION NOT ADDING TO ONE. SUM= ',F12.7, 'ivt = ', I5)
         endif
      ENDIF
!
!     THE ALLOCATION FRACTIONS CALCULATED ABOVE ARE OVERRIDDEN BY TWO
!     RULES. 
!
!     RULE 1 WHICH STATES THAT AT THE TIME OF LEAF ONSET WHICH CORRESPONDS 
!     TO LEAF STATUS EQUAL TO 1, MORE C IS ALLOCATED TO LEAVES SO 
!     THAT THEY CAN GROW ASAP. IN ADDITION WHEN LEAF STATUS IS 
!     "FALL/HARVEST" THEN NOTHING IS ALLOCATED TO LEAVES.
!
          IF( LFSTATUS(p).lt.1.1 ) THEN    !! "max growth" mode
            ALEAF(p)=ALDRLFON(ivt(p))
!
!     FOR GRASSES WE USE THE USUAL ALLOCATION EVEN AT LEAF ONSET
! lwp 2009-10-20 increase growth of grass leaves using ALDRLFON 
!
            IF(ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14) THEN
              ALEAF(p)=AFRLEAF(p)
            ENDIF
!
            DIFF  = AFRLEAF(p)-ALEAF(p)
            IF((AFRSTEM(p)+AFRROOT(p)).GT.ZERO)THEN 
              TERM1 = AFRSTEM(p)/(AFRSTEM(p)+AFRROOT(p))
              TERM2 = AFRROOT(p)/(AFRSTEM(p)+AFRROOT(p))
            ELSE
              TERM1 = 0.0
              TERM2 = 0.0
            ENDIF 
            ASTEM(p) = AFRSTEM(p) + DIFF*TERM1
            AROOT(p) = AFRROOT(p) + DIFF*TERM2
            AFRLEAF(p)=ALEAF(p)
            AFRSTEM(p)=MAX(0.0,ASTEM(p))
            AFRROOT(p)=MAX(0.0,AROOT(p))

          ELSE IF(abs(LFSTATUS(p)-3.0).lt. 0.1) THEN  !! "leaffall" mode
            ALEAF(p)=0.0
            DIFF  = AFRLEAF(p)-ALEAF(p)
!! liwp2010-01 reduce Crop growth after harvest (1/20 of CTEM original)
!! liwp2015-09 reduce allocate to stem & growth during leaffall(50% of CTEM)
               if (ivt(p).ge.15) then
                   DIFF = DIFF * 0.05 
               else
                   DIFF = DIFF * 0.5
               endif

            IF((AFRSTEM(p)+AFRROOT(p)).GT.ZERO)THEN 
              TERM1 = AFRSTEM(p)/(AFRSTEM(p)+AFRROOT(p))
              TERM2 = AFRROOT(p)/(AFRSTEM(p)+AFRROOT(p))
            ELSE
              TERM1 = 0.0
              TERM2 = 0.0
            ENDIF 
            ASTEM(p) = AFRSTEM(p) + DIFF*TERM1
            AROOT(p) = AFRROOT(p) + DIFF*TERM2
            AFRLEAF(p)=ALEAF(p)
            AFRSTEM(p)=ASTEM(p)
            AFRROOT(p)=AROOT(p)
          ENDIF
!
!     RULE 2 OVERRIDES RULE 1 ABOVE AND MAKES SURE THAT WE DO NOT ALLOW THE 
!     AMOUNT OF LEAVES ON TREES AND CROPS (I.E. PFTs 1 TO 7) TO EXCEED 
!     AN AMOUNT SUCH THAT THE REMAINING WOODY BIOMASS CANNOT SUPPORT. 
!     IF THIS HAPPENS, ALLOCATION TO LEAVES IS REDUCED AND MOST NPP 
!     IS ALLOCATED TO STEM AND ROOTS, IN A PROPORTION BASED ON CALCULATED 
!     AFRSTEM AND AFRROOT. FOR GRASSES THIS RULE ESSENTIALLY CONSTRAINS 
!     THE ROOT:SHOOT RATIO, MEANING THAT THE MODEL GRASSES CAN'T HAVE 
!     LOTS OF LEAVES WITHOUT HAVING A REASONABLE AMOUNT OF ROOTS.
!
!     FIND MIN. STEM+ROOT BIOMASS NEEDED TO SUPPORT THE GREEN LEAF BIOMASS. 
          MNSTRTMS(p)=ETA(ivt(p))*(GLEAFMAS(p)**KAPPA(ivt(p)))
!
!! lwp 2009-07 not use this rule to crops
    if(ivt(p).lt.12 .or. ivt(p).gt.14 ) then
          IF( (STEMMASS(p)+ROOTMASS(p)).LT.MNSTRTMS(p)) THEN   
  !         IF( (AFRSTEM(p)+AFRROOT(p)).GT.ZERO ) THEN  !! original
            IF( (AFRSTEM(p)+AFRROOT(p)).GT. 0.05 ) THEN
!! lwp 2009-12 use AFRLEAF/2. rather than 0.05 as allocation to leaf
!             ALEAF(p)=MIN(0.05,AFRLEAF(p))
              ALEAF(p)=AFRLEAF(p)*0.5
              DIFF  = AFRLEAF(p)-ALEAF(p)
              TERM1 = AFRSTEM(p)/(AFRSTEM(p)+AFRROOT(p))
              TERM2 = AFRROOT(p)/(AFRSTEM(p)+AFRROOT(p))
              ASTEM(p) = AFRSTEM(p) + DIFF*TERM1
              AROOT(p) = AFRROOT(p) + DIFF*TERM2
              AFRLEAF(p)=ALEAF(p)
              AFRSTEM(p)=ASTEM(p)
              AFRROOT(p)=AROOT(p)
            ELSE
!             ALEAF(p)=MIN(0.05,AFRLEAF(p))
              ALEAF(p)=AFRLEAF(p)*0.5
              DIFF  = AFRLEAF(p)-ALEAF(p)
              AFRLEAF(p)=ALEAF(p)
              AFRSTEM(p)=DIFF*0.5 + AFRSTEM(p)
              AFRROOT(p)=DIFF*0.5 + AFRROOT(p)
            ENDIF
          ENDIF
!
!     MAKE SURE THAT ROOT:SHOOT RATIO IS AT LEAST EQUAL TO RTSRMIN. IF NOT
!     ALLOCATE MORE TO ROOT AND DECREASE ALLOCATION TO STEM.
!
      IF( (STEMMASS(p)+GLEAFMAS(p)).GT.0.05)THEN
        IF( (ROOTMASS(p)/(STEMMASS(p)+GLEAFMAS(p))).LT.RTSRMIN(ivt(p)) ) THEN  
!! liwp 2016-01-20 modify the following adjustment to ASTEM, 70% rahter than 0.05
!! liwp 2016-01-20 to avoid abrupt change of AFRSTEM 
!         ASTEM(p)=MIN(0.05,AFRSTEM(p))
          ASTEM(p)=AFRSTEM(p)*0.7
          DIFF = AFRSTEM(p)-ASTEM(p)
          AFRSTEM(p)=AFRSTEM(p)-DIFF
          AFRROOT(p)=AFRROOT(p)+DIFF
        ENDIF
      ENDIF
    endif     !! end of type 12, 13, 14 (grasses)
!
!     FINALLY CHECK IF ALL ALLOCATION FRACTIONS ARE POSITIVE AND CHECK
!     AGAIN THEY ALL ADD TO ONE.
!
      IF( (AFRLEAF(p).LT.0.0).OR.(AFRSTEM(p).LT.0.0).OR.(AFRROOT(p).LT.0.0))THEN
        WRITE(6,2200) p, ivt(p)
2200  FORMAT(' AT  = (',I5,'), PFT=',I2,'  ALLOCATION FRACTIONS NEGATIVE') 
        WRITE(6,2100) AFRLEAF(p),AFRSTEM(p),AFRROOT(p)
2100    FORMAT(' ALEAF = ',F12.9,' ASTEM = ',F12.9,' AROOT = ',F12.9)
      ENDIF
!
!! lwp 2009-07 to update leaf, stem, root mass
!    ------------------------------------------------------------------
!
!     MAINTENANCE RESPIRATION ALSO REDUCES LEAF, STEM, AND ROOT BIOMASS.
!     WHEN NPP FOR A GIVEN PFT IS POSITIVE THEN THIS IS TAKEN CARE BY
!     ALLOCATING +VE NPP AMONGST THE LEAVES, STEM, AND ROOT COMPONENT.
!     WHEN NPP FOR A GIVEN PFT IS NEGATIVE THEN MAINTENANCE RESPIRATION
!     LOSS IS EXPLICITLY DEDUCTED FROM EACH COMPONENT.
!
        IF( LFSTATUS(p).LT.2.5 ) THEN       !! status=1,2 Leaves are on
            IF(npd(p).GT.0.0) THEN
              NTCHLVEG(p)=AFRLEAF(p)*npd(p)
              NTCHSVEG(p)=AFRSTEM(p)*npd(p)
              NTCHRVEG(p)=AFRROOT(p)*npd(p)
            ELSE
!! liwp 2015-09-07 reduce maintenance respiration when NPP is negative
!!            NTCHLVEG(p)=-rm1d(p)+AFRLEAF(p)*phd(p)
              NTCHLVEG(p)=-rm1d(p)*0.7 +AFRLEAF(p)*phd(p)
              NTCHSVEG(p)=-rm2d(p)*0.9 +AFRSTEM(p)*phd(p)
              NTCHRVEG(p)=-rm3d(p)*0.9 +AFRROOT(p)*phd(p)
            ENDIF
        ELSE IF ( LFSTATUS(p).LT.3.2) THEN  !! status=3, LEAF FALL 
            IF(npd(p).GT.0.0) THEN
              NTCHLVEG(p)=AFRLEAF(p)*npd(p)
              NTCHSVEG(p)=AFRSTEM(p)*npd(p)
              NTCHRVEG(p)=AFRROOT(p)*npd(p)
            ELSE
!! liwp 2015-09-07 reduce maintenance respiration when NPP is negative
              NTCHLVEG(p)=-rm1d(p)*0.6 +AFRLEAF(p)*phd(p)
              NTCHSVEG(p)=-rm2d(p)*0.8 +AFRSTEM(p)*phd(p)
              NTCHRVEG(p)=-rm3d(p)*0.8 +AFRROOT(p)*phd(p)
            ENDIF
        ELSE IF ( LFSTATUS(p).GT.3.8 ) THEN  !! status=4, No leaves 
!  AND SINCE WE DO NOT HAVE ANY REAL LEAVES ON THEN WE DO NOT TAKE
!  INTO ACCOUNT CO2 UPTAKE BY IMAGINARY LEAVES IN CARBON BUDGET.
!  RMLVGSTP(p) SHOULD BE ZERO BECAUSE WE SET MAINTENANCE
!  RESPIRATION FROM STORAGE/IMAGINARY LEAVES EQUAL TO ZERO.
!
            NTCHLVEG(p)= 0.0
            NTCHSVEG(p)=-rm2d(p)*0.8
            NTCHRVEG(p)=-rm3d(p)*0.8
!! lwp 2010-01 for Crops, no biomass after harvest
         if(ivt(p).ge.15) then
            NTCHLVEG(p)= 0.0
            NTCHSVEG(p)= 0.0
            NTCHRVEG(p)= 0.0
         endif
!
! SINCE NO REAL LEAVES ARE ON, MAKE ALLOCATION FRACTIONS EQUAL TO ZERO.
!
            AFRLEAF(p)=0.0
            AFRSTEM(p)=0.0
            AFRROOT(p)=0.0
        ENDIF
!
          GLEAFMAS(p)=GLEAFMAS(p)+NTCHLVEG(p)
          STEMMASS(p)=STEMMASS(p)+NTCHSVEG(p)
          ROOTMASS(p)=ROOTMASS(p)+NTCHRVEG(p)
!
!           WRITE(6,1901)'NTCHLVEG = ',NTCHLVEG(p)
!           WRITE(6,1902)'LFSTATUS = ',LFSTATUS(p)
!           WRITE(6,1901)'RM1D     = ',rm1d(p)
 !   print*,'LAI,deltBm1,2,3=',AILCG(p),NTCHLVEG(p),NTCHSVEG(p),NTCHRVEG(p)
1900        FORMAT(A23,2f8.2,A10,I3)
1902        FORMAT(A11,F8.1)
1901        FORMAT(A11,F12.8)
!
          IF(ROOTMASS(p).LT.0.0)THEN
     WRITE(6,1900)'ROOTMASS LT ZERO AT ',londeg(g), latdeg(g),' FOR PFT=',ivt(p)
            WRITE(6,1901)'ROOTMASS = ',ROOTMASS(p)
          ENDIF
!! lwp 2010-01 set upper limit of Crop leafmass (LAI<=4.5, assume gdef=20,gleafmas=0.225) 
!! lwp 2016-04 set upper limit of Crop leafmass (LAI<=4.5, assume gdef=18,gleafmas=0.25) 
          if(ivt(p).eq.15 ) then    !! Crop
            if( GLEAFMAS(p).ge.0.25) then
              GLEAFMAS(p)= 0.25
              DIFF= (GLEAFMAS(p)-0.25)*0.5
              STEMMASS(p)= STEMMASS(p)+DIFF
              ROOTMASS(p)= ROOTMASS(p)+DIFF
            endif
            if(LFSTATUS(p).lt.2.5 ) then  !! with leaves on 
              STEMMASS(p)= max(0.005, STEMMASS(p) )
              ROOTMASS(p)= max(0.005, ROOTMASS(p) )
            endif
          endif
#if(defined RICEWATER)
          if(ivt(p).eq.16 ) then    !! Crop
            if( GLEAFMAS(p).gt.0.20 ) then
              GLEAFMAS(p)= 0.20
              DIFF= (GLEAFMAS(p)-0.20 )*0.5
              STEMMASS(p)= STEMMASS(p)+DIFF
              ROOTMASS(p)= ROOTMASS(p)+DIFF
            endif
            if(LFSTATUS(p).lt.2.5 ) then  !! with leaves on
              STEMMASS(p)= max(0.005, STEMMASS(p) )
              ROOTMASS(p)= max(0.005, ROOTMASS(p) )
            endif
          endif
#endif
!
!         TO AVOID OVER/UNDERFLOW PROBLEMS SET GLEAFMAS, STEMMASS, AND
!         ROOTMASS TO ZERO IF THEY GET TOO SMALL
!
          IF(GLEAFMAS(p).LT.ZERO) GLEAFMAS(p)=0.0
          IF(STEMMASS(p).LT.ZERO) STEMMASS(p)=0.0
          IF(ROOTMASS(p).LT.ZERO) ROOTMASS(p)=0.0

     enddo
   end subroutine Allocate

!-------------------------------------------------------------------------
!BOP
!
! SUBROUTINE: Phenology 
!
! !INTERFACE:

      SUBROUTINE Phenology(fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                           ANVEG, TAIR, ROOTTEMP, btran, &
                           GLEAFMAS, STEMMASS, ROOTMASS, BLEAFMAS, &
!    6 ------------------ INPUTS ABOVE THIS LINE ---------------------72  
                           LFSTATUS, LEAFLITR, STEMLITR, ROOTLITR, FLHRLOSS, & 
                           PANDAYS, COLDDAYS, maturedays, &
                           normaldays, maxdays, & 
                           livefr_stem,livefr_root, &
                           surl, rotl)
!    9 --- VARIABLES WHICH ARE UPDATED AND OUTPUTS ABOVE THIS LINE ----
!
!               CANADIAN TERRESTRIAL ECOSYSTEM MODEL (CTEM) V1.0
!               PHENOLOGY, LEAF TURNOVER & MORTALITY SUBROUTINE
!
!     15  APR. 2003 - THIS SUBROUTINE CALCULATES THE LEAF STATUS 
!     V. ARORA        FOR CTEM's PFTs AND LEAF LITTER GENERATED BY
!                     NORMAL TURNOVER OF LEAVES AND COLD AND DROUGHT 
!                     STRESS. CROP HARVEST IS ALSO MODELLED IN THIS
!                     SUBROUTINE, AND FOR GRASSES GREEN LEAVES ARE
!                     CONVERTED INTO BROWN.
!
!     INPUTS 
!
!     GLEAFMAS  - GREEN OR LIVE LEAF MASS IN KG C/M2, FOR THE 9 PFTs
!     BLEAFMAS  - BROWN OR DEAD LEAF MASS IN KG C/M2, FOR THE 9 PFTs
!     TBAR      - SOIL TEMPERATURE, K
!     THLIQ     - LIQUID SOIL MOISTURE CONTENT IN 3 SOIL LAYERS
!     WILTSM    - WILTING POINT SOIL MOISTURE CONTENT
!     FIELDSM   - FIELD CAPACITY SOIL MOISTURE CONTENT
!                 BOTH CALCULATED IN ALLOCATE SUBROUTINE
!     TAIR      - AIR TEMPERATURE, K
!     ANVEG     - NET PHOTOSYNTHESIS RATE OF CTEM's PFTs, uMOL CO2/M2.S
!     IDAY      - DAY OF YEAR
!     RADL      - LATITUDE IN RADIANS
!     ROOTTEMP  - ROOT TEMPERATURE, WHICH IS A FUNCTION OF SOIL TEMPERATURE
!                 OF COURSE, K.
!     RMATCTEM  - FRACTION OF ROOTS IN EACH SOIL LAYER FOR EACH PFT
!     STEMMASS  - STEM MASS FOR EACH OF THE 9 CTEM PFTs, Kg C/M2
!     ROOTMASS  - ROOT MASS FOR EACH OF THE 9 CTEM PFTs, Kg C/M2
!
!     UPDATES
!
!     FLHRLOSS  - FALL & HARVEST LOSS FOR BDL DCD PLANTS AND CROPS, 
!                 RESPECTIVELY, Kg C/M2.
!     PANDAYS   - COUNTER FOR POSITIVE NET PHOTOSYNTHESIS (An) DAYS FOR 
!                 INITIATING LEAF ONSET
!     LFSTATUS  - INTEGER INDICATING LEAF STATUS OR MODE
!                 1 - MAX. GROWTH OR ONSET, WHEN ALL NPP IS ALLOCATED TO
!                     LEAVES
!                 2 - NORMAL GROWTH, WHEN NPP IS ALLOCATED TO LEAVES, STEM,
!                     AND ROOT
!                 3 - FALL FOR DCD TREES/HARVEST FOR CROPS, WHEN ALLOCATION
!                     TO LEAVES IS ZERO.
!                 4 - NO LEAVES
!     COLDDAYS  - COLD DAYS COUNTER FOR TRACKING DAYS BELOW A CERTAIN 
!                 TEMPERATURE THRESHOLD FOR NDL DCD AND CROP PFTs.
!
!     OUTPUTS
!
!     LEAFLITR  - LEAF LITTER GENERATED BY NORMAL TURNOVER, COLD AND
!                 DROUGHT STRESS, AND LEAF FALL/HARVEST, Kg C/M2
!     ------------------------------------------------------------------    
!
! !DESCRIPTION:
!    PHENOLOGY, LEAF TURNOVER & MORTALITY SUBROUTINE
!
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use clmtype
     use clm_varpar   , only: numcol, numrad, nlevsoi, nlevlak
     use clm_varctl, only : nsrest
#ifdef SPMD
  use spmdMod, only : masterproc, mpicom, MPI_REAL8, MPI_INTEGER, MPI_LOGICAL
#else
  use spmdMod, only : masterproc
#endif
#if (defined COUP_CAM)
  use time_manager        , only : get_step_size, get_curr_calday, get_perp_date , &
                                   get_curr_date, get_ref_date, get_nstep, &
                                   is_perpetual
#else
  use time_manager        , only : get_step_size, get_curr_calday, &
                                   get_curr_date, get_ref_date, get_nstep 
#endif
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in)    :: TAIR    (lbp:ubp)       !
     real(r8), intent(in)    :: ANVEG   (lbp:ubp)       !
     real(r8), intent(in)    :: ROOTTEMP(lbp:ubp)       !
     real(r8), intent(in)    :: btran   (lbp:ubp)       !
     real(r8), intent(inout) :: normaldays(lbp:ubp)     !
     real(r8), intent(inout) :: maxdays(lbp:ubp)        !
     real(r8), intent(inout) :: maturedays(lbp:ubp)     !
     real(r8), intent(inout) :: PANDAYS (lbp:ubp)       !
     real(r8), intent(inout) :: COLDDAYS(lbp:ubp)       !
     real(r8), intent(inout) :: LFSTATUS(lbp:ubp)       !
     real(r8), intent(inout) :: GLEAFMAS(lbp:ubp)       !
     real(r8), intent(inout) :: STEMMASS(lbp:ubp)       !
     real(r8), intent(inout) :: ROOTMASS(lbp:ubp)       !
     real(r8), intent(inout) :: BLEAFMAS(lbp:ubp)       !
     real(r8), intent(out  ) :: FLHRLOSS(lbp:ubp)       !
     real(r8), intent(out  ) :: LEAFLITR(lbp:ubp)       !
     real(r8), intent(out  ) :: STEMLITR(lbp:ubp)       !
     real(r8), intent(out  ) :: ROOTLITR(lbp:ubp)       !
     real(r8), intent(out  ) :: livefr_STEM(lbp:ubp)
     real(r8), intent(out  ) :: livefr_ROOT(lbp:ubp)
     real(r8), intent(out  ) :: surl(lbp:ubp)
     real(r8), intent(out  ) :: rotl(lbp:ubp)

! !CALLED FROM:
! subroutine CanopyFluxes in this module
!
! !REVISION HISTORY:
! author:            Vivek Arora 
! revised:           Weiping Li, 2009-07, 2013-05

! !LOCAL VARIABLES:
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)     ! pft's column index
     integer , pointer :: pgridcell(:)   ! pft's gridcell index
     integer , pointer :: ivt(:)         ! pft vegetation type
     real(r8), pointer :: lon(:)         ! latitude (radians)
     real(r8), pointer :: lat(:)         ! latitude (radians)
     real(r8), pointer :: latdeg(:)      ! latitude (degrees)
     real(r8), pointer :: londeg(:)        ! latitude (degrees)
     real(r8), pointer :: h2osoi_vol(:,:) ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
     real(r8), pointer :: watsat(:,:)     ! volumetric soil water at saturation (porosity) (nlevsoi)
     real(r8), pointer :: forc_q(:)       !atmospheric specific humidity (kg/kg)
     real(r8), pointer :: forc_t(:)       !atmospheric temperature (Kelvin) 
     real(r8), pointer :: t_grnd(:)       !ground temperature (Kelvin)
     real(r8), pointer :: t_soisno(:,:)    ! soil temperature (Kelvin)  (-nlevsno+1:nlevsoi)
     real(r8), pointer :: t_ref2m(:)     ! 2 m height surface air temperature (Kelvin)
     real(r8), pointer :: elai(:)        ! one-sided leaf area index
     real(r8), pointer :: esai(:)        ! one-sided leaf area index
     real(r8), pointer :: alai(:)        ! real LAI calculated by AVIM 
     real(r8), pointer :: bm11(:)        ! crop leaf mass when harvest begins 
     real(r8), pointer :: bm21(:)        ! crop stem mass when harvest begins 
     real(r8), pointer :: bm31(:)        ! crop root mass when harvest begins 
     real(r8), pointer :: los21(:)       ! harvest loss of crop stem
     real(r8), pointer :: los31(:)       ! harvest loss of crop root

!!! lwp 2009-07-30 to add wiltsm and fieldsm
  real(r8), pointer :: wiltsm (:,:)      ! wilting point soil moisture (nlevsoi)
  real(r8), pointer :: fieldsm(:,:)      ! field capacity soil moisture (nlevsoi)
  real(r8), pointer :: rootfr (:,:)      ! fraction of roots in each soil layer

! !OTHER LOCAL VARIABLES:
     integer  :: f,p,c,g ! indices
     integer  :: CHKMODE  (lbp:ubp)
     real(r8) :: AILCG    (lbp:ubp)
     real(r8) :: AILCB    (lbp:ubp)
     real(r8) :: SLA      (lbp:ubp)
     real(r8) :: beta_drought(lbp:ubp)
     real(r8) :: NRMLLOSS (lbp:ubp)
     real(r8) :: DRGTSTRS (lbp:ubp)
     real(r8) :: DRGTLSRT (lbp:ubp)
     real(r8) :: DRGTLOSS (lbp:ubp)
     real(r8) :: COLDSTRS (lbp:ubp)
     real(r8) :: COLDLSRT (lbp:ubp)
     real(r8) :: COLDLOSS (lbp:ubp)
     real(r8) ::  LFTHRS  (lbp:ubp)
     real(r8) ::  LFTHRS1 (lbp:ubp)
     real(r8) :: DAYLNGTH (lbp:ubp)  !! length of calendar day
     real(r8) :: BETADRGT (lbc:ubc,nlevsoi)
     real(r8) :: THLIQ    (lbc:ubc,nlevsoi)
!! liwp 2017-01-25 for Turnover
     real(r8) :: NRMLSMLR (lbp:ubp)
     real(r8) :: NRMLRTLR (lbp:ubp)
!! liwp 2017-01-25 for Turnover
     real(r8) :: TERM, DECLI, ROOTPERCENT, DTIME, TA, DAY, CALDAY, RADL
     real(r8) :: THETA
     real(r8) :: FRACBOFG 
     real(r8) :: GLEAFMAS0(lbp:ubp)  !! the initial leaf mass of crop when harvest began 
!
     INTEGER ::   J, KK, N, M
     PARAMETER (KK=16)  ! PRODUCT OF 16 CLASS PFTs AND L2MAX
     real(r8) :: LFESPANY(KK),  DRLSRTMX(KK),  CDLSRTMX(KK), & 
                    DRGTA(KK),     COLDA(KK),      GDEF(KK), & 
                 LWRTHRSH(KK),  HARVTHRS(KK),  THRPRCNT(KK), &
                      ETA(KK),     KAPPA(KK), DAYSCHK(KK)
     real(r8) :: COLDTHRS(4), FLHRSPAN(4), COLDLMT(4)
     real(r8) :: ZERO, ROOTHRSH, ZEROlai, diff
     
     integer :: nstep         ! time step number
     integer :: kyr          ! year (0, ...) for nstep+1
     integer :: kmo          ! month (1, ..., 12)
     integer :: kda          ! day of month (1, ..., 31)
     integer :: ksec         ! seconds into current date for nstep+1
     integer :: kmoda
!
!! liwp 2017-01-25 for Turnover
!
      REAL(r8) ::  STEMLIFE(KK), ROOTLIFE(KK), STMHRSPN
!     ------------------------------------------------------------------
!
!     PARAMETERS USED. ALSO NOTE THE STRUCTURE OF PARAMETER VECTORS
!     WHICH CLEARLY SHOWS THE CLASS PFTs (ALONG ROWS) AND CTEM
!     SUB-PFTs (ALONG COLUMNS)
!
!     NEEDLE LEAF |  EVG(temp) EVGboreal DCDboreal
!     BROAD LEAF  |  EVG(trop) EVGtemp   DCD-DRY  DCD-temp DCD-COLD
!     BROAD SHRUB |  EVG       DCDtemp   DCDboreal
!     GRASSES     |   C3arctic  C3       C4
!     CROPS       |   C3        C4       ---
!
!     STEMLIFE, TURNOVER TIME SCALE FOR STEM FOR DIFFERENT PFTs
      DATA  STEMLIFE/37.0, 42.0, 50.0, &
                     39.0, 37.0, 38.0, 40.0, 38.0, &
                     15.0, 15.0, 15.0, &
                     0.00, 0.00, 0.00, &
                     5.0, 12.0/
!     ROOTLIFE, TURNOVER TIME SCALE FOR ROOT FOR DIFFERENT PFTs
      DATA  ROOTLIFE/4.5, 5.0, 6.0, &
                     2.9, 2.8, 4.0, 4.3, 4.5, &
                     4.0, 5.0, 4.5, &
                     1.5, 1.8, 1.7, &
                     1.0, 2.5/
!
!     STEM HARVEST SPAN. SAME AS CROP HARVEST SPAN. PERIOD IN DAYS
!     OVER WHICH CROPS ARE HARVESTED.
!     DATA STMHRSPN/15.0/
!     DATA ZERO/1E-20/
!     ------------------------------------------------------------------
!                     CONSTANTS AND PARAMETERS USED 
!
!     ALSO NOTE THE STRUCTURE OF PARAMETER VECTORS WHICH CLEARLY 
!     SHOWS THE CLASS PFTs (ALONG ROWS) AND CTEM SUB-PFTs (ALONG 
!     COLUMNS)
!
!     NEEDLE LEAF |  EVG(temp) EVGboreal DCDboreal
!     BROAD LEAF  |  EVG(trop) EVGtemp   DCD-DRY  DCD-temp DCD-COLD(boreal)
!     BROAD SHRUB |  EVG(temp) DCDtemp   DCDboreal
!     GRASSES     |   C3arctic  C3       C4
!     CROPS       |   C3        C4       ---
!
!     MAX. LEAF LOSS RATE FOR COLD STRESS FOR ALL 16 PFTs, (1/DAY)
      DATA CDLSRTMX/ 0.005,0.005,0.01,             &
                     0.05, 0.10, 0.15, 0.10, 0.10, & 
                     0.10, 0.05, 0.06,             &
                     0.09, 0.12, 0.16,             &
                     0.15, 0.15/
!
!     MAX. LEAF LOSS RATE FOR DROUGHT STRESS FOR ALL 16 PFTs, (1/DAY)
      DATA DRLSRTMX/0.002, 0.003, 0.001,               &
                    0.001, 0.001, 0.008, 0.008, 0.004, &
                    0.005, 0.002, 0.008,               &
                    0.009, 0.006, 0.020,               & 
                    0.005, 0.003/
!
!     PARAMETER DETERMINING HOW FAST SOIL DRYNESS CAUSES LEAVES TO FALL
      DATA DRGTA/3.0, 3.0, 3.0,           &
                 4.0, 4.0, 3.0, 3.0, 3.0, &
                 3.0, 3.0, 3.0,           &
                 3.0, 3.0, 3.0,           &
                 3.0, 3.0/
!
!     PARAMETER DETERMINING HOW FAST COLD TEMPERATURES CAUSES LEAVES TO FALL
      DATA COLDA/3.0, 3.0, 3.0,           &
                 3.0, 3.0, 3.0, 3.0, 3.0, &
                 3.0, 3.0, 3.0,           &
                 3.0, 3.0, 3.0,           &
                 3.0, 3.0/
!
!     LOWER TEMPERATURE THRESHOLD FOR CTEM's 9 PFTs. THESE ARE USED
!     TO ESTIMATE COLD STRESS RELATED LEAF LOSS RATE (DEGREE C)
      DATA LWRTHRSH/-13.0, -15.0, -18.0,          &
                      5.0,   0.0,  5.0, 0.0,-1.0, &
                      0.0,  -1.0, -3.0,           &
                      0.1,   0.1,  5.0,           &
                      5.0,   5.0/
!
!     NUMBER OF DAYS OVER WHICH TO CHECK IF NET PHOTOSYNTHETIC RATE IS
!     POSITIVE BEFORE INITIATING LEAF ONSET
      DATA DAYSCHK/7., 7., 7.,         &
                   7., 7., 7., 7., 7., &
                   7., 7., 7.,         &
                   7., 7., 7.,         &
                   7., 5./
!
!     NO. OF DAYS FOR WHICH SOME TEMPERATURE HAS TO REMAIN BELOW 
!     A GIVEN THRESHOLD FOR INITIATING A PROCESS
!     1. -5 C THRESHOLD FOR INITIATING "LEAF FALL" MODE FOR NDL DCD TREES (7 days)
!     2.  5 C THRESHOLD FOR INITIATING "LEAF FALL" MODE FOR BDL DCD TREES (6 days)
!     3.  8 C THRESHOLD FOR INITIATING "HARVEST" MODE FOR CROPS (5 days)
!     THE ARRAY COLDDAYS(I,2) TRACKS DAYS CORRESPONDING TO THESE THRESHOLDS
!     DATA  COLDLMT/ 7.0, 5.0/        !! DAYS (CTEM original)
!     DATA COLDTHRS/-5.0, 8.0/        !! Celsius degree (CTEM original)
!zhangyw add for (4) is rice
      DATA  COLDLMT/ 7.0, 7.0, 5.0, 5.0/   !! DAYS (AVIM) 
      DATA COLDTHRS/-6.0,-6.0, 8.0, 10.0/  !! Celsius degree (AVIM)
!
!     LAI THRESHOLD FOR HARVESTING CROPS. VALUES ARE ZERO FOR ALL PFTs
!     OTHER THAN C3 AND C4 CROPS.
      DATA HARVTHRS/0.0, 0.0, 0.0, &
                    0.0, 0.0, 0.0, 0.0, 0.0, &
                    0.0, 0.0, 0.0, &
                    0.0, 0.0, 0.0, &
                    4.5, 3.5/
!
!     FALL SPAN (TIME IN DAYS OVER WHICH NDL/BDL COLD DCD PLANTS SHED THEIR LEAVES, ~30 DAYS) 
!     AND HARVEST SPAN (TIME IN DAYS OVER WHICH CROPS ARE HARVESTED, ~15 DAYS), 
!zhangyw add for (4) is rice harvested ~7 days
      DATA FLHRSPAN/50.0, 40.0, 15.0, 7.0/
!
!     PERCENTAGE OF MAX. LAI THAT CAN BE SUPPORTED WHICH IS USED AS A
!     THRESHOLD FOR DETERMINING LEAF PHENOLOGY STATUS
      DATA THRPRCNT/40.0, 40.0, 45.0,             & ! needleleaf
                    40.0, 45.0, 50.0, 50.0, 50.0, & ! broadleaf
                    45.0, 45.0, 45.0,             & ! Shrubs
                    40.0, 40.0, 40.0,             & ! grasses
                    50.0, 50.0/                     ! crops

      DATA LFESPANY/2.0, 1.7, 1.2,            & ! needleleaf
                    1.8, 1.4, 1.0, 1.5, 1.0,  & ! broadleaf
                    1.5, 1.1, 1.0,            & ! Shrubs
                    1.3, 1.3, 1.3,            & ! grasses
                    1.0, 1.5/                   ! crops

      DATA ETA/15.0, 20.0, 30.0,             &
               30.0, 30.0, 35.0, 40.0, 40.0, &
                8.0, 16.0,  6.0,             &
                3.0,  3.0,  3.0,             &
                5.0,  5.0/

      DATA KAPPA/1.6, 1.6, 1.6,           &
                 1.6, 1.6, 1.6, 1.6, 1.6, &
                 1.5, 1.5, 1.5,           &
                 1.2, 1.2, 1.2,           &
                 1.6, 1.6/
 
!! lwp 2009-07 to use CLM3 16 PFTs rather than AVIM 13 biome types, also 1/gdef of old AVIM
!  data gdef/17.0, 18.0, 20.0, 17.0, 17.0, 22.0, 20.0, 20.0, &  !! 2013-04-25
!            20.0, 20.0, 17.0, 18.0, 18.0, 18.0, 19.0, 20.0/
!  data gdef/19.0, 19.0, 20.0, 20.0, 19.0, 21.0, 21.0, 21.0, &  !! 2016-07-06
!            20.0, 19.0, 18.0, 19.0, 20.0, 18.0, 20.0, 20.0/
   data gdef/19.0, 19.0, 20.0, 20.0, 19.0, 19.0, 20.0, 20.0, &  !! 2017-03-03
             19.0, 19.0, 18.0, 19.0, 19.0, 18.0, 20.0, 20.0/
   data FRACBOFG/0.55/
!
!     ROOT TEMPERATURE THRESHOLD FOR INITIATING LEAF ONSET FOR COLD
!     BROADLEAF DECIDUOUS PFT, DEGREES CELCIUS
      DATA ROOTHRSH/2.0/       !! CTEM is 2 C, lower this to extend growing season
      DATA ZERO/1E-4/
!     DATA ZEROlai/0.05/
      DATA ZEROlai/0.102/

     REAL(r8),parameter :: PI=3.1415926535898d0
!     ---------------------------------------------------------------
           ! Set statement functions

     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     lon         => clm3%g%lon
     lat         => clm3%g%lat
     latdeg      => clm3%g%latdeg
     londeg      => clm3%g%londeg

     h2osoi_vol     => clm3%g%l%c%cws%h2osoi_vol
     elai           => clm3%g%l%c%p%pps%elai
     alai           => clm3%g%l%c%p%pcf%alai
     bm11           => clm3%g%l%c%p%pcf%bm11
     bm21           => clm3%g%l%c%p%pcf%bm21
     bm31           => clm3%g%l%c%p%pcf%bm31
     los21          => clm3%g%l%c%p%pcf%los21
     los31          => clm3%g%l%c%p%pcf%los31

!!! lwp 2009-07-30 to add wiltsm and fieldsm
   wiltsm          => clm3%g%l%c%cps%wiltsm
   fieldsm         => clm3%g%l%c%cps%fieldsm
   rootfr          => clm3%g%l%c%p%pps%rootfr

!! lwp 2009-07-24 added calendar day information
      dtime = get_step_size()
      calday = get_curr_calday(offset=0)

  nstep = get_nstep()
  
#ifdef COUP_CAM
      if ( is_perpetual() ) then
         call get_perp_date(kyr, kmo, kda, ksec, offset=int(dtime))
      else
         call get_curr_date(kyr, kmo, kda, ksec, offset=int(dtime))
      end if
#else
      call get_curr_date(kyr, kmo, kda, ksec, offset=int(dtime))
#endif
      kmoda = kmo*100 + kda

!     INITIALIZE REQUIRED ARRAYS TO ZERO

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        g = pgridcell(p)
          beta_drought(p) = 0._r8 ! (1 - DROUGHT STRESS)
          CHKMODE ( p )=0         ! INDICATOR FOR MAKING SURE THAT LEAF STATUS IS UPDATED 
          LEAFLITR( p )=0.0       ! LEAF LITTER
          NRMLLOSS( p )=0.0       ! LEAF LOSS DUE TO NORMAL TURNOVER
          DRGTSTRS( p )=0.0       ! DROUGHT STRESS TERM
          DRGTLSRT( p )=0.0       ! DROUGHT LOSS RATE
          DRGTLOSS( p )=0.0       ! LEAF LOSS DUE TO DROUGHT STRESS
          COLDSTRS( p )=0.0       ! COLD STRESS TERM
          COLDLSRT( p )=0.0       ! COLD LOSS RATE
          COLDLOSS( p )=0.0       ! LEAF LOSS DUE TO COLD STRESS
          LFTHRS  ( p )=0.0       ! THRESHOLD LAI FOR FINDING LEAF STATUS
          LFTHRS1 ( p )=0.0       ! THRESHOLD LAI FOR FINDING LEAF STATUS
          FLHRLOSS( p )=0.0       ! loss of leaf during leaffall or crop harvest
!     ------------------------------------------------------------------
!
!     CONVERT GREEN LEAF MASS INTO LEAF AREA INDEX USING SPECIFIC LEAF
!     AREA (SLA, M2/KG C) ESTIMATED USING LEAF LIFE SPAN. SEE BIO2STR
!     SUBROUTINE FOR MORE DETAILS. 
!
!! lwp 2013-05-20, use green leaf mass rather than LAI to determine the status of growth
        AILCG(p)=gdef(ivt(p))*GLEAFMAS(p)
!       AILCB(p)=gdef(ivt(p))*BLEAFMAS(p)*FRACBOFG
!     ALSO FIND THRESHOLD LAI AS A FUNCTION OF STEM+ROOT BIOMASS
!     WHICH IS USED TO DETERMINE LEAF STATUS
!! lwp 2009-07 rewrite the following using LOG, EXP rather than power function
!!     LFTHRS(p)=((STEMMASS(p)+ROOTMASS(p))/ETA(ivt(p)))**(1.0/KAPPA(ivt(p)))
!! lwp 2009-12 make sure LFTHRS>= 0.0
        if ((STEMMASS(p)+ROOTMASS(p)) .GT. 0.1) then
       LFTHRS1(p)=ALOG((STEMMASS(p)+ROOTMASS(p))/ETA(ivt(p))) / KAPPA(ivt(p))
        else
       LFTHRS1(p)=ALOG(0.1/ETA(ivt(p))) / KAPPA(ivt(p))
        endif
       LFTHRS (p)=(THRPRCNT(ivt(p))/100.0)*gdef(ivt(p)) * EXP(LFTHRS1(p))
!! lwp 2013-05-20 set the threshold lai for crop for the time being 
      if (ivt(p).eq.15 ) then
       LFTHRS (p)= 2.5 
      endif
!! zhangyw 2015-05-20 set the threshold lai for rice for the time being
#if(defined RICEWATER)
      if (ivt(p).eq.16 ) then
       LFTHRS (p)= 2.0
      endif
#endif
    enddo   !! end of initialization

!dir$ concurrent
!cdir nodep
    do  j=1,nlevsoi
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
          BETADRGT(c, J) = 0._r8        !! (1 - DROUGHT STRESS)
     enddo
    enddo
!
!     INITIALIZATION ENDS    
!
!     USING GREEN LEAF AREA INDEX (AILCG) DETERMINE THE LEAF STATUS FOR
!     EACH PFT. LOOPS 190 AND 200 THUS INITIALIZE LFSTATUS, IF THIS
!     THIS INFORMATION IS NOT PASSED SPECIFICALLY AS AN INITIALIZATION
!     QUANTITY. 
!
!! lwp 2009-11-2 increase "enddo" to interrupt do loops
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
           IF ( ivt(p).eq.1 .or.ivt(p).eq.2 .or.ivt(p).eq.4 ) THEN 
              LFSTATUS(p)=2.0                 ! NORMAL GROWTH for PFT=1,2,4
           ENDIF                                    
         IF(LFSTATUS(p).LT.0.1)THEN           !! i.e, LFSTATUS=0
           IF (AILCG(p).GT.LFTHRS(p) ) THEN 
              LFSTATUS(p)=2.0       
           ELSE                                  
              LFSTATUS(p)=4.0             !TREAT THIS AS NO LEAVES
              maturedays(p)=0.0
              normaldays(p)=0.0
              maxdays(p) = 0.0
    !         pandays(p) = 0.0
           ENDIF                          !START GROWING IF POSSIBLE 
         ENDIF                                    
!
!     KNOWING LFSTATUS (AFTER INITIALIZATION ABOVE OR USING VALUE FROM
!     FROM PREVIOUS TIME STEP) WE DECIDE IF WE STAY IN A GIVEN LEAF
!     MODE OR WE MOVE TO SOME OTHER MODE.
!
!     WE START WITH THE "NO LEAVES" MODE
!     ----------------------------------
!
!     ADD ONE TO PANDAYS(I,J) IF DAILY An IS POSITIVE, OTHERWISE SET IT TO ZERO.
!
        if (ivt(p).le.15) then
          IF( ANVEG(p) .GT. ZERO ) THEN
            PANDAYS(p)=PANDAYS(p)+1
             IF( PANDAYS(p).GT.DAYSCHK(ivt(p)) )THEN
!! lwp 2009-07 The following line implies that pandays never more than dayschk
              PANDAYS(p)= DAYSCHK(ivt(p)) 
             ENDIF
          ELSE
            PANDAYS(p)=0
          ENDIF
        end if          !! cumulate pandays for ivt=1~15
#if(defined RICEWATER)
          if (ivt(p).ge.16) then
!          IF( ANVEG(p) .GT. ZERO .and.dailytair(p).ge.278.and.calday.ge.75.and.calday.le.310) THEN
!          IF( ANVEG(p) .GT. ZERO .and.dailytair(p).ge.278.and.kmoda.ge.315.and.kmoda.le.1105) THEN
           IF( ANVEG(p) .GT. ZERO .and.TAIR(p).ge.278.and.kmoda.ge.315.and.kmoda.le.1105) THEN
            PANDAYS(p)=PANDAYS(p)+1
            IF( PANDAYS(p).GT.DAYSCHK(ivt(p)) )THEN
              PANDAYS(p)= DAYSCHK(ivt(p))
            ENDIF
          ELSE
            PANDAYS(p)=0
          ENDIF
          end if
#endif    
     enddo        !! end of counting PANDAYS
!
!     IF IN "NO LEAVES" MODE CHECK IF An HAS BEEN POSITIVE OVER LAST
!     DAYSCHK(J) DAYS TO MOVE INTO "MAX. GROWTH" MODE. IF NOT WE STAY
!     IN "NO LEAVES" MODE. ALSO SET THE CHKMODE(I,J) SWITCH TO 1.
!
!! lwp 2013-05-21 increase "enddo" to interrupt do loops
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
         TA = TAIR(P)
          IF(CHKMODE(p).EQ.0.AND.LFSTATUS(p).GT.3.5)THEN  !! LFSTATUS=4
            IF( PANDAYS(p).GE.DAYSCHK(ivt(p)) )THEN
              LFSTATUS(p)=1.0      ! SWITCH TO "MAX. GROWTH"
              CHKMODE(p)=1         ! MODE CHECKED, NO MORE CHECKS FURTHER DOWN
            ELSE
              LFSTATUS(p)=4.0      ! otherwise stay in "NO LEAVES" MODE
              CHKMODE(p)=1         ! MODE CHECKED, NO MORE CHECKS FURTHER DOWN
!! liwp 2015-09-18 no pandays of deciduous PFT in "no leaf" mode during winter
     if ( latdeg(g).ge.30 ) then
       if ( calday.lt.75 .or. calday.gt.260) then  !! northern hemisphere autumn-winter 
            pandays(p) = 0.
       endif
     else if ( latdeg(g).ge.5 .and. latdeg(g).lt.30 ) then
       if ( calday.lt.70 .or. calday.gt.270) then  !! northern hemisphere winter 
            pandays(p) = 0.
       endif
     endif
     if ( latdeg(g).lt.-30 ) then
       if ( calday.gt.80.and.calday.lt.257) then  !! southern hemisphere winter  
            pandays(p) = 0.
       endif
     else if ( latdeg(g).ge.-30 .and. latdeg(g).lt.-5 ) then
       if ( calday.gt.90.and.calday.lt.252) then  !! southern hemisphere winter  
            pandays(p) = 0.
       endif
     endif
            ENDIF
          ENDIF                   !! end of status=4
!!zhangyw add 20150520 for rice
#if(defined RICEWATER)
      if (ivt(p).eq.16) then
!      if ( AILCG(p).LE. 0.1 .and.calday.ge.155.and.calday.le.222.and.abs(LFSTATUS(p)-3.0).lt.ZERO) then     !! for rice zhangyyw
!     if ( AILCG(p).LE. 0.1 .and.kmoda.ge.605.and.kmoda.le.810.and.abs(LFSTATUS(p)-3.0).lt.ZERO) then     !! for rice zhangyyw
        if ( AILCG(p).LE. 0.1 .and.abs(LFSTATUS(p)-3.0).lt.ZERO) then     !! for rice zhangyyw
         normaldays(p) = 0.0
         maturedays(p) = 0.0
         maxdays(p)    = 0.0
         LFSTATUS(p)   = 1.0
         pandays(p)    = 0.0
         bm11(p)       = 0.0
         CHKMODE(p)    = 1
      endif
      endif
#endif
!
!     FIND DAY LENGTH USING DAY OF YEAR AND LATITUDE. THIS IS TO BE USED FOR
!     INITIATING LEAF OFFSET FOR BROAD LEAF DCD TREES.
!
       RADL= lat(g)
       THETA=0.2163108 + 2*ATAN(0.9671396*TAN(0.0086*(calday-186.0)))
       DECLI=ASIN(0.39795*COS(THETA))    !DECLINATION
       TERM=(SIN(RADL)*SIN(DECLI))/(COS(RADL)*COS(DECLI))
       TERM=MAX(-1.0,MIN(TERM,1.0))
       DAYLNGTH(p)=24.0-(24.0/PI)*ACOS(TERM)
!
!     EVEN IF PANDAYS CRITERIA HAS BEEN SATISFIED DO NOT GO INTO MAX. GROWTH
!     MODE IF ENVIRONMENTAL CONDITIONS ARE SUCH THAT THEY WILL FORCE LEAF FALL
!     OR HARVEST MODE.
!
!     SIMILAR TO THE WAY WE COUNT NO. OF DAYS WHEN An IS POSITIVE, WE
!     FIND NO. OF DAYS WHEN TEMPERATURE IS BELOW -5 C. WE NEED THIS TO
!     DETERMINE IF WE GO INTO "LEAF FALL" MODE FOR NEEDLE LEAF DCD TREES.
!
!     ALSO ESTIMATE NO. OF DAYS BELOW 8 C. WE USE THESE DAYS TO DECIDE 
!     IF ITS COLD ENOUGH TO HARVEST CROPS.
!
     IF( LFSTATUS(p).lt.1.2 .AND. CHKMODE(p).EQ.1 ) THEN  !! for maxgrowth mode
!       NEEDLE LEAF DCD
!! lwp 2013-10-17 lower the threshold to restrict the transition from maxgrowth to noleaf
!! lwp 2013-10-17 COLDTHRS(1)=-8.0 C, rather than -5.0 C
      if ( ivt(p).eq.3 ) then
        IF(TA.LT.(COLDTHRS(1)+273.16) )  THEN
          LFSTATUS(p)=4.0
           CHKMODE(p)=1         ! MODE CHECKED, NO MORE CHECKS FURTHER DOWN
          COLDDAYS(p)=COLDDAYS(p)+1
          COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(1) )
        ELSE
          COLDDAYS(p)=0
        ENDIF
      endif
!
!       CROPS
!! liwp 2016-04 move PFT=16 (rice) to the following part
      if (ivt(p).eq.15) then
!! lwp 2009-11-26 to seperate crops leafstatus according to latitudes
        if(abs(latdeg(g)).gt.50.) then
          IF(TA.LT.(COLDTHRS(3)+ 273.16)) THEN
            LFSTATUS(p)=4.0            !! go into "no leaf" from max growth
          ENDIF
        elseif(abs(latdeg(g)).gt.40.) then
          IF(TA.LT.(COLDTHRS(3)+ 0.5 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        elseif(abs(latdeg(g)).gt.30.) then
          IF(TA.LT.(COLDTHRS(3)+ 1.0 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        elseif(abs(latdeg(g)).gt.25.) then
          IF(TA.LT.(COLDTHRS(3)+ 1.5 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        elseif(abs(latdeg(g)).gt.20.) then
          IF(TA.LT.(COLDTHRS(3)+ 2.0 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        elseif(abs(latdeg(g)).gt.15.) then
          IF(TA.LT.(COLDTHRS(3)+ 2.5 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        elseif(abs(latdeg(g)).gt.10.) then
          IF(TA.LT.(COLDTHRS(3)+ 3.0 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        else               !! within 10S~10N
          IF(TA.LT.(COLDTHRS(3)+ 3.5 +273.16)) THEN
            LFSTATUS(p)=4.0
          ENDIF
        endif               !! end of latitudes cycling 

        if( LFSTATUS(p).gt.3.2 .and. LFSTATUS(p).lt.4.2 ) then   !! status=4
!! liwp 2016-04-09 simplified the COLDDAYS counting whatever latitudes 
            COLDDAYS(p)=COLDDAYS(p)+1
            COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(3) )
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
              maturedays(p)=0.0
              normaldays(p)=0.0
              maxdays(p)=0.0
              CHKMODE(p)= 1
        else
            COLDDAYS(p)=0
        endif
      endif       !! end of crops, ivt=15,16

#if(defined RICEWATER)
      if (ivt(p).eq.16) then
          if (kmoda.le.320.) then
             LFSTATUS(p) = 4.0
          endif
          if ( AILCG(p).LE. 0.1 .and.kmoda.ge.901) then     !! for rice zhangyyw
            LFSTATUS(p) = 4.0
          endif
          if( LFSTATUS(p).gt.3.2 .and. LFSTATUS(p).lt.4.2 ) then   !! status=4
            COLDDAYS(p)=COLDDAYS(p)+1
            COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(3) )
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
              maturedays(p)=0.0
              normaldays(p)=0.0
              maxdays(p)=0.0
              CHKMODE(p)= 1
          else
             COLDDAYS(p)=0
          endif
      endif       !! end of rice, ivt=16
#endif
!
!       BROAD LEAF DCD DRY
    if ( ivt(p).eq.6 ) then
        IF(ROOTTEMP(p).LT.(2.0+ 273.16).OR. &
          (DAYLNGTH(p).LT.11.5 .AND.ROOTTEMP(p).LT.(6.0+273.16))) THEN
            LFSTATUS(p)=4.0          !! go to "no leaf" from max growth
            CHKMODE(p) = 1
            maxdays(p) = 0.0
            normaldays(p)=0.0
        ENDIF
    endif
!       BROAD LEAF DCD CLD 
   if ( ivt(p).eq.7 ) then
     if ( abs(latdeg(g)) .ge. 35.0 ) then
       IF( DAYLNGTH(p).LT.11.0 .OR. ROOTTEMP(p).LT.(3.0+273.16) ) THEN
            LFSTATUS(p)=4.0          !! go to "no leaf" from max growth
       ENDIF
     elseif ( abs(latdeg(g)).ge.25.0 .and. abs(latdeg(g)).lt.35.0 ) then
       IF( DAYLNGTH(p).LT.11.5 .OR. ROOTTEMP(p).LT.(7.0+273.16) ) THEN
            LFSTATUS(p)=4.0
       ENDIF
     elseif ( abs(latdeg(g)).ge.15.0 .and. abs(latdeg(g)).lt.25.0 ) then
       IF( DAYLNGTH(p).LT.11.9 .OR. ROOTTEMP(p).LT.(10.0+273.16) ) THEN
            LFSTATUS(p)=4.0
       ENDIF
     else                 !! within 15S~15N
       IF( DAYLNGTH(p).LT.11.85 .OR. btran(p).lt.0.45 ) THEN
            LFSTATUS(p)=4.0
       ENDIF
     endif   !! endif latitude loop 
       if ( LFSTATUS(p) .gt. 3.8 ) then
            maxdays(p) = 0.0
            normaldays(p)=0.0
            CHKMODE(p) = 1
       endif
   endif                  !! endif ivt(p)=7
!
!       BROAD LEAF DCD CLD
    if ( ivt(p).eq.8 .or. ivt(p).eq.11 ) then
         IF(TA.LT.(COLDTHRS(2)+273.16).or.ROOTTEMP(p).LT.(3.0+273.16) &
            .or.(DAYLNGTH(p).LT.11.5.AND.ROOTTEMP(p).LT.(5.0+273.16)) ) THEN
            LFSTATUS(p)=4.0
            CHKMODE(p) = 1
            maxdays(p) = 0.0
            normaldays(p)=0.0
            COLDDAYS(p)=COLDDAYS(p)+1
            COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(2) )
         ELSE
            COLDDAYS(p)=0
         ENDIF
    endif
!
    if ( ivt(p).eq.10 ) then
          IF(ROOTTEMP(p).LT.(2.0+273.16) .OR. &
            (DAYLNGTH(p).LT.11.2.AND.ROOTTEMP(p).LT.(4.0+273.16))) THEN
            LFSTATUS(p)=4.0
            maxdays (p)=0.0
            normaldays(p)=0.0
            CHKMODE(p) = 1
          ENDIF
    endif
   ENDIF    !!  end of "recheck status4 (no leaves) transfer into status1 (maxgrowth)"
  enddo     !!  end of STATUS=4 do loop
!
!     IF IN "MAX GROWTH" MODE
!     ------------------------
!
!     IF MODE HASN'T BEEN CHECKED AND WE ARE IN "MAX. GROWTH" MODE, THEN 
!     CHECK IF WE ARE ABOVE PFT-DEPENDENT LAI THRESHOLD. IF LAI IS MORE
!     THEN THIS THRESHOLD WE MOVE INTO "NORMAL GROWTH" MODE, OTHERWISE
!     WE STAY IN "MAX GROWTH" MODE SO THAT LEAVES CAN GROW AT THEIR
!     MAX. CLIMATE-DEPENDENT RATE
!
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
         TA = TAIR(P)
      IF( CHKMODE(p).EQ.0.AND.LFSTATUS(p).lt.1.2 ) THEN  !! begin status=1 
          FLHRLOSS(p)= 0.0
          bm11 (p) =0.0
          los21(p) = 0.0
          los31(p) = 0.0
           IF( AILCG(p).LE.ZEROlai ) THEN
              LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
              CHKMODE(p) =1
    !         PANDAYS(p)=0
              maxdays(p) = 0.0
              normaldays(p)=0.0
           ELSE IF( AILCG(p).GE.(LFTHRS(p)+0.5) ) THEN
              LFSTATUS(p)=2.0      ! SWITCH TO "NORMAL GROWTH" MODE
              CHKMODE(p) =1
           ELSE 
              LFSTATUS(p)=1.0      ! STAY IN "MAX. GROWTH" MODE
              CHKMODE(p) =1
              maxdays(p)=maxdays(p)+1
              colddays(p) = 0.0
           ENDIF
!! liwp 2016-04-07 adjust phenology status for deciduous plant types
!  FOR DCD TREES GO INTO "LEAF FALL" MODE DIRECTLY FROM "MAX. GROWTH" MODE.
!
!           NDL DCD
!! lwp 2013-10-17 to restrict the maxgrowth to leaffall of NDT
          if( ivt(p).eq.3 ) then
            IF( AILCG(p).GT.ZEROlai.and.AILCG(p).LT.LFTHRS(p)  &
               .AND.COLDDAYS(p).GE.COLDLMT(1) ) THEN
             LFSTATUS(p)=3.0        ! GO INTO "LEAF FALL" from "MAXGROWTH"
             maxdays(p)=maxdays(p)-1
             CHKMODE(p)=1         
            ENDIF
          endif
!           Broadleaf Evergreen Temperate
          if( ivt(p).eq.5 ) then
            IF( DAYLNGTH(p).GT.12.5 .and. AILCG(p).GT.4.0 ) then 
             LFSTATUS(p)=2.0        ! go to "NORMAL GROWTH"
            ELSE
             LFSTATUS(p)=1.0        ! stay in "MAX GROWTH"
            ENDIF
             CHKMODE(p)=1         
          endif

!           BDL DCD DRY
    if (ivt(p).eq.6) then
      if ( abs(latdeg(g)) .ge. 25.0 ) then
        IF(AILCG(p).GT.ZEROlai .AND.DAYLNGTH(p).LT.11.6  &
           .and. ROOTTEMP(p).LT.(15.0+273.16) ) THEN
           LFSTATUS(p)=3.0      ! GO INTO "LEAF FALL" from MAX GROWTH
        ENDIF
      elseif ( abs(latdeg(g)).ge.20.0 ) then 
        IF(AILCG(p).GT.ZEROlai .AND.DAYLNGTH(p).LT.11.6  &
           .and. ROOTTEMP(p).LT.(16.0+273.16) ) THEN
           LFSTATUS(p)=3.0         
        ENDIF
      elseif ( abs(latdeg(g)).ge.15.0 ) then 
        IF(AILCG(p).GT.ZEROlai .AND.DAYLNGTH(p).LT.11.8  &
           .and. ROOTTEMP(p).LT.(16.5+273.16) ) THEN
           LFSTATUS(p)=3.0         
        ENDIF
      elseif ( abs(latdeg(g)).ge.10.0 ) then 
        IF(AILCG(p).GT.ZEROlai .AND.DAYLNGTH(p).LT.11.9  &
           .and. ROOTTEMP(p).LT.(17.5+273.16) ) THEN
           LFSTATUS(p)=3.0         
        ENDIF
      else                  !! within 10S~10N
        IF(AILCG(p).GT.ZEROlai .AND.DAYLNGTH(p).LT.11.9  &
            .and. btran(p).LT.0.50 ) THEN
            LFSTATUS(p)=3.0        
         ENDIF
       endif                 !! end of latitude loop 
!! liwp 2015-09-29 add bm11, otherwise no leaffall when status 1 shift to 3
       if (LFSTATUS(p).gt.2.9.and.LFSTATUS(p).lt.3.1) then   !! "LEAF FALL" MODE
           maxdays(p)=maxdays(p)-1
           bm11(p)   = GLEAFMAS(p)
           pandays(p)= 0.0
           CHKMODE(p)=1         
       endif
    endif                   !! end of ivt(p)=6

!!          BDL DCD Temperate
    if (ivt(p).eq.7 ) then
       if ( abs(latdeg(g)) .ge. 45.0 ) then
            IF(DAYLNGTH(p).LT.12.0 .and. ROOTTEMP(p).LT.(3.0+273.16)) THEN
                LFSTATUS(p)=3.0       ! GO INTO "LEAF FALL" from "MAX GROWTH"
            ENDIF
       elseif ( abs(latdeg(g)).ge.40.0 .and. abs(latdeg(g)).lt.45.0 ) then
            IF(DAYLNGTH(p).LT.12.1 .and. ROOTTEMP(p).LT.(4.0+273.16) ) THEN
                LFSTATUS(p)=3.0      
            ENDIF
       elseif ( abs(latdeg(g)).ge.35.0 .and. abs(latdeg(g)).lt.40.0 ) then
            IF(DAYLNGTH(p).LT.12.2 .and. ROOTTEMP(p).LT.(4.5+273.16) ) THEN
                LFSTATUS(p)=3.0      
            ENDIF
       elseif ( abs(latdeg(g)).ge.25.0 .and. abs(latdeg(g)).lt.35.0 ) then
            IF(DAYLNGTH(p).LT.12.4 .and. ROOTTEMP(p).LT.(6.0+273.16) ) THEN
                LFSTATUS(p)=3.0      
            ENDIF
       elseif ( abs(latdeg(g)).ge.15.0 .and. abs(latdeg(g)).lt.25.0 ) then
            IF(DAYLNGTH(p).LT.12.5 .and. ROOTTEMP(p).LT.(9.+273.16) ) THEN
                LFSTATUS(p)=3.0     
            ENDIF
       else                       !! within 15S~15N
            IF(btran(p).LT.0.45 .and. ROOTTEMP(p).LT.(12.+273.16) ) THEN
                LFSTATUS(p)=3.0     
            ENDIF
       endif                 !! end of latitude loop 
          if ( LFSTATUS(p).gt.2.8.and.LFSTATUS(p).lt.3.2 ) then
              maxdays(p)=maxdays(p)-1
              PANDAYS(p)=0
              CHKMODE(p)=1         
          endif
    endif                   !! end of ivt(p)=7

!           BDL DCD COLD 
       if (ivt(p).eq.8 ) then
            IF( AILCG(p).GT.ZEROlai.AND.  &
               (DAYLNGTH(p).LT.11.0.AND.ROOTTEMP(p).LT.( 6.5+273.16) & 
                .OR. ROOTTEMP(p).LT.(ROOTHRSH+273.16+3)) )THEN
                LFSTATUS(p)=3.0      ! GO INTO "LEAF FALL" MODE
                CHKMODE (p)=1         
              maxdays(p)=maxdays(p)-1
              bm11(p)=GLEAFMAS(p)
            ENDIF
       endif

     if ( ivt(p).eq.10 ) then
       if ( abs(latdeg(g)).ge.40.0 ) then
         IF(DAYLNGTH(p).LT.11.7.AND.ROOTTEMP(p).LT.(4.5+273.16)) then 
             LFSTATUS(p)=3.0      !!  GO INTO "LEAF FALL" MODE FROM MAX GROWTH
         ENDIF
       elseif ( abs(latdeg(g)).ge.30.0 ) then
         IF(DAYLNGTH(p).LT.11.8.AND.ROOTTEMP(p).LT.(5.5+273.16)) then 
             LFSTATUS(p)=3.0 
         ENDIF
       elseif ( abs(latdeg(g)).ge.20.0 ) then
         IF(DAYLNGTH(p).LT.11.9.AND.ROOTTEMP(p).LT.(6.0+273.16)) then 
             LFSTATUS(p)=3.0 
         ENDIF
       elseif ( abs(latdeg(g)).ge.10.0 ) then
         IF(DAYLNGTH(p).LT.12.0.AND.btran(p).LT.0.48) then 
             LFSTATUS(p)=3.0 
         ENDIF
       else                          !! within 10S~10N
         IF((DAYLNGTH(p).LT.12.0.or.btran(p).LT.0.50)) then 
             LFSTATUS(p)=3.0 
         ENDIF
       endif                         !! end of latitude loop
      if ( LFSTATUS(p).gt.2.8 .and. LFSTATUS(p).lt.3.2 ) then  !! "leaf fall"
              maxdays(p)=maxdays(p)-1
              bm11(p) = gleafmas(p)*0.8    !! reduce spring leaffall
              CHKMODE(p)=1         
      endif
     endif                   !! end of ivt=10

     if ( ivt(p).eq.11) then
       if ( abs(latdeg(g)).ge.60.0 ) then
         IF(DAYLNGTH(p).LT.11.6.AND.ROOTTEMP(p).LT.(6.5+273.16)) then 
                LFSTATUS(p)=3.0      ! GO INTO "LEAF FALL" MODE FROM MAX GROWTH
         ENDIF
       elseif ( abs(latdeg(g)).ge.50.0 ) then
         IF(DAYLNGTH(p).LT.11.8.AND.ROOTTEMP(p).LT.(7.5+273.16)) then 
                LFSTATUS(p)=3.0 
         ENDIF
       elseif ( abs(latdeg(g)).ge.40.0 ) then
         IF(DAYLNGTH(p).LT.12.0.AND.ROOTTEMP(p).LT.(9.0+273.16)) then 
                LFSTATUS(p)=3.0 
         ENDIF
       else                            !! within 40S~40N
         IF(DAYLNGTH(p).LT.12.0.AND.ROOTTEMP(p).LT.(10.0+273.16)) then 
                LFSTATUS(p)=3.0 
         ENDIF
       endif
      if ( LFSTATUS(p).gt.2.8 .and. LFSTATUS(p).lt.3.2 ) then  !! "leaf fall"
              bm11(p)   =GLEAFMAS(p)*0.5  !! reduce "leaf fall" in Spring
              maxdays(p)=maxdays(p)-1
              CHKMODE (p)=1
      endif
     endif                             !! end of ivt=11

!! liwp 2016-04-02 for grasses
      if (ivt(p).eq.12 .or. ivt(p).eq.13 .or. ivt(p).eq.14) then
           IF(AILCG(p).LE.ZEROlai) THEN
              LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
              PANDAYS(p)=0
              maxdays(p) = 0.0
              maturedays(p)=0.0
           ELSE IF(AILCG(p).GT.1.0 )THEN
!          ELSE IF(AILCG(p).GT.LFTHRS(p) )THEN
              LFSTATUS(p)=2.0      ! GO TO "NORMAL GROWTH" MODE
              normaldays(p)=normaldays(p)+1
              maxdays(p) = 0.0
           ELSE
              LFSTATUS(p)=1.0      ! STAY IN "MAX GROWTH" MODE
           ENDIF
              CHKMODE(p)=1
      endif                        !! end of grasses loop

!! lwp 2009-12 for crops
       if (ivt(p).eq.15) then
!          IF( AILCG(p).GE.ZEROlai .AND. COLDDAYS(p).GE.COLDLMT(3) ) THEN
!! liwp 2016-04-25 directly from status=1 to 3 when LAI never reaches LFTHRS
         if ( abs(latdeg(g)) .ge. 50.0 ) then
            IF(DAYLNGTH(p).LT.13.0 .and. ROOTTEMP(p).LT.(9.5+273.2)) THEN
                LFSTATUS(p)=3.0       ! GO INTO "LEAF FALL" from "MAX GROWTH"
            ENDIF
         else if ( abs(latdeg(g)).ge.40.0 ) then
            IF(DAYLNGTH(p).LT.12.5 .and. ROOTTEMP(p).LT.(10.5+273.2) ) THEN
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( abs(latdeg(g)).ge.30.0 ) then
            IF(DAYLNGTH(p).LT.12.3 .and. ROOTTEMP(p).LT.(12.0+273.2) ) THEN
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( latdeg(g).ge.20.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.ROOTTEMP(p).LT.(14.0+273.2).or.calday.gt.295 ) THEN
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( latdeg(g).ge.10.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.btran(p).lt.0.60 .or. calday.gt.300 ) THEN
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( latdeg(g).gt.0.0.and.latdeg(g).lt.10.0 ) then
            IF(calday.gt.305 ) THEN
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( latdeg(g).gt.-10.0.and.latdeg(g).lt.0 ) then
            IF(calday.gt.123 .and. calday.lt.143 ) THEN  !! crop harvested within 2 weeks
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( latdeg(g).gt.-20.0.and.latdeg(g).le.-10.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.btran(p).lt.0.60 .or. &
               calday.gt.118 .and. calday.lt.138 ) THEN  
                LFSTATUS(p)=3.0  
            ENDIF
         else if ( latdeg(g).gt.-30.0.and.latdeg(g).le.-20.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.ROOTTEMP(p).LT.(13.0+273.2).or. &
               calday.gt.113 .and. calday.lt.133 ) THEN  
                LFSTATUS(p)=3.0  
            ENDIF
         endif                     !! end of latitude loop
        if ( LFSTATUS(p).gt.2.8.and.LFSTATUS(p).lt.3.2 ) then 
              maxdays(p)= 0.0
              pandays(p)= 0.0
!! lwp 2009-12 to "remember" the initial leaf mass of crop when harvest begin
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
              CHKMODE(p)=1 
        endif
       endif                       !! end of PFT=15
!! zhangyw  20150520 for crops
#if(defined RICEWATER)
       if (ivt(p).eq.16) then
           IF( AILCG(p).GE.ZERO .AND. COLDDAYS(p).GE.COLDLMT(3) ) THEN
              LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE
              maxdays(p)=maxdays(p)-1
              CHKMODE(p)=1         ! REGARDLESS OF LAI
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
              FLHRLOSS(p)=bm11(p)*(0.5/FLHRSPAN(4))  !! reduce Spring loss during maxgrowth
              los21(p)   =bm21(p)*(0.5/FLHRSPAN(4))
              los31(p)   =bm31(p)*(0.5/FLHRSPAN(4))
           ENDIF
!           IF( mod(int(calday),310).eq.0.or.mod(int(calday),200).eq.0) THEN    !! for rice zhangyyw
!          IF( kmoda.eq.1105.or.kmoda.eq.719) THEN    !! for rice zhangyyw
           IF( kmoda.eq.1105                ) THEN    !! for rice zhangyyw
              LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE
              maxdays(p)=maxdays(p)-1
              CHKMODE(p)=1         ! REGARDLESS OF LAI
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
              FLHRLOSS(p)=bm11(p)*(0.5/FLHRSPAN(4))  !! reduce Spring loss during maxgrowth
              los21(p)   =bm21(p)*(0.5/FLHRSPAN(4))
              los31(p)   =bm31(p)*(0.5/FLHRSPAN(4))
           ENDIF
       endif
#endif

      ENDIF                      !! end of maxgrowth check 
    enddo                        !! end of STATUS=1 check do loop
!
!     IF IN "NORMAL GROWTH" MODE
!     --------------------------
!
!  IF IN "NORMAL GROWTH" MODE THEN GO THROUGH EVERY PFT INDIVIDUALLY
!  AND FOLLOW SET OF RULES TO DETERMINE IF GO INTO "FALL/HARVEST" MODE
!
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
         TA = TAIR(P)
!! begin normal growth
     IF(CHKMODE(p).EQ.0.AND.abs(LFSTATUS(p)-2.0).LT.0.1 ) THEN 
          FLHRLOSS(p)= 0.0
          bm11(p) = 0.0
          pandays(p) = 0.0

!       NEEDLE LEAF EVG temperate
      if ( ivt(p).eq.1 ) then
          IF( calday.lt.190.and.AILCG(p).LT.5.5 )THEN
            LFSTATUS(p)=1.0       ! GO BACK TO "MAX. GROWTH" MODE
          ELSE
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
          ENDIF
            CHKMODE(p)=1         
      endif
!       NEEDLE LEAF EVG BOREAL
      if (ivt(p).eq.2) then
          IF( calday.lt.190.and.AILCG(p).LT.5.5 )THEN
            LFSTATUS(p)=1.0       ! GO BACK TO "MAX. GROWTH" MODE
          ELSE
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
             if (calday.gt.230 ) then
               LFSTATUS(p)=3.0       ! Go Into "LEAF FALL" mode
             endif
          ENDIF
            CHKMODE(p)=1         
      endif
!
!       NEEDLE LEAF DCD
    if (ivt(p).eq.3) then
!! lwp 2013-05-27 modify the normal growth to leaffall transfer of PFT3
      IF(AILCG(p).LE.ZEROlai) THEN
            LFSTATUS(p)=4.0       ! SWITCH TO "NO LEAVES" MODE
            maxdays(p) = 0.0
      ELSE IF(AILCG(p).GT.ZEROlai.AND.COLDDAYS(p).GE.COLDLMT(1)  &
         .or. DAYLNGTH(p).LT.14.5.AND.ROOTTEMP(p).LT.(7.0+273.16) ) THEN
            LFSTATUS(p)=3.0       ! GO INTO "LEAF FALL" MODE
            bm11(p) = GLEAFMAS(p)
      ELSE
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            bm11(p) = 0.0
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
!! lwp 2013-05-28 add the following lines to count cold days to initiate leaf fall for NDL DCD
        if ( normaldays(p) .gt. 75 ) then
         IF(TA.LT.(COLDTHRS(1)+273.16))  THEN
            COLDDAYS(p)=COLDDAYS(p)+1
            COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(1) )
         ELSE
            COLDDAYS(p)=0
         ENDIF
        endif
      ENDIF
            CHKMODE(p)=1         
    endif                !! end of ivt=3
!
!       BROAD LEAF EVG
      if ( ivt(p).eq.4 ) then
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
            CHKMODE(p)=1         
      endif
      if ( ivt(p).eq.5 .or. ivt(p).eq.9 ) then
!! liwp 2016-01-07 use threshold lai=3.0 to avoid too much allocation to Leaf (status=1)
!         IF( AILCG(p).LT.LFTHRS(p).AND.AILCG(p).GT.ZEROlai ) THEN  
          IF( DAYLNGTH(p).lT.13.0 .and. AILCG(p).LT.3.0 ) THEN  
            LFSTATUS(p)=1.0       ! GO BACK TO "MAX. GROWTH" MODE
          ELSE
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
          ENDIF
            CHKMODE(p)=1         
      endif
!
!       BROAD LEAF DCD COLD
!       WE USE DAYLENGTH AND ROOTTEMP TO INITIATE LEAF OFFSET
    if ( ivt(p).eq.7 ) then
      IF( AILCG(p).GT.ZEROlai.AND. AILCG(p).LT.2.0) THEN
             LFSTATUS(p)=1.0       ! SWITCH TO "MAX. GROWTH" MODE
      ELSE
             LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
       if ( abs(latdeg(g)) .ge. 45.0 ) then
          IF(DAYLNGTH(p).LT.13.8 .and. ROOTTEMP(p).LT.(18.5+273.2)) THEN
             LFSTATUS(p)=3.0        ! GO INTO "LEAF FALL" from "NORMAL GROWTH"
          ENDIF
       elseif ( abs(latdeg(g)).ge.40.0 ) then
          IF(DAYLNGTH(p).LT.13.6 .and. ROOTTEMP(p).LT.(19.3+273.2)) THEN
             LFSTATUS(p)=3.0        ! GO INTO "LEAF FALL" from "NORMAL GROWTH"
          ENDIF
       elseif ( abs(latdeg(g)).ge.35.0 ) then
          IF(DAYLNGTH(p).LT.13.3 .and. ROOTTEMP(p).LT.(20.3+273.2)) THEN
             LFSTATUS(p)=3.0        ! GO INTO "LEAF FALL" from "NORMAL GROWTH"
          ENDIF
       elseif ( abs(latdeg(g)).ge.25.0 ) then
          IF(DAYLNGTH(p).LT.12.8 .and. ROOTTEMP(p).LT.(21.0+273.2)) THEN
              LFSTATUS(p)=3.0      
          ENDIF
       elseif ( abs(latdeg(g)).ge.15.0 ) then
          IF(DAYLNGTH(p).LT.12.4.and.(ROOTTEMP(p).LT.(22.+273.2).or.btran(p).LT.0.55) ) THEN
              LFSTATUS(p)=3.0     
          ENDIF
       else                   !! within 15S~15N
          IF(DAYLNGTH(p).LT.12.05.or. btran(p).LT.0.60 ) THEN
              LFSTATUS(p)=3.0     
          ENDIF
       endif                  !! end of latitude loop 
      ENDIF
         if ( LFSTATUS(p).lt.1.2 ) then        !! "max growth"
             maxdays(p) = maxdays(p) + 1 
             bm11(p)= 0.0
         else if ( LFSTATUS(p).lt.2.2 ) then   !! "normal growth"
             normaldays(p)=normaldays(p)+1
             maxdays(p) = 0.0
             bm11(p)= 0.0
         else if ( LFSTATUS(p) .lt. 3.2 ) then   !! "leaf fall"
             maxdays(p)=maxdays(p)-1
             normaldays(p)=normaldays(p)-1
             PANDAYS(p)=0
             bm11(p)=GLEAFMAS(p)
         endif
           CHKMODE(p)=1         
    endif                        !! end of ivt(p)=7

     if ( ivt(p).eq.8 ) then
       IF(AILCG(p).LE.ZEROlai) THEN
            LFSTATUS(p)=4.0       ! SWITCH TO "NO LEAVES" MODE
            PANDAYS(p)=0
            FLHRLOSS(p)=0.0
            maturedays(p)=0.0
            maxdays(p) = 0.0
            bm11(p)=0.0
       ELSE IF(AILCG(p).GT.ZEROlai.AND.AILCG(p).LT.LFTHRS(p))THEN   
            LFSTATUS(p)=1.0       ! SWITCH TO "MAX. GROWTH" MODE
            bm11(p)=0.0
       ELSE
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
!! lwp 2013-05-28 add the following lines to count cold days to initiate leaf fall for NDL DCD
        if ( normaldays(p) .gt. 30 ) then
         IF( TA.LT.(COLDTHRS(2)+273.16-1.0) )  THEN
            COLDDAYS(p)=COLDDAYS(p)+1
            COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(2) )
         ELSE
            COLDDAYS(p)=0
         ENDIF
        endif
         IF( DAYLNGTH(p).LT.10.8 .AND. ROOTTEMP(p).LT.( 7.0+273.16) & 
            .OR. ROOTTEMP(p).LT.(3.0+273.16+3)  &
            .OR. COLDDAYS(p).GE.COLDLMT(2) ) THEN
            LFSTATUS(p)=3.0       ! GO INTO "LEAF FALL" MODE
            bm11(p) = GLEAFMAS(p)
         ENDIF
       ENDIF
            CHKMODE(p)=1         
     endif                !! end of ivt=8

     if ( ivt(p).eq.10 ) then
        IF( AILCG(p).LE.ZEROlai) THEN
            LFSTATUS(p)=4.0       ! SWITCH TO "NO LEAVES" MODE
            PANDAYS(p)=0
            FLHRLOSS(p)=0.0
            maturedays(p)=0.0
            maxdays(p) = 0.0
            bm11(p)=0.0
  !     ELSE IF( AILCG(p).LT.LFTHRS(p) .or. AILCG(p).LT.1.0 )THEN   
  !     ELSE IF( AILCG(p).LT.1.0 )THEN   
  !         LFSTATUS(p)=1.0       ! SWITCH TO "MAX. GROWTH" MODE
  !         bm11(p)=0.0
        ELSE
            LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
!! lwp 2013-05-28 add the following lines to count cold days to initiate leaf fall for NDL DCD
            if ( normaldays(p) .gt. 40 ) then
             IF(TA.LT.(COLDTHRS(2)+273.16+1.0))  THEN
                COLDDAYS(p)=COLDDAYS(p)+1
                COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(2) )
             ELSE
                COLDDAYS(p)=0
             ENDIF
            endif
        if ( abs(latdeg(g)).ge.40.0 ) then
         IF(DAYLNGTH(p).LT.13.0.AND.ROOTTEMP(p).LT.(9.0+273.2) &
             .OR.COLDDAYS(p).GE.COLDLMT(2) ) then
             LFSTATUS(p)=3.0      ! GO INTO "LEAF FALL" MODE FROM NORMAL GROWTH
         ENDIF
    !   else if ( abs(latdeg(g)).ge.30.0 .and. abs(latdeg(g)).lt.40.0 ) then
        else if ( abs(latdeg(g)).ge.30.0 ) then
         IF(DAYLNGTH(p).LT.13.0.AND.(ROOTTEMP(p).LT.(10.0+273.2) &
            .OR.COLDDAYS(p).GE.COLDLMT(2).or.btran(p).lt.0.50) ) then
             LFSTATUS(p)=3.0
         ENDIF
        else if ( abs(latdeg(g)).ge.20.0 ) then
         IF(DAYLNGTH(p).LT.12.4.AND.(ROOTTEMP(p).LT.(12.0+273.2).OR.btran(p).lt.0.50)) then
             LFSTATUS(p)=3.0
         ENDIF
        else if ( abs(latdeg(g)).ge.10.0 ) then
         IF(DAYLNGTH(p).LT.12.2.AND.(ROOTTEMP(p).LT.(13.5+273.2).OR.btran(p).lt.0.55)) then
             LFSTATUS(p)=3.0
         ENDIF
        else                      !! within 10S~10N
         IF(DAYLNGTH(p).LT.12.0.AND.(ROOTTEMP(p).LT.(15.0+273.2).OR.btran(p).lt.0.60)) then  
             LFSTATUS(p)=3.0
         ENDIF
        endif                     !! end of latitude loop
       ENDIF                      !! end of LAI loop
      if ( LFSTATUS(p).gt.2.8.and.LFSTATUS(p).lt.3.2 ) then  !! status=3
            maxdays(p) =0
            PANDAYS(p) =0
            bm11(p)    = GLEAFMAS(p)
      endif
            CHKMODE(p)=1         
     endif                !! end of ivt(p)=10

    if( ivt(p).eq.11 ) then
      IF( AILCG(p).LE.ZEROlai ) THEN
           LFSTATUS(p)=4.0       ! SWITCH TO "NO LEAVES" MODE
           FLHRLOSS(p)=0.0
           maturedays(p)=0.0
           maxdays(p) = 0.0
           bm11(p)=0.0
      ELSE IF( AILCG(p).LT.(LFTHRS(p)-0.5) ) THEN   
           LFSTATUS(p)=1.0       ! SWITCH TO "MAX. GROWTH" MODE
           bm11(p)=0.0
      ELSE
!! lwp 2013-05-28 add the following lines to count cold days to initiate leaf fall for NDL DCD
           LFSTATUS(p)=2.0       ! STAY IN "NORMAL GROWTH" MODE
           normaldays(p)=normaldays(p)+1
           maxdays(p) = 0.0
           if ( normaldays(p) .gt. 30 ) then
            IF( TA.LT.(COLDTHRS(2)+273.16) )  THEN
               COLDDAYS(p)=COLDDAYS(p)+1
               COLDDAYS(p)=min( COLDDAYS(p), COLDLMT(2) )
            ELSE
               COLDDAYS(p)=0
            ENDIF
           endif
      if ( abs(latdeg(g)).ge.60.0 ) then
       IF( DAYLNGTH(p).LT.14.0.AND.ROOTTEMP(p).LT.(7.0+273.16)) then
            LFSTATUS(p)=3.0      ! GO INTO "LEAF FALL" MODE FROM NORMAL GROWTH
       ENDIF
      elseif ( abs(latdeg(g)).ge.50.0 ) then
       IF( DAYLNGTH(p).LT.13.0.AND.ROOTTEMP(p).LT.(8.0+273.16)) then
            LFSTATUS(p)=3.0
       ENDIF
      elseif ( abs(latdeg(g)).ge.40.0 ) then
       IF( DAYLNGTH(p).LT.12.3.AND.ROOTTEMP(p).LT.(9.0+273.16)) then
            LFSTATUS(p)=3.0
       ENDIF
      else                        !! within 40S~40N
       IF( DAYLNGTH(p).LT.12.0.AND.ROOTTEMP(p).LT.(10.0+273.16)) then
            LFSTATUS(p)=3.0
       ENDIF
      endif                       !! end of latitude loop
     ENDIF                        !! end of LAI loop
    if ( LFSTATUS(p).gt.2.8 .and. LFSTATUS(p).lt.3.2 ) then  !! "leaf fall"
         bm11(p)   =GLEAFMAS(p)
         maxdays(p)=maxdays(p)-1
         pandays(p)=0
    endif
         CHKMODE(p)=1
   endif               !! end of ivt(p)=11
!
!    BROAD LEAF DCD DRY
!    WE STILL USE DAYLENGTH AND ROOTTEMP TO INITIATE LEAF OFFSET,
!    FOR THE PATHOLOGICAL CASES OF DRY DCD TREES BEING FURTHER
!    AWAY FROM THE EQUATOR THEN WE CAN IMAGINE. OTHER WISE LEAF
!    LOSS WILL OCCUR DUE TO DROUGHT ANYWAY.
     if (ivt(p).eq.6 ) then
       IF(AILCG(p).LE.ZEROlai) THEN
            LFSTATUS(p)=4.0       ! SWITCH TO "NO LEAVES" MODE
            CHKMODE(p)=1         
            maturedays(p)=0.0
            maxdays(p) = 0.0
       ELSE IF(AILCG(p).LT.(LFTHRS(p)-0.8))THEN   
            LFSTATUS(p)=1.0       ! SWITCH TO "MAX. GROWTH" MODE
            CHKMODE(p)=1         
       ELSE
        if ( abs(latdeg(g)) .ge. 25.0 ) then
          IF( DAYLNGTH(p).LT.12.80.and.ROOTTEMP(p).LT.(23.0+273.16) ) then 
             LFSTATUS(p)=3.0     ! GO INTO "LEAF FALL" from NORMAL GROWTH
          ENDIF
        elseif ( abs(latdeg(g)).ge.20.0 ) then 
          IF( DAYLNGTH(p).LT.12.6.and.ROOTTEMP(p).LT.(24.0+273.16) ) THEN
             LFSTATUS(p)=3.0         
          ENDIF
        elseif ( abs(latdeg(g)).ge.15.0 ) then 
          IF( DAYLNGTH(p).LT.12.4.and.ROOTTEMP(p).LT.(25.0+273.16) ) THEN
             LFSTATUS(p)=3.0         
          ENDIF
        elseif ( abs(latdeg(g)).ge.10.0 ) then 
          IF(DAYLNGTH(p).LT.12.1.and.btran(p).LT.0.65 ) THEN
             LFSTATUS(p)=3.0         
          ENDIF
        else                     !! within 10s~10N
          IF(DAYLNGTH(p).LT.11.95.and.btran(p).LT.0.65 .or.DAYLNGTH(p).LT.11.85) THEN
             LFSTATUS(p)=3.0
          ENDIF
        endif                    !! end of latitude loop
       ENDIF              !! end of lai loop
         if (LFSTATUS(p).gt.1.9 .and.LFSTATUS(p).lt.2.1) then  !! "normall growth"
            normaldays(p)=normaldays(p)+1
            maxdays(p) = 0.0
            bm11(p) = 0.0
            CHKMODE(p)=1         
         endif
         if (LFSTATUS(p).gt.2.9 .and.LFSTATUS(p).lt.3.1) then  !! "leaffall"
              maxdays(p)=maxdays(p)-1
              pandays(p)= 0.0
              bm11(p)   = GLEAFMAS(P)
            CHKMODE(p)=1         
         endif
     endif               !! end of ivt(p)=6
!
!  "NORMAL GROWTH" TO "FALL/HARVEST" TRANSITION FOR CROPS IS BASED ON
!  SPECIFIED LAI. WE HARVEST IF LAI OF CROPS REACHES A THRESHOLD.
!  IF LAI DOESN'T REACH THIS THRESHOLD (SAY DUE TO A BAD YEAR)
!  WE HARVEST ANYWAY IF IT STARTS GETTING COLD, OTHERWISE DON'T HARVEST.
!
      if (ivt(p).eq.15 ) then
          IF( AILCG(p).LT.ZEROlai) THEN
              LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
              maturedays(p)=0.0
              normaldays(p)=0.0
              maxdays(p) = 0.0
!! lwp 2009-12 to reduce crop biomass after "fully harvest"
              GLEAFMAS(p) =0.0
              STEMMASS(p) =0.0
              ROOTMASS(p) =0.0
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
!! lwp 2009-12 restrain "COLD" harvest of crops only when LAI is large enough
 !        ELSE IF( AILCG(p).GE.ZEROlai .AND. COLDDAYS(p).GE.COLDLMT(3) ) Then
 !            LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE REGARDLESS OF LAI
 !            go to 2004
 !        END IF
          ELSE IF( AILCG(p).GE.ZEROlai.AND.AILCG(p).LT.HARVTHRS(ivt(p)) ) THEN
              LFSTATUS(p)=2.0         ! STAY IN "NORMAL GROWTH" MODE
              normaldays(p)= normaldays(p) + 1
              maxdays(p) = 0.0
              pandays(p) = 0.0
            if (normaldays(p).gt.80) then
              LFSTATUS(p)=3.0         ! GO INTO "HARVEST" MODE whatever LAI
!! liwp 2016-04-25 restrain "COLD" harvest of crops only when LAI is large enough
            else if ( normaldays(p) .gt. 30 ) then  !! "cold harvest" when normaldays>30 
         if ( abs(latdeg(g)) .ge. 50.0 ) then
            IF(DAYLNGTH(p).LT.13.0 .and. ROOTTEMP(p).LT.(10.0+273.2)) THEN
                LFSTATUS(p)=3.0       ! GO INTO "LEAF FALL" from "normal GROWTH"
            ENDIF
         else if ( abs(latdeg(g)).ge.40.0 ) then
            IF(DAYLNGTH(p).LT.12.5 .and. ROOTTEMP(p).LT.(12.0+273.2)) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( abs(latdeg(g)).ge.30.0 ) then
            IF(DAYLNGTH(p).LT.12.3 .and. ROOTTEMP(p).LT.(14.0+273.2)) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).ge.20.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.ROOTTEMP(p).LT.(16.0+273.2).or.calday.gt.290 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).ge.10.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.btran(p).lt.0.60 .or. calday.gt.295 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.0.0.and.latdeg(g).lt.10.0 ) then
            IF(calday.gt.300 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.-10.0.and.latdeg(g).lt.0 ) then
            IF(calday.gt.118 .and. calday.lt.138 ) THEN  !! crop harvested within 2 weeks
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.-20.0.and.latdeg(g).le.-10.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.btran(p).lt.0.60 .or. &
               calday.gt.113 .and. calday.lt.133 ) THEN 
                LFSTATUS(p)=3.0 
            ENDIF
         else if ( latdeg(g).gt.-30.0.and.latdeg(g).le.-20.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.ROOTTEMP(p).LT.(14.0+273.2).or. &
               calday.gt.108 .and. calday.lt.128 ) THEN 
                LFSTATUS(p)=3.0 
            ENDIF
         endif                     !! end of latitudes
        endif                      !! end of normaldays>30 or 80

          ELSE IF ( AILCG(p).GE.HARVTHRS(ivt(p)) ) THEN      !! when LAI>=4.5
              LFSTATUS(p)=2.0      ! STAY IN "mature" MODE when LAI is large enough
              normaldays(p)=normaldays(p)+1
!! lwp 2009-12 to "keep" growing at mature stage for 30 days for crops
              maturedays(p)= maturedays(p) + 1
            if (maturedays(p).ge.30.0.or.normaldays(p).gt.80) then
              LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE
            endif
          ENDIF                    !! end of LAI loop
2004      continue
            if (LFSTATUS(p).gt.2.9 .and.LFSTATUS(p).lt.3.1) then  !! status=3
              maturedays(p)=0.0
              normaldays(p)=0.0
              pandays(p)=0.0
!! lwp 2009-12 to "remember" the initial leaf mass of crop when harvest begin
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
            endif
              CHKMODE(p)=1         
      endif                      !! end of ivt=15
      
!!zhangyw add  for rice
#if(defined RICEWATER)
      if (ivt(p).eq.16 ) then
      if ( londeg(g).gt.95.0)then   !for two 
          IF ( AILCG(p).GE.HARVTHRS(ivt(p)) ) THEN
!! lwp 2009-12 to "keep" growing for 30 days for crops
            maturedays(p)= maturedays(p) + 1
            normaldays(p)=normaldays(p)+1
              LFSTATUS(p)=2.0      ! STAY IN "NORMAL GROWTH" MODE
            if (maturedays(p).ge.20.0.or.normaldays(p).gt.120) then
              LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE
              maturedays(p)=0.0
              normaldays(p)=0.0
!! lwp 2009-12 to "remember" the initial leaf mass of crop when harvest begin
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
              FLHRLOSS(p)=bm11(p)/FLHRSPAN(4)
              los21(p)   =bm21(p)/FLHRSPAN(4)
              los31(p)   =bm31(p)/FLHRSPAN(4)
            endif
!! lwp 2009-12 restrain "COLD" harvest of crops only when LAI is large enough
           ELSE IF( AILCG(p).GE.ZEROlai .AND. COLDDAYS(p).GE.COLDLMT(3) ) Then
              LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE REGARDLESS OF LAI
              normaldays(p)=0.0
!! lwp 2009-12 to "remember" the initial leaf mass of crop when harvest begin
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
              FLHRLOSS(p)=bm11(p)/FLHRSPAN(4)
              los21(p)   =bm21(p)/FLHRSPAN(4)
              los31(p)   =bm31(p)/FLHRSPAN(4)
           ELSE IF(AILCG(p).LT.ZEROlai) THEN
              LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
   !          PANDAYS(p)=0
              maturedays(p)=0.0
              normaldays(p)=0.0
              maxdays(p) = 0.0
!! lwp 2009-12 to reduce crop biomass after "fully harvest"
              GLEAFMAS(p) =0.0
              STEMMASS(p) =0.0
              ROOTMASS(p) =0.0
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
           ELSE
              LFSTATUS(p)=2.0      ! STAY IN "NORMAL GROWTH" MODE
              normaldays(p)= normaldays(p) + 1
              maxdays(p) = 0.0
           ENDIF

           if ( mod(int(calday),310).eq.0.or.mod(int(calday),200).eq.0) then     !! for rice zhangyyw
             LFSTATUS(p)    =  3.0      ! GO INTO "HARVEST" MODE
             maturedays(p)  =  0.0
             normaldays(p)  =  0.0
             bm11(p) = GLEAFMAS(p)
             bm21(p) = STEMMASS(p)
             bm31(p) = ROOTMASS(p)
             FLHRLOSS(p) = bm11(p)/FLHRSPAN(4)
             los21(p)    = bm21(p)/FLHRSPAN(4)
             los31(p)    = bm31(p)/FLHRSPAN(4)
           endif
              CHKMODE(p)=1
      else   !londeg.ge.95 for one
          IF( AILCG(p).LT.ZEROlai) THEN
              LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
              maturedays(p)=0.0
              normaldays(p)=0.0
              maxdays(p) = 0.0
              GLEAFMAS(p) =0.0
              STEMMASS(p) =0.0
              ROOTMASS(p) =0.0
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
          ELSE IF( AILCG(p).GE.ZEROlai.AND.AILCG(p).LT.HARVTHRS(ivt(p)) ) THEN
              LFSTATUS(p)=2.0         ! STAY IN "NORMAL GROWTH" MODE
              normaldays(p)= normaldays(p) + 1
              maxdays(p) = 0.0
              pandays(p) = 0.0
            if (normaldays(p).gt.80) then
              LFSTATUS(p)=3.0         ! GO INTO "HARVEST" MODE whatever LAI
            else if ( normaldays(p) .gt. 30 ) then  !! "cold harvest" when normaldays>30
         if ( abs(latdeg(g)) .ge. 50.0 ) then
            IF(DAYLNGTH(p).LT.13.0 .and. ROOTTEMP(p).LT.(10.0+273.2)) THEN
                LFSTATUS(p)=3.0       ! GO INTO "LEAF FALL" from "normal GROWTH"
            ENDIF
         else if ( abs(latdeg(g)).ge.40.0 ) then
            IF(DAYLNGTH(p).LT.12.5 .and. ROOTTEMP(p).LT.(12.0+273.2)) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( abs(latdeg(g)).ge.30.0 ) then
            IF(DAYLNGTH(p).LT.12.3 .and. ROOTTEMP(p).LT.(14.0+273.2)) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).ge.20.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.ROOTTEMP(p).LT.(16.0+273.2).or.calday.gt.290 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).ge.10.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.btran(p).lt.0.60 .or. calday.gt.295 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.0.0.and.latdeg(g).lt.10.0 ) then
            IF(calday.gt.300 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.-10.0.and.latdeg(g).lt.0 ) then
            IF(calday.gt.118 .and. calday.lt.138 ) THEN  !! crop harvested within 2 weeks
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.-20.0.and.latdeg(g).le.-10.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.btran(p).lt.0.60 .or. &
               calday.gt.113 .and. calday.lt.133 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         else if ( latdeg(g).gt.-30.0.and.latdeg(g).le.-20.0 ) then
            IF(DAYLNGTH(p).LT.12.0.and.ROOTTEMP(p).LT.(14.0+273.2).or. &
               calday.gt.108 .and. calday.lt.128 ) THEN
                LFSTATUS(p)=3.0
            ENDIF
         endif                     !! end of latitudes
        endif                      !! end of normaldays>30 or 80

          ELSE IF ( AILCG(p).GE.HARVTHRS(ivt(p)) ) THEN      !! when LAI>=4.5
              LFSTATUS(p)=2.0      ! STAY IN "mature" MODE when LAI is large enough
              normaldays(p)=normaldays(p)+1
              maturedays(p)= maturedays(p) + 1
            if (maturedays(p).ge.30.0.or.normaldays(p).gt.80) then
              LFSTATUS(p)=3.0      ! GO INTO "HARVEST" MODE
            endif
          ENDIF                    !! end of LAI loop
            if (LFSTATUS(p).gt.2.9 .and.LFSTATUS(p).lt.3.1) then  !! status=3
              maturedays(p)=0.0
              normaldays(p)=0.0
              pandays(p)=0.0
              bm11(p)=GLEAFMAS(p)
              bm21(p)=STEMMASS(p)
              bm31(p)=ROOTMASS(p)
            endif
              CHKMODE(p)=1
      endif  !end if londeg.ge.95
      endif
#endif
!
!     "NORMAL GROWTH" TO "MAX. GROWTH" TRANSITION FOR GRASSES 
!
      if (ivt(p).eq.12 .or. ivt(p).eq.13 .or. ivt(p).eq.14) then
            IF(AILCG(p).LE.ZEROlai) THEN
              LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
              PANDAYS(p)=0
              maxdays(p) = 0.0
              maturedays(p)=0.0
            ELSE
              LFSTATUS(p)=2.0      ! STAY IN "NORMAL GROWTH" MODE
        if ( abs(latdeg(g)).ge.40.0 ) then
         IF(DAYLNGTH(p).LT.13.1.AND.ROOTTEMP(p).LT.(14.0+273.16) ) then 
             LFSTATUS(p)=3.0      ! GO INTO "LEAF FALL" MODE FROM NORMAL GROWTH
         ENDIF
        else if ( abs(latdeg(g)).ge.30.0 ) then
         IF(DAYLNGTH(p).LT.13.0.AND.ROOTTEMP(p).LT.(15.5+273.16) ) then 
             LFSTATUS(p)=3.0
         ENDIF
        else if ( abs(latdeg(g)).ge.20.0 ) then
         IF(DAYLNGTH(p).LT.12.8.AND.ROOTTEMP(p).LT.(17.0+273.16) ) then 
             LFSTATUS(p)=3.0
         ENDIF
        else if ( abs(latdeg(g)).ge.10.0 ) then
         IF(DAYLNGTH(p).LT.12.2.AND.(ROOTTEMP(p).LT.(19.0+273.16).or.btran(p).lt.0.50) ) then 
             LFSTATUS(p)=3.0
         ENDIF
        else if ( abs(latdeg(g)).lt.10.0 ) then      !! within 10S~10N
         IF(DAYLNGTH(p).LT.12.0.AND.btran(p).lt.0.55 ) then
             LFSTATUS(p)=3.0
         ENDIF
        endif                     !! end of latitude loop
            ENDIF                 !! end of LAI loop
              CHKMODE(p)=1
      endif                       !! end of grasses loop (ivt=12,13,14)
    ENDIF           !! end of normal growth mode
   enddo            !! end of status=2 check do loop
!
!     IF IN "FALL/HARVEST" MODE
!     --------------------------
!
!   GRASSES AND EVG TREES DO NOT COME INTO THIS MODE, BECAUSE THEY WANT
!   TO STAY GREEN IF POSSIBLE. THIS MODE IS ACTIVATED FOR DCD PLANTS AND
!   CROPS. ONCE IN THIS MODE DCD TREES LOOSE THEIR LEAVES AND CROPS ARE
!   HARVESTED. NDL DCD TREES KEEP LOOSING THEIR LEAVES AT RATE DETERMINED
!   BY COLD STRESS, BDL DCD TREES LOOSE THEIR LEAVES AT A SPECIFIED
!   RATE, AND CROPS ARE HARVESTED OVER A PERIOD OF ~15 DAYS. DCD TREES
!   AND CROPS STAY IN "LEAF FALL/HARVEST" MODEL UNTIL ALL GREEN LEAVES
!   ARE GONE AT WHICH TIME THEY SWITCH INTO "NO LEAVES" MODE, AND THEN 
!   WAIT FOR THE CLIMATE TO BECOME FAVOURABLE TO GO INTO "MAX. GROWTH" MODE
!
!!!!!!!!!!!!!!!!!!!!! liwp 2015-09-12
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
         TA = TAIR(P)
          if (ivt(p).eq.2 ) then
            CHKMODE (p)=1
          endif
!!  ONLY DCD TREES AND CROPS
  if (ivt(p).eq.3 .or.ivt(p).eq.6  .or.ivt(p).eq.7  .or.ivt(p).eq.8 .or. &
      ivt(p).eq.10.or.ivt(p).ge.11 .and.ivt(p).le.14.or.ivt(p).eq.15 .or.ivt(p).eq.16) then
   IF( CHKMODE(p).EQ.0.AND.abs(LFSTATUS(p)-3.0).LT.0.1 ) THEN
     IF(AILCG(p).LE. ZEROlai) THEN      !! original criterion is 0.01
             LFSTATUS(p)=4.0          ! GO INTO "NO LEAVES" MODE
             CHKMODE (p)=1
             FLHRLOSS(p)=0.0
             los21(p) = 0.0
             los31(p) = 0.0
             bm11(p)=0.0
             bm21(p)=0.0
             bm31(p)=0.0
            maturedays(p)=0.0
!! liwp 2015-10-09 no pandays of deciduous PFT in "no leaf" mode in winter
      if ( latdeg(g).gt.10 ) then
       if(calday.lt.70 .or. calday.gt.335) then  ! northern hemisphe winter 
           pandays(p) = 0.
       endif
      endif
      if ( latdeg(g).lt.-10 ) then
       if(calday.gt.152.and.calday.lt.252) then  ! southern hemisphe winter  
           pandays(p) = 0.
       endif
      endif
     ELSE                          ! AILCG>ZEROlai
      if ( ivt(p).eq.3 ) then      ! NDL DCD TREES
         IF( PANDAYS(p).GE.DAYSCHK(ivt(p)).AND.ROOTTEMP(p).GT.(5.0+273.16)) THEN
              LFSTATUS(p)=2.0    ! GO INTO "NORMAL GROWTH" MODE
         ELSE  
            LFSTATUS(p)=3.0      ! STAY IN "FALL/HARVEST" MODE 
            pandays(p) = 0.0
            normaldays(p) = 0.0
            FLHRLOSS(p)=bm11(p)/FLHRSPAN(1)      !! loss of PFT3 leafmas within 50 days
        !      los21(p)=0.001*STEMMASS(p)/FLHRSPAN(1)
         ENDIF
            CHKMODE(p)=1
      endif                   !! end of ivt=3
!! BDL DCD TREES and shrubs
    if( ivt(p).eq.6 ) then  !! deciduous tropical 
      if (AILCG(p).LE. ZEROlai) then
           LFSTATUS(p)=4.0
      else 
       if ( abs(latdeg(g)) .ge. 25.0 ) then
         if ( DAYLNGTH(p).GT.12.5 .and.ROOTTEMP(p).GT.(19.0+273.16) ) THEN
           IF(AILCG(p).GT. ZEROlai .and. AILCG(p).LT.LFTHRS(p))THEN
             LFSTATUS(p)=1.0    ! GO INTO "MAX GROWTH" FROM "LEAF FALL" MODE
           ELSE
             LFSTATUS(p)=2.0    ! GO INTO "NORMAL GROWTH" MODE
           ENDIF
         endif
       elseif (abs(latdeg(g)).ge.17.0 .and.abs(latdeg(g)).lt.25.0) then
         if ( DAYLNGTH(p).GT.12.3 .and.ROOTTEMP(p).GT.(20.0+273.16) ) THEN
           IF(AILCG(p).GT. ZEROlai .and. AILCG(p).LT.LFTHRS(p))THEN
             LFSTATUS(p)=1.0    
           ELSE
             LFSTATUS(p)=2.0   
           ENDIF
         endif
       elseif (abs(latdeg(g)).ge.10.0 .and.abs(latdeg(g)).lt.17.0) then
         if ( DAYLNGTH(p).GT.12.2 .and. btran(p).gt. 0.5 ) THEN
           IF(AILCG(p).GT. ZEROlai .and. AILCG(p).LT.LFTHRS(p))THEN
             LFSTATUS(p)=1.0    
           ELSE
             LFSTATUS(p)=2.0   
           ENDIF
         endif
       else                       !! within 10S~10N
         if ( DAYLNGTH(p).GT.12.1 .and. btran(p).gt. 0.50 ) THEN
           IF(AILCG(p).GT. ZEROlai .and. AILCG(p).LT.LFTHRS(p))THEN
             LFSTATUS(p)=1.0  
           ELSE
             LFSTATUS(p)=2.0 
           ENDIF
         endif
       endif       !! end of latitude loop
      endif        !! end of status check(stay in status=3 if not the above) 
         if (LFSTATUS(p).gt.0.9.and.LFSTATUS(p).lt.2.1) then  !! status=1,2
             bm11(p) = 0.0
             FLHRLOSS(p)=0.0
             CHKMODE(p) =1
         endif
         if (LFSTATUS(p).gt.2.9.and.LFSTATUS(p).lt.3.1) then  !! status=3
           CHKMODE(p) =1
           pandays(p) = 0.0
           normaldays(p)= 0.0
!! liwp 2010-02 calculate "canopy loss" during "leaf-fall period"
            FLHRLOSS(p)= bm11(p)/FLHRSPAN(1)
               los21(p)=0.010*STEMMASS(p)/FLHRSPAN(1)
          endif
          if ( LFSTATUS(p).gt.3.9 ) then        !! status=4
           if ( latdeg(p).gt.5 .and. calday.gt.335 ) then
             pandays(p) = 0.0
           endif
           if (latdeg(p).lt.-5 .and.calday.gt.152.and.calday.lt.182) then
             pandays(p) = 0.0
           endif
             bm11(p) = 0.0
             FLHRLOSS(p)=0.0
             CHKMODE(p) =1
          endif
    endif               !! end of ivt=6

    if ( ivt(p).eq.7 ) then  !! broadleaf deciduous tree temperate
            LFSTATUS(p)=3.
            CHKMODE(p) =1
            pandays(p) = 0.0
            normaldays(p)= 0.0
!! liwp 2010-02 calculate "canopy loss" during "leaf-fall period"
            FLHRLOSS(p)=bm11(p)/FLHRSPAN(2)
               los21(p)=0.005*STEMMASS(p)/FLHRSPAN(2)
!       endif                 !! end of status=1,2,3
        if ( LFSTATUS(p).gt.3.9 ) then        !! status=4
          if ( latdeg(p).gt.5 .and. calday.gt.340 ) then
            pandays(p) = 0.0
          endif
          if (latdeg(p).lt.-5 .and.calday.gt.157.and.calday.lt.182) then
            pandays(p) = 0.0
          endif
            bm11(p) = 0.0
            FLHRLOSS(p)=0.0
            CHKMODE(p) =1
        endif                 !! end of status=4
    endif                     !! end of ivt=7
!!  Broadleaf Deciduous Shrub Temperate (PFT=10)
     if ( ivt(p).eq.10 ) then  
       if (AILCG(p).LE. ZEROlai) then
           LFSTATUS(p)=4.0
       else  
            LFSTATUS(p)=3.0      ! STAY IN "FALL/HARVEST" MODE 
            CHKMODE(p) =1
            FLHRLOSS(p)=bm11(p)/FLHRSPAN(2)
!           los21(p)   =0.0005*STEMMASS(p)/FLHRSPAN(2)
            normaldays(p) = 0.0
            pandays(p) = 0.0
       endif
     endif                    !! end of ivt=10
!
!! BDL DCD TREES and shrubs
!
     if ( ivt(p).eq.8 .or.ivt(p).eq.11) then         !! broadleaf deciduous cold
        IF( PANDAYS(p).GE.DAYSCHK(ivt(p)).AND.ROOTTEMP(p).GT.(6.0+273.16) &
            .or. daylngth(p).gt.13.5) then 
            IF(AILCG(p).LT.LFTHRS(p))THEN
              LFSTATUS(p)=1.0    ! GO INTO "MAX. GROWTH" MODE from "LEAF FALL"
            ELSE
              LFSTATUS(p)=2.0    ! GO INTO "NORMAL GROWTH" MODE
            ENDIF
        ELSE  
            LFSTATUS(p)=3.0      ! STAY IN "FALL/HARVEST" MODE 
            normaldays(p) = 0.0
            pandays(p) = 0.0
            FLHRLOSS(p)=bm11(p)/FLHRSPAN(2)
            los21(p)   =0.1*STEMMASS(p)/FLHRSPAN(2)
        ENDIF
            CHKMODE(p)=1
     endif                     !! end of ivt=11
!
     if ( ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14) then   !! grasses 
            LFSTATUS(p)=3.0      ! STAY IN "FALL/HARVEST" MODE 
            normaldays(p) = 0.0
            pandays(p) = 0.0
            bm11(p)    = GLEAFMAS(p)
            CHKMODE(p)=1
     endif                     !! end of ivt=12,13,14
!
     if (ivt(p).eq.15) THEN ! CROPS
         IF(AILCG(p).LE. ZEROlai) THEN
           LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
           maturedays(p)=0.0
           normaldays(p)=0.0
           pandays(p) =0.0
           FLHRLOSS(p)=0.0
           los21(p)   =0.0
           los31(p)   =0.0
!! lwp 2009-12 to reduce crop biomass after "fully harvest"
           GLEAFMAS(p)=0.0
           STEMMASS(p)=0.0
           ROOTMASS(p)=0.0
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
         ELSE
           LFSTATUS(p)=3.0        ! STAY IN "FALL/HARVEST" MODE 
           pandays(p)=0.0
           normaldays(p) = 0.0
  !        maturedays(p) = 0.0
!! lwp 2009-12 CTEM doesn't calculate harvest at this place (It's a BUG) 
      !   FLHRLOSS(p)=GLEAFMAS(p)*(1.0/FLHRSPAN(1))
!! lwp 2010-02 set bm11>=0 when leafstatus=3 from a restart run
!! lwp 2013-11-20 a bug if leafstatus=4 for crop, stem and root fall in a much faster rate
!! In spring, stem fall at a rate same as the last autumn without the bm21 and bm31 lines
           FLHRLOSS(p)=bm11(p)/FLHRSPAN(3)
           los21(p)   =bm21(p)/FLHRSPAN(3)
           los31(p)   =bm31(p)/FLHRSPAN(3)
         ENDIF       !! end of crops status 3 or 4
           CHKMODE(p)=1
     endif             !! end of ivt=15

!!zhangyw add for calculate harvest of rice
#if(defined RICEWATER)
        IF(ivt(p).eq.16) THEN ! RICE
         IF(AILCG(p).LE. ZEROlai) THEN
           LFSTATUS(p)=4.0      ! SWITCH TO "NO LEAVES" MODE
           PANDAYS(p) =0
           maturedays(p)=0.0
           normaldays(p)=0.0
           FLHRLOSS(p)=0.0
           los21(p)   =0.0
           los31(p)   =0.0
           GLEAFMAS(p)=0.0
           STEMMASS(p)=0.0
           ROOTMASS(p)=0.0
              bm11(p) = 0.0
              bm21(p) = 0.0
              bm31(p) = 0.0
         ELSE
           LFSTATUS(p)=3.0        ! STAY IN "FALL/HARVEST" MODE
           pandays(p)=0.0
           normaldays(p) = 0.0
           maturedays(p) = 0.0
           FLHRLOSS(p)=bm11(p)/FLHRSPAN(4)
           los21(p)   =bm21(p)/FLHRSPAN(4)
           los31(p)   =bm31(p)/FLHRSPAN(4)
         ENDIF       !! end of crops status 3 or 4
           CHKMODE(p)=1
        endif
#endif    
     ENDIF           !! end of LAI>ZEROlai loop
    ENDIF            !! end of status=3 check loop
   ENDIF             !! end of deciduous vegetation and crops(ivt=3,6,7,8,10,11,15,16) 
  enddo              !! end of leaf status=3 check loop
!
!     CHECK THAT LEAF STATUS OF ALL VEGETATION TYPES IN ALL GRID CELLS HAS
!     BEEN UPDATED
!
     do f = 1, fn
        p = filterp(f)
        g = pgridcell(p)
          IF(CHKMODE(p).EQ.0)THEN
           WRITE(6,2001) ivt(p),lfstatus(p),londeg(g),latdeg(g) 
           WRITE(6,2002) AILCG(p),daylngth(p),btran(p)
2001 FORMAT('LFSTATUS NOT UPDATED  PFT=',I3,' STATUS=',F4.1,' LON,LAT=', 2f6.1)
2002 FORMAT('LFSTATUS NOT UPDATED  LAI=',f4.1,' DAYLNGTH=',F6.2,' BTRAN=', f5.2)
          call endrun()
          ENDIF
     ENDDO          !! end of do loop 
!
!! lwp 2013-05-21 increase "enddo" to interrupt do loops
!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        g = pgridcell(p)
         TA = TAIR(P)
!     ------------------------------------------------------------------     
!     HAVING DECIDED LEAF STATUS FOR EVERY PFT, WE NOW CALCULATE NORMAL
!     LEAF TURNOVER, COLD AND DROUGHT STRESS MORTALITY, AND FOR BDL DCD 
!     PLANTS WE ALSO CALCULATE SPECIFIED LOSS RATE IF THEY ARE IN "LEAF FALL"
!     MODE, AND FOR CROPS WE CALCULATE HARVEST LOSS, IF THEY ARE IN
!     "HARVEST" MODE.
!
!     ALL THESE LOSS CALCULATIONS WILL YIELD LEAF LITTER IN Kg C/M2 FOR
!     THE GIVEN DAY FOR ALL PFTs 
!
!     NORMAL LEAF TURN OVER
!
!! lwp 2011-01-31 to reduce loss during maximum growth stages(status=1)
           NRMLLOSS(p)= GLEAFMAS(p)*(1.0-EXP(-1.0/(365*LFESPANY(ivt(p)) )))     
       if ( LFSTATUS(p).lt.1.2 .or. LFSTATUS(p).gt.3.5 ) then
           NRMLLOSS(p) = 0.10*NRMLLOSS(p)
       endif
!
!     FOR DROUGHT STRESS RELATED MORTALITY WE NEED FIELD CAPACITY 
!     AND WILTING POINT SOIL MOISTURE CONTENTS, WHICH WE CALCULATED
!     IN ALLOCATE SUBROUTINE
!
      DO 450 J = 1, nlevsoi
!
!         ESTIMATE (1-DROUGHT STRESS) 
!
    IF(h2osoi_vol(c,J).LE.wiltsm(c,J)) THEN
        BETADRGT(c,J)=0.0
    ELSE IF(h2osoi_vol(c,J).GT.wiltsm(c,J).AND.h2osoi_vol(c,J).LT.FIELDSM(c,J))THEN
!! liwp 2016-03-02 reduce sensitivity to soil moisture of drought induced loss
!       BETADRGT(c,J)=(h2osoi_vol(c,J)-wiltsm(c,J))/(FIELDSM(c,J)-wiltsm(c,J))
        BETADRGT(c,J)=((h2osoi_vol(c,J)-wiltsm(c,J))/(FIELDSM(c,J)-wiltsm(c,J)))**0.3
    ELSE 
        BETADRGT(c,J)=1.0
    ENDIF          
       BETADRGT(c,J)=MAX(0.0, MIN(1.0,BETADRGT(c,J)))
450   CONTINUE
!
!     ESTIMATE DROUGHT STRESS TERM AVERAGED OVER THE ROOTING DEPTH
!     FOR EACH PFT
!
        DRGTSTRS(p) =  (1.0-BETADRGT(c,1 ))*rootfr(p,1) + & 
                       (1.0-BETADRGT(c,2 ))*rootfr(p,2) + &  
                       (1.0-BETADRGT(c,3 ))*rootfr(p,3) + &  
                       (1.0-BETADRGT(c,4 ))*rootfr(p,4) + &  
                       (1.0-BETADRGT(c,5 ))*rootfr(p,5) + &  
                       (1.0-BETADRGT(c,6 ))*rootfr(p,6) + &  
                       (1.0-BETADRGT(c,7 ))*rootfr(p,7) + &  
                       (1.0-BETADRGT(c,8 ))*rootfr(p,8) + & 
                       (1.0-BETADRGT(c,9 ))*rootfr(p,9)
        ROOTPERCENT = rootfr(p,1) + rootfr(p,2) + rootfr(p,3) + &
                      rootfr(p,4) + rootfr(p,5) + rootfr(p,6) + &
                      rootfr(p,7) + rootfr(p,8) + rootfr(p,9) 
        DRGTSTRS(p) = DRGTSTRS(p) / ROOTPERCENT
        DRGTSTRS(p)=MAX(0.0, MIN(1.0,DRGTSTRS(p)))  !! (AVIM)
!! lwp 2010 to reduce soil moisture sensitivity of BET(Tropical rain forest)
         if (ivt(p).eq.4) then
!! liwp 2016-07-11 reduce DRGTSTRS
 !         DRGTSTRS(p) = min (0.40,DRGTSTRS(p))
           DRGTSTRS(p) = min (0.30,DRGTSTRS(p))
         endif
         if (ivt(p).eq.15.or.ivt(p).eq.16) then     !! crops
 !         DRGTSTRS(p) = min (0.60,DRGTSTRS(p))
           DRGTSTRS(p) = min (0.50,DRGTSTRS(p))
         endif
 
!     USING THIS DROUGHT STRESS TERM AND OUR TWO VEGETATION-DEPENDENT
!     PARAMETERS WE FIND LEAF LOSS RATE ASSOCIATED WITH DROUGHT
!
!         DROUGHT RELATED LEAF LOSS RATE
          DRGTLSRT(p)=DRLSRTMX(ivt(p))*(DRGTSTRS(p)**DRGTA(ivt(p)))
!
!     ESTIMATE LEAF LOSS IN Kg C/M2 DUE TO DROUGHT STRESS
!
!! lwp 2011-01-31 to reduce loss during maximum growth stages
           DRGTLOSS(p)=GLEAFMAS(p)*( 1.0-EXP(-DRGTLSRT(p)) )
         if ( elai(p).lt.1.0 ) then
           DRGTLOSS(p) = 0.50* DRGTLOSS(p)
         endif
!
!     SIMILAR TO DRGTSTRS WE FIND COLDSTRS FOR EACH PFT. WE ASSUME THAT
!     MAX. COLD STRESS RELATED LEAF LOSS OCCURS WHEN TEMPERATURE IS 5 C
!     OR MORE BELOW PFT's THRESHOLD
!
        IF(TA.LE.(LWRTHRSH(ivt(p))-5.0+273.16))THEN
          COLDSTRS(p)=1.0
        ELSE IF(TA.GT.(LWRTHRSH(ivt(p))-5.0+273.16).AND.  &
        TA.LT.(LWRTHRSH(ivt(p))+273.16))THEN
          COLDSTRS(p)=1.0-((TA-(LWRTHRSH(ivt(p))-5.0+273.16))/5.0)   
        ELSE 
          COLDSTRS(p)=0.0
        ENDIF

        COLDSTRS(p)=MAX(0.0, MIN(1.0,COLDSTRS(p)))
!
!     USING THIS COLD STRESS TERM AND OUR TWO VEGETATION-DEPENDENT
!     PARAMETERS WE FIND LEAF LOSS RATE ASSOCIATED WITH COLD
!
!         COLD RELATED LEAF LOSS RATE
        COLDLSRT(p)=CDLSRTMX(ivt(p))*(COLDSTRS(p)**COLDA(ivt(p)))
!
!     ESTIMATE LEAF LOSS IN Kg C/M2 DUE TO COLD STRESS
!
        COLDLOSS(p)=GLEAFMAS(p)*( 1.0-EXP(-COLDLSRT(p)) )
!! liwp 2016-03-25 to reduce coldloss during maxgrowth (status=1)
         if (LFSTATUS(p).lt.1.5 ) then   !! status=1
             COLDLOSS(p)= COLDLOSS(p)*0.20
         endif
!
!     NOW THAT WE HAVE ALL TYPES OF LEAF LOSSES (DUE TO NORMAL TURNOVER,
!     COLD AND DROUGHT STRESS, AND FALL/HARVEST) WE TAKE THE LOSSES
!     FOR GRASSES AND USE THOSE TO TURN LIVE GREEN GRASS INTO DEAD
!     BROWN GRASS. WE THEN FIND THE LEAF LITTER FROM THE BROWN GRASS
!     WHICH WILL THEN GO INTO THE LITTER POOL.
!
    if (ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14) then       !! grasses
      GLEAFMAS(p)  = GLEAFMAS(p)-NRMLLOSS(p)-DRGTLOSS(p)- COLDLOSS(p)
       IF( GLEAFMAS(p).LT.0.0) THEN
        BLEAFMAS(p) = BLEAFMAS(p)+NRMLLOSS(p)+DRGTLOSS(p)+ COLDLOSS(p)+GLEAFMAS(p)
        GLEAFMAS(p)=0.0
       ELSE
        BLEAFMAS(p) = BLEAFMAS(p)+NRMLLOSS(p)+DRGTLOSS(p)+ COLDLOSS(p)
       ENDIF
        NRMLLOSS(p) = 0.0
        DRGTLOSS(p) = 0.0
        COLDLOSS(p) = 0.0
!   ASSUME LIFE SPAN OF BROWN GRASS IS 10% (ADJUSTABLE, here 10%) THAT OF GREEN GRASS
      NRMLLOSS(p) = BLEAFMAS(p)*(1.0-EXP(-1.0/(0.10*365*LFESPANY(ivt(p)) )))       
    endif           !! end of grass loop (ivt=12, 13, 14)
!
!     COMBINE NRMLLOSS, DRGTLOSS, AND COLDLOSS TOGETHER TO GET TOTAL
!     LEAF LITTER GENERATED, WHICH IS WHAT WE USE TO UPDATE GLEAFMASS,
!     EXCEPT FOR GRASSES, FOR WHICH WE HAVE ALERADY UPDATED GLEAFMASS.
!
    LEAFLITR(p) = NRMLLOSS(p)+DRGTLOSS(p)+COLDLOSS(p)+FLHRLOSS(p)
!     los14(p) = FLHRLOSS(p)
!
!! lwp 2009-07 update green leaf biomass for trees and crops, brown leaf for grasses
     if(ivt(p).le.11 ) then    !!    trees 
      GLEAFMAS(p) = GLEAFMAS(p) - LEAFLITR(p)
       if (GLEAFMAS(p).le.0.0) then
          LEAFLITR(p) = GLEAFMAS(p) + LEAFLITR(p)
          GLEAFMAS (p) = 0.0
!! lwp 2013-11-27 to set the harvest loss of crop to zero when there is no leaf
          bm11(p) = 0.0
       endif
     endif

     if(ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14) then    !!    Grasses
       BLEAFMAS(p) = BLEAFMAS(p)-LEAFLITR(p)     !!    Brown leaf mass for grass
        if (BLEAFMAS(p).le.0.0) then
          LEAFLITR(p) = BLEAFMAS(p) + LEAFLITR(p) 
          BLEAFMAS(p) = 0.0
          bm11(p) = 0.0
        endif
!        los14(p) = LEAFLITR(p)
     endif
     
     if(ivt(p).eq.15 .or. ivt(p).eq.16 ) then    !!    crops
      GLEAFMAS(p) = GLEAFMAS(p) - LEAFLITR(p)
       if (GLEAFMAS(p).le.0.0) then
          LEAFLITR(p) = GLEAFMAS(p) + LEAFLITR(p)
          GLEAFMAS (p) = 0.0
!! lwp 2013-11-27 to set the harvest loss of crop to zero when there is no leaf
          bm11(p) = 0.0
          bm21(p) = 0.0
          bm31(p) = 0.0
       endif
     endif

      calday = get_curr_calday(offset=0)
!! lwp 2013-5-16 add the following lines to begin a new year of NORMALDAYS, PANDAYS
!! lwp 2013-5-22 restore leaf status at the beginning of a new year
!! liwp 2016-03-25 for PFT=2
    if ( ivt(p).eq.2 .and. calday.gt.30.and.calday.lt.100 ) then   !! northern hemisphere spring 
       if (LFSTATUS(p).gt.2.9 ) then   !! status=3 or 4
         LFSTATUS(p) = 1.0
         CHKMODE(p)=1
       endif
    endif

 if ( ivt(p).eq.6.or.ivt(p).eq.7.or.ivt(p).eq.10.or.ivt(p).eq.11  &
    .or.ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14.or.ivt(p).eq.15 ) then
!! liwp 2016-4-25 restore leaf status to MAX GROWTH if status>=3 in spring 
    if (calday.gt.265.and.calday.lt.300) then    !! southern hemisphere spring 
       if (LFSTATUS(p).gt.2.2 ) then             !! status=3 or 4
         LFSTATUS(p)= 1.0                        !! force into "maxgrowth" 
         bm11(p)    = 0.0
         pandays(p) = max(1.0, pandays(p) )
         GLEAFMAS(p)= max (0.020, GLEAFMAS(p) )  !! set minimum lai=0.3 
         CHKMODE(p)=1
       endif
    endif
!! liwp 2016-4-25 restore leaf status to MAX GROWTH if status>=3 in spring 
   if (latdeg(g) .le. -40.0 ) then
    if (calday.gt.165 .and. calday.lt.210) then  !! southern December~January 
      if ( LFSTATUS(p).lt.2.5 ) then      !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then  
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )    !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
   elseif (latdeg(g) .le. -25.0 ) then
    if (calday.gt.170 .and. calday.lt.212) then  !! southern December~January 
      if ( LFSTATUS(p).lt.2.5 ) then      !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then  
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )    !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
   elseif (latdeg(g).le.-10.0 ) then
    if (calday.gt.175 .and. calday.lt.210) then  !! southern late December 
      if ( LFSTATUS(p).lt.2.5 ) then      !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then  
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )    !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
!! liwp 2016-03-21 the following lines added for cases within 10S
   elseif ( latdeg(g).lt.0.0 ) then
    if (calday.gt.180 .and. calday.lt.210) then  !! southern late December 
      if ( LFSTATUS(p).lt.2.5 ) then      !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then  
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )    !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
   endif                    !! end of latitude loop
 endif                      !! end of ivt=6,7,10,11,12,13,15

 if (ivt(p).eq.3.or.ivt(p).eq.6.or.ivt(p).eq.7.or.ivt(p).eq.8.or.ivt(p).eq.10.or.ivt(p).eq.11 &
        .or.ivt(p).eq.12.or.ivt(p).eq.13.or.ivt(p).eq.14.or.ivt(p).eq.15 ) then  !! northern deciduous
!! liwp 2016-4-25 restore leaf status to MAX GROWTH if status>=3 in spring 
    if ( calday.gt.85.and.calday.lt.120 ) then   !! northern hemisphere spring 
       if (LFSTATUS(p).gt.2.2 ) then             !! status=3 or 4
         LFSTATUS(p)= 1.0
         bm11(p)    = 0.0
         pandays(p) = max(1.0, pandays(p) )
         GLEAFMAS(p)= max (0.020, GLEAFMAS(p) )  !! set minimum lai=0.3 
         CHKMODE(p)=1
       endif
    endif
!! liwp 2016-4-25 restore leaf status to MAX GROWTH if status>=3 in spring 
  if (latdeg(g) .ge. 45.0 ) then
    if ( calday.lt.35 .or. calday.gt.360 ) then  !! boreal December 
      if ( LFSTATUS(p).lt.2.5 ) then      !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then  
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )    !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
  else if (latdeg(g) .ge. 25.0 .and. latdeg(g) .lt. 45.0 ) then
    if ( calday.lt.30 .or. calday.gt.360 ) then  !! boreal December 
      if ( LFSTATUS(p).lt.2.5 ) then      !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then  
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )  !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
  elseif (latdeg(g).ge.10.0 .and. latdeg(g).lt.25.0 ) then
    if (calday.lt.25 .or. calday.gt.363) then  !! northern hemisphere December 
      if ( LFSTATUS(p).lt.2.5 ) then           !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )  !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
  elseif (latdeg(g).gt.0.0 .and. latdeg(g).lt.10.0 ) then
    if (calday.lt.20 .or. calday.gt.360) then  !! northern hemisphere December 
      if ( LFSTATUS(p).lt.2.5 ) then           !! (stop growing)
         LFSTATUS(p) = 3.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= GLEAFMAS(p) 
         CHKMODE(p)=1
      else if ( LFSTATUS(p).gt.3.5 ) then
         LFSTATUS(p) = 4.0
         normaldays(p)=0.0
         maturedays(p)=0.0
         maxdays(p) = 0.0
         pandays(p) = 0.0
         bm11(p)= 0.0
         GLEAFMAS(p)= min(0.003,GLEAFMAS(p) )  !! forced into "no leaves" 
         CHKMODE(p)=1
      endif
    endif
  endif              !! end of latitude loop
 endif               !! end of ivt=3,6,7,8,10,11,15

!!zhangyw add 20150520 for rice
#if(defined RICEWATER)
  if (ivt(p).eq.16) then
!    if ( calday.ge.316) then     !! for rice zhangyyw
    if ( kmoda.ge.1112) then     !! for rice zhangyyw
         normaldays(p) = 0.0
         maturedays(p) = 0.0
         maxdays(p)    = 0.0
         LFSTATUS(p)   = 4.0
         pandays(p)    = 0.0
         bm11(p)       = 0.0
    endif
   endif
#endif

!! lwp 2013-05 set the minimum leafmass, for AILCG in the start of this Subroutine use GLEAFMAS 
      GLEAFMAS(p) = max (0.004 , GLEAFMAS(p) ) ! Spring broadleaf deciduous minlai=0.05/20
!! liwp 2015-11-14 set bottom limit for evergreen trees
        if(ivt(p).le.2 .or.ivt(p).eq.5 ) then
           GLEAFMAS(p) = max (0.25, GLEAFMAS(p) )
        endif
        if(ivt(p).eq.4 ) then
           GLEAFMAS(p) = max (0.30, GLEAFMAS(p) )    !! PFT4 lai > 5.0
        endif
!!lwp 20130526 set upper limit of GLEAFMAS not lai, add the excess leafmass to stem&root
      diff = GLEAFMAS(p) - min (0.41, GLEAFMAS(p) )  !! lai < 8.0 
      GLEAFMAS(p) = min (0.41, GLEAFMAS(p) )
     if (ivt(p).eq.12.or.ivt(p).eq.13 .or.ivt(p).eq.14 ) then     !! for grasses
!! liwp 2016-03-05 add 70% of leafmass more than threshold 0.4Kg to leaf litter
      ROOTMASS(p) = ROOTMASS(p) + diff *0.3
      LEAFLITR(P) = LEAFLITR(P) + diff *0.7
     else
      STEMMASS(p) = STEMMASS(p) + diff *0.1
      ROOTMASS(p) = ROOTMASS(p) + diff *0.1
      LEAFLITR(P) = LEAFLITR(P) + diff *0.8
     endif

      alai(p) = GLEAFMAS(p) * gdef(ivt(p))
      elai(p) =  alai(p) 
   enddo            !! end of all PFTs grids

!! liwp 2017-01-25  combine Turnover with Phenology
!
!dir$ concurrent
!cdir nodep
   do f = 1, fn
      p = filterp(f)
      g = pgridcell(p)
          STEMLITR(p)=0.0          ! TOTAL STEM LITTER
          ROOTLITR(p)=0.0          ! TOTAL ROOT LITTER
          NRMLSMLR(p)=0.0          ! STEM LITTER FROM NORMAL TURNOVER
          NRMLRTLR(p)=0.0          ! ROOT LITTER FROM NORMAL TURNOVER
!! lwp 2009-07 initialize the following two variables
        livefr_stem(p)=1.0
        livefr_root(p)=1.0
!
!     INITIALIZATION ENDS
!
        IF(STEMLIFE(ivt(p)).GT.ZERO)THEN
            NRMLSMLR(p)=STEMMASS(p)*(1.0-EXP(-1.0/(365*STEMLIFE(ivt(p)))))
!! lwp 2015-06-01 reduce stem loss during maximum & normal growth stages
          if ( LFSTATUS(p) .lt. 1.2 )   then   !! max growth mode
            NRMLSMLR(p)= 0.40* NRMLSMLR(p)
          else if ( LFSTATUS(p).lt.2.2 ) then  !! normal growth mode
            NRMLSMLR(p)= 0.85* NRMLSMLR(p)
          else if ( LFSTATUS(p).gt.3.2 ) then  !! "no leaf" mode
            NRMLSMLR(p)= 0.40* NRMLSMLR(p)
          endif
        ENDIF

        IF(ROOTLIFE(ivt(p)).GT.ZERO)THEN
            NRMLRTLR(p)=ROOTMASS(p)*(1.0-EXP(-1.0/(365*ROOTLIFE(ivt(p)))))
!! lwp 2011-01-31 reduce root loss during maximum, normal growth, & noleaf stages
          if ( LFSTATUS(p) .lt. 1.2 ) then     !! max growth mode
            NRMLRTLR(p)= 0.40* NRMLRTLR(p)
          else if ( LFSTATUS(p).lt.2.2 ) then  !! normal growth mode
            NRMLRTLR(p)= 0.85* NRMLRTLR(p)
          else if ( LFSTATUS(p).gt.3.2 ) then  !! "no leaf" mode
            NRMLRTLR(p)= 0.40* NRMLRTLR(p)
          endif
        ENDIF
!
!     IF CROPS ARE IN HARVEST MODE THEN WE START HARVESTING STEM AS WELL.
!     IF STEM HAS ALREADY BEEN HARVESTED THEN WE SET THE STEM HARVEST
!     LOSS EQUAL TO ZERO. THE ROOTS OF THE CROP DIE IN A SIMILAR WAY.
!
!     ADD STEM AND ROOT LITTER FROM ALL SOURCES
!
          STEMLITR(p)=NRMLSMLR(p)+los21(p)
          ROOTLITR(p)=NRMLRTLR(p)+los31(p)
!
! UPDATE STEM AND ROOT BIOMASS FOR LITTER DEDUCTIONS
!! lwp 2009-07 update stem and root biomass for litter deductions
       STEMMASS(p) = STEMMASS(p) - STEMLITR(p)
           if (STEMMASS(p).lt.0.0) then
             STEMLITR(p) = STEMLITR(p) + STEMMASS(p)
             STEMMASS(p) = 0.0
           endif
       ROOTMASS(p) = ROOTMASS(p) - ROOTLITR(p)
           if (ROOTMASS(p).lt.0.0) then
             ROOTLITR(p) = ROOTLITR(p) + ROOTMASS(p)
             ROOTMASS (p) = 0.0
           endif
!! lwp 2009-07 calculate live fraction of stem and root
       if (ivt(p).le.11) then        !! Trees and shrubs
         livefr_stem(p) = exp(-0.2835*STEMMASS(p))
         livefr_stem(p) = max (0.05, min(livefr_stem(p), 1.0))
         livefr_root(p) = exp(-0.2835*ROOTMASS(p))
         livefr_root(p) = max (0.08, min(livefr_root(p), 1.0))
!! liwp 2015-09-18 reduce the live fraction of stem & root for status=3,4
        if ( LFSTATUS(p).gt.2.5 ) then
             livefr_stem(p) = 0.7 * livefr_stem(p)
             livefr_root(p) = 0.7 * livefr_root(p)
        endif
       endif
!! lwp 2010-01 set bottom limit of Crop biomass
     if(ivt(p).eq.15.or.ivt(p).eq.16) then
        if(LFSTATUS(p).lt.2.5 ) then  !! with leaves on
           STEMMASS(p)= max(0.008, STEMMASS(p) )
           ROOTMASS(p)= max(0.008, ROOTMASS(p) )
        endif
        if (LFSTATUS(p).gt.3.5 ) then  !! without leaves
         if (STEMMASS(p).gt. 0.005) then
           DIFF=STEMMASS(p)-0.005
           STEMMASS(p) = STEMMASS(p) - DIFF
           STEMLITR(p) = STEMLITR(p) + DIFF
         endif
         if (ROOTMASS(p).gt. 0.005) then
           DIFF=ROOTMASS(p)-0.005
           ROOTMASS(p) = ROOTMASS(p) - DIFF
           ROOTLITR(p) = ROOTLITR(p) + DIFF
         endif
        endif
     endif    !! loop for crop 15
!! lwp 2011-01 set top limit of grass root biomass as 1.2 Kg C/m^2
     if( ivt(p).eq.12 .or. ivt(p).eq.13 .or. ivt(p).eq.14 ) then
         STEMMASS(p)= 0.0
         ROOTMASS(p)= max(0.050, ROOTMASS(p) )

         if ( ROOTMASS(p) .gt. 1.2) then
           DIFF = ROOTMASS(p) -1.2
           ROOTMASS(p) = ROOTMASS(p) - DIFF
           ROOTLITR(p) = ROOTLITR(p) + DIFF
         endif
     endif      !! loop for grasses 13 & 14
!! liwp 2017-01-25
         surl(p) = (LEAFLITR(p) + STEMLITR(p) )* 1000.
         rotl(p) = ROOTLITR(p) * 1000.
!! liwp 2017-03-30 set limit for PHENO2 days
         normaldays(p)= min (365._r8, normaldays(p) )
         maturedays(p)= min (365._r8, maturedays(p) )
         colddays(p)  = min (365._r8, colddays(p) )
         pandays(p)   = min (365._r8, pandays(p) )
         maxdays(p)   = min (365._r8, maxdays(p) )
   enddo
!! liwp 2017-01-25 move the above Turnover to Phenology
         
  end subroutine Phenology
!
!BOP
! !IROUTINE: SoilCarbon 
! ===================================================================
! SoilCarbon is a program to simulate soil carbon decomposition and heterorespiration
!    & nitrogen dynamics(not ready yet) which is coupled to DOLY vegetation model.
! ====================================================================       
    subroutine  SoilCarbon(fn, filterp, lbp, ubp, lbc, ubc, lbg, ubg, &
                           npd, surl, rotl, &
                           ts1d,ts2d,sm1d,sm2d, &
                           polc1, polc2, polc3, polc4, &
                           polc5, polc6, polc7, polc8, &
                           dsmc, dco2, nmin,nep ) 
!dth is the time step: if time step is half hour than dth=1/48
!
! Interface routine for AVIM.
!  1995  JJJ
!  29 Aug 2007 zhangyw Revision
!    July 2009 LiWeiping Revised
!-------------------------------Commons---------------------------------
! !USES:
     use shr_kind_mod , only : r8 => shr_kind_r8
     use clmtype
     use clm_varsur, only : sand3d, clay3d
     use time_manager, only : get_curr_date, get_nstep
!! liwp 2017-01-22 the following 6 lines are not included in the earlier version
!#ifdef SPMD
!  use spmdMod, only : masterproc, mpicom, MPI_REAL8, MPI_INTEGER, MPI_LOGICAL
!#else
!  use spmdMod, only : masterproc
!#endif
!
! !ARGUMENTS:
     implicit none
     integer , intent(in)    :: fn                 ! size of pft filter
     integer , intent(in)    :: filterp(fn)        ! pft filter
     integer , intent(in)    :: lbp, ubp           ! pft bounds
     integer , intent(in)    :: lbc, ubc           ! column bounds
     integer , intent(in)    :: lbg, ubg           ! gridcell bounds
     real(r8), intent(in   ) :: npd(lbp:ubp)       !
!    real(r8), intent(in   ) :: lit1(lbp:ubp)      !
!    real(r8), intent(in   ) :: lit2(lbp:ubp)      !
!    real(r8), intent(in   ) :: lit3(lbp:ubp)      !
     real(r8), intent(in   ) :: ts1d(lbp:ubp)      !
     real(r8), intent(in   ) :: ts2d(lbp:ubp)      !
     real(r8), intent(in   ) :: sm1d(lbp:ubp)
     real(r8), intent(in   ) :: sm2d(lbp:ubp)

     real(r8), intent(inout) :: polc1(lbp:ubp)     !
     real(r8), intent(inout) :: polc2(lbp:ubp)     !
     real(r8), intent(inout) :: polc3(lbp:ubp)     !
     real(r8), intent(inout) :: polc4(lbp:ubp)     !
     real(r8), intent(inout) :: polc5(lbp:ubp)     !
     real(r8), intent(inout) :: polc6(lbp:ubp)     !
     real(r8), intent(inout) :: polc7(lbp:ubp)     !
     real(r8), intent(inout) :: polc8(lbp:ubp)     !
     real(r8), intent(inout) :: dsmc(lbp:ubp)      !
     real(r8), intent(  out) :: dco2(lbp:ubp)      !
     real(r8), intent(  out) :: nmin(lbp:ubp)      !
     real(r8), intent(  out) :: nep (lbp:ubp)      !
     real(r8), intent(inout) :: surl(lbp:ubp)      !
     real(r8), intent(  in ) :: rotl(lbp:ubp)      !
!----------------------------------------------------------------------
!     real(r8),intent(in)  :: dth

! !LOCAL VARIABLES:
!
! local pointers to implicit in variables
!
     integer , pointer :: pcolumn(:)       ! pft's column index
     integer , pointer :: clandunit(:)     ! landunit index of corresponding column
     integer , pointer :: pgridcell(:)     ! pft's gridcell index
     integer , pointer :: ivt(:)           ! pft vegetation type
     integer , pointer :: ixy(:)           ! xy lon index (column-level)
     integer , pointer :: jxy(:)           ! xy lat index (column_level)
     real(r8), pointer :: londeg(:)        ! latitude (degrees)
     real(r8), pointer :: latdeg(:)        ! latitude (degrees)
     real(r8), pointer :: h2osoi_vol(:,:)  ! volumetric soil water (0<=h2osoi_vol<=watsat) [m3/m3]
   real(r8), pointer :: carbon(:)    !zhangyw 20081121

      integer  :: f,p,c,g, i, j, l            ! indices
      real(r8) :: LIGN,NITG
      real(r8) :: TX,WTL,LC
      real(r8) :: LNSC
      real(r8) :: CAL,CAP,CAS,CSP,CSA,FM
      real(r8) :: IFSC,IFMC,IRSC,IRMC
      real(r8) :: POC,PON,TNF1,TNF2,TNF,CIN3,CIN7
!!lwp 2008-7-26 change "F" to "XF" to avoid confusion when use CLM3 structure(f,p,c,g)
      real(r8) :: K(1:8),XF(1:8),CN(1:8),CF(1:8)
      real(r8) :: NF(1:8),ND(1:8),NLIM(1:8)
      real(r8) :: NLMT,NLM
      real(r8) :: TEM,FTEM,FMOI
      real(r8) :: surfABF,rootABF
      real(r8) :: ts1, ts2, ts3
      real(r8) :: w1, w2, wm12
      real(r8) :: sand,silt,clay
      integer  :: nstep
      real(r8) :: outsurl(lbp:ubp)  ! temporal for crop direct release of CO2 to the air
!----------------------------------------------------------------------
     ! Assign local pointers to derived type members (pft-level)

     pcolumn   => clm3%g%l%c%p%column
     pgridcell => clm3%g%l%c%p%gridcell
     ivt       => clm3%g%l%c%p%itype
     ixy       => clm3%g%l%c%ixy
     jxy       => clm3%g%l%c%jxy
     latdeg    => clm3%g%latdeg
     londeg    => clm3%g%londeg
     clandunit => clm3%g%l%c%landunit
     carbon    => clm3%g%l%c%p%pef%carbon    !!zhangyw 20081121
    
!     K = 1.50 * K/30. * dth      !! original is 1.5*K(unit is day-1)
!! lwp 2011-6-3 XF means the direct partial emission of CO2 from each pool
      XF(1)=0.6
      XF(2)=0.55
      XF(3)=0.55
      XF(4)=0.6
      XF(5)=0.6
      XF(6)=0.55        !! (original is 0.55)
      XF(7)=0.55
      XF(8)=0.55

!! lwp 2008-7-28 to move the following three lines from the end of this subroutine here
!! liwp 2017-01-18 revise the following two lines from DCO2 to DCO2(:), NMIN(:)
      DSMC(:) = 0.0
      DCO2(:) = 0.0
      NMIN(:) = 0.0

      nstep = get_nstep()

!dir$ concurrent
!cdir nodep
     do f = 1, fn
        p = filterp(f)
        c = pcolumn(p)
        i = ixy(c)
        j = jxy(c)

!! lwp 2011-5-5 to accelarate grass soil respiration
       if ( ivt(p).eq.13 .or. ivt(p).eq.14 ) then
          K(1)= 0.010_r8    !! (original is 0.006) 
          K(2)= 0.011_r8    !! (original is 0.007) 
          K(3)= 0.015_r8    !! (original is 0.009) 
          K(4)= 0.012_r8    !! (original is 0.008) 
          K(5)= 0.020_r8    !! (original is 0.0125) 
          K(6)= 0.020_r8    !! (original is 0.014)
          K(7)= 0.00100_r8  !! (original 0.0009)
          K(8)= 0.00008_r8  !! (original 0.00002)
       else
!! lwp 2011-5-6 change the above 8 parameters to constants directly
         K(1)=0.006_r8      !! (original is 0.006)
         K(2)=0.007_r8      !! (original is 0.007)
         K(3)=0.0085_r8     !! (original is 0.009)
         K(4)=0.0078_r8     !! (original is 0.008)
         K(5)=0.0120_r8     !! (original is 0.0125)
         K(6)=0.0130_r8     !! (original is 0.014)
         K(7)=0.00087_r8    !! (original 0.0009)
         K(8)=0.000025_r8   !! (original 0.00002)
       endif

!! liwp 2015-11-19 reduce crop litter to soil, as direct output of CO2 to the air
     if (ivt(p).eq.15) then
       outsurl(p)=surl(p) *0.10
       surl(p)   =surl(p) *0.90
     endif

!! lwp 2008-7-26 use soil parameters of the 6th layer(36cm deep from surface??)
!! lwp 2008-7-29 the unit of sand3d is percent, therefore should be divided by 100. 
            clay = (clay3d(i,j,4)+clay3d(i,j,5))/200.
            sand = (sand3d(i,j,4)+sand3d(i,j,5))/200.
            silt = 1.0 - sand - clay
      LNSC = 0.3_r8
      LC = EXP (-3.0 * LNSC)  !! LC = 0.4065696597 for LNSC=0.3
      WTL = 0.2_r8
 
!! lwp 2008-11-3 for all grasses and crops that LIGN is small but LIGN/NITG is high
        if (ivt(p).ge.12) then
         LIGN = 0.12_r8
!! lwp 2011-02-09 to decrease soil carbon storage under grass biome
         NITG = 0.012_r8     !! ( original is 0.012 )
        else 
         LIGN = 0.2_r8
         NITG = 0.008_r8
        endif

      CN(1) = 150.0_r8
      CN(2) = 150.0_r8
      CN(3) = 8.0_r8
      CN(4) = 8.0_r8
      CN(5) = 0.18_r8/NITG
      CN(6) = 0.18_r8/NITG
      CN(7) = 20.0_r8-400.0*NITG
      CN(8) = 16.0_r8-600.0*NITG

      FM = 0.85-0.018*LIGN/NITG  
      TX = 1.0 - 0.75 * (SILT + CLAY)
      CAL = (WTL/18.0) * (0.01 + 0.04 * SAND)
!! lwp 2011-6-2 to increase CAP (POLC3 to POLC8), hence decrease CAS(POLC3 to POLC7)
!! lwp 2011-6-3 XF(3) substitute for the above constant 0.55
    if ( ivt(p).eq.13 .or. ivt(p).eq.14 ) then
      CAP = 0.006 + 0.032 * CLAY
      CSP = 0.005 - 0.009 * CLAY
    else
      CAP = 0.003 + 0.032 * CLAY
      CSP = 0.003 - 0.009 * CLAY
    endif
      XF(3) = 0.85 - 0.68 * (SILT + CLAY)  !! (original)
      CAS = 1.0 - CAL - CAP - XF(3)
!!    CSP = 0.003 - 0.009 * CLAY    !! ( original)
!     if (CSP .le. 0.0)  CSP = 0.0
!     if (CSA .le. 0.0)  CSA = 0.0 
!! liwp 2017-01-20 changed the above two lines
      CSP = max(0._r8, CSP )
      CSA = max(0._r8, 1.0-CSP-0.55 )

!     The effect of temperature and moisture on decomposition
!! lwp 2008-8-20 use daily mean soil temperature and moisture rather than instant values 

        wm12 = sm1d(p)
!! liwp 2016-03-05 reduce sensitivity of SOC decomposition to soil moisture
        if ( wm12.le.0.1) then
             FMOI=0.15
        else if (wm12.gt.0.1.and.wm12.le.0.3) then
             FMOI=0.15 +2.0 *(wm12-0.1)
        else if (wm12.gt.0.3.and.wm12.le.0.6) then
             FMOI=0.55 +0.60*(wm12-0.3)
        else if (wm12.gt.0.6.and.wm12.le.0.8) then
             FMOI=0.73 +0.50*(wm12-0.6)
        else
             FMOI=0.83 
        endif

!! lwp 2008-8-20 replace the following instant soil temperature (ts1) with daily mean(ts1d)
!       TEM=ts1-273.16
        TEM=ts1d(p)-273.16

!! lwp 2008-11-11 to increase Soil respiration at low temperatures <=-5 ( original is <=0 )
      if  (TEM.le.0) FTEM = 0.0
!! lwp 2008-9-5 increase soil respiration at low T but reduce soil respiration at high T
      if ((TEM .gt.0.0) .and. (TEM .le.  5.0))    & 
          FTEM=0.018*TEM
      if ((TEM .gt.5.0) .and. (TEM .le. 10.0))    & 
          FTEM=0.090+0.019*(TEM-5.0) 
!         FTEM=0.11 +0.013*(TEM-5.0) 
      if ((TEM .gt.10.0) .and. (TEM .le. 20.0))   & 
          FTEM=0.180+0.031*(TEM-10.0)
      if ((TEM .gt.20.0) .and. (TEM .le. 35.0))   & 
          FTEM=0.490+0.027*(TEM-20.0)
      if ((TEM.gt.35.0) .and. (TEM.le.40.0))      & 
          FTEM=0.895
!         FTEM=0.95+0.008*(TEM-35.0)
      if ((TEM .gt. 40.0) .and. (TEM .le. 47.0))  & 
          FTEM=0.895 -0.127*(TEM-40.0)
      if  (TEM.gt.47.0) FTEM = 0.0

!! lwp 2010-12-21 to increase soil carbon storage by reducing the decomposition rate
      surfABF=FTEM*FMOI*0.90    !! original is 0.9

! lwp 2008-8-25 use daily top 1.7m soil moisture for rootzone rather than wm12 as top 10cm
        wm12 = sm2d(p)
!! liwp 2016-03-05 reduce sensitivity of SOC decomposition to soil moisture
        if ( wm12.le.0.1) then
             FMOI=0.15
        else if (wm12.gt.0.1.and.wm12.le.0.3) then
             FMOI=0.15 +2.0 *(wm12-0.1)
        else if (wm12.gt.0.3.and.wm12.le.0.6) then
             FMOI=0.55 +0.60*(wm12-0.3)
        else if (wm12.gt.0.6.and.wm12.le.0.8) then
             FMOI=0.73 +0.50*(wm12-0.6)
        else
             FMOI=0.83 
        endif

!! lwp 2008-8-20 replace the following instant soil temperature (ts2) with daily mean(ts2d)
        TEM=ts2d(p)-273.16

      if  (TEM.le. 0.0 ) FTEM = 0.0
      if ((TEM .gt.0.0) .and. (TEM .le.  5.0))    & 
!! lwp 2008-7-24 the following lines add one zero (0.06 to 0.006) according to Prof Ji.
          FTEM=0.018*TEM
      if ((TEM .gt.5.0) .and. (TEM .le. 10.0))    & 
          FTEM=0.090+0.020*(TEM-5.0) 
      if ((TEM .gt.10.0) .and. (TEM .le. 20.0))   & 
!! lwp 2008-7-29 to make FTEM "smoothly" connected from 0.15 to 0.95
!         FTEM=0.15 +0.032*(TEM-10.0)   !! (original is 0.032)
          FTEM=0.190+0.030*(TEM-10.0)
      if ((TEM .gt.20.0) .and. (TEM .le. 35.0))   & 
          FTEM=0.490+0.027*(TEM-20.0)
      if ((TEM.gt.35.0) .and. (TEM.le.40.0))      & 
          FTEM=0.895
      if ((TEM .gt. 40.0) .and. (TEM .le. 47.0))  & 
!         FTEM=0.965-0.137*(TEM-40.0)
          FTEM=0.895-0.127*(TEM-40.0)
      if  (TEM.gt.47.0) FTEM = 0.0

!! lwp 2010-12-21 to increase soil carbon storage by reducing the decomposition rate
      rootABF=FTEM*FMOI*0.90    !! original is 0.9
!! liwp 2017-02-17 decrease CF of tropical PFTs, therefore increase NEP
    if ( ivt(p).le.2 .or. ivt(p).eq.4 .or. ivt(p).eq.5 .or. ivt(p).eq.7 ) then 
      surfABF = surfABF * 0.95
      rootABF = rootABF * 0.95
    elseif ( ivt(p).ge.12 ) then
      surfABF = surfABF * 0.98
      rootABF = rootABF * 0.98
    endif

!     Limitations of available nitrogen on carbon mineralization
        POC = 0.0
        PON = 0.0

!! for surface pool
      CF(1) = polc1(p) * K(1) * surfABF * LC
      CF(4) = polc4(p) * K(4) * surfABF
      CF(5) = polc5(p) * K(5) * surfABF
!! for root pool
      CF(2) = polc2(p) * K(2) * rootABF * LC
      CF(3) = polc3(p) * K(3) * rootABF * TX
      CF(6) = polc6(p) * K(6) * rootABF
      CF(7) = polc7(p) * K(7) * rootABF
      CF(8) = polc8(p) * K(8) * rootABF

!! lwp 2008-7-28 to substitute the following line in FORTRAN 90 for the above do loop 
        NF = CF/CN             !!! NF(1),NF(2),......NF(8)

        POC = 2.0*(CF(1)+CF(2)) + CF(3)+CF(4)+CF(5)+CF(6)+CF(7)+CF(8)
        PON = 2.0*(NF(1)+NF(2)) + NF(3)+NF(4)+NF(5)+NF(6)+NF(7)+NF(8)

        ND(1) = CF(1)*(0.40/CN(4)+0.7/CN(7))
        ND(2) = CF(2)*(0.45/CN(3)+0.7/CN(7))
        ND(3) = CF(3)*(CAS/CN(7)+CAP/CN(8)+CAL/CN(3))
        ND(4) = CF(4)*0.4/CN(7)
        ND(5) = CF(5)*0.4/CN(4)
        ND(6) = CF(6)*0.45/CN(3)
        ND(7) = CF(7)*(CSA/CN(3)+CSP/CN(8))
        ND(8) = CF(8)*0.45/CN(3)

        TNF1 = 2.0*NF(1) + NF(5) + NF(4)
        TNF2 = PON -TNF1

        TNF = TNF1 * CN(4) + TNF2 * CN(3)

     if (TNF .le. POC) then    !! lwp123
        NLM = 0.0
!! lwp 2008-7-28 to revised the original F77 arrays to F90, delete do loop
         if (abs(POC) .lt. 1.0E-20) then
           NLIM = 1.0 
         else
           NLIM = TNF/POC
         endif
!       ND(Pc) = ND(Pc) * NLIM(Pc)
!       NF(Pc) = NF(Pc) * NLIM(Pc)
!       CF(Pc) = CF(Pc) * NLIM(Pc)
!       NLM = NLM + NLIM(Pc)/8.0
        ND = ND * NLIM
        NF = NF * NLIM
!! lwp 2013-11-25 comment out the following line to eliminate the Nitrogen limitation
!       CF = CF * NLIM
        NLM =(NLIM(1)+NLIM(2)+NLIM(3)+NLIM(4)   &
             +NLIM(5)+NLIM(6)+NLIM(7)+NLIM(8) )/8.0

     else                      !! lwp123 TNF > POC
!! lwp 2008-7-28 the following line for NLMT is useless, for NLMT is never used thereafter.
!       NLMT = 1.0
        NLIM = 1.0
        NLM  = 1.0
     endif
  
!     Carbon and nitrogen dynamics

      IFSC = surl(p) * (1.0 - FM)
      IFMC = surl(p) * FM
      IRSC = rotl(p) * (1.0 - FM)
      IRMC = rotl(p) * FM

      CIN3 = CF(2)*(1.0- XF(2)) + CF(6)*(1.0 - XF(6)) +   &
             CF(7)*CSA + CF(8)*(1.0 - XF(8))
!! lwp 2010-12-27 to increase CIN7, so as to increase POLC7(original is *0.7)
      CIN7 = CF(1)*0.7 + CF(4)*(1.0 - XF(4)) + CF(2)*0.7 + CF(3)*CAS

!! lwp 2008-7-26 revise this do loop using polc1~polc8 rather than polc(1:8)
      polc1(p) = polc1(p) + IFSC - 2.0 * CF(1)
      polc2(p) = polc2(p) + IRSC - 2.0 * CF(2)
      polc3(p) = polc3(p) + (CIN3 - CF(3))
      polc4(p) = polc4(p) + CF(1)*(1.0-XF(1)) + CF(5)*(1.0-XF(5)) - CF(4)
      polc5(p) = polc5(p) + IFMC - CF(5)
      polc6(p) = polc6(p) + IRMC - CF(6)
      polc7(p) = polc7(p) + (CIN7 - CF(7))
      polc8(p) = polc8(p) + (CF(7)*CSP + CF(3)*CAP - CF(8))
   	   
!! lwp 2008-7-28 to revised the original F77 arrays to F90, delete do loop
      NMIN(p) = NF(1) + NF(2) + NF(3) + NF(4) + NF(5) + NF(6) + NF(7) + NF(8)  & 
              -(ND(1) + ND(2) + ND(3) + ND(4) + ND(5) + ND(6) + ND(7) + ND(8) )
!! liwp 2017-04-17 decrease DCO2 (therefore increase NEP) by times a factor less than 1
      dco2(p) =(CF(1)*XF(1) + CF(2)*XF(2) + CF(3)*XF(3) + CF(4)*XF(4)  &
               +CF(5)*XF(5) + CF(6)*XF(6) + CF(7)*XF(7) + CF(8)*XF(8)  &
               +(CF(1)+CF(2))*0.30 ) *0.95 
!! lwp 2011-01-07 decrease DCO2, so increase NEP(original is 0.3, refer CIN7, 0.7+0.3=1)
!! liwp 2017-01-04 decrease DCO2 of tropical PFTs, therefore increase NEP of these PFTs 
#if ( defined OFFLINE )
    if ( ivt(p).eq.1 .or. ivt(p).eq.2 .or. ivt(p).eq.5) then 
      dco2(p) = dco2(p) *0.96
    endif
#else
    if ( ivt(p).le.4 .or. ivt(p).eq.5 .or. ivt(p).eq.7) then 
!----
!liwp      dco2(p) = dco2(p) *0.90
! wtw
      if ( lsmlon == 320 ) then    ! T106L46 
          dco2(p) = dco2(p) *0.94  ! 0.85 
      else
          dco2(p) = dco2(p) *0.94  ! 0.90  ! T42 or T266
      endif
    endif
#endif
      dsmc(p) = polc1(p) + polc2(p) + polc3(p) + polc4(p)  &
              + polc5(p) + polc6(p) + polc7(p) + polc8(p)
      
!! lwp 2009-07 CTEM's NPP is Kg C, nep is in g C/m**2/day 
      nep(p) = npd(p)*1000. - DCO2(p)
!! liwp 2015-11-19 reduce crop litter to soil, 20% as direct output of CO2 to the air
    if (ivt(p).eq.15 ) then
       nep(p) = nep(p) - outsurl(p)
    endif
!! zhangyw 20081121 add carbon for BCC_CSM (nep( C g/m**2/day --> C g/m**2/s ,+ up to atm ,- to lnd)
       carbon(p)=-1.0*nep(p)/86400.
   enddo
  end subroutine SoilCarbon 

end module CanopyFluxes2Mod

#endif
