#include <misc.h>
#include <preproc.h>    
#ifdef LAKE_DAI
 module  DaiLakeMod		   
   
!-----------------------------------------------------------------------
!
! !MODULE: DaiLakeMod
!
! !DESCRIPTION:
! Calculates lake temperatures and surface fluxes.
!
! PUBLIC TYPES:
  IMPLICIT NONE 
  SAVE

! PUBLIC MEMBER FUNCTIONS:
  public :: DaiLake	   

  public :: newsnow_lake 
  public :: laketem
  public :: snowwater_lake

! By Anning Huang 2016.11.17
  public :: hCapacity
  public :: hConductivity  	 

  public :: WATER
  public :: snowwater
  public :: soilwater

  public :: snowcompaction
  public :: snowlayerscombine
  public :: snowlayersdivide	
  public :: rain_snow_temp

! PRIVATE MEMBER FUNCTIONS:
  private :: roughness_lake
  private :: hConductivity_lake
  
! By Anning Huang 2016.11.17   
  private :: psi  
  private :: surfacerunoff
  private :: subsurfacerunoff
  private :: combo	 
  private :: wetbulb  
  private :: tridia	 
  private :: moninobuk1
  private :: moninobukini1

!-----------------------------------------------------------------------

  CONTAINS

!-----------------------------------------------------------------------
!
! ROUTINE: DaiLake
!
 subroutine DaiLake(lbc, ubc, lbp, ubp, num_lakec, filter_lakec, &
                              num_lakep, filter_lakep) 
!
! USES:
    use shr_kind_mod, only: r8 => shr_kind_r8 

    use clmtype
    use time_manager       , only : get_step_size	  
    use clm_varpar         , only : nlevlak,nlevsoi,nlevsno ! by Huang Anning 2015.4.14	   
    use clm_varcon         , only : hvap, hsub, hfus, cpair, cpliq, tkwat, tkice, &
                                    sb, vkc, grav, denh2o, tfrz, spval,&
									ssi,wimp, tcrit         ! by Huang Anning 2015.4.15 
    implicit none
    integer, parameter :: itypwat  =  4              ! land water type (0=soil, 1 = urban and built-up, 2 = wetland,
    integer, parameter :: nl_soil  =  nlevsoi        ! number of soil layers
    integer, parameter :: nl_lake  =  nlevlak        ! number of lake layers
    integer, parameter :: maxsnl   = -nlevsno        ! max number of snow layers

    integer, intent(in) :: lbc, ubc                ! column-index bounds
    integer, intent(in) :: lbp, ubp                ! pft-index bounds
    integer, intent(in) :: num_lakec               ! number of column non-lake points in column filter
    integer, intent(in) :: filter_lakec(ubc-lbc+1) ! column filter for non-lake points
    integer, intent(in) :: num_lakep               ! number of column non-lake points in pft filter
    integer, intent(in) :: filter_lakep(ubp-lbp+1) ! pft filter for non-lake points
!
! CALLED FROM: subroutine driver
!
! REVISION HISTORY:
! Author: Gordon Bonan
! 15 September 1999: Yongjiu Dai; Initial code
! 15 December 1999:  Paul Houser and Jon Radakovich; F90 Revision
! Migrated to clm2.1 new data structures by Peter Thornton and M. Vertenstein
!
! !LOCAL VARIABLES:
!
! local pointers to implicit in arguments
!
!----------------------------gridcell-level-------------------------------------------------------
!
    real(r8), pointer :: forc_t(:)          ! atmospheric temperature (Kelvin)				  !in
    real(r8), pointer :: forc_pbot(:)       ! atmospheric pressure (Pa)	 					  !in
	real(r8), pointer :: forc_hgt(:)        ! atmospheric reference height (m)				  !in
    real(r8), pointer :: forc_hgt_u(:)      ! observational height of wind [m]   			  !in
    real(r8), pointer :: forc_hgt_t(:)      ! observational height of temperature [m] 	      !in
	real(r8), pointer :: forc_hgt_q(:)      ! observational height of humidity [m]			  !in
    real(r8), pointer :: forc_th(:)         ! atmospheric potential temperature (Kelvin)	  !in
    real(r8), pointer :: forc_q(:)          ! atmospheric specific humidity (kg/kg)			  !in
    real(r8), pointer :: forc_u(:)          ! atmospheric wind speed in east direction (m/s)  !in
    real(r8), pointer :: forc_v(:)          ! atmospheric wind speed in north direction (m/s) !in
    real(r8), pointer :: forc_rho(:)        ! density (kg/m**3)								  !in
    real(r8), pointer :: forc_lwrad(:)      ! downward infrared (longwave) radiation (W/m**2) !in	
    real(r8), pointer :: forc_snow(:)       ! snow rate [mm/s]								  !in
    real(r8), pointer :: forc_rain(:)       ! rain rate [mm/s]	 							  !in
	real(r8), pointer :: forc_solad(:,:)    ! direct beam radiation (vis=forc_sols , nir=forc_soll) (numrad) !in
    real(r8), pointer :: forc_solai(:,:)    ! diffuse radiation (vis=forc_solsd, nir=forc_solld) (numrad)	 !in	    
	real(r8), pointer :: lat(:)             ! latitude (radians)   											 !in
!
!---------------------------column-level---------------------------------------------------------
!
    integer , pointer :: cgridcell(:)       ! column's gridcell index 							       !in
    real(r8), pointer :: dz(:,:)            ! layer thickness (m)								       !in
    real(r8), pointer :: z(:,:)             ! layer depth (m)									       !in
    real(r8), pointer :: csol(:,:)          ! heat capacity, soil solids (J/m**3/Kelvin) (nlevsoi)	   !in
	real(r8), pointer :: watsat(:,:)        ! volumetric soil water at saturation (porosity) (nlevsoi) !in
    real(r8), pointer :: tkdry(:,:)         ! thermal conductivity, dry soil (W/m/Kelvin) (nlevsoi)	   !in
	real(r8), pointer :: tksatu(:,:)        ! thermal conductivity, saturated soil [W/m-K] (nlevsoi)   !in

	real(r8), pointer :: dzsoisno(:,:)      ! soil/snow layer thickness (m)						   !inout1
    real(r8), pointer :: zsoisno(:,:)       ! soil/snow layer depth (m)							   !inout2
	real(r8), pointer :: lake_icef(:,:)     ! mass fraction of lake layer that is frozen   		   !inout3

    real(r8), pointer :: snowage(:)         ! non dimensional snow age [-] (new)			       !inout4
    real(r8), pointer :: t_soisno(:,:)      ! soil temperature (Kelvin)  (-nlevsno+1:nlevsoi)	   !inout5
    real(r8), pointer :: h2osoi_liq(:,:)    ! liquid water (kg/m2) (new) (-nlevsno+1:nlevsoi)	   !inout6
    real(r8), pointer :: h2osoi_ice(:,:)    ! ice lens (kg/m2) (new) (-nlevsno+1:nlevsoi)  		   !inout7
	   																										
	real(r8), pointer :: t_lake(:,:)        ! lake temperature (Kelvin)	                           !inout8
	real(r8), pointer :: h2osno(:)          ! snow water (mm H2O)							       !inout9
    real(r8), pointer :: snowdp(:)          ! snow height (m)   							       !inout10
    real(r8), pointer :: t_grnd(:)          ! ground temperature (Kelvin)  	    			       !inout11
    real(r8), pointer :: errsoi(:)          ! soil/lake energy conservation error (W/m**2) 	       !  out12
    real(r8), pointer :: qmelt(:)           ! snow melt [mm/s]								       !  out13
!
! local pointers to implicit out arguments	
!
!----------------------------pft-level---------------------------------------------------------
!    
    integer , pointer :: pcolumn(:)         ! pft's column index									!in
    integer , pointer :: pgridcell(:)       ! pft's gridcell index									!in
    real(r8), pointer :: sabg(:)            ! solar radiation absorbed by ground (W/m**2)			!in

  	real(r8), pointer :: t_ref2m(:)         ! 2 m height surface air temperature (Kelvin)           !out14
    real(r8), pointer :: q_ref2m(:)         ! 2 m height surface specific humidity (kg/kg)	        !out15
    real(r8), pointer :: t_veg(:)           ! vegetation temperature (Kelvin)				        !out15
	real(r8), pointer :: eflx_lwrad_out(:)  ! emitted infrared (longwave) radiation (W/m**2)        !out17
    real(r8), pointer :: eflx_lwrad_net(:)  ! net infrared (longwave) rad (W/m**2) [+ = to atm]     !out18    
    real(r8), pointer :: eflx_soil_grnd(:)  ! soil heat flux (W/m**2) [+ = into soil]			    !out19
    real(r8), pointer :: eflx_lh_tot(:)     ! total latent heat flux (W/m8*2)  [+ to atm]		    !out20
    real(r8), pointer :: eflx_lh_grnd(:)    ! ground evaporation heat flux (W/m**2) [+ to atm]	    !out21
 	real(r8), pointer :: eflx_sh_grnd(:)    ! sensible heat flux from ground (W/m**2) [+ to atm]    !out22
    real(r8), pointer :: eflx_sh_tot(:)     ! total sensible heat flux (W/m**2) [+ to atm]		    !out23
    real(r8), pointer :: taux(:)            ! wind (shear) stress: e-w (kg/m/s**2)				    !out24
    real(r8), pointer :: tauy(:)            ! wind (shear) stress: n-s (kg/m/s**2)			    	!out25
    real(r8), pointer :: ram1(:)            ! aerodynamical resistance (s/m)	 				    !out26
	real(r8), pointer :: qflx_prec_grnd(:)  ! water onto ground including canopy runoff [kg/(m2 s)] !out27
    real(r8), pointer :: qflx_evap_soi(:)   ! soil evaporation (mm H2O/s) (+ = to atm)				!out28
    real(r8), pointer :: qflx_evap_tot(:)   ! qflx_evap_soi + qflx_evap_veg + qflx_tran_veg			!out29
!
! ------------------------ Dummy Argument ------------------------------
  real(r8)  deltim                 ! seconds in a time step [second]
  real(r8)  dlat                   ! latitude (radians)	   
  real(r8)  lakedepth              ! column lake depth (m) 	
  real(r8)  dz_lake(nl_lake)       ! Lake layer thickness (m)  
  real(r8)  lake_icefrac(nl_lake)  ! mass fraction of lake layer that is frozen	    	   
  real(r8)  pg_rain                ! liquid water onto ground [kg/(m2 s)]
  real(r8)  pg_snow                ! ice onto ground [kg/(m2 s)]
  real(r8)  t_precip               ! snowfall/rainfall temperature [kelvin]		 
  real(r8)  bifall                 ! bulk density of newly fallen dry snow [kg/m3]  
  real(r8)  fiold(maxsnl+1:nl_soil)! fraction of ice relative to the total water	
  integer   snl                    ! number of snow layers
  real(r8)  sag                    ! non dimensional snow age [-]
  real(r8)  scv                    ! snow mass (kg/m2--mm)

  real(r8) porsl(nl_soil)          ! soil porosity									 input
  real(r8) dkdry(nl_soil)          ! thermal conductivity for dry soil [W/m-K]		 input
  real(r8) dksatu(nl_soil)         ! Thermal conductivity of saturated soil [W/m-K]	 input
  real(r8) csol1(nl_soil)          ! heat capacity of soil soilds [J/(m3 K)]	  	 input
  real(r8) sabg1                   ! solar radiation absorbed by ground [W/m2]		 input
   
  real(r8) ram                            ! 1 aerodynamical resistance [s/m]   updated	 
  real(r8) t_soisno1(maxsnl+1:nl_soil)    ! 2 soil/snow temperature [K]        updated 	    
  real(r8) t_lake1(nl_lake)               ! 3 lake layer temperature (K)	   updated 	
  real(r8) t_grnd1                        ! 4 surface temperature (kelvin)     updated
  real(r8) taux1                          ! 5 wind stress: E-W [kg/m/s**2] 	   updated
  real(r8) tauy1                          ! 6 wind stress: N-S [kg/m/s**2]     updated
  real(r8) errsoi1                        ! 7 soil/lake energy conservation error (W/m^2)	 updated
  real(r8) snowdp1                        ! 8 snow depth (m)  				   updated
  
  real(r8) forc_hgt_u1             ! observational height of wind [m]			   
  real(r8) forc_hgt_t1             ! observational height of temperature [m]
  real(r8) forc_hgt_q1             ! observational height of humidity [m]	
  real(r8) forc_t1                 ! temperature at agcm reference height [kelvin]
  real(r8) forc_q1                 ! specific humidity at agcm reference height [kg/kg]	   	

  real(r8) forc_us                 ! wind component in eastward direction [m/s]
  real(r8) forc_vs                 ! wind component in northward direction [m/s]	 
  real(r8) forc_rhoair             ! density air [kg/m3]
  real(r8) forc_psrf               ! atmosphere pressure at the surface [pa]	
  real(r8) forc_sols               ! atm vis direct beam solar rad onto srf [W/m2]
  real(r8) forc_soll               ! atm nir direct beam solar rad onto srf [W/m2] 
  real(r8) forc_solsd              ! atm vis diffuse solar rad onto srf [W/m2]
  real(r8) forc_solld              ! atm nir diffuse solar rad onto srf [W/m2] 
  real(r8) forc_frl                ! atmospheric infrared (longwave) radiation [W/m2]   
!	 
!------------------------- Updated below ----------------------------
!
  real(r8) dz_soisno(maxsnl+1:nl_soil)    ! soil/snow layer thickness (m)
  real(r8) z_soisno(maxsnl+1:nl_soil)     ! soil/snow node depth [m]
  real(r8) zi_soisno(maxsnl:nl_soil)      ! soil/snow depth of layer interface [m] 	  
  real(r8) wliq_soisno(maxsnl+1:nl_soil)  ! soil/snow liquid water (kg/m2)
  real(r8) wice_soisno(maxsnl+1:nl_soil)  ! soil/snow ice lens (kg/m2)	 	     	 
  
  integer  imelt(maxsnl+1:nl_soil)        ! flag for: melting=1, freezing=2, Nothing happens=0	  

  real(r8) tref     ! 2 m height air temperature [kelvin]
  real(r8) qref     ! 2 m height air specific humidity	   
  real(r8) fsena    ! sensible heat from canopy height to atmosphere [W/m2]
  real(r8) fevpa    ! evapotranspiration from canopy height to atmosphere [mm/s]
  real(r8) lfevpa   ! latent heat flux from canopy height to atmosphere [W/m2]    
  real(r8) fseng    ! sensible heat flux from ground [W/m2]
  real(r8) fevpg    ! evaporation heat flux from ground [mm/s] 
  real(r8) qseva    ! ground surface evaporation rate (mm h2o/s)	
  real(r8) qsubl    ! sublimation rate from snow pack (mm H2O /s) [+]
  real(r8) qsdew    ! surface dew added to snow pack (mm H2O /s) [+]
  real(r8) qfros    ! ground surface frosting formation (mm H2O /s) [+]     
  real(r8) olrg     ! outgoing long-wave radiation from ground+canopy  
  real(r8) fgrnd    ! net ground heat flux [W/m2]  		
  real(r8) trad     ! radiative temperature [K]	   
  real(r8) emis     ! averaged bulk surface emissivity
  real(r8) z0ma     ! effective roughness [m]
  real(r8) zol      ! dimensionless height (z/L) used in Monin-Obukhov theory
  real(r8) rib      ! bulk Richardson number in surface layer
  real(r8) ustar    ! u* in similarity theory [m/s]
  real(r8) qstar    ! q* in similarity theory [kg/kg]
  real(r8) tstar    ! t* in similarity theory [K]
  real(r8) fm       ! integral of profile function for momentum
  real(r8) fh       ! integral of profile function for heat
  real(r8) fq       ! integral of profile function for moisture
  real(r8) sm       ! rate of snowmelt [mm/s, kg/(m2 s)]
!
!----------------------------------------------------------------------------------------
!
! OTHER LOCAL VARIABLES: Added by Anning Huang 2015.4.15
!
! real(r8) :: w_old      ! liquid water mass of the column at the previous time step (mm)
  real(r8) :: forc_prc   ! convective precipitation [mm/s]
  real(r8) :: forc_prl   ! large scale precipitation [mm/s]

  real(r8) :: prc_rain   ! convective rainfall [kg/(m2 s)]
  real(r8) :: prc_snow   ! convective snowfall [kg/(m2 s)] 
  real(r8) :: prl_rain   ! large scale rainfall [kg/(m2 s)]
  real(r8) :: prl_snow   ! large scale snowfall [kg/(m2 s)]	  
 
  integer  :: fp,g,c,p,j ! do loop or array index	
  real(r8) :: a,aa,aaa	 

! For global simulations with 10 body layers
! the default (50 m lake) body layer thicknesses are given by :
! Added by Anning Huang 2015/07/19/
  real(r8) :: dzlak(10)	      ! Lake layer thickness (m)   
  real(r8) :: depthratio      ! ratio of lake depth to standard
!---------------------------------------------------------------------------
! Assign local pointers to derived type members (gridcell-level)
!
    forc_t         => clm3%g%a2ls%forc_t
    forc_pbot      => clm3%g%a2ls%forc_pbot
    forc_hgt       => clm3%g%a2ls%forc_hgt
    forc_hgt_u     => clm3%g%a2ls%forc_hgt_u
    forc_hgt_t     => clm3%g%a2ls%forc_hgt_t
    forc_hgt_q     => clm3%g%a2ls%forc_hgt_q
    forc_th        => clm3%g%a2ls%forc_th
    forc_q         => clm3%g%a2ls%forc_q
    forc_u         => clm3%g%a2ls%forc_u
    forc_v         => clm3%g%a2ls%forc_v
    forc_rho       => clm3%g%a2ls%forc_rho 
    forc_lwrad     => clm3%g%a2lf%forc_lwrad
    forc_snow      => clm3%g%a2lf%forc_snow
    forc_rain      => clm3%g%a2lf%forc_rain	   
   
    lat            => clm3%g%lat
!   Added by Anning Huang 2015/07/19/
!   lakedp         => clm3%g%lakedp
			
    forc_solad     => clm3%g%a2lf%forc_solad    ! direct beam radiation (vis=forc_sols , nir=forc_soll ) (numrad)
    forc_solai     => clm3%g%a2lf%forc_solai    ! diffuse radiation (vis=forc_solsd, nir=forc_solld) (numrad)
!-----------------------------------------------------------------------------------------------------------
!
! Assign local pointers to derived type members (column-level)
!
    cgridcell      => clm3%g%l%c%gridcell	  
           dz      => clm3%g%l%c%cps%dz
            z      => clm3%g%l%c%cps%z	  
       t_lake      => clm3%g%l%c%ces%t_lake

     dzsoisno      => clm3%g%l%c%cps%dzsoisno
      zsoisno      => clm3%g%l%c%cps%zsoisno	
    lake_icef      => clm3%g%l%c%ces%lake_icef  	 ! mass fraction of lake layer that is frozen	      
	   t_grnd      => clm3%g%l%c%ces%t_grnd	 
       h2osno      => clm3%g%l%c%cws%h2osno
       snowdp      => clm3%g%l%c%cps%snowdp	  
       errsoi      => clm3%g%l%c%cebal%errsoi
        qmelt      => clm3%g%l%c%cwf%qmelt
!
!----------------Added by Anning Huang 2015.4.15
!
       snowage     => clm3%g%l%c%cps%snowage     ! non dimensional snow age [-] (new)
      t_soisno     => clm3%g%l%c%ces%t_soisno    ! soil temperature (Kelvin)  (-nlevsno+1:nlevsoi)     
    h2osoi_liq     => clm3%g%l%c%cws%h2osoi_liq  ! liquid water (kg/m2) (new) (-nlevsno+1:nlevsoi)
    h2osoi_ice     => clm3%g%l%c%cws%h2osoi_ice  ! ice lens (kg/m2) (new) (-nlevsno+1:nlevsoi)
	   	  csol     => clm3%g%l%c%cps%csol        ! heat capacity, soil solids (J/m**3/Kelvin) (nlevsoi)
        watsat     => clm3%g%l%c%cps%watsat      ! volumetric soil water at saturation (porosity) (nlevsoi)
         tkdry     => clm3%g%l%c%cps%tkdry       ! thermal conductivity, dry soil (W/m/Kelvin) (nlevsoi)
        tksatu     => clm3%g%l%c%cps%tksatu      ! thermal conductivity, saturated soil [W/m-K] (new) (nlevsoi)

!    frac_iceold   => clm3%g%l%c%cps%frac_iceold ! fraction of ice relative to the tot water (new) (-nlevsno+1:nlevsoi)
!
!-------------------------------------------------------------------------------------------------
!
! Assign local pointers to derived type members (pft-level)
!
      pcolumn      => clm3%g%l%c%p%column
      pgridcell    => clm3%g%l%c%p%gridcell
           sabg    => clm3%g%l%c%p%pef%sabg 
        t_ref2m    => clm3%g%l%c%p%pes%t_ref2m		    
        q_ref2m    => clm3%g%l%c%p%pes%q_ref2m		    
          t_veg    => clm3%g%l%c%p%pes%t_veg		    
  eflx_lwrad_out   => clm3%g%l%c%p%pef%eflx_lwrad_out   
  eflx_lwrad_net   => clm3%g%l%c%p%pef%eflx_lwrad_net   
  eflx_soil_grnd   => clm3%g%l%c%p%pef%eflx_soil_grnd   
     eflx_lh_tot   => clm3%g%l%c%p%pef%eflx_lh_tot	    
    eflx_lh_grnd   => clm3%g%l%c%p%pef%eflx_lh_grnd	    
    eflx_sh_grnd   => clm3%g%l%c%p%pef%eflx_sh_grnd	    
     eflx_sh_tot   => clm3%g%l%c%p%pef%eflx_sh_tot	    
            ram1   => clm3%g%l%c%p%pps%ram1			    
            taux   => clm3%g%l%c%p%pmf%taux			    
            tauy   => clm3%g%l%c%p%pmf%tauy			    
  qflx_prec_grnd   => clm3%g%l%c%p%pwf%qflx_prec_grnd   
   qflx_evap_soi   => clm3%g%l%c%p%pwf%qflx_evap_soi    
   qflx_evap_tot   => clm3%g%l%c%p%pwf%qflx_evap_tot    
!
! Determine step size
!	
    deltim = get_step_size() ! time step	
!
! Begin calculations 
!  
    do fp = 1, num_lakep  	       ! number of column non-lake points in pft filter     
	  
	          p = filter_lakep(fp) ! pft filter for non-lake points
              c = pcolumn(p)	   ! pft's column index
              g = pgridcell(p)	   ! column's gridcell index

!----------------------------------------------------------------------------------
! Adde by Anning Huang 2015/07/19/
!    dzlak = (/0.1, 1., 2., 3., 4., 5., 7., 7., 10.45, 10.45/)    !Lake layer thickness (m)
!        if(lakedepth> 1. .and. lakedepth< 1000.)then
!            depthratio = lakedepth/sum(dzlak(1:nl_lake))
!            dz_lake(1) = dzlak(1)
!            dz_lake(2:nl_lake-1) = dzlak(2:nl_lake-1)*depthratio
!            dz_lake(nl_lake) = dzlak(nl_lake)*depthratio-(dz_lake(1) - dzlak(1)*depthratio)
!         else if(lakedepth> 0. .and. lakedepth <= 1.)then
!            dz_lake = lakedepth/nl_lake
!         else                             ! non land water bodies or missing value of the lake depth 
!            lakedepth = sum(dzlak(1:nl_lake))
!            dz_lake = dzlak
!         end if
!--------------------------------------------------------------------------------------------------------
   	  
	   dz_lake(1:) = dz(c,1:)			    ! Lake layer thickness (m)
	     lakedepth = sum(dz_lake(1:nl_lake))! column lake depth (m)

!
!-----------------------------forcing data-----------------------------------------

!---------------------------gridcell-level------------------------------------------
!	     
	      dlat  = lat(g)           ! latitude (radians)
!  Added by Anning Huang 2015/07/19/         
!	  lakedepth = lakedp(g)	       ! lake depth (m)
		 		  	 		  	   
  	  forc_sols = forc_solad(g,1)  ! atm visible direct beam solar rad onto srf [W/m2]  
      forc_soll = forc_solad(g,2)  ! atm Near-infrared (NIR) direct beam solar rad onto srf [W/m2]
	 forc_solsd = forc_solai(g,1)  ! atm vis diffuse solar rad onto srf [W/m2]
     forc_solld = forc_solai(g,2)  ! atm nir diffuse solar rad onto srf [W/m2]
      forc_psrf = forc_pbot(g)     ! atmosphere pressure at the surface [pa]	 	   	   	       			  	 
	forc_rhoair = forc_rho(g)      ! density (kg/m**3)	    
        forc_us = forc_u(g)		   ! u wind (m/s)
        forc_vs = forc_v(g)  	   ! v wind (m/s)  	    
  	   forc_frl = forc_lwrad(g)	   ! atmospheric infrared (longwave) radiation [W/m2]  	  
	 
	    forc_q1 = forc_q(g)        ! specific humidity at agcm reference height [kg/kg]	
	    forc_t1 = forc_t(g) 	   ! temperature at agcm reference height [kelvin]	
	forc_hgt_u1 = forc_hgt_u(g)    ! observational height of wind [m]
    forc_hgt_t1 = forc_hgt_t(g)    ! observational height of temperature [m]
    forc_hgt_q1 = forc_hgt_q(g)    ! observational height of humidity [m]   
	      sabg1 = sabg(p)  		   ! solar radiation absorbed by ground [W/m2]

!
!-------------------------------------------------------------------------------------
!	forc_prl and forc_prc should be up dated   An Huang 2015.7.18
!
       forc_prc = 0                         ! convective precipitation [mm/s]
       forc_prl = 0                         ! large scale precipitation [mm/s]
	   	  
      call rain_snow_temp (forc_t1,forc_q1,forc_psrf,forc_prc,forc_prl,tcrit,&
                           prc_rain,prc_snow,prl_rain,prl_snow,t_precip,bifall)
						   	 
	  pg_rain   = prc_rain + prl_rain       ! rain rate [mm/s] 
      pg_snow   = prc_snow + prl_snow       ! snow rate [mm/s]	 
	  pg_snow   = forc_snow(g)              ! snow rate [mm/s]
      pg_rain   = forc_rain(g)              ! rain rate [mm/s]

!----------------------Column-level---------------------------------------------------------
		       scv = h2osno(c)              ! snow water (kg/m2--mm)		  !iniTimeVar.F90	
		       sag = snowage(c)	            ! non dimensional snow age [-]    !iniTimeVar.F90	   	 
		
		   t_grnd1 = t_grnd(c)		        ! ground temperature (Kelvin)     !iniTimeVar.F90  
		   snowdp1 = snowdp(c)              ! snow depth (m)			 	  !iniTimeVar.F90  	 
	    t_lake1(:) = t_lake(c,:)            ! lake layer temperature (K) 	  !iniTimeVar.F90 	    
   lake_icefrac(:) = lake_icef(c,:)  	  	! lake ice fraction			      !iniTimeVar.F90 	    
      t_soisno1(:) = t_soisno(c,:)          ! soil/snow temperature [K]		  !iniTimeVar.F90  
	   
    wliq_soisno(:) = h2osoi_liq(c,:)        ! soil/snow liquid water (kg/m2)  !iniTimeVar.F90 
    wice_soisno(:) = h2osoi_ice(c,:)        ! soil/snow liquid water (kg/m2)  !iniTimeVar.F90 
  
! Initialized in Initimeconst.F90 	nl_soil		 
	   csol1(:) = csol(c,:)                 ! heat capacity of soil soilds [J/(m3 K)] only for input
       porsl(:) = watsat(c,:)               ! soil porosity (m3/m3)	 
       dkdry(:) = tkdry(c,:)                ! thermal conductivity for dry soil [W/m-K]
      dksatu(:) = tksatu(c,:)               ! Thermal conductivity of saturated soil [W/m-K]	 
 	
!
!=========================================================================================
!								Added by Anning Huang 2015/07/18/
! ---------------------------------------------------------------
! 1 Define the soil layers's thickness 
! ............................................................... 
  	    z_soisno(:) = zsoisno(c,:)			! soil/snow Lake layer thickness (m)
  	   dz_soisno(:) = dzsoisno(c,:)		    ! soil/snow/Lake layer thickness (m)

! ---------------------------------------------------------------
! 2 define snow layers's thicknes  ! should be updated	  
! ...............................................................
		 snl = 0   	  
      do j=maxsnl+1,0
         if (wliq_soisno(j)+wice_soisno(j) > 0.) then
            snl=snl-1
         endif
      enddo	

!print*,fp,snl,wliq_soisno(maxsnl+1:0),wice_soisno(maxsnl+1:0)	 	  
	  	   
            zi_soisno(0)=0	
      if (snl <0 ) then
         do j = -1, snl, -1
            zi_soisno(j)=zi_soisno(j+1)-dz_soisno(j+1)
         enddo
      endif

      do j = 1,nl_soil
         zi_soisno(j)=zi_soisno(j-1)+dz_soisno(j)
      enddo

      fiold(:) = 0.0
      if (snl < 0) then
         fiold(snl+1:0)=wice_soisno(snl+1:0)/(wliq_soisno(snl+1:0)+wice_soisno(snl+1:0))
      endif
   

! Added by Anning Huang 2015/07/18/
!==========================================================================================	

    call newsnow_lake ( &
            ! "in" arguments
            ! ---------------
            maxsnl    , nl_lake   , deltim      , dz_lake ,&
            pg_rain   , pg_snow   , t_precip    , bifall  ,&    
           ! "inout" arguments
           ! ------------------
           t_lake1       , zi_soisno(:0), z_soisno(:0)    , &
           dz_soisno(:0) , t_soisno1(:0) , wliq_soisno(:0) , wice_soisno(:0) ,&
           fiold(:0)     , snl          , sag              , scv             ,&
           snowdp1       , lake_icefrac )

     call laketem (&
           ! "in" arguments
           ! -------------------
           itypwat     , maxsnl     , nl_soil     , nl_lake    , &
		   dlat        , deltim     , forc_hgt_u1 , forc_hgt_t1, &
		   forc_hgt_q1 , forc_us    , forc_vs     , forc_t1    , &
		   forc_q1     , forc_rhoair, forc_psrf   , forc_sols  , &
		   forc_soll   , forc_solsd , forc_solld  , sabg1      , &
           forc_frl    , dz_soisno  , z_soisno    , zi_soisno  , &
           dz_lake     , lakedepth  , csol1       , porsl      , &
           dkdry       , dksatu     , &
           ! "inout" arguments		           
		   ! -------------------
           t_grnd1      , scv         , snowdp1      , t_soisno1, &
           wliq_soisno  , wice_soisno , imelt, t_lake1         , &       
           lake_icefrac , &
           ! "out" arguments
           ! -------------------
           taux1        , tauy1       , fsena                  , &
           fevpa        , lfevpa      , fseng        , fevpg   , &
           qseva        , qsubl       , qsdew        , qfros   , &
           olrg         , fgrnd       , tref         , qref    , &
           trad         , emis        , z0ma         , zol     , &
           rib          , ustar       , qstar        , tstar   , &
           fm           , fh          , fq           , sm      , &
!------------added by Anning Huang 2015.4.15
	       ram, errsoi1)
 
	    call snowwater_lake ( &
             ! "in" arguments
             ! ---------------------------
             maxsnl      , nl_soil     , nl_lake   , deltim    , &
             ssi         , wimp        , porsl     , pg_rain   , &
             pg_snow     , dz_lake     , imelt(:0) , fiold(:0) , &
             qseva       , qsubl       , qsdew     , qfros     , &
             ! "inout" arguments
             ! ---------------------------
             z_soisno    , dz_soisno  , zi_soisno, t_soisno1    , &
             wice_soisno , wliq_soisno, t_lake1  , lake_icefrac, &
             fseng       , fgrnd      , snl      , scv ,&
             snowdp1     , sm)
			 	
!
!-----------------------PFT levels--------------------------------------------
!
! Surface fluxes of momentum, sensible and latent heat using ground temperatures from previous time step     out
! 
eflx_sh_grnd(p)   = fseng                              ! sensible heat flux from ground (W/m**2) [+ to atm]	   1
eflx_sh_tot(p)    = eflx_sh_grnd(p)                    ! total sensible heat flux (W/m**2) [+ to atm]	  	   2

qflx_evap_soi(p)  = fevpg  	                           ! soil evaporation (mm H2O/s) (+ = to atm)	           3                	                                   	
qflx_evap_tot(p)  = qflx_evap_soi(p)                   ! qflx_evap_soi + qflx_evap_veg + qflx_tran_veg  	   4

eflx_lh_grnd(p)   = lfevpa                             ! ground evaporation heat flux (W/m**2) [+ to atm]	   5
eflx_lh_tot(p)    = eflx_lh_grnd(p)                    ! total latent heat flux (W/m8*2)  [+ to atm]	  	   6

eflx_lwrad_out(p) = olrg                               ! emitted infrared (longwave) radiation (W/m**2)		   7
eflx_lwrad_net(p) = eflx_lwrad_out(p) - forc_lwrad(g)  ! net infrared (longwave) rad (W/m**2) [+ = to atm]	   8

eflx_soil_grnd(p) = fgrnd                              ! soil heat flux (W/m**2) [+ = into soil]   			   9
qflx_prec_grnd(p) = forc_rain(g) + forc_snow(g)	       ! water onto ground including canopy runoff [kg/(m2 s)] 10
 
t_veg(p)          = forc_t(g)                          ! vegetation temperature (Kelvin)					   11
t_ref2m(p)        = tref                               ! 2m height surface air temperature (Kelvin)			   12
q_ref2m(p)        = qref                               ! 2m height surface specific humidity (kg/kg)		   13

taux(p)           = taux1                              ! wind (shear) stress: e-w (kg/m/s**2)				   14
tauy(p)           = tauy1                              ! wind (shear) stress: n-s (kg/m/s**2) 				   15
 ram1(p)          = ram                               ! aerodynamical resistance (s/m)						   16

!print*,fp, fseng,fevpg,lfevpa,olrg,fgrnd,tref,qref,taux1,tauy1,ram

!
!----------------------Column Levels---------------------------------------   
!     
      t_lake(c,:) = t_lake1(:)                         ! lake layer temperature (K)          				   17
		errsoi(c) = errsoi1                            ! soil/lake energy conservation error (W/m**2) 		   18
		t_grnd(c) = t_grnd1		                       ! ground temperature (Kelvin)  			  	   		   10
 	    snowdp(c) = snowdp1                            ! snow depth (m)	 	    							   20
      
	      qmelt(c) = sm                                ! snow melt [mm/s] 	 		 	   					   21
		 h2osno(c) = scv                               ! snow mass (kg/m2--mm)								   22
	    snowage(c) = sag  		                       ! non dimensional snow age [-] 						   23
    lake_icef(c,:) = lake_icefrac(:)                   ! mass fraction of lake layer that is frozen			   24

!if(fp.eq.1000) print*,fp, t_lake1

! Set zero to the empty node
      if (snl > maxsnl) then
         t_soisno1  (maxsnl+1:snl) = 0. 
         wliq_soisno(maxsnl+1:snl) = 0.		    
         wice_soisno(maxsnl+1:snl) = 0.		 
         z_soisno   (maxsnl+1:snl) = 0.
         dz_soisno  (maxsnl+1:snl) = 0.
      endif

                     t_soisno(c,:) = t_soisno1(:)           ! soil/snow temperature [K]   					 25
                   h2osoi_liq(c,:) = wliq_soisno(:)         ! soil/snow liquid water (kg/m2)				 26
                   h2osoi_ice(c,:) = wice_soisno(:)         ! soil/snow liquid water (kg/m2)				 27
				   				    
				       zsoisno(c,:) = z_soisno(:)           ! soil/snow node depth [m]						 28
                      dzsoisno(c,:) = dz_soisno(:)          ! soil/snow layer thickness (m)					 29

! Set zero to the empty node end                 
! We assume the land water bodies have zero extra liquid water capacity 
! (i.e.,constant capacity), all excess liquid water are put into the runoff,
! this unreasonable assumption should be updated in the future version
!      a = (sum(wliq_soisno(snl+1:))-w_old)/deltim
!      aa = qseva-(qsubl-qsdew)
!      rsur = max(0., pg_rain - aa - a)
!      rnof = rsur

enddo

!stop   

 end subroutine DaiLake	   

!-----------------------------------------------------------------------
  subroutine newsnow_lake ( &
            ! "in" arguments
            ! ---------------
            maxsnl    , nl_lake   , deltim      , dz_lake ,&
            pg_rain   , pg_snow   , t_precip    , bifall  ,&

            ! "inout" arguments
            ! ------------------
            t_lake    , zi_soisno , z_soisno ,&
            dz_soisno , t_soisno  , wliq_soisno , wice_soisno ,&
            fiold     , snl       , sag         , scv         ,&
            snowdp    , lake_icefrac ) 

!=======================================================================
! Add new snow nodes. 
! Created by Yongjiu Dai, December, 2012
!                            April, 2014
! Revised by Nan Wei,          May, 2014
!=======================================================================
!
! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl, use PhysicalConstants, only : tfrz, denh2o, cpliq, cpice, hfus
  use clm_varcon, only : tfrz, denh2o, cpliq, cpice, hfus
  implicit none
! ------------------------ Dummy Argument ------------------------------
  integer, INTENT(in) :: maxsnl    ! maximum number of snow layers
  integer, INTENT(in) :: nl_lake   ! number of soil layers
  real(r8), INTENT(in) :: deltim   ! seconds in a time step [second]
  real(r8), INTENT(in) :: pg_rain  ! liquid water onto ground [kg/(m2 s)]
  real(r8), INTENT(in) :: pg_snow  ! ice onto ground [kg/(m2 s)]
  real(r8), INTENT(in) :: t_precip ! snowfall/rainfall temperature [kelvin]
  real(r8), INTENT(in) :: bifall   ! bulk density of newly fallen dry snow [kg/m3]
  real(r8), INTENT(in) :: dz_lake(1:nl_lake)      ! lake layer thickness (m)

  real(r8), INTENT(inout) ::   zi_soisno(maxsnl:0)   ! interface level below a "z" level (m)
  real(r8), INTENT(inout) ::    z_soisno(maxsnl+1:0) ! snow layer depth (m)
  real(r8), INTENT(inout) ::   dz_soisno(maxsnl+1:0) ! snow layer thickness (m)
  real(r8), INTENT(inout) ::    t_soisno(maxsnl+1:0) ! snow layer temperature [K]
  real(r8), INTENT(inout) :: wliq_soisno(maxsnl+1:0) ! snow layer liquid water (kg/m2)
  real(r8), INTENT(inout) :: wice_soisno(maxsnl+1:0) ! snow layer ice lens (kg/m2)
  real(r8), INTENT(inout) ::       fiold(maxsnl+1:0) ! fraction of ice relative to the total water
   integer, INTENT(inout) :: snl    ! number of snow layers
  real(r8), INTENT(inout) :: sag    ! non dimensional snow age [-]
  real(r8), INTENT(inout) :: scv    ! snow mass (kg/m2)
  real(r8), INTENT(inout) :: snowdp ! snow depth (m)
  real(r8), INTENT(inout) :: lake_icefrac(1:nl_lake) ! mass fraction of lake layer that is frozen
  real(r8), INTENT(inout) :: t_lake(1:nl_lake)       ! lake layer temperature (m)

! ----------------------- Local  Variables -----------------------------

  integer lb
  integer newnode    ! signification when new snow node is set, (1=yes, 0=non)
  real(r8) dz_snowf  ! layer thickness rate change due to precipitation [m/s]
  real(r8) a, b, c, d, e, f, g, h
  real(r8) wice_lake(1:nl_lake), wliq_lake(1:nl_lake), tw

      newnode = 0
      dz_snowf = pg_snow/bifall    
	              
      snowdp = snowdp + dz_snowf*deltim         
      scv = scv + pg_snow*deltim      ! snow water equivalent (mm)


      zi_soisno(0) = 0.

      IF (snl==0 .and. snowdp < 0.01) then       ! no snow layer, energy exchange between prec and lake surface

          a = cpliq*pg_rain*deltim*(t_precip-tfrz)                          !cool down rainfall to tfrz 
          b = pg_rain*deltim*hfus                                           !all rainfall frozen
          c = cpice*denh2o*dz_lake(1)*lake_icefrac(1)*(tfrz-t_lake(1))      !warm up lake surface ice to tfrz
          d = denh2o*dz_lake(1)*lake_icefrac(1)*hfus                        !all lake surface ice melt
          e = cpice*pg_snow*deltim*(tfrz-t_precip)                          !warm up snowfall to tfrz
          f = pg_snow*deltim*hfus                                           !all snowfall melt
          g = cpliq*denh2o*dz_lake(1)*(1-lake_icefrac(1))*(t_lake(1)-tfrz)  !cool down lake surface water to tfrz
          h = denh2o*dz_lake(1)*(1-lake_icefrac(1))*hfus                    !all lake surface water frozen
          sag = 0.0

          if (lake_icefrac(1) >= 0.999) then
              ! all rainfall frozen, release heat to warm up frozen lake surface 
              if (a+b<=c) then 
                  tw=min(tfrz,t_precip)
                  t_lake(1)=(a+b+cpice*(pg_rain+pg_snow)*deltim*tw+cpice*denh2o*dz_lake(1)*t_lake(1))/&
                            (cpice*denh2o*dz_lake(1)+cpice*(pg_rain+pg_snow)*deltim)
                  scv = scv+pg_rain*deltim
                  snowdp = snowdp + pg_rain*deltim/bifall
              ! prec tem at tfrz, partial rainfall frozen ->release heat -> warm up lake surface to tfrz (no latent heat)
              else if (a<=c) then
                  t_lake(1)=tfrz
                  scv = scv + (c-a)/hfus
                  snowdp = snowdp + (c-a)/(hfus*bifall)
              ! lake surface tem at tfrz, partial lake surface melt -> absorb heat -> cool down rainfall to tfrz (no latent heat)
              else if (a<=c+d) then
                  t_lake(1)=tfrz
                  wice_lake(1) = denh2o*dz_lake(1) - (a-c)/hfus
                  wliq_lake(1) = (a-c)/hfus
                  lake_icefrac(1) = (wice_lake(1)+scv)/(wice_lake(1) + wliq_lake(1)+scv)
              ! all lake surface melt, absorb heat to cool down rainfall 
              else  !(a>c+d)
                  t_lake(1)=(cpliq*pg_rain*deltim*t_precip+cpliq*denh2o*dz_lake(1)*tfrz-c-d)/&
                            (cpliq*denh2o*dz_lake(1)+cpliq*pg_rain*deltim)
                  scv = 0.0
                  snowdp = 0.0
                  lake_icefrac(1) = 0.0
              end if

              if (snowdp>=0.01) then  !frozen rain may make new snow layer
                  snl = -1
                  newnode = 1
                  dz_soisno(0)  = snowdp             ! meter
                  z_soisno (0)  = -0.5*dz_soisno(0)
                  zi_soisno(-1) = -dz_soisno(0)
                  sag = 0.                           ! snow age

                  t_soisno (0) = t_lake(1)           ! K
                  wice_soisno(0) = scv               ! kg/m2
                  wliq_soisno(0) = 0.                ! kg/m2
                  fiold(0) = 1.

              end if

          else if (lake_icefrac(1) >= 0.001) then
              if (pg_rain > 0.0 .and. pg_snow > 0.0) then
                  t_lake(1)=tfrz
                  lake_icefrac(1) = (denh2o*dz_lake(1)*lake_icefrac(1)+scv)/(denh2o*dz_lake(1)+scv)
              else if (pg_rain > 0.0) then
                  if (a>=d) then
                      t_lake(1)=(cpliq*pg_rain*deltim*t_precip+cpliq*denh2o*dz_lake(1)*tfrz-d)/&
                                (cpliq*denh2o*dz_lake(1)+cpliq*pg_rain*deltim)
                      scv = 0.0
                      snowdp = 0.0
                      lake_icefrac(1) = 0.0
                  else
                      t_lake(1)=tfrz
                      wice_lake(1) = denh2o*dz_lake(1)*lake_icefrac(1) - a/hfus
                      wliq_lake(1) = denh2o*dz_lake(1)*(1-lake_icefrac(1)) + a/hfus
                      lake_icefrac(1) = (wice_lake(1)+scv)/(wice_lake(1) + wliq_lake(1)+scv)
                  end if
              else if (pg_snow > 0.0) then
                  if (e>=h) then
                      t_lake(1)=(h+cpice*denh2o*dz_lake(1)*tfrz+cpice*pg_snow*deltim*t_precip)/&
                                (cpice*pg_snow*deltim+cpice*denh2o*dz_lake(1))
                      lake_icefrac(1) = 1.0
                  else
                      t_lake(1)=tfrz
                      wice_lake(1) = denh2o*dz_lake(1)*lake_icefrac(1) + e/hfus
                      wliq_lake(1) = denh2o*dz_lake(1)*(1-lake_icefrac(1)) - e/hfus
                      lake_icefrac(1) = (wice_lake(1)+scv)/(wice_lake(1) + wliq_lake(1)+scv)
                  end if
              end if

          else
              ! all snowfall melt, absorb heat to cool down lake surface water
              if (e+f<=g) then
                  tw=max(tfrz,t_precip)
                  t_lake(1)=(cpliq*denh2o*dz_lake(1)*t_lake(1)+cpliq*(pg_rain+pg_snow)*deltim*tw-e-f)/&
                            (cpliq*(pg_rain+pg_snow)*deltim+cpliq*denh2o*dz_lake(1))
                  scv = 0.0
                  snowdp = 0.0
              ! prec tem at tfrz, partial snowfall melt ->absorb heat -> cool down lake surface to tfrz (no latent heat)
              else if (e<=g) then
                  t_lake(1) = tfrz
                  scv = scv - (g-e)/hfus
                  snowdp = snowdp - (g-e)/(hfus*bifall)
              ! lake surface tem at tfrz, partial lake surface frozen -> release heat -> warm up snowfall to tfrz (no latent heat)
              else if (e<=g+h) then
                  t_lake(1) = tfrz
                  wice_lake(1) = (e-g)/hfus
                  wliq_lake(1) = denh2o*dz_lake(1) - (e-g)/hfus
                  lake_icefrac(1) = (wice_lake(1)+scv)/(wice_lake(1) + wliq_lake(1)+scv)
              ! all lake surface frozen, release heat to warm up snowfall
              else       !(e>g+h)
                  t_lake(1) = (g+h+cpice*denh2o*dz_lake(1)*tfrz+cpice*pg_snow*deltim*t_precip)/&
                              (cpice*pg_snow*deltim+cpice*denh2o*dz_lake(1))
                  lake_icefrac(1) = 1.0
              end if 
          end if

      ELSE IF (snl==0 .and. snowdp >= 0.01) then

          ! only ice part of snowfall is added here, the liquid part will be added later
          snl = -1
          newnode = 1
          dz_soisno(0)  = snowdp             ! meter
          z_soisno (0)  = -0.5*dz_soisno(0)
          zi_soisno(-1) = -dz_soisno(0)
          sag = 0.                           ! snow age

          t_soisno (0) = min(tfrz, t_precip) ! K
          wice_soisno(0) = scv               ! kg/m2
          wliq_soisno(0) = 0.                ! kg/m2
          fiold(0) = 1.

      ELSE                                   ! ( snl<0 .and. newnode ==0 )

          lb = snl + 1
          t_soisno(lb) = ( (wice_soisno(lb)*cpice+wliq_soisno(lb)*cpliq)*t_soisno(lb) &
                       +   (pg_rain*cpliq + pg_snow*cpice)*deltim*t_precip ) &
                       / ( wice_soisno(lb)*cpice + wliq_soisno(lb)*cpliq &
                       +   pg_rain*deltim*cpliq + pg_snow*deltim*cpice )

          t_soisno(lb) = min(tfrz, t_soisno(lb))
          wice_soisno(lb) = wice_soisno(lb)+deltim*pg_snow

          dz_soisno(lb) = dz_soisno(lb)+dz_snowf*deltim         
          z_soisno(lb) = zi_soisno(lb) - 0.5*dz_soisno(lb)
          zi_soisno(lb-1) = zi_soisno(lb) - dz_soisno(lb)

      END IF

  end subroutine newsnow_lake



  subroutine laketem (&
           ! "in" arguments
           ! -------------------
           itypwat      , maxsnl      , nl_soil      , nl_lake   ,&
           dlat         , deltim      , forc_hgt_u   , forc_hgt_t,&
           forc_hgt_q   , forc_us     , forc_vs      , forc_t    ,&
           forc_q       , forc_rhoair , forc_psrf    , forc_sols ,&
           forc_soll    , forc_solsd  , forc_solld   , sabg      ,&
           forc_frl     , dz_soisno   , z_soisno     , zi_soisno ,&
           dz_lake      , lakedepth   , csol         , porsl     ,&
           dkdry        , dksatu      , &

           ! "inout" arguments
           ! -------------------
           t_grnd       , scv         , snowdp       , t_soisno  ,&
           wliq_soisno  , wice_soisno , imelt_soisno , t_lake    ,&
           lake_icefrac , &

           ! "out" arguments
           ! -------------------
           taux         , tauy        , fsena                    ,&
           fevpa        , lfevpa      , fseng        , fevpg     ,&
           qseva        , qsubl       , qsdew        , qfros     ,&
           olrg         , fgrnd       , tref         , qref      ,&
           trad         , emis        , z0ma         , zol       ,&
           rib          , ustar       , qstar        , tstar     ,&
           fm           , fh          , fq           , sm        ,&
!------------added by Anning Huang 2015.4.15
		   ram, errsoi)	     

! ------------------------ code history ---------------------------
! purpose: lake temperature and snow on frozen lake
! initial  Yongjiu Dai, 2000
!          Zack Subin, 2009
!          Yongjiu Dai, /12/2012/, /04/2014/
!          Nan Wei, /05/2014/
!
! ------------------------ notes ----------------------------------
! Lakes have variable depth, possible snow layers above, freezing & thawing of lake water,
! and soil layers with active temperature and gas diffusion below.
!
! Calculates temperatures in the 25-30 layer column of (possible) snow,
! lake water, soil, and bedrock beneath lake.
! Snow and soil temperatures are determined as in SoilTemperature, except
! for appropriate boundary conditions at the top of the snow (the flux is fixed
! to be the ground heat flux), the bottom of the snow (adjacent to top lake layer), 
! and the top of the soil (adjacent to the bottom lake layer). 
! Also, the soil is kept fully saturated.
! The whole column is solved simultaneously as one tridiagonal matrix.
!
! calculate lake temperatures from one-dimensional thermal
! stratification model based on eddy diffusion concepts to 
! represent vertical mixing of heat
!
! d ts    d            d ts     1 ds
! ---- = -- [(km + ke) ----] + -- --
!  dt    dz             dz     cw dz   
! where: ts = temperature (kelvin)
!         t = time (s)
!         z = depth (m)
!        km = molecular diffusion coefficient (m**2/s)
!        ke = eddy diffusion coefficient (m**2/s)
!        cw = heat capacity (j/m**3/kelvin)
!         s = heat source term (w/m**2)
!
! use crank-nicholson method to set up tridiagonal system of equations to
! solve for ts at time n+1, where the temperature equation for layer i is
! r_i = a_i [ts_i-1] n+1 + b_i [ts_i] n+1 + c_i [ts_i+1] n+1
! the solution conserves energy as
! cw*([ts(  1)] n+1 - [ts(  1)] n)*dz(  1)/dt + ... +
! cw*([ts(nl_lake)] n+1 - [ts(nl_lake)] n)*dz(nl_lake)/dt = fin
! where 
! [ts] n   = old temperature (kelvin)
! [ts] n+1 = new temperature (kelvin)
! fin      = heat flux into lake (w/m**2)
!          = beta*sabg+forc_frl-olrg-fsena-lfevpa-hm + phi(1) + ... + phi(nl_lake) 
! -----------------------------------------------------------------
! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : tfrz,hvap,hfus,hsub,tkwat,tkice,tkair,stefnc,&
!shixl                               vkc,grav,cpliq,cpice,cpair,denh2o,denice,rgas
  use clm_varcon, only : tfrz,hvap,hfus,hsub,tkwat,tkice,tkair,sb,&
                                vkc,grav,cpliq,cpice,cpair,denh2o,denice,rair,cnfac
! use FRICTION_VELOCITY1
! use SOIL_thermal_parameters

  use QSatMod, only : QSat		 ! added by Anning Huang 2016.11.17


  IMPLICIT NONE
! ------------------------ input/output variables -----------------
  integer, INTENT(in) :: itypwat  ! land water type (4=deep lake, 5=shallow lake)
  integer, INTENT(in) :: maxsnl   ! maximum number of snow layers
  integer, INTENT(in) :: nl_soil  ! number of soil layers
  integer, INTENT(in) :: nl_lake  ! number of lake layers

  real(r8), INTENT(in) :: dlat    ! latitude (radians)
  real(r8), INTENT(in) :: deltim  ! seconds in a time step (s)
  real(r8), INTENT(in) :: forc_hgt_u ! observational height of wind [m]
  real(r8), INTENT(in) :: forc_hgt_t ! observational height of temperature [m]
  real(r8), INTENT(in) :: forc_hgt_q ! observational height of humidity [m]
  real(r8), INTENT(in) :: forc_us ! wind component in eastward direction [m/s]
  real(r8), INTENT(in) :: forc_vs ! wind component in northward direction [m/s]
  real(r8), INTENT(in) :: forc_t  ! temperature at agcm reference height [kelvin]
  real(r8), INTENT(in) :: forc_q  ! specific humidity at agcm reference height [kg/kg]
  real(r8), INTENT(in) :: forc_rhoair ! density air [kg/m3]
  real(r8), INTENT(in) :: forc_psrf   ! atmosphere pressure at the surface [pa]
  real(r8), INTENT(in) :: forc_sols   ! atm vis direct beam solar rad onto srf [W/m2]
  real(r8), INTENT(in) :: forc_soll   ! atm nir direct beam solar rad onto srf [W/m2]
  real(r8), INTENT(in) :: forc_solsd  ! atm vis diffuse solar rad onto srf [W/m2]
  real(r8), INTENT(in) :: forc_solld  ! atm nir diffuse solar rad onto srf [W/m2]
  real(r8), INTENT(in) :: forc_frl    ! atmospheric infrared (longwave) radiation [W/m2] upward positive
  real(r8), INTENT(in) :: sabg        ! solar radiation absorbed by ground [W/m2] downward positive

  real(r8), INTENT(in) :: dz_soisno(maxsnl+1:nl_soil) ! soil/snow layer thickness (m)
  real(r8), INTENT(in) :: z_soisno(maxsnl+1:nl_soil)  ! soil/snow node depth [m]
  real(r8), INTENT(in) :: zi_soisno(maxsnl:nl_soil)   ! soil/snow depth of layer interface [m] 

  real(r8), INTENT(in) :: dz_lake(nl_lake)  ! lake layer thickness (m)
  real(r8), INTENT(in) :: lakedepth         ! column lake depth (m)

  real(r8), INTENT(in) :: csol  (1:nl_soil) ! heat capacity of soil soilds [J/(m3 K)]
  real(r8), INTENT(in) :: porsl (1:nl_soil) ! soil porosity
  real(r8), INTENT(in) :: dkdry (1:nl_soil) ! thermal conductivity for dry soil [W/m-K]
  real(r8), INTENT(in) :: dksatu(1:nl_soil) ! Thermal conductivity of saturated soil [W/m-K]

  real(r8), INTENT(inout) :: t_grnd  ! surface temperature (kelvin)
  real(r8), INTENT(inout) :: scv     ! snow water equivalent [mm]
  real(r8), INTENT(inout) :: snowdp  ! snow depth [mm]

  real(r8), INTENT(inout) :: t_soisno    (maxsnl+1:nl_soil) ! soil/snow temperature [K]
  real(r8), INTENT(inout) :: wliq_soisno (maxsnl+1:nl_soil) ! soil/snow liquid water (kg/m2)
  real(r8), INTENT(inout) :: wice_soisno (maxsnl+1:nl_soil) ! soil/snow ice lens (kg/m2)
  integer,  INTENT(inout) :: imelt_soisno(maxsnl+1:nl_soil) ! soil/snow flag for melting (=1), freezing (=2), Not=0 (new)

  real(r8), INTENT(inout) :: t_lake(nl_lake)       ! lake temperature (kelvin)
  real(r8), INTENT(inout) :: lake_icefrac(nl_lake) ! lake mass fraction of lake layer that is frozen

  real(r8), INTENT(out) :: taux   ! wind stress: E-W [kg/m/s**2]
  real(r8), INTENT(out) :: tauy   ! wind stress: N-S [kg/m/s**2]
  real(r8), INTENT(out) :: fsena  ! sensible heat from canopy height to atmosphere [W/m2]
  real(r8), INTENT(out) :: fevpa  ! evapotranspiration from canopy height to atmosphere [mm/s]
  real(r8), INTENT(out) :: lfevpa ! latent heat flux from canopy height to atmosphere [W/m2]

  real(r8), INTENT(out) :: fseng  ! sensible heat flux from ground [W/m2]
  real(r8), INTENT(out) :: fevpg  ! evaporation heat flux from ground [mm/s]

  real(r8), INTENT(out) :: qseva  ! ground surface evaporation rate (mm h2o/s)
  real(r8), INTENT(out) :: qsubl  ! sublimation rate from snow pack (mm H2O /s) [+]
  real(r8), INTENT(out) :: qsdew  ! surface dew added to snow pack (mm H2O /s) [+]
  real(r8), INTENT(out) :: qfros  ! ground surface frosting formation (mm H2O /s) [+]

  real(r8), INTENT(out) :: olrg   ! outgoing long-wave radiation from ground+canopy
  real(r8), INTENT(out) :: fgrnd  ! ground heat flux [W/m2]

  real(r8), INTENT(out) :: tref   ! 2 m height air temperature [kelvin]
  real(r8), INTENT(out) :: qref   ! 2 m height air specific humidity
  real(r8), INTENT(out) :: trad   ! radiative temperature [K]

  real(r8), INTENT(out) :: emis   ! averaged bulk surface emissivity
  real(r8), INTENT(out) :: z0ma   ! effective roughness [m]
  real(r8), INTENT(out) :: zol    ! dimensionless height (z/L) used in Monin-Obukhov theory
  real(r8), INTENT(out) :: rib    ! bulk Richardson number in surface layer
  real(r8), INTENT(out) :: ustar  ! u* in similarity theory [m/s]
  real(r8), INTENT(out) :: qstar  ! q* in similarity theory [kg/kg]
  real(r8), INTENT(out) :: tstar  ! t* in similarity theory [K]
  real(r8), INTENT(out) :: fm     ! integral of profile function for momentum
  real(r8), INTENT(out) :: fh     ! integral of profile function for heat
  real(r8), INTENT(out) :: fq     ! integral of profile function for moisture
  real(r8), INTENT(out) :: sm     ! rate of snowmelt [mm/s, kg/(m2 s)]

!added by Anning Huang 2015.4.15
  real(r8), INTENT(out):: ram     ! aerodynamical resistance [s/m]
  real(r8), INTENT(out):: errsoi  ! soil/lake energy conservation error (W/m^2)

! ---------------- local variables in surface temp and fluxes calculation -----------------
  integer idlak     ! index of lake, 1 = deep lake, 2 = shallow lake
  real(r8) z_lake (nl_lake)  ! lake node depth (middle point of layer) (m)

  real(r8) ax       ! used in iteration loop for calculating t_grnd (numerator of NR solution)
  real(r8) bx       ! used in iteration loop for calculating t_grnd (denomin. of NR solution)
  real(r8) beta1    ! coefficient of conective velocity [-]
  real(r8) degdT    ! d(eg)/dT
  real(r8) displax  ! zero- displacement height [m]
  real(r8) dqh      ! diff of humidity between ref. height and surface
  real(r8) dth      ! diff of virtual temp. between ref. height and surface
  real(r8) dthv     ! diff of vir. poten. temp. between ref. height and surface
  real(r8) dzsur    ! 1/2 the top layer thickness (m)
  real(r8) tsur     ! top layer temperature
  real(r8) rhosnow  ! partitial density of water (ice + liquid)
  real(r8) eg       ! water vapor pressure at temperature T [pa]
  real(r8) emg      ! ground emissivity (0.97 for snow,
! real(r8) errore   ! lake temperature energy conservation error (w/m**2)
  real(r8) hm       ! energy residual [W/m2]
  real(r8) htvp     ! latent heat of vapor of water (or sublimation) [j/kg]
  real(r8) obu      ! monin-obukhov length (m)
  real(r8) obuold   ! monin-obukhov length of previous iteration
  real(r8) qsatg    ! saturated humidity [kg/kg]
  real(r8) qsatgdT  ! d(qsatg)/dT

!  real(r8) ram      ! aerodynamical resistance [s/m]

  real(r8) rah      ! thermal resistance [s/m]
  real(r8) raw      ! moisture resistance [s/m]
  real(r8) stftg3   ! emg*sb*t_grnd*t_grnd*t_grnd
  real(r8) fh2m     ! relation for temperature at 2m
  real(r8) fq2m     ! relation for specific humidity at 2m
  real(r8) fm10m    ! integral of profile function for momentum at 10m
  real(r8) t_grnd_bef0   ! initial ground temperature
  real(r8) t_grnd_bef    ! initial ground temperature
  real(r8) thm      ! intermediate variable (forc_t+0.0098*forc_hgt_t)
  real(r8) th       ! potential temperature (kelvin)
  real(r8) thv      ! virtual potential temperature (kelvin)
  real(r8) thvstar  ! virtual potential temperature scaling parameter
  real(r8) tksur    ! thermal conductivity of snow/soil (w/m/kelvin)
  real(r8) um       ! wind speed including the stablity effect [m/s]
  real(r8) ur       ! wind speed at reference height [m/s]
  real(r8) visa     ! kinematic viscosity of dry air [m2/s]
  real(r8) wc       ! convective velocity [m/s]
  real(r8) wc2      ! wc*wc
  real(r8) zeta     ! dimensionless height used in Monin-Obukhov theory
  real(r8) zii      ! convective boundary height [m]
  real(r8) zldis    ! reference height "minus" zero displacement heght [m]
  real(r8) z0mg     ! roughness length over ground, momentum [m]
  real(r8) z0hg     ! roughness length over ground, sensible heat [m]
  real(r8) z0qg     ! roughness length over ground, latent heat [m]

  real(r8) wliq_lake(nl_lake) ! lake liquid water (kg/m2)
  real(r8) wice_lake(nl_lake) ! lake ice lens (kg/m2)

! ---------------- local variables in lake/snow/soil temperature calculation --------------
  real(r8), parameter :: cur0 = 0.01        ! min. Charnock parameter
  real(r8), parameter :: curm = 0.1         ! maximum Charnock parameter
  real(r8), parameter :: fcrit = 22.        ! critical dimensionless fetch for Charnock parameter (Vickers & Mahrt 1997)
                                            ! but converted to use u instead of u* (Subin et al. 2011)
  real(r8), parameter :: depthcrit = 25.    ! (m) Depth beneath which to enhance mixing
  real(r8), parameter :: fangmult = 5.      ! Multiplier for unfrozen diffusivity
  real(r8), parameter :: minmultdepth = 20. ! (m) Minimum depth for imposing fangmult
! real(r8), parameter :: cnfac  = 0.5       ! Crank Nicholson factor between 0 and 1

  !--------------------
  real(r8) fetch     ! lake fetch (m)
  real(r8) cur       ! Charnock parameter (-)
  real(r8) betavis   ! 
  real(r8) betaprime ! Effective beta
  real(r8) tdmax     ! temperature of maximum water density
  real(r8) cfus      ! effective heat of fusion per unit volume 
  real(r8) tkice_eff ! effective conductivity since layer depth is constant
  real(r8) cice_eff  ! effective heat capacity of ice (using density of
                     ! water because layer depth is not adjusted when freezing
  real(r8) cwat      ! specific heat capacity of water (j/m**3/kelvin)

  !--------------------
  real(r8) rhow(nl_lake) ! density of water (kg/m**3)
  real(r8) fin           ! heat flux into lake - flux out of lake (w/m**2)
  real(r8) phi(nl_lake)  ! solar radiation absorbed by layer (w/m**2)
  real(r8) phi_soil      ! solar radiation into top soil layer (W/m^2)
  real(r8) phidum        ! temporary value of phi

  integer  imelt_lake(1:nl_lake)       ! lake flag for melting or freezing snow and soil layer [-]
  real(r8) cv_lake(1:nl_lake)          ! heat capacity [J/(m2 K)]
  real(r8) tk_lake(1:nl_lake)          ! thermal conductivity at layer node [W/(m K)]
  real(r8) cv_soisno(maxsnl+1:nl_soil) ! heat capacity of soil/snow [J/(m2 K)] 
  real(r8) tk_soisno(maxsnl+1:nl_soil) ! thermal conductivity of soil/snow [W/(m K)] (at interface below, except for j=0) 
  real(r8) tktopsoil                   ! thermal conductivity of the top soil layer [W/(m K)]

  real(r8) t_soisno_bef(maxsnl+1:nl_soil) ! beginning soil/snow temp for E cons. check [K]
  real(r8) t_lake_bef(1:nl_lake)          ! beginning lake temp for energy conservation check [K]

  real(r8) cvx    (maxsnl+1:nl_lake+nl_soil) ! heat capacity for whole column [J/(m2 K)]
  real(r8) tkix   (maxsnl+1:nl_lake+nl_soil) ! thermal conductivity at layer interfaces for whole column [W/(m K)]
  real(r8) phix   (maxsnl+1:nl_lake+nl_soil) ! solar source term for whole column [W/m**2]
  real(r8) zx     (maxsnl+1:nl_lake+nl_soil) ! interface depth (+ below surface) for whole column [m]
  real(r8) tx     (maxsnl:nl_lake+nl_soil) ! temperature of whole column [K]
  real(r8) tx_bef (maxsnl:nl_lake+nl_soil) ! beginning lake/snow/soil temp for energy conservation check [K]
  real(r8) factx  (maxsnl+1:nl_lake+nl_soil) ! coefficient used in computing tridiagonal matrix
  real(r8) fnx    (maxsnl+1:nl_lake+nl_soil) ! heat diffusion through the layer interface below [W/m2]
  real(r8) a      (maxsnl:nl_lake+nl_soil) ! "a" vector for tridiagonal matrix
  real(r8) b      (maxsnl:nl_lake+nl_soil) ! "b" vector for tridiagonal matrix
  real(r8) c      (maxsnl:nl_lake+nl_soil) ! "c" vector for tridiagonal matrix
  real(r8) r      (maxsnl:nl_lake+nl_soil) ! "r" vector for tridiagonal solution
  real(r8) fn1    (maxsnl+1:nl_lake+nl_soil) ! heat diffusion through the layer interface below [W/m2]
  real(r8) brr    (maxsnl+1:nl_lake+nl_soil) ! 
  integer  imelt_x(maxsnl+1:nl_lake+nl_soil) ! flag for melting (=1), freezing (=2), Not=0 (new)

  real(r8) dzm       ! used in computing tridiagonal matrix [m]
  real(r8) dzp       ! used in computing tridiagonal matrix [m]
  real(r8) zin       ! depth at top of layer (m)
  real(r8) zout      ! depth at bottom of layer (m)
  real(r8) rsfin     ! relative flux of solar radiation into layer
  real(r8) rsfout    ! relative flux of solar radiation out of layer
  real(r8) eta       ! light extinction coefficient (/m): depends on lake type
  real(r8) za(2)     ! base of surface absorption layer (m): depends on lake type
  !--------------------

  real(r8) hs        ! net ground heat flux into the surface
  real(r8) dhsdT     ! temperature derivative of "hs"
  real(r8) heatavail ! available energy for melting or freezing (J/m^2)
  real(r8) heatrem   ! energy residual or loss after melting or freezing
  real(r8) melt      ! actual melting (+) or freezing (-) [kg/m2]
  real(r8) xmf       ! total per-column latent heat abs. from phase change  (J/m^2)
  !--------------------

  real(r8) ocvts     ! (cwat*(t_lake[n  ])*dz_lake
  real(r8) ncvts     ! (cwat*(t_lake[n+1])*dz_lake
  real(r8) esum1     ! temp for checking energy (J/m^2)
  real(r8) esum2     ! ""
  real(r8) zsum      ! temp for putting ice at the top during convection (m)

! real(r8) errsoi    ! soil/lake energy conservation error (W/m^2)

  real(r8) iceav     ! used in calc aver ice for convectively mixed layers
  real(r8) qav       ! used in calc aver heat content for conv. mixed layers
  real(r8) tav       ! used in aver temp for convectively mixed layers
  real(r8) tav_froz  ! used in aver temp for convectively mixed layers (C)
  real(r8) tav_unfr  ! "    
  real(r8) nav       ! used in aver temp for convectively mixed layers

  real(r8) fevpg_lim ! temporary evap_soi limited by top snow layer content [mm/s]
  real(r8) scv_temp  ! temporary h2osno [kg/m^2]
  real(r8) tmp       ! 
  real(r8) h_fin     ! 
  real(r8) h_finDT   ! 
  real(r8) del_T_grnd

  integer iter       ! iteration index
  integer convernum  ! number of time when del_T_grnd < 0.01
  integer nmozsgn    ! number of times moz changes sign

! assign iteration parameters
  integer, parameter :: itmax  = 40   ! maximum number of iteration
  integer, parameter :: itmin  = 6    ! minimum number of iteration
  real(r8),parameter :: delmax = 3.0  ! maximum change in lake temperature [K]
  real(r8),parameter :: dtmin  = 0.01 ! max limit for temperature convergence [K]
  real(r8),parameter :: dlemin = 0.1  ! max limit for energy flux convergence [w/m2]

  !--------------------

  integer nl_sls  ! abs(snl)+nl_lake+nl_soil
  integer snl     ! number of snow layers (minimum -5)
  integer lb      ! lower bound of arrays
  integer jprime  ! j - nl_lake

  integer i,j     ! do loop or array index

! ======================================================================
!*[1] constants and model parameters
! ======================================================================

! constants for lake temperature model
      za = (/0.6, 0.5/)    
      cwat = cpliq*denh2o     ! water heat capacity per unit volume
      cice_eff = cpice*denh2o ! use water density because layer depth is not adjusted for freezing
      cfus = hfus*denh2o      ! latent heat per unit volume
      tkice_eff = tkice * denice/denh2o ! effective conductivity since layer depth is constant
      emg = 0.96              ! surface emissivity
      emg = 0.97              ! surface emissivity of ice and water
!     emg= 0.96 for soil, 0.97 for glacier, 0.96 for wetland, and 0.97 for snow-covered surfaces

!      PRINT*, denice
!	  pause

! define snow layer on ice lake
      snl = 0
      do j=maxsnl+1,0
         if(wliq_soisno(j)+wice_soisno(j)>0.) snl=snl-1
      enddo
      lb = snl + 1

! latent heat 
      if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then ! for unfrozen lake
         htvp = hvap
      else
         htvp = hsub
      end if

! define levels
      z_lake(1) = dz_lake(1) / 2.
      do j = 2, nl_lake
         z_lake(j) = z_lake(j-1) + (dz_lake(j-1) + dz_lake(j))/2.
      end do

! Base on lake depth, assuming that small lakes are likely to be shallower
! Estimate crudely based on lake depth
      if (z_lake(nl_lake) < 4.) then
          idlak = 2
          fetch = 100. ! shallow lake
      else
          idlak = 1
          fetch = 25.*z_lake(nl_lake) ! deep lake
      end if

! ======================================================================
!*[2] pre-processing for the calcilation of the surface temperature and fluxes
! ======================================================================

      if (snl == 0) then
         ! calculate the nir fraction of absorbed solar.
         betaprime = (forc_soll+forc_solld)/max(1.e-5,forc_sols+forc_soll+forc_solsd+forc_solld)
         betavis = 0. ! The fraction of the visible (e.g. vis not nir from atm) sunlight
                      ! absorbed in ~1 m of water (the surface layer za_lake).
                      ! This is roughly the fraction over 700 nm but may depend on the details
                      ! of atmospheric radiative transfer.
                      ! As long as NIR = 700 nm and up, this can be zero.
         betaprime = betaprime + (1.0-betaprime)*betavis

      else 
         ! or frozen but no snow layers or 
         ! currently ignor the transmission of solar in snow and ice layers
         ! to be updated in the future version
         betaprime = 1
      end if

      if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then
         do j = 1, nl_lake
            ! extinction coefficient from surface data (1/m), if no eta from surface data,
            ! set eta, the extinction coefficient, according to L Hakanson, Aquatic Sciences, 1995
            ! (regression of secchi depth with lake depth for small glacial basin lakes), and the
            ! Poole & Atkins expression for extinction coeffient of 1.7 / secchi Depth (m).

            eta = 1.1925*max(lakedepth,1.)**(-0.424)

            zin  = z_lake(j) - 0.5*dz_lake(j)
            zout = z_lake(j) + 0.5*dz_lake(j)
            rsfin  = exp( -eta*max(  zin-za(idlak),0. ) )  ! the radiation within surface layer (z<za)
            rsfout = exp( -eta*max( zout-za(idlak),0. ) )  ! is considered fixed at (1-beta)*sabg
                                                           ! i.e, max(z-za, 0)
            ! Let rsfout for bottom layer go into soil.
            ! This looks like it should be robust even for pathological cases,
            ! like lakes thinner than za(idlak).

            phi(j) = (rsfin-rsfout) * sabg * (1.-betaprime)
            if (j == nl_lake) phi_soil = rsfout * sabg * (1.-betaprime)
         end do
      else  !if frozen or snow-covered layers
            ! This should be improved upon; Mironov 2002 suggests that SW can penetrate thin ice and may
            ! cause spring convection.
         phi(:) = 0.
         phi_soil = 0.
      end if
	  

!    call qsadv(t_grnd,forc_psrf,eg,degdT,qsatg,qsatgdT)
     call qsat(t_grnd,forc_psrf,eg,degdT,qsatg,qsatgdT)    ! by Anning Huang 2016.11.17


! potential temperatur at the reference height
    beta1 = 1.       !(in computing W_*)
      zii = 1000.    ! m  (pbl height)

      thm = forc_t + 0.0098*forc_hgt_t  ! intermediate variable equivalent to
                                        ! forc_t*(pgcm/forc_psrf)**(rgas/cpair)

      th = forc_t*(100000./forc_psrf)**(rair/cpair) ! potential T

      thv = th*(1.+0.608*forc_q)         ! virtual potential T
      ur = max(0.1,sqrt(forc_us*forc_us+forc_vs*forc_vs))   ! limit set to 0.1

! Initialization variables
      nmozsgn = 0
      obuold = 0.
      dth   = thm-t_grnd
      dqh   = forc_q-qsatg
      dthv  = dth*(1.+0.608*forc_q)+0.608*th*dqh
      zldis = forc_hgt_u-0.

! Roughness lengths, allow all roughness lengths to be prognostic
      ustar=0.06
      wc=0.5
      z0mg = 0.04

    ! Kinematic viscosity of dry air (m2/s)- Andreas (1989) CRREL Rep. 89-11
      visa=1.326e-5*(1.+6.542e-3*(forc_t-tfrz) &
           + 8.301e-6*(forc_t-tfrz)**2 - 4.84e-9*(forc_t-tfrz)**3)

      cur = cur0 + curm * exp( max( -(fetch*grav/ur/ur)**(1./3.)/fcrit, & ! Fetch-limited
                                    -(z_lake(nl_lake)*grav)**0.5/ur ) )   ! depth-limited

      if(dthv.ge.0.) then
         um=max(ur,0.1)
      else
         um=sqrt(ur*ur+wc*wc)
      endif

      do i=1,5
         z0mg=0.013*ustar*ustar/grav+0.11*visa/ustar  
         ustar=vkc*um/log(zldis/z0mg)  
      enddo

      call roughness_lake (snl,t_grnd,t_lake(1),lake_icefrac(1),forc_psrf,&
                           cur,ustar,z0mg,z0hg,z0qg)

      call moninobukini1(ur,th,thm,thv,dth,dqh,dthv,zldis,z0mg,um,obu)
! ----------------------------------------------------------------------
   
      if (snl == 0) then
         dzsur = dz_lake(1)/2.
      else
         dzsur = z_soisno(lb)-zi_soisno(lb-1)
      end if

		 
      t_grnd_bef0     = t_grnd
      t_soisno_bef(:) = t_soisno(:)       
      t_lake_bef(:)   = t_lake(:)

!------------------------------------------------------------
! Lake density
!------------------------------------------------------------
      do j = 1, nl_lake
         rhow(j) = (1.-lake_icefrac(j))*denh2o*(1.0-1.9549e-05*(abs(t_lake(j)-277.))**1.68) &
                     + lake_icefrac(j)*denice
         ! allow for ice fraction; assume constant ice density.
         ! this is not the correct average-weighting but that's OK because the density will only
         ! be used for convection for lakes with ice, and the ice fraction will dominate the
         ! density differences between layers.
         ! using this average will make sure that surface ice is treated properly during
         ! convective mixing.
      end do


!------------------------------------------------------------
! Diffusivity and implied thermal "conductivity" = diffusivity * cwat
!------------------------------------------------------------

      do j = 1, nl_lake
         cv_lake(j) = dz_lake(j) * (cwat*(1.-lake_icefrac(j)) + cice_eff*lake_icefrac(j))
      end do

      call hConductivity_lake(nl_lake,snl,t_grnd,&
                              z_lake,t_lake,lake_icefrac,rhow,&
                              dlat,ustar,z0mg,lakedepth,depthcrit,tk_lake)
!------------------------------------------------------------
! Heat source term
!------------------------------------------------------------

   !  if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then
   !     fin = betaprime*sabg + forc_frl - (olrg+fsena+lfevpa)
   !  else  !if frozen or snow-covered layers
   !     fin = sabg + forc_frl - (olrg+fsena+lfevpa)
   !  end if

!------------------------------------------------------------
! Set the thermal properties of the snow above frozen lake and underlying soil 
! and check initial energy content.
!------------------------------------------------------------

      lb = snl+1
      call hCapacity (itypwat,lb,nl_soil,csol,porsl,&
                      wice_soisno(lb:),wliq_soisno(lb:),scv,dz_soisno(lb:),cv_soisno(lb:))

    ! snow
      if(lb < 1)then
         do j = lb, 0
            rhosnow = (wice_soisno(j)+wliq_soisno(j))/dz_soisno(j)
            tk_soisno(j) = tkair + (7.75e-5*rhosnow + 1.105e-6*rhosnow*rhosnow)*(tkice-tkair)
         enddo
      endif
      
    ! soil
      call hConductivity (itypwat,1,nl_soil,&
                          dkdry,dksatu,porsl,&
                          dz_soisno(1:),z_soisno(1:),zi_soisno(0:),&
                          t_soisno(1:),wice_soisno(1:),wliq_soisno(1:),&
                          tk_soisno(1:))
! Sum cv_lake*t_lake for energy check
! Include latent heat term, and use tfrz as reference temperature
! to prevent abrupt change in heat content due to changing heat capacity with phase change.

      ! This will need to be over all soil / lake / snow layers. Lake is below.
      ocvts = 0.
      do j = 1, nl_lake
         ocvts = ocvts + cv_lake(j)*(t_lake(j)-tfrz) + cfus*dz_lake(j)*(1.-lake_icefrac(j))
      end do

      ! Now do for soil / snow layers
      do j = lb, nl_soil
         ocvts = ocvts + cv_soisno(j)*(t_soisno(j)-tfrz) + hfus*wliq_soisno(j)
         if (j == 1 .and. scv > 0. .and. j == lb) then
            ocvts = ocvts - scv*hfus
         end if
      end do

      ! Set up solar source terms (phix)
              phix(:) = 0.
      phix(1:nl_lake) = phi(1:nl_lake)         !lake layer
      phix(nl_lake+1) = phi_soil               !top soil layer

      ! Set up interface depths(zx), and temperatures (tx).
        tx(lb-1) = t_grnd
      do j = lb, nl_lake+nl_soil
         jprime = j - nl_lake
         if (j <= 0) then                      !snow layer
            zx(j) = z_soisno(j)
            tx(j) = t_soisno(j)
         else if (j <= nl_lake) then           !lake layer
            zx(j) = z_lake(j)
            tx(j) = t_lake(j)
         else                                  !soil layer
            zx(j) = z_lake(nl_lake) + dz_lake(nl_lake)/2. + z_soisno(jprime)
            tx(j) = t_soisno(jprime)
         end if
      end do

      tx_bef(:) = tx(:)

! Heat capacity and resistance of snow without snow layers (<1cm) is ignored during diffusion,
! but its capacity to absorb latent heat may be used during phase change.

      do j = lb, nl_lake+nl_soil
         jprime = j - nl_lake

         ! heat capacity [J/(m2 K)]
         if (j <= 0) then                      !snow layer
            cvx(j) = cv_soisno(j)
         else if (j <= nl_lake) then           !lake layer
            cvx(j) = cv_lake(j)
         else                                  !soil layer
            cvx(j) = cv_soisno(jprime)
         end if

! Determine interface thermal conductivities at layer interfaces [W/(m K)]

         if (j < 0) then                       !non-bottom snow layer
            tkix(j) = tk_soisno(j)
         else if (j == 0) then                 !bottom snow layer
            rhosnow = (wice_soisno(j)+wliq_soisno(j))/dz_soisno(j)
            tk_soisno(j) = tkair+(7.75e-5*rhosnow+1.105e-6*rhosnow*rhosnow)*(tkice-tkair)
            dzp = zx(j+1) - zx(j)
            tkix(j) = tk_lake(1)*tk_soisno(j)*dzp &
                    /(tk_soisno(j)*z_lake(1) + tk_lake(1)*(-zx(j)))
                               ! tk_soisno(0) is the conductivity at the middle of that layer
         else if (j < nl_lake) then            !non-bottom lake layer
            tkix(j) = (tk_lake(j)*tk_lake(j+1) * (dz_lake(j+1)+dz_lake(j))) &
                    / (tk_lake(j)*dz_lake(j+1) + tk_lake(j+1)*dz_lake(j))
         else if (j == nl_lake) then           !bottom lake layer
            tktopsoil = tk_soisno(jprime+1)    !tktopsoil is the conductivity at the middle of that layer
                                 ! here, assumed as same value as that at the bottom of 1st soil layer
            dzp = zx(j+1) - zx(j)
            tkix(j) = (tktopsoil*tk_lake(j)*dzp &
                    / (tktopsoil*dz_lake(j)/2. + tk_lake(j)*z_soisno(1)))
         else !soil layer
            tkix(j) = tk_soisno(jprime)
         end if

      end do

! Determine heat diffusion through the layer interface and factor used in computing
! tridiagonal matrix and set up vector r and vectors a, b, c that define tridiagonal
! matrix and solve system

      do j = lb, nl_lake+nl_soil
         factx(j) = deltim/cvx(j)
         if (j < nl_lake+nl_soil) then         !top or interior layer
            fnx(j) = tkix(j)*(tx(j+1)-tx(j))/(zx(j+1)-zx(j))
         else                                  !bottom soil layer
            fnx(j) = 0. !not used
         end if
      end do


! ======================================================================
!*[3] Begin stability iteration and temperature and fluxes calculation
! ======================================================================
      iter = 1
      del_T_grnd = 1.0    ! t_grnd diff
      convernum = 0       ! number of time when del_T_grnd <= 0.01
	   	
    ! =====================================
      ITERATION : DO WHILE (iter <= itmax)       
    ! =====================================

         t_grnd_bef = t_grnd

         if ((t_grnd_bef > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then
            tksur = tkwat        !only water molecular conductivity considered, not include eddy diffusivities conductivity as CLM, why?
            tsur = t_lake(1)
            htvp = hvap
         else if (snl == 0) then ! frozen but no snow layers
            tksur = tkice        ! This is an approximation because the whole layer may not be frozen, and it is not
                                 ! accounting for the physical (but not nominal) expansion of the frozen layer.
            tsur = t_lake(1)
            htvp = hsub
         else
          ! need to calculate thermal conductivity of the top snow layer
            rhosnow = (wice_soisno(lb)+wliq_soisno(lb))/dz_soisno(lb)
            tksur = tkair + (7.75e-5*rhosnow + 1.105e-6*rhosnow*rhosnow)*(tkice-tkair)
            tsur = tx(lb)
            htvp = hsub
         end if


! Evaluated stability-dependent variables using moz from prior iteration
         displax = 0.
		  		  	
         call moninobuk1(forc_hgt_u,forc_hgt_t,forc_hgt_q,displax,z0mg,z0hg,z0qg,obu,um,&
                        ustar,fh2m,fq2m,fm10m,fm,fh,fq)

! Get derivative of fluxes with repect to ground temperature
         ram    = 1./(ustar*ustar/um)
         rah    = 1./(vkc/fh*ustar)
         raw    = 1./(vkc/fq*ustar)
         stftg3 = emg*sb*t_grnd_bef*t_grnd_bef*t_grnd_bef

       !-----------------------------------------------------------------
       ! h_fin = betaprime*sabg + emg*forc_frl + 3.*stftg3*t_grnd_bef & !
       !     + forc_rhoair*cpair/rah*thm &                              !
       !     - htvp*forc_rhoair/raw*(qsatg-qsatgdT*t_grnd_bef - forc_q) !
       ! h_finDT = 4.*stftg3 + forc_rhoair*cpair/rah &                  !
       !     + htvp*forc_rhoair/raw*qsatgdT                             !
       ! del_T_grnd = t_grnd - t_grnd_bef                               !
       !----------------------------------------------------------------!

!------------------------------------------------------------
! Set up vector r and vectors a, b, c that define tridiagonal matrix
! snow and lake and soil layer temperature
!------------------------------------------------------------
      
         a(lb-1) = 0.
         b(lb-1) = 4.*stftg3 + forc_rhoair*cpair/rah + htvp*forc_rhoair/raw*qsatgdT + tksur/dzsur
         c(lb-1) = -tksur/dzsur

         r(lb-1) = betaprime*sabg + emg*forc_frl + 3.*stftg3*t_grnd_bef + forc_rhoair*cpair/rah*thm &
                   - htvp*forc_rhoair/raw*(qsatg-qsatgdT*t_grnd_bef - forc_q) 
 

         do j = lb, nl_lake+nl_soil
            if (j == lb) then                     ! top layer
                dzp  = zx(j+1)-zx(j)
                a(j) = -factx(j)*tksur/dzsur
                b(j) = 1. + factx(j)*tksur/dzsur + (1.-cnfac)*factx(j)*tkix(j)/dzp
                c(j) = -(1.-cnfac)*factx(j)* tkix(j)/dzp
                r(j) = tx_bef(j)+ cnfac*factx(j)*fnx(j)+ factx(j)*phix(j)
            else if (j < nl_lake+nl_soil) then    ! middle layer
               dzm  = (zx(j)-zx(j-1))
               dzp  = (zx(j+1)-zx(j))
               a(j) =   - (1.-cnfac)*factx(j)* tkix(j-1)/dzm
               b(j) = 1.+ (1.-cnfac)*factx(j)*(tkix(j)/dzp + tkix(j-1)/dzm)
               c(j) =   - (1.-cnfac)*factx(j)* tkix(j)/dzp
               r(j) = tx_bef(j) + cnfac*factx(j)*(fnx(j) - fnx(j-1)) + factx(j)*phix(j)
            else                                  ! bottom soil layer
               dzm  = (zx(j)-zx(j-1))
               a(j) =   - (1.-cnfac)*factx(j)*tkix(j-1)/dzm
               b(j) = 1.+ (1.-cnfac)*factx(j)*tkix(j-1)/dzm
               c(j) = 0.
               r(j) = tx_bef(j) - cnfac*factx(j)*fnx(j-1)
            end if
         end do

!------------------------------------------------------------
! Solve for tdsolution
!------------------------------------------------------------

         nl_sls = abs(snl) + nl_lake + nl_soil

         call tridia (nl_sls+1, a(lb-1:), b(lb-1:), c(lb-1:), r(lb-1:), tx(lb-1:))

            t_grnd = tx(lb-1)
         t_lake(1) = tx(1)

! surface fluxes of momentum, sensible and latent
! using ground temperatures from previous time step
         fseng = forc_rhoair*cpair*(t_grnd-thm)/rah
         fevpg = forc_rhoair*(qsatg+qsatgdT*(t_grnd-t_grnd_bef)-forc_q)/raw

         call qsat(t_grnd,forc_psrf,eg,degdT,qsatg,qsatgdT)    
		 	
		 dth = thm-t_grnd
         dqh = forc_q-qsatg
         tstar = vkc/fh*dth
         qstar = vkc/fq*dqh
         thvstar = tstar+0.61*th*qstar	
		  
         zeta = zldis*vkc*grav*thvstar/(ustar**2*thv)
		
	    if(zeta >= 0.) then                !stable	 
           zeta = min(2.,max(zeta,1.e-6))	  		    		    		   		   	  		     
         else                              !unstable	  		 	   	   		 
           zeta = max(-100.,min(real(zeta),-1.e-6))     
         endif	
		 				 							  
         obu = zldis/zeta  

		 if(zeta >= 0.)then
           um = max(ur,0.1)
         else
           wc = (-grav*ustar*thvstar*zii/thv)**(1./3.)
          wc2 = beta1*beta1*(wc*wc)
           um = sqrt(ur*ur+wc2)
         endif

	   					  
         call roughness_lake (snl,t_grnd,t_lake(1),lake_icefrac(1),forc_psrf,&
                              cur,ustar,z0mg,z0hg,z0qg)
	   
         iter = iter + 1
		 
         del_T_grnd = abs(t_grnd - t_grnd_bef)                

         if(iter .gt. itmin) then
            if(del_T_grnd <= dtmin) then
               convernum = convernum + 1
            end if
            if(convernum >= 4) EXIT
         endif

    ! ===============================================
      END DO ITERATION   ! end of stability iteration
    ! ===============================================

      do j = lb, nl_lake + nl_soil
         jprime = j - nl_lake
         if (j < 1) then               ! snow layer
            t_soisno(j) = tx(j)
         else if (j <= nl_lake) then   ! lake layer
            t_lake(j) = tx(j)
         else                          ! soil layer
            t_soisno(jprime) = tx(j)
         end if
      end do

!*----------------------------------------------------------------------
!*Zack Subin, 3/27/09
!*Since they are now a function of whatever t_grnd was before cooling
!*to freezing temperature, then this value should be used in the derivative correction term.
!*Allow convection if ground temp is colder than lake but warmer than 4C, or warmer than
!*lake which is warmer than freezing but less than 4C.
!*    tdmax = tfrz +4.
!*    if ( (snl < 0 .or. t_lake(1) <= tfrz) .and. t_grnd > tfrz) then
!*       t_grnd_bef = t_grnd
!*       t_grnd = tfrz
!*       fseng =forc_rhoair*cpair*(t_grnd-thm)/rah
!*       fevpg = forc_rhoair*(qsatg+qsatgdT*(t_grnd-t_grnd_bef)-forc_q)/raw 
!*    else if ( (t_lake(1) > t_grnd .and. t_grnd > tdmax) .or. &
!*              (t_lake(1) < t_grnd .and. t_lake(1) > tfrz .and. t_grnd < tdmax) ) then
!*              ! Convective mixing will occur at surface
!*       t_grnd_bef = t_grnd
!*       t_grnd = t_lake(1)
!*       fseng = forc_rhoair*cpair*(t_grnd-thm)/rah
!*       fevpg = forc_rhoair*(qsatg+qsatgdT*(t_grnd-t_grnd_bef)-forc_q)/raw 
!*    end if
!*----------------------------------------------------------------------

! net longwave from ground to atmosphere
!     olrg = (1.-emg)*forc_frl + emg*stefnc*t_grnd**4
      olrg = (1.-emg)*forc_frl + emg*sb*t_grnd**4

! additional variables for CWRF or other atmospheric models
      emis = emg
      z0ma = z0mg
      zol  = zeta
      rib  = min(5.,zol*ustar**2/(vkc*vkc/fh*um**2))

! radiative temperature
      trad = (olrg/sb)**0.25

! ground heat flux
      if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then
         htvp = hvap
      else
         htvp = hsub
      end if

! solar absorption below the surface.  
      fgrnd = sabg + forc_frl - olrg - fseng - htvp*fevpg

      taux = -forc_rhoair*forc_us/ram
      tauy = -forc_rhoair*forc_vs/ram
      fsena = fseng
      fevpa = fevpg
      lfevpa = htvp*fevpg


! 2 m height air temperature and specific humidity
!     tref = thm + vkc/fh*dth * (fh2m/vkc - fh/vkc)
!     qref = forc_q + vkc/fq*dqh * (fq2m/vkc - fq/vkc)
      tref = thm + vkc/fh*dth * (fh2m/vkc - fh/vkc)
      qref = forc_q + vkc/fq*dqh * (fq2m/vkc - fq/vkc)

! calculate sublimation, frosting, dewing
      qseva = fevpg
      qsubl = 0.
      qsdew = 0.
      qfros = 0.

      if (fevpg >= 0.0) then
         ! assumed sublimation earlier than vaporation,  
         ! 0.1 is an arbitrarily assumption, should be updated in the future
         if (lb<=0 .or. lake_icefrac(1)>=0.001) qsubl = fevpg
      else
         if (t_grnd < tfrz) then
            qfros = abs(fevpg)
         else
            qsdew = abs(fevpg)
         end if
      end if


!#if(defined CLMDEBUG)
      ! sum energy content and total energy into lake for energy check. any errors will be from the
      !     tridiagonal solution.
      esum1 = 0.0
      esum2 = 0.0
      do j = lb, nl_lake + nl_soil
         esum1 = esum1 + (tx(j)-tx_bef(j))*cvx(j)
         esum2 = esum2 + (tx(j)-tfrz)*cvx(j)
      end do
                           ! fgrnd includes all the solar radiation absorbed in the lake,
      errsoi = esum1/deltim - fgrnd  
!     if(abs(errsoi) > 0.2) then
!        write(6,*)'energy conservation error in LAND WATER COLUMN during tridiagonal solution,', &
!                  'error (W/m^2):', errsoi
!     end if

!#endif


!------------------------------------------------------------
!*[4] Phase change
!------------------------------------------------------------

      sm = 0.0
      xmf = 0.0
      imelt_soisno(:) = 0
      imelt_lake(:) = 0

      ! Check for case of snow without snow layers and top lake layer temp above freezing.

      if (snl == 0 .and. scv > 0. .and. t_lake(1) > tfrz) then
         heatavail = (t_lake(1) - tfrz) * cv_lake(1)
         melt = min(scv, heatavail/hfus)
         heatrem = max(heatavail - melt*hfus, 0.) !catch small negative value to keep t at tfrz
         t_lake(1) = tfrz + heatrem/(cv_lake(1))

         snowdp = max(0., snowdp*(1. - melt/scv))
         scv = scv - melt

         if (scv < 1.e-12) scv = 0.        ! prevent tiny residuals
         if (snowdp < 1.e-12) snowdp = 0.  ! prevent tiny residuals
         sm = sm + melt/deltim
         xmf = xmf + melt*hfus
      end if

      ! Lake phase change
      do j = 1,nl_lake
         if (t_lake(j) > tfrz .and. lake_icefrac(j) > 0.) then ! melting
            imelt_lake(j) = 1
            heatavail = (t_lake(j) - tfrz) * cv_lake(j)
            melt = min(lake_icefrac(j)*denh2o*dz_lake(j), heatavail/hfus)
                       !denh2o is used because layer thickness is not adjusted for freezing
            heatrem = max(heatavail - melt*hfus, 0.)  !catch small negative value to keep t at tfrz
         else if (t_lake(j) < tfrz .and. lake_icefrac(j) < 1.) then !freezing
            imelt_lake(j) = 2
            heatavail = (t_lake(j) - tfrz) * cv_lake(j)
            melt = max(-(1.-lake_icefrac(j))*denh2o*dz_lake(j), heatavail/hfus)
                       !denh2o is used because layer thickness is not adjusted for freezing
            heatrem = min(heatavail - melt*hfus, 0.)  !catch small positive value to keep t at tfrz
         end if
         ! Update temperature and ice fraction.
         if (imelt_lake(j) > 0) then
            lake_icefrac(j) = lake_icefrac(j) - melt/(denh2o*dz_lake(j))
            if (lake_icefrac(j) > 1.-1.e-12) lake_icefrac(j) = 1.  ! prevent tiny residuals
            if (lake_icefrac(j) < 1.e-12)    lake_icefrac(j) = 0.  ! prevent tiny residuals
            cv_lake(j) = cv_lake(j) + melt*(cpliq-cpice)           ! update heat capacity
            t_lake(j) = tfrz + heatrem/cv_lake(j)
            xmf = xmf + melt*hfus
         end if
      end do

      ! snow & soil phase change. currently, does not do freezing point depression.
      do j = snl+1,nl_soil
         if (t_soisno(j) > tfrz .and. wice_soisno(j) > 0.) then ! melting
            imelt_soisno(j) = 1
            heatavail = (t_soisno(j) - tfrz) * cv_soisno(j)
            melt = min(wice_soisno(j), heatavail/hfus)
            heatrem = max(heatavail - melt*hfus, 0.) !catch small negative value to keep t at tfrz
            if (j <= 0) sm = sm + melt/deltim
         else if (t_soisno(j) < tfrz .and. wliq_soisno(j) > 0.) then !freezing
            imelt_soisno(j) = 2
            heatavail = (t_soisno(j) - tfrz) * cv_soisno(j)
            melt = max(-wliq_soisno(j), heatavail/hfus)
            heatrem = min(heatavail - melt*hfus, 0.) !catch small positive value to keep t at tfrz
         end if

         ! Update temperature and soil components.
         if (imelt_soisno(j) > 0) then
            wice_soisno(j) = wice_soisno(j) - melt
            wliq_soisno(j) = wliq_soisno(j) + melt
            if (wice_soisno(j) < 1.e-12) wice_soisno(j) = 0. ! prevent tiny residuals
            if (wliq_soisno(j) < 1.e-12) wliq_soisno(j) = 0. ! prevent tiny residuals
            cv_soisno(j) = cv_soisno(j) + melt*(cpliq-cpice) ! update heat capacity
            t_soisno(j) = tfrz + heatrem/cv_soisno(j)
            xmf = xmf + melt*hfus
         end if
      end do
      !------------------------------------------------------------

!#if (defined CLMDEBUG)
      ! second energy check and water check. now check energy balance before and after phase
      ! change, considering the possibility of changed heat capacity during phase change, by
      ! using initial heat capacity in the first step, final heat capacity in the second step,
      ! and differences from tfrz only to avoid enthalpy correction for (cpliq-cpice)*melt*tfrz.
      ! also check soil water sum.
      do j = 1, nl_lake
         esum2 = esum2 - (t_lake(j)-tfrz)*cv_lake(j)
      end do

      do j = lb, nl_soil
         esum2 = esum2 - (t_soisno(j)-tfrz)*cv_soisno(j)
      end do

      esum2 = esum2 - xmf 
      errsoi = esum2/deltim

!     if(abs(errsoi) > 0.2) then
!     write(6,*) 'energy conservation error in LAND WATER COLUMN during phase change, error (W/m^2):', errsoi
!     end if
!#endif

!------------------------------------------------------------
!*[5] Convective mixing: make sure fracice*dz is conserved, heat content c*dz*T is conserved, and
! all ice ends up at the top. Done over all lakes even if frozen.
! Either an unstable density profile or ice in a layer below an incompletely frozen layer will trigger.
!------------------------------------------------------------

      ! recalculate density
      do j = 1, nl_lake
         rhow(j) = (1.-lake_icefrac(j))*1000.*(1.0-1.9549e-05*(abs(t_lake(j)-277.))**1.68) &
                     + lake_icefrac(j)*denice
      end do

      do j = 1, nl_lake-1
         qav = 0.
         nav = 0.
         iceav = 0.

         if (rhow(j)>rhow(j+1) .or. (lake_icefrac(j)<=1.0 .and. lake_icefrac(j+1)>0.)) then
            do i = 1, j+1
               qav = qav + dz_lake(i)*(t_lake(i)-tfrz) * & 
                       ((1. - lake_icefrac(i))*cwat + lake_icefrac(i)*cice_eff)
               iceav = iceav + lake_icefrac(i)*dz_lake(i)
               nav = nav + dz_lake(i)
            end do

            qav = qav/nav
            iceav = iceav/nav
            !if the average temperature is above freezing, put the extra energy into the water.
            !if it is below freezing, take it away from the ice.
            if (qav > 0.) then
               tav_froz = 0. !Celsius
               tav_unfr = qav / ((1. - iceav)*cwat)
            else if (qav < 0.) then
               tav_froz = qav / (iceav*cice_eff)
               tav_unfr = 0. !Celsius
            else
               tav_froz = 0.
               tav_unfr = 0.
            end if
         end if

         if (nav > 0.) then
            do i = 1, j+1

            !put all the ice at the top.
            !if the average temperature is above freezing, put the extra energy into the water.
            !if it is below freezing, take it away from the ice.
            !for the layer with both ice & water, be careful to use the average temperature
            !that preserves the correct total heat content given what the heat capacity of that
            !layer will actually be.

               if (i == 1) zsum = 0.
               if ((zsum+dz_lake(i))/nav <= iceav) then
                  lake_icefrac(i) = 1.
                  t_lake(i) = tav_froz + tfrz
               else if (zsum/nav < iceav) then
                  lake_icefrac(i) = (iceav*nav - zsum) / dz_lake(i)
                  ! Find average value that preserves correct heat content.
                  t_lake(i) = ( lake_icefrac(i)*tav_froz*cice_eff &
                              + (1. - lake_icefrac(i))*tav_unfr*cwat ) &
                              / ( lake_icefrac(i)*cice_eff + (1-lake_icefrac(i))*cwat ) + tfrz
               else
                  lake_icefrac(i) = 0.
                  t_lake(i) = tav_unfr + tfrz
               end if
               zsum = zsum + dz_lake(i)

               rhow(i) = (1.-lake_icefrac(i))*1000.*(1.-1.9549e-05*(abs(t_lake(i)-277.))**1.68) &
                           + lake_icefrac(i)*denice
            end do
         end if
      end do

!------------------------------------------------------------
!*[6] Re-evaluate thermal properties and sum energy content.
!------------------------------------------------------------
      ! for lake
      do j = 1, nl_lake
         cv_lake(j) = dz_lake(j) * (cwat*(1.-lake_icefrac(j)) + cice_eff*lake_icefrac(j))
      end do

      ! for snow / soil
      call hCapacity (itypwat,lb,nl_soil,csol,porsl,&
                      wice_soisno(lb:),wliq_soisno(lb:),scv,dz_soisno(lb:),cv_soisno(lb:))

      ! do as above to sum energy content
      ncvts = 0.
      do j = 1, nl_lake
         ncvts = ncvts + cv_lake(j)*(t_lake(j)-tfrz) + cfus*dz_lake(j)*(1.-lake_icefrac(j)) 
      end do

      do j = lb, nl_soil
         ncvts = ncvts + cv_soisno(j)*(t_soisno(j)-tfrz) + hfus*wliq_soisno(j) 
         if (j == 1 .and. scv > 0. .and. j == lb) then
            ncvts = ncvts - scv*hfus
         end if
      end do

      ! check energy conservation.
      errsoi = (ncvts-ocvts)/deltim - fgrnd
      if (abs(errsoi) < 0.10) then ! send to balance check and abort
         fseng = fseng - errsoi
         fsena = fseng
         fgrnd = fgrnd + errsoi
         errsoi = 0.
      end if


  end subroutine laketem



  subroutine snowwater_lake ( &
             ! "in" arguments
             ! ---------------------------
             maxsnl      , nl_soil     , nl_lake   , deltim ,&
             ssi         , wimp        , porsl     , pg_rain ,&
             pg_snow     , dz_lake     , imelt     , fiold ,&
             qseva       , qsubl       , qsdew     , qfros ,&

             ! "inout" arguments
             ! ---------------------------
             z_soisno    , dz_soisno   , zi_soisno , t_soisno ,&
             wice_soisno , wliq_soisno , t_lake    , lake_icefrac ,&
             fseng       , fgrnd       , snl       , scv ,&
             snowdp      , sm )

!-----------------------------------------------------------------------------------------------
! Calculation of Lake Hydrology. Lake water mass is kept constant. The soil is simply maintained at
! volumetric saturation if ice melting frees up pore space. 
!
! Called: 
!    -> snowwater:             change of snow mass and snow water onto soil
!    -> snowcompaction:        compaction of snow layers
!    -> combinesnowlayers:     combine snow layers that are thinner than minimum
!    -> dividesnowlayers:      subdivide snow layers that are thicker than maximum
!
! Initial: Yongjiu Dai, December, 2012
!                          April, 2014
!-----------------------------------------------------------------------------------------------

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : denh2o, denice, hfus, tfrz, cpliq, cpice
  use clm_varcon, only : denh2o, denice, hfus, tfrz, cpliq, cpice

! use SOIL_SNOW_hydrology			! removed by Anning Huang 2016.11.17
! use SNOW_Layers_CombineDivide

  implicit none

! ------------- in/inout/out variables -----------------------------------------

  integer, INTENT(in) :: maxsnl  ! maximum number of snow layers
  integer, INTENT(in) :: nl_soil ! number of soil layers
  integer, INTENT(in) :: nl_lake ! number of soil layers

  real(r8), INTENT(in) :: deltim             ! seconds in a time step (sec)
  real(r8), INTENT(in) :: ssi                ! irreducible water saturation of snow
  real(r8), INTENT(in) :: wimp               ! water impremeable if porosity less than wimp
  real(r8), INTENT(in) :: porsl(1:nl_soil)   ! volumetric soil water at saturation (porosity)

  real(r8), INTENT(in) :: pg_rain            ! rainfall incident on ground [mm/s]
  real(r8), INTENT(in) :: pg_snow            ! snowfall incident on ground [mm/s]
  real(r8), INTENT(in) :: dz_lake(1:nl_lake) ! layer thickness for lake (m)

  integer,  INTENT(in) :: imelt(maxsnl+1:0)  ! signifies if node in melting (imelt = 1)
  real(r8), INTENT(in) :: fiold(maxsnl+1:0)  ! fraction of ice relative to the total water content at the previous time step
  real(r8), INTENT(in) :: qseva              ! ground surface evaporation rate (mm h2o/s)
  real(r8), INTENT(in) :: qsubl              ! sublimation rate from snow pack (mm H2O /s) [+]
  real(r8), INTENT(in) :: qsdew              ! surface dew added to snow pack (mm H2O /s) [+]
  real(r8), INTENT(in) :: qfros              ! ground surface frosting formation (mm H2O /s) [+]

  real(r8), INTENT(inout) :: z_soisno   (maxsnl+1:nl_soil) ! layer depth  (m)
  real(r8), INTENT(inout) :: dz_soisno  (maxsnl+1:nl_soil) ! layer thickness depth (m)
  real(r8), INTENT(inout) :: zi_soisno    (maxsnl:nl_soil) ! interface depth (m)
  real(r8), INTENT(inout) :: t_soisno   (maxsnl+1:nl_soil) ! snow temperature (Kelvin)
  real(r8), INTENT(inout) :: wice_soisno(maxsnl+1:nl_soil) ! ice lens (kg/m2)
  real(r8), INTENT(inout) :: wliq_soisno(maxsnl+1:nl_soil) ! liquid water (kg/m2)
  real(r8), INTENT(inout) :: t_lake      (1:nl_lake) ! lake temperature (Kelvin)
  real(r8), INTENT(inout) :: lake_icefrac(1:nl_lake) ! mass fraction of lake layer that is frozen

  real(r8), INTENT(inout) :: fseng  ! total sensible heat flux (W/m**2) [+ to atm]
  real(r8), INTENT(inout) :: fgrnd  ! heat flux into snow / lake (W/m**2) [+ = into soil]

  integer , INTENT(inout) :: snl    ! number of snow layers
  real(r8), INTENT(inout) :: scv    ! snow water (mm H2O)
  real(r8), INTENT(inout) :: snowdp ! snow height (m)
  real(r8), INTENT(inout) :: sm     ! rate of snow melt (mm H2O /s)


! ------------- other local variables -----------------------------------------
  integer  j          ! indices
  integer lb          ! lower bound of array

  real(r8) qout_snowb ! rate of water out of snow bottom (mm/s)
  real(r8) xmf        ! snow melt heat flux (W/m**2)

  real(r8) sumsnowice ! sum of snow ice if snow layers found above unfrozen lake [kg/m&2]
  real(r8) sumsnowliq ! sum of snow liquid if snow layers found above unfrozen lake [kg/m&2]
  logical  unfrozen   ! true if top lake layer is unfrozen with snow layers above
  real(r8) heatsum    ! used in case above [J/m^2]
  real(r8) heatrem    ! used in case above [J/m^2]

  real(r8) a, b, c, d
  real(r8) wice_lake(1:nl_lake)  ! ice lens (kg/m2)
  real(r8) wliq_lake(1:nl_lake)  ! liquid water (kg/m2)
  real(r8) t_ave, frac_
!-----------------------------------------------------------------------

      ! for runoff calculation (assumed no mass change in the land water bodies)
      lb = snl + 1
      qout_snowb = 0.0

      ! ----------------------------------------------------------
      !*[1] snow layer on frozen lake
      ! ----------------------------------------------------------
      if (snl < 0) then 
         lb = snl + 1
         call snowwater (lb,deltim,ssi,wimp,&
                         pg_rain,qseva,qsdew,qsubl,qfros,&
                         dz_soisno(lb:0),wice_soisno(lb:0),wliq_soisno(lb:0),qout_snowb)

         ! Natural compaction and metamorphosis.
         lb = snl + 1
         call snowcompaction (lb,deltim, &
                              imelt(lb:0),fiold(lb:0),t_soisno(lb:0),&
                              wliq_soisno(lb:0),wice_soisno(lb:0),dz_soisno(lb:0))

         ! Combine thin snow elements
         lb = maxsnl + 1
         call snowlayerscombine (lb, snl,&
                                 z_soisno(lb:1),dz_soisno(lb:1),zi_soisno(lb-1:0),&
                                 wliq_soisno(lb:1),wice_soisno(lb:1),&
                                 t_soisno(lb:1),scv,snowdp )

         ! Divide thick snow elements
         if (snl < 0) then
         call snowlayersdivide (lb,snl,z_soisno(lb:0),dz_soisno(lb:0),zi_soisno(lb-1:0),&
                                wliq_soisno(lb:0),wice_soisno(lb:0),t_soisno(lb:0))
         endif

      ! ----------------------------------------------------------
      !*[2] check for single completely unfrozen snow layer over lake. 
      !     Modeling this ponding is unnecessary and can cause instability after the timestep 
      !     when melt is completed, as the temperature after melt can be excessive 
      !     because the fluxes were calculated with a fixed ground temperature of freezing, but the 
      !     phase change was unable to restore the temperature to freezing.  (Zack Subnin 05/2010)
      ! ----------------------------------------------------------

         if (snl == -1 .and. wice_soisno(0) == 0.) then
            ! Remove layer
            ! Take extra heat of layer and release to sensible heat in order to maintain energy conservation.
            heatrem = cpliq*wliq_soisno(0)*(t_soisno(0) - tfrz)
            fseng = fseng + heatrem/deltim
            fgrnd = fgrnd - heatrem/deltim

            snl = 0
            scv = 0.
            snowdp = 0.
         end if

      endif 

      ! ----------------------------------------------------------
      !*[3] check for snow layers above lake with unfrozen top layer. Mechanically,
      !     the snow will fall into the lake and melt or turn to ice. If the top layer has
      !     sufficient heat to melt the snow without freezing, then that will be done.
      !     Otherwise, the top layer will undergo freezing, but only if the top layer will
      !     not freeze completely. Otherwise, let the snow layers persist and melt by diffusion.
      ! ----------------------------------------------------------

      if (t_lake(1) > tfrz .and. snl < 0 .and. lake_icefrac(1) < 0.001) then ! for unfrozen lake
         unfrozen = .true.
      else
         unfrozen = .false.
      end if

      sumsnowice = 0.
      sumsnowliq = 0.
      heatsum = 0.0
      do j = snl+1,0
         if (unfrozen) then
            sumsnowice = sumsnowice + wice_soisno(j)
            sumsnowliq = sumsnowliq + wliq_soisno(j)
            heatsum = heatsum + wice_soisno(j)*cpice*(tfrz-t_soisno(j)) &
                             + wliq_soisno(j)*cpliq*(tfrz-t_soisno(j))
         end if
      end do

      if (unfrozen) then
         ! Remove snow and subtract the latent heat from the top layer.
         sm = sm + scv/deltim
!*       xmf = xmf + scv*hfus/deltim
         scv = 0.
         snowdp = 0.
         snl = 0

         t_ave = tfrz - heatsum/(sumsnowice*cpice + sumsnowliq*cpliq)        

         a = sumsnowice*cpice*(tfrz-t_ave)
         b = sumsnowice*hfus
         c = (t_lake(1) - tfrz)*cpliq*denh2o*dz_lake(1)
         d = denh2o*dz_lake(1)*hfus

         ! lake layer all frozen, release heat to warm up sumsnowice
         if (c-a <= -d) then
            t_lake(1) = (c+d + cpice*(sumsnowice*t_ave+denh2o*dz_lake(1)*tfrz))/&
                        (cpice*(sumsnowice+denh2o*dz_lake(1)))
            lake_icefrac(1) = 1.0
         ! lake tem at tfrz, partial lake frozing -> release heat -> warm up snowall to tfrz
         else if (c-a <= 0.0) then
            t_lake(1) = tfrz
            frac_ = (a-c)/d
            wice_lake(1) = sumsnowice + frac_*denh2o*dz_lake(1)
            wliq_lake(1) = (1.0 - frac_)*denh2o*dz_lake(1)
            lake_icefrac(1) = wice_lake(1)/(wice_lake(1) + wliq_lake(1))
         ! partial sumsnowice melting, no latent heat from lake freezing
         else if (c-a <= b) then
            t_lake(1) = tfrz
            lake_icefrac(1) = sumsnowice/(denh2o*dz_lake(1) + sumsnowice) - (c-a)/b
         ! all sumsnowice melt (c-a>=b)
         else
            lake_icefrac(1) = 0.0
            t_lake(1) = (cpliq*(denh2o*dz_lake(1)*t_lake(1) + sumsnowice*tfrz) - a - b) / &
                        (cpliq*(denh2o*dz_lake(1) + sumsnowice))
         endif
      end if


      ! ----------------------------------------------------------
      !*[4] Soil water and ending water balance 
      ! ----------------------------------------------------------
      ! Here this consists only of making sure that soil is saturated even as it melts and
      ! pore space opens up. Conversely, if excess ice is melting and the liquid water exceeds the
      ! saturation value, then remove water.

      do j = 1, nl_soil 
         a = wliq_soisno(j)/(dz_soisno(j)*denh2o) + wice_soisno(j)/(dz_soisno(j)*denice)
 
         if (a < porsl(j)) then
            wliq_soisno(j) = max( 0., (porsl(j)*dz_soisno(j) - wice_soisno(j)/denice)*denh2o )
            wice_soisno(j) = max( 0., (porsl(j)*dz_soisno(j) - wliq_soisno(j)/denh2o)*denice )
         else 
            wliq_soisno(j) = max(0., wliq_soisno(j) - (a - porsl(j))*denh2o*dz_soisno(j) )
            wice_soisno(j) = max( 0., (porsl(j)*dz_soisno(j) - wliq_soisno(j)/denh2o)*denice )
         end if 

         if (wliq_soisno(j) > porsl(j)*denh2o*dz_soisno(j)) then
             wliq_soisno(j) = porsl(j)*denh2o*dz_soisno(j)
         endif
      end do


  end subroutine snowwater_lake



  subroutine roughness_lake (snl,t_grnd,t_lake,lake_icefrac,forc_psrf,&
                             cur,ustar,z0mg,z0hg,z0qg)
! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl, use PhysicalConstants, only : tfrz,vkc,grav
  use clm_varcon, only : tfrz,vkc,grav

  IMPLICIT NONE

  integer,  INTENT(in) :: snl       ! number of snow layers
  real(r8), INTENT(in) :: t_grnd    ! ground temperature
  real(r8), INTENT(in) :: t_lake(1) ! surface lake layer temperature [K]
  real(r8), INTENT(in) :: lake_icefrac(1) ! surface lake layer ice mass fraction [0-1]
  real(r8), INTENT(in) :: forc_psrf ! atmosphere pressure at the surface [pa]

  real(r8), INTENT(in) :: cur       ! Charnock parameter (-)
  real(r8), INTENT(in) :: ustar     ! u* in similarity theory [m/s]

  real(r8), INTENT(out) :: z0mg     ! roughness length over ground, momentum [m]
  real(r8), INTENT(out) :: z0hg     ! roughness length over ground, sensible heat [m]
  real(r8), INTENT(out) :: z0qg     ! roughness length over ground, latent heat [m]

  real(r8), parameter :: cus = 0.1       ! empirical constant for roughness under smooth flow
  real(r8), parameter :: kva0 = 1.51e-5  ! kinematic viscosity of air (m^2/s) at 20C and 1.013e5 Pa
  real(r8), parameter :: prn = 0.713     ! Prandtl # for air at neutral stability
  real(r8), parameter :: sch = 0.66      ! Schmidt # for water in air at neutral stability

  real(r8) kva    ! kinematic viscosity of air at ground temperature and forcing pressure
  real(r8) sqre0  ! root of roughness Reynolds number

      if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0).or. lake_icefrac(1) < 0.001) then ! for unfrozen lake
          kva = kva0 * (t_grnd/293.15)**1.5 * 1.013e5/forc_psrf ! kinematic viscosity of air
          z0mg = max(cus*kva/max(ustar,1.e-4),cur*ustar*ustar/grav) ! momentum roughness length
          z0mg = max(z0mg, 1.0e-10) ! This limit is redundant with current values.
          sqre0 = (max(z0mg*ustar/kva,0.1))**0.5   ! square root of roughness Reynolds number
          z0hg = z0mg * exp( -vkc/prn*( 4.*sqre0 - 3.2) ) ! SH roughness length
          z0qg = z0mg * exp( -vkc/sch*( 4.*sqre0 - 4.2) ) ! LH roughness length
          z0qg = max(z0qg, 1.0e-5)  ! Minimum allowed roughness length for unfrozen lakes
          z0hg = max(z0hg, 1.0e-5)  ! set low so it is only to avoid floating point exceptions
      else if (snl == 0) then ! frozen lake with ice, and no snow cover
          z0mg = 0.001              ! z0mg won't have changed
          z0hg = z0mg/exp(0.13 * (ustar*z0mg/1.5e-5)**0.45)
          z0qg = z0hg
      else                          ! use roughness over snow
          z0mg = 0.0024             ! z0mg won't have changed
          z0hg = z0mg/exp(0.13 * (ustar*z0mg/1.5e-5)**0.45)
          z0qg = z0hg
      end if

  end subroutine roughness_lake



  subroutine hConductivity_lake(nl_lake,snl,t_grnd,&
                                z_lake,t_lake,lake_icefrac,rhow,&
                                dlat,ustar,z0mg,lakedepth,depthcrit,tk_lake) 

! -------------------------------------------------------------------------
! Diffusivity and implied thermal "conductivity" = diffusivity * cwat
! -------------------------------------------------------------------------

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : tfrz,tkwat,tkice,tkair,&
!                               vkc,grav,cpliq,cpice,cpair,denh2o,denice
  use clm_varcon, only : tfrz,tkwat,tkice,tkair,&
                                vkc,grav,cpliq,cpice,cpair,denh2o,denice

  IMPLICIT NONE

  integer, INTENT(in) :: nl_lake  ! number of soil layers
  integer, INTENT(in) :: snl      ! number of snow layers
  real(r8), INTENT(in) :: t_grnd  ! ground surface temperature [k]
  real(r8), INTENT(in) :: z_lake(nl_lake)       ! lake node depth (middle point of layer) (m)
  real(r8), INTENT(in) :: t_lake(nl_lake)       ! lake temperature (kelvin)
  real(r8), INTENT(in) :: lake_icefrac(nl_lake) ! lake mass fraction of lake layer that is frozen
  real(r8), INTENT(in) :: rhow(nl_lake)         ! density of water (kg/m**3)

  real(r8), INTENT(in) :: dlat      ! latitude (radians)
  real(r8), INTENT(in) :: ustar     ! u* in similarity theory [m/s]
  real(r8), INTENT(in) :: z0mg      ! roughness length over ground, momentum [m]
  real(r8), INTENT(in) :: lakedepth ! column lake depth (m)
  real(r8), INTENT(in) :: depthcrit ! (m) Depth beneath which to enhance mixing


  real(r8), INTENT(out) :: tk_lake(nl_lake) ! thermal conductivity at layer node [W/(m K)]

! local
  real(r8) kme(nl_lake) ! molecular + eddy diffusion coefficient (m**2/s)
  real(r8) cwat   ! specific heat capacity of water (j/m**3/kelvin)
  real(r8) den    ! used in calculating ri
  real(r8) drhodz ! d [rhow] /dz (kg/m**4)
  real(r8) fangkm ! (m^2/s) extra diffusivity based on Fang & Stefan 1996
  real(r8) ke     ! eddy diffusion coefficient (m**2/s)
  real(r8) km     ! molecular diffusion coefficient (m**2/s)
  real(r8) ks     ! coefficient for calculation of decay of eddy diffusivity with depth
  real(r8) n2     ! brunt-vaisala frequency (/s**2)
  real(r8) num    ! used in calculating ri
  real(r8) ri     ! richardson number
  real(r8) tkice_eff  ! effective conductivity since layer depth is constant
  real(r8) tmp    !
  real(r8) u2m    ! 2 m wind speed (m/s
  real(r8) ws     ! surface friction velocity (m/s)

  real(r8), parameter :: mixfact = 25.   ! Mixing enhancement factor. Subin et al. 2012 set to 10 
  real(r8), parameter :: p0 = 1.        ! neutral value of turbulent prandtl number
  integer j

! -------------------------------------------------------------------

           cwat = cpliq*denh2o
      tkice_eff = tkice * denice/denh2o ! effective conductivity since layer depth is constant
             km = tkwat/cwat                   ! a constant (molecular diffusivity)
            u2m = max(0.1,ustar/vkc*log(2./z0mg))	  
             ws = 1.2e-03 * u2m
             ks = 6.6 * sqrt( abs(sin(dlat)) ) * (u2m**(-1.84))

      do j = 1, nl_lake-1
         drhodz = (rhow(j+1)-rhow(j)) / (z_lake(j+1)-z_lake(j))
         n2 = max(7.5e-5, grav / rhow(j) * drhodz)
         num = 40. * n2 * (vkc*z_lake(j))**2
         tmp = -2.*ks*z_lake(j)        ! to avoid underflow computing
         if(tmp < -40.) tmp = -40.     ! 
         den = max( (ws**2) * exp(tmp), 1.e-10 )
         ri = ( -1. + sqrt( max(1.+num/den, 0.) ) ) / 20.

         if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then
            tmp = -ks*z_lake(j)        ! to avoid underflow computing
            if(tmp < -40.) tmp = -40.  ! 

                ke = vkc*ws*z_lake(j)/p0 * exp(tmp) / (1.+37.*ri*ri)	         
		    kme(j) = km + ke
            fangkm = 1.039e-8_r8 * max(n2,7.5e-5)**(-0.43)  ! Fang & Stefan 1996, citing Ellis et al 1991	            
			kme(j) = kme(j) + fangkm

            if (lakedepth >= depthcrit) then
               kme(j) = kme(j) * mixfact    ! Mixing enhancement factor for lake deep than 25m.
            end if	

            tk_lake(j) = kme(j)*cwat

         else

            kme(j) = km	   
            fangkm = 1.039e-8 * max(n2,7.5e-5)**(-0.43)
            kme(j) = kme(j) + fangkm

            if (lakedepth >= depthcrit) then
                kme(j) = kme(j) * mixfact
            end if

            tk_lake(j) = kme(j)*cwat*tkice_eff / ((1.-lake_icefrac(j))*tkice_eff &
                       + kme(j)*cwat*lake_icefrac(j))
         end if
      end do

      kme(nl_lake) = kme(nl_lake-1)

      if ((t_grnd > tfrz .and. t_lake(1) > tfrz .and. snl == 0) .or. lake_icefrac(1) < 0.001) then
         tk_lake(nl_lake) = tk_lake(nl_lake-1)
      else
         tk_lake(nl_lake) = kme(nl_lake)*cwat*tkice_eff / ( (1.-lake_icefrac(nl_lake))*tkice_eff &
                    + kme(nl_lake)*cwat*lake_icefrac(nl_lake) )
      end if

  end subroutine hConductivity_lake

!
!------------------------------------------------
! Combined by Anning Huang 2016.11.17	

 subroutine moninobuk1(hu,ht,hq,displa,z0m,z0h,z0q,obu,um,&
		      ustar,fh2m,fq2m,fm10m,fm,fh,fq)

! ======================================================================
! Original author : Yongjiu Dai, September 15, 1999
!
! calculation of friction velocity, relation for potential temperatur
! and humidity profiles of surface boundary layer.
! the scheme is based on the work of Zeng et al. (1998):
! Intercomparison of bulk aerodynamic algorithms for the computation
! of sea surface fluxes using TOGA CORE and TAO data. J. Climate, Vol. 11: 2628-2644
! ======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : vonkar
  use clm_varcon, only : vkc
  implicit none

! ---------------------- dummy argument --------------------------------

  real(r8), INTENT(in) :: hu       ! observational height of wind [m]
  real(r8), INTENT(in) :: ht       ! observational height of temperature [m]
  real(r8), INTENT(in) :: hq       ! observational height of humidity [m]
  real(r8), INTENT(in) :: displa   ! displacement height [m]
  real(r8), INTENT(in) :: z0m      ! roughness length, momentum [m]
  real(r8), INTENT(in) :: z0h      ! roughness length, sensible heat [m]
  real(r8), INTENT(in) :: z0q      ! roughness length, latent heat [m]
  real(r8), INTENT(in) :: obu      ! monin-obukhov length (m)
  real(r8), INTENT(in) :: um       ! wind speed including the stablity effect [m/s]

  real(r8), INTENT(out) :: ustar   ! friction velocity [m/s]
  real(r8), INTENT(out) :: fh2m    ! relation for temperature at 2m
  real(r8), INTENT(out) :: fq2m    ! relation for specific humidity at 2m
  real(r8), INTENT(out) :: fm10m   ! integral of profile function for momentum at 10m
  real(r8), INTENT(out) :: fm      ! integral of profile function for momentum
  real(r8), INTENT(out) :: fh      ! integral of profile function for heat 
  real(r8), INTENT(out) :: fq      ! integral of profile function for moisture

!------------------------ local variables ------------------------------

  real(r8) zldis  ! reference height "minus" zero displacement heght [m]
  real(r8) zetam  ! transition point of flux-gradient relation (wind profile)
  real(r8) zetat  ! transition point of flux-gradient relation (temp. profile)
  real(r8) zeta   ! dimensionless height used in Monin-Obukhov theory

! real(r8), external :: psi    ! stability function for unstable case
!-----------------------------------------------------------------------
! adjustment factors for unstable (moz < 0) or stable (moz > 0) conditions.

! wind profile
        zldis=hu-displa
        zeta=zldis/obu
        zetam=1.574
        if(zeta < -zetam)then           ! zeta < -1
          fm    = log(-zetam*obu/z0m) - psi(1,-zetam) &
                + psi(1,z0m/obu) + 1.14*((-zeta)**0.333-(zetam)**0.333) 
          ustar = vkc*um / fm 
        else if(zeta < 0.)then          ! -1 <= zeta < 0
          fm    = log(zldis/z0m) - psi(1,zeta) + psi(1,z0m/obu) 
          ustar = vkc*um / fm
        else if(zeta <= 1.)then         !  0 <= ztea <= 1
          fm    = log(zldis/z0m) + 5.*zeta - 5.*z0m/obu 
          ustar = vkc*um / fm
        else                            !  1 < zeta, phi=5+zeta
          fm    = log(obu/z0m) + 5. - 5.*z0m/obu + (5.*log(zeta)+zeta-1.) 
          ustar = vkc*um / fm
        endif
        
        ! for 10 meter wind-velocity
        zldis=10.+z0m
        zeta=zldis/obu
        zetam=1.574
        if(zeta < -zetam)then           ! zeta < -1
          fm10m  = log(-zetam*obu/z0m) - psi(1,-zetam) &
                + psi(1,z0m/obu) + 1.14*((-zeta)**0.333-(zetam)**0.333) 
        else if(zeta < 0.)then          ! -1 <= zeta < 0
          fm10m  = log(zldis/z0m) - psi(1,zeta) + psi(1,z0m/obu) 
        else if(zeta <= 1.)then         !  0 <= ztea <= 1
          fm10m  = log(zldis/z0m) + 5.*zeta - 5.*z0m/obu 
        else                            !  1 < zeta, phi=5+zeta
          fm10m  = log(obu/z0m) + 5. - 5.*z0m/obu + (5.*log(zeta)+zeta-1.) 
        endif

! temperature profile
        zldis=ht-displa
        zeta=zldis/obu
        zetat=0.465
        if(zeta < -zetat)then           ! zeta < -1
          fh    = log(-zetat*obu/z0h)-psi(2,-zetat) &
                + psi(2,z0h/obu) + 0.8*((zetat)**(-0.333)-(-zeta)**(-0.333)) 
        else if(zeta < 0.)then          ! -1 <= zeta < 0
          fh    = log(zldis/z0h) - psi(2,zeta) + psi(2,z0h/obu) 
        else if(zeta <= 1.)then         !  0 <= ztea <= 1
          fh    = log(zldis/z0h) + 5.*zeta - 5.*z0h/obu 
        else                            !  1 < zeta, phi=5+zeta
          fh    = log(obu/z0h) + 5. - 5.*z0h/obu + (5.*log(zeta)+zeta-1.) 
        endif

        ! for 2 meter screen temperature
        zldis=2.+z0h  ! ht-displa
        zeta=zldis/obu
        zetat=0.465
        if(zeta < -zetat)then           ! zeta < -1
          fh2m = log(-zetat*obu/z0h)-psi(2,-zetat) &
                + psi(2,z0h/obu) + 0.8*((zetat)**(-0.333)-(-zeta)**(-0.333))
        else if(zeta < 0.)then          ! -1 <= zeta < 0
          fh2m = log(zldis/z0h) - psi(2,zeta) + psi(2,z0h/obu)
        else if(zeta <= 1.)then         !  0 <= ztea <= 1
          fh2m = log(zldis/z0h) + 5.*zeta - 5.*z0h/obu
        else                            !  1 < zeta, phi=5+zeta
          fh2m = log(obu/z0h) + 5. - 5.*z0h/obu + (5.*log(zeta)+zeta-1.)
        endif

! humidity profile
        zldis=hq-displa
        zeta=zldis/obu
        zetat=0.465
        if(zeta < -zetat)then           ! zeta < -1
          fq    = log(-zetat*obu/z0q) - psi(2,-zetat) &
                + psi(2,z0q/obu) + 0.8*((zetat)**(-0.333)-(-zeta)**(-0.333)) 
        else if(zeta < 0.)then          ! -1 <= zeta < 0
          fq    = log(zldis/z0q) - psi(2,zeta) + psi(2,z0q/obu) 
        else if(zeta <= 1.)then         !  0 <= ztea <= 1
          fq    = log(zldis/z0q) + 5.*zeta - 5.*z0q/obu 
        else                            !  1 < zeta, phi=5+zeta
          fq    = log(obu/z0q) + 5. - 5.*z0q/obu + (5.*log(zeta)+zeta-1.) 
        endif

      ! for 2 meter screen humidity
        zldis=2.+z0h
        zeta=zldis/obu
        zetat=0.465
        if(zeta < -zetat)then           ! zeta < -1
           fq2m = log(-zetat*obu/z0q)-psi(2,-zetat) &
                 + psi(2,z0q/obu) + 0.8*((zetat)**(-0.333)-(-zeta)**(-0.333))
        elseif (zeta < 0.) then         ! -1 <= zeta < 0
           fq2m = log(zldis/z0q)-psi(2,zeta)+psi(2,z0q/obu)
        else if (zeta <= 1.) then       !  0 <= zeta <= 1
           fq2m = log(zldis/z0q)+5.*zeta-5.*z0q/obu
        else                            ! 1 < zeta, phi=5+zeta
           fq2m = log(obu/z0q)+5.-5.*z0q/obu+(5.*log(zeta)+zeta-1.)
        endif

 end subroutine moninobuk1




 subroutine moninobukini1(ur,th,thm,thv,dth,dqh,dthv,zldis,z0m,um,obu)

! ======================================================================
! Original author : Yongjiu Dai, September 15, 1999
!
! initialzation of Monin-Obukhov length,
! the scheme is based on the work of Zeng et al. (1998):
! Intercomparison of bulk aerodynamic algorithms for the computation
! of sea surface fluxes using TOGA CORE and TAO data. J. Climate, Vol. 11: 2628-2644
! ======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl  use PhysicalConstants, only : grav, vonkar
   use clm_varcon, only : grav, vkc
   implicit none

! Dummy argument
  real(r8), INTENT(in) :: ur    ! wind speed at reference height [m/s]
  real(r8), INTENT(in) :: thm   ! intermediate variable (tm+0.0098*ht)
  real(r8), INTENT(in) :: th    ! potential temperature [kelvin]
  real(r8), INTENT(in) :: thv   ! virtual potential temperature (kelvin)
  real(r8), INTENT(in) :: dth   ! diff of virtual temp. between ref. height and surface
  real(r8), INTENT(in) :: dthv  ! diff of vir. poten. temp. between ref. height and surface
  real(r8), INTENT(in) :: dqh   ! diff of humidity between ref. height and surface
  real(r8), INTENT(in) :: zldis ! reference height "minus" zero displacement heght [m]
  real(r8), INTENT(in) :: z0m   ! roughness length, momentum [m]

  real(r8), INTENT(out) :: um   ! wind speed including the stablity effect [m/s]
  real(r8), INTENT(out) :: obu  ! monin-obukhov length (m)

! Local       
  real(r8) wc     ! convective velocity [m/s]
  real(r8) rib    ! bulk Richardson number
  real(r8) zeta   ! dimensionless height used in Monin-Obukhov theory
  real(r8) ustar  ! friction velocity [m/s]

!-----------------------------------------------------------------------
! Initial values of u* and convective velocity

        ustar=0.06
        wc=0.5
        if(dthv >= 0.)then
          um=max(ur,0.1)
        else
          um=sqrt(ur*ur+wc*wc)
        endif
	     	  
        rib=grav*zldis*dthv/(thv*um*um)
  	
        if(rib >= 0.)then      ! neutral or stable
          zeta = rib*log(zldis/z0m)/(1.-5.*min(rib,0.19))  
          zeta = min(2.,max(zeta,1.e-6))  

        else                   ! unstable  		 
          zeta = rib*log(zldis/z0m)		     		  	  	 
          zeta = max(-100.0,min(real(zeta),-1.0e-6))  
        endif

        obu=zldis/zeta

 end subroutine moninobukini1	 

 real(r8) function psi(k,zeta)

!=======================================================================
! stability function for unstable case (rib < 0)

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 
  implicit none
      
  integer k    
  real(r8) zeta  ! dimensionless height used in Monin-Obukhov theory
  real(r8) chik  ! 

      chik = (1.-16.*zeta)**0.25
      if(k == 1)then
        psi = 2.*log((1.+chik)*0.5)+log((1.+chik*chik)*0.5)-2.*atan(chik)+2.*atan(1.)
      else
        psi = 2.*log((1.+chik*chik)*0.5)
      endif

 end function psi	   

  subroutine hCapacity (itypwat,lb,nl_soil,csol,porsl,wice_soisno,wliq_soisno,scv,dz_soisno,cv)


!-----------------------------------------------------------------------
! Original author : Yongjiu Dai, September 15, 1999
!
! calculation of heat capacities of snow / soil layers
! the volumetric heat capacity is calculated as a linear combination
! in terms of the volumetric fraction of the constituent phases.
!-----------------------------------------------------------------------

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : cpice,cpliq
  use clm_varcon, only : cpice,cpliq
  implicit none

  integer, INTENT(in) :: lb       ! lower bound of array
  integer, INTENT(in) :: nl_soil  ! upper bound of array
  integer, INTENT(in) :: itypwat  ! land water type (0=soil, 1=urban, 2=wetland, 
  real(r8), INTENT(in) :: csol(1:nl_soil)   ! heat capacity of soil soilds [J/(m3 K)]
  real(r8), INTENT(in) :: porsl(1:nl_soil)  ! soil porosity
  real(r8), INTENT(in) :: wice_soisno(lb:nl_soil)  ! ice lens [kg/m2]
  real(r8), INTENT(in) :: wliq_soisno(lb:nl_soil)  ! liqui water [kg/m2]
  real(r8), INTENT(in) :: dz_soisno(lb:nl_soil)    ! layer thickiness [m]
  real(r8), INTENT(in) :: scv               ! snow water equivalent [mm]
  real(r8), INTENT(out) :: cv(lb:nl_soil)   ! heat capacity [J/(m2 K)]

!-----------------------------------------------------------------------
! Soil heat capacity, which from de Vires (1963)

      if(itypwat<=1)then ! soil ground
         cv(1:) = csol(1:)*(1.-porsl(1:))*dz_soisno(1:) + wice_soisno(1:)*cpice + wliq_soisno(1:)*cpliq
      else               ! wetland or glacier/ice sheet
         cv(1:) = wice_soisno(1:)*cpice + wliq_soisno(1:)*cpliq
      endif
      if(lb==1 .AND. scv>0.) cv(1) = cv(1) + cpice*scv

! Snow heat capacity
      if(lb<=0)then
        cv(:0) = cpliq*wliq_soisno(:0) + cpice*wice_soisno(:0)
      endif

  end subroutine hCapacity 



  subroutine hConductivity (itypwat,lb,nl_soil,& 
                            dkdry,dksatu,porsl,dz_soisno,z_soisno,zi_soisno,t_soisno,wice_soisno,wliq_soisno,tk)

!-----------------------------------------------------------------------
! Original author : Yongjiu Dai, September 15, 1999
!
! calculation of thermal conductivities of snow / soil layers
! The thermal conductivity of soil is computed from
! the algorithm of Johansen (as reported by Farouki 1981), and of snow is from
! the formulation used in SNTHERM (Jordan 1991).
!
! The thermal conductivities at the interfaces between two neighbor layers
! (j, j+1) are derived from an assumption that the flux across the interface
! is equal to that from the node j to the interface and the flux from the
! interface to the node j+1.
!-----------------------------------------------------------------------

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : denh2o,denice,tfrz,tkwat,tkice,tkair
  use clm_varcon, only : denh2o,denice,tfrz,tkwat,tkice,tkair
  implicit none

  integer, INTENT(in) :: lb       ! lower bound of array
  integer, INTENT(in) :: nl_soil  ! upper bound of array
  integer, INTENT(in) :: itypwat  ! land water type (0=soil, 1=urban, 2=wetland,
                                  ! 3=land ice, 4=deep lake, 5=shallow lake)
  real(r8), INTENT(in) ::  dkdry(1:nl_soil)  ! thermal conductivity for dry soil [W/m-K]
  real(r8), INTENT(in) :: dksatu(1:nl_soil)  ! Thermal conductivity of saturated soil [W/m-K]
  real(r8), INTENT(in) ::  porsl(1:nl_soil)  ! fractional volume between soil grains=1.-dmvol
  real(r8), INTENT(in) ::   dz_soisno(lb:nl_soil)   ! layer thickiness [m]
  real(r8), INTENT(in) ::    z_soisno(lb:nl_soil)   ! node depth [m]
  real(r8), INTENT(in) ::   zi_soisno(lb-1:nl_soil) ! interface depth [m]
  real(r8), INTENT(in) ::  t_soisno(lb:nl_soil)   ! Nodal temperature [K]
  real(r8), INTENT(in) :: wice_soisno(lb:nl_soil)   ! ice lens [kg/m2]
  real(r8), INTENT(in) :: wliq_soisno(lb:nl_soil)   ! liqui water [kg/m2]

  real(r8), INTENT(out) :: tk(lb:nl_soil)    ! thermal conductivity [W/(m K)]

! local
  real(r8) rhosnow  ! partitial density of water (ice + liquid)
  real(r8) dksat    ! thermal conductivity for saturated soil (j/(k s m))
  real(r8) dke      ! kersten number
  real(r8) fl       ! fraction of liquid or unfrozen water to total water
  real(r8) satw     ! relative total water content of soil.
  real(r8) thk(lb:nl_soil)  ! thermal conductivity of layer

  integer i

!-----------------------------------------------------------------------
! Thermal conductivity of soil from Farouki (1981),
      do i = 1, nl_soil

         if(itypwat<=1)then         !soil ground
            thk(i) = dkdry(i)       !rock or dry soil

            if(porsl(i)>1.e-05)then
               satw = (wliq_soisno(i)/denh2o+wice_soisno(i)/denice)/(dz_soisno(i)*porsl(i))
               satw = min(1., satw)
               if(satw>.1e-6)then          
                  fl = wliq_soisno(i)/(wice_soisno(i)+wliq_soisno(i))
                  if(t_soisno(i) >= tfrz) then       ! Unfrozen soil
                     dke = log10(satw) + 1.0
                     dke = max(dke, 0.)
                     dksat = dksatu(i)
                  else                          ! Frozen soil
                     dke = satw
                     dksat = dksatu(i)*(2.29/0.57)**((1.-fl)*porsl(i))
                  end if
                  thk(i) = dke*dksat + (1.-dke)*dkdry(i)
               endif
            endif
         else                                   ! wetland or glacier
            thk(i) = tkwat
            if(t_soisno(i)<tfrz) thk(i) = tkice
         endif

      enddo

! Thermal conductivity of snow
      if(lb < 1)then
        do i = lb, 0
          rhosnow = (wice_soisno(i)+wliq_soisno(i))/dz_soisno(i)

        ! presently option [1] is the default option
        ! [1] Jordan (1991) pp. 18
          thk(i) = tkair+(7.75e-5*rhosnow+1.105e-6*rhosnow*rhosnow)*(tkice-tkair)

        ! [2] Sturm et al (1997)
        ! thk(i) = 0.0138 + 1.01e-3*rhosnow + 3.233e-6*rhosnow**2
        ! [3] Ostin and Andersson presented in Sturm et al., (1997)
        ! thk(i) = -0.871e-2 + 0.439e-3*rhosnow + 1.05e-6*rhosnow**2
        ! [4] Jansson(1901) presented in Sturm et al. (1997)
        ! thk(i) = 0.0293 + 0.7953e-3*rhosnow + 1.512e-12*rhosnow**2
        ! [5] Douville et al., (1995)
        ! thk(i) = 2.2*(rhosnow/denice)**1.88
        ! [6] van Dusen (1992) presented in Sturm et al. (1997)
        ! thk(i) = 0.021 + 0.42e-3*rhosnow + 0.22e-6*rhosnow**2
        enddo
      endif

! Thermal conductivity at the layer interface
      do i = lb, nl_soil-1

! the following consideration is try to avoid the snow conductivity 
! to be dominant in the thermal conductivity of the interface. 
! Because when the distance of bottom snow node to the interfacee 
! is larger than that of interface to top soil node,
! the snow thermal conductivity will be dominant, and the result is that 
! lees heat tranfer between snow and soil 
         if((i==0) .AND. (z_soisno(i+1)-zi_soisno(i)<zi_soisno(i)-z_soisno(i)))then
            tk(i) = 2.*thk(i)*thk(i+1)/(thk(i)+thk(i+1))
            tk(i) = max(0.5*thk(i+1),tk(i))
         else
            tk(i) = thk(i)*thk(i+1)*(z_soisno(i+1)-z_soisno(i)) &
                  /(thk(i)*(z_soisno(i+1)-zi_soisno(i))+thk(i+1)*(zi_soisno(i)-z_soisno(i)))
         endif
      enddo
      tk(nl_soil) = 0.

  end subroutine hConductivity

!-----------------------------------------------------------------------

  subroutine WATER (itypwat,lb,nl_soil,deltim ,&
             z_soisno    ,dz_soisno   ,zi_soisno   ,&
             bsw         ,porsl       ,psi0        ,hksati  ,rootr ,&
             t_soisno    ,wliq_soisno ,wice_soisno ,pg_rain ,sm    ,&
             etr         ,qseva       ,qsdew       ,qsubl   ,qfros ,&
             rsur        ,rnof        ,wtfact      ,pondmx  ,ssi   ,&
             wimp        ,smpmin      ,zwt         ,wa      ,&
             qcharge )  
	           
!=======================================================================
! this is the main subroutine to execute the calculation of 
! hydrological processes
!
! Original author : Yongjiu Dai, /09/1999/, /08/2002/, /04/2014/
!
! FLOW DIAGRAM FOR WATER.F90
!
! WATER ===> snowwater
!            surfacerunoff 
!            soilwater 
!            subsurfacerunoff 
!
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : denice, denh2o, tfrz
  use clm_varcon, only : denice, denh2o, tfrz

  implicit none
    
!-----------------------Argument---------- ------------------------------
  integer, INTENT(in) :: &
        lb               , & ! lower bound of array
        nl_soil          , & ! upper bound of array
        itypwat              ! land water type (0=soil, 1=urban or built-up, 2=wetland, 
                             ! 3=land ice, 4=land water bodies, 99=ocean
  real(r8), INTENT(in) :: &
        deltim           , &! time step (s)
        wtfact           , &! fraction of model area with high water table
        pondmx           , &! ponding depth (mm)
        ssi              , &! irreducible water saturation of snow
        wimp             , &! water impremeable if porosity less than wimp
        smpmin           , &! restriction for min of soil poten. (mm)
        z_soisno (lb:nl_soil)   , &! layer depth (m)
        dz_soisno(lb:nl_soil)   , &! layer thickness (m)
        zi_soisno(lb-1:nl_soil) , &! interface level below a "z" level (m)

        bsw(1:nl_soil)   , &! Clapp-Hornberger "B"
        porsl(1:nl_soil) , &! saturated volumetric soil water content(porosity)
        psi0(1:nl_soil)  , &! saturated soil suction (mm) (NEGATIVE)
        hksati(1:nl_soil), &! hydraulic conductivity at saturation (mm h2o/s)
        rootr(1:nl_soil) , &! root resistance of a layer, all layers add to 1.0

        t_soisno(lb:nl_soil), &! soil/snow skin temperature (K)
        pg_rain          , &! rainfall after removal of interception (mm h2o/s)
        sm               , &! snow melt (mm h2o/s)
        etr              , &! actual transpiration (mm h2o/s)
        qseva            , &! ground surface evaporation rate (mm h2o/s)
        qsdew            , &! ground surface dew formation (mm h2o /s) [+]
        qsubl            , &! sublimation rate from snow pack (mm h2o /s) [+]
        qfros               ! surface dew added to snow pack (mm h2o /s) [+]

  real(r8), INTENT(inout) :: &
        wice_soisno(lb:nl_soil) , &! ice lens (kg/m2)
        wliq_soisno(lb:nl_soil) , &! liquid water (kg/m2)
        zwt              , &! the depth from ground (soil) surface to water table [m]
        wa                  ! water storage in aquifer [mm]

  real(r8), INTENT(out) :: &
        rsur             , &! surface runoff (mm h2o/s)
        rnof             , &! total runoff (mm h2o/s)
        qcharge             ! groundwater recharge (positive to aquifer) [mm/s]
!                    
!-----------------------Local Variables------------------------------
!                   
  integer j                 ! loop counter

  real(r8) :: & 
  eff_porosity(1:nl_soil), &! effective porosity = porosity - vol_ice
       hk(1:nl_soil)     , &! hydraulic conductivity (mm h2o/s)
       dhkdw(1:nl_soil)  , &! d(hk)/d(vol_liq)
       dwat(1:nl_soil)   , &! change in soil water
       gwat              , &! net water input from top (mm/s)
       qinfl             , &! infiltration rate (mm h2o/s)
       rsubst            , &! subsurface runoff (mm h2o/s)
       vol_liq(1:nl_soil), &! partitial volume of liquid water in layer
       vol_ice(1:nl_soil), &! partitial volume of ice lens in layer
       icefrac(1:nl_soil), &! ice fraction (-)
       zmm (1:nl_soil)   , &! layer depth (mm)
       dzmm(1:nl_soil)   , &! layer thickness (mm)
       zimm(0:nl_soil)      ! interface level below a "z" level (mm)

  real(r8) :: err_solver, w_sum

!=======================================================================
! [1] update the liquid water within snow layer and the water onto soil
!=======================================================================

      if (lb>=1)then
         gwat = pg_rain + sm - qseva
      else
         call snowwater (lb,deltim,ssi,wimp,&
                         pg_rain,qseva,qsdew,qsubl,qfros,&
                         dz_soisno(lb:0),wice_soisno(lb:0),wliq_soisno(lb:0),gwat)
      endif

!=======================================================================
! [2] surface runoff and infiltration
!=======================================================================

  if(itypwat<=1)then   ! soil ground only

      ! For water balance check, the sum of water in soil column before the calcultion
      w_sum = sum(wliq_soisno(1:)) + sum(wice_soisno(1:)) + wa

      ! porosity of soil, partitial volume of ice and liquid
      do j = 1, nl_soil
         vol_ice(j) = min(porsl(j), wice_soisno(j)/(dz_soisno(j)*denice))
         eff_porosity(j) =  max(0.01, porsl(j)-vol_ice(j))
         vol_liq(j) = min(eff_porosity(j), wliq_soisno(j)/(dz_soisno(j)*denh2o))
         if(porsl(j) < 1.e-6)then
            icefrac(j) = 0.
         else
            icefrac(j) = min(1.,vol_ice(j)/porsl(j))
         endif
      enddo

      ! surface runoff including water table and surface staturated area

      rsur = 0.
      if (gwat > 0.) then
      call surfacerunoff (nl_soil,wtfact,wimp,bsw,porsl,psi0,hksati,&
                          z_soisno(1:),dz_soisno(1:),zi_soisno(0:),&
                          eff_porosity,icefrac,zwt,gwat,rsur)
      else
           rsur = 0.
      endif

      ! infiltration into surface soil layer 
      qinfl = gwat - rsur 

!=======================================================================
! [3] determine the change of soil water
!=======================================================================

      ! convert length units from m to mm
      zmm(1:) = z_soisno(1:)*1000.
      dzmm(1:) = dz_soisno(1:)*1000.
      zimm(0:) = zi_soisno(0:)*1000.

      call soilwater(nl_soil,deltim,wimp,smpmin,&
                     qinfl,etr,z_soisno(1:),dz_soisno(1:),zi_soisno(0:),&
                     t_soisno(1:),vol_liq,vol_ice,icefrac,eff_porosity,&
                     porsl,hksati,bsw,psi0,rootr,&
                     zwt,dwat,hk,dhkdw,qcharge)

      ! update the mass of liquid water
      do j= 1, nl_soil
         wliq_soisno(j) = max(0.,wliq_soisno(j)+dwat(j)*dzmm(j))
      enddo

!=======================================================================
! [4] subsurface runoff and the corrections
!=======================================================================

      call subsurfacerunoff (nl_soil,deltim,pondmx,&
                             eff_porosity,icefrac,dz_soisno(1:),zi_soisno(0:),&
                             wice_soisno(1:),wliq_soisno(1:),&
                             porsl,psi0,bsw,hk,zwt,wa,&
                             qcharge,rsubst)

      ! total runoff (mm/s)
      rnof = rsubst + rsur                
 
      ! Renew the ice and liquid mass due to condensation
      if(lb >= 1)then
         ! make consistent with how evap_grnd removed in infiltration
         wliq_soisno(1) = max(0., wliq_soisno(1) + qsdew * deltim)
         wice_soisno(1) = max(0., wice_soisno(1) + (qfros-qsubl) * deltim)
      end if


      if(lb >= 1)then
         err_solver = (sum(wliq_soisno(1:))+sum(wice_soisno(1:))+wa) - w_sum &
                    - (gwat+qsdew+qfros-qsubl-etr-rnof)*deltim
      else
         err_solver = (sum(wliq_soisno(1:))+sum(wice_soisno(1:))+wa) - w_sum &
                    - (gwat-etr-rnof)*deltim
      endif

!#if(defined CLMDEBUG)
     if(abs(err_solver) > 1.e-3)then
        write(6,*) 'Warning: water balance violation after all soilwater calculation', err_solver
     endif
!#endif


!=======================================================================
! [6] assumed hydrological scheme for the wetland and glacier
!=======================================================================

  else                          
      if(itypwat==2)then        ! WETLAND
         qinfl = 0.
         rsur = max(0.,gwat)
         rsubst = 0.
         rnof = 0.  
         do j = 1, nl_soil
            if(t_soisno(j)>=tfrz)then
               wice_soisno(j) = 0.0
               wliq_soisno(j) = 1000.*dz_soisno(j)
            else
               wice_soisno(j) = 1000.*dz_soisno(j)
               wliq_soisno(j) = 0.0
            endif
         enddo
      endif
      if(itypwat==3)then        ! LAND ICE
         rsur = max(0.0,gwat)
         qinfl = 0.
         rsubst = 0.
         rnof = rsur
         wice_soisno(1:nl_soil) = 1000.*dz_soisno(1:nl_soil) 
         wliq_soisno(1:nl_soil) = 0.0
      endif

      wa = 4800.
      zwt = 0.
      qcharge = 0.

  endif

!-----------------------------------------------------------------------

  end subroutine WATER



  subroutine snowwater (lb,deltim,ssi,wimp, &
                        pg_rain,qseva,qsdew,qsubl,qfros, &
                        dz_soisno,wice_soisno,wliq_soisno,qout_snowb)

!-----------------------------------------------------------------------
! Original author : Yongjiu Dai, /09/1999; /04/2014
!
! Water flow wihtin snow is computed by an explicit and non-physical based scheme,
! which permits a part of liquid water over the holding capacity (a tentative value
! is used, i.e., equal to 0.033*porosity) to percolate into the underlying layer,
! except the case of that the porosity of one of the two neighboring layers is
! less than 0.05, the zero flow is assumed. The water flow out of the bottom
! snow pack will participate as the input of the soil water and runoff.
!
!-----------------------------------------------------------------------

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : denice, denh2o  ! physical constant
  use clm_varcon, only : denice, denh2o  ! physical constant
  implicit none

!----------------------- dummy argument --------------------------------
  integer, INTENT(in) :: &
        lb          ! lower bound of array

  real(r8), INTENT(in) :: &
        deltim,    &! seconds in a time step (s)
        ssi,       &! irreducible water saturation of snow
        wimp,      &! water impremeable if porosity less than wimp
        dz_soisno(lb:0),  &! layer thickness (m)

        pg_rain,   &! rainfall after removal of interception (mm h2o/s)
        qseva,     &! ground surface evaporation rate (mm h2o/s)
        qsdew,     &! ground surface dew formation (mm h2o /s) [+]
        qsubl,     &! sublimation rate from snow pack (mm h2o /s) [+]
        qfros       ! surface dew added to snow pack (mm h2o /s) [+]

  real(r8), INTENT(inout) :: &
        wice_soisno(lb:0),&! ice lens (kg/m2)
        wliq_soisno(lb:0)  ! liquid water (kg/m2)
  
  real(r8), INTENT(out) :: &
        qout_snowb  ! rate of water out of snow bottom (mm/s)

!----------------------- local variables --------------------------------
  integer j         ! k do loop/array indices

  real(r8) :: &
       qin,        &! water flow into the elmement (mm/s)
       qout,       &! water flow out of the elmement (mm/s)
       zwice,      &! the sum of ice mass of snow cover (kg/m2)
       wgdif,      &! ice mass after minus sublimation
    vol_liq(lb:0), &! partitial volume of liquid water in layer
    vol_ice(lb:0), &! partitial volume of ice lens in layer
 eff_porosity(lb:0) ! effective porosity = porosity - vol_ice

!=======================================================================
! renew the mass of ice lens (wice_soisno) and liquid (wliq_soisno) in the surface snow layer, 
! resulted by sublimation (frost) / evaporation (condense)

      wgdif = wice_soisno(lb) + (qfros - qsubl)*deltim
      wice_soisno(lb) = wgdif
      if(wgdif < 0.)then
         wice_soisno(lb) = 0.
         wliq_soisno(lb) = wliq_soisno(lb) + wgdif
      endif
      wliq_soisno(lb) = wliq_soisno(lb) + (pg_rain + qsdew - qseva)*deltim
      wliq_soisno(lb) = max(0., wliq_soisno(lb))

! Porosity and partitial volume
      do j = lb, 0
         vol_ice(j) = min(1., wice_soisno(j)/(dz_soisno(j)*denice))
         eff_porosity(j) = max(0.01, 1. - vol_ice(j))
         vol_liq(j) = min(eff_porosity(j), wliq_soisno(j)/(dz_soisno(j)*denh2o))
      enddo

! Capillary force within snow could be two or more orders of magnitude
! less than those of gravity, this term may be ignored.
! Here we could keep the garavity term only. The genernal expression 
! for water flow is "K * ss**3", however, no effective paramterization 
! for "K". Thus, a very simple treatment (not physical based) is introduced: 
! when the liquid water of layer exceeds the layer's holding 
! capacity, the excess meltwater adds to the underlying neighbor layer. 

      qin = 0.
      do j= lb, 0
         wliq_soisno(j) = wliq_soisno(j) + qin

         if(j <= -1)then
         ! no runoff over snow surface, just ponding on surface
           if(eff_porosity(j)<wimp .OR. eff_porosity(j+1)<wimp)then
             qout = 0.
           else
             qout = max(0.,(vol_liq(j)-ssi*eff_porosity(j))*dz_soisno(j))
             qout = min(qout,(1.-vol_ice(j+1)-vol_liq(j+1))*dz_soisno(j+1))
           endif
         else
           qout = max(0.,(vol_liq(j)-ssi*eff_porosity(j))*dz_soisno(j))
         endif

         qout    = qout*1000.
         wliq_soisno(j) = wliq_soisno(j) - qout
         qin     = qout

      enddo

      qout_snowb = qout/deltim

  end subroutine snowwater



  subroutine surfacerunoff (nl_soil,wtfact,wimp,bsw,porsl,psi0,hksati,&
                            z_soisno,dz_soisno,zi_soisno,&
                            eff_porosity,icefrac,zwt,gwat,rsur)

!=======================================================================
! the original code was provide by Robert E. Dickinson based on following clues:
! a water table level determination level added including highland and
! lowland levels and fractional area of wetland (water table above the surface.
! Runoff is parametrized from the lowlands in terms of precip incident on
! wet areas and a base flow, where these are estimated using ideas from TOPMODEL.
!
! Author : Yongjiu Dai, 07/29/2002, Guoyue Niu, 06/2012
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

  implicit none

!-----------------------Arguments---------------------------------------

  integer, INTENT(in) :: nl_soil   ! number of soil layers
  real(r8), INTENT(in) :: &
        wtfact,        &! fraction of model area with high water table
        wimp,          &! water impremeable if porosity less than wimp
       bsw(1:nl_soil), &! Clapp-Hornberger "B"
     porsl(1:nl_soil), &! saturated volumetric soil water content(porosity)
      psi0(1:nl_soil), &! saturated soil suction (mm) (NEGATIVE)
    hksati(1:nl_soil), &! hydraulic conductivity at saturation (mm h2o/s)
  z_soisno(1:nl_soil), &! layer depth (m)
 dz_soisno(1:nl_soil), &! layer thickness (m)
 zi_soisno(0:nl_soil), &! interface level below a "z" level (m)
 eff_porosity(1:nl_soil), &! effective porosity = porosity - vol_ice
   icefrac(1:nl_soil), &! ice fraction (-)
        gwat,          &! net water input from top
         zwt            ! the depth from ground (soil) surface to water table [m]

  real(r8), INTENT(out) :: rsur    ! surface runoff (mm h2o/s)

!-----------------------Local Variables---------------------------------

  real(r8) qinmax       ! maximum infiltration capability
  real(r8) fsat         ! fractional area with water table at surface

  real(r8), parameter :: fff = 0.5   ! runoff decay factor (m-1)

!-----------------------End Variable List-------------------------------

!  fraction of saturated area
      fsat = wtfact*min(1.0,exp(-0.5*fff*zwt))

! Maximum infiltration capacity
      qinmax = minval(10.**(-6.0*icefrac(1:3))*hksati(1:3))
      if(eff_porosity(1)<wimp) qinmax = 0.

! Surface runoff
      rsur = fsat*max(0.0,gwat) + (1.-fsat)*max(0.,gwat-qinmax)


  end subroutine surfacerunoff



  subroutine soilwater(nl_soil,deltim,wimp,smpmin,&
                       qinfl,etr,z_soisno,dz_soisno,zi_soisno,&
                       t_soisno,vol_liq,vol_ice,icefrac,eff_porosity,&
                       porsl,hksati,bsw,psi0,rootr,&
                       zwt,dwat,hk,dhkdw,qcharge)

!-----------------------------------------------------------------------
! Original author : Yongjiu Dai, 09/1999, 04/2014
!
! some new parameterization are added, which are based on CLM4.5
! 
! Soil moisture is predicted from a 10-layer model (as with soil
! temperature), in which the vertical soil moisture transport is governed
! by infiltration, runoff, gradient diffusion, gravity, and root
! extraction through canopy transpiration. The net water applied to the
! surface layer is the snowmelt plus precipitation plus the throughfall
! of canopy dew minus surface runoff and evaporation.
!
! The vertical water flow in an unsaturated porous media is described by
! Darcy's law, and the hydraulic conductivity and the soil negative
! potential vary with soil water content and soil texture based on the work
! of Clapp and Hornberger (1978) and Cosby et al. (1984). The equation is
! integrated over the layer thickness, in which the time rate of change in
! water mass must equal the net flow across the bounding interface, plus the
! rate of internal source or sink. The terms of water flow across the layer
! interfaces are linearly expanded by using first-order Taylor expansion.
! The equations result in a tridiagonal system equation.
!
! Note: length units here are all millimeter
! (in temperature subroutine uses same soil layer
! structure required but lengths are m)
!
! Richards equation:
!
! d wat      d     d wat d psi
! ----- = - -- [ k(----- ----- - 1) ] + S
!   dt      dz       dz  d wat
!
! where: wat = volume of water per volume of soil (mm**3/mm**3)
! psi = soil matrix potential (mm)
! dt  = time step (s)
! z   = depth (mm)
! dz  = thickness (mm)
! qin = inflow at top (mm h2o /s)
! qout= outflow at bottom (mm h2o /s)
! s   = source/sink flux (mm h2o /s)
! k   = hydraulic conductivity (mm h2o /s)
!
!                       d qin                  d qin
! qin[n+1] = qin[n] +  --------  d wat(j-1) + --------- d wat(j)
!                       d wat(j-1)             d wat(j)
!                ==================|=================
!                                  < qin
!
!                 d wat(j)/dt * dz = qin[n+1] - qout[n+1] + S(j)
!
!                                  > qout
!                ==================|=================
!                        d qout               d qout
! qout[n+1] = qout[n] + --------- d wat(j) + --------- d wat(j+1)
!                        d wat(j)             d wat(j+1)
!
!
! Solution: linearize k and psi about d wat and use tridiagonal
! system of equations to solve for d wat,
! where for layer j
!
!
! r_j = a_j [d wat_j-1] + b_j [d wat_j] + c_j [d wat_j+1]
!
!-----------------------------------------------------------------------
! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,    use PhysicalConstants , only : grav,hfus,tfrz,denh2o,denice
    use clm_varcon , only : grav,hfus,tfrz,denh2o,denice

    IMPLICIT NONE

    integer, INTENT(in) :: nl_soil  ! number of soil layers
    real(r8), INTENT(in) :: deltim  ! land model time step (sec)
    real(r8), INTENT(in) :: wimp    ! water impremeable if porosity less than wimp
    real(r8), INTENT(in) :: smpmin  ! restriction for min of soil potential (mm)

    real(r8), INTENT(in) :: qinfl   ! infiltration (mm H2O /s)
    real(r8), INTENT(in) :: etr     ! vegetation transpiration (mm H2O/s) (+ = to atm)

    real(r8), INTENT(in) :: z_soisno (1:nl_soil) ! layer depth (m)
    real(r8), INTENT(in) :: dz_soisno(1:nl_soil) ! layer thickness (m)
    real(r8), INTENT(in) :: zi_soisno(0:nl_soil) ! interface level below a "z" level (m)

    real(r8), INTENT(in) :: t_soisno (1:nl_soil) ! soil temperature (Kelvin)
    real(r8), INTENT(in) :: vol_liq  (1:nl_soil) ! liquid volumetric water content
    real(r8), INTENT(in) :: vol_ice  (1:nl_soil) ! ice volumetric water content
    real(r8), INTENT(in) :: icefrac  (1:nl_soil)
    real(r8), INTENT(in) :: eff_porosity(1:nl_soil) ! effective porosity = porosity - vol_ice

    real(r8), INTENT(in) :: porsl  (1:nl_soil) ! volumetric soil water at saturation (porosity)
    real(r8), INTENT(in) :: hksati (1:nl_soil) ! hydraulic conductivity at saturation (mm H2O /s)
    real(r8), INTENT(in) :: bsw    (1:nl_soil) ! Clapp and Hornberger "b"
    real(r8), INTENT(in) :: psi0   (1:nl_soil) ! minimum soil suction (mm) [-]
    real(r8), INTENT(in) :: rootr  (1:nl_soil) ! effective fraction of roots in each soil layer
    real(r8), INTENT(in) :: zwt                ! the depth from ground (soil) surface to water table [m]

    real(r8), intent(out) :: dwat(1:nl_soil)   ! change of soil water [m3/m3]
    real(r8), intent(out) :: hk(1:nl_soil)     ! hydraulic conductivity [mm h2o/s]
    real(r8), intent(out) :: dhkdw(1:nl_soil)  ! d(hk)/d(vol_liq)
    real(r8), INTENT(out) :: qcharge           ! aquifer recharge rate (positive to aquifer) (mm/s)
!
! local arguments
!
    integer  :: j                   ! do loop indices
    real(r8) :: amx(1:nl_soil+1)    ! "a" left off diagonal of tridiagonal matrix
    real(r8) :: bmx(1:nl_soil+1)    ! "b" diagonal column for tridiagonal matrix
    real(r8) :: cmx(1:nl_soil+1)    ! "c" right off diagonal tridiagonal matrix
    real(r8) :: rmx(1:nl_soil+1)    ! "r" forcing term of tridiagonal matrix
    real(r8) :: zmm(1:nl_soil+1)    ! layer depth [mm]
    real(r8) :: dzmm(1:nl_soil+1)   ! layer thickness [mm]
    real(r8) :: zimm(0:nl_soil+1)   ! layer interface depth [mm]
    real(r8) :: den                 ! used in calculating qin, qout
    real(r8) :: num                 ! used in calculating qin, qout
    real(r8) :: qin(1:nl_soil+1)    ! flux of water into soil layer [mm h2o/s]
    real(r8) :: qout(1:nl_soil+1)   ! flux of water out of soil layer [mm h2o/s]
    real(r8) :: dqidw0(1:nl_soil+1) ! d(qin)/d(vol_liq(j-1))
    real(r8) :: dqidw1(1:nl_soil+1) ! d(qin)/d(vol_liq(j))
    real(r8) :: dqodw1(1:nl_soil+1) ! d(qout)/d(vol_liq(j))
    real(r8) :: dqodw2(1:nl_soil+1) ! d(qout)/d(vol_liq(j+1))
    real(r8) :: dsmpdw(1:nl_soil+1) ! d(smp)/d(vol_liq)
    real(r8) :: s_node              ! soil wetness
    real(r8) :: s1                  ! "s" at interface of layer
    real(r8) :: s2                  ! k*s**(2b+2)
    real(r8) :: smp(1:nl_soil)      ! soil matrix potential [mm]
    integer  :: jwt                 ! index of the soil layer right above the water table (-)
    real(r8) :: dzq                 ! used in calculating qin, qout (difference in equilbirium matric potential)
    real(r8) :: zq(1:nl_soil+1)     ! equilibrium matric potential for each layer [mm]
    real(r8) :: vol_eq(1:nl_soil+1) ! equilibrium volumetric water content
    real(r8) :: tempi               ! temp variable for calculating vol_eq
    real(r8) :: temp0               ! temp variable for calculating vol_eq
    real(r8) :: voleq1              ! temp variable for calculating vol_eq
    real(r8) :: zwtmm               ! water table depth [mm]
    real(r8) :: imped(1:nl_soil)             
    real(r8) :: vwc_zwt
    real(r8) :: dwat2(1:nl_soil+1)
    real(r8) :: smp1,dsmpdw1,wh,wh_zwt,ka

    real(r8), parameter :: e_ice=6.0      !soil ice impedance factor
!-----------------------------------------------------------------------

    !compute jwt index
    ! The layer index of the first unsaturated layer, 
    ! i.e., the layer right above the water table

    jwt = nl_soil
    ! allow jwt to equal zero when zwt is in top layer
    do j = 1, nl_soil
       if(zwt <= zi_soisno(j)) then
          jwt = j-1
          exit
       end if
    enddo

   ! compute vwc at water table depth (mainly for case when t < tfrz)
   ! this will only be used when zwt is below the soil column
    vwc_zwt = porsl(nl_soil)
    if(t_soisno(nl_soil) < tfrz) then
       vwc_zwt = vol_liq(nl_soil)
       if(zwt <= zi_soisno(nl_soil)) then
          smp1 = hfus*(t_soisno(nl_soil)-tfrz)/(grav*t_soisno(nl_soil))*1000.  !(mm)
          smp1 = min(psi0(nl_soil),smp1)
          vwc_zwt = porsl(nl_soil)*(smp1/psi0(nl_soil))**(-1./bsw(nl_soil))
          ! for temperatures close to tfrz, limit vwc to total water content 
          vwc_zwt = min(vwc_zwt, 0.5*(porsl(nl_soil) + vol_liq(nl_soil)) )
       endif
    endif


    ! Because the depths in this routine are in mm, use local
    ! variable arrays instead of pointers
    do j = 1, nl_soil
       zmm(j) = z_soisno(j)*1000.
       dzmm(j) = dz_soisno(j)*1000.
       zimm(j) = zi_soisno(j)*1000.
    end do

    zimm(0) = 0.0
    zwtmm = zwt*1000.


    ! calculate the equilibrium water content based on the water table depth
            
    do j=1,nl_soil 
       if ((zwtmm <= zimm(j-1))) then 
          vol_eq(j) = porsl(j)
            
       ! use the weighted average from the saturated part (depth > wtd) and 
       ! the equilibrium solution for the rest of the layer

       else if ((zwtmm < zimm(j)) .and. (zwtmm > zimm(j-1))) then
          tempi = 1.0
          temp0 = ((psi0(j)-zwtmm+zimm(j-1))/psi0(j))**(1.-1./bsw(j))
          voleq1 = psi0(j)*porsl(j)/(1.-1./bsw(j))/(zwtmm-zimm(j-1))*(tempi-temp0)
          vol_eq(j) = (voleq1*(zwtmm-zimm(j-1)) + porsl(j)*(zimm(j)-zwtmm))/(zimm(j)-zimm(j-1))
          vol_eq(j) = min(porsl(j),vol_eq(j))
          vol_eq(j) = max(vol_eq(j),0.0)
       else
          tempi = (((psi0(j)-zwtmm+zimm(j))/psi0(j)))**(1.-1./bsw(j))
          temp0 = (((psi0(j)-zwtmm+zimm(j-1))/psi0(j)))**(1.-1./bsw(j))
          vol_eq(j) = psi0(j)*porsl(j)/(1.-1./bsw(j))/(zimm(j)-zimm(j-1))*(tempi-temp0)
          vol_eq(j) = max(vol_eq(j),0.0)
          vol_eq(j) = min(porsl(j),vol_eq(j))
       endif
       zq(j) = psi0(j)*(max(vol_eq(j)/max(porsl(j),vol_eq(j)),0.01))**(-bsw(j))
       zq(j) = max(smpmin,zq(j))
    end do

    ! If water table is below soil column calculate zq for the 11th layer
    j = nl_soil
    if(jwt == nl_soil) then 
       tempi = 1.
       temp0 = ((psi0(j)-zwtmm+zimm(j))/psi0(j))**(1.-1./bsw(j))
       vol_eq(j+1) = psi0(j)*porsl(j)/(1.-1./bsw(j))/(zwtmm-zimm(j))*(tempi-temp0)
       vol_eq(j+1) = max(vol_eq(j+1),0.0)
       vol_eq(j+1) = min(porsl(j),vol_eq(j+1))
       zq(j+1) = psi0(j)*(max(vol_eq(j+1)/max(porsl(j),vol_eq(j+1)),0.01))**(-bsw(j))
       zq(j+1) = max(smpmin,zq(j+1))
    end if

    ! Hydraulic conductivity and soil matric potential and their derivatives

    do j = 1, nl_soil
       if((eff_porosity(j) < wimp) &
               .OR. (eff_porosity(min(nl_soil,j+1)) < wimp) &
               .OR. (vol_liq(j) <= 1.e-3))then
          imped(j) = 0.
          hk(j) = 0.
          dhkdw(j) = 0.
       else
           s1 = 0.5*(vol_liq(j) + vol_liq(min(nl_soil,j+1))) / &
               (0.5*(porsl(j)+porsl(min(nl_soil,j+1))))
           s1 = min(1.,s1)
           s2 = hksati(j)*s1**(2.*bsw(j)+2.)

           ! replace fracice with impedance factor
           imped(j)=10.**(-e_ice*(0.5*(icefrac(j)+icefrac(min(nl_soil,j+1)))))

           hk(j) = imped(j)*s1*s2
           dhkdw(j) = imped(j)*(2.*bsw(j)+3.)*s2/((porsl(j)+porsl(min(nl_soil,j+1)))/2.)
       endif
    end do

    do j = 1, nl_soil
       ! compute matric potential and derivative based on liquid water content only
       if(t_soisno(j)>tfrz) then
          if(porsl(j)<1.e-6)then     ! bed rock
             s_node = 0.001
             smp(j) = psi0(j)
             dsmpdw(j) = 0.
          else
             s_node = max(vol_liq(j)/porsl(j),0.01)
             s_node = min(1.0,s_node)
             smp(j) = psi0(j)*s_node**(-bsw(j))
             smp(j) = max(smpmin,smp(j))
             dsmpdw(j) = -bsw(j)*smp(j)/(s_node*porsl(j))
          endif
       else
          ! when ice is present, the matric potential is only related to temperature
          ! by (Fuchs et al., 1978: Soil Sci. Soc. Amer. J. 42(3):379-385)
          ! Unit 1 Joule = 1 (kg m2/s2), J/kg /(m/s2) ==> m ==> 1e3 mm
          smp(j) = 1.e3 * 0.3336e6/9.80616*(t_soisno(j)-tfrz)/t_soisno(j)
          smp(j) = max(smpmin, smp(j))        ! Limit soil suction
          dsmpdw(j) = 0.
       endif
    end do

    ! aquifer (11th) layer
    zmm(nl_soil+1) = 0.5*(1000.*zwt + zmm(nl_soil))
    if(jwt < nl_soil) then
      dzmm(nl_soil+1) = dzmm(nl_soil)
    else
      dzmm(nl_soil+1) = (1000.*zwt - zmm(nl_soil))
    end if

    ! Set up r, a, b, and c vectors for tridiagonal solution

    ! Node j=1 (top)

    j = 1
    qin(j)    = qinfl
    den       = (zmm(j+1)-zmm(j))
    dzq       = (zq(j+1)-zq(j))
    num       = (smp(j+1)-smp(j)) - dzq
    qout(j)   = -hk(j)*num/den
    dqodw1(j) = -(-hk(j)*dsmpdw(j)   + num*dhkdw(j))/den
    dqodw2(j) = -( hk(j)*dsmpdw(j+1) + num*dhkdw(j))/den

    amx(j)    =  0.
    bmx(j)    =  dzmm(j)/deltim + dqodw1(j)
    cmx(j)    =  dqodw2(j)
    rmx(j)    =  qin(j) - qout(j) - etr * rootr(j)


    ! Nodes j=2 to j=nl_soil-1

    do j = 2, nl_soil - 1
       den       = (zmm(j) - zmm(j-1))
       dzq       = (zq(j)-zq(j-1))
       num       = (smp(j)-smp(j-1)) - dzq
       qin(j)    = -hk(j-1)*num/den
       dqidw0(j) = -(-hk(j-1)*dsmpdw(j-1) + num*dhkdw(j-1))/den
       dqidw1(j) = -( hk(j-1)*dsmpdw(j)   + num*dhkdw(j-1))/den

       den       = (zmm(j+1)-zmm(j))
       dzq       = (zq(j+1)-zq(j))
       num       = (smp(j+1)-smp(j)) - dzq
       qout(j)   = -hk(j)*num/den
       dqodw1(j) = -(-hk(j)*dsmpdw(j)   + num*dhkdw(j))/den
       dqodw2(j) = -( hk(j)*dsmpdw(j+1) + num*dhkdw(j))/den

       amx(j)    = -dqidw0(j)
       bmx(j)    =  dzmm(j)/deltim - dqidw1(j) + dqodw1(j)
       cmx(j)    =  dqodw2(j)
       rmx(j)    =  qin(j) - qout(j) - etr*rootr(j)
    end do

    ! Node j=nl_soil (bottom)

    j = nl_soil
    if(j > jwt) then ! water table is in soil column
      den       = (zmm(j)-zmm(j-1))
      dzq       = (zq(j)-zq(j-1))
      num       = (smp(j)-smp(j-1)) - dzq
      qin(j)    = -hk(j-1)*num/den
      dqidw0(j) = -(-hk(j-1)*dsmpdw(j-1) + num*dhkdw(j-1))/den
      dqidw1(j) = -( hk(j-1)*dsmpdw(j)   + num*dhkdw(j-1))/den

      qout(j)   = 0.
      dqodw1(j) = 0.
      dqodw2(j) = 0.

      amx(j)    = -dqidw0(j)
      bmx(j)    =  dzmm(j)/deltim - dqidw1(j) + dqodw1(j)
      cmx(j)    =  0.
      rmx(j)    =  qin(j) - qout(j) - etr*rootr(j)

      ! next set up aquifer layer; hydrologically inactive
      rmx(j+1)  = 0.
      amx(j+1)  = 0.
      bmx(j+1)  = dzmm(j+1)/deltim
      cmx(j+1)  = 0.

    else ! water table is below soil column

      ! compute aquifer soil moisture as average of layer 10 and saturation
      if(porsl(j)<1.e-6)then
         s_node = 0.001
         smp1 = psi0(j)*s_node**(-bsw(j))
         dsmpdw1 = 0.
      else
         s_node = max(0.5*((vwc_zwt+vol_liq(j))/porsl(j)), 0.01)
         s_node = min(1.0, s_node)

         ! compute smp for aquifer layer
         smp1 = psi0(j)*s_node**(-bsw(j))
         smp1 = max(smpmin, smp1)

         ! compute dsmpdw for aquifer layer
         dsmpdw1 = -bsw(j)*smp1/(s_node*porsl(j))
      endif

      ! first set up bottom layer of soil column
      den       = (zmm(j) - zmm(j-1))
      dzq       = (zq(j)-zq(j-1))
      num       = (smp(j)-smp(j-1)) - dzq
      qin(j)    = -hk(j-1)*num/den
      dqidw0(j) = -(-hk(j-1)*dsmpdw(j-1) + num*dhkdw(j-1))/den
      dqidw1(j) = -( hk(j-1)*dsmpdw(j)   + num*dhkdw(j-1))/den

      den       = (zmm(j+1)-zmm(j))
      dzq       = (zq(j+1)-zq(j))
      num       = (smp1-smp(j)) - dzq
      qout(j)   = -hk(j)*num/den
      dqodw1(j) = -(-hk(j)*dsmpdw(j) + num*dhkdw(j))/den
      dqodw2(j) = -( hk(j)*dsmpdw1   + num*dhkdw(j))/den

      rmx(j)    =  qin(j) - qout(j) - etr*rootr(j)
      amx(j)    = -dqidw0(j)
      bmx(j)    =  dzmm(j)/deltim - dqidw1(j) + dqodw1(j)
      cmx(j)    =  dqodw2(j)

      ! next set up aquifer layer; den/num unchanged, qin=qout
      qin(j+1)    = qout(j)
      dqidw0(j+1) = -(-hk(j)*dsmpdw(j) + num*dhkdw(j))/den
      dqidw1(j+1) = -( hk(j)*dsmpdw1   + num*dhkdw(j))/den
      qout(j+1)   =  0.  ! zero-flow bottom boundary condition
      dqodw1(j+1) =  0.  ! zero-flow bottom boundary condition

      rmx(j+1)    =  qin(j+1) - qout(j+1)
      amx(j+1)    = -dqidw0(j+1)
      bmx(j+1)    =  dzmm(j+1)/deltim - dqidw1(j+1) + dqodw1(j+1)
      cmx(j+1)    =  0.
    endif

    ! Solve for dwat

    call tridia (nl_soil+1, amx, bmx, cmx, rmx, dwat2 )


    ! set dwat
    do j = 1, nl_soil
       dwat(j)=dwat2(j)
    end do

!*--------------------------------------------------------------------
!*  ! calculate qcharge for case jwt < nl_soil
!*  if(jwt < nl_soil) then
!*     wh_zwt = 0.
!*        
!*     ! Recharge rate qcharge to groundwater (positive to aquifer)
!*     if(porsl(jwt+1)<1.e-6)then
!*        s_node = 0.001
!*        s1 = s_node
!*     else
!*        s_node = max(vol_liq(jwt+1)/porsl(jwt+1),0.01)
!*        s1 = min(1.,s_node)
!*     endif
!*
!*     ka = imped(jwt+1)*hksati(jwt+1)*s1**(2.*bsw(jwt+1)+3.)
!*        
!*     ! Recharge rate qcharge to groundwater (positive to aquifer)
!*     smp1 = max(smpmin, smp(max(1,jwt)))
!*     wh = smp1 - zq(max(1,jwt))
!*        
!*     if(jwt == 0) then
!*        qcharge = -ka*(wh_zwt-wh)/((zwt+1.e-3)*1000.)
!*     else
!*        qcharge = -ka*(wh_zwt-wh)/((zwt-z_soisno(jwt))*1000.*2.0)
!*     endif
!*        
!*     ! To limit qcharge (for the first several timesteps)
!*     qcharge = max(-10.0/deltim,qcharge)
!*     qcharge = min( 10.0/deltim,qcharge)
!*  else
!*     ! if water table is below soil column, compute qcharge from dwat2(11)
!*     qcharge = dwat2(nl_soil+1)*dzmm(nl_soil+1)/deltim
!*  endif
!*--------------------------------------------------------------------

    ! Recharge rate qcharge to groundwater (positive to aquifer)
    qcharge = qout(nl_soil) + dqodw1(nl_soil)*dwat2(nl_soil) &
                            + dqodw2(nl_soil)*dwat2(nl_soil+1)


  end subroutine soilwater



  subroutine subsurfacerunoff(nl_soil,deltim,pondmx,&
                              eff_porosity,icefrac,&
                              dz_soisno,zi_soisno,wice_soisno,wliq_soisno,&
                              porsl,psi0,bsw,hk,zwt,wa,&
                              qcharge,rsubst)

! -------------------------------------------------------------------------


! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 
!shixl,    use PhysicalConstants, only : tfrz
    use clm_varcon, only : tfrz
!
! ARGUMENTS:
    IMPLICIT NONE

    integer, INTENT(in) :: nl_soil       !
    real(r8), INTENT(in) :: deltim       ! land model time step (sec)
    real(r8), INTENT(in) :: pondmx       !

    real(r8), INTENT(in) :: eff_porosity(1:nl_soil) ! effective porosity = porosity - vol_ice
    real(r8), INTENT(in) :: icefrac(1:nl_soil)      ! ice fraction (-)

    real(r8), INTENT(in) :: dz_soisno  (1:nl_soil)  ! layer depth (m)
    real(r8), INTENT(in) :: zi_soisno  (0:nl_soil)  ! interface level below a "z" level (m)
    real(r8), INTENT(inout) :: wice_soisno(1:nl_soil)  ! ice lens (kg/m2)
    real(r8), INTENT(inout) :: wliq_soisno(1:nl_soil)  ! liquid water (kg/m2)

    real(r8), INTENT(in) :: porsl(1:nl_soil)        ! volumetric soil water at saturation (porosity)
    real(r8), INTENT(in) :: psi0(1:nl_soil)         ! minimum soil suction (mm) [-]
    real(r8), INTENT(in) :: bsw(1:nl_soil)          ! Clapp and Hornberger "b"
    real(r8), INTENT(in) :: hk(1:nl_soil)

    real(r8), INTENT(inout) :: zwt       ! the depth from ground (soil) surface to water table [m]
    real(r8), INTENT(inout) :: wa        ! water in the unconfined aquifer (mm)
    real(r8), INTENT(in)    :: qcharge   ! aquifer recharge rate (positive to aquifer) (mm/s)
    real(r8), INTENT(out)   :: rsubst    ! drainage drainage (positive = out of soil column) (mm H2O /s)

!
! LOCAL ARGUMENTS
!

    integer  :: j                ! indices
    integer  :: jwt              ! index of the soil layer right above the water table (-)
    real(r8) :: xs               ! water needed to bring soil moisture to watmin (mm)
    real(r8) :: dzmm(1:nl_soil)  ! layer thickness (mm)
    real(r8) :: xsi              ! excess soil water above saturation at layer i (mm)
    real(r8) :: xsia             ! available pore space at layer i (mm)
    real(r8) :: xs1              ! excess soil water above saturation at layer 1 (mm)
    real(r8) :: ws               ! summation of pore space of layers below water table (mm)
    real(r8) :: s_node           ! soil wetness (-)
    real(r8) :: available_wliq_soisno     ! available soil liquid water in a layer
    real(r8) :: qcharge_tot      !
    real(r8) :: qcharge_layer    !
    real(r8) :: drainage         !
    real(r8) :: drainage_tot     !
    real(r8) :: drainage_layer   !
    real(r8) :: s_y              !
    real(r8) :: rous             ! specific yield [-]

    real(r8) :: wt
    real(r8) :: wtsub 
    real(r8) :: dzsum
    real(r8) :: icefracsum
    real(r8) :: fracice_rsub
    real(r8) :: imped


    real(r8), parameter :: watmin = 0.01  ! Limit irreduciable wrapping liquid water 
                                          ! a tunable constant
    real(r8), parameter :: rsbmx  = 5.0   ! baseflow coefficient [mm/s]
    real(r8), parameter :: timean = 10.5  ! global mean topographic index


! -------------------------------------------------------------------------

!   ! Convert layer thicknesses from m to mm
 
    do j = 1,nl_soil
       dzmm(j) = dz_soisno(j)*1000.
    end do
 
!   ! The layer index of the first unsaturated layer, 
!   ! i.e., the layer right above the water table

    jwt = nl_soil
    ! allow jwt to equal zero when zwt is in top layer
    do j = 1, nl_soil
       if(zwt <= zi_soisno(j)) then
          jwt = j-1
          exit
       end if
    enddo
 
!============================== QCHARGE =========================================
! Water table changes due to qcharge
! use analytical expression for aquifer specific yield
    rous = porsl(nl_soil)*(1.-(1.-1.e3*zwt/psi0(nl_soil))**(-1./bsw(nl_soil)))
    rous = max(rous,0.02)
!
!---------------------------------------
! water table is below the soil column
    if(jwt == nl_soil) then             
       wa = wa + qcharge*deltim
       zwt = max(0.,zwt - (qcharge*deltim)/1000./rous)
    else                                
       ! water table within soil layers 1-9
       ! try to raise water table to account for qcharge
 
       qcharge_tot = qcharge * deltim
 
       if(qcharge_tot > 0.) then ! rising water table
          do j = jwt+1, 1,-1
             ! use analytical expression for specific yield
 
             s_y = porsl(j) * (1.-(1.-1.e3*zwt/psi0(j))**(-1./bsw(j)))
             s_y=max(s_y,0.02)
 
             qcharge_layer = min(qcharge_tot,(s_y*(zwt-zi_soisno(j-1))*1.e3))
             qcharge_layer = max(qcharge_layer,0.)
 
             zwt = max(0.,zwt - qcharge_layer/s_y/1000.)
             
             qcharge_tot = qcharge_tot - qcharge_layer
             if (qcharge_tot <= 0.) exit
          enddo
       else ! deepening water table (negative qcharge)
          do j = jwt+1, nl_soil
             ! use analytical expression for specific yield
             s_y = porsl(j) * (1.-(1.-1.e3*zwt/psi0(j))**(-1./bsw(j)))
             s_y=max(s_y,0.02)
             qcharge_layer = max(qcharge_tot,-(s_y*(zi_soisno(j) - zwt)*1.e3))
             qcharge_layer = min(qcharge_layer,0.)
             qcharge_tot = qcharge_tot - qcharge_layer
 
             if (qcharge_tot >= 0.) then 
                zwt = max(0.,zwt - qcharge_layer/s_y/1000.)
                exit
             else
                zwt = zi_soisno(j)
             endif
          enddo
          if (qcharge_tot > 0.) zwt = max(0.,zwt - qcharge_tot/1000./rous)
       endif
    endif

!-- Topographic runoff  ----------------------------------------------------------
    dzsum = 0.
    icefracsum = 0.
    do j = max(jwt,1), nl_soil
       dzsum = dzsum + dzmm(j)
       icefracsum = icefracsum + icefrac(j) * dzmm(j)
    end do
    ! add ice impedance factor to baseflow
    fracice_rsub = max(0.,exp(-3.*(1.-(icefracsum/dzsum)))-exp(-3.))/(1.0-exp(-3.))
    imped = max(0.,1.-fracice_rsub)
    drainage = imped * 5.5e-3 * exp(-2.5*zwt)  ! drainage (positive = out of soil column)

!-- Water table is below the soil column  ----------------------------------------
    if(jwt == nl_soil) then             
       wa = wa - drainage * deltim
       zwt = max(0.,zwt + (drainage * deltim)/1000./rous)
       wliq_soisno(nl_soil) = wliq_soisno(nl_soil) + max(0.,(wa-5000.))
       wa = min(wa, 5000.)
    else                                
!-- Water table within soil layers 1-9  ------------------------------------------
!============================== RSUB_TOP =========================================
       !-- Now remove water via drainage
       drainage_tot = - drainage * deltim
       do j = jwt+1, nl_soil
          ! use analytical expression for specific yield
          s_y = porsl(j) * ( 1. - (1.-1.e3*zwt/psi0(j))**(-1./bsw(j)))
          s_y = max(s_y,0.02)
             
          drainage_layer = max(drainage_tot, -(s_y*(zi_soisno(j)-zwt)*1.e3))
          drainage_layer = min(drainage_layer,0.)
          wliq_soisno(j) = wliq_soisno(j) + drainage_layer

          drainage_tot = drainage_tot - drainage_layer

          if(drainage_tot >= 0.)then 
             zwt = max(0.,zwt - drainage_layer/s_y/1000.)
             exit
          else
             zwt = zi_soisno(j)
          endif
       enddo

!-- Remove residual drainage  ------------------------------------------------
       zwt = max(0.,zwt - drainage_tot/1000./rous)
       wa = wa + drainage_tot

!-- Recompute jwt  ---------------------------------------------------------------
       ! allow jwt to equal zero when zwt is in top layer
       jwt = nl_soil
       do j = 1, nl_soil
          if(zwt <= zi_soisno(j)) then
             jwt = j-1
             exit
          end if
       enddo

    end if   ! end of jwt if construct

!*--------------------------------------------------------------------------------
!*  Niu, 06/2012
!*  if(jwt == nl_soil) then
!*     zwt = (zi_soisno(nl_soil) + 25.) - wa/1000./rous  !(m)
!*     wliq_soisno(nl_soil) = wliq_soisno(nl_soil) - qcharge * deltim  ! [mm]
!*     wliq_soisno(nl_soil) = wliq_soisno(nl_soil) + max(0.,(wa - 5000.))
!*     wa = min(wa, 5000.)
!*  else
!*     if(jwt == nl_soil-1)then
!*        zwt = zi_soisno(nl_soil) - (wt-rous*1000*25.)/(eff_porosity(nl_soil))/1000.
!*     else
!*        ws = 0.   ! water used to fill soil air pores
!*        do j = jwt+2, nl_soil
!*           ws = ws + eff_porosity(j)*dzmm(j)
!*        enddo
!*        zwt = zi_soisno(jwt+1)-(wt-rous*1000.*25.-ws)/(eff_porosity(jwt+1))/1000.
!*     endif
!*
!*     wtsub = 0.
!*     do j = 1, nl_soil
!*       wtsub = wtsub + hk(j)*dzmm(j)
!*     end do
!*
!*     do j = 1, nl_soil           ! Removing subsurface runoff
!*        wliq_soisno(j) = wliq_soisno(j) - drainage*deltim*hk(j)*dzmm(j)/wtsub
!*     end do
!*  end if
!*--------------------------------------------------------------------------------
!
    zwt = max(0.0,zwt)
    zwt = min(80.,zwt)

    rsubst = drainage


    ! Correction [1]
    ! NON-physically based corection on wliq_soisno
    ! excessive water above saturation added to the above unsaturated layer like a bucket
    ! if column fully saturated, excess water goes to runoff

    do j = nl_soil,2,-1
       xsi = max(wliq_soisno(j)-eff_porosity(j)*dzmm(j),0.)
       wliq_soisno(j) = min(eff_porosity(j)*dzmm(j), wliq_soisno(j))
       wliq_soisno(j-1) = wliq_soisno(j-1) + xsi
    end do

    xs1 = wliq_soisno(1) - (pondmx+porsl(1)*dzmm(1)-wice_soisno(1))
    if(xs1 > 0.)then
       wliq_soisno(1) = pondmx+porsl(1)*dzmm(1)-wice_soisno(1)
    else
       xs1 = 0.
    endif

    rsubst = rsubst + xs1 / deltim


    ! Correction [2]
    ! NON-physically based corection on wliq_soisno
    ! Limit wliq_soisno to be greater than or equal to watmin.
    ! Get water needed to bring wliq_soisno equal watmin from lower layer.
    ! If insufficient water in soil layers, get from aquifer water

    do j = 1, nl_soil-1
       if (wliq_soisno(j) < watmin) then
          xs = watmin - wliq_soisno(j)
          ! deepen water table if water is passed from below zwt layer
          if(j == jwt) then 
             zwt = max(0.,zwt + xs/eff_porosity(j)/1000.)
          endif
       else
          xs = 0.
       end if
       wliq_soisno(j  ) = wliq_soisno(j  ) + xs
       wliq_soisno(j+1) = wliq_soisno(j+1) - xs
    end do

    ! Get water for bottom layer from layers above if possible
    if(wliq_soisno(nl_soil) < watmin)then
       xs = watmin-wliq_soisno(nl_soil)
       do j = nl_soil-1, 1, -1
          available_wliq_soisno = max(wliq_soisno(j)-watmin-xs,0.)
          if(available_wliq_soisno >= xs)then
             wliq_soisno(nl_soil) = wliq_soisno(nl_soil) + xs
             wliq_soisno(j      ) = wliq_soisno(j      ) - xs
             xs = 0.
             exit
          else
             wliq_soisno(nl_soil) = wliq_soisno(nl_soil) + available_wliq_soisno
             wliq_soisno(j      ) = wliq_soisno(j      ) - available_wliq_soisno
             xs = xs - available_wliq_soisno
          end if
       end do
    else
       xs = 0.
    end if

    ! Needed in case there is no water to be found
    wliq_soisno(nl_soil) = wliq_soisno(nl_soil) + xs

    ! Sub-surface runoff and drainage
    rsubst = rsubst - xs/deltim

  end subroutine subsurfacerunoff		 


!-----------------------------------------------------------------------



 subroutine snowcompaction (lb,deltim,imelt,fiold,t_soisno,wliq_soisno,wice_soisno,dz_soisno)
 
!=======================================================================
! Original author: Yongjiu Dai, September 15, 1999
!
! three of metamorphisms of changing snow characteristics are implemented,
! i.e., destructive, overburden, and melt. The treatments of the former two
! are from SNTHERM.89 and SNTHERM.99 (1991, 1999). The contribution due to
! melt metamorphism is simply taken as a ratio of snow ice fraction after
! the melting versus before the melting.
!
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : denice, denh2o, tfrz
  use clm_varcon, only : denice, denh2o, tfrz
  implicit none
!
!-------------------------- Dummy argument -----------------------------
!
  integer, INTENT(in) :: &
        lb             ! lower bound of array

  integer, INTENT(in) :: & 
        imelt(lb : 0)  ! signifies if node in melting (imelt = 1)

  real(r8), INTENT(in) :: &
        deltim,       &! seconds i a time step [second]
        fiold(lb : 0),&! fraction of ice relative to 
                       ! the total water content at the previous time step 
        t_soisno (lb : 0),   &! nodal temperature [K]
        wice_soisno(lb : 0), &! ice lens [kg/m2]
        wliq_soisno(lb : 0)   ! liquid water [kg/m2]

  real(r8), INTENT(inout) :: &
        dz_soisno  (lb : 0)   ! layer thickness [m]
!
!----------------------- local variables ------------------------------
  integer i            ! Numeber of doing loop
  
  real(r8) :: &
       c1,            &! = 2.777e-7 [m2/(kg s)]
       c2,            &! = 21e-3 [m3/kg]
       c3,            &! = 2.777e-6 [1/s]
       c4,            &! = 0.04 [1/K]
       c5,            &! = 2.0 
       c6,            &! = 5.15e-7.
       c7,            &! = 4.
       dm,            &! Upper Limit on Destructive Metamorphism Compaction [kg/m3]
       eta0            ! The Viscosity Coefficient Eta0 [kg-s/m2]

  real(r8) :: &
       burden,        &! pressure of overlying snow [kg/m2]
       ddz1,          &! Rate of settling of snowpack due to destructive metamorphism.
       ddz2,          &! Rate of compaction of snowpack due to overburden.
       ddz3,          &! Rate of compaction of snowpack due to melt [1/s]
       dexpf,         &! expf=exp(-c4*(273.15-t_soisno)).
       fi,            &! Fraction of ice relative to the total water content 
                       ! at the current time step
       td,            &! t_soisno - tfrz [K]
       pdzdtc,        &! Nodal rate of change in fractional-thickness 
                       ! due to compaction [fraction/s]
       void,          &! void (1 - vol_ice - vol_liq)
       wx,            &! water mass (ice+liquid) [kg/m2]
       bi              ! partitial density of ice [kg/m3]

  data c2,c3,c4,c5/23.e-3, 2.777e-6, 0.04, 2.0/
  data c6/5.15e-7/, c7/4./
  data dm/100./         
  data eta0/9.e5/      

!=======================================================================

      burden = 0.0

      do i = lb, 0
         wx = wice_soisno(i) + wliq_soisno(i)
         void = 1.- (wice_soisno(i)/denice + wliq_soisno(i)/denh2o)/dz_soisno(i)

! Disallow compaction for water saturated node and lower ice lens node.
         if(void <= 0.001 .or. wice_soisno(i) <= .1)then
            burden = burden+wx
            CYCLE
         endif

         bi = wice_soisno(i) / dz_soisno(i)
         fi = wice_soisno(i) / wx
         td = tfrz-t_soisno(i)
         
         dexpf = exp(-c4*td)

! Settling as a result of destructive metamorphism
         ddz1 = -c3*dexpf
         if(bi > dm) ddz1 = ddz1*exp(-46.0e-3*(bi-dm))

! Liquid water term
         if(wliq_soisno(i) > 0.01*dz_soisno(i)) ddz1=ddz1*c5

! Compaction due to overburden
         ddz2 = -burden*exp(-0.08*td - c2*bi)/eta0

! Compaction occurring during melt
         if(imelt(i) == 1)then
            ddz3 = - 1./deltim * max(0.,(fiold(i) - fi)/fiold(i))
         else
            ddz3 = 0.
         endif

! Time rate of fractional change in dz_soisno (units of s-1)
         pdzdtc = ddz1+ddz2+ddz3

! The change in dz_soisno due to compaction
         dz_soisno(i) = dz_soisno(i)*(1.+pdzdtc*deltim)

! Pressure of overlying snow
         burden = burden+wx

      end do

 end subroutine snowcompaction



 subroutine snowlayerscombine (lb,snl, &
            z_soisno,dz_soisno,zi_soisno,wliq_soisno,wice_soisno,t_soisno,scv,snowdp)

!=======================================================================
! Original author : Yongjiu Dai, September 15, 1999
!
! checks for elements which are below prescribed minimum for thickness or mass.
! If snow element thickness or mass is less than a prescribed minimum,
! it is combined with neighboring element to be best combine with,
! and executes the combination of mass and energy in clm_combo.f90
! 
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

  implicit none

!-------------------------- Dummy argument -----------------------------
  integer, INTENT(in) :: lb               ! lower bound of array

! numbering from 1 (bottom) mss (surface)
  real(r8), INTENT(inout) :: wice_soisno(lb:1)   ! ice lens [kg/m2]
  real(r8), INTENT(inout) :: wliq_soisno(lb:1)   ! liquid water {kg/m2]
  real(r8), INTENT(inout) :: t_soisno (lb:1)     ! nodel temperature [K]
  real(r8), INTENT(inout) :: dz_soisno  (lb:1)   ! layer thickness [m]
  real(r8), INTENT(inout) :: z_soisno   (lb:1)   ! node depth [m]
  real(r8), INTENT(inout) :: zi_soisno  (lb-1:1) ! depth of layer interface [m]
  real(r8), INTENT(inout) :: snowdp       ! snow depth [m]
  real(r8), INTENT(inout) :: scv          ! snow mass - water equivalent [kg/m2]
  integer, INTENT(inout) :: snl           ! Number of snow

!----------------------- Local variables ------------------------------
  real(r8) :: drr           ! thickness of the combined [m]
  real(r8) :: dzmin(5)      ! minimum of snow layer 1 (top) to msn0 (bottom)
  real(r8) :: zwice         ! total ice mass in snow
  real(r8) :: zwliq         ! total liquid water in snow

  integer :: i              ! number of do looping
  integer :: j              ! node index
  integer :: k              ! number of do looping
  integer :: l              ! node index
  integer :: msn_old        ! number of snow layer 1 (top) to msn0 (bottom)
  integer :: mssi           ! node index
  integer :: neibor         ! adjacent node selected for combination

  data dzmin /0.010, 0.015, 0.025, 0.055, 0.115/

!-----------------------------------------------------------------------
! check the mass of ice lens of snow, when the total less than a small value,
! combine it with the underlying neighbor
      msn_old = snl
      do j = msn_old+1, 0
         if(wice_soisno(j) <= .1)then
            wliq_soisno(j+1) = wliq_soisno(j+1) + wliq_soisno(j)
            wice_soisno(j+1) = wice_soisno(j+1) + wice_soisno(j)

! shift all elements above this down one.
            if(j > snl+1 .AND. snl < -1)then
               do i =  j, snl+2, -1
                  t_soisno(i) = t_soisno(i-1)
                  wliq_soisno(i) = wliq_soisno(i-1)
                  wice_soisno(i) = wice_soisno(i-1)
                  dz_soisno(i) = dz_soisno(i-1)
               enddo
            endif

            snl = snl + 1
!*          write(6,*) 'one snow layer is gone'

         endif

      enddo

      if(snl == 0)then
         scv = 0.
         snowdp = 0.
!*       write(6,*) 'all snow has gone'
         return
      else
         scv = 0.
         snowdp = 0.
         zwice = 0.
         zwliq = 0.
         do j = snl + 1, 0
            scv = scv + wice_soisno(j) + wliq_soisno(j)
            snowdp = snowdp + dz_soisno(j)
            zwice = zwice + wice_soisno(j)
            zwliq = zwliq + wliq_soisno(j)
         enddo
      endif
!-----------------------------------------------------------------------
! check the snow depth

      if(snowdp < 0.01)then       !!! all snow gone 
          
         snl = 0
         scv = zwice
         if(scv <= 0.) snowdp = 0.

! the liquid water assumed ponding on soil surface
         wliq_soisno(1) = wliq_soisno(1) + zwliq
!*       write(6,'(17h all snow is gone)')
         return

      else                        !!! snow layers combined

! two or more layers 

        if(snl < -1)then
           msn_old = snl
           mssi = 1
           do i = msn_old+1, 0

! If top node is removed, combine with bottom neighbor
              if(dz_soisno(i) < dzmin(mssi))then
                 if(i == snl+1)then
                    neibor = i + 1

! If the bottom neighbor is not snow, combine with the top neighbor
                 else if(i == 0)then
                    neibor = i - 1

! If none of the above special cases apply, combine with the thinnest neighbor
                 else
                   neibor = i + 1
                   if((dz_soisno(i-1)+dz_soisno(i)) < (dz_soisno(i+1)+dz_soisno(i))) neibor = i-1
                 endif

! Node l and j are combined and stored as node j.

                 if(neibor > i)then
                    j = neibor
                    l = i
                 else
                    j = i
                    l = neibor
                 endif
                 call combo ( dz_soisno(j), wliq_soisno(j), wice_soisno(j), t_soisno(j),&
                              dz_soisno(l), wliq_soisno(l), wice_soisno(l), t_soisno(l) )

! Now shift all elements above this down one.

                 if(j-1 > snl+1) then
                    do k = j-1, snl+2, -1
                       t_soisno(k) = t_soisno(k-1)
                       wice_soisno(k) = wice_soisno(k-1)
                       wliq_soisno(k) = wliq_soisno(k-1)
                       dz_soisno(k) = dz_soisno(k-1)
                    enddo
                 endif

                 snl = snl + 1

!*    write(6,'(7h Nodes ,i4,4h and,i4,14h combined into,i4)') l,j,j

                 if(snl >= -1) EXIT

! The layer thickness great than the prescibed minimum value

              else
                 mssi = mssi + 1 
              endif
           enddo

        end if

! Reset the node depth and the depth of layer interface

        zi_soisno(0) = 0.
        do k = 0, snl+1, -1
           z_soisno(k) = zi_soisno(k) - 0.5*dz_soisno(k)
           zi_soisno(k-1) = zi_soisno(k) - dz_soisno(k)
        enddo

      endif                       !!! snow layers combined 

 end subroutine snowlayerscombine



 subroutine snowlayersdivide (lb,snl,z_soisno,dz_soisno,zi_soisno,wliq_soisno,wice_soisno,t_soisno)

!=======================================================================
! Original author : Yongjiu Dai, September 15, 1999
!
! subdivides snow layer when its thickness exceed the prescribed maximum
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

  implicit none

!-------------------------- Dummy argument -----------------------------

   integer, INTENT(in) :: lb              ! lower bound of array
   integer, INTENT(inout) :: snl          ! Number of snow
  real(r8), INTENT(inout) :: wice_soisno(lb:0)   ! ice lens [kg/m2]
  real(r8), INTENT(inout) :: wliq_soisno(lb:0)   ! liquid water [kg/m2]
  real(r8), INTENT(inout) :: t_soisno   (lb:0)   ! Nodel temperature [K]
  real(r8), INTENT(inout) :: dz_soisno  (lb:0)   ! Layer thickness [m]
  real(r8), INTENT(inout) :: z_soisno   (lb:0)   ! Node depth [m]
  real(r8), INTENT(inout) :: zi_soisno  (lb-1:0) ! Depth of layer interface [m]

!----------------------- Local variables ------------------------------

! numbering from 1 (surface) msno (bottom)
  real(r8) :: drr      ! thickness of the combined [m]
  real(r8) :: dzsno(5) ! Snow layer thickness [m] 
  real(r8) :: swice(5) ! Partial volume of ice [m3/m3]
  real(r8) :: swliq(5) ! Partial volume of liquid water [m3/m3]
  real(r8) :: tsno(5)  ! Nodel temperature [K]

  integer k            ! number of do looping
  integer msno         ! number of snow layer 1 (top) to msno (bottom)

  real(r8) zwice,zwliq,propor

!-----------------------------------------------------------------------

      msno = abs(snl)
      do k = 1, msno
         dzsno(k) = dz_soisno  (k + snl)
         swice(k) = wice_soisno(k + snl)
         swliq(k) = wliq_soisno(k + snl)
         tsno(k)  = t_soisno (k + snl)
      enddo

      if(msno == 1)then
         if(dzsno(1) > 0.03)then
         msno = 2
! Specified a new snow layer
         dzsno(1) = dzsno(1)/2.
         swice(1) = swice(1)/2.
         swliq(1) = swliq(1)/2.

         dzsno(2) = dzsno(1)
         swice(2) = swice(1)
         swliq(2) = swliq(1)
         tsno(2)  = tsno(1)
!        write(6,*)'Subdivided Top Node into two layer (1/2)'
         endif
      endif

      if(msno > 1)then
         if(dzsno(1) > 0.02)then
         drr = dzsno(1) - 0.02
         propor = drr/dzsno(1)
         zwice = propor*swice(1)
         zwliq = propor*swliq(1)

         propor = 0.02/dzsno(1)
         swice(1) = propor*swice(1)
         swliq(1) = propor*swliq(1)
         dzsno(1) = 0.02

         call combo(dzsno(2),swliq(2),swice(2),tsno(2), &
                    drr,zwliq,zwice,tsno(1))

!        write(6,*) 'Subdivided Top Node &
!                    20 mm combined into underlying neighbor'

         if(msno <= 2 .AND. dzsno(2) > 0.07)then
! subdivided a new layer
            msno = 3
            dzsno(2) = dzsno(2)/2.
            swice(2) = swice(2)/2.
            swliq(2) = swliq(2)/2.

            dzsno(3) = dzsno(2)
            swice(3) = swice(2)
            swliq(3) = swliq(2)
            tsno(3)  = tsno(2)
         endif
         endif
      endif

      if(msno > 2)then
         if(dzsno(2) > 0.05)then
         drr = dzsno(2) - 0.05
         propor = drr/dzsno(2)
         zwice = propor*swice(2)
         zwliq = propor*swliq(2)

         propor = 0.05/dzsno(2)
         swice(2) = propor*swice(2)
         swliq(2) = propor*swliq(2)
         dzsno(2) = 0.05

         call combo(dzsno(3),swliq(3),swice(3),tsno(3), &
                    drr,     zwliq,   zwice,   tsno(2))

!        write(6,*)'Subdivided 50 mm from the subsface layer &
!                   &and combined into underlying neighbor'

         if(msno <= 3 .AND. dzsno(3) > 0.18)then
! subdivided a new layer
            msno =  4
            dzsno(3) = dzsno(3)/2.
            swice(3) = swice(3)/2.
            swliq(3) = swliq(3)/2.

            dzsno(4) = dzsno(3)
            swice(4) = swice(3)
            swliq(4) = swliq(3)
            tsno(4)  = tsno(3)
         endif
         endif
      endif

      if(msno > 3)then
         if(dzsno(3) > 0.11)then
         drr = dzsno(3) - 0.11
         propor = drr/dzsno(3)
         zwice = propor*swice(3)
         zwliq = propor*swliq(3)

         propor = 0.11/dzsno(3)
         swice(3) = propor*swice(3)
         swliq(3) = propor*swliq(3)
         dzsno(3) = 0.11

         call combo(dzsno(4),swliq(4),swice(4),tsno(4), &
                    drr,     zwliq,   zwice,   tsno(3))

!        write(6,*)'Subdivided 110 mm from the third Node &
!                   &and combined into underlying neighbor'

         if(msno <= 4 .AND. dzsno(4) > 0.41)then
! subdivided a new layer
            msno = 5
            dzsno(4) = dzsno(4)/2.
            swice(4) = swice(4)/2.
            swliq(4) = swliq(4)/2.

            dzsno(5) = dzsno(4)
            swice(5) = swice(4)
            swliq(5) = swliq(4)
            tsno(5)  = tsno(4)
         endif
         endif
      endif
         
      if(msno > 4)then
         if(dzsno(4) > 0.23)then
         drr = dzsno(4) - 0.23
         propor = drr/dzsno(4)
         zwice = propor*swice(4)
         zwliq = propor*swliq(4)

         propor = 0.23/dzsno(4)
         swice(4) = propor*swice(4)
         swliq(4) = propor*swliq(4)
         dzsno(4) = 0.23

         call combo(dzsno(5),swliq(5),swice(5),tsno(5), &
                    drr,     zwliq,   zwice,   tsno(4))

!        write(6,*)'Subdivided 230 mm from the fourth Node &
!                   'and combined into underlying neighbor'
         endif
      endif
         
      snl = - msno

      do k = snl+1, 0
         dz_soisno(k)   = dzsno(k - snl)
         wice_soisno(k) = swice(k - snl)
         wliq_soisno(k) = swliq(k - snl)
         t_soisno(k)  = tsno (k - snl)
      enddo

      zi_soisno(0) = 0.
      do k = 0, snl+1, -1
         z_soisno(k)    = zi_soisno(k) - 0.5*dz_soisno(k)
         zi_soisno(k-1) = zi_soisno(k) - dz_soisno(k)
      enddo

 end subroutine snowlayersdivide



 subroutine combo ( dz_soisno, wliq_soisno, wice_soisno, t, &
                   dz2, wliq2, wice2, t2 )

!=======================================================================
! Original author: Yongjiu Dai, September 15, 1999
!
! combines two elements and returns the following combined
! variabless: dz_soisno, t, wliq_soisno, wice_soisno.
! the combined temperature is based on the equation:
! the sum of the enthalpies of the two elements = that of the combined element.
!
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : cpice, cpliq, hfus, tfrz
  use clm_varcon, only : cpice, cpliq, hfus, tfrz
  implicit none

!-------------------------- Dummy argument -----------------------------

  real(r8), INTENT(in) :: dz2     ! nodal thickness of 2 elements being combined [m]
  real(r8), INTENT(in) :: wliq2   ! liquid water of element 2 [kg/m2]
  real(r8), INTENT(in) :: wice2   ! ice of element 2 [kg/m2]
  real(r8), INTENT(in) :: t2      ! nodal temperature of element 2 [K]

  real(r8), INTENT(inout) :: dz_soisno   ! nodal thickness of 1 elements being combined [m]
  real(r8), INTENT(inout) :: wliq_soisno ! liquid water of element 1
  real(r8), INTENT(inout) :: wice_soisno ! ice of element 1 [kg/m2]
  real(r8), INTENT(inout) :: t    ! nodel temperature of elment 1 [K]

!----------------------- Local variables ------------------------------

  real(r8) dzc    ! Total thickness of nodes 1 and 2 (dzc=dz_soisno+dz2).
  real(r8) wliqc  ! Combined liquid water [kg/m2]
  real(r8) wicec  ! Combined ice [kg/m2]
  real(r8) tc     ! Combined node temperature [K]
  real(r8) h      ! enthalpy of element 1 [J/m2]
  real(r8) h2     ! enthalpy of element 2 [J/m2]
  real(r8) hc     ! temporary

!-----------------------------------------------------------------------

      dzc = dz_soisno+dz2
      wicec = (wice_soisno+wice2)
      wliqc = (wliq_soisno+wliq2)
      h   = (cpice*wice_soisno+cpliq*wliq_soisno)*(t-tfrz)+hfus*wliq_soisno
      h2  = (cpice*wice2+cpliq*wliq2)*(t2-tfrz)+hfus*wliq2

      hc = h + h2
      if(hc < 0.)then
         tc = tfrz + hc/(cpice*wicec+cpliq*wliqc)
      else if(hc.le.hfus*wliqc)then
         tc = tfrz
      else
         tc = tfrz + (hc - hfus*wliqc)/(cpice*wicec+cpliq*wliqc)
      endif

      dz_soisno = dzc
      wice_soisno = wicec 
      wliq_soisno = wliqc
      t = tc

 end subroutine combo	  


 SUBROUTINE rain_snow_temp (forc_t,forc_q,forc_psrf,forc_prc,forc_prl,tcrit,&
                            prc_rain,prc_snow,prl_rain,prl_snow,t_precip,bifall)

!=======================================================================
! define the rate of rainfall and snowfall and precipitation water temp
! Original author : Yongjiu Dai, 09/1999; 08/31/2002, 04/2014
!=======================================================================
!
! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl,  use PhysicalConstants, only : tfrz
  use clm_varcon, only : tfrz
 
  IMPLICIT NONE
 
! ------------------------ Dummy Argument ------------------------------

  real(r8), INTENT(in) :: forc_t     ! temperature at agcm reference height [kelvin]
  real(r8), INTENT(in) :: forc_q     ! specific humidity at agcm reference height [kg/kg]
  real(r8), INTENT(in) :: forc_psrf  ! atmosphere pressure at the surface [pa]
  real(r8), INTENT(in) :: forc_prc   ! convective precipitation [mm/s]
  real(r8), INTENT(in) :: forc_prl   ! large scale precipitation [mm/s]

  real(r8), INTENT(in) :: tcrit      ! critical temp. to determine rain or snow

  real(r8), INTENT(out) :: prc_rain  ! convective rainfall [kg/(m2 s)]
  real(r8), INTENT(out) :: prc_snow  ! convective snowfall [kg/(m2 s)]
  real(r8), INTENT(out) :: prl_rain  ! large scale rainfall [kg/(m2 s)]
  real(r8), INTENT(out) :: prl_snow  ! large scale snowfall [kg/(m2 s)]
  real(r8), INTENT(out) :: t_precip  ! snowfall/rainfall temperature [kelvin]
  real(r8), INTENT(out) :: bifall    ! bulk density of newly fallen dry snow [kg/m3]
  real(r8) :: flfall  ! fraction of liquid water within falling precip.

!-----------------------------------------------------------------------
! the upper limit of air temperature is set for snowfall, this cut-off 
! was selected based on Fig. 1, Plate 3-1, of Snow Hydrology (1956).
! the percentage of liquid water by mass, which is arbitrarily set to 
! vary linearly with air temp, from 0% at 273.16 to 40% max at 275.16.

      if(forc_t>tfrz+tcrit)then
        prc_rain = forc_prc       ! convective rainfall (mm/s)
        prl_rain = forc_prl       ! large scale rainfall (mm/s)
        prc_snow = 0.             ! convective snowfall (mm/s)
        prl_snow = 0.             ! large scale snowfall (mm/s)
        flfall = 1.               ! fraction of liquid water within falling precip.
        bifall = 169.             ! (not used)
      else
        if(forc_t<=tfrz)then
          flfall = 0.
        else if(forc_t<=tfrz+2.)then
          flfall = -54.632+0.2*forc_t
        else
          flfall = 0.4
        endif
 
! use Alta relationship, Anderson(1976); LaChapelle(1961), 
! U.S.Department of Agriculture Forest Service, Project F, 
! Progress Rep. 1, Alta Avalanche Study Center:Snow Layer Densification.

        if(forc_t>tfrz+2.)then
          bifall = 169.
        else if(forc_t>tfrz-15.)then
          bifall = 50.+1.7*(forc_t-tfrz+15.)**1.5
        else
          bifall = 50.
        endif
 
        prc_rain = forc_prc*flfall
        prl_rain = forc_prl*flfall
        prc_snow = forc_prc*(1.-flfall)                 
        prl_snow = forc_prl*(1.-flfall)                 

      endif

! -------------------------------------------------------------
! temperature of rainfall or snowfall
! -------------------------------------------------------------

      call wetbulb(forc_t,forc_psrf,forc_q,t_precip)

      if (forc_t > 275.65) then
         if (t_precip < tfrz) t_precip = tfrz
      else
         t_precip = min(tfrz,t_precip)
         if(flfall > 0.0)then
           t_precip = tfrz - sqrt((1.0/flfall)-1.0)/100.0
         endif
      endif	  

 END SUBROUTINE rain_snow_temp

 subroutine wetbulb(t,p,q,twc)

!=======================================================================
! Wet-bulb temperature
!    
! Yongjiu Dai, 07/2013
!=======================================================================

! use precision
  use shr_kind_mod, only: r8 => shr_kind_r8 

!shixl, use PhysicalConstants, only : tfrz, hvap, cpair
      use clm_varcon, only : tfrz, hvap, cpair	  
      use QSatMod, only : QSat		 ! added by Anning Huang 2016.11.17

      implicit none
      real(r8), intent(in) :: t     ! air temperature [K]
      real(r8), intent(in) :: p     ! atmos pressure [pa]
      real(r8), intent(in) :: q     ! air specific humidity [kg/kg]
      real(r8), intent(out) :: twc  ! wet bulb temperature [K]

      integer i
      real(r8) es, esdT, qs, qsdT, r, rws

! ----------------------------------------------------------
!     real(r8) tcair ! dry-bulb temperature in celsius
!     real(r8) bp    ! approximate average barometric pressure [mb]
!     real(r8) ea    ! water vapor pressure in air [mb]
!     real(r8) eas   ! saturated water vapor pressure in air [mb]
!     real(r8) delt  ! delt=eas*4278.63/((tcair+242.792)*(tcair+242.792))
!     real(r8) tav   ! average of dry-bulb temperature and wet bulb temperature in celsius
!     real(r8) eav   ! eav=2.7489E8*exp(-4278.63/(tav+242.792))
!     real(r8) rh    ! relative humidity
! ----------------------------------------------------------
!  WETBULB computes wet-bulb temperatures from dry-bulb (tkair) and
!  vapor pressure of air(ea). routine adapted from e. anderson, p. 188.
! ----------------------------------------------------------
!     call qsadv(t,p,es,esdT,qs,qsdT)
!     rh  = min(1.0,q/qs)
!     bp  = p/100.0             ! mb
!     eas = es/100.0            ! mb
!     ea  = eas                 ! mb
!     delt = esdT/100.          ! mb/K	 !
!     tcair = t - tfrz		!
!*    eas = 2.7489e8*exp(-4278.63/(tcair+242.792))
!*    delt = eas*4278.63/((tcair+242.792)*(tcair+242.792))
!
!     do i = 1, 3
!        twc  = delt*tcair+6.6e-4 *bp*tcair+7.59e-7*bp*tcair*tcair+ea-eas
!        twc  = twc/(delt+6.6e-4*bp+7.59e-7*bp*tcair)    ! in celsius
!
!        tav  = 0.5*(tcair+twc)+tfrz
!        call qsadv(tav,p,es,esdT,qs,qsdT)
!        eav  = es/100.
!        delt = esdT/100.
!
!*       tav  = 0.5*(tcair+twc)
!*       eav  = 2.7489e8*exp(-4278.63/(tav+242.792))
!*       delt = eav*4278.63/((tav+242.792)*(tav+242.792))
!     enddo
!     twc = twc + tfrz
! ----------------------------------------------------------

! ----------------------------------------------------------
! the defining equation for the wetbulb temp Twb is
!     f(Twb) = Twb-T - Lv/Cp [r-rs(Twb)] = 0, 
! where 
!     T = the dry-bulb temp (K), 
!     Lv = the latent heat of vaporization (J/kg/K), 
!     Cp = the specific heat of air at constant pressure,
!     r = the water vapor mixing ratio [q/(1-q)], 
!     rs(Twb) = the saturation mixing ratio at wetbulb temp.
! http://www.asp.ucar.edu/colloquium/1992/notes/paet1/node81.html
! ----------------------------------------------------------
      call qsat(t,p,es,esdT,qs,qsdT)	

      r = q/(1.0-q)
      if (q >= qs) r = qs/(1.0-qs)
      twc = t
      do i = 1, 6

      call qsat(t,p,es,esdT,qs,qsdT)
	  	  	  		  
         rws= qs/(1.0-qs)
         twc = (twc + t + hvap/cpair*(r-rws))/2.0	 

      enddo

!*----------------------------------------------------------
!*wetbulb temp as air temp and relative humidity at standard sea level pressure.
!*valid for RH% (5%-99%), T (-20C-50C). R. Stull, 2011: Wet-bulb temperature form 
!*relative humidity and air temperature. J. Appl. Meteor. and Climatol., vol 50, 2267-2269.
!*----------------------------------------------------------
!*    tcair = t - tfrz
!*    call qsadv(t,p,es,esdT,qs,qsdT)
!*    rh  = min(1.0,q/qs)
!*    twc = tcair*atan(0.151977*(rh*100.+8.313659)**0.5) &
!*        + atan(tcair+rh*100.)-atan(rh*100.-1.676331) &
!*        + 0.00391838*(rh*100.)**1.5*atan(0.023101*rh*100.)-4.686035
!*    twc = twc + tfrz
!*----------------------------------------------------------

 end subroutine wetbulb		


      subroutine tridia (n, a, b, c, r, u)	         
!     use precision
      use shr_kind_mod, only: r8 => shr_kind_r8 

      implicit none
      integer, intent(in) :: n        !length of diagonal element vector
      real(r8), intent(in) :: a(1:n)  !subdiagonal elements
      real(r8), intent(in) :: b(1:n)  !diagonal elements
      real(r8), intent(in) :: c(1:n)  !superdiagonal elements
      real(r8), intent(in) :: r(1:n)  !right hand side
      real(r8), intent(out) :: u(1:n) !solution vector

      integer j
      real(r8) gam(1:n),bet
! -----------------------------------------------------------------

      bet = b(1)
      u(1) = r(1) / bet
      do j = 2, n
            gam(j) = c(j-1) / bet
            bet = b(j) - a(j) * gam(j)
            u(j) = (r(j) - a(j)*u(j-1)) / bet
      end do
      do j = n-1, 1, -1
            u(j) = u(j) - gam(j+1) * u(j+1)
      end do

      end subroutine tridia		 

 end module DaiLakeMod
#endif
